@@ -14,59 +14,67 @@
     You should have received a copy of the GNU Library General Public
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+
+    csdemo - This is the demo application for crystal space, it reads worlds
+    and scenes with sequences and animation paths from data files allowing
+    to produce generic demos.
 */
 
 #include "cssysdef.h"
-#include "cssys/sysfunc.h"
-#include "demo.h"
-#include "demoseq.h"
-#include "csutil/cscolor.h"
-#include "csutil/cmdhelp.h"
+
+#include "csqsqrt.h"
+
 #include "csgeom/path.h"
 #include "cstool/csfxscr.h"
 #include "cstool/csview.h"
 #include "cstool/initapp.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/natwin.h"
-#include "ivideo/fontserv.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/txtmgr.h"
-#include "ivaria/conout.h"
-#include "iengine/engine.h"
-#include "iengine/sector.h"
-#include "iengine/light.h"
-#include "iengine/statlght.h"
+#include "csutil/cmdhelp.h"
+#include "csutil/cscolor.h"
+#include "csutil/event.h"
+#include "csutil/csevent.h"
+#include "csutil/stringarray.h"
+#include "csutil/sysfunc.h"
+
 #include "iengine/camera.h"
+#include "iengine/engine.h"
+#include "iengine/halo.h"
+#include "iengine/light.h"
+#include "iengine/material.h"
 #include "iengine/mesh.h"
 #include "iengine/movable.h"
-#include "iengine/halo.h"
-#include "iengine/material.h"
-#include "imesh/thing/thing.h"
-#include "imesh/thing/polygon.h"
-#include "imesh/thing/ptextype.h"
+#include "iengine/sector.h"
+#include "igraphic/imageio.h"
+#include "imap/loader.h"
+#include "imap/reader.h"
+#include "imesh/object.h"
 #include "imesh/particle.h"
 #include "imesh/sprite2d.h"
 #include "imesh/sprite3d.h"
-#include "imesh/ball.h"
 #include "imesh/stars.h"
-#include "imesh/object.h"
-#include "imap/reader.h"
-#include "imap/parser.h"
-#include "iutil/strvec.h"
+#include "imesh/thing.h"
+#include "iutil/cmdline.h"
 #include "iutil/comp.h"
-#include "iutil/eventh.h"
+#include "iutil/csinput.h"
+#include "iutil/databuff.h"
 #include "iutil/event.h"
+#include "iutil/eventh.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
-#include "iutil/virtclk.h"
-#include "iutil/csinput.h"
-#include "iutil/cmdline.h"
 #include "iutil/plugin.h"
+#include "iutil/stringarray.h"
+#include "iutil/virtclk.h"
 #include "iutil/vfs.h"
-#include "csutil/csstrvec.h"
-#include "igraphic/imageio.h"
+#include "ivideo/graph2d.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/fontserv.h"
+#include "ivideo/natwin.h"
+#include "ivaria/conout.h"
 #include "ivaria/reporter.h"
-#include "qsqrt.h"
+
+
+#include "demo.h"
+#include "demoseq.h"
 
 CS_IMPLEMENT_APPLICATION
 
@@ -79,11 +87,10 @@ void Demo::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  iReporter* rep = CS_QUERY_REGISTRY (System->object_reg, iReporter);
+  csRef<iReporter> rep (CS_QUERY_REGISTRY (System->object_reg, iReporter));
   if (rep)
   {
     rep->ReportV (severity, "crystalspace.application.demo", msg, arg);
-    rep->DecRef ();
   }
   else
   {
@@ -95,30 +102,11 @@ void Demo::Report (int severity, const char* msg, ...)
 
 Demo::Demo ()
 {
-  vc = NULL;
-  engine = NULL;
-  seqmgr = NULL;
-  loader = NULL;
-  myG3D = NULL;
-  myG2D = NULL;
-  myVFS = NULL;
-  kbd = NULL;
-  view = NULL;
-  message[0] = 0;
-  font = NULL;
+  seqmgr = 0;
 }
 
 Demo::~Demo ()
 {
-  if (vc) vc->DecRef ();
-  if (font) font->DecRef ();
-  if (view) view->DecRef ();
-  if (engine) engine->DecRef ();
-  if (myG3D) myG3D->DecRef ();
-  if (myG2D) myG2D->DecRef ();
-  if (myVFS) myVFS->DecRef ();
-  if (kbd) kbd->DecRef ();
-  if (loader) loader->DecRef ();
   delete seqmgr;
 }
 
@@ -126,43 +114,18 @@ void Cleanup ()
 {
   csPrintf ("Cleaning up...\n");
   iObjectRegistry* object_reg = System->object_reg;
-  delete System; System = NULL;
+  delete System; System = 0;
   csInitializer::DestroyApplication (object_reg);
 }
 
-iMeshWrapper* Demo::LoadObject (const char* objname, const char* filename,
-                                const char* /*classId*/, const char* loaderClassId,
-	iSector* sector, const csVector3& pos)
-{
-  iDataBuffer* databuf = myVFS->ReadFile (filename);
-  if (!databuf || !databuf->GetSize ())
-  {
-    if (databuf) databuf->DecRef ();
-    Report (CS_REPORTER_SEVERITY_ERROR, "Could not open file '%s' on VFS!",
-    	filename);
-    exit (0);
-  }
-  iMeshWrapper* obj = engine->LoadMeshWrapper (objname,
-  	loaderClassId, databuf, sector, pos);
-  databuf->DecRef ();
-  if (!obj)
-  {
-    Report (CS_REPORTER_SEVERITY_ERROR,
-    	"There was an error loading object from file '%s'!",
-	filename);
-    exit (0);
-  }
-  return obj;
-}
-
 static bool DemoEventHandler (iEvent& ev)
 {
-  if (ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess)
+  if (ev.Name == System->Process)
   {
     System->SetupFrame ();
     return true;
   }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess)
+  else if (ev.Name == System->FinalProcess)
   {
     System->FinishFrame ();
     return true;
@@ -173,22 +136,21 @@ static bool DemoEventHandler (iEvent& ev)
   }
 }
 
-static void TestDemoFile (const char* zip, iVFS* myVFS, csStrVector& demos)
+static void TestDemoFile (const char* zip, iVFS* myVFS, csStringArray& demos)
 {
-  iDataBuffer* realpath_db = myVFS->GetRealPath (zip);
+  csRef<iDataBuffer> realpath_db (myVFS->GetRealPath (zip));
   char* realpath = (char*)(realpath_db->GetData ());
   char* testpath = new char [strlen (realpath)+3];
   strcpy (testpath, realpath);
-  realpath_db->DecRef ();
   if (testpath[strlen (testpath)-1] == '/')
   {
     // We have a directory.
-    int l = strlen (testpath);
+    size_t l = strlen (testpath);
     testpath[l-1] = '$';
     testpath[l] = '/';
     testpath[l+1] = 0;
   }
-  else if (strstr (testpath, ".zip") == NULL)
+  else if (strstr (testpath, ".zip") == 0)
   {
     delete[] testpath;
     return;
@@ -197,7 +159,7 @@ static void TestDemoFile (const char* zip, iVFS* myVFS, csStrVector& demos)
   myVFS->Mount ("/tmp/csdemo_temp", testpath);
   if (myVFS->Exists ("/tmp/csdemo_temp/sequences"))
     demos.Push (csStrNew (testpath));
-  myVFS->Unmount ("/tmp/csdemo_temp", NULL);
+  myVFS->Unmount ("/tmp/csdemo_temp", 0);
   delete[] testpath;
 }
 
@@ -216,7 +178,7 @@ bool Demo::LoadDemoFile (const char* demofile)
   // This can fail, but we don't care :-)
   loader->LoadLibraryFile ("library");
 
-  if (!loader->LoadMapFile ("world", false, true))
+  if (!loader->LoadMapFile ("world", false, 0, true))
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "There was an error loading world!");
     exit (0);
@@ -228,7 +190,7 @@ bool Demo::LoadDemoFile (const char* demofile)
 
   engine->Prepare ();
 
-  view = new csView (engine, myG3D);
+  view = csPtr<iView> (new csView (engine, myG3D));
   view->GetCamera ()->SetSector (room);
   view->GetCamera ()->GetTransform ().SetOrigin (
   	  csVector3 (0.0f, 0.0f, -900.0f));
@@ -256,9 +218,23 @@ bool Demo::Initialize (int argc, const char* const argv[],
     return false;
   }
 
-  if (!csInitializer::SetupEventHandler (object_reg, DemoEventHandler))
+  Process = csevProcess (object_reg);
+  FinalProcess = csevFinalProcess (object_reg);
+  KeyboardDown = csevKeyboardDown (object_reg);
+  MouseDown = csevMouseDown (object_reg, 0);
+  MouseMove = csevMouseMove (object_reg, 0);
+
+  csEventID events[] = {
+    Process,
+    FinalProcess,
+    KeyboardDown,
+    MouseDown,
+    MouseMove,
+    CS_EVENTLIST_END };
+
+  if (!csInitializer::SetupEventHandler (object_reg, DemoEventHandler, events))
   {
-    Report (CS_REPORTER_SEVERITY_ERROR, "Couldn't initialize event handler!");
+    Report (CS_REPORTER_SEVERITY_ERROR, "Couldn't setup event handler!");
     return false;
   }
 
@@ -324,14 +300,6 @@ bool Demo::Initialize (int argc, const char* const argv[],
     exit (1);
   }
 
-  // Setup the texture manager
-  iTextureManager* txtmgr = myG3D->GetTextureManager ();
-  txtmgr->SetVerbose (true);
-
-  // Initialize the texture manager
-  txtmgr->ResetPalette ();
-  txtmgr->SetPalette ();
-
   font = myG2D->GetFontServer ()->LoadFont (CSFONT_LARGE);
 
   // Some commercials...
@@ -342,10 +310,10 @@ bool Demo::Initialize (int argc, const char* const argv[],
   Report (CS_REPORTER_SEVERITY_NOTIFY, "Creating world!...");
 
   // Check the demo file and mount it if required.
-  iCommandLineParser* cmdline = CS_QUERY_REGISTRY (object_reg,
-  	iCommandLineParser);
+  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
+  	iCommandLineParser));
   const char *val;
-  if ((val = cmdline->GetName ()) != NULL)
+  if ((val = cmdline->GetName ()) != 0)
   {
     if (!LoadDemoFile (val))
       return false;
@@ -353,40 +321,35 @@ bool Demo::Initialize (int argc, const char* const argv[],
   else
   {
     do_demo = 0;
-    selected_demo = -1;
+    selected_demo = (size_t)-1;
     // Here we can't do the demo because no data file was given.
     // However we scan for possible data files and present them to the
     // user.
-    iStrVector* zips = myVFS->FindFiles ("/this/*");
-    int i;
+    csRef<iStringArray> zips = myVFS->FindFiles ("/this/*");
+    size_t i;
     for (i = 0 ; i < zips->Length () ; i++)
     {
-      char* zip = zips->Get (i);
+      const char* zip = zips->Get (i);
       TestDemoFile (zip, myVFS, demos);
     }
-    zips->DecRef ();
     myVFS->Mount ("/tmp/csdemo_datadir", "$@data$/");
     zips = myVFS->FindFiles ("/tmp/csdemo_datadir/*");
     for (i = 0 ; i < zips->Length () ; i++)
     {
-      char* zip = zips->Get (i);
+      const char* zip = zips->Get (i);
       TestDemoFile (zip, myVFS, demos);
     }
-    zips->DecRef ();
-    myVFS->Unmount ("/tmp/csdemo_datadir", NULL);
+    myVFS->Unmount ("/tmp/csdemo_datadir", 0);
   }
 
-  cmdline->DecRef ();
-
-  txtmgr->SetPalette ();
-  col_red = txtmgr->FindRGB (255, 0, 0);
-  col_blue = txtmgr->FindRGB (0, 0, 255);
-  col_white = txtmgr->FindRGB (255, 255, 255);
-  col_gray = txtmgr->FindRGB (50, 50, 50);
-  col_black = txtmgr->FindRGB (0, 0, 0);
-  col_yellow = txtmgr->FindRGB (255, 255, 0);
-  col_cyan = txtmgr->FindRGB (0, 255, 255);
-  col_green = txtmgr->FindRGB (0, 255, 0);
+  col_red = myG2D->FindRGB (255, 0, 0);
+  col_blue = myG2D->FindRGB (0, 0, 255);
+  col_white = myG2D->FindRGB (255, 255, 255);
+  col_gray = myG2D->FindRGB (50, 50, 50);
+  col_black = myG2D->FindRGB (0, 0, 0);
+  col_yellow = myG2D->FindRGB (255, 255, 0);
+  col_cyan = myG2D->FindRGB (0, 255, 255);
+  col_green = myG2D->FindRGB (0, 255, 0);
 
   return true;
 }
@@ -401,13 +364,13 @@ static csVector2 map_br (1000, -1000);
 static int map_selpoint = 0;
 static char map_selpath[255] = { 0 };
 
-void Demo::GfxWrite (int x, int y, int fg, int bg, char *str, ...)
+void Demo::GfxWrite (int x, int y, int fg, int bg, const char *str, ...)
 {
   va_list arg;
-  char buf[256];
+  csString buf;
 
   va_start (arg, str);
-  vsprintf (buf, str, arg);
+  buf.FormatV (str, arg);
   va_end (arg);
 
   myG2D->Write (font, x, y, fg, bg, buf);
@@ -416,13 +379,13 @@ void Demo::GfxWrite (int x, int y, int fg, int bg, char *str, ...)
 void Demo::FileWrite (iFile* file, char *str, ...)
 {
   va_list arg;
-  char buf[256];
+  csString buf;
 
   va_start (arg, str);
-  vsprintf (buf, str, arg);
+  buf.FormatV (str, arg);
   va_end (arg);
 
-  file->Write (buf, strlen (buf));
+  file->Write (buf.GetData(), buf.Length());
 }
 
 void Demo::ShowMessage (const char* msg, ...)
@@ -430,7 +393,7 @@ void Demo::ShowMessage (const char* msg, ...)
   message_error = false;
   va_list arg;
   va_start (arg, msg);
-  vsprintf (message, msg, arg);
+  message.Format (msg, arg);
   va_end (arg);
   message_timer = csGetTicks () + 1500;
 }
@@ -440,7 +403,7 @@ void Demo::ShowError (const char* msg, ...)
   message_error = true;
   va_list arg;
   va_start (arg, msg);
-  vsprintf (message, msg, arg);
+  message.Format (msg, arg);
   va_end (arg);
   message_timer = csGetTicks () + 1500;
 }
@@ -452,11 +415,10 @@ void Demo::SetupFrame ()
     // Don't do the demo but print out information about
     // where to get all stuff.
     if (!myG3D->BeginDraw (CSDRAW_2DGRAPHICS)) return;
-    iTextureManager* txtmgr = myG3D->GetTextureManager ();
-    int col_bg = txtmgr->FindRGB (200, 180, 180);
-    int col_fgdata = txtmgr->FindRGB (20, 70, 20);
-    int col_bgsel = txtmgr->FindRGB (255, 255, 255);
-    int col_fgsel = txtmgr->FindRGB (0, 0, 0);
+    int col_bg = myG2D->FindRGB (200, 180, 180);
+    int col_fgdata = myG2D->FindRGB (20, 70, 20);
+    int col_bgsel = myG2D->FindRGB (255, 255, 255);
+    int col_fgsel = myG2D->FindRGB (0, 0, 0);
     myG2D->Clear (col_bg);
     int tx = 10;
     int ty = 10;
@@ -467,7 +429,7 @@ void Demo::SetupFrame ()
     GfxWrite (tx, ty, col_black, col_bg,
     	"Download 'demodata.zip' from"); ty += 10;
     GfxWrite (tx, ty, col_fgdata, col_bg,
-    	"    ftp://sunsite.dk/projects/crystal/cs094/levels/demodata.zip"); ty += 10;
+    	"    http://www.crystalspace3d.org/support/release/cs098/maps/demodata.zip"); ty += 10;
 
     ty += 10;
 
@@ -500,7 +462,7 @@ void Demo::SetupFrame ()
     	"You can select one to run that demo or press ESC to exit this program."); ty += 10;
       ty += 10;
       first_y = ty;
-      int i;
+      size_t i;
       for (i = 0 ; i < demos.Length () ; i++)
       {
 	int bg = col_bg;
@@ -649,17 +611,17 @@ void Demo::SetupFrame ()
       break;
   }
 
-  if (message[0])
+  if (!message.IsEmpty())
   {
     GfxWrite (10, 10, col_black, message_error ? col_red : col_white, message);
-    if (current_time > message_timer) message[0] = 0;
+    if (current_time > message_timer) message.Empty();
   }
 }
 
 void Demo::FinishFrame ()
 {
   myG3D->FinishDraw ();
-  myG3D->Print (NULL);
+  myG3D->Print (0);
 }
 
 void Demo::DrawEditInfo ()
@@ -688,7 +650,7 @@ void Demo::DrawEditInfo ()
     GfxWrite (ww, hh, col_black, col_white, "U(%.2g,%.2g,%.2g)",
     	up.x, up.y, up.z); hh += fh;
     float t = np->GetTimeValue (map_selpoint);
-    csTicks tms = int (t*total);
+    csTicks tms = csTicks (t*total);
     GfxWrite (ww, hh, col_black, col_white, "tot time %d ms", total); hh += fh;
     GfxWrite (ww, hh, col_black, col_white, "rel time %d ms", tms); hh += fh;
     GfxWrite (ww, hh, col_black, col_white, "Left Path Info:"); hh += fh;
@@ -696,11 +658,11 @@ void Demo::DrawEditInfo ()
     {
       csVector3 v1;
       np->GetPositionVector (map_selpoint-1, v1);
-      float d = qsqrt (csSquaredDist::PointPoint (v, v1));
+      float d = csQsqrt (csSquaredDist::PointPoint (v, v1));
       float t1 = np->GetTimeValue (map_selpoint-1);
       float dr = t-t1;
       float speed = (float) fabs (dr) / d;
-      csTicks tms1 = int (t1*total);
+      csTicks tms1 = csTicks (t1*total);
       GfxWrite (ww+20, hh, col_black, col_white, "len %g", d); hh += fh;
       GfxWrite (ww+20, hh, col_black, col_white, "dr %g", dr); hh += fh;
       GfxWrite (ww+20, hh, col_black, col_white, "speed %g", speed); hh += fh;
@@ -708,15 +670,15 @@ void Demo::DrawEditInfo ()
       	tms-tms1); hh += fh;
     }
     GfxWrite (ww, hh, col_black, col_white, "Right Path Info:"); hh += fh;
-    if (map_selpoint < np->GetPointCount ()-1)
+    if (map_selpoint < np->Length ()-1)
     {
       csVector3 v1;
       np->GetPositionVector (map_selpoint+1, v1);
       float t1 = np->GetTimeValue (map_selpoint+1);
       float dr = t1-t;
-      float d = qsqrt (csSquaredDist::PointPoint (v, v1));
+      float d = csQsqrt (csSquaredDist::PointPoint (v, v1));
       float speed = (float) fabs (dr) / d;
-      csTicks tms1 = int (t1*total);
+      csTicks tms1 = csTicks (t1*total);
       GfxWrite (ww+20, hh, col_black, col_white, "len %g", d); hh += fh;
       GfxWrite (ww+20, hh, col_black, col_white, "dr %g", dr); hh += fh;
       GfxWrite (ww+20, hh, col_black, col_white, "speed %g", speed); hh += fh;
@@ -728,26 +690,26 @@ void Demo::DrawEditInfo ()
 
 bool Demo::DemoHandleEvent (iEvent &Event)
 {
-  if (Event.Type == csevKeyDown)
+  if (Event.Name == KeyboardDown)
   {
     csTicks elapsed_time, current_time;
     elapsed_time = vc->GetElapsedTicks ();
     current_time = vc->GetCurrentTicks ();
-    bool shift = (Event.Key.Modifiers & CSMASK_SHIFT) != 0;
-    bool alt = (Event.Key.Modifiers & CSMASK_ALT) != 0;
-    bool ctrl = (Event.Key.Modifiers & CSMASK_CTRL) != 0;
+    csKeyModifiers m;
+    csKeyEventHelper::GetModifiers (&Event, m);
+    bool shift = m.modifiers[csKeyModifierTypeShift] != 0;
+    bool alt = m.modifiers[csKeyModifierTypeAlt] != 0;
+    bool ctrl = m.modifiers[csKeyModifierTypeCtrl] != 0;
+    utf32_char keyCode = csKeyEventHelper::GetCookedCode (&Event);
 
 #if 0
     if (do_demo != 3)
     {
-      if (Event.Key.Code == CSKEY_ESC)
+      if (keyCode == CSKEY_ESC)
       {
-	iEventQueue* q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+	csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
 	if (q)
-	{
-	  q->GetEventOutlet()->Broadcast (cscmdQuit);
-	  q->DecRef ();
-	}
+	  q->GetEventOutlet()->Broadcast (csevQuit (object_reg));
         return true;
       }
     }
@@ -766,7 +728,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
         if (shift) speed = dx / 20.0f;
         else if (ctrl) speed = dx / 600.0f;
         else speed = dx / 100.0f;
-        if (Event.Key.Code == CSKEY_UP)
+        if (keyCode == CSKEY_UP)
         {
           csVector3 v;
 	  np->GetPositionVector (map_selpoint, v);
@@ -775,7 +737,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  ShowMessage ("Y location set at '%g'", v.y);
           return true;
         }
-        if (Event.Key.Code == CSKEY_DOWN)
+        if (keyCode == CSKEY_DOWN)
         {
           csVector3 v;
 	  np->GetPositionVector (map_selpoint, v);
@@ -784,7 +746,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  ShowMessage ("Y location set at '%g'", v.y);
           return true;
         }
-        if (Event.Key.Code == CSKEY_LEFT)
+        if (keyCode == CSKEY_LEFT)
         {
           csVector3 up, forward;
 	  np->GetUpVector (map_selpoint, up);
@@ -797,7 +759,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  ShowMessage ("Up vector set at '%.3g,%.3g,%.3g'", up.x, up.y, up.z);
           return true;
         }
-        if (Event.Key.Code == CSKEY_RIGHT)
+        if (keyCode == CSKEY_RIGHT)
         {
           csVector3 up, forward;
 	  np->GetUpVector (map_selpoint, up);
@@ -811,7 +773,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
           return true;
         }
       }
-      switch (Event.Key.Char)
+      switch (keyCode)
       {
         case 'c':
           map_enabled = MAP_EDIT;
@@ -821,7 +783,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  // with the previous and next point.
 	  // Make the forward vector look along the path. i.e. let it look
 	  // to an average direction as specified by next and previous point.
-	  if (map_selpoint <= 0 || map_selpoint >= np->GetPointCount ()-1)
+	  if (map_selpoint <= 0 || map_selpoint >= np->Length ()-1)
 	  {
 	    ShowMessage ("The 'y' operation can't work on this point!\n");
 	  }
@@ -955,7 +917,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
       else speed = dx / 100.0f;
       if (np)
       {
-        if (Event.Key.Code == CSKEY_UP)
+        if (keyCode == CSKEY_UP)
         {
 	  if (alt)
 	  {
@@ -971,7 +933,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  }
           return true;
         }
-        if (Event.Key.Code == CSKEY_DOWN)
+        if (keyCode == CSKEY_DOWN)
         {
 	  if (alt)
 	  {
@@ -987,7 +949,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  }
           return true;
         }
-        if (Event.Key.Code == CSKEY_LEFT)
+        if (keyCode == CSKEY_LEFT)
         {
 	  if (alt)
 	  {
@@ -1003,7 +965,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  }
           return true;
         }
-        if (Event.Key.Code == CSKEY_RIGHT)
+        if (keyCode == CSKEY_RIGHT)
         {
 	  if (alt)
 	  {
@@ -1020,7 +982,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
           return true;
         }
       }
-      switch (Event.Key.Char)
+      switch (keyCode)
       {
 	case 'm':
           map_enabled = MAP_OFF;
@@ -1035,20 +997,19 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	    strcpy (backup, buf);
 	    strcat (backup, ".bak");
 	    myVFS->DeleteFile (backup);
-	    iDataBuffer* dbuf = myVFS->ReadFile (buf);
+	    csRef<iDataBuffer> dbuf (myVFS->ReadFile (buf));
 	    if (dbuf)
 	    {
 	      if (dbuf->GetSize ())
 	        myVFS->WriteFile (backup, **dbuf, dbuf->GetSize ());
-	      dbuf->DecRef ();
 	    }
 
-	    iFile* fp = myVFS->Open (buf, VFS_FILE_WRITE);
+	    csRef<iFile> fp (myVFS->Open (buf, VFS_FILE_WRITE));
 	    if (fp)
 	    {
 	      int i, num = np->GetPointCount ();
 	      FileWrite (fp, "    NUM (%d)\n", num);
-	      float* t = np->GetTimeValues ();
+	      const float* t = np->GetTimeValues ();
 	      FileWrite (fp, "    TIMES (%g", t[0]);
 	      for (i = 1 ; i < num ; i++)
 	        FileWrite (fp, ",%g", t[i]);
@@ -1077,7 +1038,6 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	        FileWrite (fp, "      V (%g,%g,%g)\n", v.x, v.y, v.z);
 	      }
 	      FileWrite (fp, "    )\n");
-	      fp->DecRef ();
 	      ShowMessage ("Wrote path to file '%s'", buf);
 	    }
 	    else
@@ -1098,8 +1058,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	      np->SetUpVector (map_selpoint, v);
 	      np->GetForwardVector (map_selpoint-1, v);
 	      np->SetForwardVector (map_selpoint, v);
-	      np->SetTimeValue (map_selpoint,
-	    	  np->GetTimeValue (map_selpoint-1));
+	      np->SetTime (map_selpoint, np->GetTime (map_selpoint-1));
 	    }
 	    else
 	    {
@@ -1113,9 +1072,9 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	      np->GetForwardVector (map_selpoint-1, v1);
 	      np->GetForwardVector (map_selpoint+1, v2);
 	      np->SetForwardVector (map_selpoint, (v1+v2)/2.);
-	      np->SetTimeValue (map_selpoint,
-	    	  (np->GetTimeValue (map_selpoint-1)+
-		   np->GetTimeValue (map_selpoint+1)) / 2.0f);
+	      np->SetTime (map_selpoint,
+	    	  (np->GetTime (map_selpoint-1)+
+		   np->GetTime (map_selpoint+1)) / 2.0f);
 	    }
 	  }
           break;
@@ -1141,7 +1100,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	      else dt /= 50.;
 	      t -= dt;
 	      if (t < t1) t = t1;
-	      np->SetTimeValue (map_selpoint, t);
+	      np->SetTime (map_selpoint, t);
 	    }
 	  }
 	  break;
@@ -1175,7 +1134,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  if (np)
 	  {
 	    int num = np->GetPointCount ();
-	    float* xv, * yv, * zv;
+	    const float* xv, * yv, * zv;
 	    xv = np->GetDimensionValues (0);
 	    yv = np->GetDimensionValues (1);
 	    zv = np->GetDimensionValues (2);
@@ -1187,7 +1146,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	    for (i = 1 ; i < num ; i++)
 	    {
 	      v1.Set (xv[i], yv[i], zv[i]);
-	      float d = qsqrt (csSquaredDist::PointPoint (v0, v1));
+	      float d = csQsqrt (csSquaredDist::PointPoint (v0, v1));
 	      totlen += d;
 	      v0 = v1;
 	    }
@@ -1200,7 +1159,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	    for (i = 1 ; i < num ; i++)
 	    {
 	      v1.Set (xv[i], yv[i], zv[i]);
-	      float d = qsqrt (csSquaredDist::PointPoint (v0, v1));
+	      float d = csQsqrt (csSquaredDist::PointPoint (v0, v1));
 	      tot += d;
 	      list[i] = tot / totlen;
 	      v0 = v1;
@@ -1281,17 +1240,14 @@ bool Demo::DemoHandleEvent (iEvent &Event)
       //==============================
       // Handle keys in demo or overlay mode.
       //==============================
-      if (Event.Key.Code == CSKEY_ESC)
+      if (keyCode == CSKEY_ESC)
       {
-	iEventQueue* q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+	csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
 	if (q)
-	{
-	  q->GetEventOutlet()->Broadcast (cscmdQuit);
-	  q->DecRef ();
-	}
+	  q->GetEventOutlet()->Broadcast (csevQuit (object_reg));
         return true;
       }
-      switch (Event.Key.Char)
+      switch (keyCode)
       {
       	case 'R':
 	  ShowMessage ("Restarted sequence manager");
@@ -1327,20 +1283,20 @@ bool Demo::DemoHandleEvent (iEvent &Event)
       }
     }
   }
-  else if (Event.Type == csevMouseDown)
+  else if (Event.Name == MouseDown)
   {
     if (do_demo == 0)
     {
-      selected_demo = (Event.Mouse.y - first_y) / 10;
-      if (selected_demo >= 0 && selected_demo < demos.Length ())
-        do_demo = 1;
+      selected_demo = (csMouseEventHelper::GetY(&Event) - first_y) / 10;
+      if ((selected_demo != (size_t)-1) && selected_demo < demos.Length ())
+	do_demo = 1;
     }
     else if (do_demo < 3)
     {
     }
-    else if (Event.Mouse.Button == 1)
+    else if (csMouseEventHelper::GetButton(&Event) == 1)
     {
-      csVector2 p (Event.Mouse.x, myG2D->GetHeight ()-Event.Mouse.y);
+      csVector2 p (csMouseEventHelper::GetX(&Event), myG2D->GetHeight ()-csMouseEventHelper::GetY(&Event));
       csVector3 v;
       view->GetCamera ()->InvPerspective (p, 1, v);
       csVector3 vw = view->GetCamera ()->GetTransform ().This2Other (v);
@@ -1349,18 +1305,18 @@ bool Demo::DemoHandleEvent (iEvent &Event)
         csNamedPath* np = seqmgr->GetSelectedPath (map_selpath);
 	if (np)
 	{
-          vw -= view->GetCamera ()->GetTransform ().GetOrigin ();
-          np->SetForwardVector (map_selpoint, vw);
+	  vw -= view->GetCamera ()->GetTransform ().GetOrigin ();
+	  np->SetForwardVector (map_selpoint, vw);
 	  csVector3 up;
 	  np->GetUpVector (map_selpoint, up);
 	  up = up % vw;
 	  up = - (up % vw);
 	  np->SetUpVector (map_selpoint, up);
-        }
+	}
       }
       else if (map_enabled == MAP_EDIT)
       {
-        p.y = Event.Mouse.y;
+	p.y = csMouseEventHelper::GetY(&Event);
 	int dim = myG2D->GetHeight ()-10;
 	float dx = (map_br.x-map_tl.x) / 2.0f;
 	float dy = (map_br.y-map_tl.y) / 2.0f;
@@ -1373,13 +1329,13 @@ bool Demo::DemoHandleEvent (iEvent &Event)
       }
     }
   }
-  else if (Event.Type == csevMouseMove)
+  else if (Event.Name == MouseMove)
   {
     if (do_demo == 0)
     {
-      selected_demo = (Event.Mouse.y - first_y) / 10;
-      if (!(selected_demo >= 0 && selected_demo < demos.Length ()))
-        selected_demo = -1;
+      selected_demo = (csMouseEventHelper::GetY(&Event) - first_y) / 10;
+      if (!((selected_demo != (size_t)-1) && selected_demo < demos.Length ()))
+        selected_demo = (size_t)-1;
     }
   }
 
@@ -1391,7 +1347,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
  *---------------------------------------------------------------------*/
 int main (int argc, char* argv[])
 {
-  srand (time (NULL));
+  srand (time (0));
 
   // Create our main class.
   System = new Demo ();
