@@ -29,7 +29,7 @@
 #include "iutil/databuff.h"
 #include "iutil/vfs.h"
 #include "csutil/xmltiny.h"
-#include "qsqrt.h"
+#include "csqsqrt.h"
 
 // Define all tokens used through this file
 enum
@@ -109,7 +109,7 @@ DemoSequenceLoader::DemoSequenceLoader (Demo* demo,
   	CS_QUERY_REGISTRY (demo->object_reg, iDocumentSystem));
   if (!xml) xml = csPtr<iDocumentSystem> (new csTinyDocumentSystem ());
   csRef<iDocument> doc = xml->CreateDocument ();
-  const char* error = doc->Parse (buf);
+  const char* error = doc->Parse (buf, true);
   if (error != 0)
   {
     demo->Report (CS_REPORTER_SEVERITY_ERROR,
@@ -125,7 +125,7 @@ DemoSequenceLoader::~DemoSequenceLoader ()
 
 iSequence* DemoSequenceLoader::GetSequence (const char* name)
 {
-  int i;
+  size_t i;
   for (i = 0 ; i < sequences.Length () ; i++)
   {
     NamedSequence* ns = sequences[i];
@@ -517,7 +517,7 @@ csNamedPath* DemoSequenceLoader::LoadPath (iDocumentNode* node,
 		CS_QUERY_REGISTRY (demo->object_reg, iDocumentSystem));
 	if (!xml) xml = csPtr<iDocumentSystem> (new csTinyDocumentSystem ());
 	csRef<iDocument> doc = xml->CreateDocument ();
-	const char* error = doc->Parse (buf);
+	const char* error = doc->Parse (buf, true);
 	if (error != 0)
 	{
 	  demo->Report (CS_REPORTER_SEVERITY_ERROR,
@@ -595,7 +595,7 @@ csNamedPath* DemoSequenceLoader::LoadPath (iDocumentNode* node,
 	for (i = 1 ; i < num ; i++)
 	{
 	  v1.Set (xv[i], yv[i], zv[i]);
-	  float d = qsqrt (csSquaredDist::PointPoint (v0, v1));
+	  float d = csQsqrt (csSquaredDist::PointPoint (v0, v1));
 	  tot += (1/list[i]) * d;
 	  list[i] = tot;
 	  v0 = v1;
@@ -632,7 +632,7 @@ csNamedPath* DemoSequenceLoader::LoadPath (iDocumentNode* node,
 	for (i = 1 ; i < num ; i++)
 	{
 	  v1.Set (xv[i], yv[i], zv[i]);
-	  float d = qsqrt (csSquaredDist::PointPoint (v0, v1));
+	  float d = csQsqrt (csSquaredDist::PointPoint (v0, v1));
 	  totlen += d;
 	  v0 = v1;
 	}
@@ -646,7 +646,7 @@ csNamedPath* DemoSequenceLoader::LoadPath (iDocumentNode* node,
 	for (i = 1 ; i < num ; i++)
 	{
 	  v1.Set (xv[i], yv[i], zv[i]);
-	  float d = qsqrt (csSquaredDist::PointPoint (v0, v1));
+	  float d = csQsqrt (csSquaredDist::PointPoint (v0, v1));
 	  tot += d;
 	  list[i] = tot / totlen;
 	  v0 = v1;
