@@ -32,8 +32,7 @@
 #include "iengine/camera.h"
 #include "iengine/mesh.h"
 #include "iengine/movable.h"
-#include "imesh/thing/polygon.h"
-#include "imesh/thing/thing.h"
+#include "imesh/thing.h"
 #include "imesh/object.h"
 #include "imesh/particle.h"
 #include "ivaria/reporter.h"
@@ -55,10 +54,10 @@ DemoSequenceManager::DemoSequenceManager (Demo* demo)
   DemoSequenceManager::demo = demo;
   demoseq = this;
   iObjectRegistry* object_reg = demo->object_reg;
-  iPluginManager* plugin_mgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
+  csRef<iPluginManager> plugin_mgr (
+  	CS_QUERY_REGISTRY (object_reg, iPluginManager));
   seqmgr = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.utilities.sequence",
   	iSequenceManager);
-  plugin_mgr->DecRef ();
   if (!seqmgr)
   {
     demo->Report (CS_REPORTER_SEVERITY_ERROR,
@@ -70,49 +69,25 @@ DemoSequenceManager::DemoSequenceManager (Demo* demo)
   fade_value = 0;
   suspended = true;
   suspend_one_frame = false;
-  main_sequence = NULL;
 }
 
 DemoSequenceManager::~DemoSequenceManager ()
 {
   Clear ();
-  int i;
-  for (i = 0 ; i < paths.Length () ; i++)
-  {
-    csNamedPath* np = (csNamedPath*)paths[i];
-    delete np;
-  }
-  paths.DeleteAll ();
-  if (seqmgr) seqmgr->DecRef ();
-  if (main_sequence) main_sequence->DecRef ();
 }
 
 void DemoSequenceManager::Clear ()
 {
   seqmgr->Clear ();
-  int i;
-  for (i = 0 ; i < pathForMesh.Length () ; i++)
-  {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
-    delete pfm;
-  }
   pathForMesh.DeleteAll ();
-  for (i = 0 ; i < meshRotation.Length () ; i++)
-  {
-    MeshRotation* mrot = (MeshRotation*)meshRotation[i];
-    if (mrot->particle) mrot->particle->DecRef ();
-    delete mrot;
-  }
   meshRotation.DeleteAll ();
 }
 
 void DemoSequenceManager::Setup (const char* sequenceFileName)
 {
-  if (main_sequence) main_sequence->DecRef ();
   DemoSequenceLoader* loader = new DemoSequenceLoader (
   	DemoSequenceManager::demo, this, seqmgr, sequenceFileName);
   main_sequence = loader->GetSequence ("main");
-  main_sequence->IncRef ();
   seqmgr->RunSequence (0, main_sequence);
   seqmgr->Resume ();
   suspended = false;
@@ -142,14 +117,8 @@ void DemoSequenceManager::Resume ()
 void DemoSequenceManager::Restart (const char* sequenceFileName)
 {
   Clear ();
-  int i;
-  for (i = 0 ; i < paths.Length () ; i++)
-  {
-    csNamedPath* np = (csNamedPath*)paths[i];
-    delete np;
-  }
   paths.DeleteAll ();
-  if (main_sequence) { main_sequence->DecRef (); main_sequence = NULL; }
+  main_sequence = 0;
   do_fade = false;
   fade_value = 0;
   suspended = true;
@@ -248,10 +217,10 @@ void DemoSequenceManager::SetupFade (float start_fade, float end_fade,
 void DemoSequenceManager::ReplacePathObject (csNamedPath* path,
 	iMeshWrapper* mesh)
 {
-  int i;
+  size_t i;
   for (i = 0 ; i < pathForMesh.Length () ; i++)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     if (pfm->path == path)
     {
       pfm->mesh = mesh;
@@ -277,11 +246,11 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
 {
   if (suspended) return;
   csTicks current_time = seqmgr->GetMainTime ();
-  int i = 0;
-  int len = pathForMesh.Length ();
+  size_t i = 0;
+  size_t len = pathForMesh.Length ();
   while (i < len)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     float r = float (current_time - pfm->start_path_time)
     	/ float (pfm->total_path_time);
     bool do_path = true;
@@ -326,7 +295,6 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
         movable->SetSector (newsector);
       movable->GetTransform ().LookAt (forward.Unit (), up.Unit ());
       movable->UpdateMove ();
-      pfm->mesh->DeferUpdateLighting (CS_NLIGHT_STATIC|CS_NLIGHT_DYNAMIC, 10);
     }
     else
     {
@@ -337,8 +305,7 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
     }
     if (!do_path)
     {
-      delete pfm;
-      pathForMesh.Delete (i);
+      pathForMesh.DeleteIndex (i);
       len--;
     }
     else i++;
@@ -348,12 +315,10 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
   len = meshRotation.Length ();
   while (i < len)
   {
-    MeshRotation* mrot = (MeshRotation*)meshRotation[i];
+    MeshRotation* mrot = meshRotation[i];
     if (current_time > mrot->start_time + mrot->total_time)
     {
-      if (mrot->particle) mrot->particle->DecRef ();
-      delete mrot;
-      meshRotation.Delete (i);
+      meshRotation.DeleteIndex (i);
       len--;
     }
     else
@@ -369,11 +334,11 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
 void DemoSequenceManager::DebugPositionObjects (iCamera* camera,
     csTicks debug_time)
 {
-  int i = 0;
-  int len = pathForMesh.Length ();
+  size_t i = 0;
+  size_t len = pathForMesh.Length ();
   while (i < len)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     float r = float (debug_time - pfm->start_path_time)
     	/ float (pfm->total_path_time);
     if (r >= 0 && r <= 1)
@@ -389,7 +354,6 @@ void DemoSequenceManager::DebugPositionObjects (iCamera* camera,
         movable->SetPosition (pos);
         movable->GetTransform ().LookAt (forward.Unit (), up.Unit ());
         movable->UpdateMove ();
-        pfm->mesh->DeferUpdateLighting (CS_NLIGHT_STATIC|CS_NLIGHT_DYNAMIC, 10);
       }
       else
       {
@@ -418,16 +382,16 @@ void DemoSequenceManager::DebugDrawPath (csNamedPath* np, bool hi,
     if (x > 0 && x < dim && y > 0 && y < dim)
       demo->myG2D->DrawPixel (x, y, col);
   }
-  float* px, * py, * pz;
+  const float* px, * py, * pz;
   px = np->GetDimensionValues (0);
   py = np->GetDimensionValues (1);
   pz = np->GetDimensionValues (2);
-  float* fx, * fy, * fz;
+  const float* fx, * fy, * fz;
   fx = np->GetDimensionValues (6);
   fy = np->GetDimensionValues (7);
   fz = np->GetDimensionValues (8);
   int j;
-  for (j = 0 ; j < np->GetPointCount () ; j++)
+  for (j = 0 ; j < np->Length () ; j++)
   {
     int col = demo->col_red;
     if (hi && selpoint == j) col = demo->col_green;
@@ -481,8 +445,8 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
 	const char* hilight, const csVector2& tl, const csVector2& br,
 	int selpoint)
 {
-  int i;
-  int len = pathForMesh.Length ();
+  size_t i;
+  size_t len = pathForMesh.Length ();
   csTicks current_time = seqmgr->GetMainTime ();
 
   //=====
@@ -506,7 +470,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
   // Get the current selected path.
   //=====
   csTicks start = 0, total = 0, seltime = 0;
-  csNamedPath* selnp = NULL;
+  csNamedPath* selnp = 0;
   if (hilight) selnp = GetSelectedPath (hilight, start, total);
   if (selnp)
   {
@@ -521,7 +485,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
   i = 0;
   while (i < len)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     bool hi = (pfm->path == selnp);
     DebugDrawPath (pfm->path, hi, tl, br, selpoint);
     i++;
@@ -535,7 +499,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
   i = 0;
   while (i < len)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     bool hi = (pfm->path == selnp);
 
     // Fetch the current time.
@@ -577,7 +541,7 @@ void DemoSequenceManager::SelectFirstPath (char* hilight)
 {
   if (pathForMesh.Length () > 0)
   {
-    strcpy (hilight, ((PathForMesh*)pathForMesh[0])->path->GetName ());
+    strcpy (hilight, pathForMesh[0]->path->GetName ());
   }
 }
 
@@ -585,8 +549,7 @@ void DemoSequenceManager::SelectLastPath (char* hilight)
 {
   if (pathForMesh.Length () > 0)
   {
-    strcpy (hilight, ((PathForMesh*)pathForMesh[
-      pathForMesh.Length ()-1])->path->GetName ());
+    strcpy (hilight, pathForMesh[pathForMesh.Length ()-1]->path->GetName ());
   }
 }
 
@@ -598,17 +561,17 @@ void DemoSequenceManager::SelectPreviousPath (char* hilight)
     SelectLastPath (hilight);
     return;
   }
-  int i;
+  size_t i;
   for (i = 0 ; i < pathForMesh.Length () ; i++)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     if (pfm->path == np)
     {
       if (i == 0)
 	return;
       else
       {
-        pfm = (PathForMesh*)pathForMesh[i-1];
+        pfm = pathForMesh[i-1];
 	strcpy (hilight, pfm->path->GetName ());
       }
       return;
@@ -627,15 +590,15 @@ void DemoSequenceManager::SelectNextPath (char* hilight)
     SelectFirstPath (hilight);
     return;
   }
-  int i;
+  size_t i;
   for (i = 0 ; i < pathForMesh.Length () ; i++)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     if (pfm->path == np)
     {
       if (i < pathForMesh.Length ()-1)
       {
-        pfm = (PathForMesh*)pathForMesh[i+1];
+        pfm = pathForMesh[i+1];
 	strcpy (hilight, pfm->path->GetName ());
       }
       return;
@@ -655,11 +618,11 @@ csNamedPath* DemoSequenceManager::GetSelectedPath (const char* hilight)
 csNamedPath* DemoSequenceManager::GetSelectedPath (const char* hilight,
 	csTicks& start, csTicks& total)
 {
-  int i = 0;
-  int len = pathForMesh.Length ();
+  size_t i = 0;
+  size_t len = pathForMesh.Length ();
   while (i < len)
   {
-    PathForMesh* pfm = (PathForMesh*)pathForMesh[i];
+    PathForMesh* pfm = pathForMesh[i];
     csNamedPath* np = pfm->path;
     bool hi = (hilight && !strcmp (np->GetName (), hilight));
     if (hi)
@@ -670,7 +633,7 @@ csNamedPath* DemoSequenceManager::GetSelectedPath (const char* hilight,
     }
     i++;
   }
-  return NULL;
+  return 0;
 }
 
 void DemoSequenceManager::SetupRotatePart (iMeshWrapper* mesh,
