@@ -20,6 +20,8 @@
 */
 
 #include "cssysdef.h"
+#include "csutil/csstring.h"
+
 #include "mapstd.h"
 #include "brush.h"
 #include "mpoly.h"
@@ -34,22 +36,25 @@
 #include "csector.h"
 #include "cworld.h"
 
+#include "dochelp.h"
 
-
-static char* BuildMaterialKey (int i, CMapEntity* pEntity)
+namespace
 {
-  char* key = new char[1024];
-  char* returnvalue = new char[1024];
-  sprintf(key,"MaterialName%d",i);
-  printf(key);
-  if(!pEntity->GetValueOfKey(key)) return NULL;
-  sprintf(returnvalue,"KEY(\"defmaterial\",\"%s\")\n",
+
+char* BuildMaterialKey (int i, CMapEntity* pEntity)
+{
+  csString key;
+  csString returnvalue;
+  key.Format ("MaterialName%d",i);
+  csPrintf("%s", key.GetData());
+  if(!pEntity->GetValueOfKey(key)) return 0;
+  returnvalue.Format ("<key name=\"defmaterial\" value=\"%s\" />\n",
 	      pEntity->GetValueOfKey(key));
-  return returnvalue;
+  return csStrNew (returnvalue);
 }
 
 //Auxiliar funtion to check if the file name is a mdl file
-static bool isMdlFile(const char* csmodelfile)
+bool isMdlFile(const char* csmodelfile)
 {
   int i = 0;
   bool dot = false;
@@ -68,10 +73,10 @@ static bool isMdlFile(const char* csmodelfile)
   if(!strcmp(extension,"mDl")) return true;
   if(!strcmp(extension,"Mdl")) return true;
   return false;
-};
+}
 
 //Auxiliar Function to contruct the Factory name of a cs_model
-static char* GetFactoryName(const char * csmodelfile)
+char* GetFactoryName(const char * csmodelfile)
 {
   /*
    * Searchs for the extension and substitutes ".extension" by "Fact" and
@@ -106,6 +111,8 @@ static char* GetFactoryName(const char * csmodelfile)
   return aux;
 }
 
+} // end of anonymous namespace
+
 
 CCSSector::CCSSector(CMapBrush* pBrush)
   : CISector(pBrush)
@@ -116,173 +123,160 @@ CCSSector::~CCSSector()
 {
 }
 
-bool CCSSector::Write(CIWorld* pIWorld)
+bool CCSSector::Write(csRef<iDocumentNode> node, CIWorld* pIWorld)
 {
   assert(pIWorld);
   CCSWorld* pWorld  = (CCSWorld*)  pIWorld;
 
   CMapFile* pMap = pWorld->GetMap();
-  FILE*     fd   = pWorld->GetFile();
 
   assert(pMap);
-  assert(fd);
 
-  pWorld->WriteIndent();
-  fprintf(fd, "SECTOR '%s' (\n", GetName());
-  pWorld->Indent();
-
-  pWorld->WriteIndent();
-  fprintf(fd, "MESHOBJ 'static'(\n");
-  pWorld->Indent();
-
-  pWorld->WriteIndent();
-  fprintf(fd, "PLUGIN('thing')\n");
-  pWorld->WriteIndent();
-  fprintf(fd, "ZFILL()\n");
-  pWorld->WriteIndent();
-  fprintf(fd, "PRIORITY('wall')\n");
-
-  CMapEntity* pEntity = m_pOriginalBrush->GetEntity();
-  CCSWorld::WriteKeys(pWorld, pEntity);
-
-  pWorld->WriteIndent();
-  fprintf(fd, "PARAMS(\n");
-  pWorld->Indent();
-
-  if (pMap->GetConfigInt("Map2CS.General.UseBSP", 0))
+  DocNode sector = CreateNode (node, "sector");
+  sector->SetAttribute ("name", GetName());
+  CMapEntity* worldspawn = pWorld->GetWorldspawn ();
+  if (worldspawn->GetBoolValueOfKey ("dynavis"))
   {
-    pWorld->WriteIndent();
-    fprintf(fd, "VISTREE()\n");
+    CreateNode (sector, "cullerp", "crystalspace.culling.dynavis");
   }
 
-  if (m_IsDefaultsector && pWorld->NeedSkysector())
+  /*if (m_IsDefaultsector && pWorld->NeedSkysector())
   {
-    /*pWorld->WriteIndent();
-    fprintf(fd, "PART 'p1' (\n");
-    pWorld->Indent();*/
-
-    pWorld->WriteSky();
-
-    /*pWorld->Unindent();
-    pWorld->WriteIndent();
-    fprintf(fd, ")\n");*/ //PART
+    pWorld->WriteSky(node);
   }
-  else
+  else*/
+
+  if (m_IsDefaultsector && 
+    !worldspawn->GetBoolValueOfKey ("skyportalsonly", false))
   {
-    if (m_Walls.Length() > 0 ||
-        m_Portals.Length() > 0)
+    bool hasSky = worldspawn->GetValueOfKey("skybox") || 
+      worldspawn->GetValueOfKey("skydome");
+
+    if (hasSky)
     {
-      pWorld->WriteIndent();
-      fprintf(fd, "PART 'p1' (\n");
-      pWorld->Indent();
+      DocNode meshobj = CreateNode (sector, "meshobj");
+      meshobj->SetAttribute ("name", "_defaultsky");
 
-      int i, j, l;
+      CreateNode (meshobj, "plugin", "thing");
+      CreateNode (meshobj, "zfill");
+      CreateNode (meshobj, "priority", "sky");
+      CreateNode (meshobj, "camera");
 
-      CVertexBuffer Vb;
-      Vb.AddVertices(&m_Walls);
-      Vb.AddVertices(&m_Portals);
-      Vb.WriteCS(pWorld);
+      DocNode params = CreateNode (meshobj, "params");
 
-      for (i=0; i<m_Walls.Length(); i++)
+      pWorld->WriteSky (params);
+    }
+  }
+
+  {
+    if (m_Portals.Length() > 0)
+    {
+      CMapEntity* pEntity = m_pOriginalBrush->GetEntity();
+
+      DocNode meshobj = CreateNode (sector, "portals");
+      csString meshname;
+      meshname.Format ("%s_portals", GetName());
+      if (pEntity)
       {
-        CMapPolygonSet* pPolySet = m_Walls[i];
-        for (j=0; j<pPolySet->GetPolygonCount(); j++)
-        {
-          CMapPolygon* pPolygon = pPolySet->GetPolygon(j);
-          pWorld->WritePolygon(pPolygon, this, true, Vb);
-        }
+	meshname = pEntity->GetValueOfKey ("cs_name", meshname);
       }
 
+      size_t i;
+      size_t j;
+
       for (i=0; i<m_Portals.Length(); i++)
       {
         CIPortal* pPortal = m_Portals[i];
         for (j=0; j<pPortal->GetPolygonCount(); j++)
         {
-          CMapPolygon*             pPolygon = pPortal->GetPolygon(j);
-          const CMapTexturedPlane* pPlane   = pPortal->GetBaseplane();
+	  csString portalName;
+	  portalName.Format ("%s_%zu_%zu", meshname.GetData(), i, j);
 
-          CTextureFile* pTexture = pPlane->GetTexture();
-
-          //Because for a sector we draw the _inside_ of the brush, we spit out the
-          //vertices in reverse order, so they will have proper orientation for
-          //backface culling in the engine.
-          pWorld->WriteIndent();
-          fprintf(fd, "POLYGON '' ( VERTICES (");
-          for (l=pPolygon->GetVertexCount()-1; l>=0; l--)
-          {
-              fprintf(fd, "%d%s", Vb.GetIndex(pPolygon->GetVertex(l)),
-                                  ((l==0) ? "" : ","));
-          }
-          fprintf(fd, ") "); //End of Vertices
-
-          //print textureinfo
-          fprintf(fd, "MATERIAL('%s') ", pTexture->GetTexturename());
-          fprintf(fd, "TEXTURE(PLANE ('%s')) ", pPolygon->GetBaseplane()->GetName());
-          fprintf(fd, "PORTAL ('%s') ", pPortal->GetTargetSector()->GetName());
-          fprintf(fd, ")\n"); //End of Polygon
-        }
+	  DocNode        portal = CreateNode(meshobj, "portal");
+	  portal->SetAttribute ("name", portalName);
+ 	  CMapPolygon*   pPolygon = pPortal->GetPolygon(j);
+ 	  
+ 	  CVertexBuffer Vb;
+ 	  for (size_t l = pPolygon->GetVertexCount(); l-- > 0; ) 
+	  {
+ 	    Vb.AddVertex(pPolygon->GetVertex(l));
+ 	  }
+ 	  Vb.WriteCS(portal, pWorld);
+ 	  
+ 	  CreateNode(portal, "sector", pPortal->GetTargetSector()->GetName());        
+	}
       }
-
-      pWorld->Unindent();
-      pWorld->WriteIndent();
-      fprintf(fd, ")\n"); //PART
     } //if contains any polygons
   }
 
-  WriteWorldspawn(pWorld);
+  //WriteWorldspawn(params, pWorld);
+  WriteWorldspawn(sector, pWorld);
 
-  pWorld->Unindent();
-  pWorld->WriteIndent();
-  fprintf(fd, ")\n"); //End of PARAMS
+  WriteFog(sector, pWorld);
+  WriteLights(sector, pWorld);
+  WriteCurves(sector, pWorld);
+  WriteThings(sector, pWorld);
+  WriteSprites(sector, pWorld);
+  WriteSprites2D(sector, pWorld);
+  WriteNodes (sector, pWorld);
 
-  pWorld->Unindent();
-  pWorld->WriteIndent();
-  fprintf(fd, ")\n"); //End of MESHOBJ
+//  if (pMap->GetConfigInt("Map2CS.General.UseBSP", 0))
+//  {
+//    pWorld->WriteIndent();
+//    csFPrintf(fd, "CULLER('static')\n");
+//  }
 
-  WriteFog(pWorld);
-  WriteLights(pWorld);
-  WriteCurves(pWorld);
-  WriteThings(pWorld);
-  WriteSprites(pWorld);
-  WriteSprites2D(pWorld);
-  WriteNodes (pWorld);
-
-  if (pMap->GetConfigInt("Map2CS.General.UseBSP", 0))
-  {
-    pWorld->WriteIndent();
-    fprintf(fd, "CULLER('static')\n");
-  }
-
-  pWorld->Unindent();
-  pWorld->WriteIndent();
-  fprintf(fd, ")\n\n"); //End of SECTOR(
   return true;
 }
 
-bool CCSSector::WriteWorldspawn(CIWorld* pWorld)
+bool CCSSector::WriteWorldspawn(csRef<iDocumentNode> node, CIWorld* pWorld)
 {
-  int i;
+  size_t i;
   for (i=0; i<m_Things.Length(); i++)
   {
     if (strcasecmp(m_Things[i]->GetClassname(), "worldspawn")==0)
     {
-      if (!((CCSThing*)m_Things[i])->WriteAsPart(pWorld, this))
+      CMapEntity* pEntity = m_pOriginalBrush->GetEntity();
+
+      DocNode meshobj = CreateNode (node, "meshobj");
+      csString meshname;
+      meshname.Format ("%s_walls", GetName());
+      if (pEntity)
+      {
+	meshname = pEntity->GetValueOfKey ("cs_name", meshname);
+      }
+      meshobj->SetAttribute ("name", meshname);
+      CreateNode (meshobj, "plugin", "thing");
+      CreateNode (meshobj, "zuse");
+
+      CCSWorld::WriteKeys(meshobj, pWorld, pEntity);
+
+      DocNode params = CreateNode (meshobj, "params");
+
+      bool Sky;
+      if (!((CCSThing*)m_Things[i])->WriteAsPart(params, pWorld, this, Sky))
       {
         return false;
       }
+
+      DocNode priority = meshobj->CreateNodeBefore (CS_NODE_ELEMENT,
+	params);
+      priority->SetValue ("priority");
+      priority = priority->CreateNodeBefore (CS_NODE_TEXT);
+      priority->SetValue ("wall");
     }
   }
   return true;
 }
 
-bool CCSSector::WriteThings(CIWorld* pWorld)
+bool CCSSector::WriteThings(csRef<iDocumentNode> node, CIWorld* pWorld)
 {
-  int i;
+  size_t i;
   for (i=0; i<m_Things.Length(); i++)
   {
     if (strcasecmp(m_Things[i]->GetClassname(), "worldspawn")!=0)
     {
-      if (!m_Things[i]->Write(pWorld, this))
+      if (!m_Things[i]->Write(node, pWorld, this))
       {
         return false;
       }
@@ -291,25 +285,21 @@ bool CCSSector::WriteThings(CIWorld* pWorld)
   return true;
 }
 
-bool CCSSector::WriteLights(CIWorld* pWorld)
+bool CCSSector::WriteLights(csRef<iDocumentNode> node, CIWorld* pWorld)
 {
   assert(pWorld);
 
   CMapFile* pMap        = pWorld->GetMap();
-  FILE*     fd          = pWorld->GetFile();
   double    ScaleFactor = pWorld->GetScalefactor();
 
   assert(pMap);
-  assert(fd);
 
-  fprintf(fd, "\n");
-
-  int i;
+  size_t i;
 
   //iterate all entities, brushes, polygons and vertices:
   for (i=0; i<pMap->GetNumEntities(); i++)
   {
-    CMapEntity* pEntity = pMap->GetEntity(i);
+    CMapEntity* pEntity = pMap->GetEntity((int)i);
     if (strcmp(pEntity->GetClassname(), "light")==0)
     {
       CdVector3 origin;
@@ -323,12 +313,17 @@ bool CCSSector::WriteLights(CIWorld* pWorld)
           //I will prefer the "_light" format provided by Half-Life, because
           //this will allow colored light. If there is no "_light" statement,
           //we will look for a "light" key, which will only contain the radius.
+	  //Also added in: "_color" from Quake3 for more support with visuals
+	  //in GtkRadiant.
 
           bool   LightOk         = false;
+	  bool	 ColorOk	 = false;
           double r               = 255;
           double g               = 255;
           double b               = 255;
           double radius          = 200;
+	  double influenceRadius = -1;
+	  double brightness      = 1.0;
           bool   dynamic         = false;
           double halo            = 0;
           double halointen       = 0;
@@ -339,20 +334,46 @@ bool CCSSector::WriteLights(CIWorld* pWorld)
           if (lightvalue)
           {
             char dummy;
-            if (sscanf(lightvalue, "%lf %lf %lf %lf%c", &r, &g, &b, &radius, &dummy)==4)
+            if (sscanf(lightvalue, "%lf %lf %lf %lf%c",
+	    	&r, &g, &b, &radius, &dummy)==4)
             {
               LightOk = true;
+	      ColorOk = true;
             }
           }
 
+	  const char* colorvalue = pEntity->GetValueOfKey("_color");
+	  if (colorvalue)
+	  {
+	    char dummy;
+	    double r2            = 1.0;
+	    double g2            = 1.0;
+	    double b2            = 1.0;
+
+	    if (sscanf(colorvalue, "%lf %lf %lf%c", &r2, &g2, &b2, &dummy)==3)
+	    {
+	      r = r2 * 255;
+	      g = g2 * 255;
+	      b = b2 * 255;
+	      ColorOk = true;
+	    }
+	  }
+
           if (!LightOk)
           {
             radius = pEntity->GetNumValueOfKey("light", 100);
-            r      = 255;
-            g      = 255;
-            b      = 255;
+	    if (!ColorOk) 
+	    {
+              r      = 255;
+              g      = 255;
+              b      = 255;
+	    }
           }
 
+	  brightness = pEntity->GetNumValueOfKey("brightness", 1.0);
+
+	  influenceRadius = pEntity->GetNumValueOfKey("influence", -1.0);
+
           //dynamic light key
           dynamic = pEntity->GetBoolValueOfKey("dynamic", false);
 
@@ -366,8 +387,10 @@ bool CCSSector::WriteLights(CIWorld* pWorld)
             char dummy;
             if (sscanf(attenuationlightvalue, "%s%c",attenuation, &dummy) == 1)
             {
-              if (!(strcmp(attenuation,"none")    || strcmp(attenuation,"linear") ||
-                    strcmp(attenuation,"inverse") || strcmp(attenuation,"realistic")))
+              if (!(strcmp(attenuation,"none") ||
+	            strcmp(attenuation,"linear") ||
+                    strcmp(attenuation,"inverse") ||
+		    strcmp(attenuation,"realistic") ))
               {
                 strcpy( attenuation , "realistic" );
               }
@@ -386,28 +409,48 @@ bool CCSSector::WriteLights(CIWorld* pWorld)
           //It looks like Crystal Space uses a different algo for lights,
           //so we will scale them up a bit. (Maybe the factor should be
           //configurable in the future)
-          pWorld->WriteIndent();
 
-          fprintf(fd, "LIGHT  (");
-          fprintf(fd, "CENTER (%g,%g,%g) ",
-                  origin.x*ScaleFactor,
-                  origin.z*ScaleFactor,
-                  origin.y*ScaleFactor);
-          fprintf(fd, "RADIUS (%g) ",radius*ScaleFactor*lightscale);
-          fprintf(fd, "COLOR  (%g,%g,%g) ",
-                  (r/255) * (radius/128),
-                  (g/255) * (radius/128),
-                  (b/255) * (radius/128));
-          if (dynamic)
+	  DocNode light = CreateNode (node, "light");
+	  DocNode center = CreateNode (light, "center");
+	  center->SetAttributeAsFloat ("x", origin.x*ScaleFactor);
+	  center->SetAttributeAsFloat ("y", origin.z*ScaleFactor);
+	  center->SetAttributeAsFloat ("z", origin.y*ScaleFactor);
+	  
+	  // New Radius / Color code
+	  DocNode radiusNode = CreateNode (light, "radius", (float)(radius*lightscale*ScaleFactor));
+	  radiusNode->SetAttributeAsFloat ("brightness", brightness);
+	  
+	  DocNode color = CreateNode (light, "color");
+	  color->SetAttributeAsFloat("red", (r/255));
+	  color->SetAttributeAsFloat("green", (g/255));
+	  color->SetAttributeAsFloat("blue", (b/255));
+
+	  /* This code is Old (keeping it incase it somehow stops working)
+	  CreateNode (light, "radius", 
+	    (float)(radius*ScaleFactor*lightscale));
+	  DocNode color = CreateNode (light, "color");
+	  color->SetAttributeAsFloat ("red", (r/255) * (radius/128));
+	  color->SetAttributeAsFloat ("green", (g/255) * (radius/128));
+	  color->SetAttributeAsFloat ("blue", (b/255) * (radius/128));
+	  */
+
+	  if (influenceRadius != -1.0)
+	  {
+	    CreateNode (light, "influenceradius", (float)(influenceRadius*ScaleFactor*lightscale));
+	  }
+
+	  if (dynamic)
           {
-            fprintf(fd, "DYNAMIC () ");
+            CreateNode (light, "dynamic");
           }
-          fprintf(fd, "HALO    (%g,%g,%g) ",
-                  (halo),
-                  (halointen),
-                  (halocross));
-          fprintf(fd, "ATTENUATION (%s) ",attenuation);
-          fprintf(fd, ")\n\n");
+	  {
+	    DocNode halo = CreateNode (light, "halo");
+	    CreateNode (halo, "type", "cross");
+	    CreateNode (halo, "intensity", (float)halointen);
+	    CreateNode (halo, "cross", (float)halointen);
+	  }
+
+	  CreateNode (light, "attenuation", attenuation);
         } //if (light is inside this sector)
       } // if (entity had origin)
     } //if (classname == "light")
@@ -416,56 +459,46 @@ bool CCSSector::WriteLights(CIWorld* pWorld)
 }
 
 /// Write all nodes inside the sector
-bool CCSSector::WriteNodes(CIWorld* pWorld)
+bool CCSSector::WriteNodes(csRef<iDocumentNode> node, CIWorld* pWorld)
 {
   assert(pWorld);
 
-  FILE* fd = pWorld->GetFile();
-  assert(fd);
-
   double ScaleFactor = pWorld->GetScalefactor();
 
   //iterate all nodes
-  int i;
+  size_t i;
   for (i=0; i<m_Nodes.Length(); i++)
   {
     CMapEntity* pEntity = m_Nodes[i];
     CdVector3 Origin(0,0,0);
     pEntity->GetOrigin(Origin);
     if (strcmp(pEntity->GetValueOfKey("classname"),"cs_model")){
-    pWorld->WriteIndent();
-    fprintf(fd, "NODE '%s' (\n", pEntity->GetName());
-    pWorld->Indent();
-    CCSWorld::WriteKeys(pWorld, pEntity);
-    pWorld->WriteIndent();
-    fprintf(fd, "POSITION(%g,%g,%g)\n",
-                Origin.x*ScaleFactor,
-                Origin.z*ScaleFactor,
-                Origin.y*ScaleFactor);
-    pWorld->Unindent();
-    pWorld->WriteIndent();
-    fprintf(fd, ")\n");
-	}
+      DocNode Node = CreateNode (node, "node");
+      Node->SetAttribute ("name", pEntity->GetName());
+      CCSWorld::WriteKeys(Node, pWorld, pEntity);
+      DocNode position = CreateNode (Node, "position");
+      position->SetAttributeAsFloat ("x", Origin.x*ScaleFactor);
+      position->SetAttributeAsFloat ("y", Origin.z*ScaleFactor);
+      position->SetAttributeAsFloat ("z", Origin.y*ScaleFactor);
+    }
   }
 
   return true;
 }
 
-bool CCSSector::WriteCurves(CIWorld* pWorld)
+bool CCSSector::WriteCurves(csRef<iDocumentNode> node, CIWorld* pWorld)
 {
   assert(pWorld);
 
   CMapFile* pMap         = pWorld->GetMap();
-  FILE*     fd           = pWorld->GetFile();
 
   assert(pMap);
-  assert(fd);
 
-  int i, curve;
+  size_t i, curve;
   for (i=0; i<pMap->GetNumEntities(); i++)
   {
     CMapEntity* pEntity = pMap->GetEntity(i);
-    int const ncurves = pEntity->GetCurveCount();
+    size_t const ncurves = pEntity->GetCurveCount();
     if (ncurves>0)
     {
       for (curve=0; curve<ncurves; curve++)
@@ -474,36 +507,22 @@ bool CCSSector::WriteCurves(CIWorld* pWorld)
         CTextureFile* pTexture = pCurve->GetTexture();
         if (pTexture->IsVisible())
         {
-          pWorld->WriteIndent();
-          fprintf(fd, "MESHOBJ '%s_e%d_c%d'(\n",pEntity->GetName(), i, curve);
-          pWorld->Indent();
+	  DocNode meshobj = CreateNode (node, "meshobj");
+	  meshobj->SetAttribute ("name",
+	    csString().Format ("%s_e%zu_c%zu", pEntity->GetName(), i, curve));
+	  CreateNode (meshobj, "plugin", "bezier");
+	  //CreateNode (meshobj, "zuse");
+	  //CreateNode (meshobj, "priority", pEntity->GetValueOfKey("priority", "object"));
 
-          pWorld->WriteIndent();
-          fprintf(fd, "PLUGIN('thing')\n");
-          pWorld->WriteIndent();
-          fprintf(fd, "ZUSE()\n");
-          pWorld->WriteIndent();
-	  fprintf(fd, "PRIORITY('%s')\n", pEntity->GetValueOfKey("priority", "object"));
+	  DocNode params = CreateNode (meshobj, "params");
+          CreateNode (params, "factory", 
+	    csString().Format ("curve_%s", pCurve->GetName()));
 
-          CCSWorld::WriteKeys(pWorld, pEntity);
-
-          pWorld->WriteIndent();
-          fprintf(fd, "PARAMS(\n");
-          pWorld->Indent();
-
-          pWorld->WriteIndent();
-          fprintf(fd, "FACTORY ('curve_%s') \n", pCurve->GetName());
-
-          pWorld->Unindent();
-          pWorld->WriteIndent();
-          fprintf(fd, ")\n"); //PARAMS
-
-          pWorld->WriteIndent();
-          fprintf(fd, "MOVE (V (0,0,0))\n");
-
-          pWorld->Unindent();
-          pWorld->WriteIndent();
-          fprintf(fd, ")\n\n"); //MESHOBJ
+	  DocNode move = CreateNode (meshobj, "move");
+	  DocNode v = CreateNode (move, "v");
+	  v->SetAttributeAsInt ("x", 0);
+	  v->SetAttributeAsInt ("y", 0);
+	  v->SetAttributeAsInt ("z", 0);
         }
       }
     }
@@ -528,8 +547,10 @@ bool CCSSector::WriteCurves(CIWorld* pWorld)
     //  TRANSPARENT ()  //=DST
     //  KEYCOLOR ()     // color 0 is transparent
 
-bool CCSSector::WriteSprites(CIWorld* pWorld)
+bool CCSSector::WriteSprites(csRef<iDocumentNode> /*node*/, CIWorld* /*pWorld*/)
 {
+  // @@@ check if this still works
+#if 0
   assert(pWorld);
 
   CMapFile* pMap        = pWorld->GetMap();
@@ -538,9 +559,6 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
   //const char*   modelscale = "1";   //df added matrix scaler
 
   assert(pMap);
-  assert(fd);
-
-  fprintf(fd, "\n");
 
   int i;
   char   mdlname[99] = "none";
@@ -579,32 +597,32 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
         if (sscanf(csnamevalue, "%s%c",mdlname, &dummy) == 1)
         {
           pWorld->WriteIndent();
-          fprintf(fd,"NODE 'SEED_MESH_OBJ' (\n");
+          csFPrintf(fd,"<node name=\"SEED_MESH_OBJ\">\n");
           // Key/Value pairs writting
           pWorld->Indent();
           pWorld->WriteIndent();
-          fprintf(fd,"KEY(\"classname\",\"SEED_MESH_OBJ\")\n");
+          csFPrintf(fd,"<key name=\"classname\" value=\"SEED_MESH_OBJ\" />\n");
           pWorld->WriteIndent();
-          fprintf(fd,"KEY(\"cs_name\",\"%s\")\n",csnamevalue);
+          csFPrintf(fd,"<key name=\"cs_name\" value=\"%s\" />\n",csnamevalue);
 
           /*Lets build all material definition
            *keys KEY("defmaterial","materialname, texturefile")
-           * It will declare materials until BuildMateriaKey returns a NULL
+           * It will declare materials until BuildMateriaKey returns a 0
            */
           int mIndex; //material Index
           char* key;
           for (mIndex = 1; (key = BuildMaterialKey(mIndex,pEntity)); mIndex++)
 	  {
             pWorld->WriteIndent();
-            fprintf(fd,key);
+            csFPrintf(fd,key);
 	  }
 
           pWorld->WriteIndent();
-	  fprintf(fd,"KEY(\"spritematerial\",\"%s\")\n",csspritematerial);
+	  csFPrintf(fd,"<key name=\"spritematerial\" value=\"%s\" />\n",csspritematerial);
           pWorld->WriteIndent();
-          fprintf(fd,"KEY(\"factory\",\"%s\")\n",csfactname);
+          csFPrintf(fd,"<key name=\"factory\" value=\"%s\" />\n",csfactname);
           pWorld->WriteIndent();
-          fprintf(fd,"KEY(\"factorymaterial\",\"%s\")\n",csfactmaterial);
+          csFPrintf(fd,"<key name=\"factorymaterial\" value=\"%s\" />\n",csfactmaterial);
           pWorld->WriteIndent();
 
           /*
@@ -614,7 +632,7 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
 	   * (factory without material error)
            */
 
-          fprintf(fd,"KEY(\"factoryfile\",\"%s\")\n",csmodelfile);
+          csFPrintf(fd,"<key name=\"factoryfile\" value=\"%s\" />\n",csmodelfile);
 
           /*
 	   * MDL files have actions defined so if model file is an mdl we
@@ -623,44 +641,44 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
            */
           pWorld->WriteIndent();
           if(!isMdlFile(csmodelfile))
-            fprintf(fd,"KEY(\"actionsfile\",\"%s\")\n",csactionsfile);
-          else fprintf(fd,"KEY(\"actionfile\",\"NONE\"\n");
+            csFPrintf(fd,"<key name=\"actionsfile\" value=\"%s\" />\n",csactionsfile);
+          else csFPrintf(fd,"<key name=\"actionfile\" value=\"NONE\" />\n");
 
 	  pWorld->WriteIndent();
-          fprintf(fd,"KEY(\"scalefactor\",\"%s\")\n",csscalefactor);
+          csFPrintf(fd,"<key name=\"scalefactor\" value=\"%s\" />\n",csscalefactor);
 
           pWorld->WriteIndent();
-          fprintf(fd,"KEY(\"staticflag\",\"%s\")\n",csstaticmodel);
+          csFPrintf(fd,"<key name=\"staticflag\" value=\"%s\" />\n",csstaticmodel);
 
           pWorld->WriteIndent();
-          fprintf(fd, "POSITION( %g, %g, %g)\n",Origin.x*ScaleFactor,
+          csFPrintf(fd, "<position x=\"%g\" y=\"%g\" z=\"%g\" />\n",Origin.x*ScaleFactor,
                   Origin.z*ScaleFactor, Origin.y*ScaleFactor);
 
 
           pWorld->Unindent();
           pWorld->WriteIndent();
 
-          fprintf(fd,")\n\n");
+          csFPrintf(fd,")\n\n");
           pWorld->Unindent();
           pWorld->WriteIndent();
 
 		/*
 		  pWorld->WriteIndent();
-          fprintf(fd, "MESHREF '%s' \n", csnamevalue);
+          csFPrintf(fd, "MESHREF '%s' \n", csnamevalue);
           pWorld->WriteIndent();
-          fprintf(fd, "( \n");
+          csFPrintf(fd, "( \n");
           pWorld->Indent();
           pWorld->WriteIndent();
-          fprintf(fd, "FACTORY ('%s')\n ", csnamevaluefact);
-		  fprintf(fd, "PRIORITY ('Object')\n ");
-		  fprintf(fd, "ZUSE( )\n ");
+          csFPrintf(fd, "FACTORY ('%s')\n ", csnamevaluefact);
+		  csFPrintf(fd, "PRIORITY ('Object')\n ");
+		  csFPrintf(fd, "ZUSE( )\n ");
 		*/
 
 
           /*const char* actionvalue = pEntity->GetValueOfKey("action");
           if (sscanf(actionvalue, "%s%c",action, &dummy) == 1)
           {
-            fprintf(fd, "'%s' )\n", action);
+            csFPrintf(fd, "'%s' )\n", action);
           }
 	  */
 	  //END OF MODIFIED CODE
@@ -671,7 +689,7 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
         {
           //The strings format matched
           pWorld->WriteIndent();
-          fprintf(fd, "MOVE (V ( %g, %g, %g) ",
+          csFPrintf(fd, "MOVE (V ( %g, %g, %g) ",
                   x*ScaleFactor,
                   z*ScaleFactor,
                   y*ScaleFactor);
@@ -680,27 +698,28 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
           //if (sscanf(alphavalue, "%s%c",alphastr, &dummy) == 1)
           //{
             //pWorld->WriteIndent();
-            //fprintf(fd, "MIXMODE (ALPHA (%s))\n", alphastr);
+            //csFPrintf(fd, "MIXMODE (ALPHA (%s))\n", alphastr);
           //}
           //added single value matrix scaler
           //need to also add 9 value matrix for rotation etc
           //if (pEntity->GetValueOfKey("uniformscale", modelscale))
           //{
-            //fprintf(fd, " MATRIX ( %s ) )\n", modelscale);
+            //csFPrintf(fd, " MATRIX ( %s ) )\n", modelscale);
           //}
           //else
           //{
-            //fprintf(fd, " )\n");  //add newline if no scale value
+            //csFPrintf(fd, " )\n");  //add newline if no scale value
           //}
 //
           //pWorld->Unindent();
           //pWorld->WriteIndent();
-          //fprintf(fd, ") \n\n");
+          //csFPrintf(fd, ") \n\n");
 	  */
 //        }
       }
     }
   }
+#endif
   return true;
 }
 
@@ -724,22 +743,18 @@ bool CCSSector::WriteSprites(CIWorld* pWorld)
 //  TRANSPARENT ()  //=DST
 //  KEYCOLOR ()     // color 0 is transparent
 
-bool CCSSector::WriteSprites2D(CIWorld* pWorld)
+bool CCSSector::WriteSprites2D(csRef<iDocumentNode> /*node*/, CIWorld* pWorld)
 {
   assert(pWorld);
 
   CMapFile* pMap        = pWorld->GetMap();
-  FILE*     fd          = pWorld->GetFile();
-  double    ScaleFactor = pWorld->GetScalefactor();
+  //double    ScaleFactor = pWorld->GetScalefactor();
   assert(pMap);
-  assert(fd);
 
-  fprintf(fd, "\n");
-
-  int i;
+  //int i;
 
   //iterate all entities, brushes, polygons and vertices:
-  for (i=0; i<pMap->GetNumEntities(); i++)
+/*  for (i=0; i<pMap->GetNumEntities(); i++)
   {
     CMapEntity* pEntity = pMap->GetEntity(i);
     assert(pEntity);
@@ -758,22 +773,22 @@ bool CCSSector::WriteSprites2D(CIWorld* pWorld)
         if (sscanf(csnamevalue, "%s%c",sprname, &dummy) == 1)
         {
           pWorld->WriteIndent();
-          fprintf(fd, "SPRITE2D '%s' \n", sprname);
+          csFPrintf(fd, "SPRITE2D '%s' \n", sprname);
           pWorld->WriteIndent();
-          fprintf(fd, "( \n");
+          csFPrintf(fd, "( \n");
 
           //vertices; add square sprite polygon
           pWorld->Indent();
           pWorld->WriteIndent();
-          fprintf(fd, "VERTICES (-1,1,1,1,1,-1,-1,-1)\n");
+          csFPrintf(fd, "VERTICES (-1,1,1,1,1,-1,-1,-1)\n");
 
           //UV; add square texture coordinates
           pWorld->WriteIndent();
-          fprintf(fd, "UV (0,0,1,0,1,1,0,1)\n");
+          csFPrintf(fd, "UV (0,0,1,0,1,1,0,1)\n");
 
           pWorld->WriteIndent();
           const char* spritetexture = pEntity->GetValueOfKey("texture");
-          fprintf(fd, "MATERIAL (%s)\n", spritetexture);
+          csFPrintf(fd, "MATERIAL (%s)\n", spritetexture);
 
           //initial start position
           double x, y, z;
@@ -781,7 +796,7 @@ bool CCSSector::WriteSprites2D(CIWorld* pWorld)
           {
             //The strings format matched
             pWorld->WriteIndent();
-            fprintf(fd, "MOVE ( %g, %g, %g )\n ",
+            csFPrintf(fd, "MOVE ( %g, %g, %g )\n ",
                     x*ScaleFactor,
                     z*ScaleFactor,
                     y*ScaleFactor);
@@ -789,59 +804,59 @@ bool CCSSector::WriteSprites2D(CIWorld* pWorld)
 
           //mixmode
           pWorld->WriteIndent();
-          fprintf(fd, "MIXMODE ( ");
+          csFPrintf(fd, "MIXMODE ( ");
           bool copy = pEntity->GetBoolValueOfKey("copy", false);
           if (copy)
           {
-            fprintf(fd, "COPY () ");
+            csFPrintf(fd, "COPY () ");
           }
           bool add = pEntity->GetBoolValueOfKey("add", false);
           if (add)
           {
-            fprintf(fd, "ADD () ");
+            csFPrintf(fd, "ADD () ");
           }
           bool multiply = pEntity->GetBoolValueOfKey("multiply", false);
           if (multiply)
           {
-            fprintf(fd, "MULTIPLY ( ) ");
+            csFPrintf(fd, "MULTIPLY ( ) ");
           }
           bool multiply2 = pEntity->GetBoolValueOfKey("multiply2", false);
           if (multiply2)
           {
-            fprintf(fd, "MULTIPLY2 ( ) ");
+            csFPrintf(fd, "MULTIPLY2 ( ) ");
           }
           char alphastr[99] = "none";
           const char* alphavalue = pEntity->GetValueOfKey("alpha");
           if (sscanf(alphavalue, "%s%c",alphastr, &dummy) == 1)
           {
-            fprintf(fd, "ALPHA (%s) ", alphastr);
+            csFPrintf(fd, "ALPHA (%s) ", alphastr);
           }
           bool transparent = pEntity->GetBoolValueOfKey("transparent", false);
           if (transparent)
           {
-            fprintf(fd, "TRANSPARENT ( ) ");
+            csFPrintf(fd, "TRANSPARENT ( ) ");
           }
           bool keycolor = pEntity->GetBoolValueOfKey("keycolor", false);
           if (keycolor)
           {
-            fprintf(fd, "KEYCOLOR ( ) ");
+            csFPrintf(fd, "KEYCOLOR ( ) ");
           }
 
-          fprintf(fd, ")\n");
+          csFPrintf(fd, ")\n");
 
           pWorld->Unindent();
           pWorld->WriteIndent();
-          fprintf(fd, ") \n\n");
+          csFPrintf(fd, ") \n\n");
         }
       }
     }
-  }
+  }*/
   return true;
 }
 
-bool CCSSector::WriteFog(CIWorld* pWorld)
+bool CCSSector::WriteFog(csRef<iDocumentNode> /*node*/, CIWorld* /*pWorld*/)
 {
-  assert(pWorld);
+/*  assert(pWorld);
 
   CMapFile* pMap        = pWorld->GetMap();
   FILE*     fd          = pWorld->GetFile();
@@ -849,7 +864,7 @@ bool CCSSector::WriteFog(CIWorld* pWorld)
   assert(pMap);
   assert(fd);
 
-  fprintf(fd, "\n");
+  csFPrintf(fd, "\n");
 
 
   double r=255.0;
@@ -874,9 +889,9 @@ bool CCSSector::WriteFog(CIWorld* pWorld)
       {
         pEntity->GetTripleNumValueOfKey("fogcolor", r, g, b);
         pWorld->WriteIndent();
-        fprintf(fd, "FOG(%g,%g,%g,%g)\n", r/255.0, g/255.0, b/255.0, fogdensity);
+        csFPrintf(fd, "FOG(%g,%g,%g,%g)\n", r/255.0, g/255.0, b/255.0, fogdensity);
       }
     }
-  }
+  }*/
   return true;
 }
