@@ -22,23 +22,11 @@
 #define __DMATH3D_H__
 
 #include "cstypes.h"
-
-#ifndef ABS
-#define ABS(x) ((x)<0?-(x):(x))
-#endif
-
-#undef SMALL_EPSILON
-#define SMALL_EPSILON 0.000000001			/* Very small value */
+#include "csgeom/math.h"
 
 class CdVector3;
 class CdMatrix3;
 
-// this is used in inline functions (we can use csSquareFloat from csUtil?)
-inline double fSqr (double f)
-{
-  return f * f;
-}
-
 /**
  * A 3D vector.
  */
@@ -461,20 +449,6 @@ public:
     if (v.z > max.z) max.z = v.z; else if (v.z < min.z ) min.z = v.z;
   }
 
-  /**
-   * Compute twice the signed area of triangle composed by three points.
-   * This function returns 2 x the area of the triangle formed by the points
-   * a, b, and c.
-   */
-  inline static double Area3 (const CdVector3 &a, const CdVector3 &b,
-                             const CdVector3 &c)
-  {
-    CdVector3 v1 = b - a;
-    CdVector3 v2 = c - a;
-    return ((v1.y * v2.z + v1.z * v2.x + v1.x * v2.y) -
-            (v1.y * v2.x + v1.x * v2.z + v1.z * v2.y));
-  }
-
   /**
    * Calculate a plane normal given three vectors.
    * This function will calculate the normal to the plane formed by vectors
@@ -537,7 +511,12 @@ class CdSquaredDist
 public:
   /// Returns the squared distance between two points.
   static double PointPoint (const CdVector3& p1, const CdVector3& p2)
-  { return fSqr (p1.x - p2.x) + fSqr (p1.y - p2.y) + fSqr (p1.z - p2.z); }
+  {
+    return
+      csSquare (p1.x - p2.x) +
+      csSquare (p1.y - p2.y) +
+      csSquare (p1.z - p2.z);
+  }
 
   /// Returns the squared distance between a point and a line.
   static double PointLine (const CdVector3& p,
@@ -646,5 +625,5 @@ public:
 };
 
 
-#endif /*__DMATH3D_H__*/
+#endif // __DMATH3D_H__
 
