@@ -43,8 +43,8 @@ CMapKeyValuePair::~CMapKeyValuePair()
 CMapEntity::CMapEntity()
 {
   static int EntityCount = 1;
-  char Buffer[100];
-  sprintf(Buffer, "e%d", EntityCount++);
+  csString Buffer;
+  Buffer.Format ("e%d", EntityCount++);
   m_UniqueName = Buffer;
 }
 
@@ -60,13 +60,10 @@ bool CMapEntity::Read(CMapParser* pParser, CMapFile* pMap)
   csString Buffer;
   csString Key;
   bool finished = false;
-  int bufferLine, bufferLine2;
 
   while (!finished)
   {
-    bufferLine = pParser->GetCurrentLine();
     if (!pParser->GetSafeToken(Buffer)) return false;
-    bufferLine2 = pParser->GetCurrentLine();
     if (strcmp(Buffer, "{") == 0)
     {
       //This entity contains some brushes or curves!
@@ -103,7 +100,7 @@ bool CMapEntity::Read(CMapParser* pParser, CMapFile* pMap)
     {
       //Now this seems to be a key/ value pair
       Key.Replace (Buffer);
-      if ((bufferLine != bufferLine2) || !pParser->GetNextToken(Buffer))
+      if (!pParser->GetNextToken(Buffer))
       {
         pParser->ReportError("Format error. Keys and values for entities must"
                              "always come in pairs. Found no match for key \"%s\"",
@@ -123,7 +120,7 @@ void CMapEntity::AddKeyValuePair(const char* Key, const char* Value)
 
 const char* CMapEntity::GetValueOfKey (const char* key, const char* defaultvalue)
 {
-  int i, NumKeys = m_Keymap.Length();
+  size_t i, NumKeys = m_Keymap.Length();
   for (i=0; i<NumKeys; i++)
   {
     if (strcmp(key, m_Keymap[i]->GetKey())==0)
@@ -223,7 +220,7 @@ bool CMapEntity::GetOrigin(CdVector3& v)
 
 void CMapEntity::CreatePolygons()
 {
-  int i;
+  size_t i;
   for (i=0; i<m_Brushes.Length(); i++)
   {
     m_Brushes[i]->CreatePolygons();
