@@ -51,12 +51,13 @@ CIWorld::~CIWorld()
 
 void CIWorld::FindSectors()
 {
-  int i, j, SectorCounter = 0;
+  size_t j, SectorCounter = 0;
+  size_t i;
 
   //iterate all entities and brushes
   for (i=0; i<m_pMap->GetNumEntities(); i++)
   {
-    CMapEntity* pEntity = m_pMap->GetEntity(i);
+    CMapEntity* pEntity = m_pMap->GetEntity((int)i);
     if (strcmp(pEntity->GetClassname(), "cs_sector")==0)
     {
       //This entity is a sector!
@@ -85,7 +86,7 @@ void CIWorld::GenerateDefaultsector()
 {
   if (m_pMap->GetPlaneCount()<=0)
   {
-    printf("Map contains no data. Aborting!");
+    csPrintf("Map contains no data. Aborting!");
     exit(1);
   }
 
@@ -146,7 +147,7 @@ void CIWorld::GenerateDefaultsector()
 
 void CIWorld::FindPortals()
 {
-  int i, j;
+  size_t i, j;
   for (i=0; i<m_Sectors.Length(); i++)
   {
     CISector* pSector1 = m_Sectors[i];
@@ -167,7 +168,7 @@ void CIWorld::FindPortals()
 
 void CIWorld::InsertThings()
 {
-  int i;
+  size_t i;
   for (i=0; i<m_Sectors.Length(); i++)
   {
     CISector* pSector = m_Sectors[i];
@@ -179,7 +180,7 @@ void CIWorld::InsertThings()
 
 CISector* CIWorld::FindSectorForPoint(CdVector3& v)
 {
-  int i;
+  size_t i;
   for (i=0; i<m_Sectors.Length(); i++)
   {
     CISector* pSector = m_Sectors[i];
@@ -197,11 +198,12 @@ CISector* CIWorld::FindSectorForPoint(CdVector3& v)
 
 void CIWorld::BuildTexturelist()
 {
-  int i, j, c;
+  size_t c;
+  size_t j, i;
 
   for (i=0; i<m_pMap->GetPlaneCount(); i++)
   {
-    CTextureFile* pTexture = m_pMap->GetPlane(i)->GetTexture();
+    CTextureFile* pTexture = m_pMap->GetPlane((int)i)->GetTexture();
 
     bool TextureFound = false;
     for (j=0; j<m_TextureFileNames.Length(); j++)
@@ -223,7 +225,7 @@ void CIWorld::BuildTexturelist()
 
   for (i=0; i<m_pMap->GetNumEntities(); i++)
   {
-    CMapEntity* pEntity = m_pMap->GetEntity(i);
+    CMapEntity* pEntity = m_pMap->GetEntity((int)i);
     for (c=0; c<pEntity->GetCurveCount(); c++)
     {
       CMapCurve*    pCurve   = pEntity->GetCurve(c);
@@ -251,12 +253,12 @@ void CIWorld::BuildTexturelist()
   //add textures for all 2D cs_sprite
   for (i=0; i<m_pMap->GetNumEntities(); i++)
   {
-    CMapEntity* pEntity = m_pMap->GetEntity(i);
+    CMapEntity* pEntity = m_pMap->GetEntity((int)i);
     if (strcmp(pEntity->GetClassname(), "cs_sprite")==0)
     {
       CTextureManager* pTexMan  = m_pMap->GetTextureManager();
 
-	  const char* texturefilename = pEntity->GetValueOfKey("texture");
+      const char* texturefilename = pEntity->GetValueOfKey("texture");
       CTextureFile* pTexture = pTexMan->GetTexture(texturefilename);
 
       bool TextureFound = false;
@@ -279,7 +281,7 @@ void CIWorld::BuildTexturelist()
   }
 }
 
-bool CIWorld::PrepareData(const char* filename, CMapFile* pMap)
+bool CIWorld::PrepareData(const char* /*filename*/, CMapFile* pMap)
 {
   m_pMap = pMap;
   m_ScaleFactor = pMap->GetConfigFloat("Map2CS.General.Scaling", 1.0/40.0);
@@ -289,9 +291,9 @@ bool CIWorld::PrepareData(const char* filename, CMapFile* pMap)
   specified in the worldspawn
   **************************/
 
-  int maxEnt = pMap->GetNumEntities();
+  size_t maxEnt = pMap->GetNumEntities();
 
-  for (int i = 0; i < maxEnt; i++) 
+  for (size_t i = 0; i < maxEnt; i++) 
   {
     CMapEntity*	curEnt = pMap->GetEntity(i);
 
