@@ -39,7 +39,7 @@ CMapPolygon::CMapPolygon(const CMapPolygon& poly)
   m_pBrush          = poly.m_pBrush;
   m_BrushLineNumber = poly.m_BrushLineNumber;
   m_PlaneNumber     = poly.m_PlaneNumber;
-  int i;
+  size_t i;
   for (i=0; i<poly.m_Planes.Length(); i++)
   {
     m_Planes.Push(poly.m_Planes[i]); //only stored by reference
@@ -72,7 +72,7 @@ void CMapPolygon::Create(CMapTexturedPlane*              pBaseplane,
   //for the upcoming handling, we copy planes into a reduced array of planes
   //this will help us to change order and to remove already used planes.
   //Also we will remove all planes that don't create an edge right here.
-  int p=0;
+  size_t p=0;
   CMapTexturedPlaneVector RedplanesStage1;
   for (p=0; p<planes.Length(); p++)
   {
@@ -123,8 +123,8 @@ void CMapPolygon::Create(CMapTexturedPlane*              pBaseplane,
   redplanes.DeleteIndex(redplanes.Find(pPlane2));
 
   //Remove the baseplane too.
-  int BaseplaneIndex = redplanes.Find(pBaseplane);
-  if (BaseplaneIndex >= 0) redplanes.DeleteIndex(BaseplaneIndex);
+  size_t BaseplaneIndex = redplanes.Find(pBaseplane);
+  if (BaseplaneIndex != csArrayItemNotFound) redplanes.DeleteIndex(BaseplaneIndex);
 
   //Insert pPlane1 as first element of redplanes. This will ensure, that
   //this plane is preferred to close the polygon again. Otherwise there
@@ -143,7 +143,7 @@ void CMapPolygon::Create(CMapTexturedPlane*              pBaseplane,
     pPlane1 = m_Planes[m_Planes.Length()-1];
     //pPlane1 = pPlane2;
 
-    int NumPlanes = redplanes.Length();
+    size_t NumPlanes = redplanes.Length();
 
     //Check all planes, if they will form a new legal vertex
     //with the baseplane and plane1 (the last found plane)
@@ -209,7 +209,7 @@ void CMapPolygon::Create(CMapTexturedPlane*              pBaseplane,
     // weird splits. (I have come to think now, that this is a rather commom
     // case instead of an exception)
 
-    ////printf("Create() failed!\n");
+    ////csPrintf("Create() failed!\n");
     ////DumpPolyinfo(pBaseplane, planes);
     ////Create(pBaseplane, planes);
 
@@ -225,7 +225,7 @@ void CMapPolygon::Create(CMapTexturedPlane*              pBaseplane,
 
   //if (GetArea()<1)
   //{
-    //printf("removed polygon due to too small area! (%g)\n", GetArea());
+    //csPrintf("removed polygon due to too small area! (%g)\n", GetArea());
     //Clear();
   //}
 }
@@ -242,7 +242,6 @@ void CMapPolygon::Clear()
 bool CMapPolygon::IsEmpty()
 {
   assert(m_Vertices.Length() == m_Planes.Length());
-  assert(m_Vertices.Length() >= 0);
 
   return m_Vertices.Length() == 0;
 }
@@ -263,7 +262,7 @@ double CMapPolygon::GetArea()
   CdVector3 Normal = m_pBaseplane->GetNormal().Unit();
   CdVector3 Sum(0,0,0);
 
-  int i;
+  size_t i;
   for (i=0; i<(m_Vertices.Length()-1); i++)
   {
     Sum += (*(m_Vertices[i])) % (*(m_Vertices[i+1]));
@@ -287,8 +286,8 @@ void CMapPolygon::Split(CMapTexturedPlane* pSplitplane,
   //unneeded planes and vertices.
   CMapTexturedPlaneVector NewPlanes;
 
-  int NumPlanes = m_Planes.Length();
-  int i;
+  size_t NumPlanes = m_Planes.Length();
+  size_t i;
   for (i=0; i<NumPlanes; i++)
   {
     //Use all current planes
@@ -306,7 +305,7 @@ void CMapPolygon::SetPolygon(CMapPolygon* pPoly)
 {
   m_pBaseplane = pPoly->m_pBaseplane;;
 
-  int i=0;
+  size_t i=0;
   for (i=0; i<pPoly->m_Vertices.Length(); i++)
   {
     m_Vertices.Push(new CdVector3(*pPoly->m_Vertices[i]));
@@ -322,7 +321,7 @@ void CMapPolygon::FlipSide()
 {
   m_pBaseplane = m_pBaseplane->GetMirror();
 
-  int i=0;
+  size_t i=0;
   for (i=0; i<m_Vertices.Length()/2; i++)
   {
     CdVector3* pHelp                    = m_Vertices[i];
@@ -349,10 +348,10 @@ void CMapPolygon::GetStartplanes(const CMapTexturedPlaneVector& planes,
                                  CMapTexturedPlane*&            pPlane2,
                                  CdVector3&                     point)
 {
-  int NumPlanes = planes.Length();
+  size_t NumPlanes = planes.Length();
 
   //Try the follwing for all possible pairs of planes.
-  int i, j;
+  size_t i, j;
   for (i=0; i<NumPlanes; i++)
   {
     pPlane1 = planes[i];
@@ -400,10 +399,10 @@ int  CMapPolygon::GetNumberOfValidVertices(CMapTexturedPlane*             pPlane
                        // getting three values happens only in case of an error,
                        // that will be explicitly tracked down.
 
-  int NumPlanes = planes.Length();
+  size_t NumPlanes = planes.Length();
 
   //Try the follwing for all planes.
-  int i;
+  size_t i;
   for (i=0; i<NumPlanes; i++)
   {
     CMapTexturedPlane* pOtherPlane = planes[i];
@@ -455,7 +454,7 @@ bool CMapPolygon::CheckIfInside(const CdVector3&               point,
   //All thre planes create at least a single point. Now we need to
   //determine, if that point is inside or outside the given polygons
 
-  int k, NumPlanes = planes.Length();
+  size_t k, NumPlanes = planes.Length();
   for (k=0; k<NumPlanes; k++)
   {
     CMapTexturedPlane* pPlane = planes[k];
@@ -475,7 +474,7 @@ bool CMapPolygon::CheckIfInside(const CdVector3&               point,
   return true;
 }
 
-void CMapPolygon::SetErrorInfo(int BrushLineNumber, int PlaneNumber)
+void CMapPolygon::SetErrorInfo(int BrushLineNumber, size_t PlaneNumber)
 {
   m_BrushLineNumber = BrushLineNumber;
   m_PlaneNumber     = PlaneNumber;
@@ -484,18 +483,18 @@ void CMapPolygon::SetErrorInfo(int BrushLineNumber, int PlaneNumber)
 void CMapPolygon::DumpPolyinfo(CMapTexturedPlane*             pBaseplane,
                                const CMapTexturedPlaneVector& planes)
 {
-  int j, i, y, c, x;
+  size_t j, i, y, c, x;
   for (j=0; j<m_Planes.Length(); j++)
   {
     for (i=0; i<planes.Length(); i++)
     {
       if (m_Planes[j] == planes[i])
       {
-        printf("%d ", i);
+        csPrintf("%zu ", i);
       }
     }
   }
-  printf("\n");
+  csPrintf("\n");
 
   for (y=0; y<planes.Length(); y++)
   {
@@ -506,7 +505,7 @@ void CMapPolygon::DumpPolyinfo(CMapTexturedPlane*             pBaseplane,
         CdVector3 point;
         if (x==y)
         {
-          printf("        * "); //10 char "No check!"
+          csPrintf("        * "); //10 char "No check!"
         }
         else
         {
@@ -516,22 +515,22 @@ void CMapPolygon::DumpPolyinfo(CMapTexturedPlane*             pBaseplane,
 
             if (CheckIfInside(point, planes, pBaseplane, planes[x], planes[y]))
             {
-              printf("%8.3fI ",v); //10 char "Inside"
+              csPrintf("%8.3fI ",v); //10 char "Inside"
             }
             else
             {
-              printf("%8.3fO ",v); //10 char "Outside"
+              csPrintf("%8.3fO ",v); //10 char "Outside"
             }
           }
           else
           {
-            printf("        M "); //10 char "Miss"
+            csPrintf("        M "); //10 char "Miss"
           }
         }
       }
-      printf("\n");
+      csPrintf("\n");
     }
-    printf("\n");
+    csPrintf("\n");
   }
-  printf("Dump done\n");
+  csPrintf("Dump done\n");
 }
