@@ -93,12 +93,12 @@ public:
   void FlipSide();
 
   /// Give some info to the polygon so better errors can be generated.
-  void SetErrorInfo(int BrushLineNumber, int PlaneNumber);
+  void SetErrorInfo (int BrushLineNumber, size_t PlaneNumber);
 
   /// Access all vertices from outside
-  int                GetVertexCount() const     {return m_Vertices.Length();}
-  CdVector3          GetVertex(int index) const {return *(m_Vertices.Get(index));}
-  CMapTexturedPlane* GetPlane (int index) const {return m_Planes.Get(index);}
+  size_t             GetVertexCount() const     {return m_Vertices.Length();}
+  CdVector3          GetVertex(size_t index) const {return *(m_Vertices.Get(index));}
+  CMapTexturedPlane* GetPlane (size_t index) const {return m_Planes.Get(index);}
 
   /// returns the Baseplane for this polygon
   CMapTexturedPlane* GetBaseplane() {return m_pBaseplane;}
@@ -167,7 +167,7 @@ protected:
   int m_BrushLineNumber;
 
   /// info on the plane in the brush
-  int m_PlaneNumber;
+  size_t m_PlaneNumber;
 
   /// pointer to the brush, that this polyon was originally part of.
   CMapBrush* m_pBrush;
