@@ -67,9 +67,9 @@ void CTextureManager::LoadTextureArchives(CMapFile* pMap)
     int nr = 1;
     do
     {
-      char keyname[200];
+      csString keyname;
       char filename[300];
-      sprintf(keyname, KeynameFormat[type], nr);
+      keyname.Format (KeynameFormat[type], nr);
       strcpy(filename, pMap->GetConfigStr(keyname, ""));
       if (filename[0])
       {
@@ -127,16 +127,14 @@ CTextureFile* CTextureManager::GetTexture(const char* TextureName)
 {
   assert(m_pMap);
 
-  char CleanedUpTextureName[200];
-  strcpy(CleanedUpTextureName, TextureName);
-  CleanupTexturename(CleanedUpTextureName);
+  csString CleanedUpTextureName (TextureName);
+  CleanupTexturename (CleanedUpTextureName);
 
-  char InternalName[200];
-  strncpy (InternalName, CleanedUpTextureName, sizeof(InternalName));
+  csString InternalName (CleanedUpTextureName);
 
   // clean out some chars like '/' etc.
   size_t p=0;
-  for (p=0; p<strlen(InternalName); p++)
+  for (p=0; p<InternalName.Length(); p++)
   {
     if (strchr ("/\\", InternalName[p]))
     {
@@ -145,7 +143,7 @@ CTextureFile* CTextureManager::GetTexture(const char* TextureName)
   }
 
   //First, we search in the array of already stored textures.
-  int i;
+  size_t i;
   for (i=0; i<m_StoredTextures.Length(); i++)
   {
     CTextureFile* pTexture = m_StoredTextures[i];
@@ -187,9 +185,9 @@ CTextureFile* CTextureManager::GetTexture(const char* TextureName)
       pTexture->SetTexturename(InternalName);
       if (pTexture->IsVisible() && pTexture->IsStored())
       {
-	printf ("Warning: texture '%s'('%s') is missing.\n"
-	  "         Using '%s' instead!\n",  TextureName, InternalName, 
-	  defaultname);
+	csPrintf ("Warning: texture '%s'('%s') is missing.\n"
+	  "         Using '%s' instead!\n",  TextureName, 
+	  InternalName.GetData(), defaultname);
       }
       m_StoredTextures.Push(pTexture);
       return pTexture;
@@ -198,8 +196,8 @@ CTextureFile* CTextureManager::GetTexture(const char* TextureName)
 
   CTextureFile* pTexture = 0;
 
-  printf("Warning: texture '%s'('%s') is missing.\n        Making a new null texture.\n",
-    TextureName, CleanedUpTextureName);
+  csPrintf("Warning: texture '%s'('%s') is missing.\n        Making a new null texture.\n",
+    TextureName, CleanedUpTextureName.GetData());
   pTexture = new CTextureFile;
   pTexture->SetTexturename (InternalName);
   pTexture->SetFilename    (CleanedUpTextureName);
@@ -212,7 +210,7 @@ CTextureFile* CTextureManager::GetTexture(const char* TextureName)
 bool CTextureManager::AddAllTexturesToVFS(csRef<iVFS> VFS, const char* path)
 {
   bool ok = true;
-  int i;
+  size_t i;
   for (i=0; i<m_StoredTextures.Length(); i++)
   {
     CTextureFile* pTexture = m_StoredTextures[i];
@@ -232,19 +230,19 @@ bool CTextureManager::AddAllTexturesToVFS(csRef<iVFS> VFS, const char* path)
   return ok;
 }
 
-void CTextureManager::CleanupTexturename(char* Name)
+void CTextureManager::CleanupTexturename (csString& Name)
 {
   int i;
 
   for (i=0; i<int(sizeof(Q3Extensions)/sizeof(Q3Extensions[0])); i++)
   {
-    int ExtensionLen = strlen(Q3Extensions[i]);
-    int NameLen      = strlen(Name);
-    if (NameLen>ExtensionLen)
+    size_t ExtensionLen = strlen(Q3Extensions[i]);
+    if (Name.Length() > ExtensionLen)
     {
-      if (strcmp(Name+NameLen-ExtensionLen, Q3Extensions[i])==0)
+      if (strcmp (Name.GetData() + Name.Length() - ExtensionLen, 
+        Q3Extensions[i])==0)
       {
-        Name[NameLen-ExtensionLen] = 0;
+        Name.Truncate (Name.Length() - ExtensionLen);
         return;
       }
     }
