@@ -41,7 +41,7 @@ CVertexBuffer::~CVertexBuffer()
 void CVertexBuffer::AddVertex(CdVector3 Vertex)
 {
   //Check if the vertex is already stored in the Buffer
-  if (GetIndex(Vertex)>=0) return;
+  if (GetIndex(Vertex) != (size_t)-1) return;
 
   //It's a new Vertex:
   m_Vertices.Push(new CdVector3(Vertex));
@@ -50,8 +50,8 @@ void CVertexBuffer::AddVertex(CdVector3 Vertex)
 void CVertexBuffer::AddVertices(CMapPolygon* pPoly)
 {
   //Add all vertices of this polygon
-  int  NumPolyVertices = pPoly->GetVertexCount();
-  int i;
+  size_t NumPolyVertices = pPoly->GetVertexCount();
+  size_t i;
   for (i=0; i<NumPolyVertices; i++)
   {
     AddVertex(pPoly->GetVertex(i));
@@ -61,8 +61,8 @@ void CVertexBuffer::AddVertices(CMapPolygon* pPoly)
 void CVertexBuffer::AddVertices(CMapPolygonSet* pPolySet)
 {
   //Add all vertices of all polygons
-  int  NumPolygons = pPolySet->GetPolygonCount();
-  int i;
+  size_t NumPolygons = pPolySet->GetPolygonCount();
+  size_t i;
   for (i=0; i<NumPolygons ; i++)
   {
     AddVertices(pPolySet->GetPolygon(i));
@@ -71,7 +71,7 @@ void CVertexBuffer::AddVertices(CMapPolygonSet* pPolySet)
 
 void CVertexBuffer::AddVertices(CMapPolygonSetVector* pPolySetVector)
 {
-  int i;
+  size_t i;
   for (i=0; i<pPolySetVector->Length(); i++)
   {
     AddVertices(pPolySetVector->Get(i));
@@ -80,17 +80,17 @@ void CVertexBuffer::AddVertices(CMapPolygonSetVector* pPolySetVector)
 
 void CVertexBuffer::AddVertices(CIPortalVector* pPortalVector)
 {
-  int i;
+  size_t i;
   for (i=0; i<pPortalVector->Length(); i++)
   {
     AddVertices(pPortalVector->Get(i));
   }
 }
 
-int CVertexBuffer::GetIndex(CdVector3 Vertex) const
+size_t CVertexBuffer::GetIndex(CdVector3 Vertex) const
 {
   //search for the vertex in the buffer
-  int i;
+  size_t i;
   for (i=0; i<m_Vertices.Length(); i++)
   {
     if (Vertex == (*m_Vertices[i]))
@@ -101,7 +101,7 @@ int CVertexBuffer::GetIndex(CdVector3 Vertex) const
   }
 
   //index not found
-  return -1;
+  return (size_t)-1;
 }
 
 bool CVertexBuffer::WriteCS(csRef<iDocumentNode> node, CIWorld* pWorld)
@@ -110,7 +110,7 @@ bool CVertexBuffer::WriteCS(csRef<iDocumentNode> node, CIWorld* pWorld)
 
   double ScaleFactor = pWorld->GetScalefactor();
 
-  int i;
+  size_t i;
   for (i=0; i<m_Vertices.Length(); i++)
   {
     DocNode v = CreateNode (node, "v");
