@@ -55,13 +55,13 @@ void WriteByte(char*& p, unsigned char c)
 
 void WriteWord(char*& p, unsigned short s)
 {
-  set_le_short ((void*)p, s);
+  csSetToAddress::UInt16 (p, csLittleEndian::UInt16 (s));
   p+=2;
 }
 
 void WriteDword(char*& p, unsigned long l)
 {
-  set_le_long ((void*)p, l);
+  csSetToAddress::UInt32 (p, csLittleEndian::UInt32 (l));
   p+=4;
 }
 
@@ -229,7 +229,7 @@ bool CWad3File::Extract(const char* Texture, char*& Data, int& Size, csString& f
     csRef<iDataBuffer> db = ImageLoader->Save (&mi, "image/png", "compress=100");
     if (db)
     {
-      Size = db->GetSize();
+      Size = (int)db->GetSize();
       Data = new char[Size];
       memcpy (Data, db->GetData(), Size);
       db = 0;
@@ -313,7 +313,7 @@ bool CWad3File::Open(const char *filename)
   m_Filehandle = fopen(filename, "rb");
   if (!m_Filehandle)
   {
-    printf("Can't open %s!\n", filename);
+    csPrintf("Can't open %s!\n", filename);
     return false;
   }
 
@@ -322,7 +322,7 @@ bool CWad3File::Open(const char *filename)
   if (!Read(FileID)) return false;
   if (FileID != 0x33444157)
   {
-    printf("%s is not a valid WAD3 file!\n", filename);
+    csPrintf("%s is not a valid WAD3 file!\n", filename);
     return false;
   }
 
