@@ -29,32 +29,32 @@ bool CZipFile::AddFile(const char* originalname, const char* zipname)
   FILE* fd = fopen(originalname, "rb");
   if (!fd)
   {
-    printf("Can't find file '%s'!\n", originalname);
+    csPrintf("Can't find file '%s'!\n", originalname);
     return false;
   }
 
   //Check the size of the file.
   fseek(fd, 0, SEEK_END); //Seek to end of file
-  int size = ftell(fd);   //Check current position
+  size_t size = ftell(fd);   //Check current position
   fseek(fd, 0, SEEK_SET); //Seek to beginning of file
 
   void* file = NewFile(zipname, size);
   if (!file)
   {
-    printf("Can't add new file '%s' to zip archive\n", originalname);
+    csPrintf("Can't add new file '%s' to zip archive\n", originalname);
     return false;
   }
 
   char Buffer[1000];
 
-  int totalread = 0;
-  int numread   = 0;
+  size_t totalread = 0;
+  size_t numread   = 0;
   do
   {
-    numread = fread(Buffer, 1, 1000, fd);
+    numread = fread (Buffer, 1, 1000, fd);
     if (!Write(file, Buffer, numread))
     {
-      printf("Can't add data of file '%s' to zip archive\n", originalname);
+      csPrintf("Can't add data of file '%s' to zip archive\n", originalname);
       return false;
     }
     totalread += numread;
@@ -74,13 +74,13 @@ bool CZipFile::AddData(const char* Data, int Size, const char* zipname)
   void* file = NewFile(zipname, Size);
   if (!file)
   {
-    printf("Can't add new file '%s' to zip archive\n", zipname);
+    csPrintf("Can't add new file '%s' to zip archive\n", zipname);
     return false;
   }
 
   if (!Write(file, Data, Size))
   {
-    printf("Can't add data (%s) to zip archive\n", zipname);
+    csPrintf("Can't add data (%s) to zip archive\n", zipname);
     return false;
   }
 
@@ -106,7 +106,7 @@ bool CZipFile::ExtractData(CBinaryData* pData, const char* zipname)
     char* pZipData = Read (zipname);
     assert(pZipData);
 
-    pData->SetData(pZipData, filesize);
+    pData->SetData (pZipData, (int)filesize);
     delete[] pZipData;
 
     return true;
