@@ -18,16 +18,17 @@
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
-#include "cssysdef.h"
 #include "m2s_mdl.h"
 #include "m2s_pal.h"
 #include "m2s_img.h"
 #include "csutil/csendian.h"
+#include "csutil/sysfunc.h"
 #include "iutil/databuff.h"
 #include "igraphic/imageio.h"
 
@@ -93,7 +94,7 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
     return setError("Invalid mdl magic", f);
 
   // check if is a correct version
-  magic.version = convert_endian (magic.version);
+  magic.version = csConvertEndian (magic.version);
   if (magic.version != 6)
     return setError("Invalid mdl version", f);
 
@@ -102,25 +103,25 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
   if (fread(&header, sizeof(mdl_t), 1, f) != 1)
     return setError("Cannot read mdl header", f);
 
-  header.scale.x = convert_endian (header.scale.x);
-  header.scale.y = convert_endian (header.scale.y);
-  header.scale.z = convert_endian (header.scale.z);
-  header.origin.x = convert_endian (header.origin.x);
-  header.origin.y = convert_endian (header.origin.y);
-  header.origin.z = convert_endian (header.origin.z);
-  header.radius = convert_endian (header.radius);
-  header.offsets.x = convert_endian (header.offsets.x);
-  header.offsets.y = convert_endian (header.offsets.y);
-  header.offsets.z = convert_endian (header.offsets.z);
-  header.numskins = convert_endian (header.numskins);
-  header.skinwidth = convert_endian (header.skinwidth);
-  header.skinheight = convert_endian (header.skinheight);
-  header.numverts = convert_endian (header.numverts);
-  header.numtris = convert_endian (header.numtris);
-  header.numframes = convert_endian (header.numframes);
-  header.synctype = convert_endian (header.synctype);
-  header.flags = convert_endian (header.flags);
-  header.size = convert_endian (header.size);
+  header.scale.x = csConvertEndian (header.scale.x);
+  header.scale.y = csConvertEndian (header.scale.y);
+  header.scale.z = csConvertEndian (header.scale.z);
+  header.origin.x = csConvertEndian (header.origin.x);
+  header.origin.y = csConvertEndian (header.origin.y);
+  header.origin.z = csConvertEndian (header.origin.z);
+  header.radius = csConvertEndian (header.radius);
+  header.offsets.x = csConvertEndian (header.offsets.x);
+  header.offsets.y = csConvertEndian (header.offsets.y);
+  header.offsets.z = csConvertEndian (header.offsets.z);
+  header.numskins = csConvertEndian (header.numskins);
+  header.skinwidth = csConvertEndian (header.skinwidth);
+  header.skinheight = csConvertEndian (header.skinheight);
+  header.numverts = csConvertEndian (header.numverts);
+  header.numtris = csConvertEndian (header.numtris);
+  header.numframes = csConvertEndian (header.numframes);
+  header.synctype = csConvertEndian (header.synctype);
+  header.flags = csConvertEndian (header.flags);
+  header.size = csConvertEndian (header.size);
 
   // sprite ops
   radiusbound = header.radius;
@@ -141,7 +142,7 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
     int32 group = 0;
     if (fread(&group, sizeof(group), 1, f) != 1)
       return setError("Error reading mdl file", f);
-    group = convert_endian (group);
+    group = csConvertEndian (group);
 
     if (group != 1 && group != 0)
       return setError("Incoherence in skin model properties", f);
@@ -160,14 +161,14 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
       skins[i].group = true;
       if (fread(&skins[i].nbtexs, sizeof(skins[i].nbtexs), 1, f) != 1)
         return setError("Error reading mdl file", f);
-      skins[i].nbtexs = convert_endian (skins[i].nbtexs);
+      skins[i].nbtexs = csConvertEndian (skins[i].nbtexs);
 
       // read time between frame
       skins[i].timebtwskin = new float [skins[i].nbtexs];
       if (fread(skins[i].timebtwskin, sizeof(float)*skins[i].nbtexs, 1, f)!=1)
         return setError("Error reading multi-tex skin", f);
       for (ii = 0 ; ii < skins[i].nbtexs ; ii++)
-        skins[i].timebtwskin[ii] = convert_endian (skins[i].timebtwskin[ii]);
+        skins[i].timebtwskin[ii] = csConvertEndian (skins[i].timebtwskin[ii]);
 
       // read all texture of group
       skins[i].texs = new unsigned char *[skins[i].nbtexs];
@@ -188,9 +189,9 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
   {
     if (fread(&vertices[i], sizeof(vertice_t), 1, f) != 1)
       return setError("Error reading mdl file", f);
-    vertices[i].onseam = convert_endian (vertices[i].onseam);
-    vertices[i].s = convert_endian (vertices[i].s);
-    vertices[i].t = convert_endian (vertices[i].t);
+    vertices[i].onseam = csConvertEndian (vertices[i].onseam);
+    vertices[i].s = csConvertEndian (vertices[i].s);
+    vertices[i].t = csConvertEndian (vertices[i].t);
   }
 
   // triangles ops
@@ -201,10 +202,10 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
   {
     if (fread(&triangles[i], sizeof(triangle_t), 1, f) != 1)
       return setError("Error reading mdl file", f);
-    triangles[i].facefront = convert_endian (triangles[i].facefront);
-    triangles[i].vertice[0] = convert_endian (triangles[i].vertice[0]);
-    triangles[i].vertice[1] = convert_endian (triangles[i].vertice[1]);
-    triangles[i].vertice[2] = convert_endian (triangles[i].vertice[2]);
+    triangles[i].facefront = csConvertEndian (triangles[i].facefront);
+    triangles[i].vertice[0] = csConvertEndian (triangles[i].vertice[0]);
+    triangles[i].vertice[1] = csConvertEndian (triangles[i].vertice[1]);
+    triangles[i].vertice[2] = csConvertEndian (triangles[i].vertice[2]);
   }
 
   // frames ops
@@ -216,7 +217,7 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
     int32 typeframe = 0;
     if (fread(&typeframe, sizeof(typeframe), 1, f) != 1)
       return setError("Error reading mdl file", f);
-    typeframe = convert_endian (typeframe);
+    typeframe = csConvertEndian (typeframe);
 
     if (typeframe == 0) // one animation frame
     {
@@ -254,7 +255,7 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
       if (fread(&framesets[i].nbframes, sizeof(framesets[i].nbframes),
                 1, f) != 1)
         return setError("Error reading mdl file", f);
-      framesets[i].nbframes = convert_endian (framesets[i].nbframes);
+      framesets[i].nbframes = csConvertEndian (framesets[i].nbframes);
 
       framesets[i].delay = new float[framesets[i].nbframes];
       framesets[i].frames = new frame_t[framesets[i].nbframes];
@@ -271,7 +272,7 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
       if (fread(framesets[i].delay,sizeof(float)*framesets[i].nbframes,1,f)!=1)
         return setError("Error reading mdl file", f);
       for (ii = 0 ; ii < framesets[i].nbframes; ii++)
-        framesets[i].delay[ii] = convert_endian (framesets[i].delay[ii]);
+        framesets[i].delay[ii] = csConvertEndian (framesets[i].delay[ii]);
 
       // read all frames in frameset
       for (j = 0; j < framesets[i].nbframes; j++)
@@ -304,11 +305,11 @@ bool Mdl::ReadMDLFile(const char* mdlfile)
 
 void Mdl::dumpstats(FILE* s) const
 {
-  fprintf(s, "\nQuake Model (MDL) Statistics:\n");
-  fprintf(s, "Skins:     %d\n", nbskins);
-  fprintf(s, "Frames:    %d\n", nbframesets);
-  fprintf(s, "Triangles: %d\n", nbtriangles);
-  fprintf(s, "Vertices:  %d\n", nbvertices);
+  csFPrintf(s, "\nQuake Model (MDL) Statistics:\n");
+  csFPrintf(s, "Skins:     %d\n", nbskins);
+  csFPrintf(s, "Frames:    %d\n", nbframesets);
+  csFPrintf(s, "Triangles: %d\n", nbtriangles);
+  csFPrintf(s, "Vertices:  %d\n", nbvertices);
 }
 
 bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
@@ -324,7 +325,7 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
 
   if (spritename == 0 || strlen(spritename) == 0)
   {
-    fprintf(stderr, "Unable to save: 0 sprite name\n");
+    csFPrintf(stderr, "Unable to save: 0 sprite name\n");
     return false;
   }
 
@@ -334,23 +335,24 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
 
   // generate skin texture
   if (nbskins < 1)
-    fprintf(stderr, "Warning: no skin in this model\n");
+    csFPrintf(stderr, "Warning: no skin in this model\n");
   else
   {
-    if (mdl2spr_imageio == 0)
+    //if (mdl2spr_imageio == 0)
+    if (!mdl2spr_imageio.IsValid ())
     {
       mdl2spr_imageio =
         SCF_CREATE_INSTANCE ("crystalspace.graphic.image.io.png", iImageIO);
     }
     if (mdl2spr_imageio == 0)
-      fprintf (stderr, "Unable to load PNG plugin; can't save skins.\n");
+      csFPrintf (stderr, "Unable to load PNG plugin; can't save skins.\n");
     else
     {
-      printf("Generate skin texture file\n");
+      csPrintf("Generate skin texture file\n");
 
       if (resizeSkin)
       {
-	printf("\tOld Skin size = %dx%d\n", skinwidth, skinheight);
+	csPrintf("\tOld Skin size = %dx%d\n", skinwidth, skinheight);
 
 	x = (float) skinwidth;
 	y = (float) skinheight;
@@ -359,11 +361,11 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
 	while (x > 1) { x /= 2; spr.skinwidth  *= 2; }
 	while (y > 1) { y /= 2; spr.skinheight *= 2; }
 
-	printf("\tNew Skin size = %dx%d\n", spr.skinwidth, spr.skinheight);
+	csPrintf("\tNew Skin size = %dx%d\n", spr.skinwidth, spr.skinheight);
       }
       else // resizeSkin == false
       {
-	printf("\tSkin size = %dx%d\n", skinwidth, skinheight);
+	csPrintf("\tSkin size = %dx%d\n", skinwidth, skinheight);
 	spr.skinwidth  = skinwidth;
 	spr.skinheight = skinheight;
       }
@@ -372,8 +374,7 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
       const unsigned char* const palette = DefaultQuakePalette;
       spr.skins = new skin_t [nbskins];
 
-      char *skinfilename;
-      skinfilename = new char [strlen(spritename)+10];
+      csString skinfilename;
 
       for (i = 0; i < nbskins; i++)
       {
@@ -384,11 +385,11 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
 	    new unsigned char [spr.skinwidth * spr.skinheight];
 
 	  if (skins[i].nbtexs > 1)
-	    sprintf(skinfilename, "%s%d%c.png", spritename, i, j + 'a');
+	    skinfilename.Format ("%s%d%c.png", spritename, i, j + 'a');
 	  else if (nbskins > 1)
-	    sprintf(skinfilename, "%s%d.png", spritename, i);
+	    skinfilename.Format ("%s%d.png", spritename, i);
 	  else
-	    sprintf(skinfilename, "%s.png", spritename);
+	    skinfilename.Format ("%s.png", spritename);
 
 	  if (!resizeSkin)
 	    for (k = 0; k < skinwidth * skinheight; k++)
@@ -418,27 +419,26 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
 	    if (f)
 	      n = fwrite (db->GetData (), 1, db->GetSize (), f);
 	    if (!f || n != db->GetSize ())
-	      fprintf(stderr, "Error when writing `%s'.\n", skinfilename);
+	      csFPrintf(stderr, "Error when writing `%s'.\n", skinfilename.GetData());
 	    if (f) fclose (f);
 	  }
 	}
       }
-      delete[] skinfilename;
     }
   }
 
   if ((f = fopen(spritefilename, "w")) == 0)
   {
-    fprintf(stderr, "Cannot open sprite file %s for writing\n", spritename);
+    csFPrintf(stderr, "Cannot open sprite file %s for writing\n", spritename);
     return false;
   }
 
   // begin hard work now
-  fprintf(f, "<meshfact name=\"%s\">\n", spritename);
-  fprintf(f, "\t<plugin>crystalspace.mesh.loader.factory.sprite.3d</plugin>\n");
-  fprintf(f, "\t<params>\n");
-  fprintf(f, "\t\t<material>%s</material>\n", spritename);
-  printf("Generate MDL/SPR vertex correspondence\n");
+  csFPrintf(f, "<meshfact name=\"%s\">\n", spritename);
+  csFPrintf(f, "\t<plugin>crystalspace.mesh.loader.factory.sprite.3d</plugin>\n");
+  csFPrintf(f, "\t<params>\n");
+  csFPrintf(f, "\t\t<material>%s</material>\n", spritename);
+  csPrintf("Generate MDL/SPR vertex correspondence\n");
 
   // count back seam vertices
   long back_seam_verts = 0;
@@ -464,7 +464,7 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
       back_seam_verts++;
     }
 
-  printf("\t%ld back seam vertices detected\n", back_seam_verts);
+  csPrintf("\t%ld back seam vertices detected\n", back_seam_verts);
 
   // create sprite skin vertices
   spr.triangles = new triangle_t [nbtriangles];
@@ -526,12 +526,12 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
   if (maxFrames!=-1)
 	outFrames = maxFrames;
 
-  printf("Generate Frames\n");
+  csPrintf("Generate Frames\n");
   for (i = 0; i < outFrames; i++)
   {
     for (j = 0; j < framesets[i].nbframes; j++)
     {
-      fprintf(f, "\t\t<frame name=\"%s\">", framesets[i].frames[j].name);
+      csFPrintf(f, "\t\t<frame name=\"%s\">", framesets[i].frames[j].name);
       for (k = 0; k < nbvertices + back_seam_verts; k++)
       {
         // it seem y and z are switched
@@ -552,28 +552,27 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
         u = u / (float) skinwidth;
         v = v / (float) skinheight;
 
-        fprintf(f, " <v x=\"%.3f\" y=\"%.3f\" z=\"%.3f\" u=\"%.2f\" v=\"%.2f\"/>", x, y, z, u, v);
+        csFPrintf(f, " <v x=\"%.3f\" y=\"%.3f\" z=\"%.3f\" u=\"%.2f\" v=\"%.2f\"/>", x, y, z, u, v);
       }
-      fprintf(f, " </frame>\n");
+      csFPrintf(f, " </frame>\n");
     }
   }
 
-  printf("Generate Actions\n");
+  csPrintf("Generate Actions\n");
   for (i = 0; i < outFrames; i++)
   {
     if (framesets[i].group)
     {
-      char name_action[64];
-      memset(name_action, 0, 64);
+      csString name_action;
 
-      int base_action = strlen(framesets[i].frames[0].name);
-      for (j = strlen(framesets[i].frames[0].name) - 1; j > 1; j--)
+      size_t base_action = strlen(framesets[i].frames[0].name);
+      for (j = (int)strlen(framesets[i].frames[0].name) - 1; j > 1; j--)
         if (!isdigit(framesets[i].frames[0].name[j]))
 	  break;
       base_action=j + 1;
-      strncpy(name_action, framesets[i].frames[0].name, base_action);
+      name_action.Append (framesets[i].frames[0].name, base_action);
 
-      fprintf(f, "\t\t<action name=\"%s\">", name_action);
+      csFPrintf(f, "\t\t<action name=\"%s\">", name_action.GetData());
 
       for (j = 0; j < framesets[i].nbframes; j++)
       {
@@ -581,31 +580,30 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
         if (j > 0)
 	  delay -= framesets[i].delay[j - 1];
         delay *= 1000.0;
-        fprintf(f, " <f name=\"%s\" delay=\"%d\"/>", framesets[i].frames[j].name, (int)delay);
+        csFPrintf(f, " <f name=\"%s\" delay=\"%d\"/>", framesets[i].frames[j].name, (int)delay);
       }
-      fprintf(f, " </action>\n");
+      csFPrintf(f, " </action>\n");
     }
     else if (actionNamingMdl)
     {
       if (!isdigit(
         framesets[i].frames[0].name[strlen(framesets[i].frames[0].name) - 1]))
-        fprintf(f, "\t\t<action name=\"%s\"> <f name=\"%s\" delay=\"%d\"/> </action>\n",
+        csFPrintf(f, "\t\t<action name=\"%s\"> <f name=\"%s\" delay=\"%d\"/> </action>\n",
 	  framesets[i].frames[0].name, framesets[i].frames[0].name, delayMdl);
       else
       {
-        char name_action[64];
-        memset(name_action, 0, 64);
+        csString name_action;
 
         int base_action;
-        for (j = strlen(framesets[i].frames[0].name) - 1; j > 1; j--)
+        for (j = (int)strlen(framesets[i].frames[0].name) - 1; j > 1; j--)
           if (!isdigit(framesets[i].frames[0].name[j]))
 	    break;
         base_action=j + 1;
-        strncpy(name_action, framesets[i].frames[0].name, base_action);
+        name_action.Append (framesets[i].frames[0].name, base_action);
 
-        fprintf(f, "\t\t<action name=\"%s\">", name_action);
+        csFPrintf(f, "\t\t<action name=\"%s\">", name_action.GetData());
 
-        fprintf(f, " <f name=\"%s\" delay=\"%d\"/>", framesets[i].frames[0].name, delayMdl);
+        csFPrintf(f, " <f name=\"%s\" delay=\"%d\"/>", framesets[i].frames[0].name, delayMdl);
         for (j = i + 1; j < outFrames; j++, i++)
         {
           if (framesets[j].group)
@@ -627,25 +625,25 @@ bool Mdl::WriteSPR(const char* spritename, float scaleMdl, int delayMdl,
 	    break;
 
           if (strncmp(name_action,framesets[j].frames[0].name,base_action)==0)
-            fprintf(f, " <f name=\"%s\" delay=\"%d\"/>", framesets[j].frames[0].name, delayMdl);
+            csFPrintf(f, " <f name=\"%s\" delay=\"%d\"/>", framesets[j].frames[0].name, delayMdl);
           else
 	    break;
         }
-        fprintf(f, " </action>\n");
+        csFPrintf(f, " </action>\n");
       }
     }
   }
 
-  printf("Generate Triangles\n");
+  csPrintf("Generate Triangles\n");
   for (i = 0; i < nbtriangles; i++)
   {
-    fprintf(f, "\t\t<t v1=\"%d\" v2=\"%d\" v3=\"%d\"/>\n",
+    csFPrintf(f, "\t\t<t v1=\"%" PRId32 "\" v2=\"%" PRId32 "\" v3=\"%" PRId32 "\"/>\n",
       spr.triangles[i].vertice[0],
       spr.triangles[i].vertice[1],
       spr.triangles[i].vertice[2]);
   }
 
-  fprintf(f, "\t</params>\n</meshfact>\n");
+  csFPrintf(f, "\t</params>\n</meshfact>\n");
   fclose(f);
   return true;
 }
