@@ -20,33 +20,35 @@
     It only accepts models within zip files (for now).
 */
 
+#include "cssysdef.h"
 #include <stdarg.h>
 #include <ctype.h>
 
-#include "cssysdef.h"
-#include "md32spr.h"
-#include "bufio.h"
-#include "ivaria/reporter.h"
-#include "iutil/document.h"
-#include "iutil/vfs.h"
-#include "iutil/databuff.h"
-#include "iutil/cmdline.h"
-#include "iutil/objreg.h"
 #include "csgeom/transfrm.h"
 #include "cstool/initapp.h"
-#include "csutil/xmltiny.h"
+#include "csutil/archive.h"
 #include "csutil/csstring.h"
 #include "csutil/memfile.h"
-#include "csutil/archive.h"
+#include "csutil/xmltiny.h"
+#include "iutil/cmdline.h"
+#include "iutil/databuff.h"
+#include "iutil/document.h"
+#include "iutil/objreg.h"
+#include "iutil/stringarray.h"
+#include "iutil/vfs.h"
+#include "ivaria/reporter.h"
+
+#include "md32spr.h"
+#include "bufio.h"
 
 CS_IMPLEMENT_APPLICATION
 
-#if defined (OS_DOS)
+#if defined (CS_PLATFORM_DOS)
   #define IS_A_PATH_SEPARATOR(c)	\
-    (((c) == PATH_SEPARATOR) || ((c) == '/') || ((c) == ':'))
+    (((c) == CS_PATH_SEPARATOR) || ((c) == '/') || ((c) == ':'))
 #else
   #define IS_A_PATH_SEPARATOR(c)	\
-    (((c) == PATH_SEPARATOR) || ((c) == '\\') || ((c) == '/'))
+    (((c) == CS_PATH_SEPARATOR) || ((c) == '\\') || ((c) == '/'))
 #endif
 
 #define THRESHOLD 0.0000001
@@ -55,20 +57,20 @@ CS_IMPLEMENT_APPLICATION
 void splitpath (const char *iPathName, char *oPath, size_t iPathSize,
 		char *oName, size_t iNameSize);
 char *stristr(const char *String, const char *Pattern);
-char *basename(char *path, char *base);
+char *basename(const char *path, char *base);
 char *filename(char *path, char *file);
 char *lowercase(char *str);
 
 void Usage() {
-  printf("MD3 to CS XML converter\n");
-  printf("by Manjunath Sripadarao\n");
-  printf("Usage: md32spr [options] <model-file.zip> [-o=<output-file.zip>]\n");
-  printf("Options:\n");
-  printf("-wdir=<dir>\tDirectory in which the weapon files are stored.\n");
-  printf("\t\texample: -wdir=railgun\n");
-  printf("-o=<file.zip>\tOutput zip file name. \n");
-  printf("-scale=<float>\tHow much the model should be scaled. Default is 1.\n");
-  printf("\t\texample: -scale=4096. Model will be scaled as 1/4096.\n");
+  csPrintf("MD3 to CS XML converter\n");
+  csPrintf("by Manjunath Sripadarao\n");
+  csPrintf("Usage: md32spr [options] <model-file.zip> [-o=<output-file.zip>]\n");
+  csPrintf("Options:\n");
+  csPrintf("-wdir=<dir>\tDirectory in which the weapon files are stored.\n");
+  csPrintf("\t\texample: -wdir=railgun\n");
+  csPrintf("-o=<file.zip>\tOutput zip file name. \n");
+  csPrintf("-scale=<float>\tHow much the model should be scaled. Default is 1.\n");
+  csPrintf("\t\texample: -scale=4096. Model will be scaled as 1/4096.\n");
 }
 
 const int MD32spr::NUM_ACTIONS = 25;
@@ -141,7 +143,7 @@ void MD3Model::Init(csRef < iVFS > *vfsRef)
   vfs = vfsRef;
 }
 
-char *MD3Model::GetFileName()
+const char *MD3Model::GetFileName()
 {
   return fileName.GetData();
 }
@@ -179,8 +181,9 @@ MD32spr::~MD32spr()
 
 void MD32spr::Main()
 {
-  char *fileName, *dirName, *mountName, *weaponPath;
-  int i = 0;
+  char *fileName, *dirName;
+  csString mountName, weaponPath;
+  size_t i = 0;
   bool head = false, torso = false, leg = false;
   //csString baseName;
 
@@ -231,7 +234,6 @@ void MD32spr::Main()
 
   fileName = (char *) malloc(sizeof(char) * name.Length() + 1);
   dirName = (char *) malloc(sizeof(char) * name.Length() + 1);
-  mountName = (char *) malloc(sizeof(char) * (name.Length() + 12));	// /tmp/ + _data + 2 bytes padding.
 
   /*
      splitpath(name.GetData(), dirName, name.Length(), fileName, name.Length());
@@ -246,7 +248,7 @@ void MD32spr::Main()
      there is an ending slash, like /tmp/%s_data/. It mounts the zip file but vfs->FindFiles(char *dirname)
      doesn't work without the ending slash.
    */
-  sprintf(mountName, "/tmp/%s_data/", fileName);
+  mountName.Format ("/tmp/%s_data/", fileName);
   mountPath = mountName;
   /* Now we mount the .zip file for further processing */
   vfs->Mount(mountName, name.GetData());
@@ -348,10 +350,9 @@ void MD32spr::Main()
 
   }
 
-  if (weaponDir.Length() > 0) {
-    weaponPath =
-      new char[strlen(mountName) + weaponDir.Length() + 2];
-    sprintf(weaponPath, "%s%s/", mountName, weaponDir.GetData());
+  if (weaponDir.Length() > 0) 
+  {
+    weaponPath.Format ("%s%s/", mountName.GetData(), weaponDir.GetData());
     vfs->ChDir(weaponPath);
     weaponFiles = vfs->FindFiles(weaponPath);
     for (i = 0; i < weaponFiles->Length(); i++) {
@@ -394,7 +395,7 @@ bool MD3Model::ReadHeader(char **buf)
   /* The header is 108 bytes long... */
   header = (md3Header *) malloc(sizeof(md3Header));
   memcpy(header, bufPtr, sizeof(md3Header));
-  //printf("%c%c%c%c\n", header->ID[0], header->ID[1], 
+  //csPrintf("%c%c%c%c\n", header->ID[0], header->ID[1], 
   // header->ID[2], header->ID[3]);
 
   if (header->ID[0] != 'I' || header->ID[1] != 'D'
@@ -414,9 +415,10 @@ bool MD3Model::ReadHeader(char **buf)
 bool MD32spr::ReadVfsDir()
 {
   char *str;
-  int i = 0;
+  size_t i = 0;
 
-  for (i = 0; i < fileNames->Length(); i++) {
+  for (i = 0; i < fileNames->Length(); i++) 
+  {
     str = (char*)fileNames->Get(i);
 
     if (str) {
@@ -541,12 +543,15 @@ bool MD3Model::LoadSkin(char *skinFile)
     return false;
 
   bufPtr = **buf;
-  for (i = 0; i < (int) buf->GetSize(); i++) {
-    if (bufPtr[i] == '\n' || bufPtr[i] == '\r') {
+  for (i = 0; i < (int) buf->GetSize(); i++) 
+  {
+    if (bufPtr[i] == '\n' || bufPtr[i] == '\r') 
+    {
       for (j = 0; j < header->numMeshes; j++)
-	if (stristr(line, meshes[j].meshHeader->name)) {
-	  int k = line.Length();
-	  char *name = new char[line.Length() + 1];
+	if (stristr(line, meshes[j].meshHeader->name)) 
+	{
+	  size_t k = line.Length();
+	  char *name = new char[k + 1];
 	  strcpy(name, line.GetData());
 	  if(strtok(name, ",") != 0)
 	    strcpy(meshes[j].skins[0].name, strtok(0, ","));
@@ -556,7 +561,9 @@ bool MD3Model::LoadSkin(char *skinFile)
 	  strcpy(fileName, (line.GetData() + k + 1));
 	}
       line.Clear();
-    } else {
+    } 
+    else 
+    {
       line.Append(bufPtr[i]);
     }
   }
@@ -566,7 +573,8 @@ bool MD3Model::LoadSkin(char *skinFile)
 
 bool MD32spr::LoadAnimation(char *animFile)
 {
-  int lineLen = 0, tmp = -1, i = 0;
+  size_t lineLen = 0;
+  int tmp = -1, i = 0;
   int headActions = 1, upperActions = 0, lowerActions = 0;
   char *line;
   size_t fileSz;
@@ -632,8 +640,8 @@ bool MD32spr::LoadAnimation(char *animFile)
 
 void MD32spr::Write()
 {
-  int i = 0;
-  char* vfspath = 0;
+  size_t i = 0;
+  csString vfspath;
   char* fileName = 0;
   char* mdlName = 0;
 
@@ -650,10 +658,9 @@ void MD32spr::Write()
       }
 
       delete zipFile;
-      vfspath = new char[outZipName.Length() + 12];
       fileName = new char[outZipName.Length() + 1];
       basename(outZipName.GetData(), fileName);
-      sprintf(vfspath, "/tmp/%s_out/", fileName);
+      vfspath.Format ("/tmp/%s_out/", fileName);
 
       if (!out->Mount(vfspath, outZipName.GetData()))
       {
@@ -688,12 +695,14 @@ void MD32spr::Write()
     }
   }
 
-  if (player) {
-    if (headModel) {
+  if (player) 
+  {
+    if (headModel) 
+    {
       char *headName = new char[strlen(headModel->GetFileName()) + 1];
-      char *tagFileName = new char[strlen(headModel->GetFileName()) + strlen(vfspath) + strlen(".tag") + 1];
+      csString tagFileName;
       basename(headModel->GetFileName(), headName);
-      sprintf(tagFileName,"%s%s.tag", vfspath, headName);
+      tagFileName.Format ("%s%s.tag", vfspath.GetData(), headName);
       WriteXMLTags(headModel, tagFileName);
       csRef < iDocumentSystem > xml(csPtr < iDocumentSystem >
 				    (new csTinyDocumentSystem()));
@@ -709,9 +718,9 @@ void MD32spr::Write()
     }
     if (upperModel) {
       char *upperName = new char[strlen(headModel->GetFileName()) + 1];
-      char *tagFileName = new char[strlen(upperModel->GetFileName()) + strlen(vfspath) + strlen(".tag") + 1];
+      csString tagFileName;
       basename(upperModel->GetFileName(), upperName);
-      sprintf(tagFileName,"%s%s.tag", vfspath, upperName);
+      tagFileName.Format ("%s%s.tag", vfspath.GetData(), upperName);
       WriteXMLTags(upperModel, tagFileName);
       csRef < iDocumentSystem > xml(csPtr < iDocumentSystem >
 				    (new csTinyDocumentSystem()));
@@ -728,9 +737,9 @@ void MD32spr::Write()
     }
     if (lowerModel) {
       char *lowerName = new char[strlen(headModel->GetFileName()) + 1];
-      char *tagFileName = new char[strlen(upperModel->GetFileName()) + strlen(vfspath) + strlen(".tag") + 1];
+      csString tagFileName;
       basename(lowerModel->GetFileName(), lowerName);
-      sprintf(tagFileName,"%s%s.tag", vfspath, lowerName);
+      tagFileName.Format ("%s%s.tag", vfspath.GetData(), lowerName);
       WriteXMLTags(lowerModel, tagFileName);
       csRef < iDocumentSystem > xml(csPtr < iDocumentSystem >
 				    (new csTinyDocumentSystem()));
@@ -760,10 +769,7 @@ void MD32spr::Write()
 
 	WriteGeneric(mdl, parent);
 	char *fileName = new char[mdl->fileName.Length() + 1];
-	char *tagFileName = new char[strlen(vfspath) + 
-				     strlen(weaponDir) + 
-				     strlen("/") + mdl->fileName.Length() + 
-				     strlen(".tag") + 1];
+	csString tagFileName;
 	basename(mdl->fileName.GetData(), fileName);
 
 
@@ -772,7 +778,7 @@ void MD32spr::Write()
 	outFile += "/";
 	outFile += fileName;
 	doc->Write(out, outFile.GetData());
-	sprintf(tagFileName,"%s%s/%s.tag", vfspath, weaponDir.GetData(), fileName);
+	tagFileName.Format ("%s%s/%s.tag", vfspath.GetData(), weaponDir.GetData(), fileName);
 	WriteXMLTags(mdl, tagFileName);
       }
     }
@@ -790,7 +796,7 @@ void MD32spr::Write()
 }
 
 void MD32spr::WriteXMLTags(md3Model * model,
-			char *tagFileName) 
+			const char *tagFileName) 
 {
   int i = 0, j = 0, k = 0;
   float *rotationMatrix, *translationVector;
@@ -921,12 +927,12 @@ void MD32spr::WriteGeneric(md3Model * model,
 
       csRef < iDocumentNode > frame;
       for (j = 0; j < model->meshes[i].meshHeader->numMeshFrames; j++) {
-	char fNum[10];
+	csString fNum;
 	int numVertices = model->meshes[i].meshHeader->numVertices;
 
 	frame = child->CreateNodeBefore(CS_NODE_ELEMENT, 0);
 	frame->SetValue("frame");
-	sprintf(fNum, "f%d", j);
+	fNum.Format ("f%d", j);
 	frame->SetAttribute("name", fNum);
 
 	for (k = 0; k < model->meshes[i].meshHeader->numVertices; k++) {
@@ -957,11 +963,12 @@ void MD32spr::WriteGeneric(md3Model * model,
 	    action->SetValue("action");
 	    action->SetAttribute("name", "default");
 
-	    for (k = 0; k < model->meshes[i].meshHeader->numMeshFrames; k++) {
-	      char fNum[10];
+	    for (k = 0; k < model->meshes[i].meshHeader->numMeshFrames; k++) 
+	    {
+	      csString fNum;
 	      csRef < iDocumentNode > f =
 		action->CreateNodeBefore(CS_NODE_ELEMENT, 0);
-	      sprintf(fNum, "f%d", k);
+	      fNum.Format ("f%d", k);
 	      f->SetValue("f");
 	      f->SetAttribute("name", fNum);
 	      f->SetAttribute("delay", "1000");	/// 1000 is the default delay.
@@ -974,11 +981,12 @@ void MD32spr::WriteGeneric(md3Model * model,
 	  
 	    for(k = model->animInfo[j].startFrame; 
 		k < (model->animInfo[j].startFrame + model->animInfo[j].numFrames); 
-		k++) {
-	      char fNum[10];
+		k++) 
+	    {
+	      csString fNum;
 	      csRef < iDocumentNode > f =
 		action->CreateNodeBefore(CS_NODE_ELEMENT, 0);
-	      sprintf(fNum, "f%d", k);
+	      fNum.Format ("f%d", k);
 	      f->SetValue("f");
 	      f->SetAttribute("name", fNum);
 	      float delay = (1.0f/(float)model->animInfo[j].fps) * 1000;
@@ -992,11 +1000,12 @@ void MD32spr::WriteGeneric(md3Model * model,
 	action->SetValue("action");
 	action->SetAttribute("name", "default");
 
-	for (j = 0; j < model->meshes[i].meshHeader->numMeshFrames; j++) {
-	  char fNum[10];
+	for (j = 0; j < model->meshes[i].meshHeader->numMeshFrames; j++) 
+	{
+	  csString fNum;
 	  csRef < iDocumentNode > f =
 	    action->CreateNodeBefore(CS_NODE_ELEMENT, 0);
-	  sprintf(fNum, "f%d", j);
+	  fNum.Format ("f%d", j);
 	  f->SetValue("f");
 	  f->SetAttribute("name", fNum);
 	  f->SetAttribute("delay", "1000");	/// 1000 is the default delay.
@@ -1041,14 +1050,16 @@ void MD32spr::WriteGeneric(md3Model * model,
 
 void MD32spr::WriteTextures(const char *inPath, const char *outPath)
 {
-  int i = 0;
+  size_t i = 0;
   size_t fileSize;
   csRef <iStringArray> files = vfs->FindFiles(inPath);
   char fileName[100];
-  for(i = 0; i < files->Length(); i++) {
+  for(i = 0; i < files->Length(); i++) 
+  {
     char *str = (char*)files->Get(i);
 
-    if(stristr(str,".jpg") || stristr(str, ".png") || stristr(str, ".bmp") || stristr(str, ".tga")) { 
+    if(stristr(str,".jpg") || stristr(str, ".png") || stristr(str, ".bmp") || stristr(str, ".tga")) 
+    {
       filename(str, fileName);
       lowercase(fileName);
       csString csStr(outPath);
@@ -1111,7 +1122,7 @@ int main(int argc, char *argv[])
 char *stristr(const char *String, const char *Pattern)
 {
   char *pptr, *sptr, *start;
-  uint slen, plen;
+  size_t slen, plen;
 
   for (start = (char *) String,
        pptr = (char *) Pattern,
@@ -1147,11 +1158,11 @@ char *stristr(const char *String, const char *Pattern)
 
 /*----------------------------------------------------------------------------*/
 
-char *basename(char *path, char *base)
+char *basename(const char *path, char *base)
 {
   char *dir, *file;
   char *ptr, *point;
-  int sz = 0;
+  size_t sz = 0;
   if (!path || !base)
     return 0;
   if(!strlen(path))
@@ -1161,13 +1172,16 @@ char *basename(char *path, char *base)
   splitpath(path, dir, strlen(path), file, strlen(path));
   point = stristr(file, ".");
   ptr = file;
-  if(point != 0) {
+  if(point != 0) 
+  {
     while (ptr++ != point)
       sz++;
-  } else {
+  } 
+  else 
+  {
     sz = strlen(file);
   }
-  strncpy(base, file, sz);
+  memcpy (base, file, sz);
   base[sz] = '\0';
   return base;
 }
@@ -1175,7 +1189,7 @@ char *basename(char *path, char *base)
 char *filename(char *path, char *file)
 {
   char *dir, *fname;
-  int sz = 0;
+  size_t sz = 0;
   if (!path || !file)
     return 0;
   if(!strlen(path))
@@ -1184,8 +1198,10 @@ char *filename(char *path, char *file)
   fname = new char[strlen(path)];
   splitpath(path, dir, strlen(path), fname, strlen(path));
   sz = strlen(fname);
-  strncpy(file, fname, sz);
+  memcpy (file, fname, sz);
   file[sz] = '\0';
+  delete[] dir;
+  delete[] fname;
   return file;
 }
 
