@@ -1,10 +1,18 @@
 #include "cssysdef.h"
+#include "awstest_config.h"
+
 #include "cstool/initapp.h"
 #include "csutil/scf.h"
 #include "csutil/cfgfile.h"
 #include "ivaria/reporter.h"
-#include "iaws/aws.h"
-#include "awstest.h"
+
+#ifdef TEST_AWS2
+ #include "iaws/aws2.h"
+ #include "awstest2.h"
+#else
+ #include "iaws/aws.h"
+ #include "awstest.h"
+#endif
 
 #include <stdio.h>
 
@@ -15,10 +23,11 @@ CS_IMPLEMENT_APPLICATION
 awsTest *System;
 
 
+
 int
 main(int argc, char *argv[])
 {
-  printf("Beginning test of AWS as a plugin...\n");
+  csPrintf("Beginning test of AWS as a plugin...\n");
 
   // Create our main class.
   System = new awsTest();
@@ -32,9 +41,12 @@ main(int argc, char *argv[])
   }
 
   // Main loop.
+  iObjectRegistry* object_reg = System->object_reg;
   csDefaultRunLoop(System->object_reg);
 
-  delete System;
- return 0;
+  delete System; System = 0;
+  csInitializer::DestroyApplication (object_reg);
+
+  return 0;
 }
 
