@@ -1,5 +1,5 @@
-#ifndef AWSTEST_H
-#define AWSTEST_H
+#ifndef __AWSTEST_H__
+#define __AWSTEST_H__
 
 #include <stdarg.h>
 #include "csgeom/math2d.h"
@@ -29,25 +29,28 @@ class awsTest
 {
 public:
   iObjectRegistry* object_reg;
+  csEventID Process;
+  csEventID FinalProcess;
+  csEventID KeyboardDown;
 
 private:
-  iEngine* engine;
-  iAws *aws;
-  iAwsPrefManager *awsprefs;
-  iAwsCanvas *awsCanvas;
+  csRef<iEngine> engine;
+  csRef<iAws> aws;
+  iAwsPrefManager* awsprefs;
+  csRef<iAwsCanvas> awsCanvas;
   iSector* room;
-  iView* view, *wview;
-  iGraphics3D *myG3D;
-  iGraphics2D *myG2D;
-  iVFS *myVFS;
-  iConsoleOutput *myConsole;
-  iLoader* loader;
-  iVirtualClock* vc;
+  csRef<iView> view, wview;
+  csRef<iGraphics3D> myG3D;
+  csRef<iGraphics2D> myG2D;
+  csRef<iVFS> myVFS;
+  csRef<iConsoleOutput> myConsole;
+  csRef<iLoader> loader;
+  csRef<iVirtualClock> vc;
 
-  iFont* font;
+  csRef<iFont> font;
   int col_red, col_blue, col_white, col_black;
   int col_yellow, col_cyan, col_green, col_gray;
-  char message[255];
+  csString message;
   csTicks message_timer;
   bool message_error;
 
@@ -65,4 +68,4 @@ public:
   bool HandleEvent (iEvent &Event);
 };
 
-#endif // AWSTEST_H
+#endif // __AWSTEST_H__
