@@ -18,46 +18,52 @@
 */
 
 #include "cssysdef.h"
+
+#include "csgeom/tri.h"
+#include "csgeom/vector3.h"
+#include "cstool/csview.h"
+#include "cstool/initapp.h"
+#include "csutil/cmdhelp.h"
+#include "csutil/cscolor.h"
+#include "csutil/event.h"
 #include "csutil/sysfunc.h"
 #include "csutil/xmltiny.h"
-#include "cstool/initapp.h"
-#include "csgeom/vector3.h"
-#include "apps/tests/csbench/csbench.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/natwin.h"
-#include "ivideo/txtmgr.h"
-#include "ivideo/fontserv.h"
-#include "ivideo/material.h"
-#include "ivaria/conout.h"
-#include "igraphic/imageio.h"
-#include "igraphic/image.h"
-#include "iutil/cmdline.h"
-#include "iutil/event.h"
-#include "iutil/objreg.h"
-#include "iutil/eventh.h"
-#include "iutil/eventq.h"
-#include "iutil/comp.h"
-#include "iutil/virtclk.h"
-#include "csutil/cmdhelp.h"
-#include "ivaria/reporter.h"
-#include "iutil/plugin.h"
-#include "iutil/vfs.h"
-#include "csutil/event.h"
-#include "imesh/thing.h"
-#include "imesh/genmesh.h"
-#include "imesh/object.h"
-#include "iengine/mesh.h"
-#include "iengine/sector.h"
 #include "iengine/camera.h"
 #include "iengine/material.h"
+#include "iengine/mesh.h"
 #include "iengine/renderloop.h"
-#include "iengine/rendersteps/irsfact.h"
+#include "iengine/rendersteps/icontainer.h"
 #include "iengine/rendersteps/igeneric.h"
 #include "iengine/rendersteps/irenderstep.h"
-#include "iengine/rendersteps/icontainer.h"
+#include "iengine/rendersteps/irsfact.h"
+#include "iengine/sector.h"
+#include "igraphic/image.h"
+#include "igraphic/imageio.h"
+#include "imesh/genmesh.h"
+#include "imesh/object.h"
+#include "imesh/thing.h"
+#include "iutil/cmdline.h"
+#include "iutil/comp.h"
+#include "iutil/databuff.h"
+#include "iutil/event.h"
+#include "iutil/eventh.h"
+#include "iutil/eventq.h"
+#include "iutil/object.h"
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "iutil/vfs.h"
+#include "iutil/virtclk.h"
+#include "ivaria/conout.h"
+#include "ivaria/reporter.h"
 #include "ivaria/view.h"
-#include "cstool/csview.h"
+#include "ivideo/fontserv.h"
+#include "ivideo/graph2d.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/material.h"
+#include "ivideo/natwin.h"
+#include "ivideo/txtmgr.h"
+
+#include "apps/tests/csbench/csbench.h"
 
 CS_IMPLEMENT_APPLICATION
 
@@ -114,52 +120,14 @@ static void Cleanup ()
   csInitializer::DestroyApplication (object_reg);
 }
 
-bool CsBench::CreateGeometry ()
+iMeshFactoryWrapper* CsBench::CreateGenmeshLattice (int dim, float size,
+	const char* name)
 {
-  // First disable the lighting cache. Our app is simple enough
-  // not to need this.
-  engine->SetLightingCacheMode (0);
-
-  // Load the texture from the standard library.  This is located in
-  // CS/data/standard.zip and mounted as /lib/std using the Virtual
-  // File System (VFS) plugin.
-  if (!loader->LoadTexture ("stone", "/lib/std/stone4.gif"))
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple1",
-        "Error loading 'stone4' texture!");
-    return false;
-  }
-  if (!loader->LoadTexture ("stone_normal", "/lib/std/stone2DOT3.png", CS_TEXTURE_3D,
-    0, false, false))
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple1",
-        "Error loading 'stone2DOT3' texture!");
-    return false;
-  }
-  material = engine->GetMaterialList ()->FindByName ("stone");
-  csShaderVariable* normalSV = 
-    material->GetMaterial()->GetVariableAdd (strings->Request ("tex normal"));
-  normalSV->SetValue (engine->GetTextureList()->FindByName ("stone_normal"));
-
-  room2 = engine->CreateSector ("room2");
-  csRef<iMeshWrapper> walls (engine->CreateSectorWallsMesh (room2, "walls"));
-  csRef<iThingState> ws =
-    SCF_QUERY_INTERFACE (walls->GetMeshObject (), iThingState);
-  csRef<iThingFactoryState> walls_state = ws->GetFactory ();
-  walls_state->AddInsideBox (csVector3 (-5, -5, 5), csVector3 (5, 5, 15));
-  walls_state->SetPolygonMaterial (CS_POLYRANGE_LAST, material);
-  walls_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST, 3);
-
   // Create our object.
   csRef<iMeshFactoryWrapper> fact = engine->CreateMeshFactory (
-    "crystalspace.mesh.object.genmesh", "complexmesh");
+    "crystalspace.mesh.object.genmesh", name);
   csRef<iGeneralFactoryState> factstate = SCF_QUERY_INTERFACE (
     fact->GetMeshObjectFactory (), iGeneralFactoryState);
-  //int dim = 128;
-  int dim = 64;
-  float size = 5.0f;
   factstate->SetVertexCount (dim * dim);
   factstate->SetTriangleCount (2 * (dim-1) * (dim-1));
   int x, y;
@@ -185,29 +153,80 @@ bool CsBench::CreateGeometry ()
     }
 
   factstate->CalculateNormals ();
+  return fact;
+}
+
+bool CsBench::SetupMaterials ()
+{
+  // Load the texture from the standard library.  This is located in
+  // CS/data/standard.zip and mounted as /lib/std using the Virtual
+  // File System (VFS) plugin.
+  if (!loader->LoadTexture ("stone", "/lib/std/stone4.gif"))
+  {
+    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
+        "crystalspace.application.simple1",
+        "Error loading 'stone4' texture!");
+    return false;
+  }
+  if (!loader->LoadTexture ("stone_normal", "/lib/stdtex/stone2DOT3.png", 
+  	CS_TEXTURE_3D, 0, false, false))
+  {
+    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
+        "crystalspace.application.simple1",
+        "Error loading 'stone2DOT3' texture!");
+    return false;
+  }
+  material = engine->GetMaterialList ()->FindByName ("stone");
+  csShaderVariable* normalSV = 
+    material->GetMaterial()->GetVariableAdd (strings->Request ("tex normal"));
+  iTextureWrapper* stoneDot3 = 
+    engine->GetTextureList()->FindByName ("stone_normal");
+  stoneDot3->SetTextureClass ("normalmap");
+  normalSV->SetValue (stoneDot3);
+  return true;
+}
+
+iSector* CsBench::CreateRoom (const char* name, const char* meshname,
+	const csVector3& p1, const csVector3& p2)
+{
+  iSector* room2 = engine->CreateSector (name);
+  csRef<iMeshWrapper> walls = engine->CreateSectorWallsMesh (room2, meshname);
+  csRef<iThingFactoryState> walls_state = 
+    scfQueryInterface<iThingFactoryState> (walls->GetMeshObject ()->GetFactory());
+  walls_state->AddInsideBox (p1, p2);
+  walls_state->SetPolygonMaterial (CS_POLYRANGE_LAST, material);
+  walls_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST, 3);
+  return room2;
+}
+
+bool CsBench::CreateTestCaseSingleBigObject ()
+{
+  iSector* room2 = CreateRoom ("room2_single", "room_single",
+  	csVector3 (-5, -5, 5), csVector3 (5, 5, 15));
+  // Create our factory.
+  iMeshFactoryWrapper* fact = CreateGenmeshLattice (BIGOBJECT_DIM,
+  	5.0f, "complexmesh");
+  if (!fact) return false;
 
   // Now create an instance:
   csRef<iMeshWrapper> mesh =
     engine->CreateMeshWrapper (fact, "complex", room2, csVector3 (0, 0, 10.0));
-  genmesh = SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iGeneralMeshState);
+  csRef<iGeneralMeshState> genmesh = 
+    SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iGeneralMeshState);
   genmesh->SetMaterialWrapper (material);
+  
+  gmSingle = genmesh;
 
-  csRef<iLight> light;
+  csRef<iLight> l;
   iLightList* ll = room2->GetLights ();
+  l = engine->CreateLight (0, csVector3 (-3, 3, 10), 10, csColor (1, 0, 0));
+  ll->Add (l);
+  l = engine->CreateLight (0, csVector3 (3, 3,  10), 10, csColor (0, 0, 1));
+  ll->Add (l);
+  l = engine->CreateLight (0, csVector3 (0, 3, 7), 10, csColor (0, 1, 0));
+  ll->Add (l);
 
-  light = engine->CreateLight (0, csVector3 (-3, 3, 10), 10,
-    csColor (1, 0, 0));
-  ll->Add (light);
-
-  light = engine->CreateLight (0, csVector3 (3, 3,  10), 10,
-    csColor (0, 0, 1));
-  ll->Add (light);
-
-  light = engine->CreateLight (0, csVector3 (0, 3, 7), 10,
-    csColor (0, 1, 0));
-  ll->Add (light);
-
-  room1 = engine->CreateSector ("room1");
+  room_single = engine->CreateSector ("room_single");
   csVector3 portal_vts[4];
   portal_vts[0].Set (-1.5, 1.5, 0);
   portal_vts[1].Set (1.5, 1.5, 0);
@@ -215,16 +234,56 @@ bool CsBench::CreateGeometry ()
   portal_vts[3].Set (-1.5, -1.5, 0);
   iPortal* portal;
   csRef<iMeshWrapper> portal_mesh = engine->CreatePortal (
-  	"portal_room2", room1, csVector3 (0, 0, 5),
+  	"portal_room2_single", room_single, csVector3 (0, 0, 5),
   	room2, portal_vts, 4, portal);
+  return true;
+}
 
-  engine->Prepare ();
+bool CsBench::CreateTestCaseMultipleObjects ()
+{
+  iSector* room2 = CreateRoom ("room2_multi", "room_multi",
+  	csVector3 (-5, -5, 5), csVector3 (5, 5, 15));
+  // Create our factory.
+  iMeshFactoryWrapper* fact = CreateGenmeshLattice (SMALLOBJECT_DIM,
+  	4.0f, "smallmesh");
+  if (!fact) return false;
 
-  view = csPtr<iView> (new csView (engine, g3d));
-  view->GetCamera ()->SetSector (room1);
-  view->GetCamera ()->GetTransform ().SetOrigin (csVector3 (0, 0, 0));
-  iGraphics2D* g2d = g3d->GetDriver2D ();
-  view->SetRectangle (0, 0, g2d->GetWidth (), g2d->GetHeight ());
+  // Now create the instances:
+  int i;
+  csVector3 p (-9, -9, 9);
+  float step = 18.0f / float (SMALLOBJECT_NUM);
+  float zstep = 2.0f / float (SMALLOBJECT_NUM);
+  for (i = 0 ; i < SMALLOBJECT_NUM ; i++)
+  {
+    csRef<iMeshWrapper> mesh =
+      engine->CreateMeshWrapper (fact, "small", room2, p);
+    csRef<iGeneralMeshState> genmesh = 
+      SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iGeneralMeshState);
+    genmesh->SetMaterialWrapper (material);
+    p.x += step;
+    p.y += step;
+    p.z += zstep;
+  }
+
+  csRef<iLight> l;
+  iLightList* ll = room2->GetLights ();
+  l = engine->CreateLight (0, csVector3 (-3, 3, 10), 10, csColor (1, 0, 0));
+  ll->Add (l);
+  l = engine->CreateLight (0, csVector3 (3, 3,  10), 10, csColor (0, 0, 1));
+  ll->Add (l);
+  l = engine->CreateLight (0, csVector3 (0, 3, 7), 10, csColor (0, 1, 0));
+  ll->Add (l);
+
+  room_multi = engine->CreateSector ("room_multi");
+  csVector3 portal_vts[4];
+  portal_vts[0].Set (-1.5, 1.5, 0);
+  portal_vts[1].Set (1.5, 1.5, 0);
+  portal_vts[2].Set (1.5, -1.5, 0);
+  portal_vts[3].Set (-1.5, -1.5, 0);
+  iPortal* portal;
+  csRef<iMeshWrapper> portal_mesh = engine->CreatePortal (
+  	"portal_room2_multi", room_multi, csVector3 (0, 0, 5),
+  	room2, portal_vts, 4, portal);
   return true;
 }
 
@@ -237,7 +296,8 @@ bool CsBench::Initialize (int argc, const char* const argv[],
   // Make sure the commandline has -verbose and -console for consistent
   // results.
   cmdline = CS_QUERY_REGISTRY (object_reg, iCommandLineParser);
-  cmdline->AddOption ("verbose", "true");
+  cmdline->AddOption ("verbose", "-scf");
+  cmdline->AddOption ("console", 0);
 
   if (!csInitializer::SetupConfigManager (object_reg, iConfigName))
     return ReportError ("Couldn't initialize app!");
@@ -274,6 +334,7 @@ bool CsBench::Initialize (int argc, const char* const argv[],
   iNativeWindow* nw = g2d->GetNativeWindow ();
   if (nw) nw->SetTitle ("Crystal Space Graphics Benchmark App");
 
+  unlink ("csbench_report.zip");
   vfs->Mount ("/lib/report", "csbench_report.zip");
 
   csRef<iStandardReporterListener> stdrep = CS_QUERY_REGISTRY (object_reg,
@@ -295,15 +356,26 @@ bool CsBench::Initialize (int argc, const char* const argv[],
   if (!csInitializer::OpenApplication (object_reg))
     return ReportError ("Error opening system!");
 
-  if (!CreateGeometry ())
-    return false;
+  // First disable the lighting cache. Our app is simple enough
+  // not to need this.
+  engine->SetLightingCacheMode (0);
+
+  if (!SetupMaterials ()) return false;
+  if (!CreateTestCaseSingleBigObject ()) return false;
+  if (!CreateTestCaseMultipleObjects ()) return false;
+
+  engine->Prepare ();
+
+  view = csPtr<iView> (new csView (engine, g3d));
+  view->GetCamera ()->SetSector (room_single);
+  view->GetCamera ()->GetTransform ().SetOrigin (csVector3 (0, 0, 0));
+  view->SetRectangle (0, 0, g2d->GetWidth (), g2d->GetHeight ());
 
   return true;
 }
 
-#define BENCHTIME 3000
-
-void CsBench::BenchMark (const char* name, const char* description)
+float CsBench::BenchMark (const char* name, const char* description, 
+			 uint drawFlags)
 {
   Report ("================================================================");
   Report ("Benchmark %s (%s)...", name, description);
@@ -315,7 +387,7 @@ void CsBench::BenchMark (const char* name, const char* description)
   {
     cnt++;
 
-    g3d->BeginDraw (CSDRAW_CLEARSCREEN | CSDRAW_3DGRAPHICS);
+    g3d->BeginDraw (CSDRAW_CLEARSCREEN | CSDRAW_3DGRAPHICS | drawFlags);
     view->Draw ();
     g3d->FinishDraw ();
     g3d->Print (0);
@@ -326,7 +398,8 @@ void CsBench::BenchMark (const char* name, const char* description)
   csRef<iImage> shot = csPtr<iImage> (g2d->ScreenShot ());
   if (shot)
   {
-    csRef<iDataBuffer> shotbuf = imageio->Save (shot, "image/jpg", "");
+    csRef<iDataBuffer> shotbuf = imageio->Save (shot, "image/jpg", 
+      "compress=10");
     if (shotbuf)
     {
       csString filename;
@@ -337,8 +410,10 @@ void CsBench::BenchMark (const char* name, const char* description)
       	shotbuf->GetSize ());
     }
   }
-  Report ("Performance is %d frames in %d seconds: %g fps",
-  	cnt, BENCHTIME / 1000, float (cnt * 1000) / float (BENCHTIME));
+  float fps = float (cnt * 1000) / float (BENCHTIME);
+  Report ("PERF:%s:%d:%d:%g: (%d frames in %d seconds: %g fps)",
+  	name, cnt, BENCHTIME / 1000, fps,
+  	cnt, BENCHTIME / 1000, fps);
 
   // Due to some limitation in VFS we cannot let the stdrep plugin write
   // directly to a file in a zip archive. So at this point we copy the
@@ -348,6 +423,8 @@ void CsBench::BenchMark (const char* name, const char* description)
   	(const char*)buf->GetData (), buf->GetSize ());
 
   vfs->Sync ();
+  fflush (stdout);
+  return fps;
 }
 
 iShaderManager* CsBench::GetShaderManager ()
@@ -372,17 +449,40 @@ iDocumentSystem* CsBench::GetDocumentSystem ()
   return docsys;
 }
 
-void CsBench::PerformShaderTest (const char* shaderPath, const char* shname,
-		const char* shtype)
+void CsBench::PerformShaderTest (const char* shaderPath, const char* shtype, 
+				 const char* shaderPath2, const char* shtype2,
+				 iGeneralMeshState* genmesh)
 {
-  csRef<iDocument> shaderDoc = GetDocumentSystem ()->CreateDocument ();
   csRef<iShaderCompiler> shcom = GetShaderManager ()->GetCompiler ("XMLShader");
+  csRef<iDocument> shaderDoc = GetDocumentSystem ()->CreateDocument ();
   csRef<iFile> shaderFile = vfs->Open (shaderPath, VFS_FILE_READ);
-  shaderDoc->Parse (shaderFile);
+  shaderDoc->Parse (shaderFile, true);
   csRef<iDocumentNode> shadernode = shaderDoc->GetRoot ()->GetNode ("shader");
   csStringID shadertype = strings->Request (shtype);
+
+  csStringID shadertype2 = 0;
+  csRef<iShader> shader2;
+  if (shaderPath2)
+  {
+    csRef<iDocument> shaderDoc2 = GetDocumentSystem ()->CreateDocument ();
+    csRef<iFile> shaderFile2 = vfs->Open (shaderPath2, VFS_FILE_READ);
+    shaderDoc2->Parse (shaderFile2, true);
+    csRef<iDocumentNode> shadernode2 = shaderDoc2->GetRoot ()
+    	->GetNode ("shader");
+    shadertype2 = strings->Request (shtype2);
+    shader2 = shcom->CompileShader (shadernode2);
+  }
+
   csRef<iShaderPriorityList> prilist = shcom->GetPriorities (shadernode);
-  int i;
+  size_t i;
+  iMaterialWrapper* oldmat = material;
+  //csRef<iMeshWrapper> walls (engine->FindMeshObject ("walls"));
+  //csRef<iMeshWrapper> walls (
+    //view->GetCamera()->GetSector()->GetMeshes()->FindByName ("walls"));
+  csRef<iMeshWrapper> walls (engine->FindMeshObject (
+    view->GetCamera()->GetSector()->QueryObject()->GetName()));
+  csRef<iThingState> ws =
+    SCF_QUERY_INTERFACE (walls->GetMeshObject (), iThingState);
   for (i = 0 ; i < prilist->GetCount () ; i++)
   {
     int pri = prilist->GetPriority (i);
@@ -390,16 +490,26 @@ void CsBench::PerformShaderTest (const char* shaderPath, const char* shname,
     if (shader)
     {
       csRef<iMaterial> matinput = engine->CreateBaseMaterial (
-		      engine->GetTextureList ()->FindByName ("stone"));
+	engine->GetTextureList ()->FindByName ("stone"));
+      csShaderVariable* normalSV = 
+	matinput->GetVariableAdd (strings->Request ("tex normal"));
+      normalSV->SetValue (engine->GetTextureList()->FindByName (
+      	"stone_normal"));
       matinput->SetShader (shadertype, shader);
-      iMaterialWrapper* mat = engine->GetMaterialList ()->NewMaterial (matinput);
-      mat->Register (g3d->GetTextureManager ());
+      if (shader2)
+	matinput->SetShader (shadertype2, shader2);
+      iMaterialWrapper* mat = engine->GetMaterialList ()->NewMaterial (
+      	matinput, 0);
       genmesh->SetMaterialWrapper (mat);
-      char name[256];
-      sprintf (name, "%s_%d", shname, pri);
-      char description[256];
-      sprintf (description, "Shader %s with priority %d", shname, pri);
-      BenchMark (name, description);
+      ws->ReplaceMaterial (oldmat, mat);
+
+      csString name;
+      name.Format ("%s_%d", shader->QueryObject()->GetName(), pri);
+      csString description;
+      description.Format ("Shader %s with priority %d", 
+	shader->QueryObject()->GetName(), pri);
+      BenchMark (name, description, CSDRAW_CLEARZBUFFER);
+      ws->ClearReplacedMaterials();
     }
   }
 }
@@ -414,54 +524,44 @@ void CsBench::PerformTests ()
 # endif
   Report ("Compiler '%s', platform '%s', processor '%s'",
   	CS_COMPILER_NAME, CS_PLATFORM_NAME, CS_PROCESSOR_NAME);
-#ifdef CS_QINT_WORKAROUND
-  Report ("QInt() workaround enabled!");
-#endif
 #ifdef CS_NO_QSQRT
-  Report ("qsqrt() disabled!");
+  Report ("csQsqrt() disabled!");
 #endif
+  Report ("Big object in test has %d triangles.",
+  	2*(BIGOBJECT_DIM-1)*(BIGOBJECT_DIM-1));
+  Report ("Small object in test has %d triangles. We use %d of them.",
+  	2*(SMALLOBJECT_DIM-1)*(SMALLOBJECT_DIM-1), SMALLOBJECT_NUM);
 
   g3d->SetOption ("StencilThreshold", "0");
-  BenchMark ("stencilclip", "Stencil clipping is used");
-  g3d->SetOption ("StencilThreshold", "100000000");
-  BenchMark ("planeclip", "glClipPlane clipping is used");
+  view->GetCamera ()->SetSector (room_single);
+  float stencil0 = BenchMark ("stencilclip_single", 
+    "Stencil clipping, single object");
+  view->GetCamera ()->SetSector (room_multi);
+  BenchMark ("stencilclip_multi", "Stencil clipping, multiple objects");
 
-  PerformShaderTest ("/shader/or_lighting.xml", "or_lighting", "OR compatibility");
+  g3d->SetOption ("StencilThreshold", "100000000");
+  view->GetCamera ()->SetSector (room_single);
+  float stencil1 = BenchMark ("planeclip_single", "glClipPlane, single object");
+  view->GetCamera ()->SetSector (room_multi);
+  BenchMark ("planeclip_multi", "glClipPlane, multiple objects");
+  // Set back to stencil0 if that method was faster.
+  if (stencil0 > stencil1)
+  {
+    Report ("Switching back to stencil thresshold 0.");
+    g3d->SetOption ("StencilThreshold", "0");
+  }
+
+  view->GetCamera ()->SetSector (room_single);
+  vfs->PushDir("/shader");
+  PerformShaderTest ("/shader/std_lighting.xml", "standard", 0, 0, gmSingle);
 
   iRenderLoopManager* rlmgr = engine->GetRenderLoopManager ();
-  csRef<iRenderLoop> loop = rlmgr->Create ();
-  csRef<iPluginManager> plugin_mgr = CS_QUERY_REGISTRY (object_reg,
-	iPluginManager);
-  csRef<iRenderStepType> genType = CS_LOAD_PLUGIN (plugin_mgr,
-	"crystalspace.renderloop.step.generic.type", iRenderStepType);
-  csRef<iRenderStepType> lightType = CS_LOAD_PLUGIN (plugin_mgr,
-	"crystalspace.renderloop.step.lightiter.type", iRenderStepType);
-  csRef<iRenderStepFactory> genFact = genType->NewFactory ();
-  csRef<iRenderStepFactory> lightFact = lightType->NewFactory ();
-  csRef<iRenderStep> step;
-  csRef<iGenericRenderStep> genStep;
-  step = genFact->Create ();
-  loop->AddStep (step);
-  genStep = SCF_QUERY_INTERFACE (step, iGenericRenderStep);
-  genStep->SetShaderType ("ambient");
-  genStep->SetZBufMode (CS_ZBUF_MESH);
-  genStep->SetZOffset (true);
-  genStep->SetPortalTraversal (true);
-  step = lightFact->Create ();
-  loop->AddStep (step);
-  csRef<iRenderStepContainer> contStep = SCF_QUERY_INTERFACE (step,
-	iRenderStepContainer);
-  step = genFact->Create ();
-  contStep->AddStep (step);
-  genStep = SCF_QUERY_INTERFACE (step, iGenericRenderStep);
-  genStep->SetShaderType ("diffuse");
-  genStep->SetZBufMode (CS_ZBUF_MESH);
-  genStep->SetZOffset (false);
-  genStep->SetPortalTraversal (false);
-  engine->GetRenderLoopManager ()->Register ("bump", loop);
+  csRef<iRenderLoop> loop = rlmgr->Load ("/shader/std_rloop_diffuse.xml");
   engine->SetCurrentDefaultRenderloop (loop);
 
-  PerformShaderTest ("/shader/light_bumpmap.xml", "light_bumpmap", "diffuse");
+  PerformShaderTest ("/shader/light_bumpmap.xml", "diffuse", 
+    "/shader/ambient.xml", "ambient", gmSingle);
+  vfs->PopDir ();
 }
 
 /*---------------------------------------------------------------------*
