@@ -20,33 +20,36 @@
 
 #include <stdarg.h>
 
-#define CS_SYSDEF_PROVIDE_SOFTWARE2D
 #include "cssysdef.h"
-#include "csutil/sysfunc.h"
-#include "csutil/csuctransform.h"
-#include "csutil/randomgen.h"
-#include "csutil/csstring.h"
-#include "cstool/initapp.h"
-#include "ivaria/reporter.h"
-#include "qint.h"
-#include "qsqrt.h"
+#include "csqint.h"
+#include "csqsqrt.h"
+
 #include "csgeom/polyaa.h"
 #include "csgeom/vector2.h"
-
-#include "iutil/vfs.h"
-#include "iutil/plugin.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/natwin.h"
-#include "ivideo/fontserv.h"
+#include "cstool/initapp.h"
 #include "csutil/cmdhelp.h"
+#include "csutil/csstring.h"
+#include "csutil/csuctransform.h"
+#include "csutil/event.h"
+#include "csutil/randomgen.h"
+#include "csutil/refarr.h"
+#include "csutil/sysfunc.h"
 #include "csutil/util.h"
+#include "iutil/cfgmgr.h"
 #include "iutil/cmdline.h"
+#include "iutil/comp.h"
+#include "iutil/databuff.h"
 #include "iutil/event.h"
+#include "iutil/eventh.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
-#include "csutil/event.h"
+#include "iutil/plugin.h"
+#include "iutil/vfs.h"
+#include "ivaria/reporter.h"
+#include "ivideo/custcursor.h"
+#include "ivideo/fontserv.h"
+#include "ivideo/graph2d.h"
+#include "ivideo/natwin.h"
 
 CS_IMPLEMENT_APPLICATION
 
@@ -67,6 +70,8 @@ class G2DTestSystemDriver
     stContextInfo,
     stWindowFixed,
     stWindowResize,
+    stCustomCursor,
+    stAlphaTest,
     stBackBufferON,
     stBackBufferOFF,
     stTestUnicode1,
@@ -80,6 +85,7 @@ class G2DTestSystemDriver
     stLineClipTest,
     stBoxClipTest,
     stFontClipTest,
+    stBlitTest,
     stPause,
     stWaitKey
   };
@@ -97,7 +103,8 @@ class G2DTestSystemDriver
   // some handy colors
   int white, yellow, green, red, blue, black, gray, dsteel;
   // Last pressed key
-  int lastkey, lastkey2, lastkey3, lastkey4, lastkey5, lastkey6, lastkey7, lastkey8;
+  int lastkey, lastkey2, lastkey3, lastkey4, lastkey5, lastkey6, lastkey7, 
+    lastkey8, lastkey9;
   // Switch backbuffer while waiting for a key
   bool SwitchBB;
   // Current font
@@ -106,12 +113,21 @@ class G2DTestSystemDriver
   csRef<iFont> fontItalic;
   csRef<iFont> fontCourier;
   csRef<iFont> fontSmall;
+  csRef<iImage> blitTestImage;
+  csRef<iImage> alphaBlitImage;
   // Event Outlet
   iEventOutlet* EventOutlet;
 
 public:
   csRef<iGraphics2D> myG2D;
+  csRef<iGraphics3D> myG3D;
   iObjectRegistry* object_reg;
+  csRef<iCursor> cursorPlugin;
+  csEventID SystemOpen;
+  csEventID KeyboardDown;
+  csEventID Process;
+  csEventID FinalProcess;
+  csEventID CanvasResize;
 
 public:
   G2DTestSystemDriver (int argc, char* argv[]);
@@ -127,17 +143,22 @@ private:
   void EnterState (appState newstate, int arg = 0);
   void LeaveState ();
 
-  int MakeColor (int r, int g, int b);
+  int MakeColor (int r, int g, int b, int a = 255);
   void WriteCentered (int mode, int dy, int fg, int bg, const char *format, ...);
   void WriteCenteredWrapped (int mode, int dy, int &h, int fg, int bg, 
     const char *format, ...);
 
   void ResizeContext ();
 
+  void SetCustomCursor ();
+  void SetNormalCursor ();
+
   void DrawStartupScreen ();
   void DrawContextInfoScreen ();
   void DrawWindowScreen ();
   void DrawWindowResizeScreen ();
+  void DrawCustomCursorScreen ();
+  void DrawAlphaTestScreen ();
   void DrawBackBufferText ();
   void DrawBackBufferON ();
   void DrawBackBufferOFF ();
@@ -154,6 +175,8 @@ private:
   void BoxClipTest ();
   void FontClipTest ();
 
+  void BlitTest ();
+
   void DrawClipRect(int x, int y, int w, int h);
 };
 
@@ -191,6 +214,7 @@ G2DTestSystemDriver::G2DTestSystemDriver (int argc, char* argv[])
     EventOutlet = q->GetEventOutlet();
     EventOutlet->IncRef();
   }
+
 }
 
 G2DTestSystemDriver::~G2DTestSystemDriver ()
@@ -202,6 +226,9 @@ G2DTestSystemDriver::~G2DTestSystemDriver ()
   fontCourier = 0;
   fontLarge = 0;
   fontSmall = 0;
+  blitTestImage = 0;
+  alphaBlitImage = 0;
+  cursorPlugin = 0;
   csInitializer::DestroyApplication (object_reg);
 }
 
@@ -234,6 +261,9 @@ void G2DTestSystemDriver::EnterState (appState newstate, int arg)
     case stFontClipTest:
       lastkey8 = 0;
       break;
+    case stCustomCursor:
+      lastkey9 = 0;
+      break;
     case stWaitKey:
       lastkey = 0;
       break;
@@ -260,12 +290,12 @@ void G2DTestSystemDriver::SetupFrame ()
     blue = MakeColor (0, 0, 255);
     gray = MakeColor (128, 128, 128);
     dsteel = MakeColor (80, 100, 112);
-    black = 0;
+    black = MakeColor (0, 0, 0);
   }
 
   if (state_sptr == 0)
   {
-    EventOutlet->Broadcast (cscmdQuit);
+    EventOutlet->Broadcast (csevQuit (object_reg));
     return;
   }
 
@@ -277,6 +307,8 @@ void G2DTestSystemDriver::SetupFrame ()
     case stContextInfo:
     case stWindowFixed:
     case stWindowResize:
+    case stCustomCursor:
+    case stAlphaTest:
     case stBackBufferON:
     case stBackBufferOFF:
     case stTestUnicode1:
@@ -290,8 +322,9 @@ void G2DTestSystemDriver::SetupFrame ()
     case stLineClipTest:
     case stBoxClipTest:
     case stFontClipTest:
+    case stBlitTest:
     {
-      if (!myG2D->BeginDraw ())
+      if (!myG3D->BeginDraw (CSDRAW_2DGRAPHICS))
         break;
 
       myG2D->Clear (black);
@@ -303,11 +336,34 @@ void G2DTestSystemDriver::SetupFrame ()
 	  fontItalic = GetFont (CSFONT_ITALIC);
 	  fontCourier = GetFont (CSFONT_COURIER);
 	  fontSmall = GetFont (CSFONT_SMALL);
+	  {
+	    csRef<iVFS> vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+	    csRef<iImageIO> iio = CS_QUERY_REGISTRY (object_reg,
+	      iImageIO);
+	    if (vfs.IsValid () && iio.IsValid ())
+	    {
+	      csRef<iFile> testFile = vfs->Open ("/lib/g2dtest/up.png", 
+		VFS_FILE_READ);
+	      if (testFile.IsValid ())
+	      {
+		csRef<iDataBuffer> fileData = testFile->GetAllData ();
+		blitTestImage = iio->Load (fileData, CS_IMGFMT_TRUECOLOR 
+		  | CS_IMGFMT_ALPHA);
+	      }
+	      testFile = vfs->Open ("/lib/std/cslogo2.png", VFS_FILE_READ);
+	      if (testFile.IsValid ())
+	      {
+		csRef<iDataBuffer> fileData = testFile->GetAllData ();
+		alphaBlitImage = iio->Load (fileData, CS_IMGFMT_TRUECOLOR |
+		  CS_IMGFMT_ALPHA);
+	      }
+	    }
+	  }
 	  EnterState (stStartup);
 	  break;
         case stStartup:
           DrawStartupScreen ();
-          EnterState (stContextInfo);
+	  EnterState (stContextInfo);
           EnterState (stPause, 5000);
           break;
         case stContextInfo:
@@ -322,6 +378,20 @@ void G2DTestSystemDriver::SetupFrame ()
           break;
         case stWindowResize:
           DrawWindowResizeScreen ();
+          EnterState (stCustomCursor);
+          EnterState (stWaitKey);
+          break;
+	case stCustomCursor:
+          DrawCustomCursorScreen ();
+	  SetCustomCursor ();
+	  if (lastkey9)
+            EnterState (stAlphaTest);
+	  else
+            EnterState (stCustomCursor);
+          break;
+	case stAlphaTest:
+          SetNormalCursor ();
+          DrawAlphaTestScreen ();
           EnterState (stBackBufferON);
           EnterState (stWaitKey);
           break;
@@ -419,10 +489,14 @@ void G2DTestSystemDriver::SetupFrame ()
         case stFontClipTest:
           FontClipTest ();
           if (lastkey8)
-            ;//EnterState ();
+	    EnterState (stBlitTest);
           else
             EnterState (stFontClipTest);
           break;
+	case stBlitTest:
+	  BlitTest ();
+          EnterState (stWaitKey);
+          break;
         default:
           break;
       }
@@ -456,39 +530,27 @@ void G2DTestSystemDriver::SetupFrame ()
 
 void G2DTestSystemDriver::FinishFrame ()
 {
-  myG2D->FinishDraw ();
-  myG2D->Print (0);
+  myG3D->FinishDraw ();
+  myG3D->Print (0);
 }
 
 bool G2DTestSystemDriver::HandleEvent (iEvent &Event)
 {
-  switch (Event.Type)
+  if (myG2D && (Event.Name == SystemOpen))
   {
-    case csevBroadcast:
-      switch (Event.Command.Code)
-      {
-        case cscmdSystemOpen:
-          if (myG2D)
-          {
             // Create a uniform palette: r(3)g(3)b(2)
             int r,g,b;
             for (r = 0; r < 8; r++)
               for (g = 0; g < 8; g++)
                 for (b = 0; b < 4; b++)
                   myG2D->SetRGB (r * 32 + g * 4 + b, r * 32, g * 32, b * 64);
-            break;
-          }
-        case cscmdContextResize:
-          if (myG2D)
-          {
-            ResizeContext ();
-            break;
-          }
-      }
-      break;
-    case csevKeyboard:
-      if (csKeyEventHelper::GetEventType (&Event) == csKeyEventTypeDown)
-      {
+  }
+  else if (myG2D && (Event.Name == CanvasResize))
+  {
+    ResizeContext ();
+  }
+  else if (Event.Name == KeyboardDown)
+  {
 	if (state_sptr)
 	  switch (state [state_sptr - 1])
 	  {
@@ -506,30 +568,37 @@ bool G2DTestSystemDriver::HandleEvent (iEvent &Event)
 	      break;
 	    case stPixelClipTest:
 	      lastkey5 = csKeyEventHelper::GetCookedCode (&Event);
+	      break;
 	    case stLineClipTest:
 	      lastkey6 = csKeyEventHelper::GetCookedCode (&Event);
+	      break;
 	    case stBoxClipTest:
 	      lastkey7 = csKeyEventHelper::GetCookedCode (&Event);
+	      break;
 	    case stFontClipTest:
 	      lastkey8 = csKeyEventHelper::GetCookedCode (&Event);
+	      break;
+	    case stCustomCursor:
+	      lastkey9 = csKeyEventHelper::GetCookedCode (&Event);
+	      break;
 	    default:
 	      break;
 	  }
-      }
-      break;
   }
   return false;
 }
 
-int G2DTestSystemDriver::MakeColor (int r, int g, int b)
+int G2DTestSystemDriver::MakeColor (int r, int g, int b, int a)
 {
-  if (!pfmt.PalEntries)
+  /*if (!pfmt.PalEntries)
     return ((r >> (8 - pfmt.RedBits)) << pfmt.RedShift)
          | ((g >> (8 - pfmt.GreenBits)) << pfmt.GreenShift)
-         | ((b >> (8 - pfmt.BlueBits)) << pfmt.BlueShift);
+         | ((b >> (8 - pfmt.BlueBits)) << pfmt.BlueShift)
+         | ((a >> (8 - pfmt.AlphaBits)) << pfmt.AlphaShift);
 
   // In paletted mode this is easy since we have a uniform 3-3-2 palette
-  return ((r >> 5) << 5) | ((g >> 5) << 2) | (b >> 6);
+  return ((r >> 5) << 5) | ((g >> 5) << 2) | (b >> 6);*/
+  return myG2D->FindRGB (r, g, b, a);
 }
 
 csPtr<iFont> G2DTestSystemDriver::GetFont (const char *fontID, int size)
@@ -548,11 +617,11 @@ void G2DTestSystemDriver::WriteCentered (int mode, int dy, int fg, int bg,
 {
   if (!font) return;
 
-  char text [1024];
+  csString text;
   va_list arg;
 
   va_start (arg, format);
-  vsprintf (text, format, arg);
+  text.FormatV (format, arg);
   va_end (arg);
 
   int fw, fh;
@@ -584,11 +653,11 @@ void G2DTestSystemDriver::WriteCenteredWrapped (int mode, int dy, int &h,
 {
   if (!font) return;
 
-  char text [1024];
+  csString text;
   va_list arg;
 
   va_start (arg, format);
-  vsprintf (text, format, arg);
+  text.FormatV (format, arg);
   va_end (arg);
 
   int y = 0, w = myG2D->GetWidth ();
@@ -671,18 +740,25 @@ void G2DTestSystemDriver::DrawContextInfoScreen ()
 
   WriteCentered (0,-16*3, white, -1, "Some information about graphics context");
   WriteCentered (0,-16*2, gray,  -1, "Screen size: %d x %d", myG2D->GetWidth (), myG2D->GetHeight ());
-  char pixfmt [50];
+  csString pixfmt;
   if (pfmt.PalEntries)
-    sprintf (pixfmt, "%d colors (Indexed)", pfmt.PalEntries);
+    pixfmt.Format ("%d colors (Indexed)", pfmt.PalEntries);
   else
-    sprintf (pixfmt, "R%dG%dB%d", pfmt.RedBits, pfmt.GreenBits, pfmt.BlueBits);
-  WriteCentered (0,-16*1, gray,  -1, "Pixel format: %d BPP, %s", pfmt.PixelBytes * 8, pixfmt);
+    pixfmt.Format ("R%dG%dB%dA%d", pfmt.RedBits, pfmt.GreenBits, pfmt.BlueBits, 
+      pfmt.AlphaBits);
+  WriteCentered (0,-16*1, gray,  -1, "Pixel format: %d BPP, %s", pfmt.PixelBytes * 8, 
+    pixfmt.GetData());
 
   if (pfmt.PalEntries)
-    sprintf (pixfmt, "not available");
+    pixfmt = "not available";
   else
-    sprintf (pixfmt, "R[%08X] G[%08X] B[%08X] ", pfmt.RedMask, pfmt.GreenMask, pfmt.BlueMask);
-  WriteCentered (0, 16*0, gray,  -1, "R/G/B masks: %s", pixfmt);
+    pixfmt.Format (
+      "R[%08" PRIX32 "] "
+      "G[%08" PRIX32 "] "
+      "B[%08" PRIX32 "] "
+      "A[%08" PRIX32 "]",
+      pfmt.RedMask, pfmt.GreenMask, pfmt.BlueMask, pfmt.AlphaMask);
+  WriteCentered (0, 16*0, gray,  -1, "R/G/B/A masks: %s", pixfmt.GetData());
 
   WriteCentered (0, 16*1, gray,  -1, "More than one backbuffer available: %s",
     myG2D->GetDoubleBufferState () ? "yes" : "no");
@@ -738,8 +814,8 @@ void G2DTestSystemDriver::ResizeContext ()
   myG2D->Clear (black);
   DrawWindowResizeScreen ();
 
-  char text [50];
-  sprintf (text, "Canvas [%d x %d]", myG2D->GetWidth (), myG2D->GetHeight ());
+  csString text;
+  text.Format ("Canvas [%d x %d]", myG2D->GetWidth (), myG2D->GetHeight ());
   SetFont (fontLarge);
   int fw, fh;
   font->GetDimensions (text, fw, fh);
@@ -750,6 +826,19 @@ void G2DTestSystemDriver::ResizeContext ()
   myG2D->Print (0);
 }
 
+void G2DTestSystemDriver::SetCustomCursor ()
+{
+  if (cursorPlugin)
+  {
+    cursorPlugin->SwitchCursor ("Hand");
+  }
+}
+
+void G2DTestSystemDriver::SetNormalCursor ()
+{
+  myG2D->SetMouseCursor (csmcArrow);
+}
+
 void G2DTestSystemDriver::DrawBackBufferText ()
 {
   SetFont (fontItalic);
@@ -799,6 +888,73 @@ void G2DTestSystemDriver::DrawBackBufferOFF ()
   WriteCentered (0, 16*2, gray,  -1, "wrong support for single-backbuffer mode.");
 }
 
+void G2DTestSystemDriver::DrawCustomCursorScreen ()
+{
+  int w = myG2D->GetWidth ();
+  int h = myG2D->GetHeight ();
+  myG2D->SetClipRect(0,0,w,h);
+  myG2D->DrawBox(0,0,w,h, dsteel);
+
+  SetFont (fontItalic);
+  int tpos = -h / 2;
+  WriteCentered (0, tpos, white, -1, "CUSTOM MOUSE CURSOR");
+
+  SetFont (fontLarge);
+  WriteCentered (0, tpos + 16*2, black,  -1, "If your current canvas supports custom mouse cursors");
+  WriteCentered (0, tpos + 16*3, black,  -1, "you shouldn't see your systems default cursor now.");
+}
+
+void G2DTestSystemDriver::DrawAlphaTestScreen ()
+{
+  int w = myG2D->GetWidth ();
+  int h = myG2D->GetHeight ();
+  myG2D->SetClipRect(0,0,w,h);
+  myG2D->DrawBox(0,0,w,h, dsteel);
+
+  SetFont (fontItalic);
+  WriteCentered (1, 1, white, -1, "ALPHA COLOR TEST");
+
+  SetFont (fontLarge);
+  WriteCentered (1, 16*2, black, -1, "If your current canvas is in 32-bit mode, you should");
+  WriteCentered (1, 16*3, black, -1, "see various text and geometry at various transparencies.");
+
+  myG2D->DrawBox (190, 80, 50, 100, black);
+  myG2D->DrawBox (20, 100, 150, 75, myG2D->FindRGB (205, 0, 125, 200));
+  myG2D->DrawBox (120, 100, 100, 50, myG2D->FindRGB (120, 50, 50, 100));
+  myG2D->DrawLine (30, 110, 120, 60, myG2D->FindRGB (255, 128, 128, 128));
+  myG2D->DrawLine (120, 60, 70, 120, myG2D->FindRGB (128, 255, 128, 128));
+  myG2D->DrawLine (70, 120, 30, 110, myG2D->FindRGB (128, 128, 255, 128));
+
+  if (alphaBlitImage.IsValid ())
+  {
+    myG2D->Blit (20, 160, alphaBlitImage->GetWidth (), alphaBlitImage->GetHeight (), 
+      (unsigned char*)alphaBlitImage->GetImageData ());
+  }
+
+  myG2D->Write (font, 50, 140, myG2D->FindRGB (255, 255, 255, 100), -1,
+    "Here is some partially transparent text");
+  myG2D->Write (font, 50, 150, myG2D->FindRGB (0, 0, 255, 150), -1,
+    "overlaying partially transparent boxes.");
+
+  csString str;
+  int i;
+  int y = 140;
+  int tw, th;
+  font->GetMaxSize (tw, th);
+  for (i = 0; i < 6; i++)
+  {
+    const uint8 alpha = (i * 51);
+    str.Format ("FG has alpha %" PRIu8 , alpha);
+    myG2D->Write (font, 320, y, MakeColor (255, 255, 255, alpha), 
+      black, str);
+    y += th;
+    str.Format ("BG has alpha %" PRIu8, alpha);
+    myG2D->Write (font, 320, y, white, MakeColor (0, 0, 0, alpha), 
+      str);
+    y += th;
+  }
+}
+
 void G2DTestSystemDriver::DrawUnicodeTest1 ()
 {
   int w = myG2D->GetWidth ();
@@ -880,8 +1036,8 @@ void G2DTestSystemDriver::DrawUnicodeTest2 ()
 
 void G2DTestSystemDriver::DrawFreetypeTest ()
 {
-  const char* fontFaces[] = {"VeraSans", "VeraSansBoldOblique", 
-    "VeraSansMono", "VeraSerif", 0};
+  const char* fontFaces[] = {"DejaVuSans", "DejaVuSansBoldOblique", 
+    "DejaVuSansMono", "DejaVuSerif", 0};
   const int fontSizes[] = {4, 8, 12, 24, 0};
 
   int w = myG2D->GetWidth ();
@@ -901,6 +1057,10 @@ void G2DTestSystemDriver::DrawFreetypeTest ()
   WriteCentered (0, tpos + 16*4, black,  -1, 
     "in various faces and sizes below.");
 
+  csRefArray<iFont> fonts;
+  // The used fonts are all kept until the end of this function, to provide
+  // some more "stress" on the font cache.
+
   int y = tpos + 16*7;
   int i = 0;
   while (fontFaces[i] != 0)
@@ -913,6 +1073,7 @@ void G2DTestSystemDriver::DrawFreetypeTest ()
       csRef<iFont> font = GetFont (fontFaces[i], fontSizes[j]);
       if (font)
       {
+	fonts.Push (font);
 	SetFont (font);
 	str.Clear ();
         str << fontFaces[i] << ", Size " << fontSizes[j];
@@ -1043,8 +1204,8 @@ void G2DTestSystemDriver::DrawLinePerf ()
       float x2 = sx + rng.Get () * sw;
       float y2 = sy + rng.Get () * sh;
       myG2D->DrawLine (x1, y1, x2, y2, colors [rng.Get (4)]);
-      x2 = QInt (x2 - x1); y2 = QInt (y2 - y1);
-      pix_count += qsqrt (x2 * x2 + y2 * y2);
+      x2 = csQint (x2 - x1); y2 = csQint (y2 - y1);
+      pix_count += csQsqrt (x2 * x2 + y2 * y2);
     }
     myG2D->PerformExtension ("flush");
     delta_time = csGetTicks () - start_time;
@@ -1081,7 +1242,7 @@ void G2DTestSystemDriver::DrawTextTest ()
   const char *text = "Crystal Space rulez";
   int tw, th;
   font->GetDimensions (text, tw, th);
-  int cc = strlen (text);
+  size_t cc = strlen (text);
 
   // Test text drawing performance for 1/4 seconds
   int colors [4] = { red, green, blue, yellow };
@@ -1089,7 +1250,7 @@ void G2DTestSystemDriver::DrawTextTest ()
   sy += 10; sh -= 20 + th;
   csRandomGen rng (csGetTicks ());
   csTicks start_time = csGetTicks (), delta_time;
-  int char_count = 0;
+  size_t char_count = 0;
   do
   {
     for (i = 0; i < 2000; i++)
@@ -1132,7 +1293,7 @@ void G2DTestSystemDriver::DrawTextTest2 ()
   const char *text = "Crystal Space rulez";
   int tw,th;
   font->GetDimensions (text, tw, th);
-  int cc = strlen (text);
+  size_t cc = strlen (text);
 
   // Test text drawing performance for 1/4 seconds
   int colors [4] = { red, green, blue, yellow };
@@ -1140,7 +1301,7 @@ void G2DTestSystemDriver::DrawTextTest2 ()
   sy += 10; sh -= 20 + th;
   csRandomGen rng (csGetTicks ());
   csTicks start_time = csGetTicks (), delta_time;
-  int char_count = 0;
+  size_t char_count = 0;
   do
   {
 	int i;
@@ -1453,14 +1614,45 @@ void G2DTestSystemDriver::FontClipTest()
   } while (delta_time < 100);
 }
 
+void G2DTestSystemDriver::BlitTest ()
+{
+  int w = myG2D->GetWidth ();
+  int h = myG2D->GetHeight ();
+  myG2D->SetClipRect(0,0,w,h);
+  myG2D->DrawBox(0,0,w,h, dsteel);
+  
+  SetFont (fontItalic);
+  WriteCentered (0,-16*8, white, -1, "BLIT() TEST");
+
+  SetFont (fontLarge);
+  WriteCentered (0,-16*7, black, dsteel, "This will test whether iGraphics2D->Blit() works correctly");
+  WriteCentered (0,-16*6, black, dsteel, "on this canvas.");
+
+  WriteCentered (0,-16*4, black, dsteel, "You should see an image of an arrow and the word \"up\".");
+  WriteCentered (0,-16*3, black, dsteel, "It is surrounded by a green rectangle, and the image");
+  WriteCentered (0,-16*2, black, dsteel, "itself has a black border. No red should be visible");
+  WriteCentered (0,-16*1, black, dsteel, "and the border has to be complete, too.");
+
+  if (blitTestImage.IsValid ())
+  {
+    const int imW = blitTestImage->GetWidth ();
+    const int imH = blitTestImage->GetHeight ();
+    const int bx = (w - imW) / 2;
+    const int by = h / 2;
+    DrawClipRect (bx, by, imW + 1, imH + 1);
+    myG2D->Blit (bx + 1, by + 1, imW, imH, 
+      (unsigned char*)blitTestImage->GetImageData ());
+  }
+}
+
 static bool G2DEventHandler (iEvent& ev)
 {
-  if (ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess)
+  if (ev.Name == Sys->Process)
   {
     Sys->SetupFrame ();
     return true;
   }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess)
+  else if (ev.Name == Sys->FinalProcess)
   {
     Sys->FinishFrame ();
     return true;
@@ -1497,35 +1689,42 @@ int main (int argc, char *argv[])
   csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
   	iCommandLineParser));
 
-  System.myG2D = CS_QUERY_REGISTRY (object_reg, iGraphics2D);
-  // Now load the canvas plugin
-  if (!System.myG2D)
+  System.myG3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  // Now load the renderer plugin
+  if (!System.myG3D)
   {
-    csString canvas = cmdline->GetOption ("canvas");
+    csString canvas = cmdline->GetOption ("video");
     if (!canvas || !*canvas)
-      canvas = CS_SOFTWARE_2D_DRIVER;
+      canvas = "crystalspace.graphics3d.software"; //CS_SOFTWARE_2D_DRIVER;
     else if (strncmp ("crystalspace.", canvas, 13))
     {
-      canvas = "crystalspace.graphics2d." + canvas;
+      canvas = "crystalspace.graphics3d." + canvas;
     }
-    System.myG2D = CS_LOAD_PLUGIN (plugin_mgr, canvas, iGraphics2D);
-    if (!object_reg->Register (System.myG2D, "iGraphics2D"))
+    System.myG3D = CS_LOAD_PLUGIN (plugin_mgr, canvas, iGraphics3D);
+    if (!object_reg->Register (System.myG3D, "iGraphics3D"))
     {
       csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
     	  "crystalspace.application.g2dtest",
-	  "Unable to register canvas!");
+	  "Unable to register renderer!");
       return -1;
     }
   }
 
-  if (!System.myG2D)
+  if (!System.myG3D)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
     	"crystalspace.application.g2dtest",
 	"Unable to load canvas driver!");
     return -1;
   }
+  System.myG2D = System.myG3D->GetDriver2D ();
     
+  System.SystemOpen = csevSystemOpen (object_reg);
+  System.KeyboardDown = csevKeyboardDown (object_reg);
+  System.Process = csevProcess (object_reg);
+  System.FinalProcess = csevFinalProcess (object_reg);
+  System.CanvasResize = csevCanvasResize (object_reg, System.myG2D);
+
   if (!csInitializer::OpenApplication (object_reg))
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -1534,12 +1733,16 @@ int main (int argc, char *argv[])
     return -1;
   }
   
-  if (!System.myG2D->Open ())
+  System.cursorPlugin = CS_QUERY_REGISTRY(object_reg, iCursor);
+  if (System.cursorPlugin)
   {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-    	"crystalspace.application.g2dtest",
-        "Unable to open graphics context!");
-    return -1;
+    csRef<iConfigManager> cfg (CS_QUERY_REGISTRY (object_reg, iConfigManager));
+    if (System.cursorPlugin->Setup (System.myG3D))
+    {
+      System.cursorPlugin->ParseConfigFile ((iConfigManager*)cfg);
+    }
+    else
+      System.cursorPlugin = 0;
   }
 
   iNativeWindow* nw = System.myG2D->GetNativeWindow ();
@@ -1547,10 +1750,11 @@ int main (int argc, char *argv[])
 
   csDefaultRunLoop(object_reg);
 
-  System.myG2D->Close();
-  System.myG2D = NULL;
-  plugin_mgr = NULL;
-  cmdline = NULL;
+  System.myG2D = 0;
+  System.myG3D->Close();
+  System.myG3D = 0;
+  plugin_mgr = 0;
+  cmdline = 0;
 
   return 0;
 }
