@@ -17,29 +17,31 @@
 */
 
 #include "cssysdef.h"
-#include "csutil/sysfunc.h"
-#include "csutil/cscolor.h"
 #include "cstool/initapp.h"
 #include "csutil/cmdhelp.h"
-#include "cslight.h"
+#include "csutil/cscolor.h"
+#include "csutil/stringarray.h"
+#include "csutil/sysfunc.h"
 #include "iengine/engine.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/natwin.h"
-#include "ivideo/txtmgr.h"
-#include "ivideo/texture.h"
-#include "ivideo/material.h"
-#include "ivideo/fontserv.h"
-#include "imap/parser.h"
+#include "igraphic/imageio.h"
+#include "imap/loader.h"
 #include "iutil/cmdline.h"
-#include "iutil/objreg.h"
-#include "iutil/eventh.h"
 #include "iutil/comp.h"
-#include "iutil/virtclk.h"
+#include "iutil/eventh.h"
+#include "iutil/objreg.h"
 #include "iutil/vfs.h"
+#include "iutil/virtclk.h"
 #include "ivaria/reporter.h"
 #include "ivaria/stdrep.h"
-#include "igraphic/imageio.h"
+#include "ivideo/fontserv.h"
+#include "ivideo/graph2d.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/material.h"
+#include "ivideo/natwin.h"
+#include "ivideo/texture.h"
+#include "ivideo/txtmgr.h"
+
+#include "cslight.h"
 
 CS_IMPLEMENT_APPLICATION
 
@@ -76,41 +78,37 @@ void csCsLightProgressMeter::SetProgressDescription (const char* id,
 void csCsLightProgressMeter::SetProgressDescriptionV (const char* /*id*/,
 	const char* description, va_list list)
 {
-  vsprintf (cur_description, description, list);
+  cur_description.FormatV (description, list);
 }
 
-void csCsLightProgressMeter::Step()
+void csCsLightProgressMeter::Step(unsigned int n)
 {
   if (current < total)
   {
-    current++;
+    current+=n;
+    if (current>total) current = total;
 
     int const units = (current == total ? 100 :
       (((100 * current) / total) / granularity) * granularity);
     int const extent = units / tick_scale;
     if (anchor < extent)
     {
-      char buff [256]; // Batch the update here before emitting it.
-      char const* safety_margin = buff + sizeof(buff) - 5;
-      char* p = buff;
-	  int i;
-      for (i = anchor + 1; i <= extent && p < safety_margin; i++)
+      csString buff; // Batch the update here before emitting it.
+      int i;
+      for (i = anchor + 1; i <= extent; i++)
       {
         if (i % (10 / tick_scale) != 0)
-	  *p++ = '.';
+	  buff << '.';
 	else
 	{
-          int n;
-	  sprintf(p, "%d%%%n", i * tick_scale, &n );
-	  p += n;
+	  buff.AppendFmt ("%d%%", i * tick_scale);
 	}
       }
-      *p = '\0';
-      printf ("%s", buff);
+      csPrintf ("%s", buff.GetData());
       anchor = extent;
     }
     if (current == total)
-      printf ("\n");
+      csPrintf ("\n");
 
     int fw = System->g2d->GetWidth ();
     int fh = System->g2d->GetHeight ();
@@ -118,7 +116,7 @@ void csCsLightProgressMeter::Step()
     System->g3d->BeginDraw (CSDRAW_2DGRAPHICS);
     System->g2d->Clear (System->color_bg);
     int lw, lh;
-    System->logo->GetMipMapDimensions (0, lw, lh);
+    System->logo->GetRendererDimensions (lw, lh);
     int w = lw * fw / 300;
     int h = lh * fh / 200;
     System->g3d->DrawPixmap (System->logo, (fw - w)/2, 20, w, h,
@@ -136,19 +134,19 @@ void csCsLightProgressMeter::Step()
 void csCsLightProgressMeter::Restart()
 {
   Reset();
-  printf ("0%%");
+  csPrintf ("0%%");
 }
 
 void csCsLightProgressMeter::Abort ()
 {
   current = total;
-  printf ("\n");
+  csPrintf ("\n");
 }
 
 void csCsLightProgressMeter::Finalize ()
 {
   current = total;
-  printf ("\n");
+  csPrintf ("\n");
 }
 
 void csCsLightProgressMeter::SetGranularity(int n)
@@ -192,38 +190,13 @@ void Cleanup ()
 
 bool Lighter::SetMapDir (const char* map_dir)
 {
-  char tmp[512];
-  sprintf (tmp, "%s/", map_dir);
-  csRef<iVFS> myVFS (CS_QUERY_REGISTRY (object_reg, iVFS));
-  if (!myVFS->Exists (map_dir))
+  csRef<iVFS> myVFS = CS_QUERY_REGISTRY (object_reg, iVFS);
+  csStringArray paths;
+  paths.Push ("/lev/");
+  if (!myVFS->ChDirAuto (map_dir, &paths, 0, "world"))
   {
-    char *name = strrchr (map_dir, '/');
-    if (name)
-    {
-      name++;
-      //sprintf (tmp, "$.$/data$/%s.zip, $.$/%s.zip, $(..)$/data$/%s.zip",
-      //  name, name, name);
-      const char *valfiletype = "zip";
-      char* extension = strrchr (name, '.');
-      if (extension && !strcmp (extension+1, valfiletype))
-      {
-        // The file already ends with the correct extension.
-        sprintf (tmp, "$.$/data$/%s, $.$/%s, $(..)$/data$/%s, %s",
-            name, name, name, name);
-      }
-      else
-      {
-        // Add the extension.
-        sprintf (tmp, "$.$/data$/%s.%s, $.$/%s.%s, $(..)$/data$/%s.%s",
-            name, valfiletype, name, valfiletype, name, valfiletype);
-      }
-      myVFS->Mount (map_dir, tmp);
-    }
-  }
-  if (!myVFS->ChDir (map_dir))
-  {
-    Report (CS_REPORTER_SEVERITY_ERROR,
-    	"The directory on VFS for map file does not exist!");
+    Report (CS_REPORTER_SEVERITY_ERROR, "Error setting directory '%s'!",
+    	map_dir);
     return false;
   }
   return true;
@@ -353,25 +326,14 @@ bool Lighter::Initialize (int argc, const char* const argv[],
     if (strlen (val) > 7 && !strncmp ("cache:", val, 6))
     {
       val += 6;
-      char map_dir[512];
-      // if an absolute path is given, copy it. Otherwise prepend "/lev/".
-      if (val[0] == '/')
-        strcpy (map_dir, val);
-      else
-        sprintf (map_dir, "/lev/%s", val);
-
-      if (!SetMapDir (map_dir))
+      if (!SetMapDir (val))
       {
         Cleanup ();
         exit (1);
       }
 
-      // First we force a clear of the cache manager in the engine
-      // so that a new one will be made soon.
-      engine->SetCacheManager (0);
-      // And then we get the cache manager which will force it
-      // to be created based on current VFS dir.
-      engine->GetCacheManager ();
+      // Set the cache manager based on current VFS dir.
+      engine->SetVFSCacheManager ();
       break;
     }
   }
@@ -380,7 +342,6 @@ bool Lighter::Initialize (int argc, const char* const argv[],
   int map_idx = 0;
   for (;;)
   {
-    char map_dir[512];
     csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
   	  iCommandLineParser));
     const char* val = cmdline->GetName (cmd_idx);
@@ -402,13 +363,7 @@ bool Lighter::Initialize (int argc, const char* const argv[],
       // We already treated the cache entry so we just continue here.
       continue;
     }
-    // if an absolute path is given, copy it. Otherwise prepend "/lev/".
-    if (val[0] == '/')
-      strcpy (map_dir, val);
-    else
-      sprintf (map_dir, "/lev/%s", val);
-
-    if (!SetMapDir (map_dir))
+    if (!SetMapDir (val))
     {
       Cleanup ();
       exit (1);
@@ -417,7 +372,7 @@ bool Lighter::Initialize (int argc, const char* const argv[],
     // Load the level file which is called 'world'.
     if (!loader->LoadMapFile ("world", map_idx == 0))	// Only clear engine for first level.
     {
-      Report (CS_REPORTER_SEVERITY_ERROR, "Couldn't load level '%s'!", map_dir);
+      Report (CS_REPORTER_SEVERITY_ERROR, "Couldn't load level '%s'!", val);
       Cleanup ();
       exit (1);
     }
