@@ -16,39 +16,13 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include "cssysdef.h"
-#include "csutil/sysfunc.h"
-#include "csutil/cscolor.h"
-#include "csutil/csevent.h"
-#include "csutil/cmdhelp.h"
-#include "csgeom/csrect.h"
-#include "csgeom/csrectrg.h"
-#include "cstool/csfxscr.h"
-#include "cstool/initapp.h"
-#include "qsqrt.h"
-#include "csutil/event.h"
-
-#include "ivideo/graph3d.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/fontserv.h"
-#include "ivideo/txtmgr.h"
-#include "ivideo/natwin.h"
-#include "igraphic/imageio.h"
-#include "ivaria/reporter.h"
-#include "iutil/eventq.h"
-#include "iutil/virtclk.h"
-#include "iutil/plugin.h"
-#include "iutil/vfs.h"
-
 #include "awstut.h"
 
-
 CS_IMPLEMENT_APPLICATION
 
 // The global system driver
 AwsTutorial *System;
 
-
 //-----------------------------------------------------------------------------
 
 #define QUERY_REG(myPlug, iFace, errMsg) \
@@ -69,41 +43,46 @@ AwsTutorial::~AwsTutorial()
 
 static bool AwsEventHandler (iEvent& ev)
 {
-  if (ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess)
+  if (System)
   {
-    System->SetupFrame ();
-    return true;
-  }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess)
-  {
-    System->FinishFrame ();
-    return true;
+    if (ev.Name == csevProcess (System->object_reg))
+    {
+      System->SetupFrame ();
+      return true;
+    }
+    else if (ev.Name == csevFinalProcess (System->object_reg))
+    {
+      System->FinishFrame ();
+      return true;
+    }
+    else
+    {
+      return System->HandleEvent (ev);
+    }
   }
   else
-  {
-    return System ? System->HandleEvent (ev) : false;
-  }
+    return false;
 }
 
-void AwsTutorial::SetPass (void* awst, iAwsSource *)
+void AwsTutorial::SetPass (unsigned long, intptr_t awst, iAwsSource *)
 {
   AwsTutorial* tut = (AwsTutorial*)awst;
   (void)tut; // @@@ TODO
-  printf ("SetPass\n"); fflush (stdout);
+  csPrintf ("SetPass\n"); fflush (stdout);
 }
 
-void AwsTutorial::SetUser (void* awst, iAwsSource *)
+void AwsTutorial::SetUser (unsigned long, intptr_t awst, iAwsSource *)
 {
   AwsTutorial* tut = (AwsTutorial*)awst;
   (void)tut; // @@@ TODO
-  printf ("SetUser\n"); fflush (stdout);
+  csPrintf ("SetUser\n"); fflush (stdout);
 }
 
-void AwsTutorial::Login (void* awst, iAwsSource *)
+void AwsTutorial::Login (unsigned long, intptr_t awst, iAwsSource *)
 {
   AwsTutorial* tut = (AwsTutorial*)awst;
   (void)tut; // @@@ TODO
-  printf ("Login\n"); fflush (stdout);
+  csPrintf ("Login\n"); fflush (stdout);
 }
 
 bool AwsTutorial::Initialize (int argc, const char* const argv[])
@@ -123,7 +102,7 @@ bool AwsTutorial::Initialize (int argc, const char* const argv[])
 
   if (!csInitializer::RequestPlugins (object_reg,
   	CS_REQUEST_VFS,
-	CS_REQUEST_SOFTWARE3D,
+	CS_REQUEST_OPENGL3D,
 	CS_REQUEST_FONTSERVER,
 	CS_REQUEST_IMAGELOADER,
 	CS_REQUEST_PLUGIN("crystalspace.window.alternatemanager", iAws),
@@ -133,6 +112,10 @@ bool AwsTutorial::Initialize (int argc, const char* const argv[])
     return false;
   }
 
+  Process = csevProcess (object_reg);
+  FinalProcess = csevFinalProcess (object_reg);
+  KeyboardDown = csevKeyboardDown (object_reg);
+
   if (!csInitializer::SetupEventHandler (object_reg, AwsEventHandler))
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "Could not setup event handler!");
@@ -172,7 +155,7 @@ bool AwsTutorial::Initialize (int argc, const char* const argv[])
   aws->SetupCanvas(0, myG2D, myG3D);
 
   // Setup sink.
-  iAwsSink* sink = aws->GetSinkMgr ()->CreateSink ((void*)this);
+  iAwsSink* sink = aws->GetSinkMgr ()->CreateSink ((intptr_t)this);
   sink->RegisterTrigger ("SetUserName", &SetUser);
   sink->RegisterTrigger ("SetPassword", &SetPass);
   sink->RegisterTrigger ("Login", &Login);
@@ -209,17 +192,17 @@ void AwsTutorial::FinishFrame ()
 
 bool AwsTutorial::HandleEvent (iEvent &Event)
 {
-  if ((Event.Type == csevKeyboard) && 
-    (csKeyEventHelper::GetEventType (&Event) == csKeyEventTypeDown) &&
+  if ((Event.Name == csevKeyboardDown(object_reg)) && 
     (csKeyEventHelper::GetCookedCode (&Event) == CSKEY_ESC))
   {
     csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
     if (q)
-      q->GetEventOutlet()->Broadcast (cscmdQuit);
+      q->GetEventOutlet()->Broadcast (csevQuit(object_reg));
     return true;
   }
 
-  return aws->HandleEvent (Event);
+  if (aws) return aws->HandleEvent (Event);
+  return false;
 }
 
 void AwsTutorial::Report (int severity, const char* msg, ...)
