@@ -20,23 +20,17 @@
 #define __AWSTUT_H__
 
 #include <stdarg.h>
-#include "csgeom/math2d.h"
-#include "csgeom/math3d.h"
-#include "iaws/aws.h"
-#include "iaws/awscnvs.h"
-
-struct iObjectRegistry;
-struct iVirtualClock;
-struct iEvent;
-struct iGraphics3D;
-struct iGraphics2D;
-struct iAwsSource;
+#include <crystalspace.h>
 
 class AwsTutorial
 {
 public:
   iObjectRegistry* object_reg;
 
+  csEventID Process;
+  csEventID FinalProcess;
+  csEventID KeyboardDown;
+
 private:
   csRef<iAws> aws;
   iAwsPrefManager* awsprefs;
@@ -45,9 +39,9 @@ private:
   csRef<iGraphics2D> myG2D;
   csRef<iVirtualClock> vc;
 
-  static void SetPass (void* awstut, iAwsSource *source);
-  static void SetUser (void* awstut, iAwsSource *source);
-  static void Login (void* awstut, iAwsSource *source);
+  static void SetPass (unsigned long, intptr_t awstut, iAwsSource *source);
+  static void SetUser (unsigned long, intptr_t awstut, iAwsSource *source);
+  static void Login (unsigned long, intptr_t awstut, iAwsSource *source);
 
 public:
   AwsTutorial();
