@@ -15,59 +15,24 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-
 #include "cssysdef.h"
-#include "csutil/sysfunc.h"
-#include "iutil/vfs.h"
-#include "csutil/cscolor.h"
-#include "cstool/csview.h"
-#include "cstool/initapp.h"
+
 #include "simple2.h"
-#include "iutil/eventq.h"
-#include "iutil/event.h"
-#include "iutil/objreg.h"
-#include "iutil/csinput.h"
-#include "iutil/virtclk.h"
-#include "iengine/sector.h"
-#include "iengine/engine.h"
-#include "iengine/camera.h"
-#include "iengine/light.h"
-#include "iengine/texture.h"
-#include "iengine/mesh.h"
-#include "iengine/movable.h"
-#include "iengine/material.h"
-#include "imesh/thing.h"
-#include "imesh/sprite3d.h"
-#include "imesh/object.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/texture.h"
-#include "ivideo/material.h"
-#include "ivideo/fontserv.h"
-#include "igraphic/imageio.h"
-#include "imap/parser.h"
-#include "ivaria/reporter.h"
-#include "ivaria/stdrep.h"
-#include "csutil/cmdhelp.h"
-#include "csutil/event.h"
 
 CS_IMPLEMENT_APPLICATION
 
 //-----------------------------------------------------------------------------
 
-// The global pointer to simple
-Simple* simple = 0;
-
-Simple::Simple (iObjectRegistry* object_reg)
+Simple::Simple ()
 {
-  Simple::object_reg = object_reg;
+  SetApplicationName ("CrystalSpace.Simple2");
 }
 
 Simple::~Simple ()
 {
 }
 
-void Simple::SetupFrame ()
+void Simple::ProcessFrame ()
 {
   // First get elapsed time from the virtual clock.
   csTicks elapsed_time = vc->GetElapsedTicks ();
@@ -99,15 +64,15 @@ void Simple::SetupFrame ()
     if (kbd->GetKeyState (CSKEY_RIGHT))
       rotY += speed;
     if (kbd->GetKeyState (CSKEY_LEFT))
-        rotY -= speed;
+      rotY -= speed;
     if (kbd->GetKeyState (CSKEY_PGUP))
-        rotX += speed;
+      rotX += speed;
     if (kbd->GetKeyState (CSKEY_PGDN))
-        rotX -= speed;
+      rotX -= speed;
     if (kbd->GetKeyState (CSKEY_UP))
-        c->Move (CS_VEC_FORWARD * 4 * speed);
+      c->Move (CS_VEC_FORWARD * 4 * speed);
     if (kbd->GetKeyState (CSKEY_DOWN))
-        c->Move (CS_VEC_BACKWARD * 4 * speed);
+      c->Move (CS_VEC_BACKWARD * 4 * speed);
   }
 
   // We now assign a new rotation transformation to the camera.  You
@@ -122,7 +87,6 @@ void Simple::SetupFrame ()
   csOrthoTransform ot (rot, c->GetTransform().GetOrigin ());
   c->SetTransform (ot);
 
-
   // Tell 3D driver we're going to display 3D things.
   if (!g3d->BeginDraw (engine->GetBeginDrawFlags () | CSDRAW_3DGRAPHICS))
     return;
@@ -133,42 +97,42 @@ void Simple::SetupFrame ()
 
 void Simple::FinishFrame ()
 {
+  // Just tell the 3D renderer that everything has been rendered.
   g3d->FinishDraw ();
   g3d->Print (0);
 }
 
-bool Simple::HandleEvent (iEvent& ev)
+bool Simple::OnKeyboard(iEvent& ev)
 {
-  if (ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess)
+  // We got a keyboard event.
+  csKeyEventType eventtype = csKeyEventHelper::GetEventType(&ev);
+  if (eventtype == csKeyEventTypeDown)
   {
-    SetupFrame ();
-    return true;
+    // The user pressed a key (as opposed to releasing it).
+    utf32_char code = csKeyEventHelper::GetCookedCode(&ev);
+    if (code == CSKEY_ESC)
+    {
+      // The user pressed escape to exit the application.
+      // The proper way to quit a Crystal Space application
+      // is by broadcasting a csevQuit event. That will cause the
+      // main runloop to stop. To do that we get the event queue from
+      // the object registry and then post the event.
+      csRef<iEventQueue> q = 
+        CS_QUERY_REGISTRY(GetObjectRegistry(), iEventQueue);
+      if (q.IsValid()) q->GetEventOutlet()->Broadcast(csevQuit(GetObjectRegistry()));
+    }
   }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess)
-  {
-    FinishFrame ();
-    return true;
-  }
-  else if ((ev.Type == csevKeyboard) &&
-    (csKeyEventHelper::GetEventType (&ev) == csKeyEventTypeDown) &&
-    (csKeyEventHelper::GetCookedCode (&ev) == CSKEY_ESC))
-  {
-    csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
-    if (q) q->GetEventOutlet()->Broadcast (cscmdQuit);
-    return true;
-  }
-
   return false;
 }
 
-bool Simple::SimpleEventHandler (iEvent& ev)
+bool Simple::OnInitialize(int /*argc*/, char* /*argv*/ [])
 {
-  return simple ? simple->HandleEvent (ev) : false;
-}
-
-bool Simple::Initialize ()
-{
-  if (!csInitializer::RequestPlugins (object_reg,
+  // RequestPlugins() will load all plugins we specify. In addition
+  // it will also check if there are plugins that need to be loaded
+  // from the config system (both the application config and CS or
+  // global configs). In addition it also supports specifying plugins
+  // on the commandline.
+  if (!csInitializer::RequestPlugins(GetObjectRegistry(),
     CS_REQUEST_VFS,
     CS_REQUEST_OPENGL3D,
     CS_REQUEST_ENGINE,
@@ -178,159 +142,144 @@ bool Simple::Initialize ()
     CS_REQUEST_REPORTER,
     CS_REQUEST_REPORTERLISTENER,
     CS_REQUEST_END))
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-    "Can't initialize plugins!");
-    return false;
-  }
+    return ReportError("Failed to initialize plugins!");
 
-  if (!csInitializer::SetupEventHandler (object_reg, SimpleEventHandler))
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-    "Can't initialize event handler!");
-    return false;
-  }
+  csBaseEventHandler::Initialize(GetObjectRegistry());
 
-  // Check for commandline help.
-  if (csCommandLineHelper::CheckHelp (object_reg))
-  {
-    csCommandLineHelper::Help (object_reg);
-    return false;
-  }
+  // Now we need to setup an event handler for our application.
+  // Crystal Space is fully event-driven. Everything (except for this
+  // initialization) happens in an event.
+  if (!RegisterQueue(GetObjectRegistry(), csevAllEvents(GetObjectRegistry())))
+    return ReportError("Failed to set up event handler!");
 
-  // The virtual clock.
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
-  if (vc == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-    "Can't find the virtual clock!");
-    return false;
-  }
+  return true;
+}
 
-  // Find the pointer to engine plugin
-  engine = CS_QUERY_REGISTRY (object_reg, iEngine);
-  if (engine == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-    "No iEngine plugin!");
-    return false;
-  }
-
-  loader = CS_QUERY_REGISTRY (object_reg, iLoader);
-  if (loader == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "No iLoader plugin!");
-    return false;
-  }
-
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
-  if (g3d == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "No iGraphics3D plugin!");
-    return false;
-  }
-
-  kbd = CS_QUERY_REGISTRY (object_reg, iKeyboardDriver);
-  if (kbd == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "No iKeyboardDriver plugin!");
-    return false;
-  }
+void Simple::OnExit()
+{
+}
 
+bool Simple::Application()
+{
   // Open the main system. This will open all the previously loaded plug-ins.
-  if (!csInitializer::OpenApplication (object_reg))
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "Error opening system!");
-    return false;
+  // i.e. all windows will be opened.
+  if (!OpenApplication(GetObjectRegistry()))
+    return ReportError("Error opening system!");
+
+  if (SetupModules()) {
+    // This calls the default runloop. This will basically just keep
+    // broadcasting process events to keep the game going.
+    Run();
   }
+  return true;
+}
+
+bool Simple::SetupModules()
+{
+  // Now get the pointer to various modules we need. We fetch them
+  // from the object registry. The RequestPlugins() call we did earlier
+  // registered all loaded plugins with the object registry.
+  // The virtual clock.
+  g3d = CS_QUERY_REGISTRY(GetObjectRegistry(), iGraphics3D);
+  if (!g3d) return ReportError("Failed to locate 3D renderer!");
+
+  engine = CS_QUERY_REGISTRY(GetObjectRegistry(), iEngine);
+  if (!engine) return ReportError("Failed to locate 3D engine!");
+
+  vc = CS_QUERY_REGISTRY(GetObjectRegistry(), iVirtualClock);
+  if (!vc) return ReportError("Failed to locate Virtual Clock!");
+
+  kbd = CS_QUERY_REGISTRY(GetObjectRegistry(), iKeyboardDriver);
+  if (!kbd) return ReportError("Failed to locate Keyboard Driver!");
+
+  loader = CS_QUERY_REGISTRY(GetObjectRegistry(), iLoader);
+  if (!loader) return ReportError("Failed to locate Loader!");
+
+  // We need a View to the virtual world.
+  view.AttachNew(new csView (engine, g3d));
+  iGraphics2D* g2d = g3d->GetDriver2D ();
+  // We use the full window to draw the world.
+  view->SetRectangle (0, 0, g2d->GetWidth (), g2d->GetHeight ());
 
   // First disable the lighting cache. Our app is simple enough
   // not to need this.
   engine->SetLightingCacheMode (0);
 
+  // Here we create our world.
+  CreateRoom();
+
+  // Here we create our world.
+  CreateSprites();
+
+  // Let the engine prepare all lightmaps for use and also free all images 
+  // that were loaded for the texture manager.
+  engine->Prepare ();
+
+  // these are used store the current orientation of the camera
+  rotY = rotX = 0;
+
+  // Now we need to position the camera in our world.
+  view->GetCamera ()->SetSector (room);
+  view->GetCamera ()->GetTransform ().SetOrigin (csVector3 (0, 5, -3));
+
+  return true;
+}
+
+
+void Simple::CreateRoom ()
+{
+  // Load the texture from the standard library.  This is located in
+  // CS/data/standard.zip and mounted as /lib/std using the Virtual
+  // File System (VFS) plugin.
   if (!loader->LoadTexture ("stone", "/lib/std/stone4.gif"))
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "Error loading 'stone4' texture!");
-    return false;
-  }
+    ReportError("Error loading 'stone4' texture!");
+
   iMaterialWrapper* tm = engine->GetMaterialList ()->FindByName ("stone");
 
-  rotX = rotY = 0;
-
+  // We create a new sector called "room".
   room = engine->CreateSector ("room");
+
+  // Creating the walls for our room.
   csRef<iMeshWrapper> walls (engine->CreateSectorWallsMesh (room, "walls"));
-  csRef<iThingState> ws =
-    SCF_QUERY_INTERFACE (walls->GetMeshObject (), iThingState);
-  csRef<iThingFactoryState> walls_state = ws->GetFactory ();
+  csRef<iThingFactoryState> walls_state = 
+    scfQueryInterface<iThingFactoryState> (walls->GetMeshObject ()->GetFactory());
   walls_state->AddInsideBox (csVector3 (-5, 0, -5), csVector3 (5, 20, 5));
   walls_state->SetPolygonMaterial (CS_POLYRANGE_LAST, tm);
   walls_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST, 3);
 
+  // Now we need light to see something.
   csRef<iLight> light;
   iLightList* ll = room->GetLights ();
 
-  light = engine->CreateLight (0, csVector3 (-3, 5, 0), 10,
-    csColor (1, 0, 0));
+  light = engine->CreateLight(0, csVector3(-3, 5, 0), 10, csColor(1, 0, 0));
   ll->Add (light);
 
-  light = engine->CreateLight (0, csVector3 (3, 5,  0), 10,
-    csColor (0, 0, 1));
+  light = engine->CreateLight(0, csVector3(3, 5,  0), 10, csColor(0, 0, 1));
   ll->Add (light);
 
-  light = engine->CreateLight (0, csVector3 (0, 5, -3), 10,
-    csColor (0, 1, 0));
+  light = engine->CreateLight(0, csVector3(0, 5, -3), 10, csColor(0, 1, 0));
   ll->Add (light);
+}
 
-  engine->Prepare ();
-
-  view = csPtr<iView> (new csView (engine, g3d));
-  view->GetCamera ()->SetSector (room);
-  view->GetCamera ()->GetTransform ().SetOrigin (csVector3 (0, 5, -3));
-  iGraphics2D* g2d = g3d->GetDriver2D ();
-  view->SetRectangle (0, 0, g2d->GetWidth (), g2d->GetHeight ());
-
+void Simple::CreateSprites ()
+{
   // Load a texture for our sprite.
-  iTextureManager* txtmgr = g3d->GetTextureManager ();
   iTextureWrapper* txt = loader->LoadTexture ("spark",
-    "/lib/std/spark.png", CS_TEXTURE_3D, txtmgr, true);
+    "/lib/std/spark.png");
   if (txt == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "Error loading texture!");
-    return false;
-  }
+    ReportError("Error loading texture!");
 
   // Load a sprite template from disk.
   csRef<iMeshFactoryWrapper> imeshfact (
     loader->LoadMeshObjectFactory ("/lib/std/sprite1"));
   if (imeshfact == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.simple2",
-        "Error loading mesh object factory!");
-    return false;
-  }
+    ReportError("Error loading mesh object factory!");
 
   // Create the sprite and add it to the engine.
   csRef<iMeshWrapper> sprite (engine->CreateMeshWrapper (
     imeshfact, "MySprite", room,
     csVector3 (-3, 5, 3)));
-  csMatrix3 m; m.Identity (); m *= 5.;
+  csMatrix3 m; m.Identity ();
   sprite->GetMovable ()->SetTransform (m);
   sprite->GetMovable ()->UpdateMove ();
   csRef<iSprite3DState> spstate (
@@ -343,29 +292,19 @@ bool Simple::Initialize ()
   // can do this.
   sprite->SetZBufMode (CS_ZBUF_USE);
   sprite->SetRenderPriority (engine->GetObjectRenderPriority ());
-
-  return true;
 }
 
-void Simple::Start ()
-{
-  csDefaultRunLoop (object_reg);
-}
-
-/*---------------------------------------------------------------------*
+/*-------------------------------------------------------------------------*
  * Main function
- *---------------------------------------------------------------------*/
+ *-------------------------------------------------------------------------*/
 int main (int argc, char* argv[])
 {
-  iObjectRegistry* object_reg = csInitializer::CreateEnvironment (argc, argv);
-
-  simple = new Simple (object_reg);
-  if (simple->Initialize ())
-    simple->Start ();
-  delete simple;
-  simple = 0;
-
-  csInitializer::DestroyApplication (object_reg);
-  return 0;
+  /* Runs the application. 
+   *
+   * csApplicationRunner<> is a small wrapper to support "restartable" 
+   * applications (ie where CS needs to be completely shut down and loaded 
+   * again). Simple1 does not use that functionality itself, however, it
+   * allows you to later use "Simple.Restart();" and it'll just work.
+   */
+  return csApplicationRunner<Simple>::Run (argc, argv);
 }
-
