@@ -16,43 +16,7 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include "cssysdef.h"
-#include "csutil/sysfunc.h"
-#include "csutil/cscolor.h"
-#include "cstool/csview.h"
-#include "cstool/initapp.h"
-#include "csutil/cmdhelp.h"
 #include "simplept.h"
-#include "iutil/eventq.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/event.h"
-#include "iutil/objreg.h"
-#include "iutil/csinput.h"
-#include "iutil/virtclk.h"
-#include "iengine/sector.h"
-#include "iengine/engine.h"
-#include "iengine/camera.h"
-#include "iengine/light.h"
-#include "iengine/texture.h"
-#include "iengine/mesh.h"
-#include "iengine/movable.h"
-#include "iengine/material.h"
-#include "imesh/thing.h"
-#include "imesh/genmesh.h"
-#include "imesh/object.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/txtmgr.h"
-#include "ivideo/texture.h"
-#include "ivideo/material.h"
-#include "ivideo/fontserv.h"
-#include "igraphic/imageio.h"
-#include "imap/parser.h"
-#include "ivaria/reporter.h"
-#include "ivaria/stdrep.h"
-#include "iutil/vfs.h"
-#include "csutil/event.h"
 
 CS_IMPLEMENT_APPLICATION
 
@@ -220,20 +184,25 @@ Simple::~Simple ()
 
 static bool SimpleEventHandler (iEvent& ev)
 {
-  if (ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess)
+  if (simple)
   {
-    simple->SetupFrame ();
-    return true;
-  }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess)
-  {
-    simple->FinishFrame ();
-    return true;
+    if (ev.Name == csevProcess(simple->object_reg))
+    {
+      simple->SetupFrame ();
+      return true;
+    }
+    else if (ev.Name == csevFinalProcess(simple->object_reg))
+    {
+      simple->FinishFrame ();
+      return true;
+    }
+    else
+    {
+      return simple->HandleEvent (ev);
+    }
   }
   else
-  {
-    return simple ? simple->HandleEvent (ev) : false;
-  }
+    return false;
 }
 
 bool Simple::Initialize ()
@@ -255,6 +224,10 @@ bool Simple::Initialize ()
     return false;
   }
 
+  Process = csevProcess (object_reg);
+  FinalProcess = csevFinalProcess (object_reg);
+  KeyboardDown = csevKeyboardDown (object_reg);
+
   if (!csInitializer::SetupEventHandler (object_reg, SimpleEventHandler))
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -369,9 +342,8 @@ bool Simple::Initialize ()
   ProcTexture->DecRef ();
   room = engine->CreateSector ("proctex-room");
   csRef<iMeshWrapper> walls (engine->CreateSectorWallsMesh (room, "walls"));
-  csRef<iThingState> ws =
-  	SCF_QUERY_INTERFACE (walls->GetMeshObject (), iThingState);
-  csRef<iThingFactoryState> walls_state = ws->GetFactory ();
+  csRef<iThingFactoryState> walls_state = 
+    scfQueryInterface<iThingFactoryState> (walls->GetMeshObject ()->GetFactory());
 
   walls_state->CreateVertex (csVector3 (-8, -8, -5));
   walls_state->CreateVertex (csVector3 (-3, -3, +8));
@@ -412,13 +384,12 @@ bool Simple::Initialize ()
 
 bool Simple::HandleEvent (iEvent& Event)
 {
-  if ((Event.Type == csevKeyboard) && 
-    (csKeyEventHelper::GetEventType (&Event) == csKeyEventTypeDown) &&
+  if ((Event.Name == csevKeyboardDown(object_reg)) && 
     (csKeyEventHelper::GetCookedCode (&Event) == CSKEY_ESC))
   {
     csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
     if (q)
-      q->GetEventOutlet()->Broadcast (cscmdQuit);
+      q->GetEventOutlet()->Broadcast (csevQuit(object_reg));
     return true;
   }
 
@@ -488,7 +459,7 @@ int main (int argc, char* argv[])
   if (simple->Initialize ())
     simple->Start ();
 
-  delete simple;
+  delete simple; simple = 0;
 
   csInitializer::DestroyApplication (object_reg);
 
