@@ -19,42 +19,45 @@
 #ifndef __SIMPVS_H__
 #define __SIMPVS_H__
 
-#include <stdarg.h>
+#include <crystalspace.h>
+#include "vidprefs.h"
 
-struct iEngine;
-struct iLoader;
-struct iGraphics3D;
-struct iKeyboardDriver;
-struct iVirtualClock;
-struct iObjectRegistry;
-struct iEvent;
-struct iSector;
-struct iView;
-
-class Simple
+class Simple : public csApplicationFramework, public csBaseEventHandler
 {
 private:
-  iObjectRegistry* object_reg;
-  iEngine* engine;
-  iLoader* loader;
-  iGraphics3D* g3d;
-  iKeyboardDriver* kbd;
-  iVirtualClock* vc;
+  csRef<iEngine> engine;
+  csRef<iLoader> loader;
+  csRef<iGraphics3D> g3d;
+  csRef<iKeyboardDriver> kbd;
+  csRef<iVirtualClock> vc;
   iSector* room;
-  iView* view;
-
-  static bool SimpleEventHandler (iEvent& ev);
+  csRef<iView> view;
+  
   bool HandleEvent (iEvent& ev);
   void SetupFrame ();
   void FinishFrame ();
-
+  
+  csVideoPreferences* vidprefs;
+  void SaveVideoPreference();
+  
+  bool Setup ();
+  
+  csEventID Process;
+  csEventID FinalProcess;
+  csEventID KeyboardDown;
+  csEventID Quit;
+  
 public:
   Simple ();
-  ~Simple ();
+  virtual ~Simple ();
 
-  bool Initialize (int argc, const char* const argv[]);
-  bool Initialize2 ();
   void Start ();
+
+  bool OnInitialize (int argc, char* argv[]);
+  bool Application ();
+
+  CS_EVENTHANDLER_NAMES("crystalspace.simpvs")
+  CS_EVENTHANDLER_NIL_CONSTRAINTS
 };
 
 #endif // __SIMPVS_H__
