@@ -17,17 +17,19 @@
 */
 
 #include "cssysdef.h"
+
 #include "cstool/collider.h"
+#include "csutil/flags.h"
 #include "csutil/sparse3d.h"
-#include "imesh/object.h"
-#include "imesh/lighting.h"
-#include "imesh/thing.h"
-#include "igeom/polymesh.h"
-#include "igeom/objmodel.h"
 #include "iengine/light.h"
 #include "iengine/material.h"
-#include "iengine/sector.h"
 #include "iengine/movable.h"
+#include "iengine/sector.h"
+#include "igeom/objmodel.h"
+#include "igeom/polymesh.h"
+#include "imesh/lighting.h"
+#include "imesh/object.h"
+#include "imesh/thing.h"
 
 #include "infmaze.h"
 #include "walktest.h"
@@ -42,6 +44,9 @@ InfiniteMaze::~InfiniteMaze ()
   delete infinite_world;
 }
 
+#define ROOM_SIZE 3.0f
+#define LEDGE_OFFSET 0.1f
+
 void InfiniteMaze::create_one_side (iThingFactoryState* walls_state,
 	char* pname,
 	iMaterialWrapper* tm, iMaterialWrapper* tm2,
@@ -60,9 +65,10 @@ void InfiniteMaze::create_one_side (iThingFactoryState* walls_state,
   x3 -= cx; y3 -= cy; z3 -= cz;
   x4 -= cx; y4 -= cy; z4 -= cz;
   float sx, sy, sz;
-  if (dx) { sy = sz = 0.9f; sx = 0; }
-  else if (dy) { sx = sz = 0.9f; sy = 0; }
-  else { sx = sy = 0.9f; sz = 0; }
+# define L_FACTOR ((ROOM_SIZE/2.0f-LEDGE_OFFSET)/(ROOM_SIZE/2.0f))
+  if (dx) { sy = sz = L_FACTOR; sx = 0; }
+  else if (dy) { sx = sz = L_FACTOR; sy = 0; }
+  else { sx = sy = L_FACTOR; sz = 0; }
 
   walls_state->AddQuad (
     csVector3 (cx+sx*x1, cy+sy*y1, cz+sz*z1),
@@ -119,37 +125,39 @@ float rand2 (float max)
 
 InfRoomData* InfiniteMaze::create_six_room (iEngine* engine, int x, int y, int z)
 {
-  char buf[50];
-  sprintf (buf, "r%d_%d_%d", x, y, z);
+  csString buf;
+  buf.Format ("r%d_%d_%d", x, y, z);
   iSector* room = engine->CreateSector (buf);
   csRef<iMeshWrapper> walls (engine->CreateSectorWallsMesh (room, "walls"));
-  csRef<iThingState> walls_state (SCF_QUERY_INTERFACE (walls->GetMeshObject (),
-  	iThingState));
-  csRef<iThingFactoryState> walls_fact_state = walls_state->GetFactory ();
+  walls->SetZBufMode (CS_ZBUF_USE);
+  walls->SetRenderPriority (engine->GetObjectRenderPriority ());
+  csRef<iThingFactoryState> walls_fact_state = 
+    scfQueryInterface<iThingFactoryState> (
+    walls->GetMeshObject ()->GetFactory());
   float dx, dy, dz;
-  dx = 2.0*(float)x;
-  dy = 2.0*(float)y;
-  dz = 2.0*(float)z;
+  dx = ROOM_SIZE*(float)x;
+  dy = ROOM_SIZE*(float)y;
+  dz = ROOM_SIZE*(float)z;
   iMaterialWrapper* t = engine->GetMaterialList ()->FindByName ("txt");
   iMaterialWrapper* t2 = engine->GetMaterialList ()->FindByName ("txt2");
-  float s = 1;
+  float s = (ROOM_SIZE/2.0f);
 
   create_one_side (walls_fact_state, "n", t, t2, dx-s,dy+s,dz+s,
-  		dx+s,dy+s,dz+s, dx+s,dy-s,dz+s,  dx-s,dy-s,dz+s, 0,0,-0.1f);
+  		dx+s,dy+s,dz+s, dx+s,dy-s,dz+s,  dx-s,dy-s,dz+s, 0,0,-LEDGE_OFFSET);
   create_one_side (walls_fact_state, "e", t, t2, dx+s,dy+s,dz+s,
-  		dx+s,dy+s,dz-s, dx+s,dy-s,dz-s,  dx+s,dy-s,dz+s, -0.1f,0,0);
+  		dx+s,dy+s,dz-s, dx+s,dy-s,dz-s,  dx+s,dy-s,dz+s, -LEDGE_OFFSET,0,0);
   create_one_side (walls_fact_state, "w", t, t2, dx-s,dy+s,dz+s,
-  		dx-s,dy-s,dz+s, dx-s,dy-s,dz-s,  dx-s,dy+s,dz-s, 0.1f,0,0);
+  		dx-s,dy-s,dz+s, dx-s,dy-s,dz-s,  dx-s,dy+s,dz-s, LEDGE_OFFSET,0,0);
   create_one_side (walls_fact_state, "s", t, t2, dx+s,dy+s,dz-s,
-  		dx-s,dy+s,dz-s, dx-s,dy-s,dz-s,  dx+s,dy-s,dz-s, 0,0,0.1f);
+  		dx-s,dy+s,dz-s, dx-s,dy-s,dz-s,  dx+s,dy-s,dz-s, 0,0,LEDGE_OFFSET);
   create_one_side (walls_fact_state, "f", t, t2, dx-s,dy-s,dz+s,
-  		dx+s,dy-s,dz+s, dx+s,dy-s,dz-s,  dx-s,dy-s,dz-s, 0,0.1f,0);
+  		dx+s,dy-s,dz+s, dx+s,dy-s,dz-s,  dx-s,dy-s,dz-s, 0,LEDGE_OFFSET,0);
   create_one_side (walls_fact_state, "c", t, t2, dx-s,dy+s,dz-s,
-  		dx+s,dy+s,dz-s, dx+s,dy+s,dz+s,  dx-s,dy+s,dz+s, 0,-0.1f,0);
+  		dx+s,dy+s,dz-s, dx+s,dy+s,dz+s,  dx-s,dy+s,dz+s, 0,-LEDGE_OFFSET,0);
 
   csRef<iLight> light (engine->CreateLight ("",
   	csVector3 (dx+rand2 (.9*s), dy+rand2 (.9*s), dz+rand2 (.9*s)),
-	1+rand1 (3),
+	1+rand1 (ROOM_SIZE * 1.5f),
   	csColor (rand1 (1), rand1 (1), rand1 (1))));
   room->GetLights ()->Add (light);
 
@@ -169,10 +177,11 @@ InfRoomData* InfiniteMaze::create_six_room (iEngine* engine, int x, int y, int z
 }
 
 void InfiniteMaze::connect_infinite (int x1, int y1, int z1, int x2, int y2,
-	int z2, bool create_portal1)
+	int z2, bool /*create_portal1*/)
 {
-  //InfRoomData* s1 = (InfRoomData*)(infinite_world->Get (x1, y1, z1));
-  //InfRoomData* s2 = (InfRoomData*)(infinite_world->Get (x2, y2, z2));
+  int a;
+  InfRoomData* s1 = (InfRoomData*)(infinite_world->Get (x1, y1, z1));
+  InfRoomData* s2 = (InfRoomData*)(infinite_world->Get (x2, y2, z2));
   char* p1, * p2;
   if (x1 == x2)
     if (y1 == y2)
@@ -184,11 +193,38 @@ void InfiniteMaze::connect_infinite (int x1, int y1, int z1, int x2, int y2,
   else
     if (x1 < x2) { p1 = "e"; p2 = "w"; }
     else { p1 = "w"; p2 = "e"; }
-  //@@@@@@@@@
-  //iPolygon3DStatic* po1 = s1->walls_fact_state->GetPolygon (p1);
-  //iPolygon3DStatic* po2 = s2->walls_fact_state->GetPolygon (p2);
-  //if (create_portal1) po1->CreatePortal (s2->sector);
-  //po2->CreatePortal (s1->sector);
+
+  int po1 = s1->walls_fact_state->FindPolygonByName (p1);
+  int tmpCount = s1->walls_fact_state->GetPolygonVertexCount(po1);
+  csVector3 *vertices = new csVector3[tmpCount];
+  for(a=0;a<tmpCount;a++)
+  {
+    vertices[a] = s1->walls_fact_state->GetPolygonVertex(po1,a);
+  }
+  iPortal* portalBack;
+  csRef<iMeshWrapper> portalMeshBack = Sys->Engine->CreatePortal (
+    "new_portal_back", s1->sector, csVector3 (0),
+    s2->sector, vertices, tmpCount, portalBack);
+  delete[] vertices;
+
+  int po2 = s2->walls_fact_state->FindPolygonByName (p2);
+  tmpCount = s2->walls_fact_state->GetPolygonVertexCount(po2);
+  vertices = new csVector3[tmpCount];
+  for(a=0;a<tmpCount;a++)
+  {
+    vertices[a] = s2->walls_fact_state->GetPolygonVertex(po2,a);
+  }
+  iPortal* portalBack2;
+  csRef<iMeshWrapper> portalMeshBack2 = Sys->Engine->CreatePortal (
+    "new_portal_back2", s2->sector, csVector3 (0),
+    s1->sector, vertices, tmpCount, portalBack2);
+  portalBack->GetFlags ().Set (CS_PORTAL_ZFILL);
+  portalBack->GetFlags ().Set (CS_PORTAL_CLIPDEST);
+  portalBack2->GetFlags ().Set (CS_PORTAL_ZFILL);
+  portalBack2->GetFlags ().Set (CS_PORTAL_CLIPDEST);
+  s1->walls_fact_state->RemovePolygon(po1);
+  s2->walls_fact_state->RemovePolygon(po2);
+  delete[] vertices;
 }
 
 SCF_IMPLEMENT_IBASE (InfPortalCS)
@@ -229,7 +265,6 @@ bool InfPortalCS::Traverse (iPortal* portal, iBase* context)
         lv->ctxt = fv->CopyFrustumContext ();
       }
     }
-    return false;
   }
   else
   {
@@ -281,13 +316,10 @@ bool InfPortalCS::Traverse (iPortal* portal, iBase* context)
     }
     csRef<iPolygonMesh> mesh = 
       ird->walls->GetMeshObject ()->GetObjectModel()->GetPolygonMeshColldet();
-    csRef<iObject> io (SCF_QUERY_INTERFACE (ird->walls, iObject));
-    csColliderWrapper *cw = new csColliderWrapper (
-    	io, Sys->collide_system, mesh);
-    cw->SetName ("inf maze");
-    cw->DecRef ();
+    csColliderHelper::InitializeCollisionWrapper (Sys->collide_system, ird->walls);
     return true;
   }
+  return false;
 }
 
 void InfiniteMaze::create_loose_portal (int x1, int y1, int z1,
