@@ -17,15 +17,19 @@
 */
 
 #include "cssysdef.h"
-#include "cssys/sysfunc.h"
-#include "qint.h"
+#include "csutil/sysfunc.h"
+#include "csqint.h"
 #include "walktest.h"
 #include "bot.h"
 #include "infmaze.h"
 #include "command.h"
-#include "imesh/thing/polygon.h"
+#include "imesh/thing.h"
+#include "iengine/camera.h"
+#include "iengine/mesh.h"
+#include "iengine/sector.h"
+#include "imesh/object.h"
+#include "csutil/csuctransform.h"
 #include "csutil/scanstr.h"
-#include "csutil/dataobj.h"
 #include "csgeom/math3d.h"
 #include "cstool/collider.h"
 #include "cstool/cspixmap.h"
@@ -37,12 +41,12 @@
 #include "ivaria/view.h"
 #include "iutil/event.h"
 #include "iutil/csinput.h"
-
-#include "csengine/wirefrm.h"
+#include "iutil/virtclk.h"
+#include "csutil/event.h"
 
 extern WalkTest* Sys;
 
-csKeyMap* mapping = NULL;
+csKeyMap* mapping = 0;
 
 iMeshWrapper *FindNextClosestMesh (iMeshWrapper *baseMesh, iCamera *camera, csVector2 *screenCoord);
 
@@ -50,64 +54,83 @@ iMeshWrapper *FindNextClosestMesh (iMeshWrapper *baseMesh, iCamera *camera, csVe
 // Everything for key mapping and binding.
 //===========================================================================
 
-void map_key (const char* keyname, csKeyMap* map)
+void map_key (const char* _keyname, csKeyMap* map)
 {
   map->shift = 0;
   map->alt = 0;
   map->ctrl = 0;
   map->need_status = 0;
-  char* dash = strchr (keyname, '-');
+  char* keyname = new char[strlen(_keyname) + 1];
+  strcpy (keyname, _keyname);
+  char* wordstart = keyname;
+  char* dash = strchr (wordstart, '-');
   while (dash)
   {
     *dash = 0;
-    if (!strcmp (keyname, "shift")) map->shift = 1;
-    else if (!strcmp (keyname, "alt")) map->alt = 1;
-    else if (!strcmp (keyname, "ctrl")) map->ctrl = 1;
-    else if (!strcmp (keyname, "status")) map->need_status = 1;
+    if (!strcmp (wordstart, "shift")) map->shift = 1;
+    else if (!strcmp (wordstart, "alt")) map->alt = 1;
+    else if (!strcmp (wordstart, "ctrl")) map->ctrl = 1;
+    else if (!strcmp (wordstart, "status")) map->need_status = 1;
     else Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-    	"Bad modifier '%s'!", keyname);
+    	"Bad modifier '%s'!", wordstart);
 
     *dash = '-';
-    keyname = dash+1;
-    dash = strchr (dash+1, '-');
+    wordstart = dash+1;
+    dash = strchr (wordstart, '-');
   }
 
-  if (!strcmp (keyname, "tab")) map->key = CSKEY_TAB;
-  else if (!strcmp (keyname, "space")) map->key = ' ';
-  else if (!strcmp (keyname, "esc")) map->key = CSKEY_ESC;
-  else if (!strcmp (keyname, "enter")) map->key = CSKEY_ENTER;
-  else if (!strcmp (keyname, "bs")) map->key = CSKEY_BACKSPACE;
-  else if (!strcmp (keyname, "up")) map->key = CSKEY_UP;
-  else if (!strcmp (keyname, "down")) map->key = CSKEY_DOWN;
-  else if (!strcmp (keyname, "right")) map->key = CSKEY_RIGHT;
-  else if (!strcmp (keyname, "left")) map->key = CSKEY_LEFT;
-  else if (!strcmp (keyname, "pgup")) map->key = CSKEY_PGUP;
-  else if (!strcmp (keyname, "pgdn")) map->key = CSKEY_PGDN;
-  else if (!strcmp (keyname, "home")) map->key = CSKEY_HOME;
-  else if (!strcmp (keyname, "end")) map->key = CSKEY_END;
-  else if (!strcmp (keyname, "ins")) map->key = CSKEY_INS;
-  else if (!strcmp (keyname, "del")) map->key = CSKEY_DEL;
-  else if (!strcmp (keyname, "f1")) map->key = CSKEY_F1;
-  else if (!strcmp (keyname, "f2")) map->key = CSKEY_F2;
-  else if (!strcmp (keyname, "f3")) map->key = CSKEY_F3;
-  else if (!strcmp (keyname, "f4")) map->key = CSKEY_F4;
-  else if (!strcmp (keyname, "f5")) map->key = CSKEY_F5;
-  else if (!strcmp (keyname, "f6")) map->key = CSKEY_F6;
-  else if (!strcmp (keyname, "f7")) map->key = CSKEY_F7;
-  else if (!strcmp (keyname, "f8")) map->key = CSKEY_F8;
-  else if (!strcmp (keyname, "f9")) map->key = CSKEY_F9;
-  else if (!strcmp (keyname, "f10")) map->key = CSKEY_F10;
-  else if (!strcmp (keyname, "f11")) map->key = CSKEY_F11;
-  else if (!strcmp (keyname, "f12")) map->key = CSKEY_F12;
-  else if (*(keyname+1) != 0) Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-  	"Bad key '%s'!", keyname);
-  else if ((*keyname >= 'A' && *keyname <= 'Z') || strchr ("!@#$%^&*()_+", *keyname))
+  if (!strcmp (wordstart, "tab")) map->key = CSKEY_TAB;
+  else if (!strcmp (wordstart, "space")) map->key = ' ';
+  else if (!strcmp (wordstart, "esc")) map->key = CSKEY_ESC;
+  else if (!strcmp (wordstart, "enter")) map->key = CSKEY_ENTER;
+  else if (!strcmp (wordstart, "bs")) map->key = CSKEY_BACKSPACE;
+  else if (!strcmp (wordstart, "up")) map->key = CSKEY_UP;
+  else if (!strcmp (wordstart, "down")) map->key = CSKEY_DOWN;
+  else if (!strcmp (wordstart, "right")) map->key = CSKEY_RIGHT;
+  else if (!strcmp (wordstart, "left")) map->key = CSKEY_LEFT;
+  else if (!strcmp (wordstart, "pgup")) map->key = CSKEY_PGUP;
+  else if (!strcmp (wordstart, "pgdn")) map->key = CSKEY_PGDN;
+  else if (!strcmp (wordstart, "home")) map->key = CSKEY_HOME;
+  else if (!strcmp (wordstart, "end")) map->key = CSKEY_END;
+  else if (!strcmp (wordstart, "ins")) map->key = CSKEY_INS;
+  else if (!strcmp (wordstart, "del")) map->key = CSKEY_DEL;
+  else if (!strcmp (wordstart, "f1")) map->key = CSKEY_F1;
+  else if (!strcmp (wordstart, "f2")) map->key = CSKEY_F2;
+  else if (!strcmp (wordstart, "f3")) map->key = CSKEY_F3;
+  else if (!strcmp (wordstart, "f4")) map->key = CSKEY_F4;
+  else if (!strcmp (wordstart, "f5")) map->key = CSKEY_F5;
+  else if (!strcmp (wordstart, "f6")) map->key = CSKEY_F6;
+  else if (!strcmp (wordstart, "f7")) map->key = CSKEY_F7;
+  else if (!strcmp (wordstart, "f8")) map->key = CSKEY_F8;
+  else if (!strcmp (wordstart, "f9")) map->key = CSKEY_F9;
+  else if (!strcmp (wordstart, "f10")) map->key = CSKEY_F10;
+  else if (!strcmp (wordstart, "f11")) map->key = CSKEY_F11;
+  else if (!strcmp (wordstart, "f12")) map->key = CSKEY_F12;
+  /*
+  else if (*(wordstart+0) != 0) Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
+  	"Bad key '%s'!", wordstart);
+  else if ((*wordstart >= 'A' && *wordstart <= 'Z') ||
+    strchr ("!@#$%^&*()_+", *wordstart))
   {
     map->shift = 1;
-    map->key = *keyname;
+    map->key = *wordstart;
   }
+  */
   else
-    map->key = *keyname;
+  {
+    utf32_char key;
+    size_t nameLen = strlen (wordstart);
+    bool charValid;
+    int encLen = csUnicodeTransform::UTF8Decode ((utf8_char*)wordstart, 
+      nameLen, key, &charValid);
+    if (!charValid || ((size_t)encLen < nameLen))
+    {
+      Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Bad key '%s'!", wordstart);
+    }
+    else
+      map->key = key;
+  }
+  delete[] keyname;
 }
 
 char* keyname (csKeyMap* map)
@@ -170,12 +193,12 @@ csKeyMap* find_mapping (const char* keyname)
       return m;
     m = m->next;
   }
-  return NULL;
+  return 0;
 }
 
-void bind_key (const char* arg)
+void bind_key (const char* _arg)
 {
-  if (!arg)
+  if (!_arg)
   {
     csKeyMap* map = mapping;
     while (map)
@@ -186,6 +209,8 @@ void bind_key (const char* arg)
     }
     return;
   }
+  char* arg = new char[strlen(_arg) + 1];
+  strcpy (arg, _arg);
   char* space = strchr (arg, ' ');
   if (space)
   {
@@ -199,7 +224,7 @@ void bind_key (const char* arg)
     {
       map = new csKeyMap ();
       map->next = mapping;
-      map->prev = NULL;
+      map->prev = 0;
       if (mapping) mapping->prev = map;
       mapping = map;
       map_key (arg, map);
@@ -215,6 +240,7 @@ void bind_key (const char* arg)
     	"Key bound to '%s'!", map->cmd);
     else Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Key not bound!");
   }
+  delete[] arg;
 }
 
 void free_keymap ()
@@ -227,250 +253,157 @@ void free_keymap ()
     delete [] prev->cmd;
     delete prev;
   }
-  mapping = NULL;
+  mapping = 0;
 }
 
-/*------------------------------------------------------------------
- * The following handle_key_... routines are general movement
- * routines that are called by do_update() for the new movement
- * system and by do_keypress() for the old movement system (see
- * system.h for an explanation of the difference between the two
- * systems).
- *------------------------------------------------------------------*/
-
 extern iCamera* c;
 extern WalkTest* Sys;
 
-void WalkTest::strafe (float speed,int keep_old)
+void WalkTest::Strafe (float speed)
 {
-  if (move_3d || map_mode) return;
-  static bool pressed = false;
-  static float strafe_speed = 0;
-  static long start_time = csGetTicks ();
-
-  long cur_time = csGetTicks ();
-  if (!keep_old)
-  {
-    bool new_pressed = ABS (speed) > 0.001;
-    if (new_pressed != pressed)
-    {
-      pressed = new_pressed;
-      strafe_speed = speed * cfg_walk_accelerate;
-      start_time = cur_time - 100;
-    }
-  }
-
-  while ((cur_time - start_time) >= 100)
-  {
-    if (pressed)
-    {
-      // accelerate
-      if (ABS (velocity.x) < cfg_walk_maxspeed)
-        velocity.x += strafe_speed;
-    }
-    else
-    {
-      // brake!
-      if (velocity.x > cfg_walk_brake)
-        velocity.x -= cfg_walk_brake;
-      else if (velocity.x < -cfg_walk_brake)
-        velocity.x += cfg_walk_brake;
-      else
-        velocity.x = 0;
-    }
-    start_time += 100;
-  }
+  if (move_3d) return;
+  speed *= cfg_walk_maxspeed_multreal;
+  desired_velocity.x = 140.0f * speed * cfg_walk_maxspeed
+  	* cfg_walk_maxspeed_multreal;
 }
 
-void WalkTest::step (float speed,int keep_old)
+void WalkTest::Step (float speed)
 {
-  if (move_3d || map_mode) return;
-
-  static bool pressed = false;
-  static float step_speed = 0;
-  static long start_time = csGetTicks ();
-
-  long cur_time = csGetTicks ();
-  if (!keep_old)
-  {
-    bool new_pressed = ABS (speed) > 0.001;
-    if (new_pressed != pressed)
-    {
-      pressed = new_pressed;
-      step_speed = speed * cfg_walk_accelerate;
-      start_time = cur_time - 100;
-    }
-  }
-
-  float max_speed = cfg_walk_maxspeed * (kbd->GetKeyState (CSKEY_SHIFT) ? 2 : 1);
-
-  while ((cur_time - start_time) >= 100)
-  {
-    if (pressed)
-    {
-      // accelerate
-      if (ABS (velocity.z) < max_speed)
-        velocity.z += step_speed;
-      else if (ABS (velocity.z) > max_speed)
-        velocity.z -= step_speed;
-    }
-    else
-    {
-      // brake!
-      if (velocity.z > cfg_walk_brake)
-        velocity.z -= cfg_walk_brake;
-      else if (velocity.z < -cfg_walk_brake)
-        velocity.z += cfg_walk_brake;
-      else
-        velocity.z = 0;
-    }
-    start_time += 100;
-  }
+  if (move_3d) return;
+  speed *= cfg_walk_maxspeed_multreal;
+  desired_velocity.z = 140.0f * speed * cfg_walk_maxspeed
+  	* cfg_walk_maxspeed_multreal;
 }
 
-void WalkTest::rotate (float speed,int keep_old)
+void WalkTest::Jump ()
 {
-  if (move_3d || map_mode) return;
-
-  static bool pressed = false;
-  static float angle_accel = 0;
-  static long start_time = csGetTicks ();
-
-  long cur_time = csGetTicks ();
-  if (!keep_old)
-  {
-    bool new_pressed = ABS (speed) > 0.001;
-    if (new_pressed != pressed)
-    {
-      pressed = new_pressed;
-      angle_accel = speed * cfg_rotate_accelerate;
-      start_time = cur_time - 100;
-    }
-  }
-
-  float max_speed = cfg_rotate_maxspeed * (kbd->GetKeyState (CSKEY_SHIFT) ? 2 : 1);
-
-  while ((cur_time - start_time) >= 100)
-  {
-    if (pressed)
-    {
-      // accelerate rotation
-      if (ABS (angle_velocity.y) < max_speed)
-        angle_velocity.y += angle_accel;
-      else if (ABS (angle_velocity.y) > max_speed)
-        angle_velocity.y -= angle_accel;
-    }
-    else
-    {
-      // brake!
-      if (angle_velocity.y > cfg_rotate_brake)
-        angle_velocity.y -= cfg_rotate_brake;
-      else if (angle_velocity.y < -cfg_rotate_brake)
-        angle_velocity.y += cfg_rotate_brake;
-      else
-        angle_velocity.y = 0;
-    }
-    start_time += 100;
-  }
+  velocity.y = 110.0f * cfg_jumpspeed;
+  desired_velocity.y = 0.0f;
 }
 
-void WalkTest::look (float speed,int keep_old)
+void WalkTest::Look (float speed)
 {
-  (void) speed; (void) keep_old;
-  if (move_3d || map_mode) return;
-  static float step_speed = 0;
-  if (!keep_old)
-    step_speed = speed*cfg_look_accelerate;
-  
-  //XXX: how to do this without angle?
-#if 0
-  if (ABS (angle.x+step_speed) <= (355.0/113.0/4))
-    angle.x += step_speed;
-#endif
-  RotateCam (-step_speed, 0);
+  if (move_3d) return;
+  desired_angle_velocity.x = 150.0f * speed * cfg_rotate_maxspeed;
 }
 
-void WalkTest::RotateCam(float x, float y)
+void WalkTest::Rotate (float speed)
 {
-  csMatrix3 mat = view->GetCamera ()->GetTransform ().GetO2T ();
-  if(x)
-    mat = csXRotMatrix3(x) * mat;
-  if(y)
-    mat *= csYRotMatrix3(y);
-  view->GetCamera ()->SetTransform ( csOrthoTransform 
-	  (mat, view->GetCamera ()->GetTransform ().GetOrigin ()));
+  if (move_3d) return;
+  desired_angle_velocity.y = 100.0f * speed * cfg_rotate_maxspeed
+  	* cfg_walk_maxspeed_multreal;
+}
+
+#undef WLK_ACCELERATE
+#define WLK_ACCELERATE(c) \
+  if (velocity. c < desired_velocity. c) \
+  { \
+    velocity. c += cfg_walk_accelerate * elapsed; \
+    if (velocity. c > desired_velocity. c) velocity. c = desired_velocity. c; \
+  } \
+  else \
+  { \
+    velocity. c -= cfg_walk_accelerate * elapsed; \
+    if (velocity. c < desired_velocity. c) velocity. c = desired_velocity. c; \
+  }
+
+#undef WLK_ROT_ACCELERATE
+#define WLK_ROT_ACCELERATE(c) \
+  if (angle_velocity. c < desired_angle_velocity. c) \
+  { \
+    angle_velocity. c += cfg_rotate_accelerate * elapsed; \
+    if (angle_velocity. c > desired_angle_velocity. c) angle_velocity. c = desired_angle_velocity. c; \
+  } \
+  else \
+  { \
+    angle_velocity. c -= cfg_rotate_accelerate * elapsed; \
+    if (angle_velocity. c < desired_angle_velocity. c) angle_velocity. c = desired_angle_velocity. c; \
+  }
+
+void WalkTest::InterpolateMovement ()
+{
+  float elapsed = vc->GetElapsedTicks () / 1000.0f;
+  elapsed *= 1700.0f;
+
+  WLK_ACCELERATE(x)
+  WLK_ACCELERATE(y)
+  WLK_ACCELERATE(z)
+
+  WLK_ROT_ACCELERATE(x)
+  WLK_ROT_ACCELERATE(y)
+  WLK_ROT_ACCELERATE(z)
+}
+
+void WalkTest::RotateCam (float x, float y)
+{
+  csVector3 rot = collider_actor.GetRotation ();
+  rot.x += x;
+  rot.y += y;
+  collider_actor.SetRotation (rot);
 }
 
 void WalkTest::imm_forward (float speed, bool slow, bool fast)
 {
-  if (map_mode) { wf->KeyUp (speed, slow, fast); return; }
   if (slow)
-    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_FORWARD, do_cd);
+    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_FORWARD, collider_actor.HasCD ());
   else if (fast)
-    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_FORWARD, do_cd);
+    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_FORWARD, collider_actor.HasCD ());
   else
-    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_FORWARD, do_cd);
+    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_FORWARD, collider_actor.HasCD ());
 }
 
 void WalkTest::imm_backward (float speed, bool slow, bool fast)
 {
-  if (map_mode) { wf->KeyDown (speed, slow, fast); return; }
   if (slow)
-    view->GetCamera ()->Move (speed*.01*CS_VEC_BACKWARD, do_cd);
+    view->GetCamera ()->Move (speed*.01*CS_VEC_BACKWARD, collider_actor.HasCD ());
   else if (fast)
-    view->GetCamera ()->Move (speed*1.2*CS_VEC_BACKWARD, do_cd);
+    view->GetCamera ()->Move (speed*1.2*CS_VEC_BACKWARD, collider_actor.HasCD ());
   else
-    view->GetCamera ()->Move (speed*.6*CS_VEC_BACKWARD, do_cd);
+    view->GetCamera ()->Move (speed*.6*CS_VEC_BACKWARD, collider_actor.HasCD ());
 }
 
 void WalkTest::imm_left (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
-    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_LEFT, do_cd);
+    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_LEFT, collider_actor.HasCD ());
   else if (fast)
-    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_LEFT, do_cd);
+    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_LEFT, collider_actor.HasCD ());
   else
-    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_LEFT, do_cd);
+    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_LEFT, collider_actor.HasCD ());
 }
 
 void WalkTest::imm_right (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
-    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_RIGHT, do_cd);
+    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_RIGHT, collider_actor.HasCD ());
   else if (fast)
-    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_RIGHT, do_cd);
+    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_RIGHT, collider_actor.HasCD ());
   else
-    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_RIGHT, do_cd);
+    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_RIGHT, collider_actor.HasCD ());
 }
 
 void WalkTest::imm_up (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
-    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_UP, do_cd);
+    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_UP, collider_actor.HasCD ());
   else if (fast)
-    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_UP, do_cd);
+    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_UP, collider_actor.HasCD ());
   else
-    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_UP, do_cd);
+    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_UP, collider_actor.HasCD ());
 }
 
 void WalkTest::imm_down (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
-    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_DOWN, do_cd);
+    view->GetCamera ()->Move (speed * 0.01 * CS_VEC_DOWN, collider_actor.HasCD ());
   else if (fast)
-    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_DOWN, do_cd);
+    view->GetCamera ()->Move (speed * 4.0 * CS_VEC_DOWN, collider_actor.HasCD ());
   else
-    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_DOWN, do_cd);
+    view->GetCamera ()->Move (speed * 1.0 * CS_VEC_DOWN, collider_actor.HasCD ());
 }
 
 void WalkTest::imm_rot_left_camera (float speed, bool slow, bool fast)
 {
-  if (map_mode == MAP_TXT) { wf->KeyLeftStrafe (speed, slow, fast); return; }
-  if (map_mode) { wf->KeyLeft (speed, slow, fast); return; }
   if (slow)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_ROT_LEFT, speed * .01);
   else if (fast)
@@ -481,7 +414,6 @@ void WalkTest::imm_rot_left_camera (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_left_world (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
     view->GetCamera ()->GetTransform ().RotateOther (CS_VEC_ROT_LEFT, speed * .01);
   else if (fast)
@@ -492,8 +424,6 @@ void WalkTest::imm_rot_left_world (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_right_camera (float speed, bool slow, bool fast)
 {
-  if (map_mode == MAP_TXT) { wf->KeyRightStrafe (speed, slow, fast); return; }
-  if (map_mode) { wf->KeyRight (speed, slow, fast); return; }
   if (slow)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_ROT_RIGHT, speed * .01);
   else if (fast)
@@ -504,7 +434,6 @@ void WalkTest::imm_rot_right_camera (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_right_world (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
     view->GetCamera ()->GetTransform ().RotateOther (CS_VEC_ROT_RIGHT, speed * .01);
   else if (fast)
@@ -515,7 +444,6 @@ void WalkTest::imm_rot_right_world (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_left_xaxis (float speed, bool slow, bool fast)
 {
-  if (map_mode) { wf->KeyPgDn (speed, slow, fast); return; }
   if (slow)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_TILT_DOWN, speed * .01);
   else if (fast)
@@ -526,7 +454,6 @@ void WalkTest::imm_rot_left_xaxis (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_right_xaxis (float speed, bool slow, bool fast)
 {
-  if (map_mode) { wf->KeyPgUp (speed, slow, fast); return; }
   if (slow)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_TILT_UP, speed * .01);
   else if (fast)
@@ -537,7 +464,6 @@ void WalkTest::imm_rot_right_xaxis (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_left_zaxis (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_TILT_LEFT, speed * .01);
   else if (fast)
@@ -548,7 +474,6 @@ void WalkTest::imm_rot_left_zaxis (float speed, bool slow, bool fast)
 
 void WalkTest::imm_rot_right_zaxis (float speed, bool slow, bool fast)
 {
-  if (map_mode) return;
   if (slow)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_TILT_RIGHT, speed * .01);
   else if (fast)
@@ -557,23 +482,31 @@ void WalkTest::imm_rot_right_zaxis (float speed, bool slow, bool fast)
     view->GetCamera ()->GetTransform ().RotateThis (CS_VEC_TILT_RIGHT, speed * .2);
 }
 
-void WalkTest::eatkeypress (iEvent &Event)
+void WalkTest::eatkeypress (iEvent* Event)
 {
-  int key = Event.Key.Code;
-  int status = (Event.Type == csevKeyDown);
-  bool shift = (Event.Key.Modifiers & CSMASK_SHIFT) != 0;
-  bool alt = (Event.Key.Modifiers & CSMASK_ALT) != 0;
-  bool ctrl = (Event.Key.Modifiers & CSMASK_CTRL) != 0;
+  uint32 key = 0;
+  Event->Retrieve ("keyCodeRaw", key);
+
+  uint8 evType = 0;
+  Event->Retrieve ("keyEventType", evType);
+  bool status = (evType == csKeyEventTypeDown);
 
   if (myConsole && myConsole->GetVisible () && status)
   {
     if (ConsoleInput)
-      ConsoleInput->HandleEvent (Event);
+      ConsoleInput->HandleEvent (*Event);
     //@@KLUDGE
     if (key != CSKEY_TAB)
       return;
   }
 
+  csKeyModifiers modifiers;
+  csKeyEventHelper::GetModifiers (Event, modifiers);
+
+  bool shift = modifiers.modifiers[csKeyModifierTypeShift] != 0;
+  bool alt = modifiers.modifiers[csKeyModifierTypeAlt] != 0;
+  bool ctrl = modifiers.modifiers[csKeyModifierTypeCtrl] != 0;
+
   csKeyMap *m = mapping;
   while (m)
   {
@@ -585,8 +518,8 @@ void WalkTest::eatkeypress (iEvent &Event)
         // Don't perform the command again if the key is already down
         if (m->is_on != status)
         {
-          char buf [256];
-          sprintf (buf,"%s %d", m->cmd, status);
+          csString buf;
+          buf.Format ("%s %d", m->cmd, status);
           csCommandProcessor::perform_line (buf);
           m->is_on = status;
         }
@@ -601,8 +534,8 @@ void WalkTest::eatkeypress (iEvent &Event)
     {
       if (key == m->key || shift != m->shift || alt != m->alt || ctrl != m->ctrl)
       {
-        char buf [256];
-        sprintf (buf,"%s 0", m->cmd);
+        csString buf;
+        buf.Format ("%s 0", m->cmd);
         csCommandProcessor::perform_line (buf);
         m->is_on = 0;
       }
@@ -619,11 +552,12 @@ void WalkTest::MouseClick1Handler(iEvent &Event)
   move_forward = true;
 }
 
-// middle mouse button
+// right mouse button
 void WalkTest::MouseClick2Handler(iEvent &Event)
 {
   csVector3 v;
-  csVector2 p (Event.Mouse.x, FRAME_HEIGHT-Event.Mouse.y);
+  csVector2 p (csMouseEventHelper::GetX(&Event), 
+	       FRAME_HEIGHT-csMouseEventHelper::GetY(&Event));
 
   view->GetCamera ()->InvPerspective (p, 1, v);
   csVector3 vw = view->GetCamera ()->GetTransform ().This2Other (v);
@@ -631,7 +565,9 @@ void WalkTest::MouseClick2Handler(iEvent &Event)
   iSector* sector = view->GetCamera ()->GetSector ();
   csVector3 origin = view->GetCamera ()->GetTransform ().GetO2TTranslation ();
   csVector3 isect;
-  iPolygon3D* sel = sector->HitBeam (origin, origin + (vw-origin) * 20, isect);
+  int sel;
+  iMeshWrapper* mesh = sector->HitBeamPortals (origin,
+  	origin + (vw-origin) * 20, isect, &sel);
 
   vw = isect;
   v = view->GetCamera ()->GetTransform ().Other2This (vw);
@@ -642,24 +578,25 @@ void WalkTest::MouseClick2Handler(iEvent &Event)
   	"LMB down : cam:(%f,%f,%f) world:(%f,%f,%f)",
 	v.x, v.y, v.z, vw.x, vw.y, vw.z);
 
-  if (sel)
+  if (mesh && sel != -1)
   {
-    if (Sys->selected_polygon == sel)
-      Sys->selected_polygon = NULL;
-    else
-      Sys->selected_polygon = sel;
+    csRef<iThingFactoryState> ps = 
+      scfQueryInterface<iThingFactoryState> (
+      mesh->GetMeshObject ()->GetFactory());
+    Sys->selected_polygon = sel;
 
-    iThingState* ps = sel->GetParent ();
-    iObject* psobj = SCF_QUERY_INTERFACE (ps, iObject);
+    iMeshObject* obj = mesh->GetMeshObject ();
+    csRef<iObject> psobj = SCF_QUERY_INTERFACE (obj->GetMeshWrapper (),
+    	iObject);
     Sys->Report (CS_REPORTER_SEVERITY_DEBUG, "Hit polygon '%s/%s'",
-    	psobj->GetName (), sel->QueryObject ()->GetName ());
+    	psobj ? psobj->GetName () : "<null>",
+	ps ? ps->GetPolygonName (sel) : "<null>");
     //Dumper::dump (sel);
-    psobj->DecRef ();
   }
 
   extern csVector2 coord_check_vector;
-  coord_check_vector.x = Event.Mouse.x;
-  coord_check_vector.y = FRAME_HEIGHT-Event.Mouse.y;
+  coord_check_vector.x = csMouseEventHelper::GetX(&Event);
+  coord_check_vector.y = FRAME_HEIGHT-csMouseEventHelper::GetY(&Event);
   extern bool check_light;
   extern void select_object (iRenderView* rview, int type, void* entity);
   check_light = true;
@@ -667,15 +604,15 @@ void WalkTest::MouseClick2Handler(iEvent &Event)
     //view->GetClipper (), select_object);
 }
 
-// right mouse button
+// middle mouse button
 void WalkTest::MouseClick3Handler(iEvent &Event)
 {
   csVector2   screenPoint;
   iMeshWrapper *closestMesh;
 
-  screenPoint.x = Event.Mouse.x;
-  screenPoint.y = Event.Mouse.y;
-  closestMesh = FindNextClosestMesh (NULL, view->GetCamera(), &screenPoint);
+  screenPoint.x = csMouseEventHelper::GetX(&Event);
+  screenPoint.y = csMouseEventHelper::GetY(&Event);
+  closestMesh = FindNextClosestMesh (0, view->GetCamera(), &screenPoint);
   if (closestMesh)
     Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
     	"Selected mesh %s", closestMesh->
@@ -687,104 +624,82 @@ void WalkTest::MouseClick3Handler(iEvent &Event)
 
 bool WalkTest::WalkHandleEvent (iEvent &Event)
 {
-  switch (Event.Type)
+  if (Event.Name == CanvasHidden)
   {
-    case csevBroadcast:
+    canvas_exposed = false;
+#ifdef CS_DEBUG
+    Report (CS_REPORTER_SEVERITY_NOTIFY, "canvas hidden");
+#endif
+  }
+  else if (Event.Name == CanvasExposed)
+  {
+    canvas_exposed = true;
+#ifdef CS_DEBUG
+    Report (CS_REPORTER_SEVERITY_NOTIFY, "canvas exposed");
+#endif
+  }
+  else if (Event.Name == CanvasResize)
+  {
+    Sys->FrameWidth = Sys->myG2D->GetWidth();
+    Sys->FrameHeight = Sys->myG2D->GetHeight();
+  }
+  else if (CS_IS_KEYBOARD_EVENT(name_reg, Event))
+  {
+    eatkeypress (&Event);
+  }
+  else if (CS_IS_MOUSE_EVENT(name_reg, Event))
+  {
+    switch(csMouseEventHelper::GetEventType(&Event))
     {
-      if (Event.Command.Code == cscmdContextResize)
+    case csMouseEventTypeDown:
+      switch(csMouseEventHelper::GetButton(&Event))
       {
-        int oldCWidth = FRAME_WIDTH;
-        int oldCHeight = FRAME_HEIGHT;
-	FRAME_WIDTH = myG2D->GetWidth();
-	FRAME_HEIGHT = myG2D->GetHeight();
-        if (split == -1)
-            view->GetCamera ()->SetPerspectiveCenter (FRAME_WIDTH/2, FRAME_HEIGHT/2);
-        else
-        {
-            csBox2 bbox1, bbox2;
-            BoundingBoxForView(Sys->views[0], &bbox1);
-            BoundingBoxForView(Sys->views[1], &bbox2);
-            
-            int oldVWidth = QInt(bbox2.MaxX() - bbox1.MinX());
-            int oldVHeight = QInt(bbox1.MaxY() - bbox1.MinY());
-            int newVWidth = FRAME_WIDTH - oldCWidth + oldVWidth;
-            int newVHeight = FRAME_HEIGHT - oldCHeight + oldVHeight;
-            if ((oldVWidth != newVWidth) || (oldVHeight != newVHeight))
-            {
-                views[0]->GetCamera()->SetPerspectiveCenter(bbox1.MinX() + (newVWidth / 4),
-                                                            bbox1.MinY() + (newVHeight / 2));
-                views[1]->GetCamera()->SetPerspectiveCenter(bbox1.MinX() + (3 * newVWidth / 4),
-                                                            bbox1.MinY() + (newVHeight / 2));
-            }
-        }
-        
-	if (wf)
-	  wf->GetCamera ()->SetPerspectiveCenter (FRAME_WIDTH/2, FRAME_HEIGHT/2);
-	break;
+	case csmbLeft:
+	  MouseClick1Handler(Event); break;
+	case csmbRight:
+	  MouseClick2Handler(Event); break;
+	case csmbMiddle:
+	  MouseClick3Handler(Event); break;
       }
-      else if (Event.Command.Code == cscmdCanvasHidden)
-      {
-	canvas_exposed = false;
-      #ifdef CS_DEBUG
-	Report (CS_REPORTER_SEVERITY_NOTIFY, "canvas hidden");
-      #endif
-	break;
-      }
-      else if (Event.Command.Code == cscmdCanvasExposed)
-      {
-	canvas_exposed = true;
-      #ifdef CS_DEBUG
-	Report (CS_REPORTER_SEVERITY_NOTIFY, "canvas exposed");
-      #endif
-	break;
-      }
-    }
-    case csevKeyDown:
-    case csevKeyUp:
-      eatkeypress (Event);
       break;
-    case csevMouseDown:
-      {
-      if (Event.Mouse.Button == 1)
-        MouseClick1Handler(Event);
-      else if (Event.Mouse.Button == 2)
-        MouseClick2Handler(Event);
-      else if (Event.Mouse.Button == 3)
-        MouseClick3Handler(Event);
-      break;
-      }
 
-    case csevMouseMove:
-      // additional command by Leslie Saputra -> freelook mode.
+    case csMouseEventTypeMove:
       {
-        static bool first_time = true;
-        if (do_freelook)
-        {
-          int last_x, last_y;
-          last_x = Event.Mouse.x;
-          last_y = Event.Mouse.y;
-
-          myG2D->SetMousePosition (FRAME_WIDTH / 2, FRAME_HEIGHT / 2);
-          if (!first_time)
-          {
+	// additional command by Leslie Saputra -> freelook mode.
+	static bool first_time = true;
+	if (do_freelook)
+	{
+	  int last_x, last_y;
+	  last_x = csMouseEventHelper::GetX(&Event);
+	  last_y = csMouseEventHelper::GetY(&Event);
+  	
+	  myG2D->SetMousePosition (FRAME_WIDTH / 2, FRAME_HEIGHT / 2);
+	  if (!first_time)
+	  {
 	    RotateCam (
-		-((float)(last_y - (FRAME_HEIGHT / 2) )) / (FRAME_HEIGHT*2)*(1-2*(int)inverse_mouse),
-		((float)(last_x - (FRAME_WIDTH / 2) )) / (FRAME_WIDTH*2));
-          }
-          else
-            first_time = false;
-        }
-        else
-          first_time = true;
+		      -((float)(last_y - (FRAME_HEIGHT / 2) )) / (FRAME_HEIGHT*2)*(1-2*(int)inverse_mouse),
+		      ((float)(last_x - (FRAME_WIDTH / 2) )) / (FRAME_WIDTH*2));
+	  }
+	  else
+	    first_time = false;
+	}
+	else
+	  first_time = true;
       }
       break;
-    case csevMouseUp:
-      if (Event.Mouse.Button == 1)
+
+    case csMouseEventTypeUp:
+      if (csMouseEventHelper::GetButton(&Event) == csmbLeft) 
       {
-        move_forward = false;
-	step (0, 0);
+	move_forward = false;
+	Step (0);
       }
       break;
+
+    default:
+      // ignore
+      break;
+    }
   }
 
   return false;
@@ -808,11 +723,11 @@ iMeshWrapper *FindNextClosestMesh (iMeshWrapper *baseMesh,
     	(camera, screenBoundingBox, bbox3);
     // if the baseMesh isn't in front of the camera, return
     if (closestZLocation < 0)
-      return NULL;
+      return 0;
   }
   else
   {
-    closestMesh = NULL;
+    closestMesh = 0;
     closestZLocation = 32000;
   }
 
