@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1998-2001 by Jorrit Tyberghein
+    Copyright (C) 1998-2005 by Jorrit Tyberghein
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,73 +22,27 @@
 */
 #include <stdarg.h>
 
-#define CS_SYSDEF_PROVIDE_ACCESS
-#include "cssysdef.h"
-#include "csutil/sysfunc.h"
+// Some external projects use gcc's -ansi and -pedantic options. We need to
+// ensure that CS's public headers are usable when these options are enabled,
+// Unfortunately, we can not enable these options globally since CS uses some
+// features (such as `long long') which are not available in the present
+// language standard. As a compromise, we enable these restrictions on an
+// individual module basis (see Jamfile) so as to ensure that the CS headers
+// are tested against these options on a regular basis. We purposely #include
+// "crystalspace.h" rather than including only the headers we need so that we
+// test as many headers for standards conformance as possible.
+#include "crystalspace.h"
+
 #include "walktest.h"
 #include "infmaze.h"
 #include "command.h"
-#include "igeom/polymesh.h"
-#include "csgeom/frustum.h"
-#include "iengine/region.h"
-#include "iengine/light.h"
-#include "iengine/camera.h"
-#include "iengine/sector.h"
-#include "csgeom/csrect.h"
-#include "csutil/scanstr.h"
-#include "csutil/csobject.h"
-#include "csutil/cspmeter.h"
-#include "cstool/cspixmap.h"
-#include "cstool/csfxscr.h"
-#include "cstool/csview.h"
-#include "cstool/initapp.h"
-#include "csver.h"
-#include "qint.h"
-#include "iutil/cfgmgr.h"
-#include "iutil/cmdline.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/natwin.h"
-#include "ivideo/txtmgr.h"
-#include "isound/handle.h"
-#include "isound/source.h"
-#include "isound/listener.h"
-#include "isound/renderer.h"
-#include "igraphic/image.h"
-#include "igraphic/imageio.h"
-#include "ivaria/collider.h"
-#include "ivaria/reporter.h"
-#include "ivaria/stdrep.h"
-#include "imap/parser.h"
-#include "csutil/cmdhelp.h"
-#include "iutil/event.h"
-#include "iutil/objreg.h"
-#include "iutil/csinput.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/virtclk.h"
-#include "isound/wrapper.h"
-#include "imesh/thing.h"
-#include "imesh/terrfunc.h"
-#include "imesh/object.h"
-#include "imesh/mdlconv.h"
-#include "imesh/crossbld.h"
-#include "iengine/movable.h"
-#include "iengine/campos.h"
-#include "iutil/plugin.h"
 
-#if defined(OS_WIN32)
-#include "csutil/win32/minidump.h"
-#endif
-
-#if defined(OS_DOS) || defined(OS_WIN32)
+#if defined(CS_PLATFORM_DOS) || defined(CS_PLATFORM_WIN32)
 #  include <io.h>
-#elif defined(OS_UNIX)
+#elif defined(CS_PLATFORM_UNIX)
 #  include <unistd.h>
 #endif
 
-#include "csutil/fpu80x86.h"	// for debugging numerical instabilities
-
 WalkTest *Sys;
 
 #define Gfx3D Sys->myG3D
@@ -118,11 +72,9 @@ WalkTest::WalkTest () :
   do_show_z = false;
   do_show_palette = false;
   do_infinite = false;
-  do_cd = true;
   do_freelook = false;
   do_logo = true;
   player_spawned = false;
-  do_gravity = true;
   inverse_mouse = false;
   selected_light = 0;
   selected_polygon = 0;
@@ -143,7 +95,9 @@ WalkTest::WalkTest () :
   do_fs_fadeout = false;
 
   velocity.Set (0, 0, 0);
+  desired_velocity.Set (0, 0, 0);
   angle_velocity.Set (0, 0, 0);
+  desired_angle_velocity.Set (0, 0, 0);
 
   bgcolor_txtmap = 255;
   bgcolor_map = 0;
@@ -201,7 +155,8 @@ void WalkTest::Report (int severity, const char* msg, ...)
 void WalkTest::SetDefaults ()
 {
   csRef<iConfigManager> Config (CS_QUERY_REGISTRY (object_reg, iConfigManager));
-  do_cd = Config->GetBool ("Walktest.Settings.Colldet", true);
+  bool do_cd = Config->GetBool ("Walktest.Settings.Colldet", true);
+  collider_actor.SetCD (do_cd);
   do_logo = Config->GetBool ("Walktest.Settings.DrawLogo", true);
 
   csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
@@ -217,7 +172,6 @@ void WalkTest::SetDefaults ()
   {
     num_maps++;
     idx++;
-    char map_dir[512];
     csMapToLoad* map = new csMapToLoad;
 
     // If the given value starts with "cache:" the given location will not
@@ -230,13 +184,7 @@ void WalkTest::SetDefaults ()
       val += 6;
     }
 
-    // if an absolute path is given, copy it. Otherwise prepend "/lev/".
-    if (val[0] == '/')
-      strcpy (map_dir, val);
-    else
-      sprintf (map_dir, "/lev/%s", val);
-
-    map->map_dir = csStrNew (map_dir);
+    map->map_dir = csStrNew (val);
     map->next_map = 0;
     if (last_map)
       last_map->next_map = map;
@@ -255,12 +203,12 @@ void WalkTest::SetDefaults ()
 
   if (cmdline->GetOption ("colldet"))
   {
-    do_cd = true;
+    collider_actor.SetCD (true);
     Report (CS_REPORTER_SEVERITY_NOTIFY, "Enabled collision detection system.");
   }
   else if (cmdline->GetOption ("nocolldet"))
   {
-    do_cd = false;
+    collider_actor.SetCD (false);
     Report (CS_REPORTER_SEVERITY_NOTIFY, "Disabled collision detection system.");
   }
 
@@ -280,21 +228,28 @@ void WalkTest::SetDefaults ()
     do_logo = false;
     Report (CS_REPORTER_SEVERITY_NOTIFY, "Logo disabled.");
   }
+
+  bool doSave = Config->GetBool ("Walktest.Settings.EnableEngineSaving", 
+    false);
+  doSave = cmdline->GetBoolOption ("saveable", doSave);
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "World saving %s.", 
+    doSave ? "enabled" : "disabled");
 }
 
 void WalkTest::Help ()
 {
   csRef<iConfigManager> cfg (CS_QUERY_REGISTRY (object_reg, iConfigManager));
-  printf ("Options for WalkTest:\n");
-  printf ("  -exec=<script>     execute given script at startup\n");
-  printf ("  -[no]colldet       collision detection system (default '%scolldet')\n", do_cd ? "" : "no");
-  printf ("  -[no]logo          draw logo (default '%slogo')\n", do_logo ? "" : "no");
-  printf ("  -regions           load every map in a separate region (default off)\n");
-  printf ("  -dupes             check for duplicate objects in multiple maps (default off)\n");
-  printf ("  -prepare           after loading, pre-prepare all things (default off)\n");
-  printf ("  -infinite          special infinite level generation (ignores map file!)\n");
-  printf ("  -bots              allow random generation of bots\n");
-  printf ("  <path>             load map from VFS <path> (default '%s')\n",
+  csPrintf ("Options for WalkTest:\n");
+  csPrintf ("  -exec=<script>     execute given script at startup\n");
+  csPrintf ("  -[no]colldet       collision detection system (default '%scolldet')\n", collider_actor.HasCD () ? "" : "no");
+  csPrintf ("  -[no]logo          draw logo (default '%slogo')\n", do_logo ? "" : "no");
+  csPrintf ("  -regions           load every map in a separate region (default off)\n");
+  csPrintf ("  -dupes             check for duplicate objects in multiple maps (default off)\n");
+  csPrintf ("  -noprecache        after loading don't precache to speed up rendering\n");
+  csPrintf ("  -infinite          special infinite level generation (ignores map file!)\n");
+  csPrintf ("  -bots              allow random generation of bots\n");
+  csPrintf ("  -[no]saveable      enable/disable engine 'saveable' flag\n");
+  csPrintf ("  <path>             load map from VFS <path> (default '%s')\n",
         cfg->GetStr ("Walktest.Settings.WorldFile", "world"));
 }
 
@@ -376,23 +331,11 @@ void WalkTest::MoveSystems (csTicks elapsed_time, csTicks current_time)
     }
     move->UpdateMove ();
   }
-  // Move the directional light if any.
-  if (anim_dirlight)
-  {
-    csRef<iTerrFuncState> state (SCF_QUERY_INTERFACE (
-    	anim_dirlight->GetMeshObject (),
-	iTerrFuncState));
-    csVector3 pos = state->GetDirLightPosition ();
-    csColor col = state->GetDirLightColor ();
-    csYRotMatrix3 mat (.05 * TWO_PI * (float)elapsed_time/1000.);
-    pos = mat * pos;
-    state->SetDirLight (pos, col);
-  }
 
   // Update all busy entities.
   // We first push all entities in a vector so that NextFrame() can safely
   // remove it self from the busy_entities list (or add other entities).
-  int i;
+  size_t i;
   busy_vector.DeleteAll ();
   csWalkEntity* wentity;
   for (i = 0 ; i < busy_entities.Length () ; i++)
@@ -426,22 +369,7 @@ void WalkTest::MoveSystems (csTicks elapsed_time, csTicks current_time)
   }
   if (!myConsole || !myConsole->GetVisible ())
   {
-    int alt,shift,ctrl;
-    float speed = 1;
-
-    alt = kbd->GetKeyState (CSKEY_ALT);
-    ctrl = kbd->GetKeyState (CSKEY_CTRL);
-    shift = kbd->GetKeyState (CSKEY_SHIFT);
-    if (ctrl)
-      speed = .5;
-    if (shift)
-      speed = 2;
-
-    /// Act as usual...
-    strafe (0,1);
-    look (0,1);
-    step (0,1);
-    rotate (0,1);
+    InterpolateMovement ();
 
     if (mySound)
     {
@@ -463,7 +391,7 @@ void WalkTest::MoveSystems (csTicks elapsed_time, csTicks current_time)
   move_bots (elapsed_time);
 
   if (move_forward)
-    step (1, 0);
+    Step (1);
 }
 
 
@@ -515,10 +443,10 @@ void WalkTest::DrawFrameDebug3D ()
 void WalkTest::GfxWrite (int x, int y, int fg, int bg, char *str, ...)
 {
   va_list arg;
-  char buf[256];
+  csString buf;
 
   va_start (arg, str);
-  vsprintf (buf, str, arg);
+  buf.FormatV (str, arg);
   va_end (arg);
 
   myG2D->Write (Font, x, y, fg, bg, buf);
@@ -536,8 +464,8 @@ void WalkTest::DrawFrameConsole ()
 
     if (do_show_coord)
     {
-      char buffer[100];
-      sprintf (buffer, "%2.2f,%2.2f,%2.2f: %s",
+      csString buffer;
+      buffer.Format ("%2.2f,%2.2f,%2.2f: %s",
       view->GetCamera ()->GetTransform ().GetO2TTranslation ().x,
       view->GetCamera ()->GetTransform ().GetO2TTranslation ().y,
       view->GetCamera ()->GetTransform ().GetO2TTranslation ().z,
@@ -548,11 +476,11 @@ void WalkTest::DrawFrameConsole ()
 
       GfxWrite ( FRAME_WIDTH - buffWidth - 1, 
                  FRAME_HEIGHT - fh - 3, 0, -1, 
-                 buffer);
+                 "%s", buffer.GetData());
 
       GfxWrite (FRAME_WIDTH - buffWidth, 
                 FRAME_HEIGHT - fh - 2, fgcolor_stats, -1, 
-                buffer);
+                "%s", buffer.GetData());
     }
   }
 }
@@ -562,8 +490,7 @@ void WalkTest::DrawFullScreenFX2D (csTicks /*elapsed_time*/,
 {
   if (do_fs_inter)
   {
-    iTextureManager* txtmgr = Gfx3D->GetTextureManager ();
-    csfxInterference (Gfx2D, txtmgr, fs_inter_amount, fs_inter_anim,
+    csfxInterference (Gfx2D, fs_inter_amount, fs_inter_anim,
     	fs_inter_length);
     fs_inter_anim = fmod (fabs (float (current_time)/3000.0), 1.);
   }
@@ -590,7 +517,7 @@ void WalkTest::DrawFullScreenFX3D (csTicks /*elapsed_time*/,
   }
   if (do_fs_fadetxt)
   {
-    csfxFadeTo (Gfx3D, fs_fadetxt_mat, fs_fadetxt_fade);
+    csfxFadeTo (Gfx3D, fs_fadetxt_txt, fs_fadetxt_fade);
     float t3 = fabs (float (current_time)/3000.0);
     fs_fadetxt_fade = fmod (t3, 1.0f);
     fs_fadetxt_dir = fmod (t3, 2.0f) >= 1;
@@ -635,10 +562,6 @@ void WalkTest::DrawFullScreenFX3D (csTicks /*elapsed_time*/,
   }
 }
 
-SCF_IMPLEMENT_IBASE_EXT (WalkDataObject)
-  SCF_IMPLEMENTS_INTERFACE (WalkDataObject)
-SCF_IMPLEMENT_IBASE_EXT_END
-
 void WalkTest::DrawFrame3D (int drawflags, csTicks /*current_time*/)
 {
   // Tell Gfx3D we're going to display 3D things
@@ -647,7 +570,7 @@ void WalkTest::DrawFrame3D (int drawflags, csTicks /*current_time*/)
     return;
 
   // Apply lighting BEFORE the very first frame
-  int i;
+  size_t i;
   for (i = 0 ; i < dynamic_lights.Length () ; i++)
   {
     iLight* dyn = dynamic_lights[i];
@@ -695,8 +618,7 @@ void WalkTest::DrawFrame2D (void)
      * to look better on various backgrounds than the usual logo, which only looks
      * good on black.
      * The logo was drawn as an SVG image using Sodipodi, so it should be usable in
-     * many other contexts if desired. The originals should appear on sunsite.dk
-     * sometime soon. If not, email Micah to get them.
+     * many other contexts if desired.
      */
 
     // Margin to the edge of the screen, as a fraction of screen width
@@ -755,7 +677,7 @@ void WalkTest::DrawFrame (csTicks elapsed_time, csTicks current_time)
       csRecordedCamera* reccam = (csRecordedCamera*)recording[cfg_playrecording];
       cfg_playrecording++;
       record_frame_count++;
-      if (cfg_playrecording >= recording.Length ())
+      if ((size_t)cfg_playrecording >= recording.Length ())
       {
 	csTicks t1 = record_start_time;
 	csTicks t2 = csGetTicks ();
@@ -764,7 +686,7 @@ void WalkTest::DrawFrame (csTicks elapsed_time, csTicks current_time)
 	  "End demo: %f secs to render %d frames: %f fps",
 	  (float) (t2 - t1) / 1000.,
 	  num, float (num) * 1000. / (float) (t2 - t1));
-	printf (
+	csPrintf (
 	  "End demo: %f secs to render %d frames: %f fps\n",
 	  (float) (t2 - t1) / 1000.,
 	  num, float (num) * 1000. / (float) (t2 - t1));
@@ -824,63 +746,27 @@ void WalkTest::DrawFrame (csTicks elapsed_time, csTicks current_time)
 }
 
 int cnt = 1;
-csTicks time0 = (csTicks)-1;
 
-void WalkTest::PrepareFrame (csTicks elapsed_time, csTicks current_time)
+void WalkTest::PrepareFrame (csTicks elapsed_time, csTicks /*current_time*/)
 {
-  static csTicks prev_time = 0;
-  if (prev_time == 0) prev_time = csGetTicks () - 10;
-
-  // If the time interval is too big, limit to something reasonable
-  // This will help a little for software OpenGL :-)
-  elapsed_time = current_time - prev_time;
-  if (elapsed_time > 250)
-    prev_time = current_time - (elapsed_time = 250);
-
-  if (do_cd)
+  if (!player_spawned)
   {
-    extern void DoGravity (iEngine* engine, csVector3& pos, csVector3& vel);
-    if (!player_spawned)
-    {
-      CreateColliders ();
-      player_spawned=true;
-    }
-
-    for (; elapsed_time >= 10; elapsed_time -= 10, prev_time += 10)
-    {
-      if (move_3d)
-      {
-        // If we are moving in 3d then don't do any camera correction.
-      }
-      else
-      {
-	// Rotate Camera
-	RotateCam(angle_velocity.x, angle_velocity.y);
-      }
-
-      csVector3 vel = view->GetCamera ()->GetTransform ().GetT2O ()*velocity;
-
-      static bool check_once = false;
-      if (ABS (vel.x) < SMALL_EPSILON && ABS (vel.y)
-      	< SMALL_EPSILON && ABS (vel.z) < SMALL_EPSILON)
-      {
-        // If we don't move we don't have to do the collision detection tests.
-	// However, we need to do it once to make sure that we are standing
-	// on solid ground. So we first set 'check_once' to true to enable
-	// one more test.
-	if (check_once == false)
-	{
-	  check_once = true;
-	  DoGravity (Engine, pos, vel);
-	}
-      }
-      else
-      {
-        check_once = false;
-	DoGravity (Engine, pos, vel);
-      }
-    }
+    CreateColliders ();
+    player_spawned=true;
   }
+
+  int shift, ctrl;
+  float speed = 1;
+
+  ctrl = kbd->GetKeyState (CSKEY_CTRL);
+  shift = kbd->GetKeyState (CSKEY_SHIFT);
+  if (ctrl)
+    speed = .5;
+  if (shift)
+    speed = 2;
+
+  float delta = float (elapsed_time) / 1000.0f;
+  collider_actor.Move (delta, speed, velocity, angle_velocity);
 }
 
 void perf_test (int num)
@@ -904,12 +790,11 @@ void perf_test (int num)
   Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
     "%f secs to render %d frames: %f fps", (float) (t2 - t1) / 1000., num,
     float (num) * 1000. / (float) (t2 - t1));
-  printf (
+  csPrintf (
     "%f secs to render %d frames: %f fps\n", (float) (t2 - t1) / 1000., num,
     float (num) * 1000. / (float) (t2 - t1));
   fflush (stdout);
   cnt = 1;
-  time0 = (csTicks)-1;
   Sys->busy_perf_test = false;
 }
 
@@ -929,8 +814,6 @@ void Cleanup ()
 
 void start_console ()
 {
-  iTextureManager* txtmgr = Gfx3D->GetTextureManager ();
-  txtmgr->PrepareTextures ();
 }
 
 void WalkTest::EndEngine ()
@@ -940,12 +823,9 @@ void WalkTest::EndEngine ()
 
 void WalkTest::InitCollDet (iEngine* engine, iRegion* region)
 {
-  if (do_cd)
-  {
-    Report (CS_REPORTER_SEVERITY_NOTIFY, "Computing OBBs ...");
-    csColliderHelper::InitializeCollisionWrappers (collide_system,
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "Computing OBBs ...");
+  csColliderHelper::InitializeCollisionWrappers (collide_system,
     	engine, region);
-  }
 }
 
 void WalkTest::LoadLibraryData (iRegion* region)
@@ -970,7 +850,6 @@ void WalkTest::Inititalize2DTextures ()
 
 void WalkTest::Create2DSprites ()
 {
-  int w, h;
   iTextureWrapper *texh;
   iTextureHandle* phTex;
 
@@ -981,75 +860,45 @@ void WalkTest::Create2DSprites ()
     phTex = texh->GetTextureHandle();
     if (phTex)
     {
-      phTex->GetMipMapDimensions (0, w, h);
-      cslogo = new csSimplePixmap (phTex, 0, 0, w, h);
+      cslogo = new csSimplePixmap (phTex);
     }
   }
 }
 
 static bool WalkEventHandler (iEvent& ev)
 {
+  if (!Sys)
+    return false;
 
-  if (ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess)
+  if (ev.Name == Sys->Process)
   {
     Sys->SetupFrame ();
     return true;
   }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess)
+  else if (ev.Name == Sys->FinalProcess)
   {
     Sys->FinishFrame ();
     return true;
   }
-  else if (ev.Type == csevBroadcast && ev.Command.Code == cscmdCommandLineHelp)
+  else if (ev.Name == Sys->CommandLineHelp)
   {
     Sys->Help ();
     return true;
   }
   else
   {
-    return Sys ? Sys->WalkHandleEvent (ev) : false;
+    return Sys->WalkHandleEvent (ev);
   }
 }
 
 bool WalkTest::SetMapDir (const char* map_dir)
 {
-  char tmp[512];
-  sprintf (tmp, "%s/", map_dir);
-  csRef<iConfigManager> cfg (CS_QUERY_REGISTRY (object_reg, iConfigManager));
-  if (!myVFS->Exists (map_dir))
+  csStringArray paths;
+  paths.Push ("/lev/");
+  if (!myVFS->ChDirAuto (map_dir, &paths, 0, "world"))
   {
-    char *name = strrchr (map_dir, '/');
-    if (name)
-    {
-      name++;
-      //sprintf (tmp, "$.$/data$/%s.zip, $.$/%s.zip, $(..)$/data$/%s.zip",
-      //  name, name, name);
-      const char *valfiletype = "";
-      valfiletype = cfg->GetStr ("Walktest.Settings.WorldZipType", "");
-      if (strcmp (valfiletype, "") ==0)
-      {
-        valfiletype = "zip";
-      }
-      char* extension = strrchr (name, '.');
-      if (extension && !strcmp (extension+1, valfiletype))
-      {
-        // The file already ends with the correct extension.
-        sprintf (tmp, "$.$/data$/%s, $.$/%s, $(..)$/data$/%s, %s",
-            name, name, name, name);
-      }
-      else
-      {
-        // Add the extension.
-        sprintf (tmp, "$.$/data$/%s.%s, $.$/%s.%s, $(..)$/data$/%s.%s ",
-            name, valfiletype, name, valfiletype, name, valfiletype);
-      }
-      myVFS->Mount (map_dir, tmp);
-    }
-  }
-  if (!myVFS->ChDir (map_dir))
-  {
-    Report (CS_REPORTER_SEVERITY_ERROR,
-    	"The directory on VFS for map file does not exist!");
+    Report (CS_REPORTER_SEVERITY_ERROR, "Error setting directory '%s'!",
+    	map_dir);
     return false;
   }
   return true;
@@ -1076,13 +925,13 @@ public:
   virtual void Run ()
   {
     csSleep (2000);
-    printf ("================ START PARSING!\n"); fflush (stdout);
-    const char* error = doc->Parse (buf);
+    csPrintf ("================ START PARSING!\n"); fflush (stdout);
+    const char* error = doc->Parse (buf, true);
     if (error != 0)
     {
-      printf ("Document system error for file '%s'!", error);
+      csPrintf ("Document system error for file '%s'!", error);
     }
-    printf ("================ END PARSING!\n"); fflush (stdout);
+    csPrintf ("================ END PARSING!\n"); fflush (stdout);
   }
   virtual void IncRef () { ref++; }
   virtual void DecRef () { ref--; if (ref <= 0) delete this; }
@@ -1100,7 +949,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   object_reg = csInitializer::CreateEnvironment (argc, argv);
   if (!object_reg) return false;
 
-#if defined(OS_WIN32)
+#if defined(CS_PLATFORM_WIN32)
   cswinMinidumpWriter::SetCrashMinidumpObjectReg (Sys->object_reg);
 #endif
 
@@ -1111,9 +960,9 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   }
 
   csRef<iConfigManager> cfg (CS_QUERY_REGISTRY (object_reg, iConfigManager));
-#if defined(OS_WIN32)
+#if defined(CS_PLATFORM_WIN32)
   const bool mdumpDefault =
-#if defined(COMP_VC)
+#if defined(CS_COMPILER_MSVC)
     true;
 #else
     false;
@@ -1185,6 +1034,14 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     return false;
   }
 
+  name_reg = csEventNameRegistry::GetRegistry (object_reg);
+  Process = csevProcess (name_reg);
+  FinalProcess = csevFinalProcess (name_reg);
+  CommandLineHelp = csevCommandLineHelp (name_reg);
+  CanvasHidden = csevCanvasHidden (name_reg, myG2D);
+  CanvasExposed = csevCanvasExposed (name_reg, myG2D);
+  CanvasResize = csevCanvasResize (name_reg, myG2D);
+
   myConsole = CS_QUERY_REGISTRY (object_reg, iConsoleOutput);
   mySound = CS_QUERY_REGISTRY (object_reg, iSoundRender);
 
@@ -1196,6 +1053,8 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   cfg_jumpspeed = cfg->GetFloat ("Walktest.CollDet.JumpSpeed", 0.08f);
   cfg_walk_accelerate = cfg->GetFloat ("Walktest.CollDet.WalkAccelerate", 0.007f);
   cfg_walk_maxspeed = cfg->GetFloat ("Walktest.CollDet.WalkMaxSpeed", 0.1f);
+  cfg_walk_maxspeed_mult = cfg->GetFloat ("Walktest.CollDet.WalkMaxSpeedMult", 10.0f);
+  cfg_walk_maxspeed_multreal = 1.0f;
   cfg_walk_brake = cfg->GetFloat ("Walktest.CollDet.WalkBrake", 0.014f);
   cfg_rotate_accelerate = cfg->GetFloat ("Walktest.CollDet.RotateAccelerate", 0.005f);
   cfg_rotate_maxspeed = cfg->GetFloat ("Walktest.CollDet.RotateMaxSpeed", 0.03f);
@@ -1253,6 +1112,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     Report (CS_REPORTER_SEVERITY_ERROR, "No iEngine plugin!");
     return false;
   }
+  Engine->SetSaveableFlag (doSave);
 
   // Find the level loader plugin
   LevelLoader = CS_QUERY_REGISTRY (object_reg, iLoader);
@@ -1262,12 +1122,6 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     return false;
   }
 
-  // Find the model converter plugin
-  ModelConverter = CS_QUERY_REGISTRY (object_reg, iModelConverter);
-
-  // Find the model crossbuilder plugin
-  CrossBuilder = CS_QUERY_REGISTRY (object_reg, iCrossBuilder);
-
   // csView is a view encapsulating both a camera and a clipper.
   // You don't have to use csView as you can do the same by
   // manually creating a camera and a clipper but it makes things a little
@@ -1278,13 +1132,14 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
 
   // Get the collide system plugin.
   const char* p = cfg->GetStr ("Walktest.Settings.CollDetPlugin",
-  	"crystalspace.collisiondetection.rapid");
+  	"crystalspace.collisiondetection.opcode");
   collide_system = CS_LOAD_PLUGIN (plugin_mgr, p, iCollideSystem);
   if (!collide_system)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No Collision Detection plugin found!");
     return false;
   }
+  object_reg->Register (collide_system, "iCollideSystem");
 
   // Initialize the command processor with the engine and camera.
   csCommandProcessor::Initialize (Engine, view->GetCamera (),
@@ -1309,7 +1164,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     // though collision detection does not really make sense in this context.
     // Hopefully the movement system will be fixed some day so that the user
     // can move around even with collision detection disabled.
-    do_cd = true;
+    collider_actor.SetCD (true);
 
     // Load two textures that are used in the maze.
     if (!LevelLoader->LoadTexture ("txt", "/lib/std/stone4.gif"))
@@ -1358,6 +1213,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       infinite_maze->connect_infinite (0, 1, 3, 0, 2, 3);
       infinite_maze->create_loose_portal (-2, 0, 4, -2, 1, 4);
       view->GetCamera ()->SetSector(room);
+      printf ("Done creation of infinite maze!\n"); fflush (stdout);
     }
 
     // Prepare the engine. This will calculate all lighting and
@@ -1397,15 +1253,11 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       
     if ((!do_regions) && cache_map != 0)
     {
-      // First we force a clear of the cache manager in the engine
-      // so that a new one will be made soon.
-      Engine->SetCacheManager (0);
       // Then we set the current directory right.
       if (!SetMapDir (cache_map->map_dir))
 	return false;
-      // And finally we get the cache manager which will force it
-      // to be created based on current VFS dir.
-      Engine->GetCacheManager ();
+      // Set the cache manager based on current VFS dir.
+      Engine->SetVFSCacheManager ();
     }
 
     // Check the map and mount it if required.
@@ -1439,12 +1291,8 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       }
       if (do_regions)
       {
-        // First we force a clear of the cache manager in the engine
-        // so that a new one will be made soon.
-        Engine->SetCacheManager (0);
-        // And finally we get the cache manager which will force it
-        // to be created based on current VFS dir.
-        Engine->GetCacheManager ();
+        // Set the cache manager based on current VFS dir.
+        Engine->SetVFSCacheManager ();
         region->Prepare ();
       }
     }
@@ -1468,34 +1316,16 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       Engine->Prepare (meter);
       delete meter;
     }
-    else
-    {
-      // Here we free all loaded images. Normally engine->Prepare()
-      // will do that for us.
-      iTextureManager *txtmgr = Gfx3D->GetTextureManager ();
-      txtmgr->FreeImages ();
-    }
 
-    if (cmdline->GetOption ("prepare"))
+    if (!cmdline->GetOption ("noprecache"))
     {
-      Report (CS_REPORTER_SEVERITY_NOTIFY, "Preparing all things...");
-      iMeshList* ml = Engine->GetMeshes ();
-      int i;
-      for (i = 0 ; i < ml->GetCount () ; i++)
-      {
-        iMeshWrapper* m = ml->Get (i);
-	if (m->GetMeshObject ())
-	{
-	  csRef<iThingState> thing = SCF_QUERY_INTERFACE (m->GetMeshObject (),
-	  	iThingState);
-	  if (thing) thing->Prepare ();
-        }
-      }
-      Report (CS_REPORTER_SEVERITY_NOTIFY, "Preparing finished...");
+      Report (CS_REPORTER_SEVERITY_NOTIFY, "Precaching all things...");
+      Engine->PrecacheDraw ();
+      Report (CS_REPORTER_SEVERITY_NOTIFY, "Precaching finished...");
     }
 
     csTicks stop_time = csGetTicks ();
-    printf ("Total level load time: %g seconds\n",
+    csPrintf ("Total level load time: %g seconds\n",
     	float (stop_time-start_time) / 1000.0); fflush (stdout);
 
     Create2DSprites ();
@@ -1585,16 +1415,17 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   // real window for example).
   int w3d = Gfx3D->GetWidth ();
   int h3d = Gfx3D->GetHeight ();
+  // clear all backbuffers to black
+  // and Zbuffer, mainly to make better zbufdumps
+  Gfx3D->BeginDraw (CSDRAW_2DGRAPHICS | CSDRAW_CLEARZBUFFER);
+  myG2D->ClearAll (myG2D->FindRGB(0,0,0));
+  Gfx3D->FinishDraw ();
 #ifdef CS_DEBUG
   view->SetRectangle (2, 2, w3d - 4, h3d - 4);
   myG2D->SetClipRect (2, 2, w3d - 2, h3d - 2);
 #else
   view->SetRectangle (0, 0, w3d, h3d);
 #endif
-  // clear all backbuffers to black
-  myG2D->BeginDraw ();
-  myG2D->ClearAll (myG2D->FindRGB(0,0,0));
-  myG2D->FinishDraw ();
 
 #if 0
 {
@@ -1632,7 +1463,7 @@ static void CreateSystem(void)
  *---------------------------------------------------------------------*/
 int main (int argc, char* argv[])
 {
-#if defined(OS_WIN32) && defined(COMP_VC)
+#if defined(CS_PLATFORM_WIN32) && defined(CS_COMPILER_MSVC)
   cswinMinidumpWriter::EnableCrashMinidumps ();
 #endif
   // Initialize the random number generator
