@@ -33,6 +33,7 @@
 #include "ivaria/conin.h"
 #include "iutil/vfs.h"
 #include "iutil/plugin.h"
+#include "iutil/eventnames.h"
 #include "ivideo/fontserv.h"
 #include "bot.h"
 
@@ -55,8 +56,6 @@ struct iMeshWrapper;
 struct iLight;
 struct iView;
 struct iSoundRender;
-struct iModelConverter;
-struct iCrossBuilder;
 struct iKeyboardDriver;
 struct iVirtualClock;
 struct iGraphics3D;
@@ -68,16 +67,19 @@ struct iGraphics2D;
 #define MAP_ON 2
 #define MAP_TXT 3
 
-SCF_VERSION (WalkDataObject, 0, 0, 1);
-class WalkDataObject : public csObject
+class WalkDataObject : public scfImplementationExt1<WalkDataObject,
+						    csObject, 
+						    scfFakeInterface<WalkDataObject> >
 {
 protected:
   /// Pointer to data.
   void* data;
 
 public:
+  SCF_INTERFACE(WalkDataObject, 2, 0, 0);
+
   /// Initialize this object with data pointer initialized to 'd'
-  WalkDataObject (void *d) : csObject (), data (d)
+  WalkDataObject (void *d) : scfImplementationType (this), data (d)
   {
   }
   /// Destroy object.
@@ -96,8 +98,6 @@ public:
     void *res = (d ? d->GetData () : 0);
     return res;
   }
-
-  SCF_DECLARE_IBASE_EXT (csObject);
 };
 
 ///
@@ -107,7 +107,7 @@ struct csKeyMap
   utf32_char key;
   bool shift, alt, ctrl;
   char* cmd;
-  int need_status,is_on;
+  bool need_status,is_on;
 };
 
 /**
@@ -158,6 +158,14 @@ public:
   csRef<iKeyboardDriver> kbd;
   csRef<iVirtualClock> vc;
 
+  csRef<iEventNameRegistry> name_reg;
+  csEventID Process;
+  csEventID FinalProcess;
+  csEventID CommandLineHelp;
+  csEventID CanvasHidden;
+  csEventID CanvasExposed;
+  csEventID CanvasResize;
+
   csRefArray<iLight> dynamic_lights;
 
   int FrameWidth, FrameHeight;
@@ -171,15 +179,14 @@ public:
 
   /// Player position, orientation, and velocity
   csVector3 pos;
+  csVector3 desired_velocity;
   csVector3 velocity;
 
-  /// Camera angles. X and Y are user controllable, Z is not.
-  //csVector3 angle;
-
   /**
    * Angular velocity: angle_velocity.x is constantly added to angle.x
    * and so on.
    */
+  csVector3 desired_angle_velocity;
   csVector3 angle_velocity;
 
   /// Colliders for "legs" and "body". Intersections are handled differently.
@@ -187,6 +194,8 @@ public:
   csRef<iCollider> legs;
   csVector3 body_radius, body_center, legs_radius, legs_center;
 
+  csColliderActor collider_actor;
+
   /// A list with all busy entities.
   csArray<csWalkEntity*> busy_entities;
   /// A vector that is used to temporarily store references to busy entities.
@@ -213,6 +222,10 @@ public:
   float cfg_walk_accelerate;
   /// Walk maximum speed.
   float cfg_walk_maxspeed;
+  /// Multiplier for maximum speed.
+  float cfg_walk_maxspeed_mult;
+  /// Is multiplier used?
+  float cfg_walk_maxspeed_multreal;
   /// Walk brake deceleration.
   float cfg_walk_brake;
   /// Rotate acceleration.
@@ -276,7 +289,7 @@ public:
   bool do_fs_fadetxt;
   float fs_fadetxt_fade;
   bool fs_fadetxt_dir;
-  iMaterialHandle* fs_fadetxt_mat;
+  iTextureHandle* fs_fadetxt_txt;
 
   bool do_fs_red;
   float fs_red_fade;
@@ -298,14 +311,10 @@ public:
 
   /**
    * The main engine interface
-   * (when interface will be complete, csEngine will not be needed anymore)
    */
   csRef<iEngine> Engine;
   /// The level loader
   csRef<iLoader> LevelLoader;
-  /// The model importer and crossbuilder
-  csRef<iModelConverter> ModelConverter;
-  csRef<iCrossBuilder> CrossBuilder;
   ///
   csRef<iGraphics2D> myG2D;
   csRef<iGraphics3D> myG3D;
@@ -343,11 +352,10 @@ public:
   bool do_show_palette;
   bool do_infinite;
   bool do_huge;
-  bool do_cd;
   bool do_freelook;
-  bool do_gravity;
   bool do_light_frust;
   bool do_logo;
+  bool doSave;
   int cfg_debug_check_frustum;
   int fgcolor_stats;
 
@@ -501,14 +509,18 @@ public:
   /// Creates Colliders
   virtual void CreateColliders();
 
-  /// Gravity correct movement function: strafe().
-  void strafe(float speed,int keep_old);
-  /// Gravity correct movement function: step().
-  void step(float speed,int keep_old);
-  /// Gravity correct movement function: rotate().
-  void rotate(float speed,int keep_old);
-  /// Gravity correct movement function: look().
-  void look(float speed,int keep_old);
+  /// Gravity correct movement function.
+  void Strafe (float speed);
+  /// Gravity correct movement function.
+  void Step (float speed);
+  /// Gravity correct movement function.
+  void Rotate (float speed);
+  /// Gravity correct movement function.
+  void Look (float speed);
+  /// Jump
+  void Jump ();
+  /// Do the actual acceleration calculation.
+  void InterpolateMovement ();
 
   /// Immediate gravity incorrect movement functions.
   void imm_forward (float speed, bool slow, bool fast);
@@ -540,10 +552,17 @@ public:
 
   // Bot stuff
   csPDelArray<Bot> bots;
+  csPDelArray<Bot> manual_bots;
   void add_bot (float size, iSector* where, csVector3 const& pos,
-    float dyn_radius);
-  void del_bot ();
+    float dyn_radius, bool manual = false);
+  void del_bot (bool manual = true);
   void move_bots (csTicks);
+
+  //@{
+  /// Save/load camera functions
+  void SaveCamera (const char *fName);
+  bool LoadCamera (const char *fName);
+  //@}
 };
 
 extern csVector2 coord_check_vector;
@@ -555,9 +574,6 @@ extern void perf_test (int num);
 extern void CaptureScreen ();
 extern void free_keymap ();
 
-extern void SaveCamera (iVFS*, const char *fName);
-extern bool LoadCamera (iVFS*, const char *fName);
-
 // Use a view's clipping rect to calculate a bounding box
 void BoundingBoxForView(iView *view, csBox2 *box);
 
