@@ -27,10 +27,6 @@
 
 extern WalkTest* Sys;
 
-SCF_IMPLEMENT_IBASE_EXT (csWalkEntity)
-  SCF_IMPLEMENTS_INTERFACE (csWalkEntity)
-SCF_IMPLEMENT_IBASE_EXT_END
-
 //--------------------------------------------------------------------------
 
 struct AnimPortalCallback : public iPortalCallback
@@ -82,8 +78,8 @@ csAnimatedPortal::csAnimatedPortal (iPortal* p,
 void csAnimatedPortal::Activate ()
 {
   // Push ourselves on to the busy list if we're not already there.
-  int idx = Sys->busy_entities.Find (this);
-  if (idx != -1) Sys->busy_entities.DeleteIndex (idx);
+  size_t idx = Sys->busy_entities.Find (this);
+  if (idx != csArrayItemNotFound) Sys->busy_entities.DeleteIndex (idx);
   Sys->busy_entities.Push (this);
 }
 
