@@ -17,11 +17,35 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __FASTSQRT_H__
-#define __FASTSQRT_H__
+#ifndef __CS_FASTSQRT_H__
+#define __CS_FASTSQRT_H__
 
-extern void BuildSqrtTable ();
-extern float FastSqrt (float n);
+/**\file 
+ */
+/**
+ * \addtogroup geom_utils
+ * @{ */
 
-#endif // __FASTSQRT_H__
+#include "csextern.h"
+
+
+#ifdef CS_COMPILER_GCC
+  #warning FastSqrt is deprecated. Use sqrtf instead.
+#endif
+#ifdef CS_COMPILER_MSVC
+  #pragma message ("FastSqrt is deprecated. Use sqrtf instead.")
+#endif
+
+/// declare table of square roots
+CS_CRYSTALSPACE_EXPORT void BuildSqrtTable ()
+{}
+/// fast square root, looks up table build by BuildSqrtTable().
+CS_CRYSTALSPACE_EXPORT float FastSqrt (float n)
+{
+  return sqrtf(n);
+}
+
+/** @} */
+
+#endif // __CS_FASTSQRT_H__
 
