@@ -21,24 +21,18 @@
 #define __CS_MATH2D_H__
 
 /**\file 
+ * 2D mathematic utility functions.
  */
 /**
  * \addtogroup geom_utils
  * @{ */
 
-// These are also defined in plane2.h
-#ifndef __CS_POLY_MACROS__
-#define __CS_POLY_MACROS__
-#define CS_POLY_IN 1
-#define CS_POLY_ON 0
-#define CS_POLY_OUT -1
-#endif
 
 #include "csextern.h"
 
-#include "csgeom/vector2.h"
 #include "csgeom/plane2.h"
 #include "csgeom/segment.h"
+#include "csgeom/vector2.h"
 
 class csBox2;
 class csPoly2D;
@@ -47,12 +41,12 @@ class csPoly2D;
  * Various functions in 2D, such as 2D vector functions.
  * This is a static class and contains only static member functions.
  */
-class CS_CSGEOM_EXPORT csMath2
+class CS_CRYSTALSPACE_EXPORT csMath2
 {
 public:
   /**
    * Calculates which side of a line a given point is on.
-   * Returns -1 if point v is left of line segment 's1-s2',
+   * \return -1 if point v is left of line segment 's1-s2',
    *          1 if point v is right of segment 's1-s2'
    *       or 0 if point v lies on segment 's1-s2'.
    */
@@ -68,7 +62,7 @@ public:
 
   /**
    * Calculates which side of a line a given point is on.
-   * Returns -1 if point v is left of line segment 'seg'
+   * \return -1 if point v is left of line segment 'seg'
    *          1 if point v is right of segment 'seg'
    *       or 0 if point v lies on segment 'seg'.
    */
@@ -80,7 +74,7 @@ public:
 
   /**
    * Calculates whether a vector lies inside a given 2D polygon.
-   * Return CS_POLY_IN, CS_POLY_OUT, or CS_POLY_ON for this vector with
+   * \return CS_POLY_IN, CS_POLY_OUT, or CS_POLY_ON for this vector with
    * respect to the given polygon. The polygon is given as an array of 2D
    * vectors with a bounding box.
    * WARNING: does no safety checking for P or bounding_box.
@@ -90,7 +84,7 @@ public:
 
   /**
    * Calculates 2 x the area of a given triangle.
-   * Returns twice the signed area of the triangle determined by a,b,c,
+   * \return twice the signed area of the triangle determined by a,b,c,
    * positive if a,b,c are oriented ccw, and negative if cw.
    */
   static float Area2 (const csVector2& a,
@@ -105,7 +99,7 @@ public:
 
   /**
    * Calculates whether a point lies to the right of a given line.
-   * Returns true iff c is strictly to the right of the directed
+   * \return true iff c is strictly to the right of the directed
    * line through a to b.
    */
   static float Right (const csVector2& a,
@@ -137,7 +131,7 @@ public:
 
   /**
    * Check if two planes are almost equal.
-   * The function returns true iff each component of the plane equation for
+   * \return true iff each component of the plane equation for
    * one plane is within .001 of the corresponding component of the other
    * plane.
    */
@@ -160,29 +154,31 @@ public:
  * line segments.  This is a static class and contains only static member
  * functions.
  */
-class CS_CSGEOM_EXPORT csIntersect2
+class CS_CRYSTALSPACE_EXPORT csIntersect2
 {
 public:
   /**
    * Intersect a plane with a 2D polygon and return
    * the line segment corresponding with this intersection.
-   * Returns true if there is an intersection. If false
+   * \return true if there is an intersection. If false
    * then 'segment' will not be valid.
    */
-  static bool IntersectPolygon (const csPlane2& plane, csPoly2D* poly,
+  static bool PlanePolygon (const csPlane2& plane, csPoly2D* poly,
   	csSegment2& segment);
 
   /**
-   * Compute the intersection of the 2D segments.  Return true if they
+   * Compute the intersection of the 2D segments.
+   * \return true if they
    * intersect, with the intersection point returned in isect,  and the
    * distance from a1 of the intersection in dist.
    */
-  static bool Segments (
+  static bool SegmentSegment (
     const csSegment2& a, const csSegment2& b,	// Two segments.
     csVector2& isect, float& dist);         // intersection point and distance
 
   /**
-   * Compute the intersection of a 2D segment and a line.  Return true if they
+   * Compute the intersection of a 2D segment and a line.
+   * \return true if they
    * intersect, with the intersection point returned in isect,  and the
    * distance from a1 of the intersection in dist.
    */
@@ -192,50 +188,53 @@ public:
     csVector2& isect, float& dist);     // intersection point and distance
 
   /**
-   * Compute the intersection of 2D lines.  Return true if they
+   * Compute the intersection of 2D lines.
+   * \return true if they
    * intersect, with the intersection point returned in isect.
    */
-  static bool Lines (
+  static bool LineLine (
     // Two lines (end is only direction).
     const csSegment2& a, const csSegment2& b,
     csVector2& isect);                      // intersection point
 
   /**
-   * Intersect a 2D segment with a plane.  Returns true if there is an
+   * Intersect a 2D segment with a plane.
+   * \return true if there is an
    * intersection, with the intersection point returned in isect.
    * The distance from u to the intersection point is returned in dist.
    * The distance that is returned is a normalized distance with respect
    * to the given input vector. i.e. a distance of 0.5 means that the
    * intersection point is halfway u and v.
    */
-  static bool Plane (
+  static bool SegmentPlane (
     const csVector2& u, const csVector2& v,
     const csPlane2& p,                     // plane Ax+By+Cz+D=0
     csVector2& isect,                     // intersection point
     float& dist);                       // distance from u to isect
 
   /**
-   * Intersect a 2D segment with a plane.  Returns true if there is an
+   * Intersect a 2D segment with a plane.
+   * \return true if there is an
    * intersection, with the intersection point returned in isect.
    * The distance from u to the intersection point is returned in dist.
    * The distance that is returned is a normalized distance with respect
    * to the given input vector. i.e. a distance of 0.5 means that the
    * intersection point is halfway u and v.
    */
-  static bool Plane (
+  static bool SegmentPlane (
     const csSegment2& uv,	// Segment.
     const csPlane2& p,                     // plane Ax+By+Cz+D=0
     csVector2& isect,                     // intersection point
     float& dist)                        // distance from u to isect
   {
-    return Plane (uv.Start (), uv.End (), p, isect, dist);
+    return SegmentPlane (uv.Start (), uv.End (), p, isect, dist);
   }
 
   /**
    * Return the intersection point. This version does not test if
    * there really is an intersection. It just assumes there is one.
    */
-  static void PlaneNoTest (const csVector2& u, const csVector2& v,
+  static void SegmentPlaneNoTest (const csVector2& u, const csVector2& v,
                      const csPlane2& p, csVector2& isect, float& dist)
   {
     float x,y, denom;
@@ -249,18 +248,19 @@ public:
    * Return the intersection point. This version does not test if
    * there really is an intersection. It just assumes there is one.
    */
-  static void PlaneNoTest (const csSegment2& uv,
+  static void SegmentPlaneNoTest (const csSegment2& uv,
                      const csPlane2& p, csVector2& isect, float& dist)
   {
-    PlaneNoTest (uv.Start (), uv.End (), p, isect, dist);
+    SegmentPlaneNoTest (uv.Start (), uv.End (), p, isect, dist);
   }
 
   /**
    * Intersect 2 planes to get the point that is part of all two
-   * planes. Returns true, if there is a single point that fits.
+   * planes.
+   * \return true if there is a single point that fits.
    * If the planes are parallel, then it will return false.
    */
-  static bool Planes (const csPlane2& p1, const csPlane2& p2,
+  static bool PlanePlane (const csPlane2& p1, const csPlane2& p2,
                       csVector2& isect);
 
 };
