@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1998-2001 by Jorrit Tyberghein
+    Copyright (C) 1998-2005 by Jorrit Tyberghein
     Largely rewritten by Ivan Avramovic <ivan@avramovic.com>
 
     This library is free software; you can redistribute it and/or
@@ -20,34 +20,39 @@
 #ifndef __CS_MATH3D_H__
 #define __CS_MATH3D_H__
 
-#ifndef __CS_CSSYSDEFS_H__
-#error "cssysdef.h must be included in EVERY source file!"
-#endif
+/**\file 
+ * 3D mathematic utility functions.
+ */
+/**
+ * \addtogroup geom_utils
+ * @{ */
 
-#include "csgeom/vector3.h"
+#include "csextern.h"
+
+#include "csgeom/box.h"
+#include "csgeom/frustum.h"
 #include "csgeom/plane3.h"
-#include "csgeom/plane2.h"
 #include "csgeom/segment.h"
+#include "csgeom/vector3.h"
+#include "csutil/ref.h"
+#include "csutil/scf_implementation.h"
 
-class csDVector3;
+#include "iutil/dbghelp.h"
+#include "iutil/string.h"
+
+class csPlane2;
 class csPoly3D;
-class csBox3;
-
-inline float fSqr (float f)
-{
-  return f * f;
-}
 
 /**
  * Various assorted 3D mathematical functions.
  * This is a static class and contains only static member functions.
  */
-class csMath3
+class CS_CRYSTALSPACE_EXPORT csMath3
 {
 public:
   /**
    * Tests which side of a plane the given 3D point is on.
-   * Return -1 if point p is left of plane '0-v1-v2',
+   * \return -1 if point p is left of plane '0-v1-v2',
    *         1 if point p is right of plane '0-v1-v2',
    *      or 0 if point p lies on plane '0-v1-v2'.
    * Plane '0-v1-v2' is the plane passing through points <0,0,0>, v1, and v2.
@@ -55,7 +60,7 @@ public:
    * Warning: the result of this function when 'p' is exactly on the plane
    * 0-v1-v2 is undefined. It should return 0 but it will not often do that
    * due to numerical inaccuracies. So you should probably test for this
-   * case seperatelly.
+   * case separately.
    */
   static int WhichSide3D (const csVector3& p,
                           const csVector3& v1, const csVector3& v2)
@@ -84,17 +89,6 @@ public:
   static bool Visible (const csVector3& p, const csPlane3& pl)
   { return pl.Classify (p) <= 0; }
 
-  /**
-   * Calculate the line, that is the result of the Intersection of
-   * triangle 1  and triangle 2. This Method returns false, if there is
-   * no intersection. If there is an intersection, the start of the line
-   * is in line[0] and the end of the line is in line[1] and the method
-   * return true;
-   */
-  static bool FindIntersection(const csVector3  tri1[3],
-                               const csVector3  tri2[3],
-                               csVector3        line[2]);
-
   /**
    * Calculates a vector lying a specified distance between two other vectors.
    * Given vectors v1 and v2, this function will calculate and return vector
@@ -122,11 +116,22 @@ public:
   }
 
   /**
-   * Compute twice the signed area of triangle composed by three points.
+   * Compute twice the area of triangle composed by three points.
    * This function returns 2 x the area of the triangle formed by the points
    * a, b, and c.
    */
-  inline static float Area3 (const csVector3 &a, const csVector3 &b,
+  inline static float DoubleArea3 (const csVector3 &a, const csVector3 &b,
+                             const csVector3 &c)
+  {
+    csVector3 v1 = b - a;
+    csVector3 v2 = c - a;
+    return (v1 % v2).Norm ();
+  }
+
+  /**
+   * Returns < 0 or > 0 depending on the direction of the triangle.
+   */
+  inline static float Direction3 (const csVector3 &a, const csVector3 &b,
                              const csVector3 &c)
   {
     csVector3 v1 = b - a;
@@ -220,12 +225,15 @@ public:
  * Some functions to perform squared distance calculations.
  * This is a static class and contains only static member functions.
  */
-class csSquaredDist
+class CS_CRYSTALSPACE_EXPORT csSquaredDist
 {
 public:
   /// Returns the squared distance between two points.
   static float PointPoint (const csVector3& p1, const csVector3& p2)
-  { return fSqr (p1.x - p2.x) + fSqr (p1.y - p2.y) + fSqr (p1.z - p2.z); }
+  { 
+    csVector3 d = (p1-p2);
+    return d*d;
+  }
 
   /// Returns the squared distance between a point and a line.
   static float PointLine (const csVector3& p,
@@ -250,8 +258,12 @@ public:
  * line segments.  This is a static class and contains only static member
  * functions.
  */
-class csIntersect3
+class CS_CRYSTALSPACE_EXPORT csIntersect3
 {
+private:
+  static bool BoxPlaneInternal (const csVector3& normal,
+	const csVector3& vert, const csVector3& boxhalfsize);
+
 public:
   /**
    * Intersect a plane with a 3D polygon and return
@@ -259,7 +271,7 @@ public:
    * Returns true if there is an intersection. If false
    * then 'segment' will not be valid.
    */
-  static bool IntersectPolygon (const csPlane3& plane, csPoly3D* poly,
+  static bool PlanePolygon (const csPlane3& plane, csPoly3D* poly,
   	csSegment3& segment);
 
   /**
@@ -271,23 +283,49 @@ public:
    * @@@ WARNING! This function may not work completely ok. It has only
    * barely been tested and is now unused.
    */
-  static int IntersectSegment (csPlane3* planes, int num_planes,
+  static int SegmentFrustum (csPlane3* planes, int num_planes,
   	csSegment3& seg);
 
   /**
-   * Intersect a 3D segment with a triangle. Returns true if there
+   * Intersect a 3D segment with a triangle.
+   * \return true if there
    * is an intersection. In that case the intersection point will
    * be in 'isect'.
    */
-  static bool IntersectTriangle (const csVector3& tr1,
+  static bool SegmentTriangle (const csSegment3& seg,
+  	const csVector3& tr1,
   	const csVector3& tr2, const csVector3& tr3,
-	const csSegment3& seg, csVector3& isect);
+	csVector3& isect);
 
   /**
-   * Intersect a 3D segment with a plane.  Returns true if there is an
+   * Intersect a 3D segment with a polygon.
+   * \return true if there
+   * is an intersection. In that case the intersection point will
+   * be in 'isect'. Note that this function doesn't do
+   * backface culling.
+   */
+  static bool SegmentPolygon (const csSegment3& seg, const csPoly3D& poly,
+  	const csPlane3& poly_plane, csVector3& isect);
+
+  /**
+   * If a number of planes enclose a convex space (with their normals
+   * pointing outwards).
+   * \return true if they are intersected by a segment.
+   * isect contains the closest intersection point.
+   * dist contains the distance to that point (with distance between u and v
+   * being 1)
+   */
+  static bool SegmentPlanes (
+     const csVector3& u, const csVector3& v,
+     const csPlane3* planes, int length,
+     csVector3& isect, float& dist);
+
+  /**
+   * Intersect a 3D segment with a plane.
+   * \return true if there is an
    * intersection, with the intersection point returned in isect.
    */
-  static bool Plane (
+  static bool SegmentPlane (
     const csVector3& u, const csVector3& v,
     const csVector3& normal, const csVector3& a, // plane
     csVector3& isect, float& dist);              // intersection point
@@ -299,8 +337,22 @@ public:
    * The distance that is returned is a normalized distance with respect
    * to the given input vector. i.e. a distance of 0.5 means that the
    * intersection point is halfway u and v.
+   * There are two cases in which this method will return false:
+   * - If the plane and the segment are parallel, then 'dist' will be set
+   *   equal to 0, and 'isect' equal to 'v'.
+   * - If the segment does not cross the plane (i.e. if 'dist'>1+epsilon or
+   *   'dist'<-epsilon, where epsilon is a very small value near to zero)
+   *   then 'isect's value is (0, 0, 0).
+   *
+   * \remarks
+   * 'p' is the plane, expressed as: A x + B y + C z + D = 0 , where (A,B,C) is 
+   * the normal vector of the plane.
+   * 'u' and 'v' are the start (U point) and the end (V point) of the segment.
+   * 'isect' is searched along the segment U + x (V - U); the unknown 'x' value
+   * is got by: x = [(A,B,C) * U + D ] / (A,B,C) * (U - V), where * is the dot 
+   * product.  
    */
-  static bool Plane (
+  static bool SegmentPlane (
     const csVector3& u, const csVector3& v,
     const csPlane3& p,                     // plane Ax+By+Cz+D=0
     csVector3& isect,                     // intersection point
@@ -308,17 +360,18 @@ public:
 
   /**
    * Intersect 3 planes to get the point that is part of all three
-   * planes. Returns true, if there is a single point that fits.
+   * planes.
+   * \return true if there is a single point that fits.
    * If some planes are parallel, then it will return false.
    */
-  static bool Planes (const csPlane3& p1, const csPlane3& p2,
+  static bool ThreePlanes (const csPlane3& p1, const csPlane3& p2,
   	const csPlane3& p3, csVector3& isect);
 
   /**
    * Intersect a regular plane and an axis aligned plane and
    * return the intersection (line) as a 2D plane. This intersection
    * is defined on the axis aligned plane.
-   * Returns false if there is no intersection.
+   * \return false if there is no intersection.
    */
   static bool PlaneXPlane (const csPlane3& p1, float x2, csPlane2& isect);
 
@@ -326,7 +379,7 @@ public:
    * Intersect a regular plane and an axis aligned plane and
    * return the intersection (line) as a 2D plane. This intersection
    * is defined on the axis aligned plane.
-   * Returns false if there is no intersection.
+   * \return false if there is no intersection.
    */
   static bool PlaneYPlane (const csPlane3& p1, float y2, csPlane2& isect);
 
@@ -334,7 +387,7 @@ public:
    * Intersect a regular plane and an axis aligned plane and
    * return the intersection (line) as a 2D plane. This intersection
    * is defined on the axis aligned plane.
-   * Returns false if there is no intersection.
+   * \return false if there is no intersection.
    */
   static bool PlaneZPlane (const csPlane3& p1, float z2, csPlane2& isect);
 
@@ -342,7 +395,7 @@ public:
    * Intersect a regular plane and an axis aligned plane and
    * return the intersection (line) as a 2D plane. This intersection
    * is defined on the axis aligned plane.
-   * Returns false if there is no intersection.
+   * \return false if there is no intersection.
    */
   static bool PlaneAxisPlane (const csPlane3& p1, int nr, float pos,
   	csPlane2& isect)
@@ -362,7 +415,7 @@ public:
    * and returns the distance from u to the intersection point.
    * The intersection point is returned in isect.
    */
-  static float Z0Plane (
+  static float SegmentZ0Plane (
     const csVector3& v1, const csVector3& v2,
     csVector3& isect);                    // intersection point
 
@@ -372,11 +425,61 @@ public:
    * and returns the distance from u to the intersection point.
    * The intersection point is returned in isect.
    */
-  static float Z0Plane (
+  static float SegmentZ0Plane (
     const csSegment3& uv,
     csVector3& isect)                    // intersection point
   {
-    return Z0Plane (uv.Start (), uv.End (), isect);
+    return SegmentZ0Plane (uv.Start (), uv.End (), isect);
+  }
+
+  /**
+   * Intersect a 3D segment with the plane x = xval. Assumes that there
+   * is an intersection (fails if the segment is parallel to the plane),
+   * and returns the distance from u to the intersection point.
+   * The intersection point is returned in isect.
+   */
+  static float SegmentXPlane (
+    const csVector3& u, const csVector3& v,
+    float xval,
+    csVector3& isect);                    // intersection point
+
+  /**
+   * Intersect a 3D segment with the plane x = xval. Assumes that there
+   * is an intersection (fails if the segment is parallel to the plane),
+   * and returns the distance from u to the intersection point.
+   * The intersection point is returned in isect.
+   */
+  static float SegmentXPlane (
+    const csSegment3& uv,
+    float xval,
+    csVector3& isect)                     // intersection point
+  {
+    return SegmentXPlane (uv.Start (), uv.End (), xval, isect);
+  }
+
+  /**
+   * Intersect a 3D segment with the plane y = yval. Assumes that there
+   * is an intersection (fails if the segment is parallel to the plane),
+   * and returns the distance from u to the intersection point.
+   * The intersection point is returned in isect.
+   */
+  static float SegmentYPlane (
+    const csVector3& u, const csVector3& v,
+    float yval,      // plane y = yval
+    csVector3& isect);                    // intersection point
+
+  /**
+   * Intersect a 3D segment with the plane y = yval. Assumes that there
+   * is an intersection (fails if the segment is parallel to the plane),
+   * and returns the distance from u to the intersection point.
+   * The intersection point is returned in isect.
+   */
+  static float SegmentYPlane (
+    const csSegment3& uv,
+    float yval,      // plane y = yval
+    csVector3& isect)                     // intersection point
+  {
+    return SegmentYPlane (uv.Start (), uv.End (), yval, isect);
   }
 
   /**
@@ -385,8 +488,9 @@ public:
    * and returns the distance from u to the intersection point.
    * The intersection point is returned in isect.
    */
-  static float ZPlane (float zval,      // plane z = zval
+  static float SegmentZPlane (
     const csVector3& u, const csVector3& v,
+    float zval,      // plane z = zval
     csVector3& isect);                    // intersection point
 
   /**
@@ -395,57 +499,210 @@ public:
    * and returns the distance from u to the intersection point.
    * The intersection point is returned in isect.
    */
-  static float ZPlane (float zval,      // plane z = zval
+  static float SegmentZPlane (
     const csSegment3& uv,
+    float zval,      // plane z = zval
     csVector3& isect)                     // intersection point
   {
-    return ZPlane (zval, uv.Start (), uv.End (), isect);
+    return SegmentZPlane (uv.Start (), uv.End (), zval, isect);
+  }
+
+  /**
+   * Intersect a 3D segment with an axis aligned plane and
+   * return the intersection (fails if segment is parallel to the plane),
+   * and returns the distance from u to the intersection point.
+   * The intersection point is returned in isect.
+   */
+  static float SegmentAxisPlane (const csVector3& u, const csVector3& v,
+  	int nr, float pos, csVector3& isect)
+  {
+    switch (nr)
+    {
+      case 0: return SegmentXPlane (u, v, pos, isect);
+      case 1: return SegmentYPlane (u, v, pos, isect);
+      case 2: return SegmentZPlane (u, v, pos, isect);
+    }
+    return 0.0;
   }
 
   /**
    * Intersect a 3D segment with the frustum plane Ax + z = 0.
    * Assumes an intersection, and returns the intersection point in isect.
    */
-  static float XFrustum (
-    float A, const csVector3& u, const csVector3& v, csVector3& isect);
+  static float SegmentXFrustum (
+    const csVector3& u, const csVector3& v, float A, csVector3& isect);
 
   /**
    * Intersect a 3D segment with the frustum plane Ax + z = 0.
    * Assumes an intersection, and returns the intersection point in isect.
    */
-  static float XFrustum (
-    float A, const csSegment3& uv, csVector3& isect)
+  static float SegmentXFrustum (
+    const csSegment3& uv, float A, csVector3& isect)
   {
-    return XFrustum (A, uv.Start (), uv.End (), isect);
+    return SegmentXFrustum (uv.Start (), uv.End (), A, isect);
   }
 
   /**
    * Intersect a 3D segment with the frustum plane By + z = 0.
    * Assumes an intersection, and returns the intersection point in isect.
    */
-  static float YFrustum (
-    float B, const csVector3& u, const csVector3& v, csVector3& isect);
+  static float SegmentYFrustum (
+    const csVector3& u, const csVector3& v, float B, csVector3& isect);
 
   /**
    * Intersect a 3D segment with the frustum plane By + z = 0.
    * Assumes an intersection, and returns the intersection point in isect.
    */
-  static float YFrustum (
-    float B, const csSegment3& uv, csVector3& isect)
+  static float SegmentYFrustum (
+    const csSegment3& uv, float B, csVector3& isect)
   {
-    return YFrustum (B, uv.Start (), uv.End (), isect);
+    return SegmentYFrustum (uv.Start (), uv.End (), B, isect);
   }
 
   /**
-   * Intersect a segment with a box and returns true if it intersects.
-   * The intersection point is also returned.
-   * If 'pr' is given then a number between 0 and 1 is returned which
+   * Intersect a segment with a box.
+   * \param box The box with which to perform the intersection.
+   * \param segment The intersection candidate.
+   * \param isect The returned intersection point.
+   * \param pr If not null then a number between 0 and 1 is returned which
    * corresponds to the position on the segment. If we were in the box
-   * this this function will also return true. In this case 'isect' will
+   * this this function will return CS_BOX_INSIDE. In this case 'isect' will
    * be set to the start of the segment and *pr to 0.
+   * \return one of CS_BOX_SIDE_... if it intersects, CS_BOX_INSIDE if inside,
+   * or -1 otherwise.
    */
   static int BoxSegment (const csBox3& box, const csSegment3& segment,
-  	csVector3& isect, float* pr = NULL);
+  	csVector3& isect, float* pr = 0);
+
+  /**
+   * Intersect an AABB with a frustum. The frustum may contain up to
+   * 32 planes. Active planes are defined using the 'inClipMask'. It will
+   * return true if AABB is visible in frustum. If the AABB intersects
+   * with the frustum then 'outClipMask' will contain the mask for all planes
+   * intersecting with the AABB. This can be used as 'inClipMask' for subsequent
+   * frustum tests with children of the AABB (i.e. other AABB inside
+   * this AABB).
+   */
+  static bool BoxFrustum (const csBox3& box, csPlane3* frustum,
+  	uint32 inClipMask, uint32& outClipMask);
+
+  /**
+   * Intersect an AABB with a frustum.
+   */
+  static bool BoxFrustum (const csBox3& box, const csFrustum* frustum);
+
+  /**
+   * Test if a box intersects with a sphere. The intersection is not
+   * computed. The sphere is given with squared radius.
+   */
+  static bool BoxSphere (const csBox3& box, const csVector3& center,
+		  float sqradius);
+
+  /**
+   * Test if a plane intersects with a box.
+   */
+  static bool BoxPlane (const csBox3& box, const csPlane3& plane);
+
+  /**
+   * Test if a plane intersects with a box.
+   * 'vert' is one point on the plane.
+   */
+  static bool BoxPlane (const csBox3& box, const csVector3& normal,
+  	const csVector3& vert);
+
+  /**
+   * Test if a triangle intersects with a box.
+   */
+  static bool BoxTriangle (const csBox3& box,
+	const csVector3& tri0, const csVector3& tri1, const csVector3& tri2);
+
+  /**
+   * Test if two boxes intersect.
+   */
+  static bool BoxBox (const csBox3& box1, const csBox3& box2)
+  {
+    return box1.TestIntersect (box2);
+  }
+
+  /**
+   * Calculate intersection of two frustums.
+   */
+  static csPtr<csFrustum> FrustumFrustum (const csFrustum& f1,
+  	const csFrustum& f2)
+  {
+    return f1.Intersect (f2);
+  }
+
+  /**
+   * Calculate intersection of two frustums.
+   */
+  static csPtr<csFrustum> FrustumFrustum (const csFrustum& f1,
+  	csVector3* poly, int num)
+  {
+    return f1.Intersect (poly, num);
+  }
+
+  /**
+   * Test intersection between two triangles.
+   * \param tri1 Vertices of triangle 1
+   * \param tri2 Vertices of triangle 2
+   * \return true if the triangles intersect, otherwise false
+   */
+  static bool TriangleTriangle (const csVector3 tri1[3],
+			 const csVector3 tri2[3]);
+
+  /**
+   * Calculate intersection between two triangles and return it
+   * in isectline.
+   * \param tri1 Vertices of triangle 1
+   * \param tri2 Vertices of triangle 2
+   * \param[out] isectline The line segment where they intersect
+   * \param[out] coplanar Returns whether the triangles are coplanar
+   * \return true if the triangles intersect, otherwise false
+   */
+  static bool TriangleTriangle (const csVector3 tri1[3],
+			 const csVector3 tri2[3],
+			 csSegment3& isectline, bool& coplanar);
 };
 
+/**
+ * This is a class that does unit testing (and other debug stuff) for most
+ * of csgeom classes.
+ */
+class CS_CRYSTALSPACE_EXPORT csGeomDebugHelper :
+  public scfImplementation1<csGeomDebugHelper, iDebugHelper>
+{
+public:
+  csGeomDebugHelper ();
+  virtual ~csGeomDebugHelper ();
+
+  virtual int GetSupportedTests () const
+  {
+    return CS_DBGHELP_UNITTEST;
+  }
+  virtual csPtr<iString> UnitTest ();
+  virtual csPtr<iString> StateTest ()
+  {
+    return 0;
+  }
+  virtual csTicks Benchmark (int /*num_iterations*/)
+  {
+    return 0;
+  }
+  virtual csPtr<iString> Dump ()
+  {
+    return 0;
+  }
+  virtual void Dump (iGraphics3D* /*g3d*/)
+  {
+  }
+  virtual bool DebugCommand (const char*)
+  {
+    return false;
+  }
+};
+
+/** @} */
+
 #endif // __CS_MATH3D_H__
+
