@@ -21,25 +21,24 @@
 #define __CS_PLANE3_H__
 
 /**\file 
+ * 3D space plane.
  */
 /**
  * \addtogroup geom_utils
  * @{ */
 
-#ifndef __CS_CSSYSDEFS_H__
-#error "cssysdef.h must be included in EVERY source file!"
-#endif
-
 #include "csextern.h"
 
 #include "csgeom/vector3.h"
 
+class csString;
+
 /**
  * A plane in 3D space.
  * The plane is given by the equation AAx + BBy + CCz + DD = 0,
  * Where (AA,BB,CC) is given by the vector 'norm'.
  */
-class CS_CSGEOM_EXPORT csPlane3
+class CS_CRYSTALSPACE_EXPORT csPlane3
 {
 public:
   /// The normal vector (or the (A,B,C) components).
@@ -104,6 +103,9 @@ public:
   /// Return the D component of this plane.
   inline float& D () { return DD; }
 
+  /// Return the normal of this plane.
+  inline const csVector3& GetNormal () const { return norm; }
+
   /// Set the value of the four plane components.
   inline void Set (float a, float b, float c, float d)
   { norm.x = a; norm.y = b; norm.z = c; DD = d; }
@@ -125,11 +127,20 @@ public:
    * is expressed as (N,D) with N the A,B,C components of the plane then
    * this will initialize the plane to (v2%v3,0).
    */
-  void Set (const csVector3& v2, const csVector3& v3)
+  inline void Set (const csVector3& v2, const csVector3& v3)
   {
     norm = v2 % v3; DD = 0;
   }
 
+  /**
+   * Set one point ("origin") through which the plane goes.
+   * This is equal to setting DD = -N'*p where N' is the normal
+   */
+  inline void SetOrigin (const csVector3& p)
+  {
+    DD = -norm * p;
+  }
+
   /**
    * Classify the given vector with regards to this plane. If the plane
    * is expressed as (N,D) with N the A,B,C components of the plane then
@@ -165,17 +176,22 @@ public:
    * Reverses the direction of the plane while maintianing the plane itself.
    * This will basically reverse the result of Classify().
    */
-  void Invert () { norm = -norm;  DD = -DD; }
+  inline void Invert () { norm = -norm;  DD = -DD; }
 
   /**
    * Normalizes the plane equation so that 'norm' is a unit vector.
    */
-  void Normalize ()
+  inline void Normalize ()
   {
     float f = norm.Norm ();
     if (f) { norm /= f;  DD /= f; }
   }
 
+  /**
+   * Find a point on this plane.
+   */
+  csVector3 FindPoint () const;
+
   /**
    * Clip the polygon in pverts (having num_verts vertices) to this plane.
    * The vertices are expected in camera space.
@@ -187,6 +203,9 @@ public:
    * is set to true then the positive side will be used instead.
    */
   bool ClipPolygon (csVector3*& pverts, int& num_verts, bool reversed = false);
+
+  /// Return a textual representation of the plane in the form "aa,bb,cc,dd".
+  csString Description() const;
 };
 
 /** @} */
