@@ -20,9 +20,21 @@
 #ifndef __CS_TRANSFORM_H__
 #define __CS_TRANSFORM_H__
 
+/**\file
+ * Transformation from one coordinate system to another.
+ */
+/**
+ * \addtogroup geom_utils
+ * @{ */
+
+#include "csextern.h"
+
+
 #include "csgeom/matrix3.h"
-#include "csgeom/plane3.h"
-#include "csgeom/sphere.h"
+#include "csgeom/vector3.h"
+
+class csPlane3;
+class csSphere;
 
 class csReversibleTransform;
 
@@ -32,7 +44,7 @@ class csReversibleTransform;
  * and 'this'. The transform defines a transformation from 'other'
  * to 'this'.
  */
-class csTransform
+class CS_CRYSTALSPACE_EXPORT csTransform
 {
 protected:
   /// Transformation matrix from 'other' space to 'this' space.
@@ -41,6 +53,9 @@ protected:
   csVector3 v_o2t;
 
 public:
+  // Needed for GCC4. Otherwise emits a flood of "virtual functions but
+  // non-virtual destructor" warnings.
+  virtual ~csTransform() {}
   /**
    * Initialize with the identity transformation.
    */
@@ -56,6 +71,36 @@ public:
   csTransform (const csMatrix3& other2this, const csVector3& origin_pos) :
   	m_o2t (other2this), v_o2t (origin_pos) {}
 
+  /**
+   * Reset this transform to the identity transform.
+   */
+  inline void Identity ()
+  {
+    SetO2TTranslation (csVector3 (0));
+    SetO2T (csMatrix3 ());
+  }
+
+  /**
+   * Returns true if this transform is an identity transform.
+   * This tests all fields so don't call this before every operation.
+   */
+  inline bool IsIdentity () const
+  {
+    if (ABS (v_o2t.x) >= SMALL_EPSILON) return false;
+    if (ABS (v_o2t.y) >= SMALL_EPSILON) return false;
+    if (ABS (v_o2t.z) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m11-1) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m12) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m13) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m21) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m22-1) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m23) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m31) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m32) >= SMALL_EPSILON) return false;
+    if (ABS (m_o2t.m33-1) >= SMALL_EPSILON) return false;
+    return true;
+  }
+
   /**
    * Get 'other' to 'this' transformation matrix. This is the 3x3
    * matrix M from the transform equation T=M*(O-V).
@@ -63,14 +108,15 @@ public:
   inline const csMatrix3& GetO2T () const { return m_o2t; }
 
   /**
-   * Get 'world' to 'this' translation. This is the vector V
+   * Get 'other' to 'this' translation. This is the vector V
    * from the transform equation T=M*(O-V). This is equivalent
    * to calling GetOrigin().
    */
   inline const csVector3& GetO2TTranslation () const { return v_o2t; }
 
   /**
-   * Get origin of transformed coordinate system. This is equivalent
+   * Get origin of transformed coordinate system. In other words, the vector
+   * that gets 0,0,0 after transforming with Other2This(). This is equivalent
    * to calling GetO2TTranslation().
    */
   inline const csVector3& GetOrigin () const { return v_o2t; }
@@ -116,7 +162,7 @@ public:
    * Use the origin of 'other' space. This will calculate and return
    * M*v (so the translation or V of this transform is ignored).
    */
-  csVector3 Other2ThisRelative (const csVector3& v) const
+  inline csVector3 Other2ThisRelative (const csVector3& v) const
   { return m_o2t * v; }
 
   /**
@@ -153,86 +199,97 @@ public:
    * Apply a transformation to a 3D vector. This corresponds exactly
    * to calling t.Other2This (v).
    */
-  friend csVector3 operator* (const csVector3& v, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csVector3 operator* (const csVector3& v, 
+    const csTransform& t);
 
   /**
    * Apply a transformation to a 3D vector. This corresponds exactly
    * to calling t.Other2This (v).
    */
-  friend csVector3 operator* (const csTransform& t, const csVector3& v);
+  friend CS_CRYSTALSPACE_EXPORT csVector3 operator* (const csTransform& t, 
+    const csVector3& v);
 
   /**
    * Apply a transformation to a 3D vector. This corresponds exactly
    * to calling v = t.Other2This(v).
    */
-  friend csVector3& operator*= (csVector3& v, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csVector3& operator*= (csVector3& v, 
+    const csTransform& t);
 
   /**
    * Apply a transformation to a Plane. This corresponds exactly
    * to calling t.Other2This(p).
    */
-  friend csPlane3 operator* (const csPlane3& p, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csPlane3 operator* (const csPlane3& p, 
+    const csTransform& t);
 
   /**
    * Apply a transformation to a Plane. This corresponds exactly
    * to calling t.Other2This(p).
    */
-  friend csPlane3 operator* (const csTransform& t, const csPlane3& p);
+  friend CS_CRYSTALSPACE_EXPORT csPlane3 operator* (const csTransform& t, 
+    const csPlane3& p);
 
   /**
    * Apply a transformation to a Plane. This corresponds exactly
    * to calling p = t.Other2This(p).
    */
-  friend csPlane3& operator*= (csPlane3& p, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csPlane3& operator*= (csPlane3& p, 
+    const csTransform& t);
 
   /**
    * Apply a transformation to a sphere. This corresponds exactly
    * to calling t.Other2This(p).
    */
-  friend csSphere operator* (const csSphere& p, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csSphere operator* (const csSphere& p, 
+    const csTransform& t);
 
   /**
    * Apply a transformation to a sphere. This corresponds exactly
    * to calling t.Other2This(p).
    */
-  friend csSphere operator* (const csTransform& t, const csSphere& p);
+  friend CS_CRYSTALSPACE_EXPORT csSphere operator* (const csTransform& t, 
+    const csSphere& p);
 
   /**
    * Apply a transformation to a sphere. This corresponds exactly
    * to calling p = t.Other2This(p).
    */
-  friend csSphere& operator*= (csSphere& p, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csSphere& operator*= (csSphere& p, 
+    const csTransform& t);
 
   /**
    * Multiply a matrix with the transformation matrix. This will calculate
    * and return m*M.
    */
-  friend csMatrix3 operator* (const csMatrix3& m, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator* (const csMatrix3& m, 
+    const csTransform& t);
 
   /**
    * Multiply a matrix with the transformation matrix. This will calculate
    * and return M*m.
    */
-  friend csMatrix3 operator* (const csTransform& t, const csMatrix3& m);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator* (const csTransform& t, 
+    const csMatrix3& m);
 
   /**
    * Multiply a matrix with the transformation matrix.
    * This corresponds exactly to m*=M.
    */
-  friend csMatrix3& operator*= (csMatrix3& m, const csTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3& operator*= (csMatrix3& m, 
+    const csTransform& t);
 
   /**
    * Combine two transforms, rightmost first. Given the following
    * definitions:
-   * <ul>
-   * <li>'t1' expressed as T=t1.M*(O-t1.V)
-   * <li>'t2' expressed as T=t2.M*(O-t2.V)
-   * <li>t2.Minv is the inverse of t2.M
-   * </ul>
+   * - 't1' expressed as T=t1.M*(O-t1.V)
+   * - 't2' expressed as T=t2.M*(O-t2.V)
+   * - t2.Minv is the inverse of t2.M
+   *
    * Then this will return a new transform
    * T=(t1.M*t2.M)*(O-(t2.V+t2.Minv*t1.V)).
    */
-  friend csTransform operator* (const csTransform& t1,
+  friend CS_CRYSTALSPACE_EXPORT csTransform operator* (const csTransform& t1,
                               const csReversibleTransform& t2);
 
   /**
@@ -241,6 +298,27 @@ public:
    * across the plane pl.
    */
   static csTransform GetReflect (const csPlane3& pl);
+
+  /**
+   * Get the front vector in 'other' space. This is basically equivalent
+   * to doing: tr.This2OtherRelative (csVector3 (0, 0, 1)) but it is
+   * more efficient.
+   */
+  csVector3 GetFront () { return csVector3 (m_o2t.m31, m_o2t.m32, m_o2t.m33); }
+
+  /**
+   * Get the up vector in 'other' space. This is basically equivalent
+   * to doing: tr.This2OtherRelative (csVector3 (0, 1, 0)) but it is
+   * more efficient.
+   */
+  csVector3 GetUp () { return csVector3 (m_o2t.m21, m_o2t.m22, m_o2t.m23); }
+
+  /**
+   * Get the right vector in 'other' space. This is basically equivalent
+   * to doing: tr.This2OtherRelative (csVector3 (1, 0, 0)) but it is
+   * more efficient.
+   */
+  csVector3 GetRight () { return csVector3 (m_o2t.m11, m_o2t.m12, m_o2t.m13); }
 };
 
 /**
@@ -249,8 +327,12 @@ public:
  * This version is similar to csTransform (in fact, it is a sub-class)
  * but it is more efficient if you plan to do inverse transformations
  * often.
+ * \remarks Despite that the superclass csTransform transforms from 'other'
+ *   to 'this' space, commonly csReversibleTransform instances are named like
+ *   'this2other' - e.g. 'object2world' where 'this' space is object space and
+ *   'other' space is world space.
  */
-class csReversibleTransform : public csTransform
+class CS_CRYSTALSPACE_EXPORT csReversibleTransform : public csTransform
 {
 protected:
   /// Inverse transformation matrix ('this' to 'other' space).
@@ -328,7 +410,7 @@ public:
    * This is the basic inverse transform operation and it corresponds
    * with the calculation of V+Minv*v (with Minv the inverse of M).
    */
-  csVector3 This2Other (const csVector3& v) const
+  inline csVector3 This2Other (const csVector3& v) const
   { return v_o2t + m_t2o * v; }
 
   /**
@@ -417,42 +499,45 @@ public:
    * Reverse a transformation on a 3D vector. This corresponds exactly
    * to calling t.This2Other(v).
    */
-  friend csVector3 operator/ (const csVector3& v,
+  friend CS_CRYSTALSPACE_EXPORT csVector3 operator/ (const csVector3& v,
   	const csReversibleTransform& t);
 
   /**
    * Reverse a transformation on a 3D vector. This corresponds exactly
    * to calling v=t.This2Other(v).
    */
-  friend csVector3& operator/= (csVector3& v, const csReversibleTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csVector3& operator/= (csVector3& v, 
+    const csReversibleTransform& t);
 
   /**
    * Reverse a transformation on a Plane. This corresponds exactly
    * to calling t.This2Other(p).
    */
-  friend csPlane3 operator/ (const csPlane3& p, const csReversibleTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csPlane3 operator/ (const csPlane3& p, 
+    const csReversibleTransform& t);
 
   /**
    * Reverse a transformation on a Plane. This corresponds exactly to
    * calling p = t.This2Other(p).
    */
-  friend csPlane3& operator/= (csPlane3& p, const csReversibleTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csPlane3& operator/= (csPlane3& p, 
+    const csReversibleTransform& t);
 
   /**
    * Reverse a transformation on a sphere. This corresponds exactly to
    * calling t.This2Other(p).
    */
-  friend csSphere operator/ (const csSphere& p, const csReversibleTransform& t);
+  friend CS_CRYSTALSPACE_EXPORT csSphere operator/ (const csSphere& p, 
+    const csReversibleTransform& t);
 
   /**
    * Combine two transforms, rightmost first. Given the following
    * definitions:
-   * <ul>
-   * <li>'t1' expressed as T=t1.M*(O-t1.V)
-   * <li>'t2' expressed as T=t2.M*(O-t2.V)
-   * <li>t1.Minv is the inverse of t1.M
-   * <li>t2.Minv is the inverse of t2.M
-   * </ul>
+   * - 't1' expressed as T=t1.M*(O-t1.V)
+   * - 't2' expressed as T=t2.M*(O-t2.V)
+   * - t1.Minv is the inverse of t1.M
+   * - t2.Minv is the inverse of t2.M
+   *
    * Then this will calculate a new transformation in 't1' as follows:
    * T=(t1.M*t2.M)*(O-(t2.Minv*t1.V+t2.V)).
    */
@@ -469,12 +554,11 @@ public:
   /**
    * Combine two transforms, rightmost first. Given the following
    * definitions:
-   * <ul>
-   * <li>'t1' expressed as T=t1.M*(O-t1.V)
-   * <li>'t2' expressed as T=t2.M*(O-t2.V)
-   * <li>t1.Minv is the inverse of t1.M
-   * <li>t2.Minv is the inverse of t2.M
-   * </ul>
+   * - 't1' expressed as T=t1.M*(O-t1.V)
+   * - 't2' expressed as T=t2.M*(O-t2.V)
+   * - t1.Minv is the inverse of t1.M
+   * - t2.Minv is the inverse of t2.M
+   *
    * Then this will calculate a new transformation in 't1' as follows:
    * T=(t1.M*t2.M)*(O-(t2.Minv*t1.V+t2.V)).
    */
@@ -485,50 +569,49 @@ public:
                              t2.v_o2t + t2.m_t2o*t1.v_o2t);
   }
 
+#if !defined(SWIG) /* Otherwise Swig 1.3.22 thinks this is multiply declared */
   /**
    * Combine two transforms, rightmost first. Given the following
    * definitions:
-   * <ul>
-   * <li>'t1' expressed as T=t1.M*(O-t1.V)
-   * <li>'t2' expressed as T=t2.M*(O-t2.V)
-   * <li>t1.Minv is the inverse of t1.M
-   * <li>t2.Minv is the inverse of t2.M
-   * </ul>
+   * - 't1' expressed as T=t1.M*(O-t1.V)
+   * - 't2' expressed as T=t2.M*(O-t2.V)
+   * - t1.Minv is the inverse of t1.M
+   * - t2.Minv is the inverse of t2.M
+   *
    * Then this will calculate a new transformation in 't1' as follows:
    * T=(t1.M*t2.M)*(O-(t2.Minv*t1.V+t2.V)).
    */
-  friend csTransform operator* (const csTransform& t1,
+  friend CS_CRYSTALSPACE_EXPORT csTransform operator* (const csTransform& t1,
                               const csReversibleTransform& t2);
+#endif
 
   /**
    * Combine two transforms, reversing t2 then applying t1.
    * Given the following definitions:
-   * <ul>
-   * <li>'t1' expressed as T=t1.M*(O-t1.V)
-   * <li>'t2' expressed as T=t2.M*(O-t2.V)
-   * <li>t1.Minv is the inverse of t1.M
-   * <li>t2.Minv is the inverse of t2.M
-   * </ul>
+   * - 't1' expressed as T=t1.M*(O-t1.V)
+   * - 't2' expressed as T=t2.M*(O-t2.V)
+   * - t1.Minv is the inverse of t1.M
+   * - t2.Minv is the inverse of t2.M
+   *
    * Then this will calculate a new transformation in 't1' as follows:
    * T=(t1.M*t2.Minv)*(O-(t2.M*(t1.V-t2.V))).
    */
-  friend csReversibleTransform& operator/= (csReversibleTransform& t1,
-                                          const csReversibleTransform& t2);
+  friend CS_CRYSTALSPACE_EXPORT csReversibleTransform& operator/= (
+    csReversibleTransform& t1, const csReversibleTransform& t2);
 
   /**
    * Combine two transforms, reversing t2 then applying t1.
    * Given the following definitions:
-   * <ul>
-   * <li>'t1' expressed as T=t1.M*(O-t1.V)
-   * <li>'t2' expressed as T=t2.M*(O-t2.V)
-   * <li>t1.Minv is the inverse of t1.M
-   * <li>t2.Minv is the inverse of t2.M
-   * </ul>
+   * - 't1' expressed as T=t1.M*(O-t1.V)
+   * - 't2' expressed as T=t2.M*(O-t2.V)
+   * - t1.Minv is the inverse of t1.M
+   * - t2.Minv is the inverse of t2.M
+   *
    * Then this will calculate a new transformation in 't1' as follows:
    * T=(t1.M*t2.Minv)*(O-(t2.M*(t1.V-t2.V))).
    */
-  friend csReversibleTransform operator/ (const csReversibleTransform& t1,
-                                        const csReversibleTransform& t2);
+  friend CS_CRYSTALSPACE_EXPORT csReversibleTransform operator/ (
+    const csReversibleTransform& t1, const csReversibleTransform& t2);
 };
 
 /**
@@ -575,5 +658,7 @@ public:
   { m_t2o = m;  m_o2t = m_t2o.GetTranspose (); }
 };
 
+/** @} */
+
 #endif // __CS_TRANSFORM_H__
 
