@@ -20,44 +20,66 @@
 #ifndef __CS_VECTOR3_H__
 #define __CS_VECTOR3_H__
 
-/**\file 
+/**\file
+ * 3D vector.
  */
 /**
  * \addtogroup geom_utils
  * @{ */
 
-#ifndef __CS_CSSYSDEFS_H__
-#error "cssysdef.h must be included in EVERY source file!"
-#endif
-
 #include "csextern.h"
+#include "math3d_d.h"
 
-#include "csgeom/math3d_d.h"
+class csString;
+
+/**
+ * Constants for axes. Can also be used as indices to csVector2::operator[], 
+ * csVector3::operator[] and csVector4::operator[] for more readability.
+ */
+enum
+{
+  /// Invalid axis
+  CS_AXIS_NONE = -1,
+  /// X axis
+  CS_AXIS_X = 0,
+  /// Y axis
+  CS_AXIS_Y = 1,
+  /// Z axis
+  CS_AXIS_Z = 2,
+  /// W axis
+  CS_AXIS_W = 3
+};
 
 /**
  * A 3D vector.
  */
-class CS_CSGEOM_EXPORT csVector3
+class CS_CRYSTALSPACE_EXPORT csVector3
 {
 public:
+
+#if !defined(__STRICT_ANSI__) && !defined(SWIG)
   union
   {
-    float f[3];
-    struct
+    struct 
     {
+#endif
       /// The X component of the vector
       float x;
       /// The Y component of the vector
       float y;
       /// The Z component of the vector
       float z;
+#if !defined(__STRICT_ANSI__) && !defined(SWIG)
     };
+    /// All components
+    float m[3];
   };
-  
+#endif
+
   /**
-   * Make a new vector. The vector is not
-   * initialized. This makes the code slightly faster as
-   * csVector3 objects are used a lot.
+   * Make a new vector.
+   * \warning The vector is not initialized. This makes the code slightly
+   *   faster as csVector3 objects are used a lot.
    */
   csVector3 () {}
 
@@ -77,36 +99,39 @@ public:
   /// Conversion from double precision vector to single.
   csVector3 (const csDVector3&);
 
+  /// Return a textual representation of the vector in the form "x,y,z".
+  csString Description() const;
+
   /// Add two vectors.
-  inline friend csVector3 operator+ (const csVector3& v1, const csVector3& v2)
+  inline friend csVector3 operator+(const csVector3& v1, const csVector3& v2)
   { return csVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z); }
 
   /// Add two vectors of differing type, raise the csVector3 to DVector3.
-  inline friend csDVector3 operator+ (const csDVector3& v1, const csVector3& v2)
+  inline friend csDVector3 operator+(const csDVector3& v1, const csVector3& v2)
   { return csDVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z); }
 
   /// Add two vectors of differing type, raise the csVector3 to DVector3.
-  inline friend csDVector3 operator+ (const csVector3& v1, const csDVector3& v2)
+  inline friend csDVector3 operator+(const csVector3& v1, const csDVector3& v2)
   { return csDVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z); }
 
   /// Subtract two vectors.
-  inline friend csVector3 operator- (const csVector3& v1, const csVector3& v2)
+  inline friend csVector3 operator-(const csVector3& v1, const csVector3& v2)
   { return csVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z); }
 
   /// Subtract two vectors of differing type, cast to double.
-  inline friend csDVector3 operator- (const csVector3& v1, const csDVector3& v2)
+  inline friend csDVector3 operator-(const csVector3& v1, const csDVector3& v2)
   { return csDVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z); }
 
   /// Subtract two vectors of differing type, cast to double.
-  inline friend csDVector3 operator- (const csDVector3& v1, const csVector3& v2)
+  inline friend csDVector3 operator-(const csDVector3& v1, const csVector3& v2)
   { return csDVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z); }
 
   /// Take the dot product of two vectors.
-  inline friend float operator* (const csVector3& v1, const csVector3& v2)
+  inline friend float operator*(const csVector3& v1, const csVector3& v2)
   { return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z; }
 
   /// Take the cross product of two vectors.
-  inline friend csVector3 operator% (const csVector3& v1, const csVector3& v2)
+  inline friend csVector3 operator%(const csVector3& v1, const csVector3& v2)
   {
     return csVector3 (v1.y*v2.z-v1.z*v2.y,
                       v1.z*v2.x-v1.x*v2.z,
@@ -182,10 +207,18 @@ public:
   { return ABS(v.x)<f && ABS(v.y)<f && ABS(v.z)<f; }
 
   /// Returns n-th component of the vector.
+#ifdef __STRICT_ANSI__
   inline float operator[] (int n) const { return !n?x:n&1?y:z; }
+#else
+  inline float operator[] (int n) const { return m[n]; }
+#endif
 
   /// Returns n-th component of the vector.
+#ifdef __STRICT_ANSI__
   inline float & operator[] (int n) { return !n?x:n&1?y:z; }
+#else
+  inline float & operator[] (int n) { return m[n]; }
+#endif
 
   /// Add another vector to this vector.
   inline csVector3& operator+= (const csVector3& v)
@@ -225,7 +258,16 @@ public:
   inline void Set (float sx, float sy, float sz) { x = sx; y = sy; z = sz; }
 
   /// Set the value of this vector.
-  inline void Set (const csVector3& v) { x = v.x; y = v.y; z = v.z; }
+  inline void Set (csVector3 const& v) { x = v.x; y = v.y; z = v.z; }
+
+  /// Set the value of this vector.
+  inline void Set (float const* v) { x = v[0]; y = v[1]; z = v[2]; }
+
+  /// Set the value of this vector so that all components are the same.
+  inline void Set (float v) { x = y = z = v; }
+
+  /// Get the value of this vector.
+  inline void Get (float* v) { v[0] = x; v[1] = y; v[2] = z; }
 
   /// Returns the norm of this vector.
   float Norm () const;
