@@ -67,7 +67,7 @@ struct csColorBox;
  * case for big endian machines for the INDEX_B macro (see below). With
  * big-endian machines the B component starts at bit 8, thus the right
  * shift counter becomes negative. Probably the best solution is to add
- * and #if B_BIT + 8 - HIST_B_BITS - HIST_G_BITS - HIST_R_BITS < 0.
+ * an \#if B_BIT + 8 - HIST_B_BITS - HIST_G_BITS - HIST_R_BITS < 0.
  * 
  * <b>Quantizing</b><br>
  * The following routines can be used to split the quantization process
@@ -126,10 +126,17 @@ struct csColorBox;
  * Palette() will allocate color index 0 for that color, and
  * Remap will map all such pixel values to index 0.
  */
-class CS_CSGFX_EXPORT csColorQuantizer
+class CS_CRYSTALSPACE_EXPORT csColorQuantizer
 {
 private:
   friend struct csColorBox;
+  struct ColorIndex
+  {
+    int index;
+    csColorBox* box;
+
+    ColorIndex() : box(0) {}
+  };
 
   // The storage for color usage histogram.
   uint16 *hist;
@@ -141,7 +148,7 @@ private:
   // Number of valid color boxes.
   int boxcount;
   // The storage for color indices.
-  uint8 *color_index;
+  ColorIndex* color_index;
 
   // The state of quantization variables
   enum
