@@ -20,18 +20,25 @@
 #ifndef __CS_CSVIEW_H__
 #define __CS_CSVIEW_H__
 
+/**\file
+ * Crystal Space 3D renderer view
+ */
+
 #include "csextern.h"
 
+#include "csutil/ref.h"
+#include "csutil/scf_implementation.h"
 #include "ivaria/view.h"
 
-class csPoly2D;
 class csBox2;
+class csPoly2D;
 
 /**
  * The csView class encapsulates the top-level Crystal Space
  * renderer interface. It is basically a camera and a clipper.
  */
-class CS_CSTOOL_EXPORT csView : public iView
+class CS_CRYSTALSPACE_EXPORT csView : 
+  public scfImplementation1<csView, iView>
 {
 private:
   /// the engine
@@ -96,8 +103,6 @@ public:
   virtual iClipper2D* GetClipper ();
   /// Draw 3D world as seen from the camera.
   virtual void Draw ();
-
-  SCF_DECLARE_IBASE;
 };
 
 #endif // __CS_CSVIEW_H__
