@@ -29,8 +29,8 @@
 
 #include "csextern.h"
 
-#include "cstypes.h"
 #include "csutil/array.h"
+#include "csutil/eventnames.h"
 #include "csutil/csstring.h"
 #include "csutil/scf.h"
 #include "iengine/engine.h"
@@ -53,24 +53,25 @@ struct iPluginManager;
 struct iVirtualClock;
 struct iCommandLineParser;
 struct iConfigManager;
+struct iVerbosityManager;
 
 /**\name Plugin request macros
  * Utility macros to select what plugins you want to have loaded.
  * @{ */
 /// Request a plugin.
-#define CS_REQUEST_PLUGIN(Name,Interface) Name, #Interface, \
-  scfInterface<Interface>::GetID(), scfInterface<Interface>::GetVersion()
+#define CS_REQUEST_PLUGIN(Name,Interface) Name, scfInterfaceTraits<Interface>::GetName(), \
+  scfInterfaceTraits<Interface>::GetID(), scfInterfaceTraits<Interface>::GetVersion()
 
 // !!! NOTE !!!
 // When editing this list, you *must* ensure that initapp.h #include the
 // appropriate header for each SCF interface mentioned by a CS_REQUEST_PLUGIN()
 // invocation. This is necessary to guarantee that the interface-specialized
-// version of scfInterface<> is seen by clients rather than the non-specialized
+// version of scfInterfaceTraits<> is seen by clients rather than the non-specialized
 // template.
 
 /// Marker for the end of the requested plugins list.
 #define CS_REQUEST_END \
-  0
+  (const char*)0
 /// Request VFS plugin.
 #define CS_REQUEST_VFS \
   CS_REQUEST_PLUGIN("crystalspace.kernel.vfs", iVFS)
@@ -79,7 +80,7 @@ struct iConfigManager;
   CS_REQUEST_PLUGIN("crystalspace.font.server.default", iFontServer)
 /// Request default image loader.
 #define CS_REQUEST_IMAGELOADER \
-  CS_REQUEST_PLUGIN("crystalspace.graphic.image.io.multiplex", iImageIO)
+  CS_REQUEST_PLUGIN("crystalspace.graphic.image.io.multiplexer", iImageIO)
 /// Request null 3D renderer
 #define CS_REQUEST_NULL3D \
   CS_REQUEST_PLUGIN("crystalspace.graphics3d.null",iGraphics3D)
@@ -125,7 +126,7 @@ typedef bool (*csEventHandlerFunc) (iEvent&);
  * csPluginRequest r2(CS_REQUEST_PLUGIN("myproj.foobar",iFoobar));
  * \endcode
  */
-class CS_CSTOOL_EXPORT csPluginRequest
+class CS_CRYSTALSPACE_EXPORT csPluginRequest
 {
 private:
   csString class_name;
@@ -152,29 +153,30 @@ public:
  * setup an application. It is possible to do all the setup on your own
  * but using the functions below will help considerably.
  */
-class CS_CSTOOL_EXPORT csInitializer
+class CS_CRYSTALSPACE_EXPORT csInitializer
 {
 public:
   /**
    * Create everything needed to get a CS application operational.
    * This function is completely equivalent to calling:
-   * <ul>
-   * <li>#CS_INITIALIZE_PLATFORM_APPLICATION macro</li>
-   * <li>InitializeSCF()</li>
-   * <li>CreateObjectRegistry()</li>
-   * <li>CreatePluginManager()</li>
-   * <li>CreateEventQueue()</li>
-   * <li>CreateVirtualClock()</li>
-   * <li>CreateCommandLineParser()</li>
-   * <li>CreateConfigManager()</li>
-   * <li>CreateInputDrivers()</li>
-   * <li>CreateStringSet()</li>
-   * <li>csPlatformStartup()</li>
-   * </ul>
+   * - #CS_INITIALIZE_PLATFORM_APPLICATION macro
+   * - InitializeSCF()
+   * - CreateObjectRegistry()
+   * - CreatePluginManager()
+   * - CreateEventQueue()
+   * - CreateVirtualClock()
+   * - CreateCommandLineParser()
+   * - CreateVerbosityManager()
+   * - CreateConfigManager()
+   * - CreateInputDrivers()
+   * - CreateStringSet()
+   * - csPlatformStartup()
+   *
    * You may want to call all or some of those manually if you don't wish
    * to use this function. It is suggested that 
-   * #CS_INITIALIZE_PLATFORM_APPLICATION, InitializeSCF() and 
-   * csPlatformStartup() are always invoked; the other calls can be replaced 
+   * #CS_INITIALIZE_PLATFORM_APPLICATION, InitializeSCF(),
+   * and csPlatformStartup() are always invoked; 
+   * the other calls can be replaced 
    * with manual creation of the respective objects. However, the order above
    * should be retained when doing so.
    * \param argc argc argument from main().
@@ -229,6 +231,12 @@ public:
   static iCommandLineParser* CreateCommandLineParser (
     iObjectRegistry*, int argc, char const* const argv[]);
 
+  /**
+   * Create and, if needed, register the verbosity manager. It is used by a 
+   * lot of plugins to control diagnostoc output while running.
+   */
+  static iVerbosityManager* CreateVerbosityManager (iObjectRegistry*);
+
   /**
    * Create the config manager. This function will register the created
    * config manager with the object registry as the default config manager
@@ -260,14 +268,15 @@ public:
    * call this routine using a 0 parameter. If you don't call this then
    * either RequestPlugins() or Initialize() will call this routine with
    * 0 parameter. The 'ApplicationID' parameter is used to determine the
-   * correct user-specific domain. It is possibly overriden by the application
-   * config file option "System.ApplicationID".
+   * correct user-specific domain; if 0, the return value of GetDefaultAppID()
+   * is used. It is possibly overriden by the application config file option 
+   * "System.ApplicationID".
    *
    * This method will load the VFS plugin if not already present in the given
    * object registry.
    */
   static bool SetupConfigManager (iObjectRegistry*, const char* configName,
-    const char *ApplicationID = "CrystalSpace.Noname");
+    const char *ApplicationID = 0);
 
   /**
    * Find or load the VFS plugin, add it to the given object registry, and
@@ -296,6 +305,13 @@ public:
    */
   static bool RequestPlugins (iObjectRegistry*, ...);
 
+  /**
+   * This is just like RequestPlugins(...), which accepts a variable list of
+   * arguments at compile-time, except that arguments are passed as a
+   * `va_list'.
+   */
+  static bool RequestPluginsV (iObjectRegistry*, va_list);
+
   /**
    * Request a few widely used standard plugins and also read the standard
    * configuration file and command line for potential other plugins.  This
@@ -318,13 +334,13 @@ public:
   static bool RequestPlugins(iObjectRegistry*,csArray<csPluginRequest> const&);
 
   /**
-   * Send the cscmdSystemOpen command to all loaded plugins.
+   * Send the csevSystemOpen command to all loaded plugins.
    * This should be done after initializing them (Initialize()).
    */
   static bool OpenApplication (iObjectRegistry*);
 
   /**
-   * Send the cscmdSystemClose command to all loaded plugins.
+   * Send the csevSystemClose command to all loaded plugins.
    */
   static void CloseApplication (iObjectRegistry*);
 
@@ -334,30 +350,26 @@ public:
    * that are sent through the event manager. Use this function to know
    * about keyboard, mouse and other events. Note that you also have to
    * use this function to be able to render something as rendering
-   * happens as a result of one event (cscmdProcess).
+   * happens as a result of one event (csevProcess).
    */
-  static bool SetupEventHandler (
-    iObjectRegistry*, iEventHandler*, unsigned int eventmask);
+  static bool SetupEventHandler (iObjectRegistry*, iEventHandler*, const csEventID[]);
 
   /**
    * Initialize an event handler function. This is an easier version
    * of SetupEventHandler() that takes a function and will register an
-   * event handler to call that function for all relevant events if `eventmask'
-   * is not specified, or for the requested event types if it is specified.
+   * event handler to call that function for all stated events and their
+   * children, or (if the third argument is omitted) for all events.
    */
-  static bool SetupEventHandler (
-    iObjectRegistry*, csEventHandlerFunc, unsigned int eventmask =
-      CSMASK_Nothing          |
-      CSMASK_Broadcast        |
-      CSMASK_MouseUp          |
-      CSMASK_MouseDown        |
-      CSMASK_MouseMove        |
-      CSMASK_Keyboard         |
-      CSMASK_MouseClick       |
-      CSMASK_MouseDoubleClick |
-      CSMASK_JoystickMove     |
-      CSMASK_JoystickDown     |
-      CSMASK_JoystickUp);
+  static bool SetupEventHandler (iObjectRegistry*, csEventHandlerFunc, const csEventID events[]);
+
+  /**
+   * Initialize an event handler function.  This function will receive EVERY
+   * event that passes through the system.  This is usually not what you 
+   * want, since (1) it means you'll probably receive a lot of events you're
+   * not interested in, and (2) it's the most costly operation for the event
+   * subscription scheduler.
+   */
+  static bool SetupEventHandler (iObjectRegistry*, csEventHandlerFunc);
 
   /**
    * Destroy the application.<p>
@@ -388,6 +400,14 @@ public:
    * application.)
    */
   static void DestroyApplication (iObjectRegistry*);
+  
+  /**
+   * The default application string name; computed from the executable name
+   * as passed in argv[0] to CreateEnvironment(). If the latter was not called
+   * or argv[0] did not contain a sensible name, a default string is used.
+   */
+  static const char* GetDefaultAppID();
+protected:
 };
 
 /** @} */
