@@ -19,12 +19,16 @@
 #ifndef __CS_KEYVAL_H__
 #define __CS_KEYVAL_H__
 
+/**\file
+ * Key/Value pair
+ */
+
 #include "csextern.h"
 
-#include "csgeom/vector3.h"
 #include "csutil/csobject.h"
 #include "csutil/hash.h"
-#include "csutil/hashhandlers.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/set.h"
 #include "ivaria/keyval.h"
 
 /**
@@ -33,13 +37,15 @@
  * the iObject.
  * <p>
  * One way to attach key value pairs to CS objects is to add the following
- * xml to your object: <key name="somename" value="somevalue" />
+ * xml to your object: \<key name="somename" value="somevalue" /\>
  */
-class CS_CSTOOL_EXPORT csKeyValuePair : public csObject, public iKeyValuePair
+class CS_CRYSTALSPACE_EXPORT csKeyValuePair : 
+  public scfImplementationExt1<csKeyValuePair, csObject, iKeyValuePair>
 {
 private:
   const char *m_Value;	// Points to a string in the hash below.
-  csHash<csStrKey, csStrKey, csConstCharHashKeyHandler> values;
+  csHash<csStrKey, csStrKey> values;
+  csSet<csStrKey> names;
 
 public:
   /// The constructor. Requires both key and value. Data is being copied!
@@ -49,7 +55,6 @@ public:
   /// The destructor as usual
   virtual ~csKeyValuePair ();
 
-  SCF_DECLARE_IBASE_EXT (csObject);
   //----------------------- iKeyValuePair --------------------------
   virtual iObject *QueryObject() { return (csObject*)this; }
   virtual const char *GetKey () const;
@@ -58,6 +63,9 @@ public:
   virtual const char *GetValue (const char* vname) const;
   virtual void SetValue (const char* value);
   virtual void SetValue (const char* vname, const char* value);
+
+  virtual csRef<iStringArray> GetValueNames () const;
+
 };
 
 #endif // __CS_KEYVAL_H__
