@@ -21,6 +21,12 @@
 #ifndef __CSUTIL_ARRAY_H__
 #define __CSUTIL_ARRAY_H__
 
+/**\file
+ * Generic Array Template
+ */
+
+#include "comparator.h"
+
 // Hack: Work around problems caused by #defining 'new'.
 #if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
 # undef new
@@ -29,36 +35,12 @@
 
 #if defined(CS_MEMORY_TRACKER)
 #include "csutil/memdebug.h"
+#include "csutil/snprintf.h"
 #include <typeinfo>
 #endif
 
-/**
- * A template providing various comparison and ordering functions.
- */
-template <class T1, class T2>
-class csOrdering
-{
-public:
-  /**
-   * Compare two objects of the same type or different types (T1 and T2).
-   * \param r1 Reference to first object.
-   * \param r2 Reference to second object.
-   * \return Zero if the objects are equal; less-than-zero if the first object
-   *   is less than the second; or greater-than-zero if the first object is
-   *   greater than the second.
-   * \remarks Assumes the existence of T1::operator<(T2) and T2::operator<(T1).
-   *   If T1 and T2 are the same type T, then only T::operator<(T) is assumed
-   *   (of course).  This is the default comparison function used by csArray
-   *   for searching and sorting if the client does not provide a custom
-   *   function.
-   */
-  static int Compare(T1 const &r1, T2 const &r2)
-  {
-    if (r1 < r2) return -1;
-    else if (r2 < r1) return 1;
-    else return 0;
-  }
-};
+/**\addtogroup util_containers
+ * @{ */
 
 // Define CSARRAY_INHIBIT_TYPED_KEYS if the compiler is too old or too buggy to
 // properly support templated functions within a templated class.  When this is
@@ -122,7 +104,7 @@ public:
    *   Default comparison function if client does not supply one.
    */
   static int DefaultCompare(T const& r, K const& k)
-    { return csOrdering<T,K>::Compare(r,k); }
+    { return csComparator<T,K>::Compare(r,k); }
 private:
   K key;
   CF cmp;
@@ -155,7 +137,7 @@ public:
   virtual operator CF() const { return cmp; }
   operator K const&() const { return key; }
   static int DefaultCompare(T const& r, K const& k)
-    { return csOrdering<T,K>::Compare(r,k); }
+    { return csComparator<T,K>::Compare(r,k); }
 private:
   K key;
   CF cmp;
@@ -174,6 +156,11 @@ template <class T>
 class csArrayElementHandler
 {
 public:
+  static void Construct (T* address)
+  {
+    new (CS_STATIC_CAST(void*,address)) T();
+  }
+
   static void Construct (T* address, T const& src)
   {
     new (CS_STATIC_CAST(void*,address)) T(src);
@@ -184,10 +171,10 @@ public:
     address->~T();
   }
 
-  static void InitRegion (T* address, int count)
+  static void InitRegion (T* address, size_t count)
   {
-    for (int i = 0 ; i < count ; i++)
-      Construct (address + i, T ());
+    for (size_t i = 0 ; i < count ; i++)
+      Construct (address + i);
   }
 };
 
@@ -198,7 +185,7 @@ template <class T>
 class csArrayMemoryAllocator
 {
 public:
-  static T* Alloc (int count)
+  static T* Alloc (size_t count)
   {
     return (T*)malloc (count * sizeof(T));
   }
@@ -210,26 +197,33 @@ public:
 
   // The 'relevantcount' parameter should be the number of items
   // in the old array that are initialized.
-  static T* Realloc (T* mem, int relevantcount, int oldcount, int newcount)
+  static T* Realloc (T* mem, size_t relevantcount, size_t oldcount,
+    size_t newcount)
   {
     (void)relevantcount; (void)oldcount;
     return (T*)realloc (mem, newcount * sizeof(T));
   }
+
+  // Move memory.
+  static void MemMove (T* mem, size_t dest, size_t src, size_t count)
+  {
+    memmove (mem + dest, mem + src, count * sizeof(T));
+  }
 };
 
 /**
  * Special allocator for csArray that makes sure that when
  * the array is reallocated that the objects are properly constructed
  * and destructed at their new position. This is needed for objects
- * that can't be safely moved around in memory (like weak references).
- * This is of course slower and that's the reason that this is not
+ * that can not be safely moved around in memory (like weak references).
+ * This is of course slower and that is the reason that this is not
  * done by default.
  */
 template <class T, class ElementHandler = csArrayElementHandler<T> >
 class csSafeCopyArrayMemoryAllocator
 {
 public:
-  static T* Alloc (int count)
+  static T* Alloc (size_t count)
   {
     return (T*)malloc (count * sizeof(T));
   }
@@ -239,7 +233,8 @@ public:
     free (mem);
   }
 
-  static T* Realloc (T* mem, int relevantcount, int oldcount, int newcount)
+  static T* Realloc (T* mem, size_t relevantcount, size_t oldcount,
+    size_t newcount)
   {
     if (newcount <= oldcount)
     {
@@ -250,7 +245,7 @@ public:
     }
 
     T* newmem = Alloc (newcount);
-    int i;
+    size_t i;
     for (i = 0 ; i < relevantcount ; i++)
     {
       ElementHandler::Construct (newmem + i, mem[i]);
@@ -259,15 +254,44 @@ public:
     Free (mem);
     return newmem;
   }
+
+  static void MemMove (T* mem, size_t dest, size_t src, size_t count)
+  {
+    size_t i;
+    if (dest < src)
+    {
+      for (i = 0 ; i < count ; i++)
+      {
+        ElementHandler::Construct (mem + dest + i, mem[src + i]);
+	ElementHandler::Destroy (mem + src + i);
+      }
+    }
+    else
+    {
+      i = count;
+      while (i > 0)
+      {
+	i--;
+        ElementHandler::Construct (mem + dest + i, mem[src + i]);
+	ElementHandler::Destroy (mem + src + i);
+      }
+    }
+  }
 };
 
+/**
+ * This value is returned whenever an array item could not be located or does
+ * not exist.
+ */
+const size_t csArrayItemNotFound = (size_t)-1;
 
 /**
  * A templated array class.  The objects in this class are constructed via
  * copy-constructor and are destroyed when they are removed from the array or
- * the array is destroyed.  Note: If you want to store reference-counted object
- * pointers (such as iSomething*), then you should look at csRefArray instead
- * of this class.
+ * the array is destroyed.
+ * \note If you want to store reference-counted object pointers, such as iFoo*,
+ * then you should consider csRefArray<>, which is more idiomatic than
+ * csArray<csRef<iFoo> >.
  */
 template <class T,
 	class ElementHandler = csArrayElementHandler<T>,
@@ -275,19 +299,23 @@ template <class T,
 class csArray
 {
 private:
-  int count;
-  int capacity;
-  int threshold;
+  size_t count;
+  size_t capacity;
+  size_t threshold;
   T* root;
-# ifdef CS_MEMORY_TRACKER
-  MemTrackerInfo* mti;
+#ifdef CS_MEMORY_TRACKER
+  csMemTrackerInfo* mti;
   void UpdateMti (int dn, int curcapacity)
   {
     if (!mti)
     {
       if (!curcapacity) return;
-      char buf[255];
-      sprintf (buf, "csArray<%s>", typeid (T).name());
+      char buf[1024];
+      strcpy (buf, "csArray<");
+      size_t l = strlen (typeid (T).name ());
+      if (l > 1000) l = 1000;
+      strncat (buf, typeid (T).name (), l);
+      strcat (buf, ">");
       mti = mtiRegisterAlloc (1 * sizeof (T), buf);
       if (!mti) return;
       curcapacity--;
@@ -297,60 +325,71 @@ private:
     }
     mtiUpdateAmount (mti, dn, dn * sizeof (T));
   }
-# endif
+#endif
 
 protected:
   /**
    * Initialize a region. This is a dangerous function to use because it
    * does not properly destruct the items in the array.
    */
-  void InitRegion (int start, int count)
+  void InitRegion (size_t start, size_t count)
   {
     ElementHandler::InitRegion (root+start, count);
   }
 
 private:
-  /// Copy from one array to this.
+  /// Copy from one array to this one, properly constructing the copied items.
   void CopyFrom (const csArray& source)
   {
     if (&source != this)
     {
       DeleteAll ();
       threshold = source.threshold;
-      SetLengthUnsafe (source.Length ());
-      for (int i=0 ; i<source.Length() ; i++)
+      SetSizeUnsafe (source.Length ());
+      for (size_t i=0 ; i<source.Length() ; i++)
         ElementHandler::Construct (root + i, source[i]);
     }
   }
 
-  // Adjust internal capacity of this array.
-  void AdjustCapacity (int n)
+  /// Set the capacity of the array precisely to \c n elements.
+  void InternalSetCapacity (size_t n)
+  {
+    if (root == 0)
+    {
+      root = MemoryAllocator::Alloc (n);
+#ifdef CS_MEMORY_TRACKER
+      UpdateMti (n, n);
+#endif
+    }
+    else
+    {
+      root = MemoryAllocator::Realloc (root, count, capacity, n);
+#ifdef CS_MEMORY_TRACKER
+      UpdateMti (n-capacity, n);
+#endif
+    }
+    capacity = n;
+  }
+
+  /**
+   * Adjust capacity of this array to \c n elements rounded up to a multiple of
+   * \c threshold.
+   */
+  void AdjustCapacity (size_t n)
   {
     if (n > capacity || (capacity > threshold && n < capacity - threshold))
     {
-      n = ((n + threshold - 1) / threshold ) * threshold;
-      if (root == 0)
-      {
-        root = MemoryAllocator::Alloc (n);
-#	ifdef CS_MEMORY_TRACKER
-	UpdateMti (n, n);
-#	endif
-      }
-      else
-      {
-        root = MemoryAllocator::Realloc (root, count, capacity, n);
-#	ifdef CS_MEMORY_TRACKER
-	UpdateMti (n-capacity, n);
-#	endif
-      }
-      capacity = n;
+      InternalSetCapacity (((n + threshold - 1) / threshold ) * threshold);
     }
   }
 
-  // Set array length.  NOTE: Do not make this public since it does not
-  // properly construct/destroy elements.  To safely truncate the array, use
-  // Truncate().  To safely set the capacity, use SetCapacity().
-  void SetLengthUnsafe (int n)
+  /**
+   * Set array length.
+   * \warning Do not make this public since it does not properly
+   *   construct/destroy elements.  To safely truncate the array, use
+   *   Truncate().  To safely set the capacity, use SetCapacity().
+   */
+  void SetSizeUnsafe (size_t n)
   {
     if (n > capacity)
       AdjustCapacity (n);
@@ -371,27 +410,28 @@ public:
    */
   static int DefaultCompare(T const& r1, T const& r2)
   {
-    return csOrdering<T,T>::Compare(r1,r2);
+    return csComparator<T,T>::Compare(r1,r2);
   }
 
   /**
-   * Initialize object to have initial capacity of 'icapacity' elements, and to
-   * increase storage by 'ithreshold' each time the upper bound is exceeded.
+   * Initialize object to have initial capacity of \c in_capacity elements, and
+   * to increase storage by \c in_threshold each time the upper bound is
+   * exceeded.
    */
-  csArray (int icapacity = 0, int ithreshold = 0)
+  csArray (size_t in_capacity = 0, size_t in_threshold = 0)
   {
-#   ifdef CS_MEMORY_TRACKER
+#ifdef CS_MEMORY_TRACKER
     mti = 0;
-#   endif
+#endif
     count = 0;
-    capacity = (icapacity > 0 ? icapacity : 0);
-    threshold = (ithreshold > 0 ? ithreshold : 16);
+    capacity = (in_capacity > 0 ? in_capacity : 0);
+    threshold = (in_threshold > 0 ? in_threshold : 16);
     if (capacity != 0)
     {
       root = MemoryAllocator::Alloc (capacity);
-#     ifdef CS_MEMORY_TRACKER
+#ifdef CS_MEMORY_TRACKER
       UpdateMti (capacity, capacity);
-#     endif
+#endif
     }
     else
     {
@@ -399,6 +439,7 @@ public:
     }
   }
 
+  /// Destroy array and all contained elements.
   ~csArray ()
   {
     DeleteAll ();
@@ -407,15 +448,15 @@ public:
   /// Copy constructor.
   csArray (const csArray& source)
   {
-#   ifdef CS_MEMORY_TRACKER
+#ifdef CS_MEMORY_TRACKER
     mti = 0;
-#   endif
+#endif
     root = 0;
     capacity = 0;
     count = 0;
     CopyFrom (source);
   }
-  
+
   /// Assignment operator.
   csArray<T,ElementHandler>& operator= (const csArray& other)
   {
@@ -423,14 +464,23 @@ public:
     return *this;
   }
 
-  /// Return the number of elements in the Array
-  int Length () const
+  /// Return the number of elements in the array.
+  size_t GetSize () const
   {
     return count;
   }
 
+  /**
+   * Return the number of elements in the array.
+   * \deprecated Use GetSize() instead.
+   */
+  size_t Length () const
+  {
+    return GetSize();
+  }
+
   /// Query vector capacity.  Note that you should rarely need to do this.
-  int Capacity () const
+  size_t Capacity () const
   {
     return capacity;
   }
@@ -450,23 +500,25 @@ public:
       destination.count = count;
       destination.capacity = capacity;
       destination.threshold = threshold;
-#     ifdef CS_MEMORY_TRACKER
+#ifdef CS_MEMORY_TRACKER
       destination.mti = mti;
       mti = 0;
-#     endif
+#endif
       root = 0;
       capacity = count = 0;
     }
   }
 
   /**
-   * Set the actual number of items in this array. This can be used to
-   * shrink an array (works like Truncate() then in which case it will properly
-   * destroy all truncated objects) or to enlarge an array in which case
-   * it will properly set the new capacity and construct all new items
-   * based on the given item.
+   * Set the actual number of items in this array. This can be used to shrink
+   * an array (like Truncate()) or to enlarge an array, in which case it will
+   * properly construct all new items based on the given item.
+   * \param n New array length.
+   * \param what Object used as template to construct each newly added object
+   *   using the object's copy constructor when the array size is increased by
+   *   this method.
    */
-  void SetLength (int n, T const& what)
+  void SetSize (size_t n, T const& what)
   {
     if (n <= count)
     {
@@ -474,15 +526,21 @@ public:
     }
     else
     {
-      int old_len = Length ();
-      SetLengthUnsafe (n);
-      for (int i = old_len ; i < n ; i++)
+      size_t old_len = Length ();
+      SetSizeUnsafe (n);
+      for (size_t i = old_len ; i < n ; i++)
         ElementHandler::Construct (root + i, what);
     }
   }
 
-  /// Set vector length to n.
-  void SetLength (int n)
+  /**
+   * Set the actual number of items in this array. This can be used to shrink
+   * an array (like Truncate()) or to enlarge an array, in which case it will
+   * properly construct all new items using their default (zero-argument)
+   * constructor.
+   * \param n New array length.
+   */
+  void SetSize (size_t n)
   {
     if (n <= count)
     {
@@ -490,104 +548,119 @@ public:
     }
     else
     {
-      int old_len = Length ();
-      SetLengthUnsafe (n);
+      size_t old_len = Length ();
+      SetSizeUnsafe (n);
       ElementHandler::InitRegion (root + old_len, n-old_len);
     }
   }
 
+  /** @{ */
+  /**
+   * Set the actual number of items in this array.
+   * \deprecated Use SetSize() instead.
+   */
+  void SetLength (size_t n, T const& what) { SetSize(n, what); }
+  void SetLength (size_t n) { SetSize(n); }
+  /** @} */
+
   /// Get an element (non-const).
-  T& Get (int n)
+  T& Get (size_t n)
   {
-    CS_ASSERT (n >= 0 && n < count);
+    CS_ASSERT (n < count);
     return root[n];
   }
 
   /// Get an element (const).
-  T const& Get (int n) const
+  T const& Get (size_t n) const
   {
-    CS_ASSERT (n >= 0 && n < count);
+    CS_ASSERT (n < count);
     return root[n];
   }
 
   /**
-   * Get an item from the array. If the number of elements in this
-   * array is too small the array will be automatically extended.
+   * Get an item from the array. If the number of elements in this array is too
+   * small the array will be automatically extended, and the newly added
+   * objects will be created using their default (no-argument) constructor.
    */
-  T& GetExtend (int n)
+  T& GetExtend (size_t n)
   {
-    CS_ASSERT (n >= 0);
     if (n >= count)
-      SetLength (n+1);
+      SetSize (n+1);
     return root[n];
   }
 
   /// Get an element (non-const).
-  T& operator [] (int n)
+  T& operator [] (size_t n)
   {
     return Get(n);
   }
 
   /// Get a const reference.
-  T const& operator [] (int n) const
+  T const& operator [] (size_t n) const
   {
     return Get(n);
   }
 
-  /// Put an element at some position.
-  void Put (int n, T const& what)
+  /// Insert a copy of element at the indicated position.
+  void Put (size_t n, T const& what)
   {
-    CS_ASSERT (n >= 0);
     if (n >= count)
-      SetLength (n+1);
+      SetSize (n+1);
     ElementHandler::Destroy (root + n);
     ElementHandler::Construct (root + n, what);
   }
 
   /**
-   * Find an element based on some key.
+   * Find an element based upon some arbitrary key, which may be embedded
+   * within an element, or otherwise derived from it. The incoming key \e
+   * functor defines the relationship between the key and the array's element
+   * type.
+   * \return csArrayItemNotFound if not found, else item index.
    */
   csArrayTemplate(K)
-  int FindKey (csArrayCmpDecl(T,K) comparekey) const
+  size_t FindKey (csArrayCmpDecl(T,K) comparekey) const
   {
-    for (int i = 0 ; i < Length () ; i++)
+    for (size_t i = 0 ; i < Length () ; i++)
       if (csArrayCmpInvoke(comparekey, root[i]) == 0)
         return i;
-    return -1;
+    return csArrayItemNotFound;
   }
 
-  /// Push an element onto the tail end of the array. Returns index of element.
-  int Push (T const& what)
+  /**
+   * Push a copy of an element onto the tail end of the array.
+   * \return Index of newly added element.
+   */
+  size_t Push (T const& what)
   {
-    if (((&what >= root) && (&what < root + Length())) && 
+    if (((&what >= root) && (&what < root + Length())) &&
       (capacity < count + 1))
     {
       /*
-        Special case: An element from this very array is pushed, and a 
+        Special case: An element from this very array is pushed, and a
 	reallocation is needed. This could cause the passed ref to the
 	element to be pushed to be read from deleted memory. Work
 	around this.
        */
-      int whatIndex = &what - root;
-      SetLengthUnsafe (count + 1);
+      size_t whatIndex = &what - root;
+      SetSizeUnsafe (count + 1);
       ElementHandler::Construct (root + count - 1, root[whatIndex]);
     }
     else
     {
-      SetLengthUnsafe (count + 1);
+      SetSizeUnsafe (count + 1);
       ElementHandler::Construct (root + count - 1, what);
     }
     return count - 1;
   }
 
-  /*
+  /**
    * Push a element onto the tail end of the array if not already present.
-   * Returns index of newly pushed element or index of already present element.
+   * \return Index of newly pushed element or index of already present element.
    */
-  int PushSmart (T const& what)
+  size_t PushSmart (T const& what)
   {
-    int const n = Find (what);
-    return (n < 0) ? Push (what) : n;
+    size_t const n = Find (what);
+    return (n == csArrayItemNotFound) ? Push (what) : n;
   }
 
   /// Pop an element from tail end of array.
@@ -596,33 +669,33 @@ public:
     CS_ASSERT (count > 0);
     T ret(root [count - 1]);
     ElementHandler::Destroy (root + count - 1);
-    SetLengthUnsafe (count - 1);
+    SetSizeUnsafe (count - 1);
     return ret;
   }
 
-  /// Return the top element but do not remove it.
+  /// Return the top element but do not remove it. (const)
   T const& Top () const
   {
     CS_ASSERT (count > 0);
     return root [count - 1];
   }
 
-  /// Return the top element but do not remove it.
+  /// Return the top element but do not remove it. (non-const)
   T& Top ()
   {
     CS_ASSERT (count > 0);
     return root [count - 1];
   }
 
-  /// Insert element 'item' before element 'n'.
-  bool Insert (int n, T const& item)
+  /// Insert element \c item before element \c n.
+  bool Insert (size_t n, T const& item)
   {
     if (n <= count)
     {
-      SetLengthUnsafe (count + 1); // Increments 'count' as a side-effect.
-      int const nmove = (count - n - 1);
+      SetSizeUnsafe (count + 1); // Increments 'count' as a side-effect.
+      size_t const nmove = (count - n - 1);
       if (nmove > 0)
-        memmove (root + n + 1, root + n, nmove * sizeof(T));
+	MemoryAllocator::MemMove (root, n+1, n, nmove);
       ElementHandler::Construct (root + n, item);
       return true;
     }
@@ -631,52 +704,56 @@ public:
   }
 
   /**
-   * Get the portion of the array between low and high inclusive
+   * Get the portion of the array between \c low and \c high inclusive.
    */
-  csArray<T> Section (int low, int high) const
+  csArray<T> Section (size_t low, size_t high) const
   {
-    CS_ASSERT (low >= 0 && high < count && high >= low);
+    CS_ASSERT (high < count && high >= low);
     csArray<T> sect (high - low + 1);
-    for (int i = low; i <= high; i++) sect.Push (root[i]);
+    for (size_t i = low; i <= high; i++) sect.Push (root[i]);
     return sect;
   }
 
   /**
    * Find an element based on some key, using a comparison function.
-   * The array must be sorted. Returns -1 if element does not exist.
+   * \return csArrayItemNotFound if not found, else the item index.
+   * \remarks The array must be sorted.
    */
   csArrayTemplate(K)
-  int FindSortedKey (csArrayCmpDecl(T,K) comparekey, int* candidate = 0) const
+  size_t FindSortedKey (csArrayCmpDecl(T,K) comparekey,
+                        size_t* candidate = 0) const
   {
-    int m = 0, l = 0, r = Length () - 1;
-    while (l <= r)
+    size_t m = 0, l = 0, r = Length ();
+    while (l < r)
     {
       m = (l + r) / 2;
       int cmp = csArrayCmpInvoke(comparekey, root[m]);
       if (cmp == 0)
       {
-        if (candidate) *candidate = -1;
+        if (candidate) *candidate = csArrayItemNotFound;
         return m;
       }
       else if (cmp < 0)
         l = m + 1;
       else
-        r = m - 1;
+        r = m;
     }
     if (candidate) *candidate = m;
-    return -1;
+    return csArrayItemNotFound;
   }
 
   /**
    * Insert an element at a sorted position, using an element comparison
-   * function.  Assumes array is already sorted.
+   * function.
+   * \return The index of the inserted item.
+   * \remarks The array must be sorted.
    */
-  int InsertSorted (const T& item,
+  size_t InsertSorted (const T& item,
     int (*compare)(T const&, T const&) = DefaultCompare,
-    int* equal_index = 0)
+    size_t* equal_index = 0)
   {
-    int m = 0, l = 0, r = Length () - 1;
-    while (l <= r)
+    size_t m = 0, l = 0, r = Length ();
+    while (l < r)
     {
       m = (l + r) / 2;
       int cmp = compare (root [m], item);
@@ -690,37 +767,48 @@ public:
       else if (cmp < 0)
         l = m + 1;
       else
-        r = m - 1;
+        r = m;
     }
-    if (r == m)
+    if ((m + 1) == r)
       m++;
-    if (equal_index) *equal_index = -1;
+    if (equal_index) *equal_index = csArrayItemNotFound;
     Insert (m, item);
     return m;
   }
 
-  /// Find a element in array and return its index (or -1 if not found).
-  int Find (T const& which) const
+  /**
+   * Find an element in array.
+   * \return csArrayItemNotFound if not found, else the item index.
+   * \warning Performs a slow linear search. For faster searching, sort the
+   *   array and then use FindSortedKey().
+   */
+  size_t Find (T const& which) const
   {
-    for (int i = 0 ; i < Length () ; i++)
+    for (size_t i = 0 ; i < Length () ; i++)
       if (root[i] == which)
         return i;
-    return -1;
+    return csArrayItemNotFound;
   }
 
+  /// An alias for Find() which may be considered more idiomatic by some.
+  size_t Contains(T const& which) const
+  { return Find(which); }
+
   /**
    * Given a pointer to an element in the array this function will return
-   * the index. Note that this function doesn't check if the returned
+   * the index. Note that this function does not check if the returned
    * index is actually valid. The caller of this function is responsible
    * for testing if the returned index is within the bounds of the array.
    */
-  int GetIndex (const T* which) const
+  size_t GetIndex (const T* which) const
   {
+    CS_ASSERT (which >= root);
+    CS_ASSERT (which < (root + count));
     return which-root;
   }
 
   /**
-   * Sort array.
+   * Sort array using a comparison function.
    */
   void Sort (int (*compare)(T const&, T const&) = DefaultCompare)
   {
@@ -729,13 +817,13 @@ public:
   }
 
   /**
-   * Clear entire vector.
+   * Clear entire array, releasing all allocated memory.
    */
   void DeleteAll ()
   {
     if (root)
     {
-      int i;
+      size_t i;
       for (i = 0 ; i < count ; i++)
         ElementHandler::Destroy (root + i);
       MemoryAllocator::Free (root);
@@ -749,18 +837,23 @@ public:
 
   /**
    * Truncate array to specified number of elements. The new number of
-   * elements cannot exceed the current number of elements. Use SetLength()
-   * for a more general way to enlarge the array.
+   * elements cannot exceed the current number of elements.
+   * \remarks Does not reclaim memory used by the array itself, though the
+   *   removed objects are destroyed. To reclaim the array's memory invoke
+   *   ShrinkBestFit(), or DeleteAll() if you want to release all allocated
+   *   resources.
+   * <p>
+   * \remarks The more general-purpose SetSize() method can also enlarge the
+   *   array.
    */
-  void Truncate (int n)
+  void Truncate (size_t n)
   {
-    CS_ASSERT(n >= 0);
     CS_ASSERT(n <= count);
     if (n < count)
     {
-      for (int i = n; i < count; i++)
+      for (size_t i = n; i < count; i++)
         ElementHandler::Destroy (root + i);
-      SetLengthUnsafe(n);
+      SetSizeUnsafe(n);
     }
   }
 
@@ -775,20 +868,44 @@ public:
   }
 
   /**
-   * Set vector capacity to approximately 'n' elements.  Never sets the
-   * capacity to fewer than the current number of elements in the array.  See
-   * Truncate() or SetLength() if you need to adjust the number of actual
-   * array elements.
+   * Return true if the array is empty.
+   * \remarks Rigidly equivalent to <tt>return GetSize() == 0</tt>, but more
+   *   idiomatic.
    */
-  void SetCapacity (int n)
+  bool IsEmpty() const
+  {
+    return GetSize() == 0;
+  }
+
+  /**
+   * Set vector capacity to approximately \c n elements.
+   * \remarks Never sets the capacity to fewer than the current number of
+   *   elements in the array.  See Truncate() or SetSize() if you need to
+   *   adjust the number of actual array elements.
+   */
+  void SetCapacity (size_t n)
   {
     if (n > Length ())
-      AdjustCapacity (n);
+      InternalSetCapacity (n);
+  }
+
+  /**
+   * Set vector capacity to at least \c n elements.
+   * \remarks Never sets the capacity to fewer than the current number of
+   *   elements in the array.  See Truncate() or SetSize() if you need to
+   *   adjust the number of actual array elements. This function will also
+   *   never shrink the current capacity.
+   */
+  void SetMinimalCapacity (size_t n)
+  {
+    if (n < Capacity ()) return;
+    if (n > Length ())
+      InternalSetCapacity (n);
   }
 
   /**
    * Make the array just as big as it needs to be. This is useful in cases
-   * where you know the array isn't going to be modified anymore in order
+   * where you know the array is not going to be modified anymore in order
    * to preserve memory.
    */
   void ShrinkBestFit ()
@@ -800,24 +917,31 @@ public:
     else if (count != capacity)
     {
       root = MemoryAllocator::Realloc (root, count, capacity, count);
-#     ifdef CS_MEMORY_TRACKER
+#ifdef CS_MEMORY_TRACKER
       UpdateMti (count-capacity, count);
-#     endif
+#endif
       capacity = count;
     }
   }
 
-  /// Delete element number 'n' from vector.
-  bool DeleteIndex (int n)
+  /**
+   * Delete an element from the array.
+   * return True if the indicated item index was valid, else false.
+   * \remarks Deletion speed is proportional to the size of the array and the
+   *   location of the element being deleted. If the order of the elements in
+   *   the array is not important, then you can instead use DeleteIndexFast()
+   *   for constant-time deletion.
+  */
+  bool DeleteIndex (size_t n)
   {
-    if (n >= 0 && n < count)
+    if (n < count)
     {
-      int const ncount = count - 1;
-      int const nmove = ncount - n;
+      size_t const ncount = count - 1;
+      size_t const nmove = ncount - n;
       ElementHandler::Destroy (root + n);
       if (nmove > 0)
-        memmove (root + n, root + n + 1, nmove * sizeof(T));
-      SetLengthUnsafe (ncount);
+	MemoryAllocator::MemMove (root, n, n+1, nmove);
+      SetSizeUnsafe (ncount);
       return true;
     }
     else
@@ -825,37 +949,89 @@ public:
   }
 
   /**
-   * Delete a given range (inclusive). This routine will clamp start and end
-   * to the size of the array.
+   * Delete an element from the array in constant-time, regardless of the
+   * array's size.
+   * return True if the indicated item index was valid, else false.
+   * \remarks This is a special version of DeleteIndex() which does not
+   *   preserve the order of the remaining elements. This characteristic allows
+   *   deletions to be performed in constant-time, regardless of the size of
+   *   the array.
    */
-  void DeleteRange (int start, int end)
+  bool DeleteIndexFast (size_t n)
   {
-    if (start >= count) return;
-    if (end < 0) return;
-    if (start < 0) start = 0;
-    if (end >= count) end = count-1;
-    int i;
-    for (i = start ; i < end ; i++)
-      ElementHandler::Destroy (root + i);
-
-    int const range_size = end-start+1;
-    int const ncount = count - range_size;
-    int const nmove = count - end - 1;
-    if (nmove > 0)
-      memmove (root + start, root + start + range_size, nmove * sizeof(T));
-    SetLengthUnsafe (ncount);
+    if (n < count)
+    {
+      size_t const ncount = count - 1;
+      size_t const nmove = ncount - n;
+      ElementHandler::Destroy (root + n);
+      if (nmove > 0)
+        MemoryAllocator::MemMove (root, n, ncount, 1);
+      SetSizeUnsafe (ncount);
+      return true;
+    }
+    else
+      return false;
   }
 
-  /// Delete the given element from vector.
+  /**
+   * Delete a given range (inclusive).
+   * \remarks Will clamp \c start and \c end to the array limits.
+   */
+  void DeleteRange (size_t start, size_t end)
+  {
+    if (start >= count) return;
+    // Treat 'csArrayItemNotFound' as invalid indices, do nothing.
+    // @@@ Assert that?
+    if (end == csArrayItemNotFound) return;
+    if (start == csArrayItemNotFound) return;//start = 0;
+    if (end >= count) end = count - 1;
+    size_t i;
+    for (i = start ; i <= end ; i++)
+      ElementHandler::Destroy (root + i);
+
+    size_t const range_size = end - start + 1;
+    size_t const ncount = count - range_size;
+    size_t const nmove = count - end - 1;
+    if (nmove > 0)
+      MemoryAllocator::MemMove (root, start, start + range_size, nmove);
+    SetSizeUnsafe (ncount);
+  }
+
+  /**
+   * Delete the given element from the array.
+   * \remarks Performs a linear search of the array to locate \c item, thus it
+   *   may be slow for large arrays.
+   */
   bool Delete (T const& item)
   {
-    int const n = Find (item);
-    if (n >= 0)
+    size_t const n = Find (item);
+    if (n != csArrayItemNotFound)
       return DeleteIndex (n);
     return false;
   }
 
-  /** Iterator for the Array object */
+  /**
+   * Delete the given element from the array.
+   * \remarks This is a special version of Delete() which does not
+   *   preserve the order of the remaining elements. This characteristic allows
+   *   deletions to be performed somewhat more quickly than by Delete(),
+   *   however the speed gain is largely mitigated by the fact that a linear
+   *   search is performed in order to locate \c item, thus this optimization
+   *   is mostly illusory.
+   * \deprecated The speed gain promised by this method is mostly illusory on
+   *   account of the linear search for the item. In many cases, it will be
+   *   faster to keep the array sorted, search for \c item using
+   *   FindSortedKey(), and then remove it using the plain DeleteIndex().
+   */
+  bool DeleteFast (T const& item)
+  {
+    size_t const n = Find (item);
+    if (n != csArrayItemNotFound)
+      return DeleteIndexFast (n);
+    return false;
+  }
+
+  /** Iterator for the Array<> class */
   class Iterator
   {
   public:
@@ -878,24 +1054,33 @@ public:
     /** Reset the array to the first element */
     void Reset()
     { currentelem = 0; }
+
   protected:
     Iterator(const csArray<T, ElementHandler>& newarray)
-	: currentelem(0), array(newarray)
-    { }
+	: currentelem(0), array(newarray) {}
     friend class csArray<T, ElementHandler>;
-    
+
   private:
-    int currentelem;
+    size_t currentelem;
     const csArray<T, ElementHandler>& array;
   };
 
-  /** Returns an Iterator which traverses the Array */
+  /** Returns an Iterator which traverses the array. */
   Iterator GetIterator() const
   { return Iterator(*this); }
+  
+  /// Check if this array has the exact same contents as \a other.
+  bool operator== (const csArray& other) const
+  {
+    if (other.GetSize() != GetSize()) return false;
+    for (size_t i = 0; i < GetSize(); i++)
+      if (Get (i) != other[i]) return false;
+    return true;
+  }
 };
 
 /**
- * Convenience class to make a version of csArray that does a
+ * Convenience class to make a version of csArray<> that does a
  * safe-copy in case of reallocation of the array. Useful for weak
  * references.
  */
@@ -907,12 +1092,12 @@ class csSafeCopyArray
 {
 public:
   /**
-   * Initialize object to hold initially 'ilimit' elements, and increase
-   * storage by 'ithreshold' each time the upper bound is exceeded.
+   * Initialize object to hold initially \c limit elements, and increase
+   * storage by \c threshold each time the upper bound is exceeded.
    */
-  csSafeCopyArray (int ilimit = 0, int ithreshold = 0)
+  csSafeCopyArray (size_t limit = 0, size_t threshold = 0)
   	: csArray<T, csArrayElementHandler<T>,
-		     csSafeCopyArrayMemoryAllocator<T> > (ilimit, ithreshold)
+		     csSafeCopyArrayMemoryAllocator<T> > (limit, threshold)
   {
   }
 };
@@ -921,4 +1106,6 @@ public:
 # define new CS_EXTENSIVE_MEMDEBUG_NEW
 #endif
 
+/** @} */
+
 #endif
