@@ -19,10 +19,15 @@
 #ifndef __CS_UTIL_CFGFILE_H__
 #define __CS_UTIL_CFGFILE_H__
 
+/**\file
+ * Implementation for iConfigFile with configurations stored in VFS files.
+ */
+
 #include "csextern.h"
+#include "csutil/array.h"
+#include "csutil/scf_implementation.h"
 #include "iutil/cfgfile.h"
 #include "iutil/vfs.h"
-#include "array.h"
 
 class csConfigNode;
 class csConfigIterator;
@@ -30,11 +35,10 @@ class csConfigIterator;
 /**
  * Configuration file which implements the iConfigFile SCF interface.
  */
-class CS_CSUTIL_EXPORT csConfigFile : public iConfigFile
+class CS_CRYSTALSPACE_EXPORT csConfigFile : 
+  public scfImplementation1<csConfigFile, iConfigFile>
 {
 public:
-  SCF_DECLARE_IBASE;
-
   /// Create a new configuration object from the given file.
   csConfigFile(const char *Filename = 0, iVFS* = 0);
   /// Create a new empty configuration object.
@@ -42,6 +46,9 @@ public:
   /// Delete this configuration.
   virtual ~csConfigFile();
 
+  /// Is the configuration object empty?
+  virtual bool IsEmpty() const;
+
   /// Get configuration file name.
   virtual const char *GetFileName () const;
 
