@@ -20,25 +20,31 @@
 #ifndef __CS_UTIL_EVOUTLET_H__
 #define __CS_UTIL_EVOUTLET_H__
 
+/**\file
+ * Implementation for iEventOutlet
+ */
+
 #include "csextern.h"
-#include "csunicode.h"
+#include "csutil/csunicode.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/weakref.h"
 #include "iutil/event.h"
 #include "iutil/csinput.h"
+
 struct iObjectRegistry;
 class csEventQueue;
 
 /**
  * A class which implements the iEventOutlet interface.
  */
-class CS_CSUTIL_EXPORT csEventOutlet : public iEventOutlet
+class CS_CRYSTALSPACE_EXPORT csEventOutlet : 
+  public scfImplementation1<csEventOutlet, iEventOutlet>
 {
 private:
-  /// The mask of events to allow from this plug
-  unsigned EnableMask;
   /// The event plug object
   iEventPlug *Plug;
   /// The owning event queue
-  csEventQueue *Queue;
+  csWeakRef<csEventQueue> Queue;
   /// The shared-object registry.
   iObjectRegistry* Registry;
   /// The shared keyboard driver.
@@ -53,7 +59,6 @@ private:
   iJoystickDriver* GetJoystickDriver();
 
 public:
-  SCF_DECLARE_IBASE;
 
   /// Initialize the outlet
   csEventOutlet (iEventPlug*, csEventQueue*, iObjectRegistry*);
@@ -66,14 +71,18 @@ public:
   virtual void Post (iEvent*);
   /// Put a keyboard event into event queue.
   virtual void Key (utf32_char codeRaw, utf32_char codeCooked, bool iDown);
-  /// Put a mouse event into event queue.
+  /// Put a mouse event into event queue (old interface)
   virtual void Mouse (int iButton, bool iDown, int x, int y);
+  /// Put a mouse event into event queue (new interface)
+  virtual void Mouse (uint iNumber, int iButton, bool iDown, 
+		      const int32 *axes, uint numAxes);
   /// Put a joystick event into event queue.
-  virtual void Joystick (int iNumber, int iButton, bool iDown, int x, int y);
+  virtual void Joystick (uint iNumber, int iButton, bool iDown, 
+    const int32 *axes, uint numAxes);
   /// Put a broadcast event into event queue.
-  virtual void Broadcast (int iCode, void *iInfo);
+  virtual void Broadcast (csEventID iName, intptr_t iInfo);
   /// Broadcast a event to all plugins
-  virtual void ImmediateBroadcast (int iCode, void *iInfo);
+  virtual void ImmediateBroadcast (csEventID iName, intptr_t iInfo);
 };
 
 #endif // __CS_UTIL_EVOUTLET_H__
