@@ -19,6 +19,10 @@
 #ifndef __CS_FLAGS_H__
 #define __CS_FLAGS_H__
 
+/**\file
+ * Set of flags
+ */
+
 #include "csextern.h"
 
 /**
@@ -67,7 +71,7 @@ public:
    * but only the flags you specify in the mask will be affected.
    */
   void Set (uint32 mask, uint32 value)
-  { flags = (flags & ~mask) | value; }
+  { flags = (flags & ~mask) | (value & mask); }
 
   /**
    * Set all flags with the given value.
