@@ -19,49 +19,233 @@
 #ifndef __CS_UTIL_HASH_H__
 #define __CS_UTIL_HASH_H__
 
+/**\file
+ * A generic hash table
+ */
+
 #include "csextern.h"
-#include "array.h"
+#include "csutil/array.h"
+#include "csutil/comparator.h"
+
+
+/**\addtogroup util_containers
+ * @{ */
 
 /**
  * Compute a hash key for a null-terminated string.
- *
+ * <p>
  * Note that these keys are non-unique; some dissimilar strings may generate
  * the same key. For unique keys, see csStringSet.
  */
-CS_CSUTIL_EXPORT unsigned int csHashCompute (char const*);
+CS_CRYSTALSPACE_EXPORT unsigned int csHashCompute (char const*);
 
 /**
  * Compute a hash key for a string of a given length.
- *
+ * <p>
  * Note that these keys are non-unique; some dissimilar strings may generate
  * the same key. For unique keys, see csStringSet.
  */
-CS_CSUTIL_EXPORT unsigned int csHashCompute (char const*, int length);
+CS_CRYSTALSPACE_EXPORT unsigned int csHashCompute (char const*, size_t length);
 
 /**
- * A hash key handler for integral types and types that can be casted to such.
+ * Template for hash value computing.
+ * Expects that T has a method 'uint GetHash() const'.
  */
 template <class T>
-class csIntegralHashKeyHandler
+class csHashComputer
 {
 public:
-  static unsigned int ComputeHash (const T& key)
+  /// Compute a hash value for \a key.
+  static uint ComputeHash (const T& key)
   {
-    return (unsigned int)key;
-  }
-
-  static bool CompareKeys (const T& key1, const T& key2)
-  {
-    return (key1 == key2);
+    return key.GetHash();
   }
 };
 
+/**
+ * Template for hash value computing, suitable for integral types and types 
+ * that can be casted to such.
+ */
+template <class T>
+class csHashComputerIntegral
+{
+public:
+  /// Compute a hash value for \a key.
+  static uint ComputeHash (const T& key)
+  {
+    return (uintptr_t)key;
+  }
+};
+
+//@{
+/**
+ * csHashComputer<> specialization for an integral type.
+ */
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<void*> : public csHashComputerIntegral<void*> {};
+  
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<int> : public csHashComputerIntegral<int> {}; 
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<unsigned int> : 
+  public csHashComputerIntegral<unsigned int> {}; 
+    
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<long> : public csHashComputerIntegral<long> {}; 
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<unsigned long> : 
+  public csHashComputerIntegral<unsigned long> {}; 
+
+#if (CS_LONG_SIZE < 8)    
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<longlong> : 
+  public csHashComputerIntegral<longlong> {}; 
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<ulonglong> : 
+  public csHashComputerIntegral<ulonglong> {}; 
+#endif
+    
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<float>
+{
+public:
+  /// Compute a hash value for \a key.
+  static uint ComputeHash (float key)
+  {
+    union
+    {
+      float f;
+      uint u;
+    } float2uint;
+    float2uint.f = key;
+    return float2uint.u;
+  }
+};
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<double>
+{
+public:
+  /// Compute a hash value for \a key.
+  static uint ComputeHash (double key)
+  {
+    union
+    {
+      double f;
+      uint u;
+    } double2uint;
+    double2uint.f = key;
+    return double2uint.u;
+  }
+};
+//@}
+
+/**
+ * A helper template to use pointers as keys for hashes.
+ */
+template <typename T>
+class csPtrKey
+{
+  T* ptr;
+public:
+  csPtrKey () : ptr(0) {}
+  csPtrKey (T* ptr) : ptr(ptr) {}
+  csPtrKey (csPtrKey const& other) : ptr (other.ptr) {}
+
+  uint GetHash () const { return (uintptr_t)ptr; }
+  inline friend bool operator < (const csPtrKey& r1, const csPtrKey& r2)
+  { return r1.ptr < r2.ptr; }
+  operator T* () const
+  { return ptr; }
+  T* operator -> () const
+  { return ptr; }
+  csPtrKey& operator = (csPtrKey const& other)
+  { ptr = other.ptr; return *this; }
+};
+
+/**
+ * Template that can be used as a base class for hash computers for 
+ * string types (must support cast to const char*).
+ * Example:
+ * \code
+ * CS_SPECIALIZE_TEMPLATE csHashComputer<MyString> : 
+ *   public csHashComputerString<MyString> {};
+ * \endcode
+ */
+template <class T>
+class csHashComputerString
+{
+public:
+  static uint ComputeHash (const T& key)
+  {
+    return csHashCompute ((const char*)key);
+  }
+};
+
+/**
+ * csHashComputer<> specialization for strings that uses csHashCompute().
+ */
+CS_SPECIALIZE_TEMPLATE
+class csHashComputer<const char*> : public csHashComputerString<const char*> {};
+
+/**
+ * Template that can be used as a base class for hash computers for POD 
+ * structs.
+ * Example:
+ * \code
+ * CS_SPECIALIZE_TEMPLATE csHashComputer<MyStruct> : 
+ *   public csHashComputerStruct<MyStruct> {};
+ * \endcode
+ */
+template <class T>
+class csHashComputerStruct
+{
+public:
+  static uint ComputeHash (const T& key)
+  {
+    return csHashCompute ((char*)&key, sizeof (T));
+  }
+};
+
+/**
+ * This is a simple helper class to make a copy of a const char*.
+ * This can be used to have a hash that makes copies of the keys.
+ * \deprecated csString can also be used for hash keys.
+ */
+class csStrKey
+{
+private:
+  char* str;
+
+public:
+  csStrKey () { str = 0; }
+  csStrKey (const char* s) { str = csStrNew (s); }
+  csStrKey (const csStrKey& c) { str = csStrNew (c.str); }
+  ~csStrKey () { delete[] str; }
+  csStrKey& operator=(const csStrKey& o)
+  {
+    delete[] str; str = csStrNew (o.str);
+    return *this;
+  }
+  operator const char* () const { return str; }
+  uint GetHash() const { return csHashCompute (str); }
+};
+
+/**
+ * csComparator<> specialization for csStrKey that uses strcmp().
+ */
+CS_SPECIALIZE_TEMPLATE
+class csComparator<csStrKey, csStrKey> : public csComparatorString<csStrKey> {};
+
 /**
  * A generic hash table class,
  * which grows dynamically and whose buckets are unsorted arrays.
+ * The hash value of a key is computed using csHashComputer<>, two keys are
+ * compared using csComparator<>. You need to provide appropriate 
+ * specializations of those templates if you want use non-integral types 
+ * (other than const char*, csStrKey and csString for which appropriate 
+ * specializations are already provided) or special hash algorithms. 
  */
-template <class T, class K = unsigned int, 
-  class KeyHandler = csIntegralHashKeyHandler<K> > 
+template <class T, class K = unsigned int> 
 class csHash
 {
 protected:
@@ -75,17 +259,17 @@ protected:
   };
   csArray< csArray<Element> > Elements;
 
-  int Modulo;
+  size_t Modulo;
 
 private:
-  int InitModulo;
-  int GrowRate;
-  int MaxSize;
-  int Size;
+  size_t InitModulo;
+  size_t GrowRate;
+  size_t MaxSize;
+  size_t Size;
 
   void Grow ()
   {
-    static const int Primes[] =
+    static const size_t Primes[] =
     {
       53,         97,         193,       389,       769, 
       1543,       3079,       6151,      12289,     24593,
@@ -95,27 +279,27 @@ private:
       1610612741, 0
     };
 
-    const int *p;
-    int elen = Elements.Length ();
+    const size_t *p;
+    size_t elen = Elements.Length ();
     for (p = Primes; *p && *p <= elen; p++) ;
     Modulo = *p;
     CS_ASSERT (Modulo);
 
-    Elements.SetLength (Modulo);
+    Elements.SetLength(Modulo, csArray<Element>(0, MIN(Modulo / GrowRate, 8)));
 
-    for (int i = 0; i < elen; i++)
+    for (size_t i = 0; i < elen; i++)
     {
       csArray<Element>& src = Elements[i];
-      int slen = src.Length ();
-      for (int j = slen - 1; j >= 0; j--)
+      size_t slen = src.Length ();
+      for (size_t j = slen; j > 0; j--)
       {
-        const Element& srcElem = src[j];
+        const Element& srcElem = src[j - 1];
         csArray<Element>& dst = 
-	  Elements.Get (KeyHandler::ComputeHash (srcElem.key) % Modulo);
+	  Elements.Get (csHashComputer<K>::ComputeHash (srcElem.key) % Modulo);
         if (&src != &dst)
         {
           dst.Push (srcElem);
-          src.DeleteIndex (j);
+          src.DeleteIndex (j - 1);
         }
       }
     }
@@ -125,21 +309,22 @@ public:
   /**
    * Construct a hash table with an array of the given size,
    * which for optimisation reasons should be a prime number.
-   * 
+   * <p>
    * Grow_rate is the rate at which the hash table grows:
    * Size doubles once there are size/grow_rate collisions.
    * It will not grow after it reaches max_size.
-   *
+   * <p>
    * Here are a few primes: 7, 11, 19, 29, 59, 79, 101, 127, 151, 199, 251,
    * 307, 401, 503, 809, 1009, 1499, 2003, 3001, 5003, 12263, 25247, 36923,
    * 50119, 70951, 90313, 104707.
+   * <p>
    * For a bigger list go to http://www.utm.edu/research/primes/
    */
-  csHash (int size = 257, int grow_rate = 64, int max_size = 20000)
+  csHash (size_t size = 23, size_t grow_rate = 5, size_t max_size = 20000)
     : Elements (size), Modulo (size), InitModulo (size),
-      GrowRate (grow_rate), MaxSize (max_size), Size (0)
+      GrowRate (MIN (grow_rate, size)), MaxSize (max_size), Size (0)
   {
-    Elements.SetLength (size, csArray<Element> (0, 8));
+    Elements.SetLength (size, csArray<Element> (0, MIN (size / GrowRate, 8)));
   }
 
   /// Copy constructor.
@@ -147,11 +332,17 @@ public:
     Modulo (o.Modulo), InitModulo (o.InitModulo),
     GrowRate (o.GrowRate), MaxSize (o.MaxSize), Size (o.Size) {}
 
-  /// Add an element to the hash table.
+  /**
+   * Add an element to the hash table.
+   * \remarks If `key' is already present, does NOT replace the existing value,
+   *   but merely adds `value' as an additional value of `key'. To retrieve all
+   *   values for a given key, use GetAll(). If you instead want to replace an
+   *   existing value for 'key', use PutUnique().
+   */
   void Put (const K& key, const T &value)
   {
     csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
     values.Push (Element (key, value));
     Size++;
     if (values.Length () > Elements.Length () / GrowRate
@@ -162,28 +353,28 @@ public:
   csArray<T> GetAll (const K& key) const
   {
     const csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
     csArray<T> ret (values.Length () / 2);
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
     {
       const Element& v = values[i];
-      if (KeyHandler::CompareKeys (v.key, key)) 
+      if (csComparator<K, K>::Compare (v.key, key) == 0) 
 	ret.Push (v.value);
     }
     return ret;
   }
 
   /// Add an element to the hash table, overwriting if the key already exists.
-  void PutFirst (const K& key, const T &value)
+  void PutUnique (const K& key, const T &value)
   {
     csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
     {
       Element& v = values[i];
-      if (KeyHandler::CompareKeys (v.key, key))
+      if (csComparator<K, K>::Compare (v.key, key) == 0)
       {
         v.value = value;
         return;
@@ -196,19 +387,35 @@ public:
      && Elements.Length () < MaxSize) Grow ();
   }
 
+  /**
+   * Add an element to the hash table, overwriting if the key already exists.
+   * \deprecated Use PutUnique() instead.
+   */
+  CS_DEPRECATED_METHOD void PutFirst (const K& key, const T &value)
+  {
+    PutUnique(key, value);
+  }
+
   /// Returns whether at least one element matches the given key.
-  bool In (const K& key) const
+  bool Contains (const K& key) const
   {
     const csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
-      if (KeyHandler::CompareKeys (values[i].key, key)) 
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
+      if (csComparator<K, K>::Compare (values[i].key, key) == 0) 
 	return true;
-
     return false;
   }
 
+  /**
+   * Returns whether at least one element matches the given key.
+   * \remarks This is rigidly equivalent to Contains(key), but may be
+   *   considered more idiomatic by some.
+   */
+  bool In (const K& key) const
+  { return Contains(key); }
+
   /**
    * Get a pointer to the first element matching the given key, 
    * or 0 if there is none.
@@ -216,12 +423,12 @@ public:
   const T* GetElementPointer (const K& key) const
   {
     const csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
     {
       const Element& v = values[i];
-      if (KeyHandler::CompareKeys (v.key, key))
+      if (csComparator<K, K>::Compare (v.key, key) == 0)
 	return &v.value;
     }
 
@@ -235,12 +442,12 @@ public:
   T* GetElementPointer (const K& key)
   {
     csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
     {
       Element& v = values[i];
-      if (KeyHandler::CompareKeys (v.key, key))
+      if (csComparator<K, K>::Compare (v.key, key) == 0)
 	return &v.value;
     }
 
@@ -254,12 +461,12 @@ public:
   const T& Get (const K& key, const T& fallback) const
   {
     const csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
     {
       const Element& v = values[i];
-      if (KeyHandler::CompareKeys (v.key, key))
+      if (csComparator<K, K>::Compare (v.key, key) == 0)
 	return v.value;
     }
 
@@ -273,12 +480,12 @@ public:
   T& Get (const K& key, T& fallback)
   {
     csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    const int len = values.Length ();
-    for (int i = 0; i < len; ++i)
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    const size_t len = values.Length ();
+    for (size_t i = 0; i < len; ++i)
     {
       Element& v = values[i];
-      if (KeyHandler::CompareKeys (v.key, key))
+      if (csComparator<K, K>::Compare (v.key, key) == 0)
 	return v.value;
     }
 
@@ -289,25 +496,31 @@ public:
   void DeleteAll ()
   {
     Elements.SetLength (Modulo = InitModulo);
-    int elen = Elements.Length ();
-    for (int i = 0; i < elen; i++)
+    size_t elen = Elements.Length ();
+    for (size_t i = 0; i < elen; i++)
       Elements[i].Empty ();
     Size = 0;
   }
 
+  /// Delete all the elements. (Idiomatic alias for DeleteAll().)
+  void Empty() { DeleteAll(); }
+
   /// Delete all the elements matching the given key.
   bool DeleteAll (const K& key)
   {
     bool ret = false;
     csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    for (int i = values.Length () - 1; i >= 0; i--)
-      if (KeyHandler::CompareKeys (values[i].key, key))
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    for (size_t i = values.Length (); i > 0; i--)
+    {
+      const size_t idx = i - 1;
+      if (csComparator<K, K>::Compare (values[idx].key, key) == 0)
       {
-        values.DeleteIndex (i);
+	values.DeleteIndexFast (idx);
         ret = true;
         Size--;
       }
+    }
     return ret;
   }
   
@@ -316,48 +529,62 @@ public:
   {
     bool ret = false;
     csArray<Element> &values = 
-      Elements[KeyHandler::ComputeHash (key) % Modulo];
-    for (int i = values.Length () - 1; i >= 0; i--)
-      if (KeyHandler::CompareKeys (values[i].key, key) && 
-	(values[i].value == value))
+      Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
+    for (size_t i = values.Length (); i > 0; i--)
+    {
+      const size_t idx = i - 1;
+      if ((csComparator<K, K>::Compare (values[idx].key, key) == 0) && 
+	(values[idx].value == value))
       {
-        values.DeleteIndex (i);
+        values.DeleteIndexFast (idx);
         ret = true;
         Size--;
       }
+    }
     return ret;
   }
 
   /// Get the number of elements in the hash.
-  int GetSize () const
+  size_t GetSize () const
   {
     return Size;
   }
 
+  /**
+   * Return true if the hash is empty.
+   * \remarks Rigidly equivalent to <tt>return GetSize() == 0</tt>, but more
+   *   idiomatic.
+   */
+  bool IsEmpty() const
+  {
+    return GetSize() == 0;
+  }
+
   /// An iterator class for the hash.
   class Iterator
   {
   private:
-    const csHash<T, K, KeyHandler>* hash;
+    const csHash<T, K>* hash;
     const K key;
-    int bucket, size, element;
+    size_t bucket, size, element;
 
     void Seek ()
     {
       while ((element < size) && 
-        ! KeyHandler::CompareKeys (hash->Elements[bucket][element].key, key))
+        (csComparator<K, K>::Compare (hash->Elements[bucket][element].key, 
+	key) != 0))
           element++;
     }
 
   protected:
-    Iterator (const csHash<T, K, KeyHandler>* hash0, const K& key0) :
+    Iterator (const csHash<T, K>* hash0, const K& key0) :
       hash(hash0),
       key(key0), 
-      bucket(KeyHandler::ComputeHash(key) % hash->Modulo),
+      bucket(csHashComputer<K>::ComputeHash (key) % hash->Modulo),
       size(hash->Elements[bucket].Length ())
       { Reset (); }
 
-    friend class csHash;
+    friend class csHash<T, K>;
   public:
     /// Copy constructor.
     Iterator (const Iterator &o) :
@@ -402,8 +629,8 @@ public:
   class GlobalIterator
   {
   private:
-    const csHash<T, K, KeyHandler> *hash;
-    int bucket, size, element;
+    const csHash<T, K> *hash;
+    size_t bucket, size, element;
 
     void Zero () { bucket = element = 0; }
     void Init () { size = hash->Elements[bucket].Length (); }
@@ -425,14 +652,14 @@ public:
     }
 
   protected:
-    GlobalIterator (const csHash<T, K, KeyHandler> *hash0) : hash (hash0) 
+    GlobalIterator (const csHash<T, K> *hash0) : hash (hash0) 
     { 
       Zero (); 
       Init (); 
       FindItem ();
     }
 
-    friend class csHash;
+    friend class csHash<T, K>;
   public:
     /// Copy constructor.
     GlobalIterator (const Iterator &o) :
@@ -479,6 +706,7 @@ public:
       return ret;
     }
 
+    /// Get the next element's value and key, don't move the iterator.
     const T& NextNoAdvance (K &key)
     {
       key = hash->Elements[bucket][element].key;
@@ -499,7 +727,7 @@ public:
 
   /// Delete the element pointed by the iterator. This is safe for this
   /// iterator, not for the others.
-  void DeleteElement (GlobalIterator iterator)
+  void DeleteElement (GlobalIterator& iterator)
   {
     Elements[iterator.bucket].DeleteIndex(iterator.element);
     Size--;
@@ -510,8 +738,8 @@ public:
   /**
    * Return an iterator for the hash, to iterate only over the elements
    * with the given key.
-   * Modifying the hash (except with DeleteElement) while you have open 
-   * iterators will cause undefined behaviour.
+   * \warning Modifying the hash (except with DeleteElement()) while you have
+   *   open iterators will result in undefined behaviour.
    */
   Iterator GetIterator (const K& key) const
   {
@@ -520,8 +748,8 @@ public:
 
   /**
    * Return an iterator for the hash, to iterate over all elements.
-   * Modifying the hash (except with DeleteElement) while you have open 
-   * iterators will cause undefined behaviour.
+   * \warning Modifying the hash (except with DeleteElement()) while you have
+   *   open iterators will result in undefined behaviour.
    */
   GlobalIterator GetIterator () const
   {
@@ -529,114 +757,6 @@ public:
   }
 };
 
-/**
- * This class implements a basic set for objects.
- * You can basically use this to test for the occurrence
- * of some object quickly.
- */
-template <class T, class KeyHandler = csIntegralHashKeyHandler<T> > 
-class csSet
-{
-private:
-  csHash<T, T, KeyHandler> map;
-
-public:
-  /// An iterator class for the set.
-  class Iterator : public csHash<T, T, KeyHandler>::Iterator
-  {
-  protected:
-    Iterator () {}
-  public:
-  };
-  /// An iterator class for the set.
-  class GlobalIterator : public csHash<T, T, KeyHandler>::GlobalIterator
-  {
-  protected:
-    GlobalIterator () {}
-    GlobalIterator (const csSet<T, KeyHandler> *set0) : 
-      csHash<T, T, KeyHandler>::GlobalIterator(&set0->map)
-    { }
-
-  public:
-    friend class csSet;
-  };
-  friend class GlobalIterator;
-
-  /**
-   * Construct a new empty set.
-   * The given size will be passed to the hashmap.
-   */
-  csSet (int size = 257, int grow_rate = 64, int max_size = 20000)
-  	: map (size, grow_rate, max_size)
-  {
-  }
-
-  /**
-   * Add an object to this set.
-   * This will do nothing if the object is already present.
-   */
-  void Add (const T& object)
-  {
-    if (In (object)) return;
-    AddNoTest (object);
-  }
-
-  /**
-   * Add an object to this set.
-   * This function does not test if the object is already
-   * there. This is used for efficiency reasons. But use
-   * with care!
-   */
-  void AddNoTest (const T& object)
-  {
-    map.Put (object, object);
-  }
-
-  /**
-   * Test if an object is in this set.
-   */
-  bool In (const T& object) const
-  {
-    return map.In (object);
-  }
-
-  /**
-   * Delete all elements in the set.
-   */
-  void DeleteAll ()
-  {
-    map.DeleteAll ();
-  }
-
-  /**
-   * Delete an object from the set. This function
-   * does nothing if the object is not in the set.
-   * Return true if the object was present.
-   */
-  bool Delete (const T& object)
-  {
-    return map.Delete (object, object);
-  }
-
-  /// Get the number of elements in the set.
-  int GetSize () const
-  {
-    return map.GetSize ();
-  }
-
-  /// Return the hash map for this hash set.
-  csHash<T, T, KeyHandler>* GetHash () { return &map; }
-
-  /**
-   * Return an iterator for the hash set, to iterate over all elements.
-   * Modifying the set while you have open iterators will cause undefined 
-   * behaviour.
-   */
-  GlobalIterator GetIterator () const
-  {
-    return GlobalIterator(this);
-  }
-};
-
+/** @} */
 
 #endif
