@@ -27,7 +27,7 @@
 //-----------------------------------------------------------------------------
 
 #include "csextern.h"
-#include "refarr.h"
+#include "csutil/refarr.h"
 #include "iutil/object.h"
 
 /**\file 
@@ -49,9 +49,9 @@ public:
   {
   }
 
-  int GetIndexByName (const char* name) const
+  size_t GetIndexByName (const char* name) const
   {
-    int i;
+    size_t i;
     for (i = 0 ; i < this->Length () ; i++) // see *1*
     {
       T* o = (*this)[i];
@@ -59,13 +59,13 @@ public:
       if (n && !strcmp (n, name))
         return i;
     }
-    return -1;
+    return csArrayItemNotFound;
   }
 
   T* FindByName (const char* name) const
   {
-    int i = GetIndexByName (name);
-    if (i != -1)
+    size_t i = GetIndexByName (name);
+    if (i != csArrayItemNotFound)
       return (*this)[i];
     else
       return 0;
