@@ -19,18 +19,25 @@
 #ifndef __CS_PLUGLDR_H__
 #define __CS_PLUGLDR_H__
 
+/**\file
+ * Plugin loader
+ */
+
 #include "csextern.h"
-#include "parray.h"
-#include "util.h"
+#include "csutil/parray.h"
+#include "csutil/ref.h"
+#include "csutil/scf.h"
 
 struct iObjectRegistry;
 
-/// \internal Helper class for csPluginLoader.
+/** \internal
+ * Helper class for csPluginLoader.
+ */
 struct csPluginLoadRec
 {
   char* Tag;
   char* ClassID;
-  iBase* plugin;
+  csRef<iBase> plugin;
 
   csPluginLoadRec (const char* iTag, const char* iClassID)
   { Tag = csStrNew (iTag); ClassID = csStrNew (iClassID); }
@@ -38,13 +45,15 @@ struct csPluginLoadRec
   { delete [] ClassID; delete [] Tag; }
 };
 
-/// \internal Helper class for csPluginLoader.
-class CS_CSUTIL_EXPORT csPluginList : public csPDelArray<csPluginLoadRec>
+/** \internal
+ * Helper class for csPluginLoader.
+ */
+class CS_CRYSTALSPACE_EXPORT csPluginList : public csPDelArray<csPluginLoadRec>
 {
 public:
   bool Sort (iObjectRegistry* object_reg);
 private:
-  bool RecurseSort (iObjectRegistry*, int row, int *order, int *loop,
+  bool RecurseSort (iObjectRegistry*, size_t row, size_t* order, size_t* loop,
     bool *matrix);
 };
 
@@ -52,7 +61,7 @@ private:
  * This utility class helps to load plugins based on request,
  * config file, and commandline.
  */
-class CS_CSUTIL_EXPORT csPluginLoader
+class CS_CRYSTALSPACE_EXPORT csPluginLoader
 {
   friend class csPluginList;
 
