@@ -20,32 +20,31 @@
 #define __CS_IENGINE_FVIEW_H__
 
 /**\file
+ * Frustum view
  */
 /**
  * \addtogroup engine3d_vis
  * @{ */
  
+#include "csutil/ref.h"
 #include "csutil/scf.h"
-#include "csgeom/plane3.h"
-#include "csgeom/transfrm.h"
-#include "csgeom/box.h"
+
+#include "csgeom/frustum.h"
+
 #include "iengine/shadows.h"
 
+
 struct iFrustumView;
 struct iMeshWrapper;
 class csFrustum;
 class csFrustumContext;
-class csObject;
-class csOctreeNode;
-
-
-SCF_VERSION (iFrustumViewUserdata, 0, 0, 1);
 
 /**
  * User data which can be attached to iFrustumView.
  */
-struct iFrustumViewUserdata : public iBase
+struct iFrustumViewUserdata : public virtual iBase
 {
+  SCF_INTERFACE(iFrustumViewUserdata,2,0,0);
 };
 
 /**
@@ -134,14 +133,15 @@ public:
   bool IsMirrored () { return mirror; }
 };
 
-SCF_VERSION (iFrustumView, 0, 4, 1);
 
 /**
  * This structure represents all information needed for the frustum
  * visibility calculator.
  */
-struct iFrustumView : public iBase
+struct iFrustumView : public virtual iBase
 {
+  SCF_INTERFACE(iFrustumView,2,0,0);
+
   /// Get the current frustum context.
   virtual csFrustumContext* GetFrustumContext () const = 0;
   /**
@@ -178,8 +178,6 @@ struct iFrustumView : public iBase
   virtual float GetRadius () const = 0;
   /// Get the squared radius.
   virtual float GetSquaredRadius () const = 0;
-  /// Return true if shadowing for things is enabled.
-  virtual bool ThingShadowsEnabled () = 0;
   /// Check if a mask corresponds with the shadow mask.
   virtual bool CheckShadowMask (unsigned int mask) = 0;
   /// Check if a mask corresponds with the process mask.
@@ -192,9 +190,6 @@ struct iFrustumView : public iBase
   virtual void SetUserdata (iFrustumViewUserdata* data) = 0;
   /// Get userdata.
   virtual iFrustumViewUserdata* GetUserdata () = 0;
-
-  /// Create a new empty shadow block.
-  virtual csPtr<iShadowBlock> CreateShadowBlock () = 0;
 };
 
 /** @} */
