@@ -20,13 +20,13 @@
 #define __CS_IENGINE_HALO_H__
 
 /**\file
+ * Halo interfaces
  */
 /**
  * \addtogroup engine3d_light
  * @{ */
 
 #include "csutil/scf.h"
-#include "csutil/flags.h"
 
 struct iMaterialWrapper;
 
@@ -43,26 +43,26 @@ enum csHaloType
   cshtFlare
 };
 
-SCF_VERSION (iBaseHalo, 0, 0, 1);
-
 /**
  * This is the basic interface for all types of halos.
  */
-struct iBaseHalo : public iBase
+struct iBaseHalo : public virtual iBase
 {
+  SCF_INTERFACE(iBaseHalo,2,0,0);
   /// Set intensity.
   virtual void SetIntensity (float i) = 0;
   /// Get intensity.
   virtual float GetIntensity () = 0;
+  /// Get halo type.
+  virtual csHaloType GetType () = 0;
 };
 
-SCF_VERSION (iCrossHalo, 0, 0, 1);
-
 /**
  * This is a halo which resembles a cross.
  */
-struct iCrossHalo : public iBase
+struct iCrossHalo : public virtual iBase
 {
+  SCF_INTERFACE(iCrossHalo,2,0,0);
   /// Set intensity factor.
   virtual void SetIntensityFactor (float i) = 0;
   /// Get intensity factor.
@@ -73,13 +73,12 @@ struct iCrossHalo : public iBase
   virtual float GetCrossFactor () = 0;
 };
 
-SCF_VERSION (iNovaHalo, 0, 0, 1);
-
 /**
  * This is a halo which resembles a nova.
  */
-struct iNovaHalo : public iBase
+struct iNovaHalo : public virtual iBase
 {
+  SCF_INTERFACE(iNovaHalo,2,0,0);
   /// Set random seed for generating the halo.
   virtual void SetRandomSeed (int s) = 0;
   /// Get random seed.
@@ -94,13 +93,12 @@ struct iNovaHalo : public iBase
   virtual float GetRoundnessFactor () = 0;
 };
 
-SCF_VERSION (iFlareHalo, 0, 0, 1);
-
 /**
  * This is a halo which resembles a (solar) flare.
  */
-struct iFlareHalo : public iBase
+struct iFlareHalo : public virtual iBase
 {
+  SCF_INTERFACE(iFlareHalo,2,0,0);
   /**
    * Add a visual component to the flare.
    * give position, size, image and mixmode.
