@@ -23,64 +23,48 @@
 #include "iutil/strset.h"
 
 /**\file
+ * Material interfaces
  */
 /**
  * \addtogroup engine3d_textures
  * @{ */
- 
-class csMaterialWrapper;
+
+
 struct iMaterial;
-struct iMaterialHandle;
 struct iTextureManager;
 struct iTextureWrapper;
 struct iObject;
 
-SCF_VERSION (iMaterialWrapper, 0, 0, 5);
-
 /**
  * A material wrapper is an engine-level object that wraps around an actual
- * material (iMaterialHandle). Every material in the engine is represented
- * by a material wrapper, which keeps the pointer to the material handle, its
- * name, and possibly the base material object that was registered to create
- * the material handle.
- * <p>
+ * material (iMaterial). Every material in the engine is represented by a 
+ * material wrapper, which keeps the pointer to the material and its name, and 
+ * possibly the base material object.
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEngine::CreateMaterial()
- *   <li>iMaterialList::NewMaterial()
- *   </ul>
+ * - iEngine::CreateMaterial()
+ * - iMaterialList::NewMaterial()
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iEngine::FindMaterial()
- *   <li>iMaterialList::Get()
- *   <li>iMaterialList::FindByName()
- *   <li>iLoaderContext::FindMaterial()
- *   <li>iLoaderContext::FindNamedMaterial()
- *   <li>iMeshObject::GetMaterialWrapper()
- *   <li>Various state interfaces for mesh objects.
- *   </ul>
+ * - iEngine::FindMaterial()
+ * - iMaterialList::Get()
+ * - iMaterialList::FindByName()
+ * - iLoaderContext::FindMaterial()
+ * - iLoaderContext::FindNamedMaterial()
+ * - iMeshObject::GetMaterialWrapper()
+ * - Various state interfaces for mesh objects.
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>iEngine
- *   <li>Mesh objects
- *   </ul>
+ * - iEngine
+ * - Mesh objects
  */
-struct iMaterialWrapper : public iBase
+struct iMaterialWrapper : public virtual iBase
 {
+  SCF_INTERFACE (iMaterialWrapper, 0, 0, 5);
+
   /// Get the iObject for this material.
   virtual iObject *QueryObject() = 0;
 
-  /// Create a clone this material wrapper, using the same material handle
-  virtual iMaterialWrapper *Clone () const = 0;
-
-  /**
-   * Change the material handle. Note: This will also change the base
-   * material to 0.
-   */
-  virtual void SetMaterialHandle (iMaterialHandle *mat) = 0;
-  /// Get the material handle.
-  virtual iMaterialHandle* GetMaterialHandle () = 0;
-
   /**
    * Change the base material. Note: The changes will not be visible until
    * you re-register the material.
@@ -89,9 +73,6 @@ struct iMaterialWrapper : public iBase
   /// Get the original material.
   virtual iMaterial* GetMaterial () = 0;
 
-  /// Register the material with the texture manager
-  virtual void Register (iTextureManager *txtmng) = 0;
-
   /**
    * Visit this material. This should be called by the engine right
    * before using the material. It will call Visit() on all textures
@@ -114,11 +95,9 @@ SCF_VERSION (iMaterialEngine, 0, 0, 2);
  * that implements iMaterial you can query for iMaterialEngine.
  * So this interface basically augments iMaterial with engine specific
  * features.
- * <p>
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE() from iMaterial
- *   </ul>
+ *   - SCF_QUERY_INTERFACE() from iMaterial
  */
 struct iMaterialEngine : public iBase
 {
@@ -130,7 +109,7 @@ struct iMaterialEngine : public iBase
   /**
    * Get a texture by name.
    */
-  virtual iTextureWrapper* GetTextureWrapper (csStringID name) = 0;
+    virtual iTextureWrapper* GetTextureWrapper (csStringID name) = 0;
 
   /**
    * Visit all textures.
@@ -147,26 +126,18 @@ SCF_VERSION (iMaterialList, 0, 0, 1);
 
 /**
  * This class represents a list of materials.
- * <p>
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iEngine::GetMaterialList()
- *   </ul>
+ *   - iEngine::GetMaterialList()
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>iEngine
- *   </ul>
+ *   - iEngine
  */
 struct iMaterialList : public iBase
 {
   /// Create a new material.
-  virtual iMaterialWrapper* NewMaterial (iMaterial* material) = 0;
-
-  /**
-   * Create a engine wrapper for a pre-prepared iTextureHandle
-   * The handle will be IncRefed.
-   */
-  virtual iMaterialWrapper *NewMaterial (iMaterialHandle *ith) = 0;
+  virtual iMaterialWrapper* NewMaterial (iMaterial* material,
+  	const char* name) = 0;
 
   /// Return the number of materials in this list
   virtual int GetCount () const = 0;
