@@ -21,6 +21,7 @@
 #define __CS_IENGINE_REGION_H__
 
 /**\file
+ * Engine regions
  */
 /**
  * \addtogroup engine3d
@@ -28,32 +29,35 @@
  
 #include "csutil/scf.h"
 
-struct iSector;
-struct iSprite;
-struct iSpriteTemplate;
+struct iCameraPosition;
+struct iCollection;
+struct iMaterialWrapper;
 struct iMeshWrapper;
 struct iMeshFactoryWrapper;
-struct iTextureWrapper;
-struct iMaterialWrapper;
-struct iCameraPosition;
 struct iObject;
-struct iCollection;
+struct iSector;
+struct iTextureWrapper;
 
-SCF_VERSION (iRegion, 0, 1, 6);
 
 /**
  * A region. A region is basically a collection of objects in the
  * 3D engine that can be treated as a unit.
  */
-struct iRegion : public iBase
+struct iRegion : public virtual iBase
 {
+  SCF_INTERFACE(iRegion, 2,0,0);
   /// Get the iObject for this region.
   virtual iObject *QueryObject() = 0;
 
   /**
    * Add an object to this region.
    */
-  virtual void Add(iObject *obj) = 0;
+  virtual void Add (iObject *obj) = 0;
+
+  /**
+   * Remove an object from this region.
+   */
+  virtual void Remove (iObject *obj) = 0;
 
   /**
    * Clear this region without removing the entities in it. The entities
@@ -107,14 +111,12 @@ struct iRegion : public iBase
   virtual bool IsInRegion (iObject* obj) = 0;
 };
 
-
-SCF_VERSION (iRegionList, 0, 0, 1);
-
 /**
  * A list of region objects.
  */
-struct iRegionList : public iBase
+struct iRegionList : public virtual iBase
 {
+  SCF_INTERFACE(iRegionList, 2,0,0);
   /// Return the number of regions in this list.
   virtual int GetCount () const = 0;
 
