@@ -20,20 +20,28 @@
 #ifndef __CS_IENGINE_RENDERSTEPS_IRENDERSTEP_H__
 #define __CS_IENGINE_RENDERSTEPS_IRENDERSTEP_H__
 
-#include "csgfx/shadervar.h"
-#include "csutil/refarr.h"
+#include "csutil/scf.h"
 #include "ivideo/shader/shader.h"
 
+/**\file
+ * Render step base interface.
+ */
+
+/**\addtogroup engine3d_rloop
+ * @{ */
+
 struct iRenderView;
 struct iSector;
 
-SCF_VERSION (iRenderStep, 0, 0, 1);
-
 /// Document me!@@@
-struct iRenderStep : public iBase
+struct iRenderStep : public virtual iBase
 {
+  SCF_INTERFACE (iRenderStep, 2, 0, 0);
+
   virtual void Perform (iRenderView* rview, iSector* sector,
     csShaderVarStack &stacks) = 0;
 };
 
+/** @} */
+
 #endif
