@@ -22,47 +22,76 @@
 #define __CS_IENGINE_SECTOR_H__
 
 /**\file
+ * Sector
  */
 /**
  * \addtogroup engine3d
  * @{ */
 
-#include "cstypes.h"
 #include "csutil/scf.h"
-#include "csutil/hash.h"
-#include "iutil/object.h"
+#include "csutil/set.h"
 
-class csVector3;
-class csSector;
-class csColor;
-class csBox3;
-class csReversibleTransform;
 struct iMeshWrapper;
+struct iMeshGenerator;
 struct iMeshList;
 struct iLightList;
 struct iLight;
-struct iThing;
 struct iVisibilityCuller;
-struct iVisibilityObject;
+
 struct iObject;
-struct csFog;
-struct iGraphics3D;
+
 struct iRenderView;
 struct iRenderLoop;
 struct iFrustumView;
 struct iSector;
-struct iPortal;
-class csRenderMeshList;
+struct iDocumentNode;
 
-SCF_VERSION (iSectorCallback, 0, 0, 1);
+class csBox3;
+class csColor;
+class csRenderMeshList;
+class csReversibleTransform;
+class csVector3;
+
+enum csFogMode
+{
+  CS_FOG_MODE_NONE = 0,
+  CS_FOG_MODE_LINEAR,
+  CS_FOG_MODE_EXP,
+  CS_FOG_MODE_EXP2
+};
+
+/**
+ * Sector fog structure.
+ */
+struct csFog
+{
+  /// If true then fog is enabled.
+  bool enabled;
+  /// Density (0 is off).
+  float density;
+  /// Color (red).
+  float red;
+  /// Color (green).
+  float green;
+  /// Color (blue).
+  float blue;
+  /// Fog start.
+  float start;
+  /// Fog end.
+  float end;
+  /// Fog mode.
+  csFogMode mode;
+};
 
 /**
  * Set a callback which is called when this sector is traversed.
  * The given context will be either an instance of iRenderView, iFrustumView,
  * or else 0.
  */
-struct iSectorCallback : public iBase
+struct iSectorCallback : public virtual iBase
 {
+  SCF_INTERFACE (iSectorCallback, 0, 0, 1);
+
   /**
    * Sector will be traversed. It is safe to delete this callback
    * in this function.
@@ -70,7 +99,36 @@ struct iSectorCallback : public iBase
   virtual void Traverse (iSector* sector, iBase* context) = 0;
 };
 
-SCF_VERSION (iSector, 0, 5, 3);
+/**
+ * Set a callback which is called when a mesh is added or removed
+ * from this sector.
+ */
+struct iSectorMeshCallback : public virtual iBase
+{
+  SCF_INTERFACE (iSectorMeshCallback, 0, 0, 1);
+
+  /**
+   * New mesh. Note that this is also called if the mesh is added as child
+   * of another mesh that is in the sector.
+   */
+  virtual void NewMesh (iSector* sector, iMeshWrapper* mesh) = 0;
+
+  /**
+   * Remove mesh.
+   */
+  virtual void RemoveMesh (iSector* sector, iMeshWrapper* mesh) = 0;
+};
+
+/**
+ * A callback that is fired whenever a light is visible.
+ */
+struct iLightVisibleCallback : public virtual iBase
+{
+  SCF_INTERFACE (iLightVisibleCallback, 0, 0, 1);
+
+  /// The light is visible.
+  virtual void LightVisible (iSector* sector, iLight* light) = 0;
+};
 
 /**
  * The iSector interface is used to work with "sectors". A "sector"
@@ -80,92 +138,86 @@ SCF_VERSION (iSector, 0, 5, 3);
  * The sector is the basic building block for any Crystal Space level.
  * A level can be made from one or more sectors. Using the thing mesh
  * object one can use portals to connect multiple sectors.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEngine::CreateSector()
- *   </ul>
+ * - iEngine::CreateSector()
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iEngine::FindSector()
- *   <li>iSectorList::Get()
- *   <li>iSectorList::FindByName()
- *   <li>iLoaderContext::FindSector()
- *   <li>iPortal::GetSector()
- *   <li>iCamera::GetSector()
- *   </ul>
+ * - iEngine::FindSector()
+ * - iSectorList::Get()
+ * - iSectorList::FindByName()
+ * - iLoaderContext::FindSector()
+ * - iPortal::GetSector()
+ * - iCamera::GetSector()
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>iEngine
- *   </ul>
+ * - iEngine
  */
-struct iSector : public iBase
+struct iSector : public virtual iBase
 {
-  /// @@@ Used by the engine to retrieve internal sector object (ugly)
-  virtual csSector *GetPrivateObject () = 0;
+  SCF_INTERFACE(iSector,2,0,0);
   /// Get the iObject for this sector.
   virtual iObject *QueryObject () = 0;
 
-  /**
-   * Set the renderloop to use for this sector. If this is not set then
-   * the default engine renderloop will be used.
-   */
-  virtual void SetRenderLoop (iRenderLoop* rl) = 0;
-
-  /**
-   * Get the renderloop for this sector. If this returns 0 then it
-   * means there is no specific renderloop for this sector. In that case
-   * the default renderloop in the engine will be used.
-   */
-  virtual iRenderLoop* GetRenderLoop () = 0;
-
-  /// Has this sector fog?
-  virtual bool HasFog () const = 0;
-  /// Return the fog structure (even if fog is disabled)
-  virtual csFog *GetFog () const = 0;
-  /// Fill the fog structure with the given values
-  virtual void SetFog (float density, const csColor& color) = 0;
-  /// Disable fog in this sector
-  virtual void DisableFog () = 0;
-
+  /**\name Mesh handling
+   * @{ */
   /// Get the list of meshes in this sector.
   virtual iMeshList* GetMeshes () = 0;
-  /**
-   * Get the list of static and pseudo-dynamic lights in this sector.
-   */
-  virtual iLightList* GetLights () = 0;
-
-  /// Calculate lighting for all objects in this sector
-  virtual void ShineLights () = 0;
-  /// Version of ShineLights() which only affects one mesh object.
-  virtual void ShineLights (iMeshWrapper*) = 0;
-
-  /// Sets dynamic ambient light for all things in the sector
-  virtual void SetDynamicAmbientLight(const csColor& color) = 0;
-
-  /// Get the last set dynamic ambient light for this sector.
-  virtual csColor GetDynamicAmbientLight() const = 0;
 
   /**
-   * Calculate the bounding box of all objects in this sector.
-   * This function is not very efficient as it will traverse all objects
-   * in the sector one by one and compute a bounding box from that.
+   * Get a set of visible meshes for given camera. These will be cached for
+   * a given frame and camera, but if the cached result isn't enough it will
+   * be reculled. The returned pointer is valid as long as the sector exists
+   * (the sector will delete it)
    */
-  virtual void CalculateSectorBBox (csBox3& bbox,
-    bool do_meshes) const = 0;
+  virtual csRenderMeshList* GetVisibleMeshes (iRenderView *) = 0;
 
   /**
-   * Use the specified plugin as the visibility culler for
-   * this sector. Returns false if the culler could not be
-   * loaded for some reason.
+   * Get the set of meshes containing portals that leave from this sector.
+   * Note that portals are uni-directional. The portals represented
+   * by this list are portals that are on some mesh object that is
+   * actually located in this sector.
    */
-  virtual bool SetVisibilityCullerPlugin (const char *Name) = 0;
+  virtual const csSet<csPtrKey<iMeshWrapper> >& GetPortalMeshes () const = 0;
   /**
-   * Get the visibility culler that is used for this sector.
-   * If there is no culler yet a culler of type 'crystalspace.culling.frustvis'
-   * will be created and used for this sector.
+   * Register a mesh with a portal. @@@ TO BE REMOVED...
    */
-  virtual iVisibilityCuller* GetVisibilityCuller () = 0;
+  virtual void RegisterPortalMesh (iMeshWrapper* mesh) = 0;
+  /**
+   * Unregister a mesh with a portal. @@@ TO BE REMOVED...
+   */
+  virtual void UnregisterPortalMesh (iMeshWrapper* mesh) = 0;
+
+  /**
+   * Unlink all mesh objects from this sector. This will not remove
+   * the mesh objects but simply unlink them so that they are no longer
+   * part of this sector. This happens automatically when the sector
+   * is removed but you can force it by calling this function here.
+   */
+  virtual void UnlinkObjects () = 0;
+
+  /**
+   * Add a mesh callback. This will call IncRef() on the callback
+   * So make sure you call DecRef() to release your own reference.
+   */
+  virtual void AddSectorMeshCallback (iSectorMeshCallback* cb) = 0;
+
+  /**
+   * Remove a mesh callback.
+   */
+  virtual void RemoveSectorMeshCallback (iSectorMeshCallback* cb) = 0;
+  /** @} */
+
+  /**\name Drawing related
+   * @{ */
+  /// Draw the sector with the given render view
+  virtual void Draw (iRenderView* rview) = 0;
+
+  /**
+   * Prepare the sector to draw.
+   * Must be called before any rendermesh is requested.
+   */
+  virtual void PrepareDraw (iRenderView* rview) = 0;
 
   /**
    * Get the current draw recursion level.
@@ -182,6 +234,126 @@ struct iSector : public iBase
    */
   virtual void DecRecLevel () = 0;
 
+  /**
+   * Set the renderloop to use for this sector. If this is not set then
+   * the default engine renderloop will be used.
+   */
+  virtual void SetRenderLoop (iRenderLoop* rl) = 0;
+
+  /**
+   * Get the renderloop for this sector. If this returns 0 then it
+   * means there is no specific renderloop for this sector. In that case
+   * the default renderloop in the engine will be used.
+   */
+  virtual iRenderLoop* GetRenderLoop () = 0;
+  /** @} */
+
+  /**\name Mesh generator handling
+   * @{ */
+  /**
+   * Create a mesh generator.
+   */
+  virtual iMeshGenerator* CreateMeshGenerator (const char* name) = 0;
+
+  /**
+   * Get the number of mesh generators.
+   */
+  virtual size_t GetMeshGeneratorCount () const = 0;
+
+  /**
+   * Get the specific mesh generator.
+   */
+  virtual iMeshGenerator* GetMeshGenerator (size_t idx) = 0;
+
+  /**
+   * Get the specific mesh generator by name.
+   */
+  virtual iMeshGenerator* GetMeshGeneratorByName (const char* name) = 0;
+
+  /**
+   * Remove a mesh generator.
+   */
+  virtual void RemoveMeshGenerator (size_t idx) = 0;
+
+  /**
+   * Remove all mesh generators.
+   */
+  virtual void RemoveMeshGenerators () = 0;
+  /** @} */
+
+  /**\name Fog handling
+   * @{ */
+  /// Has this sector fog?
+  virtual bool HasFog () const = 0;
+  /// Return the fog structure (even if fog is disabled)
+  virtual csFog *GetFog () const = 0;
+  /// Fill the fog structure with the given values
+  virtual void SetFog (float density, const csColor& color) = 0;
+  /// Disable fog in this sector
+  virtual void DisableFog () = 0;
+  /** @} */
+
+  /**\name Light handling
+   * @{ */
+  /**
+   * Get the list of static and pseudo-dynamic lights in this sector.
+   */
+  virtual iLightList* GetLights () = 0;
+
+  /// Calculate lighting for all objects in this sector
+  virtual void ShineLights () = 0;
+  /// Version of ShineLights() which only affects one mesh object.
+  virtual void ShineLights (iMeshWrapper*) = 0;
+
+  /**
+   * Sets dynamic ambient light this sector. This works in addition
+   * to the dynamic light you can specify for every object.
+   */
+  virtual void SetDynamicAmbientLight (const csColor& color) = 0;
+
+  /// Get the last set dynamic ambient light for this sector.
+  virtual csColor GetDynamicAmbientLight () const = 0;
+
+  /**
+   * Get the version number of the dynamic ambient color. This
+   * number is increased whenever dynamic ambient changes.
+   */
+  virtual uint GetDynamicAmbientVersion () const = 0;
+  /** @} */
+
+  /**\name Visculling
+   * @{ */
+  /**
+   * Calculate the bounding box of all objects in this sector.
+   * This function is not very efficient as it will traverse all objects
+   * in the sector one by one and compute a bounding box from that.
+   */
+  virtual void CalculateSectorBBox (csBox3& bbox,
+    bool do_meshes) const = 0;
+
+  /**
+   * Use the specified plugin as the visibility culler for
+   * this sector. Returns false if the culler could not be
+   * loaded for some reason.
+   * The optional culler parameters will be given to the new
+   * visibility culler.
+   */
+  virtual bool SetVisibilityCullerPlugin (const char* name,
+  	iDocumentNode* culler_params = 0) = 0;
+  /**
+   * Get the visibility culler that is used for this sector.
+   * If there is no culler yet a culler of type 'crystalspace.culling.frustvis'
+   * will be created and used for this sector.
+   */
+  virtual iVisibilityCuller* GetVisibilityCuller () = 0;
+
+  /**
+   * Check visibility in a frustum way for all things and polygons in
+   * this sector and possibly traverse through portals to other sectors.
+   */
+  virtual void CheckFrustum (iFrustumView* lview) = 0;
+
+  
   /**
    * Follow a beam from start to end and return the first polygon that
    * is hit. This function correctly traverse portals and space warping
@@ -223,24 +395,10 @@ struct iSector : public iBase
    */
   virtual iSector* FollowSegment (csReversibleTransform& t,
     csVector3& new_position, bool& mirror, bool only_portals = false) = 0;
+  /** @} */
 
-  /// Draw the sector with the given render view
-  virtual void Draw (iRenderView* rview) = 0;
-
-  /**
-   * Prepare the sector to draw.
-   * Must be called before any rendermesh is requested.
-   */
-  virtual void PrepareDraw (iRenderView* rview) = 0;
-
-  /**
-   * Get a set of visible meshes for given camera. These will be cached for
-   * a given frame and camera, but if the cached result isn't enough it will
-   * be reculled. The returned pointer is valid as long as the sector exsist
-   * (the sector will delete it)
-   */
-  virtual csRenderMeshList* GetVisibleMeshes (iRenderView *) = 0;
-
+  /**\name Various  
+   * @ { */
   /**
    * Set the sector callback. This will call IncRef() on the callback
    * So make sure you call DecRef() to release your own reference.
@@ -258,44 +416,43 @@ struct iSector : public iBase
   /// Get the specified sector callback.
   virtual iSectorCallback* GetSectorCallback (int idx) const = 0;
 
-  /// Used for portal traversal.
-  virtual void CheckFrustum (iFrustumView* lview) = 0;
-
   /**
-   * Get the set of meshes containing portals that leave from this sector.
-   * Note that portals are uni-directional. The portals represented
-   * by this list are portals that are on some mesh object that is
-   * actually located in this sector.
+   * Set/reset culling objects for all lights in the sector.
+   * This can be used for hardware accelerated lighting techniques that
+   * want to know what lights (influence object) are visible for camera.
+   * With this enabled every light will be registered to the culler of
+   * this sector and a callback (see AddLightVisibleCallback) will be
+   * called.
    */
-  virtual const csSet<iMeshWrapper*>& GetPortalMeshes () const = 0;
+  virtual void SetLightCulling (bool enable) = 0;
+  /// Return true if light culling objects are enabled.
+  virtual bool IsLightCullingEnabled () const = 0;
   /**
-   * Register a mesh with a portal. @@@ TO BE REMOVED...
+   * Add a callback that is called whenever a light is visible.
+   * This only works if SetLightCulling() is enabled.
    */
-  virtual void RegisterPortalMesh (iMeshWrapper* mesh) = 0;
+  virtual void AddLightVisibleCallback (iLightVisibleCallback* cb) = 0;
   /**
-   * Unregister a mesh with a portal. @@@ TO BE REMOVED...
+   * Remove a light visible callback.
    */
-  virtual void UnregisterPortalMesh (iMeshWrapper* mesh) = 0;
+  virtual void RemoveLightVisibleCallback (iLightVisibleCallback* cb) = 0;
+  /** @} */
 };
 
 
-SCF_VERSION (iSectorList, 0, 0, 2);
-
 /**
  * A list of sectors.
- * <p>
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iEngine::GetSectors()
- *   <li>iMovable::GetSectors()
- *   </ul>
+ *   - iEngine::GetSectors()
+ *   - iMovable::GetSectors()
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>iEngine
- *   </ul>
+ *   - iEngine
  */
-struct iSectorList : public iBase
+struct iSectorList : public virtual iBase
 {
+  SCF_INTERFACE(iSectorList, 2,0,0);
   /// Return the number of sectors in this list.
   virtual int GetCount () const = 0;
 
@@ -321,19 +478,17 @@ struct iSectorList : public iBase
   virtual iSector *FindByName (const char *Name) const = 0;
 };
 
-SCF_VERSION (iSectorIterator, 0, 1, 0);
 
 /**
  * An iterator to iterate over sectors. Some functions in CS
  * return this.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEngine::GetNearbySectors()
- *   </ul>
+ * - iEngine::GetNearbySectors()
  */
-struct iSectorIterator : public iBase
+struct iSectorIterator : public virtual iBase
 {
+  SCF_INTERFACE(iSectorIterator,2,0,0);
   /// Return true if there are more elements.
   virtual bool HasNext () = 0;
 
