@@ -20,62 +20,56 @@
 #define __CS_IENGINE_TEXTURE_H__
 
 /**\file
+ * Texture interfaces
  */
 /**
  * \addtogroup engine3d_textures
  * @{ */
  
 #include "csutil/scf.h"
-#include "cstypes.h"
 
 struct iImage;
+struct iObject;
 struct iTextureHandle;
 struct iTextureManager;
 struct iTextureWrapper;
-struct iObject;
-
-SCF_VERSION (iTextureCallback, 0, 0, 1);
 
 /**
  * A callback for when a iTextureWrapper is used.
  */
-struct iTextureCallback : public iBase
+struct iTextureCallback : public virtual iBase
 {
+  SCF_INTERFACE(iTextureCallback, 2,0,0);
   /// Get height.
   virtual void UseTexture (iTextureWrapper* wrap) = 0;
 };
 
-SCF_VERSION (iTextureWrapper, 0, 1, 0);
-
 /**
  * A texture wrapper is an engine-level object that wraps around an actual
  * texture (iTextureHandle). Every texture in the engine is represented
  * by a texture wrapper, which keeps the pointer to the texture handle, its
  * name, and possibly the base image object.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEngine::CreateTexture()
- *   <li>iEngine::CreateBlackTexture()
- *   <li>iTextureList::NewTexture()
- *   <li>iLoader::LoadTexture()
- *   </ul>
+ * - iEngine::CreateTexture()
+ * - iEngine::CreateBlackTexture()
+ * - iTextureList::NewTexture()
+ * - iLoader::LoadTexture()
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iEngine::FindTexture()
- *   <li>iTextureList::Get()
- *   <li>iTextureList::FindByName()
- *   <li>iLoaderContext::FindTexture()
- *   <li>iLoaderContext::FindNamedTexture()
- *   <li>iMaterialEngine::GetTextureWrapper()
- *   </ul>
+ * - iEngine::FindTexture()
+ * - iTextureList::Get()
+ * - iTextureList::FindByName()
+ * - iLoaderContext::FindTexture()
+ * - iLoaderContext::FindNamedTexture()
+ * - iMaterialEngine::GetTextureWrapper()
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>iEngine
- *   </ul>
+ * - iEngine
  */
-struct iTextureWrapper : public iBase
+struct iTextureWrapper : public virtual iBase
 {
+  SCF_INTERFACE(iTextureWrapper, 2,0,0);
   /// Get the iObject for this texture
   virtual iObject *QueryObject() = 0;
 
@@ -102,12 +96,12 @@ struct iTextureWrapper : public iBase
   /// Set the transparent color.
   virtual void SetKeyColor (int red, int green, int blue) = 0;
   /// Query the transparent color.
-  virtual void GetKeyColor (int &red, int &green, int &blue) = 0;
+  virtual void GetKeyColor (int &red, int &green, int &blue) const = 0;
 
   /// Set the flags which are used to register the texture
   virtual void SetFlags (int flags) = 0;
   /// Return the flags which are used to register the texture
-  virtual int GetFlags () = 0;
+  virtual int GetFlags () const = 0;
 
   /// Register the texture with the texture manager
   virtual void Register (iTextureManager *txtmng) = 0;
@@ -123,7 +117,7 @@ struct iTextureWrapper : public iBase
    * Get the use callback. If there are multiple use callbacks you can
    * use this function to chain.
    */
-  virtual iTextureCallback* GetUseCallback () = 0;
+  virtual iTextureCallback* GetUseCallback () const = 0;
 
   /**
    * Visit this texture. This should be called by the engine right
@@ -148,6 +142,17 @@ struct iTextureWrapper : public iBase
    * remove the image pointer from this texture wrapper. False by default.
    */
   virtual bool KeepImage () const = 0;
+
+  /**
+   * Set the "class" of this texture.
+   * For more information, see iTextureHandle::SetTextureClass.
+   */
+  virtual void SetTextureClass (const char* className) = 0;
+  /**
+   * Get the "class" of this texture.
+   * For more information, see iTextureHandle::GetTextureClass.
+   */
+  virtual const char* GetTextureClass () = 0;
 };
 
 
@@ -155,11 +160,9 @@ SCF_VERSION (iTextureList, 0, 0, 1);
 
 /**
  * This class represents a list of texture wrappers.
- * <p>
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iEngine::GetTextureList()
- *   </ul>
+ *   - iEngine::GetTextureList()
  */
 struct iTextureList : public iBase
 {
