@@ -21,6 +21,7 @@
 #define __CS_IENGINE_VISCULL_H__
 
 /**\file
+ * Visibility culling
  */
 /**
  * \addtogroup engine3d_vis
@@ -28,18 +29,19 @@
  
 #include "csutil/scf.h"
 
-struct iRenderView;
+struct iDocumentNode;
 struct iFrustumView;
-struct iVisibilityObject;
-struct iMovable;
-struct iShadowReceiver;
 struct iMeshWrapper;
-struct iPolygonMesh;
+struct iMovable;
 struct iObjectModel;
-class csVector3;
+struct iRenderView;
+struct iVisibilityObject;
+
 class csBox3;
-class csSphere;
 class csFlags;
+class csPlane3;
+class csSphere;
+class csVector3;
 
 SCF_VERSION (iVisibilityObjectIterator, 0, 1, 0);
 
@@ -59,14 +61,13 @@ struct iVisibilityObjectIterator : public iBase
 };
 
 
-SCF_VERSION (iVisibilityCullerListener, 0, 0, 1);
-
 /**
  * Implement this interface when you want to get notified about visible
  * objects detected by the visibility cullers.
  */
-struct iVisibilityCullerListener : public iBase
+struct iVisibilityCullerListener : public virtual iBase
 {
+  SCF_INTERFACE(iVisibilityCullerListener, 2,0,0);
   /**
    * This function is called whenever the visibilty culler discovers a new
    * visible mesh.  The frustum_mask is a mask that is compatible with
@@ -78,31 +79,27 @@ struct iVisibilityCullerListener : public iBase
     iMeshWrapper *mesh, uint32 frustum_mask) = 0;
 };
 
-SCF_VERSION (iVisibilityCuller, 0, 4, 0);
+SCF_VERSION (iVisibilityCuller, 0, 7, 0);
 
 /**
  * This interface represents a visibility culling system.
  * To use it you first register visibility objects (which are all the
  * objects for which you want to test visibility) to this culler.
  * A visibility culler can usually also support shadow calculation.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Dynavis culler plugin (crystalspace.culling.dynavis)
- *   <li>Frustvis culler plugin (crystalspace.culling.frustvis)
- *   </ul>
+ * - Dynavis culler plugin (crystalspace.culling.dynavis)
+ * - Frustvis culler plugin (crystalspace.culling.frustvis)
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   <li>CS_LOAD_PLUGIN()
- *   <li>iSector::GetVisibilityCuller()
- *   </ul>
+ * - CS_QUERY_REGISTRY()
+ * - CS_LOAD_PLUGIN()
+ * - iSector::GetVisibilityCuller()
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>iEngine
- *   </ul>
+ * - iEngine
  */
-struct iVisibilityCuller : public iBase
+struct iVisibilityCuller : public virtual iBase
 {
   /**
    * Setup all data for this visibility culler. This needs
@@ -128,6 +125,13 @@ struct iVisibilityCuller : public iBase
    */
   virtual bool VisTest (iRenderView* irview, 
     iVisibilityCullerListener* viscallback) = 0;
+  /**
+   * Precache visibility culling. This can be useful in case you want
+   * to ensure that render speed doesn't get any hickups as soon as a portal
+   * to this sector becomes visible. iEngine->PrecacheDraw() will call this
+   * function.
+   */
+  virtual void PrecacheCulling () = 0;
 
   /**
    * Mark all objects as visible that intersect with the given bounding
@@ -163,7 +167,16 @@ struct iVisibilityCuller : public iBase
    */
   virtual void VisTest (csPlane3* plane, int num_planes, 
     iVisibilityCullerListener* viscallback) = 0;
-	
+
+  /**
+   * Intersect a segment with all objects in the visibility culler and
+   * return them all in an iterator. This function is less accurate
+   * then IntersectSegment() because it might also return objects that
+   * are not even hit by the beam but just close to it.
+   */
+  virtual csPtr<iVisibilityObjectIterator> IntersectSegmentSloppy (
+    const csVector3& start, const csVector3& end) = 0;
+
   /**
    * Intersect a segment with all objects in the visibility culler and
    * return them all in an iterator.
@@ -199,12 +212,10 @@ struct iVisibilityCuller : public iBase
   virtual void CastShadows (iFrustumView* fview) = 0;
 
   /**
-   * Get the current visibility number. You can compare this number
-   * to the visibility number as returned by
-   * iVisibilityObject->GetVisibilityNumber(). If equal then the object
-   * was visible.
+   * Parse a document node with additional parameters for this culler.
+   * Returns error message on error or 0 otherwise.
    */
-  //virtual uint32 GetCurrentVisibilityNumber () const = 0;
+  virtual const char* ParseCullerParameters (iDocumentNode* node) = 0;
 };
 
 /** \name GetCullerFlags() flags
@@ -224,45 +235,28 @@ struct iVisibilityCuller : public iBase
 
 /** @} */
 
-SCF_VERSION (iVisibilityObject, 0, 2, 1);
-
 /**
  * An object that wants to know if it is visible or not
  * for some visibility culler needs to implement this interface.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Mesh wrapper in the engine implements this.
- *   </ul>
+ * - Mesh wrapper in the engine implements this.
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE() from iMeshWrapper.
- *   </ul>
+ * - SCF_QUERY_INTERFACE() from iMeshWrapper.
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>Visibility culler plugins (iVisibilityCuller).
- *   </ul>
+ * - Visibility culler plugins (iVisibilityCuller).
  */
-struct iVisibilityObject : public iBase
+struct iVisibilityObject : public virtual iBase
 {
+  SCF_INTERFACE(iVisibilityObject, 2, 0, 0);
+
   /// Get the reference to the movable from this object.
   virtual iMovable* GetMovable () const = 0;
   /// Get the reference to the mesh wrapper from this object.
   virtual iMeshWrapper* GetMeshWrapper () const = 0;
 
-  /**
-   * Set the visibility number for this object. A visibility culler
-   * will set the visibility number of an object equal to the current
-   * visibility culler number if the object is visible.
-   */
-  virtual void SetVisibilityNumber (uint32 visnr) = 0;
-  /**
-   * Get the visibility number. You can compare this with
-   * iVisibilityCuller->GetCurrentVisibilityNumber(). If equal then
-   * this object is visible.
-   */
-  virtual uint32 GetVisibilityNumber () const = 0;
-
   /**
    * Get the object model corresponding with this object.
    */
@@ -271,10 +265,8 @@ struct iVisibilityObject : public iBase
   /**
    * Get flags for this object. This is a combination of zero or more of the
    * following flags. See the documentation with these flags for more info:
-   * <ul>
-   * <li>#CS_CULLER_HINT_GOODOCCLUDER
-   * <li>#CS_CULLER_HINT_BADOCCLUDER
-   * </ul>
+   * - #CS_CULLER_HINT_GOODOCCLUDER
+   * - #CS_CULLER_HINT_BADOCCLUDER
    */
   virtual csFlags& GetCullerFlags () = 0;
 };
