@@ -25,19 +25,24 @@
 
 /**
  * \addtogroup gfx2d
- * @{
- */
+ * @{ */
 
 #include "csutil/scf.h"
 #include "csutil/array.h"
-#include "igraphic/image.h"
-#include "iutil/databuff.h"
 
-SCF_VERSION (iImageIO, 1, 0, 0);
+struct iDataBuffer;
+struct iImage;
 
-/// \internal format can be read
+
+SCF_VERSION (iImageIO, 1, 0, 1);
+
+/** \internal
+ * Format can be read.
+ */
 #define CS_IMAGEIO_LOAD 1
-/// \internal format can be written
+/** \internal
+ * Format can be written.
+ */
 #define CS_IMAGEIO_SAVE 2
 
 /// Description for a file format supported by an image loader.
@@ -52,13 +57,13 @@ struct csImageIOFileFormatDescription
 };
 
 /// Description for the array of file formats.
-typedef csArray<csImageIOFileFormatDescription*>
+typedef csArray<csImageIOFileFormatDescription const*>
 	csImageIOFileFormatDescriptions;
 
 /**
  * The iImageIO interface is used to save and load graphic files.
  */
-struct iImageIO : public iBase
+struct iImageIO : public virtual iBase
 {
   /// Description for a file format supported by an image loader.
   typedef csImageIOFileFormatDescription FileFormatDescription;
@@ -70,7 +75,7 @@ struct iImageIO : public iBase
 
   /**
    * Load an image from a buffer.<p>
-   * This routine will read from the buffer buf of length size, try to
+   * This routine will read from the buffer buf , try to
    * recognize the type of image contained within, and return an csImageFile
    * of the appropriate type.  Returns a pointer to the iImage on
    * success, or 0 on failure. The bits that fit the CS_IMGFMT_MASK
@@ -79,7 +84,7 @@ struct iImageIO : public iBase
    * are optional: if the image does not contain alpha mask, the GetFormat()
    * method of the image will return a value without that bit set.
    */
-  virtual csPtr<iImage> Load (uint8* iBuffer, uint32 iSize, int iFormat) = 0;
+  virtual csPtr<iImage> Load (iDataBuffer* buf, int iFormat) = 0;
 
   /**
    * Set global image dithering option.<p>
@@ -95,8 +100,8 @@ struct iImageIO : public iBase
    * Those options consist of a comma-separated list and can be either 'option' 
    * or 'option=value'. The available options vary from plugin to plugin, some
    * common ones are:<p>
-   * <code>compress=#</code> - Set image compression, from 0..100. Higher values 
-   * give smaller files, but at the expense of quality(e.g. JPEG) or 
+   * <code>compress=#</code> - Set image compression, from 0..100. Higher
+   * values give smaller files, but at the expense of quality(e.g. JPEG) or 
    * speed(e.g. PNG).<br>
    * <code>progressive</code> - Progressive/interlaced encoding.<p>
    * Examples:<br>
@@ -107,7 +112,7 @@ struct iImageIO : public iBase
     const char* extraoptions = 0) = 0;
 
   /**
-   * Save an image using format <mime>.
+   * Save an image using format MIME.
    * If omitted format selection is left to the plugin.
    */
   virtual csPtr<iDataBuffer> Save (iImage *image, const char *mime = 0,
