@@ -20,9 +20,14 @@
 #ifndef __CS_IMESH_EMIT_H__
 #define __CS_IMESH_EMIT_H__
 
+/**\file
+ * Emitter particle mesh object
+ */ 
+
 #include "csutil/scf.h"
 
 class csColor;
+class csVector3;
 
 SCF_VERSION (iEmitGen3D, 0, 0, 1);
 SCF_VERSION (iEmitFixed, 0, 0, 1);
@@ -38,11 +43,9 @@ SCF_VERSION (iEmitCylinderTangent, 0, 0, 1);
 /**
  * This interface is for objects that can generate 3d vectors, which
  * are used for emitting.
- * <p>
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitGen3D : public iBase
 {
@@ -52,15 +55,12 @@ struct iEmitGen3D : public iBase
 
 /**
  * Fixed value emitter - returns a particular point value.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateFixed()
- *   </ul>
+ * - iEmitFactoryState::CreateFixed()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitFixed : public iEmitGen3D
 {
@@ -70,15 +70,12 @@ struct iEmitFixed : public iEmitGen3D
 
 /**
  * Sphere value emitter - returns points in a sphere.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateSphere()
- *   </ul>
+ * - iEmitFactoryState::CreateSphere()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitSphere : public iEmitGen3D
 {
@@ -90,15 +87,12 @@ struct iEmitSphere : public iEmitGen3D
 
 /**
  * Box value emitter - returns points in an (axis aligned) box.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateBox()
- *   </ul>
+ * - iEmitFactoryState::CreateBox()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitBox : public iEmitGen3D
 {
@@ -110,15 +104,12 @@ struct iEmitBox : public iEmitGen3D
 
 /**
  * Cone value emitter - returns points in a cone.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateCone()
- *   </ul>
+ * - iEmitFactoryState::CreateCone()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitCone : public iEmitGen3D
 {
@@ -135,30 +126,31 @@ struct iEmitCone : public iEmitGen3D
 
 /**
  * Mix value emitter - returns a weighted random mix of other emitters.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateMix()
- *   </ul>
+ * - iEmitFactoryState::CreateMix()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitMix : public iEmitGen3D
 {
   /// add a weighted emitter to the mix
   virtual void AddEmitter(float weight, iEmitGen3D* emit) = 0;
-  /** removes an emitter from the mix given a zero based emitter number.
-   *  Use GetEmitterCount() and GetContent() to enumerate through the mix and find the index of an emitter.
+  /**
+   * Removes an emitter from the mix given a zero based emitter number.
+   * Use GetEmitterCount() and GetContent() to enumerate through the mix
+   * and find the index of an emitter.
    */
   virtual void RemoveEmitter(int num) = 0;
   /// get the total weight in this mix
   virtual float GetTotalWeight() = 0;
   /// get the number of emitters in this mix
   virtual int GetEmitterCount() = 0;
-  /** adjust the weight of an emitter given a zero based emitter number
-   *  Use GetEmitterCount() and GetContent() to enumerate through the mix and find the index of an emitter.
+  /**
+   * Adjust the weight of an emitter given a zero based emitter number
+   * Use GetEmitterCount() and GetContent() to enumerate through the mix
+   * and find the index of an emitter.
    */
   virtual void AdjustEmitterWeight(int num,float weight) = 0;
   /// get content, returns emitters and their weight by a number (0..number-1)
@@ -167,15 +159,12 @@ struct iEmitMix : public iEmitGen3D
 
 /**
  * Line value emitter - returns values on the line between start and end.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateLine()
- *   </ul>
+ * - iEmitFactoryState::CreateLine()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitLine : public iEmitGen3D
 {
@@ -187,15 +176,12 @@ struct iEmitLine : public iEmitGen3D
 
 /**
  * Cylinder value emitter - returns values in a cylinder.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateCylinder()
- *   </ul>
+ * - iEmitFactoryState::CreateCylinder()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitCylinder : public iEmitGen3D
 {
@@ -210,15 +196,12 @@ struct iEmitCylinder : public iEmitGen3D
 /**
  * Sphere tangential value emitter - gives direction tangential to sphere
  * Uses the given point, gives a tangential direction for that.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateSphereTangent()
- *   </ul>
+ * - iEmitFactoryState::CreateSphereTangent()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitSphereTangent : public iEmitGen3D
 {
@@ -231,15 +214,12 @@ struct iEmitSphereTangent : public iEmitGen3D
 /**
  * Cylinder tangential value emitter - gives direction tangential to cylinder
  * Uses the given point, gives a tangential direction for that
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iEmitFactoryState::CreateCylinderTangent()
- *   </ul>
+ * - iEmitFactoryState::CreateCylinderTangent()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEmitState
- *   </ul>
+ * - iEmitState
  */
 struct iEmitCylinderTangent : public iEmitGen3D
 {
@@ -251,66 +231,57 @@ struct iEmitCylinderTangent : public iEmitGen3D
     float& min, float& max) = 0;
 };
 
-SCF_VERSION (iEmitFactoryState, 0, 0, 1);
+SCF_VERSION (iEmitFactoryState, 0, 0, 2);
 
 /**
  * This interface describes the API for the emitter mesh factory object.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Emit mesh object plugin (crystalspace.mesh.object.emit)
- *   <li>iMeshObjectType::NewFactory()
- *   </ul>
+ * - Emit mesh object plugin (crystalspace.mesh.object.emit)
+ * - iMeshObjectType::NewFactory()
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE() on iMeshFactoryWrapper::GetMeshObjectFactory()
- *   </ul>
+ * - scfQueryInterface() on iMeshFactoryWrapper::GetMeshObjectFactory()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>Emit Factory Loader plugin (crystalspace.mesh.loader.factory.emit)
- *   </ul>
+ * - Emit Factory Loader plugin (crystalspace.mesh.loader.factory.emit)
  */
 struct iEmitFactoryState : public iBase
 {
   /// create an emitter, you have to set the content
-  virtual iEmitFixed* CreateFixed() = 0;
+  virtual csRef<iEmitFixed> CreateFixed() = 0;
   /// create an emitter
-  virtual iEmitBox* CreateBox() = 0;
+  virtual csRef<iEmitBox> CreateBox() = 0;
   /// create an emitter
-  virtual iEmitSphere* CreateSphere() = 0;
+  virtual csRef<iEmitSphere> CreateSphere() = 0;
   /// create an emitter
-  virtual iEmitCone* CreateCone() = 0;
+  virtual csRef<iEmitCone> CreateCone() = 0;
   /// create an emitter
-  virtual iEmitMix* CreateMix() = 0;
+  virtual csRef<iEmitMix> CreateMix() = 0;
   /// create an emitter
-  virtual iEmitLine* CreateLine() = 0;
+  virtual csRef<iEmitLine> CreateLine() = 0;
   /// create an emitter
-  virtual iEmitCylinder* CreateCylinder() = 0;
+  virtual csRef<iEmitCylinder> CreateCylinder() = 0;
   /// create an emitter
-  virtual iEmitSphereTangent* CreateSphereTangent() = 0;
+  virtual csRef<iEmitSphereTangent> CreateSphereTangent() = 0;
   /// create an emitter
-  virtual iEmitCylinderTangent* CreateCylinderTangent() = 0;
-
+  virtual csRef<iEmitCylinderTangent> CreateCylinderTangent() = 0;
 };
 
 SCF_VERSION (iEmitState, 0, 0, 1);
 
 /**
  * This interface describes the API for the emitter mesh object.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Emit mesh object plugin (crystalspace.mesh.object.emit)
- *   <li>iMeshObjectFactory::NewInstance()
- *   </ul>
+ * - Emit mesh object plugin (crystalspace.mesh.object.emit)
+ * - iMeshObjectFactory::NewInstance()
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE() on iMeshWrapper::GetMeshObject()
- *   </ul>
+ * - scfQueryInterface() on iMeshWrapper::GetMeshObject()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>Emit Loader plugin (crystalspace.mesh.loader.emit)
- *   </ul>
+ * - Emit Loader plugin (crystalspace.mesh.loader.emit)
  */
 struct iEmitState : public iBase
 {
@@ -404,5 +375,7 @@ struct iEmitState : public iBase
   virtual bool GetContainerBox(csVector3& min, csVector3& max) const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_EMIT_H__
 
