@@ -19,11 +19,17 @@
 #ifndef __CS_IMESH_EXPLODE_H__
 #define __CS_IMESH_EXPLODE_H__
 
+/**\file
+ * Explosion particle mesh object
+ */ 
+
 #include "csutil/scf.h"
 
 class csColor;
-struct iEngine;
-struct iSector;
+class csVector3;
+
+/**\addtogroup meshplugins
+ * @{ */
 
 SCF_VERSION (iExplosionState, 0, 0, 2);
 
@@ -77,5 +83,7 @@ struct iExplosionState : public iBase
   virtual bool GetFadeSprites (csTicks& fade_time) const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_EXPLODE_H__
 
