@@ -20,9 +20,18 @@
 #ifndef __CS_IMESH_FIRE_H__
 #define __CS_IMESH_FIRE_H__
 
+/**\file
+ * Fire particle mesh object
+ */ 
+
 #include "csutil/scf.h"
 
+/**\addtogroup meshplugins
+ * @{ */
+
+class csBox3;
 class csColor;
+class csVector3;
 
 SCF_VERSION (iFireState, 0, 0, 1);
 
@@ -65,5 +74,7 @@ struct iFireState : public iBase
   virtual float GetTotalTime () const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_FIRE_H__
 
