@@ -20,9 +20,19 @@
 #ifndef __CS_IMESH_RAIN_H__
 #define __CS_IMESH_RAIN_H__
 
+/**\file
+ * Rain particle mesh object
+ */ 
+
 #include "csutil/scf.h"
 
+/**\addtogroup meshplugins
+ * @{ */
+
+struct iMaterialWrapper;
+
 class csColor;
+class csVector3;
 
 SCF_VERSION (iRainState, 0, 0, 1);
 
@@ -69,5 +79,7 @@ struct iRainState : public iBase
   virtual const csColor& GetColor () const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_RAIN_H__
 
