@@ -20,9 +20,17 @@
 #ifndef __CS_IMESH_SNOW_H__
 #define __CS_IMESH_SNOW_H__
 
+/**\file
+ * Snow particle mesh object
+ */ 
+
 #include "csutil/scf.h"
 
+/**\addtogroup meshplugins
+ * @{ */
+
 class csColor;
+class csVector3;
 
 SCF_VERSION (iSnowState, 0, 0, 1);
 
@@ -57,5 +65,7 @@ struct iSnowState : public iBase
   virtual float GetSwirl () const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_SNOW_H__
 
