@@ -19,26 +19,36 @@
 #ifndef __CS_IMESH_SPRITE3D_H__
 #define __CS_IMESH_SPRITE3D_H__
 
+/**\file
+ * 3D keyframe animated sprite mesh object
+ */ 
+
 #include "csutil/scf.h"
-#include "csutil/garray.h"
-#include "csutil/cscolor.h"
-#include "ivideo/graph3d.h"
+
+/**\addtogroup meshplugins
+ * @{ */
 
-class csColor;
 struct iMaterialWrapper;
 struct iMeshObject;
-struct iMeshWrapper;
 struct iMeshObjectFactory;
+struct iMeshWrapper;
 struct iRenderView;
-struct iRenderView;
+
+class csColor;
+class csVector2;
+class csVector3;
+struct csTriangle;
 
 /**
  * Macros for the csSprite3D lighting levels.
  */
-#define CS_SPR_LIGHTING_HQ 0
-#define CS_SPR_LIGHTING_LQ 1
-#define CS_SPR_LIGHTING_FAST 2
-#define CS_SPR_LIGHTING_RANDOM 3
+enum
+{
+  CS_SPR_LIGHTING_HQ = 0,
+  CS_SPR_LIGHTING_LQ = 1,
+  CS_SPR_LIGHTING_FAST = 2,
+  CS_SPR_LIGHTING_RANDOM = 3
+};
 
 /**
  * Use the global value for determining which lighting level is used by the
@@ -77,13 +87,13 @@ struct iRenderView;
 // @@@ CONFIG TODO: global_lighting_quality
 // @@@ CONFIG TODO: global_lod_level
 
-SCF_VERSION (iSpriteFrame, 0, 0, 2);
-
 /**
  * A frame for 3D sprite animation.
  */
-struct iSpriteFrame : public iBase
+struct iSpriteFrame : public virtual iBase
 {
+  SCF_INTERFACE (iSpriteFrame, 1, 0, 0);
+
   /// Set the name.
   virtual void SetName (char const*) = 0;
   /// Get the name.
@@ -94,13 +104,13 @@ struct iSpriteFrame : public iBase
   virtual int GetTexIndex () const = 0;
 };
 
-SCF_VERSION (iSpriteAction, 0, 0, 1);
-
 /**
  * An action frameset for 3D sprite animation.
  */
-struct iSpriteAction : public iBase
+struct iSpriteAction : public virtual iBase
 {
+  SCF_INTERFACE (iSpriteAction, 1, 0, 0);
+
   /// Set the name.
   virtual void SetName (char const*) = 0;
   /// Get the name.
@@ -116,17 +126,18 @@ struct iSpriteAction : public iBase
   /// Get the displacement for the specified frame.
   virtual float GetFrameDisplacement (int f) = 0;
   /// Add a frame to this action.
-  virtual void AddFrame (iSpriteFrame* frame, int delay, float displacement) = 0;
+  virtual void AddFrame (iSpriteFrame* frame, int delay,
+  	float displacement) = 0;
 };
 
-SCF_VERSION (iSpriteSocket, 0, 0, 1);
-
 /**
  * A socket for specifying where sprites can plug into
  * other sprites.
  */
-struct iSpriteSocket : public iBase
+struct iSpriteSocket : public virtual iBase
 {
+  SCF_INTERFACE (iSpriteSocket, 1, 0, 0);
+
   /// Set the name.
   virtual void SetName (char const*) = 0;
   /// Get the name.
@@ -143,13 +154,13 @@ struct iSpriteSocket : public iBase
   virtual int GetTriangleIndex () const = 0;
 };
 
-SCF_VERSION (iSprite3DFactoryState, 0, 0, 3);
-
 /**
  * This interface describes the API for the 3D sprite factory mesh object.
  */
-struct iSprite3DFactoryState : public iBase
+struct iSprite3DFactoryState : public virtual iBase
 {
+  SCF_INTERFACE (iSprite3DFactoryState, 1, 0, 0);
+
   /// Set material of sprite.
   virtual void SetMaterialWrapper (iMaterialWrapper* material) = 0;
   /// Get material of sprite.
@@ -158,7 +169,7 @@ struct iSprite3DFactoryState : public iBase
   /**
    * Reserve space for the given number of vertices. A vertex includes
    * information about its position, normal and texel. This function will
-   * not write any information into the reserved space. <p>
+   * not write any information into the reserved space. 
    *
    * Note that this function requires that at least one frame exists in
    * the sprite factory, otherwise this function will fail!
@@ -262,11 +273,9 @@ struct iSprite3DFactoryState : public iBase
    * Sets which lighting config variable that all new sprites created
    * from this template will use.
    * The options are:
-   * <ul>
-   * <li>CS_SPR_LIGHT_GLOBAL (default)
-   * <li>CS_SPR_LIGHT_TEMPLATE
-   * <li>CS_SPR_LIGHT_LOCAL
-   * </ul>
+   * - CS_SPR_LIGHT_GLOBAL (default)
+   * - CS_SPR_LIGHT_TEMPLATE
+   * - CS_SPR_LIGHT_LOCAL
    */
   virtual void SetLightingQualityConfig (int qual) = 0;
   /// Get the lighting quality config.
@@ -276,11 +285,9 @@ struct iSprite3DFactoryState : public iBase
    * Sets which lod config variable that all new sprites created
    * from this template will use.
    * The options are:
-   * <ul>
-   * <li>CS_SPR_LOD_GLOBAL (default)
-   * <li>CS_SPR_LOD_TEMPLATE
-   * <li>CS_SPR_LOD_LOCAL
-   * </ul>
+   * - CS_SPR_LOD_GLOBAL (default)
+   * - CS_SPR_LOD_TEMPLATE
+   * - CS_SPR_LOD_LOCAL
    */
   virtual void SetLodLevelConfig (int config_flag) = 0;
 
@@ -314,17 +321,13 @@ struct iSprite3DFactoryState : public iBase
   virtual uint GetMixMode () const = 0;
 };
 
-SCF_VERSION (iSprite3DState, 0, 0, 6);
-
 /**
  * This interface describes the API for the 3D sprite mesh object.
  */
-struct iSprite3DState : public iBase
+struct iSprite3DState : public virtual iBase
 {
-  /// Set material of sprite.
-  virtual void SetMaterialWrapper (iMaterialWrapper* material) = 0;
-  /// Get material of sprite.
-  virtual iMaterialWrapper* GetMaterialWrapper () const = 0;
+  SCF_INTERFACE (iSprite3DState, 1, 0, 0);
+
   /// Set mix mode.
   virtual void SetMixMode (uint mode) = 0;
   /// Get mix mode.
@@ -412,11 +415,9 @@ struct iSprite3DState : public iBase
   /**
    * Sets which lighting config variable this sprite will use.
    * The options are:
-   * <ul>
-   * <li>CS_SPR_LIGHT_GLOBAL (default)
-   * <li>CS_SPR_LIGHT_TEMPLATE
-   * <li>CS_SPR_LIGHT_LOCAL
-   * </ul>
+   * - CS_SPR_LIGHT_GLOBAL (default)
+   * - CS_SPR_LIGHT_TEMPLATE
+   * - CS_SPR_LIGHT_LOCAL
    */
   virtual void SetLightingQualityConfig (int config_flag) = 0;
 
@@ -428,11 +429,9 @@ struct iSprite3DState : public iBase
   /**
    * Sets which lighting config variable this sprite will use.
    * The options are:
-   * <ul>
-   *   <li>CS_SPR_LOD_GLOBAL (default)
-   *   <li>CS_SPR_LOD_TEMPLATE
-   *   <li>CS_SPR_LOD_LOCAL
-   * </ul>
+   * - CS_SPR_LOD_GLOBAL (default)
+   * - CS_SPR_LOD_TEMPLATE
+   * - CS_SPR_LOD_LOCAL
    */
   virtual void SetLodLevelConfig (int config_flag) = 0;
 
@@ -446,18 +445,6 @@ struct iSprite3DState : public iBase
    */
   virtual bool IsLodEnabled () const = 0;
 
-  /**
-   * Set the base color. This color will be added to the vertex
-   * colors of the sprite. If no lighting is used then this will
-   * be the color.
-   */
-  virtual void SetBaseColor (const csColor& col) = 0;
-
-  /**
-   * Get the base color.
-   */
-  virtual void GetBaseColor (csColor& col) const = 0;
-
   /// find a socked based on the sprite attached to it.
   virtual iSpriteSocket* FindSocket (iMeshWrapper *mesh) const = 0;  
 
@@ -465,5 +452,7 @@ struct iSprite3DState : public iBase
   virtual iSpriteSocket* FindSocket (const char * name) const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_SPRITE3D_H__
 
