@@ -19,10 +19,18 @@
 #ifndef __CS_IMESH_STARS_H__
 #define __CS_IMESH_STARS_H__
 
+/**\file
+ * Stars particle mesh object
+ */ 
+
 #include "csutil/scf.h"
 
-class csVector3;
+class csBox3;
 class csColor;
+class csVector3;
+
+/**\addtogroup meshplugins
+ * @{ */
 
 SCF_VERSION (iStarsState, 0, 0, 2);
 
@@ -61,5 +69,7 @@ struct iStarsState : public iBase
   virtual float GetMaxDistance () const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_STARS_H__
 
