@@ -20,23 +20,28 @@
 #define __CS_IMESH_THING_H__
 
 #include "csutil/scf.h"
-#include "csutil/flags.h"
 
-class csVector2;
-class csVector3;
-class csMatrix3;
-class csPlane3;
-struct iSector;
-struct iGraphics3D;
-struct iFrustumView;
+/**\file
+ * Thing mesh interfaces
+ */
+
+/**
+ * \addtogroup meshplugins
+ * @{ */
+
 struct iMaterialWrapper;
-struct iMaterialList;
-struct iMovable;
 struct iMeshObject;
 struct iMeshObjectFactory;
+struct iRenderBuffer;
 struct iThingState;
 struct iThingFactoryState;
 
+class csFlags;
+class csMatrix3;
+class csPlane3;
+class csVector2;
+class csVector3;
+
 /**
  * A range structure for specifing polygon ranges.
  */
@@ -126,8 +131,6 @@ struct csPolygonRange
 #define CS_THING_MOVE_OCCASIONAL 2
 /** @} */
 
-SCF_VERSION (iPolygonHandle, 0, 0, 1);
-
 /**
  * This is an interface that can be used to represent a polygon in
  * situations where a SCF object is required. Create an instance of
@@ -135,20 +138,20 @@ SCF_VERSION (iPolygonHandle, 0, 0, 1);
  * iThingState->CreatePolygonHandle(). Note that this handle will make
  * sure that the returned pointers are cleared if the thing or thing
  * factory happens to be removed.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iThingState::CreatePolygonHandle()
- *   <li>iThingFactoryState::CreatePolygonHandle()
- *   </ul>
+ * - iThingState::CreatePolygonHandle()
+ * - iThingFactoryState::CreatePolygonHandle()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iThingState
- *   <li>iThingFactoryState
- *   </ul>
+ * - iThingState
+ * - iThingFactoryState
+ *   
  */
-struct iPolygonHandle : public iBase
+struct iPolygonHandle : public virtual iBase
 {
+  SCF_INTERFACE (iPolygonHandle, 1, 0, 0);
+
   /**
    * Get the factory state for this polygon. Or 0 if the factory is
    * removed.
@@ -180,31 +183,25 @@ struct iPolygonHandle : public iBase
 };
 
 
-SCF_VERSION (iThingFactoryState, 0, 2, 0);
-
 /**
  * This is the state interface to access the internals of a thing
  * mesh factory.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Thing mesh object plugin (crystalspace.mesh.object.thing)
- *   <li>iMeshObjectType::NewFactory()
- *   </ul>
+ * - Thing mesh object plugin (crystalspace.mesh.object.thing)
+ * - iMeshObjectType::NewFactory()
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE() on iMeshFactoryWrapper::GetMeshObjectFactory()
- *   <li>iThingState::GetFactory()
- *   </ul>
+ * - scfQueryInterface() on iMeshFactoryWrapper::GetMeshObjectFactory()
+ * - scfQueryInterface() on iMeshObject::GetFactory()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>Thing Factory Loader plugin (crystalspace.mesh.loader.factory.thing)
- *   </ul>
+ * - Thing Factory Loader plugin (crystalspace.mesh.loader.factory.thing)
+ *   
  */
-struct iThingFactoryState : public iBase
+struct iThingFactoryState : public virtual iBase
 {
-  /// @@@ UGLY
-  virtual void* GetPrivateObject () = 0;
+  SCF_INTERFACE(iThingFactoryState, 2, 0, 0);
 
   /**
    * Compress the vertex table so that all nearly identical vertices
@@ -234,7 +231,7 @@ struct iThingFactoryState : public iBase
 
   /**
    * Add a triangle.
-   * <p>
+   * 
    * By default the texture mapping is set so that the texture
    * is aligned on the u-axis with the 'v1'-'v2' vector and the scale is set
    * so that the texture tiles once for every unit (i.e. if you have the
@@ -247,7 +244,7 @@ struct iThingFactoryState : public iBase
   /**
    * Add a quad. Note that quads are the most optimal kind of polygon
    * for a thing so you should try to use these as much as possible.
-   * <p>
+   * 
    * By default the texture mapping is set so that the texture
    * is aligned on the u-axis with the 'v1'-'v2' vector and the scale is set
    * so that the texture tiles once for every unit (i.e. if you have the
@@ -260,7 +257,7 @@ struct iThingFactoryState : public iBase
 
   /**
    * Add a general polygon.
-   * <p>
+   * 
    * By default the texture mapping is set so that the texture
    * is aligned on the u-axis with the 'v1'-'v2' vector and the scale is set
    * so that the texture tiles once for every unit (i.e. if you have the
@@ -272,7 +269,7 @@ struct iThingFactoryState : public iBase
 
   /**
    * Add a general polygon using vertex indices.
-   * <p>
+   * 
    * By default the texture mapping is set so that the texture
    * is aligned on the u-axis with the 'v1'-'v2' vector and the scale is set
    * so that the texture tiles once for every unit (i.e. if you have the
@@ -284,7 +281,7 @@ struct iThingFactoryState : public iBase
 
   /**
    * Add a box that can be seen from the outside. This will add six polygons.
-   * <p>
+   * 
    * By default the texture mapping is set so that the texture
    * is aligned on the u-axis with the 'v1'-'v2' vector and the scale is set
    * so that the texture tiles once for every unit (i.e. if you have the
@@ -296,7 +293,7 @@ struct iThingFactoryState : public iBase
 
   /**
    * Add a box that can be seen from the inside. This will add six polygons.
-   * <p>
+   * 
    * By default the texture mapping is set so that the texture
    * is aligned on the u-axis with the 'v1'-'v2' vector and the scale is set
    * so that the texture tiles once for every unit (i.e. if you have the
@@ -310,6 +307,7 @@ struct iThingFactoryState : public iBase
    * Set the name of all polygons in the given range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param name the name to be assigned to the indicated polygons.
    */
   virtual void SetPolygonName (const csPolygonRange& range,
   	const char* name) = 0;
@@ -337,6 +335,7 @@ struct iThingFactoryState : public iBase
    * Set the material of all polygons in the given range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param material the material to be assigned to the indicated polygons.
    */
   virtual void SetPolygonMaterial (const csPolygonRange& range,
   	iMaterialWrapper* material) = 0;
@@ -352,6 +351,7 @@ struct iThingFactoryState : public iBase
    * Add a vertex to all polygons in the given range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param vt the vertex index to be added to the polygons.
    */
   virtual void AddPolygonVertex (const csPolygonRange& range,
   	const csVector3& vt) = 0;
@@ -360,6 +360,7 @@ struct iThingFactoryState : public iBase
    * Add a vertex index to all polygons in the given range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param vt the vertex index to be added to the polygons.
    */
   virtual void AddPolygonVertex (const csPolygonRange& range, int vt) = 0;
 
@@ -368,6 +369,8 @@ struct iThingFactoryState : public iBase
    * It is more optimal to call this routine as opposed to calling
    * AddPolygonVertex() all the time.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
+   * \param num number of entires in `indices'.
+   * \param indices polygon index table to be assigned to range.
    * range.
    */
   virtual void SetPolygonVertexIndices (const csPolygonRange& range,
@@ -384,6 +387,7 @@ struct iThingFactoryState : public iBase
    * Get a vertex from a polygon.
    * \param polygon_idx is a polygon index or #CS_POLYINDEX_LAST for last
    * created polygon.
+   * \param vertex_idx the desired vertex.
    */
   virtual const csVector3& GetPolygonVertex (int polygon_idx,
   	int vertex_idx) = 0;
@@ -402,9 +406,13 @@ struct iThingFactoryState : public iBase
    * to specify texture mapping like this. It is recommended to use one
    * of the other texture mapping routines.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
+   * \param m the matrix which defines the mapping.
+   * \param v the vector which defines the mapping.
    * range.
+   * \return false in case the texture mapping was invalid. In that case
+   * a default texture mapping will be set on the polygon.
    */
-  virtual void SetPolygonTextureMapping (const csPolygonRange& range,
+  virtual bool SetPolygonTextureMapping (const csPolygonRange& range,
   	const csMatrix3& m, const csVector3& v) = 0;
 
   /**
@@ -412,8 +420,13 @@ struct iThingFactoryState : public iBase
    * given uv coordinates for the first three vertices of every polygon.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param uv1 uv coordinates for vertex 1.
+   * \param uv2 uv coordinates for vertex 2.
+   * \param uv3 uv coordinates for vertex 3.
+   * \return false in case the texture mapping was invalid. In that case
+   * a default texture mapping will be set on the polygon.
    */
-  virtual void SetPolygonTextureMapping (const csPolygonRange& range,
+  virtual bool SetPolygonTextureMapping (const csPolygonRange& range,
   	const csVector2& uv1, const csVector2& uv2, const csVector2& uv3) = 0;
 
   /**
@@ -423,8 +436,16 @@ struct iThingFactoryState : public iBase
    * the same plane.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param p1 vertex 1.
+   * \param uv1 uv coordinates for vertex 1.
+   * \param p2 vertex 2.
+   * \param uv2 uv coordinates for vertex 2.
+   * \param p3 vertex 3.
+   * \param uv3 uv coordinates for vertex 3.
+   * \return false in case the texture mapping was invalid. In that case
+   * a default texture mapping will be set on the polygon.
    */
-  virtual void SetPolygonTextureMapping (const csPolygonRange& range,
+  virtual bool SetPolygonTextureMapping (const csPolygonRange& range,
   	const csVector3& p1, const csVector2& uv1,
   	const csVector3& p2, const csVector2& uv2,
   	const csVector3& p3, const csVector2& uv3) = 0;
@@ -435,15 +456,20 @@ struct iThingFactoryState : public iBase
    * first vertex is seen as the origin and the second as the u-axis of the
    * texture space coordinate system. The v-axis is calculated on the plane
    * of the polygons and orthogonal to the given u-axis. The length of the
-   * u-axis and the v-axis is given as the 'len1' parameter.
-   * <p>
-   * For example, if 'len1' is equal to 2 this means that texture will be
-   * tiled exactly two times between vertex 'v_orig' and 'v1'.
+   * u-axis and the v-axis is given as the 'len' parameter.
+   * 
+   * For example, if 'len' is equal to 2 this means that texture will be
+   * tiled exactly two times between vertex 'v_orig' and 'v'.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param v_orig origin.
+   * \param v u-axis in texture space coordinate system.
+   * \param len length of u-axis.
+   * \return false in case the texture mapping was invalid. In that case
+   * a default texture mapping will be set on the polygon.
    */
-  virtual void SetPolygonTextureMapping (const csPolygonRange& range,
-  	const csVector3& v_orig, const csVector3& v1, float len1) = 0;
+  virtual bool SetPolygonTextureMapping (const csPolygonRange& range,
+  	const csVector3& v_orig, const csVector3& v, float len) = 0;
 
   /**
    * Set texture mapping of all polygons in the given range to use the
@@ -452,13 +478,20 @@ struct iThingFactoryState : public iBase
    * texture space coordinate system, and the third as the v-axis.
    * The length of the u-axis and the v-axis is given with the 'len1' and
    * 'len2' parameters.
-   * <p>
+   * 
    * For example, if 'len1' is equal to 2 this means that texture will be
    * tiled exactly two times between vertex 'v_orig' and 'v1'.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param v_orig origin.
+   * \param v1 u-axis in texture space coordinate system.
+   * \param len1 length of u-axis.
+   * \param v2 v-axis in texture space coordinate system.
+   * \param len2 length of v-axis.
+   * \return false in case the texture mapping was invalid. In that case
+   * a default texture mapping will be set on the polygon.
    */
-  virtual void SetPolygonTextureMapping (const csPolygonRange& range,
+  virtual bool SetPolygonTextureMapping (const csPolygonRange& range,
   	const csVector3& v_orig,
 	const csVector3& v1, float len1,
 	const csVector3& v2, float len2) = 0;
@@ -469,28 +502,36 @@ struct iThingFactoryState : public iBase
    * first vertex is seen as the origin and the second as the u-axis of the
    * texture space coordinate system. The v-axis is calculated on the plane
    * of the polygons and orthogonal to the given u-axis. The length of the
-   * u-axis and the v-axis is given as the 'len1' parameter.
-   * <p>
+   * u-axis and the v-axis is given as the 'len' parameter.
+   * 
    * For example, if 'len1' is equal to 2 this means that texture will be
    * tiled exactly two times between the two first vertices.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param len length of the u-axis and the v-axis is given as the 'len1'
+   * parameter.
+   * \return false in case the texture mapping was invalid. In that case
+   * a default texture mapping will be set on the polygon.
    */
-  virtual void SetPolygonTextureMapping (const csPolygonRange& range,
-  	float len1) = 0;
+  virtual bool SetPolygonTextureMapping (const csPolygonRange& range,
+  	float len) = 0;
 
   /**
    * Get the texture space information for the specified polygon.
    * \param polygon_idx is a polygon index or #CS_POLYINDEX_LAST for last
    * created polygon.
+   * \param m the matrix which will receive the mapping.
+   * \param v the vector which will receive the mapping.
    */
   virtual void GetPolygonTextureMapping (int polygon_idx,
   	csMatrix3& m, csVector3& v) = 0;
 
   /**
-   * Disable or enable texture mapping for the range of polygons.
+   * Enable or disable texture mapping for the range of polygons.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param enabled boolean flag indicating if mapping should be enabled or
+   * disabled.
    */
   virtual void SetPolygonTextureMappingEnabled (const csPolygonRange& range,
   	bool enabled) = 0;
@@ -506,6 +547,8 @@ struct iThingFactoryState : public iBase
    * Set the given flags to all polygons in the range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param flags a mask of the polygon flags, such as #CS_POLY_LIGHTING,
+   * #CS_POLY_COLLDET, etc.
    */
   virtual void SetPolygonFlags (const csPolygonRange& range, uint32 flags) = 0;
 
@@ -513,6 +556,11 @@ struct iThingFactoryState : public iBase
    * Set the given flags to all polygons in the range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param mask a mask of the polygon flags, such as #CS_POLY_LIGHTING,
+   * #CS_POLY_COLLDET, etc. Only the indicated bits from `flags' will be
+   * assigned.
+   * \param flags The set of bits to assign to the corresponding flags
+   * indicated by `mask'.
    */
   virtual void SetPolygonFlags (const csPolygonRange& range, uint32 mask,
   	uint32 flags) = 0;
@@ -521,6 +569,8 @@ struct iThingFactoryState : public iBase
    * Reset the given flags to all polygons in the range.
    * \param range is one of the #CS_POLYRANGE defines to specify a polygon
    * range.
+   * \param flags a mask of the polygon flags, such as #CS_POLY_LIGHTING,
+   * #CS_POLY_COLLDET, etc.
    */
   virtual void ResetPolygonFlags (const csPolygonRange& range,
   	uint32 flags) = 0;
@@ -551,6 +601,7 @@ struct iThingFactoryState : public iBase
    * the given polygon.
    * \param polygon_idx is a polygon index or #CS_POLYINDEX_LAST for last
    * created polygon.
+   * \param v the point in object space to test against the polygon.
    */
   virtual bool PointOnPolygon (int polygon_idx, const csVector3& v) = 0;
 
@@ -604,35 +655,34 @@ struct iThingFactoryState : public iBase
    * default is used.
    */
   virtual void SetCosinusFactor (float cosfact) = 0;
-};
 
-SCF_VERSION (iThingState, 0, 7, 0);
+  virtual bool AddPolygonRenderBuffer (int polygon_idx, const char* name,
+    iRenderBuffer* buffer) = 0;
+
+  /// Set mix mode.
+  virtual void SetMixMode (uint mode) = 0;
+  /// Get mix mode.
+  virtual uint GetMixMode () const = 0;
+};
 
 /**
  * This is the state interface to access the internals of a thing
  * mesh object.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Thing mesh object plugin (crystalspace.mesh.object.thing)
- *   <li>iMeshObjectFactory::NewInstance()
- *   </ul>
+ * - Thing mesh object plugin (crystalspace.mesh.object.thing)
+ * - iMeshObjectFactory::NewInstance()
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE() on iMeshWrapper::GetMeshObject()
- *   </ul>
+ * - scfQueryInterface() on iMeshWrapper::GetMeshObject()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>Thing Loader plugin (crystalspace.mesh.loader.thing)
- *   </ul>
+ * - Thing Loader plugin (crystalspace.mesh.loader.thing)
+ *   
  */
-struct iThingState : public iBase
+struct iThingState : public virtual iBase
 {
-  /// @@@ UGLY
-  virtual void* GetPrivateObject () = 0;
-
-  /// Get the factory.
-  virtual iThingFactoryState* GetFactory () = 0;
+  SCF_INTERFACE (iThingState, 1, 0, 0);
 
   /// Get the given vertex coordinates in world space
   virtual const csVector3 &GetVertexW (int idx) const = 0;
@@ -647,8 +697,7 @@ struct iThingState : public iBase
   /**
    * Control how this thing will be moved.
    * There are currently two options.
-   * <ul>
-   *   <li>CS_THING_MOVE_NEVER: this option is set for a thing that cannot
+   * - CS_THING_MOVE_NEVER: this option is set for a thing that cannot
    *       move at all. In this case the movable will be ignored and only
    *       hard transforms can be used to move a thing with this flag. This
    *       setting is both efficient for memory (object space coordinates are
@@ -656,7 +705,7 @@ struct iThingState : public iBase
    *       render speed (only the camera transform is needed). This option
    *       is very useful for static geometry like walls.
    *       This option is default.
-   *   <li>CS_THING_MOVE_OCCASIONAL: this option is set for a thing that
+   * - CS_THING_MOVE_OCCASIONAL: this option is set for a thing that
    *       is movable but doesn't move all the time usually. Setting this
    *       option means that the world space vertices will be cached (taking
    *       up more memory that way) but the coordinates will be recalculated
@@ -665,8 +714,7 @@ struct iThingState : public iBase
    *       move but more memory is used as all the vertices are duplicated.
    *       Use this option for geometry that is not too big (in number of
    *       vertices) and only moves occasionally like doors of elevators.
-   * </ul>
-   * <p>
+   * 
    * Note: it is no longer needed to manually set this option. By default
    * things will use CS_THING_MOVE_NEVER and they will automatically switch
    * to the slightly less efficient CS_THING_MOVE_OCCASIONAL if needed.
@@ -729,23 +777,21 @@ struct iThingState : public iBase
   virtual const csPlane3& GetPolygonWorldPlane (int polygon_idx) = 0;
 };
 
-SCF_VERSION (iThingEnvironment, 0, 3, 0);
-
 /**
  * This interface is implemented by the iObjectType for things.
  * Using this interface you can access some global information for things.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Thing mesh plugin.
- *   </ul>
+ * - Thing mesh plugin.
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>SCF_QUERY_INTERFACE on thing mesh plugin (iMeshObjectType).
- *   </ul>
+ * - scfQueryInterface on thing mesh plugin (iMeshObjectType).
+ *   
  */
-struct iThingEnvironment : public iBase
+struct iThingEnvironment : public virtual iBase
 {
+  SCF_INTERFACE (iThingEnvironment, 1, 0, 0);
+
   /**
    * Reset the thing environment (clear all stuff related to things).
    */
@@ -759,4 +805,6 @@ struct iThingEnvironment : public iBase
   virtual int GetDefaultLightmapCellSize () const = 0;
 };
 
+/** @} */
+
 #endif // __CS_IMESH_THING_H__
