@@ -16,15 +16,15 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __ISOUND_LOADER_H__
-#define __ISOUND_LOADER_H__
+#ifndef __CS_ISOUND_LOADER_H__
+#define __CS_ISOUND_LOADER_H__
 
 #include "csutil/scf.h"
 
 struct iSoundData;
 struct csSoundFormat;
 
-SCF_VERSION (iSoundLoader, 1, 0, 0);
+SCF_VERSION (iSoundLoader, 1, 1, 0);
 
 /**
  * The sound loader is used to load sound files given a raw input data stream.
@@ -32,7 +32,7 @@ SCF_VERSION (iSoundLoader, 1, 0, 0);
 struct iSoundLoader : public iBase
 {
   /// Create a sound object from raw input data.
-  virtual iSoundData *LoadSound(void *Data, unsigned long Size) const = 0;
+  virtual csPtr<iSoundData> LoadSound(void *Data, size_t Size) = 0;
 };
 
-#endif // __ISOUND_LOADER_H__
+#endif // __CS_ISOUND_LOADER_H__
