@@ -16,21 +16,20 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __ISOUND_WRAPPER_H__
-#define __ISOUND_WRAPPER_H__
+#ifndef __CS_ISOUND_WRAPPER_H__
+#define __CS_ISOUND_WRAPPER_H__
 
 #include "csutil/scf.h"
 
 struct iSoundHandle;
 struct iObject;
 
-SCF_VERSION (iSoundWrapper, 0, 0, 1);
-
 /**
  * This class is a csObject (iObject) wrapper for sound handles.
  */
-struct iSoundWrapper : public iBase
+struct iSoundWrapper : public virtual iBase
 {
+  SCF_INTERFACE(iSoundWrapper, 2,0,0);
   /// Return the sound handle
   virtual iSoundHandle *GetSound () = 0;
 
@@ -38,4 +37,4 @@ struct iSoundWrapper : public iBase
   virtual iObject *QueryObject () = 0;
 };
 
-#endif // __ISOUND_WRAPPER_H__
+#endif // __CS_ISOUND_WRAPPER_H__
