@@ -20,29 +20,27 @@
 #define __CS_IUTIL_CMDLINE_H__
 
 /**\file
+ * Command line parsing utility
  */
 /**\addtogroup util
  * @{ */
-#include "csutil/scf.h"
-#include "csutil/csstring.h"
+#include "csutil/scf_interface.h"
+
 
-SCF_VERSION (iCommandLineParser, 0, 0, 2);
 
 /**
  * A utility class that makes it easier to parse the command line.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>csInitializer::CreateEnvironment()
- *   <li>csInitializer::CreateCommandLineParser()
- *   </ul>
+ * - csInitializer::CreateEnvironment()
+ * - csInitializer::CreateCommandLineParser()
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   </ul>
+ * - csQueryRegistry()
  */
-struct iCommandLineParser : public iBase
+struct iCommandLineParser : public virtual iBase
 {
+  SCF_INTERFACE(iCommandLineParser, 2,0,0);
   /**
    * Initialize for the given command line.  Options from command line are
    * added to any options already present --- i.e. those added via AddName()
@@ -52,18 +50,19 @@ struct iCommandLineParser : public iBase
   /// Clear all options and names.
   virtual void Reset () = 0;
   /// Query specific commandline option (you can query second etc. such option)
-  virtual const char *GetOption (const char *iName, int iIndex = 0) const = 0;
+  virtual const char *GetOption (const char *iName, size_t iIndex = 0)
+  	const = 0;
   /// Query filename specified on commandline (that is, without leading '-')
-  virtual const char *GetName (int iIndex = 0) const = 0;
+  virtual const char *GetName (size_t iIndex = 0) const = 0;
   /// Add a command-line option to the command-line option array
   virtual void AddOption (const char *iName, const char *iValue) = 0;
   /// Add a command-line name to the command-line names array
   virtual void AddName (const char *iName) = 0;
   /// Replace the Nth command-line option with a new value
   virtual bool ReplaceOption (const char *iName, const char *iValue,
-    int iIndex = 0) = 0;
+    size_t iIndex = 0) = 0;
   /// Replace the Nth command-line name with a new value
-  virtual bool ReplaceName (const char *iValue, int iIndex = 0) = 0;
+  virtual bool ReplaceName (const char *iValue, size_t iIndex = 0) = 0;
   /**
    * Check for a -[no]option toggle. The difference to using GetOption() to
    * check for the two possibilities is that this function respects the
@@ -86,18 +85,18 @@ struct iCommandLineParser : public iBase
    * GetAppDir(); however, on MacOS/X, it is the "Resources" directory within
    * the Cocoa application wrapper.
    */
-  virtual const csString& GetResourceDir () = 0;
+  virtual const char* GetResourceDir () = 0;
 
   /**
    * Returns the directory in which the application executable resides; or the
    * directory in which the Cocoa application wrapper resides on MacOS/X.
    */
-  virtual const csString& GetAppDir () = 0;
+  virtual const char* GetAppDir () = 0;
 
   /**
    * Returns the full path to the application executable.
    */
-  virtual const csString& GetAppPath () = 0;
+  virtual const char* GetAppPath () = 0;
 };
 
 /** @} */
