@@ -20,64 +20,79 @@
 #define __CS_IOBJECT_OBJECT_H__
 
 /**\file
+ * Generic object interface
  */
 /**\addtogroup util
  * @{ */
-#include "csutil/scf.h"
+#include "csutil/scf_interface.h"
 #include "csutil/ref.h"
 
 struct iObjectIterator;
+struct iObject;
 
 /**
  * You can use this macro to get a child object from a csObject. The returned
  * object will be IncRef'ed. This version requires a correctly set-up interface
  * ID variable.
  */
-#define CS_GET_CHILD_OBJECT(object,Interface)			\
-  csPtr<Interface> ((Interface*)(object)->GetChild(		\
-  scfInterface<Interface>::GetID(), scfInterface<Interface>::GetVersion()))
+#define CS_GET_CHILD_OBJECT(object,Interface)                        \
+  scfQueryInterfaceSafe<Interface>((object)->GetChild(               \
+    scfInterfaceTraits<Interface>::GetID(),                          \
+    scfInterfaceTraits<Interface>::GetVersion()))
 
 /**
  * You can use this macro to get a child object with the given name and
  * interface from a csObject. The returned object will be IncRef'ed.
  */
-#define CS_GET_NAMED_CHILD_OBJECT(object,Interface,name)		\
-  csPtr<Interface> ((Interface*)(object)->GetChild(			\
-  scfInterface<Interface>::GetID(), scfInterface<Interface>::GetVersion(), \
-  name))
+#define CS_GET_NAMED_CHILD_OBJECT(object,Interface,name)             \
+  scfQueryInterfaceSafe<Interface>((object)->GetChild(               \
+    scfInterfaceTraits<Interface>::GetID(),                          \
+    scfInterfaceTraits<Interface>::GetVersion(),                     \
+    name))
 
 /**
  * This is the same as CS_GET_CHILD_OBJECT, but stops at the first object
  * with the given name, even if it does not implement the requested interface.
  */
-#define CS_GET_FIRST_NAMED_CHILD_OBJECT(object,Interface,name)		\
-  csPtr<Interface> ((Interface*)(object)->GetChild(			\
-  scfInterface<Interface>::GetID(), scfInterface<Interface>::GetVersion(), \
-  name, true))
+#define CS_GET_FIRST_NAMED_CHILD_OBJECT(object,Interface,name)       \
+  scfQueryInterfaceSafe<Interface>((object)->GetChild(               \
+    scfInterfaceTraits<Interface>::GetID(),                          \
+    scfInterfaceTraits<Interface>::GetVersion(),                     \
+    name, true))
 
+/**
+ * A callback that you can implement to get notified of name changes
+ * in an iObject.
+ */
+struct iObjectNameChangeListener : public virtual iBase
+{
+  SCF_INTERFACE (iObjectNameChangeListener, 0, 0, 1);
+
+  /// Object name has changed.
+  virtual void NameChanged (iObject* obj, const char* oldname,
+  	const char* newname) = 0;
+};
 
-SCF_VERSION (iObject, 0, 3, 0);
 
 /**
  * This interface is an SCF interface for encapsulating csObject.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Many objects implement this (especially objects in the
- *       3D engine like meshes, lights, sectors, materials, ...)
- *   </ul>
+ * - Many objects implement this (especially objects in the
+ *   3D engine like meshes, lights, sectors, materials, ...)
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>Many objects have a QueryObject() method that you can use.
- *   <li>SCF_QUERY_INTERFACE() on the object.
- *   <li>iObject::GetObjectParent()
- *   <li>iObject::GetChild()
- *   <li>iObjectIterator::Next()
- *   <li>iObjectIterator::GetParentObj()
- *   </ul>
+ * - Many objects have a QueryObject() method that you can use.
+ * - scfQueryInterface() on the object.
+ * - iObject::GetObjectParent()
+ * - iObject::GetChild()
+ * - iObjectIterator::Next()
+ * - iObjectIterator::GetParentObj()
  */
-struct iObject : public iBase
+struct iObject : public virtual iBase
 {
+  SCF_INTERFACE(iObject,2,0,1);
+
   /// Set object name
   virtual void SetName (const char *iName) = 0;
 
@@ -85,7 +100,7 @@ struct iObject : public iBase
   virtual const char *GetName () const = 0;
 
   /// Get the unique ID associated with this object.
-  virtual CS_ID GetID () const = 0;
+  virtual uint GetID () const = 0;
 
   /**
    * Set the parent iObject. Note that this only sets the 'parent' pointer but
@@ -113,16 +128,15 @@ struct iObject : public iBase
    * optionally pass a name to look for. If FirstName is true then the
    * method will stop at the first object with the requested name, even
    * if it did not implement the requested type. Note that the returned
-   * object may only be cast to the requested type, no other type, not
-   * even iObject! <p>
+   * object must still be queried for the requested type. <p>
    *
    * Note that the returned object will be IncRef'ed.
    */
-  virtual void* GetChild (int iInterfaceID, int iVersion,
+  virtual iObject* GetChild (int iInterfaceID, int iVersion,
     const char *Name = 0, bool FirstName = false) const = 0;
 
   /// Return the first child object with the given name
-  virtual iObject *GetChild (const char *Name) const = 0;
+  virtual iObject* GetChild (const char *Name) const = 0;
 
   /**
    * Return an iterator for all child objects. Note that you should not
@@ -132,25 +146,34 @@ struct iObject : public iBase
 
   // @@@ temporary fix
   virtual void ObjReleaseOld (iObject *obj) = 0;
+
+  /**
+   * Add a name change listener.
+   */
+  virtual void AddNameChangeListener (
+  	iObjectNameChangeListener* listener) = 0;
+
+  /**
+   * Remove a name change listener.
+   */
+  virtual void RemoveNameChangeListener (
+  	iObjectNameChangeListener* listener) = 0;
 };
 
 
-SCF_VERSION (iObjectIterator, 0, 1, 0);
-
 /**
  * This is an iterator for child objects of a csObject. Note that this
  * iterator only contains type-independent functionality and is therefore
  * a bit complicated to use (i.e. you'll have to do a lot of
  * SCF_QUERY_INTERFACE_CALLS if you use it directly). Check out typed object
  * iterators instead.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iObject::GetIterator()
- *   </ul>
+ * - iObject::GetIterator()
  */
-struct iObjectIterator : public iBase
+struct iObjectIterator : public virtual iBase
 {
+  SCF_INTERFACE(iObjectIterator,2,0,0);
   /// Move forward
   virtual iObject* Next () = 0;
 
