@@ -20,8 +20,13 @@
 #ifndef __CS_IVARIA_COLLIDER_H__
 #define __CS_IVARIA_COLLIDER_H__
 
+/**\file
+ * Mesh collider interfaces
+ */
+
 #include "csutil/scf.h"
 #include "csgeom/vector3.h"
+#include "csutil/array.h"
 
 struct iPolygonMesh;
 struct iMeshObject;
@@ -33,27 +38,37 @@ class csReversibleTransform;
  */
 struct csCollisionPair
 {
-  csVector3 a1, b1, c1;	// First triangle
-  csVector3 a2, b2, c2;	// Second triangle
+  //@{
+  /// First triangle
+  csVector3 a1, b1, c1;	
+  //@}
+  //@{
+  // Second triangle
+  csVector3 a2, b2, c2;	
+  //@}
+};
+
+/**
+ * An intersection triangle for CollideRay.
+ */
+struct csIntersectingTriangle
+{
+  csVector3 a, b, c;
 };
 
 SCF_VERSION (iCollider, 0, 2, 0);
 
 /**
  * A mesh collider.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iCollideSystem::CreateCollider()
- *   </ul>
+ * - iCollideSystem::CreateCollider()
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>csColliderWrapper::GetCollider()
- *   </ul>
+ * - csColliderWrapper::GetCollider()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>csColliderWrapper
- *   </ul>
+ * - csColliderWrapper
  */
 struct iCollider : public iBase
 {
@@ -66,21 +81,16 @@ SCF_VERSION (iCollideSystem, 0, 0, 4);
  * iCollider entities. A collider represents an entity in the
  * collision detection world. It uses the geometry data as given by
  * iPolygonMesh.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>RAPID plugin (crystalspace.collisiondetection.rapid)
- *   <li>OPCODE plugin (crystalspace.collisiondetection.opcode)
- *   </ul>
+ * - OPCODE plugin (crystalspace.collisiondetection.opcode)
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   </ul>
+ * - csQueryRegistry()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>csColliderWrapper
- *   <li>csColliderHelper
- *   </ul>
+ * - csColliderWrapper
+ * - csColliderHelper
  */
 struct iCollideSystem : public iBase
 {
@@ -119,6 +129,7 @@ struct iCollideSystem : public iBase
   virtual bool Collide (
   	iCollider* collider1, const csReversibleTransform* trans1,
   	iCollider* collider2, const csReversibleTransform* trans2) = 0;
+
   /**
    * Get pointer to current array of collision pairs.
    * This array will grow with every call to Collide until you clear
@@ -133,7 +144,7 @@ struct iCollideSystem : public iBase
    * Get number of collision pairs in array.
    * \return the number of collision pairs.
    */
-  virtual int GetCollisionPairCount () = 0;
+  virtual size_t GetCollisionPairCount () = 0;
 
   /**
    * Reset the array with collision pairs. It is very important to call
@@ -142,6 +153,45 @@ struct iCollideSystem : public iBase
    */
   virtual void ResetCollisionPairs () = 0;
 
+  /**
+   * Collide a collider with a world space ray.
+   * \param collider is the collider to test with.
+   * \param trans is the transform for the object represented by the
+   * collider. If the collider belongs to a mesh object then you can get
+   * the transform by calling mesh->GetMovable()->GetFullTransform().
+   * \param start is the start of the ray.
+   * \param end is the end of the ray.
+   * \return true if there was a collision. The array with intersecting
+   * triangles will be updated (see GetIntersectingTriangles()).
+   */
+  virtual bool CollideRay (
+  	iCollider* collider, const csReversibleTransform* trans,
+	const csVector3& start, const csVector3& end) = 0;
+
+  /**
+   * Collide a collider with a world space segment. This will not
+   * return collisions with triangles behind the end of the segment.
+   * \param collider is the collider to test with.
+   * \param trans is the transform for the object represented by the
+   * collider. If the collider belongs to a mesh object then you can get
+   * the transform by calling mesh->GetMovable()->GetFullTransform().
+   * \param start is the start of the ray.
+   * \param end is the end of the ray.
+   * \return true if there was a collision. The array with intersecting
+   * triangles will be updated (see GetIntersectingTriangles()).
+   */
+  virtual bool CollideSegment (
+  	iCollider* collider, const csReversibleTransform* trans,
+	const csVector3& start, const csVector3& end) = 0;
+
+  /**
+   * Get the array of intersection points as returned by CollideRay().
+   * Note that the coordinates in the array of triangles is in object
+   * space of the collider object and not world space!
+   */
+  virtual const csArray<csIntersectingTriangle>& GetIntersectingTriangles ()
+  	const = 0;
+
   /**
    * Indicate if we are interested only in the first hit that is found.
    * This is only valid for CD algorithms that actually allow the
