@@ -20,51 +20,56 @@
 #ifndef __CS_IVARIA_DYNAMICS_H__
 #define __CS_IVARIA_DYNAMICS_H__
 
+/**\file
+ * Physics interfaces
+ */
+
 #include "csutil/scf.h"
-#include "csutil/ref.h"
 
-class csVector3;
-class csOrthoTransform;
-class csMatrix3;
-class csPlane3;
-struct iMeshWrapper;
 
-struct iObject;
-struct iDynamicSystem;
-struct iRigidBody;
 struct iBodyGroup;
-struct iDynamicsMoveCallback;
-struct iDynamicsCollisionCallback;
 struct iJoint;
+struct iMeshWrapper;
+struct iLight;
+struct iCamera;
+struct iObject;
 struct iPolygonMesh;
+struct iRigidBody;
 
-SCF_VERSION (iDynamics, 0, 0, 2);
+class csMatrix3;
+class csOrthoTransform;
+class csPlane3;
+class csVector3;
+
+struct iDynamicsCollisionCallback;
+struct iDynamicsMoveCallback;
+struct iDynamicSystem;
 
 /**
  * This is the interface for the actual plugin.
  * It is responsible for creating iDynamicSystem.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>ODE Physics plugin (crystalspace.dynamics.ode)
- *   </ul>
+ * - ODE Physics plugin (crystalspace.dynamics.ode)
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   </ul>
+ * - csQueryRegistry()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>Dynamics loader plugin (crystalspace.dynamics.loader)
- *   </ul>
+ * - Dynamics loader plugin (crystalspace.dynamics.loader)
  */
-struct iDynamics : public iBase
+struct iDynamics : public virtual iBase
 {
+  SCF_INTERFACE(iDynamics,0,0,2);
   /// Create a rigid body and add it to the simulation
   virtual csPtr<iDynamicSystem> CreateSystem () = 0;
 
-  /// Create a rigid body and add it to the simulation
+  /// Remove dynamic system from the simulation
   virtual void RemoveSystem (iDynamicSystem* system) = 0;
 
+  /// Remove all dynamic systems from the simulation
+  virtual void RemoveSystems () = 0;
+
   /// Finds a system by name
   virtual iDynamicSystem* FindSystem (const char *name) = 0;
 
@@ -72,25 +77,24 @@ struct iDynamics : public iBase
   virtual void Step (float stepsize) = 0;
 };
 
-SCF_VERSION (iDynamicSystem, 0, 0, 2);
+struct iDynamicsSystemCollider;
 
 /**
  * This is the interface for the dynamics core.
  * It handles all bookkeeping for rigid bodies and joints.
  * It also handles collision response.
  * Collision detection is done in another plugin.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iDynamicSystem::CreateSystem()
- *   </ul>
+ * - iDynamicSystem::CreateSystem()
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iDynamicSystem::FindSystem()
- *   </ul>
+ * - iDynamicSystem::FindSystem()
  */
-struct iDynamicSystem : public iBase
+struct iDynamicSystem : public virtual iBase
 {
+  SCF_INTERFACE (iDynamicSystem, 0, 0, 2);
+
   /// returns the underlying object
   virtual iObject *QueryObject (void) = 0;
   /// Set the global gravity.
@@ -106,6 +110,25 @@ struct iDynamicSystem : public iBase
   /// Get the global rolling dampener setting.
   virtual float GetRollingDampener () const = 0;
 
+  /**
+   * Turn on/off AutoDisable functionality.
+   * AutoDisable will stop moving objects if they are stable in order
+   * to save processing time. By default this is enabled.
+   */
+  virtual void EnableAutoDisable (bool enable) = 0;
+  virtual bool AutoDisableEnabled () =0;
+  /**
+   * Set the parameters for AutoDisable.
+   * \param linear Maximum linear movement to disable a body
+   * \param angular Maximum angular movement to disable a body
+   * \param steps Minimum number of steps the body meets linear and angular
+   * requirements before it is disabled.
+   * \param time Minimum time the body needs to meet linear and angular
+   * movement requirements before it is disabled.
+   */
+  virtual void SetAutoDisableParams (float linear, float angular, int steps,
+  	float time)=0;
+
   /// Step the simulation forward by stepsize.
   virtual void Step (float stepsize) = 0;
 
@@ -118,6 +141,12 @@ struct iDynamicSystem : public iBase
   /// Finds a body within a system
   virtual iRigidBody *FindBody (const char *name) = 0;
 
+  /// Get Rigid Body by its index
+  virtual iRigidBody *GetBody (unsigned int index) = 0;
+
+  /// Get rigid bodys count
+  virtual int GetBodysCount () = 0;
+
   /// Create a body group.  Bodies in a group don't collide with each other
   virtual csPtr<iBodyGroup> CreateGroup () = 0;
 
@@ -190,7 +219,6 @@ struct iDynamicSystem : public iBase
    * \param radius the radius of the sphere
    * \param offset a translation of the sphere's center
    * from the default (0,0,0)
-   * \param trans a hard transform to apply to the mesh
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
    * \param elasticity the "bouncyness" of this object, from 0
@@ -206,7 +234,6 @@ struct iDynamicSystem : public iBase
   /**
    * Attaches a static collider plane to world
    * \param plane describes the plane to added
-   * \param trans a hard transform to apply to the mesh
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
    * \param elasticity the "bouncyness" of this object, from 0
@@ -218,27 +245,48 @@ struct iDynamicSystem : public iBase
    */
   virtual bool AttachColliderPlane (const csPlane3 &plane, float friction,
     float elasticity, float softness = 0.01f) = 0;
-};
 
-SCF_VERSION (iDynamicsMoveCallback, 0, 0, 1);
+  /// Destroy all static colliders
+  virtual void DestroyColliders () = 0;
+
+  /// Destroy static collider
+  virtual void DestroyCollider (iDynamicsSystemCollider* collider) = 0;
+
+  /// Attach collider to dynamic system 
+  virtual void AttachCollider (iDynamicsSystemCollider* collider) = 0;
+
+  /**
+   * Create static collider and put it into simulation. After collision it
+   * will remain in the same place, but it will affect collided dynamic
+   * colliders (to make it dynamic, just attach it to the rigid body).
+   */
+  virtual csRef<iDynamicsSystemCollider> CreateCollider () = 0;
+
+  /// Get static collider.
+  virtual csRef<iDynamicsSystemCollider> GetCollider (unsigned int index) = 0;
+
+  /// Get static colliders count.
+  virtual int GetColliderCount () = 0;
+};
 
 /**
  * This is the interface for a dynamics move callback.
  * Set on iRigidBody, it can update attachments after each step.
- * <p>
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>application specific
- *   <li>iDynamicSystem::GetDefaultMoveCallback()
- *   </ul>
+ * - application specific
+ * - iDynamicSystem::GetDefaultMoveCallback()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>iDynamicSystem
- *   </ul>
+ * - iDynamicSystem
  */
-struct iDynamicsMoveCallback : public iBase
+struct iDynamicsMoveCallback : public virtual iBase
 {
+  SCF_INTERFACE (iDynamicsMoveCallback, 0, 0, 1);
+
   virtual void Execute (iMeshWrapper* mesh, csOrthoTransform& t) = 0;
+  virtual void Execute (iLight* light, csOrthoTransform& t) = 0;
+  virtual void Execute (iCamera* camera, csOrthoTransform& t) = 0;
   virtual void Execute (csOrthoTransform& t) = 0;
 };
 
@@ -246,15 +294,13 @@ SCF_VERSION (iDynamicsCollisionCallback, 0, 0, 1);
 
 /**
  * This is the interface for attaching a collider callback to the body
- * <p>
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>application specific
- *   </ul>
+ * - application specific
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iDynamicSystem
- *   </ul>
+ * - iDynamicSystem
+ *   
  */
 struct iDynamicsCollisionCallback : public iBase
 {
@@ -269,19 +315,15 @@ SCF_VERSION (iBodyGroup, 0, 0, 1);
  * certain collisions.  For instance if you have a car built of
  * many different bodies.  The bodies can be collected into a group
  * and the car will be treated as a single object.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iDynamicSystem::CreateGroup()
- *   </ul>
+ * - iDynamicSystem::CreateGroup()
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iRigidBody::GetGroup()
- *   </ul>
+ * - iRigidBody::GetGroup()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>iDynamicSystem
- *   </ul>
+ * - iDynamicSystem
  */
 struct iBodyGroup : public iBase
 {
@@ -293,29 +335,25 @@ struct iBodyGroup : public iBase
    virtual bool BodyInGroup (iRigidBody *body) = 0;
 };
 
-SCF_VERSION (iRigidBody, 0, 0, 2);
-
 /**
  * This is the interface for a rigid body.
  * It keeps all properties for the body.
  * It can also be attached to a movable or a bone,
  * to automatically update it.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iDynamicSystem::CreateBody()
- *   </ul>
+ * - iDynamicSystem::CreateBody()
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iDynamicSystem::FindBody()
- *   </ul>
+ * - iDynamicSystem::FindBody()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>iDynamicSystem
- *   </ul>
+ * - iDynamicSystem
  */
-struct iRigidBody : public iBase
+struct iRigidBody : public virtual iBase
 {
+  SCF_INTERFACE (iRigidBody, 0, 0, 2);
+
   /// returns the underlying object
   virtual iObject *QueryObject (void) = 0;
   /**
@@ -328,12 +366,21 @@ struct iRigidBody : public iBase
   virtual bool MakeDynamic (void) = 0;
   /// Tells whether a body has been made static or not
   virtual bool IsStatic (void) = 0;
+  /**
+    * Temporarily ignores the body until something collides with it.
+  */
+  virtual bool Disable (void) = 0;
+  /// Re-enables a body after calling Disable, or by being auto disabled
+  virtual bool Enable (void) = 0;
+  /// Returns true if a body is enabled.
+  virtual bool IsEnabled (void) = 0;  
 
   /// Returns which group a body belongs to
   virtual csRef<iBodyGroup> GetGroup (void) = 0;
 
   /**
    * Add a collider with a associated friction coefficient
+   * \param mesh the mesh object which will act as collider
    * \param trans a hard transform to apply to the mesh
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
@@ -352,6 +399,8 @@ struct iRigidBody : public iBase
 
   /**
    * Cylinder orientated along its local z axis
+   * \param length length of the cylinder
+   * \param radius radius of the cylinder
    * \param trans a hard transform to apply to the mesh
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
@@ -370,6 +419,7 @@ struct iRigidBody : public iBase
 
   /**
    * Add a collider box with given properties
+   * \param size the box's dimensions
    * \param trans a hard transform to apply to the mesh
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
@@ -388,7 +438,8 @@ struct iRigidBody : public iBase
 
   /**
    * Add a collider sphere with given properties
-   * \param trans a hard transform to apply to the mesh
+   * \param radius radius of sphere
+   * \param offset position of sphere
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
    * \param density the density of this rigid body (used to calculate
@@ -406,7 +457,7 @@ struct iRigidBody : public iBase
 
   /**
    * Add a collider plane with given properties
-   * \param trans a hard transform to apply to the mesh
+   * \param plane the plane which will act as collider
    * \param friction how much friction this body has,
    * ranges from 0 (no friction) to infinity (perfect friction)
    * \param density the density of this rigid body (used to calculate
@@ -421,6 +472,20 @@ struct iRigidBody : public iBase
   virtual bool AttachColliderPlane (const csPlane3 &plane, float friction,
     float density, float elasticity, float softness = 0.01f) = 0;
 
+  /** 
+   * Attach collider to rigid body. If you have set colliders transform before
+   * then it will be considered as relative to attached body (but still if you 
+   * will use colliders "GetTransform ()" it will be in the world coordinates.
+   * Colider become dynamic (which means that it will follow rigid body).
+   */
+  virtual void AttachCollider (iDynamicsSystemCollider* collider) = 0;
+
+  /// Destroy body colliders.
+  virtual void DestroyColliders () = 0;
+
+  /// Destroy body collider.
+  virtual void DestroyCollider (iDynamicsSystemCollider* collider) = 0;
+
   /// Set the position
   virtual void SetPosition (const csVector3& trans) = 0;
   /// Get the position
@@ -448,6 +513,13 @@ struct iRigidBody : public iBase
   /// Get the physic properties. 0 parameters are ignored
   virtual void GetProperties (float* mass, csVector3* center,
     csMatrix3* inertia) = 0;
+  /// Get mass
+  virtual float GetMass () = 0;
+  /// Get center
+  virtual csVector3 GetCenter () = 0;
+  /// Get inertia
+  virtual csMatrix3 GetInertia () = 0;
+
   /// Set total mass to targetmass, and adjust properties
   virtual void AdjustTotalMass (float targetmass) = 0;
 
@@ -500,10 +572,18 @@ struct iRigidBody : public iBase
   virtual int GetJointCount () const = 0;
   */
 
-  /// Attach a iMeshWrapper to this body
+  /// Attach an iMeshWrapper to this body
   virtual void AttachMesh (iMeshWrapper* mesh) = 0;
   /// Returns the attached MeshWrapper
-  virtual csRef<iMeshWrapper> GetAttachedMesh () = 0;
+  virtual iMeshWrapper* GetAttachedMesh () = 0;
+  /// Attach an iLight to this body
+  virtual void AttachLight (iLight* light) = 0;
+  /// Returns the attached light
+  virtual iLight* GetAttachedLight () = 0;
+  /// Attach an iCamera to this body
+  virtual void AttachCamera (iCamera* camera) = 0;
+  /// Returns the attached camera
+  virtual iCamera* GetAttachedCamera () = 0;
 
   /**
    * Set a callback to be executed when this body moves.
@@ -521,6 +601,182 @@ struct iRigidBody : public iBase
 
   /// Update transforms for mesh and/or bone
   virtual void Update () = 0;
+
+  /// Get body collider by its index
+  virtual csRef<iDynamicsSystemCollider> GetCollider (unsigned int index) = 0;
+
+  /// Get body colliders count 
+  virtual int GetColliderCount () = 0;
+};
+
+enum csColliderGeometryType
+{
+  NO_GEOMETRY,
+  BOX_COLLIDER_GEOMETRY,
+  PLANE_COLLIDER_GEOMETRY,
+  TRIMESH_COLLIDER_GEOMETRY,
+  CYLINDER_COLLIDER_GEOMETRY,
+  SPHERE_COLLIDER_GEOMETRY
+};
+
+
+SCF_VERSION (iDynamicsColliderCollisionCallback, 0, 0, 1);
+
+/**
+ * This is the interface for attaching a collider callback to the body
+ *
+ * Main ways to get pointers to this interface:
+ * - application specific
+ * 
+ * Main users of this interface:
+ * - iDynamicSystem
+ */
+struct iDynamicsColliderCollisionCallback : public iBase
+{
+  virtual void Execute (iDynamicsSystemCollider *thiscollider, 
+    iDynamicsSystemCollider *othercollider) = 0;
+  virtual void Execute (iDynamicsSystemCollider *thiscollider, 
+    iRigidBody *otherbody) = 0;
+};
+
+
+struct iGeneralFactoryState;
+class csBox3;
+class csSphere;
+class csReversibleTransform;
+
+/**
+ * This is the interface for a dynamics system collider.
+ * It keeps all properties that system uses for collision 
+ * detection and after collision behaviour (like surface 
+ * properties, collider geometry). It can be placed into 
+ * dynamic system (then this will be "static" collider by default) 
+ * or attached to body (then it will be "dynamic"). 
+ *
+ * Main creators of instances implementing this interface:
+ * - iDynamicSystem::CreateCollider()
+ * 
+ * Main ways to get pointers to this interface:
+ * - iDynamicSystem::GetCollider()
+ * - iRigidBody::GetCollider()
+ * 
+ * Main users of this interface:
+ * - iDynamicSystem
+ * - iRigidBody
+ */
+struct iDynamicsSystemCollider : public virtual iBase
+{
+  SCF_INTERFACE (iDynamicsSystemCollider, 0, 0, 1);
+
+  /// Create Collider Geometry with given sphere
+  virtual bool CreateSphereGeometry (const csSphere& sphere) = 0;
+
+  /// Create Collider Geometry with given plane
+  virtual bool CreatePlaneGeometry (const csPlane3& plane) = 0;
+
+  /// Create Collider Geometry with given mesh geometry
+  virtual bool CreateMeshGeometry (iMeshWrapper *mesh) = 0;
+
+  /// Create Collider Geometry with given box (given by its size)
+  virtual bool CreateBoxGeometry (const csVector3& box_size) = 0;
+
+  /// Create Collider Geometry with Cylinder (given by its length and radius)
+  virtual bool CreateCCylinderGeometry (float length, float radius) = 0;
+
+  //FIXME: This should be implememented, but it is not so obvious - it
+  //should be valid also for static colliders.
+  virtual void SetCollisionCallback (
+  	iDynamicsColliderCollisionCallback* cb) = 0;
+
+  /// Set friction of collider surface
+  virtual void SetFriction (float friction) = 0;
+
+  /// Set softness of collider surface
+  virtual void SetSoftness (float softness) = 0;
+
+  /**
+   * Set body density. This could look strange that collider needs to know
+   * this parameter, but it is used for reseting mass of conected body  
+   * (it should depend on collider geometry)
+   */
+  virtual void SetDensity (float density) = 0;
+
+  /// Set softness of collider elasticity
+  virtual void SetElasticity (float elasticity) = 0;
+  
+  /// Get collider surface friction
+  virtual float GetFriction () = 0;
+
+  /// Get collider surface friction
+  virtual float GetSoftness () = 0;
+
+  /// Get bodys density
+  virtual float GetDensity () = 0;
+
+  /// Get collider surface elasticity
+  virtual float GetElasticity () = 0;
+
+  /// Fills given General Mesh factory with collider geometry 
+  virtual void FillWithColliderGeometry (
+  	csRef<iGeneralFactoryState> genmesh_fact) = 0;
+
+  /// Get geometry type
+  virtual csColliderGeometryType GetGeometryType () = 0;
+
+  /// Get collider transform (it will be alwas in world cooridnates)
+  virtual csOrthoTransform GetTransform () = 0;
+
+  /**
+   * Get collider transform (when collider is attached to body transform 
+   * will be in body space)
+   */
+  virtual csOrthoTransform GetLocalTransform () = 0;
+
+  /**
+   * Set Collider transform. If this is "static" collider then given transform
+   * will be in world space, otherwise it will be in attached rigid body space.
+   */ 
+  virtual void SetTransform (const csOrthoTransform& trans) = 0;
+
+  /**
+   * If collider has box geometry method will return true and size of
+   * box, otherwise it will return false.
+   */
+  virtual bool GetBoxGeometry (csVector3& size) = 0; 
+
+  /**
+   * If collider has sphere geometry method will return true and sphere, 
+   * otherwise it will return false.
+   */
+  virtual bool GetSphereGeometry (csSphere& sphere) = 0;
+
+  /**
+   * If collider has plane geometry method will return true and plane, 
+   * otherwise it will return false.
+   */
+  virtual bool GetPlaneGeometry (csPlane3& plane) = 0; 
+
+  /**
+   * If collider has plane geometry method will return true and cylinder's
+   * length and radius, otherwise it will return false.
+   */
+  virtual bool GetCylinderGeometry (float& length, float& radius) = 0; 
+
+  /**
+   * Make collider static. Static collider acts on dynamic colliders and bodies,
+   * but ignores other static colliders (it won't do precise collision detection 
+   * in that case).
+   */
+  virtual void MakeStatic () = 0;
+
+  /**
+   * Make collider dynamic. Dynamic colliders acts (it collision is checked, and 
+   * collision callbacks called) on every other collider and body.
+   */
+  virtual void MakeDynamic () = 0;
+
+  /// Check if collider is static. 
+  virtual bool IsStatic () = 0;
 };
 
 SCF_VERSION (iJoint, 0, 0, 1);
@@ -531,15 +787,12 @@ SCF_VERSION (iJoint, 0, 0, 1);
  * instance if all motion in along the local X axis is constrained
  * then the bodies will stay motionless relative to each other
  * along an x axis rotated and positioned by the Joint's transform.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iDynamicSystem::CreateJoint()
- *   </ul>
+ * - iDynamicSystem::CreateJoint()
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>iDynamicSystem
- *   </ul>
+ * - iDynamicSystem
  */
 struct iJoint : public iBase
 {
