@@ -19,9 +19,10 @@
 #ifndef __CS_IVARIA_REPORTER_H__
 #define __CS_IVARIA_REPORTER_H__
 
-#include <stdarg.h>
+#include "csutil/ansicolor.h"
 #include "csutil/scf.h"
 #include "csutil/sysfunc.h"
+#include "csutil/util.h"
 #include "iutil/objreg.h"
 
 /**\file
@@ -73,16 +74,14 @@ SCF_VERSION (iReporterListener, 0, 0, 1);
 /**
  * Implement this interface if you're interested in hearing about
  * new messages on the reporter.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Application.
- *   <li>Standard reporter listener plugin (crystalspace.utilities.stdrep)
- *   </ul>
+ * - Application.
+ * - Standard reporter listener plugin (crystalspace.utilities.stdrep)
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iReporter
- *   </ul>
+ * - iReporter
+ *   
  */
 struct iReporterListener : public iBase
 {
@@ -99,11 +98,10 @@ SCF_VERSION (iReporterIterator, 0, 0, 1);
 
 /**
  * An iterator to iterate over all messages in the reporter.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iReporter::GetMessageIterator()
- *   </ul>
+ * - iReporter::GetMessageIterator()
+ *   
  */
 struct iReporterIterator : public iBase
 {
@@ -141,33 +139,32 @@ SCF_VERSION (iReporter, 0, 1, 0);
  * standard reporter listener (iStandardReporterListener) is such
  * a plugin. It will print out the messages that arrive
  * on the reporter.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Reporter plugin (crystalspace.utilities.reporter)
- *   </ul>
+ * - Reporter plugin (crystalspace.utilities.reporter)
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   </ul>
+ * - CS_QUERY_REGISTRY()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>Entire CS and application.
- *   <li>csReporterHelper
- *   </ul>
+ * - Entire CS and application.
+ * - csReporterHelper
+ *   
  */
 struct iReporter : public iBase
 {
   /**
    * Report something. The given message ID should be formed like:
-   * 'crystalspace.<source>.<type>.<detail>'. Example:
+   * 'crystalspace.{source}.{type}.{detail}'. Example:
    * 'crystalspace.sprite2dloader.parse.material'.
+   * \sa \ref FormatterNotes
    */
   virtual void Report (int severity, const char* msgId,
   	const char* description, ...) CS_GNUC_PRINTF(4, 5) = 0;
 
   /**
    * Report something. va_list version.
+   * \sa \ref FormatterNotes
    */
   virtual void ReportV (int severity, const char* msgId,
   	const char* description, va_list) CS_GNUC_PRINTF(4, 0) = 0;
@@ -220,30 +217,35 @@ struct iReporter : public iBase
 
   /**
    * Report error.
+   * \sa \ref FormatterNotes
    */
   inline void ReportError (const char* msgId, const char* description, ...)
     CS_GNUC_PRINTF (3, 4);
 
   /**
    * Report warning.
+   * \sa \ref FormatterNotes
    */
   inline void ReportWarning (const char* msgId, const char* description, ...)
     CS_GNUC_PRINTF (3, 4);
 
   /**
    * Report notification.
+   * \sa \ref FormatterNotes
    */
   inline void ReportNotify (const char* msgId, const char* description, ...)
     CS_GNUC_PRINTF (3, 4);
 
   /**
    * Report bug.
+   * \sa \ref FormatterNotes
    */
   inline void ReportBug (const char* msgId, const char* description, ...)
     CS_GNUC_PRINTF (3, 4);
 
   /**
    * Report debug.
+   * \sa \ref FormatterNotes
    */
   inline void ReportDebug (const char* msgId, const char* description, ...)
     CS_GNUC_PRINTF (3, 4);
@@ -308,6 +310,7 @@ public:
    * Helper function to use a reporter easily.  This function will also work if
    * no reporter is present and use stdout in that case.
    * \remark You can use the #csReportV macro for even more convenience.
+   * \sa \ref FormatterNotes
    */
   static inline void ReportV(iObjectRegistry* reg, int severity,
     char const* msgId, char const* description, va_list args)
@@ -317,6 +320,7 @@ public:
    * Helper function to use a reporter easily.  This function will also work if
    * no reporter is present and use stdout in that case.
    * \remark You can use the #csReport macro for even more convenience.
+   * \sa \ref FormatterNotes
    */
   static inline void Report(iObjectRegistry* reg, int severity,
     char const* msgId, char const* description, ...)
@@ -332,7 +336,7 @@ inline void csReporterHelper::ReportV(iObjectRegistry* reg, int severity,
   else
   {
     /*
-      @@@ The strncasecmp()s are there because sometimes reported messages
+      @@@ The csStrNCaseCmp()s are there because sometimes reported messages
       start with "Warning", and a "Warning: Warning" output looks rather
       crappy. The correct fix is obviously to remove "Warning" prefixes
       when the reporter is used.
@@ -340,21 +344,21 @@ inline void csReporterHelper::ReportV(iObjectRegistry* reg, int severity,
     switch (severity)
     {
       case CS_REPORTER_SEVERITY_BUG:
-	csPrintf("BUG: ");
+	csPrintf(CS_ANSI_BK CS_ANSI_FM CS_ANSI_FI "BUG: " CS_ANSI_RST);
 	break;
       case CS_REPORTER_SEVERITY_ERROR:
-        if (strncasecmp (description, "error", 5) != 0)
-	  csPrintf("ERROR: ");
+        if (csStrNCaseCmp (description, "error", 5) != 0)
+	  csPrintf(CS_ANSI_BK CS_ANSI_FR CS_ANSI_FI "ERROR: " CS_ANSI_RST);
 	break;
       case CS_REPORTER_SEVERITY_WARNING:
-        if (strncasecmp (description, "warning", 7) != 0)
-	  csPrintf("WARNING: ");
+        if (csStrNCaseCmp (description, "warning", 7) != 0)
+	  csPrintf(CS_ANSI_BK CS_ANSI_FY CS_ANSI_FI "WARNING: " CS_ANSI_RST);
 	break;
       case CS_REPORTER_SEVERITY_NOTIFY:
-        csPrintf("NOTIFY: ");
+	csPrintf ("NOTIFY: ");
 	break;
       case CS_REPORTER_SEVERITY_DEBUG:
-        csPrintf("DEBUG: ");
+	csPrintf(CS_ANSI_BK CS_ANSI_FW CS_ANSI_FI "DEBUG: " CS_ANSI_RST);
 	break;
     }
     csPrintfV(description, args);
