@@ -19,6 +19,10 @@
 #ifndef __CS_IVARIA_SEQUENCE_H__
 #define __CS_IVARIA_SEQUENCE_H__
 
+/**\file
+ * Sequences
+ */
+
 #include "csutil/scf.h"
 
 struct iSequenceManager;
@@ -28,15 +32,13 @@ SCF_VERSION (iSequenceOperation, 0, 2, 0);
 /**
  * A sequence operation. This is effectively a callback
  * to the application.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Application using the sequence manager.
- *   </ul>
+ * - Application using the sequence manager.
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iSequence
- *   </ul>
+ * - iSequence
+ *   
  */
 struct iSequenceOperation : public iBase
 {
@@ -63,15 +65,13 @@ SCF_VERSION (iSequenceCondition, 0, 1, 0);
 /**
  * A sequence condition. This is also a callback to the application.
  * This condition returns true on success.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Application using the sequence manager.
- *   </ul>
+ * - Application using the sequence manager.
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iSequence
- *   </ul>
+ * - iSequence
+ *   
  */
 struct iSequenceCondition : public iBase
 {
@@ -92,6 +92,8 @@ struct csSequenceOp
   csTicks time;
   csRef<iBase> params;
   csRef<iSequenceOperation> operation;
+  // Unique id to identify all operations belonging to a running sequence.
+  uint sequence_id;
 
   csSequenceOp () { }
   ~csSequenceOp () { }
@@ -105,15 +107,13 @@ SCF_VERSION (iSequence, 0, 0, 2);
  * All operations added to this sequence will be executed relative to the
  * time the sequence itself is executed. The execute order of operations
  * added at the same relative time is undefined.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>iSequenceManager::NewSequence()
- *   </ul>
+ * - iSequenceManager::NewSequence()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iSequenceManager
- *   </ul>
+ * - iSequenceManager
+ *   
  */
 struct iSequence : public iBase
 {
@@ -127,14 +127,14 @@ struct iSequence : public iBase
    * on the operation.
    */
   virtual void AddOperation (csTicks time, iSequenceOperation* operation,
-  	iBase* params = 0) = 0;
+  	iBase* params = 0, uint sequence_id = 0) = 0;
 
   /**
    * Add a standard operation to execute another sequence. This function
    * will NOT call IncRef() on the sequence.
    */
   virtual void AddRunSequence (csTicks time, iSequence* sequence,
-  	iBase* params = 0) = 0;
+  	iBase* params = 0, uint sequence_id = 0) = 0;
 
   /**
    * Add a standard operation to perform a condition and execute the right
@@ -143,7 +143,7 @@ struct iSequence : public iBase
    */
   virtual void AddCondition (csTicks time, iSequenceCondition* condition,
   	iSequence* trueSequence, iSequence* falseSequence,
-	iBase* params = 0) = 0;
+	iBase* params = 0, uint sequence_id = 0) = 0;
 
   /**
    * Perform the sequence for as long as the condition is valid.
@@ -151,7 +151,7 @@ struct iSequence : public iBase
    * sequence.
    */
   virtual void AddLoop (csTicks time, iSequenceCondition* condition,
-  	iSequence* sequence, iBase* params = 0) = 0;
+  	iSequence* sequence, iBase* params = 0, uint sequence_id = 0) = 0;
 
   /**
    * Clear all operations in this sequence (call DecRef()).
@@ -170,19 +170,16 @@ SCF_VERSION (iSequenceManager, 0, 1, 0);
  * The sequence manager. The sequence manager is a plugin that will perform
  * sequences of operations depending on elapsed time. It is mostly useful
  * for demo's or intros of games.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Sequence Manager plugin (crystalspace.utilities.sequence)
- *   </ul>
+ * - Sequence Manager plugin (crystalspace.utilities.sequence)
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   </ul>
+ * - CS_QUERY_REGISTRY()
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>iEngineSequenceManager
- *   </ul>
+ * - iEngineSequenceManager
+ *   
  */
 struct iSequenceManager : public iBase
 {
@@ -265,14 +262,33 @@ struct iSequenceManager : public iBase
    * Modifications on a sequence after it has been added have no effect.
    * You can also remove the sequence (with DecRef()) immediatelly after
    * running it.
-   * <p>
+   * 
    * The optional params instance will be given to all operations that
    * are added on the main sequence. Ref counting is used to keep track
    * of this object. So you can safely DecRef() your own reference after
    * calling RunSequence.
+   *
+   * \param time Time at which the sequence should be executed.
+   * \param sequence Sequence to execute.
+   * \param params Optional instance given to all operations added to the
+   *   main sequence.
+   * \param sequence_id This identifier can be used to get track of
+   *   a given sequence. You can use this id to remove all operations that
+   *   have this id.
    */
   virtual void RunSequence (csTicks time, iSequence* sequence,
-  	iBase* params = 0) = 0;
+  	iBase* params = 0, uint sequence_id = 0) = 0;
+
+  /**
+   * Destroy all operations with a given sequence id.
+   */
+  virtual void DestroySequenceOperations (uint sequence_id) = 0;
+
+  /**
+   * Return a unique id that you can use for identifying the sequence
+   * operations.
+   */
+  virtual uint GetUniqueID () = 0;
 };
 
 #endif // __CS_IVARIA_SEQUENCE_H__
