@@ -20,9 +20,11 @@
 #ifndef __CS_IVARIA_VIEW_H__
 #define __CS_IVARIA_VIEW_H__
 
-#include "csutil/scf.h"
+/**\file
+ * Renderer view interface
+ */
 
-SCF_VERSION (iView, 0, 1, 0);
+#include "csutil/scf_interface.h"
 
 struct iCamera;
 struct iEngine;
@@ -32,22 +34,20 @@ struct iClipper2D;
 /**
  * The iView class encapsulates the top-level Crystal Space
  * renderer interface. It is basically a camera and a clipper.
- * <p>
+ * 
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>Applications using csView.
- *   </ul>
+ * - Applications using csView.
+ *   
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>Application stores it.
- *   </ul>
+ * - Application stores it.
+ *   
  * Main users of this interface:
- *   <ul>
- *   <li>Application uses it.
- *   </ul>
+ * - Application uses it.
+ *   
  */
-struct iView : public iBase
+struct iView : public virtual iBase
 {
+  SCF_INTERFACE(iView, 2,0,0);
   /// Get engine handle.
   virtual iEngine* GetEngine () = 0;
   /// Set engine handle.
@@ -63,7 +63,12 @@ struct iView : public iBase
   /// Set Context
   virtual void SetContext (iGraphics3D *ig3d) = 0;
 
-  /// Set clipping rectangle.
+  /**
+   * Set clipping rectangle.
+   * \remarks The coordinates are vertically mirrored in comparison to screen
+   *   space, i.e. y=0 is at the bottom of the viewport, y=GetHeight() at the 
+   *   top.
+   */
   virtual void SetRectangle (int x, int y, int w, int h) = 0;
   /// Clear clipper in order to start building a polygon-based clipper.
   virtual void ClearView () = 0;
