@@ -28,20 +28,19 @@
  * \addtogroup gfx2d
  * @{ */
  
-#include <stdarg.h>
 #include "csutil/scf.h"
 #include "csgfx/rgbpixel.h"
-#include "ivideo/texture.h"
-#include "iengine/texture.h"
 #include "ivideo/cursor.h"
 
-class csRect;
+
 struct iImage;
 struct iFontServer;
 struct iFont;
 struct iNativeWindow;
 struct iGraphics2D;
 
+class csRect;
+
 /// iGraphics2D::Write() flags.
 enum
 {
@@ -53,7 +52,7 @@ enum
   /**
    * Don't use anti-aliased glyphs.
    */
-  CS_WRITE_NOANTIALIAS = (1 << 1), 
+  CS_WRITE_NOANTIALIAS = (1 << 1)
 };
 
 /// Simple 2D pixel coordinate
@@ -75,14 +74,14 @@ struct csPixelFormat
    * only). Ignore the Mask and Shift fields of this structure if
    * PalEntries != 0.
    */
-  uint32 RedMask, GreenMask, BlueMask;
+  uint32 RedMask, GreenMask, BlueMask, AlphaMask;
   /**
    * The shifts to extract the color information from a pixel (truecolor mode
    * only).
    */
-  int RedShift, GreenShift, BlueShift;
+  int RedShift, GreenShift, BlueShift, AlphaShift;
   /// The number of significant bits for every color.
-  int RedBits, GreenBits, BlueBits;
+  int RedBits, GreenBits, BlueBits, AlphaBits;
 
   /**
    * Number of palette entries. 0 for truecolor, else the number of palette
@@ -95,11 +94,9 @@ struct csPixelFormat
   /**
    * Number of bytes for every pixel.
    * The only supported values currently are:<p>
-   * <ul>
-   * <li>1: for palette mode (256 palette entries)
-   * <li>2: for truecolor 15/16-bit mode (no palette entries)
-   * <li>4: for truecolor 32-bit mode (no palette entries)
-   * </ul>
+   * - 1: for palette mode (256 palette entries)
+   * - 2: for truecolor 15/16-bit mode (no palette entries)
+   * - 4: for truecolor 32-bit mode (no palette entries)
    */
   int PixelBytes;
 
@@ -120,6 +117,7 @@ struct csPixelFormat
     COMPUTE (Red);
     COMPUTE (Green);
     COMPUTE (Blue);
+    COMPUTE (Alpha);
 #undef COMPUTE
   }
 };
@@ -149,37 +147,33 @@ struct iOffscreenCanvasCallback : public iBase
   virtual void SetRGB (iGraphics2D* canvas, int idx, int r, int g, int b) = 0;
 };
 
-SCF_VERSION (iGraphics2D, 2, 4, 0);
-
 /**
  * This is the interface for 2D renderer. The 2D renderer is responsible
  * for all 2D operations such as creating the window, switching pages,
  * returning pixel format and so on.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>OpenGL/Windows canvas plugin (crystalspace.graphics2d.glwin32)
- *   <li>OpenGL/X11 canvas plugin (crystalspace.graphics2d.glx)
- *   <li>DirectDraw canvas plugin (crystalspace.graphics2d.directdraw)
- *   <li>X11 canvas plugin (crystalspace.graphics2d.x2d)
- *   <li>Memory canvas plugin (crystalspace.graphics2d.memory)
- *   <li>Null 2D canvas plugin (crystalspace.graphics2d.null2d)
- *   <li>Some others.
- *   <li>Note that it is the 3D renderer that will automatically create
+ * - OpenGL/Windows canvas plugin (crystalspace.graphics2d.glwin32)
+ * - OpenGL/X11 canvas plugin (crystalspace.graphics2d.glx)
+ * - DirectDraw canvas plugin (crystalspace.graphics2d.directdraw)
+ * - X11 canvas plugin (crystalspace.graphics2d.x2d)
+ * - Memory canvas plugin (crystalspace.graphics2d.memory)
+ * - Null 2D canvas plugin (crystalspace.graphics2d.null)
+ * - Some others.
+ * - Note that it is the 3D renderer that will automatically create
  *       the right instance of the canvas that it requires.
- *   </ul>
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   <li>iGraphics3D::GetDriver2D()
- *   </ul>
+ * - CS_QUERY_REGISTRY()
+ * - iGraphics3D::GetDriver2D()
+ *
  * Main users of this interface:
- *   <ul>
- *   <li>3D renderers (iGraphics3D implementations)
- *   </ul>
+ * - 3D renderers (iGraphics3D implementations)
  */
-struct iGraphics2D : public iBase
+struct iGraphics2D : public virtual iBase
 {
+  SCF_INTERFACE (iGraphics2D, 3, 0, 0);
+  
   /// Open the device.
   virtual bool Open () = 0;
 
@@ -223,11 +217,30 @@ struct iGraphics2D : public iBase
   /// Get the palette (if there is one)
   virtual csRGBpixel *GetPalette () = 0;
 
-  /// Set a color index to given R,G,B (0..255) values
+  /**
+   * Set a color index to given R,G,B (0..255) values. Only use if there
+   * is a palette.
+   */
   virtual void SetRGB (int i, int r, int g, int b) = 0;
-  /// Find an RGB color.
-  virtual int FindRGB (int r, int g, int b) = 0;
+  /**
+   * Find an RGB (0..255) color. If there is a palette, this returns an
+   * entry index set with SetRGB(). If the returned value is -1, a
+   * suitable palette entry was not found.
+   * Without a palette, the actual color bytes are returned.
+   * <p>
+   * Use returned value for color arguments in iGraphics2D.
+   */
+  virtual int FindRGB (int r, int g, int b, int a = 255) = 0;
 
+  /**
+   * Retrieve the R,G,B tuple for a given color index.
+   */
+  virtual void GetRGB (int color, int& r, int& g, int& b) = 0;
+  /**
+   * Retrieve the R,G,B,A tuple for a given color index.
+   */
+  virtual void GetRGB (int color, int& r, int& g, int& b, int& a) = 0;
+  
   /**
    * Set clipping rectangle.
    * The clipping rectangle is inclusive the top and left edges and exclusive
@@ -289,6 +302,9 @@ struct iGraphics2D : public iBase
 
   /// Query pixel R,G,B at given screen location
   virtual void GetPixel (int x, int y, uint8 &oR, uint8 &oG, uint8 &oB) = 0;
+  /// As GetPixel() above, but with alpha
+  virtual void GetPixel (int x, int y, uint8 &oR, uint8 &oG, uint8 &oB,
+  	uint8 &oA) = 0;
 
   /**
    * Save a subarea of screen and return a handle to saved buffer.
@@ -304,21 +320,25 @@ struct iGraphics2D : public iBase
   virtual void FreeArea (csImageArea *Area) = 0;
 
   /**
-   * Write a text string into the back buffer. A negative value for bg
+   * Write a text string into the back buffer. A value of -1 for \p bg
    * color will not draw the background.
+   * \remarks \p str is expected to be UTF-8 encoded.
+   * \remarks For transparent backgrounds, it is recommended to obtain a color
+   *  value from FindRGB() that has the same R, G, B components as the 
+   *  foreground color, but an alpha component of 0.
    */
   virtual void Write (iFont *font, int x, int y, int fg, int bg,
     const char *str, uint flags = 0) = 0;
 
   /**
-   * Write a text string into the back buffer. A negative value for bg
+   * Write a text string into the back buffer. A value of -1 for \p bg
    * color will not draw the background. x and y are the pen position on
    * a baseline. The actual font baseline is shifted up by the font's descent.
    * \deprecated
    * Instead, use Write() with the #CS_WRITE_BASELINE flag set.
    */
-  virtual void WriteBaseline (iFont *font, int x, int y, int fg, int bg,
-    const char *str) = 0;
+  CS_DEPRECATED_METHOD virtual void WriteBaseline (iFont *font, 
+    int x, int y, int fg, int bg, const char *str) = 0;
 
   /// Enable/disable canvas resizing
   virtual void AllowResize (bool iAllow) = 0;
@@ -382,7 +402,7 @@ struct iGraphics2D : public iBase
    * all black colors in the image will become the value of 'bg' and all 
    * non-black colors will become 'fg'
    */
-  virtual bool SetMouseCursor (iImage *image, csRGBcolor keycolor, 
+  virtual bool SetMouseCursor (iImage *image, const csRGBcolor* keycolor = 0, 
                                int hotspot_x = 0, int hotspot_y = 0,
                                csRGBcolor fg = csRGBcolor(255,255,255),
                                csRGBcolor bg = csRGBcolor(0,0,0)) = 0;
@@ -399,6 +419,11 @@ struct iGraphics2D : public iBase
    */
   virtual float GetGamma () const = 0;
 
+  /**
+   * Get the name of the canvas
+   */
+  virtual const char* GetName () const = 0;
+
   /**
    * Create an off-screen canvas so you can render on a given memory
    * area. If depth==8 then the canvas will use palette mode. In that
