@@ -1,5 +1,6 @@
 /*
     Copyright (C) 1998-2001 by Jorrit Tyberghein
+                       2004 by Marten Svanfeldt
     Written by Jorrit Tyberghein, Dan Ogles, and Gary Clark.
 
     This library is free software; you can redistribute it and/or
@@ -28,47 +29,35 @@
  * \addtogroup gfx3d
  * @{ */
  
-#include "csutil/refarr.h"
 #include "csutil/scf.h"
-#include "csgeom/plane3.h"
-#include "csgeom/vector2.h"
+
 #include "csgeom/transfrm.h"
-#include "csgeom/tri.h"
-#include "csutil/cscolor.h"
+#include "csutil/flags.h"
+#include "csutil/strset.h"
+
 #include "ivideo/rndbuf.h"
-#include "ivideo/polyrender.h"
+
+struct iClipper2D;
+struct iGraphics2D;
+struct iHalo;
+struct iRenderBuffer;
+struct iRendererLightmap;
+struct iShader;
+struct iShaderVariableContext;
+struct iTextureHandle;
+struct iTextureManager;
+
+class csRect;
+class csPlane3;
+class csShaderVariable;
+class csVector2;
+class csVector3;
+class csVector4;
+
 
 struct csCoreRenderMesh;
 struct csRenderMeshModes;
-class csMatrix3;
-class csVector4;
-class csVector3;
-class csVector2;
-class csPlane3;
-class csRect;
-class csReversibleTransform;
-class csShaderVariable;
-
-struct iGraphics2D;
-struct iPolygonBuffer;
-struct iPolygonRenderer;
-struct iVertexBuffer;
-struct iVertexBufferManager;
-struct iTextureManager;
-struct iTextureHandle;
-struct iMaterialHandle;
-struct iMaterial;
-struct iClipper2D;
-struct iHalo;
-struct iRendererLightmap;
-struct csRGBpixel;
-struct csPixelFormat;
-struct csPolyTextureMapping;
-struct iRenderBuffer;
-struct iRenderBufferManager;
-struct iLightingManager;
-struct iShader;
-struct iShaderVariableContext;
+class csRenderBufferHolder;
 
 
 /**\name iGraphics3D::BeginDraw() flags
@@ -118,60 +107,41 @@ struct iShaderVariableContext;
 #define CS_CLIP_NEEDED 1
 /** @} */
 
-/**
- * Fog structure.
- */
-struct csFog
-{
-  /// If true then fog is enabled.
-  bool enabled;
-  /// Density (0 is off).
-  float density;
-  /// Color (red).
-  float red;
-  /// Color (green).
-  float green;
-  /// Color (blue).
-  float blue;
-};
-
 /// Z-buffer modes
 enum csZBufMode
 {
-  // values below are sometimes used as bit masks, so don't change them!
-  /// Don't test/write
+  /// Don't test or write
   CS_ZBUF_NONE     = 0x00000000,
-  /// write
+  /// Write unconditionally
   CS_ZBUF_FILL     = 0x00000001,
-  /// test
+  /// Test only
   CS_ZBUF_TEST     = 0x00000002,
-  /// write/test
+  /// Test, write if successful
   CS_ZBUF_USE      = 0x00000003,
-  /// only write
-  CS_ZBUF_FILLONLY = 0x00000004,
-  /// test if equal
-  CS_ZBUF_EQUAL    = 0x00000005,
-  /// inverted test
-  CS_ZBUF_INVERT   = 0x00000006,
-  /// use the z mode of the mesh (NOTE: NOT VALID AS MESH ZMODE)
-  CS_ZBUF_MESH     = 0x00000007,
+  /// Test if equal
+  CS_ZBUF_EQUAL    = 0x00000004,
+  /// Inverted test
+  CS_ZBUF_INVERT   = 0x00000005,
+  
+  /// Use the z mode of the render mesh (NOTE: NOT VALID AS MESH ZMODE)
+  CS_ZBUF_MESH     = 0x80000000,
   /**
-   * Use a z mode depending on the mesh zmode.
+   * Use a "pass 2" z mode depending on the render mesh zmode.
    * The mesh zmode is used a to choose a zmode that makes sure only pixels
    * that are changed by the mesh zmode can be touched, e.g. if the mesh has a
    * zmode of "zuse", zmesh2 will resolve to "ztest". This is useful for multi-
    * pass stuff.
    * (NOTE: NOT VALID AS MESH ZMODE)
    */
-  CS_ZBUF_MESH2    = 0x00000008
+  CS_ZBUF_MESH2    = 0x80000001
 };
 
 // @@@ Keep in sync with values below
 // @@@ Document me better!
-#define CS_VATTRIB_GENERIC_FIRST     0
-#define CS_VATTRIB_GENERIC_LAST     15
-#define CS_VATTRIB_SPECIFIC_FIRST  100
-#define CS_VATTRIB_SPECIFIC_LAST   (CS_VATTRIB_SPECIFIC_FIRST + 15)
+#define CS_VATTRIB_SPECIFIC_FIRST    0
+#define CS_VATTRIB_SPECIFIC_LAST    15
+#define CS_VATTRIB_GENERIC_FIRST   100
+#define CS_VATTRIB_GENERIC_LAST    (CS_VATTRIB_GENERIC_FIRST + 15)
 
 #define CS_VATTRIB_IS_GENERIC(va)   \
   ((va >= CS_VATTRIB_GENERIC_FIRST) && (va <=CS_VATTRIB_GENERIC_LAST))
@@ -179,32 +149,12 @@ enum csZBufMode
   ((va >= CS_VATTRIB_SPECIFIC_FIRST) && (va <=CS_VATTRIB_SPECIFIC_LAST))
 
 /**
- * For NR:
  * Vertex attributes.
  */
 enum csVertexAttrib
 {
-  //@{
-  /**
-   * General vertex attribute
-   */
-  CS_VATTRIB_0	= 0,
-  CS_VATTRIB_1	= 1,
-  CS_VATTRIB_2	= 2,
-  CS_VATTRIB_3	= 3,
-  CS_VATTRIB_4	= 4,
-  CS_VATTRIB_5	= 5,
-  CS_VATTRIB_6	= 6,
-  CS_VATTRIB_7	= 7,
-  CS_VATTRIB_8	= 8,
-  CS_VATTRIB_9	= 9,
-  CS_VATTRIB_10 = 10,
-  CS_VATTRIB_11 = 11,
-  CS_VATTRIB_12 = 12,
-  CS_VATTRIB_13 = 13,
-  CS_VATTRIB_14 = 14,
-  CS_VATTRIB_15 = 15,
-  //@}
+  /// Invalid attribute
+  CS_VATTRIB_INVALID	      = -1,
   /// Position vertex attribute
   CS_VATTRIB_POSITION	      = CS_VATTRIB_SPECIFIC_FIRST + 0,
   /// Vertex weight attribute
@@ -237,57 +187,224 @@ enum csVertexAttrib
   CS_VATTRIB_TEXCOORD6	      = CS_VATTRIB_SPECIFIC_FIRST + 14,
   /// TU 7 texture coordinates
   CS_VATTRIB_TEXCOORD7	      = CS_VATTRIB_SPECIFIC_FIRST + 15,
-};
-
-/// 
-enum G3D_FOGMETHOD
-{
-  G3DFOGMETHOD_NONE = 0x00,
-  G3DFOGMETHOD_ZBUFFER = 0x01,
-  G3DFOGMETHOD_VERTEX = 0x02
-};
-
-/**\name Mix modes for DrawPolygonFX ()
- * The constants can be ORed together if they belong to different masks.
- * @{ */
-/// SRC/DST mixing mode mask
-#define CS_FX_MASK_MIXMODE 0xF0000000 
-/// =SRC
-#define CS_FX_COPY         0x00000000 
-/// =SRC*DST
-#define CS_FX_MULTIPLY     0x10000000 
-/// =2*SRC*DST
-#define CS_FX_MULTIPLY2    0x20000000 
-/// =SRC+DST
-#define CS_FX_ADD          0x30000000 
-/// =(1-alpha)*SRC + alpha*DST
-#define CS_FX_ALPHA        0x40000000 
-/// =DST
-#define CS_FX_TRANSPARENT  0x50000000 
-/// =(dstalpha)*SRC + DST
-#define CS_FX_DESTALPHAADD 0x60000000 
-/// =(srcalpha)*SRC + DST
-#define CS_FX_SRCALPHAADD  0x70000000
-/**
-  * Use the mix mode of the mesh zmode.
-  * (NOTE: NOT VALID AS MESH ZMODE - only for shader pass mixmodes)
+  //@{
+  /**
+  * General vertex attribute
   */
-#define CS_FX_MESH	   0xf0000000
-/// color 0 is transparent
-#define CS_FX_KEYCOLOR     0x08000000 
-/// Flat shading
-#define CS_FX_FLAT         0x04000000 
-/// Tiling
-#define CS_FX_TILING       0x02000000 
-/// alpha = 0..FF (opaque..transparent)
-#define CS_FX_MASK_ALPHA   0x000000FF 
+  CS_VATTRIB_0	= CS_VATTRIB_GENERIC_FIRST + 0,
+  CS_VATTRIB_1	= CS_VATTRIB_GENERIC_FIRST + 1,
+  CS_VATTRIB_2	= CS_VATTRIB_GENERIC_FIRST + 2,
+  CS_VATTRIB_3	= CS_VATTRIB_GENERIC_FIRST + 3,
+  CS_VATTRIB_4	= CS_VATTRIB_GENERIC_FIRST + 4,
+  CS_VATTRIB_5	= CS_VATTRIB_GENERIC_FIRST + 5,
+  CS_VATTRIB_6	= CS_VATTRIB_GENERIC_FIRST + 6,
+  CS_VATTRIB_7	= CS_VATTRIB_GENERIC_FIRST + 7,
+  CS_VATTRIB_8	= CS_VATTRIB_GENERIC_FIRST + 8,
+  CS_VATTRIB_9	= CS_VATTRIB_GENERIC_FIRST + 9,
+  CS_VATTRIB_10 = CS_VATTRIB_GENERIC_FIRST + 10,
+  CS_VATTRIB_11 = CS_VATTRIB_GENERIC_FIRST + 11,
+  CS_VATTRIB_12 = CS_VATTRIB_GENERIC_FIRST + 12,
+  CS_VATTRIB_13 = CS_VATTRIB_GENERIC_FIRST + 13,
+  CS_VATTRIB_14 = CS_VATTRIB_GENERIC_FIRST + 14,
+  CS_VATTRIB_15 = CS_VATTRIB_GENERIC_FIRST + 15
+  //@}
+};
 
+/**\name Mix mode: Types
+ * The mix mode specifies how a shaded fragment (denoted as \c SRC) is mixed 
+ * (or \e blended) with the framebuffer fragment (\c DST).
+ * @{ */
+/**
+ * Automatic blending mode. Whether the texture is alpha-blended or not is 
+ * taken from csRenderMesh::alphaMode and whether the alpha part of the
+ * mixmode is non-zero.
+ */
+#define CS_MIXMODE_TYPE_AUTO (0x00000000)
+/**
+ * Blend with a blending operation.
+ * The fragment value written to the framebuffer is computed from the 
+ * formula \c SRC * \c srcFactor + \c DST * \c dstFactor. 
+ * \c srcFactor and \c dstFactor are one of 
+ * \link #CS_MIXMODE_FACT_ZERO CS_MIXMODE_FACT_xxx \endlink, encoded into
+ * the mixmode specifier with #CS_MIXMODE_BLEND.
+ */
+#define CS_MIXMODE_TYPE_BLENDOP (0x40000000)
+/**
+ * Use the mix mode of the mesh mix mode.
+ * \warning NOT VALID AS MESH MIXMODE - only for shader pass mixmodes.
+ */
+#define CS_MIXMODE_TYPE_MESH (0x80000000)
+
+/// Bit mask to extract the type from a mixmode specifier.
+#define CS_MIXMODE_TYPE_MASK (0xc0000000)
+/** @} */
+
+/// Mix mode: Blending op factors
+enum
+{
+  /// 0
+  CS_MIXMODE_FACT_ZERO		= 0x0,
+  /// -(e^(i*Pi))
+  CS_MIXMODE_FACT_ONE		= 0x1,
+  /// Source fragment (R,G,B,A) components
+  CS_MIXMODE_FACT_SRCCOLOR	= 0x2,
+  /// Source fragment (1-R,1-G,1-B,-1A) components
+  CS_MIXMODE_FACT_SRCCOLOR_INV	= 0x3,
+  /// Destination fragment (R,G,B,A) components
+  CS_MIXMODE_FACT_DSTCOLOR	= 0x4,
+  /// Destination fragment (1-R,1-G,1-B,-1A) components
+  CS_MIXMODE_FACT_DSTCOLOR_INV	= 0x5,
+  /// Source fragment alpha
+  CS_MIXMODE_FACT_SRCALPHA	= 0x6,
+  /// Source fragment 1-alpha
+  CS_MIXMODE_FACT_SRCALPHA_INV	= 0x7,
+  /// Destination fragment alpha
+  CS_MIXMODE_FACT_DSTALPHA	= 0x8,
+  /// Destination fragment 1-alpha
+  CS_MIXMODE_FACT_DSTALPHA_INV	= 0x9,
+  
+  /// Number of available mixmodes
+  CS_MIXMODE_FACT_COUNT		= 0xa,
+  
+  /// Mask to extract factors
+  CS_MIXMODE_FACT_MASK		= 0xf
+};
+
+/**\name Mix mode: Alpha test flags
+ * Enabled alpha test (or <i>binary alpha</i>) means that a fragment is only
+ * drawn when its alpha component is above a certain threshold, and discarded
+ * otherwise.
+ * @{ */
+/**
+ * Automatic alpha test. Whether the texture is alpha-blended binarily is 
+ * taken from csRenderMesh::alphaMode and whether the alpha part of the
+ * mixmode is non-zero.
+ */
+#define CS_MIXMODE_ALPHATEST_AUTO (0x00000000)
+/// Unconditionally enable alpha test.
+#define CS_MIXMODE_ALPHATEST_ENABLE (0x10000000)
+/// Unconditionally disable alpha test.
+#define CS_MIXMODE_ALPHATEST_DISABLE (0x20000000)
+  
+/// Bit mask to extract the alpha test flag from a mixmode specifier.
+#define CS_MIXMODE_ALPHATEST_MASK (0x30000000)
+/** @} */
+
+/**\name Mix mode: Blending mode helpers
+ * @{ */
+/**
+ * Helper macro to construct a blending operation mixmode
+ * \a Src and \a Dst are 
+ * \link #CS_MIXMODE_FACT_ZERO blending op factors \endlink, however sans the
+ * CS_MIXMODE_FACT_ prefix. E.g.:
+ * \code
+ * uint mixmode = CS_MIXMODE_BLEND(SRCALPHA, SRCALPHA_INV);
+ * \endcode
+ * will generate a blending operation for alpha blending.
+ */
+#define CS_MIXMODE_BLEND(Src, Dst)					\
+  (CS_MIXMODE_TYPE_BLENDOP 						\
+  | (CS_MIXMODE_FACT_ ## Src << 20) | (CS_MIXMODE_FACT_ ## Dst << 16))
+/// Helper macro to extract the \c srcFactor from a blending op mixmode.
+#define CS_MIXMODE_BLENDOP_SRC(mode)	((mode >> 20) & CS_MIXMODE_FACT_MASK)
+/// Helper macro to extract the \c dstFactor from a blending op mixmode.
+#define CS_MIXMODE_BLENDOP_DST(mode)	((mode >> 16) & CS_MIXMODE_FACT_MASK)
+/** @} */
+
+/**\name Mix mode: Default modes
+ * A set of commonly used mix modes.
+ * @{ */
+/**
+ * This mixmode uses alpha smooth blending, binary blending (ie enabled
+ * alpha test) and no blending depending on the contents of 
+ * csRenderMesh::alphaMode.
+ * \remarks For a "true" copy mixmode that just writes the source fragment
+ *   color to the frame buffer, you can use:
+ *   \code
+ *   CS_MIXMODE_BLEND(ONE, ZERO) | CS_MIXMODE_ALPHATEST_DISABLE
+ *   \endcode
+ */
+#define CS_FX_COPY (CS_MIXMODE_TYPE_AUTO | CS_MIXMODE_ALPHATEST_AUTO)
+/// Multiplicative blending. Formula: <tt>=SRC*DST</tt>
+#define CS_FX_MULTIPLY \
+    (CS_MIXMODE_BLEND(DSTCOLOR, ZERO) | CS_MIXMODE_ALPHATEST_DISABLE)
+/// Multiplicative doubling blending. Formula: <tt>=2*SRC*DST</tt>
+#define CS_FX_MULTIPLY2 \
+    (CS_MIXMODE_BLEND(DSTCOLOR, SRCCOLOR) | CS_MIXMODE_ALPHATEST_DISABLE)
+/// Additive blending. Formula: <tt>=SRC+DST</tt>
+#define CS_FX_ADD \
+    (CS_MIXMODE_BLEND(ONE, ONE) | CS_MIXMODE_ALPHATEST_DISABLE)
+/** 
+ * Alpha blending. Formula: <tt>=(1-srcAlpha)*SRC + srcAlpha*DST</tt>
+ * \remarks Usually used with a non-zero alpha part.
+ *  \see CS_FX_MASK_ALPHA, \see CS_FX_SETALPHA
+ */
+#define CS_FX_ALPHA \
+    (CS_MIXMODE_BLEND(SRCALPHA, SRCALPHA_INV) | CS_MIXMODE_ALPHATEST_DISABLE)
+/**
+ * Transparent blending (keep framebuffer unmodified). 
+ * Formula: <tt>=DST</tt>
+ * \remarks The Z buffer will still be modified!
+ */
+#define CS_FX_TRANSPARENT \
+    (CS_MIXMODE_BLEND(ZERO, ONE) | CS_MIXMODE_ALPHATEST_DISABLE)
+/** 
+ * Multiply source color with destination alpha and add.
+ * Formula: <tt>=(dstalpha)*SRC + DST</tt>
+ */
+#define CS_FX_DESTALPHAADD \
+    (CS_MIXMODE_BLEND(DSTALPHA, ONE) | CS_MIXMODE_ALPHATEST_DISABLE)
+/** 
+ * Multiply source color with source alpha and add.
+ * Formula: <tt>=(srcalpha)*SRC + DST</tt>
+ */
+#define CS_FX_SRCALPHAADD \
+    (CS_MIXMODE_BLEND(SRCALPHA, ONE) | CS_MIXMODE_ALPHATEST_DISABLE)
+/** 
+ * Multiply destination color with inverse source alpha and add source color.
+ * Formula: <tt>=SRC + DST*(1-srcalpha)</tt>
+ * \remarks When the \c SRC alpha component was multiplied into the source
+ *  color, this acts like alpha blending; if it was not, it acts like
+ *  additive blending. Hence, this mixmode can be used to use both additive
+ *  and alpha blending on the same triangle even and interpolate between
+ *  those two "extremes" by appropriate choice of the color and alpha values.
+ */
+#define CS_FX_PREMULTALPHA \
+    (CS_MIXMODE_BLEND(ONE, SRCALPHA_INV) | CS_MIXMODE_ALPHATEST_DISABLE)
+/**
+ * Use the mix mode of the mesh mix mode.
+ * \warning NOT VALID AS MESH MIXMODE - only for shader pass mixmodes.
+ */
+#define CS_FX_MESH (CS_MIXMODE_TYPE_MESH)
+  
+/// Flat shading flag.
+#define CS_FX_FLAT (0x04000000)
+/**
+ * Mixmode alpha part. 
+ * Values range from 0(opaque) to 255 (transparent) (note that this is the
+ * reverse of the "common" alpha meaning). When this part is non-null, the
+ * renderer will scale the incoming color buffer alpha components by the
+ * (inverse of) this value.
+ * \remarks The scaling may be relatively expensive, it is recommended to
+ *  scale the vertex alpha by other means, e.g. through a vertex program.
+ */
+#define CS_FX_MASK_ALPHA (0x000000FF)
+  
+/**
+ * Bit mask for bits relevant to mix mode comparison; contains type, alpha
+ * test flags and blending op factors.
+ */
+#define CS_FX_MASK_MIXMODE (0xf0ff0000)
+/** @} */
+
+/**\name Mix mode: alpha helpers
+ * @{ */
 /// Macro for setting of alpha bits into mixmode (alpha between 0 and 1).
 #define CS_FX_SETALPHA(alpha) \
-  (CS_FX_ALPHA | uint (alpha * CS_FX_MASK_ALPHA))
+  (CS_FX_ALPHA | uint ((alpha) * CS_FX_MASK_ALPHA))
 /// Macro for setting of alpha bits into mixmode (alpha between 0 and 255).
 #define CS_FX_SETALPHA_INT(alpha) \
-  (CS_FX_ALPHA | uint (alpha & CS_FX_MASK_ALPHA))
+  (CS_FX_ALPHA | uint ((alpha) & CS_FX_MASK_ALPHA))
 /** @} */
 
 /**
@@ -315,6 +432,7 @@ struct csAlphaMode
     csStringID autoModeTexture;
   };
 };
+/** @} */
 
 /**\name Light parameters
  * @{ */
@@ -385,268 +503,24 @@ struct csGraphics3DCaps
   bool CanClip;
   int minTexHeight, minTexWidth;
   int maxTexHeight, maxTexWidth;
-  G3D_FOGMETHOD fog;
   bool NeedsPO2Maps;
   int MaxAspectRatio;
+
+  // The following caps are only used by NR
+
+  /**
+   * Whether point sprites are supported. If \a true, geometry of the type
+   * CS_MESHTYPE_POINT_SPRITES can be drawn.
+   */
   bool SupportsPointSprites;
-};
-
-//===========================================================================
-// Not for new renderer! @@@NR@@@
-// The stuff below is either to be ported to new renderer or else
-// it will be removed.
-//===========================================================================
-
-#define CS_FOG_FRONT  0
-#define CS_FOG_BACK   1
-#define CS_FOG_VIEW   2
-
-//======================================================================
-// For vertex based fog the following defines are used:
-#define CS_FOGTABLE_SIZE 64
-
-// Each texel in the fog table holds the fog alpha value at a certain
-// (distance*density).  The median distance parameter determines the
-// (distance*density) value represented by the texel at the center of
-// the fog table.  The fog calculation is:
-// alpha = 1.0 - exp( -(density*distance) / CS_FOGTABLE_MEDIANDISTANCE)
-#define CS_FOGTABLE_MEDIANDISTANCE 10.0f
-#define CS_FOGTABLE_MAXDISTANCE (CS_FOGTABLE_MEDIANDISTANCE * 2.0f)
-#define CS_FOGTABLE_DISTANCESCALE (1.0f / CS_FOGTABLE_MAXDISTANCE)
-
-// Fog (distance*density) is mapped to a texture coordinate and then
-// clamped.  This determines the clamp value.  Some drivers don't
-// like clamping textures so we must do it ourself
-#define CS_FOGTABLE_CLAMPVALUE 0.85f
-#define CS_FOG_MAXVALUE (CS_FOGTABLE_MAXDISTANCE * CS_FOGTABLE_CLAMPVALUE)
-//======================================================================
-
-/// Extra information for vertex fogging.
-class G3DFogInfo
-{
-public:
-  /// Color.
-  float r, g, b;
   /**
-   * Intensity (== density * thickness).
-   * The second intensity value is always 0 and is put there
-   * to make it easier for 3D implementations to just use the
-   * two values below as a coordinate in a texture of 64*1.
+   * Mixmodes utilizing destination alpha are properly supported.
    */
-  float intensity;
-  float intensity2;
-};
-
-/// Information about a texture plane.
-class G3DCam2TextureTransform
-{
-public:
-  /// Transformation from camera space to texture space.
-  csMatrix3* m_cam2tex;
-  /// Transformation from camera space to texture space.
-  csVector3* v_cam2tex;
-};
-
-/// Structure containing all info needed by DrawPolygonFX (DPFX)
-struct G3DPolygonDPFX
-{
-  /// Current number of vertices.
-  int num;
-  /// Vertices that form the polygon.
-  csVector2 vertices[100];
-  /// 1/z for every vertex.
-  float z[100];
-  /// Texels per vertex.
-  csVector2 texels[100];
-  /// Lighting info per vertex.
-  csColor colors[100];
-
-  /// Extra optional fog information.
-  G3DFogInfo fog_info[100];
-  /// Use fog info?
-  bool use_fog;
-
-  /// The material handle as returned by iTextureManager.
-  iMaterialHandle *mat_handle;
-  /// Mixmode to use. If CS_FX_COPY then no mixmode is used.
-  uint mixmode;
-
-  //@{
-  /// Use this color for drawing (if txt_handle == 0) instead of a material.
-  uint8 flat_color_r;
-  uint8 flat_color_g;
-  uint8 flat_color_b;
-  //@}
-
-  // A dummy constructor to appease NextStep compiler which otherwise
-  // complains that it is unable to create this object.  This happens when
-  // a subcomponent such as csVector2 has a constructor.
-  G3DPolygonDPFX() {}
-};
-
-/// Structure containing all info needed by DrawFogPolygon (DFP)
-struct G3DPolygonDFP
-{
-  /// Current number of vertices.
-  int num;
-  /// Vertices that form the polygon.
-  csVector2 vertices[100];
-
-  /// The plane equation in camera space of this polygon.
-  csPlane3 normal;
-};
-
-/// Structure containing all info needed by DrawPolygon (DP)
-struct G3DPolygonDP : public G3DPolygonDFP
-{
-  /// Extra optional fog information.
-  G3DFogInfo fog_info[100];
-  /// Use fog info?
-  bool use_fog;
-
-  /// The material handle as returned by iTextureManager.
-  iMaterialHandle* mat_handle;
-
-  /// Transformation matrices for the texture. @@@ BAD NAME
-  G3DCam2TextureTransform cam2tex;
-
-  /// Handle to lighted textures (texture + lightmap)
-  csPolyTextureMapping* texmap;
-  iRendererLightmap* rlm;
-  
-  /// Draw fullbright?
-  bool do_fullbright;
-
-  /// Mixmode to use. If CS_FX_COPY then no mixmode is used.
-  uint mixmode;
-
-  /// Z value (in camera space) of vertex[0].
-  float z_value;
-
-  iTextureHandle* txt_handle;
-};
-
-/// Structure containing all info needed by DrawPolygonFlat (DPF)
-typedef G3DPolygonDP G3DPolygonDPF;
-
-/**
- * Structure containing all info needed by DrawTriangeMesh.
- * This function is capable of:<br>
- * <ul>
- * <li>Object2camera transformation and perspective.
- * <li>Linear interpolation between two sets of vertices.
- * <li>Clipping.
- * <li>Whatever else DrawPolygonFX can do.
- * </ul>
- * To disable the use of one of the components, set it to 0.
- */
-struct G3DTriangleMesh
-{
-  enum
-  {
-    /// Maximum number of vertex pool, used for vertex weighting/morphing.
-    MAX_VERTEXPOOL = 2
-  };
-
-  /// Number of vertex sets, if > 1, morphing will be applied.
-  int num_vertices_pool;
-
-  /// Number of triangles.
-  int num_triangles;
-  /// Pointer to array of triangles.
-  csTriangle* triangles;
-
-  /// Clip to portal? One of CS_CLIP_???.
-  int clip_portal;
-  /// Clip to near plane? One of CS_CLIP_???.
-  int clip_plane;
-  /// Clip to z plane? One of CS_CLIP_???.
-  int clip_z_plane;
-
-  /// Use precalculated vertex color?
-  bool use_vertex_color;
-
-  /// Apply fogging?
-  bool do_fog;
-  /// Consider triangle vertices in anti-clockwise order if true.
-  bool do_mirror;
-  /// If morphing is applied then morph texels too if true.
-  bool do_morph_texels;
-  /// If morphing is applied then morph vertex colors too if true.
-  bool do_morph_colors;
-
-  /// Types of vertices supplied.
-  enum VertexMode
-  {
-    /// Must apply transformation and perspective.
-    VM_WORLDSPACE,
-    /// Must apply perspective.
-    VM_VIEWSPACE
-  };
-  
-  /// Type of vertices supplied.
-  VertexMode vertex_mode;
-
-  /// DrawPolygonFX flag.
-  uint mixmode;
-  float morph_factor;
+  bool DestinationAlpha;
   /**
-   * Vertex buffers. Note that all vertex buffers used here MUST
-   * have the same number of vertices.
+   * Enough stencil bits for stencil shadows are available.
    */
-  iVertexBuffer* buffers[MAX_VERTEXPOOL];
-  iMaterialHandle* mat_handle;
-  /// Information for fogging the vertices.
-  G3DFogInfo* vertex_fog;
-
-  // TODO : store information required for lighting calculation
-};
-
-/**
- * Structure containing all info needed by DrawPolygonMesh.
- * In theory this function is capable of:<br>
- * <ul>
- * <li>Object2camera transformation and perspective.
- * <li>Clipping.
- * <li>Whatever else DrawPolygon can do.
- * </ul>
- * To disable the use of one of the components, set it to 0.
- */
-struct G3DPolygonMesh
-{
-  /// Polygon buffer.
-  iPolygonBuffer* polybuf;
-
-  // Apply fogging?
-  bool do_fog;
-
-  /// Mixmode.
-  uint mixmode;
-
-  /// Clip to portal? One of CS_CLIP_???.
-  int clip_portal;
-  /// Clip to near plane? One of CS_CLIP_???.
-  int clip_plane;
-  /// Clip to z plane? One of CS_CLIP_???.
-  int clip_z_plane;
-
-  /// Consider polygon vertices in anti-clockwise order if true.
-  bool do_mirror;
-
-  /// Types of vertices supplied.
-  enum VertexMode
-  {
-    /// Must apply transformation and perspective.
-    VM_WORLDSPACE,
-    /// Must apply perspective.
-    VM_VIEWSPACE
-  };
-  
-  /// Type of vertices supplied.
-  VertexMode vertex_mode;
-
-  /// Information for fogging the vertices.
-  G3DFogInfo* vertex_fog;
+  bool StencilShadows;
 };
 
 /// Primitive type of a mesh
@@ -690,14 +564,7 @@ enum csRenderMeshType
    * Line strip.
    * A line is defined from the prebviously and recently added vertex.
    */
-  CS_MESHTYPE_LINESTRIP,
-  /**
-   * Render polygons. Note that you <b>*must*</b> supply geometry with the
-   * help of an iPolygonRenderer. In the common case, if you want to draw a
-   * polygon, you probably want to use CS_MESHTYPE_TRIANGLES or 
-   * CS_MESHTYPE_TRIANGLEFAN and triangulate the poly yourself. 
-   */
-  CS_MESHTYPE_POLYGON,
+  CS_MESHTYPE_LINESTRIP
 };
 
 /**
@@ -708,9 +575,10 @@ enum csSimpleMeshFlags
   /**
    * Ignore the object2camera transform in the csSimpleRenderMesh struct and
    * replace it with a transformation that effectively lets you specify the
-   * vertices in screen space.
+   * vertices in screen space. The Z components of the mesh vertices should be
+   * set to 0 when this flag is specified.
    */
-  csSimpleMeshScreenspace     = 0x01,
+  csSimpleMeshScreenspace = 0x01
 };
 
 /**
@@ -721,9 +589,15 @@ struct csSimpleRenderMesh
   /// Type of the geometry to draw.
   csRenderMeshType meshtype;
 
-  /// Number of vertex indices
+  /// (optional) Number of vertex indices
   uint indexCount;
-  /// Vertex indices
+  /** 
+   * (optional) Vertex indices.
+   * If this field is 0, \a vertexCount indices are generated with the values
+   * 0 to \a vertexCount -1. \a indexCount is ignored in that case.
+   * In other words, not specifying indices assumes all vertices are in order
+   * and only used once.
+   */
   const uint* indices;
 
   /// Number of vertices
@@ -761,42 +635,42 @@ struct csSimpleRenderMesh
   /**
    * (Optional) Transform to apply to the mesh.
    * \remark This transform is initialized to an identity transform.
-   *  This effectively means that geometry is drawn in eye space.
+   *  This effectively means that geometry is drawn in world space.
    *  To draw in screen space, supply the \a csSimpleMeshScreenspace
    *  flag to DrawSimpleMesh(). For anything else supply an appropriate
    *  transformation.
+   * \remark Keep in mind that the renderer's world-to-camera transform is in
+   *  effect, too.
    */
-  csReversibleTransform object2camera;
+  csReversibleTransform object2world;
 
-  csSimpleRenderMesh () : colors(0), texture (0), shader (0), dynDomain (0), 
-    z_buf_mode (CS_ZBUF_NONE), mixmode (CS_FX_COPY)
+  csSimpleRenderMesh () : indexCount(0), indices(0), texcoords(0), colors(0), 
+    texture (0), shader (0), dynDomain (0), z_buf_mode (CS_ZBUF_NONE), 
+    mixmode (CS_FX_COPY)
   {  
     alphaType.autoAlphaMode = true;
     alphaType.autoModeTexture = csInvalidStringID;
   };
 };
 
-SCF_VERSION (iGraphics3D, 5, 4, 1);
-
 /**
  * This is the standard 3D graphics interface.
  * All 3D graphics rasterizer servers for Crystal Space should implement this
  * interface, as well as the iGraphics2D interface.  The standard
  * implementation is csGraphics3DSoftware.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>OpenGL Renderer plugin (crystalspace.graphics3d.opengl)
- *   <li>Software Renderer plugin (crystalspace.graphics3d.software)
- *   <li>Null 3D Renderer plugin (crystalspace.graphics3d.null)
- *   </ul>
+ * - OpenGL Renderer plugin (crystalspace.graphics3d.opengl)
+ * - Software Renderer plugin (crystalspace.graphics3d.software)
+ * - Null 3D Renderer plugin (crystalspace.graphics3d.null)
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>CS_QUERY_REGISTRY()
- *   </ul>
+ * - csQueryRegistry()
  */
-struct iGraphics3D : public iBase
+struct iGraphics3D : public virtual iBase
 {
+  SCF_INTERFACE(iGraphics3D, 2, 0, 0);
+  
   /// Open the 3D graphics display.
   virtual bool Open () = 0;
   /// Close the 3D graphics display.
@@ -834,11 +708,18 @@ struct iGraphics3D : public iBase
 
   /**
    * Set center of projection for perspective projection.
-   * Center is set in screen space coordinates.
+   * \remarks The coordinates are vertically mirrored in comparison to screen
+   *   space, i.e. y=0 is at the bottom of the viewport, y=GetHeight() at the 
+   *   top.
    */
   virtual void SetPerspectiveCenter (int x, int y) = 0;
 
-  /// Get perspective center.
+  /**
+   * Get perspective center.
+   * \remarks The coordinates are vertically mirrored in comparison to screen
+   *   space, i.e. y=0 is at the bottom of the viewport, y=GetHeight() at the 
+   *   top.
+   */
   virtual void GetPerspectiveCenter (int& x, int& y) const = 0;
 
   /**
@@ -848,18 +729,7 @@ struct iGraphics3D : public iBase
 
   /// Get aspect ratio.
   virtual float GetPerspectiveAspect () const = 0;
-
-  /**
-   * Set object to camera transformation (currently only used by
-   * DrawTriangleMesh and DrawPolygonMesh).
-   */
-  virtual void SetObjectToCamera (csReversibleTransform* o2c) = 0;
-
-  /**
-   * Get object to camera transformation.
-   */
-  virtual const csReversibleTransform& GetObjectToCamera () = 0;
-  
+ 
   /**
    * Set the target of rendering. If this is 0 then the target will
    * be the main screen. Otherwise it is a texture. After calling
@@ -894,6 +764,29 @@ struct iGraphics3D : public iBase
    */
   virtual void Print (csRect const* area) = 0;
 
+  /// Drawroutine. Only way to draw stuff
+  virtual void DrawMesh (const csCoreRenderMesh* mymesh,
+                         const csRenderMeshModes& modes,
+                         const csArray<csShaderVariable*> &stacks) = 0;
+  /**
+  * Draw a csSimpleRenderMesh on the screen.
+  * Simple render meshes are intended for cases where setting up
+  * a render mesh and juggling with render buffers would be too much
+  * effort - e.g. when you want to draw a single polygon on the screen.
+  * <p>
+  * DrawSimpleMesh () hides the complexity of csRenderMesh, it cares
+  * about setting up render buffers, activating the texture etc.
+  * Note that you can still provide shaders and shader variables, but those
+  * are optional.
+  * \param mesh The mesh to draw.
+  * \param flags Drawing flags, a combination of csSimpleMeshFlags values.
+  * \remark This operation can also be called from 2D mode. In this case,
+  *  the csSimpleMeshScreenspace flag should be specified. If this is not the
+  *  case, you are responsible for the mess that is likely created.
+  */
+  virtual void DrawSimpleMesh (const csSimpleRenderMesh& mesh,
+    uint flags = 0) = 0;
+
   /**
    * Draw a pixmap using a rectangle from given texture.
    * The sx,sy(sw,sh) rectangle defines the screen rectangle within
@@ -918,6 +811,32 @@ struct iGraphics3D : public iBase
   virtual void DrawLine (const csVector3& v1, const csVector3& v2,
     float fov, int color) = 0;
 
+  /**
+  * Activate the buffers in the default buffer holder.
+  */
+  virtual bool ActivateBuffers (csRenderBufferHolder* holder, 
+    csRenderBufferName mapping[CS_VATTRIB_SPECIFIC_LAST+1]) = 0;
+
+  /**
+  * Activate all given buffers.
+  */
+  virtual bool ActivateBuffers (csVertexAttrib *attribs,
+    iRenderBuffer **buffers, unsigned int count) = 0;
+
+  /**
+  * Deactivate all given buffers.
+  * If attribs is 0, all buffers are deactivated;
+  */
+  virtual void DeactivateBuffers (csVertexAttrib *attribs, unsigned int count) = 0;
+
+  /**
+  * Activate or deactivate all given textures depending on the value
+  * of 'textures' for that unit (i.e. deactivate if 0).
+  */
+  virtual void SetTextureState (int* units, iTextureHandle** textures,
+    int count) = 0;
+
+
   /**
    * Set optional clipper to use. If clipper == null
    * then there is no clipper.
@@ -937,7 +856,6 @@ struct iGraphics3D : public iBase
 
   /**
    * Set near clip plane.
-   * Currently only used by DrawTriangleMesh.
    */
   virtual void SetNearPlane (const csPlane3& pl) = 0;
 
@@ -967,47 +885,7 @@ struct iGraphics3D : public iBase
    * support that option.
    */
   virtual bool SetOption (const char*, const char*) = 0;
-
-  /**
-   * Create a renderbuffer.
-   * \param size Size of the buffer in bytes.
-   * \param type Type of buffer; CS_BUF_DYNAMIC or CS_BUF_STATIC
-   * \param componentType Components Types; CS_BUFCOMP_BYTE, CS_BUFCOMP_INT, etc
-   * \param componentCount Number of components per element (e.g. 4 for RGBA)
-   * \param index True if this buffer will contain indices. (Triangle buffer)
-   */
-  virtual csPtr<iRenderBuffer> CreateRenderBuffer (int size, 
-    csRenderBufferType type, csRenderBufferComponentType componentType, 
-    int componentCount, bool index) = 0;
-
-  /**
-   * Create an interleaved renderbuffer (You would use this then set stride to
-   * determine offset and stride of the interleaved buffer
-   * \param size Size of the buffer in bytes
-   * \param count Number of renderbuffers you want
-   */
-  virtual void CreateInterleavedRenderBuffers (int size, 
-    csRenderBufferType type, int count, csArray<iRenderBuffer*> &buffers) = 0;
-
-  /**
-   * Activate or deactivate all given buffers depending on the value of
-   * 'buffers' for that index.
-   */
-  virtual void SetBufferState (csVertexAttrib* attribs,
-  	iRenderBuffer** buffers, int count) = 0;
-
-  /**
-   * Activate or deactivate all given textures depending on the value
-   * of 'textures' for that unit (i.e. deactivate if 0).
-   */
-  virtual void SetTextureState (int* units, iTextureHandle** textures,
-  	int count) = 0;
-
-  /// Drawroutine. Only way to draw stuff
-  virtual void DrawMesh (const csCoreRenderMesh* mymesh,
-    const csRenderMeshModes& modes,
-    const csArray< csArray<csShaderVariable*> > &stacks) = 0;
-
+  
   /// Set the masking of color and/or alpha values to framebuffer
   virtual void SetWriteMask (bool red, bool green, bool blue, bool alpha) = 0;
 
@@ -1018,6 +896,9 @@ struct iGraphics3D : public iBase
   /// Set the z buffer write/test mode
   virtual void SetZMode (csZBufMode mode) = 0;
 
+  /// Get the z buffer write/test mode
+  virtual csZBufMode GetZMode () = 0;
+
   /// Enables offsetting of Z values
   virtual void EnableZOffset () = 0;
 
@@ -1027,90 +908,12 @@ struct iGraphics3D : public iBase
   /// Controls shadow drawing
   virtual void SetShadowState (int state) = 0;
 
-  //=========================================================================
-  // Below this line are all functions that are not yet implemented by
-  // the new renderer or are not going to be implemented ever. In the
-  // last case they will be removed as soon as we permanently switch
-  // to the new renderer. @@@NR@@@
-  //=========================================================================
-  /// Debugging only: get a pointer to Z-buffer at some location
-  virtual uint32 *GetZBuffAt (int x, int y) = 0;
-
-  /// Get Z-buffer value at given X,Y position
+   /// Get Z-buffer value at given X,Y position
   virtual float GetZBuffValue (int x, int y) = 0;
 
-  /// Draw the projected polygon with light and texture.
-  virtual void DrawPolygon (G3DPolygonDP& poly) = 0;
-
   /**
-   * Draw the projected polygon with light and texture.
-   * Debugging version. This one does not actually draw anything
-   * but it just prints debug information about what it would have
-   * done.
-   */
-  virtual void DrawPolygonDebug (G3DPolygonDP& poly) = 0;
-
-  /**
-   * Draw a polygon with special effects. This is the most rich and slowest
-   * variant of DrawPolygonXxx. (If you use these features)
-   *
-   * Warning! Do not rely on this method to handle Color keying under
-   * all circumstances. Color Keying will only work reliable in Mixmodes
-   * FX_Copy, FX_Add and FX_Transparent. When using FX_Multiply
-   * and FX_Multiply2, it depends very much on the driver if it works or
-   * not. For example the RivaTNT Detonator 0.48 driver can display
-   * Multiply with color keying, while newer versions can't. They will
-   * then not display anything at all. It is always safer to use a texture
-   * where transparent sections are white or 50% gray if you want to achieve
-   * transparent sections in Multiply, Multiply2.
-   * There are also some drivers (which I would consider buggy...), that won't
-   * display FX_Alpha correctly with Color Keying. I can't provide a valid
-   * workaround for that except using FX_Multiplay and FX_Add, to manually
-   * create the image, but that would be very expensive.
-   */
-  virtual void DrawPolygonFX (G3DPolygonDPFX& poly) = 0;
-
-  /**
-   * Draw a triangle mesh using features similar to DrawPolygonFX.
-   */
-  virtual void DrawTriangleMesh (G3DTriangleMesh& mesh) = 0;
-
-  /**
-   * Draw a triangle mesh using features similar to DrawPolygon.
-   */
-  virtual void DrawPolygonMesh (G3DPolygonMesh& mesh) = 0;
-
-  /**
-   * Initiate a volumetric fog object. This function will be called
-   * before front-facing and back-facing fog polygons are added to
-   * the object. The fog object will be convex but not necesarily closed.
-   * The given CS_ID can be used to identify multiple fog objects when
-   * multiple objects are started.
-   */
-  virtual void OpenFogObject (CS_ID id, csFog* fog) = 0;
-
-  /**
-   * Add a front or back-facing fog polygon in the current fog object.
-   * Note that it is guaranteed that all back-facing fog polygons
-   * will have been added before the first front-facing polygon.
-   * fogtype can be:<br>
-   * <ul>
-   *  <li>CS_FOG_FRONT: a front-facing polygon
-   *  <li>CS_FOG_BACK:  a back-facing polygon
-   *  <li>CS_FOG_VIEW:  the view-plane
-   * </ul>
-   */
-  virtual void DrawFogPolygon (CS_ID id, G3DPolygonDFP& poly, int fogtype) = 0;
-
-  /**
-   * Close a volumetric fog object. After the volumetric object is
-   * closed it should be rendered on screen (whether you do it here
-   * or in DrawFogPolygon is not important).
-   */
-  virtual void CloseFogObject (CS_ID id) = 0;
-
-  /**
-   * Enter a new portal. If 'floating' is true then this routine will restrict
+   * Enter a new portal. 
+   * If 'flags' contains CS_PORTAL_FLOAT then this routine will restrict
    * all further drawing to the given 2D area and it will also respect
    * the current contents of the Z-buffer so that geometry will only
    * render where the Z-buffer allows it (even if zfill or znone is used).
@@ -1119,24 +922,18 @@ struct iGraphics3D : public iBase
    * portal must be fully contained in the previous ones.
    */
   virtual void OpenPortal (size_t numVertices, const csVector2* vertices,
-    const csPlane3& normal, bool floating) = 0;
+    const csPlane3& normal, csFlags flags) = 0;
 
   /**
    * Close a portal previously opened with OpenPortal().
    * If 'zfill_portal' then the portal area will be zfilled.
    */
-  virtual void ClosePortal (bool zfill_portal) = 0;
+  virtual void ClosePortal () = 0;
 
   /// Create a halo of the specified color and return a handle.
   virtual iHalo *CreateHalo (float iR, float iG, float iB,
     unsigned char *iAlpha, int iWidth, int iHeight) = 0;
 
-  /// Dump the texture cache.
-  virtual void DumpCache () = 0;
-
-  /// Clear the texture cache.
-  virtual void ClearCache () = 0;
-
   /**
    * Remove some polygon from the cache.
    * You have to call this function before deleting a polygon
@@ -1145,45 +942,11 @@ struct iGraphics3D : public iBase
   virtual void RemoveFromCache (iRendererLightmap* rlm) = 0;
 
   /**
-   * Get the vertex buffer manager. This will not increment the ref count
-   * of the vertex buffer manager!
+   * Set the world to camera transform.
+   * This affects rendering in DrawMesh and DrawSimpleMesh.
+   * \remarks 'this' space is world space, 'other' space is camera space
    */
-  virtual iVertexBufferManager* GetVertexBufferManager () = 0;
-  
-  //=========================================================================
-  // Here ends the zone of unimplemented methods.
-  //=========================================================================
-
-  /**
-   * Check if renderer can handle a lightmap.
-   * Returns true if it can, false if not.
-   */
-  virtual bool IsLightmapOK (int lmw, int lmh, 
-    int lightCellSize) = 0;
-    
-  virtual csPtr<iPolygonRenderer> CreatePolygonRenderer () = 0;
-
-  /*
-    @@@ Needed for SW poly drawing ATM.
-   */
-  virtual void SetWorldToCamera (csReversibleTransform* w2c) = 0;
-
-  /**
-   * Draw a csSimpleRenderMesh on the screen.
-   * Simple render meshes are intended for cases where setting up
-   * a render mesh and juggling with render buffers would be too much
-   * effort - e.g. when you want to draw a single polygon on the screen.
-   * <p>
-   * DrawSimpleMesh () hides the complexity of csRenderMesh, it cares
-   * about setting up render buffers, activating the texture etc.
-   * Note that you can still provide shaders and shader variables, but those
-   * are optional.
-   */
-  virtual void DrawSimpleMesh (const csSimpleRenderMesh& mesh,
-    uint flags = 0) = 0;
-
-  /// Get the z buffer write/test mode
-  virtual csZBufMode GetZMode () = 0;
+  virtual void SetWorldToCamera (const csReversibleTransform& w2c) = 0;
 };
 
 /** @} */
