@@ -27,7 +27,6 @@
  * \addtogroup gfx2d
  * @{ */
  
-#include <stdarg.h>
 #include "csutil/scf.h"
 
 /**\name Alert types
@@ -40,40 +39,45 @@
 #define CS_ALERT_NOTE 3
 /** @} */
 
-SCF_VERSION (iNativeWindowManager, 0, 0, 1);
-
 /**
  * This interface represents the native window manager system.
  * At this moment this interface is nearly empty. In the future it
  * will be extended with more functionality to manage windows and so on.
  */
-struct iNativeWindowManager : public iBase
+struct iNativeWindowManager : public virtual iBase
 {
+  SCF_INTERFACE (iNativeWindowManager, 2, 0, 0);
+  
   /**
    * Show an alert.
    * Type is one of CS_ALERT_???.
+   * \remarks All strings are expected to be UTF-8 encoded.
    * \sa #CS_ALERT_ERROR
+   * \sa \ref FormatterNotes
    */
   virtual void Alert (int type, const char* title, const char* okMsg,
   	const char* msg, ...) CS_GNUC_PRINTF (5, 6) = 0;
   /**
    * Show an alert.
    * Type is one of CS_ALERT_???.
+   * \remarks All strings are expected to be UTF-8 encoded.
    * \sa #CS_ALERT_ERROR
+   * \sa \ref FormatterNotes
    */
   virtual void AlertV (int type, const char* title, const char* okMsg,
   	const char* msg, va_list arg) CS_GNUC_PRINTF (5, 0) = 0;
 };
 
-SCF_VERSION (iNativeWindow, 0, 0, 1);
-
 /**
  * This interface represents a native window.
  */
-struct iNativeWindow : public iBase
+struct iNativeWindow : public virtual iBase
 {
+  SCF_INTERFACE (iNativeWindow, 2, 0, 0);
+  
   /**
    * Set the title for this window.
+   * \remarks \p title is expected to be UTF-8 encoded.
    */
   virtual void SetTitle (const char* title) = 0;
 };
