@@ -40,8 +40,6 @@ struct csLightMapMapping;
 struct iImage;
 struct iTextureHandle;
 struct iMaterial;
-struct iMaterialHandle;
-struct iImageVector;
 
 /**\name Texture registration flags.
  * During texture registration you should tell
@@ -53,15 +51,6 @@ struct iImageVector;
 #define CS_TEXTURE_2D			0x00000001
 /// You're going to use the texture for 3D drawing
 #define CS_TEXTURE_3D			0x00000002
-/**
- * Dither texture or not.
- * Some renderers may use dithering while converting textures to internal
- * format (say from truecolor to 8-bit paletted). For most textures dithering
- * won't give any visual effect, but very seldom there are textures that looks
- * relatively bad after being converted. In this case you can enable this
- * per-texture flag.
- */
-#define CS_TEXTURE_DITHER		0x00000004
 /**
  * Create mipmaps for this texture?
  * Sometimes we know in advance that some texture will need just one
@@ -81,15 +70,51 @@ struct iImageVector;
  * This texture will not be filtered, even if texture filtering is available.
  */
 #define CS_TEXTURE_NOFILTER		0x00000020
+/**
+ * Store texture as non-power-of-two sized (NPOTS) if possible.
+ * \remarks On a lot of hardware, support for NPOTS-texture is rather limited;
+ *  for example, commonly only clamped and textures without mipmaps are 
+ *  supported, hence a texture may only be considered for NPOTS use if the 
+ *  CS_TEXTURE_CLAMP and CS_TEXTURE_NOMIPMAPS flags are also set.
+ *  Some hardware may not support NPOTS textures at all. It is at the 
+ *  discretion of the renderer to decide whether a texture really becomes
+ *  NPOTS or not. This flag serves merely as an expression of the wish that
+ *  a texture should be NPOTS, if possible. If that cannot be granted, the 
+ *  flag will be removed (thus you can query whether a texture was stored
+ *  NPOTS or not by checking if this flag is set on it).
+ */
+#define CS_TEXTURE_NPOTS		0x00000040
+/**
+ * Texture resizing control flag.
+ * If a texture needs to be resized to power-of-two dimensions (either because
+ * non-power-of-two textures are not supported or just not requested), specify
+ * that powers of two larger than or equal to the original dimensions should
+ * be selected.
+ * \sa #CS_TEXTURE_SCALE_DOWN
+ * \remarks If none or both texture scale flags are specified, the renderer
+ *   will decide whether a dimension is scaled up or down.
+ */
+#define CS_TEXTURE_SCALE_UP		0x00000080
+/**
+ * Texture resizing control flag.
+ * If a texture needs to be resized to power-of-two dimensions (either because
+ * non-power-of-two textures are not supported or just not requested), specify
+ * that powers of two smaller than or equal to the original dimensions should
+ * be selected.
+ * \sa #CS_TEXTURE_SCALE_UP
+ * \remarks If none or both texture scale flags are specified, the renderer
+ *   will decide whether a dimension is scaled up or down.
+ */
+#define CS_TEXTURE_SCALE_DOWN		0x00000100
 /** @} */
 
-SCF_VERSION (iRendererLightmap, 1, 1, 0);
-
 /**
  * A lightmap registered with a renderer.
  */
-struct iRendererLightmap : public iBase
+struct iRendererLightmap : public virtual iBase
 {
+  SCF_INTERFACE (iRendererLightmap, 2, 0, 0);
+
   /**
    * Retrieve the coordinates of this lightmap in the superlightmap, in the
    * 'absolute' system used by iSuperLightmap::RegisterLightmap().
@@ -103,13 +128,13 @@ struct iRendererLightmap : public iBase
   virtual void SetLightCellSize (int size) = 0;
 };
 
-SCF_VERSION (iSuperLightmap, 1, 0, 1);
-
 /**
  * A super light map.
  */
-struct iSuperLightmap : public iBase
+struct iSuperLightmap : public virtual iBase
 {
+  SCF_INTERFACE (iSuperLightmap, 2, 0, 0);
+
   /// Add a lightmap to this SLM.
   virtual csPtr<iRendererLightmap> RegisterLightmap (int left, int top, 
     int width, int height) = 0;
@@ -120,8 +145,6 @@ struct iSuperLightmap : public iBase
   virtual iTextureHandle* GetTexture () = 0;
 };
 
-SCF_VERSION (iTextureManager, 2, 3, 2);
-
 /**
  * This is the standard texture manager interface.
  * A 3D rasterizer will have to implement a subclass of this one and
@@ -131,59 +154,23 @@ SCF_VERSION (iTextureManager, 2, 3, 2);
  * needed, calculating a palette if needed, and calculating all
  * lookup tables related to the textures. Mipmap creation is
  * also done in this class.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>The 3D renderers create an instance of this.
- *   </ul>
+ * - The 3D renderers create an instance of this.
+ *
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>iGraphics3D::GetTextureManager()
- *   </ul>
+ * - iGraphics3D::GetTextureManager()
  */
-struct iTextureManager : public iBase
+struct iTextureManager : public virtual iBase
 {
+  SCF_INTERFACE(iTextureManager, 3,0,0);
   /**
    * Register a texture. The given input image is IncRef'd and DecRef'ed
-   * later when FreeImages () is called. If you want to keep the input image
+   * later when no longer needed. If you want to keep the input image
    * make sure you have called IncRef yourselves.
    *<p>
    * The texture is not converted immediately. Instead, you can make
-   * intermediate calls to iTextureHandle::SetKeyColor (). Finally,
-   * if you want to merge the texture into the current environment, you
-   * should call txt->Prepare(). Alternatively you can call the
-   * PrepareTextures () method to compute a optimal palette and convert
-   * all the textures into the internal format.
-   *<p>
-   * This function returns a handle which should be given
-   * to the 3D rasterizer or 2D driver when drawing or otherwise using
-   * the texture.
-   *<p>
-   * The `flags' contains one or several of CS_TEXTURE_XXX flags OR'ed
-   * together. They define the mode texture is going to be used in.
-   *<p>
-   * The texture manager will reject the texture if it is an inappropiate
-   * format (see GetTextureFormat () method).
-   *<p>
-   * The texture is unregistered at destruction, i.e. as soon as the last
-   * reference to the texture handle is released.
-   *<p>
-   * Note! This function will NOT scale the texture to fit hardware
-   * restrictions. This is done later when Prepare() is called.
-   */
-  virtual csPtr<iTextureHandle> RegisterTexture (iImage *image, int flags) = 0;
-
-  /**
-   * Register a texture. The given input image is IncRef'd and DecRef'ed
-   * later when FreeImages () is called. If you want to keep the input image
-   * make sure you have called IncRef yourselves.
-   *<p>
-   * The texture is not converted immediately. Instead, you can make
-   * intermediate calls to iTextureHandle::SetKeyColor (). Finally,
-   * if you want to merge the texture into the current environment, you
-   * should call txt->Prepare(). Alternatively you can call the
-   * PrepareTextures () method to compute a optimal palette and convert
-   * all the textures into the internal format.
+   * intermediate calls to iTextureHandle::SetKeyColor ().
    *<p>
    * This function returns a handle which should be given
    * to the 3D rasterizer or 2D driver when drawing or otherwise using
@@ -202,65 +189,9 @@ struct iTextureManager : public iBase
    * found in ivideo/texture.h
    *<p>
    * Note! This function will NOT scale the texture to fit hardware
-   * restrictions. This is done later when Prepare() is called.
+   * restrictions. This is done later before texture is first used.
    */
-  virtual csPtr<iTextureHandle> RegisterTexture (iImageVector *image,
-  	int flags, int target) = 0;
-
-  /**
-   * After all textures have been added, this function does all
-   * needed calculations (palette, lookup tables, mipmaps, ...).
-   * PrepareTextures () must be able to handle being called twice
-   * or more without ill effects.
-   * Note that it is in this stage that the original image that is
-   * attached to a texture is scaled so that it fits hardware
-   * requirements. So it is important to realize that calling this
-   * function may actually change the images from which you created
-   * the textures!
-   */
-  virtual void PrepareTextures () = 0;
-
-  /**
-   * Call this function if you want to release all iImage's as
-   * given to this texture manager. After FreeImages() has been called
-   * it is no longer allowed to call Prepare() again. So the advantage
-   * of calling FreeImages() is that you gain memory (may be a lot)
-   * but the disadvantage is that when you want to add textures later
-   * you have to reload them all and start all over.
-   */
-  virtual void FreeImages () = 0;
-
-  /**
-   * Register a material. The input material is IncRef'd and DecRef'ed
-   * later when FreeMaterials () is called or the material handle is destroyed
-   * by calling DecRef on it enough times. If you want to keep the input
-   * material make sure you have called IncRef yourselves. <p>
-   *
-   * The material is unregistered at destruction, i.e. as soon as the last
-   * reference to the material handle is released.
-   */
-  virtual csPtr<iMaterialHandle> RegisterMaterial (iMaterial* material) = 0;
-
-  /**
-   * Register a material based on a texture handle. This is a short-cut
-   * to quickly make materials based on a single texture.
-   *
-   * The material is unregistered at destruction, i.e. as soon as the last
-   * reference to the material handle is released.
-   */
-  virtual csPtr<iMaterialHandle> RegisterMaterial (
-  	iTextureHandle* txthandle) = 0;
-
-  /**
-   * Prepare all materials.
-   */
-  virtual void PrepareMaterials () = 0;
-
-  /**
-   * Call this function if you want to release all iMaterial's as
-   * given to this texture manager.
-   */
-  virtual void FreeMaterials () = 0;
+  virtual csPtr<iTextureHandle> RegisterTexture (iImage *image, int flags) = 0;
 
   /**
    * Query the basic format of textures that can be registered with this
