@@ -17,14 +17,15 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <math.h>
 #include "cssysdef.h"
+
 #include "csgeom/box.h"
-#include "csgeom/transfrm.h"
+#include "csgeom/math3d.h"
 #include "csgeom/poly2d.h"
+#include "csgeom/transfrm.h"
 
 //---------------------------------------------------------------------------
-csBox2::bEdge csBox2:: edges[8] =
+csBox2::bEdge csBox2::edges[8] =
 {
   { CS_BOX_CORNER_xy, CS_BOX_CORNER_Xy },
   { CS_BOX_CORNER_Xy, CS_BOX_CORNER_xy },
@@ -36,6 +37,13 @@ csBox2::bEdge csBox2:: edges[8] =
   { CS_BOX_CORNER_xy, CS_BOX_CORNER_xY }
 };
 
+csString csBox2::Description() const
+{
+  csString s;
+  s.Format("(%g,%g)-(%g,%g)", MinX(), MinY(), MaxX(), MaxY());
+  return s;
+}
+
 csVector2 csBox2::GetCorner (int corner) const
 {
   switch (corner)
@@ -264,6 +272,57 @@ float csBox2::SquaredOriginMaxDist () const
   return res;
 }
 
+float csBox2::SquaredPosDist (const csVector2& pos) const
+{
+  // Thanks to Ivan Avramovic for the original.
+  // Adapted by Norman Kramer, Jorrit Tyberghein and Wouter Wijngaards.
+  float res = 0;
+  float mm = minbox.x - pos.x;
+  if (mm > 0)
+    res = mm * mm;
+  else
+  {
+    mm = maxbox.x - pos.x;
+    if (mm < 0)
+      res = mm * mm;
+  }
+  mm = minbox.y - pos.y;
+  if (mm > 0)
+    res += mm * mm;
+  else
+  {
+    mm = maxbox.y - pos.y;
+    if (mm < 0)
+      res += mm * mm;
+  }
+  return res;
+}
+
+float csBox2::SquaredPosMaxDist (const csVector2& pos) const
+{
+  // Thanks to Ivan Avramovic for the original.
+  // Adapted by Norman Kramer, Jorrit Tyberghein and Wouter Wijngaards.
+  float res;
+  float mi = minbox.x - pos.x;
+  float ma = maxbox.x - pos.x;
+  if (mi > 0)
+    res = ma * ma;
+  else if (ma < 0)
+    res = mi * mi;
+  else
+    res = MAX (ma * ma, mi * mi);
+
+  mi = minbox.y - pos.y;
+  ma = maxbox.y - pos.y;
+  if (mi > 0)
+    res += ma * ma;
+  else if (ma < 0)
+    res += mi * mi;
+  else
+    res += MAX (ma * ma, mi * mi);
+  return res;
+}
+
 //---------------------------------------------------------------------------
 
 /*
@@ -404,6 +463,14 @@ csBox3::bFace csBox3:: faces[6] =
       CS_BOX_EDGE_XyZ_XYZ }
 };
 
+csString csBox3::Description() const
+{
+  csString s;
+  s.Format("(%g,%g,%g)-(%g,%g,%g)",
+    MinX(), MinY(), MinZ(), MaxX(), MaxY(), MaxZ());
+  return s;
+}
+
 csVector3 csBox3::GetCorner (int corner) const
 {
   switch (corner)
@@ -454,11 +521,38 @@ csBox2 csBox3::GetSide (int side) const
   return csBox2 ();
 }
 
-bool csBox3::AdjacentX (const csBox3 &other) const
+void csBox3::GetAxisPlane (int side, int& axis, float& where) const
+{
+  axis = side / 2;
+
+  switch (side)
+  {
+    case CS_BOX_SIDE_x:
+      where = MinX ();
+      break;
+    case CS_BOX_SIDE_X:
+      where = MaxX ();
+      break;
+    case CS_BOX_SIDE_y:
+      where = MinY ();
+      break;
+    case CS_BOX_SIDE_Y:
+      where = MaxY ();
+      break;
+    case CS_BOX_SIDE_z:
+      where = MinZ ();
+      break;
+    case CS_BOX_SIDE_Z:
+      where = MaxZ ();
+      break;
+  }
+}
+
+bool csBox3::AdjacentX (const csBox3 &other, float epsilon) const
 {
   if (
-    ABS (other.MinX () - MaxX ()) < SMALL_EPSILON ||
-    ABS (other.MaxX () - MinX ()) < SMALL_EPSILON)
+    ABS (other.MinX () - MaxX ()) < epsilon ||
+    ABS (other.MaxX () - MinX ()) < epsilon)
   {
     if (MaxY () < other.MinY () || MinY () > other.MaxY ()) return false;
     if (MaxZ () < other.MinZ () || MinZ () > other.MaxZ ()) return false;
@@ -468,11 +562,11 @@ bool csBox3::AdjacentX (const csBox3 &other) const
   return false;
 }
 
-bool csBox3::AdjacentY (const csBox3 &other) const
+bool csBox3::AdjacentY (const csBox3 &other, float epsilon) const
 {
   if (
-    ABS (other.MinY () - MaxY ()) < SMALL_EPSILON ||
-    ABS (other.MaxY () - MinY ()) < SMALL_EPSILON)
+    ABS (other.MinY () - MaxY ()) < epsilon ||
+    ABS (other.MaxY () - MinY ()) < epsilon)
   {
     if (MaxX () < other.MinX () || MinX () > other.MaxX ()) return false;
     if (MaxZ () < other.MinZ () || MinZ () > other.MaxZ ()) return false;
@@ -482,11 +576,11 @@ bool csBox3::AdjacentY (const csBox3 &other) const
   return false;
 }
 
-bool csBox3::AdjacentZ (const csBox3 &other) const
+bool csBox3::AdjacentZ (const csBox3 &other, float epsilon) const
 {
   if (
-    ABS (other.MinZ () - MaxZ ()) < SMALL_EPSILON ||
-    ABS (other.MaxZ () - MinZ ()) < SMALL_EPSILON)
+    ABS (other.MinZ () - MaxZ ()) < epsilon ||
+    ABS (other.MaxZ () - MinZ ()) < epsilon)
   {
     if (MaxX () < other.MinX () || MinX () > other.MaxX ()) return false;
     if (MaxY () < other.MinY () || MinY () > other.MaxY ()) return false;
@@ -496,9 +590,9 @@ bool csBox3::AdjacentZ (const csBox3 &other) const
   return false;
 }
 
-int csBox3::Adjacent (const csBox3 &other) const
+int csBox3::Adjacent (const csBox3 &other, float epsilon) const
 {
-  if (AdjacentX (other))
+  if (AdjacentX (other, epsilon))
   {
     if (other.MaxX () > MaxX ())
       return CS_BOX_SIDE_X;
@@ -506,7 +600,7 @@ int csBox3::Adjacent (const csBox3 &other) const
       return CS_BOX_SIDE_x;
   }
 
-  if (AdjacentY (other))
+  if (AdjacentY (other, epsilon))
   {
     if (other.MaxY () > MaxY ())
       return CS_BOX_SIDE_Y;
@@ -514,7 +608,7 @@ int csBox3::Adjacent (const csBox3 &other) const
       return CS_BOX_SIDE_y;
   }
 
-  if (AdjacentZ (other))
+  if (AdjacentZ (other, epsilon))
   {
     if (other.MaxZ () > MaxZ ())
       return CS_BOX_SIDE_Z;
@@ -692,6 +786,77 @@ float csBox3::SquaredOriginMaxDist () const
   return res;
 }
 
+float csBox3::SquaredPosDist (const csVector3& pos) const
+{
+  // Thanks to Ivan Avramovic for the original.
+  // Adapted by Norman Kramer, Jorrit Tyberghein and Wouter Wijngaards.
+  float res = 0;
+  float mm = minbox.x - pos.x;
+  if (mm > 0)
+    res = mm * mm;
+  else
+  {
+    mm = maxbox.x - pos.x;
+    if (mm < 0)
+      res = mm * mm;
+  }
+
+  mm = minbox.y - pos.y;
+  if (mm > 0)
+    res += mm * mm;
+  else
+  {
+    mm = maxbox.y - pos.y;
+    if (mm < 0)
+      res += mm * mm;
+  }
+
+  mm = minbox.z - pos.z;
+  if (mm > 0)
+    res += mm * mm;
+  else
+  {
+    mm = maxbox.z - pos.z;
+    if (mm < 0)
+      res += mm * mm;
+  }
+  return res;
+}
+
+float csBox3::SquaredPosMaxDist (const csVector3& pos) const
+{
+  // Thanks to Ivan Avramovic for the original.
+  // Adapted by Norman Kramer, Jorrit Tyberghein and Wouter Wijngaards.
+  float res;
+  float mi = minbox.x - pos.x;
+  float ma = maxbox.x - pos.x;
+  if (mi > 0)
+    res = ma * ma;
+  else if (ma < 0)
+    res = mi * mi;
+  else
+    res = MAX (ma * ma, mi * mi);
+
+  mi = minbox.y - pos.y;
+  ma = maxbox.y - pos.y;
+  if (mi > 0)
+    res += ma * ma;
+  else if (ma < 0)
+    res += mi * mi;
+  else
+    res += MAX (ma * ma, mi * mi);
+
+  mi = minbox.z - pos.z;
+  ma = maxbox.z - pos.z;
+  if (mi > 0)
+    res += ma * ma;
+  else if (ma < 0)
+    res += mi * mi;
+  else
+    res += MAX (ma * ma, mi * mi);
+  return res;
+}
+
 // Version to cope with z <= 0. This is wrong but it in the places where
 // it is used below the result is acceptable because it generates a
 // conservative result (i.e. a box or outline that is bigger then reality).
@@ -778,6 +943,59 @@ bool csBox3::ProjectBox (const csTransform& trans, float fov,
   return true;
 }
 
+bool csBox3::ProjectOutline (const csVector3& origin,
+	int axis, float where,
+  	csArray<csVector2>& poly) const
+{
+  int idx = CalculatePointSegment (origin);
+  const Outline &ol = outlines[idx];
+  int num_array = MIN (ol.num, 6);
+
+  int i;
+  for (i = 0 ; i < num_array ; i++)
+  {
+    csVector3 isect;
+    if (!csIntersect3::SegmentAxisPlane (origin, GetCorner (ol.vertices[i]),
+    	axis, where, isect))
+      return false;
+    csVector2 v;
+    switch (axis)
+    {
+      case CS_AXIS_X: v.x = isect.y; v.y = isect.z; break;
+      case CS_AXIS_Y: v.x = isect.x; v.y = isect.z; break;
+      case CS_AXIS_Z: v.x = isect.x; v.y = isect.y; break;
+    }
+    poly.Push (v);
+  }
+  return true;
+}
+
+bool csBox3::ProjectOutline (const csVector3& origin,
+	int axis, float where,
+  	csPoly2D& poly) const
+{
+  int idx = CalculatePointSegment (origin);
+  const Outline &ol = outlines[idx];
+  int num_array = MIN (ol.num, 6);
+  poly.SetVertexCount (num_array);
+
+  int i;
+  for (i = 0 ; i < num_array ; i++)
+  {
+    csVector3 isect;
+    if (!csIntersect3::SegmentAxisPlane (origin, GetCorner (ol.vertices[i]),
+    	axis, where, isect))
+      return false;
+    switch (axis)
+    {
+      case CS_AXIS_X: poly[i].x = isect.y; poly[i].y = isect.z; break;
+      case CS_AXIS_Y: poly[i].x = isect.x; poly[i].y = isect.z; break;
+      case CS_AXIS_Z: poly[i].x = isect.x; poly[i].y = isect.y; break;
+    }
+  }
+  return true;
+}
+
 bool csBox3::ProjectOutline (const csTransform& trans, float fov,
 	float sx, float sy,
   	csPoly2D& poly, float& min_z, float& max_z) const
