@@ -16,17 +16,24 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
+
+#include "cssysdef.h"
 #include <math.h>
 #include <float.h>
-#include "cssysdef.h"
-#include "qint.h"
-#include "qsqrt.h"
+#include "csqsqrt.h"
 #include "csgeom/vector3.h"
+#include "csutil/csstring.h"
+
+csString csVector3::Description() const
+{
+  csString s;
+  s.Format("%g,%g,%g", x, y, z);
+  return s;
+}
 
-//---------------------------------------------------------------------------
 float csVector3::Norm () const
 {
-  return qsqrt (x * x + y * y + z * z);
+  return csQsqrt (x * x + y * y + z * z);
 }
 
 void csVector3::Normalize ()
@@ -34,7 +41,7 @@ void csVector3::Normalize ()
   float sqlen = x * x + y * y + z * z;
   if (sqlen < SMALL_EPSILON) return ;
 
-  float invlen = qisqrt (sqlen);
+  float invlen = csQisqrt (sqlen);
   *this *= invlen;
 }
 
@@ -44,5 +51,3 @@ csVector3::csVector3 (const csDVector3 &v)
   y = (float)v.y;
   z = (float)v.z;
 }
-
-//---------------------------------------------------------------------------
