@@ -21,15 +21,15 @@
 #include "igraphic/image.h"
 #include "csgfx/memimage.h"
 #include "csgfx/rgbpixel.h"
-#include "qint.h"
+#include "csqint.h"
 #include "csgfx/xorpat.h"
 
 csPtr<iImage> csCreateXORPatternImage(int width, int height, int recdepth,
 				      float red, float green, float blue)
 {
   int x,y;
-  iImage *image = new csImageMemory(width, height);
-  csRGBpixel *pixel = (csRGBpixel*)image->GetImageData();
+  csImageMemory* image = new csImageMemory (width, height);
+  csRGBpixel *pixel = (csRGBpixel*)image->GetImagePtr();
 
   if (recdepth<1) recdepth = 1;
   if (recdepth>8) recdepth = 8;
@@ -44,14 +44,14 @@ csPtr<iImage> csCreateXORPatternImage(int width, int height, int recdepth,
     {
       unsigned char value = ((x & coordmask) ^ (y & coordmask));
       int v = (value << shlpixel) + ((value >> valueshr) * valueadd);
-      pixel->red = QInt (v * red);
-      pixel->green = QInt (v * green);
-      pixel->blue = QInt (v * blue);
+      pixel->red = csQint (v * red);
+      pixel->green = csQint (v * green);
+      pixel->blue = csQint (v * blue);
       pixel++;
     }
   }
 
-  return image;
+  return csPtr<iImage> ((iImage*)image);
 }
 
 
