@@ -18,7 +18,7 @@
 */
 
 #include "cssysdef.h"
-#include "qint.h"
+#include "csqint.h"
 #include "csgeom/polyclip.h"
 #include "csgeom/poly2d.h"
 #include "ivideo/graph3d.h"
@@ -26,16 +26,11 @@
 #include "iengine/camera.h"
 #include "iengine/engine.h"
 
-SCF_IMPLEMENT_IBASE (csView)
-  SCF_IMPLEMENTS_INTERFACE (iView)
-SCF_IMPLEMENT_IBASE_END
 
-
-csView::csView (iEngine *e, iGraphics3D* ig3d) :
+csView::csView (iEngine *e, iGraphics3D* ig3d) 
+  : scfImplementationType (this),
   Engine (e), G3D (ig3d), RectView (0), PolyView (0), AutoResize (true)
 {
-  SCF_CONSTRUCT_IBASE (0);
-
   Camera = e->CreateCamera ();
 
   OldWidth = G3D->GetWidth ();
@@ -46,7 +41,6 @@ csView::~csView ()
 {
   delete RectView;
   delete PolyView;
-  SCF_DESTRUCT_IBASE();
 }
 
 iEngine* csView::GetEngine ()
@@ -85,7 +79,6 @@ void csView::SetRectangle (int x, int y, int w, int h)
   OldHeight = G3D->GetHeight ();
   delete PolyView; PolyView = 0;
   Clipper = 0;
-
   // Do not allow the rectangle to go out of the screen
   if (x < 0) { w += x; x = 0; }
   if (y < 0) { h += y; y = 0; }
@@ -137,9 +130,9 @@ void csView::UpdateView ()
   
   if (PolyView)
   {
-    int i;
+    size_t i;
     csVector2 *pverts = PolyView->GetVertices ();
-    int InCount = PolyView->GetVertexCount ();
+    size_t InCount = PolyView->GetVertexCount ();
     // scale poly
     for (i = 0; i < InCount; i++)
     {
@@ -149,10 +142,10 @@ void csView::UpdateView ()
   }
   else if (RectView)
   {
-    RectView->Set (QRound (scale_x * RectView->MinX()),
-		   QRound (scale_y * RectView->MinY()),
-		   QRound (scale_x * RectView->MaxX()),
-		   QRound (scale_y * RectView->MaxY()) );
+    RectView->Set (csQround (scale_x * RectView->MinX()),
+		   csQround (scale_y * RectView->MinY()),
+		   csQround (scale_x * RectView->MaxX()),
+		   csQround (scale_y * RectView->MaxY()) );
   }
 
   Clipper = 0;
@@ -190,7 +183,7 @@ void csView::RestrictClipperToScreen ()
   // so we only have to update polygon-based views
   if (PolyView)
   {
-    int InCount = PolyView->GetVertexCount (), OutCount;
+    size_t InCount = PolyView->GetVertexCount (), OutCount;
     csBoxClipper bc (0., 0., (float)G3D->GetWidth (), (float)G3D->GetHeight());
     csVector2 *TempPoly = new csVector2[InCount + 5];
     uint8 rc = bc.Clip (PolyView->GetVertices (), InCount , TempPoly, OutCount);
@@ -198,7 +191,7 @@ void csView::RestrictClipperToScreen ()
     {
       PolyView->MakeRoom (OutCount);
       PolyView->SetVertices (TempPoly, OutCount);
-      PolyView->UpdateBoundingBox ();
+      //@@@PolyView->UpdateBoundingBox ();
     }
     delete [] TempPoly;
   }
