@@ -16,20 +16,18 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-
+#include "cssysdef.h"
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#define CS_SYSDEF_PROVIDE_DIR
-#define CS_SYSDEF_PROVIDE_PATH
-#define CS_SYSDEF_PROVIDE_TEMP
-#define CS_SYSDEF_PROVIDE_UNLINK
-#define CS_SYSDEF_PROVIDE_ACCESS
-#include "cssysdef.h"
-#include "cssys/csendian.h"
-#include "csutil/util.h"
+
 #include "csutil/archive.h"
+#include "csutil/csendian.h"
+#include "csutil/csstring.h"
+#include "csutil/set.h"
 #include "csutil/snprintf.h"
+#include "csutil/sysfunc.h"
+#include "csutil/util.h"
 #include "iutil/vfs.h"	// For csFileTime
 
 // Default compression method to use when adding entries (there is no choice for now)
@@ -42,6 +40,16 @@
 #  define DEFAULT_COMPRESSION_LEVEL Z_DEFAULT_COMPRESSION
 #endif
 
+// helper macro: appends a slash to the string if no slash is present
+#  define APPEND_SLASH(str,len)                 \
+     if ((len)                                  \
+      && (str[len - 1] != '/')                  \
+      && (str[len - 1] != CS_PATH_SEPARATOR))   \
+     {                                          \
+       str[len++] = CS_PATH_SEPARATOR;          \
+       str[len] = 0;                            \
+     } /* endif */
+
 //---------------------------------------------------------------------------
 
 char csArchive::hdr_central[4] = {'P', 'K', CENTRAL_HDR_SIG};
@@ -51,16 +59,22 @@ char csArchive::hdr_extlocal[4] = {'P', 'K', EXTD_LOCAL_SIG};
 
 //-- Endianess handling -----------------------------------------------------
 
-#define BUFF_GET_SHORT(ofs)     get_le_short ((uint8 *)&buff[ofs])
-#define BUFF_GET_LONG(ofs)      get_le_long  ((uint8 *)&buff[ofs])
-#define BUFF_SET_SHORT(ofs,val) set_le_short ((uint8 *)&buff[ofs], val)
-#define BUFF_SET_LONG(ofs,val)  set_le_long  ((uint8 *)&buff[ofs], val)
+#define BUFF_GET_(ofs, Type) \
+  csLittleEndian::Type (csGetFromAddress::Type ((uint8 *)&buff[ofs]))
+#define BUFF_GET_SHORT(ofs)     BUFF_GET_(ofs, UInt16)
+#define BUFF_GET_LONG(ofs)      BUFF_GET_(ofs, UInt32)
+
+#define BUFF_SET_(ofs, val, Type, Type2)      \
+  csSetToAddress::Type ((uint8 *)&buff[ofs],  \
+    csLittleEndian::Convert ((Type2)val))
+#define BUFF_SET_SHORT(ofs,val) BUFF_SET_(ofs, val, UInt16, uint16)
+#define BUFF_SET_LONG(ofs,val)  BUFF_SET_(ofs, val, UInt32, uint32)
 
 //-- Archive class implementation -------------------------------------------
 
 csArchive::csArchive (const char *filename)
 {
-  comment = NULL;
+  comment = 0;
   comment_length = 0;
   csArchive::filename = csStrNew (filename);
 
@@ -76,6 +90,43 @@ csArchive::~csArchive ()
   delete [] filename;
   delete [] comment;
   if (file) fclose (file);
+
+  size_t i;
+  for (i = 0; i < lazy.Length (); i++)
+  {
+    ArchiveEntry* e = lazy[i];
+    delete e;
+  }
+}
+
+void csArchive::ResetArchiveEntry (ArchiveEntry *f, size_t size, bool pack)
+{
+  f->info.compression_method = (pack) ?
+    DEFAULT_COMPRESSION_METHOD : ZIP_STORE;
+  f->info.ucsize = (u32)size;
+  f->buffer_pos = 0;
+
+  time_t curtime = time (0);
+  struct tm *curtm = localtime (&curtime);
+  csFileTime ft;
+  ASSIGN_FILETIME (ft, *curtm);
+  SetFileTime ((void *)f, ft);
+}
+
+csArchive::ArchiveEntry *csArchive::CreateArchiveEntry (const char *name, size_t size, bool pack)
+{
+  ZIP_central_directory_file_header cdfh;
+
+  memset (&cdfh, 0, sizeof (cdfh));
+  cdfh.version_made_by[0] = 0x16;		/* Zip version 2.2 rev 6 ??? */
+  cdfh.version_made_by[1] = 0x06;
+  cdfh.version_needed_to_extract[0] = 20;       /* Unzip version 2.0 rev 0 */
+  cdfh.version_needed_to_extract[1] = 00;
+
+  ArchiveEntry *f = new ArchiveEntry (name, cdfh);
+  ResetArchiveEntry (f, size, pack);
+
+  return f;
 }
 
 void csArchive::ReadDirectory ()
@@ -84,6 +135,46 @@ void csArchive::ReadDirectory ()
     return;                     /* Directory already read */
 
   ReadZipDirectory (file);
+
+  //// After reading, ensure that a node entry exists in memory for every
+  //// directory component even if the physical archive file does not contain
+  //// the corresponding directory entries (which are optional in zip files).
+
+  // First, make a list of all possible directory components.
+  csString filename, slice;
+  csSet<csStrKey> dset;
+  for (size_t i = 0, n = dir.Length(); i < n; i++)
+  {
+    ArchiveEntry const* e = dir.Get (i);
+    filename = e->filename;
+    size_t sep = 0;
+    while (sep < filename.Length())
+    {
+      size_t slash = filename.FindFirst ('/', sep);
+      if (slash == (size_t)-1)
+	slash = filename.FindFirst (CS_PATH_SEPARATOR, sep);
+      sep = slash;
+      if (sep != (size_t)-1)
+      {
+	filename.SubString (slice, 0, ++sep);
+        if (!dset.In (slice.GetData()))
+          dset.AddNoTest (csStrKey (slice)); 
+      }
+    }
+  }
+
+  // Now, iterate over `dset' and create fake directory components.
+  csSet<csStrKey>::GlobalIterator it = dset.GetIterator();
+  while (it.HasNext())
+  {
+    csString dname (it.Next());
+    if (!FileExists (dname))
+    {
+      ArchiveEntry* f = CreateArchiveEntry(dname, 0, 0);
+      f->faked = true;
+      dir.InsertSorted (f, ArchiveEntryVector::Compare);
+    }
+  }
 }
 
 void csArchive::ReadZipDirectory (FILE *infile)
@@ -101,8 +192,10 @@ void csArchive::ReadZipDirectory (FILE *infile)
   if ((long) (cur_offs = ftell (infile)) == -1)
     return;                     /* Can't get file position */
 
-  if (cur_offs >= (65535 + ZIP_END_CENTRAL_DIR_RECORD_SIZE + sizeof (hdr_endcentral)))
-    min_offs = cur_offs - (65535 + ZIP_END_CENTRAL_DIR_RECORD_SIZE + sizeof (hdr_endcentral));
+  if (cur_offs >= (65535 + ZIP_END_CENTRAL_DIR_RECORD_SIZE +
+  	sizeof (hdr_endcentral)))
+    min_offs = cur_offs - (65535 + ZIP_END_CENTRAL_DIR_RECORD_SIZE +
+    	sizeof (hdr_endcentral));
   else
     min_offs = 0;
 
@@ -111,14 +204,14 @@ void csArchive::ReadZipDirectory (FILE *infile)
   /* after which follows a two-byte END_CENTRAL_SIG */
   while (cur_offs > min_offs)
   {
-    uint search_pos;
+    size_t search_pos;
 
     if (cur_offs >= sizeof (buff) - step)
       cur_offs -= sizeof (buff) - step;
     else
       cur_offs = 0;
 
-    fseek (infile, cur_offs, SEEK_SET);
+    fseek (infile, (long)cur_offs, SEEK_SET);
     search_pos = fread (buff, 1, sizeof (buff), infile);
     if (search_pos >= step)
     {
@@ -129,9 +222,11 @@ void csArchive::ReadZipDirectory (FILE *infile)
             (memcmp (search_ptr, hdr_endcentral, sizeof (hdr_endcentral)) == 0))
         {
           /* Central directory structure found */
-          central_directory_offset = cur_offs + (uint32) search_ptr - (uint32)buff;
+          central_directory_offset = cur_offs + (size_t)(search_ptr - buff);
           LoadECDR (ecdr, &search_ptr[sizeof (hdr_endcentral)]);
-          if (fseek (infile, central_directory_offset + sizeof (hdr_endcentral) + ZIP_END_CENTRAL_DIR_RECORD_SIZE, SEEK_SET)
+          if (fseek (infile, (long)(central_directory_offset +
+	  	sizeof (hdr_endcentral) +
+		ZIP_END_CENTRAL_DIR_RECORD_SIZE), SEEK_SET)
            || !ReadArchiveComment (infile, ecdr.zipfile_comment_length)
            || fseek (infile, ecdr.offset_start_central_directory, SEEK_SET))
             goto rebuild_cdr;   /* Broken central directory */
@@ -139,8 +234,9 @@ void csArchive::ReadZipDirectory (FILE *infile)
           /* Now read central directory structure */
           for (;;)
           {
-            if ((fread (buff, 1, sizeof (hdr_central), infile) < sizeof (hdr_central))
-             || (memcmp (buff, hdr_central, sizeof (hdr_central)) != 0))
+            if ((fread (buff, 1, sizeof (hdr_central), infile) <
+	    	sizeof (hdr_central)) ||
+		(memcmp (buff, hdr_central, sizeof (hdr_central)) != 0))
             {
               if (dir.Length ())
                 return;         /* Finished reading central directory */
@@ -149,21 +245,14 @@ void csArchive::ReadZipDirectory (FILE *infile)
             }
             if ((!ReadCDFH (cdfh, infile))
              || (cdfh.filename_length > sizeof (buff))
-             || (fread (buff, 1, cdfh.filename_length, infile) < cdfh.filename_length))
+             || (fread (buff, 1, cdfh.filename_length, infile) <
+	     	cdfh.filename_length))
               return;           /* Broken zipfile? */
             buff[cdfh.filename_length] = 0;
 
-            if ((buff[cdfh.filename_length - 1] == '/')
-             || (buff[cdfh.filename_length - 1] == PATH_SEPARATOR))
-            {
-              if (fseek (infile, cdfh.extra_field_length + cdfh.file_comment_length, SEEK_CUR))
-                return;         /* Broken zipfile? */
-              continue;
-            } /* endif */
-
             ArchiveEntry *curentry = InsertEntry (buff, cdfh);
             if (!curentry->ReadExtraField (infile, cdfh.extra_field_length)
-             || !curentry->ReadFileComment (infile, cdfh.file_comment_length))
+              || !curentry->ReadFileComment (infile, cdfh.file_comment_length))
               return;         /* Broken zipfile? */
           } /* endfor */
         } /* endif */
@@ -198,7 +287,7 @@ void csArchive::ReadZipEntries (FILE *infile)
     buff[lfh.filename_length] = 0;
 
     if ((buff[lfh.filename_length - 1] != '/')
-        && (buff[lfh.filename_length - 1] != PATH_SEPARATOR))
+        && (buff[lfh.filename_length - 1] != CS_PATH_SEPARATOR))
     {
       /* Partialy convert lfh to cdfh */
       memset (&cdfh, 0, sizeof (cdfh));
@@ -211,14 +300,15 @@ void csArchive::ReadZipEntries (FILE *infile)
       cdfh.crc32 = lfh.crc32;
       cdfh.csize = lfh.csize;
       cdfh.ucsize = lfh.ucsize;
-      cdfh.relative_offset_local_header = cur_offs;
+      cdfh.relative_offset_local_header = (u32)cur_offs;
 
       ArchiveEntry *curentry = InsertEntry (buff, cdfh);
 
       if (!curentry->ReadExtraField (infile, lfh.extra_field_length))
         return;                 /* Broken zipfile */
     } /* endif */
-    if (fseek (infile, cur_offs = new_offs, SEEK_SET))
+    cur_offs = new_offs;
+    if (fseek (infile, (long)new_offs, SEEK_SET))
       return;                   /* Broken zipfile */
   } /* endwhile */
 }
@@ -226,11 +316,11 @@ void csArchive::ReadZipEntries (FILE *infile)
 csArchive::ArchiveEntry *csArchive::InsertEntry (const char *name,
   ZIP_central_directory_file_header &cdfh)
 {
-  int dupentry;
+  size_t dupentry;
   ArchiveEntry *e = new ArchiveEntry (name, cdfh);
-  dir.InsertSorted (e, &dupentry);
-  if (dupentry >= 0)
-    dir.Delete (dupentry);
+  dir.InsertSorted (e, ArchiveEntryVector::Compare, &dupentry);
+  if (dupentry != (size_t)-1)
+    dir.DeleteIndex (dupentry);
   return e;
 }
 
@@ -239,7 +329,7 @@ bool csArchive::ReadArchiveComment (FILE *infile, size_t zipfile_comment_length)
   if (comment && (comment_length != zipfile_comment_length))
   {
     delete [] comment;
-    comment = NULL;
+    comment = 0;
   }
 
   comment_length = zipfile_comment_length;
@@ -248,28 +338,31 @@ bool csArchive::ReadArchiveComment (FILE *infile, size_t zipfile_comment_length)
 
   if (!comment)
     comment = new char [zipfile_comment_length];
-  return (fread (comment, 1, zipfile_comment_length, infile) == zipfile_comment_length);
+  return (fread (comment, 1, zipfile_comment_length, infile) ==
+  	zipfile_comment_length);
 }
 
 void csArchive::Dir () const
 {
-  printf (" Comp |Uncomp| File |CheckSum| File\n");
-  printf (" size | size |offset| (CRC32)| name\n");
-  printf ("------+------+------+--------+------\n");
-  int fn;
+  csPrintf (" Comp |Uncomp| File |CheckSum| File\n");
+  csPrintf (" size | size |offset| (CRC32)| name\n");
+  csPrintf ("------+------+------+--------+------\n");
+  size_t fn;
   for (fn = 0; fn < dir.Length (); fn++)
   {
     ArchiveEntry *e = dir.Get (fn);
-    printf ("%6ld|%6ld|%6ld|%08x|%s\n", e->info.csize, e->info.ucsize,
-      e->info.relative_offset_local_header, (uint)e->info.crc32, e->filename);
+    csPrintf ("%6" PRIu32 "|%6" PRIu32 "|%6" PRIu32 "|%08" PRIx32 "|%s\n",
+	  e->info.csize, e->info.ucsize,
+      e->info.relative_offset_local_header, e->info.crc32, e->filename);
   }
 }
 
 void *csArchive::FindName (const char *name) const
 {
-  int idx = dir.FindSortedKey (name);
-  if (idx < 0)
-    return NULL;
+  size_t idx = dir.FindSortedKey(csArrayCmp<ArchiveEntry*,char const*>(name,
+    ArchiveEntryVector::CompareKey));
+  if (idx == (size_t)-1)
+    return 0;
   return dir.Get (idx);
 }
 
@@ -278,7 +371,7 @@ char *csArchive::Read (const char *name, size_t *size)
   ArchiveEntry *f = (ArchiveEntry *) FindName (name);
 
   if (!f)
-    return NULL;
+    return 0;
   if (size)
     *size = f->info.ucsize;
 
@@ -298,17 +391,18 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
 
   out_buff = new char[f->info.ucsize + 1];
   if (!out_buff)
-    return NULL;
+    return 0;
   out_buff [f->info.ucsize] = 0;
 
   if ((fseek (infile, f->info.relative_offset_local_header, SEEK_SET))
       || (fread (buff, 1, sizeof (hdr_local), infile) < sizeof (hdr_local))
       || (memcmp (buff, hdr_local, sizeof (hdr_local)) != 0)
       || (!ReadLFH (lfh, infile))
-      || (fseek (infile, lfh.filename_length + lfh.extra_field_length, SEEK_CUR)))
+      || (fseek (infile, lfh.filename_length + lfh.extra_field_length,
+      	SEEK_CUR)))
   {
     delete [] out_buff;
-    return NULL;
+    return 0;
   }
   switch (f->info.compression_method)
   {
@@ -317,7 +411,7 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
         if (fread (out_buff, 1, f->info.csize, infile) < f->info.csize)
         {
           delete [] out_buff;
-          return NULL;
+          return 0;
         } /* endif */
         break;
       }
@@ -336,7 +430,7 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
         if (err != Z_OK)
         {
           delete [] out_buff;
-          return NULL;
+          return 0;
         }
         while (bytes_left)
         {
@@ -347,7 +441,7 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
             size = sizeof (buff);
           else
             size = bytes_left;
-          zs.avail_in = fread (buff, 1, size, infile);
+	  zs.avail_in = (uInt)fread (buff, 1, size, infile);
 
           err = inflate (&zs, bytes_left > size ? Z_PARTIAL_FLUSH : Z_FINISH);
           bytes_left -= size;
@@ -356,12 +450,13 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
 
         /* discard dynamically allocated buffers on error */
         // Kludge warning: I've encountered a file where zlib 1.1.1 returned
-        // Z_BUF_ERROR although everything was ok (a slightly compressed PNG file)
+        // Z_BUF_ERROR although everything was ok (a slightly compressed PNG
+	// file)
         if ((err != Z_STREAM_END)
          && ((err != Z_BUF_ERROR) || bytes_left || zs.avail_out))
         {
           //delete [] out_buff;
-          //return NULL;
+          //return 0;
         }
         break;
       }
@@ -369,7 +464,7 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
       {
         /* Can't handle this compression algorythm */
         delete [] out_buff;
-        return NULL;
+        return 0;
       }
   } /* endswitch */
   return out_buff;
@@ -378,30 +473,21 @@ char *csArchive::ReadEntry (FILE *infile, ArchiveEntry * f)
 void *csArchive::NewFile (const char *name, size_t size, bool pack)
 {
   DeleteFile (name);
-
-  ZIP_central_directory_file_header cdfh;
-
-  memset (&cdfh, 0, sizeof (cdfh));
-  cdfh.version_made_by[0] = 0x16;		/* Zip version 2.2 rev 6 ??? */
-  cdfh.version_made_by[1] = 0x06;
-  cdfh.version_needed_to_extract[0] = 20;       /* Unzip version 2.0 rev 0 */
-  cdfh.version_needed_to_extract[1] = 00;
-  if (pack)
-    cdfh.compression_method = DEFAULT_COMPRESSION_METHOD;
+  size_t idx = lazy.FindKey (csArrayCmp<ArchiveEntry*,char const*>(name,
+    ArchiveEntryVector::CompareKey));
+  ArchiveEntry *f;
+  if (idx != csArrayItemNotFound)
+  {
+    // If already added without flushing, reuse the previously added
+    // entry and just reset the write pointer and compression method.
+    f = lazy.Get(idx);
+    ResetArchiveEntry (f, size, pack);
+  }
   else
-    cdfh.compression_method = ZIP_STORE;
-  cdfh.ucsize = size;
-
-  ArchiveEntry *f = new ArchiveEntry (name, cdfh);
-
-  time_t curtime = time (NULL);
-  struct tm *curtm = localtime (&curtime);
-  csFileTime ft;
-  ASSIGN_FILETIME (ft, *curtm);
-  SetFileTime ((void *)f, ft);
-
-  lazy.Push (f);
-
+  {
+    f = CreateArchiveEntry(name, size, pack);
+    lazy.Push (f);
+  }
   return (void *)f;
 }
 
@@ -410,7 +496,7 @@ bool csArchive::DeleteFile (const char *name)
   if (!FileExists (name))
     return false;
 
-  del.InsertSorted (csStrNew (name));
+  del.InsertSorted (name);
   return true;
 }
 
@@ -448,27 +534,27 @@ bool csArchive::WriteZipArchive ()
   FILE *temp;
   char buff [16 * 1024];
   bool success = false;
-  int n = 0;
+  size_t n = 0;
 
   // Check if file is opened for reading first
   if (!file) return false;
 
   // Step one: Copy archive file into a temporary file,
   // skipping entries marked as 'deleted'
-  strcpy (temp_file, TEMP_DIR);
-  int tmplen = strlen (temp_file);
+  strcpy (temp_file, CS_TEMP_DIR);
+  size_t tmplen = strlen (temp_file);
 
   APPEND_SLASH (temp_file, tmplen);
-  cs_snprintf (&temp_file[tmplen], CS_MAXPATHLEN - tmplen, TEMP_FILE);
-  //sprintf (&temp_file[tmplen], TEMP_FILE);
-  if ((temp = fopen (temp_file, "w+b")) == NULL)
+    
+  cs_snprintf (&temp_file[tmplen], CS_MAXPATHLEN - tmplen, CS_TEMP_FILE);
+  if ((temp = fopen (temp_file, "w+b")) == 0)
     return false;               /* Cannot create temporary file */
   fseek (file, 0, SEEK_SET);
 
   while (fread (buff, 1, sizeof (hdr_local), file) == sizeof (hdr_local))
   {
     size_t bytes_to_copy, bytes_to_skip;
-    ArchiveEntry *this_file = NULL;
+    ArchiveEntry *this_file = 0;
 
     if (memcmp (buff, hdr_local, sizeof (hdr_local)) == 0)
     {
@@ -526,7 +612,8 @@ skip_entry:
         goto temp_failed;
 
       bytes_to_copy = 0;
-      bytes_to_skip = cdfh.filename_length + cdfh.extra_field_length + cdfh.file_comment_length;
+      bytes_to_skip = cdfh.filename_length + cdfh.extra_field_length +
+      	cdfh.file_comment_length;
     }
     else if (memcmp (buff, hdr_endcentral, sizeof (hdr_endcentral)) == 0)
     {
@@ -534,7 +621,8 @@ skip_entry:
       ZIP_end_central_dir_record ecdr;
       char buff [ZIP_END_CENTRAL_DIR_RECORD_SIZE];
 
-      if (fread (buff, 1, ZIP_END_CENTRAL_DIR_RECORD_SIZE, file) < ZIP_END_CENTRAL_DIR_RECORD_SIZE)
+      if (fread (buff, 1, ZIP_END_CENTRAL_DIR_RECORD_SIZE, file) <
+      	ZIP_END_CENTRAL_DIR_RECORD_SIZE)
         goto temp_failed;
       LoadECDR (ecdr, buff);
 
@@ -548,7 +636,7 @@ skip_entry:
     } /* endif */
 
     if (bytes_to_skip)
-      fseek (file, bytes_to_skip, SEEK_CUR);
+	fseek (file, (long)bytes_to_skip, SEEK_CUR);
     while (bytes_to_copy)
     {
       size_t size;
@@ -587,7 +675,7 @@ skip_entry:
     fseek (temp, 0, SEEK_SET);
     fclose (file);
 
-    if ((file = fopen (filename, "wb")) == NULL)
+    if ((file = fopen (filename, "wb")) == 0)
     {
       file = fopen (filename, "rb");
       goto temp_failed;
@@ -624,13 +712,14 @@ temp_failed:
 
 bool csArchive::WriteCentralDirectory (FILE *temp)
 {
-  int n, count = 0;
+  size_t n, count = 0;
   size_t cdroffs = ftell (temp);
 
   for (n = 0; n < dir.Length (); n++)
   {
     ArchiveEntry *f = dir.Get (n);
-    if (IsDeleted (f->filename))
+    // Don't write deleted or faked entries
+    if (IsDeleted (f->filename) || f->faked)
       continue;
     if (!f->WriteCDFH (temp))
       return false;
@@ -649,11 +738,11 @@ bool csArchive::WriteCentralDirectory (FILE *temp)
   ZIP_end_central_dir_record ecdr;
 
   memset (&ecdr, 0, sizeof (ecdr));
-  ecdr.num_entries_centrl_dir_ths_disk = count;
-  ecdr.total_entries_central_dir = count;
-  ecdr.size_central_directory = ftell (temp) - cdroffs;
-  ecdr.offset_start_central_directory = cdroffs;
-  ecdr.zipfile_comment_length = comment_length;
+  ecdr.num_entries_centrl_dir_ths_disk = (ush)count;
+  ecdr.total_entries_central_dir = (ush)count;
+  ecdr.size_central_directory = (u32)(ftell (temp) - cdroffs);
+  ecdr.offset_start_central_directory = (u32)cdroffs;
+  ecdr.zipfile_comment_length = (ush)comment_length;
   if (!WriteECDR (ecdr, temp))
     return false;
   return true;
@@ -662,12 +751,13 @@ bool csArchive::WriteCentralDirectory (FILE *temp)
 void csArchive::UpdateDirectory ()
 {
   /* Update archive directory: remove deleted entries first */
-  int n;
-  for (n = dir.Length () - 1; n >= 0; n--)
+  size_t n = dir.Length ();
+  while (n > 0)
   {
+    n--;
     ArchiveEntry *e = dir.Get (n);
     if (IsDeleted (e->filename))
-      dir.Delete (n);
+      dir.DeleteIndex (n);
   }
   del.DeleteAll ();
 
@@ -675,15 +765,15 @@ void csArchive::UpdateDirectory ()
   {
     ArchiveEntry *e = lazy.Get (n);
     e->FreeBuffer ();
-    dir.InsertSorted (e);
-    lazy [n] = NULL;
+    dir.InsertSorted (e, ArchiveEntryVector::Compare);
+    lazy.Put (n, 0);
   }
   lazy.DeleteAll ();
 }
 
 bool csArchive::IsDeleted (const char *name) const
 {
-  return (del.FindSortedKey (name) >= 0);
+  return (del.FindSortedKey (name) != csArrayItemNotFound);
 }
 
 void csArchive::UnpackTime (ush zdate, ush ztime, csFileTime & rtime) const
@@ -699,7 +789,8 @@ void csArchive::UnpackTime (ush zdate, ush ztime, csFileTime & rtime) const
   rtime.sec = ((ztime & 0x1f) * 2);
 }
 
-void csArchive::PackTime (const csFileTime & ztime, ush & rdate, ush & rtime) const
+void csArchive::PackTime (const csFileTime & ztime, ush & rdate,
+	ush & rtime) const
 {
   rdate = (((ztime.year - 1980) & 0x7f) << 9)
         | (((ztime.mon & 0x0f) + 1) << 5)
@@ -713,10 +804,12 @@ void csArchive::LoadECDR (ZIP_end_central_dir_record & ecdr, char *buff)
 {
   ecdr.number_this_disk = BUFF_GET_SHORT (E_NUMBER_THIS_DISK);
   ecdr.num_disk_start_cdir = BUFF_GET_SHORT (E_NUM_DISK_WITH_START_CENTRAL_DIR);
-  ecdr.num_entries_centrl_dir_ths_disk = BUFF_GET_SHORT (E_NUM_ENTRIES_CENTRL_DIR_THS_DISK);
+  ecdr.num_entries_centrl_dir_ths_disk = BUFF_GET_SHORT (
+  	E_NUM_ENTRIES_CENTRL_DIR_THS_DISK);
   ecdr.total_entries_central_dir = BUFF_GET_SHORT (E_TOTAL_ENTRIES_CENTRAL_DIR);
   ecdr.size_central_directory = BUFF_GET_LONG (E_SIZE_CENTRAL_DIRECTORY);
-  ecdr.offset_start_central_directory = BUFF_GET_LONG (E_OFFSET_START_CENTRAL_DIRECTORY);
+  ecdr.offset_start_central_directory = BUFF_GET_LONG (
+  	E_OFFSET_START_CENTRAL_DIRECTORY);
   ecdr.zipfile_comment_length = BUFF_GET_SHORT (E_ZIPFILE_COMMENT_LENGTH);
 }
 
@@ -724,7 +817,8 @@ bool csArchive::ReadCDFH (ZIP_central_directory_file_header & cdfh, FILE *infile
 {
   char buff[ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE];
 
-  if (fread (buff, 1, ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE, infile) < ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE)
+  if (fread (buff, 1, ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE, infile) <
+  	ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE)
     return false;
 
   cdfh.version_made_by[0] = buff[C_VERSION_MADE_BY_0];
@@ -744,7 +838,8 @@ bool csArchive::ReadCDFH (ZIP_central_directory_file_header & cdfh, FILE *infile
   cdfh.disk_number_start = BUFF_GET_SHORT (C_DISK_NUMBER_START);
   cdfh.internal_file_attributes = BUFF_GET_SHORT (C_INTERNAL_FILE_ATTRIBUTES);
   cdfh.external_file_attributes = BUFF_GET_LONG (C_EXTERNAL_FILE_ATTRIBUTES);
-  cdfh.relative_offset_local_header = BUFF_GET_LONG (C_RELATIVE_OFFSET_LOCAL_HEADER);
+  cdfh.relative_offset_local_header = BUFF_GET_LONG (
+  	C_RELATIVE_OFFSET_LOCAL_HEADER);
 
   return true;
 }
@@ -753,7 +848,8 @@ bool csArchive::ReadLFH (ZIP_local_file_header & lfh, FILE *infile)
 {
   char buff[ZIP_LOCAL_FILE_HEADER_SIZE];
 
-  if (fread (buff, 1, ZIP_LOCAL_FILE_HEADER_SIZE, infile) < ZIP_LOCAL_FILE_HEADER_SIZE)
+  if (fread (buff, 1, ZIP_LOCAL_FILE_HEADER_SIZE, infile) <
+  	ZIP_LOCAL_FILE_HEADER_SIZE)
     return false;
 
   lfh.version_needed_to_extract[0] = buff[L_VERSION_NEEDED_TO_EXTRACT_0];
@@ -775,19 +871,23 @@ bool csArchive::WriteECDR (ZIP_end_central_dir_record & ecdr, FILE *outfile)
 {
   char buff[ZIP_END_CENTRAL_DIR_RECORD_SIZE];
 
-  if (fwrite (hdr_endcentral, 1, sizeof (hdr_endcentral), outfile) != sizeof (hdr_endcentral))
+  if (fwrite (hdr_endcentral, 1, sizeof (hdr_endcentral), outfile) !=
+  	sizeof (hdr_endcentral))
     return false;
 
   BUFF_SET_SHORT (E_NUMBER_THIS_DISK, ecdr.number_this_disk);
   BUFF_SET_SHORT (E_NUM_DISK_WITH_START_CENTRAL_DIR, ecdr.num_disk_start_cdir);
-  BUFF_SET_SHORT (E_NUM_ENTRIES_CENTRL_DIR_THS_DISK, ecdr.num_entries_centrl_dir_ths_disk);
+  BUFF_SET_SHORT (E_NUM_ENTRIES_CENTRL_DIR_THS_DISK,
+  	ecdr.num_entries_centrl_dir_ths_disk);
   BUFF_SET_SHORT (E_TOTAL_ENTRIES_CENTRAL_DIR, ecdr.total_entries_central_dir);
   BUFF_SET_LONG (E_SIZE_CENTRAL_DIRECTORY, ecdr.size_central_directory);
-  BUFF_SET_LONG (E_OFFSET_START_CENTRAL_DIRECTORY, ecdr.offset_start_central_directory);
+  BUFF_SET_LONG (E_OFFSET_START_CENTRAL_DIRECTORY,
+  	ecdr.offset_start_central_directory);
   BUFF_SET_SHORT (E_ZIPFILE_COMMENT_LENGTH, ecdr.zipfile_comment_length);
 
-  if ((fwrite (buff, 1, ZIP_END_CENTRAL_DIR_RECORD_SIZE, outfile) != ZIP_END_CENTRAL_DIR_RECORD_SIZE)
-      || (fwrite (comment, 1, comment_length, outfile) != comment_length))
+  if ((fwrite (buff, 1, ZIP_END_CENTRAL_DIR_RECORD_SIZE, outfile) !=
+  	ZIP_END_CENTRAL_DIR_RECORD_SIZE) ||
+	(fwrite (comment, 1, comment_length, outfile) != comment_length))
     return false;
   return true;
 }
@@ -798,11 +898,12 @@ csArchive::ArchiveEntry::ArchiveEntry (const char *name,
   filename = new char[strlen (name) + 1];
   strcpy (filename, name);
   info = cdfh;
-  buffer = NULL;
-  extrafield = NULL;
-  comment = NULL;
+  buffer = 0;
+  extrafield = 0;
+  comment = 0;
   buffer_pos = 0;
   buffer_size = 0;
+  faked = false;
 }
 
 csArchive::ArchiveEntry::~ArchiveEntry ()
@@ -816,7 +917,7 @@ csArchive::ArchiveEntry::~ArchiveEntry ()
 void csArchive::ArchiveEntry::FreeBuffer ()
 {
   if (buffer) free (buffer);
-  buffer = NULL;
+  buffer = 0;
   buffer_pos = 0;
   buffer_size = 0;
 }
@@ -839,7 +940,7 @@ bool csArchive::ArchiveEntry::Append (const void *data, size_t size)
   } /* endif */
 
   if (info.ucsize < buffer_pos + size)
-    info.ucsize = buffer_pos + size;
+    info.ucsize = (u32)(buffer_pos + size);
 
   memcpy (buffer + buffer_pos, data, size);
   buffer_pos += size;
@@ -860,17 +961,22 @@ bool csArchive::ArchiveEntry::WriteLFH (FILE *outfile)
   BUFF_SET_LONG (L_CRC32, info.crc32);
   BUFF_SET_LONG (L_COMPRESSED_SIZE, info.csize);
   BUFF_SET_LONG (L_UNCOMPRESSED_SIZE, info.ucsize);
-  BUFF_SET_SHORT (L_FILENAME_LENGTH, info.filename_length = strlen (filename));
+  info.filename_length = (ush)strlen (filename);
+  BUFF_SET_SHORT (L_FILENAME_LENGTH, info.filename_length);
+  info.extra_field_length = extrafield ? info.extra_field_length : 0;
   BUFF_SET_SHORT (L_EXTRA_FIELD_LENGTH,
-                  info.extra_field_length = extrafield ? info.extra_field_length : 0);
+                  info.extra_field_length);
 
   if ((fwrite (hdr_local, 1, sizeof (hdr_local), outfile) < sizeof (hdr_local))
-      || (fwrite (buff, 1, ZIP_LOCAL_FILE_HEADER_SIZE, outfile) < ZIP_LOCAL_FILE_HEADER_SIZE)
-      || (fwrite (filename, 1, info.filename_length, outfile) < info.filename_length)
-      || (fwrite (extrafield, 1, info.extra_field_length, outfile) < info.extra_field_length))
+      || (fwrite (buff, 1, ZIP_LOCAL_FILE_HEADER_SIZE, outfile) <
+          ZIP_LOCAL_FILE_HEADER_SIZE)
+      || (fwrite (filename, 1, info.filename_length, outfile) <
+          info.filename_length)
+      || (fwrite (extrafield, 1, info.extra_field_length, outfile) <
+          info.extra_field_length))
     return false;
 
-  info.relative_offset_local_header = lfhpos;
+  info.relative_offset_local_header = (u32)lfhpos;
   return true;
 }
 
@@ -892,56 +998,68 @@ bool csArchive::ArchiveEntry::WriteCDFH (FILE *outfile)
   BUFF_SET_LONG (C_COMPRESSED_SIZE, info.csize);
   BUFF_SET_LONG (C_UNCOMPRESSED_SIZE, info.ucsize);
 
-  BUFF_SET_SHORT (C_FILENAME_LENGTH, info.filename_length = strlen (filename));
-  /* We're ignoring extra field for central directory, although InfoZIP puts there a field containing EF_TIME -
-     universal timestamp - but for example DOS pkzip/pkunzip does not put nothing there. */
+  info.filename_length = (ush)strlen (filename);
+  BUFF_SET_SHORT (C_FILENAME_LENGTH, info.filename_length);
+  /* We're ignoring extra field for central directory, although InfoZIP puts 
+   * there a field containing EF_TIME - universal timestamp - but for example
+   * DOS pkzip/pkunzip does not put nothing there. */
   BUFF_SET_SHORT (C_EXTRA_FIELD_LENGTH, 0);
+  info.file_comment_length = comment ? info.file_comment_length : 0;
   BUFF_SET_SHORT (C_FILE_COMMENT_LENGTH,
-                  info.file_comment_length = comment ? info.file_comment_length : 0);
+                  info.file_comment_length);
   BUFF_SET_SHORT (C_DISK_NUMBER_START, info.disk_number_start);
   BUFF_SET_SHORT (C_INTERNAL_FILE_ATTRIBUTES, info.internal_file_attributes);
   BUFF_SET_LONG (C_EXTERNAL_FILE_ATTRIBUTES, info.external_file_attributes);
-  BUFF_SET_LONG (C_RELATIVE_OFFSET_LOCAL_HEADER, info.relative_offset_local_header);
+  BUFF_SET_LONG (C_RELATIVE_OFFSET_LOCAL_HEADER,
+  	info.relative_offset_local_header);
 
-  if ((fwrite (hdr_central, 1, sizeof (hdr_central), outfile) < sizeof (hdr_central))
-      || (fwrite (buff, 1, ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE, outfile) < ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE)
-      || (fwrite (filename, 1, info.filename_length, outfile) < info.filename_length)
-      || (fwrite (comment, 1, info.file_comment_length, outfile) < info.file_comment_length))
+  if ((fwrite (hdr_central, 1, sizeof (hdr_central), outfile) <
+  	sizeof (hdr_central))
+      || (fwrite (buff, 1, ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE, outfile) <
+      	  ZIP_CENTRAL_DIRECTORY_FILE_HEADER_SIZE)
+      || (fwrite (filename, 1, info.filename_length, outfile) <
+      	  info.filename_length)
+      || (fwrite (comment, 1, info.file_comment_length, outfile) <
+      	  info.file_comment_length))
     return false;
 
   return true;
 }
 
-bool csArchive::ArchiveEntry::ReadExtraField (FILE *infile, size_t extra_field_length)
+bool csArchive::ArchiveEntry::ReadExtraField (FILE *infile,
+	size_t extra_field_length)
 {
   if (extrafield && (info.extra_field_length != extra_field_length))
   {
     delete [] extrafield;
-    extrafield = NULL;
+    extrafield = 0;
   }
-  info.extra_field_length = extra_field_length;
+  info.extra_field_length = (ush)extra_field_length;
   if (extra_field_length)
   {
     if (!extrafield)
       extrafield = new char[extra_field_length];
-    return (fread (extrafield, 1, extra_field_length, infile) == extra_field_length);
+    return (fread (extrafield, 1, extra_field_length, infile) ==
+    	extra_field_length);
   }
   else return true;
 }
 
-bool csArchive::ArchiveEntry::ReadFileComment (FILE *infile, size_t file_comment_length)
+bool csArchive::ArchiveEntry::ReadFileComment (FILE *infile,
+	size_t file_comment_length)
 {
   if (comment && (info.file_comment_length != file_comment_length))
   {
     delete [] comment;
-    comment = NULL;
+    comment = 0;
   }
-  info.file_comment_length = file_comment_length;
+  info.file_comment_length = (ush)file_comment_length;
   if (file_comment_length)
   {
     if (!comment)
       comment = new char[file_comment_length];
-    return (fread (comment, 1, file_comment_length, infile) == file_comment_length);
+    return (fread (comment, 1, file_comment_length, infile) ==
+    	file_comment_length);
   }
   else return true;
 }
@@ -950,13 +1068,15 @@ bool csArchive::ArchiveEntry::WriteFile (FILE *outfile)
 {
   size_t lfhoffs = ftell (outfile);
 
-  info.crc32 = crc32 (CRCVAL_INITIAL, (z_Byte *)buffer, buffer_pos);
+  info.crc32 = crc32 (CRCVAL_INITIAL, (z_Byte *)buffer, (uInt)buffer_pos);
   bool finished = false;
 
   while (!finished)
   {
-    if (fseek (outfile, lfhoffs + sizeof (hdr_local) + ZIP_LOCAL_FILE_HEADER_SIZE +
-               strlen (filename) + (extrafield ? info.extra_field_length : 0), SEEK_SET))
+    if (fseek (outfile, (long)(lfhoffs + sizeof (hdr_local) +
+    	ZIP_LOCAL_FILE_HEADER_SIZE +
+	strlen (filename) + (extrafield ? info.extra_field_length : 0)),
+		SEEK_SET))
       return false;
 
     switch (info.compression_method)
@@ -965,7 +1085,7 @@ bool csArchive::ArchiveEntry::WriteFile (FILE *outfile)
         {
           if (fwrite (buffer, 1, buffer_pos, outfile) < buffer_pos)
             return false;       /* Write error */
-          info.csize = info.ucsize = buffer_pos;
+          info.csize = info.ucsize = (u32)buffer_pos;
           finished = true;
           break;
         }
@@ -976,11 +1096,11 @@ bool csArchive::ArchiveEntry::WriteFile (FILE *outfile)
           zs.zalloc = (alloc_func) 0;
           zs.zfree = (free_func) 0;
           zs.next_in = (z_Byte *) buffer;
-          zs.avail_in = buffer_pos;
+          zs.avail_in = (uInt)buffer_pos;
           if (deflateInit (&zs, DEFAULT_COMPRESSION_LEVEL) != Z_OK)
             return false;
           info.csize = 0;
-          info.ucsize = buffer_pos;
+          info.ucsize = (u32)buffer_pos;
 
           char buff[16384];
           int buffofs = 2;      /* Skip inflated data header */
@@ -993,7 +1113,7 @@ bool csArchive::ArchiveEntry::WriteFile (FILE *outfile)
             int rc = deflate (&zs, Z_FINISH);   /* Do actual compression */
             size_t size = sizeof (buff) - zs.avail_out - buffofs;
 
-            info.csize += size;
+            info.csize += (u32)size;
 
             if (fwrite (&buff[buffofs], 1, size, outfile) != size)
             {
@@ -1016,7 +1136,7 @@ bool csArchive::ArchiveEntry::WriteFile (FILE *outfile)
     } /* endswitch */
   } /* endwhile */
 
-  fseek (outfile, lfhoffs, SEEK_SET);
+  fseek (outfile, (long)lfhoffs, SEEK_SET);
   if (!WriteLFH (outfile))
     return false;
   fseek (outfile, info.csize, SEEK_CUR);
