@@ -18,10 +18,13 @@
 
 #include "cssysdef.h"
 #include "csutil/cfgfile.h"
-#include "csutil/databuf.h"
 #include "csutil/csstring.h"
-#include "csutil/util.h"
+#include "csutil/databuf.h"
+#include "csutil/physfile.h"
+#include "csutil/scf_implementation.h"
 #include "csutil/snprintf.h"
+#include "csutil/sysfunc.h"
+#include "csutil/util.h"
 #include "iutil/vfs.h"
 #include <ctype.h>
 
@@ -129,9 +132,8 @@ void csConfigNode::SetStr(const char *s)
 
 void csConfigNode::SetInt(int n)
 {
-  char output [32];
-  cs_snprintf (output, sizeof(output), "%d", n);
-  //sprintf (output, "%d", n);
+  csString output;
+  output.Format ("%d", n);
   SetStr (output);
 }
 
@@ -139,7 +141,6 @@ void csConfigNode::SetFloat(float f)
 {
   char output [64];
   cs_snprintf (output, sizeof(output), "%g", f);
-  //sprintf (output, "%g", f);
   SetStr (output);
 }
 
@@ -185,10 +186,10 @@ const char *csConfigNode::GetComment() const
 
 /* config iterator */
 
-class csConfigIterator : public iConfigIterator
+class csConfigIterator : public scfImplementation1<csConfigIterator, 
+                                                   iConfigIterator>
 {
 public:
-  SCF_DECLARE_IBASE;
 
   /// Returns the configuration object for this iterator.
   virtual iConfigFile *GetConfigFile() const;
@@ -218,6 +219,11 @@ public:
   /// Get the comment of the given key, or 0 if no comment exists.
   virtual const char *GetComment() const;
 
+  // Create a new iterator.
+  csConfigIterator(csConfigFile *Config, const char *Subsection);
+  // Delete this iterator
+  virtual ~csConfigIterator();
+
 private:
   friend class csConfigFile;
   csConfigFile *Config;
@@ -225,10 +231,6 @@ private:
   char *Subsection;
   size_t SubsectionLength;
 
-  // Create a new iterator.
-  csConfigIterator(csConfigFile *Config, const char *Subsection);
-  // Delete this iterator
-  virtual ~csConfigIterator();
   // Utility function to check if a key meets subsection requirement
   bool CheckSubsection(const char *Key) const;
   // Move to the previous node, ignoring subsection
@@ -239,14 +241,9 @@ private:
   bool Prev ();
 };
 
-SCF_IMPLEMENT_IBASE(csConfigIterator);
-  SCF_IMPLEMENTS_INTERFACE(iConfigIterator);
-SCF_IMPLEMENT_IBASE_END;
-
 csConfigIterator::csConfigIterator(csConfigFile *c, const char *sub)
+  : scfImplementationType (this), Config (c)
 {
-  SCF_CONSTRUCT_IBASE(0);
-  Config = c;
   Node = Config->FirstNode;
   Subsection = csStrNew(sub);
   SubsectionLength = (Subsection ? strlen(Subsection) : 0);
@@ -258,7 +255,6 @@ csConfigIterator::~csConfigIterator()
   Config->RemoveIterator(this);
   delete[] Subsection;
   Config->DecRef();
-  SCF_DESTRUCT_IBASE ();
 }
 
 iConfigFile *csConfigIterator::GetConfigFile() const
@@ -350,10 +346,6 @@ const char *csConfigIterator::GetComment() const
 
 /* configuation object */
 
-SCF_IMPLEMENT_IBASE(csConfigFile);
-  SCF_IMPLEMENTS_INTERFACE(iConfigFile);
-SCF_IMPLEMENT_IBASE_END;
-
 void csConfigFile::InitializeObject ()
 {
   FirstNode = new csConfigNode(0);
@@ -367,14 +359,14 @@ void csConfigFile::InitializeObject ()
 }
 
 csConfigFile::csConfigFile(iBase *pBase)
+  : scfImplementationType (this, pBase)
 {
-  SCF_CONSTRUCT_IBASE (pBase);
   InitializeObject ();
 }
 
 csConfigFile::csConfigFile(const char *file, iVFS *vfs)
+  : scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE(0);
   InitializeObject ();
   if (file)
     Load(file, vfs);
@@ -390,7 +382,12 @@ csConfigFile::~csConfigFile()
   CS_ASSERT(Iterators->Length() == 0);
   delete Iterators;
   delete[] Filename;
-  SCF_DESTRUCT_IBASE ();
+}
+
+bool csConfigFile::IsEmpty() const
+{
+  csConfigNode* const n = FirstNode->GetNext();
+  return n == 0 || n->GetName() == 0;
 }
 
 const char* csConfigFile::GetFileName() const
@@ -533,7 +530,7 @@ void csConfigFile::Clear()
   // delete all nodes but the first and last one
   FirstNode->DeleteDataNodes();
   // rewind all iterators
-  for (long i = 0; i < Iterators->Length(); i++)
+  for (size_t i = 0; i < Iterators->Length(); i++)
   {
     csConfigIterator *it = Iterators->Get(i);
     it->Rewind();
@@ -663,7 +660,7 @@ void csConfigFile::DeleteKey(const char *Name)
   if (!Node) return;
 
   // look for iterators on that node
-  for (long i = 0; i < Iterators->Length(); i++)
+  for (size_t i = 0; i < Iterators->Length(); i++)
   {
     csConfigIterator *it = (csConfigIterator*)Iterators->Get(i);
     if (it->Node == Node) it->Prev();
@@ -682,23 +679,17 @@ bool csConfigFile::LoadNow(const char *fName, iVFS *vfs, bool overwrite)
   if (vfs)
   {
     Filedata = vfs->ReadFile(fName);
-    if (!Filedata) return false;
   }
   else
   {
-    FILE *fp = fopen(fName, "rb");
-    if (!fp) return false;
-    fseek(fp, 0, SEEK_END);
-    size_t Size = ftell(fp);
-    fseek(fp, 0, SEEK_SET);
-    Filedata = csPtr<iDataBuffer> (new csDataBuffer(Size + 1));
-    fread(Filedata->GetData(), sizeof(char), Size, fp);
-    fclose(fp);
-    Filedata->GetInt8()[Size] = 0;
+    csRef<iFile> file;
+    file.AttachNew (new csPhysicalFile (fName, "rb"));
+    Filedata = file->GetAllData (true);
   }
+  if (!Filedata) return false;
 
   // parse the data
-  LoadFromBuffer((char*)Filedata->GetInt8(), overwrite);
+  LoadFromBuffer ((char*)Filedata->GetInt8(), overwrite);
 
   return true;
 }
@@ -738,7 +729,7 @@ void csConfigFile::LoadFromBuffer(char *Filedata, bool overwrite)
       // if no equal sign, this is an invalid line
       if (!t)
       {
-        fprintf(stderr, "Missing `=' on line %d of %s\n", Line,
+        csFPrintf(stderr, "Missing `=' on line %d of %s\n", Line,
 	  (Filename ? Filename : "configuration data"));
         CurrentComment.Clear();
         continue;
@@ -746,7 +737,7 @@ void csConfigFile::LoadFromBuffer(char *Filedata, bool overwrite)
       // check for missing key name
       if (t == Filedata)
       {
-        fprintf(stderr, "Missing key name (before `=') on line %d of %s\n",
+        csFPrintf(stderr, "Missing key name (before `=') on line %d of %s\n",
 	  Line, (Filename ? Filename : "configuration data"));
         CurrentComment.Clear();
         continue;
@@ -811,8 +802,8 @@ csConfigNode *csConfigFile::CreateNode(const char *Name)
 
 void csConfigFile::RemoveIterator(csConfigIterator *it) const
 {
-  int n = Iterators->Find(it);
-  CS_ASSERT(n != -1);
+  size_t n = Iterators->Find(it);
+  CS_ASSERT(n != csArrayItemNotFound);
   Iterators->DeleteIndex(n);
 }
 
