@@ -16,20 +16,21 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <stdio.h>
 #include <string.h>
-#include "cssysdef.h"
 #include "csutil/cmdhelp.h"
 #include "csutil/csevent.h"
 #include "csutil/snprintf.h"
+#include "csutil/sysfunc.h"
 #include "iutil/objreg.h"
 #include "iutil/cmdline.h"
-#include "iutil/config.h"
 #include "iutil/eventq.h"
 #include "iutil/plugin.h"
+#include "iutil/pluginconfig.h"
 
 
-void csCommandLineHelper::Help (iConfig* config)
+void csCommandLineHelper::Help (iPluginConfig* config)
 {
   int i;
   for (i = 0 ; ; i++)
@@ -37,36 +38,37 @@ void csCommandLineHelper::Help (iConfig* config)
     csOptionDescription option;
     if (!config->GetOptionDescription (i, &option))
       break;
-    char opt [30], desc [80];
+    csString opt;
+    csStringFast<80> desc;
     csVariant def;
     config->GetOption (i, &def);
     switch (option.type)
     {
       case CSVAR_BOOL:
-        sprintf (opt,  "  -[no]%s", option.name);
-	sprintf (desc, "%s (%s) ", option.description, def.GetBool ()
+        opt.Format ("  -[no]%s", option.name);
+	desc.Format ("%s (%s) ", option.description, def.GetBool ()
 		? "yes" : "no");
 	break;
       case CSVAR_CMD:
-        sprintf (opt, "  -%s", option.name);
-	strcpy (desc, option.description);
+        opt.Format ("  -%s", option.name);
+	desc = option.description;
 	break;
       case CSVAR_FLOAT:
-        sprintf (opt,  "  -%s=<val>", option.name);
-	sprintf (desc, "%s (%g)", option.description, def.GetFloat ());
+        opt.Format ("  -%s=<val>", option.name);
+	desc.Format ("%s (%g)", option.description, def.GetFloat ());
 	break;
       case CSVAR_LONG:
-        sprintf (opt,  "  -%s=<val>", option.name);
-	sprintf (desc, "%s (%ld)", option.description, def.GetLong ());
+        opt.Format ("  -%s=<val>", option.name);
+	desc.Format ("%s (%ld)", option.description, def.GetLong ());
 	break;
       case CSVAR_STRING:
-        sprintf (opt,  "  -%s=<val>", option.name);
-	sprintf (desc, "%s (%s)", option.description, def.GetString ()
+        opt.Format ("  -%s=<val>", option.name);
+	desc.Format ("%s (%s)", option.description, def.GetString ()
 		? def.GetString () : "none");
 	break;
     }
     //@@@????
-    printf ("%-21s%s\n", opt, desc);
+    csPrintf ("%-21s%s\n", opt.GetData(), desc.GetData());
     //ReportSys (CS_MSG_STDOUT, "%-21s%s\n", opt, desc);
   }
 }
@@ -79,7 +81,7 @@ void csCommandLineHelper::Help (iObjectRegistry* object_reg,
     cmd = CS_QUERY_REGISTRY (object_reg, iCommandLineParser);
   CS_ASSERT (cmd != 0);
 
-  // First send a global cscmdCommandLineHelp event.
+  // First send a global csevCommandLineHelp event.
   csRef<iEventQueue> evq (CS_QUERY_REGISTRY (object_reg, iEventQueue));
   if (evq)
   {
@@ -88,7 +90,7 @@ void csCommandLineHelper::Help (iObjectRegistry* object_reg,
     // We use ImmediateBroadcast here because after processing commandline
     // help the application usually exits. This means there is no chance
     // to actually process the event in the queue.
-    evout->ImmediateBroadcast (cscmdCommandLineHelp, 0);
+    evout->ImmediateBroadcast (csevCommandLineHelp(object_reg), 0);
   }
 
   csRef<iPluginManager> plgmgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
@@ -96,26 +98,26 @@ void csCommandLineHelper::Help (iObjectRegistry* object_reg,
   while (it->HasNext ())
   {
     iBase* plug = it->Next ();
-    csRef<iConfig> config (SCF_QUERY_INTERFACE (plug, iConfig));
+    csRef<iPluginConfig> config (SCF_QUERY_INTERFACE (plug, iPluginConfig));
     if (config)
     {
       csRef<iFactory> fact (SCF_QUERY_INTERFACE (plug, iFactory));
       if (fact)
-        printf ("Options for %s:\n", fact->QueryDescription ());
+        csPrintf ("Options for %s:\n", fact->QueryDescription ());
       else
-        printf ("Options for unknown plugin:\n");
+        csPrintf ("Options for unknown plugin:\n");
       Help (config);
     }
   }
 
   //@@@???
-  printf (
+  csPrintf (
 "General options:\n"
 "  -help              this help\n"
 "  -video=<s>         the 3D rendering driver (opengl, software, ...)\n"
 "  -canvas=<s>        the 2D canvas driver (asciiart, x2d, ...)\n"
 "  -plugin=<s>        load the plugin after all others\n"
-"  -verbose           be more verbose; print better diagonstic messages\n");
+"  -verbose           be more verbose; print better diagnostic messages\n");
 }
 
 bool csCommandLineHelper::CheckHelp (iObjectRegistry* object_reg,
