@@ -20,32 +20,26 @@
 #include "csutil/syspath.h"
 #include "csutil/cmdline.h"
 
-SCF_IMPLEMENT_IBASE (csCommandLineParser)
-  SCF_IMPLEMENTS_INTERFACE (iCommandLineParser)
-SCF_IMPLEMENT_IBASE_END
-
-csCommandLineParser::csCommandLineParser (iBase *Parent) : Names (16, 16)
+csCommandLineParser::csCommandLineParser (iBase *Parent) 
+  : scfImplementationType (this, Parent), Names (16, 16)
 {
-  SCF_CONSTRUCT_IBASE (Parent);
 }
 
-csCommandLineParser::csCommandLineParser (int argc, const char* const argv[]) :
-  Names (16, 16)
+csCommandLineParser::csCommandLineParser (int argc, const char* const argv[]) 
+  : scfImplementationType (this),  Names (16, 16)
 {
-  SCF_CONSTRUCT_IBASE (0);
   Initialize (argc, argv);
 }
 
 csCommandLineParser::~csCommandLineParser()
 {
-  SCF_DESTRUCT_IBASE ();
 }
 
 void csCommandLineParser::Initialize (int argc, const char* const argv[])
 {
-  resDir  = csGetResourceDir (argv[0]);
-  appDir  = csGetAppDir      (argv[0]);
-  appPath = csGetAppPath     (argv[0]);
+  resDir  = csInstallationPathsHelper::GetResourceDir (argv[0]);
+  appDir  = csInstallationPathsHelper::GetAppDir      (argv[0]);
+  appPath = csInstallationPathsHelper::GetAppPath     (argv[0]);
 
   for (int i = 1; i < argc; i++)
   {
@@ -78,9 +72,9 @@ void csCommandLineParser::Reset()
 }
 
 csCommandLineOption*
-csCommandLineParser::FindOption (const char *name, int iIndex) const
+csCommandLineParser::FindOption (const char *name, size_t iIndex) const
 {
-  int idx;
+  size_t idx;
   for (idx = 0 ; idx < Options.Length () ; idx++)
   {
     csCommandLineOption* cl = Options[idx];
@@ -104,7 +98,7 @@ csCommandLineParser::FindOption (const char *name, int iIndex) const
 }
 
 bool csCommandLineParser::ReplaceOption (
-  const char *iName, const char *iValue, int iIndex)
+  const char *iName, const char *iValue, size_t iIndex)
 {
   csCommandLineOption *clo = FindOption (iName, iIndex);
   if (clo)
@@ -117,26 +111,26 @@ bool csCommandLineParser::ReplaceOption (
     return false;
 }
 
-bool csCommandLineParser::ReplaceName (const char *iValue, int iIndex)
+bool csCommandLineParser::ReplaceName (const char *iValue, size_t iIndex)
 {
-  if ((iIndex >= 0) && (iIndex < Names.Length ()))
+  if (iIndex < Names.Length ())
   {
-    Names.Put (iIndex, (char*)iValue);
+    Names.Put (iIndex, iValue);
     return true;
   }
   else
     return false;
 }
 
-const char *csCommandLineParser::GetOption(const char *iName, int iIndex) const
+const char *csCommandLineParser::GetOption(const char *iName, size_t iIndex) const
 {
   csCommandLineOption *clo = FindOption (iName, iIndex);
   return clo ? (clo->Value ? clo->Value : "") : 0;
 }
 
-const char *csCommandLineParser::GetName (int iIndex) const
+const char *csCommandLineParser::GetName (size_t iIndex) const
 {
-  if ((iIndex >= 0) && (iIndex < Names.Length ()))
+  if (iIndex < Names.Length ())
     return Names[iIndex];
   return 0;
 }
@@ -158,10 +152,10 @@ bool csCommandLineParser::GetBoolOption(const char *iName, bool defaultValue)
   CS_ALLOC_STACK_ARRAY (char, negName, strlen(iName)+3);
   strcpy (negName, "no");
   strcpy (negName+2, iName);
-  int idx;
-  for (idx = Options.Length ()-1; idx >= 0; idx--)
+  size_t idx;
+  for (idx = Options.Length (); idx > 0; idx--)
   {
-    csCommandLineOption* cl = Options[idx];
+    csCommandLineOption* cl = Options[idx - 1];
     if (!strcmp (cl->Name, iName))
     {
       result = true;
@@ -177,17 +171,17 @@ bool csCommandLineParser::GetBoolOption(const char *iName, bool defaultValue)
   return result;
 }
 
-const csString& csCommandLineParser::GetResourceDir ()
+const char* csCommandLineParser::GetResourceDir ()
 {
   return resDir;
 }
 
-const csString& csCommandLineParser::GetAppDir ()
+const char* csCommandLineParser::GetAppDir ()
 {
   return appDir;
 }
 
-const csString& csCommandLineParser::GetAppPath ()
+const char* csCommandLineParser::GetAppPath ()
 {
   return appPath;
 }
