@@ -24,25 +24,41 @@
 #include "csutil/csevent.h"
 #include "csutil/csinput.h"
 #include "csutil/sysfunc.h"
+#include "csutil/event.h"
+#include "csutil/csuctransform.h"
+#include "csutil/eventnames.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
 
+#ifdef CS_KEY_DEBUG_ENABLE
+#include "cskey_identstrs.h"
+#endif
+
+#undef NOP
+#define NOP ((char)-1)
 // This array defines first 32..128 character codes with SHIFT key applied
 static char ShiftedKey [128-32] =
 {
 ' ', '!', '"', '#', '$', '%', '&', '"', '(', ')', '*', '+', '<', '_', '>', '?',
 ')', '!', '@', '#', '$', '%', '^', '&', '*', '(', ':', ':', '<', '+', '>', '?',
-'@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
-'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '{', '|', '}', '^', '_',
-'~', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
-'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '{', '|', '}', '~', 127
+'@', NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
+NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, '{', '|', '}', '^', '_',
+'~', NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
+NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, '{', '|', '}', '~', NOP
 };
+#undef NOP
 
 //--//--//--//--//--//--//--//--//--//--//--//--//--/> Input driver <--//--//--
 
 csInputDriver::csInputDriver(iObjectRegistry* r) :
-  Registered(false), Registry(r), Listener(0)
+  Registered(false), 
+  Registry(r), NameRegistry(csEventNameRegistry::GetRegistry(r)), 
+  Listener(0), 
+  FocusChanged(csevFocusChanged(r)),
+  FocusGained(csevFocusGained(r)),
+  FocusLost(csevFocusLost(r))
 {
+  CS_ASSERT(r != 0);
 }
 
 csInputDriver::~csInputDriver()
@@ -52,7 +68,7 @@ csInputDriver::~csInputDriver()
 
 csPtr<iEventQueue> csInputDriver::GetEventQueue()
 {
-  return CS_QUERY_REGISTRY(Registry, iEventQueue);
+  return csQueryRegistry<iEventQueue> (Registry);
 }
 
 void csInputDriver::StartListening()
@@ -62,7 +78,7 @@ void csInputDriver::StartListening()
     csRef<iEventQueue> q (GetEventQueue());
     if (q != 0)
     {
-      q->RegisterListener(Listener, CSMASK_Broadcast);
+      q->RegisterListener(Listener, FocusChanged);
       Registered = true;
     }
   }
@@ -89,33 +105,28 @@ void csInputDriver::Post(iEvent* e)
 
 bool csInputDriver::HandleEvent(iEvent& e)
 {
-  const bool focusChanged = (e.Type == csevBroadcast &&
-    e.Command.Code == cscmdFocusChanged);
-  if (focusChanged) 
+  if (e.Name == FocusLost)
   {
-    if (!e.Command.Info) // Application lost focus.
-      LostFocus();
-    if (e.Command.Info) // Application gained focus.
-      GainFocus();
+    LostFocus();
+    return true;
   }
-  return focusChanged;
+  else if (e.Name == FocusGained)
+  {
+    GainFocus();
+    return true;
+  }
+  return false;
 }
 
 //-----------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(csKeyComposer)
-  SCF_IMPLEMENTS_INTERFACE(iKeyComposer)
-SCF_IMPLEMENT_IBASE_END
-
 csKeyComposer::csKeyComposer ()
+  : scfImplementationType (this), lastDead (0)
 {
-  SCF_CONSTRUCT_IBASE(0);
-  lastDead = 0;
 }
 
 csKeyComposer::~csKeyComposer ()
 {
-  SCF_DESTRUCT_IBASE ();
 }
 
 /*
@@ -152,7 +163,8 @@ const utf32_char latinLetters[] = {
 const int numLetters = sizeof (latinLetters) / sizeof (utf32_char);
 
 /// A combination is not available.
-#define NOPE	  (utf32_char)-1
+#undef NOPE
+#define NOPE ((utf32_char)-1)
 
 /*
   All combinations of marks and letters we support.
@@ -300,31 +312,22 @@ void csKeyComposer::ResetState ()
 
 //--//--//--//--//--//--//--//--//--//--//--//--//--/> Keyboard driver <--//--/
 
-SCF_IMPLEMENT_IBASE(csKeyboardDriver)
-  SCF_IMPLEMENTS_INTERFACE(iKeyboardDriver)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE(csKeyboardDriver::eiEventHandler)
-  SCF_IMPLEMENTS_INTERFACE(iEventHandler)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 csKeyboardDriver::csKeyboardDriver (iObjectRegistry* r) :
-  csInputDriver(r)
+  csInputDriver(r), scfImplementationType (this),
+  KeyboardUp(csevKeyboardUp(r)),
+  KeyboardDown(csevKeyboardDown(r))
 {
-  SCF_CONSTRUCT_IBASE(0);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiEventHandler);
-
   memset (&modifiersState, 0, sizeof (modifiersState));
 
-  Listener = &scfiEventHandler;
+  keyDebugChecked = false;
+
+  Listener = this;
   StartListening();
 }
 
 csKeyboardDriver::~csKeyboardDriver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiEventHandler);
-  SCF_DESTRUCT_IBASE();
 }
 
 void csKeyboardDriver::Reset ()
@@ -365,6 +368,13 @@ void csKeyboardDriver::DoKey (utf32_char codeRaw, utf32_char codeCooked,
     SynthesizeCooked (codeRaw, modifiersState, codeCooked);
   }
 
+  if (IsKeyboardDebugging())
+  {
+    csPrintf ("raw: %s cooked: %s %s%s\n", GetKeycodeString (codeRaw), 
+      GetKeycodeString (codeCooked), iDown ? "down" : "up", 
+      autoRepeat ? " autoRepeat" : "");
+  }
+
   /*
     Set state before the event if the key is released, otherwise
     after. This is mainly for modifier keys, so if you press Alt
@@ -376,9 +386,9 @@ void csKeyboardDriver::DoKey (utf32_char codeRaw, utf32_char codeCooked,
   // @@@ Pooled events, somehow?
   csRef<iEvent> ev;
   ev.AttachNew (new csEvent ());
-  ev->Type = csevKeyboard;
-  ev->Category = ev->SubCategory = ev->Flags = 0;
-  ev->Add ("keyEventType", (uint8)(iDown ? csKeyEventTypeDown : csKeyEventTypeUp));
+  ev->Name = (iDown ? KeyboardDown : KeyboardUp);
+  ev->Add ("keyEventType", 
+    (uint8)(iDown ? csKeyEventTypeDown : csKeyEventTypeUp));
   ev->Add ("keyCodeRaw", (uint32)codeRaw);
   ev->Add ("keyCodeCooked", (uint32)codeCooked);
   ev->Add ("keyModifiers", &modifiersState, sizeof (modifiersState));
@@ -420,16 +430,73 @@ void csKeyboardDriver::SynthesizeCooked (utf32_char codeRaw,
     }
     else if (modifiers.modifiers[csKeyModifierTypeShift] != 0)
     {
-      if ((codeRaw >= 32) && (codeRaw <= 127))
-	codeCooked = ShiftedKey [codeRaw - 32];
+      char newCode;
+      if ((codeRaw < 32) || (codeRaw > 127)
+        || ((newCode = ShiftedKey [codeRaw - 32]) == (char)-1))
+      {
+        csUnicodeTransform::MapToUpper(codeRaw, &codeCooked, 1, csUcMapSimple);
+      }
       else
-	codeCooked = 0;
+        codeCooked = newCode;
     }
     else
       codeCooked = 0;
   }
 }
 
+csEventError csKeyboardDriver::SynthesizeCooked (iEvent *ev)
+{
+  utf32_char codeRaw, codeCooked;
+  csKeyModifiers mods;
+
+  csEventError err = ev->Retrieve ("keyCodeRaw", codeRaw);
+  if (err != csEventErrNone) return err;
+
+  SynthesizeCooked (codeRaw, mods, codeCooked);
+
+  ev->Add ("keyCodeCooked", codeCooked);
+  ev->Add ("keyModifiers", (void *) & mods, sizeof (mods));
+
+  return csEventErrNone;
+}
+
+// This really should be inside csKeyboardDriver::GetKeycodeString, but Cygwin
+// crashes on exit if functions have local static variables with complex types
+static csString genName;
+
+#ifdef CS_KEY_DEBUG_ENABLE
+const char* csKeyboardDriver::GetKeycodeString (utf32_char code)
+{
+  const char* str = KeyCodeNames.StringForIdent (code);
+  if (str != 0) return str;
+
+  genName.Format ("[%" PRIu32 "]", code);
+  return genName;
+}
+#else
+const char* csKeyboardDriver::GetKeycodeString (utf32_char /*code*/)
+{
+  return 0;
+}
+#endif
+
+bool csKeyboardDriver::IsKeyboardDebugging ()
+{
+#ifndef CS_KEY_DEBUG_ENABLE
+  return false;
+#endif
+  // Delay checking b/c at construction time the config manager is prolly
+  // not set up yet.
+  // @@@ Should probably controlled via "--verbose".
+  if (!keyDebugChecked)
+  {
+    csConfigAccess cfgAccess (Registry);
+    keyDebug = cfgAccess->GetBool ("KeyboardDriver.EnableDebugging");
+    keyDebugChecked = true;
+  }
+  return keyDebug;
+}
+
 void csKeyboardDriver::SetKeyState (utf32_char codeRaw, bool iDown,
 				    bool autoRepeat)
 {
@@ -452,7 +519,7 @@ void csKeyboardDriver::SetKeyState (utf32_char codeRaw, bool iDown,
         modifiersState.modifiers[modType] = state ? 0 : 1;
       }
 
-      keyStates.PutFirst (codeRaw, iDown);
+      keyStates.PutUnique (codeRaw, iDown);
     }
     else
     {
@@ -467,15 +534,15 @@ void csKeyboardDriver::SetKeyState (utf32_char codeRaw, bool iDown,
 	else
 	  modifiersState.modifiers[modType] &= ~(1 << modNum);
 
-	keyStates.PutFirst (codeRaw, iDown);
+	keyStates.PutUnique (codeRaw, iDown);
       }
     }
   }
   else
-    keyStates.PutFirst (codeRaw, iDown);
+    keyStates.PutUnique (codeRaw, iDown);
 }
 
-bool csKeyboardDriver::GetKeyState (utf32_char codeRaw)
+bool csKeyboardDriver::GetKeyState (utf32_char codeRaw) const
 {
   if (CSKEY_IS_MODIFIER (codeRaw) && 
     (CSKEY_MODIFIER_NUM(codeRaw) == csKeyModifierNumAny))
@@ -486,7 +553,7 @@ bool csKeyboardDriver::GetKeyState (utf32_char codeRaw)
     return keyStates.Get (codeRaw, false);
 }
 
-uint32 csKeyboardDriver::GetModifierState (utf32_char codeRaw)
+uint32 csKeyboardDriver::GetModifierState (utf32_char codeRaw) const
 {
   if (CSKEY_IS_MODIFIER (codeRaw))
   {
@@ -514,25 +581,17 @@ csPtr<iKeyComposer> csKeyboardDriver::CreateKeyComposer ()
 
 //--//--//--//--//--//--//--//--//--//--//--//--//--//--> Mouse driver <--//--/
 
-SCF_IMPLEMENT_IBASE(csMouseDriver)
-  SCF_IMPLEMENTS_INTERFACE(iMouseDriver)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE(csMouseDriver::eiEventHandler)
-  SCF_IMPLEMENTS_INTERFACE(iEventHandler)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 csMouseDriver::csMouseDriver (iObjectRegistry* r) :
-  csInputDriver(r)
+  csInputDriver(r), scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE(0);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiEventHandler);
-  Listener = &scfiEventHandler;
+  Listener = this;
   StartListening();
-
-  LastX = LastY = 0;
-  memset (&Button, 0, sizeof (Button));
+  for (int iter=0 ; iter<CS_MAX_MOUSE_COUNT ; iter++)
+  {
+    memset (Button[iter], 0, sizeof(bool) * CS_MAX_MOUSE_BUTTONS);
+    memset (Last[iter], 0, sizeof(int) * CS_MAX_MOUSE_AXES);
+  }
+  memset (Axes, 0, sizeof(Axes));
   Reset();
 
   csConfigAccess cfg;
@@ -544,16 +603,17 @@ csMouseDriver::csMouseDriver (iObjectRegistry* r) :
 
 csMouseDriver::~csMouseDriver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiEventHandler);
-  SCF_DESTRUCT_IBASE();
 }
 
 void csMouseDriver::Reset ()
 {
-  for (int i = 0; i < CS_MAX_MOUSE_BUTTONS; i++)
-    if (Button[i])
-      DoButton (i + 1, false, LastX, LastY);
-  LastClickButton = -1;
+  for (int i = 0; i < CS_MAX_MOUSE_COUNT; i++)
+  {
+    for (int j = 0; j < CS_MAX_MOUSE_BUTTONS; j++)
+      if (Button [i][j])
+	DoButton (i, j, false, Last[i], Axes[i]);
+    LastClickButton[i] = csmbNone;
+  }
 }
 
 iKeyboardDriver* csMouseDriver::GetKeyboardDriver()
@@ -563,12 +623,22 @@ iKeyboardDriver* csMouseDriver::GetKeyboardDriver()
   return Keyboard;
 }
 
-void csMouseDriver::DoButton (int button, bool down, int x, int y)
+/**
+ * Try to post a new mouse button event.
+ * 
+ * \todo Building the key-modifiers mask is broken, needs \see iKeyboardDriver 
+ *   support to fix.
+ */
+void csMouseDriver::DoButton (uint number, int button, bool down, 
+                              const int32 *axes, uint numAxes)
 {
-  if (x != LastX || y != LastY)
-    DoMotion (x, y);
+  if (number >= CS_MAX_MOUSE_COUNT)
+    return;
 
-  if (button <= 0 || button >= CS_MAX_MOUSE_BUTTONS)
+  if (memcmp(Last[number], axes, numAxes * sizeof(int))!=0)
+    DoMotion (number, axes, numAxes);
+
+  if (button < 0 || button >= CS_MAX_MOUSE_BUTTONS)
     return;
 
   iKeyboardDriver* k = GetKeyboardDriver();
@@ -576,53 +646,104 @@ void csMouseDriver::DoButton (int button, bool down, int x, int y)
             | (k->GetKeyState (CSKEY_ALT  ) ? CSMASK_ALT   : 0)
             | (k->GetKeyState (CSKEY_CTRL ) ? CSMASK_CTRL  : 0);
 
-  Button [button - 1] = down;
+  Button [number][button] = down;
+
+  uint32 buttonMask = 0;
+  for (int i=31 ; i>=0 ; i--) 
+  {
+    buttonMask <<= 1;
+    if (Button[number][i])
+      buttonMask |= 0x1;
+  }
 
-  csTicks evtime = csGetTicks ();
-  
   csRef<iEvent> ev;
-  ev.AttachNew (new csEvent (evtime, down ? csevMouseDown : csevMouseUp, 
-    x, y, button, smask));
+  csTicks evtime = csGetTicks();
+
+  ev.AttachNew (csMouseEventHelper::NewEvent
+		(NameRegistry,
+		 evtime, ((down)?
+			  csevMouseDown(NameRegistry, number):
+			  csevMouseUp(NameRegistry, number)),
+		 number, down?csMouseEventTypeDown:csMouseEventTypeUp,
+		 axes, numAxes, 0, button, down, buttonMask, smask));
   Post(ev);
 
-  if ((button == LastClickButton)
-   && (evtime - LastClickTime <= DoubleClickTime)
-   && (unsigned (ABS (x - LastClickX)) <= DoubleClickDist)
-   && (unsigned (ABS (y - LastClickY)) <= DoubleClickDist))
+  if ((button == LastClickButton[number])
+      && (evtime - LastClickTime[number] <= DoubleClickTime))
   {
+    for (uint iter=0 ; iter<Axes[number] ; iter++)
+      if (unsigned 
+	  (ABS (axes[iter] - LastClick[number][iter])) > DoubleClickDist)
+	goto mousedown;
     csRef<iEvent> ev;
-    ev.AttachNew (new csEvent (evtime,
-      down ? csevMouseDoubleClick : csevMouseClick, x, y, button, smask));
+    ev.AttachNew (csMouseEventHelper::NewEvent
+		  (NameRegistry,
+		   evtime, 
+		   ((down)?
+		    csevMouseDoubleClick(NameRegistry, number):
+		    csevMouseClick(NameRegistry, number)),
+		   number, (down?csMouseEventTypeDoubleClick:
+			    csMouseEventTypeClick),
+		   axes, numAxes, 0, button, down, 
+		   buttonMask, smask));
     Post (ev);
     // Don't allow for sequential double click events
     if (down)
-      LastClickButton = -1;
+      LastClickButton [number] = csmbNone;
   }
   else if (down)
   {
+  mousedown:
     // Remember the coordinates/button/position of last mousedown event
-    LastClickButton = button;
-    LastClickTime = evtime;
-    LastClickX = x;
-    LastClickY = y;
+    LastClickButton[number] = button;
+    LastClickTime[number] = evtime;
+    for (uint iter=0; iter<Axes[number]; iter++)
+      LastClick[number][iter] = axes[iter];
   }
 }
 
-void csMouseDriver::DoMotion (int x, int y)
+/**
+ * Try to post a new mouse motion event.
+ * 
+ * \todo Building the key-modifiers mask is broken, needs \see iKeyboardDriver 
+ *   support to fix.
+ */
+void csMouseDriver::DoMotion (uint number, const int32 *axes, uint numAxes)
 {
-  if (x != LastX || y != LastY)
+  uint32 cflags = 0;
+  if (number >= CS_MAX_MOUSE_COUNT)
+    return;
+
+  for (uint iter=0; iter<numAxes ; iter++)
+    if (Last [number][iter] != axes[iter])
+      cflags |= (1 << iter);
+
+  if (cflags==0)
+    return; /* no change to report */
+
+  iKeyboardDriver* k = GetKeyboardDriver();
+  int smask = (k->GetKeyState (CSKEY_SHIFT) ? CSMASK_SHIFT : 0)
+    | (k->GetKeyState (CSKEY_ALT  ) ? CSMASK_ALT   : 0)
+    | (k->GetKeyState (CSKEY_CTRL ) ? CSMASK_CTRL  : 0);
+  memcpy(Last [number], axes, numAxes * sizeof(int));
+  Axes [number] = numAxes;
+
+  uint32 buttonMask = 0;
+  for (int i=31 ; i>=0 ; i--)
   {
-    iKeyboardDriver* k = GetKeyboardDriver();
-    int smask = (k->GetKeyState (CSKEY_SHIFT) ? CSMASK_SHIFT : 0)
-              | (k->GetKeyState (CSKEY_ALT  ) ? CSMASK_ALT   : 0)
-              | (k->GetKeyState (CSKEY_CTRL ) ? CSMASK_CTRL  : 0);
-    LastX = x;
-    LastY = y;
-    csRef<iEvent> event;
-    event.AttachNew (new csEvent (csGetTicks (), 
-      csevMouseMove, x, y, 0, smask));
-    Post (event);
+    buttonMask <<= 1;
+    if (Button[number][i])
+      buttonMask |= 0x1;
   }
+
+  csRef<iEvent> ev;
+  ev.AttachNew (csMouseEventHelper::NewEvent
+		(NameRegistry,
+		 csGetTicks (), csevMouseMove(NameRegistry, number),
+		 number, csMouseEventTypeMove,
+		 axes, numAxes, cflags, 0, false, 
+		 buttonMask, smask));
+  Post (ev);
 }
 
 void csMouseDriver::SetDoubleClickTime (int iTime, size_t iDist)
@@ -633,31 +754,21 @@ void csMouseDriver::SetDoubleClickTime (int iTime, size_t iDist)
 
 //--//--//--//--//--//--//--//--//--//--//--//--//--/> Joystick driver <--//--/
 
-SCF_IMPLEMENT_IBASE(csJoystickDriver)
-  SCF_IMPLEMENTS_INTERFACE(iJoystickDriver)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csJoystickDriver::eiEventHandler)
-  SCF_IMPLEMENTS_INTERFACE (iEventHandler)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 csJoystickDriver::csJoystickDriver (iObjectRegistry* r) :
-  csInputDriver(r)
+  csInputDriver(r), scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE(0);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiEventHandler);
-  Listener = &scfiEventHandler;
+  Listener = this;
   StartListening();
-  memset (&Button, 0, sizeof (Button));
-  memset (&LastX, 0, sizeof (LastX));
-  memset (&LastY, 0, sizeof (LastY));
+  for (int iter=0 ; iter<CS_MAX_JOYSTICK_COUNT ; iter++)
+  {
+    memset (Button[iter], 0, sizeof(bool) * CS_MAX_JOYSTICK_BUTTONS);
+    memset (Last[iter], 0, sizeof(int) * CS_MAX_JOYSTICK_AXES);
+  }
+  memset (Axes, 0, sizeof(Axes));
 }
 
 csJoystickDriver::~csJoystickDriver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiEventHandler);
-  SCF_DESTRUCT_IBASE();
 }
 
 void csJoystickDriver::Reset ()
@@ -665,7 +776,7 @@ void csJoystickDriver::Reset ()
   for (int i = 0; i < CS_MAX_JOYSTICK_COUNT; i++)
     for (int j = 0; j < CS_MAX_JOYSTICK_BUTTONS; j++)
       if (Button [i][j])
-        DoButton (i + 1, j + 1, false, LastX [i], LastY [i]);
+        DoButton (i, j, false, Last [i], Axes [i]);
 }
 
 iKeyboardDriver* csJoystickDriver::GetKeyboardDriver()
@@ -675,16 +786,22 @@ iKeyboardDriver* csJoystickDriver::GetKeyboardDriver()
   return Keyboard;
 }
 
-void csJoystickDriver::DoButton (int number, int button, bool down,
-  int x, int y)
+/**
+ * Try to post a new joystick button event.
+ * 
+ * \todo Building the key-modifiers mask is broken, needs \see iKeyboardDriver 
+ *  support to fix.
+ */
+void csJoystickDriver::DoButton (uint number, int button, bool down,
+				 const int32 *axes, uint numAxes)
 {
-  if (number < 0 || number >= CS_MAX_JOYSTICK_COUNT)
+  if (number >= CS_MAX_JOYSTICK_COUNT)
     return;
 
-  if (x != LastX [number] || y != LastY [number])
-    DoMotion (number, x, y);
+  if (memcmp(Last[number], axes, numAxes * sizeof(int))!=0)
+    DoMotion(number, axes, numAxes);
 
-  if (button <= 0 || button > CS_MAX_JOYSTICK_BUTTONS)
+  if (button < 0 || button >= CS_MAX_JOYSTICK_BUTTONS)
     return;
 
   iKeyboardDriver* k = GetKeyboardDriver();
@@ -692,32 +809,68 @@ void csJoystickDriver::DoButton (int number, int button, bool down,
             | (k->GetKeyState (CSKEY_ALT)   ? CSMASK_ALT   : 0)
             | (k->GetKeyState (CSKEY_CTRL)  ? CSMASK_CTRL  : 0);
 
-  Button [number] [button - 1] = down;
-  csRef<iEvent> ev;
-  ev.AttachNew (new csEvent (csGetTicks (),
-    down ? csevJoystickDown : csevJoystickUp, number, x, y, button, smask));
-  Post(ev);
+  Button [number][button] = down;
 
+  uint32 buttonMask = 0;
+  for (int i=31 ; i>=0 ; i--)
+  {
+    buttonMask <<= 1;
+    if (Button[number][i])
+      buttonMask |= 0x1;
+  }
+
+  csRef<iEvent> ev;
+  ev.AttachNew (csJoystickEventHelper::NewEvent
+		(NameRegistry,
+		 csGetTicks (), ((down)?
+				 csevJoystickDown(NameRegistry, number):
+				 csevJoystickUp(NameRegistry, number)),
+		 number, axes, numAxes, 0, button, down, 
+		 buttonMask, smask));
+  Post(ev);
 }
 
-void csJoystickDriver::DoMotion (int number, int x, int y)
+/**
+ * Try to post a new joystick motion event.
+ * 
+ * \todo Building the key-modifiers mask is broken, needs \see iKeyboardDriver 
+ *  support to fix.
+ */
+void csJoystickDriver::DoMotion (uint number, const int32 *axes, uint numAxes)
 {
-  if (number < 0 || number >= CS_MAX_JOYSTICK_COUNT)
+  uint32 cflags = 0;
+  if (number >= CS_MAX_JOYSTICK_COUNT)
     return;
 
-  if (x != LastX [number] || y != LastY [number])
+  for (uint iter=0 ; iter<numAxes ; iter++)
+    if (Last[number][iter] != axes[iter])
+    {
+      Last[number][iter] = axes[iter];
+      cflags |= (1 << iter);
+    }
+
+  if (cflags==0)
+    return; /* no change to report */
+
+  iKeyboardDriver* k = GetKeyboardDriver();
+  int smask = (k->GetKeyState (CSKEY_SHIFT) ? CSMASK_SHIFT : 0)
+    | (k->GetKeyState (CSKEY_ALT)   ? CSMASK_ALT   : 0)
+    | (k->GetKeyState (CSKEY_CTRL)  ? CSMASK_CTRL  : 0);
+  Axes [number] = numAxes;
+
+  uint32 buttonMask = 0;
+  for (int i=31 ; i>=0 ; i--)
   {
-    iKeyboardDriver* k = GetKeyboardDriver();
-    int smask = (k->GetKeyState (CSKEY_SHIFT) ? CSMASK_SHIFT : 0)
-              | (k->GetKeyState (CSKEY_ALT)   ? CSMASK_ALT   : 0)
-              | (k->GetKeyState (CSKEY_CTRL)  ? CSMASK_CTRL  : 0);
-    LastX [number] = x;
-    LastY [number] = y;
-
-    csRef<iEvent> ev;
-    ev.AttachNew (new csEvent(csGetTicks(), csevJoystickMove, 
-      number, x, y, 0, smask));
-    Post(ev);
-
+    buttonMask <<= 1;
+    if (Button[number][i])
+      buttonMask |= 0x1;
   }
+
+  csRef<iEvent> ev;
+  ev.AttachNew (csJoystickEventHelper::NewEvent
+		(NameRegistry,
+		 csGetTicks(), csevJoystickMove(NameRegistry, number),
+		 number, axes, numAxes, cflags, 0, false, 
+		 buttonMask, smask));
+  Post(ev);
 }
