@@ -18,7 +18,6 @@
 */	
 
 #include "cssysdef.h"
-#include "csutil/garray.h"
 #include "csutil/csuctransform.h"
 #include "iutil/event.h"
 #include <windows.h>
@@ -26,6 +25,11 @@
 #include "win32kbd.h"
 #include "csutil/win32/wintools.h"
 
+#ifdef CS_KEY_DEBUG_ENABLE
+#include "csutil/sysfunc.h"
+#include "win32kbd_identstrs.h"
+#endif
+
 static void ModifiersToKeyState (const csKeyModifiers& modifiersState,
 				 BYTE* keystate)
 {
@@ -101,20 +105,14 @@ static bool CookedCodeToVKey (utf32_char codeCooked, SHORT& vKey,
 
 //-----------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(csWin32KeyComposer)
-  SCF_IMPLEMENTS_INTERFACE(iKeyComposer)
-SCF_IMPLEMENT_IBASE_END
 
 csWin32KeyComposer::csWin32KeyComposer ()
+  : scfImplementationType (this), lastDead (0), lastDeadVk (0)
 {
-  SCF_CONSTRUCT_IBASE(0);
-  lastDead = 0;
-  lastDeadVk = 0;
 }
 
 csWin32KeyComposer::~csWin32KeyComposer ()
 {
-  SCF_DESTRUCT_IBASE();
 }
 
 csKeyComposeResult csWin32KeyComposer::HandleKey (
@@ -152,7 +150,9 @@ csKeyComposeResult csWin32KeyComposer::HandleKey (
       RETURN1(ret, b);	      \
   }
 
-  if (CSKEY_IS_SPECIAL (keyEventData.codeRaw))
+  if (CSKEY_IS_SPECIAL (keyEventData.codeRaw)
+    && (!CSKEY_IS_PAD_KEY (keyEventData.codeRaw)))
+	 // PAD keys may result in characters
     RETURN0(csComposeNoChar)
 
   if (lastDeadVk != 0)
@@ -161,7 +161,8 @@ csKeyComposeResult csWin32KeyComposer::HandleKey (
       Urg. Windows saves the dead key state in a variable somewhere
       when ToAscii() or ToUnicode() is called with one.
       So to get a properly composed char, we need to first call
-      ToAscii() with the dead key, so the 
+      ToAscii() with the dead key, so that internal variable is 
+      properly set.
       */
     char outCh[2];
     int ret = ToAscii (lastDeadVk, MapVirtualKey (lastDeadVk, 0), 
@@ -221,6 +222,10 @@ csKeyComposeResult csWin32KeyComposer::HandleKey (
     }
     else
     {
+      // Since we allow PAD keys above, the cooked code may be special
+      if (CSKEY_IS_SPECIAL (keyEventData.codeCooked))
+	RETURN0(csComposeNoChar);
+
       RETURN1(csComposeNormalChar, keyEventData.codeCooked);
     }
   }
@@ -314,21 +319,12 @@ void csWin32KeyboardDriver::RestoreKeys ()
   }
 }
 
-CS_IMPLEMENT_STATIC_VAR (GetIMEComposeBuf, csDirtyAccessArray<WCHAR>, ());
-
 bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message, 
 					      WPARAM wParam, LPARAM lParam)
 {
   csRef<iEvent> ev;
   switch (message)
   {
-    case WM_CHAR:
-    case WM_UNICHAR:
-      if (wParam != UNICODE_NOCHAR)
-      {
-	return true;
-      }
-      return false;
     case WM_IME_COMPOSITION:
       if (Imm32::ImmAvailable())
       {
@@ -337,19 +333,17 @@ bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message,
 	{
 	  hIMC = Imm32::ImmGetContext (hWnd);
 	  
-	  csDirtyAccessArray<WCHAR>& buf = *GetIMEComposeBuf();
-
 	  if (cswinIsWinNT ())
 	  {
 	    // Get the size of the result string.
 	    DWORD dwSize = Imm32::ImmGetCompositionStringW (hIMC, 
 	      GCS_RESULTSTR, 0, 0);
 
-	    buf.SetLength (dwSize / sizeof (WCHAR));
+	    IMEComposeBuf.SetLength (dwSize / sizeof (WCHAR));
 
 	    // Get the result strings that is generated by the IME.
 	    Imm32::ImmGetCompositionStringW (hIMC, GCS_RESULTSTR, 
-	      buf.GetArray(), dwSize);
+	      IMEComposeBuf.GetArray(), dwSize);
 	  }
 	  else
 	  {
@@ -365,15 +359,15 @@ bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message,
 
 	    int bufsize = MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, 
 	      composeResult, dwSize, 0, 0);
-	    buf.SetLength (bufsize);
+	    IMEComposeBuf.SetLength (bufsize);
 	    MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, 
-	      composeResult, -1, buf.GetArray(), bufsize);
+	      composeResult, -1, IMEComposeBuf.GetArray(), bufsize);
 	  }
 
 	  Imm32::ImmReleaseContext (hWnd, hIMC);
 
-	  size_t remaining = buf.Length();
-	  utf16_char* str = (utf16_char*)buf.GetArray();
+	  size_t remaining = IMEComposeBuf.Length();
+	  utf16_char* str = (utf16_char*)IMEComposeBuf.GetArray();
           while (remaining > 0)
 	  {
 	    utf32_char code;
@@ -399,15 +393,99 @@ bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message,
 	    (message == WM_SYSKEYDOWN);
 	bool autoRep = ((lParam & 0x40000000) != 0);
 	csKeyCharType type;
+      #ifdef CS_KEY_DEBUG_ENABLE
+	if (IsKeyboardDebugging())
+	{
+	  csPrintf ("msg: %s wp: %s lp: %.8x\n", 
+	    Win32KeyMsgNames.StringForIdent (message), GetVKName (wParam), 
+	    (uint)lParam);
+	}
+      #endif
 	if (Win32KeyToCSKey (wParam, lParam, raw, cooked, type))
 	{
-  	  DoKey (raw, cooked, down, autoRep, type);
+	  bool doDoKey = true;
+	  if (raw == CSKEY_CTRL_LEFT)
+	  {
+	    /*
+	      Quirk: When pressing AltGr, messages for both LCtrl and RAlt are 
+	      emitted. To work this around, peek into the queue and check 
+	      whether an RAlt stroke is ahead. If yes, discard the LCtrl. The 
+	      message time is compared to make sure no "legal" 
+	      LCtrl-before-RAlt strokes are discarded.
+	    */
+	    DWORD strokeTime = (DWORD)GetMessageTime ();
+	    MSG msg;
+	    if (PeekMessage (&msg, hWnd, WM_KEYFIRST, WM_KEYLAST, PM_NOREMOVE))
+	    {
+	      if ((msg.message & ~4) == (message & ~4)) 
+		// mask out 4 so WM_KEY* and WM_SYSKEY* messages can be compared
+	      {
+		utf32_char nextRaw, nextCooked;
+		csKeyCharType nextType;
+		if (Win32KeyToCSKey (msg.wParam, msg.lParam, nextRaw, 
+		  nextCooked, nextType))
+		{
+		  doDoKey = !((nextRaw == CSKEY_ALT_RIGHT) 
+		    && (strokeTime == msg.time));
+		}
+	      }
+	    }
+	  }
+	  else if (raw == CSKEY_SHIFT_LEFT)
+	  {
+	    /*
+	      Quirk: when RShift is pressed, an LShift is emitted nevertheless.
+	      So check which shifts are _really_ down and call DoKey()
+	      appropriately.
+	    */
+	    bool lshiftState = keyStates.Get (CSKEY_SHIFT_LEFT, false);
+	    bool rshiftState = keyStates.Get (CSKEY_SHIFT_RIGHT, false);
+	    bool lshiftDown, rshiftDown;
+            if (cswinIsWinNT())
+            {
+              lshiftDown = (::GetKeyState (VK_LSHIFT) & 0x8000) != 0;
+              rshiftDown = (::GetKeyState (VK_RSHIFT) & 0x8000) != 0;
+            }
+            else
+            {
+              /* @@@ Bah. Win9x can't really distinguish between left and right shift.
+                Can't help it. Only emit LShift in all cases. */
+              lshiftDown = (::GetKeyState (VK_SHIFT) & 0x8000) != 0;
+              rshiftDown = false;
+            }
+
+	    if ((lshiftState != lshiftDown) || lshiftDown)
+	    {
+	    #ifdef CS_KEY_DEBUG_ENABLE
+	      if (IsKeyboardDebugging()) csPrintf ("Shift quirk: ");
+	    #endif
+	      DoKey (CSKEY_SHIFT_LEFT, CSKEY_SHIFT, lshiftDown, autoRep, type);
+	    }
+	    if ((rshiftState != rshiftDown) || rshiftDown)
+	    {
+	    #ifdef CS_KEY_DEBUG_ENABLE
+	      if (IsKeyboardDebugging()) csPrintf ("Shift quirk: ");
+	    #endif
+	      DoKey (CSKEY_SHIFT_RIGHT, CSKEY_SHIFT, rshiftDown, autoRep, type);
+	    }
+
+	    doDoKey = false;
+	  }
+  	  if (doDoKey) DoKey (raw, cooked, down, autoRep, type);
 	}
       }
       return !(wParam == VK_F4 && 
 	(modifiersState.modifiers[csKeyModifierTypeAlt] != 0));   // so Alt+F4 still works
     case WM_DEADCHAR:
     case WM_SYSDEADCHAR:
+    case WM_CHAR:
+    case WM_UNICHAR:
+    case WM_SYSCHAR:
+      if (wParam != UNICODE_NOCHAR)
+      {
+	return true; // Pretend we handle it. May get beeps otherwise.
+      }
+      return false;
     default:
       return false;
   }
@@ -704,6 +782,13 @@ bool csWin32KeyboardDriver::Win32KeyToCSKey (LONG vKey, LONG keyFlags,
 
 	// Set up modifiers
 	ModifiersToKeyState (modifiersState, keystate);
+        /* "Inverse ALtGr quirk": to get proper AltGr combos, Ctrl+Alt must be 
+         * "pressed", too */
+        if (modifiersState.modifiers[csKeyModifierTypeAlt] & 
+          (1 << csKeyModifierNumRight))
+        {
+          keystate[VK_CONTROL] |= 0x80;
+        }
 
 	if (cswinIsWinNT ())
 	{
@@ -743,3 +828,31 @@ bool csWin32KeyboardDriver::Win32KeyToCSKey (LONG vKey, LONG keyFlags,
   return false;
 }
 
+#ifdef CS_KEY_DEBUG_ENABLE
+// This really should be inside csWin32KeyboardDriver, but Cygwin
+// crashes on exit if functions have local static variables with complex types
+static csString genName;
+#endif
+
+#ifdef CS_KEY_DEBUG_ENABLE
+const char* csWin32KeyboardDriver::GetVKName (LONG vKey)
+{
+  const char* vkName = Win32VKeyNames.StringForIdent (vKey);
+  if (vkName != 0) return vkName;
+
+  if (((vKey >= 'A') && (vKey <= 'Z')) || ((vKey >= '0') && (vKey <= '9')))
+  {
+    genName = (char)vKey;
+  }
+  else
+  {
+    genName.Format ("[%ld]", vKey);
+  }
+  return genName;
+}
+#else
+const char* csWin32KeyboardDriver::GetVKName (LONG /*vKey*/)
+{
+  return 0;
+}
+#endif
