@@ -19,27 +19,27 @@
 #ifndef __CS_AWS_H__
 #define __CS_AWS_H__
 
-#include "iaws/aws.h"
-#include "iaws/awsparm.h"
-#include "iaws/awscnvs.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "csutil/array.h"
-#include "csutil/array.h"
 #include "csgeom/csrect.h"
 #include "csgeom/csrectrg.h"
 #include "cstool/proctex.h"
+#include "csutil/array.h"
+#include "csutil/array.h"
+#include "iaws/aws.h"
+#include "iaws/awscnvs.h"
+#include "iaws/awsparm.h"
+#include "igraphic/imageio.h"
+#include "iutil/comp.h"
+#include "iutil/eventh.h"
+#include "iutil/strset.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
-#include "igraphic/imageio.h"
 #include "awscomp.h"
 #include "awswin.h"
 
-/**
- * This is the alternate windowing system plugin.  It defines a simple,
- * lightweight alternative to the current CSWS windowing system.  It supports
- * simple skinning via the .skn defintions, and creation of windows from
- * .win definitions.
+/**\file
+ * This is the Alternate Windowing System plugin.  It defines a simple,
+ * lightweight windowing system.  It supports simple skinning via the .skn
+ * defintions, and creation of windows from .win definitions.
  */
 
 /**
@@ -75,6 +75,9 @@ private:
   /// Handle to the sink manager.
   csRef<iAwsSinkManager> sinkmgr;
 
+  /// Shared string table.
+  csRef<iStringSet> strset;
+
   /**
    * This is the dirty region.  All clean/dirty code now utilizes the
    * update region facility for non-contiguous rectangular spaces.  This
@@ -183,6 +186,9 @@ public:
   /// Set the preference manager used by the window system.
   virtual void SetPrefMgr (iAwsPrefManager *pmgr);
 
+  /// Get the shared string table.
+  virtual iStringSet* GetStringTable ();
+
   /// Register a component factory.
   virtual void RegisterComponentFactory (
     iAwsComponentFactory *factory,
@@ -329,7 +335,8 @@ public:
   virtual iAwsComponent *CreateWindowFrom (const char *defname);
 
   /// Creates a new embeddable component.
-  virtual iAwsComponent *CreateEmbeddableComponent (iAwsComponent *forComponent);
+  virtual iAwsComponent *CreateEmbeddableComponent (
+    iAwsComponent *forComponent);
 
   /// Creates a new parameter list.
   virtual iAwsParmList *CreateParmList ();
@@ -356,22 +363,6 @@ public:
   /// Get the current context.
   virtual iAwsCanvas *GetCanvas ();
 
-  /// Create a default canvas, covering the whole screen.
-  /*virtual iAwsCanvas *CreateDefaultCanvas (
-      iEngine *engine,
-      iTextureManager *txtmgr);*/
-
-  /// Create a default canvas, just a single proctex.
-  /*virtual iAwsCanvas *CreateDefaultCanvas (
-      iEngine *engine,
-      iTextureManager *txtmgr,
-      int width,
-      int height,
-      const char *name);*/
-
-  /// Create a canvas that uses custom graphics devices (e.g. the screen).
-  //virtual iAwsCanvas *CreateCustomCanvas (iGraphics2D *g2d, iGraphics3D *g3d);
-
   /// Get the iGraphics2D interface so that components can use it.
   virtual iGraphics2D *G2D ();
 
@@ -428,8 +419,23 @@ public:
   /// Dispatch event to the component and all its children.
   static void DispatchEventRecursively(iAwsComponent *c, iEvent &ev);
 
+  /// Call this if you want to delete marked components immediately.
+  virtual void DeleteMarkedComponents()
+  {
+    DeleteMarkedComponentsRecursively(top);
+  }
+
+  /// Call this if you want to delete marked components immediately.
+  static void DeleteMarkedComponentsRecursively(iAwsComponent *&c);
+
   /// Notify the manager about component destruction.
   virtual void ComponentDestroyed(iAwsComponent *comp);
+
+  /// Mark component and all its children to delete.
+  virtual void MarkToDeleteRecursively( iAwsComponent *c );
+  
+  static void MarkChildToDeleteRecursively( iAwsComponent *c );
+  
 };
 
 #endif // __CS_AWS_H__
