@@ -1,5 +1,5 @@
 /*
-    Copyright (C) ???
+    Copyright (C) 2001 by Christopher Nelson
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -19,6 +19,7 @@
 #include "cssysdef.h"
 #include "awsMenu.h"
 #include "csutil/util.h"
+#include "csutil/event.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/fontserv.h"
@@ -29,7 +30,7 @@ const int awsMenuEntry::signalClicked = 1;
 const int awsMenuEntry::signalSelected = 2;
 
 awsMenuEntry::awsMenuEntry ()
-  : caption (0),
+  : /*caption (0),*/
     popup (0),
     selected (false),
     mouse_down (false),
@@ -45,7 +46,6 @@ awsMenuEntry::awsMenuEntry ()
 
 awsMenuEntry::~awsMenuEntry () 
 {
-  if (caption) caption->DecRef ();
   if (popup) popup->DecRef ();
   if (image) image->DecRef ();
   if (sub_menu_image) sub_menu_image->DecRef ();
@@ -57,11 +57,12 @@ bool awsMenuEntry::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
 
+  caption.AttachNew (new scfString (""));
   pm->GetString (settings, "Caption", caption);
   
-  iString* image_name = 0;
-  pm->GetString (settings, "Image", image_name);
-  if (image_name)
+  csRef<iString> image_name = 0;
+  image_name.AttachNew (new scfString (""));
+  if (pm->GetString (settings, "Image", image_name))
     image = pm->GetTexture (image_name->GetData (), image_name->GetData ());
 
   pm->LookupIntKey ("MenuItemImageWidth", image_width);
@@ -69,12 +70,11 @@ bool awsMenuEntry::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   pm->LookupIntKey ("MenuItemImageHeigth", image_width);
   pm->GetInt (settings, "ImageHeight", image_width);
 
-  image_name = 0;
-  pm->LookupStringKey ("MenuItemSubMenuImage", image_name);
-  if (image_name)
+  iString* image_name2 = 0;
+  if (pm->LookupStringKey ("MenuItemSubMenuImage", image_name2))
   {
-    sub_menu_image = pm->GetTexture (image_name->GetData (),
-      image_name->GetData ());
+    sub_menu_image = pm->GetTexture (image_name2->GetData (),
+      image_name2->GetData ());
     
     if (sub_menu_image)
     {
@@ -91,28 +91,28 @@ const char* awsMenuEntry::Type ()
   return "Menu Entry";
 }
 
-bool awsMenuEntry::GetProperty (const char* name, void **parm)
+bool awsMenuEntry::GetProperty (const char* name, intptr_t *parm)
 {
   if (awsPanel::GetProperty (name, parm)) return true;
 	
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
     
     if (caption) st = caption->GetData ();
     
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else if (strcmp (name, "PopupMenu") == 0)
   {
-    *parm = (void *)popup;
+    *parm = (intptr_t)popup;
     return true;
   }
   else if (strcmp (name, "Selected") == 0)
   {
-    *parm = (void*)selected;
+    *parm = selected;
     return true;
   }
   else if (strcmp (name, "UserParam") == 0)
@@ -122,23 +122,23 @@ bool awsMenuEntry::GetProperty (const char* name, void **parm)
   }
   else if (strcmp (name, "CloseSignal") == 0)
   {
-    *parm = (void *)signalClicked;
+    *parm = signalClicked;
     return true;
   }
   else if (strcmp (name, "SelectSignal") == 0)
   {
-    *parm = (void *)signalSelected;
+    *parm = signalSelected;
     return true;
   }
   else if (strcmp (name, "Image") == 0)
   {
-    *parm = (void*) image;
+    *parm = (intptr_t) image;
     return true;
   }
   return false;
 }
 
-bool awsMenuEntry::SetProperty (const char *name, void *parm)
+bool awsMenuEntry::SetProperty (const char *name, intptr_t parm)
 {
   if (awsPanel::SetProperty (name, parm)) return true;
 	
@@ -148,15 +148,12 @@ bool awsMenuEntry::SetProperty (const char *name, void *parm)
     
     if (s && s->Length ())
     {
-      if (caption) caption->DecRef ();
       caption = s;
-      caption->IncRef (); 
       SizeToFit ();
       Invalidate ();
     }
     else
     {
-      if (caption) caption->DecRef ();
       caption = 0;
     }
     return true;
@@ -174,7 +171,7 @@ bool awsMenuEntry::SetProperty (const char *name, void *parm)
   }
   else if (strcmp (name, "Selected") == 0)
   {
-    selected = (bool)parm;
+    selected = (parm != 0);
     return true;
   }
   else if (strcmp (name, "UserParam") == 0)
@@ -380,23 +377,23 @@ const char* awsMenuBarEntry::Type ()
   return "Menu Bar Entry";
 }
 
-bool awsMenuBarEntry::GetProperty (const char* name, void **parm)
+bool awsMenuBarEntry::GetProperty (const char* name, intptr_t *parm)
 {
   if (awsCmdButton::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
 
     if (caption) st = caption->GetData ();
 
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else if (strcmp (name, "PopupMenu") == 0)
   {
-    *parm = (void *)popup;
+    *parm = (intptr_t)popup;
     return true;
   }
   else if (strcmp (name, "Selected") == 0)
@@ -406,13 +403,13 @@ bool awsMenuBarEntry::GetProperty (const char* name, void **parm)
   }
   else if (strcmp (name, "SelectSignal") == 0)
   {
-    *parm = (void *)signalClicked;
+    *parm = signalClicked;
     return true;
   }
   return false;
 }
 
-bool awsMenuBarEntry::SetProperty (const char *name, void *parm)
+bool awsMenuBarEntry::SetProperty (const char *name, intptr_t parm)
 {
   if (awsCmdButton::SetProperty (name, parm)) return true;
 
@@ -437,10 +434,10 @@ bool awsMenuBarEntry::SetProperty (const char *name, void *parm)
 bool awsMenuBarEntry::OnMouseEnter ()
 {
   iAwsComponent* cmp = 0;
-  Parent ()->GetProperty ("Selected", (void**) &cmp);
+  Parent ()->GetProperty ("Selected", (intptr_t*)&cmp);
   if (cmp)
   {
-    SetProperty ("Selected", (void**)true);
+    SetProperty ("Selected", true);
 
     // Our selection signal.
     Broadcast (signalClicked);
@@ -470,11 +467,10 @@ awsMenu::awsMenu ()
     child_menu (0),
     parent_menu (0),
     mouse_pos (0, 0),
-    sink (),
+    sink (0),
     mouse_captured (false),
     let_mouse_exit (true)
 {
-  sink.SetParm (this);
 }
 
 awsMenu::~awsMenu ()
@@ -484,13 +480,17 @@ awsMenu::~awsMenu ()
     child_menu->Hide ();
     child_menu->DecRef ();
   }
+  delete sink;
 }
 
 bool awsMenu::Setup (iAws *wmgr, iAwsComponentNode *settings)
 {
-  sink.RegisterTrigger ("Select", &OnSelect);
-  sink.RegisterTrigger ("Close", &OnClose);
-  if(!awsControlBar::Setup (wmgr, settings)) return false;
+  if (!awsControlBar::Setup (wmgr, settings)) return false;
+
+  sink = new awsSink(WindowManager());
+  sink->SetParm ((intptr_t)this);
+  sink->RegisterTrigger ("Select", &OnSelect);
+  sink->RegisterTrigger ("Close", &OnClose);
 
   // We initially fit it to size, but after that you can change
   // it to whatever you like.
@@ -498,12 +498,12 @@ bool awsMenu::Setup (iAws *wmgr, iAwsComponentNode *settings)
   return true;
 }
 
-bool awsMenu::GetProperty (const char* name, void** parm)
+bool awsMenu::GetProperty (const char* name, intptr_t *parm)
 {
   if (awsControlBar::GetProperty (name, parm)) return true;
   else if (strcmp (name, "Selected") == 0)
   {
-    *parm = (void*) select;
+    *parm = (intptr_t)select;
     return true;
   }
   return false;
@@ -517,17 +517,17 @@ void awsMenu::SetMenuParent (awsMenu *_parent_menu)
 void awsMenu::AddChild (iAwsComponent* comp)
 {
   int selectSignal, closeSignal;
-  if (comp->GetProperty ("SelectSignal", (void**) &selectSignal))
+  if (comp->GetProperty ("SelectSignal", (intptr_t*)&selectSignal))
   {
     slot_select.Connect (
       comp,
       selectSignal,
-      &sink,
-      sink.GetTriggerID ("Select"));
+      sink,
+      sink->GetTriggerID ("Select"));
   }
-  if (comp->GetProperty ("CloseSignal", (void**) &closeSignal))
+  if (comp->GetProperty ("CloseSignal", (intptr_t*)&closeSignal))
   {
-    slot_close.Connect (comp, closeSignal, &sink, sink.GetTriggerID ("Close"));
+    slot_close.Connect (comp, closeSignal, sink, sink->GetTriggerID ("Close"));
   }
   awsControlBar::AddChild (comp);
 }
@@ -539,9 +539,9 @@ iAwsSource* awsMenu::AddChild (
 {
   iAwsComponent* child = GetNewDefaultEntry ();
   // XXX: Memory leak here, should delete the string later.
-  child->SetProperty ("Caption", csStrNew(caption));
-  child->SetProperty ("Image", image);
-  child->SetProperty ("PopupMenu", popup);
+  child->SetProperty ("Caption", (intptr_t)csStrNew(caption));
+  child->SetProperty ("Image", (intptr_t)image);
+  child->SetProperty ("PopupMenu", (intptr_t)popup);
 
   // Create will link the component to us.
   child->Create (WindowManager (), this, 0);
@@ -552,20 +552,20 @@ iAwsSource* awsMenu::AddChild (
 void awsMenu::RemoveChild (iAwsComponent* comp)
 {
   int selectSignal, closeSignal;
-  if (comp->GetProperty ("SelectionSignal", (void**) &selectSignal))
+  if (comp->GetProperty ("SelectionSignal", (intptr_t*)&selectSignal))
   {
     slot_select.Disconnect (
       comp,
       selectSignal,
-      &sink,
-      sink.GetTriggerID ("Select"));
+      sink,
+      sink->GetTriggerID ("Select"));
   }
-  if (comp->GetProperty ("CloseSignal", (void**) &closeSignal))
+  if (comp->GetProperty ("CloseSignal", (intptr_t*)&closeSignal))
   {
     slot_close.Disconnect (comp,
       closeSignal,
-      &sink,
-      sink.GetTriggerID ("Close"));
+      sink,
+      sink->GetTriggerID ("Close"));
   }
   awsControlBar::RemoveChild(comp);
 }
@@ -575,7 +575,8 @@ void awsMenu::RemoveChild (const char* caption)
   for (iAwsComponent* cmp = GetTopChild (); cmp; cmp = cmp->ComponentBelow ())
   {
     iString* temp_caption;
-    if (!cmp->GetProperty ("Caption", (void**)&temp_caption)) continue;
+    if (!cmp->GetProperty ("Caption", (intptr_t*)&temp_caption))
+      continue;
     if (!caption)
     {
       if (!temp_caption)
@@ -604,17 +605,12 @@ void awsMenu::RemoveChild (iAwsSource* src)
 
 bool awsMenu::HandleEvent (iEvent &Event)
 {
-  switch (Event.Type)
-  {
-  case csevMouseMove:
-  case csevMouseUp:
-  case csevMouseClick:
-  case csevMouseDown:
-  case csevMouseEnter:
-  case csevMouseExit:
-    mouse_pos.Set (Event.Mouse.x, Event.Mouse.y);
-    break;
-  }
+  iObjectRegistry *object_reg = WindowManager()->GetObjectRegistry();
+  if (csEventNameRegistry::IsKindOf(object_reg,
+				    Event.Name, 
+				    csevMouseEvent(object_reg)))
+    mouse_pos.Set (csMouseEventHelper::GetX(&Event), 
+		   csMouseEventHelper::GetY(&Event));
   return awsControlBar::HandleEvent (Event);
 }
 
@@ -655,9 +651,9 @@ bool awsMenu::OnMouseExit ()
   // If the mouse attempts to leave the border of the popup we
   // capture it unless it is entering another popup or one of our
   // children or if let_mouse_exit is set.
-  if (!IsOverChildMenu (mouse_pos.x, mouse_pos.y) &&
-    !IsOverParentMenu (mouse_pos.x, mouse_pos.y) &&
-    !Frame ().Contains (mouse_pos.x, mouse_pos.y) &&
+  if (!IsOverChildMenu ((int)mouse_pos.x, (int)mouse_pos.y) &&
+    !IsOverParentMenu ((int)mouse_pos.x, (int)mouse_pos.y) &&
+    !Frame ().Contains ((int)mouse_pos.x, (int)mouse_pos.y) &&
     ShouldTrackMouse ())
   {
     WindowManager ()->CaptureMouse (this);
@@ -706,22 +702,22 @@ void awsMenu::Select (iAwsComponent* child)
 
   // First we deactivate the old active child.
   if (select)
-    select->SetProperty ("Selected", (void*)false);
+    select->SetProperty ("Selected", (intptr_t)false);
 
   select = child;
 
   if (select)
-    select->SetProperty ("Selected", (void*)true);
+    select->SetProperty ("Selected", (intptr_t)true);
 
   StartPopupChange ();
 }
 
-void awsMenu::OnSelect (void* p, iAwsSource* src)
+void awsMenu::OnSelect (unsigned long, intptr_t p, iAwsSource* src)
 {
   awsMenu* m = (awsMenu*)p;
   iAwsComponent* menu_entry = src->GetComponent ();
   bool selected = false;
-  menu_entry->GetProperty("Selected", (void**) &selected);
+  menu_entry->GetProperty("Selected", (intptr_t*)&selected);
   // An inactive entry is telling us its inactive.
   if (!selected && menu_entry != m->select) return;
 
@@ -731,7 +727,7 @@ void awsMenu::OnSelect (void* p, iAwsSource* src)
     m->Select (0);
 }
 
-void awsMenu::OnClose (void* p, iAwsSource* src)
+void awsMenu::OnClose (unsigned long, intptr_t p, iAwsSource* /*src*/)
 {
   awsMenu* m = (awsMenu*)p;
   m->HideAllPopups ();
@@ -767,7 +763,7 @@ void awsMenu::SwitchPopups ()
 
   // Get the new popup if any.
   if (select)
-    select->GetProperty ("PopupMenu", (void**) &child_menu);
+    select->GetProperty ("PopupMenu", (intptr_t*)&child_menu);
 
   // Display the new popup if necessary.
   if (child_menu)
@@ -839,7 +835,7 @@ bool awsMenuBar::Create (
 
   if (Parent ())
   {
-    if (!Parent ()->SetProperty ("Menu", this))
+    if (!Parent ()->SetProperty ("Menu", (intptr_t)this))
     {
       Parent ()->AddChild (this);
       
@@ -910,7 +906,7 @@ bool awsPopupMenu::Create (
   if (!Setup (wmgr, settings)) return false;
 
   if (parent)
-    return parent->SetProperty ("PopupMenu", this);
+    return parent->SetProperty ("PopupMenu", (intptr_t)this);
   else
   {
     // Link into the current hierarchy, at the top.
@@ -930,16 +926,17 @@ bool awsPopupMenu::Create (
 bool awsPopupMenu::Setup (iAws *wmgr, iAwsComponentNode *settings)
 {
   timer = new awsTimer (wmgr->GetObjectRegistry (), this);
-  
-  sink.RegisterTrigger ("Timer", &OnTimer);
-  slot_timer.Connect (
-    timer,
-    awsTimer::signalTick,
-    &sink,
-    sink.GetTriggerID ("Timer"));
 
   if (!awsMenu::Setup (wmgr, settings))
     return false;
+
+  sink->RegisterTrigger ("Timer", &OnTimer);
+  slot_timer.Connect (
+    timer,
+    awsTimer::signalTick,
+    sink,
+    sink->GetTriggerID ("Timer"));
+
   return true;
 }
 
@@ -968,7 +965,7 @@ void awsPopupMenu::HideAllPopups ()
     Hide ();
 }
 
-void awsPopupMenu::OnTimer (void* param, iAwsSource* src)
+void awsPopupMenu::OnTimer (unsigned long, intptr_t param, iAwsSource* /*src*/)
 {
   awsPopupMenu* pm = (awsPopupMenu*)param;
   pm->SwitchPopups ();
