@@ -1,5 +1,5 @@
 /*
-    Copyright (C) ???
+    Copyright (C) 2001 by Christopher Nelson
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -16,9 +16,8 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <stdio.h>
-
 #include "cssysdef.h"
+#include <stdio.h>
 #include "awscmdbt.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
@@ -26,6 +25,7 @@
 #include "csutil/scfstr.h"
 #include "csutil/csevent.h"
 #include "iutil/evdefs.h"
+#include "ivaria/reporter.h"
 
 awsCmdButton::awsCmdButton ()
   : is_down (false),
@@ -33,8 +33,7 @@ awsCmdButton::awsCmdButton ()
     is_switch (false),
     was_down (false),
     icon_align (0),
-    stretched (false),
-    caption (0)
+    stretched (false)
 {
   tex[0] = tex[1] = tex[2] = 0;
   style = fsNormal;
@@ -42,10 +41,6 @@ awsCmdButton::awsCmdButton ()
 
 awsCmdButton::~awsCmdButton ()
 {
-  if (caption != 0)
-  {
-    caption->DecRef();
-  }
 }
 
 const char *awsCmdButton::Type ()
@@ -61,11 +56,12 @@ bool awsCmdButton::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   // The command button can use "Image" rather than "BitmapOverlay" to
   // setup its overlay image therefore we create a BitmapOverlay key
   // from the Image key for back compatibility.
-  iString *tn = 0;
+  csRef<iString> tn;
+  tn.AttachNew (new scfString (""));
   if (!pm->GetString (settings, "BitmapOverlay", tn) &&
     pm->GetString (settings, "Image", tn))
   {
-    awsStringKey* temp = new awsStringKey ("BitmapOverlay", tn);
+    awsStringKey* temp = new awsStringKey (_wmgr, "BitmapOverlay", tn);
     csRef<iAwsStringKey> key (SCF_QUERY_INTERFACE (temp, iAwsStringKey));
     settings->Add (key);
     temp->DecRef ();
@@ -75,62 +71,77 @@ bool awsCmdButton::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 
   pm->GetInt (settings, "Toggle", switch_style);
   pm->GetInt (settings, "IconAlign", icon_align);
+  caption.AttachNew (new scfString (""));
   pm->GetString (settings, "Caption", caption);
 
-  if (caption != 0)
-  {
-    caption->IncRef();
-  }
-
-  is_switch = switch_style;
+  is_switch = (switch_style != 0);
 
   if (style == fsNormal || style == fsToolbar)
   {
-    iString *in = 0;
-    pm->GetString (settings, "Icon", in);
-    if (in) tex[0] = pm->GetTexture (in->GetData (), in->GetData ());
+    csRef<iString> in;
+    in.AttachNew (new scfString (""));
+    if (pm->GetString (settings, "Icon", in))
+      tex[0] = pm->GetTexture (in->GetData (), in->GetData ());
   }
   else if (style == fsBitmap)
   {
-    iString *tn1 = 0, *tn2 = 0, *tn3 = 0;
+    csRef<iString> tn1 = csPtr<iString> (new scfString ());
+    csRef<iString> tn2 = csPtr<iString> (new scfString ());
+    csRef<iString> tn3 = csPtr<iString> (new scfString ());
 
     int stretch;
     pm->GetString (settings, "BitmapNormal", tn1);
     pm->GetString (settings, "BitmapFocused", tn2);
     pm->GetString (settings, "BitmapClicked", tn3);
-    if (pm->GetInt (settings, "Stretched", stretch)) stretched = stretch;
-    if (tn1) tex[0] = pm->GetTexture (tn1->GetData (), tn1->GetData ());
-    if (tn2) tex[1] = pm->GetTexture (tn2->GetData (), tn2->GetData ());
-    if (tn3) tex[2] = pm->GetTexture (tn3->GetData (), tn3->GetData ());
+    if (pm->GetInt (settings, "Stretched", stretch) != 0) stretched = (stretch != 0);
+
+#define GET_TEXTURE( t_id )     \
+  if (tn##t_id) \
+  {\
+    tex[t_id - 1] = pm->GetTexture (tn##t_id->GetData (), tn##t_id->GetData ());\
+    if( 0 == tex[t_id - 1] )\
+    {\
+      csReport( _wmgr->GetObjectRegistry(), CS_REPORTER_SEVERITY_ERROR, "crystalspace.aws", \
+        "Texture \"%s\" could not be found", tn##t_id->GetData () );\
+      return false;\
+    }\
+  }
+
+    GET_TEXTURE( 1 );
+    GET_TEXTURE( 2 );
+    GET_TEXTURE( 3 );
+
+#undef GET_TEXTURE
+
   }
   else
     return false;
   return true;
 }
 
-bool awsCmdButton::GetProperty (const char *name, void **parm)
+bool awsCmdButton::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
 
     if (caption) st = caption->GetData ();
 
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else if (strcmp ("State", name) == 0)
   {
-   *parm = (void*)is_down;
+   *parm = is_down;
     return true;
   }
   return false;
 }
 
-bool awsCmdButton::SetProperty (const char *name, void *parm)
+bool awsCmdButton::SetProperty (const char *name, intptr_t parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
 
@@ -138,20 +149,21 @@ bool awsCmdButton::SetProperty (const char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
+    //TODO Investigatre if calling Invalidate () always (also
+    //when the the string is 0 does not harm. luca
+#if 0
     if (s && s->Length ())
     {
-      if (caption) caption->DecRef ();
       caption = s;
-      caption->IncRef ();
       Invalidate ();
     }
     else
     {
-      if (caption)
-        caption->DecRef ();
-
       caption = 0;
     }
+#endif
+    caption->Replace (s);
+    Invalidate();
     return true;
   }
   else if (strcmp ("Image", name) == 0)
@@ -172,16 +184,16 @@ bool awsCmdButton::SetProperty (const char *name, void *parm)
   {
     if (is_switch)
     {
-      is_down  = (bool)parm;
-      was_down = (bool)parm;
+      is_down  = (parm != 0);
+      was_down = (parm != 0);
       ClearGroup ();
     }
     else
     {
-      if (((bool)parm) == is_down)
+      if ((parm != 0) == is_down)
         return true;
 
-      is_down = (bool)parm;
+      is_down = (parm != 0);
     }
     Invalidate ();
     return true;
@@ -193,7 +205,7 @@ void awsCmdButton::ClearGroup ()
 {
   csEvent Event;
 
-  Event.Type = csevGroupOff;
+  Event.Name = awsGroupOff (WindowManager()->GetObjectRegistry());
 
   iAwsComponent* cmp = Parent ()->GetTopChild ();
   while (cmp)
@@ -207,21 +219,19 @@ bool awsCmdButton::HandleEvent (iEvent &Event)
 {
   if (awsComponent::HandleEvent (Event)) return true;
 
-  switch (Event.Type)
+  if (Event.Name == awsGroupOff (WindowManager()->GetObjectRegistry()))
   {
-  case csevGroupOff:
     if (is_down && is_switch)
     {
       is_down = false;
       Invalidate ();
     }
     return true;
-    break;
   }
   return false;
 }
 
-void awsCmdButton::OnDraw (csRect clip)
+void awsCmdButton::OnDraw (csRect /*clip*/)
 {
   int tw = 0, th = 0;
 
@@ -251,18 +261,18 @@ void awsCmdButton::OnDraw (csRect clip)
         showing_style = fsSunken;
       else
         showing_style = fsRaised;
-      }
-      else if (style == fsToolbar)
-      {
-        if (is_down)
-          showing_style = fsSunken;
-        else if (can_raise)
-          showing_style = fsRaised;
-        else
-          showing_style = fsFlat;
-      }
-      frame_drawer.Draw (frame, showing_style, Window ()->Frame ());
     }
+    else if (style == fsToolbar)
+    {
+      if (is_down)
+        showing_style = fsSunken;
+      else if (can_raise)
+        showing_style = fsRaised;
+      else
+        showing_style = fsFlat;
+    }
+    frame_drawer.Draw (frame, showing_style, Window ()->Frame ());
+  }
   
   int tx = Frame ().Width () >> 1;
   int ty = Frame ().Height () >> 1;
@@ -276,7 +286,7 @@ void awsCmdButton::OnDraw (csRect clip)
       th);
   }
   
-  if (tex[0])
+  if (style != fsBitmap && tex[0])
   {
     int img_w, img_h;
     int itx = tx;
@@ -316,8 +326,8 @@ void awsCmdButton::OnDraw (csRect clip)
     
     g3d->DrawPixmap (
       tex[0],
-      Frame ().xmin + is_down + itx + 2,
-      Frame ().ymin + is_down + ity + 2,
+      Frame ().xmin + itx + 2,
+      Frame ().ymin + ity + 2,
       img_w,
       img_h,
       0,
@@ -333,38 +343,41 @@ void awsCmdButton::OnDraw (csRect clip)
   }
   
   // Draw the caption, if there is one and the style permits it.
-  if (caption)
+  if (style != fsBitmap)
   {
-    // Draw the text.
-    g2d->Write (
-      WindowManager ()->GetPrefMgr ()->GetDefaultFont (),
-      Frame ().xmin + tx + is_down,
-      Frame ().ymin + ty + is_down,
-      WindowManager ()->GetPrefMgr ()->GetColor (AC_TEXTFORE),
-      -1,
-      caption->GetData ());
-
-    if (can_raise && style == fsNormal)
+    if( caption )
     {
-      int x, y;
-      int y1 = Frame ().ymin + ty + th + 2 + is_down, y2 = Frame ().ymin +
-        ty - 2 + is_down, x1 = Frame ().xmin + is_down + 4,
-        x2 = Frame ().xmax + is_down - 4;
+      // Draw the text.
+      g2d->Write (
+        WindowManager ()->GetPrefMgr ()->GetDefaultFont (),
+        Frame ().xmin + tx + is_down,
+        Frame ().ymin + ty + is_down,
+        WindowManager ()->GetPrefMgr ()->GetColor (AC_TEXTFORE),
+        -1,
+        caption->GetData ());
 
-      for (x = x1; x < x2; ++x)
+      if (can_raise && style == fsNormal)
       {
-        g2d->DrawPixel (x, y1, (x & 1 ? hi : lo));
-        g2d->DrawPixel (x, y2, (x & 1 ? hi : lo));
-      }
+        int x, y;
+        int y1 = Frame ().ymin + ty + th + 2 + is_down, y2 = Frame ().ymin +
+          ty - 2 + is_down, x1 = Frame ().xmin + is_down + 4,
+          x2 = Frame ().xmax + is_down - 4;
 
-      for (y = y2; y < y1; ++y)
-      {
-        g2d->DrawPixel (x1, y, (y & 1 ? hi : lo));
-        g2d->DrawPixel (x2, y, (y & 1 ? hi : lo));
+        for (x = x1; x < x2; ++x)
+        {
+          g2d->DrawPixel (x, y1, (x & 1 ? hi : lo));
+          g2d->DrawPixel (x, y2, (x & 1 ? hi : lo));
+        }
+
+        for (y = y2; y < y1; ++y)
+        {
+          g2d->DrawPixel (x1, y, (y & 1 ? hi : lo));
+          g2d->DrawPixel (x2, y, (y & 1 ? hi : lo));
+        }
       }
     }
   }
-  else if (style == fsBitmap)
+  else
   {
     int texindex;
     int w, h;
@@ -380,8 +393,8 @@ void awsCmdButton::OnDraw (csRect clip)
 
     g3d->DrawPixmap (
       tex[texindex],
-      Frame ().xmin+is_down,
-      Frame ().ymin+is_down,
+      Frame ().xmin,
+      Frame ().ymin,
       stretched ? Frame ().Width () : w,
       stretched ? Frame ().Height () : h,
       0,
@@ -456,7 +469,7 @@ csRect awsCmdButton::getMinimumSize ()
   }
 }
 
-bool awsCmdButton::OnMouseDown (int button, int x, int y)
+bool awsCmdButton::OnMouseDown (int /*button*/, int /*x*/, int /*y*/)
 {
   was_down = is_down;
 
@@ -466,7 +479,7 @@ bool awsCmdButton::OnMouseDown (int button, int x, int y)
   return true;
 }
 
-bool awsCmdButton::OnMouseUp (int button, int x, int y)
+bool awsCmdButton::OnMouseUp (int /*button*/, int /*x*/, int /*y*/)
 {
   if (!is_switch)
   {
