@@ -50,7 +50,7 @@ protected:
   bool stretched;
   
   /// Caption text for this component.
-  iString *caption;
+  csRef<iString> caption;
 
   void ClearGroup ();
 public:
@@ -79,10 +79,10 @@ public:
   virtual bool Setup (iAws *wmgr, iAwsComponentNode *settings);
 
   /// Gets properties.
-  bool GetProperty (const char* name, void **parm);
+  bool GetProperty (const char* name, intptr_t *parm);
 
   /// Sets properties.
-  bool SetProperty (const char* name, void *parm);
+  bool SetProperty (const char* name, intptr_t parm);
 
   /// Returns the named TYPE of the component, like "Radio Button", etc.
   virtual const char* Type ();
