@@ -1,35 +1,46 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
-#include "awslayot.h"
-#include "awsgbl.h"
-#include "awsgbc.h"
-#include "iaws/awsdefs.h"
 #include <stdlib.h>
 #include <math.h>
 
-const int awsGridBagLayout:: MAXGRIDSIZE = 512;
-const int awsGridBagLayout:: MINSIZE = 1;
-const int awsGridBagLayout:: PREFERREDSIZE = 2;
+#include "awsgbc.h"
+#include "awsgbl.h"
+#include "awslayot.h"
+#include "csgeom/math.h"
+#include "iaws/awsdefs.h"
+#include "csutil/sysfunc.h"
 
-const int MAXINT = 0xeffffff;
+const int GBS_MAXINT = 0xeffffff;
 
-inline int max (int i, int j)
-{
-  return i > j ? i : j;
-}
-
-/////////////////////////////////////////////////////////////////////////////////
-awsGridBagConstraints::awsGridBagConstraints () :
-  gridx(GBS_RELATIVE),
-  gridy(GBS_RELATIVE),
-  gridwidth(1),
-  gridheight(1),
-  weightx(0),
-  weighty(0),
-  anchor(GBS_CENTER),
-  fill(GBS_NONE),
-  insets(0, 0, 0, 0),
-  ipadx(0),
-  ipady(0)
+awsGridBagConstraints::awsGridBagConstraints ()
+  : gridx (GBS_RELATIVE),
+    gridy (GBS_RELATIVE),
+    gridwidth (1),
+    gridheight (1),
+    weightx (0),
+    weighty (0),
+    anchor (GBS_CENTER),
+    fill (GBS_NONE),
+    insets (0, 0, 0, 0),
+    ipadx (0),
+    ipady (0)
 {
 }
 
@@ -44,43 +55,53 @@ awsGridBagConstraints::awsGridBagConstraints (
   int fill,
   csRect insets,
   int ipadx,
-  int ipady) :
-    gridx(gridx),
-    gridy(gridy),
-    gridwidth(gridwidth),
-    gridheight(gridheight),
-    weightx(weightx),
-    weighty(weighty),
-    anchor(anchor),
-    fill(fill),
-    insets(insets),
-    ipadx(ipadx),
-    ipady(ipady)
+  int ipady)
+  : gridx (gridx),
+    gridy (gridy),
+    gridwidth (gridwidth),
+    gridheight (gridheight),
+    weightx (weightx),
+    weighty (weighty),
+    anchor (anchor),
+    fill (fill),
+    insets (insets),
+    ipadx (ipadx),
+    ipady (ipady)
 {
 }
 
-awsGridBagConstraints *awsGridBagConstraints::Clone ()
+awsGridBagConstraints::awsGridBagConstraints(awsGridBagConstraints const& c)
 {
-  return new awsGridBagConstraints (
-      gridx,
-      gridy,
-      gridwidth,
-      gridheight,
-      weightx,
-      weighty,
-      anchor,
-      fill,
-      insets,
-      ipadx,
-      ipady);
+  Assign(c);
 }
 
-/////////////////////////////////////////////////////////////////////////////////////////
-awsGridBagLayout::GridBagLayoutInfo::GridBagLayoutInfo () :
-  width(0),
-  height(0),
-  startx(0),
-  starty(0)
+awsGridBagConstraints&
+awsGridBagConstraints::operator=(awsGridBagConstraints const& c)
+{
+  Assign(c);
+  return *this;
+}
+
+void awsGridBagConstraints::Assign (awsGridBagConstraints const& c)
+{
+  gridx      = c.gridx;
+  gridy      = c.gridy;
+  gridwidth  = c.gridwidth;
+  gridheight = c.gridheight;
+  weightx    = c.weightx;
+  weighty    = c.weighty;
+  anchor     = c.anchor;
+  fill       = c.fill;
+  insets     = c.insets;
+  ipadx      = c.ipadx;
+  ipady      = c.ipady;
+}
+
+awsGridBagLayout::GridBagLayoutInfo::GridBagLayoutInfo ()
+  : width  (0),
+    height (0),
+    startx (0),
+    starty (0)
 {
   minWidth = new int[awsGridBagLayout::MAXGRIDSIZE];
   minHeight = new int[awsGridBagLayout::MAXGRIDSIZE];
@@ -95,19 +116,19 @@ awsGridBagLayout::GridBagLayoutInfo::GridBagLayoutInfo () :
 
 awsGridBagLayout::GridBagLayoutInfo::~GridBagLayoutInfo ()
 {
-  if (minWidth) delete[] minWidth;
-  if (minHeight) delete[] minHeight;
-  if (weightX) delete[] weightX;
-  if (weightY) delete[] weightY;
+  delete[] minWidth;
+  delete[] minHeight;
+  delete[] weightX;
+  delete[] weightY;
 }
 
 void awsGridBagLayout::GridBagLayoutInfo::Set (
   awsGridBagLayout::GridBagLayoutInfo &l2)
 {
-  if (minWidth) delete[] minWidth;
-  if (minHeight) delete[] minHeight;
-  if (weightX) delete[] weightX;
-  if (weightY) delete[] weightY;
+  delete[] minWidth;
+  delete[] minHeight;
+  delete[] weightX;
+  delete[] weightY;
 
   width = l2.width;
   height = l2.height;
@@ -120,33 +141,37 @@ void awsGridBagLayout::GridBagLayoutInfo::Set (
 }
 
 awsGridBagLayout::GridBagLayoutInfo *
-  awsGridBagLayout::GridBagLayoutInfo::Clone()
-  {
-    GridBagLayoutInfo *i = new GridBagLayoutInfo();
+  awsGridBagLayout::GridBagLayoutInfo::Clone ()
+{
+  GridBagLayoutInfo *i = new GridBagLayoutInfo ();
 
   i->width = width;
   i->height = height;
   i->startx = startx;
   i->starty = starty;
 
-  memcpy(i->minWidth, minWidth, sizeof (int) * awsGridBagLayout::MAXGRIDSIZE);
-  memcpy(i->minHeight, minHeight, sizeof (int) * awsGridBagLayout::MAXGRIDSIZE);
-  memcpy(i->weightX, weightX, sizeof (double) * awsGridBagLayout::MAXGRIDSIZE);
-  memcpy(i->weightY, weightY, sizeof (double) * awsGridBagLayout::MAXGRIDSIZE);
+  memcpy (i->minWidth, minWidth, sizeof (int) * awsGridBagLayout::MAXGRIDSIZE);
+  memcpy (i->minHeight, minHeight, sizeof (int) * awsGridBagLayout::MAXGRIDSIZE);
+  memcpy (i->weightX, weightX, sizeof (double) * awsGridBagLayout::MAXGRIDSIZE);
+  memcpy (i->weightY, weightY, sizeof (double) * awsGridBagLayout::MAXGRIDSIZE);
 
   return i;
 }
-awsGridBagLayout::awsGridBagLayout (iAwsComponent *o) :
-  awsLayoutManager(o),
-  layoutInfo(NULL),
-  columnWidths(NULL),
-  columnWidthsLength(0),
-  rowHeights(NULL),
-  rowHeightsLength(0),
-  columnWeights(NULL),
-  columnWeightsLength(0),
-  rowWeights(NULL),
-  rowWeightsLength(0)
+
+awsGridBagLayout::awsGridBagLayout (
+  iAwsComponent *o,
+  iAwsComponentNode* settings,
+  iAwsPrefManager* pm )
+  : awsLayoutManager(o, settings, pm),
+    layoutInfo(0),
+    columnWidths(0),
+    columnWidthsLength(0),
+    rowHeights(0),
+    rowHeightsLength(0),
+    columnWeights(0),
+    columnWeightsLength(0),
+    rowWeights(0),
+    rowWeightsLength(0)
 {
 }
 
@@ -156,9 +181,8 @@ void awsGridBagLayout::LayoutComponents ()
 }
 
 csRect awsGridBagLayout::AddComponent (
-  iAwsPrefManager *pm,
-  awsComponentNode *settings,
-  iAwsComponent *cmp)
+  iAwsComponent *cmp,
+  iAwsComponentNode *settings)
 {
   awsGridBagConstraints c;
 
@@ -180,6 +204,7 @@ csRect awsGridBagLayout::AddComponent (
   c.weighty = wy / 100.0;
 
   setConstraints (cmp, c);
+  comps.Push (cmp);
 
   return csRect (0, 0, 0, 0);
 }
@@ -188,86 +213,53 @@ void awsGridBagLayout::setConstraints (
   iAwsComponent *cmp,
   awsGridBagConstraints &constraints)
 {
-  // check if the component exists.
-  awsGridBagConstraints *p = STATIC_CAST (
-      awsGridBagConstraints *,
-      comptable.Get ((csHashKey) cmp));
-
-  // kill it if so.
-  if (p) comptable.DeleteAll ((csHashKey) cmp);
-
-  // add in the new constraints.
-  comptable.Put ((csHashKey) cmp, constraints.Clone ());
+  comptable.DeleteAll (cmp);
+  comptable.Put (cmp, constraints);
 }
 
 awsGridBagConstraints awsGridBagLayout::getConstraints (iAwsComponent *cmp)
 {
-  awsGridBagConstraints *constraints = STATIC_CAST (
-      awsGridBagConstraints *,
-      comptable.Get ((csHashKey) cmp));
-
-  if (constraints == NULL)
-  {
-    setConstraints (cmp, defaultConstraints);
-    constraints = (awsGridBagConstraints *)comptable.Get ((csHashKey) cmp);
-  }
-
-  return awsGridBagConstraints (
-      constraints->gridx,
-      constraints->gridy,
-      constraints->gridwidth,
-      constraints->gridheight,
-      constraints->weightx,
-      constraints->weighty,
-      constraints->anchor,
-      constraints->fill,
-      constraints->insets,
-      constraints->ipadx,
-      constraints->ipady);
+  awsGridBagConstraints* c = lookupConstraints(cmp);
+  CS_ASSERT(c != 0);
+  return *c;
 }
 
 awsGridBagConstraints *awsGridBagLayout::lookupConstraints (
   iAwsComponent *cmp)
 {
-  awsGridBagConstraints *constraints = STATIC_CAST (
-      awsGridBagConstraints *,
-      comptable.Get ((csHashKey) cmp));
-
-  if (constraints == NULL)
-  {
-    setConstraints (cmp, defaultConstraints);
-    constraints = (awsGridBagConstraints *)comptable.Get ((csHashKey) cmp);
-  }
-
-  return constraints;
+  if (!comptable.In(cmp))
+    setConstraints(cmp, defaultConstraints);
+  awsGridBagConstraints* c = comptable.GetElementPointer(cmp);
+  CS_ASSERT(c != 0);
+  return c;
 }
 
 void awsGridBagLayout::removeConstraints (iAwsComponent *cmp)
 {
-  comptable.DeleteAll ((csHashKey) cmp);
+  comptable.DeleteAll (cmp);
 }
 
-csPoint awsGridBagLayout::getLayoutOrigin ()
+csVector2 awsGridBagLayout::getLayoutOrigin ()
 {
-  csPoint origin (0, 0);
-  if (layoutInfo != NULL)
+  csVector2 origin (0, 0);
+  if (layoutInfo != 0)
   {
     origin.x = layoutInfo->startx;
     origin.y = layoutInfo->starty;
   }
-
   return origin;
 }
 
 void awsGridBagLayout::getLayoutDimensions (int **row, int **col)
 {
-  if (!row || !col) return ;
+  if (!row || !col)
+    return;
 
-  if (layoutInfo == NULL)
+  if (layoutInfo == 0)
   {
-    *row = NULL;
-    *col = NULL;
-    return ;
+    *row = 0;
+    *col = 0;
+    return;
   }
 
   *col = new int[layoutInfo->width];
@@ -279,13 +271,14 @@ void awsGridBagLayout::getLayoutDimensions (int **row, int **col)
 
 void awsGridBagLayout::getLayoutWeights (double **row, double **col)
 {
-  if (!row || !col) return ;
+  if (!row || !col)
+    return;
 
-  if (layoutInfo == NULL)
+  if (layoutInfo == 0)
   {
-    *row = NULL;
-    *col = NULL;
-    return ;
+    *row = 0;
+    *col = 0;
+    return;
   }
 
   *col = new double[layoutInfo->width];
@@ -295,14 +288,14 @@ void awsGridBagLayout::getLayoutWeights (double **row, double **col)
   memcpy (*row, layoutInfo->weightY, layoutInfo->height * sizeof (double));
 }
 
-csPoint awsGridBagLayout::location (int x, int y)
+csVector2 awsGridBagLayout::location (int x, int y)
 {
-  csPoint loc (0, 0);
-  int i, d;
+  csVector2 loc (0, 0);
 
-  if (layoutInfo == NULL) return loc;
+  if (layoutInfo == 0)
+    return loc;
 
-  d = layoutInfo->startx;
+  int i, d = layoutInfo->startx;
   for (i = 0; i < layoutInfo->width; ++i)
   {
     d += layoutInfo->minWidth[i];
@@ -335,37 +328,39 @@ void awsGridBagLayout::removeLayoutComponent (iAwsComponent *cmp)
   removeConstraints (cmp);
 }
 
-awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
-  (iAwsComponent * parent, int sizeflag)
+awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo (
+  iAwsComponent *,
+  int sizeflag)
 {
   GridBagLayoutInfo r;
   iAwsComponent *cmp;
   awsGridBagConstraints *constraints;
 
-  int compindex, i, /*j,*/ k, px, py, pixels_diff, nextSize;
+  size_t compindex;
+  int i, k, px, py, pixels_diff, nextSize;
   int curX, curY, curWidth, curHeight, curRow, curCol;
-  double weight_diff, weight /*, start, size*/ ;
+  double weight_diff, weight;
   int xMax[MAXGRIDSIZE], yMax[MAXGRIDSIZE];
   csRect d;
 
-  /*
-     * Pass #1
-     *
-     * Figure out the dimensions of the layout grid (use a value of 1 for
-     * zero or negative widths and heights).
-     */
+  /**
+   * Pass #1
+   *<p>
+   * Figure out the dimensions of the layout grid (use a value of 1 for
+   * zero or negative widths and heights).
+   */
   r.width = r.height = 0;
   curRow = curCol = -1;
-  memset(xMax, 0, sizeof (int) * MAXGRIDSIZE);
-  memset(yMax, 0, sizeof (int) * MAXGRIDSIZE);
+  memset (xMax, 0, sizeof (int) * MAXGRIDSIZE);
+  memset (yMax, 0, sizeof (int) * MAXGRIDSIZE);
 
-  for (compindex = 0; compindex < parent->GetChildCount(); ++compindex)
+  for (compindex = 0; compindex < comps.Length (); ++compindex)
   {
-    cmp = parent->GetChildAt(compindex);
+    cmp = comps.Get (compindex);
 
-    if (cmp->isHidden()) continue;
+    if (cmp->isHidden ()) continue;
 
-    constraints = lookupConstraints(cmp);
+    constraints = lookupConstraints (cmp);
 
     curX = constraints->gridx;
     curY = constraints->gridy;
@@ -376,7 +371,7 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
     curHeight = constraints->gridheight;
     if (curHeight <= 0) curHeight = 1;
 
-    /* If x or y is negative, then use relative positioning: */
+    /// If x or y is negative, then use relative positioning.
     if (curX < 0 && curY < 0)
     {
       if (curRow >= 0)
@@ -389,8 +384,8 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
     if (curX < 0)
     {
       px = 0;
-
-      for (i = curY; i < (curY + curHeight); i++) px = max(px, xMax[i]);
+      for (i = curY; i < (curY + curHeight); i++)
+        px = csMax(px, xMax[i]);
 
       curX = px - curX - 1;
       if (curX < 0) curX = 0;
@@ -398,17 +393,18 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
     else if (curY < 0)
     {
       py = 0;
-      for (i = curX; i < (curX + curWidth); i++) py = max(py, yMax[i]);
+      for (i = curX; i < (curX + curWidth); i++)
+        py = csMax(py, yMax[i]);
 
       curY = py - curY - 1;
       if (curY < 0) curY = 0;
     }
 
-    /* Adjust the grid width and height */
+    /// Adjust the grid width and height.
     for (px = curX + curWidth; r.width < px; r.width++);
     for (py = curY + curHeight; r.height < py; r.height++);
 
-    /* Adjust the xMax and yMax arrays */
+    /// Adjust the xMax and yMax arrays.
     for (i = curX; i < (curX + curWidth); i++)
     {
       yMax[i] = py;
@@ -418,64 +414,62 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
       xMax[i] = px;
     }
 
-    /* Cache the current slave's size. */
+    /// Cache the current slave's size.
     if (sizeflag == PREFERREDSIZE)
-      d = cmp->getPreferredSize();
+      d = cmp->getPreferredSize ();
     else
-      d = cmp->getMinimumSize();
+      d = cmp->getMinimumSize ();
 
-    constraints->minWidth = d.Width();
-    constraints->minHeight = d.Height();
+    constraints->minWidth = d.Width ();
+    constraints->minHeight = d.Height ();
 
-    /* Zero width and height must mean that this is the last item (or
-       * else something is wrong). */
+    /**
+     * Zero width and height must mean that this is the last item (or
+     * else something is wrong).
+     */
     if (constraints->gridheight == 0 && constraints->gridwidth == 0)
       curRow = curCol = -1;
 
-    /* Zero width starts a new row */
+    /// Zero width starts a new row.
     if (constraints->gridheight == 0 && curRow < 0)
       curCol = curX + curWidth;
-
-    /* Zero height starts a new column */
+    /// Zero height starts a new column.
     else if (constraints->gridwidth == 0 && curCol < 0)
       curRow = curY + curHeight;
-  } // end first pass
+  } /// End first pass.
 
-  /*
-     * Apply minimum row/column dimensions
-     */
-  if (columnWidths != NULL && r.width < columnWidthsLength)
+  /// Apply minimum row/column dimensions
+  if (columnWidths != 0 && r.width < columnWidthsLength)
     r.width = columnWidthsLength;
-
-  if (rowHeights != NULL && r.height < rowHeightsLength)
+  if (rowHeights != 0 && r.height < rowHeightsLength)
     r.height = rowHeightsLength;
 
-  /*
-     * Pass #2
-     *
-     * Negative values for gridX are filled in with the current x value.
-     * Negative values for gridY are filled in with the current y value.
-     * Negative or zero values for gridWidth and gridHeight end the current
-     *  row or column, respectively.
-     */
+  /**
+   * Pass #2
+   *<p>
+   * Negative values for gridX are filled in with the current x value.
+   * Negative values for gridY are filled in with the current y value.
+   * Negative or zero values for gridWidth and gridHeight end the current
+   * row or column, respectively.
+   */
   curRow = curCol = -1;
-  memset(xMax, 0, sizeof (int) * MAXGRIDSIZE);
-  memset(yMax, 0, sizeof (int) * MAXGRIDSIZE);
+  memset (xMax, 0, sizeof (int) * MAXGRIDSIZE);
+  memset (yMax, 0, sizeof (int) * MAXGRIDSIZE);
 
-  for (compindex = 0; compindex < parent->GetChildCount(); compindex++)
+  for (compindex = 0; compindex < comps.Length(); compindex++)
   {
-    cmp = parent->GetChildAt(compindex);
+    cmp = comps.Get (compindex);
 
-    if (cmp->isHidden()) continue;
+    if (cmp->isHidden ()) continue;
 
-    constraints = lookupConstraints(cmp);
+    constraints = lookupConstraints (cmp);
 
     curX = constraints->gridx;
     curY = constraints->gridy;
     curWidth = constraints->gridwidth;
     curHeight = constraints->gridheight;
 
-    /* If x or y is negative, then use relative positioning: */
+    /// If x or y is negative, then use relative positioning.
     if (curX < 0 && curY < 0)
     {
       if (curRow >= 0)
@@ -493,8 +487,10 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
         if (curHeight < 1) curHeight = 1;
       }
     }
+
     px = 0;
-    for (i = curY; i < (curY + curHeight); i++) px = max(px, xMax[i]);
+    for (i = curY; i < (curY + curHeight); i++)
+      px = csMax(px, xMax[i]);
 
     curX = px - curX - 1;
     if (curX < 0)
@@ -506,8 +502,10 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
         curWidth += r.width - curX;
         if (curWidth < 1) curWidth = 1;
       }
+
       py = 0;
-      for (i = curX; i < (curX + curWidth); i++) py = max(py, yMax[i]);
+      for (i = curX; i < (curX + curWidth); i++)
+        py = csMax(py, yMax[i]);
 
       curY = py - curY - 1;
       if (curY < 0) curY = 0;
@@ -534,7 +532,7 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
       xMax[i] = px;
     }
 
-    /* Make negative sizes start a new row/column */
+    /// Make negative sizes start a new row/column.
     if (constraints->gridheight == 0 && constraints->gridwidth == 0)
       curRow = curCol = -1;
     if (constraints->gridheight == 0 && curRow < 0)
@@ -542,52 +540,50 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
     else if (constraints->gridwidth == 0 && curCol < 0)
       curRow = curY + curHeight;
 
-    /* Assign the new values to the gridbag child */
+    /// Assign the new values to the gridbag child.
     constraints->tempX = curX;
     constraints->tempY = curY;
     constraints->tempWidth = curWidth;
     constraints->tempHeight = curHeight;
-  } // end pass 2
+  } /// End pass 2.
 
-  /*
-     * Apply minimum row/column dimensions and weights
-     */
-  if (columnWidths != NULL)
-    memcpy(r.minWidth, columnWidths, columnWidthsLength * sizeof (int));
-  if (rowHeights != NULL)
-    memcpy(r.minHeight, rowHeights, rowHeightsLength * sizeof (int));
-  if (columnWeights != NULL)
-    memcpy(r.weightX, columnWeights, columnWeightsLength * sizeof (double));
-  if (rowWeights != NULL)
-    memcpy(r.weightY, rowWeights, rowWeightsLength * sizeof (double));
+  /// Apply minimum row/column dimensions and weights.
+  if (columnWidths != 0)
+    memcpy (r.minWidth, columnWidths, columnWidthsLength * sizeof (int));
+  if (rowHeights != 0)
+    memcpy (r.minHeight, rowHeights, rowHeightsLength * sizeof (int));
+  if (columnWeights != 0)
+    memcpy (r.weightX, columnWeights, columnWeightsLength * sizeof (double));
+  if (rowWeights != 0)
+    memcpy (r.weightY, rowWeights, rowWeightsLength * sizeof (double));
 
-  /*
-     * Pass #3
-     *
-     * Distribute the minimun widths and weights:
-     */
-  nextSize = MAXINT;
+  /**
+   * Pass #3
+   *<p>
+   * Distribute the minimun widths and weights:
+   */
+  nextSize = GBS_MAXINT;
 
-  for (i = 1; i != MAXINT; i = nextSize, nextSize = MAXINT)
+  for (i = 1; i != GBS_MAXINT; i = nextSize, nextSize = GBS_MAXINT)
   {
-    for (compindex = 0; compindex < parent->GetChildCount(); ++compindex)
+    for (compindex = 0; compindex < comps.Length(); ++compindex)
     {
-      cmp = parent->GetChildAt(compindex);
+      cmp = comps.Get (compindex);
 
-      if (cmp->isHidden()) continue;
+      if (cmp->isHidden ()) continue;
 
-      constraints = lookupConstraints(cmp);
+      constraints = lookupConstraints (cmp);
 
       if (constraints->tempWidth == i)
       {
-        px = constraints->tempX + constraints->tempWidth;   /* right column */
+        px = constraints->tempX + constraints->tempWidth; /// Right column.
 
-        /*
-	   * Figure out if we should use this child\'s weight.  If the weight
-	   * is less than the total weight spanned by the width of the cell,
-	   * then discard the weight.  Otherwise split the difference
-	   * according to the existing weights.
-	   */
+        /**
+	 * Figure out if we should use this child\'s weight.  If the weight
+	 * is less than the total weight spanned by the width of the cell,
+	 * then discard the weight.  Otherwise split the difference
+	 * according to the existing weights.
+	 */
         weight_diff = constraints->weightx;
         for (k = constraints->tempX; k < px; k++)
           weight_diff -= r.weightX[k];
@@ -604,17 +600,17 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
             weight -= wt;
           }
 
-          /* Assign the remainder to the rightmost cell */
-          r.weightX[px - 1] += weight_diff;
+          /// Assign the remainder to the rightmost cell.
+          r.weightX[px > 0 ? px - 1 : 0] += weight_diff;
         }
 
-        /*
-	   * Calculate the minWidth array values.
-	   * First, figure out how wide the current child needs to be.
-	   * Then, see if it will fit within the current minWidth values.
-	   * If it will not fit, add the difference according to the
-	   * weightX array.
-	   */
+        /**
+	 * Calculate the minWidth array values.
+	 * First, figure out how wide the current child needs to be.
+	 * Then, see if it will fit within the current minWidth values.
+	 * If it will not fit, add the difference according to the
+	 * weightX array.
+	 */
         pixels_diff = constraints->minWidth +
           constraints->ipadx +
           constraints->insets.xmin +
@@ -622,6 +618,7 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
 
         for (k = constraints->tempX; k < px; k++)
           pixels_diff -= r.minWidth[k];
+
         if (pixels_diff > 0)
         {
           weight = 0.0;
@@ -635,28 +632,27 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
             weight -= wt;
           }
 
-          /* Any leftovers go into the rightmost cell */
-          r.minWidth[px - 1] += pixels_diff;
+          /// Any leftovers go into the rightmost cell.
+          r.minWidth[px > 0 ? px - 1 : 0] += pixels_diff;
         }
       }
-      else if (
-            constraints->tempWidth > i &&
-            constraints->tempWidth < nextSize)
+      else if (constraints->tempWidth > i && constraints->tempWidth < nextSize)
         nextSize = constraints->tempWidth;
 
       if (constraints->tempHeight == i)
       {
-        py = constraints->tempY + constraints->tempHeight;  /* bottom row */
+        py = constraints->tempY + constraints->tempHeight; /// Bottom row.
 
-        /*
-	   * Figure out if we should use this child\'s weight.  If the weight
-	   * is less than the total weight spanned by the height of the cell,
-	   * then discard the weight.  Otherwise split it the difference
-	   * according to the existing weights.
-	   */
+        /**
+	 * Figure out if we should use this child\'s weight.  If the weight
+	 * is less than the total weight spanned by the height of the cell,
+	 * then discard the weight.  Otherwise split it the difference
+	 * according to the existing weights.
+	 */
         weight_diff = constraints->weighty;
         for (k = constraints->tempY; k < py; k++)
           weight_diff -= r.weightY[k];
+
         if (weight_diff > 0.0)
         {
           weight = 0.0;
@@ -670,27 +666,30 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
             weight -= wt;
           }
 
-          /* Assign the remainder to the bottom cell */
-          r.weightY[py - 1] += weight_diff;
+          /// Assign the remainder to the bottom cell.
+          r.weightY[py > 0 ? py - 1 : 0] += weight_diff;
         }
 
-        /*
-	   * Calculate the minHeight array values.
-	   * First, figure out how tall the current child needs to be.
-	   * Then, see if it will fit within the current minHeight values.
-	   * If it will not fit, add the difference according to the
-	   * weightY array.
-	   */
+        /**
+	 * Calculate the minHeight array values.
+	 * First, figure out how tall the current child needs to be.
+	 * Then, see if it will fit within the current minHeight values.
+	 * If it will not fit, add the difference according to the
+	 * weightY array.
+	 */
         pixels_diff = constraints->minHeight +
           constraints->ipady +
           constraints->insets.ymin +
           constraints->insets.ymax;
+
         for (k = constraints->tempY; k < py; k++)
           pixels_diff -= r.minHeight[k];
+
         if (pixels_diff > 0)
         {
           weight = 0.0;
-          for (k = constraints->tempY; k < py; k++) weight += r.weightY[k];
+          for (k = constraints->tempY; k < py; k++)
+            weight += r.weightY[k];
           for (k = constraints->tempY; weight > 0.0 && k < py; k++)
           {
             double wt = r.weightY[k];
@@ -700,17 +699,18 @@ awsGridBagLayout::GridBagLayoutInfo * awsGridBagLayout::GetLayoutInfo
             weight -= wt;
           }
 
-          /* Any leftovers go into the bottom cell */
-          r.minHeight[py - 1] += pixels_diff;
+          /// Any leftovers go into the bottom cell.
+          r.minHeight[py > 0 ? py - 1 : 0] += pixels_diff;
         }
       }
-      else if (
-            constraints->tempHeight > i &&
-            constraints->tempHeight < nextSize)
+      else if (constraints->tempHeight > i &&
+        constraints->tempHeight < nextSize)
+      {
         nextSize = constraints->tempHeight;
+      }
     }
   }
-  return r.Clone();
+  return r.Clone ();
 }
 void awsGridBagLayout::AdjustForGravity (
   awsGridBagConstraints *constraints,
@@ -724,11 +724,8 @@ void awsGridBagLayout::AdjustForGravity (
   r.ymax -= constraints->insets.ymax;
 
   diffx = 0;
-  if (
-    (
-      constraints->fill != awsGridBagConstraints::GBS_HORIZONTAL &&
-      constraints->fill != awsGridBagConstraints::GBS_BOTH
-    ) &&
+  if ((constraints->fill != awsGridBagConstraints::GBS_HORIZONTAL &&
+    constraints->fill != awsGridBagConstraints::GBS_BOTH) &&
     (r.Width () > (constraints->minWidth + constraints->ipadx)))
   {
     diffx = r.Width () - (constraints->minWidth + constraints->ipadx);
@@ -736,11 +733,8 @@ void awsGridBagLayout::AdjustForGravity (
   }
 
   diffy = 0;
-  if (
-    (
-      constraints->fill != awsGridBagConstraints::GBS_VERTICAL &&
-      constraints->fill != awsGridBagConstraints::GBS_BOTH
-    ) &&
+  if ((constraints->fill != awsGridBagConstraints::GBS_VERTICAL &&
+    constraints->fill != awsGridBagConstraints::GBS_BOTH) &&
     (r.Height () > (constraints->minHeight + constraints->ipady)))
   {
     diffy = r.Height () - (constraints->minHeight + constraints->ipady);
@@ -749,35 +743,35 @@ void awsGridBagLayout::AdjustForGravity (
 
   switch (constraints->anchor)
   {
-    case awsGridBagConstraints::GBS_CENTER:
-      r.Move (diffx / 2, diffx / 2);
-      break;
-    case awsGridBagConstraints::GBS_NORTH:
-      r.Move (diffx / 2, 0);
-      break;
-    case awsGridBagConstraints::GBS_NORTHEAST:
-      r.Move (diffx, 0);
-      break;
-    case awsGridBagConstraints::GBS_EAST:
-      r.Move (diffx, diffy / 2);
-      break;
-    case awsGridBagConstraints::GBS_SOUTHEAST:
-      r.Move (diffx, diffy);
-      break;
-    case awsGridBagConstraints::GBS_SOUTH:
-      r.Move (diffx / 2, diffy);
-      break;
-    case awsGridBagConstraints::GBS_SOUTHWEST:
-      r.Move (0, diffy);
-      break;
-    case awsGridBagConstraints::GBS_WEST:
-      r.Move (0, diffy / 2);
-      break;
-    case awsGridBagConstraints::GBS_NORTHWEST:
-      break;
-    default:
-      printf ("GridBag: bad gravity!\n");
-      break;
+  case awsGridBagConstraints::GBS_CENTER:
+    r.Move (diffx / 2, diffx / 2);
+    break;
+  case awsGridBagConstraints::GBS_NORTH:
+    r.Move (diffx / 2, 0);
+    break;
+  case awsGridBagConstraints::GBS_NORTHEAST:
+    r.Move (diffx, 0);
+    break;
+  case awsGridBagConstraints::GBS_EAST:
+    r.Move (diffx, diffy / 2);
+    break;
+  case awsGridBagConstraints::GBS_SOUTHEAST:
+    r.Move (diffx, diffy);
+    break;
+  case awsGridBagConstraints::GBS_SOUTH:
+    r.Move (diffx / 2, diffy);
+    break;
+  case awsGridBagConstraints::GBS_SOUTHWEST:
+    r.Move (0, diffy);
+    break;
+  case awsGridBagConstraints::GBS_WEST:
+    r.Move (0, diffy / 2);
+    break;
+  case awsGridBagConstraints::GBS_NORTHWEST:
+    break;
+  default:
+    csPrintf ("GridBag: bad gravity!\n");
+    break;
   }
 }
 
@@ -807,7 +801,7 @@ csRect awsGridBagLayout::GetMinSize (
 void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
 {
   iAwsComponent *cmp;
-  int compindex;
+  size_t compindex;
   awsGridBagConstraints *constraints;
   csRect insets (0, 0, 0, 0);
   csRect d;
@@ -816,47 +810,45 @@ void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
   double weight;
   GridBagLayoutInfo *info;
 
-  /*
-     * If the parent has no slaves anymore, then don't do anything
-     * at all:  just leave the parent's size as-is.
-     */
-  if (
-    parent->GetChildCount () == 0 &&
-    (columnWidths == NULL || columnWidthsLength == 0) &&
-    (rowHeights == NULL || rowHeightsLength == 0))
+  /**
+   * If the parent has no slaves anymore, then don't do anything
+   * at all:  just leave the parent's size as-is.
+   */
+  if (parent->GetChildCount () == 0 &&
+    (columnWidths == 0 || columnWidthsLength == 0) &&
+    (rowHeights == 0 || rowHeightsLength == 0))
   {
-    return ;
+    return;
   }
 
-  /*
-     * Pass #1: scan all the children to figure out the total amount
-     * of space needed.
-     */
+  /**
+   * Pass #1: scan all the children to figure out the total amount
+   * of space needed.
+   */
   info = GetLayoutInfo (parent, PREFERREDSIZE);
   d = GetMinSize (parent, info);
 
-  if (
-    parent->Frame ().Width () < d.Width () ||
+  if (parent->Frame ().Width () < d.Width () ||
     parent->Frame ().Height () < d.Height ())
   {
-    if (info) delete info;
+    delete info;
 
     info = GetLayoutInfo (parent, MINSIZE);
     d = GetMinSize (parent, info);
   }
 
-  if (layoutInfo) delete layoutInfo;
+  delete layoutInfo;
 
   layoutInfo = info;
 
   r.xmax = d.xmax;
   r.ymax = d.ymax;
 
-  /*
-     * If the current dimensions of the window don't match the desired
-     * dimensions, then adjust the minWidth and minHeight arrays
-     * according to the weights.
-     */
+  /**
+   * If the current dimensions of the window don't match the desired
+   * dimensions, then adjust the minWidth and minHeight arrays
+   * according to the weights.
+   */
   diffw = parent->Frame ().Width () - r.Width ();
   if (diffw != 0)
   {
@@ -876,7 +868,6 @@ void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
         }
       }
     }
-
     diffw = parent->Frame ().Width () - r.Width ();
   }
   else
@@ -903,7 +894,6 @@ void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
         }
       }
     }
-
     diffh = parent->Frame ().Height () - r.Height ();
   }
   else
@@ -911,16 +901,16 @@ void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
     diffh = 0;
   }
 
-  /*
-     * Now do the actual layout of the children using the layout information
-     * that has been collected.
-     */
+  /**
+   * Now do the actual layout of the children using the layout information
+   * that has been collected.
+   */
   info->startx = diffw / 2 + insets.xmin;
   info->starty = diffh / 2 + insets.ymin;
 
-  for (compindex = 0; compindex < parent->GetChildCount (); ++compindex)
+  for (compindex = 0; compindex < comps.Length (); ++compindex)
   {
-    cmp = parent->GetChildAt (compindex);
+    cmp = comps.Get (compindex);
     if (cmp->isHidden ()) continue;
 
     constraints = lookupConstraints (cmp);
@@ -950,16 +940,16 @@ void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
 
     AdjustForGravity (constraints, r);
 
-    /*
-       * When is a window ever too small to be interesting in AWS?
-       * Not sure, but we don't have an unmap paradigm, so we just make
-       * it really, really small.  I need to add a pseudo flag which means
-       * that the component is too small to be seen but not actually hidden.
-       */
+    /**
+     * When is a window ever too small to be interesting in AWS?
+     * Not sure, but we don't have an unmap paradigm, so we just make
+     * it really, really small.  I need to add a pseudo flag which means
+     * that the component is too small to be seen but not actually hidden.
+     */
     if ((r.Width () <= 0) || (r.Height () <= 0))
     {
-      cmp->Frame ().MakeEmpty ();
-      cmp->SetFlag(AWSF_CMP_INVISIBLE);
+      cmp->ResizeTo (r);
+      cmp->SetFlag (AWSF_CMP_INVISIBLE);
     }
     else
     {
@@ -969,9 +959,10 @@ void awsGridBagLayout::ArrangeGrid (iAwsComponent *parent)
         cmp->Frame ().Width () != r.Width () ||
         cmp->Frame ().Height () != r.Height ())
       {
-	cmp->ClearFlag(AWSF_CMP_INVISIBLE);
-        cmp->Frame ().Set (r);
-        cmp->OnResized ();
+	cmp->ClearFlag (AWSF_CMP_INVISIBLE);
+        cmp->ResizeTo (r);
+        cmp->Move (owner->ClientFrame ().xmin, owner->ClientFrame ().ymin);
+        //cmp->OnResized ();
       }
     }
   }
