@@ -17,51 +17,44 @@
 */
 
 #include "cssysdef.h"
-#include "iutil/string.h"
-#include "csutil/scfstr.h"
 #include "csgeom/csrect.h"
+#include "csutil/scfstr.h"
+#include "csutil/sysfunc.h"
 #include "iaws/aws.h"
+#include "iutil/string.h"
 #include "awsprefs.h"
-#include "awsadler.h"
 
-iAwsKey* awsKeyContainer::Find (iString *n)
+iAwsKey* awsKeyContainer::Find (iString *n) const
 {
-  return Find (
-    aws_adler32 (
-      aws_adler32 (0, 0, 0),
-      (unsigned char *)n->GetData (),
-      n->Length ()));
+  return Find (ComputeKeyID(n->GetData()));
 }
 
-iAwsKey* awsKeyContainer::Find (const char* n)
+iAwsKey* awsKeyContainer::Find (const char* n) const
 {
-  return Find (
-    aws_adler32 (
-      aws_adler32 (0, 0, 0),
-      (unsigned char*) n,
-      strlen(n)));
+  return Find (ComputeKeyID(n));
 }
 
-iAwsKey *awsKeyContainer::Find (unsigned long idname)
+iAwsKey *awsKeyContainer::Find (unsigned long idname) const
 {
   if (aws_debug)
-    printf (
-      "aws-debug: searching for %lu (%i items)\n",
+    csPrintf (
+      "aws-debug: searching for %lu (%zu items)\n",
       idname,
       children.Length ());
 
-  int i;
+  size_t i;
   for (i = 0; i < children.Length (); ++i)
   {
     iAwsKey *key = children[i];
 
     if (aws_debug)
-      printf ("aws-debug: item %d=%lu ? %lu\n", i, key->Name (), idname);
+      csPrintf ("aws-debug: item %zu=%lu ? %lu\n", i,
+	      key->Name (), idname);
 
     if (key && key->Name () == idname) return key;
   }
 
-  if (aws_debug) printf ("aws-debug: search failed.\n");
+  if (aws_debug) csPrintf ("aws-debug: search failed.\n");
 
   return 0;
 }
@@ -96,8 +89,8 @@ void awsKeyContainer::Consume (iAwsKeyContainer *c)
 {
   if (aws_debug)
   {
-    printf (
-      "aws-debug: Consuming %d items (%d items currently).\n",
+    csPrintf (
+      "aws-debug: Consuming %d items (%zu items currently).\n",
       c->Length (),
       children.Length ());
   }
@@ -119,7 +112,8 @@ void awsKeyContainer::Consume (iAwsKeyContainer *c)
   }
 
   if (aws_debug)
-    printf ("aws-debug: Now contains %d items.\n", children.Length ());
+    csPrintf ("aws-debug: Now contains %zu items.\n",
+	    children.Length ());
 
   /**
    * Do NOT delete awsKeyContainer!  This is NOT a memory leak! The
@@ -127,8 +121,7 @@ void awsKeyContainer::Consume (iAwsKeyContainer *c)
    */
 }
 
-awsConnectionNode::awsConnectionNode ()
-  : awsKeyContainer ("Connect")
+awsConnectionNode::awsConnectionNode (iAws* a) : awsKeyContainer (a,"Connect")
 {
 }
 
