@@ -34,7 +34,7 @@ private:
   int alignment;
 
   /// Caption text for this component.
-  iString *caption;
+  csRef<iString> caption;
 public:
   awsLabel ();
   virtual ~awsLabel ();
@@ -63,10 +63,10 @@ public:
   virtual bool Setup (iAws *wmgr, iAwsComponentNode *settings);
 
   /// Get properties.
-  bool GetProperty (const char *name, void **parm);
+  bool GetProperty (const char *name, intptr_t *parm);
 
   /// Set properties.
-  bool SetProperty (const char *name, void *parm);
+  bool SetProperty (const char *name, intptr_t parm);
 
   /// Returns the named TYPE of the component, like "Radio Button", etc.
   virtual const char *Type ();
