@@ -1,3 +1,21 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
 #include "awslstbx.h"
 #include "awsfparm.h"
@@ -9,22 +27,18 @@
 #include "csutil/scfstr.h"
 #include "csutil/csevent.h"
 #include "csutil/snprintf.h"
+#include "csutil/sysfunc.h"
 #include "iutil/evdefs.h"
 
 #include <stdio.h>
 
-const int awsListBox:: fsBump = 0x0;
-const int awsListBox:: fsSimple = 0x1;
-const int awsListBox:: fsRaised = 0x2;
-const int awsListBox:: fsSunken = 0x3;
-const int awsListBox:: fsFlat = 0x4;
-const int awsListBox:: fsNone = 0x5;
-
 const int awsListBox:: ctList = 0x0;
 const int awsListBox:: ctTree = 0x1;
 
-const int awsListBox:: signalSelected = 0x1;
-const int awsListBox:: signalScrolled = 0x2;
+const int awsListBox:: signalSelected      = 0x1;
+const int awsListBox:: signalScrolled      = 0x2;
+const int awsListBox:: signalFocused       = 0x3;
+const int awsListBox:: signalStateChanged  = 0x4;
 
 const int alignLeft = 0;
 const int alignCenter = 1;
@@ -34,14 +48,13 @@ const int hsTreeBox = 0;
 const int hsState = 1;
 const int hsRow = 2;
 
-//////////////////////////////////////////////////////////////////////////
-
+///////////////////////////////////////////////////////////////////////////////
 // awsListRow implementation
+///////////////////////////////////////////////////////////////////////////////
 
-//
 awsListRow::~awsListRow ()
 {
-  if (cols) delete[] cols;
+  delete[] cols;
 }
 
 int awsListRow::GetHeight (iAwsPrefManager *pm, int colcount)
@@ -59,15 +72,14 @@ int awsListRow::GetHeight (iAwsPrefManager *pm, int colcount)
   return minheight;
 }
 
-//////////////////////////////////////////////////////////////////////////
-
+///////////////////////////////////////////////////////////////////////////////
 // awsListItem implementation
+///////////////////////////////////////////////////////////////////////////////
 
-//
 awsListItem::~awsListItem ()
 {
-  SCF_DEC_REF (text);
-  SCF_DEC_REF (image);
+  if (text) text->DecRef ();
+  if (image) image->DecRef ();
 }
 
 int awsListItem::GetHeight (iAwsPrefManager *pm)
@@ -84,67 +96,93 @@ int awsListItem::GetHeight (iAwsPrefManager *pm)
     return th;
 }
 
-//////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+// awsListRowVector implementation
+///////////////////////////////////////////////////////////////////////////////
+int awsListRowVector::sortcol = 0;
 
+int awsListRowVector::Compare (awsListRow* const& r1, awsListRow* const& r2)
+{
+  if (r1->cols[sortcol].text && r2->cols[sortcol].text)
+    return strcmp (r1->cols[sortcol].text->GetData (),
+		   r2->cols[sortcol].text->GetData ());
+  else if (r1->cols[sortcol].text)
+    return 1;
+  else if (r2->cols[sortcol].text)
+    return -1;
+  else
+    return 0;
+}
+
+int awsListRowVector::CompareKey (awsListRow* const& r1, iString* const& Key)
+{
+  if (r1->cols[sortcol].text)
+    return strcmp (r1->cols[sortcol].text->GetData (), Key->GetData ());
+  else
+    return -1;
+}
+
+///////////////////////////////////////////////////////////////////////////////
 // awsListBox implementation
+///////////////////////////////////////////////////////////////////////////////
 
-//
 awsListBox::awsListBox () :
   is_down(false),
   mouse_is_over(false),
   is_switch(false),
   was_down(false),
-  bkg(NULL),
-  highlight(NULL),
-  tree_collapsed(NULL),
-  tree_expanded(NULL),
-  tree_hline(NULL),
-  tree_vline(NULL),
-  frame_style(0),
-  alpha_level(92),
+  highlight(0),
+  tree_collapsed(0),
+  tree_expanded(0),
+  tree_hline(0),
+  tree_vline(0),
   hi_alpha_level(128),
   control_type(0),
   ncolumns(1),
-  sel(NULL),
-  map(NULL),
+  sel(0),
+  map(0),
   map_size(0),
   map_dirty(true),
   scroll_start(0),
-  drawable_count(0)
+  drawable_count(0),
+  sink(0),
+  slot(0),
+  scrollbar(0),
+  actions(0)
 {
-  actions.Register ("InsertItem", &InsertItem);
-  actions.Register ("DeleteItem", &DeleteItem);
-  actions.Register ("GetSelectedItem", &GetSelectedItem);
-  actions.Register ("ClearList", &ClearList);
 }
 
 awsListBox::~awsListBox ()
 {
+  delete actions;
 }
 
-char *awsListBox::Type ()
+const char *awsListBox::Type ()
 {
   return "List Box";
 }
 
-bool awsListBox::Setup (iAws *_wmgr, awsComponentNode *settings)
+bool awsListBox::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
-  iString *tn1 = NULL, *tn2 = NULL;
   char buf[64];
   int i;
   int sb_h, sb_w;
   int border = 3;
-  int min = 0, max = 0, change = 1, bigchange = 1;
+  float min = 0, max = 0, change = 1, bigchange = 1;
 
-  if (!awsComponent::Setup (_wmgr, settings)) return false;
+  if (!awsPanel::Setup (_wmgr, settings)) return false;
 
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
 
-  pm->LookupIntKey ("OverlayTextureAlpha", alpha_level);  // global get
+  actions = new awsActionDispatcher(WindowManager());
+  actions->Register ("InsertItem", &InsertItem);
+  actions->Register ("DeleteItem", &DeleteItem);
+  actions->Register ("GetSelectedItem", &GetSelectedItem);
+  actions->Register ("GetItem", &GetItem);
+  actions->Register ("ClearList", &ClearList);
+
   pm->LookupIntKey ("ScrollBarHeight", sb_h);
   pm->LookupIntKey ("ScrollBarWidth", sb_w);
-  pm->GetInt (settings, "Style", frame_style);
-  pm->GetInt (settings, "Alpha", alpha_level);            // local overrides, if present.
   pm->GetInt (settings, "HiAlpha", hi_alpha_level);
   pm->GetInt (settings, "Columns", ncolumns);
   pm->GetInt (settings, "Type", control_type);
@@ -159,23 +197,20 @@ bool awsListBox::Setup (iAws *_wmgr, awsComponentNode *settings)
   tree_grpe = pm->GetTexture ("TreeGrpUnmarked");
   tree_grpf = pm->GetTexture ("TreeGrpMarked");
 
+  csRef<iString> tn1 (csPtr<iString> (new scfString ()));
+  csRef<iString> tn2 (csPtr<iString> (new scfString ()));
   pm->GetString (settings, "Background", tn1);
   pm->GetString (settings, "Highlight", tn2);
 
-  switch (frame_style)
-  {
-    case fsBump:    border = 5; break;
-    case fsSimple:  border = 1; break;
-  }
+  if(style == fsBump) border = 5;
+  else if(style == fsSimple) border = 1;
 
   rows.SetSortCol (sortcol);
 
-  if (tn1)
+  if (!tn1->IsEmpty())
     bkg = pm->GetTexture (tn1->GetData (), tn1->GetData ());
-  else
-    bkg = pm->GetTexture ("Texture");
 
-  if (tn2) highlight = pm->GetTexture (tn2->GetData (), tn2->GetData ());
+  if (!tn2->IsEmpty()) highlight = pm->GetTexture (tn2->GetData (), tn2->GetData ());
 
   // Make sure we have at least one column
   ncolumns = (ncolumns < 1 ? 1 : ncolumns);
@@ -188,53 +223,53 @@ bool awsListBox::Setup (iAws *_wmgr, awsComponentNode *settings)
   for (i = 0; i < ncolumns; ++i)
   {
     cs_snprintf (buf, 64, "Column%dImg", i);
-    pm->GetString (settings, buf, tn1);
+    if (!pm->GetString (settings, buf, tn1)) tn1->Clear();
     cs_snprintf (buf, 64, "Column%dBkg", i);
-    pm->GetString (settings, buf, tn2);
+    if (!pm->GetString (settings, buf, tn2)) tn2->Clear();
     cs_snprintf (buf, 64, "Column%dCaption", i);
+    columns[i].caption.AttachNew (new scfString (""));
     pm->GetString (settings, buf, columns[i].caption);
     cs_snprintf (buf, 64, "Column%dWidth", i);
     pm->GetInt (settings, buf, columns[i].width);
     cs_snprintf (buf, 64, "Column%dAlign", i);
     pm->GetInt (settings, buf, columns[i].align);
 
-    if (tn1)
+    if (!tn1->IsEmpty())
       columns[i].image = pm->GetTexture (tn1->GetData (), tn1->GetData ());
 
-    if (tn2)
+    if (!tn2->IsEmpty())
       columns[i].bkg = pm->GetTexture (tn2->GetData (), tn2->GetData ());
   }
 
   // Setup embedded scrollbar
   scrollbar = new awsScrollBar;
 
-  awsKeyFactory sbinfo;
+  awsKeyFactory sbinfo(WindowManager());
 
-  sbinfo.Initialize (
-      new scfString ("vertscroll"),
-      new scfString ("Scroll Bar"));
+  sbinfo.Initialize ("vertscroll", "Scroll Bar");
 
   sbinfo.AddRectKey (
-      new scfString ("Frame"),
+      "Frame",
       csRect (
         Frame ().Width () - sb_w - 1,
         border,
         Frame ().Width () - 1,
         Frame ().Height () - 1));
 
-  sbinfo.AddIntKey (new scfString ("Orientation"), awsScrollBar::sboVertical);
+  sbinfo.AddIntKey ("Orientation", awsScrollBar::sboVertical);
 
-  scrollbar->SetWindow (Window ());
   scrollbar->SetParent (this);
   scrollbar->Setup (_wmgr, sbinfo.GetThisNode ());
 
-  scrollbar->SetProperty ("Change", &change);
-  scrollbar->SetProperty ("BigChange", &bigchange);
-  scrollbar->SetProperty ("Max", &max);
-  scrollbar->SetProperty ("Min", &min);
+  scrollbar->SetProperty ("Change", (intptr_t)&change);
+  scrollbar->SetProperty ("BigChange", (intptr_t)&bigchange);
+  scrollbar->SetProperty ("Max", (intptr_t)&max);
+  scrollbar->SetProperty ("Min", (intptr_t)&min);
 
   // Setup trigger
-  sink = new awsSink (this);
+  awsSink* _sink = new awsSink (WindowManager());
+  _sink->SetParm ((intptr_t)this);
+  sink = _sink;
 
   sink->RegisterTrigger ("ScrollChanged", &ScrollChanged);
 
@@ -249,26 +284,22 @@ bool awsListBox::Setup (iAws *_wmgr, awsComponentNode *settings)
   return true;
 }
 
-bool awsListBox::GetProperty (char *name, void **parm)
+bool awsListBox::GetProperty (const char *name, intptr_t *parm)
 {
-  if (awsComponent::GetProperty (name, parm)) return true;
-
-  return false;
+  return awsPanel::GetProperty (name, parm);
 }
 
-bool awsListBox::SetProperty (char *name, void *parm)
+bool awsListBox::SetProperty (const char *name, intptr_t parm)
 {
-  if (awsComponent::SetProperty (name, parm)) return true;
-
-  return false;
+  return awsPanel::SetProperty (name, parm);
 }
 
-void awsListBox::ScrollChanged (void *sk, iAwsSource *source)
+void awsListBox::ScrollChanged (unsigned long, intptr_t sk, iAwsSource *source)
 {
   awsListBox *lb = (awsListBox *)sk;
   float *curval = 0;
 
-  source->GetComponent ()->GetProperty ("Value", (void **) &curval);
+  source->GetComponent ()->GetProperty ("Value", (intptr_t*)&curval);
 
   lb->UpdateMap ();
   lb->scroll_start = (int) *curval;
@@ -277,28 +308,31 @@ void awsListBox::ScrollChanged (void *sk, iAwsSource *source)
 
   if (lb->scroll_start < 0) lb->scroll_start = 0;
 
+  lb->Broadcast(awsListBox::signalScrolled);
   lb->Invalidate ();
 }
 
 void awsListBox::UpdateMap ()
 {
-  int map_size_m1;
-
   if (map_dirty)
   {
     int start = 0;
 
     map_dirty = false;
     map_size = 0;
-    if (map) delete[] map;
+    delete[] map;
 
     // Traverse once for the count of visible items
     map_size = CountVisibleItems (&rows);
     map = new awsListRow *[map_size];
 
-    // Set the scroll bar's max position
-    map_size_m1 = map_size - 1;
-    scrollbar->SetProperty ("Max", &map_size_m1);
+    // Set the scroll bar's max position.  Since we're using the pagesize
+    // (amount visible) property of the scrollbar, the max position will be the
+    // total number of rows.  The pagesize will be the number of visible rows,
+    // and the current value of the scrollbar will be the topmost visible row.
+    float max_scroll;
+    max_scroll = map_size;
+    scrollbar->SetProperty ("Max", (intptr_t)&max_scroll);
 
     // Map out items
     MapVisibleItems (&rows, start, map);
@@ -307,7 +341,7 @@ void awsListBox::UpdateMap ()
 
 int awsListBox::CountVisibleItems (awsListRowVector *v)
 {
-  int i;
+  size_t i;
   int count = 0;
   for (i = 0; i < v->Length (); ++i)
   {
@@ -322,12 +356,13 @@ int awsListBox::CountVisibleItems (awsListRowVector *v)
   return count;
 }
 
+
 void awsListBox::MapVisibleItems (
   awsListRowVector *v,
   int &start,
   awsListRow **map)
 {
-  int i;
+  size_t i;
   for (i = 0; i < v->Length (); ++i)
   {
     awsListRow *r = (awsListRow *) ((*v)[i]);
@@ -341,25 +376,25 @@ void awsListBox::MapVisibleItems (
 
 static int DoFindItem (awsListRowVector *v, iString *text, bool with_delete)
 {
-  int i = v->Find (text);
+  v->sortcol = v->local_sortcol;
+  size_t key = v->FindKey (v->KeyCmp(text));
 
-  if (i)
+  if (key != csArrayItemNotFound)
   {
-    if (with_delete) v->Delete (i, true);
+    if (with_delete) v->DeleteIndex (key);
 
-    return i;
+    return (int)key;
   }
   else
   {
     int j;
-    for (i = 0; i < v->Length (); ++i)
+    for (size_t i = 0; i < v->Length (); ++i)
     {
       awsListRow *r = (awsListRow *) ((*v)[i]);
-      if (
-        r->children &&
+      if (r->children &&
         (j = DoFindItem (r->children, text, with_delete)) >= 0)
       {
-        if (with_delete) r->children->Delete (j, true);
+        if (with_delete) r->children->DeleteIndex (j);
 
         return j;
       }
@@ -371,7 +406,7 @@ static int DoFindItem (awsListRowVector *v, iString *text, bool with_delete)
 
 static void DoRecursiveClearList (awsListRowVector *v)
 {
-  int i;
+  size_t i;
 
   for (i = 0; i < v->Length (); ++i)
   {
@@ -383,12 +418,16 @@ static void DoRecursiveClearList (awsListRowVector *v)
     }
   }
 
-  v->DeleteAll (true);
+  v->DeleteAll ();
 }
 
-/////////////// Scripted Actions /////////////////////////////////////////////////////////////
-void awsListBox::InsertItem (void *owner, iAwsParmList &parmlist)
+/////////////// Scripted Actions //////////////////////////////////////////////
+
+void awsListBox::InsertItem (intptr_t owner, iAwsParmList* parmlist)
 {
+  if (!parmlist)
+    return;
+
   awsListBox *lb = (awsListBox *)owner;
 
   char buf[50];
@@ -403,45 +442,50 @@ void awsListBox::InsertItem (void *owner, iAwsParmList &parmlist)
   row->cols = new awsListItem[lb->ncolumns];
   memset (row->cols, 0, sizeof (awsListItem) * lb->ncolumns);
 
-  parmlist.GetInt ("parent", (int *) &(row->parent));
-  parmlist.GetBool ("selectable", &(row->selectable));
+  parmlist->GetOpaque ("parent", (intptr_t*)&row->parent);
+  row->selectable = true;
+  parmlist->GetBool ("selectable", &(row->selectable));
 
-  /* Fill in the columns by looking for several parameters:
-   *   textX, imageX, txtalignX, imgalignX, statefulX, stateX, groupstateX,
-   *   selectableX
-   */
+  // Fill in the columns by looking for several parameters: textX, imageX,
+  // txtalignX, imgalignX, statefulX, stateX, groupstateX, selectableX
   for (i = 0; i < lb->ncolumns; ++i)
   {
     cs_snprintf (buf, 50, "text%d", i);
-    if (parmlist.GetString (buf, &(row->cols[i].text)))
+    if (parmlist->GetString (buf, &(row->cols[i].text)))
       row->cols[i].text->IncRef ();
 
     cs_snprintf (buf, 50, "image%d", i);
-    if (parmlist.GetString (buf, &str))
+    if (parmlist->GetString (buf, &str))
+    {
       row->cols[i].image = lb->WindowManager ()->GetPrefMgr ()->GetTexture (
           str->GetData (),
           str->GetData ());
+      if (row->cols[i].image) row->cols[i].image->IncRef ();
+    }
 
     cs_snprintf (buf, 50, "stateful%d", i);
-    parmlist.GetBool (buf, &(row->cols[i].has_state));
+    parmlist->GetBool (buf, &(row->cols[i].has_state));
 
     cs_snprintf (buf, 50, "state%d", i);
-    parmlist.GetBool (buf, &(row->cols[i].state));
+    parmlist->GetBool (buf, &(row->cols[i].state));
 
     cs_snprintf (buf, 50, "groupstate%d", i);
-    parmlist.GetBool (buf, &(row->cols[i].group_state));
+    parmlist->GetBool (buf, &(row->cols[i].group_state));
 
     cs_snprintf (buf, 50, "aligntxt%d", i);
-    parmlist.GetInt (buf, &(row->cols[i].txt_align));
+    parmlist->GetInt (buf, &(row->cols[i].txt_align));
 
     cs_snprintf (buf, 50, "alignimg%d", i);
-    parmlist.GetInt (buf, &(row->cols[i].txt_align));
+    parmlist->GetInt (buf, &(row->cols[i].txt_align));
+
+    cs_snprintf (buf, 50, "param%d", i);
+    parmlist->GetInt (buf, &(row->cols[i].param));
   }
 
   // Add the item
   if (row->parent)
   {
-    if (row->parent->children == NULL)
+    if (row->parent->children == 0)
     {
       row->parent->children = new awsListRowVector ();
       row->parent->children->SetSortCol (lb->sortcol);
@@ -453,108 +497,189 @@ void awsListBox::InsertItem (void *owner, iAwsParmList &parmlist)
     lb->rows.Push (row);
 
   // Pass back the id of this row, in case they want it.
-  parmlist.AddInt ("id", (int)row);
+  parmlist->AddOpaque ("id", (intptr_t)row);
 
   lb->map_dirty = true;
 }
 
-void awsListBox::DeleteItem (void *owner, iAwsParmList &parmlist)
+void awsListBox::DeleteItem (intptr_t owner, iAwsParmList* parmlist)
 {
+  if (!parmlist)
+    return;
+  
   awsListBox *lb = (awsListBox *)owner;
 
-  int i;
-  iString *str = NULL;
+  int i, selidx = -1;
+  iString *str = 0;
 
   // Try and find out what they're searching for
-  if (!parmlist.GetString ("text", &str))
-    if (!parmlist.GetString ("id", &str)) return ;
+  if (!parmlist->GetString ("text", &str))
+    if (!parmlist->GetString ("id", &str)) return ;
+
+  if (lb->sel)
+    selidx = (int)lb->rows.Find (lb->sel);
 
   i = DoFindItem (&lb->rows, str, true);
+  if (i == selidx && selidx > -1)
+  {
+    int startidx=selidx;
+    while (selidx < (int)lb->rows.Length () &&
+	   !((awsListRow*)lb->rows[selidx])->selectable)
+      selidx++;
 
+    if (selidx >= (int)lb->rows.Length ())
+    {
+      selidx = MIN (startidx, (int)lb->rows.Length ()-1);
+      while (selidx >= 0 && !((awsListRow*)lb->rows[selidx])->selectable)
+        selidx--;
+    }
+
+    if (selidx > -1 && (size_t)selidx < lb->rows.Length ())
+    {
+      lb->sel = (awsListRow*) lb->rows[selidx];
+      lb->Broadcast (awsListBox::signalSelected);
+    }
+    else
+      lb->sel = 0;
+    lb->Invalidate ();
+  }
   // Pass back the result, in case they want it
-  parmlist.AddInt ("result", (int)i);
+  parmlist->AddInt ("result", (int)i);
 
   lb->map_dirty = true;
 }
 
-void awsListBox::GetSelectedItem (void *owner, iAwsParmList &parmlist)
+void awsListBox::GetSelectedItem (intptr_t owner, iAwsParmList* parmlist)
 {
+  if (!parmlist)
+    return;
+  
   awsListBox *lb = (awsListBox *)owner;
-  int i;
-  char buf[50];
-
-  //bool state[lb->ncolumns];
-  bool *state = new bool[lb->ncolumns];
-
-  //iString *str[lb->ncolumns];
-  iString **str = new iString *[lb->ncolumns];
-
-  //bool usedt[lb->ncolumns], useds[lb->ncolumns];
-  bool *usedt = new bool[lb->ncolumns];
-  bool *useds = new bool[lb->ncolumns];
-
-  for (i = 0; i < lb->ncolumns; ++i)
-  {
-    usedt[i] = false;
-    useds[i] = false;
-  }
-
-  // check if they want the text or state or what. then return those in the parmlist
-  for (i = 0; i < lb->ncolumns; ++i)
-  {
-    cs_snprintf (buf, 50, "text%d", i);
-    if (parmlist.GetString (buf, &str[i]))
-    {
-      str[i] = lb->sel->cols[i].text;
-      usedt[i] = true;
-    }
-
-    cs_snprintf (buf, 50, "state%d", i);
-    if (parmlist.GetBool (buf, &state[i]))
-    {
-      state[i] = lb->sel->cols[i].state;
-      useds[i] = true;
-    }
-  }
-
-  parmlist.Clear ();
-
-  // return parmlist
-  for (i = 0; i < lb->ncolumns; ++i)
-  {
-    if (usedt[i])
-    {
-      cs_snprintf (buf, 50, "text%d", i);
-      parmlist.AddString (buf, str[i]);
-    }
-
-    if (useds[i])
-    {
-      cs_snprintf (buf, 50, "state%d", i);
-      parmlist.AddBool (buf, state[i]);
-    }
-  }
-
-  delete state;
-  delete str;
-  delete useds;
-  delete usedt;
+  parmlist->AddBool ("success", lb->GetItems (lb->sel, parmlist));
 }
 
-void awsListBox::ClearList (void *owner, iAwsParmList &)
+void awsListBox::GetItem (intptr_t owner, iAwsParmList* parmlist)
+{
+  if (!parmlist)
+    return;
+  
+  awsListBox *lb = (awsListBox *)owner;
+  int row=-1;
+  if (parmlist->GetInt ("row", &row) && row >= -1 &&
+      (size_t)row < lb->rows.Length ())
+    parmlist->AddBool ("success", lb->GetItems ((awsListRow*)lb->rows[row],
+      parmlist));
+  else
+    parmlist->AddBool ("success", false);
+}
+
+bool awsListBox::GetItems (awsListRow *row, iAwsParmList* parmlist)
+{
+  if (!parmlist)
+    return false;
+  
+  bool succ = false;
+
+  if (row)
+  {
+    int i;
+    char buf[50];
+
+    bool *state = new bool[ncolumns];
+    int  *param = new int [ncolumns];
+
+    iString **str = new iString *[ncolumns];
+
+    bool *usedt = new bool[ncolumns];
+    bool *useds = new bool[ncolumns];
+    bool *usedp = new bool[ncolumns];
+
+    for (i = 0; i < ncolumns; ++i)
+    {
+      usedt[i] = false;
+      useds[i] = false;
+      usedp[i] = false;
+    }
+
+    // check if they want the text or state or what. then return those in the
+    // parmlist
+    for (i = 0; i < ncolumns; ++i)
+    {
+      cs_snprintf (buf, 50, "text%d", i);
+      if (parmlist->GetString (buf, &str[i]))
+      {
+        str[i] = row->cols[i].text;
+        usedt[i] = true;
+      }
+
+      cs_snprintf (buf, 50, "state%d", i);
+      if (parmlist->GetBool (buf, &state[i]))
+      {
+        state[i] = row->cols[i].state;
+        useds[i] = true;
+      }
+
+      cs_snprintf (buf, 50, "param%d", i);
+      if (parmlist->GetInt (buf, &param[i]))
+      {
+        param[i] = row->cols[i].param;
+        usedp[i] = true;
+      }
+    }
+
+    parmlist->Clear ();
+
+    // return parmlist
+    for (i = 0; i < ncolumns; ++i)
+    {
+      if (usedt[i])
+      {
+        cs_snprintf (buf, 50, "text%d", i);
+        parmlist->AddString (buf, str[i]->GetData());
+      }
+
+      if (useds[i])
+      {
+        cs_snprintf (buf, 50, "state%d", i);
+        parmlist->AddBool (buf, state[i]);
+      }
+
+      if (usedp[i])
+      {
+        cs_snprintf (buf, 50, "param%d", i);
+        parmlist->AddInt (buf, param[i]);
+      }
+    }
+
+    delete [] state;
+    delete [] str;
+    delete [] param;
+    delete [] useds;
+    delete [] usedt;
+    delete [] usedp;
+    succ = true;
+  }
+  return succ;
+}
+
+void awsListBox::ClearList (intptr_t owner, iAwsParmList* )
 {
   awsListBox *lb = (awsListBox *)owner;
 
   DoRecursiveClearList (&lb->rows);
 
+  // Clear the selected item.
+  lb->sel = 0;
+  lb->scroll_start = 0;
+
   lb->map_dirty = true;
 }
 
-bool awsListBox::Execute (char *action, iAwsParmList &parmlist)
+bool awsListBox::Execute (const char *action, iAwsParmList* parmlist)
 {
-  if (awsComponent::Execute (action, parmlist)) return true;
+  if (awsPanel::Execute (action, parmlist)) return true;
 
-  actions.Execute (action, this, parmlist);
+  actions->Execute (action, (intptr_t)this, parmlist);
 
   return false;
 }
@@ -563,14 +688,13 @@ void awsListBox::ClearGroup ()
 {
   csEvent Event;
 
-  Event.Type = csevGroupOff;
+  Event.Name = WindowManager()->GroupOff;
 
-  int i;
-  for (i = 0; i < Parent ()->GetChildCount (); ++i)
+  iAwsComponent* cmp = Parent()->GetTopChild();
+  while(cmp)
   {
-    iAwsComponent *cmp = Parent ()->GetChildAt (i);
-
     if (cmp && cmp != this) cmp->HandleEvent (Event);
+	cmp = cmp->ComponentBelow();
   }
 }
 
@@ -584,7 +708,7 @@ bool awsListBox::RecursiveClearPeers (awsListItem *itm, awsListRow *row)
     {
       if (row->parent)
       {
-        int j;
+        size_t j;
         for (j = 0; j < row->parent->children->Length (); ++j)
         {
           awsListRow *crow = (awsListRow *)row->parent->children->Get (j);
@@ -597,7 +721,7 @@ bool awsListBox::RecursiveClearPeers (awsListItem *itm, awsListRow *row)
     else if (row->children)
     {
       // Otherwise, recusively descend the tree.
-      int j;
+      size_t j;
 
       // Search through list for this guy, and clear his peers
       for (j = 0; j < row->children->Length (); ++j)
@@ -613,7 +737,7 @@ bool awsListBox::RecursiveClearPeers (awsListItem *itm, awsListRow *row)
 
 void awsListBox::ClearPeers (awsListItem *itm)
 {
-  int j;
+  size_t j;
 
   // Search through list for this guy, and clear his peers
   for (j = 0; j < rows.Length (); ++j)
@@ -625,23 +749,15 @@ void awsListBox::ClearPeers (awsListItem *itm)
 
 void awsListBox::ClearHotspots ()
 {
-  int i;
-  for (i = 0; i < hotspots.Length (); ++i)
-  {
-    awsListHotspot *hs = (awsListHotspot *)hotspots[i];
-    delete hs;
-  }
-
-  hotspots.SetLength (0);
+  hotspots.DeleteAll ();
 }
 
 bool awsListBox::HandleEvent (iEvent &Event)
 {
-  if (awsComponent::HandleEvent (Event)) return true;
+  if (awsPanel::HandleEvent (Event)) return true;
 
-  switch (Event.Type)
+  if (Event.Name == WindowManager ()->GroupOff)
   {
-    case csevGroupOff:
       if (is_down && is_switch)
       {
         is_down = false;
@@ -649,7 +765,6 @@ bool awsListBox::HandleEvent (iEvent &Event)
       }
 
       return true;
-      break;
   }
 
   return false;
@@ -673,7 +788,7 @@ int awsListBox::GetRowDepth (awsListRow *row)
 bool awsListBox::IsLastChild (awsListRow *row)
 {
   awsListRow *v = row->parent;
-  int i;
+  size_t i;
 
   if (!v)
   {
@@ -687,8 +802,11 @@ bool awsListBox::IsLastChild (awsListRow *row)
   }
 }
 
-void awsListBox::OnDraw (csRect /*clip*/)
+void awsListBox::OnDraw (csRect clip)
 {
+
+  awsPanel::OnDraw(clip);
+
   iGraphics2D *g2d = WindowManager ()->G2D ();
 
   //iGraphics3D *g3d = WindowManager()->G3D();
@@ -703,21 +821,9 @@ void awsListBox::OnDraw (csRect /*clip*/)
 
   ClearHotspots ();
 
-  aws3DFrame frame3d;
 
-  frame3d.Draw (
-      WindowManager (),
-      Window (),
-      Frame (),
-      frame_style,
-      bkg,
-      alpha_level);
-
-  switch (frame_style)
-  {
-    case fsBump:    border = 5; break;
-    case fsSimple:  border = 1; break;
-  }
+  if(style == fsBump) border = 5;
+  else if(style == fsSimple) border = 1;
 
   int starty = Frame ().ymin + border;
   int startx = Frame ().xmin + border;
@@ -798,7 +904,7 @@ void awsListBox::OnDraw (csRect /*clip*/)
 
     UpdateMap ();
 
-    if (map == NULL)
+    if (map == 0)
       start = (awsListRow *)rows[0];
     else
       start = map[scroll_start];
@@ -822,42 +928,43 @@ void awsListBox::OnDraw (csRect /*clip*/)
 
       draw_this_time = true;
 
-      // Find parent of current row.  If there is no parent, then go to the next row item.
+      // Find parent of current row.  If there is no parent, then go to the
+      // next row item.
       awsListRow *parent = row->parent;
 
-      if (parent == NULL)
+      if (parent == 0)
       { // Get next item from main list
-        int i = rows.Find (row);
+        int i = (int)rows.Find (row);
 
         // This should never occur
         if (i == -1)
         {
-          printf ("awslistbox: bug: couldn't find current row!\n");
+          csPrintf ("awslistbox: bug: couldn't find current row!\n");
           return ;
         }
         else
           ++i;
 
         // If we're done, leave
-        if (i >= rows.Length ())
+        if ((size_t)i >= rows.Length ())
           break;
         else
           row = (awsListRow *)rows[i];
       } // end if no parent.
       else
       {
-        int i = parent->children->Find (row);
+        int i = (int)parent->children->Find (row);
 
         // This should never occur
         if (i == -1)
         {
-          printf ("awslistbox: bug: couldn't find current row!\n");
+          csPrintf ("awslistbox: bug: couldn't find current row!\n");
           return ;
         }
         else
           ++i;
 
-        if (i >= parent->children->Length ())
+        if ((size_t)i >= parent->children->Length ())
         {
           row = parent;
           draw_this_time = false;
@@ -866,9 +973,11 @@ void awsListBox::OnDraw (csRect /*clip*/)
           row = (awsListRow *)parent->children->Get (i);
       }
     }   // end while draw items recursively
-    scrollbar->SetProperty ("PageSize", &drawable_count);
 
-  }     // end if there are any rows to draw
+    // Map to a float to set PageSize for scrollbar
+    float drawable_float=drawable_count;
+    scrollbar->SetProperty ("PageSize", (intptr_t)&drawable_float);
+  } // end if there are any rows to draw
 }
 
 bool awsListBox::DrawItemsRecursively (
@@ -947,7 +1056,7 @@ bool awsListBox::DrawItemsRecursively (
     // Create hot spot for expand/collapse
     awsListHotspot *hs = new awsListHotspot;
 
-    hs->obj = row;
+    hs->obj = (intptr_t)row;
     hs->type = hsTreeBox;
     hs->r.Set (
         x + 2 + (tbw * (depth + 1)),
@@ -975,7 +1084,7 @@ bool awsListBox::DrawItemsRecursively (
     // Create hot spot for expand/collapse
     awsListHotspot *hs = new awsListHotspot;
 
-    hs->obj = row;
+    hs->obj = (intptr_t)row;
     hs->type = hsTreeBox;
     hs->r.Set (x + 2, y, x + 2 + tbw, y + tbh);
 
@@ -1023,7 +1132,7 @@ bool awsListBox::DrawItemsRecursively (
   // Draw columns
   for (i = 0; i < ncolumns; ++i)
   {
-    int tw = 0, th = 0, tx = 0, ty = 0, mcc;  // text width, height, position, max len.
+    int tw = 0, th = 0, tx = 0, ty = 0, mcc; // text width, hght, pos, max len.
     int cw;                 // column width
     int iw = 0, ih = 0;     // stateful image width and height
     int iws = 0;            // stateful image spacer
@@ -1191,7 +1300,7 @@ bool awsListBox::DrawItemsRecursively (
       // Create hot spot for expand/collapse
       awsListHotspot *hs = new awsListHotspot;
 
-      hs->obj = &(row->cols[i]);
+      hs->obj = (intptr_t)&(row->cols[i]);
       hs->type = hsState;
       hs->r.Set (x + tx, y, x + tx + iw, y + ih);
 
@@ -1205,7 +1314,7 @@ bool awsListBox::DrawItemsRecursively (
   // Create hot spot for this row
   awsListHotspot *hs = new awsListHotspot;
 
-  hs->obj = row;
+  hs->obj = (intptr_t)row;
   hs->type = hsRow;
   hs->r.Set (Frame ().xmin + border, y, Frame ().xmax - border, y + ith);
 
@@ -1239,10 +1348,10 @@ bool awsListBox::DrawItemsRecursively (
   // Draw children
   if (row->children && row->expanded)
   {
-    for (i = 0; i < row->children->Length (); ++i)
+    for (size_t j = 0; j < row->children->Length (); ++j)
     {
       int cx = orgx;
-      awsListRow *newrow = (awsListRow *)row->children->Get (i);
+      awsListRow *newrow = (awsListRow *)row->children->Get (j);
 
       if (
         DrawItemsRecursively (
@@ -1251,7 +1360,7 @@ bool awsListBox::DrawItemsRecursively (
             y,
             border,
             (depth ? depth + 2 : depth + 1),
-            (i == row->children->Length () - 1 ? true : false)))
+            ((size_t)i == row->children->Length () - 1 ? true : false)))
         return true;
     }
   }
@@ -1264,11 +1373,11 @@ bool awsListBox::OnMouseDown (int
 
 /*button*/, int x, int y)
 {
-  int i;
+  size_t i;
 
   for (i = 0; i < hotspots.Length (); ++i)
   {
-    awsListHotspot *hs = (awsListHotspot *)hotspots[i];
+    awsListHotspot *hs = hotspots[i];
     if (hs->r.Contains (x, y))
     {
       switch (hs->type)
@@ -1298,6 +1407,7 @@ bool awsListBox::OnMouseDown (int
             else
               itm->state = true;
 
+            Broadcast (awsListBox::signalStateChanged);
             Invalidate ();
             return true;
           }
@@ -1306,9 +1416,12 @@ bool awsListBox::OnMouseDown (int
         case hsRow:
           {
             awsListRow *row = (awsListRow *)hs->obj;
-            sel = row;
-            Broadcast (awsListBox::signalSelected);
-            Invalidate ();
+            if (row->selectable)
+            {
+              sel = row;
+              Broadcast (awsListBox::signalSelected);
+              Invalidate ();
+            }
             return true;
           }
           break;
@@ -1318,24 +1431,94 @@ bool awsListBox::OnMouseDown (int
   return false;
 }
 
-bool awsListBox::OnMouseUp (int, int, int)
+bool awsListBox::OnKeyboard (const csKeyEventData& eventData)
 {
-  return false;
+  ///Changing selected row with arrow keys
+  /// !!! Worked correctly only for awsListHotspot.type == hsRow
+  if( !(WindowManager()->GetFlags() & AWSF_KeyboardControl) )
+    return false;
+
+  switch (eventData.codeCooked)
+  {
+    case CSKEY_DOWN:
+    {
+      awsListRow *parent = sel ? sel->parent: 0;
+
+      if (parent == 0)
+      { 
+	int i = (int)rows.Find (sel);
+	
+	if(i < (int)rows.Length() -1 && rows.Length() > 0)
+	{
+	  ++i;
+	  sel = (awsListRow *)rows[i];
+	  Broadcast (awsListBox::signalSelected);
+
+	  /// @@@ HACK ;-]
+
+	  /// Change scroll bar position if selected component 
+	  /// is in the bottom of our list box
+
+	  awsListRow *row = 0;
+	  UpdateMap ();
+	  if (map)
+	    row = map[drawable_count + scroll_start];
+
+	  if (sel == row)
+	    awsScrollBar::IncClicked (0, (intptr_t)scrollbar, 0);
+
+	  ///@@@ END_HACK
+
+	  return true;
+	}
+      }
+    }
+
+    return true;
+
+    case CSKEY_UP:
+    {
+      awsListRow *parent = sel ? sel->parent: 0;
+
+      if (parent == 0)
+      { 
+	int i = (int)rows.Find (sel);
+
+	if(i > 0 && rows.Length () > 0)
+	{
+	  --i;
+	  sel = (awsListRow *)rows[i];
+	  Broadcast (awsListBox::signalSelected);
+
+	  /// @@@ HACK ;-]
+
+	  /// Change scroll bar position if selected component 
+	  /// is in the top of our list box
+
+	  awsListRow *row = 0;
+	  UpdateMap ();
+	  if (map)
+		  row = map[scroll_start -1];
+
+	  if (sel == row)
+		  awsScrollBar::DecClicked (0, (intptr_t)scrollbar, 0);
+
+	  ///@@@ END_HACK
+
+	  return true;
+	}
+      }
+    }
+  }
+
+  Invalidate ();
+
+  return true;
 }
 
-bool awsListBox::OnMouseMove (int, int, int)
+void awsListBox::OnSetFocus ()
 {
-  return false;
-}
-
-bool awsListBox::OnMouseClick (int, int, int)
-{
-  return false;
-}
-
-bool awsListBox::OnMouseDoubleClick (int, int, int)
-{
-  return false;
+	Broadcast (signalFocused);
 }
 
 bool awsListBox::OnMouseExit ()
@@ -1355,38 +1538,23 @@ bool awsListBox::OnMouseEnter ()
   return true;
 }
 
-bool awsListBox::OnKeypress (int, int)
-{
-  return false;
-}
-
-bool awsListBox::OnLostFocus ()
-{
-  return false;
-}
-
-bool awsListBox::OnGainFocus ()
-{
-  return false;
-}
-
 void awsListBox::OnAdded ()
 {
-  AddChild (scrollbar, (Parent ()->Layout () != NULL));
+  AddChild (scrollbar);
 }
 
 void awsListBox::OnResized ()
 {
   int w = scrollbar->Frame ().Width (); //, h = scrollbar->Frame ().Width ();
 
-  scrollbar->Frame ().SetPos (Frame ().xmax - w - 1, Frame ().ymin + 2);
-  scrollbar->Frame ().xmax = Frame ().xmax - 1;
-  scrollbar->Frame ().ymax = Frame ().ymax - 2;
-
-  scrollbar->OnResized ();
+  csRect newFrame;
+  newFrame.SetPos(Frame ().xmax - w - 1, Frame ().ymin + 2);
+  newFrame.xmax = Frame ().xmax - 1;
+  newFrame.ymax = Frame ().ymax - 2;
+  scrollbar->ResizeTo(newFrame);
 }
 
-/************************************* Command Button Factory ****************/
+/************************************* List Box Factory ****************/
 
 awsListBoxFactory::awsListBoxFactory (iAws *wmgr) :
   awsComponentFactory(wmgr)
@@ -1406,10 +1574,12 @@ awsListBoxFactory::awsListBoxFactory (iAws *wmgr) :
   RegisterConstant ("lbAlignRight", alignRight);
   RegisterConstant ("lbAlignCenter", alignCenter);
 
-  RegisterConstant (
-    "signalListBoxSelectionChanged",
-    awsListBox::signalSelected);
+  RegisterConstant ("signalListBoxSelectionChanged",
+		    awsListBox::signalSelected);
   RegisterConstant ("signalListBoxScrolled", awsListBox::signalScrolled);
+  RegisterConstant ("signalListBoxFocused", awsListBox::signalFocused);
+  RegisterConstant ("signalListBoxStateChanged",
+		    awsListBox::signalStateChanged);
 }
 
 awsListBoxFactory::~awsListBoxFactory ()
