@@ -132,7 +132,7 @@ struct awsListColumn
   iTextureHandle *bkg;
 
   /// The caption of the header, if it has one.
-  iString *caption;
+  csRef<iString> caption;
 
   /// Alignment.
   int align;
@@ -150,7 +150,7 @@ struct awsListHotspot
   csRect r;
 
   /// The owner of the hotspot (either an item or row).
-  void *obj;
+  intptr_t obj;
 
   /// The type of hotspot (tree box, stateful item, etc.).
   int type;
@@ -174,9 +174,6 @@ private:
   /// True if button was down, and button is in switch mode (toggle = yes).
   bool was_down;
 
-  /// Holds the background texture (either the global one, or an overridden one.
-  //iTextureHandle *bkg;
-
   /// Holds the highlight overlay texture.
   iTextureHandle *highlight;
 
@@ -204,12 +201,6 @@ private:
   /// Image of radio button type item (marked).
   iTextureHandle *tree_grpf;
 
-  /// Flags for frame style.
-  //int frame_style;
-
-  /// Alpha level for this component.
-  //int alpha_level;
-
   /// Alpha level for highlight bitmap.
   int hi_alpha_level;
 
@@ -258,26 +249,26 @@ private:
   /// Our embedded scrollbar.
   awsScrollBar *scrollbar;
 
-  /// Trigger for catching scroll signals.
-  static void ScrollChanged (void *sk, iAwsSource *source);
-
   /// Action dispatcher.
-  awsActionDispatcher actions;
+  awsActionDispatcher* actions;
+
+  /// Trigger for catching scroll signals.
+  static void ScrollChanged (unsigned long, intptr_t sk, iAwsSource *source);
 
   /// Inserts an item.
-  static void InsertItem (void *owner, iAwsParmList* parmlist);
+  static void InsertItem (intptr_t owner, iAwsParmList* parmlist);
 
   /// Deletes an item.
-  static void DeleteItem (void *owner, iAwsParmList* parmlist);
+  static void DeleteItem (intptr_t owner, iAwsParmList* parmlist);
 
   /// Get the selected item.
-  static void GetSelectedItem (void *owner, iAwsParmList* parmlist);
+  static void GetSelectedItem (intptr_t owner, iAwsParmList* parmlist);
 
   /// Get an item by row number.
-  static void GetItem (void *owner, iAwsParmList* parmlist);
+  static void GetItem (intptr_t owner, iAwsParmList* parmlist);
 
   /// Clears the entire list.
-  static void ClearList (void *owner, iAwsParmList* parmlist);
+  static void ClearList (intptr_t owner, iAwsParmList* parmlist);
 
   /**
    * Counts the number of visible items recursively, given the starting vector.
@@ -351,10 +342,10 @@ public:
   virtual bool Setup (iAws *wmgr, iAwsComponentNode *settings);
 
   /// Get properties.
-  bool GetProperty (const char *name, void **parm);
+  bool GetProperty (const char *name, intptr_t *parm);
 
   /// Set properties.
-  bool SetProperty (const char *name, void *parm);
+  bool SetProperty (const char *name, intptr_t parm);
 
   /// Executes some actions.
   bool Execute (const char *action, iAwsParmList* parmlist);
