@@ -1,17 +1,35 @@
-#include "cssysdef.h"
-#include "awsntbk.h"
-#include "aws3dfrm.h"
-#include "awskcfct.h"
-#include "awsslot.h"
-#include "awsscrbr.h"
-#include "awsfparm.h"
+/*
+    Copyright (C) 2002 by Norman Kraemer
+  
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+  
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+  
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
 
+#include "cssysdef.h"
+#include "aws3dfrm.h"
+#include "awsfparm.h"
+#include "awskcfct.h"
+#include "awsntbk.h"
+#include "awsscrbr.h"
+#include "awsslot.h"
+
+#include "csutil/csevent.h"
+#include "csutil/scfstr.h"
+#include "iutil/evdefs.h"
+#include "ivideo/fontserv.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
-#include "ivideo/fontserv.h"
-#include "csutil/scfstr.h"
-#include "csutil/csevent.h"
-#include "iutil/evdefs.h"
 
 #include <stdio.h>
 
@@ -32,14 +50,15 @@ awsNotebook::awsNotebook () :
   bb_location(nbTop),
   bb_style(nbSlide),
   maxheight (0),
-  sink()
+  sink(0)
 {
-  sink.SetParm (this);
   SetFlag (AWSF_CMP_ALWAYSERASE);
 }
 
 awsNotebook::~awsNotebook ()
-{}
+{
+  delete sink;
+}
 
 const char *awsNotebook::Type ()
 {
@@ -50,15 +69,18 @@ bool awsNotebook::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
   if (!awsPanel::Setup (_wmgr, settings)) return false;
 
-  iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
+  iAws* const w = WindowManager();
+  iAwsPrefManager *pm = w->GetPrefMgr ();
 
   pm->GetInt (settings, "Location", bb_location);
   pm->GetInt (settings, "Mode", bb_style);
 
-  sink.RegisterTrigger("ActivateTab", &OnActivateTab);
-  sink.RegisterTrigger("DeactivateTab", &OnDeactivateTab);
+  sink = new awsSink(w);
+  sink->SetParm ((intptr_t)this);
+  sink->RegisterTrigger("ActivateTab", &OnActivateTab);
+  sink->RegisterTrigger("DeactivateTab", &OnDeactivateTab);
 
-  awsKeyFactory info;
+  awsKeyFactory info(w);
 
   info.Initialize ("ButtonBar", "Notebook ButtonBar");
 
@@ -73,22 +95,22 @@ bool awsNotebook::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   return true;
 }
 
-bool awsNotebook::GetProperty (const char *name, void **parm)
+bool awsNotebook::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsPanel::GetProperty (name, parm)) return true;
 
   if (strcmp ("Location", name) == 0)
-    *parm = (void *) &bb_location;
+    *parm = (intptr_t)&bb_location;
   else
   if (strcmp ("Mode", name) == 0)
-    *parm = (void *) &bb_style;
+    *parm = (intptr_t)&bb_style;
   else
     return false;
 
   return true;
 }
 
-bool awsNotebook::SetProperty (const char *name, void *parm)
+bool awsNotebook::SetProperty (const char *name, intptr_t parm)
 {
   if (awsPanel::SetProperty (name, parm)) return true;
 
@@ -137,7 +159,7 @@ bool awsNotebook::Execute (const char* action, iAwsParmList* parmlist)
     iAwsComponent* comp = FindChild (comp_name->GetData ());
     if (!comp) return false;
 
-    tab_ctrl.ActivateTab((void*) comp);
+    tab_ctrl.ActivateTabParam ((intptr_t)comp);
 
     return true;
   }
@@ -145,7 +167,7 @@ bool awsNotebook::Execute (const char* action, iAwsParmList* parmlist)
   return awsComponent::Execute (action, parmlist);
 }
 
-void awsNotebook::OnDraw (csRect r)
+void awsNotebook::OnDraw (csRect /*r*/)
 {
   csRect f (Frame ());
   if (bb_location == nbTop)
@@ -168,26 +190,28 @@ void awsNotebook::AddChild (iAwsComponent *child)
   awsComponent::AddChild (child);
 
   iString *str = 0;
-  child->GetProperty ("Caption", (void**)&str);
-  iAwsSource* src = tab_ctrl.AddTab (str, (void*)child);
+  child->GetProperty ("Caption", (intptr_t*)&str);
+  iAwsSource* src = tab_ctrl.AddTab (str, (intptr_t)child);
 
   // hook up the source so we receive the signals
-  slot.Connect(src, awsTab::signalActivateTab, &sink, sink.GetTriggerID("ActivateTab"));
-  slot.Connect(src, awsTab::signalDeactivateTab, &sink, sink.GetTriggerID("DeactivateTab"));
+  slot.Connect(src, awsTab::signalActivateTab,
+    sink, sink->GetTriggerID("ActivateTab"));
+  slot.Connect(src, awsTab::signalDeactivateTab,
+    sink, sink->GetTriggerID("DeactivateTab"));
 }
 
-void awsNotebook::OnActivateTab(void* param, iAwsSource* src)
+void awsNotebook::OnActivateTab(unsigned long, intptr_t param, iAwsSource* src)
 {
   iAwsComponent* child;
-  src->GetComponent()->GetProperty("User Param", (void**)&child);
+  src->GetComponent()->GetProperty("User Param", (intptr_t*)&child);
   child->Show();
   ((awsNotebook*)param)->Invalidate();
 }
 
-void awsNotebook::OnDeactivateTab(void* param, iAwsSource* src)
+void awsNotebook::OnDeactivateTab(unsigned long, intptr_t param, iAwsSource* src)
 {
   iAwsComponent* child;
-  src->GetComponent()->GetProperty("User Param", (void**)&child);
+  src->GetComponent()->GetProperty("User Param", (intptr_t*)&child);
   child->Hide();
   ((awsNotebook*)param)->Invalidate();
 }
@@ -218,12 +242,10 @@ iAwsComponent *awsNotebookFactory::Create ()
 }
 
 
-/********************************** Notebook Page ********************************/
+/****************************** Notebook Page ********************************/
 
 awsNotebookPage::awsNotebookPage ():
   tex(0),
-  caption (0),
-  icon (0),
   iconalign(0)
 {
   SetFlag (AWSF_CMP_ALWAYSERASE);
@@ -231,7 +253,7 @@ awsNotebookPage::awsNotebookPage ():
 
 awsNotebookPage::~awsNotebookPage ()
 {
-  if (caption) caption->DecRef ();
+  //??if (caption) caption->DecRef ();
 }
 
 bool awsNotebookPage::Setup (iAws *_wmgr, iAwsComponentNode *settings)
@@ -241,36 +263,38 @@ bool awsNotebookPage::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
 
   tex = pm->GetTexture ("Texture");
+  caption.AttachNew (new scfString (""));
   pm->GetString (settings, "Caption", caption);
+  icon.AttachNew (new scfString (""));
   pm->GetString (settings, "Icon", icon);
   pm->GetInt (settings, "IconAlign", iconalign);
 
   return true;
 }
 
-bool awsNotebookPage::GetProperty (const char *name, void **parm)
+bool awsNotebookPage::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
 
     if (caption) st = caption->GetData ();
 
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else
   if (strcmp ("Icon", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
 
     if (icon) st = icon->GetData ();
 
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else
@@ -284,7 +308,7 @@ bool awsNotebookPage::GetProperty (const char *name, void **parm)
   return false;
 }
 
-bool awsNotebookPage::SetProperty (const char *name, void *parm)
+bool awsNotebookPage::SetProperty (const char *name, intptr_t parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
   
@@ -292,10 +316,8 @@ bool awsNotebookPage::SetProperty (const char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
-    if (caption) caption->DecRef ();
-
     if (s && s->Length ())
-      (caption = s)->IncRef ();
+      caption = s;
     else
       caption = 0;
     Invalidate ();
@@ -307,10 +329,8 @@ bool awsNotebookPage::SetProperty (const char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
-    if (icon) icon->DecRef ();
-
     if (s && s->Length ())
-      (icon = s)->IncRef ();
+      icon = s;
     else
       icon = 0;
 
@@ -330,9 +350,8 @@ const char *awsNotebookPage::Type ()
   return "Notebook Page";
 }
 
-awsNotebookPageFactory::awsNotebookPageFactory (
-  iAws *wmgr) :
-    awsComponentFactory(wmgr)
+awsNotebookPageFactory::awsNotebookPageFactory (iAws *wmgr) :
+  awsComponentFactory(wmgr)
 {
   Register ("Notebook Page");
 
@@ -353,7 +372,6 @@ awsNotebookButton::awsNotebookButton ():
   is_active(false),
   is_first(false),
   is_top(true),
-  caption (0),
   captured(false),
   icon_align(0),
   alpha_level(92)
@@ -372,20 +390,21 @@ bool awsNotebookButton::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
 
   pm->LookupIntKey ("OverlayTextureAlpha", alpha_level);  // global get
-  pm->GetInt (settings, "Alpha", alpha_level);            // local overrides, if present.
+  pm->GetInt (settings, "Alpha", alpha_level); // local overrides, if present.
   pm->GetInt (settings, "IconAlign", icon_align);
+  caption.AttachNew (new scfString (""));
   pm->GetString (settings, "Caption", caption);
 
-  iString *tn = 0;
-
   tex[0] = pm->GetTexture ("Texture");
-  pm->GetString (settings, "Image", tn);
+  csRef<iString> tn;
+  tn.AttachNew (new scfString (""));
+  if (pm->GetString (settings, "Image", tn))
+    tex[1] = pm->GetTexture (tn->GetData (), tn->GetData ());
 
-  if (tn) tex[1] = pm->GetTexture (tn->GetData (), tn->GetData ());
-
-  iString *in = 0;
-  pm->GetString (settings, "Icon", in);
-  if (in) tex[2] = pm->GetTexture (in->GetData (), in->GetData ());
+  csRef<iString> in;
+  in.AttachNew (new scfString (""));
+  if (pm->GetString (settings, "Icon", in))
+    tex[2] = pm->GetTexture (in->GetData (), in->GetData ());
 
   return true;
 }
@@ -427,7 +446,8 @@ void awsNotebookButton::OnDraw (csRect)
 
   if (is_active)
   {
-    g2d->DrawLine (r.xmin + 1, is_top ? r.ymin : r.ymax, r.xmax-1, is_top ? r.ymin : r.ymax, hi);
+    g2d->DrawLine (r.xmin + 1, is_top ? r.ymin : r.ymax, r.xmax-1,
+		   is_top ? r.ymin : r.ymax, hi);
     g2d->DrawLine (r.xmin, r.ymin + 1, r.xmin, r.ymax, hi);
     g2d->DrawLine (r.xmax-1, r.ymin + 1, r.xmax-1, r.ymax, lo);
     g2d->DrawLine (r.xmax, r.ymin + 1, r.xmax, r.ymax, lo2);
@@ -435,22 +455,23 @@ void awsNotebookButton::OnDraw (csRect)
   else
   {
     g2d->DrawLine (r.xmin, r.ymin + 1, r.xmin, r.ymax, is_first ? hi2 : lo);
-    g2d->DrawLine (r.xmin + 1, is_top ? r.ymin : r.ymax, r.xmax, is_top ? r.ymin : r.ymax, hi2);
+    g2d->DrawLine (r.xmin + 1, is_top ? r.ymin : r.ymax, r.xmax,
+		   is_top ? r.ymin : r.ymax, hi2);
     g2d->DrawLine (r.xmax, r.ymin + 1, r.xmax, r.ymax, lo);
   }
 
-  g2d->DrawBox (r.xmin + 1, r.ymin + 1, r.Width () - 1, r.Height () - 1, is_active ? fill : dfill);
+  g2d->DrawBox (r.xmin + 1, r.ymin + 1, r.Width () - 1, r.Height () - 1,
+		is_active ? fill : dfill);
 
   if (tex[0])
     g3d->DrawPixmap (tex[0], r.xmin+1, r.ymin+1, r.Width ()-1, r.Height ()-1,
-                     r.xmin+1, r.ymin+1, r.Width ()-1, r.Height ()-1, alpha_level);
+                     r.xmin+1, r.ymin+1, r.Width ()-1, r.Height ()-1,
+		     alpha_level);
 
   if (tex[1])
   {
     int img_w, img_h;
-
     tex[1]->GetOriginalDimensions (img_w, img_h);
-
     g3d->DrawPixmap (tex[1], r.xmin+1, r.ymin+1, r.Width ()-1, r.Height ()-1,
                      0, 0, img_w, img_h, 0);
   }
@@ -511,33 +532,31 @@ void awsNotebookButton::OnDraw (csRect)
   // Draw the caption, if there is one and the style permits it.
   if (caption)
   {
-
     // Draw the text
     g2d->Write (pm->GetDefaultFont (), r.xmin + tx, r.ymin + ty,
                 pm->GetColor (AC_TEXTFORE), -1, caption->GetData ());
 
   }
-
 }
 
-bool awsNotebookButton::GetProperty (const char *name, void **parm)
+bool awsNotebookButton::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
 
     if (caption) st = caption->GetData ();
 
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   return false;
 }
 
-bool awsNotebookButton::SetProperty (const char *name, void *parm)
+bool awsNotebookButton::SetProperty (const char *name, intptr_t parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
   
@@ -545,10 +564,8 @@ bool awsNotebookButton::SetProperty (const char *name, void *parm)
   {
     iString *s = (iString *) (parm);
 
-    if (caption) caption->DecRef ();
-
     if (s && s->Length ())
-      (caption = s)->IncRef ();
+      caption = s;
     else
       caption = 0;
     Invalidate ();
@@ -691,12 +708,13 @@ awsNotebookButtonBar::~awsNotebookButtonBar ()
 bool awsNotebookButtonBar::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
   if (!awsComponent::Setup (_wmgr, settings)) return false;
+  iAws* const w = WindowManager();
 
   // set up the next/prev buttons and hide them initially
   next = new awsSliderButton;
   prev = new awsSliderButton;
 
-  awsKeyFactory previnfo, nextinfo;
+  awsKeyFactory previnfo(w), nextinfo(w);
 
   previnfo.Initialize ("prev", "Slider Button");
   nextinfo.Initialize ("next", "Slider Button");
@@ -723,11 +741,11 @@ bool awsNotebookButtonBar::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   prev->Setup (_wmgr, previnfo.GetThisNode ());
   next->Setup (_wmgr, nextinfo.GetThisNode ());
 
-  prev->SetProperty ("Image", previmg);
-  next->SetProperty ("Image", nextimg);
+  prev->SetProperty ("Image", (intptr_t)previmg);
+  next->SetProperty ("Image", (intptr_t)nextimg);
 
-  awsSink* _sink = new awsSink ();
-  _sink->SetParm (this);
+  awsSink* _sink = new awsSink (w);
+  _sink->SetParm ((intptr_t)this);
   sink = _sink;
 
   sink->RegisterTrigger ("Prev", &PrevClicked);
@@ -763,7 +781,8 @@ void awsNotebookButtonBar::SetTopBottom (bool to_top)
 
 void awsNotebookButtonBar::DoLayout ()
 {
-  int i, x=0;
+  size_t i;
+  int x=0;
   csRect r = Frame ();
   csRect cr = Parent ()->Frame ();
 
@@ -787,8 +806,10 @@ void awsNotebookButtonBar::DoLayout ()
 
   ResizeTo(r);
 
-  for (i=first-1; i >=0 ; i--)
+  i = first;
+  while (i > 0)
   {
+    i--;
     awsNotebookButton *btn = vTabs.Get (i)->button;
     csRect br =  btn->Frame ();
     btn->Hide ();
@@ -853,12 +874,12 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
   // determine caption of tab
   iString *str = 0;
 
-  comp->GetProperty ("Caption", (void**)&str);
+  comp->GetProperty ("Caption", (intptr_t*)&str);
   if (!str || !str->GetData ())
   {
     if (str) str->DecRef ();
     csString theCap ("Tab ");
-    theCap += vTabs.Length ()+1;
+    theCap += (uint)vTabs.Length ()+1;
     str = new scfString ((const char*)theCap);
   }
 
@@ -866,27 +887,28 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
   awsNotebookButton *btn = new awsNotebookButton;
 
   // initialize and setup the button
-  awsKeyFactory btninfo;
+  awsKeyFactory btninfo(WindowManager());
 
   btninfo.Initialize (str->GetData(), "Notebook Button");
-  btninfo.AddRectKey ("Frame", csRect (0, 0, Frame ().Width (), Frame ().Height ()));
+  btninfo.AddRectKey (
+    "Frame", csRect (0, 0, Frame ().Width (), Frame ().Height ()));
 
   iString *icon = 0;
-  if (comp->GetProperty ("Icon", (void**)&icon) && icon && icon->Length ())
+  if (comp->GetProperty ("Icon", (intptr_t*)&icon) && icon && icon->Length ())
   {
     btninfo.AddStringKey ("Icon", icon ? icon->GetData() : "");
     int *iconalign;
-    if (comp->GetProperty ("IconAlign", (void**)&iconalign))
+    if (comp->GetProperty ("IconAlign", (intptr_t*)&iconalign))
       btninfo.AddIntKey ("IconAlign", *iconalign);
   }
 
   btn->SetParent (this);
   btn->Setup (WindowManager (), btninfo.GetThisNode ());
-  btn->SetProperty ("Caption", str);
+  btn->SetProperty ("Caption", (intptr_t)str);
 
   // resize button
   csRect r(btn->getPreferredSize ());
-  int last = vTabs.Length ();
+  int last = (int)vTabs.Length ();
   if (r.Height () > Frame ().Height())
   {
     int delta = r.Height () - Frame ().Height ();
@@ -916,7 +938,8 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
 
   // connect myself with button to keep informed about state changes
   awsSlot *slot = new awsSlot;
-  slot->Connect (btn, awsNotebookButton::signalActivateTab, sink, sink->GetTriggerID ("ActivateTab"));
+  slot->Connect (btn, awsNotebookButton::signalActivateTab,
+    sink, sink->GetTriggerID ("ActivateTab"));
   vTabs.Push (btn, slot, comp, sink);
   //csRect newFrame(Frame());
   //newFrame.ymin = newFrame.ymax+1;
@@ -930,24 +953,25 @@ bool awsNotebookButtonBar::Add (iAwsComponent *comp)
 
 bool awsNotebookButtonBar::Remove (iAwsComponent *comp)
 {
-  int idx = vTabs.Find ((awsNotebookButtonBar::tabEntry*)comp);
+  size_t idx = vTabs.Find ((awsNotebookButtonBar::tabEntry*)comp);
 
-  if (idx!=-1)
+  if (idx != csArrayItemNotFound)
   {
-    if (idx == active)
-      if (vTabs.Length () -1 == active)
+    if (idx == (size_t)active)
+      if (vTabs.Length () - 1 == (size_t)active)
         Activate (active-1);
       else
         Activate (active+1);
 
     vTabs.Get (first)->button->SetFirst (false);
-    if ((idx < first) || (idx == first && (first > 0 || vTabs.Length () < 2)))
+    if ((idx < (size_t)first) || (idx == (size_t)first && 
+      (first > 0 || vTabs.Length () < 2)))
       first--;
 
     if (first > -1)
       vTabs.Get (first)->button->SetFirst (true);
 
-    if (idx < active)
+    if (idx < (size_t)active)
       active--;
 
     vTabs.FreeItem (vTabs[idx]);
@@ -994,11 +1018,11 @@ void awsNotebookButtonBar::OnDraw (csRect)
 
 void awsNotebookButtonBar::ScrollLeft ()
 {
-  if (vTabs.Length () && first != vTabs.Length ()-1)
+  if (vTabs.Length () && (size_t)first != vTabs.Length ()-1)
   {
     int xdelta = vTabs.Get (first)->button->Frame ().Width ()+1;
     vTabs.Get (first)->button->SetFirst (false);
-    for (int i=0; i < vTabs.Length (); i++)
+    for (size_t i = 0; i < vTabs.Length (); i++)
     {
       awsNotebookButton *btn = vTabs.Get (i)->button;
       csRect f = btn->Frame ();
@@ -1019,7 +1043,7 @@ void awsNotebookButtonBar::ScrollRight ()
   {
     int xdelta = vTabs.Get (first-1)->button->Frame ().Width ()+1;
     vTabs.Get (first)->button->SetFirst (false);
-    for (int i=0; i < vTabs.Length (); i++)
+    for (size_t i=0; i < vTabs.Length (); i++)
     {
       awsNotebookButton *btn = vTabs.Get (i)->button;
       csRect f = btn->Frame ();
@@ -1070,10 +1094,11 @@ void awsNotebookButtonBar::Activate (int idx)
   active = idx;
 }
 
-void awsNotebookButtonBar::ActivateTab (void *sk, iAwsSource *source)
+void awsNotebookButtonBar::ActivateTab (unsigned long,
+    intptr_t sk, iAwsSource *source)
 {
   awsNotebookButtonBar *bb = (awsNotebookButtonBar *)sk;
-  int idx = bb->vTabs.FindKey (
+  int idx = (int)bb->vTabs.FindKey (
     bb->vTabs.KeyButtonFunctor((awsNotebookButton*)source->GetComponent ()));
   if (idx != -1 && bb->active != idx)
   {
@@ -1083,13 +1108,15 @@ void awsNotebookButtonBar::ActivateTab (void *sk, iAwsSource *source)
   }
 }
 
-void awsNotebookButtonBar::PrevClicked (void *sk, iAwsSource *)
+void awsNotebookButtonBar::PrevClicked (unsigned long,
+    intptr_t sk, iAwsSource *)
 {
   awsNotebookButtonBar *bb = (awsNotebookButtonBar *)sk;
   bb->ScrollRight ();
 }
 
-void awsNotebookButtonBar::NextClicked (void *sk, iAwsSource *)
+void awsNotebookButtonBar::NextClicked (unsigned long,
+    intptr_t sk, iAwsSource *)
 {
   awsNotebookButtonBar *bb = (awsNotebookButtonBar *)sk;
   bb->ScrollLeft ();
