@@ -43,7 +43,7 @@ protected:
   bool is_top;
 
   /// Caption.
-  iString *caption;
+  csRef<iString> caption;
 
   /// Is the mouse currently held doen?
   bool captured;
@@ -64,8 +64,8 @@ public:
   virtual ~awsNotebookButton ();
 
   virtual bool Setup (iAws *_wmgr, iAwsComponentNode *settings);
-  virtual bool GetProperty (const char *name, void **parm);
-  virtual bool SetProperty (const char *name, void *parm);
+  virtual bool GetProperty (const char *name, intptr_t *parm);
+  virtual bool SetProperty (const char *name, intptr_t parm);
 
   virtual void OnDraw (csRect clip);
   bool OnMouseDown (int, int, int);
@@ -145,12 +145,12 @@ protected:
       te->comp = comp;
       te->sink = sink;
       sink->IncRef ();
-      return csPDelArray<tabEntry>::Push (te);
+      return (int)csPDelArray<tabEntry>::Push (te);
     }
 
     void FreeAll ()
     {
-      int i;
+      size_t i;
       for (i = 0 ; i < Length () ; i++)
         FreeItem (Get (i));
       DeleteAll ();
@@ -259,9 +259,9 @@ public:
   /// Activate the <idx>-th tab.
   void Activate (int idx);
 
-  static void ActivateTab (void *sk, iAwsSource *source);
-  static void PrevClicked (void *sk, iAwsSource *source);
-  static void NextClicked (void *sk, iAwsSource *source);
+  static void ActivateTab (unsigned long, intptr_t sk, iAwsSource *source);
+  static void PrevClicked (unsigned long, intptr_t sk, iAwsSource *source);
+  static void NextClicked (unsigned long, intptr_t sk, iAwsSource *source);
 
   static const int HandleSize;
 
@@ -286,8 +286,8 @@ protected:
   iTextureHandle *tex;
 
   /// Caption we show in the tab button.
-  iString *caption;
-  iString *icon;
+  csRef<iString> caption;
+  csRef<iString> icon;
   int iconalign;
 public:
   awsNotebookPage ();
@@ -296,10 +296,10 @@ public:
   virtual bool Setup (iAws *wmgr, iAwsComponentNode *settings);
 
   /// Get properties.
-  bool GetProperty (const char *name, void **parm);
+  bool GetProperty (const char *name, intptr_t *parm);
 
   /// Set properties.
-  bool SetProperty (const char *name, void *parm);
+  bool SetProperty (const char *name, intptr_t parm);
 
   /// Returns the named TYPE of the component, like "Notebook Page", etc.
   virtual const char *Type ();
@@ -331,7 +331,7 @@ protected:
   int maxheight;
 
   /// Our kitchen sink.
-  awsSink sink;
+  awsSink* sink;
 
   /// Our slot.
   awsSlot slot;
@@ -361,10 +361,10 @@ public:
   virtual bool Setup (iAws *wmgr, iAwsComponentNode *settings);
 
   /// Get properties.
-  bool GetProperty (const char *name, void **parm);
+  bool GetProperty (const char *name, intptr_t *parm);
 
   /// Set properties.
-  bool SetProperty (const char *name, void *parm);
+  bool SetProperty (const char *name, intptr_t parm);
 
   /// Executres a scriptable action. Currently supported (on top of standard
   /// ones from awsComponent) : ActivateTab.
@@ -378,9 +378,8 @@ public:
 
   virtual void AddChild (iAwsComponent *child);
 
-  static void OnActivateTab(void* param, iAwsSource* src);
-  static void OnDeactivateTab(void* param, iAwsSource* src);
-
+  static void OnActivateTab(unsigned long, intptr_t param, iAwsSource* src);
+  static void OnDeactivateTab(unsigned long, intptr_t param, iAwsSource* src);
 };
 
 class awsNotebookFactory : public awsComponentFactory
