@@ -1,5 +1,5 @@
-/**************************************************************************
-    Copyright (C) 2003 by Christopher Nelson
+/*
+    Copyright (C) 2001 by Christopher Nelson
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
     You should have received a copy of the GNU Library General Public
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*****************************************************************************/
+*/
 
 #include "cssysdef.h"
 
@@ -28,36 +28,30 @@
 #include "ivaria/reporter.h"
 #include "iutil/objreg.h"
 #include "iutil/vfs.h"
-#include "awsprefs.h"
-#include "awstex.h"
-#include "awsadler.h"
 #include "awskcfct.h"
 #include "awsparser.h"
+#include "awsprefs.h"
+#include "awstex.h"
 
 extern int awsparse (void *prefscont);
 
 const bool DEBUG_KEYS = false;
 const bool DEBUG_INIT = false;
 
-/// The gradient step for the slightly darker and lighter versions of the highlight and shadow.
+// The gradient step for the slightly darker and lighter versions of the
+// highlight and shadow.
 const unsigned char GRADIENT_STEP = 25;
 
-/***************************************************************************************************************
-*   This constructor converts the text-based name into a fairly unique numeric ID.  The ID's are then used for *
-* comparison.  The method of text-to-id mapping may be somewhat slower than a counter, but it does not have to *
-* worry about wrapping or collisions or running out during long execution cycles.                              *
-***************************************************************************************************************/
-void awsKey::ComputeKeyID (const char* n, size_t len)
+unsigned long awsKey::ComputeKeyID (const char* n) const
 {
-  CS_ASSERT (n);
-  
-  name = aws_adler32 (aws_adler32 (0, 0, 0), (unsigned char *)n, len);
-  
+  CS_ASSERT (n != 0);
+  unsigned long const x = strset->Request(n);
   if (DEBUG_KEYS)
-    printf ("aws-debug: new key %s mapped to %lu\n", n, name);
+    csPrintf ("aws-debug: new key %s mapped to %lu\n", n, x);
+  return x;
 }
 
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////////////////
 awsPrefManager::awsPrefManager (iBase *iParent) :
   def_skin(0),
   awstxtmgr(0),
@@ -75,18 +69,18 @@ awsPrefManager::~awsPrefManager ()
 
 bool awsPrefManager::Setup (iObjectRegistry *obj_reg)
 {
-  if (DEBUG_INIT) printf ("aws-debug: initializing AWS Texture Manager\n");
-  if (DEBUG_INIT) printf ("aws-debug: creating texture manager.\n");
+  if (DEBUG_INIT) csPrintf ("aws-debug: initializing AWS Texture Manager\n");
+  if (DEBUG_INIT) csPrintf ("aws-debug: creating texture manager.\n");
 
   g2d = CS_QUERY_REGISTRY (obj_reg, iGraphics2D);
   if (!g2d)
   {
-    printf ("aws-debug: Couldn't find iGraphics2D plugin!!\n");
+    csPrintf ("aws-debug: Couldn't find iGraphics2D plugin!!\n");
     return false;
   }
   awstxtmgr = new awsTextureManager ();
   if (!awstxtmgr) return false;
-  if (DEBUG_INIT) printf ("aws-debug: initing texture manager\n");
+  if (DEBUG_INIT) csPrintf ("aws-debug: initing texture manager\n");
 
   awstxtmgr->Initialize (obj_reg);
 
@@ -99,17 +93,12 @@ unsigned long awsPrefManager::NameToId (const char *n)
 {
   if (n)
   {
-    unsigned long id = aws_adler32 (
-        aws_adler32 (0, 0, 0),
-        (unsigned char *)n,
-        strlen (n));
-
-    if (DEBUG_KEYS) printf ("aws-debug: mapped %s to %lu\n", n, id);
-
+    unsigned long id = wmgr->GetStringTable()->Request(n);
+    if (DEBUG_KEYS) csPrintf ("aws-debug: mapped %s to %lu\n", n, id);
     return id;
   }
   else
-    return 0;
+    return csInvalidStringID;
 }
 
 void awsPrefManager::SetColor (int index, int color)
@@ -122,12 +111,15 @@ int awsPrefManager::GetColor (int index)
   return sys_colors[index];
 }
 
-int awsPrefManager::FindColor(unsigned char r, unsigned char g, unsigned char b)
+int awsPrefManager::FindColor(unsigned char r,
+			      unsigned char g,
+			      unsigned char b)
 {
   return g2d->FindRGB (r, g, b);
 }
 
-iTextureHandle *awsPrefManager::GetTexture (const char* name, const char* filename)
+iTextureHandle *awsPrefManager::GetTexture (const char* name,
+					    const char* filename)
 {
   if (awstxtmgr)
     return awstxtmgr->GetTexture (name, filename, false);
@@ -135,7 +127,8 @@ iTextureHandle *awsPrefManager::GetTexture (const char* name, const char* filena
     return 0;
 }
 
-iTextureHandle *awsPrefManager::GetTexture (const char* name,const char* filename, 
+iTextureHandle *awsPrefManager::GetTexture (const char* name,
+					    const char* filename, 
                                             unsigned char key_r,
                                             unsigned char key_g,
                                             unsigned char key_b)
@@ -181,7 +174,7 @@ void awsPrefManager::SetWindowMgr (iAws *_wmgr)
 
 void awsPrefManager::SetupPalette ()
 {
-  printf ("aws-debug: setting up global AWS palette...\n");
+  csPrintf ("aws-debug: setting up global AWS palette...\n");
 
   unsigned char red, green, blue;
   iTextureManager *txtmgr = 0;
@@ -256,41 +249,41 @@ void awsPrefManager::SetupPalette ()
   else
 	  sys_colors[AC_BACKFILL] = sys_colors[AC_FILL];
 
-  printf ("aws-debug: finished palette setup.\n");
+  csPrintf ("aws-debug: finished palette setup.\n");
 }
 
 bool awsPrefManager::Load (const char *def_file)
 {
   if (wmgr == 0)
   {
-    printf (
-      "\tunable to load definitions because of an internal error: no window manager.\n");
+    csPrintf ("\tunable to load definitions because of an internal error: "
+	    "no window manager: %s\n", def_file);
     return false;
   }
 
-  printf ("\tloading definitions file %s...\n", def_file);
+  csPrintf ("\tloading definitions file %s...\n", def_file);
 
   delete static_awsparser;
   static_awsparser = new awsParser(objreg, wmgr, this);
   if (!static_awsparser->Initialize (def_file))
   {
-    printf ("Couldn't open def file\n");
+    csPrintf ("Couldn't open def file: %s\n", def_file);
     delete static_awsparser;
     static_awsparser = 0;
     return false;
   }
 
-  int ncw = win_defs.Length();
-  int ncs = skin_defs.Length();
+  size_t ncw = win_defs.Length();
+  size_t ncs = skin_defs.Length();
 
   if (awsparse (wmgr))
   {
-    printf ("\tsyntax error in definition file, load failed.\n");
+    csPrintf ("\tsyntax error in definition file, load failed: %s\n", def_file);
     return false;
   }
 
-  printf (
-    "\tload successful (%i windows, %i skins loaded.)\n",
+  csPrintf (
+    "\tload successful (%zu windows, %zu skins loaded.)\n",
     win_defs.Length() - ncw,
     skin_defs.Length() - ncs);
 
@@ -301,7 +294,7 @@ bool awsPrefManager::SelectDefaultSkin (const char* skin_name)
 {
   unsigned long id = NameToId (skin_name);
 
-  for (int i=0;i<skin_defs.Length(); i++)
+  for (size_t i = 0; i < skin_defs.Length(); i++)
   {
     if (skin_defs[i]->Name () == id)
     {
@@ -460,12 +453,12 @@ bool awsPrefManager::LookupRGBKey (
   return false;
 }
 
-bool awsPrefManager::LookupPointKey (const char *name, csPoint &val)
+bool awsPrefManager::LookupPointKey (const char *name, csVector2 &val)
 {
   return LookupPointKey (NameToId (name), val);
 }
 
-bool awsPrefManager::LookupPointKey (unsigned long id, csPoint &val)
+bool awsPrefManager::LookupPointKey (unsigned long id, csVector2 &val)
 {
   iAwsKey *k = ((iAwsKeyContainer *)def_skin)->Find (id);
 
@@ -482,7 +475,8 @@ bool awsPrefManager::LookupPointKey (unsigned long id, csPoint &val)
   return false;
 }
 
-bool awsPrefManager::GetInt (iAwsComponentNode *node, const char *name, int &val)
+bool awsPrefManager::GetInt (iAwsComponentNode *node, const char *name,
+			     int &val)
 {
   if (!node) return false;
 
@@ -501,7 +495,8 @@ bool awsPrefManager::GetInt (iAwsComponentNode *node, const char *name, int &val
   return false;
 }
 
-bool awsPrefManager::GetFloat (iAwsComponentNode *node, const char *name, float &val)
+bool awsPrefManager::GetFloat (iAwsComponentNode *node, const char *name,
+			       float &val)
 {
   if (!node) return false;
 
@@ -546,17 +541,18 @@ bool awsPrefManager::GetRGB (iAwsComponentNode *node, const char *name,
   return false;
 }
 
-bool awsPrefManager::GetRect (iAwsComponentNode *node, const char *name, csRect &val)
+bool awsPrefManager::GetRect (iAwsComponentNode *node, const char *name,
+			      csRect &val)
 {
   if (!node) return false;
 
   if (DEBUG_KEYS)
-    printf ("aws-debug: Getting \"%s\" from %p\n", name, node);
+    csPrintf ("aws-debug: Getting \"%s\" from %p\n", name, (void*)node);
 
   iAwsKey *k = ((iAwsKeyContainer *)node)->Find (NameToId (name));
 
   if (DEBUG_KEYS)
-    printf ("aws-debug: Node retrieved: %p [%s]\n", node, name);
+    csPrintf ("aws-debug: Node retrieved: %p [%s]\n", (void*)node, name);
 
   if (k)
   {
@@ -571,12 +567,11 @@ bool awsPrefManager::GetRect (iAwsComponentNode *node, const char *name, csRect
   return false;
 }
 
-bool awsPrefManager::GetString (
-  iAwsComponentNode *node,
-  const char *name,
-  iString * &val)
+bool 
+awsPrefManager::GetString (iAwsComponentNode *node,  const char *name, iString* val)
 {
   if (!node) return false;
+  if (!val) return false;
 
   iAwsKey *k = ((iAwsKeyContainer *)node)->Find (NameToId (name));
 
@@ -585,7 +580,7 @@ bool awsPrefManager::GetString (
     if (k->Type () == KEY_STR)
     {
       csRef<iAwsStringKey> stringKey (SCF_QUERY_INTERFACE(k, iAwsStringKey));
-      val = stringKey->Value ();
+      val->Replace (stringKey->Value());
       return true;
     }
   }
@@ -593,11 +588,12 @@ bool awsPrefManager::GetString (
   return false;
 }
 
+
 iAwsComponentNode *awsPrefManager::FindWindowDef (const char *name)
 {
   unsigned long id = NameToId (name);
 
-  for (int i=0;i<win_defs.Length();i++)
+  for (size_t i = 0; i < win_defs.Length(); i++)
     if (win_defs[i]->Name () == id)
       return win_defs[i];
 
@@ -608,7 +604,7 @@ iAwsKeyContainer *awsPrefManager::FindSkinDef (const char *name)
 {
   unsigned long id = NameToId (name);
 
-  for (int i=0;i<skin_defs.Length();i++)
+  for (size_t i = 0; i < skin_defs.Length(); i++)
     if (skin_defs[i]->Name () == id)
       return skin_defs[i];
 
@@ -629,7 +625,7 @@ int awsPrefManager::GetConstantValue (const char *name)
 {
   unsigned int namev = NameToId (name);
 
-  int i;
+  size_t i;
   for (i = 0; i < constants.Length (); ++i)
   {
     constant_entry *c = constants.Get (i);
@@ -644,7 +640,7 @@ bool awsPrefManager::ConstantExists (const char *name)
 {
   unsigned int namev = NameToId (name);
 
-  int i;
+  size_t i;
   for (i = 0; i < constants.Length (); ++i)
   {
     constant_entry *c = constants.Get (i);
@@ -657,11 +653,20 @@ bool awsPrefManager::ConstantExists (const char *name)
 
 iAwsKeyFactory *awsPrefManager::CreateKeyFactory ()
 {
-  return new awsKeyFactory ();
+  return new awsKeyFactory (wmgr);
 }
 
 iAwsConnectionNodeFactory *awsPrefManager::CreateConnectionNodeFactory ()
 {
-  return new awsConnectionNodeFactory ();
+  return new awsConnectionNodeFactory (wmgr);
 }
 
+void awsPrefManager::AddCustomStringProperty (const char *prop)
+{
+  _customStringProps.Push( prop );
+}
+
+const csStringArray &awsPrefManager::GetCustomStringProperties()
+{
+  return _customStringProps;
+}
