@@ -1,4 +1,23 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
+#include "aws.h"
 #include "awsradbt.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
@@ -15,8 +34,7 @@ awsRadButton::awsRadButton () :
   is_on(false),
   frame_style(0),
   alpha_level(96),
-  alignment(0),
-  caption(0)
+  alignment(0)
 {
   tex[0] = tex[1] = tex[2] = tex[3] = 0;
   SetFlag (AWSF_CMP_ALWAYSERASE);
@@ -39,6 +57,7 @@ bool awsRadButton::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 
   pm->GetInt (settings, "Alpha", alpha_level);
   pm->GetInt (settings, "Align", alignment);
+  caption.AttachNew (new scfString ());
   pm->GetString (settings, "Caption", caption);
 
   tex[0] = pm->GetTexture ("RadioButtonUp");
@@ -48,24 +67,24 @@ bool awsRadButton::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 
   int _focusable = 0;
   pm->GetInt (settings, "Focusable", _focusable);
-	focusable = _focusable;
+  focusable = (_focusable != 0);
 
 
   return true;
 }
 
-bool awsRadButton::GetProperty (const char *name, void **parm)
+bool awsRadButton::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Caption", name) == 0)
   {
-    char *st = 0;
+    const char *st = 0;
 
     if (caption) st = caption->GetData ();
 
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else if (strcmp ("State", name) == 0)
@@ -80,7 +99,7 @@ bool awsRadButton::GetProperty (const char *name, void **parm)
   return false;
 }
 
-bool awsRadButton::SetProperty (const char *name, void *parm)
+bool awsRadButton::SetProperty (const char *name, intptr_t parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
 
@@ -90,14 +109,11 @@ bool awsRadButton::SetProperty (const char *name, void *parm)
 
     if (s && s->Length ())
     {
-      if (caption) caption->DecRef ();
       caption = s;
-      caption->IncRef ();
       Invalidate ();
     }
     else
     {
-      if (caption) caption->DecRef ();
       caption = 0;
     }
 
@@ -123,7 +139,7 @@ void awsRadButton::ClearGroup ()
 {
   csEvent Event;
 
-  Event.Type = csevGroupOff;
+  Event.Name = ((awsManager*) WindowManager ())->GroupOff;
 
   for (iAwsComponent* cmp = Parent()->GetTopChild(); cmp; cmp = cmp->ComponentBelow())
   {
@@ -139,9 +155,8 @@ bool awsRadButton::HandleEvent (iEvent &Event)
 {
   if (awsComponent::HandleEvent (Event)) return true;
 
-  switch (Event.Type)
+  if (Event.Name == ((awsManager*) WindowManager ())->GroupOff)
   {
-  case csevGroupOff:
     if (is_on)
     {
       is_on = false;
@@ -151,7 +166,6 @@ bool awsRadButton::HandleEvent (iEvent &Event)
     }
 
     return true;
-    break;
   }
 
   return false;
