@@ -1,5 +1,5 @@
 /*
-    Copyright (C) ???
+    Copyright (C) 2001 by Christopher Nelson
   
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -20,6 +20,8 @@
 #define __CS_AWS_SCR_H__
 
 #include "csutil/array.h"
+#include "iutil/strset.h"
+struct iAws;
 
 struct awsActionMap
 {
@@ -27,7 +29,7 @@ struct awsActionMap
   unsigned long name;
 
   /// The action to execute.
-  void (*Action) (void *owner, iAwsParmList* parmlist);
+  void (*Action) (intptr_t owner, iAwsParmList* parmlist);
 };
 
 class awsActionDispatcher
@@ -35,15 +37,17 @@ class awsActionDispatcher
 private:
   /// List of actions to execute.
   csArray<awsActionMap*> actions;
+  /// Shared string table.
+  csRef<iStringSet> strset;
 public:
+  /// Constructor.
+  awsActionDispatcher(iAws*);
   /// Register an action.
-  void Register (
-    const char *name,
-    void (Action) (void *owner,
-    iAwsParmList* parmlist));
+  void Register (const char *name,
+    void (Action) (intptr_t owner, iAwsParmList* parmlist));
 
   /// Execute the corresponding action.
-  void Execute (const char *action, void *owner, iAwsParmList* parmlist);
+  void Execute (const char *action, intptr_t owner, iAwsParmList* parmlist);
 };
 
 #endif // __CS_AWS_SCR_H__
