@@ -1,3 +1,21 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
 #include "awsscrbr.h"
 #include "aws3dfrm.h"
@@ -12,34 +30,28 @@
 
 #include <stdio.h>
 
-const int awsScrollBar:: signalChanged = 0x1;
-const int awsScrollBar:: sboVertical = 0x0;
-const int awsScrollBar:: sboHorizontal = 0x1;
-
 awsScrollBar::awsScrollBar () :
   is_down(false),
   mouse_is_over(false),
   was_down(false),
-  tex(NULL),
   orientation(0),
-  alpha_level(92),
-  decVal(NULL),
-  incVal(NULL),
-  knob(NULL),
-  timer(NULL),
-  sink(NULL),
-  dec_slot(NULL),
-  inc_slot(NULL),
-  knob_slot(NULL),
-  tick_slot(NULL),
+  decVal(0),
+  incVal(0),
+  knob(0),
+  timer(0),
+  sink(0),
+  dec_slot(0),
+  inc_slot(0),
+  knob_slot(0),
+  tick_slot(0),
   value(0),
   max(1),
   min(0),
   amntvis(0),
-  value_delta(0.1),
-  value_page_delta(0.25)
+  value_delta(0.1f),
+  value_page_delta(0.25f)
 {
-  SetFlag (AWSF_CMP_ALWAYSERASE);
+  //SetFlag (AWSF_CMP_ALWAYSERASE);
   captured = false;
 }
 
@@ -70,70 +82,102 @@ awsScrollBar::~awsScrollBar ()
         sink,
         sink->GetTriggerID ("TickTock"));
 
-  SCF_DEC_REF (incVal);
-  SCF_DEC_REF (decVal);
-  SCF_DEC_REF (knob);
-  SCF_DEC_REF (sink);
-  SCF_DEC_REF (inc_slot);
-  SCF_DEC_REF (dec_slot);
-  SCF_DEC_REF (knob_slot);
-  SCF_DEC_REF (tick_slot);
-  SCF_DEC_REF (timer);
+  if (incVal) incVal->DecRef ();
+  if (decVal) decVal->DecRef ();
+  if (knob) knob->DecRef ();
+  if (sink) sink->DecRef ();
+  if (inc_slot) inc_slot->DecRef ();
+  if (dec_slot) dec_slot->DecRef ();
+  if (knob_slot) knob_slot->DecRef ();
+  if (tick_slot) tick_slot->DecRef ();
+  if (timer) timer->DecRef ();
 
   if (captured) WindowManager ()->ReleaseMouse ();
 }
 
-char *awsScrollBar::Type ()
+const char *awsScrollBar::Type ()
 {
   return "Scroll Bar";
 }
 
-bool awsScrollBar::Setup (iAws *_wmgr, awsComponentNode *settings)
+bool awsScrollBar::Setup (iAws *_wmgr, iAwsComponentNode *settings)
 {
-  if (!awsComponent::Setup (_wmgr, settings)) return false;
+  if (!awsPanel::Setup (_wmgr, settings)) return false;
 
   iAwsPrefManager *pm = WindowManager ()->GetPrefMgr ();
 
-  pm->LookupIntKey ("OverlayTextureAlpha", alpha_level);  // global get
   pm->GetInt (settings, "Orientation", orientation);
-  pm->GetInt (settings, "Alpha", alpha_level);            // local overrides, if present.
-  int h = (int)min;
-  h = (int)min;
-  pm->GetInt (settings, "Min", h);
-  min = (float)h;
-  h = (int)value;
-  pm->GetInt (settings, "Value", h);
-  value = (float)h;
-  h = 1;
-  pm->GetInt (settings, "Max", h);
-  max = (float)h;
-  h = (int)amntvis;
-  pm->GetInt (settings, "PageSize", h);
-  amntvis = (float)h;
 
-  tex = pm->GetTexture ("Texture");
+  /*  Numbers are parsed into either floats or ints depending on
+   *   the presence of a decimal point.
+   *  If we're expecting a float and we don't find one, we should
+   *   check for an int, since the numeric value may have just been
+   *   specified as a whole number - in which case it would be parsed
+   *   as an int.
+   */
+
+  int ival;
+
+  min=0.0f;
+  if (!pm->GetFloat (settings, "Min", min))
+  {
+	  if (pm->GetInt(settings,"Min",ival))
+		  min=(float)ival;
+  }
+
+  value=0.0f;
+  if (!pm->GetFloat (settings, "Value", value))
+  {
+	  if (pm->GetInt(settings,"Value",ival))
+		  value=(float)ival;
+  }
+
+  max=1.0f;
+  if (!pm->GetFloat (settings, "Max", max))
+  {
+	  if (pm->GetInt(settings,"Max",ival))
+		  max=(float)ival;
+  }
+
+  amntvis=1.0f;
+  if (!pm->GetFloat (settings, "PageSize", amntvis))
+  {
+	  if (pm->GetInt(settings,"PageSize",ival))
+		  amntvis=(float)ival;
+  }
+
+  
+  value_delta=1.0f;
+  if (!pm->GetFloat (settings, "Change", value_delta))
+  {
+	  if (pm->GetInt(settings,"Change",ival))
+		  value_delta=(float)ival;
+  }
+
+  value_page_delta=1.0f;
+  if (!pm->GetFloat (settings, "BigChange", value_page_delta))
+  {
+	  if (pm->GetInt(settings,"BigChange",ival))
+		  value_page_delta=(float)ival;
+  }
+
+  iAws* const w = WindowManager();
 
   // Setup embedded buttons
   incVal = new awsSliderButton;
   decVal = new awsSliderButton;
   knob = new awsSliderButton;
-  timer = new awsTimer (WindowManager ()->GetObjectRegistry (), this);
+  timer = new awsTimer (w->GetObjectRegistry (), this);
 
-  awsKeyFactory incinfo, decinfo, knobinfo;
+  awsKeyFactory incinfo(w), decinfo(w), knobinfo(w);
 
-  decinfo.Initialize (
-      new scfString ("decVal"),
-      new scfString ("Slider Button"));
-  incinfo.Initialize (
-      new scfString ("incVal"),
-      new scfString ("Slider Button"));
-  knobinfo.Initialize (
-      new scfString ("knob"),
-      new scfString ("Slider Button"));
+  decinfo.Initialize ("decVal", "Slider Button");
+  incinfo.Initialize ("incVal", "Slider Button");
+  knobinfo.Initialize ("knob", "Slider Button");
 
-  decinfo.AddIntKey (new scfString ("Style"), awsCmdButton::fsToolbar);
-  incinfo.AddIntKey (new scfString ("Style"), awsCmdButton::fsToolbar);
-  knobinfo.AddIntKey (new scfString ("Style"), awsCmdButton::fsToolbar);
+  decinfo.AddIntKey ("Style", awsCmdButton::fsNormal);
+  incinfo.AddIntKey ("Style", awsCmdButton::fsNormal);
+  knobinfo.AddIntKey ("Style", awsCmdButton::fsNormal);
 
   switch (orientation)
   {
@@ -149,25 +193,20 @@ bool awsScrollBar::Setup (iAws *_wmgr, awsComponentNode *settings)
 
         incimg->GetOriginalDimensions (img_w, img_h);
 
-        decinfo.AddRectKey (
-            new scfString ("Frame"),
-            csRect (0, 0, Frame ().Width (), img_h));
+        decinfo.AddRectKey ("Frame", csRect (0, 0, Frame ().Width (), img_h));
 
-        incinfo.AddRectKey (
-            new scfString ("Frame"),
-            csRect (
+        incinfo.AddRectKey ("Frame", csRect (
               0,
               Frame ().Height () - img_h,
               Frame ().Width (),
               Frame ().Height ()));
 
-        knobinfo.AddRectKey (
-            new scfString ("Frame"),
-            csRect (0, img_h + 1, Frame ().Width (), 2 * img_h + 1));
+        knobinfo.AddRectKey ("Frame",
+	      csRect (0, img_h + 1, Frame ().Width (), 2 * img_h + 1));
       }
       break;
 
-    default:
+      default:
       {
         incimg = pm->GetTexture ("ScrollBarRt");
         decimg = pm->GetTexture ("ScrollBarLt");
@@ -179,37 +218,19 @@ bool awsScrollBar::Setup (iAws *_wmgr, awsComponentNode *settings)
 
         incimg->GetOriginalDimensions (img_w, img_h);
 
-        decinfo.AddRectKey (
-            new scfString ("Frame"),
-            csRect (
-              Frame ().xmin,
-              Frame ().ymin,
-              Frame ().xmin + img_w + 5,
-              Frame ().ymax));
+        decinfo.AddRectKey ("Frame", csRect (0, 0, img_w, Frame().Height()));
 
-        incinfo.AddRectKey (
-            new scfString ("Frame"),
-            csRect (
-              Frame ().xmax - img_w - 5,
-              Frame ().ymin,
-              Frame ().xmax,
-              Frame ().ymax));
+        incinfo.AddRectKey ("Frame",
+              csRect (Frame ().Width () - img_w, 0,
+                      Frame ().Width (), Frame().Height()));
 
-        knobinfo.AddRectKey (
-            new scfString ("Frame"),
-            csRect (
-              Frame ().xmin + img_w + 6,
-              Frame ().ymin,
-              Frame ().xmin + 2 * img_w + 6,
-              Frame ().ymax));
+        knobinfo.AddRectKey ("Frame",
+			     csRect (img_w + 1, 0, 2 * img_w + 1, Frame().Height()));
       }
       break;
   } // end switch framestyle
 
   //  DEBUG_BREAK;
-  decVal->SetWindow (Window ());
-  incVal->SetWindow (Window ());
-  knob->SetWindow (Window ());
 
   decVal->SetParent (this);
   incVal->SetParent (this);
@@ -219,15 +240,18 @@ bool awsScrollBar::Setup (iAws *_wmgr, awsComponentNode *settings)
   incVal->Setup (_wmgr, incinfo.GetThisNode ());
   knob->Setup (_wmgr, knobinfo.GetThisNode ());
 
-  decVal->SetProperty ("Image", decimg);
-  incVal->SetProperty ("Image", incimg);
+  decVal->SetProperty ("Image", (intptr_t)decimg);
+  incVal->SetProperty ("Image", (intptr_t)incimg);
+
 
   csTicks t = (csTicks) 10;
-  incVal->SetProperty ("TicksPerSecond", (void *) &t);
-  decVal->SetProperty ("TicksPerSecond", (void *) &t);
-  knob->SetProperty ("TicksPerSecond", (void *) &t);
+  incVal->SetProperty ("TicksPerSecond", (intptr_t)&t);
+  decVal->SetProperty ("TicksPerSecond", (intptr_t)&t);
+  knob->SetProperty ("TicksPerSecond", (intptr_t)&t);
 
-  sink = new awsSink (this);
+  awsSink* _sink = new awsSink (w);
+  _sink->SetParm ((intptr_t)this);
+  sink = _sink;
 
   sink->RegisterTrigger ("DecValue", &DecClicked);
   sink->RegisterTrigger ("IncValue", &IncClicked);
@@ -264,64 +288,111 @@ bool awsScrollBar::Setup (iAws *_wmgr, awsComponentNode *settings)
   return true;
 }
 
-bool awsScrollBar::GetProperty (char *name, void **parm)
+bool awsScrollBar::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
   if (strcmp ("Value", name) == 0)
   {
-    *parm = (void *) &value;
+    *parm = (intptr_t)&value;
     return true;
   }
-
-  return false;
-}
-
-bool awsScrollBar::SetProperty (char *name, void *parm)
-{
-  if (awsComponent::SetProperty (name, parm)) return true;
-
-  if (strcmp ("Change", name) == 0)
+  else if (strcmp ("Change", name) == 0)
   {
-    value_delta = *(int *)parm;
+    *parm = (intptr_t)&value_delta;
     return true;
   }
   else if (strcmp ("BigChange", name) == 0)
   {
-    value_page_delta = *(int *)parm;
+    *parm = (intptr_t)&value_page_delta;
+    return true;
+  }
+  else if (strcmp ("Max", name) == 0)
+  {
+    *parm = (intptr_t)&max;
     return true;
   }
   else if (strcmp ("Min", name) == 0)
   {
-    min = *(int *)parm;
+    *parm = (intptr_t)&min;
+    return true;
+  }
+  else if (strcmp ("PageSize", name) == 0)
+  {
+    *parm = (intptr_t)&amntvis;
+    return true;
+  }
+ 
+  return false;
+}
+
+bool awsScrollBar::SetProperty (const char *name, intptr_t parm)
+{
+  if (awsComponent::SetProperty (name, parm)) 
+    return true;
+
+  if (strcmp ("Change", name) == 0)
+  {
+    value_delta = *(float *)parm;
+    
+    Invalidate ();
+    return true;
+  }
+  else if (strcmp ("BigChange", name) == 0)
+  {
+    value_page_delta = *(float *)parm;
+
+    Invalidate ();
+    return true;
+  }
+  else if (strcmp ("Min", name) == 0)
+  {
+    min = *(float *)parm;
+
+    // Fix value in case it's out of range
+    value = (value < min) ? min : value;
 
     Invalidate ();
     return true;
   }
   else if (strcmp ("Max", name) == 0)
   {
-    max = *(int *)parm;
+    max = *(float *)parm;
 
     // Fix the page size
-    if (amntvis > max) amntvis = max;
-    int maxval = (int)(max-amntvis);
+    if (amntvis > max) 
+      amntvis = max + 1.0f;
+    int maxval = (int)(max - amntvis + 1.0f);
 
     // Fix value in case it's out of range
-    value = (value < 0 ? 0 : (value > maxval ? maxval : value));
+    value = (value < min ? min : (value > maxval ? maxval : value));
 
     Invalidate ();
     return true;
   }
   else if (strcmp ("PageSize", name) == 0)
   {
-    amntvis = *(int *)parm;
+    amntvis = *(float *)parm;
 
     // Fix the page size
-    if (amntvis > max) amntvis = max;
-    int maxval = (int)(max-amntvis);
+    if (amntvis > max) 
+      amntvis = max + 1.0f;
+    int maxval = (int)(max - amntvis + 1.0f);
 
     // Fix value in case it's out of range
-    value = (value < 0 ? 0 : (value > maxval ? maxval : value));
+    value = (value < min ? min : (value > maxval ? maxval : value));
+
+    Invalidate ();
+    return true;
+  }
+  else if (strcmp ("Value", name) == 0)
+  {
+    value = *(float *)parm;
+
+    // Fix value in case it's out of range
+    int maxval = (int)(max - amntvis + 1.0f);
+
+    value = (value < min ? min : (value > maxval ? maxval : value));
 
     Invalidate ();
     return true;
@@ -330,39 +401,61 @@ bool awsScrollBar::SetProperty (char *name, void *parm)
   return false;
 }
 
-void awsScrollBar::KnobTick (void *sk, iAwsSource *)
+/*  This function handles at least mouse scrolling on the bar by "grabbing" the scroll "knob".
+ */
+void awsScrollBar::KnobTick (unsigned long, intptr_t sk, iAwsSource *)
 {
-  // adjust position of knob and scrollbar value
   awsScrollBar *sb = (awsScrollBar *)sk;
-  int maxval = (int)(sb->max - sb->amntvis);
+  /* amntvis relates to "proportional scrollbars" which are probably broken
+   *  because most everything else was prior to some fixups and I don't
+   *  know how "proportional scrollbars" are supposed to work so I haven't
+   *  attempted to fix them.
+   */
 
   if (sb->orientation == sboVertical)
   {
-    int height = 10;
+    // Code for Vertical scrollbars.  We use height for calculations here.
+    int height = 10; 
+
+    // We get the rectangle that surrounds the entire scrollbar, including the end buttons (up/down arrows)
     csRect f (sb->Frame ());
 
-    // Get the bar height
+    // Then we subtract the height of the up and down arrows to get the height of the actual "bar" area we can scroll through
     f.ymin += sb->decVal->Frame ().Height () + 1;
     f.ymax -= sb->incVal->Frame ().Height () + 1;
 
-    // Get the knob height
+    /* This retrieves the knob height.  If amntvis isn't 0 you're using a "proportional" scrollbar
+     *  which apparently changes the size of the knob based on some factors.
+     */
     if (sb->amntvis == 0)
       sb->WindowManager ()->GetPrefMgr ()->LookupIntKey (
           "ScrollBarHeight",
           height);
     else
-      height = (int)((sb->amntvis * f.Height ()) / sb->max);
+      height = (int)((sb->amntvis * f.Height ()) / (sb->max - sb->min) );
 
-    // Get the actual height that we can traverse with the knob
+    // bh becomes the actual "usable" height in pixels - after we subtract the height of the knob from the scroll area
     int bh = f.Height () - height;
     
-    if (maxval == 0)
+    // If there's no area to scroll through or the range (max - min) is less than 0, then there is only one possible position to be at.
+    if (((sb->max - sb->min) <= 0) || (bh == 0)) 
       sb->value = 0;
     else
-      sb->value = (sb->knob->last_y - sb->decVal->Frame ().ymax) * maxval / bh;
+      /*  sb->knob->last_y is actually the last y position of the mouse while dragging the knob
+       *  We take that position, subtract the y of the bottom edge of the top arrow button - that gives us how far the mouse is 
+       *   "into" the scroll range.  We also subtract half of the height of the knob, otherwise the knob would end up
+       *   with the top edge at the mouse cursor (instead of the more desirable and logical center of the button).
+       *  We take this distance in, and divide by the total navigatable pixels in the scroll bar (bh) - this should give us a value 0.00 - 1.00 
+       *  That value is multiplied by the range this bar is supposed to cover (max - min) which gives us the number in range-units that we are into
+       *  the bar.  Finally we add the minimum to get the actual range-unit the mouse is at.
+       */
+      sb->value = ((sb->knob->last_y - (sb->knob->Frame().Height()/2) - sb->decVal->Frame ().ymax) * (sb->max - sb->min) / bh) + sb->min;
+
   }
   else if (sb->orientation == sboHorizontal)
   {
+    // Code for Horizontal scrollbars.  We use width for calculations here.
+    // For detailed documentation of what is going on here, see above in the sboVertical case.
     int width = 10;
     csRect f (sb->Frame ());
 
@@ -374,29 +467,29 @@ void awsScrollBar::KnobTick (void *sk, iAwsSource *)
           "ScrollBarWidth",
           width);
     else
-      width = (int)((sb->amntvis * f.Width ()) / sb->max);
+      width = (int)((sb->amntvis * f.Width ()) / (sb->max - sb->min));
 
-    // Get the actual height that we can traverse with the knob
+   
     int bw = f.Width () - width;
 
-    if (maxval == 0)
-      sb->value = 0;
+    if ((sb->max - sb->min <= 0) || (bw == 0))
+      sb->value = sb->min;
     else
-      sb->value = (sb->knob->last_x - sb->decVal->Frame ().xmax) * maxval / bw;
+      sb->value = ((sb->knob->last_x - (sb->knob->Frame().Width()/2) - sb->decVal->Frame ().xmax) * (sb->max - sb->min) / bw) + sb->min ;
   }
   else
     return ;
 
   sb->value =
     (
-      sb->value < sb->min ? 0 :
-        (sb->value > maxval ? maxval : sb->value)
+      sb->value < sb->min ? sb->min :
+        (sb->value > sb->max ? sb->max : sb->value)
     );
   sb->Broadcast (signalChanged);
   sb->Invalidate ();
 }
 
-void awsScrollBar::TickTock (void *sk, iAwsSource *)
+void awsScrollBar::TickTock (unsigned long, intptr_t sk, iAwsSource *)
 {
   awsScrollBar *sb = (awsScrollBar *)sk;
 
@@ -419,10 +512,10 @@ void awsScrollBar::TickTock (void *sk, iAwsSource *)
       return ;
   }
 
- int maxval = (int)(sb->max - sb->amntvis);
+  float maxval = (sb->max - sb->amntvis + 1);
   sb->value =
     (
-      sb->value < sb->min ? 0 :
+      sb->value < sb->min ? sb->min :
         (sb->value > maxval ? maxval : sb->value)
     );
 
@@ -430,45 +523,42 @@ void awsScrollBar::TickTock (void *sk, iAwsSource *)
   sb->Invalidate ();
 }
 
-void awsScrollBar::IncClicked (void *sk, iAwsSource *)
+void awsScrollBar::IncClicked (unsigned long, intptr_t sk, iAwsSource *)
 {
   awsScrollBar *sb = (awsScrollBar *)sk;
 
   sb->value += sb->value_delta;
 
   /// Check floor and ceiling
- int maxval = (int)(sb->max - sb->amntvis);
   sb->value =
     (
-     sb->value < sb->min ? 0 :
-     (sb->value > maxval ? maxval : sb->value)
+     sb->value < sb->min ? sb->min :
+     (sb->value > sb->max ? sb->max : sb->value)
      );
 
   sb->Broadcast (signalChanged);
   sb->Invalidate ();
 }
 
-void awsScrollBar::DecClicked (void *sk, iAwsSource *)
+void awsScrollBar::DecClicked (unsigned long, intptr_t sk, iAwsSource *)
 {
   awsScrollBar *sb = (awsScrollBar *)sk;
 
   sb->value -= sb->value_delta;
 
   /// Check floor and ceiling
- int maxval = (int)(sb->max - sb->amntvis);
   sb->value =
     (
-      sb->value < sb->min ? 0 :
-        (sb->value > maxval ? maxval : sb->value)
+      sb->value < sb->min ? sb->min :
+        (sb->value > sb->max ? sb->max : sb->value)
     );
 
   sb->Broadcast (signalChanged);
   sb->Invalidate ();
 }
 
-void awsScrollBar::OnDraw (csRect /*clip*/)
+void awsScrollBar::OnDraw (csRect clip)
 {
-  aws3DFrame frame3d;
   int height = 10, width = 10;
 
   csRect f (Frame ());
@@ -485,17 +575,22 @@ void awsScrollBar::OnDraw (csRect /*clip*/)
           "ScrollBarHeight",
           height);
     else
-      height = (int)((amntvis * f.Height ()) / max);
+	{
+      height = (int)((amntvis * f.Height ()) / (max-min));
+	  // Minimum height of the scrollbar is 5 pixels
+	  if (height<5)
+		  height=5;
+	}
 
     // Get the actual height that we can traverse with the knob
     int bh = f.Height () - height;
 
     // Get the knob's position
     int ky;
-    if ((max-amntvis) == 0)
+    if ((max-min) - amntvis == 0)
       ky = 0;
     else
-      ky = (int)((value * bh) / (max-amntvis));
+      ky = (int)(((value-min) * bh) / ((max-min)-amntvis));
 
     f.ymin += ky;
     f.ymax = f.ymin + height;
@@ -509,19 +604,25 @@ void awsScrollBar::OnDraw (csRect /*clip*/)
     f.xmax -= incVal->Frame ().Width () + 1;
 
     if (amntvis == 0)
-      WindowManager ()->GetPrefMgr ()->LookupIntKey ("ScrollBarWidth", width);
+      WindowManager ()->GetPrefMgr ()->LookupIntKey 
+        ("ScrollBarWidth", width);
     else
-      width = (int)((amntvis * f.Width ()) / max);
+	{
+      width = (int)((amntvis * f.Width ()) / (max-min));
+	  // Minimum width of the scrollbar is 5 pixels
+	  if (width<5)
+		  width=5;
+	}
 
     // Get the actual height that we can traverse with the knob
     int bw = f.Width () - width;
 
     // Get the knob's position
     int kx;
-    if ((max-amntvis)==0)
+    if (((max-min) - amntvis) == 0)
       kx = 0;
     else
-      kx = (int)((value * bw) / max);
+      kx = (int)(((value-min) * bw) / (max-min));
 
     f.xmin += kx;
     f.xmax = f.xmin + width;
@@ -530,9 +631,9 @@ void awsScrollBar::OnDraw (csRect /*clip*/)
       f.xmax = incVal->Frame ().xmin - 1;
   }
 
-  knob->Frame () = f;
+  knob->ResizeTo(f);
 
-  //  frame3d.Draw(WindowManager(), Window(), f, aws3DFrame::fsRaised, tex, alpha_level);
+  awsPanel::OnDraw(clip);
 }
 
 bool awsScrollBar::OnMouseDown (int btn, int x, int y)
@@ -633,19 +734,52 @@ bool awsScrollBar::OnMouseEnter ()
   return true;
 }
 
-bool awsScrollBar::OnKeypress (int, int)
+bool awsScrollBar::OnKeyboard (const csKeyEventData& eventData)
 {
-  return false;
+  switch(eventData.codeCooked)
+ {
+  case CSKEY_UP:
+    if(orientation == sboVertical)
+		{
+		 value -= value_delta;
+		 Broadcast (signalChanged);
+		 value = (value < min ? min : (value > max ? max : value));
+		}
+		break;
+  case CSKEY_DOWN:
+    if(orientation == sboVertical)
+		{
+		 value += value_delta;
+		 Broadcast (signalChanged);
+		 value = (value < min ? min : (value > max ? max : value));
+		}
+		break;
+  case CSKEY_LEFT:
+    if(orientation == sboHorizontal)
+		{
+		 value -= value_delta;
+		 Broadcast (signalChanged);
+		 value = (value < min ? min : (value > max ? max : value));
+		}
+		break;
+  case CSKEY_RIGHT:
+    if(orientation == sboHorizontal)
+		{
+		 value += value_delta;
+		 Broadcast (signalChanged);
+		 value = (value < min ? min : (value > max ? max : value));
+		}
+		break;
+	}
+
+	Invalidate ();
+
+	return true;
 }
 
-bool awsScrollBar::OnLostFocus ()
+void awsScrollBar::OnSetFocus ()
 {
-  return false;
-}
-
-bool awsScrollBar::OnGainFocus ()
-{
-  return false;
+	Broadcast (signalFocused);
 }
 
 void awsScrollBar::OnAdded ()
@@ -660,11 +794,9 @@ void awsScrollBar::OnResized ()
   int h = incVal->Frame ().Height ();
   int w = incVal->Frame ().Width ();
 
-  decVal->Frame ().SetPos (Frame ().xmax - w, Frame ().ymin);
-  incVal->Frame ().SetPos (Frame ().xmax - w, Frame ().ymax - h);
+  decVal->MoveTo(Frame ().xmax - w, Frame ().ymin);
+  incVal->MoveTo(Frame ().xmax - w, Frame ().ymax - h);
 
-  incVal->Frame ().SetSize (w, h);
-  decVal->Frame ().SetSize (w, h);
 }
 
 /************************************* Command Button Factory ****************/
@@ -678,6 +810,7 @@ awsScrollBarFactory::awsScrollBarFactory (
   RegisterConstant ("sboHorizontal", awsScrollBar::sboHorizontal);
 
   RegisterConstant ("signalScrollBarChanged", awsScrollBar::signalChanged);
+  RegisterConstant ("signalScrollBarFocused", awsScrollBar::signalFocused);
 }
 
 awsScrollBarFactory::~awsScrollBarFactory ()
@@ -693,11 +826,11 @@ iAwsComponent *awsScrollBarFactory::Create ()
 /************************************* Slider Button ****************/
 
 awsSliderButton::awsSliderButton () :
-  timer(NULL),
+  timer(0),
   captured(false),
   nTicks((csTicks) 0),
-  sink(NULL),
-  tick_slot(NULL)
+  sink(0),
+  tick_slot(0)
 {
 }
 
@@ -712,16 +845,18 @@ awsSliderButton::~awsSliderButton ()
 
   if (captured) WindowManager ()->ReleaseMouse ();
 
-  SCF_DEC_REF (tick_slot);
-  SCF_DEC_REF (timer);
+  if (tick_slot) tick_slot->DecRef ();
+  if (timer) timer->DecRef ();
 }
 
-bool awsSliderButton::Setup (iAws *wmgr, awsComponentNode *settings)
+bool awsSliderButton::Setup (iAws *wmgr, iAwsComponentNode *settings)
 {
   if (!awsCmdButton::Setup (wmgr, settings)) return false;
 
   timer = new awsTimer (WindowManager ()->GetObjectRegistry (), this);
-  sink = new awsSink (this);
+  awsSink* _sink = new awsSink (WindowManager());
+  _sink->SetParm ((intptr_t)this);
+  sink = _sink;
 
   sink->RegisterTrigger ("TickTock", &TickTock);
 
@@ -735,20 +870,20 @@ bool awsSliderButton::Setup (iAws *wmgr, awsComponentNode *settings)
   return true;
 }
 
-bool awsSliderButton::GetProperty (char *name, void **parm)
+bool awsSliderButton::GetProperty (const char *name, intptr_t *parm)
 {
   if (awsCmdButton::GetProperty (name, parm)) return true;
 
   if (strcmp ("TicksPerSecond", name) == 0)
   {
-    *parm = (void *) &nTicks;
+    *parm = (intptr_t)&nTicks;
     return true;
   }
 
   return false;
 }
 
-bool awsSliderButton::SetProperty (char *name, void *parm)
+bool awsSliderButton::SetProperty (const char *name, intptr_t parm)
 {
   if (awsCmdButton::SetProperty (name, parm)) return true;
 
@@ -767,12 +902,12 @@ bool awsSliderButton::SetProperty (char *name, void *parm)
   return false;
 }
 
-char *awsSliderButton::Type ()
+const char *awsSliderButton::Type ()
 {
   return "Slider Button";
 }
 
-void awsSliderButton::TickTock (void *sk, iAwsSource *)
+void awsSliderButton::TickTock (unsigned long, intptr_t sk, iAwsSource *)
 {
   awsSliderButton *sb = (awsSliderButton *)sk;
   sb->Broadcast (signalClicked);
