@@ -1,3 +1,21 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
 #include "awsstbar.h"
 #include "ivideo/graph2d.h"
@@ -71,7 +89,7 @@ bool awsStatusBar::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   return true;
 }
 
-bool awsStatusBar::GetProperty (const char *name, void **parm)
+bool awsStatusBar::GetProperty (const char *name, intptr_t*parm)
 {
   if (awsComponent::GetProperty (name, parm)) return true;
 
@@ -85,7 +103,7 @@ bool awsStatusBar::GetProperty (const char *name, void **parm)
   return false;
 }
 
-bool awsStatusBar::SetProperty (const char *name, void *parm)
+bool awsStatusBar::SetProperty (const char *name, intptr_t parm)
 {
   if (awsComponent::SetProperty (name, parm)) return true;
 
