@@ -1,3 +1,21 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
 #include "awsstdsk.h"
 #include "awscomp.h"
@@ -5,7 +23,7 @@
 
 #include <stdio.h>
 
-void awsStandardSink::Hide (void *sink, iAwsSource *source)
+void awsStandardSink::Hide (unsigned long, intptr_t sink, iAwsSource *source)
 {
   (void)sink;
 
@@ -14,7 +32,7 @@ void awsStandardSink::Hide (void *sink, iAwsSource *source)
   c->Hide ();
 }
 
-void awsStandardSink::Show (void *sink, iAwsSource *source)
+void awsStandardSink::Show (unsigned long, intptr_t sink, iAwsSource *source)
 {
   (void)sink;
 
@@ -23,28 +41,25 @@ void awsStandardSink::Show (void *sink, iAwsSource *source)
   c->Show ();
 }
 
-void awsStandardSink::HideWindow (void *sink, iAwsSource *source)
+void awsStandardSink::HideWindow (unsigned long, intptr_t /*sink*/, iAwsSource *source)
 {
-  sink = (awsStandardSink *)sink;
   iAwsComponent *c = source->GetComponent ();
   c->Window()->Hide ();
 }
 
-void awsStandardSink::MaximizeWindow (void *sink, iAwsSource *source)
+void awsStandardSink::MaximizeWindow (unsigned long, intptr_t /*sink*/, iAwsSource *source)
 {
-  sink = (awsStandardSink *)sink;
   iAwsComponent *c = source->GetComponent ();
   c->Window ()->Maximize ();
 }
 
-void awsStandardSink::UnMaximizeWindow (void *sink, iAwsSource *source)
+void awsStandardSink::UnMaximizeWindow (unsigned long, intptr_t /*sink*/, iAwsSource *source)
 {
-  sink = (awsStandardSink *)sink;
   iAwsComponent *c = source->GetComponent ();
   c->Window ()->UnMaximize ();
 }
 
-void awsStandardSink::WindowSlideOutLeft (void *_sink, iAwsSource *source)
+void awsStandardSink::WindowSlideOutLeft (unsigned long, intptr_t _sink, iAwsSource *source)
 {
   awsStandardSink *sink = (awsStandardSink *)_sink;
 
@@ -57,7 +72,7 @@ void awsStandardSink::WindowSlideOutLeft (void *_sink, iAwsSource *source)
     sink->wmgr->CreateTransition(win, AWS_TRANSITION_SLIDE_OUT_LEFT);
 }
 
-void awsStandardSink::WindowSlideOutRight (void *_sink, iAwsSource *source)
+void awsStandardSink::WindowSlideOutRight (unsigned long, intptr_t _sink, iAwsSource *source)
 {
    awsStandardSink *sink = (awsStandardSink *)_sink;
 
@@ -70,7 +85,7 @@ void awsStandardSink::WindowSlideOutRight (void *_sink, iAwsSource *source)
     sink->wmgr->CreateTransition(win, AWS_TRANSITION_SLIDE_OUT_RIGHT);
 }
 
-void awsStandardSink::WindowSlideOutUp (void *_sink, iAwsSource *source)
+void awsStandardSink::WindowSlideOutUp (unsigned long, intptr_t _sink, iAwsSource *source)
 {
    awsStandardSink *sink = (awsStandardSink *)_sink;
 
@@ -83,7 +98,7 @@ void awsStandardSink::WindowSlideOutUp (void *_sink, iAwsSource *source)
     sink->wmgr->CreateTransition(win, AWS_TRANSITION_SLIDE_OUT_UP);
 }
 
-void awsStandardSink::WindowSlideOutDown (void *_sink, iAwsSource *source)
+void awsStandardSink::WindowSlideOutDown (unsigned long, intptr_t _sink, iAwsSource *source)
 {
    awsStandardSink *sink = (awsStandardSink *)_sink;
 
@@ -97,7 +112,7 @@ void awsStandardSink::WindowSlideOutDown (void *_sink, iAwsSource *source)
 }
 
 
-void awsStandardSink::Invalidate (void *sink, iAwsSource *source)
+void awsStandardSink::Invalidate (unsigned long, intptr_t sink, iAwsSource *source)
 {
   (void)sink;
 
@@ -107,9 +122,9 @@ void awsStandardSink::Invalidate (void *sink, iAwsSource *source)
 }
 
 awsStandardSink::awsStandardSink (iAws *_wmgr) :
-  awsSink(), wmgr(_wmgr)
+  awsSink(_wmgr), wmgr(_wmgr)
 {
-  SetParm (this);
+  SetParm ((intptr_t)this);
   RegisterTrigger ("Show", Show);
   RegisterTrigger ("Hide", Hide);
   RegisterTrigger ("HideWindow", HideWindow);
