@@ -29,41 +29,40 @@
  * is not encouraged, since it may make assumptions based on the internal
  * architecture of AWS that other components and sinks may not be privy to.
  */
-
 class awsStandardSink : public awsSink
 {
 private:
   iAws *wmgr;
 
   /// Hides the source component.
-  static void Hide (void *sink, iAwsSource *source);
+  static void Hide (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Shows the source component.
-  static void Show (void *sink, iAwsSource *source);
+  static void Show (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Hides the window that the source component belongs to.
-  static void HideWindow (void *sink, iAwsSource *source);
+  static void HideWindow (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Maximizes the window that the source component belongs to.
-  static void MaximizeWindow (void* sink, iAwsSource *source);
+  static void MaximizeWindow (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// UnMaximizes the window that the source component belongs to.
-  static void UnMaximizeWindow (void* sink, iAwsSource *source);
+  static void UnMaximizeWindow (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Invalidates the source component.
-  static void Invalidate (void *sink, iAwsSource *source);
+  static void Invalidate (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Slides a window down and out.
-  static void WindowSlideOutDown (void *sink, iAwsSource *source);
+  static void WindowSlideOutDown (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Slides a window up and out.
-  static void WindowSlideOutUp (void *sink, iAwsSource *source);
+  static void WindowSlideOutUp (unsigned long, intptr_t sink, iAwsSource *source);
 
   /// Slides a window left and out.
-  static void WindowSlideOutLeft (void *sink, iAwsSource *source);
+  static void WindowSlideOutLeft (unsigned long, intptr_t sink, iAwsSource *source);
   
   /// Slides a window right and out.
-  static void WindowSlideOutRight (void *sink, iAwsSource *source);
+  static void WindowSlideOutRight (unsigned long, intptr_t sink, iAwsSource *source);
 public:
   awsStandardSink (iAws *_wmgr);
   virtual ~awsStandardSink ();
