@@ -1,27 +1,41 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/vfs.h"
-#include "iutil/plugin.h"
-#include "ivaria/reporter.h"
-#include "igraphic/imageio.h"
-#include "igraphic/image.h"
-#include "ivideo/txtmgr.h"
-#include "ivideo/texture.h"
-#include "awstex.h"
-#include "awsadler.h"
 #include <string.h>
 #include <stdio.h>
 
-const bool DEBUG_GETTEX = false;
 
-static unsigned long NameToId (const char *txt)
-{
-  return aws_adler32 (
-      aws_adler32 (0, 0, 0),
-      (unsigned char *)txt,
-      strlen (txt));
-}
+#include "igraphic/image.h"
+#include "igraphic/imageio.h"
+#include "iutil/comp.h"
+#include "iutil/databuff.h"
+#include "iutil/eventh.h"
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "iutil/vfs.h"
+#include "ivaria/reporter.h"
+#include "ivideo/texture.h"
+#include "ivideo/txtmgr.h"
+
+#include "awstex.h"
+
+const bool DEBUG_GETTEX = false;
 
 awsTextureManager::awsTexture::~awsTexture ()
 {
@@ -29,19 +43,23 @@ awsTextureManager::awsTexture::~awsTexture ()
 
 awsTextureManager::awsTextureManager () : object_reg(0)
 {
-  // empty
 }
 
 awsTextureManager::~awsTextureManager ()
 {
 }
 
+unsigned long awsTextureManager::NameToId (const char *name) const
+{
+  return strset->Request(name);
+}
+
 void awsTextureManager::Initialize (iObjectRegistry *obj_reg)
 {
   object_reg = obj_reg;
 
-  if (!obj_reg) printf ("aws-debug:  bad obj_reg (%s)\n", __FILE__);
-  if (!object_reg) printf ("aws-debug:  bad object_reg (%s)\n", __FILE__);
+  if (!obj_reg) csPrintf ("aws-debug:  bad obj_reg (%s)\n", __FILE__);
+  if (!object_reg) csPrintf ("aws-debug:  bad object_reg (%s)\n", __FILE__);
 
   loader = CS_QUERY_REGISTRY (object_reg, iImageIO);
   vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
@@ -51,7 +69,7 @@ void awsTextureManager::Initialize (iObjectRegistry *obj_reg)
     csReport (
       object_reg,
       CS_REPORTER_SEVERITY_ERROR,
-      "crystalspace.awsprefs",
+      "crystalspace.awstex",
       "could not load the image loader plugin. This is a fatal error.");
   }
 
@@ -60,7 +78,7 @@ void awsTextureManager::Initialize (iObjectRegistry *obj_reg)
     csReport (
       object_reg,
       CS_REPORTER_SEVERITY_ERROR,
-      "crystalspace.awsprefs",
+      "crystalspace.awstex",
       "could not load the VFS plugin. This is a fatal error.");
   }
 
@@ -69,9 +87,18 @@ void awsTextureManager::Initialize (iObjectRegistry *obj_reg)
     csReport (
       object_reg,
       CS_REPORTER_SEVERITY_ERROR,
-      "crystalspace.awsprefs",
+      "crystalspace.awstex",
       "could not mount the default aws skin (awsdef.zip)aws.");
   }
+
+  strset = CS_QUERY_REGISTRY_TAG_INTERFACE(object_reg,
+    "crystalspace.shared.stringset", iStringSet);
+  if (!strset.IsValid())
+  {
+    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR, "crystalspace.awstex",
+      "could not locate the global shared string set \""
+      "crystalspace.shared.stringset\". This is a serious error.");
+  }
 }
 
 iTextureHandle *awsTextureManager::GetTexture (
@@ -99,22 +126,22 @@ iTextureHandle *awsTextureManager::GetTexturebyID (
   awsTexture *awstxt = 0;
   bool txtfound = false;
 
-  /*  Perform a lookup on the texture list.  We may consider doing this a little more
-  * optimally in the future, like with a QuickSort. */
+  // Perform a lookup on the texture list.  We may consider doing this a little
+  // more optimally in the future, like with a QuickSort.
   if (DEBUG_GETTEX)
-    printf (
-      "aws-debug: (%s) texture count is: %d\n",
+    csPrintf (
+      "aws-debug: (%s) texture count is: %lu\n",
       __FILE__,
-      textures.Length ());
+      (unsigned long)textures.Length ());
 
-  int i;
+  size_t i;
   for (i = 0; i < textures.Length () && txtfound == false; ++i)
   {
     awsTexture *awstxt = textures[i];
 
     if (DEBUG_GETTEX)
-      printf ("aws-debug: (%s) texture is: %p\n", __FILE__,
-      	(iTextureHandle*)(awstxt->tex));
+      csPrintf ("aws-debug: (%s) texture is: %p\n", __FILE__,
+      	(void*)awstxt->tex);
 
     if (awstxt && id == awstxt->id)
     {
@@ -128,20 +155,19 @@ iTextureHandle *awsTextureManager::GetTexturebyID (
   if (!txtfound && filename == 0) return 0;
   if (!txtfound) awstxt = 0;
 
-  /*  If we have arrived here, then we know that the texture does not exist in the cache.
-  * Therefore, we will now attempt to load it from the disk, register it, and pass back a handle.
-  * If this fails, we'll return 0.
-  */
+  // If we have arrived here, then we know that the texture does not exist in
+  // the cache.  Therefore, we will now attempt to load it from the disk,
+  // register it, and pass back a handle.  If this fails, we'll return 0.
   if (DEBUG_GETTEX)
   {
     if (txtmgr == 0)
-      printf (
+      csPrintf (
         "aws-debug: GetTexturebyID (%s) no texture manager.\n",
         __FILE__);
     if (vfs == 0)
-      printf ("aws-debug: GetTexturebyID (%s) no vfs.\n", __FILE__);
+      csPrintf ("aws-debug: GetTexturebyID (%s) no vfs.\n", __FILE__);
     if (loader == 0)
-      printf ("aws-debug: GetTexturebyID (%s) no loader.\n", __FILE__);
+      csPrintf ("aws-debug: GetTexturebyID (%s) no loader.\n", __FILE__);
 
     if (!txtmgr || !vfs || !loader) return 0;
   }
@@ -149,38 +175,37 @@ iTextureHandle *awsTextureManager::GetTexturebyID (
   int Format = txtmgr->GetTextureFormat ();
 
   csRef<iImage> ifile;
-  csRef<iDataBuffer> buf (vfs->ReadFile (filename));
+  csRef<iDataBuffer> buf (vfs->ReadFile (filename, false));
 
   if (buf == 0 || buf->GetSize () == 0)
   {
     csReport (
       object_reg,
       CS_REPORTER_SEVERITY_ERROR,
-      "crystalspace.awsprefs",
+      "crystalspace.awstex",
       "Could not open image file '%s' on VFS!",
       filename);
 
     return 0;
   }
 
-  ifile = loader->Load (buf->GetUint8 (), buf->GetSize (), Format);
+  ifile = loader->Load (buf, Format);
 
   if (!ifile)
   {
     csReport (
       object_reg,
       CS_REPORTER_SEVERITY_ERROR,
-      "crystalspace.awsprefs",
+      "crystalspace.awstex",
       "Could not load image '%s'. Unknown format or wrong extension!",
       filename);
 
     return 0;
   }
 
-  /*  At this point, we have loaded the file from the disk, and all we're doing now is creating
-  * a texture handle to the image.  The texture handle is necessary to draw a pixmap with
-  * iGraphics3D::DrawPixmap()
-  */
+  // At this point, we have loaded the file from the disk, and all we're doing
+  // now is creating a texture handle to the image.  The texture handle is
+  // necessary to draw a pixmap with iGraphics3D::DrawPixmap()
   if (awstxt == 0)
   {
     awstxt = new awsTexture;
@@ -188,14 +213,13 @@ iTextureHandle *awsTextureManager::GetTexturebyID (
   }
 
   awstxt->img = ifile;
-  awstxt->tex = txtmgr->RegisterTexture (
-      ifile,
+  awstxt->tex = txtmgr->RegisterTexture (ifile,
       CS_TEXTURE_2D | CS_TEXTURE_3D | CS_TEXTURE_NOMIPMAPS);
   awstxt->id = id;
 
   // Post load work...
   awstxt->tex->SetKeyColor (key_r, key_g, key_b);
-  awstxt->tex->Prepare ();
+  awstxt->tex->SetTextureClass( "nocompress" );
 
   textures.Push (awstxt);
 
