@@ -47,13 +47,13 @@ private:
   int alpha_level;
 
   /// Text of this component, editable.
-  iString *text;
+  csRef<iString> text;
 
   /// Text that's not allowed to add.
-  iString *disallow;
+  csRef<iString> disallow;
 
   /// Character to replace text with.
-  iString *maskchar;
+  csRef<iString> maskchar;
 
   /// Position of first character we display.
   //int start;
@@ -111,10 +111,10 @@ public:
   virtual bool Setup (iAws *wmgr, iAwsComponentNode *settings);
 
   /// Get properties.
-  bool GetProperty (const char *name, void **parm);
+  bool GetProperty (const char *name, intptr_t *parm);
 
   /// Set properties
-  bool SetProperty (const char *name, void *parm);
+  bool SetProperty (const char *name, intptr_t parm);
 
   /// Returns the named TYPE of the component, like "Radio Button", etc.
   virtual const char *Type ();
