@@ -1,4 +1,23 @@
+/*
+    Copyright (C) 2001 by Christopher Nelson
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #include "cssysdef.h"
+#include "csgeom/vector4.h"
 #include "awswin.h"
 #include "awslayot.h"
 #include "ivideo/graph2d.h"
@@ -44,16 +63,17 @@ const int awsWindow:: foDontCaptureMouseMove = 0x400;
 
 const int grip_size = 16;
 
-// Set to true to get printf info about events, false to disable them.
+// Set to true to get csPrintf info about events, false to disable them.
 const bool DEBUG_WINDOW_EVENTS = false;
 
 awsWindow::awsWindow () :
-  frame_options(foControl | foZoom | foClose | foTitle | foGrip | foRoundBorder),
+  frame_options(foControl | foZoom | foClose | foTitle | foGrip |
+		foRoundBorder),
   title_bar_height(0),
   title(0),
   resizing_mode(false),
   moving_mode(false),
-  sink(),
+  sink(0),
   is_minimized(false),
   popup(0),
   menu(0),
@@ -63,14 +83,11 @@ awsWindow::awsWindow () :
   SetFlag (AWSF_CMP_HIDDEN);
   SetFlag (AWSF_CMP_WINDOW);
   SetFlag (AWSF_CMP_TOP_SELECT);
-
-  sink.SetParm (this);
 }
 
 awsWindow::~awsWindow ()
 {
-  if (title)
-    title->DecRef();
+  delete sink;
 }
 
 bool awsWindow::Setup (iAws *_wmgr, iAwsComponentNode *settings)
@@ -87,45 +104,46 @@ bool awsWindow::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   if(style == fsBitmap)
     frame_options = 0;
 
+  title.AttachNew (new scfString (""));
   pm->GetInt (settings, "Options", frame_options);
   pm->GetString (settings, "Title", title);
  
-  // We have to incref this or else if our title is changed we release the copy in the prefmanager!
-  if (title)
-    title->IncRef();
   pm->LookupIntKey ("TitleBarHeight", title_bar_height);
 
   unsigned char red, green, blue;
   if(pm->LookupRGBKey ("TitleBarTextColor", red, green, blue))
-	  title_text_color = pm->FindColor(red, green, blue);
+    title_text_color = pm->FindColor(red, green, blue);
   else
-	  title_text_color = pm->GetColor(AC_TEXTFORE);
+    title_text_color = pm->GetColor(AC_TEXTFORE);
 
   // setup the colors for the title bar
+  for (int i = 0; i < 12; i++)
+    title_color[i] = 128;
 
-  for(int i = 0; i < 12; i++) title_color[i] = 128;
-
-  pm->LookupRGBKey("ActiveTitleBarColor1", title_color[0], title_color[1], title_color[2]);
-  if(!pm->LookupRGBKey("ActiveTitleBarColor2", title_color[3], title_color[4], title_color[5]))
+  pm->LookupRGBKey("ActiveTitleBarColor1", title_color[0], title_color[1],
+		   title_color[2]);
+  if(!pm->LookupRGBKey("ActiveTitleBarColor2", title_color[3], title_color[4],
+		       title_color[5]))
   {
-	  title_color[3] = title_color[0];
-	  title_color[4] = title_color[1];
-	  title_color[5] = title_color[2];
+    title_color[3] = title_color[0];
+    title_color[4] = title_color[1];
+    title_color[5] = title_color[2];
   }
 
-  pm->LookupRGBKey("InactiveTitleBarColor1", title_color[6], title_color[7], title_color[8]);
-  if(!pm->LookupRGBKey("InactiveTitleBarColor2", title_color[9], title_color[10], title_color[11]))
+  pm->LookupRGBKey("InactiveTitleBarColor1", title_color[6], title_color[7],
+		   title_color[8]);
+  if(!pm->LookupRGBKey("InactiveTitleBarColor2", title_color[9],
+		       title_color[10], title_color[11]))
   {
-	  title_color[9] = title_color[6];
-	  title_color[10] = title_color[7];
-	  title_color[11] = title_color[8];
+    title_color[9]  = title_color[6];
+    title_color[10] = title_color[7];
+    title_color[11] = title_color[8];
   }
 
   // setup title bar height and offset
+  int tw = 0, th = 0;
 
-  int tw=0, th=0;
-
-    // Get the size of the text
+  // Get the size of the text
   WindowManager ()->GetPrefMgr ()->GetDefaultFont ()->GetMaxSize (tw, th);
   // Get a good offset
   title_offset = th >> 1;
@@ -135,10 +153,12 @@ bool awsWindow::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   title_bar_height = MAX(th + 3, title_bar_height);
 
   // register triggers with our sink
-  sink.RegisterTrigger("Close", &OnCloseClick);
-  sink.RegisterTrigger("Zoom", &OnZoomClick);
-  sink.RegisterTrigger("Min", &OnMinClick);
+  sink = new awsSink(WindowManager()->GetStringTable());
+  sink->SetParm ((intptr_t)this);
 
+  sink->RegisterTrigger("Close", &OnCloseClick);
+  sink->RegisterTrigger("Zoom", &OnZoomClick);
+  sink->RegisterTrigger("Min", &OnMinClick);
 
   // setup the control buttons
   iString* close_button_txt;
@@ -163,8 +183,7 @@ bool awsWindow::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   closeinfo->DecRef();
 
   slot_close.Connect(&close_button, awsCmdButton::signalClicked, 
-    &sink, sink.GetTriggerID("Close"));
-  
+    sink, sink->GetTriggerID("Close"));
   
   iString* zoom_button_txt;
   pm->LookupStringKey("WindowZoom", zoom_button_txt);
@@ -188,10 +207,8 @@ bool awsWindow::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   zoominfo->DecRef();
 
   slot_zoom.Connect(&zoom_button, awsCmdButton::signalClicked, 
-    &sink, sink.GetTriggerID("Zoom"));
-  
-  
-  
+    sink, sink->GetTriggerID("Zoom"));
+
   iString* min_button_txt;
   pm->LookupStringKey("WindowMin", min_button_txt);
   
@@ -212,84 +229,79 @@ bool awsWindow::Setup (iAws *_wmgr, iAwsComponentNode *settings)
   min_button.Create(WindowManager(), this, mininfo->GetThisNode());
   mininfo->DecRef();
   min_button.ResizeTo(minp);
-  
 
   slot_min.Connect(&min_button, awsCmdButton::signalClicked, 
-    &sink, sink.GetTriggerID("Min"));
+    sink, sink->GetTriggerID("Min"));
   
   // Hide any of the undesired controls
-  if(~frame_options & foClose)
+  if ((~frame_options & foClose) != 0)
     close_button.Hide();
   
-
-  if(~frame_options & foZoom)
+  if ((~frame_options & foZoom) != 0)
     zoom_button.Hide();
-  
 
-  if(~frame_options & foMin)
+  if ((~frame_options & foMin) != 0)
     min_button.Hide();
 
   return true;
 }
 
-bool awsWindow::GetProperty (const char *name, void **parm)
+bool awsWindow::GetProperty (const char *name, intptr_t *parm)
 {
-  if (awsComponent::GetProperty (name, parm)) return true;
+  if (awsComponent::GetProperty (name, parm))
+    return true;
 
   if (strcmp ("Title", name) == 0)
   {
-    char *st = 0;
-
-    if (title) st = title->GetData ();
-
+    const char *st = 0;
+    if (title)
+      st = title->GetData ();
     iString *s = new scfString (st);
-    *parm = (void *)s;
+    *parm = (intptr_t)s;
     return true;
   }
   else if( strcmp("Active", name) == 0)
   {
-    *parm = (void*) IsActiveWindow();
+    *parm = IsActiveWindow();
     return true;
   }
   else if( strcmp("PopupMenu", name) == 0)
   {
-    *parm = popup;
+    *parm = (intptr_t)popup;
     return true;
   }
   else if( strcmp("Menu", name) == 0)
   {
-    *parm = menu;
+    *parm = (intptr_t)menu;
     return true;
   }
 
   return false;
 }
 
-bool awsWindow::SetProperty (const char *name, void *parm)
+bool awsWindow::SetProperty (const char *name, intptr_t parm)
 {
-  if (awsComponent::SetProperty (name, parm)) return true;
+  if (awsComponent::SetProperty (name, parm))
+    return true;
 
   if (strcmp ("Title", name) == 0)
   {
     iString *t = (iString *) (parm);
     if (t)
     {
-      title->DecRef ();
-      title = new scfString (t->GetData ());
-
+      title->Replace (t);
       Invalidate ();
     }
-
     return true;
   }
-  else if(strcmp("PopupMenu", name) == 0)
+  else if (strcmp("PopupMenu", name) == 0)
   {
-    if(popup) popup->DecRef();
+    if (popup) popup->DecRef();
     popup = (awsPopupMenu*) parm;
-    if(popup) popup->IncRef();
+    if (popup) popup->IncRef();
     return true;
   }
-  else if(strcmp("Menu", name) == 0)
+  else if (strcmp("Menu", name) == 0)
   {
     SetMenu((awsMenuBar*)parm);
     return true;
@@ -300,9 +312,7 @@ bool awsWindow::SetProperty (const char *name, void *parm)
 
 bool awsWindow::Execute (const char *action, iAwsParmList* parmlist)
 {
-  if (awsComponent::Execute (action, parmlist)) return true;
-
-  return false;
+  return awsComponent::Execute (action, parmlist);
 }
 
 void awsWindow::Show ()
@@ -312,10 +322,10 @@ void awsWindow::Show ()
   // Focusing last focused window child
   iAwsComponent *comp = GetFocusedChild ();
 
-  if(!comp)
-   comp = GetFirstFocusableChild(this);
+  if (!comp)
+    comp = GetFirstFocusableChild(this);
 
-  if(comp)
+  if (comp)
   {
     WindowManager ()->SetFocusedComponent (comp);
     comp->SetFocus();
@@ -327,57 +337,55 @@ void awsWindow::Show ()
 void awsWindow::Hide ()
 {
   awsPanel::Hide ();
-
   // Save last focused component
   SetFocusedChild (WindowManager ()->GetFocusedComponent());
-
   Broadcast (sWindowHidden);
 }
 
 bool awsWindow::IsActiveWindow()
 {
   // check to see if there are any sibbling windows above this one
-    iAwsComponent* cmp = ComponentAbove();
-    while(cmp)
-    {
-      if(cmp->Flags() & AWSF_CMP_WINDOW)
-        return false;
-      cmp = cmp->ComponentAbove();
-    }
+  iAwsComponent* cmp = ComponentAbove();
+  while (cmp)
+  {
+    if (cmp->Flags() & AWSF_CMP_WINDOW)
+      return false;
+    cmp = cmp->ComponentAbove();
+  }
 
-    // check to see if our Parent window is active, if we have one
-    if(Parent())
-    {
-      bool active = false;
-      Parent()->Window()->GetProperty("Active", (void**) &active);
-      return active;
-    }
-
-    return true;
+  // check to see if our Parent window is active, if we have one
+  if (Parent())
+  {
+    bool active = false;
+    Parent()->Window()->GetProperty("Active", (intptr_t*)&active);
+    return active;
+  }
+  return true;
 }
 
 
 void awsWindow::SetMenu(awsMenuBar* _menu)
 {
-  if(menu)
+  if (menu)
   {
     menu->DecRef();
     RemoveChild(menu);
     Invalidate();
   }
   menu = _menu;
-  if(menu)
+  if (menu)
   {
     menu->IncRef();
     AddChild(menu);
     menu->SetFlag(AWSF_CMP_NON_CLIENT);
 
     csRect insets = frame_drawer.GetInsets(style);
-    if(frame_options & foTitle)
+    if (frame_options & foTitle)
       insets.ymin += title_bar_height;
 
     menu->MoveTo(Frame().xmin + insets.xmin, Frame().ymin + insets.ymin);
-    menu->Resize(Frame().Width() - insets.xmin - insets.xmax, menu->Frame().Height());
+    menu->Resize(Frame().Width() - insets.xmin - insets.xmax,
+		 menu->Frame().Height());
     menu->Show();
 
     Invalidate();
@@ -391,25 +399,23 @@ awsMenuBar* awsWindow::GetMenu() { return menu; }
 void awsWindow::OnRaise ()
 {
   iAwsComponent *comp = GetFocusedChild ();
-  if(comp)
+  if (comp)
   {
-     WindowManager ()->SetFocusedComponent (comp);
-     comp->SetFocus();
+    WindowManager ()->SetFocusedComponent (comp);
+    comp->SetFocus();
   }
   Broadcast (sWindowRaised);
-  return ;
 }
 
 void awsWindow::OnLower ()
 {
   SetFocusedChild(WindowManager()->GetFocusedComponent());
   Broadcast (sWindowLowered);
-  return ;
 }
 
 bool awsWindow::OnMouseDown (int button, int x, int y)
 {
-  if(button == 2 && popup)
+  if (button == 1 && popup)
   {
     popup->MoveTo(x,y);
     popup->Show();
@@ -417,32 +423,31 @@ bool awsWindow::OnMouseDown (int button, int x, int y)
     popup->TrackMouse();
   }
 
-  if(style == fsBitmap || style == fsNone || style == fsFlat)
+  if (style == fsBitmap || style == fsNone || style == fsFlat)
     return false;
 
-  if (IsMaximized()) return false;
+  if (IsMaximized())
+    return false;
 
   down_x = x;
   down_y = y;
 
-  ///// Check for resizing
-    if (
-      (frame_options & foGrip) &&
+  // Check for resizing
+  if ((frame_options & foGrip) &&
       x < Frame ().xmax &&
       x > Frame ().xmax - grip_size &&
       y < Frame ().ymax &&
       y > Frame ().ymax - grip_size)
-    {
-      orig_x = Frame().Width();
-      orig_y = Frame().Height();
-      resizing_mode = true;
-      WindowManager ()->CaptureMouse (this);
-      return true;
-    }
+  {
+    orig_x = Frame().Width();
+    orig_y = Frame().Height();
+    resizing_mode = true;
+    WindowManager ()->CaptureMouse (this);
+    return true;
+  }
 
-    ///// Check for moving
-    else if (
-	!(frame_options & foNoDrag)
+  // Check for moving
+  else if (!(frame_options & foNoDrag)
 	&& (
           // Move using titlebar if it's a normal window
             (
@@ -458,20 +463,18 @@ bool awsWindow::OnMouseDown (int button, int x, int y)
           // Move using whole window frame if it's not
             (style != fsNormal || (frame_options & foBeveledBorder)))
 	)
-    {
-      orig_x = Frame().xmin;
-      orig_y = Frame().ymin;
-      moving_mode = true;
-      WindowManager ()->CaptureMouse (this);
-      return true;
-    }
-    return false;
+  {
+    orig_x = Frame().xmin;
+    orig_y = Frame().ymin;
+    moving_mode = true;
+    WindowManager ()->CaptureMouse (this);
+    return true;
+  }
+  return false;
 }
 
-bool awsWindow::OnMouseUp (int button, int , int )
+bool awsWindow::OnMouseUp (int /* button */, int, int)
 {
-  (void)button;
-
   if (resizing_mode || moving_mode)
   {
     resizing_mode = false;
@@ -479,7 +482,6 @@ bool awsWindow::OnMouseUp (int button, int , int )
     WindowManager ()->ReleaseMouse ();
     return true;
   }
-
   return false;
 }
 
@@ -493,14 +495,8 @@ bool awsWindow::OnMouseMove (int button, int x, int y)
   else if (moving_mode)
     MoveTo(orig_x + x - down_x, orig_y + y - down_y);
 
-  //
-  //  If we are set to not capture mouse movements
-  //  return false here. 
-  //
-  if (frame_options & foDontCaptureMouseMove)
-    return false;
- 
-  return true;
+  // If we are set to not capture mouse movements return false here. 
+  return (frame_options & foDontCaptureMouseMove) == 0;
 }
 
 void awsWindow::OnDraw (csRect clip)
@@ -508,7 +504,7 @@ void awsWindow::OnDraw (csRect clip)
   iGraphics2D *g2d = WindowManager ()->G2D ();
 
   awsPanel::OnDraw(clip);
-  if(style == fsNormal && !(frame_options & foNoBorder))
+  if (style == fsNormal && !(frame_options & foNoBorder))
   {
     csRect r = Frame();
     csRect insets = frame_drawer.GetInsets(fsNormal);
@@ -520,16 +516,15 @@ void awsWindow::OnDraw (csRect clip)
     r.ymax -= insets.ymax;
     csRectRegion reg;
     reg.makeEmpty();
-    if(!r.IsEmpty())
+    if (!r.IsEmpty())
       frame_drawer.Draw(r, fsSunken, Frame(), Frame(), &reg);
   }
 
-
-  if (frame_options & foTitle)
+  if ((frame_options & foTitle) != 0)
   {
     const int step = 6;
     
-    if(style != fsBitmap)
+    if (style != fsBitmap)
     {
       csRect title_frame = Frame();
       csRect insets = frame_drawer.GetInsets(style);
@@ -541,40 +536,46 @@ void awsWindow::OnDraw (csRect clip)
       // if the title is active
       if (IsActiveWindow())
       {
-        DrawGradient(title_frame, title_color[0], title_color[1], title_color[2],
-          title_color[3], title_color[4], title_color[5]);
+        DrawGradient(title_frame,
+		     title_color[0],
+		     title_color[1],
+		     title_color[2],
+		     title_color[3],
+		     title_color[4],
+		     title_color[5]);
       }
       else
       {
-        DrawGradient(title_frame, title_color[6], title_color[7], title_color[8],
-          title_color[9], title_color[10], title_color[11]);
+        DrawGradient(title_frame,
+		     title_color[6],
+		     title_color[7],
+		     title_color[8],
+		     title_color[9],
+		     title_color[10],
+		     title_color[11]);
       }
-      
     }
+
     if (title)
     {
-      // find how far to the right can we write
-      // the title bar
+      // find how far to the right can we write the title bar
       int right_border = ClientFrame().xmax;
-      if(frame_options & foMin)
+      if (frame_options & foMin)
         right_border = MIN(min_button.Frame().xmin, right_border);
-      if(frame_options & foZoom)
+      if (frame_options & foZoom)
         right_border = MIN(zoom_button.Frame().xmin, right_border);
-      if(frame_options & foClose)
+      if (frame_options & foClose)
         right_border = MIN(close_button.Frame().xmin, right_border);
       
-      
-      
-      int mcc = WindowManager ()->GetPrefMgr ()->GetDefaultFont ()
-        ->GetLength (title->GetData (), right_border - ClientFrame().xmin - 10);
+      int mcc = WindowManager ()->GetPrefMgr ()->GetDefaultFont ()->
+        GetLength (title->GetData (), right_border - ClientFrame().xmin - 10);
       
       scfString tmp (title->GetData ());
       tmp.Truncate (mcc);
-      
       if(mcc < (int) title->Length())
       {
         // set the last 3 characters to ...
-        for(unsigned int i = MAX(0, (int)tmp.Length() - 3); i < tmp.Length(); i++)
+        for(unsigned int i = MAX(0, (int)tmp.Length()-3); i<tmp.Length(); i++)
           tmp.SetAt(i, '.');
       }
       
@@ -588,54 +589,66 @@ void awsWindow::OnDraw (csRect clip)
         tmp.GetData ());
     }
   }   // end if title bar
-		
 }
 
-void awsWindow::DrawGradient(csRect frame, unsigned char r1, unsigned char g1, unsigned char b1,
-							 unsigned char r2, unsigned char g2, unsigned char b2)
+void awsWindow::DrawGradient(csRect frame,
+			     unsigned char r1,
+			     unsigned char g1,
+			     unsigned char b1,
+			     unsigned char r2,
+			     unsigned char g2,
+			     unsigned char b2)
 {
-	iGraphics2D *g2d = WindowManager()->G2D();
-    iAwsPrefManager *pm = WindowManager()->GetPrefMgr();
+  csVector4 c1 (r1 / 255.0f, g1 / 255.0f, b1 / 255.0f, 1.0f);
+  csVector4 c2 (r2 / 255.0f, g2 / 255.0f, b2 / 255.0f, 1.0f);
 
-	float r_step, g_step, b_step;
-    r_step = (float)(r2 - r1) / frame.Width();
-	g_step = (float)(g2 - g1) / frame.Width();
-	b_step = (float)(b2 - b1) / frame.Width();
+  csSimpleRenderMesh mesh;
+  csVector3 verts[4];
+  csVector4 colors[4];
 
-	for(int i = 0; i < frame.Width(); i++)
-	{
-		int color = pm->FindColor(r1 + (unsigned char)(i*r_step),
-			                      g1 + (unsigned char)(i*g_step),
-								  b1 + (unsigned char)(i*b_step));
-		g2d->DrawLine(frame.xmin + i, frame.ymin, frame.xmin + i, frame.ymax, color);
-	}
+  mesh.meshtype = CS_MESHTYPE_QUADS;
+  mesh.vertexCount = 4;
+  mesh.vertices = verts;
+  mesh.colors = colors;
+  mesh.texture = 0;
+
+  verts[0].Set (frame.xmin, frame.ymin, 0.0f);
+  colors[0] = c1;
+  
+  verts[1].Set (frame.xmax, frame.ymin, 0.0f);
+  colors[1] = c2;
+
+  verts[2].Set (frame.xmax, frame.ymax, 0.0f);
+  colors[2] = c2;
+
+  verts[3].Set (frame.xmin, frame.ymax, 0.0f);
+  colors[3] = c1;
+
+  WindowManager()->G3D()->DrawSimpleMesh (mesh, csSimpleMeshScreenspace);
 }
 
 void awsWindow::SetOptions(int o)
 {
-	frame_options = o;
+  frame_options = o;
 }
 
 void awsWindow::Resize(int width, int height)
 {
-	// don't shrink too far
+  // don't shrink too far
   csRect insets = frame_drawer.GetInsets(style);
-  if(frame_options & foTitle) insets.ymin += title_bar_height;
+  if (frame_options & foTitle) insets.ymin += title_bar_height;
   int min_height = insets.ymin + insets.ymax;
-	
-	int right_border = ClientFrame().xmax;
-        if(frame_options & foMin)
-			right_border = MIN(min_button.Frame().xmin, right_border);
-		if(frame_options & foZoom)
-            right_border = MIN(zoom_button.Frame().xmin, right_border);
-		if(frame_options & foClose)
-			right_border = MIN(close_button.Frame().xmin, right_border);
-	int min_width = (Frame().xmax - right_border) + insets.xmin; 
+  int right_border = ClientFrame().xmax;
+  if (frame_options & foMin)
+    right_border = MIN(min_button.Frame().xmin, right_border);
+  if (frame_options & foZoom)
+    right_border = MIN(zoom_button.Frame().xmin, right_border);
+  if (frame_options & foClose)
+    right_border = MIN(close_button.Frame().xmin, right_border);
+  int min_width = (Frame().xmax - right_border) + insets.xmin; 
 
-
-
-	width = MAX(width, min_width);
-	height = MAX(height, min_height);
+  width = MAX(width, min_width);
+  height = MAX(height, min_height);
 
   int delta_x = width - Frame().Width();
   min_button.Move(delta_x, 0);
@@ -643,8 +656,7 @@ void awsWindow::Resize(int width, int height)
   close_button.Move(delta_x, 0);
 
   // resize the menu bar
-  
-  if(menu)
+  if (menu)
   {
     insets = frame_drawer.GetInsets(style);
     menu->SizeToFitVert();
@@ -652,9 +664,8 @@ void awsWindow::Resize(int width, int height)
       MIN(menu->Frame().Height(), height - min_height));
   }
 
-	// change size
-	awsComponent::Resize(width, height);
-
+  // change size
+  awsComponent::Resize(width, height);
 }
 
 iAwsComponent *awsWindow::GetFocusedChild ()
@@ -675,11 +686,11 @@ csRect awsWindow::getMinimumSize ()
 csRect awsWindow::getInsets ()
 {
   csRect r = awsPanel::getInsets();
-  if(frame_options & foTitle)
+  if (frame_options & foTitle)
     r.ymin += title_bar_height;
-  if(menu)
+  if (menu)
     r.ymin += menu->Frame().Height();
-  if(style == fsNormal && !(frame_options & foNoBorder))
+  if (style == fsNormal && !(frame_options & foNoBorder))
   {
     csRect more_insets = frame_drawer.GetInsets(fsSunken);
     r.xmin += more_insets.xmin;
@@ -687,7 +698,6 @@ csRect awsWindow::getInsets ()
     r.xmax += more_insets.xmax;
     r.ymax += more_insets.ymax;
   }
-
   return r;
 }
 
@@ -696,12 +706,12 @@ bool awsWindow::IsMoving()
   return moving_mode;
 }
 
-void awsWindow::OnCloseClick(void *p, iAwsSource *)
+void awsWindow::OnCloseClick(unsigned long, intptr_t p, iAwsSource *)
 {
   ((iAwsComponent*)p)->Broadcast(sWindowClosed);
 }
 
-void awsWindow::OnZoomClick(void *p, iAwsSource *)
+void awsWindow::OnZoomClick(unsigned long, intptr_t p, iAwsSource *)
 {
   iAwsComponent* comp = (iAwsComponent*)p;
   if(comp->IsMaximized())
@@ -710,7 +720,7 @@ void awsWindow::OnZoomClick(void *p, iAwsSource *)
     comp->Maximize();
 }
 
-void awsWindow::OnMinClick(void *p, iAwsSource *)
+void awsWindow::OnMinClick(unsigned long, intptr_t p, iAwsSource *)
 {
   ((iAwsComponent*)p)->Broadcast(sWindowMinimized);
 }
@@ -747,8 +757,8 @@ awsComponentFactory(wmgr)
   RegisterConstant ("wfoRoundBorder", awsWindow::foRoundBorder);
   RegisterConstant ("wfoBeveledBorder", awsWindow::foBeveledBorder);
   RegisterConstant ("wfoNoBorder", awsWindow::foNoBorder);
-  RegisterConstant ("wfoDontCaptureMouseMove", awsWindow::foDontCaptureMouseMove);
-
+  RegisterConstant ("wfoDontCaptureMouseMove",
+		    awsWindow::foDontCaptureMouseMove);
 }
 
 awsWindowFactory::~awsWindowFactory()
@@ -757,7 +767,5 @@ awsWindowFactory::~awsWindowFactory()
 
 iAwsComponent* awsWindowFactory::Create()
 {
-	return (new awsWindow())->GetComponent();
+  return (new awsWindow())->GetComponent();
 }
-
-
