@@ -39,7 +39,6 @@
  * being peers, also have a depth ordering that is implicit in their
  * hierarchy.
  */
-
 class awsWindow : public awsPanel
 {
 private:
@@ -53,7 +52,7 @@ private:
   int title_offset;
 
   /// The title.
-  iString *title;
+  csRef<iString> title;
 
   /// The title bar text color.
   int title_text_color;
@@ -87,7 +86,7 @@ private:
 
   /// Slots and a sink for getting events from the buttons.
   awsSlot slot_min, slot_zoom, slot_close;
-  awsSink sink;
+  awsSink* sink;
 
   /// True if the window is in one of these various states.
   bool is_minimized;
@@ -189,10 +188,10 @@ public:
   virtual bool Setup (iAws *_wmgr, iAwsComponentNode *settings);
 
   /// Gets properties for this window.
-  bool GetProperty (const char *name, void **parm);
+  bool GetProperty (const char *name, intptr_t *parm);
 
   /// Sets properties for this window.
-  bool SetProperty (const char *name, void *parm);
+  bool SetProperty (const char *name, intptr_t parm);
 
   /// Executes scriptable actions for this window.
   bool Execute (const char *action, iAwsParmList* parmlist);
@@ -215,9 +214,9 @@ public:
   virtual void SetMenu (awsMenuBar* menu);
   virtual awsMenuBar* GetMenu ();
 
-  static void OnCloseClick (void* p, iAwsSource* source);
-  static void OnZoomClick (void* p, iAwsSource* source);
-  static void OnMinClick (void* p, iAwsSource* source);
+  static void OnCloseClick (unsigned long, intptr_t p, iAwsSource* source);
+  static void OnZoomClick (unsigned long, intptr_t p, iAwsSource* source);
+  static void OnMinClick (unsigned long, intptr_t p, iAwsSource* source);
 
   /// Event triggered when a window is about to be raised.
   virtual void OnRaise ();
