@@ -19,12 +19,16 @@
 #ifndef __CS_SHADOW_H__
 #define __CS_SHADOW_H__
 
-#include "imesh/object.h"
 #include "csgeom/box.h"
+#include "csutil/flags.h"
 #include "igeom/objmodel.h"
+#include "imesh/object.h"
+#include "ivideo/graph3d.h"
 
 struct iEngine;
 struct iMeshWrapper;
+struct iCamera;
+
 
 /**
  * BugPlug is the hiding place for many dark creatures. While Spider only
@@ -33,33 +37,39 @@ struct iMeshWrapper;
  * to it with all its power. Unless Shadow willingly releases the creature
  * there is nothing that can be done to unbind Shadow.
  * The Shadow slowly devours the poor creature from all its knowledge.
- * <p>
+ *
  * For the more technically minded:
- * <ul>
- * <li>Shadow: mesh object
- * <li>Creature: another mesh object
- * </ul>
+ * - Shadow: mesh object
+ * - Creature: another mesh object
+ *
  * This mesh object follows another mesh object and it will render a bounding
  * box for that object.
  */
 class csShadow : public iMeshObject
 {
 private:
-  iBase* logparent;
+  iMeshWrapper* logparent;
   iMeshWrapper* wrap;
-  iMeshWrapper* shadow_mesh;
   bool do_bbox;	// Show bounding box.
   bool do_rad;	// Show bounding sphere.
-  bool do_beam; // Show the intersection beam.
-  csVector3 beam[2];
-  csVector3 isec;
   csFlags flags;
+  iCamera* keep_camera;
 
 public:
 
   csShadow ();
   virtual ~csShadow ();
 
+  /**
+   * Get renderview found when rendering the shadow mesh.
+   */
+  iCamera* GetCamera () const { return keep_camera; }
+
+  /**
+   * Clear renderview.
+   */
+  void ClearCamera () { keep_camera = 0; }
+
   /**
    * Add Shadow to the engine.
    */
@@ -70,44 +80,30 @@ public:
    */
   void RemoveFromEngine (iEngine* engine);
 
-  /**
-   * Set the mesh that we are shadowing.
-   */
-  void SetShadowMesh (iMeshWrapper* sh);
-
   /**
    * Set what we are showing.
    */
-  void SetShowOptions (bool bbox, bool rad, bool beam)
+  void SetShowOptions (bool bbox, bool rad)
   {
     do_bbox = bbox;
     do_rad = rad;
-    do_beam = beam;
   }
 
   /**
    * Get what we are showing.
    */
-  void GetShowOptions (bool& bbox, bool& rad, bool& beam) const
+  void GetShowOptions (bool& bbox, bool& rad) const
   {
     bbox = do_bbox;
     rad = do_rad;
-    beam = do_beam;
   }
 
-  void SetBeam (csVector3& start, csVector3& finish, csVector3& intersect)
+  void GetObjectBoundingBox (csBox3& bbox)
   {
-    beam[0] = start;
-    beam[1] = finish;
-    isec = intersect;
-  }
-
-  void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL)
-  {
-    (void)type;
     bbox.Set (-100000, -100000, -100000, 100000, 100000, 100000);
     return;
   }
+  void SetObjectBoundingBox (const csBox3&) { }
   void GetRadius (csVector3& rad, csVector3& cent)
   {
      rad.Set (200000, 200000, 200000);
@@ -119,10 +115,8 @@ public:
   virtual iMeshObjectFactory* GetFactory () const { return 0; }
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone () { return 0; }
-  virtual bool DrawTest (iRenderView* rview, iMovable* movable, uint32);
-  virtual bool Draw (iRenderView*, iMovable*, csZBufMode);
   virtual csRenderMesh** GetRenderMeshes (int& n, iRenderView* rview, 
-    iMovable* movable, uint32) { n = 0; return 0; }
+    iMovable* movable, uint32);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback*) { }
   virtual iMeshObjectDrawCallback* GetVisibleCallback () const { return 0; }
   virtual void NextFrame (csTicks, const csVector3& /*pos*/) { }
@@ -132,9 +126,9 @@ public:
         csVector3&, float*)
   { return false; }
   virtual bool HitBeamObject (const csVector3&, const csVector3&,
-  	csVector3&, float*, int* = 0) { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  	csVector3&, float*, int* = 0, iMaterialWrapper** = 0) { return false; }
+  virtual void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
+  virtual iMeshWrapper* GetMeshWrapper () const { return logparent; }
 
   //------------------------- iObjectModel implementation ----------------
   class ObjectModel : public iObjectModel
@@ -150,9 +144,13 @@ public:
     virtual void SetPolygonMeshShadows (iPolygonMesh*) { }
     virtual csPtr<iPolygonMesh> CreateLowerDetailPolygonMesh (float)
     { return 0; }
-    virtual void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL)
+    virtual void GetObjectBoundingBox (csBox3& bbox)
     {
-      scfParent->GetObjectBoundingBox (bbox, type);
+      scfParent->GetObjectBoundingBox (bbox);
+    }
+    virtual void SetObjectBoundingBox (const csBox3& bbox)
+    {
+      scfParent->SetObjectBoundingBox (bbox);
     }
     virtual void GetRadius (csVector3& rad, csVector3& cent)
     {
@@ -172,12 +170,14 @@ public:
   virtual bool GetColor (csColor&) const { return false; }
   virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
   virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
+  virtual void SetMixMode (uint) { }
+  virtual uint GetMixMode () const { return CS_FX_COPY; }
   virtual void InvalidateMaterialHandles () { }
   /**
    * see imesh/object.h for specification. The default implementation
    * does nothing.
    */
-  virtual void PositionChild (iMeshObject* child,csTicks current_time) { }
+  virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
 };
 
 #endif // __CS_SHADOW_H__
