@@ -1,5 +1,6 @@
 /*
     Copyright (C) 2000 by Norman Krmer
+              (C) 2004 by Marten Svanfeldt
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -16,29 +17,35 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include "cssysdef.h"
-#include "cssys/sysfunc.h"
-#include "fancycon.h"
+
+#include "csgeom/csrect.h"
+#include "csutil/cfgacc.h"
+#include "csutil/csstring.h"
+#include "csutil/event.h"
+#include "csutil/eventnames.h"
+#include "csutil/sysfunc.h"
+#include "igraphic/image.h"
+#include "igraphic/imageio.h"
+#include "iutil/comp.h"
+#include "iutil/databuff.h"
+#include "iutil/event.h"
+#include "iutil/eventh.h"
+#include "iutil/eventq.h"
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "iutil/vfs.h"
 #include "ivaria/reporter.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
-#include "ivideo/txtmgr.h"
 #include "ivideo/material.h"
-#include "iutil/vfs.h"
-#include "iutil/plugin.h"
-#include "csgeom/csrect.h"
-#include "csutil/csstring.h"
-#include "csutil/cfgacc.h"
-#include "igraphic/image.h"
-#include "igraphic/imageio.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/event.h"
-#include "iutil/eventq.h"
-#include "iutil/objreg.h"
+#include "ivideo/txtmgr.h"
+
+#include "fancycon.h"
+
 
 CS_IMPLEMENT_PLUGIN
 
@@ -57,23 +64,15 @@ SCF_IMPLEMENT_IBASE_END
 
 SCF_IMPLEMENT_FACTORY(csFancyConsole)
 
-SCF_EXPORT_CLASS_TABLE (fancycon)
-  SCF_EXPORT_CLASS_DEP (csFancyConsole, "crystalspace.console.output.fancy",
-    "Crystal Space fancy output console",
-    "crystalspace.kernel., crystalspace.graphics3d., crystalspace.graphics2d.")
-SCF_EXPORT_CLASS_TABLE_END
 
 
 void csFancyConsole::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  iReporter* rep = CS_QUERY_REGISTRY (object_reg, iReporter);
+  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
   if (rep)
-  {
     rep->ReportV (severity, "crystalspace.console.output.fancy", msg, arg);
-    rep->DecRef ();
-  }
   else
   {
     csPrintfV (msg, arg);
@@ -83,36 +82,25 @@ void csFancyConsole::Report (int severity, const char* msg, ...)
 }
 
 csFancyConsole::csFancyConsole (iBase *p) :
-  object_reg(0), VFS(0), base(0), G2D(0), G3D(0), ImageLoader(NULL), border_computed(false),
+  object_reg(0), border_computed(false),
   pix_loaded(false), system_ready(false), auto_update(true), visible(true)
 {
   SCF_CONSTRUCT_IBASE (p);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
-  scfiEventHandler = NULL;
+  scfiEventHandler = 0;
 }
 
 csFancyConsole::~csFancyConsole ()
 {
   if (scfiEventHandler)
   {
-    iEventQueue* q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+    csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
     if (q)
-    {
       q->RemoveListener (scfiEventHandler);
-      q->DecRef ();
-    }
     scfiEventHandler->DecRef ();
   }
-  if (ImageLoader)
-    ImageLoader->DecRef();
-  if (G2D)
-    G2D->DecRef ();
-  if (G3D)
-    G3D->DecRef ();
-  if (base)
-    base->DecRef ();
-  if (VFS)
-    VFS->DecRef ();
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
+  SCF_DESTRUCT_IBASE ();
 }
 
 bool csFancyConsole::Initialize (iObjectRegistry *object_reg)
@@ -126,9 +114,9 @@ bool csFancyConsole::Initialize (iObjectRegistry *object_reg)
   csConfigAccess ini(object_reg, "/config/fancycon.cfg");
   char const* baseclass = ini->GetStr("FancyConsole.General.Superclass",
     "crystalspace.console.output.standard");
-  iPluginManager* plugin_mgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
+  csRef<iPluginManager> plugin_mgr (
+  	CS_QUERY_REGISTRY (object_reg, iPluginManager));
   base = CS_LOAD_PLUGIN (plugin_mgr, baseclass, iConsoleOutput);
-  plugin_mgr->DecRef ();
   if (!base)
     return false;
 
@@ -136,18 +124,18 @@ bool csFancyConsole::Initialize (iObjectRegistry *object_reg)
   if (!G3D)
     return false;
   G2D = G3D->GetDriver2D ();
-  G2D->IncRef ();
 
-  ImageLoader = NULL;
+  ImageLoader = 0;
 
   // Tell event queue that we want to handle broadcast events
+  CS_INITIALIZE_SYSTEM_EVENT_SHORTCUTS (object_reg);
   if (!scfiEventHandler)
     scfiEventHandler = new EventHandler (this);
-  iEventQueue* q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
+  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
   if (q != 0)
   {
-    q->RegisterListener (scfiEventHandler, CSMASK_Broadcast);
-    q->DecRef ();
+    csEventID events[3] = { SystemOpen, SystemClose, CS_EVENTLIST_END };
+    q->RegisterListener (scfiEventHandler, events);
   }
 
   int x, y, w, h;
@@ -159,12 +147,8 @@ bool csFancyConsole::Initialize (iObjectRegistry *object_reg)
 
 bool csFancyConsole::HandleEvent (iEvent &Event)
 {
-  switch (Event.Type)
+  if (Event.Name == SystemOpen)
   {
-    case csevBroadcast:
-      switch (Event.Command.Code)
-      {
-        case cscmdSystemOpen:
           system_ready = true;
 	  if (!pix_loaded)
 	  {
@@ -173,11 +157,11 @@ bool csFancyConsole::HandleEvent (iEvent &Event)
 	    pix_loaded = true;
 	  }
           return true;
-        case cscmdSystemClose:
+  }
+  else if (Event.Name == SystemClose)
+  {
           system_ready = false;
           return true;
-      }
-      break;
   }
   return false;
 }
@@ -211,17 +195,17 @@ void csFancyConsole::Draw3D (csRect *oArea)
   bool btext, bgour;
   int i;
   long int zBuf;
-  G3DPolygonDPFX poly;
+  csSimpleRenderMesh mesh;
   if (!border_computed)
   {
     // determine what space left to draw the actual console
     memset (&bordersize, 0, sizeof (bordersize));
-    if (deco.border [0].mat)
-      deco.border [0].mat->GetTexture ()->GetMipMapDimensions(
-        0, bordersize.xmin, bordersize.ymin);
-    if (deco.border[4].mat)
-      deco.border [4].mat->GetTexture ()->GetMipMapDimensions(
-        0, bordersize.xmax, bordersize.ymax);
+    if (deco.border [0].txt)
+      deco.border [0].txt->GetRendererDimensions (
+        bordersize.xmin, bordersize.ymin);
+    if (deco.border[4].txt)
+      deco.border [4].txt->GetRendererDimensions (
+        bordersize.xmax, bordersize.ymax);
 
     SetTransparency (true); // Otherwise 2D-part will overdraw what we paint.
     border_computed = true;
@@ -230,17 +214,15 @@ void csFancyConsole::Draw3D (csRect *oArea)
   }
 
   zBuf = G3D->GetRenderState (G3DRENDERSTATE_ZBUFFERMODE);
-  btext = G3D->GetRenderState (G3DRENDERSTATE_TEXTUREMAPPINGENABLE);
-  bgour = G3D->GetRenderState (G3DRENDERSTATE_GOURAUDENABLE);
+  btext = (G3D->GetRenderState (G3DRENDERSTATE_TEXTUREMAPPINGENABLE) != 0);
+  bgour = (G3D->GetRenderState (G3DRENDERSTATE_GOURAUDENABLE) != 0);
 
   G3D->SetRenderState (G3DRENDERSTATE_ZBUFFERMODE, CS_ZBUF_NONE);
 
-  int height = G3D->GetHeight () - 1;
-
   // first draw the background
   // do we draw gouraud/flat or with texture ?
 
-  bool with_color = deco.bgnd.mat == NULL;
+  bool with_color = (deco.bgnd.txt == 0);
 
   csRect size (outersize);
   size.xmin +=  bordersize.xmin - deco.p2lx - deco.lx;
@@ -248,80 +230,100 @@ void csFancyConsole::Draw3D (csRect *oArea)
   size.ymin +=  bordersize.ymin - deco.p2ty - deco.ty;
   size.ymax += -bordersize.ymax + deco.p2by + deco.by;
 
-  poly.num = 4;
-  poly.vertices [0].x = size.xmin;
-  poly.vertices [0].y = height - size.ymin;
-  poly.vertices [1].x = size.xmax;
-  poly.vertices [1].y = height - size.ymin;
-  poly.vertices [2].x = size.xmax;
-  poly.vertices [2].y = height - size.ymax;
-  poly.vertices [3].x = size.xmin;
-  poly.vertices [3].y = height - size.ymax;
-  poly.use_fog = false;
+  static uint indices[4] = {0, 1, 2, 3};
+  csVector3 vertices[4];
+  csVector2 texels[4];
+  csVector4 colors[4];
+
+  vertices[0].Set (size.xmin, size.ymin, 0);
+  vertices[1].Set (size.xmax,  size.ymin, 0);
+  vertices[2].Set (size.xmax,  size.ymax, 0);
+  vertices[3].Set (size.xmin, size.ymax, 0);
+
 
   float u_stretch = 1.0, v_stretch = 1.0;
 
-  if (!with_color && !deco.bgnd.do_stretch)
+  if (!deco.bgnd.do_stretch)
   {
     int w, h;
-    deco.bgnd.mat->GetTexture ()->GetMipMapDimensions (0, w, h);
+    deco.bgnd.txt->GetRendererDimensions (w, h);
     u_stretch = ((float)(size.xmax - size.xmin)) / ((float)w);
     v_stretch = ((float)(size.ymax - size.ymin)) / ((float)h);
   }
 
-  poly.vertices [0].u = 0;
-  poly.vertices [0].v = 0;
-  poly.vertices [1].u = u_stretch;
-  poly.vertices [1].v = 0;
-  poly.vertices [2].u = u_stretch;
-  poly.vertices [2].v = v_stretch;
-  poly.vertices [3].u = 0;
-  poly.vertices [3].v = v_stretch;
+  texels [0].x = 0;
+  texels [0].y = 0;
+  texels [1].x = u_stretch;
+  texels [1].y = 0;
+  texels [2].x = u_stretch;
+  texels [2].y = v_stretch;
+  texels [3].x = 0;
+  texels [3].y = v_stretch;
 
-  for (i = 0; i < poly.num; i++)
+  float alpha = 1;
+  if (deco.bgnd.do_alpha)
   {
-    poly.vertices [i].r = ((float)deco.bgnd.kr) * (1 / 255.0);
-    poly.vertices [i].g = ((float)deco.bgnd.kg) * (1 / 255.0);
-    poly.vertices [i].b = ((float)deco.bgnd.kb) * (1 / 255.0);
-    poly.vertices [i].z = 1;
+    alpha = deco.bgnd.alpha;
+    mesh.mixmode = CS_FX_COPY | CS_FX_FLAT;
+    mesh.alphaType.autoAlphaMode = false;
+    mesh.alphaType.alphaType = csAlphaMode::alphaSmooth;
   }
+  else
+  {
+    mesh.mixmode = CS_FX_COPY | CS_FX_FLAT;
+  }
+  
 
-  poly.mat_handle = deco.bgnd.mat;
-  if (with_color)
-    G3D->SetRenderState (G3DRENDERSTATE_TEXTUREMAPPINGENABLE, false);
+  for (i = 0; i < 4; i++)
+  {
+    if (with_color)
+    {
+      colors [i].x = ((float)deco.bgnd.kr) * (1 / 255.0);
+      colors [i].y = ((float)deco.bgnd.kg) * (1 / 255.0);
+      colors [i].z = ((float)deco.bgnd.kb) * (1 / 255.0);
+    }
+    else
+      colors [i].x = colors [i].y = colors [i].z = 1.0f;
+    colors [i].w = alpha;
+  }
+  
+  mesh.vertices = vertices;
+  mesh.texcoords = texels;
+  mesh.indices = indices;
+  mesh.indexCount = 4;
+  mesh.vertexCount = 4;
 
-  float alpha = deco.bgnd.do_alpha ? deco.bgnd.alpha : 0.0;
+  if (!with_color)
+    mesh.texture = deco.bgnd.txt;
+  mesh.colors = colors;
 
-  poly.mixmode = CS_FX_SETALPHA (alpha) |
-    CS_FX_COPY | (with_color && deco.bgnd.do_keycolor ? CS_FX_GOURAUD : 0);
-  G3D->DrawPolygonFX (poly);
-
-  if (with_color)
-    G3D->SetRenderState (G3DRENDERSTATE_TEXTUREMAPPINGENABLE, true);
+  mesh.meshtype = CS_MESHTYPE_QUADS;
+  
+  G3D->DrawSimpleMesh (mesh, csSimpleMeshScreenspace);
 
   // draw the top left decoration
-  DrawBorder (outersize.xmin, height-outersize.ymin, bordersize.xmin,
+  DrawBorder (outersize.xmin, outersize.ymin, bordersize.xmin,
     bordersize.ymin, deco.border[0], 0);
   // draw the top decoration
-  DrawBorder (p2size.xmin-deco.p2lx, height-outersize.ymin,
+  DrawBorder (p2size.xmin-deco.p2lx, outersize.ymin,
     p2size.Width()+deco.p2lx+deco.p2rx, bordersize.ymin,  deco.border[1], 1);
   // draw the top right decoration
-  DrawBorder (p2size.xmax, height-outersize.ymin, bordersize.xmax,
+  DrawBorder (p2size.xmax, outersize.ymin, bordersize.xmax,
     bordersize.ymin, deco.border[2], 0);
   // draw the right decoration
-  DrawBorder (p2size.xmax, height-p2size.ymin+deco.p2ty, bordersize.xmax,
+  DrawBorder (p2size.xmax, p2size.ymin-deco.p2ty, bordersize.xmax,
     p2size.Height()+deco.p2by+deco.p2ty, deco.border[3], 2);
   // draw the bottom right decoration
-  DrawBorder (p2size.xmax, height-p2size.ymax, bordersize.xmax,
-    bordersize.ymax, deco.border[4], 0);
+  DrawBorder (p2size.xmax, p2size.ymax, bordersize.xmax,
+      bordersize.ymax, deco.border[4], 0);
   // draw the bottom decoration
-  DrawBorder (p2size.xmin-deco.p2lx, height-p2size.ymax,
+  DrawBorder (p2size.xmin-deco.p2lx, p2size.ymax,
     p2size.Width()+deco.p2lx+deco.p2rx, bordersize.ymax, deco.border[5], 3);
   // draw the bottom left decoration
-  DrawBorder (outersize.xmin, height-p2size.ymax, bordersize.xmin,
+  DrawBorder (outersize.xmin, p2size.ymax, bordersize.xmin,
     bordersize.ymax, deco.border[6], 0);
   // draw the left decoration
-  DrawBorder (outersize.xmin, height-p2size.ymin+deco.p2ty, bordersize.xmin,
+  DrawBorder (outersize.xmin, p2size.ymin-deco.p2ty, bordersize.xmin,
     p2size.Height()+deco.p2by+deco.p2ty, deco.border[7], 4);
 
   G3D->SetRenderState (G3DRENDERSTATE_ZBUFFERMODE, zBuf);
@@ -335,15 +337,19 @@ void csFancyConsole::Draw3D (csRect *oArea)
 void csFancyConsole::DrawBorder (int x, int y, int width, int height,
   ConDecoBorder &border, int align)
 {
-  if (border.mat)
+  if (border.txt)
   {
-    G3DPolygonDPFX poly;
+    csSimpleRenderMesh mesh;
+    static uint indices[4] = {0, 1, 2, 3};
+    csVector3 vertices[4];
+    csVector2 texels[4];
+    csVector4 colors[4];
     int i;
 
     float u_stretch = 1.0, v_stretch = 1.0;
     int w, h;
 
-    border.mat->GetTexture ()->GetMipMapDimensions (0, w, h);
+    border.txt->GetRendererDimensions (w, h);
     switch (align)
     {
       case 1:
@@ -356,7 +362,7 @@ void csFancyConsole::DrawBorder (int x, int y, int width, int height,
         w = width;
         break;
       case 3:
-        y -= MAX (0, height - h);
+        y += MAX (0, height - h);
         height = MIN (h, height);
         h = height;
         break;
@@ -372,42 +378,59 @@ void csFancyConsole::DrawBorder (int x, int y, int width, int height,
       v_stretch = ((float)height) / ((float)h);
     }
 
-    poly.num = 4;
-    poly.use_fog = false;
-    poly.vertices [0].u = 0;
-    poly.vertices [0].v = 0;
-    poly.vertices [1].u = u_stretch;
-    poly.vertices [1].v = 0;
-    poly.vertices [2].u = u_stretch;
-    poly.vertices [2].v = v_stretch;
-    poly.vertices [3].u = 0;
-    poly.vertices [3].v = v_stretch;
+    texels [0].x = 0;
+    texels [0].y = 0;
+    texels [1].x = u_stretch;
+    texels [1].y = 0;
+    texels [2].x = u_stretch;
+    texels [2].y = v_stretch;
+    texels [3].x = 0;
+    texels [3].y = v_stretch;
 
-    poly.vertices [0].x = x;
-    poly.vertices [0].y = y;
-    poly.vertices [1].x = x + width ;
-    poly.vertices [1].y = y;
-    poly.vertices [2].x = x + width;
-    poly.vertices [2].y = y - height;
-    poly.vertices [3].x = x;
-    poly.vertices [3].y = y - height;
+    vertices [0].x = x;
+    vertices [0].y = y;
+    vertices [0].z = 0;
+    vertices [1].x = x + width ;
+    vertices [1].y = y;
+    vertices [1].z = 0;
+    vertices [2].x = x + width;
+    vertices [2].y = y + height;
+    vertices [2].z = 0;
+    vertices [3].x = x;
+    vertices [3].y = y + height;
+    vertices [3].z = 0;
+
+    float alpha = 1;
+
+    if (border.do_alpha)
+    {
+      alpha = deco.bgnd.alpha;
+      mesh.alphaType.autoAlphaMode = false;
+      mesh.alphaType.alphaType = csAlphaMode::alphaSmooth;
+    }
+    
 
     for (i = 0; i < 4; i++)
     {
-      poly.vertices [i].x -= border.offx;
-      poly.vertices [i].y += border.offy;
-      poly.vertices [i].z = 1;
-      poly.vertices [i].r = 1;
-      poly.vertices [i].g = 1;
-      poly.vertices [i].b = 1;
+      vertices [i].x -= border.offx;
+      vertices [i].y += border.offy;
+      colors [i].x = 1;
+      colors [i].y = 1;
+      colors [i].z = 1;
+      colors [i].w = alpha;
     }
 
-    poly.mat_handle = border.mat;
+    mesh.texture = border.txt;
+    mesh.mixmode = CS_FX_COPY | CS_FX_FLAT;
+    
+    mesh.vertices = vertices;
+    mesh.texcoords = texels;
+    mesh.indices = indices;
+    mesh.indexCount = 4;
+    mesh.vertexCount = 4;
+    mesh.meshtype = CS_MESHTYPE_QUADS;
 
-    float alpha = border.do_alpha ? border.alpha : 0.0;
-    poly.mixmode = CS_FX_SETALPHA (alpha)
-      | (border.do_keycolor ? CS_FX_KEYCOLOR : 0);
-    G3D->DrawPolygonFX (poly);
+    G3D->DrawSimpleMesh (mesh, csSimpleMeshScreenspace);
   }
 }
 
@@ -450,41 +473,45 @@ void csFancyConsole::LoadPix ()
 
   const char* dir = ini->GetStr ("FancyConsole.General.Archive");
   const char* mountdir = ini->GetStr ("FancyConsole.General.Mount");
-  if (!dir || !mountdir)
+  if (!*mountdir)
     Report (CS_REPORTER_SEVERITY_WARNING,
       "FancyConsole: Data resource location unknown");
-  else if (VFS->Mount (mountdir, dir))
+  else 
   {
-    VFS->PushDir ();
-    VFS->ChDir (mountdir);
+    if (!*dir || VFS->Mount (mountdir, dir))
+    {
+      VFS->PushDir ();
+      VFS->ChDir (mountdir);
 
-    // scan in all sections
-    PrepPix (ini, "Background", deco.bgnd, true);
-    PrepPix (ini, "TopLeft", deco.border[0], false);
-    PrepPix (ini, "Top", deco.border[1], false);
-    PrepPix (ini, "TopRight", deco.border[2], false);
-    PrepPix (ini, "Right", deco.border[3], false);
-    PrepPix (ini, "BottomRight", deco.border[4], false);
-    PrepPix (ini, "Bottom", deco.border[5], false);
-    PrepPix (ini, "BottomLeft", deco.border[6], false);
-    PrepPix (ini, "Left", deco.border[7], false);
+      // scan in all sections
+      PrepPix (ini, "Background", deco.bgnd, true);
+      PrepPix (ini, "TopLeft", deco.border[0], false);
+      PrepPix (ini, "Top", deco.border[1], false);
+      PrepPix (ini, "TopRight", deco.border[2], false);
+      PrepPix (ini, "Right", deco.border[3], false);
+      PrepPix (ini, "BottomRight", deco.border[4], false);
+      PrepPix (ini, "Bottom", deco.border[5], false);
+      PrepPix (ini, "BottomLeft", deco.border[6], false);
+      PrepPix (ini, "Left", deco.border[7], false);
 
-    // internal increase/decrease
-    deco.p2lx = ini->GetInt ("FancyConsole.General.p2lx");
-    deco.p2rx = ini->GetInt ("FancyConsole.General.p2rx");
-    deco.p2ty = ini->GetInt ("FancyConsole.General.p2ty");
-    deco.p2by = ini->GetInt ("FancyConsole.General.p2by");
-    deco.lx = ini->GetInt ("FancyConsole.General.lx");
-    deco.rx = ini->GetInt ("FancyConsole.General.rx");
-    deco.ty = ini->GetInt ("FancyConsole.General.ty");
-    deco.by = ini->GetInt ("FancyConsole.General.by");
+      // internal increase/decrease
+      deco.p2lx = ini->GetInt ("FancyConsole.General.p2lx");
+      deco.p2rx = ini->GetInt ("FancyConsole.General.p2rx");
+      deco.p2ty = ini->GetInt ("FancyConsole.General.p2ty");
+      deco.p2by = ini->GetInt ("FancyConsole.General.p2by");
+      deco.lx = ini->GetInt ("FancyConsole.General.lx");
+      deco.rx = ini->GetInt ("FancyConsole.General.rx");
+      deco.ty = ini->GetInt ("FancyConsole.General.ty");
+      deco.by = ini->GetInt ("FancyConsole.General.by");
 
-    VFS->PopDir ();
-    VFS->Unmount (mountdir, dir);
+      VFS->PopDir ();
+      if (strlen (dir))
+	VFS->Unmount (mountdir, dir);
+    }
+    else
+      Report (CS_REPORTER_SEVERITY_WARNING,
+    	  "Could not mount %s on %s", dir, mountdir);
   }
-  else
-    Report (CS_REPORTER_SEVERITY_WARNING,
-    	"Could not mount %s on %s", dir, mountdir);
 }
 
 void csFancyConsole::PrepPix (iConfigFile *ini, const char *sect,
@@ -494,35 +521,24 @@ void csFancyConsole::PrepPix (iConfigFile *ini, const char *sect,
   Keyname.Clear() << "FancyConsole." << sect << ".pic";
   const char* pix = ini->GetStr (Keyname, "");
 
-  border.mat = NULL;
+  border.txt = 0;
   border.do_keycolor = false;
   border.do_alpha = false;
   border.do_stretch = false;
 
   if (strlen (pix))
   {
-    size_t len = 0;
-    char *data = NULL;
-    iFile *F = VFS->Open (pix, VFS_FILE_READ);
-    if (F)
-    {
-      len = F->GetSize ();
-      data = new char [len];
-      if (data) len = F->Read (data, len);
-      F->DecRef ();
-    }
-    if (len)
+    csRef<iDataBuffer> Fbuf = VFS->ReadFile (pix, false);
+    if (Fbuf)
     {
       iTextureManager *tm = G3D->GetTextureManager ();
-      iImage *image =
-        ImageLoader->Load ((uint8 *)data, len, tm->GetTextureFormat ());
+      CS_ASSERT (tm != 0);
+      csRef<iImage> image (
+        ImageLoader->Load (Fbuf, tm->GetTextureFormat ()));
       if (image)
       {
-	iTextureHandle* txt =
-	  tm->RegisterTexture ( image, CS_TEXTURE_3D | CS_TEXTURE_NOMIPMAPS );
-	iMaterialHandle* mat = tm->RegisterMaterial (txt);
-	border.mat = mat;
-	image->DecRef();
+	border.txt =
+	  tm->RegisterTexture (image, CS_TEXTURE_3D | CS_TEXTURE_NOMIPMAPS);
 
         Keyname.Clear() << "FancyConsole." << sect << ".x";
 	border.offx = ini->GetInt (Keyname, 0);
@@ -538,14 +554,13 @@ void csFancyConsole::PrepPix (iConfigFile *ini, const char *sect,
 	  const char *kc = ini->GetStr(Keyname, "0,0,0" );
 	  sscanf( kc, "%d,%d,%d", &r, &g, &b );
 	  border.kr=r; border.kg=g; border.kb=b;
-	  border.mat->GetTexture ()->SetKeyColor(
+	  border.txt->SetKeyColor(
 	    border.kr, border.kg, border.kb);
 	}
 
         Keyname.Clear() << "FancyConsole." << sect << ".do_stretch";
 	border.do_stretch = ini->GetBool (Keyname, false);
       }
-      delete [] data;
     }
     else
       Report (CS_REPORTER_SEVERITY_WARNING, "Could not read %s", pix);
