@@ -17,11 +17,11 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "cssysdef.h"
 #include "csutil/sysfunc.h"
 #include "csutil/csuctransform.h"
 #include "simpcon.h"
@@ -29,6 +29,8 @@
 #include "csgeom/csrect.h"
 #include "csutil/cfgacc.h"
 #include "csutil/csevent.h"
+#include "csutil/eventnames.h"
+#include "csutil/event.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/txtmgr.h"
@@ -128,6 +130,7 @@ bool csSimpleConsole::Initialize (iObjectRegistry *object_reg)
     return false;
   }
 
+  // @@@ Not really good. G2D may not have been opened yet
   FrameWidth = G2D->GetWidth ();
   FrameHeight = G2D->GetHeight ();
 
@@ -185,12 +188,15 @@ bool csSimpleConsole::Initialize (iObjectRegistry *object_reg)
   CursorTime = csGetTicks ();
 
   // We want to see broadcast events
+  CS_INITIALIZE_SYSTEM_EVENT_SHORTCUTS (object_reg);
   if (!scfiEventHandler)
     scfiEventHandler = new EventHandler (this);
   csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
   if (q != 0)
-    q->RegisterListener (scfiEventHandler, CSMASK_Broadcast);
-
+  {
+    csEventID events[3] = { SystemOpen, SystemClose, CS_EVENTLIST_END };
+    q->RegisterListener (scfiEventHandler, events);
+  }
   return true;
 }
 
@@ -279,7 +285,10 @@ void csSimpleConsole::PutMessage (bool advance, const char *iText)
     LineMessageNumber--;
   }
 
-  strncpy (LineMessage [LineMessageNumber], iText, SIZE_LINE - 1);
+  size_t len = strlen (iText);
+  len = MIN (len, SIZE_LINE-1);
+  memcpy (LineMessage [LineMessageNumber], iText, len);
+  LineMessage [LineMessageNumber][len] = 0;
   LinesChanged [LineMessageNumber] = true;
 
   LineTime = csGetTicks () + 4000;
@@ -287,7 +296,7 @@ void csSimpleConsole::PutMessage (bool advance, const char *iText)
     LineMessageNumber++;
 }
 
-CS_IMPLEMENT_STATIC_VAR(GetTextBuf, csString, ());
+CS_IMPLEMENT_STATIC_VAR(GetTextBuf, csString, ())
 
 void csSimpleConsole::PutTextV (const char *iText2, va_list args)
 {
@@ -295,14 +304,12 @@ void csSimpleConsole::PutTextV (const char *iText2, va_list args)
 
   putTextLevel++;
 
-  int len;
-  char *dst;
-  const char *src;
-  char c;
-
   if (iText2 == 0 || *iText2 == 0)
     goto Done;
 
+  size_t len;
+  char *dst;
+  const char *src;
   {
     csString& iText = *GetTextBuf();
     iText.FormatV (iText2, args);
@@ -311,6 +318,8 @@ void csSimpleConsole::PutTextV (const char *iText2, va_list args)
     dst = Line [LineNumber] + len;
     src = iText;
   }
+  
+  char c;
   for (c = *src; c != '\0'; c = *++src)
   {
     if (ClearInput)
@@ -481,10 +490,10 @@ void csSimpleConsole::Draw2D (csRect* area)
       cursor[cursorSize] = 0;
 
       char *tmp = csStrNew (Line [LineNumber]);
-      int curx = strlen (tmp);
-      if ((CursorPos >= 0) && (CursorPos < curx))
+      size_t curPosX = strlen (tmp);
+      if ((CursorPos >= 0) && ((size_t)CursorPos < curPosX))
       {
-	size_t tl = strlen (tmp);
+	size_t tl = curPosX;
 	size_t cp = CursorPos;
 	size_t tp = 0;
 	while (cp > 0)
@@ -499,7 +508,7 @@ void csSimpleConsole::Draw2D (csRect* area)
 
       //if ((CursorPos >= 0) && (CursorPos < curx))
       //  tmp [CursorPos] = 0;
-      int temp_h;
+      int temp_h, curx;
       console_font->GetDimensions (tmp, curx, temp_h);
       delete [] tmp;
 
@@ -533,10 +542,10 @@ void csSimpleConsole::CacheColors ()
 void csSimpleConsole::GfxWrite (int x, int y, int fg, int bg, char *iText, ...)
 {
   va_list arg;
-  char buf[256];
+  csString buf;
 
   va_start (arg, iText);
-  vsprintf (buf, iText, arg);
+  buf.FormatV (iText, arg);
   va_end (arg);
 
   G2D->Write (console_font, x, y, fg, bg, buf);
@@ -558,20 +567,16 @@ const char *csSimpleConsole::GetLine (int iLine) const
 
 bool csSimpleConsole::HandleEvent (iEvent &Event)
 {
-  switch (Event.Type)
+  if (Event.Name == SystemOpen)
   {
-    case csevBroadcast:
-      switch (Event.Command.Code)
-      {
-        case cscmdSystemOpen:
           SystemReady = true;
           CacheColors ();
           return true;
-        case cscmdSystemClose:
+  }
+  else if (Event.Name == SystemClose)
+  {
           SystemReady = false;
           return true;
-      }
-      break;
   }
   return false;
 }
