@@ -16,10 +16,10 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "cssysdef.h"
 #include "csutil/sysfunc.h"
 #include "csutil/cfgacc.h"
 #include "csconout.h"
@@ -32,6 +32,7 @@
 #include "ivideo/txtmgr.h"
 #include "ivideo/fontserv.h"
 #include "csutil/csevent.h"
+#include "csutil/event.h"
 #include "csgeom/csrect.h"
 #include "csutil/csstring.h"
 #include "iutil/eventq.h"
@@ -110,6 +111,7 @@ bool csConsoleOutput::Initialize (iObjectRegistry *object_reg)
 
   csConfigAccess Config (object_reg, "/config/standardcon.cfg");
   const char* fontname = Config->GetStr ("StandardConsole.ConFont", "auto");
+  int fontsize = Config->GetInt ("StandardConsole.ConFontSize", 10);
 
   // Initialize the display rectangle to the entire display
   size.Set (0, 0, G2D->GetWidth () - 1, G2D->GetHeight () - 1);
@@ -127,8 +129,9 @@ bool csConsoleOutput::Initialize (iObjectRegistry *object_reg)
         fontname = CSFONT_COURIER;
       else
         fontname = CSFONT_LARGE;
+      fontsize = 10;
     }
-    font = fserv->LoadFont (fontname);
+    font = fserv->LoadFont (fontname, fontsize);
     font->GetMaxSize (fw, fh);
   }
   else
@@ -149,11 +152,15 @@ bool csConsoleOutput::Initialize (iObjectRegistry *object_reg)
   flash_time = csGetTicks ();
 
   // We want to see broadcast events
+  CS_INITIALIZE_SYSTEM_EVENT_SHORTCUTS (object_reg);
   if (!scfiEventHandler)
     scfiEventHandler = new EventHandler (this);
   csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
   if (q != 0)
-    q->RegisterListener (scfiEventHandler, CSMASK_Broadcast);
+  {
+    csEventID events[3] = { SystemOpen, SystemClose, CS_EVENTLIST_END };
+    q->RegisterListener (scfiEventHandler, events);
+  }
   return true;
 }
 
@@ -185,14 +192,14 @@ void csConsoleOutput::PutTextV (const char *text2, va_list args)
 {
   csScopedMutexLock lock (mutex);
 
-  int i;
+  size_t i;
   csString *curline = 0;
 
-  char text[4096];
-  vsprintf (text, text2, args);
+  csString text;
+  text.FormatV (text2, args);
 
   // Scan the string for escape characters
-  for (i = 0; text && text [i] != 0; i++)
+  for (i = 0; i < text.Length(); i++)
   {
     if (clear_input)
     {
@@ -279,7 +286,7 @@ const char *csConsoleOutput::GetLine (int line) const
 void csConsoleOutput::DeleteText (int start, int end)
 {
   csString *text = buffer->WriteLine ();
-  int length = text->Length ();
+  int length = (int)text->Length ();
 
   // Avoid invalid start points
   if (start > length)
@@ -289,7 +296,7 @@ void csConsoleOutput::DeleteText (int start, int end)
   if ((end == -1) || (end >= length))
   {
     text->DeleteAt (start, length - start);
-    cx = text->Length ();
+    cx = (int)text->Length ();
   }
   else
   {
@@ -379,7 +386,7 @@ void csConsoleOutput::Draw2D (csRect *area)
     {
 #ifdef CS_DEBUG
       if (cx != 0)
-	printf (
+	csPrintf (
 	  "csConsoleOutput:  Current line is empty but cursor x != 0!\n");
 #endif // CS_DEBUG
       cx_pix = 1;
@@ -411,7 +418,7 @@ void csConsoleOutput::Draw2D (csRect *area)
         break;
 #ifdef CS_DEBUG
       default:
-        printf ("csConsoleOutput:  Invalid cursor setting!\n");
+        csPrintf ("csConsoleOutput:  Invalid cursor setting!\n");
 #endif // CS_DEBUG
     }
   }
@@ -561,7 +568,7 @@ void csConsoleOutput::SetCursorPos(int x, int y)
   const csString *curline = buffer->GetLine (cy);
 
   if (curline)
-    max_x = curline->Length ();
+    max_x = (int)curline->Length ();
   else
     max_x = 0;
 
@@ -586,7 +593,7 @@ void csConsoleOutput::SetCursorPos (int iCharNo)
     int max_x;
     const csString *curline = buffer->GetLine (cy);
 
-    max_x = curline ? curline->Length () : 0;
+    max_x = curline ? (int)curline->Length () : 0;
     cx = (iCharNo > max_x) ? max_x : (iCharNo <= 0) ? 0 : iCharNo;
   }
 }
@@ -671,20 +678,16 @@ bool csConsoleOutput::PerformExtensionV (const char *iCommand, va_list args)
 
 bool csConsoleOutput::HandleEvent (iEvent &Event)
 {
-  switch (Event.Type)
+  if (Event.Name == SystemOpen)
   {
-    case csevBroadcast:
-      switch (Event.Command.Code)
-      {
-        case cscmdSystemOpen:
           system_ready = true;
           CacheColors ();
           return true;
-        case cscmdSystemClose:
+  }
+  else if (Event.Name == SystemClose)
+  {
           system_ready = false;
           return true;
-      }
-      break;
   }
   return false;
 }
