@@ -21,6 +21,7 @@
 
 #include "ivaria/conout.h"
 #include "csutil/scopedmutexlock.h"
+#include "csutil/eventnames.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
 #include "csgeom/csrect.h"
@@ -34,6 +35,7 @@ class csConsoleOutput : public iConsoleOutput
 {
 private:
   csRef<csMutex> mutex;
+  CS_DECLARE_SYSTEM_EVENT_SHORTCUTS;
 
 public:
   SCF_DECLARE_IBASE;
@@ -183,6 +185,8 @@ public:
       SCF_DESTRUCT_IBASE ();
     }
     virtual bool HandleEvent (iEvent& e) { return parent->HandleEvent(e); }
+    CS_EVENTHANDLER_NAMES("crystalspace.console");
+    CS_EVENTHANDLER_NIL_CONSTRAINTS
   } * scfiEventHandler;
 
 private:
