@@ -18,33 +18,33 @@
 */
 
 #include "cssysdef.h"
-#include "qint.h"
-#include "ivideo/graph3d.h"
-#include "csloader.h"
-#include "imap/ldrctxt.h"
-#include "imap/reader.h"
-#include "csutil/array.h"
-#include "csutil/scanstr.h"
-#include "iutil/document.h"
-#include "iutil/strset.h"
-#include "cstool/proctex.h"
-#include "cstool/prdots.h"
-#include "cstool/prfire.h"
-#include "cstool/prplasma.h"
-#include "cstool/prwater.h"
+#include "csqint.h"
+
+#include "csgfx/imagecubemapmaker.h"
 #include "csgfx/rgbpixel.h"
-#include "ivideo/texture.h"
-#include "ivideo/material.h"
+#include "cstool/proctex.h"
+#include "csutil/array.h"
+#include "csutil/cscolor.h"
+#include "csutil/scanstr.h"
 #include "iengine/engine.h"
-#include "iengine/texture.h"
 #include "iengine/material.h"
 #include "iengine/region.h"
-#include "iutil/objreg.h"
-#include "ivaria/reporter.h"
+#include "iengine/texture.h"
 #include "igraphic/animimg.h"
-#include "csgfx/csimgvec.h"
-#include "loadtex.h"
+#include "imap/ldrctxt.h"
+#include "imap/reader.h"
+#include "iutil/document.h"
+#include "iutil/objreg.h"
+#include "iutil/strset.h"
+#include "ivaria/keyval.h"
+#include "ivaria/reporter.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/material.h"
 #include "ivideo/shader/shader.h"
+#include "ivideo/texture.h"
+
+#include "loadtex.h"
+#include "csloader.h"
 
 #define PLUGIN_LEGACY_TEXTYPE_PREFIX  "crystalspace.texture.loader."
 
@@ -119,16 +119,12 @@ bool csLoader::ParseTextureList (iLoaderContext* ldr_context,
 	        return false;
         break;
       case XMLTOKEN_CUBEMAP:
-#ifdef CS_USE_NEW_RENDERER
         if (!ParseCubemap (ldr_context, child))
           return false;
-#endif
         break;
       case XMLTOKEN_TEXTURE3D:
-#ifdef CS_USE_NEW_RENDERER
         if (!ParseTexture3D (ldr_context, child))
           return false;
-#endif // CS_USE_NEW_RENDERER
         break;
       default:
         SyntaxService->ReportBadToken (child);
@@ -154,14 +150,18 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
   csRef<iTextureWrapper> tex;
   csRef<iLoaderPlugin> plugin;
 
-  char filename[256] = "";
+  csString filename = node->GetAttributeValue ("file");
   csColor transp (0, 0, 0);
   bool do_transp = false;
   bool keep_image = false;
   bool always_animate = false;
   TextureLoaderContext context (txtname);
   csRef<iDocumentNode> ParamsNode;
-  char* type = 0;
+  csString type;
+  csAlphaMode::AlphaType alphaType = csAlphaMode::alphaNone;
+  bool overrideAlphaType = false;
+
+  csRefArray<iDocumentNode> key_nodes;
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -172,11 +172,14 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
     csStringID id = xmltokens.Request (value);
     switch (id)
     {
+      case XMLTOKEN_KEY:
+	key_nodes.Push (child);
+	break;
       case XMLTOKEN_FOR2D:
 	{
 	  bool for2d;
 	  if (!SyntaxService->ParseBool (child, for2d, true))
-	    goto error;
+	    return 0;
           if (for2d)
 	    context.SetFlags (context.GetFlags() | CS_TEXTURE_2D);
           else
@@ -187,7 +190,7 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
 	{
 	  bool for3d;
 	  if (!SyntaxService->ParseBool (child, for3d, true))
-	    goto error;
+	    return 0;
           if (for3d)
 	    context.SetFlags (context.GetFlags() | CS_TEXTURE_3D);
           else
@@ -197,7 +200,7 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
       case XMLTOKEN_TRANSPARENT:
         do_transp = true;
 	if (!SyntaxService->ParseColor (child, transp))
-	  goto error;
+	  return 0;
         break;
       case XMLTOKEN_FILE:
 	{
@@ -207,16 +210,16 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
 	    SyntaxService->ReportError (
 	      "crystalspace.maploader.parse.texture",
 	      child, "Expected VFS filename for 'file'!");
-	    goto error;
+	    return 0;
 	  }
-          strcpy (filename, fname);
+          filename = fname;
 	}
         break;
       case XMLTOKEN_MIPMAP:
 	{
 	  bool mm;
 	  if (!SyntaxService->ParseBool (child, mm, true))
-	    goto error;
+	    return 0;
           if (!mm)
 	    context.SetFlags (context.GetFlags() | CS_TEXTURE_NOMIPMAPS);
           else
@@ -225,32 +228,41 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
         break;
       case XMLTOKEN_DITHER:
 	{
-	  bool di;
-	  if (!SyntaxService->ParseBool (child, di, true))
-	    goto error;
-          if (di)
-	    context.SetFlags (context.GetFlags() | CS_TEXTURE_DITHER);
+	  SyntaxService->Report (
+	    "crystalspace.maploader.parse.texture",
+	    CS_REPORTER_SEVERITY_NOTIFY,
+	    node,
+	    "\"dither\" flag has been dropped");
+	}
+        break;
+      case XMLTOKEN_NPOTS:
+	{
+	  bool npots;
+	  if (!SyntaxService->ParseBool (child, npots, true))
+	    return 0;
+          if (npots)
+	    context.SetFlags (context.GetFlags() | CS_TEXTURE_NPOTS);
           else
-	    context.SetFlags (context.GetFlags() & ~CS_TEXTURE_DITHER);
+	    context.SetFlags (context.GetFlags() & ~CS_TEXTURE_NPOTS);
 	}
         break;
       case XMLTOKEN_KEEPIMAGE:
         {
 	  if (!SyntaxService->ParseBool (child, keep_image, true))
-	    goto error;
+	    return 0;
 	}
 	break;
       case XMLTOKEN_PARAMS:
 	ParamsNode = child;
 	break;
       case XMLTOKEN_TYPE:
-	type = csStrNew (child->GetContentsValue ());
-	if (!type)
+	type = child->GetContentsValue ();
+	if (type.IsEmpty ())
 	{
 	  SyntaxService->ReportError (
 	    "crystalspace.maploader.parse.texture",
 	    child, "Expected plugin ID for <type>!");
-	  goto error;
+	  return 0;
 	}
 	break;
       case XMLTOKEN_SIZE:
@@ -266,13 +278,13 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
 	break;
       case XMLTOKEN_ALWAYSANIMATE:
 	if (!SyntaxService->ParseBool (child, always_animate, true))
-	  goto error;
+	  return 0;
 	break;
       case XMLTOKEN_CLAMP:
         {
           bool c;
           if (!SyntaxService->ParseBool (child, c, true))
-            goto error;
+            return 0;
           if (c)
             context.SetFlags (context.GetFlags() | CS_TEXTURE_CLAMP);
           else
@@ -283,41 +295,54 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
         {
           bool c;
           if (!SyntaxService->ParseBool (child, c, true))
-            goto error;
+            return 0;
           if (c)
             context.SetFlags (context.GetFlags() & ~CS_TEXTURE_NOFILTER);
           else
             context.SetFlags (context.GetFlags() | CS_TEXTURE_NOFILTER);
         }
         break;
+      case XMLTOKEN_CLASS:
+	{
+	  context.SetClass (child->GetContentsValue ());
+	}
+	break;
+      case XMLTOKEN_ALPHA:
+	{
+	  csAlphaMode am;
+	  if (!SyntaxService->ParseAlphaMode (child, 0, am, false))
+            return 0;
+	  overrideAlphaType = true;
+	  alphaType = am.alphaType;
+	}
+	break;
       default:
         SyntaxService->ReportBadToken (child);
-	goto error;
+	return 0;
     }
   }
 
   // @@@ some more comments
-  if ((!type || (*type == 0)) && (filename[0] == 0))
+  if (type.IsEmpty () && filename.IsEmpty ())
   {
-    strcpy (filename, txtname);
+    filename = txtname;
   }
 
   iTextureManager* tm;
   tm = G3D ? G3D->GetTextureManager() : 0;
   int Format;
   Format = tm ? tm->GetTextureFormat () : CS_IMGFMT_TRUECOLOR;
-  if (filename && (filename[0] != 0))
+  if (!filename.IsEmpty ())
   {
     csRef<iImage> image = LoadImage (filename, Format);
     context.SetImage (image);
-    if (image && !type)
+    if (image && type.IsEmpty ())
     {
       // special treatment for animated textures
-      csRef<iAnimatedImage> anim = csPtr<iAnimatedImage>
-	(SCF_QUERY_INTERFACE (image, iAnimatedImage));
+      csRef<iAnimatedImage> anim = SCF_QUERY_INTERFACE (image, iAnimatedImage);
       if (anim && anim->IsAnimated())
       {
-	type = csStrNew (PLUGIN_TEXTURELOADER_ANIMIMG);
+	type = PLUGIN_TEXTURELOADER_ANIMIMG;
       }
       else
       {
@@ -335,15 +360,15 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
   
   iLoaderPlugin* Plug; Plug = 0;
   iBinaryLoaderPlugin* Binplug;
-  if (type && !plugin)
+  if ((!type.IsEmpty ()) && !plugin)
   {
     iDocumentNode* defaults = 0;
     if (!loaded_plugins.FindPlugin (type, Plug, Binplug, defaults))
     {
-      if ((!strcasecmp (type, "dots")) ||
-	  (!strcasecmp (type, "plasma")) ||
-	  (!strcasecmp (type, "water")) ||
-	  (!strcasecmp (type, "fire")))
+      if ((type == "dots") ||
+	  (type == "plasma") ||
+	  (type == "water") ||
+	  (type == "fire"))
       {
 	// old style proctex type
 	if (!deprecated_warned)
@@ -358,12 +383,8 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
 	  deprecated_warned = true;
 	}
 
-	char* newtype = new char[
-		strlen (PLUGIN_LEGACY_TEXTYPE_PREFIX) +
-		strlen (type) + 1];
-	strcpy (newtype, PLUGIN_LEGACY_TEXTYPE_PREFIX);
-	strcat (newtype, type);
-	delete[] type;
+	csString newtype = PLUGIN_LEGACY_TEXTYPE_PREFIX;
+	newtype += type;
 	type = newtype;
 
 	loaded_plugins.FindPlugin (type, Plug, Binplug, defaults);
@@ -379,12 +400,12 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
     }
   }
 
-  if (type && !plugin)
+  if ((!type.IsEmpty ()) && !plugin)
   {
     SyntaxService->Report (
       "crystalspace.maploader.parse.texture",
       CS_REPORTER_SEVERITY_WARNING,
-      node, "Could not get plugin '%s', using default", type);
+      node, "Could not get plugin '%s', using default", (const char*)type);
 
     if (!BuiltinImageTexLoader)
     {
@@ -397,7 +418,7 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
   if (plugin)
   {
     csRef<iBase> b = plugin->Parse (ParamsNode,
-      ldr_context, CS_STATIC_CAST(iBase*, &context));
+      0/*ssource*/, ldr_context, CS_STATIC_CAST(iBase*, &context));
     if (b) tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
   }
 
@@ -415,33 +436,52 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
       BuiltinCheckerTexLoader.AttachNew (ctl);
     }
     csRef<iBase> b = BuiltinCheckerTexLoader->Parse (ParamsNode,
-      ldr_context, CS_STATIC_CAST(iBase*, &context));
+      0/*ssource*/, ldr_context, CS_STATIC_CAST(iBase*, &context));
+    CS_ASSERT(b);
     tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
+    CS_ASSERT(tex);
   }
 
   if (tex)
   {
+    CS_ASSERT_MSG("Texture loader did not register texture", 
+      tex->GetTextureHandle());
     tex->QueryObject ()->SetName (txtname);
     tex->SetKeepImage (keep_image);
     if (do_transp)
-      tex->SetKeyColor (QInt (transp.red * 255.99),
-        QInt (transp.green * 255.99), QInt (transp.blue * 255.99));
+      tex->SetKeyColor (csQint (transp.red * 255.99),
+        csQint (transp.green * 255.99), csQint (transp.blue * 255.99));
+    tex->SetTextureClass (context.GetClass ());
+    if (overrideAlphaType)
+      tex->GetTextureHandle()->SetAlphaType (alphaType);
 
-    csRef<iProcTexture> ipt = csPtr<iProcTexture>
-      (SCF_QUERY_INTERFACE (tex, iProcTexture));
+    csRef<iProcTexture> ipt = SCF_QUERY_INTERFACE (tex, iProcTexture);
     if (ipt)
-    {
       ipt->SetAlwaysAnimate (always_animate);
+    AddToRegion (ldr_context, tex->QueryObject ());
+
+    size_t i;
+    for (i = 0 ; i < key_nodes.Length () ; i++)
+    {
+      iKeyValuePair* kvp = 0;
+      SyntaxService->ParseKey (key_nodes[i], kvp);
+      if (kvp)
+      {
+        tex->QueryObject()->ObjAdd (kvp->QueryObject ());
+	kvp->DecRef ();
+      }
+      else
+	return 0;
     }
   }
-  AddToRegion (ldr_context, tex->QueryObject ());
 
-  delete[] type;
+  if (tm)
+  {
+    if (!tex->GetTextureHandle ()) tex->Register (tm);
+    if (!tex->KeepImage ())
+      tex->SetImageFile (0);
+  }
   return tex;
-
-error:
-  delete[] type;
-  return 0;
 }
 
 iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
@@ -462,22 +502,14 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
   float diffuse = CS_DEFMAT_DIFFUSE;
   float ambient = CS_DEFMAT_AMBIENT;
   float reflection = CS_DEFMAT_REFLECTION;
-#ifndef CS_USE_NEW_RENDERER
-  int num_txt_layer = 0;
-  iTextureWrapper* txt_layers[4];
-  csTextureLayer layers[4];
-#endif
+
   
-#ifdef CS_USE_NEW_RENDERER
   bool shaders_mentioned = false;	// If true there were shaders.
   csArray<csStringID> shadertypes;
-  //csArray<iShader*> shaders;
   csArray<iShader*> shaders;
   csRefArray<csShaderVariable> shadervars;
 
-  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (
-    object_reg, "crystalspace.shared.stringset", iStringSet);
-#endif // CS_USE_NEW_RENDERER
+  csRefArray<iDocumentNode> key_nodes;
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -488,6 +520,9 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
     csStringID id = xmltokens.Request (value);
     switch (id)
     {
+      case XMLTOKEN_KEY:
+	key_nodes.Push (child);
+	break;
       case XMLTOKEN_TEXTURE:
         {
 	  const char* txtname = child->GetContentsValue ();
@@ -507,9 +542,9 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
 	  csColor color;
           if (!SyntaxService->ParseColor (child, color))
 	    return 0;
-	  col.red = QInt (color.red * 255.99f);
-	  col.green = QInt (color.green * 255.99f);
-	  col.blue = QInt (color.blue * 255.99f);
+	  col.red = csQint (color.red * 255.99f);
+	  col.green = csQint (color.green * 255.99f);
+	  col.blue = csQint (color.blue * 255.99f);
 	}
         break;
       case XMLTOKEN_DIFFUSE:
@@ -521,82 +556,7 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
       case XMLTOKEN_REFLECTION:
 	reflection = child->GetContentsValueAsFloat ();
         break;
-      case XMLTOKEN_LAYER:
-        // @@@ TODO: add shortcut to support NR!
-#ifndef CS_USE_NEW_RENDERER
-	{
-	  if (num_txt_layer >= 4)
-	  {
-	    SyntaxService->ReportError (
-	      "crystalspace.maploader.parse.material",
-	      child, "Only four texture layers supported!");
-	    return 0;
-	  }
-	  txt_layers[num_txt_layer] = 0;
-	  layers[num_txt_layer].txt_handle = 0;
-	  layers[num_txt_layer].uscale = 1;
-	  layers[num_txt_layer].vscale = 1;
-	  layers[num_txt_layer].ushift = 0;
-	  layers[num_txt_layer].vshift = 0;
-          layers[num_txt_layer].mode = CS_FX_ADD | CS_FX_TILING;
-	  csRef<iDocumentNodeIterator> layer_it = child->GetNodes ();
-	  while (layer_it->HasNext ())
-	  {
-	    csRef<iDocumentNode> layer_child = layer_it->Next ();
-	    if (layer_child->GetType () != CS_NODE_ELEMENT) continue;
-	    const char* layer_value = layer_child->GetValue ();
-	    csStringID layer_id = xmltokens.Request (layer_value);
-	    switch (layer_id)
-	    {
-	      case XMLTOKEN_TEXTURE:
-		{
-		  const char* txtname = layer_child->GetContentsValue ();
-		  iTextureWrapper* texh;
-		  if (ldr_context->GetRegion ()
-		  	&& ldr_context->CurrentRegionOnly ())
-		    texh = ldr_context->GetRegion ()->FindTexture (txtname);
-		  else
-                    texh = Engine->GetTextureList ()->FindByName (txtname);
-                  if (texh)
-                    txt_layers[num_txt_layer] = texh;
-                  else
-                  {
-		    SyntaxService->ReportError (
-			"crystalspace.maploader.parse.material",
-		    	child, "Cannot find texture `%s' for material `%s'!",
-			txtname, matname);
-		    return 0;
-                  }
-		}
-		break;
-	      case XMLTOKEN_SCALE:
-		layers[num_txt_layer].uscale = layer_child
-				->GetAttributeValueAsFloat ("u");
-		layers[num_txt_layer].vscale = layer_child
-				->GetAttributeValueAsFloat ("v");
-	        break;
-	      case XMLTOKEN_SHIFT:
-		layers[num_txt_layer].ushift = layer_child
-				->GetAttributeValueAsFloat ("u");
-		layers[num_txt_layer].vshift = layer_child
-				->GetAttributeValueAsFloat ("v");
-	        break;
-	      case XMLTOKEN_MIXMODE:
-		if (!SyntaxService->ParseMixmode (layer_child,
-		    layers[num_txt_layer].mode))
-		  return 0;
-	        break;
-	      default:
-	        SyntaxService->ReportBadToken (layer_child);
-		return 0;
-	    }
-	  }
-	  num_txt_layer++;
-	}
-#endif
-        break;
       case XMLTOKEN_SHADER:
-#ifdef CS_USE_NEW_RENDERER
         {
 	  shaders_mentioned = true;
           csRef<iShaderManager> shaderMgr = CS_QUERY_REGISTRY (object_reg,
@@ -607,7 +567,7 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
             break;
           }
           const char* shadername = child->GetContentsValue ();
-          iShader* shader = shaderMgr->GetShader (shadername);
+	  iShader* shader = ldr_context->FindShader (shadername);
           if (!shader)
           {
             ReportNotify (
@@ -623,15 +583,13 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
 		shadername, matname);
             break;
           }
-          shadertypes.Push (strings->Request(shadertype));
+          shadertypes.Push (stringSet->Request(shadertype));
           //shaders.Push (shader);
 	  //csRef<iShaderWrapper> wrapper = shaderMgr->CreateWrapper (shader);
 	  shaders.Push (shader);
         }
-#endif //CS_USE_NEW_RENDERER
         break;
       case XMLTOKEN_SHADERVAR:
-#ifdef CS_USE_NEW_RENDERER
         {
           csRef<iShaderManager> shaderMgr = CS_QUERY_REGISTRY (object_reg,
             iShaderManager);
@@ -645,7 +603,7 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
           /*csRef<csShaderVariable> var = 
             shaderMgr->CreateVariable (strings->Request (varname));*/
 	  csRef<csShaderVariable> var;
-	  var.AttachNew (new csShaderVariable (strings->Request (varname)));
+	  var.AttachNew (new csShaderVariable (stringSet->Request (varname)));
 
           if (!SyntaxService->ParseShaderVar (child, *var))
           {
@@ -655,7 +613,6 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
           }
           shadervars.Push (var);
         }
-#endif //CS_USE_NEW_RENDERER
         break;
       default:
 	SyntaxService->ReportBadToken (child);
@@ -663,35 +620,13 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
     }
   }
 
-#ifdef CS_USE_NEW_RENDERER
-  // If we didn't have any shaders then we automatically use
-  // the default ambient and diffuse shader.
-#if 0 // This is moved to the engine
-  if (!shaders_mentioned)
-  {
-    csRef<iShaderManager> shaderMgr = CS_QUERY_REGISTRY (object_reg,
-	  	iShaderManager);
-    if (shaderMgr)
-    {
-      iShader* shader_light = shaderMgr->GetShader ("or_lighting");
-      if (shader_light)
-      {
-        shadertypes.Push (strings->Request ("lighting"));
-	shaders.Push (shader_light);
-      }
-    }
-  }
-#endif
   csRef<iMaterial> material = Engine->CreateBaseMaterial (texh);
-#else
-  csRef<iMaterial> material = Engine->CreateBaseMaterial (texh,
-  	num_txt_layer, txt_layers, layers);
-#endif
+
   if (col_set)
     material->SetFlatColor (col);
   material->SetReflection (diffuse, ambient, reflection);
 
-  iMaterialWrapper *mat = Engine->GetMaterialList ()->NewMaterial (material);
+  iMaterialWrapper *mat;
  
   if (prefix)
   {
@@ -699,201 +634,108 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
     strcpy (prefixedname, prefix);
     strcat (prefixedname, "_");
     strcat (prefixedname, matname);
-    mat->QueryObject()->SetName (prefixedname);
+    mat = Engine->GetMaterialList ()->NewMaterial (material, prefixedname);
     delete [] prefixedname;
   }
   else
   {
-    mat->QueryObject()->SetName (matname);
+    mat = Engine->GetMaterialList ()->NewMaterial (material, matname);
   }
-#ifdef CS_USE_NEW_RENDERER
-  int i;
+  
+  size_t i;
   for (i=0; i<shaders.Length (); i++)
     //if (shaders[i]->Prepare ())
       material->SetShader (shadertypes[i], shaders[i]);
   for (i=0; i<shadervars.Length (); i++)
     material->AddVariable (shadervars[i]);
-#endif // CS_USE_NEW_RENDERER
+
   // dereference material since mat already incremented it
 
+  for (i = 0 ; i < key_nodes.Length () ; i++)
+  {
+    iKeyValuePair* kvp = 0;
+    SyntaxService->ParseKey (key_nodes[i], kvp);
+    if (kvp)
+    {
+      mat->QueryObject ()->ObjAdd (kvp->QueryObject ());
+      kvp->DecRef ();
+    } else
+      return 0;
+  }
   AddToRegion (ldr_context, mat->QueryObject ());
+
   return mat;
 }
 
-#ifdef CS_USE_NEW_RENDERER
 /// Parse a Cubemap texture definition and add the texture to the engine
 iTextureWrapper* csLoader::ParseCubemap (iLoaderContext* ldr_context,
     iDocumentNode* node)
 {
-  csRef<csImageVector> cubetextures = new csImageVector();
-
-  csRef<iDocumentNodeIterator> it = node->GetNodes ();
-  iTextureManager* tm = G3D ? G3D->GetTextureManager() : 0;
-  int Format = tm ? tm->GetTextureFormat () : CS_IMGFMT_TRUECOLOR;
- 
-  csRef<iTextureHandle> itex;
-  csRef<iTextureWrapper> itexwrap;
-
-  csRef<iImage> north, south, east, west, top, bottom;
-  const char* texname = node->GetAttributeValue ("name");
-  char *fname= 0;
-
-  while (it->HasNext ())
+  static bool cubemapDeprecationWarning = false;
+  if (!cubemapDeprecationWarning)
   {
-    csRef<iDocumentNode> child = it->Next ();
-    if (child->GetType () != CS_NODE_ELEMENT) continue;
-    const char* value = child->GetValue ();
-    csStringID id = xmltokens.Request (value);
-    switch (id)
-    {
-      case XMLTOKEN_NORTH:
-        fname = csStrNew (child->GetContentsValue ());
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  goto error;
-	}
-      
-        north = LoadImage(fname, Format);
-        break;
-    
-      case XMLTOKEN_SOUTH:
-        fname = csStrNew (child->GetContentsValue ());
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  goto error;
-	}
-      
-        south = LoadImage(fname, Format);
-        break;
-    
-      case XMLTOKEN_EAST:
-        fname = csStrNew (child->GetContentsValue ());
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  goto error;
-	}
-      
-        east = LoadImage(fname, Format);
-        break;
-    
-      case XMLTOKEN_WEST:
-        fname = csStrNew (child->GetContentsValue ());
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  goto error;
-	}
-      
-        west = LoadImage(fname, Format);
-        break;
-    
-      case XMLTOKEN_TOP:
-        fname = csStrNew (child->GetContentsValue ());
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  goto error;
-	}
-      
-        top = LoadImage(fname, Format);
-        break;
-    
-      case XMLTOKEN_BOTTOM:
-        fname = csStrNew (child->GetContentsValue ());
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  goto error;
-	}
-      
-        bottom = LoadImage(fname, Format);
-        break;
-    }
+    cubemapDeprecationWarning = true;
+    SyntaxService->Report ("crystalspace.maploader.parse.texture",
+      CS_REPORTER_SEVERITY_WARNING, node,
+      "'<cubemap>...' is deprecated, use '<texture><type>"
+      PLUGIN_TEXTURELOADER_CUBEMAP "</type><params>...' instead");
   }
 
-  cubetextures->AddImage (north);
-  cubetextures->AddImage (south);
-  cubetextures->AddImage (east);
-  cubetextures->AddImage (west);
-  cubetextures->AddImage (top);
-  cubetextures->AddImage (bottom);
+  csRef<csCubemapTextureLoader> plugin;
+  plugin.AttachNew (new csCubemapTextureLoader (0));
+  plugin->Initialize (object_reg);
 
-  itex = tm->RegisterTexture(cubetextures,0,
-  	iTextureHandle::CS_TEX_IMG_CUBEMAP);
+  csRef<TextureLoaderContext> context;
+  const char* txtname = node->GetAttributeValue ("name");
+  context.AttachNew (new TextureLoaderContext (txtname));
 
-  itexwrap = Engine->GetTextureList()->NewTexture(itex);
-  itexwrap->QueryObject()->SetName(texname);
+  csRef<iBase> b = plugin->Parse (node, 0/*ssource*/, ldr_context, context);
+  csRef<iTextureWrapper> tex;
+  if (b) tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
 
-  delete[] fname;
-  return itexwrap;
+  if (tex)
+  {
+    tex->QueryObject ()->SetName (txtname);
+    AddToRegion (ldr_context, tex->QueryObject ());
+    iTextureManager* tm = G3D ? G3D->GetTextureManager() : 0;
+    if (tm) tex->Register (tm);
+  }
 
-error:
-  delete[] fname;
-  return 0;
+  return tex;
 }
 
 iTextureWrapper* csLoader::ParseTexture3D (iLoaderContext* ldr_context,
     iDocumentNode* node)
 {
-   csRef<csImageVector> cubetextures = new csImageVector();
-
-  csRef<iDocumentNodeIterator> it = node->GetNodes ();
-  iTextureManager* tm = G3D ? G3D->GetTextureManager() : 0;
-  int Format = tm ? tm->GetTextureFormat () : CS_IMGFMT_TRUECOLOR;
-  
-  csRef<iImageVector> imagevec = new csImageVector();
-  csRef<iImage> image;
-
-  const char* texname = node->GetAttributeValue ("name");
-
-  int imagecount = 0;
-  while (it->HasNext ())
+  static bool volmapDeprecationWarning = false;
+  if (!volmapDeprecationWarning)
   {
-    csRef<iDocumentNode> child = it->Next ();
-    if (child->GetType () != CS_NODE_ELEMENT) continue;
-    const char* value = child->GetValue ();
-    csStringID id = xmltokens.Request (value);
-    switch (id)
-    {
-      case XMLTOKEN_LAYER:
-      {
-        const char* fname = child->GetContentsValue ();
-	if (!fname)
-	{
-	  SyntaxService->ReportError (
-	       "crystalspace.maploader.parse.texture",
-	       child, "Expected VFS filename for 'file'!");
-	  return 0;
-	}
-        image = LoadImage(fname, Format);
-        imagevec->AddImage(image);
-        imagecount++;
-        break;
-      }
-    }
+    volmapDeprecationWarning = true;
+    SyntaxService->Report ("crystalspace.maploader.parse.texture",
+      CS_REPORTER_SEVERITY_WARNING, node,
+      "'<texture3d>...' is deprecated, use '<texture><type>"
+      PLUGIN_TEXTURELOADER_TEX3D "</type><params>...' instead");
   }
-  
-  csRef<iTextureHandle> itex (tm->RegisterTexture(imagevec,0,
-  	iTextureHandle::CS_TEX_IMG_3D));
 
-  csRef<iTextureWrapper> itexwrap = Engine->GetTextureList()->NewTexture(itex);
-  itexwrap->QueryObject()->SetName(texname);
+  csRef<csTexture3DLoader> plugin;
+  plugin.AttachNew (new csTexture3DLoader (0));
+  plugin->Initialize (object_reg);
 
-  return itexwrap;
+  csRef<TextureLoaderContext> context;
+  const char* txtname = node->GetAttributeValue ("name");
+  context.AttachNew (new TextureLoaderContext (txtname));
+
+  csRef<iBase> b = plugin->Parse (node, 0/*ssource*/, ldr_context, context);
+  csRef<iTextureWrapper> tex;
+  if (b) tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
+
+  if (tex)
+  {
+    tex->QueryObject ()->SetName (txtname);
+    AddToRegion (ldr_context, tex->QueryObject ());
+    iTextureManager* tm = G3D ? G3D->GetTextureManager() : 0;
+    if (tm) tex->Register (tm);
+  }
+
+  return tex;
 }
-#endif // CS_USE_NEW_RENDERER
