@@ -19,8 +19,8 @@
 #include "csutil/sysfunc.h"
 #include "csutil/scfstr.h"
 #include "iutil/string.h"
-#include "qint.h"
-#include "qsqrt.h"
+#include "csqint.h"
+#include "csqsqrt.h"
 #include "csgeom/math3d.h"
 #include "csgeom/pmtools.h"
 #include "igeom/objmodel.h"
