@@ -16,13 +16,16 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <string.h>
 #include "cssysdef.h"
+#include <string.h>
 #include "csver.h"
 #include "csutil/sysfunc.h"
 #include "csutil/scf.h"
 #include "csutil/util.h"
 #include "csutil/scfstr.h"
+#include "csutil/event.h"
+#include "iutil/event.h"
+#include "iutil/eventq.h"
 #include "csgeom/frustum.h"
 #include "csgeom/matrix3.h"
 #include "csgeom/math3d.h"
@@ -64,13 +67,17 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csFrustumVis::eiComponent)
   SCF_IMPLEMENTS_INTERFACE (iComponent)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
+SCF_IMPLEMENT_IBASE (csFrustumVis::eiEventHandler)
+  SCF_IMPLEMENTS_INTERFACE (iEventHandler)
+SCF_IMPLEMENT_IBASE_END
+
 //----------------------------------------------------------------------
 
 class csFrustVisObjIt : public iVisibilityObjectIterator
 {
 private:
   csArray<iVisibilityObject*>* vector;
-  int position;
+  size_t position;
   bool* vistest_objects_inuse;
 
 public:
@@ -99,25 +106,25 @@ public:
 
   virtual iVisibilityObject* Next()
   {
-    if (position < 0) return 0;
+    if (position == (size_t)-1) return 0;
     iVisibilityObject* vo = vector->Get (position);
     position++;
     if (position == vector->Length ())
-      position = -1;
+      position = (size_t)-1;
     return vo;
   }
 
   virtual void Reset()
   {
     if (vector == 0 || vector->Length () < 1)
-      position = -1;
+      position = (size_t)-1;
     else
       position = 0;
   }
 
   virtual bool HasNext () const
   {
-    return (position >= 0 && position <= vector->Length ());
+    return ((position != (size_t)-1) && position <= vector->Length ());
   }
 };
 
@@ -142,9 +149,32 @@ void csFrustVisObjectWrapper::MovableChanged (iMovable* /*movable*/)
   frustvis->AddObjectToUpdateQueue (this);
 }
 
+class csFrustVisObjectDescriptor : public scfImplementation1<
+	csFrustVisObjectDescriptor, iKDTreeObjectDescriptor>
+{
+public:
+  csFrustVisObjectDescriptor () : scfImplementationType (this) { }
+  ~csFrustVisObjectDescriptor () { }
+  virtual csPtr<iString> DescribeObject (csKDTreeChild* child)
+  {
+    scfString* str = new scfString ();
+    const csBox3& b = child->GetBBox ();
+    csFrustVisObjectWrapper* obj = (csFrustVisObjectWrapper*)(
+    	child->GetObject ());
+    str->Format ("'%s' (%g,%g,%g)-(%g,%g,%g)",
+    	obj->mesh->QueryObject ()->GetName (),
+	b.MinX (), b.MinY (), b.MinZ (),
+	b.MaxX (), b.MaxY (), b.MaxZ ());
+    return str;
+  }
+};
+
 //----------------------------------------------------------------------
 
-csFrustumVis::csFrustumVis (iBase *iParent)
+csFrustumVis::csFrustumVis (iBase *iParent) :
+	vistest_objects (256, 256),
+	visobj_vector (256, 256),
+	update_queue (151, 59)
 {
   SCF_CONSTRUCT_IBASE (iParent);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
@@ -153,13 +183,23 @@ csFrustumVis::csFrustumVis (iBase *iParent)
   current_vistest_nr = 1;
   vistest_objects_inuse = false;
   updating = false;
+
+  scfiEventHandler = new eiEventHandler (this);
 }
 
 csFrustumVis::~csFrustumVis ()
 {
+  if (scfiEventHandler)
+  {
+    csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+    if (q)
+      q->RemoveListener (scfiEventHandler);
+    scfiEventHandler->DecRef ();
+  }
+
   while (visobj_vector.Length () > 0)
   {
-    csFrustVisObjectWrapper* visobj_wrap = visobj_vector[0];
+    csFrustVisObjectWrapper* visobj_wrap = visobj_vector.Pop ();
     iVisibilityObject* visobj = visobj_wrap->visobj;
     visobj->GetObjectModel ()->RemoveListener (
 		      (iObjectModelListener*)visobj_wrap);
@@ -167,20 +207,31 @@ csFrustumVis::~csFrustumVis ()
     movable->RemoveListener ((iMovableListener*)visobj_wrap);
     kdtree->RemoveObject (visobj_wrap->child);
     visobj->DecRef ();
-    visobj_vector.DeleteIndex (0);
   }
   delete kdtree;
   SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
   SCF_DESTRUCT_IBASE ();
 }
 
+bool csFrustumVis::HandleEvent (iEvent& ev)
+{
+  if (ev.Name == CanvasResize)
+  {
+    csRef<iGraphics3D> g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+    scr_width = g3d->GetWidth ();
+    scr_height = g3d->GetHeight ();
+    //printf ("Got resize %dx%d!\n", scr_width, scr_height);fflush (stdout);
+  }
+  return false;
+}
+
 bool csFrustumVis::Initialize (iObjectRegistry *object_reg)
 {
   csFrustumVis::object_reg = object_reg;
 
   delete kdtree;
 
-  csRef<iGraphics3D> g3d (CS_QUERY_REGISTRY (object_reg, iGraphics3D));
+  csRef<iGraphics3D> g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
   if (g3d)
   {
     scr_width = g3d->GetWidth ();
@@ -194,6 +245,15 @@ bool csFrustumVis::Initialize (iObjectRegistry *object_reg)
   }
 
   kdtree = new csKDTree ();
+  csFrustVisObjectDescriptor* desc = new csFrustVisObjectDescriptor ();
+  kdtree->SetObjectDescriptor (desc);
+  desc->DecRef ();
+
+  csRef<iGraphics2D> g2d = csQueryRegistry<iGraphics2D> (object_reg);
+  CanvasResize = csevCanvasResize(object_reg, g2d);
+  csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+  if (q)
+    q->RegisterListener (scfiEventHandler, CanvasResize);
 
   return true;
 }
@@ -207,12 +267,12 @@ void csFrustumVis::CalculateVisObjBBox (iVisibilityObject* visobj, csBox3& bbox)
   iMovable* movable = visobj->GetMovable ();
   if (movable->IsFullTransformIdentity ())
   {
-    visobj->GetObjectModel ()->GetObjectBoundingBox (bbox, CS_BBOX_MAX);
+    visobj->GetObjectModel ()->GetObjectBoundingBox (bbox);
   }
   else
   {
     csBox3 box;
-    visobj->GetObjectModel ()->GetObjectBoundingBox (box, CS_BBOX_MAX);
+    visobj->GetObjectModel ()->GetObjectBoundingBox (box);
     csReversibleTransform trans = movable->GetFullTransform ();
     bbox.StartBoundingBox (trans.This2Other (box.GetCorner (0)));
     bbox.AddBoundingVertexSmart (trans.This2Other (box.GetCorner (1)));
@@ -228,7 +288,7 @@ void csFrustumVis::CalculateVisObjBBox (iVisibilityObject* visobj, csBox3& bbox)
 void csFrustumVis::RegisterVisObject (iVisibilityObject* visobj)
 {
 #ifdef CS_DEBUG
-  int i;
+  size_t i;
   for (i = 0 ; i < visobj_vector.Length () ; i++)
   {
     if (visobj_vector[i]->visobj == visobj)
@@ -269,7 +329,7 @@ void csFrustumVis::RegisterVisObject (iVisibilityObject* visobj)
 
 void csFrustumVis::UnregisterVisObject (iVisibilityObject* visobj)
 {
-  int i;
+  size_t i;
   for (i = 0 ; i < visobj_vector.Length () ; i++)
   {
     csFrustVisObjectWrapper* visobj_wrap = visobj_vector[i];
@@ -286,7 +346,7 @@ void csFrustumVis::UnregisterVisObject (iVisibilityObject* visobj)
       // To easily recognize that the vis wrapper has been deleted:
       visobj_wrap->frustvis = (csFrustumVis*)0xdeadbeef;
 #endif
-      visobj_vector.DeleteIndex (i);
+      visobj_vector.DeleteIndexFast (i);
       return;
     }
   }
@@ -303,7 +363,8 @@ void csFrustumVis::UpdateObjects ()
 {
   updating = true;
   {
-    csSet<csFrustVisObjectWrapper*>::GlobalIterator it = update_queue.GetIterator ();
+    csSet<csPtrKey<csFrustVisObjectWrapper> >::GlobalIterator it = 
+      update_queue.GetIterator ();
     while (it.HasNext ())
     {
       csFrustVisObjectWrapper* vw = it.Next ();
@@ -461,13 +522,16 @@ void csFrustumVis::FrustTest_Traverse (csKDTree* treenode,
 bool csFrustumVis::VisTest (iRenderView* rview, 
                             iVisibilityCullerListener* viscallback)
 {
+  // We update the objects before testing the callback so that
+  // we can use this VisTest() call to make sure the objects in the
+  // culler are precached.
+  UpdateObjects ();
+  current_vistest_nr++;
+
   // just make sure we have a callback
   if (viscallback == 0)
     return false;
 
-  UpdateObjects ();
-  current_vistest_nr++;
-
   // Data for the vis tester.
   FrustTest_Front2BackData data;
 
@@ -572,7 +636,7 @@ csPtr<iVisibilityObjectIterator> csFrustumVis::VisTest (csPlane3* planes,
   else
   {
     v = &vistest_objects;
-    vistest_objects.DeleteAll ();
+    vistest_objects.Empty ();
   }
   
   FrustTestPlanes_Front2BackData data;
@@ -672,7 +736,7 @@ csPtr<iVisibilityObjectIterator> csFrustumVis::VisTest (const csBox3& box)
   else
   {
     v = &vistest_objects;
-    vistest_objects.DeleteAll ();
+    vistest_objects.Empty ();
   }
   
   FrustTestBox_Front2BackData data;
@@ -765,7 +829,7 @@ csPtr<iVisibilityObjectIterator> csFrustumVis::VisTest (const csSphere& sphere)
   else
   {
     v = &vistest_objects;
-    vistest_objects.DeleteAll ();
+    vistest_objects.Empty ();
   }
 
   FrustTestSphere_Front2BackData data;
@@ -811,6 +875,54 @@ struct IntersectSegment_Front2BackData
   bool accurate;
 };
 
+static bool IntersectSegmentSloppy_Front2Back (csKDTree* treenode,
+	void* userdata, uint32 cur_timestamp, uint32&)
+{
+  IntersectSegment_Front2BackData* data
+  	= (IntersectSegment_Front2BackData*)userdata;
+
+  const csBox3& node_bbox = treenode->GetNodeBBox ();
+
+  // In the first part of this test we are going to test if the
+  // start-end vector intersects with the node. If not then we don't
+  // need to continue.
+  csVector3 box_isect;
+  if (csIntersect3::BoxSegment (node_bbox, data->seg, box_isect) == -1)
+  {
+    return false;
+  }
+
+  treenode->Distribute ();
+
+  int num_objects;
+  csKDTreeChild** objects;
+  num_objects = treenode->GetObjectCount ();
+  objects = treenode->GetObjects ();
+
+  int i;
+  for (i = 0 ; i < num_objects ; i++)
+  {
+    if (objects[i]->timestamp != cur_timestamp)
+    {
+      objects[i]->timestamp = cur_timestamp;
+      csFrustVisObjectWrapper* visobj_wrap = (csFrustVisObjectWrapper*)
+      	objects[i]->GetObject ();
+
+      // First test the bounding box of the object.
+      const csBox3& obj_bbox = visobj_wrap->child->GetBBox ();
+
+      if (csIntersect3::BoxSegment (obj_bbox, data->seg, box_isect) != -1)
+      {
+        // This object is possibly intersected by this beam.
+	if (visobj_wrap->mesh)
+	  if (!visobj_wrap->mesh->GetFlags ().Check (CS_ENTITY_NOHITBEAM))
+	    data->vector->Push (visobj_wrap->visobj);
+      }
+    }
+  }
+  return true;
+}
+
 static bool IntersectSegment_Front2Back (csKDTree* treenode,
 	void* userdata, uint32 cur_timestamp, uint32&)
 {
@@ -935,6 +1047,7 @@ bool csFrustumVis::IntersectSegment (const csVector3& start,
   data.polygon_idx = -1;
   data.vector = 0;
   data.accurate = accurate;
+  data.isect = 0;
   kdtree->Front2Back (start, IntersectSegment_Front2Back, (void*)&data, 0);
 
   if (p_mesh) *p_mesh = data.mesh;
@@ -964,6 +1077,21 @@ csPtr<iVisibilityObjectIterator> csFrustumVis::IntersectSegment (
   return csPtr<iVisibilityObjectIterator> (vobjit);
 }
 
+csPtr<iVisibilityObjectIterator> csFrustumVis::IntersectSegmentSloppy (
+    const csVector3& start, const csVector3& end)
+{
+  UpdateObjects ();
+  current_vistest_nr++;
+  IntersectSegment_Front2BackData data;
+  data.seg.Set (start, end);
+  data.vector = new csArray<iVisibilityObject*> ();
+  kdtree->Front2Back (start, IntersectSegmentSloppy_Front2Back,
+  	(void*)&data, 0);
+
+  csFrustVisObjIt* vobjit = new csFrustVisObjIt (data.vector, 0);
+  return csPtr<iVisibilityObjectIterator> (vobjit);
+}
+
 //======== CastShadows =====================================================
 
 struct ShadObj
@@ -1040,7 +1168,7 @@ static bool CastShadows_Front2Back (csKDTree* treenode, void* userdata,
       if (b.SquaredOriginDist () > sqrad)
 	continue;
 
-      if (visobj_wrap->caster && fview->ThingShadowsEnabled () &&
+      if (visobj_wrap->caster &&
             fview->CheckShadowMask (visobj_wrap->mesh->GetFlags ().Get ()))
       {
         data->shadobjs[data->num_shadobjs].sqdist = b.SquaredOriginDist ();
@@ -1101,7 +1229,7 @@ void csFrustumVis::CastShadows (iFrustumView* fview)
   CS_ASSERT (lf->GetVertexCount () <= 31);
   if (lf->GetVertexCount () > 31)
   {
-    printf ("INTERNAL ERROR! #vertices in GetVisibleObjects() exceeded!\n");
+    csPrintf ("INTERNAL ERROR! #vertices in GetVisibleObjects() exceeded!\n");
     fflush (stdout);
     return;
   }
