@@ -1,5 +1,6 @@
 /*
     Copyright (C) 2002 by Norman Kramer
+    Multi-axis support (C) 2005 Adam D. Bradley <artdodge@cs.bu.edu>
   
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -19,22 +20,28 @@
 #include "cssysdef.h"
 #include "csver.h"
 #include "csjoylin.h"
+#include "iutil/verbositymanager.h"
 #include "ivaria/reporter.h"
 #include "csutil/csstring.h"
+#include "csutil/event.h"
 #include "csutil/array.h"
+#include "iutil/csinput.h" /* for JS max/mins */
 
+#include <errno.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 #include <linux/joystick.h>
 
 #define CS_LINUX_JOYSTICK_CFG "/config/joystick.cfg"
-#define CS_LINUX_JOYSTICK_KEY "Device.Joystick."
-
-#define CS_LINUX_JOYSTICK_OLD_EVENTS // the values of the first two axis are sent only
+#define CS_LINUX_JOYSTICK_KEY "Device.Joystick." CS_PLATFORM_NAME "."
 
 CS_IMPLEMENT_PLUGIN;
 
+SCF_IMPLEMENT_FACTORY (csLinuxJoystick);
+
 SCF_IMPLEMENT_IBASE (csLinuxJoystick)
   SCF_IMPLEMENTS_INTERFACE (iComponent)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iEventPlug)
@@ -75,29 +82,36 @@ bool csLinuxJoystick::Initialize (iObjectRegistry *oreg)
   return Init ();
 }
 
-bool csLinuxJoystick::HandleEvent (iEvent &)
+#define CS_MAX_LINUX_JOYSTICK_AXES CS_MAX_JOYSTICK_AXES
+bool csLinuxJoystick::HandleEvent (iEvent& ev)
 {
+  if (ev.Name != PreProcess)
+    return false;
+
   struct js_event js;
 
-  for (int i=0; i < nJoy; i++)
+  for (int i = 0; i < nJoy; i++)
   {
-    while (read (joystick [i].fd, &js, sizeof(struct js_event)) == sizeof(struct js_event)) 
+    joydata& jd = joystick[i];
+    while (read (jd.fd, &js, sizeof(js)) == sizeof(js)) 
     {
+      int32 axisread[CS_MAX_LINUX_JOYSTICK_AXES];
+      if ((js.type & ~JS_EVENT_INIT) == JS_EVENT_AXIS)
+	jd.axis[js.number] = js.value;
+      for (int iter=0 ; 
+	   iter<MIN(jd.nAxes,CS_MAX_LINUX_JOYSTICK_AXES) ; 
+	   iter++)
+	axisread[iter] = jd.axis[iter]; /* "int16" to "int" */
       switch (js.type & ~JS_EVENT_INIT) 
       {
       case JS_EVENT_BUTTON:
-        //        DEBUG_BREAK;
-        joystick[i].button[js.number] = js.value;
-        EventOutlet->Joystick (i, js.number+1, js.value, -1, 0);
+        jd.button[js.number] = js.value;
+        EventOutlet->Joystick (jd.number, js.number, js.value,
+			       axisread, jd.nAxes);
         break;
       case JS_EVENT_AXIS:
-        joystick[i].axis[js.number] = js.value;
-#ifdef CS_LINUX_JOYSTICK_OLD_EVENTS
-        if (js.number < 2)
-          EventOutlet->Joystick (i, 0, 0, joystick[i].axis[0], (joystick[i].nAxes > 1 ? joystick[i].axis[1] : 0));
-#else
-        EventOutlet->Joystick (i, 0, 0, js.number, js.value);
-#endif
+        EventOutlet->Joystick (jd.number, (uint) -1, 0, 
+			       axisread, jd.nAxes);
         break;
       }
     }
@@ -107,20 +121,16 @@ bool csLinuxJoystick::HandleEvent (iEvent &)
 
 bool csLinuxJoystick::Init ()
 {
-  // read list of devices
+  bool verbose = false;
+  csRef<iVerbosityManager> verbosemgr (
+    CS_QUERY_REGISTRY (object_reg, iVerbosityManager));
+  if (verbosemgr) 
+    verbose = verbosemgr->Enabled ("joystick");
+
   config.AddConfig (object_reg, CS_LINUX_JOYSTICK_CFG);
-
-  // create the key
-  csString key(CS_LINUX_JOYSTICK_KEY);
-  key += CS_PLATFORM_NAME;
-  key += ".";
-  key += CS_COMPILER_NAME;
-  key += ".";
-
   csRef<iConfigIterator> it (config->Enumerate (CS_LINUX_JOYSTICK_KEY));
-
   csArray<int> fds;
-  int fd;
+  csArray<dev_t> devids;
 
   nJoy=0;
   bHooked = false;
@@ -128,16 +138,42 @@ bool csLinuxJoystick::Init ()
 
   while (it->Next ())
   {
-    if ((fd = open (it->GetStr (), O_RDONLY)) < 0) 
+    int fd = open (it->GetStr (), O_RDONLY);
+    if (fd >= 0) 
     {
-      Report (CS_REPORTER_SEVERITY_NOTIFY, 
+      Report (CS_REPORTER_SEVERITY_DEBUG,
+	      "Opened joystick device %s as fd#%d\n",
+	      it->GetStr(), fd);
+      struct stat st;
+      fstat(fd, &st);
+
+      /* check devids for a dup st.st_dev */
+      for (csArray<dev_t>::Iterator i = devids.GetIterator(); i.HasNext(); )
+      {
+	if (i.Next() == st.st_dev)
+	{
+	  close(fd);
+	  fd = -1;
+	  if (verbose)
+	    Report (CS_REPORTER_SEVERITY_WARNING,
+		    "Found duplicate joystick device %s\n", it->GetStr());
+	  break;
+	}
+      }
+      if (fd != -1)
+      {
+	nJoy++;
+	fds.Push((const int) fd);
+	devids.Push(st.st_dev);
+      }
+    }
+    else if (verbose || errno != ENOENT)
+    {
+      Report (CS_REPORTER_SEVERITY_WARNING,
               "Failed to open joystick device %s - error: %s\n",
               it->GetStr (),
               strerror (errno));
     }
-    else
-      nJoy++;
-    fds.Push(fd);
   }
 
   if (nJoy)
@@ -149,40 +185,41 @@ bool csLinuxJoystick::Init ()
     int n = 0;
     while (i.HasNext())
     {
-      fd = i.Next();
-      if (fd > -1)
-      {
-	unsigned char axes = 2;
-	unsigned char buttons = 2;
-	int version = 0x000800;
-	char name[128] = "Unknown";
+      int const fd = i.Next();
+      unsigned char axes = 2;
+      unsigned char buttons = 2;
+      int version = 0x000800;
+      char name[128] = "Unknown";
 
-	ioctl (fd, JSIOCGVERSION, &version);
-	ioctl (fd, JSIOCGAXES, &axes);
-	ioctl (fd, JSIOCGBUTTONS, &buttons);
-	ioctl (fd, JSIOCGNAME(128), name);
+      ioctl (fd, JSIOCGVERSION, &version);
+      ioctl (fd, JSIOCGAXES, &axes);
+      ioctl (fd, JSIOCGBUTTONS, &buttons);
+      ioctl (fd, JSIOCGNAME(128), name);
 
-	Report (CS_REPORTER_SEVERITY_NOTIFY,
-                "Joystick number %d (%s) has %d axes and %d buttons. Driver version is %d.%d.%d.\n",
-		n, name, axes, buttons, version >> 16, (version >> 8) & 0xff, version & 0xff);
-        joystick[n].number = n;
-        joystick[n].fd = fd;
-        joystick[n].nButtons = buttons;
-        joystick[n].nAxes = axes;
-        joystick[n].axis = new int16[axes];
-        joystick[n].button = new int16[buttons];
+      Report (CS_REPORTER_SEVERITY_NOTIFY,
+              "Joystick number %d (%s) has %hhu axes and %hhu buttons.\n"
+	      "Driver version is %d.%d.%d.\n",
+	      n, name, axes, buttons,
+	      version >> 16, (version >> 8) & 0xff, version & 0xff);
 
-        // make device non blocking
-        fcntl(fd, F_SETFL, O_NONBLOCK);
-        n++;
-      }
+      joydata& jd = joystick[n];
+      jd.number = n; // CS joystick numbers are 0-based.
+      jd.fd = fd;
+      jd.nButtons = buttons;
+      jd.nAxes = axes;
+      jd.axis = new int16[axes];
+      jd.button = new int16[buttons];
+
+      fcntl(fd, F_SETFL, O_NONBLOCK);
+      n++;
     }
 
     // hook into eventqueue
+    PreProcess = csevPreProcess (object_reg);
     csRef<iEventQueue> eq (CS_QUERY_REGISTRY(object_reg, iEventQueue));
     if (eq != 0)
     {
-      eq->RegisterListener (&scfiEventHandler, CSMASK_Nothing);
+      eq->RegisterListener (&scfiEventHandler, PreProcess);
       EventOutlet = eq->CreateEventOutlet (&scfiEventPlug);
       bHooked = true;
     }
@@ -232,6 +269,3 @@ void csLinuxJoystick::Report (int severity, const char* msg, ...)
   }
   va_end (arg);
 }
-
-SCF_IMPLEMENT_FACTORY (csLinuxJoystick);
-
