@@ -22,28 +22,33 @@
 #ifndef __CS_FONTPLEX_H__
 #define __CS_FONTPLEX_H__
 
+#include "csutil/scf_implementation.h"
 #include "ivideo/fontserv.h"
+#include "csutil/weakref.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
 #include "csutil/refarr.h"
 #include "csutil/cfgacc.h"
 #include "csutil/hash.h"
-#include "csutil/hashhandlers.h"
 
-class csFontServerMultiplexor;
+namespace cspluginFontplex
+{
+
+class csFontServerMultiplexer;
 class csFontPlexer;
 
 struct csFontLoadOrderEntry
 {
-  char* fontName;
-  csRef<iFontServer> server;
+  csString fontName;
+  csRefArray<iFontServer> servers;
+  bool fallback;
 
   bool loaded;
   csRef<iFont> font;
   float scale;
 
-  csFontLoadOrderEntry (iFontServer* server, const char* fontName,
-    float scale);
+  csFontLoadOrderEntry (csRefArray<iFontServer> servers, const char* fontName,
+    float scale, bool fallback);
   csFontLoadOrderEntry (const csFontLoadOrderEntry& other);
   ~csFontLoadOrderEntry ();
 
@@ -58,27 +63,26 @@ public:
   void AppendSmart (const csFontLoaderOrder& other);
 };
 
-class csFontPlexer : public iFont
+class csFontPlexer : public scfImplementation1<csFontPlexer, 
+                                               iFont>
 {
 private:
   friend struct csFontLoadOrderEntry;
 
-  csRef<csFontServerMultiplexor> parent;
-  char* fontid;
-  int size;
-  iFont* primaryFont;
+  csString fontid;
+  float size;
 
   csFontLoaderOrder* order;
   csRefArray<iFontDeleteNotify> DeleteCallbacks;
 public:
-  SCF_DECLARE_IBASE;
+  csRef<csFontServerMultiplexer> parent;
+  iFont* primaryFont;
 
-  csFontPlexer (csFontServerMultiplexor* parent, 
-    char* fontid, iFont* primary, 
-    int size, csFontLoaderOrder* order);
+  csFontPlexer (csFontServerMultiplexer* parent, 
+    const char* fontid, float size, csFontLoaderOrder* order);
   virtual ~csFontPlexer ();
 
-  virtual int GetSize ();
+  virtual float GetSize ();
   virtual void GetMaxSize (int &oW, int &oH);
   virtual bool GetGlyphMetrics (utf32_char c, csGlyphMetrics& metrics);
 
@@ -97,6 +101,11 @@ public:
   virtual int GetDescent (); 
   virtual int GetAscent (); 
   virtual bool HasGlyph (utf32_char c); 
+
+  virtual int GetTextHeight ();
+  virtual int GetUnderlinePosition ();
+  virtual int GetUnderlineThickness ();
+
 };
 
 /**
@@ -109,48 +118,58 @@ public:
  * to auxiliary font servers. Example extract from config file:
  * <code>
  * ...
- * System.Plugins.iFontServer = crystalspace.font.server.multiplexor
+ * System.Plugins.iFontServer = crystalspace.font.server.multiplexer
  * System.Plugins.iFontServer.1 = crystalspace.font.server.freetype2
  * System.Plugins.iFontServer.2 = crystalspace.font.server.default
  * ...
  * </code>
  */
-class csFontServerMultiplexor : public iFontServer
+class csFontServerMultiplexer : 
+  public scfImplementation2<csFontServerMultiplexer, 
+                            iFontServer,
+                            iComponent>
 {
 private:
-  csRef<iObjectRegistry> object_reg;
+  iObjectRegistry* object_reg;
   csRefArray<iFontServer> fontservers;
 
   csConfigAccess config;
-  const char *fontset;
+  csString fontset;
 
   struct FontServerMapEntry
   {
     csRef<iFontServer> server;
-    char* name;
+    csString name;
 
     FontServerMapEntry (const char* name, iFontServer* server);
     FontServerMapEntry (const FontServerMapEntry& source);
     ~FontServerMapEntry ();
   };
-  csHash<FontServerMapEntry, csStrKey, 
-    csConstCharHashKeyHandler> fontServerMap;
-  csHash<iFont*, const char*, csConstCharHashKeyHandler> loadedFonts;
+  csHash<FontServerMapEntry, csStrKey> fontServerMap;
+  csHash<iFont*, csString> loadedFonts;
 
   csFontLoaderOrder fallbackOrder;
+  bool emitErrors;
+
+  int GetErrorSeverity () const
+  { 
+    return emitErrors ? 
+      CS_REPORTER_SEVERITY_WARNING : CS_REPORTER_SEVERITY_NOTIFY; 
+  }
+  csHash<int, csString> fontsNotFound;
 
   void ParseFontLoaderOrder (csFontLoaderOrder& order, 
-    const char* str);
+    const char* str, bool fallback);
   csPtr<iFontServer> ResolveFontServer (const char* name);
 public:
-  void NotifyDelete (csFontPlexer* font, char* fontid);
+  void ReportFontNotFound (bool fallback, const char* font);
 
-  SCF_DECLARE_IBASE;
+  void NotifyDelete (csFontPlexer* font, const char* fontid);
 
   /// Create the plugin object
-  csFontServerMultiplexor (iBase *pParent);
+  csFontServerMultiplexer (iBase *pParent);
   /// Destructor: nothing to do
-  virtual ~csFontServerMultiplexor ();
+  virtual ~csFontServerMultiplexer ();
 
   /// Register plugin with the system driver
   virtual bool Initialize (iObjectRegistry *object_reg);
@@ -159,14 +178,14 @@ public:
    * Load a font by name.
    * Returns a new iFont object or 0 on failure.
    */
-  virtual csPtr<iFont> LoadFont (const char *filename, int size = 10);
+  virtual csPtr<iFont> LoadFont (const char *filename, float size = 10.0f);
 
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csFontServerMultiplexor);
-    virtual bool Initialize (iObjectRegistry* p)
-    { return scfParent->Initialize(p); }
-  } scfiComponent;
+  virtual void SetWarnOnError (bool enable)
+  { emitErrors = enable; }
+  virtual bool GetWarnOnError ()
+  { return emitErrors; }
 };
 
+} // namespace cspluginFontplex
+
 #endif // __CS_FONTPLEX_H__
