@@ -17,46 +17,49 @@
 */
 
 #include "cssysdef.h"
-#include "csutil/csendian.h"
-#include "qint.h"
-#include "beziermsh.h"
-#include "lightpool.h"
-#include "curvebase.h"
-#include "csgeom/polypool.h"
-#include "csgeom/poly3d.h"
+#include "csqint.h"
+#include "csqsqrt.h"
+
 #include "csgeom/frustum.h"
-#include "iengine/light.h"
+#include "csgeom/math3d.h"
+#include "csgeom/poly3d.h"
+#include "csgeom/polypool.h"
+#include "csgeom/sphere.h"
+#include "csgfx/renderbuffer.h"
+#include "csutil/array.h"
+#include "csutil/csendian.h"
+#include "csutil/csmd5.h"
+#include "csutil/csstring.h"
+#include "csutil/debug.h"
+#include "csutil/dirtyaccessarray.h"
+#include "csutil/memfile.h"
+#include "iengine/camera.h"
 #include "iengine/engine.h"
-#include "iengine/sector.h"
-#include "iengine/movable.h"
+#include "iengine/fview.h"
+#include "iengine/light.h"
 #include "iengine/material.h"
 #include "iengine/mesh.h"
-#include "iengine/camera.h"
-#include "iengine/shadows.h"
-#include "csgeom/sphere.h"
-#include "csgeom/math3d.h"
-#include "csutil/csstring.h"
-#include "csutil/memfile.h"
-#include "csutil/hashmap.h"
-#include "csutil/debug.h"
-#include "csutil/csmd5.h"
-#include "csutil/array.h"
-#include "csutil/garray.h"
-#include "ivideo/txtmgr.h"
-#include "ivideo/vbufmgr.h"
-#include "ivideo/texture.h"
-#include "iengine/texture.h"
-#include "iengine/shadcast.h"
-#include "iutil/vfs.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/cache.h"
-#include "iutil/cmdline.h"
+#include "iengine/movable.h"
 #include "iengine/rview.h"
-#include "iengine/fview.h"
-#include "qsqrt.h"
-#include "ivideo/graph3d.h"
+#include "iengine/sector.h"
+#include "iengine/shadcast.h"
+#include "iengine/shadows.h"
+#include "iengine/texture.h"
+#include "iutil/cache.h"
+#include "iutil/comp.h"
+#include "iutil/databuff.h"
+#include "iutil/eventh.h"
+#include "iutil/verbositymanager.h"
+#include "iutil/vfs.h"
 #include "ivaria/reporter.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/rendermesh.h"
+#include "ivideo/texture.h"
+#include "ivideo/txtmgr.h"
+
+#include "lightpool.h"
+#include "curvebase.h"
+#include "beziermsh.h"
 
 CS_IMPLEMENT_PLUGIN
 
@@ -76,10 +79,10 @@ SCF_IMPLEMENT_IBASE(csBezierMesh)
     if (iPolygonMesh_scfID == (scfInterfaceID)-1)				
       iPolygonMesh_scfID = iSCF::SCF->GetInterfaceID ("iPolygonMesh");		
     if (iInterfaceID == iPolygonMesh_scfID &&				
-      scfCompatibleVersion(iVersion, scfInterface<iPolygonMesh>::GetVersion()))
+      scfCompatibleVersion(iVersion, scfInterfaceTraits<iPolygonMesh>::GetVersion()))
     {
 #ifdef CS_DEBUG
-      printf ("Deprecated feature use: iPolygonMesh queried from Bezier "
+      csPrintf ("Deprecated feature use: iPolygonMesh queried from Bezier "
 	"object; use iObjectModel->GetPolygonMeshColldet() instead.\n");
 #endif
       iPolygonMesh* Object = scfiObjectModel.GetPolygonMeshColldet();
@@ -125,11 +128,10 @@ int csBezierMesh:: last_thing_id = 0;
 
 //----------------------------------------------------------------------------
 
-csBezierMeshStatic::csBezierMeshStatic (csBezierMesh* thing,
+csBezierMeshStatic::csBezierMeshStatic (
 	csBezierMeshObjectType* thing_type,
 	iBezierFactoryState* factory_state)
 {
-  csBezierMeshStatic::thing = thing;
   csBezierMeshStatic::thing_type = thing_type;
   csBezierMeshStatic::factory_state = factory_state;
 
@@ -161,6 +163,11 @@ void csBezierMeshStatic::Prepare ()
 // Option variable: cosinus factor.
 float csBezierMesh:: cfg_cosinus_factor = 0;
 
+csStringID csBezierMesh::vertex_name = csInvalidStringID;
+csStringID csBezierMesh::texel_name = csInvalidStringID;
+csStringID csBezierMesh::color_name = csInvalidStringID;
+csStringID csBezierMesh::index_name = csInvalidStringID;
+
 csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
   curves (4, 16)
 {
@@ -175,7 +182,7 @@ csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiMeshObjectFactory);
 
   static_data = new csBezierMeshStatic (
-  	this, thing_type, &scfiBezierFactoryState);
+  	thing_type, &scfiBezierFactoryState);
 
   scfiPolygonMesh.SetThing (this);
   scfiPolygonMeshLOD.SetThing (this);
@@ -187,8 +194,9 @@ csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
   last_thing_id++;
   thing_id = last_thing_id;
   logparent = 0;
+  logparent_factory = 0;
+  beziermsh_type = thing_type;
 
-  dynamic_ambient.Set (0,0,0);
   light_version = 1;
 
   ParentTemplate = 0;
@@ -208,6 +216,23 @@ csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
 
   current_visnr = 1;
   cosinus_factor = -1;
+
+  dynamic_ambient_version = 0;
+
+  csRef<iStringSet> strings;
+  strings = CS_QUERY_REGISTRY_TAG_INTERFACE (thing_type->object_reg,
+    "crystalspace.shared.stringset", iStringSet);
+
+  if ((vertex_name == csInvalidStringID) ||
+    (texel_name == csInvalidStringID) ||
+    (color_name == csInvalidStringID) ||
+    (index_name == csInvalidStringID))
+  {
+    vertex_name = strings->Request ("vertices");
+    texel_name = strings->Request ("texture coordinates");
+    color_name = strings->Request ("colors");
+    index_name = strings->Request ("indices");
+  }
 }
 
 csBezierMesh::~csBezierMesh ()
@@ -232,39 +257,23 @@ char* csBezierMesh::GenerateCacheName ()
 
   csMemFile mf;
   int32 l;
-  l = convert_endian ((int32)static_data->num_curve_vertices);
+  l = csLittleEndian::Convert ((int32)static_data->num_curve_vertices);
   mf.Write ((char*)&l, 4);
-  l = convert_endian ((int32)curves.Length ());
+  l = csLittleEndian::Convert ((int32)curves.Length ());
   mf.Write ((char*)&l, 4);
 
   if (logparent)
   {
-    csRef<iMeshWrapper> mw (SCF_QUERY_INTERFACE (logparent, iMeshWrapper));
-    if (mw)
-    {
-      if (mw->QueryObject ()->GetName ())
-        mf.Write (mw->QueryObject ()->GetName (),
+    iMeshWrapper* mw = logparent;
+    if (mw->QueryObject ()->GetName ())
+      mf.Write (mw->QueryObject ()->GetName (),
 		strlen (mw->QueryObject ()->GetName ()));
-      iSector* sect = mw->GetMovable ()->GetSectors ()->Get (0);
-      if (sect && sect->QueryObject ()->GetName ())
-        mf.Write (sect->QueryObject ()->GetName (),
+    iSector* sect = mw->GetMovable ()->GetSectors ()->Get (0);
+    if (sect && sect->QueryObject ()->GetName ())
+      mf.Write (sect->QueryObject ()->GetName (),
 		strlen (sect->QueryObject ()->GetName ()));
-    }
   }
 
-  l = convert_endian ((int32)QInt ((b.MinX () * 1000)+.5));
-  mf.Write ((char*)&l, 4);
-  l = convert_endian ((int32)QInt ((b.MinY () * 1000)+.5));
-  mf.Write ((char*)&l, 4);
-  l = convert_endian ((int32)QInt ((b.MinZ () * 1000)+.5));
-  mf.Write ((char*)&l, 4);
-  l = convert_endian ((int32)QInt ((b.MaxX () * 1000)+.5));
-  mf.Write ((char*)&l, 4);
-  l = convert_endian ((int32)QInt ((b.MaxY () * 1000)+.5));
-  mf.Write ((char*)&l, 4);
-  l = convert_endian ((int32)QInt ((b.MaxZ () * 1000)+.5));
-  mf.Write ((char*)&l, 4);
-
   csMD5::Digest digest = csMD5::Encode (mf.GetData (), mf.GetSize ());
   csString hex(digest.HexString());
   return hex.Detach();
@@ -283,11 +292,11 @@ void csBezierMesh::LightChanged (iLight* /*light*/)
 void csBezierMesh::LightDisconnect (iLight* light)
 {
   MarkLightmapsDirty ();
-  int i;
+  size_t i;
   int dt = light->GetDynamicType ();
   for (i = 0; i < curves.Length (); i++)
   {
-    csCurve *c = GetCurve (i);
+    csCurve *c = GetCurve ((int)i);
     if (dt == CS_LIGHT_DYNAMICTYPE_DYNAMIC)
       c->DynamicLightDisconnect (light);
     else
@@ -295,6 +304,17 @@ void csBezierMesh::LightDisconnect (iLight* light)
   }
 }
 
+void csBezierMesh::DisconnectAllLights ()
+{
+  MarkLightmapsDirty ();
+  size_t i;
+  for (i = 0; i < curves.Length (); i++)
+  {
+    csCurve *c = GetCurve ((int)i);
+    c->DisconnectAllLights ();
+  }
+}
+
 void csBezierMesh::WorUpdate ()
 {
   if (cached_movable && cached_movable->GetUpdateNumber () != movablenr)
@@ -411,7 +431,7 @@ iPolygonMesh* csBezierMesh::GetWriteObject ()
 
 csCurve *csBezierMesh::GetCurve (char *name) const
 {
-  int i;
+  size_t i;
   for (i = 0 ; i < curves.Length () ; i++)
   {
     const char* n = curves[i]->GetName ();
@@ -439,7 +459,7 @@ iCurve *csBezierMesh::CreateCurve ()
 
 int csBezierMesh::FindCurveIndex (iCurve *curve) const
 {
-  return curves.Find (curve->GetOriginalObject ());
+  return (int)curves.Find (curve->GetOriginalObject ());
 }
 
 void csBezierMesh::RemoveCurve (int idx)
@@ -458,17 +478,18 @@ void csBezierMesh::RemoveCurves ()
 
 void csBezierMesh::HardTransform (const csReversibleTransform &t)
 {
-  int i;
+  size_t i;
 
   static_data->curves_center = t.This2Other (static_data->curves_center);
   if (static_data->curve_vertices)
-    for (i = 0; i < static_data->num_curve_vertices; i++)
-      static_data->curve_vertices[i] = t.This2Other (static_data->curve_vertices[i]);
+    for (i = 0; i < (size_t)static_data->num_curve_vertices; i++)
+      static_data->curve_vertices[i] = t.This2Other (
+      	static_data->curve_vertices[i]);
 
   curves_transf_ok = false;
   for (i = 0; i < curves.Length (); i++)
   {
-    csCurve *c = GetCurve (i);
+    csCurve *c = GetCurve ((int)i);
     c->HardTransform (t);
   }
 }
@@ -553,7 +574,7 @@ float csBezierMesh::GetScreenBoundingBox (
 void csBezierMesh::AppendShadows (
   iMovable* movable,
   iShadowBlockList *shadows,
-  const csVector3 &origin)
+  const csVector3& /*origin*/)
 {
   Prepare ();
   //@@@ Ok?
@@ -630,11 +651,18 @@ void csBezierMesh::GetBoundingBox (csBox3 &box)
   }
 
   static_data->obj_radius = (static_data->obj_bbox.Max () - static_data->obj_bbox.Min ()) * 0.5f;
-  static_data->max_obj_radius = qsqrt (csSquaredDist::PointPoint (
+  static_data->max_obj_radius = csQsqrt (csSquaredDist::PointPoint (
   	static_data->obj_bbox.Max (), static_data->obj_bbox.Min ())) * 0.5f;
   box = static_data->obj_bbox;
 }
 
+void csBezierMesh::SetBoundingBox (const csBox3& b)
+{
+  static_data->obj_bbox_valid = true;
+  static_data->obj_bbox = b;
+  scfiObjectModel.ShapeChanged ();
+}
+
 void csBezierMesh::GetBoundingBox (iMovable *movable, csBox3 &box)
 {
   if (wor_bbox_movablenr != movable->GetUpdateNumber ())
@@ -707,8 +735,8 @@ void BezierPolyMeshHelper::Setup ()
   {
     csCurve *c = thing->curves.Get (i);
     csCurveTesselated *tess = c->Tesselate (1000);    // @@@ High quality?
-    num_poly += tess->GetTriangleCount ();
-    num_verts += tess->GetVertexCount ();
+    num_poly += (int)tess->GetTriangleCount ();
+    num_verts += (int)tess->GetVertexCount ();
   }
 
   if (!num_verts || !num_poly) return;
@@ -724,7 +752,7 @@ void BezierPolyMeshHelper::Setup ()
     csCurve *c = thing->curves.Get (i);
     csCurveTesselated *tess = c->Tesselate (1000);  // @@@ High quality?
     csTriangle *tris = tess->GetTriangles ();
-    int tri_count = tess->GetTriangleCount ();
+    int tri_count = (int)tess->GetTriangleCount ();
     for (j = 0; j < tri_count; j++)
     {
       polygons[num_poly].num_vertices = 3;
@@ -739,7 +767,7 @@ void BezierPolyMeshHelper::Setup ()
     }
 
     csVector3 *vts = tess->GetVertices ();
-    int num_vt = tess->GetVertexCount ();
+    int num_vt = (int)tess->GetVertexCount ();
     memcpy (vertices + num_verts, vts, sizeof (csVector3) * num_vt);
     num_verts += num_vt;
   }
@@ -801,16 +829,10 @@ void csBezierMesh::UpdateCurveTransform ()
   }
 }
 
-/// The list of fog vertices
-typedef csDirtyAccessArray<G3DFogInfo> engine3d_StaticFogVerts;
-CS_IMPLEMENT_STATIC_VAR (GetStaticFogVerts, engine3d_StaticFogVerts,())
-
-bool csBezierMesh::DrawCurves (
-  iRenderView *rview,
-  iMovable *movable,
-  csZBufMode zMode)
+csRenderMesh** csBezierMesh::GetRenderMeshes (int &n, iRenderView* rview,
+					      iMovable* movable, 
+					      uint32 frustum_mask)
 {
-  static engine3d_StaticFogVerts &fog_verts = *GetStaticFogVerts ();
   if (GetCurveCount () <= 0) return false;
 
   iCamera *icam = rview->GetCamera ();
@@ -841,25 +863,43 @@ bool csBezierMesh::DrawCurves (
   // combining object to world and world to camera.
   csReversibleTransform obj_cam = camtrans;
   obj_cam /= movtrans;
-  rview->GetGraphics3D ()->SetObjectToCamera (&obj_cam);
-  rview->GetGraphics3D ()->SetRenderState (G3DRENDERSTATE_ZBUFFERMODE, zMode);
+ 
+  int clip_portal, clip_plane, clip_z_plane;
+  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
+      clip_z_plane);
 
-  // Base of the mesh.
-  G3DTriangleMesh mesh;
-  mesh.morph_factor = 0;
-  mesh.num_vertices_pool = 1;
-  mesh.do_mirror = icam->IsMirrored ();
-  mesh.do_morph_texels = false;
-  mesh.do_morph_colors = false;
-  mesh.vertex_mode = G3DTriangleMesh::VM_WORLDSPACE;
+  const uint currentFrame = rview->GetCurrentFrameNumber();
+  bool listCreated;
+  csDirtyAccessArray<csRenderMesh*>& meshes = rmListHolder.GetUnusedData (
+    listCreated, currentFrame);
+  meshes.SetLength (GetCurveCount(), 0);
 
-  iVertexBufferManager *vbufmgr = rview->GetGraphics3D ()
-    ->GetVertexBufferManager ();
-
-  // Loop over all curves
+  iSector* s = movable->GetSectors ()->Get (0);
+  csColor ambient = s->GetDynamicAmbientLight ();
+  bool update_ambient = false;
+  if (dynamic_ambient_version != s->GetDynamicAmbientVersion ())
+  {
+    update_ambient = true;
+    dynamic_ambient_version = s->GetDynamicAmbientVersion ();
+  }
   csCurve *c;
   for (i = 0; i < GetCurveCount (); i++)
   {
+    bool meshCreated;
+    csRenderMesh*& rm = rmHolder.GetUnusedMesh (meshCreated, currentFrame);
+    meshes[i] = rm;
+    if (meshCreated)
+    {
+      rm->buffers.AttachNew (new csRenderBufferHolder);
+      rm->variablecontext.AttachNew (new csShaderVariableContext);
+    }
+    rm->worldspace_origin = world_coord;
+    rm->clip_portal = clip_portal;
+    rm->clip_plane = clip_plane;
+    rm->clip_z_plane = clip_z_plane;
+    rm->do_mirror = icam->IsMirrored ();
+    rm->object2world = movtrans;
+
     c = curves.Get (i);
 
     // First get a bounding box in camera space.
@@ -876,72 +916,53 @@ bool csBezierMesh::DrawCurves (
 
     // If the lightmap was updated or the new tesselation doesn't yet
     // have a valid colors table we need to update colors here.
-    if (updated_lm || !tess->AreColorsValid ())
-      tess->UpdateColors (c->LightMap);
-
-    // Setup the structure for DrawTriangleMesh.
-    if (tess->GetVertexCount () > fog_verts.Length ())
-    {
-      fog_verts.SetLength (tess->GetVertexCount ());
-    }
+    if (update_ambient || updated_lm || !tess->AreColorsValid ())
+      tess->UpdateColors (c->LightMap, ambient);
 
     c->GetMaterial ()->Visit ();
 
-    iVertexBuffer *vbuf = c->GetVertexBuffer ();
-
-    mesh.mat_handle = c->GetMaterialHandle ();
-    mesh.buffers[0] = vbuf;
-    mesh.num_triangles = tess->GetTriangleCount ();
-    mesh.triangles = tess->GetTriangles ();
-    mesh.clip_portal = clip_portal;
-    mesh.clip_plane = clip_plane;
-    mesh.clip_z_plane = clip_z_plane;
-    mesh.vertex_fog = fog_verts.GetArray ();
-
     bool gouraud = !!c->LightMap;
-    mesh.mixmode = CS_FX_COPY | (gouraud ? 0 : CS_FX_FLAT);
-    mesh.use_vertex_color = gouraud;
-    if (mesh.mat_handle == 0)
+    rm->mixmode = CS_FX_COPY | (gouraud ? 0 : CS_FX_FLAT);
+    
+    csRenderBufferHolder* holder = rm->buffers;
+    /* @@@ TODO: use an SV accessor for geometry delivery */
+    bool frameBuffersCreated;
+    PerFrameData& frameBuffers = frameBufferHolder.GetUnusedData (
+      frameBuffersCreated, currentFrame);
+    if (frameBuffersCreated || (frameBuffers.vertCount < tess->GetVertexCount ()))
     {
-      static_data->thing_type->Warn ("Warning! Curve without material!");
-      continue;
+      frameBuffers.vertices = csRenderBuffer::CreateRenderBuffer (
+	tess->GetVertexCount (), CS_BUF_STREAM, 
+	CS_BUFCOMP_FLOAT, 3);
+      frameBuffers.colors = csRenderBuffer::CreateRenderBuffer (
+	tess->GetVertexCount (), CS_BUF_STREAM, 
+	CS_BUFCOMP_FLOAT, 3);
+      frameBuffers.texcoords = csRenderBuffer::CreateRenderBuffer (
+	tess->GetVertexCount (), CS_BUF_STREAM, 
+	CS_BUFCOMP_FLOAT, 2);
+      frameBuffers.indices = csRenderBuffer::CreateIndexRenderBuffer (
+	tess->GetTriangleCount () * 3, CS_BUF_STREAM, 
+	CS_BUFCOMP_UNSIGNED_INT, 0, tess->GetVertexCount ()-1);
+      frameBuffers.vertCount = tess->GetVertexCount ();
     }
 
-    CS_ASSERT (!vbuf->IsLocked ());
-    vbufmgr->LockBuffer (
-        vbuf,
-        tess->GetVertices (),
-        tess->GetTxtCoords (),
-        tess->GetColors (),
-        tess->GetVertexCount (),
-        0, static_data->obj_bbox);
-    rview->CalculateFogMesh (obj_cam, mesh);
-    rview->GetGraphics3D ()->DrawTriangleMesh (mesh);
-    vbufmgr->UnlockBuffer (vbuf);
+    frameBuffers.vertices->CopyInto (tess->GetVertices(), tess->GetVertexCount ());
+    holder->SetRenderBuffer (CS_BUFFER_POSITION, frameBuffers.vertices);
+    frameBuffers.colors->CopyInto (tess->GetColors(), tess->GetVertexCount ());
+    holder->SetRenderBuffer (CS_BUFFER_COLOR, frameBuffers.colors);
+    frameBuffers.texcoords->CopyInto (tess->GetTxtCoords(), tess->GetVertexCount ());
+    holder->SetRenderBuffer (CS_BUFFER_TEXCOORD0, frameBuffers.texcoords);
+    frameBuffers.indices->CopyInto (tess->GetTriangles(), tess->GetTriangleCount ()*3);
+    holder->SetRenderBuffer (CS_BUFFER_INDEX, frameBuffers.indices);
+
+    rm->indexstart = 0;
+    rm->indexend = (uint)tess->GetTriangleCount() * 3;
+    rm->meshtype = CS_MESHTYPE_TRIANGLES;
+    rm->material = c->GetMaterial ();
   }
 
-  return true;                                  //@@@ RETURN correct vis info
-}
-
-bool csBezierMesh::DrawTest (iRenderView *rview, iMovable *movable,
-	uint32 frustum_mask)
-{
-  Prepare ();
-
-  // Only get the transformation if this thing can move.
-  cached_movable = movable;
-  WorUpdate ();
-
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
-  	clip_z_plane);
-  return true;
-}
-
-bool csBezierMesh::Draw (iRenderView *rview, iMovable *movable,
-	csZBufMode zMode)
-{
-  DrawCurves (rview, movable, zMode);
-  return true;                                  // @@@@ RETURN correct vis info
+  n = (int)meshes.Length();
+  return meshes.GetArray();
 }
 
 //----------------------------------------------------------------------
@@ -991,12 +1012,11 @@ bool csBezierMesh::ReadFromCache (iCacheManager* cache_mgr)
   const char* thing_name = 0;
   if (static_data->thing_type->do_verbose && logparent)
   {
-    csRef<iMeshWrapper> mw (SCF_QUERY_INTERFACE (logparent, iMeshWrapper));
-    if (mw) thing_name = mw->QueryObject ()->GetName ();
+    thing_name = logparent->QueryObject ()->GetName ();
   }
 
   bool rc = true;
-  csRef<iDataBuffer> db = cache_mgr->ReadCache ("bezier_lm", 0, ~0);
+  csRef<iDataBuffer> db = cache_mgr->ReadCache ("bezier_lm", 0, (uint32)~0);
   if (db)
   {
     csMemFile mf ((const char*)(db->GetData ()), db->GetSize ());
@@ -1009,7 +1029,7 @@ bool csBezierMesh::ReadFromCache (iCacheManager* cache_mgr)
         rc = false;
         if (static_data->thing_type->do_verbose)
 	{
-	  printf ("  Bezier '%s' Curve '%s': %s\n",
+	  csPrintf ("  Bezier '%s' Curve '%s': %s\n",
 	  	thing_name, curves.Get (i)->GetName (),
 		error);
 	  fflush (stdout);
@@ -1021,7 +1041,7 @@ bool csBezierMesh::ReadFromCache (iCacheManager* cache_mgr)
   {
     if (static_data->thing_type->do_verbose)
     {
-      printf ("  Bezier '%s': Couldn't find cached lightmap file for bezier!\n",
+      csPrintf ("  Bezier '%s': Couldn't find cached lightmap file for bezier!\n",
       	thing_name);
       fflush (stdout);
     }
@@ -1044,7 +1064,7 @@ bool csBezierMesh::WriteToCache (iCacheManager* cache_mgr)
   for (i = 0; i < GetCurveCount (); i++)
     if (!curves.Get (i)->WriteToCache (&mf)) goto stop;
   if (!cache_mgr->CacheData ((void*)(mf.GetData ()), mf.GetSize (),
-    	"bezier_lm", 0, ~0))
+    	"bezier_lm", 0, (uint32)~0))
     goto stop;
 
   rc = true;
@@ -1087,8 +1107,9 @@ void csBezierMesh::MergeTemplate (
   static_data->curves_scale = tpl->GetCurvesScale ();
 
   //@@@ TEMPORARY
-  csRef<iBezierState> ith (SCF_QUERY_INTERFACE (tpl, iBezierState));
-  ParentTemplate = (csBezierMesh *) (ith->GetPrivateObject ());
+  csRef<iBezierState> ith = SCF_QUERY_INTERFACE (tpl, iBezierState);
+  ParentTemplate = ((csBezierMesh::BezierState*)(iBezierState*)ith)
+  	->GetPrivateObject ();
 
   for (i = 0; i < tpl->GetCurveVertexCount (); i++)
   {
@@ -1147,15 +1168,15 @@ SCF_IMPLEMENT_IBASE_END
 SCF_IMPLEMENT_IBASE(csBezierMeshObjectType)
   SCF_IMPLEMENTS_INTERFACE(iMeshObjectType)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iComponent)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iConfig)
+  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iPluginConfig)
 SCF_IMPLEMENT_IBASE_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMeshObjectType::eiComponent)
   SCF_IMPLEMENTS_INTERFACE(iComponent)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMeshObjectType::eiConfig)
-  SCF_IMPLEMENTS_INTERFACE(iConfig)
+SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMeshObjectType::eiPluginConfig)
+  SCF_IMPLEMENTS_INTERFACE(iPluginConfig)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csBezierMeshObjectType)
@@ -1166,7 +1187,7 @@ csBezierMeshObjectType::csBezierMeshObjectType (
 {
   SCF_CONSTRUCT_IBASE (pParent);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiConfig);
+  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiPluginConfig);
   lightpatch_pool = 0;
   do_verbose = false;
 }
@@ -1176,7 +1197,7 @@ csBezierMeshObjectType::~csBezierMeshObjectType ()
   delete lightpatch_pool;
 
   SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiConfig);
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiPluginConfig);
   SCF_DESTRUCT_IBASE ();
 }
 
@@ -1190,12 +1211,10 @@ bool csBezierMeshObjectType::Initialize (iObjectRegistry *object_reg)
 
   lightpatch_pool = new csBezierLightPatchPool ();
 
-  csRef<iCommandLineParser> cmdline = CS_QUERY_REGISTRY (object_reg,
-  	iCommandLineParser);
-  if (cmdline)
-  {
-    do_verbose = cmdline->GetOption ("verbose") != 0;
-  }
+  csRef<iVerbosityManager> verbosemgr (
+    CS_QUERY_REGISTRY (object_reg, iVerbosityManager));
+  if (verbosemgr) 
+    do_verbose = verbosemgr->Enabled ("bezier");
 
   return true;
 }
@@ -1324,7 +1343,7 @@ const int NUM_OPTIONS =
     sizeof (config_options[0])
   );
 
-bool csBezierMeshObjectType::eiConfig::SetOption (int id, csVariant *value)
+bool csBezierMeshObjectType::eiPluginConfig::SetOption (int id, csVariant *value)
 {
   switch (id)
   {
@@ -1338,7 +1357,7 @@ bool csBezierMeshObjectType::eiConfig::SetOption (int id, csVariant *value)
   return true;
 }
 
-bool csBezierMeshObjectType::eiConfig::GetOption (int id, csVariant *value)
+bool csBezierMeshObjectType::eiPluginConfig::GetOption (int id, csVariant *value)
 {
   switch (id)
   {
@@ -1349,7 +1368,7 @@ bool csBezierMeshObjectType::eiConfig::GetOption (int id, csVariant *value)
   return true;
 }
 
-bool csBezierMeshObjectType::eiConfig::GetOptionDescription (
+bool csBezierMeshObjectType::eiPluginConfig::GetOptionDescription (
   int idx,
   csOptionDescription *option)
 {
