@@ -20,16 +20,17 @@
 #include "cssysdef.h"
 #include "csgeom/matrix3.h"
 #include "csgeom/transfrm.h"
-#include "emit.h"
+#include "csutil/floatrand.h"
 #include "ivideo/material.h"
 #include "iengine/material.h"
 #include "iengine/rview.h"
 #include "iengine/camera.h"
 #include "iengine/movable.h"
-#include "qsqrt.h"
-#include "qint.h"
+#include "csqsqrt.h"
+#include "csqint.h"
 #include "csgeom/math3d.h"
-#include "csutil/garray.h"
+#include "csutil/dirtyaccessarray.h"
+#include "emit.h"
 #include <math.h>
 #include <stdlib.h>
 
@@ -39,56 +40,54 @@ SCF_IMPLEMENT_IBASE (csEmitFixed)
   SCF_IMPLEMENTS_INTERFACE (iEmitFixed)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitSphere)
   SCF_IMPLEMENTS_INTERFACE (iEmitSphere)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitBox)
   SCF_IMPLEMENTS_INTERFACE (iEmitBox)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitCone)
   SCF_IMPLEMENTS_INTERFACE (iEmitCone)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitMix)
   SCF_IMPLEMENTS_INTERFACE (iEmitMix)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitCylinder)
   SCF_IMPLEMENTS_INTERFACE (iEmitCylinder)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitLine)
   SCF_IMPLEMENTS_INTERFACE (iEmitLine)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitSphereTangent)
   SCF_IMPLEMENTS_INTERFACE (iEmitSphereTangent)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
+
 SCF_IMPLEMENT_IBASE (csEmitCylinderTangent)
   SCF_IMPLEMENTS_INTERFACE (iEmitCylinderTangent)
   SCF_IMPLEMENTS_INTERFACE (iEmitGen3D)
 SCF_IMPLEMENT_IBASE_END
 
 /// helper particle sorting structure
-struct csEmitCompPart {
+struct csEmitCompPart
+{
   float z;
   iParticle* part;
 };
 
-/// growing drawing sort array
-CS_TYPEDEF_GROWING_ARRAY (csCompPartArray, struct csEmitCompPart);
-CS_IMPLEMENT_STATIC_VAR (GetStaticCompPartArray,csCompPartArray,())
-
-/// utility function
-static float GetRandomFloat(float min, float max)
-{
-  float w = max - min;
-  return min + (w*rand()/(RAND_MAX+1.0));
-}
-
 csEmitFixed::csEmitFixed(iBase *parent)
 {
   SCF_CONSTRUCT_IBASE(parent);
@@ -97,9 +96,10 @@ csEmitFixed::csEmitFixed(iBase *parent)
 
 csEmitFixed::~csEmitFixed()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitFixed::GetValue(csVector3& value, csVector3 & /*given*/)
+void csEmitFixed::GetValue(csVector3& value, csVector3& /*given*/)
 {
   value = val;
 }
@@ -118,19 +118,23 @@ csEmitBox::csEmitBox(iBase *parent)
 
 csEmitBox::~csEmitBox()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitBox::GetValue(csVector3& value, csVector3 & /*given*/)
+void csEmitBox::GetValue(csVector3& value, csVector3& /*given*/)
 {
-  value.x = GetRandomFloat(min.x, max.x);
-  value.y = GetRandomFloat(min.y, max.y);
-  value.z = GetRandomFloat(min.z, max.z);
+  value.x = min.x + mult.x * randgen.Get();
+  value.y = min.y + mult.y * randgen.Get();
+  value.z = min.z + mult.z * randgen.Get();
 }
 
 void csEmitBox::SetContent(const csVector3& min, const csVector3& max)
 {
   csEmitBox::min = min;
   csEmitBox::max = max;
+  mult.x = (max.x - min.x);
+  mult.y = (max.y - min.y);
+  mult.z = (max.z - min.z);
 }
 
 void csEmitBox::GetContent(csVector3& min, csVector3& max)
@@ -149,34 +153,36 @@ csEmitSphere::csEmitSphere(iBase *parent)
 
 csEmitSphere::~csEmitSphere()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitSphere::GetValue(csVector3& value, csVector3 & /*given*/)
+void csEmitSphere::GetValue(csVector3& value, csVector3& /*given*/)
 {
   //because the volume of a slice at a certain distance
   // is the dist*dist(*pi..) in size. Taking random min..max will
   // cause an uneven spread of points in the sphere.
-  float sqdist = GetRandomFloat( min*min*min, max*max*max );
+  float sqdist = rand_min + (rand_mult * randgen.Get());
   float dist = pow(sqdist, (float)(1./3.));
   value.Set(dist, 0, 0);
-  float rotz_open = TWO_PI * (rand() / (1.0+RAND_MAX));
+  float rotz_open = randgen.GetAngle();
   csZRotMatrix3 openrot(rotz_open);
   value = openrot * value;
-  float rot_around = TWO_PI * (rand() / (1.0+RAND_MAX));
+  float rot_around = randgen.GetAngle();
   csXRotMatrix3 xaround(rot_around);
   value = xaround * value;
   value += center;
-
+#if 0
   // slow but gives a good even spreading
-  //while(1)
-  //{
-    //value.Set (GetRandomFloat(-max,max), GetRandomFloat(-max,max),
-      //GetRandomFloat(-max,max));
-    //float dist = value.SquaredNorm();
-    //if( (min*min <= dist) && (dist <= max*max) )
-      //break;
-  //}
-  //value += center;
+  while(1)
+  {
+    value.Set (randgen.Get(-max,max), randgen.Get(-max,max),
+      randgen.Get(-max,max));
+    float dist = value.SquaredNorm();
+    if (min*min <= dist && dist <= max*max)
+      break;
+  }
+  value += center;
+#endif
 }
 
 void csEmitSphere::SetContent(const csVector3& center, float min, float max)
@@ -184,6 +190,8 @@ void csEmitSphere::SetContent(const csVector3& center, float min, float max)
   csEmitSphere::center = center;
   csEmitSphere::min = min;
   csEmitSphere::max = max;
+  rand_min = min * min * min;
+  rand_mult = max * max * max - rand_min;
 }
 
 void csEmitSphere::GetContent(csVector3& center, float& min, float& max)
@@ -206,18 +214,19 @@ csEmitCone::csEmitCone(iBase *parent)
 
 csEmitCone::~csEmitCone()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitCone::GetValue(csVector3& value, csVector3 & /*given*/)
+void csEmitCone::GetValue(csVector3& value, csVector3& /*given*/)
 {
-  csVector3 dest(GetRandomFloat(min, max), 0, 0);
+  csVector3 dest(randgen.Get(min, max), 0, 0);
   /// from fountain code
 
   // now make it shoot to a circle in the x direction
-  float rotz_open = 2.0 * aperture * (rand() / (1.0+RAND_MAX)) - aperture;
+  float rotz_open = randgen.Get(2.0 * aperture) - aperture;
   csZRotMatrix3 openrot(rotz_open);
   dest = openrot * dest;
-  float rot_around = TWO_PI * (rand() / (1.0+RAND_MAX));
+  float rot_around = randgen.GetAngle();
   csXRotMatrix3 xaround(rot_around);
   dest = xaround * dest;
   // now dest point to somewhere in a circular cur of a sphere around the
@@ -262,7 +271,7 @@ void csEmitCone::GetContent(csVector3& origin, float& elevation,
 csEmitMix::csEmitMix(iBase *parent)
 {
   SCF_CONSTRUCT_IBASE(parent);
-  list = NULL;
+  list = 0;
   totalweight = 0.0;
   nr = 0;
 }
@@ -273,15 +282,15 @@ csEmitMix::~csEmitMix()
   while(p)
   {
     np = p->next;
-    if(p->emit) p->emit->DecRef();
     delete p;
     p = np;
   }
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitMix::GetValue(csVector3& value, csVector3 & given)
+void csEmitMix::GetValue(csVector3& value, csVector3& given)
 {
-  float num = GetRandomFloat(0., totalweight);
+  float num = randgen.Get(totalweight);
   float passed = 0.0;
   struct part *p = list;
   struct part *found = list;
@@ -299,18 +308,63 @@ void csEmitMix::GetValue(csVector3& value, csVector3 & given)
   else found->emit->GetValue(value, given);
 }
 
-void csEmitMix::AddEmitter(float weight, iEmitGen3D* emit)
+void csEmitMix::AddEmitter (float weight, iEmitGen3D* emit)
 {
   struct part *np = new struct csEmitMix::part;
   np->next = list;
   np->emit = emit;
   list = np;
-  if(emit) emit->IncRef();
   np->weight = weight;
   nr++;
   totalweight += weight;
 }
 
+void csEmitMix::RemoveEmitter(int num)
+{
+  int i;
+  struct part *p = list,*pp=0;
+  if(num >= nr) return;
+
+  // Find the entry to remove and the previous entry in the linked list
+  for (i=0;i<num;i++)
+  {
+    pp=p;
+    p = p->next;
+  }
+
+  // If this is the first entry, set the second entry as the head of the list
+  if (!pp)
+    list=p->next;
+  else
+    pp->next=p->next; // Otherwise remove this entry from the list by
+                      // adjusting linkage
+
+  // Drop our reference
+  p->emit=0;
+  // Remove the weight from the mix total
+  totalweight-=p->weight;
+  // Decrease the count of total mix elements by 1
+  nr--;
+}
+
+void csEmitMix::AdjustEmitterWeight(int num,float weight)
+{
+  int i;
+  struct part *p = list;
+  if(num >= nr) return;
+  
+  // Find the emitter for which to adjust weight
+  for (i=0;i<num;i++)
+    p = p->next;
+
+  // Reduce old weight from total and add new weight
+  totalweight-=p->weight;
+  totalweight+=weight;
+
+  // Set new emitter weight
+  p->weight=weight;
+}
+
 void csEmitMix::GetContent(int num, float& weight, iEmitGen3D*& emit)
 {
   struct part *p = list;
@@ -334,11 +388,12 @@ csEmitLine::csEmitLine(iBase *parent)
 
 csEmitLine::~csEmitLine()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitLine::GetValue(csVector3& value, csVector3 & /*given*/)
+void csEmitLine::GetValue(csVector3& value, csVector3& /*given*/)
 {
-  float v = GetRandomFloat(0., 1.);
+  float v = randgen.Get(1.0);
   value = start + (end-start)*v;
 }
 
@@ -381,25 +436,26 @@ static void FindAxis(const csVector3& n, csVector3& res)
 
 csEmitCylinder::~csEmitCylinder()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitCylinder::GetValue(csVector3& value, csVector3 & /*given*/)
+void csEmitCylinder::GetValue(csVector3& value, csVector3& /*given*/)
 {
   // point on the center line of the cylinder
-  float v = GetRandomFloat(0., 1.);
-  value = start + (end-start)*v;
+  float v = randgen.Get(1.0);
+  value = start + (end-start) * v;
 
   // setup 3 axis for the cylinder
   csVector3 normal = (end-start).Unit();
   csVector3 udir; FindAxis(normal, udir);
   csVector3 vdir = udir % normal;
-  float angle = GetRandomFloat(0, TWO_PI);
+  float angle = randgen.GetAngle();
   // direction on the circle
   csVector3 oncirc = udir*cos(angle) + vdir*sin(angle);
 
   // distance from cylinder line
-  float amount = GetRandomFloat(min*min, max*max);
-  amount = qsqrt(amount); // cause even spread of points in the circle
+  float amount = randgen.Get(min*min, max*max);
+  amount = csQsqrt(amount); // cause even spread of points in the circle
   value += oncirc * amount;
 }
 
@@ -432,9 +488,10 @@ csEmitCylinderTangent::csEmitCylinderTangent(iBase *parent)
 
 csEmitCylinderTangent::~csEmitCylinderTangent()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitCylinderTangent::GetValue(csVector3& value, csVector3 &given)
+void csEmitCylinderTangent::GetValue(csVector3& value, csVector3& given)
 {
   // cylinder direction
   csVector3 cyldir = (end - start).Unit();
@@ -445,13 +502,13 @@ void csEmitCylinderTangent::GetValue(csVector3& value, csVector3 &given)
   pl.Set(cyldir, pldist); // cylinder plane through given point.
   csVector3 isect;
   float sectdist;
-  if(!csIntersect3::Plane(start, end, pl, isect, sectdist))
+  if(!csIntersect3::SegmentPlane(start, end, pl, isect, sectdist))
     isect.Set(end);
   csVector3 normal = (given - isect).Unit();
 
   // need a direction tangential to normal and cyldirection
   csVector3 direction = normal % cyldir;
-  float amount = GetRandomFloat(min, max);
+  float amount = randgen.Get(min, max);
   value = direction * amount;
 }
 
@@ -483,22 +540,23 @@ csEmitSphereTangent::csEmitSphereTangent(iBase *parent)
 
 csEmitSphereTangent::~csEmitSphereTangent()
 {
+  SCF_DESTRUCT_IBASE();
 }
 
-void csEmitSphereTangent::GetValue(csVector3& value, csVector3 & given)
+void csEmitSphereTangent::GetValue(csVector3& value, csVector3& given)
 {
   csVector3 path = given - center;
   // setup axis
   csVector3 normal = path.Unit();
   csVector3 udir; FindAxis(normal, udir);
   csVector3 vdir = udir % normal;
-  float angle = GetRandomFloat(0, TWO_PI);
+  float angle = randgen.GetAngle();
   // direction on the circle
   csVector3 oncirc = udir*cos(angle) + vdir*sin(angle);
 
   // size of direction
-  float amount = GetRandomFloat(min*min, max*max);
-  amount = qsqrt(amount); // cause even spread of points in the circle
+  float amount = randgen.Get(min*min, max*max);
+  amount = csQsqrt(amount); // cause even spread of points in the circle
   value = oncirc * amount;
 }
 
@@ -517,6 +575,7 @@ void csEmitSphereTangent::GetContent(csVector3& center, float& min, float& max)
   max = csEmitSphereTangent::max;
 }
 
+
 //----------------- csEmitMeshObject -----------------------------------
 
 SCF_IMPLEMENT_IBASE_EXT (csEmitMeshObject)
@@ -532,8 +591,9 @@ void csEmitMeshObject::SetupObject ()
 {
   if (!initialized)
   {
-    RemoveParticles ();
+    csParticleSystem::SetupObject ();
     initialized = true;
+    RemoveParticles ();
     delete[] ages;
     delete[] part_pos;
     delete[] part_speed;
@@ -547,9 +607,10 @@ void csEmitMeshObject::SetupObject ()
     part_attract = new csVector3[number];
     bbox.StartBoundingBox();
 
+
     /// create new particles and add to particle system
-	int i;
-    for (i=0 ; i < number ; i++)
+    int i;
+    for (i=0 ; i < (int)number ; i++)
     {
       if(using_rect_sprites)
         AppendRectSprite (drop_width, drop_height, mat, lighted_particles);
@@ -557,65 +618,15 @@ void csEmitMeshObject::SetupObject ()
         lighted_particles);
       StartParticle(i);
       /// age each particle randomly, to spread out particles over ages.
-      int elapsed = QInt(GetRandomFloat(0,timetolive));
+      int elapsed = csQint(randgen.Get(timetolive));
       MoveAgeParticle(i, elapsed, elapsed/1000.);
     }
     SetupColor ();
     SetupMixMode ();
+    ShapeChanged ();
   }
 }
 
-
-/// particle backtofront sorting
-static int compareparticle(const void* p1, const void* p2)
-{
-  struct csEmitCompPart *cp1 = (struct csEmitCompPart*)p1;
-  struct csEmitCompPart *cp2 = (struct csEmitCompPart*)p2;
-  /// pos.z is smaller when close to camera.
-  /// bigger z's should be drawn first.
-  /// thus the bigger z should be considered first.
-  /// so if cp1.z > cp2.z --- return -1 (draw p1 before p2)
-  /// if cp1.z == cp2.z --- return 0 (same)
-  /// if cp1.z < cp2.z --- return +1 (draw p1 after p2)
-  /// this is the sign of cp2.z - cp1.z
-  float val = cp2->z - cp1->z;
-  if(val<0) return -1;
-  if(val>0) return +1;
-  return 0;
-}
-
-bool csEmitMeshObject::Draw (iRenderView* rview, iMovable* movable,
-        csZBufMode mode)
-{
-  static csCompPartArray *cpa = GetStaticCompPartArray ();
-
-  // detect if back to front is needed.
-  if(MixMode & CS_FX_ADD)
-    return csParticleSystem::Draw(rview, movable, mode);
-
-  // draw back to front
-  if (vis_cb) if (!vis_cb->BeforeDrawing (this, rview)) return false;
-  csReversibleTransform trans = movable->GetFullTransform ();
-
-  // sort
-  int i;
-  csReversibleTransform tr_o2c = rview->GetCamera()->GetTransform()
-    * trans.GetInverse(); // object to camera space
-  cpa->SetLength(number);
-  for (i = 0 ; i < number ; i++)
-  {
-    (*cpa)[i].z = (tr_o2c * part_pos[i]).z;
-    (*cpa)[i].part = GetParticle(i);
-  }
-  qsort(cpa->GetArray(), number, sizeof( struct csEmitCompPart ),
-    compareparticle);
-  
-  for (i = 0 ; i < number ; i++)
-    (*cpa)[i].part->Draw (rview, trans, mode);
-
-  return true;
-}
-
 csEmitMeshObject::csEmitMeshObject (iObjectRegistry* object_reg,
   iMeshObjectFactory* factory)
 	: csParticleSystem (object_reg, factory)
@@ -623,24 +634,23 @@ csEmitMeshObject::csEmitMeshObject (iObjectRegistry* object_reg,
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiEmitState);
   lighted_particles = false;
   number = 50;
-  ages = NULL;
-  part_pos = NULL;
-  part_speed = NULL;
-  part_accel = NULL;
-  part_attract = NULL;
+  ages = 0;
+  part_pos = 0;
+  part_speed = 0;
+  part_accel = 0;
+  part_attract = 0;
   attractor_force = 1.0;
-  attractor = NULL;
-  startpos = NULL;
-  startspeed = NULL;
-  startaccel = NULL;
   timetolive = 1000;
-  aging = NULL;
+  aging = 0;
   nr_aging_els = 0;
   using_rect_sprites = true;
-  drop_width = 0.2;
-  drop_height = 0.2;
+  drop_width = 0.2f;
+  drop_height = 0.2f;
   drop_sides = 6;
-  drop_radius = 0.1;
+  drop_radius = 0.1f;
+  has_container_box = false;
+  container_min.Set(0,0,0);
+  container_max.Set(0,0,0);
 }
 
 csEmitMeshObject::~csEmitMeshObject()
@@ -650,10 +660,6 @@ csEmitMeshObject::~csEmitMeshObject()
   delete[] part_speed;
   delete[] part_accel;
   delete[] part_attract;
-  if(startpos) startpos->DecRef();
-  if(startspeed) startspeed->DecRef();
-  if(startaccel) startaccel->DecRef();
-  if(attractor) attractor->DecRef();
   csEmitAge *p = aging, *np = 0;
   while(p)
   {
@@ -661,6 +667,7 @@ csEmitMeshObject::~csEmitMeshObject()
     delete p;
     p = np;
   }
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiEmitState);
 }
 
 void csEmitMeshObject::StartParticle (int i)
@@ -756,6 +763,8 @@ void csEmitMeshObject::MoveAgeParticle (int i, int elapsed, float delta_t)
     col = between*belowage->color + invbet*aboveage->color;
   }
   // adjust the particle
+  // Jorrit: @@@ This fix is needed or we sometimes get a division by zero!
+  if (ABS (oldscale) < .0001) oldscale = 1;
   GetParticle(i)->ScaleBy(newscale / oldscale);
   GetParticle(i)->Rotate(rotspeed * delta_t);
   if(MixMode & CS_FX_ADD)
@@ -772,11 +781,13 @@ void csEmitMeshObject::MoveAgeParticle (int i, int elapsed, float delta_t)
   }
 
   /// move the particle
+  if(fieldaccel) fieldaccel->GetValue(part_accel[i], part_pos[i]);
+  if(fieldspeed) fieldspeed->GetValue(part_speed[i], part_pos[i]);
   if(attractor)
   {
     // do attractor influence
     csVector3 d = part_attract[i] - part_pos[i];
-    part_accel[i] += d * (attractor_force * delta_t);
+	part_speed[i] += d * (attractor_force * delta_t);
   }
   csVector3 swirl = GetRandomDirection() * swirlamount;
   part_speed[i] += swirl * delta_t;
@@ -796,7 +807,7 @@ void csEmitMeshObject::Update (csTicks elapsed_time)
   int elapsed = elapsed_time;
   // age particles;
   int i;
-  for (i=0 ; i < particles.Length () ; i++)
+  for (i=0 ; i < (int)particles.Length () ; i++)
   {
     if(ages[i] + elapsed > timetolive)
     {
@@ -818,6 +829,9 @@ void csEmitMeshObject::Update (csTicks elapsed_time)
           float(aboveage->time - belowage->time);
         oldscale = between*belowage->scale + (1.-between)*aboveage->scale;
       }
+      // Jorrit: @@@ This fix is needed or we sometimes get a division by zero!
+      if (ABS (oldscale) < .0001) oldscale = 1;
+
       GetParticle(i)->ScaleBy(1./oldscale); // reset the scale
       /// restart the particle
       int afterstart = (ages[i] + elapsed_time) % timetolive;
@@ -833,7 +847,7 @@ void csEmitMeshObject::Update (csTicks elapsed_time)
   }
 }
 
-void csEmitMeshObject::HardTransform (const csReversibleTransform& /*t*/)
+void csEmitMeshObject::HardTransform (const csReversibleTransform&)
 {
 }
 
@@ -860,6 +874,46 @@ void csEmitMeshObject::AddAge(int time, const csColor& color, float alpha,
   np->scale = scale;
 }
 
+void csEmitMeshObject::RemoveAge(int time, const csColor& color, float alpha,
+        float swirl, float rotspeed, float scale)
+{
+  bool found=false;
+  csEmitAge *p = aging, *prevp = 0;
+  // Find the aging moment to remove and the aging moment entry prior to it
+  while(p && (p->time <= time))
+  {
+    // Compare all elements.  We don't stop multiple entries with the same time
+    // from being added, so we have to check all parts for match.
+    if (p->time == time &&
+        p->alpha == alpha &&
+	p->swirl == swirl &&
+	p->rotspeed == rotspeed &&
+	p->scale == scale &&
+        p->color.blue == color.blue &&
+	p->color.red == color.red &&
+	p->color.green == color.green)
+    {
+      found=true;
+      break;
+    }
+    prevp = p;
+    p=p->next;
+  }
+  if (!found)
+    return;
+
+  // Unlink
+  if (!prevp)
+    aging=p->next;
+  else
+    prevp->next=p->next;
+
+  // Decrease total aging moment count
+  nr_aging_els--;
+
+  // Delete aging moment structure
+  delete p;
+}
 
 void csEmitMeshObject::GetAgingMoment(int i, int& time, csColor& color,
   float &alpha, float& swirl, float& rotspeed, float& scale)
@@ -887,7 +941,7 @@ void csEmitMeshObject::ReplaceAge(int time, const csColor& color, float alpha,
   csEmitAge *p = aging;
   while(p && (p->time != time))
   {
-    if(fabs(p->time-time) < SMALL_EPSILON)
+    if(fabs((float)p->time-time) < SMALL_EPSILON)
       break;  /// almost the same, assume the diff is due to float inaccuracy
     p=p->next;
   }
@@ -911,25 +965,29 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csEmitMeshObjectFactory::EmitFactoryState)
   SCF_IMPLEMENTS_INTERFACE (iEmitFactoryState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-csEmitMeshObjectFactory::csEmitMeshObjectFactory (iBase *p, iObjectRegistry* s)
+csEmitMeshObjectFactory::csEmitMeshObjectFactory (iMeshObjectType *p,
+  iObjectRegistry* s)
 {
   SCF_CONSTRUCT_IBASE (p);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiEmitFactoryState);
   object_reg = s;
-  logparent = NULL;
+  logparent = 0;
+  emit_type = p;
 }
 
 csEmitMeshObjectFactory::~csEmitMeshObjectFactory ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiEmitFactoryState);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObject* csEmitMeshObjectFactory::NewInstance ()
+csPtr<iMeshObject> csEmitMeshObjectFactory::NewInstance ()
 {
   csEmitMeshObject* cm =
     new csEmitMeshObject (object_reg, (iMeshObjectFactory*)this);
-  iMeshObject* im = SCF_QUERY_INTERFACE (cm, iMeshObject);
-  im->DecRef ();
-  return im;
+  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
+  cm->DecRef ();
+  return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
@@ -945,10 +1003,6 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csEmitMeshObjectType)
 
-SCF_EXPORT_CLASS_TABLE (emit)
-  SCF_EXPORT_CLASS (csEmitMeshObjectType, "crystalspace.mesh.object.emit",
-    "Crystal Space Emit Mesh Type")
-SCF_EXPORT_CLASS_TABLE_END
 
 csEmitMeshObjectType::csEmitMeshObjectType (iBase* pParent)
 {
@@ -958,12 +1012,15 @@ csEmitMeshObjectType::csEmitMeshObjectType (iBase* pParent)
 
 csEmitMeshObjectType::~csEmitMeshObjectType ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObjectFactory* csEmitMeshObjectType::NewFactory ()
+csPtr<iMeshObjectFactory> csEmitMeshObjectType::NewFactory ()
 {
   csEmitMeshObjectFactory* cm = new csEmitMeshObjectFactory (this, object_reg);
-  iMeshObjectFactory* ifact = SCF_QUERY_INTERFACE (cm, iMeshObjectFactory);
-  ifact->DecRef ();
-  return ifact;
+  csRef<iMeshObjectFactory> ifact (
+    SCF_QUERY_INTERFACE (cm, iMeshObjectFactory));
+  cm->DecRef ();
+  return csPtr<iMeshObjectFactory> (ifact);
 }
