@@ -25,7 +25,7 @@
 #include "ivideo/material.h"
 #include "iengine/material.h"
 #include "iengine/engine.h"
-#include "qsqrt.h"
+#include "csqsqrt.h"
 #include <math.h>
 #include <stdlib.h>
 
@@ -46,7 +46,7 @@ void csExploMeshObject::SetupObject ()
     csNewtonianParticleSystem::SetupObject ();
     initialized = true;
     RemoveParticles ();
-    int i;
+    size_t i;
     csVector3 pos;
     /// add particles
     bbox.StartBoundingBox (center);
@@ -78,11 +78,11 @@ void csExploMeshObject::SetupObject ()
     }
     startbox = bbox;
     radiusnow = 1.0;
-    maxspeed = qsqrt (sqmaxspeed);
-    maxaccel = qsqrt (sqmaxaccel);
+    maxspeed = csQsqrt (sqmaxspeed);
+    maxaccel = csQsqrt (sqmaxaccel);
     SetupColor ();
     SetupMixMode ();
-    float r = qsqrt (csSquaredDist::PointPoint (bbox.Max (), bbox.Min ())) / 2;
+    float r = csQsqrt (csSquaredDist::PointPoint (bbox.Max (), bbox.Min ())) / 2;
     radius.Set (r, r, r);
   }
 }
@@ -102,7 +102,7 @@ csExploMeshObject::csExploMeshObject (iObjectRegistry* object_reg,
   spread_pos = 0.6f;
   spread_speed = 2.0f;
   spread_accel = 2.0f;
-  SetParticleCount (number);
+  SetParticleCount ((int)number);
 }
 
 csExploMeshObject::~csExploMeshObject()
@@ -129,7 +129,7 @@ void csExploMeshObject::HardTransform (const csReversibleTransform& t)
 {
   center = t.This2Other (center);
   initialized = false;
-  scfiObjectModel.ShapeChanged ();
+  ShapeChanged ();
 }
 
 //----------------------------------------------------------------------
@@ -138,11 +138,12 @@ SCF_IMPLEMENT_IBASE (csExploMeshObjectFactory)
   SCF_IMPLEMENTS_INTERFACE (iMeshObjectFactory)
 SCF_IMPLEMENT_IBASE_END
 
-csExploMeshObjectFactory::csExploMeshObjectFactory (iBase *p,
+csExploMeshObjectFactory::csExploMeshObjectFactory (iMeshObjectType *p,
 	iObjectRegistry* s)
 {
   SCF_CONSTRUCT_IBASE (p);
   logparent = 0;
+  explo_type = p;
   object_reg = s;
 }
 
