@@ -23,7 +23,7 @@
 #include "csgeom/box.h"
 #include "csutil/cscolor.h"
 #include "csutil/refarr.h"
-#include "plugins/mesh/partgen/partgen.h"
+#include "csplugincommon/particlesys/partgen.h"
 #include "imesh/explode.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
@@ -77,16 +77,16 @@ public:
   virtual void SetParticleCount (int num)
   {
     csParticleSystem::SetParticleCount (num);
-    SetCount (number);
+    SetCount ((int)number);
   }
   /// Get the number of particles
-  int GetParticleCount () const {return number;}
+  int GetParticleCount () const {return (int)number;}
   /// Set the explosion center.
   void SetCenter (const csVector3& center)
   {
     initialized = false;
     csExploMeshObject::center = center;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the explosion center
   const csVector3 &GetCenter () const {return center;}
@@ -95,7 +95,7 @@ public:
   {
     initialized = false;
     csExploMeshObject::push = push;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the push vector.
   const csVector3& GetPush () const {return push;}
@@ -104,7 +104,7 @@ public:
   {
     initialized = false;
     csExploMeshObject::nr_sides = nr_sides;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the number of sides.
   int GetNrSides () const {return nr_sides;}
@@ -113,7 +113,7 @@ public:
   {
     initialized = false;
     csExploMeshObject::part_radius = part_radius;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the radius of all particles.
   float GetPartRadius () const {return part_radius;}
@@ -130,7 +130,7 @@ public:
   {
     initialized = false;
     csExploMeshObject::spread_pos = spread_pos;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the spread position.
   float GetSpreadPos () const {return spread_pos;}
@@ -139,7 +139,7 @@ public:
   {
     initialized = false;
     csExploMeshObject::spread_speed = spread_speed;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the spread speed.
   float GetSpreadSpeed () const {return spread_speed;}
@@ -148,7 +148,7 @@ public:
   {
     initialized = false;
     csExploMeshObject::spread_accel = spread_accel;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the spread acceleration.
   float GetSpreadAcceleration () const {return spread_accel;}
@@ -269,12 +269,14 @@ class csExploMeshObjectFactory : public iMeshObjectFactory
 {
 private:
   iObjectRegistry* object_reg;
-  iBase* logparent;
+  iMeshFactoryWrapper* logparent;
+  iMeshObjectType* explo_type;
   csFlags flags;
 
 public:
   /// Constructor.
-  csExploMeshObjectFactory (iBase *pParent, iObjectRegistry* object_reg);
+  csExploMeshObjectFactory (iMeshObjectType *pParent,
+  	iObjectRegistry* object_reg);
 
   /// Destructor.
   virtual ~csExploMeshObjectFactory ();
@@ -287,8 +289,11 @@ public:
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  { logparent = lp; }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  { return logparent; }
+  virtual iMeshObjectType* GetMeshObjectType () const { return explo_type; }
   virtual iObjectModel* GetObjectModel () { return 0; }
 };
 
