@@ -23,7 +23,7 @@
 #include "csgeom/box.h"
 #include "csutil/cscolor.h"
 #include "csutil/refarr.h"
-#include "plugins/mesh/partgen/partgen.h"
+#include "csplugincommon/particlesys/partgen.h"
 #include "imesh/fire.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
@@ -110,7 +110,7 @@ public:
     initialized = false;
     drop_width = dropwidth;
     drop_height = dropheight;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the size of the fire drops.
   void GetDropSize (float& dropwidth, float& dropheight) const
@@ -120,7 +120,7 @@ public:
   {
     initialized = false;
     csFireMeshObject::origin = origin;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get origin of the fire.
   const csBox3& GetOrigin () const
@@ -130,7 +130,7 @@ public:
   {
     initialized = false;
     csFireMeshObject::direction = direction;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get direction of the fire.
   const csVector3& GetDirection () const
@@ -149,7 +149,7 @@ public:
   {
     initialized = false;
     csFireMeshObject::swirl = swirl;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get swirl.
   float GetSwirl () const
@@ -194,7 +194,7 @@ public:
     }
     virtual int GetParticleCount () const
     {
-      return scfParent->GetParticleCount ();
+      return (int)scfParent->GetParticleCount ();
     }
     virtual void SetDropSize (float dropwidth, float dropheight)
     {
@@ -263,12 +263,14 @@ class csFireMeshObjectFactory : public iMeshObjectFactory
 {
 private:
   iObjectRegistry* object_reg;
-  iBase* logparent;
+  iMeshFactoryWrapper* logparent;
+  iMeshObjectType* fire_type;
   csFlags flags;
 
 public:
   /// Constructor.
-  csFireMeshObjectFactory (iBase *pParent, iObjectRegistry* object_reg);
+  csFireMeshObjectFactory (iMeshObjectType *pParent,
+  	iObjectRegistry* object_reg);
 
   /// Destructor.
   virtual ~csFireMeshObjectFactory ();
@@ -281,8 +283,11 @@ public:
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  { logparent = lp; }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  { return logparent; }
+  virtual iMeshObjectType* GetMeshObjectType () const { return fire_type; }
   virtual iObjectModel* GetObjectModel () { return 0; }
 };
 
