@@ -16,22 +16,31 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define CS_SYSDEF_PROVIDE_ALLOCA
 #include "cssysdef.h"
+#include "csqsqrt.h"
+
 #include "csgeom/math3d.h"
 #include "csgeom/poly2d.h"
-#include "haze.h"
-#include "iengine/movable.h"
-#include "iengine/rview.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/graph2d.h"
-#include "ivideo/material.h"
-#include "iengine/material.h"
+#include "csgfx/renderbuffer.h"
+#include "csgfx/shadervarcontext.h"
+#include "cstool/rbuflock.h"
+#include "csutil/dirtyaccessarray.h"
+#include "csutil/sysfunc.h"
 #include "iengine/camera.h"
-#include "igeom/clip2d.h"
 #include "iengine/engine.h"
 #include "iengine/light.h"
-#include "qsqrt.h"
+#include "iengine/material.h"
+#include "iengine/movable.h"
+#include "iengine/rview.h"
+#include "igeom/clip2d.h"
+#include "iutil/objreg.h"
+#include "ivideo/graph2d.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/material.h"
+#include "ivideo/rendermesh.h"
+
+#include "haze.h"
+
 
 CS_IMPLEMENT_PLUGIN
 
@@ -41,36 +50,37 @@ SCF_IMPLEMENT_IBASE_END
 
 csHazeHull::csHazeHull()
 {
-  SCF_CONSTRUCT_IBASE (NULL);
+  SCF_CONSTRUCT_IBASE (0);
   total_poly = 0;
   total_vert = 0;
   total_edge = 0;
-  verts = NULL;
-  edgept1 = NULL;
-  edgept2 = NULL;
-  pol_num = NULL;
-  pol_verts = NULL;
-  pol_edges = NULL;
+  verts = 0;
+  edgept1 = 0;
+  edgept2 = 0;
+  pol_num = 0;
+  pol_verts = 0;
+  pol_edges = 0;
 }
 
 csHazeHull::~csHazeHull()
 {
   /// delete and clear the object
-  delete[] verts; verts = NULL;
-  delete[] edgept1; edgept1 = NULL;
-  delete[] edgept2; edgept2 = NULL;
+  delete[] verts; verts = 0;
+  delete[] edgept1; edgept1 = 0;
+  delete[] edgept2; edgept2 = 0;
   int p;
   for(p=0; p<total_poly; p++)
   {
-    delete[] pol_verts[p]; pol_verts[p] = NULL;
-    delete[] pol_edges[p]; pol_edges[p] = NULL;
+    delete[] pol_verts[p]; pol_verts[p] = 0;
+    delete[] pol_edges[p]; pol_edges[p] = 0;
   }
-  delete[] pol_verts; pol_verts = NULL;
-  delete[] pol_edges; pol_edges = NULL;
-  delete[] pol_num; pol_num = NULL;
+  delete[] pol_verts; pol_verts = 0;
+  delete[] pol_edges; pol_edges = 0;
+  delete[] pol_num; pol_num = 0;
   total_poly = 0;
   total_vert = 0;
   total_edge = 0;
+  SCF_DESTRUCT_IBASE ();
 }
 
 
@@ -87,7 +97,7 @@ void csHazeHull::ComputeEdges()
   int p;
   for(p=0; p<total_poly; p++)
   {
-    //printf("poly %d: ", p);
+    //csPrintf("poly %d: ", p);
     for(i=0; i<pol_num[p]; i++)
     {
       /// get two indices
@@ -101,10 +111,10 @@ void csHazeHull::ComputeEdges()
 	idx2 = swp;
       }
       /// mark
-      //printf("(%d-%d) ", idx1, idx2);
+      //csPrintf("(%d-%d) ", idx1, idx2);
       matrix[ idx1*total_vert + idx2 ] = 1;
     }
-    //printf("\n");
+    //csPrintf("\n");
   }
   /// use matrix to fill fields.
 
@@ -119,8 +129,8 @@ void csHazeHull::ComputeEdges()
         total_edge++;
     }
   /// fill edgept tables.
-  if(edgept1) delete[] edgept1;
-  if(edgept2) delete[] edgept2;
+  delete[] edgept1;
+  delete[] edgept2;
   edgept1 = new int[total_edge];
   edgept2 = new int[total_edge];
   int edge_now = 0;
@@ -191,12 +201,12 @@ void csHazeHull::ComputeOutline(iHazeHull *hull, const csVector3& campos,
     hull->GetVertex(v1, hull->GetPolVertex(p, 1));
     hull->GetVertex(v2, hull->GetPolVertex(p, 2));
     if(csMath3::WhichSide3D(campos-v0, v1-v0, v2-v0) > 0) {
-      //printf("polygon %d visible\n", p);
+      //csPrintf("polygon %d visible\n", p);
       for(i=0; i<hull->GetPolVerticeCount(p); i++)
       {
         int edge, i1, i2;
 	edge = hull->GetPolEdge(p, i, i1, i2);
-	//printf("increasing edge %d (from %d - %d)\n", edge, i1, i2);
+	//csPrintf("increasing edge %d (from %d - %d)\n", edge, i1, i2);
 	use_edge[edge] ++;
 	use_start[edge] = i1;
 	use_end[edge] = i2;
@@ -238,7 +248,7 @@ void csHazeHull::ComputeOutline(iHazeHull *hull, const csVector3& campos,
     pt = nextvert[pt];
     if(pt == -1)
     {
-      printf("Error: pt==-1 in Outline.\n");
+      csPrintf("Error: pt==-1 in Outline.\n");
       delete[] use_edge;
       delete[] use_start;
       delete[] use_end;
@@ -256,10 +266,9 @@ void csHazeHull::ComputeOutline(iHazeHull *hull, const csVector3& campos,
 
 //------------ csHazeHullBox -----------------------------------
 
-SCF_IMPLEMENT_IBASE (csHazeHullBox)
-  SCF_IMPLEMENTS_INTERFACE (iHazeHull)
+SCF_IMPLEMENT_IBASE_EXT (csHazeHullBox)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeHullBox)
-SCF_IMPLEMENT_IBASE_END
+SCF_IMPLEMENT_IBASE_EXT_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeHullBox::HazeHullBox)
   SCF_IMPLEMENTS_INTERFACE (iHazeHullBox)
@@ -268,7 +277,6 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 csHazeHullBox::csHazeHullBox(const csVector3& a, const csVector3& b)
   : csHazeHull()
 {
-  SCF_CONSTRUCT_IBASE (NULL);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeHullBox);
   min = a;
   max = b;
@@ -312,14 +320,14 @@ csHazeHullBox::csHazeHullBox(const csVector3& a, const csVector3& b)
 
 csHazeHullBox::~csHazeHullBox()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeHullBox);
 }
 
 //------------ csHazeHullCone -----------------------------------
 
-SCF_IMPLEMENT_IBASE (csHazeHullCone)
-  SCF_IMPLEMENTS_INTERFACE (iHazeHull)
+SCF_IMPLEMENT_IBASE_EXT (csHazeHullCone)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeHullCone)
-SCF_IMPLEMENT_IBASE_END
+SCF_IMPLEMENT_IBASE_EXT_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeHullCone::HazeHullCone)
   SCF_IMPLEMENTS_INTERFACE (iHazeHullCone)
@@ -343,7 +351,6 @@ csHazeHullCone::csHazeHullCone(int nr_sides, const csVector3& start,
     const csVector3& end, float srad, float erad)
   : csHazeHull()
 {
-  SCF_CONSTRUCT_IBASE (NULL);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeHullCone);
   csHazeHullCone::nr_sides = nr_sides;
   csHazeHullCone::start = start;
@@ -410,31 +417,41 @@ csHazeHullCone::csHazeHullCone(int nr_sides, const csVector3& start,
 
 csHazeHullCone::~csHazeHullCone()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeHullCone);
 }
 
 //------------ csHazeMeshObject -------------------------------
 
 SCF_IMPLEMENT_IBASE (csHazeMeshObject)
   SCF_IMPLEMENTS_INTERFACE (iMeshObject)
+  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeState)
 SCF_IMPLEMENT_IBASE_END
 
+SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObject::ObjectModel)
+  SCF_IMPLEMENTS_INTERFACE (iObjectModel)
+SCF_IMPLEMENT_EMBEDDED_IBASE_END
+
 SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObject::HazeState)
   SCF_IMPLEMENTS_INTERFACE (iHazeState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
+csStringID csHazeMeshObject::vertex_name = csInvalidStringID;
+csStringID csHazeMeshObject::texel_name = csInvalidStringID;
+csStringID csHazeMeshObject::index_name = csInvalidStringID;
+
 csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory)
 {
-  SCF_CONSTRUCT_IBASE (NULL);
+  SCF_CONSTRUCT_IBASE (0);
+  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeState);
   csHazeMeshObject::factory = factory;
-  logparent = NULL;
+  logparent = 0;
   ifactory = SCF_QUERY_INTERFACE (factory, iMeshObjectFactory);
   material = factory->GetMaterialWrapper ();
   MixMode = factory->GetMixMode ();
   initialized = false;
-  vis_cb = NULL;
-  shapenr = 0;
+  vis_cb = 0;
   current_lod = 1;
   current_features = 0;
   origin.Set(0,0,0);
@@ -444,20 +461,36 @@ csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory)
   /// copy the factory settings
   origin = factory->GetOrigin();
   directional = factory->GetDirectional();
-  csHazeLayerVector *factlayers = factory->GetLayers();
-  int i;
-  for(i=0; i<factlayers->Length(); i++)
+  csPDelArray<csHazeLayer> *factlayers = factory->GetLayers();
+  size_t i;
+  for(i = 0; i < factlayers->Length(); i++)
   {
-    csHazeLayer *p = new csHazeLayer (factlayers->GetLayer(i)->hull,
-      factlayers->GetLayer(i)->scale);
+    csHazeLayer *p = new csHazeLayer (factlayers->Get(i)->hull,
+      factlayers->Get(i)->scale);
     layers.Push(p);
   }
+
+  csRef<iStringSet> strings;
+  strings = CS_QUERY_REGISTRY_TAG_INTERFACE (factory->object_reg,
+    "crystalspace.shared.stringset", iStringSet);
+
+  if ((vertex_name == csInvalidStringID) ||
+    (texel_name == csInvalidStringID) ||
+    (index_name == csInvalidStringID))
+  {
+    vertex_name = strings->Request ("vertices");
+    texel_name = strings->Request ("texture coordinates");
+    index_name = strings->Request ("indices");
+  }
 }
 
 csHazeMeshObject::~csHazeMeshObject ()
 {
   if (vis_cb) vis_cb->DecRef ();
-  if (ifactory) ifactory->DecRef ();
+
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeState);
+  SCF_DESTRUCT_IBASE ();
 }
 
 void csHazeMeshObject::SetupObject ()
@@ -466,11 +499,12 @@ void csHazeMeshObject::SetupObject ()
   {
     bbox.StartBoundingBox( origin );
     csVector3 pos;
-	int l, i;
+    size_t l;
+    int i;
     for(l=0; l<layers.Length(); l++)
-      for(i=0; i<layers.GetLayer(l)->hull->GetVerticeCount(); i++)
+      for(i=0; i<layers[l]->hull->GetVerticeCount(); i++)
       {
-	layers.GetLayer(l)->hull->GetVertex(pos, i);
+	layers[l]->hull->GetVertex(pos, i);
         bbox.AddBoundingVertex( pos );
       }
     initialized = true;
@@ -548,192 +582,281 @@ float csHazeMeshObject::GetScreenBoundingBox (long cameranr,
 }
 
 
-bool csHazeMeshObject::DrawTest (iRenderView* rview, iMovable* movable)
+
+CS_IMPLEMENT_STATIC_VAR(GetTempVertices, csDirtyAccessArray<csVector3>, ())
+CS_IMPLEMENT_STATIC_VAR(GetTempTexels, csDirtyAccessArray<csVector2>, ())
+CS_IMPLEMENT_STATIC_VAR(GetTempIndices, csDirtyAccessArray<uint>, ())
+
+void csHazeMeshObject::GenGeometryAdapt (iRenderView *rview, iGraphics3D *g3d, 
+					 int num_sides, csVector3* scrpts, 
+					 csVector3* campts, csVector2* uvs,
+					 float layer_scale, float quality, 
+					 int depth, int maxdepth)
 {
-  SetupObject ();
-
-  if(layers.Length() <= 0) return false;
-
-  //iGraphics3D* g3d = rview->GetGraphics3D ();
-  iCamera* camera = rview->GetCamera ();
-
-  csReversibleTransform tr_o2c = camera->GetTransform ()
-        * movable->GetFullTransform ().GetInverse ();
-  float fov = camera->GetFOV ();
-  float shiftx = camera->GetShiftX ();
-  float shifty = camera->GetShiftY ();
-  csBox2 sbox;
-  csBox3 cbox;
-
-  if (GetScreenBoundingBox (camera->GetCameraNumber (),
-        movable->GetUpdateNumber (), fov, shiftx, shifty,
-        tr_o2c, sbox, cbox) < 0)
+  /// only triangles
+  (void)num_sides;
+  CS_ASSERT(num_sides == 3);
+  // check if the angle is OK
+  csVector2 dir1;
+  dir1.x = scrpts[1].x - scrpts[0].x;
+  dir1.y = scrpts[1].y - scrpts[0].y;
+  csVector2 dir2;
+  dir2.x = scrpts[2].x - scrpts[0].x;
+  dir2.y = scrpts[2].y - scrpts[0].y;
+  csVector2 normdir1 = dir1 / dir1.Norm();
+  csVector2 normdir2 = dir2 / dir2.Norm();
+  float cosangle = normdir1 * normdir2;
+  //csPrintf("cosangle %g, quality %g\n", cosangle, quality);
+  if(cosangle > quality || depth >= maxdepth)
   {
-    //printf("No draw\n");
-    return false;
-  }
-  int clip_portal, clip_plane, clip_z_plane;
-  if (rview->ClipBBox (sbox, cbox, clip_portal, clip_plane,
-        clip_z_plane) == false)
-  {
-    //printf("No draw(2)\n");
-    return false;
-  }
-
-  return true;
-}
-
-void csHazeMeshObject::UpdateLighting (iLight** /*lights*/, int /*num_lights*/,
-    const csVector3& /*pos*/)
-{
-  SetupObject ();
-}
-
-void csHazeMeshObject::UpdateLighting (iLight** lights, int num_lights,
-    iMovable* movable)
-{
-  csVector3 pos = movable->GetFullPosition ();
-  UpdateLighting (lights, num_lights, pos);
-}
-
-#define INTERPOLATE1(component) \
-  g3dpoly->vertices [i].component = inpoly [vt].component + \
-    t * (inpoly [vt2].component - inpoly [vt].component);
-
-#define INTERPOLATE(component) \
-{ \
-  float v1 = inpoly [edge_from [0]].component + \
-    t1 * (inpoly [edge_to [0]].component - inpoly [edge_from [0]].component); \
-  float v2 = inpoly [edge_from [1]].component + \
-    t2 * (inpoly [edge_to [1]].component - inpoly [edge_from [1]].component); \
-  g3dpoly->vertices [i].component = v1 + t * (v2 - v1); \
-}
-
-static void PreparePolygonFX2 (G3DPolygonDPFX* g3dpoly,
-  csVector2* clipped_verts, int num_vertices, csVertexStatus* clipped_vtstats,
-  int orig_num_vertices, bool gouraud)
-{
-  // first we copy the first texture coordinates to a local buffer
-  // to avoid that they are overwritten when interpolating.
-  ALLOC_STACK_ARRAY (inpoly, G3DTexturedVertex, orig_num_vertices);
-  int i;
-  for (i = 0; i < orig_num_vertices; i++)
-    inpoly[i] = g3dpoly->vertices[i];
-
-  int vt, vt2;
-  float t;
-  for (i = 0; i < num_vertices; i++)
-  {
-    g3dpoly->vertices [i].x = clipped_verts [i].x;
-    g3dpoly->vertices [i].y = clipped_verts [i].y;
-    switch (clipped_vtstats[i].Type)
+    // emit geometry
+    for (int i = 0; i < 3; i++)
     {
-      case CS_VERTEX_ORIGINAL:
-        vt = clipped_vtstats[i].Vertex;
-        g3dpoly->vertices [i].z = inpoly [vt].z;
-        g3dpoly->vertices [i].u = inpoly [vt].u;
-        g3dpoly->vertices [i].v = inpoly [vt].v;
-	if (gouraud)
-	{
-          g3dpoly->vertices [i].r = inpoly [vt].r;
-          g3dpoly->vertices [i].g = inpoly [vt].g;
-          g3dpoly->vertices [i].b = inpoly [vt].b;
-	}
-	break;
-      case CS_VERTEX_ONEDGE:
-        vt = clipped_vtstats[i].Vertex;
-	vt2 = vt + 1; if (vt2 >= orig_num_vertices) vt2 = 0;
-	t = clipped_vtstats [i].Pos;
-	INTERPOLATE1 (z);
-	INTERPOLATE1 (u);
-	INTERPOLATE1 (v);
-	if (gouraud)
-	{
-	  INTERPOLATE1 (r);
-	  INTERPOLATE1 (g);
-	  INTERPOLATE1 (b);
-	}
-	break;
-      case CS_VERTEX_INSIDE:
-        float x = clipped_verts [i].x;
-        float y = clipped_verts [i].y;
-        int edge_from [2], edge_to [2];
-	int edge = 0;
-	int j, j1;
-	j1 = orig_num_vertices - 1;
-	for (j = 0; j < orig_num_vertices; j++)
-	{
-          if ((y >= inpoly [j].y && y <= inpoly [j1].y) ||
-	      (y <= inpoly [j].y && y >= inpoly [j1].y))
-	  {
-	    edge_from [edge] = j;
-	    edge_to [edge] = j1;
-	    edge++;
-	    if (edge >= 2) break;
-	  }
-	  j1 = j;
-	}
-	if (edge == 1)
-	{
-	  // Safety if we only found one edge.
-	  edge_from [1] = edge_from [0];
-	  edge_to [1] = edge_to [0];
-	}
-	G3DTexturedVertex& A = inpoly [edge_from [0]];
-	G3DTexturedVertex& B = inpoly [edge_to [0]];
-	G3DTexturedVertex& C = inpoly [edge_from [1]];
-	G3DTexturedVertex& D = inpoly [edge_to [1]];
-	float t1 = (y - A.y) / (B.y - A.y);
-	float t2 = (y - C.y) / (D.y - C.y);
-	float x1 = A.x + t1 * (B.x - A.x);
-	float x2 = C.x + t2 * (D.x - C.x);
-	t = (x - x1) / (x2 - x1);
-	INTERPOLATE (z);
-	INTERPOLATE (u);
-	INTERPOLATE (v);
-	if (gouraud)
-	{
-	  INTERPOLATE (r);
-	  INTERPOLATE (g);
-	  INTERPOLATE (b);
-	}
-	break;
+      GetTempIndices()->Push ((uint)GetTempVertices()->Length());
+      GetTempVertices()->Push (campts[i]);
+      GetTempTexels()->Push (uvs[i]);
+    }
+    return;
+  }
+  // split up
+  csVector3 oldpos = scrpts[2];
+  csVector3 oldcam = campts[2];
+  csVector2 olduv = uvs[2];
+  scrpts[2] = (scrpts[1] + scrpts[2])*0.5;
+  campts[2] = (campts[1] + campts[2])*0.5;
+  csVector2 newdir;
+  newdir.x = scrpts[2].x - scrpts[0].x;
+  newdir.y = scrpts[2].y - scrpts[0].y;
+  newdir /= newdir.Norm();
+  uvs[2].Set(0.5, 0.5);
+  uvs[2] += newdir * layer_scale;
+  GenGeometryAdapt (rview, g3d, 3, scrpts, campts, uvs, layer_scale, quality, 
+    depth+1, maxdepth);
+  // other half
+  csVector3 oldpos1 = scrpts[1];
+  csVector3 oldcam1 = campts[1];
+  csVector2 olduv1 = uvs[1];
+  scrpts[1] = scrpts[2];
+  campts[1] = campts[2];
+  uvs[1] = uvs[2];
+  scrpts[2] = oldpos;
+  campts[2] = oldcam;
+  uvs[2] = olduv;
+  GenGeometryAdapt (rview, g3d, 3, scrpts, campts, uvs, layer_scale, quality, 
+    depth+1, maxdepth);
+  scrpts[1] = oldpos1;
+  campts[1] = oldcam1;
+  uvs[1] = olduv1;
+}
+
+csRenderMesh** csHazeMeshObject::GetRenderMeshes (int &n, iRenderView* rview,
+						  iMovable* movable, 
+						  uint32 frustum_mask)
+{ 
+  SetupObject ();
+
+  if(layers.Length() <= 0)
+  {
+    n = 0; 
+    return 0; 
+  }
+
+  /// prepare to transform the points
+  iGraphics3D* g3d = rview->GetGraphics3D ();
+  iCamera* camera = rview->GetCamera ();
+  csVector3 campos = camera->GetTransform().GetOrigin();
+  if (!movable->IsFullTransformIdentity ())
+    campos = movable->GetFullTransform() * campos;
+  float fov = camera->GetFOV ();
+  float shx = camera->GetShiftX ();
+  float shy = camera->GetShiftY ();
+  /// obj to camera space
+  csReversibleTransform tr_o2c = camera->GetTransform ();
+  if (!movable->IsFullTransformIdentity ())
+    tr_o2c /= movable->GetFullTransform ();
+
+
+  // project origin to screenspace
+  csVector2 center(0.5, 0.5);
+  csVector3 scr_orig;
+  csVector3 cam_orig;
+  ProjectO2S(tr_o2c, fov, shx, shy, origin, scr_orig, &cam_orig);
+
+  // get hull 0 outline in screenspace
+  iHazeHull *hull = layers[0]->hull;
+  float layer_scale = layers[0]->scale;
+  int layer_num = 0;
+  int *layer_poly = 0;
+  csVector3* layer_pts = 0;
+  csVector2* layer_uvs = 0;
+  csVector3* cam_pts = 0;
+  ComputeHullOutline (hull, layer_scale, campos, tr_o2c, fov, shx, shy,
+    layer_num, layer_poly, layer_pts, &cam_pts, layer_uvs);
+  if(layer_num <= 0)
+  {
+    n = 0; 
+    return 0; 
+  }
+
+  int i;
+  // additional test if origin inside the outline
+  csVector2* incheck = new csVector2[layer_num];
+  for(i=0; i<layer_num; i++)
+  {
+    incheck[i].x = layer_pts[layer_num-1 - i].x;
+    incheck[i].y = layer_pts[layer_num-1 - i].y;
+  }
+  csVector2 checkpt( scr_orig.x, scr_orig.y );
+  if(!csPoly2D::In (incheck, layer_num, checkpt))
+  {
+    // origin not inside outline.
+    delete[] incheck;
+    delete[] layer_poly;
+    delete[] layer_pts;
+    delete[] layer_uvs;
+    delete[] cam_pts;
+    n = 0; 
+    return 0; 
+  }
+  delete[] incheck;
+
+  GetTempVertices()->Empty();
+  GetTempTexels()->Empty();
+  GetTempIndices()->Empty();
+
+  {
+    // draw triangles from orig to layer 0
+    csVector3 tri_pts[3];
+    csVector3 tri_campts[3];
+    csVector2 tri_uvs[3];
+    tri_pts[0] = scr_orig;
+    tri_campts[0] = cam_orig;
+    tri_uvs[0] = center;
+    for(i=0; i<layer_num; i++)
+    {
+      int nexti = (i+1)%layer_num;
+      tri_pts[2] = layer_pts[i];
+      tri_pts[1] = layer_pts[nexti];
+      tri_campts[2] = cam_pts[i];
+      tri_campts[1] = cam_pts[nexti];
+      tri_uvs[2] = layer_uvs[i];
+      tri_uvs[1] = layer_uvs[nexti];
+
+
+      //csPrintf("drawing a polygon\n");
+      //DrawPoly(rview, g3d, mat, 3, tri_pts, tri_uvs);
+
+      float quality = 0.90f;
+      int maxdepth = 10;
+      GenGeometryAdapt (rview, g3d, 3, tri_pts, tri_campts, tri_uvs, 
+	layer_scale, quality, 0, maxdepth);
     }
   }
-}
 
-#undef INTERPOLATE
-#undef INTERPOLATE1
+  bool bufferCreated;
+  const size_t vertCount = GetTempVertices()->Length();
+  HazeRenderBuffer& vertices = renderBuffers.GetUnusedData (bufferCreated,
+    rview->GetCurrentFrameNumber());
+  if (bufferCreated || (vertices.count < vertCount))
+  {
+    vertices.buffer = csRenderBuffer::CreateRenderBuffer (vertCount,
+      CS_BUF_STREAM, CS_BUFCOMP_FLOAT, 3);
+  }
+  vertices.buffer->CopyInto (GetTempVertices()->GetArray(), 
+    vertCount);
+  HazeRenderBuffer& texels = renderBuffers.GetUnusedData (bufferCreated,
+    rview->GetCurrentFrameNumber());
+  if (bufferCreated || (texels.count < vertCount))
+  {
+    texels.buffer = csRenderBuffer::CreateRenderBuffer (vertCount,
+      CS_BUF_STREAM, CS_BUFCOMP_FLOAT, 2);
+  }
+  texels.buffer->CopyInto (GetTempTexels()->GetArray(), 
+    vertCount);
+  HazeRenderBuffer& indices = indexBuffers.GetUnusedData (bufferCreated,
+    rview->GetCurrentFrameNumber());
+  if (bufferCreated || (indices.count < GetTempIndices()->Length()))
+  {
+    indices.buffer = csRenderBuffer::CreateIndexRenderBuffer (
+      GetTempIndices()->Length(), CS_BUF_STREAM, 
+      CS_BUFCOMP_UNSIGNED_INT, 0, vertCount-1);
+  }
+  indices.buffer->CopyInto (GetTempIndices()->GetArray(), 
+    GetTempIndices()->Length());
+
+  bool rmCreated;
+  csRenderMesh*& rm = rmHolder.GetUnusedMesh (rmCreated, 
+    rview->GetCurrentFrameNumber());
+  if (rmCreated)
+  {
+    rm->buffers.AttachNew (new csRenderBufferHolder);
+    rm->variablecontext.AttachNew (new csShaderVariableContext);
+    rm->meshtype = CS_MESHTYPE_TRIANGLES;
+    rm->material = material;
+    rm->mixmode = MixMode;
+  }
+
+  int clip_portal, clip_plane, clip_z_plane;
+  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
+      clip_z_plane);
+
+  rm->worldspace_origin = movable->GetFullPosition ();
+  rm->clip_portal = clip_portal;
+  rm->clip_plane = clip_plane;
+  rm->clip_z_plane = clip_z_plane;
+  rm->do_mirror = camera->IsMirrored ();
+
+  rm->object2world = camera->GetTransform ();
+
+  rm->indexend = (uint)GetTempIndices()->Length();
+
+  rm->buffers->SetRenderBuffer (CS_BUFFER_INDEX, indices.buffer);
+  rm->buffers->SetRenderBuffer (CS_BUFFER_POSITION, vertices.buffer);
+  rm->buffers->SetRenderBuffer (CS_BUFFER_TEXCOORD0, texels.buffer);
+
+  delete[] layer_poly;
+  delete[] layer_pts;
+  delete[] layer_uvs;
+  delete[] cam_pts;
+
+  n = 1; 
+  return &rm; 
+}
 
 
 void csHazeMeshObject::ComputeHullOutline(iHazeHull *hull, float layer_scale,
   const csVector3& campos, csReversibleTransform& tr_o2c, float fov, float shx,
   float shy, int &layer_num, int *& layer_poly, csVector3 *& layer_pts,
-  csVector2 *&layer_uvs)
+  csVector3** cam_pts, csVector2 *&layer_uvs)
 {
   int i;
   // get hull outline in screenspace
   layer_num = 0;
   layer_poly = 0;
-  //printf("campos %g,%g,%g\n", campos.x, campos.y, campos.z);
+  //csPrintf("campos %g,%g,%g\n", campos.x, campos.y, campos.z);
   csHazeHull::ComputeOutline(hull, campos, layer_num, layer_poly);
-  //printf("has outline of size %d: ", layer_num);
+  //csPrintf("has outline of size %d: ", layer_num);
   if(layer_num <= 0) return;
   layer_pts = new csVector3[layer_num];
+  *cam_pts = new csVector3[layer_num];
+
   for(i=0; i<layer_num; i++)
   {
-    //printf(" %d", layer_poly[i]);
+    //csPrintf(" %d", layer_poly[i]);
     csVector3 objpos;
     hull->GetVertex(objpos, layer_poly[i] );
-    ProjectO2S(tr_o2c, fov, shx, shy, objpos, layer_pts[i]);
+
+    ProjectO2S(tr_o2c, fov, shx, shy, objpos, layer_pts[i],
+      &((*cam_pts)[i]));
   }
-  //printf("\n");
+  //csPrintf("\n");
   // get hull 0 uv values
   layer_uvs = new csVector2[layer_num];
   csVector2 center(0.5, 0.5);
   // project to screenspace
   csVector3 scr_orig;
-  ProjectO2S(tr_o2c, fov, shx, shy, origin, scr_orig);
+  csVector3 cam_orig;
+  ProjectO2S (tr_o2c, fov, shx, shy, origin, scr_orig, &cam_orig);
   csVector2 dir;
   for(i=0; i<layer_num; i++)
   {
@@ -744,250 +867,11 @@ void csHazeMeshObject::ComputeHullOutline(iHazeHull *hull, float layer_scale,
   }
 }
 
-bool csHazeMeshObject::Draw (iRenderView* rview, iMovable* movable,
-	csZBufMode mode)
-{
-  int i;
-  if (!material)
-  {
-    printf ("INTERNAL ERROR: haze used without material!\n");
-    return false;
-  }
-  iMaterialHandle* mat = material->GetMaterialHandle ();
-  if (!mat)
-  {
-    printf ("INTERNAL ERROR: haze used without valid material handle!\n");
-    return false;
-  }
-  if(layers.Length() <= 0) return false;
-
-  if (vis_cb) if (!vis_cb->BeforeDrawing (this, rview)) return false;
-
-  //printf("drawing\n");
-
-  /// prepare to transform the points
-  iGraphics3D* g3d = rview->GetGraphics3D ();
-  iCamera* camera = rview->GetCamera ();
-  csVector3 campos = camera->GetTransform().GetOrigin();
-  campos = movable->GetFullTransform() * campos;
-  float fov = camera->GetFOV ();
-  float shx = camera->GetShiftX ();
-  float shy = camera->GetShiftY ();
-  /// obj to camera space
-  csReversibleTransform tr_o2c = camera->GetTransform ()
-        * movable->GetFullTransform ().GetInverse ();
-
-  // Prepare for rendering.
-  g3d->SetRenderState (G3DRENDERSTATE_ZBUFFERMODE, mode);
-  material->Visit ();
-
-  // project origin to screenspace
-  csVector2 center(0.5, 0.5);
-  csVector3 scr_orig;
-  ProjectO2S(tr_o2c, fov, shx, shy, origin, scr_orig);
-
-  // get hull 0 outline in screenspace
-  iHazeHull *hull = layers.GetLayer(0)->hull;
-  float layer_scale = layers.GetLayer(0)->scale;
-  int layer_num = 0;
-  int *layer_poly = 0;
-  csVector3* layer_pts = 0;
-  csVector2* layer_uvs = 0;
-  ComputeHullOutline(hull, layer_scale, campos, tr_o2c, fov, shx, shy,
-    layer_num, layer_poly, layer_pts, layer_uvs);
-  if(layer_num <= 0) return false;
-
-  // additional test if origin inside the outline
-  csVector2* incheck = new csVector2[layer_num];
-  for(i=0; i<layer_num; i++)
-  {
-    incheck[i].x = layer_pts[layer_num-1 - i].x;
-    incheck[i].y = layer_pts[layer_num-1 - i].y;
-  }
-  csVector2 checkpt( scr_orig.x, scr_orig.y );
-  if(!csPoly2D::In(incheck, layer_num, checkpt))
-  {
-    // origin not inside outline.
-    delete[] incheck;
-    delete[] layer_poly;
-    delete[] layer_pts;
-    delete[] layer_uvs;
-    return false;
-  }
-  delete[] incheck;
-
-  // draw triangles from orig to layer 0
-  csVector3 tri_pts[3];
-  csVector2 tri_uvs[3];
-  tri_pts[0] = scr_orig;
-  tri_uvs[0] = center;
-  for(i=0; i<layer_num; i++)
-  {
-    int nexti = (i+1)%layer_num;
-    tri_pts[2] = layer_pts[i];
-    tri_pts[1] = layer_pts[nexti];
-    tri_uvs[2] = layer_uvs[i];
-    tri_uvs[1] = layer_uvs[nexti];
-    //printf("drawing a polygon\n");
-    //DrawPoly(rview, g3d, mat, 3, tri_pts, tri_uvs);
-    float quality = 0.90;
-    DrawPolyAdapt(rview, g3d, mat, 3, tri_pts, tri_uvs, layer_scale,
-      quality);
-
-#if 0
-    // debug drawing of the outline
-    iGraphics2D *g2d = g3d->GetDriver2D();
-    int m2dy = g2d->GetHeight();
-    g2d->DrawLine(scr_orig.x, m2dy-scr_orig.y, scr_orig.x+1,
-      m2dy-scr_orig.y+1, -1);
-    // only outline
-    //g2d->DrawLine( layer_pts[i].x, layer_pts[i].y,
-      //layer_pts[nexti].x, layer_pts[nexti].y, -1);
-    // show direction of lines (from black to white)
-    float midx = (layer_pts[i].x + layer_pts[nexti].x)*0.5;
-    float midy = (layer_pts[i].y + layer_pts[nexti].y)*0.5;
-    g2d->DrawLine( layer_pts[i].x, m2dy-layer_pts[i].y, midx, m2dy-midy, 0);
-    g2d->DrawLine( midx, m2dy-midy, layer_pts[nexti].x,
-      m2dy-layer_pts[nexti].y, -1);
-#endif
-  }
-
-
-#if 0 // draw multiple hulls
-  int  curlay;
-  for(curlay = 1; curlay < layers.Length(); curlay++)
-  {
-    // get hull [curlay] outline in screenspace
-    iHazeHull *hull2 = layers.GetLayer(curlay)->hull;
-    float layer_scale2 = layers.GetLayer(curlay)->scale;
-    int layer_num2 = 0;
-    int *layer_poly2 = 0;
-    csVector3* layer_pts2 = 0;
-    csVector2* layer_uvs2 = 0;
-    ComputeHullOutline(hull2, layer_scale2, campos, tr_o2c, fov, shx, shy,
-      layer_num2, layer_poly2, layer_pts2, layer_uvs2);
-    if(layer_num2 <= 0)
-    {
-      delete[] layer_poly;
-      delete[] layer_pts;
-      delete[] layer_uvs;
-      return true;
-    }
-
-#if 1
-    for(i=0; i<layer_num2; i++)
-    {
-      int nexti = (i+1)%layer_num;
-      iGraphics2D *g2d = g3d->GetDriver2D();
-      int m2dy = g2d->GetHeight();
-      float midx = (layer_pts2[i].x + layer_pts2[nexti].x)*0.5;
-      float midy = (layer_pts2[i].y + layer_pts2[nexti].y)*0.5;
-      g2d->DrawLine( layer_pts2[i].x, m2dy-layer_pts2[i].y, midx, m2dy-midy, 0);
-      g2d->DrawLine( midx, m2dy-midy, layer_pts2[nexti].x,
-        m2dy-layer_pts2[nexti].y, -1);
-    }
-#endif
-
-    /// got  this outline, draw between hull to hull2
-    if(layer_num != layer_num2) {
-      printf("haze: outlines differ for hull %d,%d!\n", layer_num, layer_num2);
-      /*
-      delete[] layer_poly;
-      delete[] layer_pts;
-      delete[] layer_uvs;
-      delete[] layer_poly2;
-      delete[] layer_pts2;
-      delete[] layer_uvs2;
-      return true;
-      */
-    }
-    /// draw
-    for(i=0; i<layer_num; i++)
-    {
-      int ni = (i+1)%layer_num;
-      tri_pts[0] = layer_pts[i];
-      tri_uvs[0] = layer_uvs[i];
-      tri_pts[1] = layer_pts[ni];
-      tri_uvs[1] = layer_uvs[ni];
-      tri_pts[2] = layer_pts2[ni];
-      tri_uvs[2] = layer_uvs2[ni];
-      DrawPoly(rview, g3d, mat, 3, tri_pts, tri_uvs);
-      tri_pts[0] = layer_pts[i];
-      tri_uvs[0] = layer_uvs[i];
-      tri_pts[1] = layer_pts2[ni];
-      tri_uvs[1] = layer_uvs2[ni];
-      tri_pts[2] = layer_pts2[i];
-      tri_uvs[2] = layer_uvs2[i];
-      DrawPoly(rview, g3d, mat, 3, tri_pts, tri_uvs);
-    }
-
-    /// get ready for the next hull
-    hull = hull2;
-    layer_scale = layer_scale2;
-    layer_num = layer_num2;
-    delete[] layer_poly; layer_poly = layer_poly2;
-    delete[] layer_pts; layer_pts = layer_pts2;
-    delete[] layer_uvs; layer_uvs = layer_uvs2;
-  }
-#endif // draw multiple hulls
-
-  delete[] layer_poly;
-  delete[] layer_pts;
-  delete[] layer_uvs;
-
-  return true;
-}
-
-
-void csHazeMeshObject::DrawPolyAdapt(iRenderView *rview, iGraphics3D *g3d,
-  iMaterialHandle *mat, int num_sides, csVector3* pts, csVector2* uvs,
-        float layer_scale, float quality)
-{
-  /// only triangles
-  CS_ASSERT(num_sides == 3);
-  // check if the angle is OK
-  csVector2 dir1;
-  dir1.x = pts[1].x - pts[0].x;
-  dir1.y = pts[1].y - pts[0].y;
-  csVector2 dir2;
-  dir2.x = pts[2].x - pts[0].x;
-  dir2.y = pts[2].y - pts[0].y;
-  csVector2 normdir1 = dir1 / dir1.Norm();
-  csVector2 normdir2 = dir2 / dir2.Norm();
-  float cosangle = normdir1 * normdir2;
-  //printf("cosangle %g, quality %g\n", cosangle, quality);
-  if(cosangle > quality)
-  {
-    // draw it
-    DrawPoly(rview, g3d, mat, 3, pts, uvs);
-    return;
-  }
-  // split up
-  csVector3 oldpos = pts[2];
-  csVector2 olduv = uvs[2];
-  pts[2] = (pts[1] + pts[2])*0.5;
-  csVector2 newdir;
-  newdir.x = pts[2].x - pts[0].x;
-  newdir.y = pts[2].y - pts[0].y;
-  newdir /= newdir.Norm();
-  uvs[2].Set(0.5, 0.5);
-  uvs[2] += newdir * layer_scale;
-  DrawPolyAdapt(rview, g3d, mat, 3, pts, uvs, layer_scale, quality);
-  // other half
-  csVector3 oldpos1 = pts[1];
-  csVector2 olduv1 = uvs[1];
-  pts[1] = pts[2];
-  uvs[1] = uvs[2];
-  pts[2] = oldpos;
-  uvs[2] = olduv;
-  DrawPolyAdapt(rview, g3d, mat, 3, pts, uvs, layer_scale, quality);
-  pts[1] = oldpos1;
-  uvs[1] = olduv1;
-}
-
 void csHazeMeshObject::ProjectO2S(csReversibleTransform& tr_o2c, float fov,
-  float shiftx, float shifty, const csVector3& objpos, csVector3& scrpos)
+  float shiftx, float shifty, const csVector3& objpos, csVector3& scrpos, 
+  csVector3* campos)
 {
+  *campos =
   scrpos = tr_o2c * objpos;  // to camera space
   scrpos.z = 1. / scrpos.z; // = iz
   float inv_z = fov * scrpos.z; // = iz
@@ -995,63 +879,26 @@ void csHazeMeshObject::ProjectO2S(csReversibleTransform& tr_o2c, float fov,
   scrpos.y = scrpos.y * inv_z + shifty;
 }
 
-void csHazeMeshObject::DrawPoly(iRenderView *rview, iGraphics3D *g3d,
-  iMaterialHandle *mat, int num, const csVector3* pts, const csVector2* uvs)
-{
-  g3dpolyfx.use_fog = false;
-  g3dpolyfx.num = num;
-  g3dpolyfx.mat_handle = mat;
-  g3dpolyfx.mat_handle->GetTexture ()->GetMeanColor (g3dpolyfx.flat_color_r,
-    g3dpolyfx.flat_color_g, g3dpolyfx.flat_color_b);
-
-  ALLOC_STACK_ARRAY (poly2d, csVector2, num);
-  csVector2 clipped_poly2d[MAX_OUTPUT_VERTICES];
-  csVertexStatus clipped_vtstats[MAX_OUTPUT_VERTICES];
-
-  int i;
-  for(i = 0; i < num; i++)
-  {
-    poly2d[i].x = pts[i].x;
-    poly2d[i].y = pts[i].y;
-    g3dpolyfx.vertices [i].x = pts[i].x;
-    g3dpolyfx.vertices [i].y = pts[i].y;
-    g3dpolyfx.vertices [i].z = pts[i].z;
-    g3dpolyfx.vertices [i].r = 1.0;
-    g3dpolyfx.vertices [i].g = 1.0;
-    g3dpolyfx.vertices [i].b = 1.0;
-    g3dpolyfx.vertices [i].u = uvs [i].x;
-    g3dpolyfx.vertices [i].v = uvs [i].y;
-  }
-
-  int num_clipped_verts;
-  uint8 clip_result = rview->GetClipper ()->Clip (poly2d, num,
-    clipped_poly2d, num_clipped_verts, clipped_vtstats);
-  if (clip_result == CS_CLIP_OUTSIDE) return;
-  g3dpolyfx.num = num_clipped_verts;
-
-  if (clip_result != CS_CLIP_INSIDE)
-    PreparePolygonFX2 (&g3dpolyfx, clipped_poly2d, num_clipped_verts,
-    	clipped_vtstats, num, false); // no gouraud interpol, no true);
-
-  rview->CalculateFogPolygon (g3dpolyfx);
-  g3dpolyfx.mixmode = MixMode; // | CS_FX_GOURAUD;
-  g3d->DrawPolygonFX (g3dpolyfx);
-
-}
-
-void csHazeMeshObject::GetObjectBoundingBox (csBox3& retbbox, int /*type*/)
+void csHazeMeshObject::GetObjectBoundingBox (csBox3& retbbox)
 {
   SetupObject ();
   retbbox = bbox;
 }
 
+void csHazeMeshObject::SetObjectBoundingBox (const csBox3& inbbox)
+{
+  bbox = inbbox;
+  scfiObjectModel.ShapeChanged ();
+}
+
 void csHazeMeshObject::HardTransform (const csReversibleTransform& t)
 {
   (void)t;
   //@@@ TODO
 }
 
-void csHazeMeshObject::NextFrame (csTicks /*current_time*/)
+void csHazeMeshObject::NextFrame (csTicks /*current_time*/,
+	const csVector3& /*pos*/)
 {
   // nothing
 }
@@ -1072,28 +919,33 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObjectFactory::HazeHullCreation)
   SCF_IMPLEMENTS_INTERFACE (iHazeHullCreation)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-csHazeMeshObjectFactory::csHazeMeshObjectFactory (iBase *pParent)
+csHazeMeshObjectFactory::csHazeMeshObjectFactory (csHazeMeshObjectType* pParent)
 {
   SCF_CONSTRUCT_IBASE (pParent);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeFactoryState);
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeHullCreation);
-  material = NULL;
+  material = 0;
   MixMode = 0;
   origin.Set(0,0,0);
   directional.Set(0,0,0);
-  logparent = NULL;
+  logparent = 0;
+  haze_type = pParent;
+  object_reg = pParent->object_reg;
 }
 
 csHazeMeshObjectFactory::~csHazeMeshObjectFactory ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeFactoryState);
+  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeHullCreation);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObject* csHazeMeshObjectFactory::NewInstance ()
+csPtr<iMeshObject> csHazeMeshObjectFactory::NewInstance ()
 {
   csHazeMeshObject* cm = new csHazeMeshObject (this);
-  iMeshObject* im = SCF_QUERY_INTERFACE (cm, iMeshObject);
-  im->DecRef ();
-  return im;
+  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
+  cm->DecRef ();
+  return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
@@ -1109,10 +961,6 @@ SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csHazeMeshObjectType)
 
-SCF_EXPORT_CLASS_TABLE (haze)
-  SCF_EXPORT_CLASS (csHazeMeshObjectType, "crystalspace.mesh.object.haze",
-    "Crystal Space Haze Mesh Type")
-SCF_EXPORT_CLASS_TABLE_END
 
 csHazeMeshObjectType::csHazeMeshObjectType (iBase* pParent)
 {
@@ -1122,13 +970,16 @@ csHazeMeshObjectType::csHazeMeshObjectType (iBase* pParent)
 
 csHazeMeshObjectType::~csHazeMeshObjectType ()
 {
+  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
+  SCF_DESTRUCT_IBASE ();
 }
 
-iMeshObjectFactory* csHazeMeshObjectType::NewFactory ()
+csPtr<iMeshObjectFactory> csHazeMeshObjectType::NewFactory ()
 {
   csHazeMeshObjectFactory* cm = new csHazeMeshObjectFactory (this);
-  iMeshObjectFactory* ifact = SCF_QUERY_INTERFACE (cm, iMeshObjectFactory);
-  ifact->DecRef ();
-  return ifact;
+  csRef<iMeshObjectFactory> ifact (
+  	SCF_QUERY_INTERFACE (cm, iMeshObjectFactory));
+  cm->DecRef ();
+  return csPtr<iMeshObjectFactory> (ifact);
 }
 
