@@ -16,28 +16,34 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef _HAZE_H_
-#define _HAZE_H_
+#ifndef __CS_HAZE_H__
+#define __CS_HAZE_H__
 
-#include "csgeom/vector3.h"
+#include "csgeom/objmodel.h"
 #include "csgeom/transfrm.h"
+#include "csgeom/vector3.h"
+#include "cstool/rendermeshholder.h"
 #include "csutil/cscolor.h"
-#include "csutil/csvector.h"
-#include "imesh/object.h"
+#include "csutil/flags.h"
+#include "csutil/parray.h"
+#include "csutil/refarr.h"
 #include "imesh/haze.h"
+#include "imesh/object.h"
 #include "imesh/particle.h"
-#include "ivideo/graph3d.h"
-#include "iutil/config.h"
-#include "iutil/eventh.h"
 #include "iutil/comp.h"
+#include "iutil/eventh.h"
+#include "ivideo/graph3d.h"
 
 struct iMaterialWrapper;
 struct iCamera;
 class csHazeMeshObjectFactory;
+class csHazeMeshObjectType;
 
-/** haze layer
-*/
-class csHazeLayer {
+/**
+ * haze layer
+ */
+class csHazeLayer
+{
 public:
   /// hull
   iHazeHull *hull;
@@ -50,7 +56,8 @@ public:
 };
 
 /// a haze hull
-class csHazeHull : public iHazeHull {
+class csHazeHull : public iHazeHull
+{
 protected:
   /// total counts
   int total_poly, total_vert, total_edge;
@@ -97,26 +104,31 @@ public:
   virtual int GetEdgeCount() const {return total_edge;}
 
   virtual void GetVertex(csVector3& res, int vertex_idx) const
-  { CS_ASSERT( vertex_idx >= 0 && vertex_idx < total_vert);
+  {
+    CS_ASSERT( vertex_idx >= 0 && vertex_idx < total_vert);
     res = verts[vertex_idx];}
   virtual void GetEdge(int edge_num, int& vertex_idx_1, int& vertex_idx_2) const
-  { CS_ASSERT( edge_num >= 0 && edge_num < total_edge);
+  {
+    CS_ASSERT( edge_num >= 0 && edge_num < total_edge);
     vertex_idx_1 = edgept1[edge_num];
     vertex_idx_2 = edgept2[edge_num];
   }
 
   virtual int GetPolVerticeCount(int polygon_num) const
-  { CS_ASSERT( polygon_num >= 0 && polygon_num < total_poly);
+  {
+    CS_ASSERT( polygon_num >= 0 && polygon_num < total_poly);
     return pol_num[polygon_num];
   }
   virtual int GetPolVertex(int polygon_num, int vertex_num) const
-  { CS_ASSERT( polygon_num >= 0 && polygon_num < total_poly);
+  {
+    CS_ASSERT( polygon_num >= 0 && polygon_num < total_poly);
     CS_ASSERT( vertex_num >= 0 && vertex_num < pol_num[polygon_num]);
     return pol_verts[polygon_num][vertex_num];
   }
   virtual int GetPolEdge(int polygon_num, int vertex_num, int& start_idx,
     int& end_idx) const
-  { CS_ASSERT( polygon_num >= 0 && polygon_num < total_poly);
+  {
+    CS_ASSERT( polygon_num >= 0 && polygon_num < total_poly);
     CS_ASSERT( vertex_num >= 0 && vertex_num < pol_num[polygon_num]);
     start_idx = pol_verts[polygon_num][vertex_num];
     int inc = (vertex_num+1)%pol_num[polygon_num];
@@ -127,10 +139,11 @@ public:
 
 
 /** box haze hull */
-class csHazeHullBox : public csHazeHull {
+class csHazeHullBox : public csHazeHull
+{
   csVector3 min, max;
 public:
-  SCF_DECLARE_IBASE;
+  SCF_DECLARE_IBASE_EXT(csHazeHull);
   /// create new.
   csHazeHullBox(const csVector3& a, const csVector3& b);
   ///
@@ -147,12 +160,13 @@ public:
 };
 
 /** cone haze hull */
-class csHazeHullCone : public csHazeHull {
+class csHazeHullCone : public csHazeHull
+{
   int nr_sides;
   csVector3 start, end;
   float start_radius, end_radius;
 public:
-  SCF_DECLARE_IBASE;
+  SCF_DECLARE_IBASE_EXT(csHazeHull);
   /// create new.
   csHazeHullCone(int nr, const csVector3& a, const csVector3& b, float ra,
     float rb);
@@ -165,53 +179,36 @@ public:
     /// get settings
     virtual void GetSettings(int &nr, csVector3& a, csVector3& b, float &ra,
       float &rb)
-    {nr = scfParent->nr_sides; a=scfParent->start; b=scfParent->end;
-     ra = scfParent->start_radius; rb = scfParent->end_radius;}
+    {
+      nr = scfParent->nr_sides;
+      a=scfParent->start;
+      b=scfParent->end;
+      ra = scfParent->start_radius;
+      rb = scfParent->end_radius;
+    }
   } scfiHazeHullCone;
   friend class HazeHullCone;
 };
 
 
-/**
- * haze layer vector
- */
-class csHazeLayerVector : public csVector
-{
-public:
-  /// construct
-  csHazeLayerVector(int ilimit = 8, int ithreshold = 16)
-    : csVector(ilimit, ithreshold) {}
-  /// destroy
-  virtual ~csHazeLayerVector() {DeleteAll ();}
-  /// delete correctly
-  virtual bool FreeItem(csSome Item)
-  { delete (csHazeLayer*)Item; return true; }
-  /// get a layer
-  csHazeLayer* GetLayer(int nr) {return (csHazeLayer*)Get(nr);}
-};
-
-
 /**
  * Haze mesh object.
  */
 class csHazeMeshObject : public iMeshObject
 {
 private:
-  iMeshObjectFactory* ifactory;
-  iBase* logparent;
+  csRef<iMeshObjectFactory> ifactory;
+  iMeshWrapper* logparent;
   csHazeMeshObjectFactory* factory;
 
-  iMaterialWrapper* material;
+  csRef<iMaterialWrapper> material;
   uint MixMode;
   bool initialized;
   iMeshObjectDrawCallback* vis_cb;
   csVector3 radius;
-  long shapenr;
   float current_lod;
   uint32 current_features;
-
-  /// Polygon.
-  G3DPolygonDPFX g3dpolyfx;
+  csFlags flags;
 
   /// bbox in object space
   csBox3 bbox;
@@ -223,7 +220,6 @@ private:
   csBox3 camera_bbox;
   csBox3 world_bbox;
 
-
   /// Current camera number.
   long cur_cameranr;
   /// Current movable number.
@@ -232,16 +228,26 @@ private:
   /// haze data
   csVector3 origin, directional;
   /// vector of csHazeLayer
-  csHazeLayerVector layers;
+  csPDelArray<csHazeLayer> layers;
+
+  csRenderMeshHolder rmHolder;
+
+  struct HazeRenderBuffer
+  {
+    size_t count;
+    csRef<iRenderBuffer> buffer;
+
+    HazeRenderBuffer() : count(0) { }
+  };
+  csFrameDataHolder<HazeRenderBuffer> renderBuffers;
+  csFrameDataHolder<HazeRenderBuffer> indexBuffers;
+  static csStringID vertex_name, texel_name, index_name;
 
   /**
    * Setup this object. This function will check if setup is needed.
    */
   void SetupObject ();
 
-  /// Update lighting given a position.
-  void UpdateLighting (iLight** lights, int num_lights, const csVector3& pos);
-
   /// Get the bounding box in transformed space.
   void GetTransformedBoundingBox (long cameranr, long movablenr,
       const csReversibleTransform& trans, csBox3& cbox);
@@ -263,11 +269,6 @@ public:
   /// Destructor.
   virtual ~csHazeMeshObject ();
 
-  /** draw (and clip) a polygon, give nr of vertices, arrays of vert, uv
-   * pts are in screenspace (sx, sy, iz), uvs in texturespace (u, v)
-   */
-  void DrawPoly(iRenderView *rview, iGraphics3D *g3d, iMaterialHandle *mat,
-    int num, const csVector3* pts, const csVector2* uvs);
   /**
    *  Compute the outline of a particular hull.
    *  Pass a hull and its layerscale.
@@ -281,31 +282,38 @@ public:
   void ComputeHullOutline(iHazeHull *hull, float layer_scale,
     const csVector3& campos, csReversibleTransform& tr_o2c, float fov,
     float shx, float shy, int &layer_num, int *& layer_poly,
-    csVector3 *& layer_pts, csVector2 *&layer_uvs);
+    csVector3 *& layer_pts, csVector3** cam_pts, csVector2 *&layer_uvs);
   /** project a vertice in object space to screenspace */
   void ProjectO2S(csReversibleTransform& tr_o2c, float fov, float shiftx,
-    float shifty, const csVector3& objpos, csVector3& scrpos);
-  /** recursively draw a polygon - type 0=center 1,2 is rim, with
-    alternative interpolation.
-    quality is the minimal cos of the angle.
-      .9999 is maximum quality (infinite polygons).
-      .90 is nice quality.
-      0.70 is lower quality.
-      0.50 is low quality.
-      -1 is lowest quality (no adaptation)
-    */
-  void DrawPolyAdapt(iRenderView *rview, iGraphics3D *g3d, iMaterialHandle *mat,
-    int num_sides, csVector3* pts, csVector2* uvs,
-    float layer_scale, float quality);
+    float shifty, const csVector3& objpos, csVector3& scrpos, 
+    csVector3* campos);
+  void GetObjectBoundingBox (csBox3& bbox);
+  void SetObjectBoundingBox (const csBox3& bbox);
+  void GetRadius (csVector3& rad, csVector3& cent)
+  { rad =  radius; cent.Set(0,0,0); }
 
-  ///------------------------ iMeshObject implementation ------------------------
+  ///--------------------- iMeshObject implementation ------------------------
   SCF_DECLARE_IBASE;
 
   virtual iMeshObjectFactory* GetFactory () const { return ifactory; }
-  virtual bool DrawTest (iRenderView* rview, iMovable* movable);
-  virtual void UpdateLighting (iLight** lights, int num_lights,
-      	iMovable* movable);
-  virtual bool Draw (iRenderView* rview, iMovable* movable, csZBufMode mode);
+  virtual csFlags& GetFlags () { return flags; }
+  virtual csPtr<iMeshObject> Clone () { return 0; }
+  /** 
+   * Recursively generate geometry. 
+   * \a type 0=center 1,2 is rim, with alternative interpolation.
+   * \a quality is the minimal cos of the angle.
+   *  .9999 is maximum quality (infinite polygons).
+   *  .90 is nice quality.
+   *  0.70 is lower quality.
+   *  0.50 is low quality.
+   *  -1 is lowest quality (no adaptation)
+   * Recursion will only go until >= maxdepth. Pass depth=0 at start.
+   */
+  void GenGeometryAdapt (iRenderView *rview, iGraphics3D *g3d, 
+    int num_sides, csVector3* scrpts, csVector3* campts, csVector2* uvs,
+    float layer_scale, float quality, int depth, int maxdepth);
+  virtual csRenderMesh **GetRenderMeshes (int &n, iRenderView*, 
+    iMovable*, uint32);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
     if (cb) cb->IncRef ();
@@ -316,21 +324,49 @@ public:
   {
     return vis_cb;
   }
-  virtual void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL);
-  virtual void GetRadius (csVector3& rad, csVector3& cent)
-	{ rad =  radius; cent.Set(0,0,0); }
-  virtual void NextFrame (csTicks current_time);
-  virtual bool WantToDie () const { return false; }
+  virtual void NextFrame (csTicks current_time, const csVector3& /*pos*/);
   virtual void HardTransform (const csReversibleTransform& t);
   virtual bool SupportsHardTransform () const { return false; }
   virtual bool HitBeamOutline (const csVector3&, const csVector3&,
         csVector3&, float*)
   { return false; }
   virtual bool HitBeamObject (const csVector3&, const csVector3&,
-  	csVector3&, float*) { return false; }
-  virtual long GetShapeNumber () const { return shapenr; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  	csVector3&, float*, int* = 0, iMaterialWrapper** = 0) { return false; }
+  virtual void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
+  virtual iMeshWrapper* GetMeshWrapper () const { return logparent; }
+
+  //------------------------- iObjectModel implementation ----------------
+  class ObjectModel : public csObjectModel
+  {
+    SCF_DECLARE_EMBEDDED_IBASE (csHazeMeshObject);
+    virtual void GetObjectBoundingBox (csBox3& bbox)
+    {
+      scfParent->GetObjectBoundingBox (bbox);
+    }
+    virtual void SetObjectBoundingBox (const csBox3& bbox)
+    {
+      scfParent->SetObjectBoundingBox (bbox);
+    }
+    virtual void GetRadius (csVector3& rad, csVector3& cent)
+    {
+      scfParent->GetRadius (rad, cent);
+    }
+  } scfiObjectModel;
+  friend class ObjectModel;
+
+  virtual iObjectModel* GetObjectModel () { return &scfiObjectModel; }
+  virtual bool SetColor (const csColor&) { return false; }
+  virtual bool GetColor (csColor&) const { return false; }
+  virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
+  virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
+  virtual void SetMixMode (uint mode) { MixMode = mode; }
+  virtual uint GetMixMode () const { return MixMode; }
+  virtual void InvalidateMaterialHandles () { }
+  /**
+   * see imesh/object.h for specification. The default implementation
+   * does nothing.
+   */
+  virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
 
   //------------------------- iHazeState implementation ----------------
   class HazeState : public iHazeState
@@ -340,7 +376,8 @@ public:
     {
       scfParent->material = material;
     }
-    virtual iMaterialWrapper* GetMaterialWrapper () const { return scfParent->material; }
+    virtual iMaterialWrapper* GetMaterialWrapper () const
+    { return scfParent->material; }
     virtual void SetMixMode (uint mode) { scfParent->MixMode = mode; }
     virtual uint GetMixMode () const { return scfParent->MixMode; }
     virtual void SetOrigin(const csVector3& pos) { scfParent->origin = pos; }
@@ -349,22 +386,25 @@ public:
     { scfParent->directional=pos;}
     virtual const csVector3& GetDirectional() const
     {return scfParent->directional;}
-    virtual int GetLayerCount() const {return scfParent->layers.Length();}
+    virtual int GetLayerCount() const {return (int)scfParent->layers.Length();}
     virtual void AddLayer(iHazeHull *hull, float scale)
-    { csHazeLayer *lay = new csHazeLayer(hull, scale);
-      scfParent->layers.Push(lay); }
+    {
+      csHazeLayer *lay = new csHazeLayer(hull, scale);
+      scfParent->layers.Push(lay);
+    }
     virtual void SetLayerHull(int layer, iHazeHull* hull)
-    { if(hull) hull->IncRef();
-      if(scfParent->layers.GetLayer(layer)->hull)
-        scfParent->layers.GetLayer(layer)->hull->DecRef();
-      scfParent->layers.GetLayer(layer)->hull = hull;
+    {
+      if (hull) hull->IncRef();
+      if (scfParent->layers[layer]->hull)
+        scfParent->layers[layer]->hull->DecRef();
+      scfParent->layers[layer]->hull = hull;
     }
     virtual iHazeHull* GetLayerHull(int layer) const
-    { return scfParent->layers.GetLayer(layer)->hull; }
+    { return scfParent->layers[layer]->hull; }
     virtual void SetLayerScale(int layer, float scale)
-    { scfParent->layers.GetLayer(layer)->scale = scale; }
+    { scfParent->layers[layer]->scale = scale; }
     virtual float GetLayerScale(int layer) const
-    { return scfParent->layers.GetLayer(layer)->scale; }
+    { return scfParent->layers[layer]->scale; }
   } scfiHazeState;
   friend class HazeState;
 };
@@ -375,17 +415,20 @@ public:
 class csHazeMeshObjectFactory : public iMeshObjectFactory
 {
 private:
-  iMaterialWrapper* material;
+  csRef<iMaterialWrapper> material;
   uint MixMode;
   /// haze state info
   csVector3 origin, directional;
   /// vector of csHazeLayer
-  csHazeLayerVector layers;
-  iBase* logparent;
-
+  csPDelArray<csHazeLayer> layers;
+  iMeshFactoryWrapper* logparent;
+  iMeshObjectType* haze_type;
+  csFlags flags;
 public:
+  iObjectRegistry* object_reg;
+
   /// Constructor.
-  csHazeMeshObjectFactory (iBase *pParent);
+  csHazeMeshObjectFactory (csHazeMeshObjectType* pParent);
 
   /// Destructor.
   virtual ~csHazeMeshObjectFactory ();
@@ -395,7 +438,7 @@ public:
   /// Get mixmode.
   uint GetMixMode () const { return MixMode; }
   /// Get the layers vector
-  csHazeLayerVector* GetLayers() {return &layers;}
+  csPDelArray<csHazeLayer>* GetLayers() {return &layers;}
   /// get the origin
   const csVector3& GetOrigin() const {return origin;}
   /// get the directional
@@ -404,11 +447,17 @@ public:
   //------------------------ iMeshObjectFactory implementation --------------
   SCF_DECLARE_IBASE;
 
-  virtual iMeshObject* NewInstance ();
+  virtual csFlags& GetFlags () { return flags; }
+  virtual csPtr<iMeshObject> NewInstance ();
+  virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  { logparent = lp; }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  { return logparent; }
+  virtual iMeshObjectType* GetMeshObjectType () const { return haze_type; }
+  virtual iObjectModel* GetObjectModel () { return 0; }
 
   //------------------------- iHazeFactoryState implementation ----------------
   class HazeFactoryState : public iHazeFactoryState
@@ -418,7 +467,8 @@ public:
     {
       scfParent->material = material;
     }
-    virtual iMaterialWrapper* GetMaterialWrapper () const { return scfParent->material; }
+    virtual iMaterialWrapper* GetMaterialWrapper () const
+    { return scfParent->material; }
     virtual void SetMixMode (uint mode) { scfParent->MixMode = mode; }
     virtual uint GetMixMode () const { return scfParent->MixMode; }
     virtual void SetOrigin(const csVector3& pos) { scfParent->origin = pos; }
@@ -427,22 +477,25 @@ public:
     { scfParent->directional=pos;}
     virtual const csVector3& GetDirectional() const
     {return scfParent->directional;}
-    virtual int GetLayerCount() const {return scfParent->layers.Length();}
+    virtual int GetLayerCount() const {return (int)scfParent->layers.Length();}
     virtual void AddLayer(iHazeHull *hull, float scale)
-    { csHazeLayer *lay = new csHazeLayer(hull, scale);
-      scfParent->layers.Push(lay); }
+    {
+      csHazeLayer *lay = new csHazeLayer(hull, scale);
+      scfParent->layers.Push(lay);
+    }
     virtual void SetLayerHull(int layer, iHazeHull* hull)
-    { if(hull) hull->IncRef();
-      if(scfParent->layers.GetLayer(layer)->hull)
-        scfParent->layers.GetLayer(layer)->hull->DecRef();
-      scfParent->layers.GetLayer(layer)->hull = hull;
+    {
+      if(hull) hull->IncRef();
+      if(scfParent->layers[layer]->hull)
+        scfParent->layers[layer]->hull->DecRef();
+      scfParent->layers[layer]->hull = hull;
     }
     virtual iHazeHull* GetLayerHull(int layer) const
-    { return scfParent->layers.GetLayer(layer)->hull; }
+    { return scfParent->layers[layer]->hull; }
     virtual void SetLayerScale(int layer, float scale)
-    { scfParent->layers.GetLayer(layer)->scale = scale; }
+    { scfParent->layers[layer]->scale = scale; }
     virtual float GetLayerScale(int layer) const
-    { return scfParent->layers.GetLayer(layer)->scale; }
+    { return scfParent->layers[layer]->scale; }
   } scfiHazeFactoryState;
   friend class HazeFactoryState;
 
@@ -450,12 +503,23 @@ public:
   class HazeHullCreation : public iHazeHullCreation
   {
     SCF_DECLARE_EMBEDDED_IBASE (csHazeMeshObjectFactory);
-    virtual iHazeHullBox* CreateBox(const csVector3& a, const csVector3& b)
-      const { return &(new csHazeHullBox(a, b))->scfiHazeHullBox; }
-    virtual iHazeHullCone* CreateCone(int nr_sides, const csVector3& start,
-      const csVector3& end, float srad, float erad) const
-      { return &(new csHazeHullCone(nr_sides, start, end, srad, erad))->
-        scfiHazeHullCone; }
+    virtual csRef<iHazeHullBox> CreateBox(
+	const csVector3& a, const csVector3& b) const
+    {
+      csHazeHullBox* c = new csHazeHullBox(a, b);
+      csRef<iHazeHullBox> x;
+      x.AttachNew(&c->scfiHazeHullBox);
+      return x;
+    }
+    virtual csRef<iHazeHullCone> CreateCone(int nr_sides,
+      const csVector3& start, const csVector3& end, float srad,
+      float erad) const
+    {
+      csHazeHullCone* c = new csHazeHullCone(nr_sides, start, end, srad, erad);
+      csRef<iHazeHullCone> x;
+      x.AttachNew(&c->scfiHazeHullCone);
+      return x;
+    }
 
   } scfiHazeHullCreation;
   friend class HazeHullCreation;
@@ -468,6 +532,7 @@ public:
 class csHazeMeshObjectType : public iMeshObjectType
 {
 public:
+  iObjectRegistry* object_reg;
   SCF_DECLARE_IBASE;
 
   /// Constructor.
@@ -475,13 +540,14 @@ public:
   /// Destructor.
   virtual ~csHazeMeshObjectType ();
   /// New Factory.
-  virtual iMeshObjectFactory* NewFactory ();
+  virtual csPtr<iMeshObjectFactory> NewFactory ();
 
   struct eiComponent : public iComponent
   {
     SCF_DECLARE_EMBEDDED_IBASE(csHazeMeshObjectType);
-    virtual bool Initialize (iObjectRegistry*) { return true; }
+    virtual bool Initialize (iObjectRegistry* object_reg) 
+    { scfParent->object_reg = object_reg; return true; }
   } scfiComponent;
 };
 
-#endif // _HAZE_H_
+#endif // __CS_HAZE_H__
