@@ -18,31 +18,34 @@
 
 #include "cssysdef.h"
 
-#include "iutil/objreg.h"
-#include "iutil/plugin.h"
+#include "csqint.h"
+#include "csqsqrt.h"
 
+#include "csgeom/box.h"
+#include "csgfx/renderbuffer.h"
+#include "csutil/cscolor.h"
+#include "csutil/randomgen.h"
+#include "csutil/util.h"
 #include "iengine/camera.h"
 #include "iengine/movable.h"
 #include "iengine/rview.h"
-
 #include "imesh/particles.h"
-
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "ivaria/reporter.h"
 #include "ivideo/material.h"
 #include "ivideo/rndbuf.h"
 #include "ivideo/texture.h"
 #include "ivideo/txtmgr.h"
 
-#include "ivaria/reporter.h"
-
-#include "csutil/randomgen.h"
-#include "csutil/util.h"
-
-#include "qsqrt.h"
-#include "qint.h"
 
 #include "particles.h"
 #include <limits.h>
 
+CS_LEAKGUARD_IMPLEMENT (csParticlesFactory);
+CS_LEAKGUARD_IMPLEMENT (csParticlesObject);
+CS_LEAKGUARD_IMPLEMENT (csParticlesObject::eiRenderBufferAccessor);
+
 CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_IBASE (csParticlesType)
@@ -85,7 +88,7 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csParticlesFactory::eiParticlesFactoryState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 csParticlesFactory::csParticlesFactory (csParticlesType* p,
-	iObjectRegistry* objreg) : parent (p), object_reg (objreg)
+  iObjectRegistry* objreg) : particles_type(p), object_reg (objreg)
 {
   SCF_CONSTRUCT_IBASE (p)
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiParticlesFactoryState)
@@ -101,6 +104,7 @@ csParticlesFactory::csParticlesFactory (csParticlesType* p,
   force_type = CS_PART_FORCE_RADIAL;
 
   force_direction = csVector3(0.0f, 0.0f, 0.0f);
+  force_direction_variation = csVector3(0.0f, 0.0f, 0.0f);
   force_range = 1.0f;
   force_falloff = CS_PART_FALLOFF_LINEAR;
   force_cone_radius = 0.0f;
@@ -129,9 +133,13 @@ csParticlesFactory::csParticlesFactory (csParticlesType* p,
   transform_mode = false;
 
   physics_plugin = "crystalspace.particles.physics.simple";
+  physics_zsort = false;
 
   color_method = CS_PART_COLOR_CONSTANT;
-  constant_color = csColor(1,1,1);
+  constant_color = csColor4 (1.0f, 1.0f, 1.0f, 1.0f);
+
+  mixmode = CS_FX_COPY;
+  zsort_enabled = false;
 }
 
 csParticlesFactory::~csParticlesFactory ()
@@ -147,7 +155,6 @@ SCF_IMPLEMENT_IBASE (csParticlesObject)
   SCF_IMPLEMENTS_INTERFACE (iMeshObject)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iParticlesObjectState)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iShaderVariableAccessor)
 SCF_IMPLEMENT_IBASE_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csParticlesObject::eiParticlesObjectState)
@@ -159,17 +166,16 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csParticlesObject::eiObjectModel)
   SCF_IMPLEMENTS_INTERFACE (iObjectModel)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csParticlesObject::eiShaderVariableAccessor)
-  SCF_IMPLEMENTS_INTERFACE (iShaderVariableAccessor)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
+SCF_IMPLEMENT_IBASE (csParticlesObject::eiRenderBufferAccessor)
+  SCF_IMPLEMENTS_INTERFACE (iRenderBufferAccessor)
+SCF_IMPLEMENT_IBASE_END
 
 csParticlesObject::csParticlesObject (csParticlesFactory* p)
-  : logparent (p), pFactory (p)
+  : logparent (0), pFactory (p)
 {
   SCF_CONSTRUCT_IBASE (p)
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiParticlesObjectState)
   SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel)
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiShaderVariableAccessor)
 
   mesh = 0;
   meshpp = 0;
@@ -179,10 +185,6 @@ csParticlesObject::csParticlesObject (csParticlesFactory* p)
     CS_QUERY_REGISTRY_TAG_INTERFACE (p->object_reg,
 	  "crystalspace.shared.stringset", iStringSet);
 
-  vertex_name = strings->Request ("vertices");
-  color_name = strings->Request ("colors");
-  texcoord_name = strings->Request ("texture coordinates");
-  index_name = strings->Request ("indices");
   radius_name = strings->Request ("point radius");
   scale_name = strings->Request ("point scale");
 
@@ -198,6 +200,7 @@ csParticlesObject::csParticlesObject (csParticlesFactory* p)
   force_type = p->force_type;
 
   force_direction = p->force_direction;
+  force_direction_variation = p->force_direction_variation;
   force_range = p->force_range;
   force_falloff = p->force_falloff;
   force_cone_radius = p->force_cone_radius;
@@ -234,7 +237,7 @@ csParticlesObject::csParticlesObject (csParticlesFactory* p)
 
   transform_mode = p->transform_mode;
 
-  point_data = NULL; // This gets created by the physics plugin
+  point_data = 0; // This gets created by the physics plugin
 
   emitter = csVector3(0.0f, 0.0f, 0.0f);
   radius = 1.0f;
@@ -243,22 +246,22 @@ csParticlesObject::csParticlesObject (csParticlesFactory* p)
 
   running = false;
 
-  svcontext = new csShaderVariableContext ();
+  svcontext.AttachNew (new csShaderVariableContext);
+  bufferHolder.AttachNew (new csRenderBufferHolder);
+
+  scfiRenderBufferAccessor.AttachNew (new eiRenderBufferAccessor (this));
+
+  bufferHolder->SetAccessor (scfiRenderBufferAccessor, (uint32)~0);
 
   csShaderVariable* sv;
-  sv = svcontext->GetVariableAdd (vertex_name);
-  sv->SetAccessor (&scfiShaderVariableAccessor);
-  sv = svcontext->GetVariableAdd (color_name);
-  sv->SetAccessor (&scfiShaderVariableAccessor);
-  sv = svcontext->GetVariableAdd (texcoord_name);
-  sv->SetAccessor (&scfiShaderVariableAccessor);
-  sv = svcontext->GetVariableAdd (index_name);
-  sv->SetAccessor (&scfiShaderVariableAccessor);
   sv = svcontext->GetVariableAdd (radius_name);
   sv->SetValue (particle_radius);
   sv = svcontext->GetVariableAdd (scale_name);
   sv->SetValue (1.0f);
 
+  mixmode = p->mixmode;
+  zsort_enabled = p->zsort_enabled;
+
   LoadPhysicsPlugin (p->physics_plugin);
 
   if(autostart) Start();
@@ -266,15 +269,15 @@ csParticlesObject::csParticlesObject (csParticlesFactory* p)
 
 csParticlesObject::~csParticlesObject ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiShaderVariableAccessor)
+  if (physics)
+    physics->RemoveParticles (&scfiParticlesObjectState);
+
   SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel)
   SCF_DESTRUCT_EMBEDDED_IBASE (scfiParticlesObjectState)
   SCF_DESTRUCT_IBASE ()
   
-  if (meshpp)
-    delete [] meshpp;
-  if (mesh)
-    delete mesh;
+  delete [] meshpp;
+  delete mesh;
 }
 
 csPtr<iMeshObject> csParticlesObject::Clone ()
@@ -312,6 +315,7 @@ csPtr<iMeshObject> csParticlesObject::Clone ()
   new_obj->force_type = force_type;
 
   new_obj->force_direction = force_direction;
+  new_obj->force_direction_variation = force_direction_variation;
   new_obj->force_range = force_range;
   new_obj->force_falloff = force_falloff;
   new_obj->force_cone_radius = force_cone_radius;
@@ -358,6 +362,8 @@ csPtr<iMeshObject> csParticlesObject::Clone ()
   new_obj->radius = radius;
 
   new_obj->flags = flags;
+  new_obj->mixmode = mixmode;
+  new_obj->zsort_enabled = zsort_enabled;
   
   return csPtr<iMeshObject> (new_obj);
 }
@@ -388,6 +394,17 @@ bool csParticlesObject::LoadPhysicsPlugin (const char *plugin_id)
   return true;
 }
 
+void csParticlesObject::EnableZSort (bool en)
+{
+  if (zsort_enabled == en) return;
+  zsort_enabled = en;
+  if (physics)
+  {
+    physics->RemoveParticles (&scfiParticlesObjectState);
+    physics->RegisterParticles (&scfiParticlesObjectState);
+  }
+}
+
 void csParticlesObject::SetParticleRadius (float rad)
 {
   particle_radius = rad;
@@ -399,71 +416,51 @@ void csParticlesObject::SetParticleRadius (float rad)
   }
 }
 
-bool csParticlesObject::DrawTest (iRenderView*, iMovable*,
-	uint32)
+void csParticlesObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer)
 {
-  return true;
+  if (!holder) return;
+  holder->SetRenderBuffer (buffer, GetRenderBuffer(buffer));
 }
 
-void csParticlesObject::PreGetShaderVariableValue (csShaderVariable* var)
-{
-  var->SetValue(GetRenderBuffer(var->Name));
-}
-
-iRenderBuffer *csParticlesObject::GetRenderBuffer (csStringID name)
+iRenderBuffer *csParticlesObject::GetRenderBuffer (csRenderBufferName name)
 {
   if (!vertex_buffer || buffer_length != point_data->Length ())
   {
     buffer_length = point_data->Length ();
-    csArray<iRenderBuffer*> buffers;
-    int bufsize = (point_sprites ? buffer_length : buffer_length * 4);
-    pFactory->g3d->CreateInterleavedRenderBuffers (bufsize
-      * sizeof(csParticlesData), CS_BUF_DYNAMIC, 2, buffers);
-    vertex_buffer = buffers.Get(0);
-    vertex_buffer->SetOffset (0);
-    vertex_buffer->SetComponentType (CS_BUFCOMP_FLOAT);
-    vertex_buffer->SetComponentCount (3);
-    color_buffer = buffers.Get (1);
-    color_buffer->SetOffset (sizeof(csVector3));
-    color_buffer->SetComponentType (CS_BUFCOMP_FLOAT);
-    color_buffer->SetComponentCount (4);
-    texcoord_buffer = pFactory->g3d->CreateRenderBuffer (
-      sizeof (csVector2) * bufsize, CS_BUF_DYNAMIC,
-      CS_BUFCOMP_FLOAT, 2, false);
+    static const csInterleavedSubBufferOptions interleavedElements[2] =
+      {{CS_BUFCOMP_FLOAT, 3}, {CS_BUFCOMP_FLOAT, 4}};
+    csRef<iRenderBuffer> buffers[2];
+    int bufsize = (int)(point_sprites ? buffer_length : buffer_length * 4);
+    masterBuffer = csRenderBuffer::CreateInterleavedRenderBuffers (bufsize, 
+      CS_BUF_DYNAMIC, 2, interleavedElements, buffers);
+    vertex_buffer = buffers[0];
+    color_buffer = buffers[1];
+
+    texcoord_buffer = csRenderBuffer::CreateRenderBuffer (
+      bufsize, CS_BUF_DYNAMIC,
+      CS_BUFCOMP_FLOAT, 2);
     csVector2 *texcoords = new csVector2 [bufsize];
+    unsigned int *indices;
+    size_t indices_size;
     if (point_sprites)
     {
-      vertex_buffer->SetStride (sizeof(i_vertex));
-      color_buffer->SetStride (sizeof(i_vertex));
-
-      unsigned int *indices = new unsigned int[buffer_length];
-      for (int i = 0; i < buffer_length; i++)
-      {
-        indices[i] = i;
-      }
-      index_buffer = pFactory->g3d->CreateRenderBuffer (
-        sizeof (unsigned int) * buffer_length, CS_BUF_STATIC,
-        CS_BUFCOMP_UNSIGNED_INT, 1, true);
-      index_buffer->CopyToBuffer(indices, sizeof(unsigned int)*buffer_length);
-      delete [] indices;
+      indices_size = buffer_length;
+      indices = new unsigned int[indices_size];
+      for (size_t i = 0; i < buffer_length; i++)
+        indices[i] = (uint)i;
     }
     else
     {
-      vertex_buffer->SetStride (sizeof(i_vertex));
-      color_buffer->SetStride (sizeof(i_vertex));
+      indices_size = buffer_length * 6;
       int i;
       for (i = 0; i < bufsize - 4; i += 4)
       {
-        texcoords[i].x = 0;
-        texcoords[i].y = 0;
-        texcoords[i+1].x = 0;
-        texcoords[i+1].y = 1;
-        texcoords[i+2].x = 1;
-        texcoords[i+2].y = 1;
-        texcoords[i+3].x = 1;
-        texcoords[i+3].y = 0;
+        texcoords[i].x = 0; texcoords[i].y = 0;
+        texcoords[i+1].x = 0; texcoords[i+1].y = 1;
+        texcoords[i+2].x = 1; texcoords[i+2].y = 1;
+        texcoords[i+3].x = 1; texcoords[i+3].y = 0;
       }
-      unsigned int *indices = new unsigned int[buffer_length * 6];
+      indices = new unsigned int[indices_size];
       int j;
       for (i = 0, j = 0; i < bufsize-4; i += 4, j += 6)
       {
@@ -476,23 +473,21 @@ iRenderBuffer *csParticlesObject::GetRenderBuffer (csStringID name)
         indices[j+4] = i + 2;
         indices[j+5] = i + 3;
       }
-      index_buffer = pFactory->g3d->CreateRenderBuffer (
-        sizeof (unsigned int) * buffer_length * 6, CS_BUF_STATIC,
-        CS_BUFCOMP_UNSIGNED_INT, 1, true);
-      index_buffer->CopyToBuffer(
-        indices, sizeof(unsigned int) * buffer_length * 6);
-      delete [] indices;
     }
-    texcoord_buffer->CopyToBuffer (texcoords, sizeof(csVector2) * bufsize);
+    index_buffer = csRenderBuffer::CreateIndexRenderBuffer (
+        indices_size, CS_BUF_STATIC,
+        CS_BUFCOMP_UNSIGNED_INT, 0, bufsize - 1);
+    index_buffer->CopyInto (indices, indices_size);
+    delete [] indices;
+
+    texcoord_buffer->CopyInto (texcoords, bufsize);
     delete [] texcoords;
   }
-  if (name == vertex_name)
+  if (name == CS_BUFFER_POSITION)
   {
-    void *data;
-    int size;
     if (point_sprites)
     {
-      int len = point_data->Length ();
+      int len = (int)point_data->Length ();
       vertex_data.SetLength (len);
       for (int i = 0; i < len - 1; i++)
       {
@@ -502,13 +497,10 @@ iRenderBuffer *csParticlesObject::GetRenderBuffer (csStringID name)
         vertex.color = point.color;
         vertex_data.Put (i, vertex);
       }
-
-      data = vertex_data.GetArray ();
-      size = vertex_data.Length () * sizeof(i_vertex);
     }
     else
     {
-      int len = point_data->Length ();
+      int len = (int)point_data->Length ();
       vertex_data.SetLength (len * 4);
       int i,j;
       for (i = 0, j = 0; i < len - 1; i++, j += 4)
@@ -528,23 +520,20 @@ iRenderBuffer *csParticlesObject::GetRenderBuffer (csStringID name)
         vertex.color = point.color;
         vertex_data.Put (j+3, vertex);
       }
-
-      data = vertex_data.GetArray ();
-      size = vertex_data.Length () * sizeof(i_vertex);
     }
     // Vertex buffer is an interleaved buffer, so copy all the data into it
-    vertex_buffer->CopyToBuffer (data, size);
+    masterBuffer->CopyInto (vertex_data.GetArray (), vertex_data.Length ());
     return vertex_buffer;
   }
-  else if (name == color_name)
+  else if (name == CS_BUFFER_COLOR)
   {
     return color_buffer;
   }
-  else if (name == texcoord_name)
+  else if (name == CS_BUFFER_TEXCOORD0)
   {
     return texcoord_buffer;
   }
-  else if (name == index_name)
+  else if (name == CS_BUFFER_INDEX)
   {
     return index_buffer;
   }
@@ -556,28 +545,26 @@ csRenderMesh** csParticlesObject::GetRenderMeshes (int& n, iRenderView* rview,
 {
   iCamera* cam = rview->GetCamera ();
 
-  tr_o2c = cam->GetTransform ();
+  csReversibleTransform o2wt;
+
   if (!transform_mode)
   {
     emitter = movable->GetFullPosition();
-    csReversibleTransform trans;
     // @@@ The code below is not very efficient!
-    trans.Identity();
-    trans.SetOrigin (emitter);
-    tr_o2c /= trans;
+    o2wt.Identity();
+    o2wt.SetOrigin (emitter);
   }
   else
   {
-    if (!movable->IsFullTransformIdentity ())
-      tr_o2c /= movable->GetFullTransform ();
+    o2wt = movable->GetFullTransform ();
   }
   // @@@ GetTransform??? Shouldn't this be GetFullTransform?
-  rotation_matrix = movable->GetTransform ().GetT2O ();
+  rotation_matrix = movable->GetFullTransform ().GetT2O ();
 
   int vertnum = 0;
   float new_radius = 0.0f;
 
-  int i;
+  size_t i;
   for (i=0 ; i<point_data->Length () ; i++)
   {
     const csParticlesData &point = point_data->Get(i);
@@ -596,7 +583,12 @@ csRenderMesh** csParticlesObject::GetRenderMeshes (int& n, iRenderView* rview,
 
   if (vertnum>0) 
   {
-    radius = qsqrt (new_radius);
+    new_radius = csQsqrt (new_radius);
+    if (new_radius>radius)
+    {
+      radius = new_radius;
+      scfiObjectModel.ShapeChanged ();
+    }
     running = true;
   }
   else
@@ -608,7 +600,8 @@ csRenderMesh** csParticlesObject::GetRenderMeshes (int& n, iRenderView* rview,
   int clip_portal, clip_plane, clip_z_plane;
   rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
   	clip_z_plane);
-  csVector3 camera_origin = tr_o2c.GetT2OTranslation ();
+
+  tr_o2c = cam->GetTransform () / o2wt;
 
   if (!point_sprites)
   {
@@ -621,7 +614,7 @@ csRenderMesh** csParticlesObject::GetRenderMeshes (int& n, iRenderView* rview,
   }
   else
   {
-    int fov = QInt (cam->GetFOVAngle ());
+    int fov = csQint (cam->GetFOVAngle ());
     int fov_pixels = cam->GetFOV ();
     if (radius_changed || camera_fov != fov || camera_pixels != fov_pixels)
     {
@@ -637,20 +630,19 @@ csRenderMesh** csParticlesObject::GetRenderMeshes (int& n, iRenderView* rview,
   if (!mesh)
     mesh = new csRenderMesh;
 
-  mesh->z_buf_mode = CS_ZBUF_USE;
-  mesh->mixmode = CS_FX_ALPHA;
+  mesh->mixmode = mixmode;
   mesh->clip_plane = clip_plane;
   mesh->clip_portal = clip_portal;
   mesh->clip_z_plane = clip_z_plane;
   mesh->do_mirror = rview->GetCamera()->IsMirrored();
   matwrap->Visit ();
   mesh->material = matwrap;
-  mesh->object2camera = tr_o2c;
-  mesh->camera_origin = camera_origin;
+  mesh->worldspace_origin = o2wt.GetOrigin ();
   mesh->indexstart = 0;
   mesh->indexend = vertnum;
-  mesh->variablecontext = 
-    (iShaderVariableContext*)svcontext; // Cast for gcc 2.95.x.
+  mesh->variablecontext = svcontext;
+  mesh->object2world = o2wt;
+  mesh->buffers = bufferHolder;
   if (point_sprites)
     mesh->meshtype = CS_MESHTYPE_POINT_SPRITES;
   else
@@ -668,42 +660,29 @@ csRenderMesh** csParticlesObject::GetRenderMeshes (int& n, iRenderView* rview,
   return meshpp;
 }
 
-bool csParticlesObject::HitBeamOutline (const csVector3& start,
-	const csVector3& end, csVector3& isect, float* pr)
+bool csParticlesObject::HitBeamOutline (const csVector3& /*start*/,
+	const csVector3& /*end*/, csVector3& /*isect*/, float* /*pr*/)
 {
   return false;
 }
 
-bool csParticlesObject::HitBeamObject (const csVector3& start,
-	const csVector3& end, csVector3& isect, float* pr,
-	int* polygon_idx)
+bool csParticlesObject::HitBeamObject (const csVector3& /*start*/,
+	const csVector3& /*end*/, csVector3& /*isect*/, float* /*pr*/,
+	int* polygon_idx, iMaterialWrapper**)
 {
   if (polygon_idx) *polygon_idx = -1;
   return false;
 }
 
-void csParticlesObject::GetObjectBoundingBox (csBox3& bbox, int type)
+void csParticlesObject::GetObjectBoundingBox (csBox3& bbox)
 {
-  bbox.StartBoundingBox ();
-  switch (type)
-  {
-  case CS_BBOX_NORMAL:
-    bbox.AddBoundingVertex (-radius, -radius, -radius);
-    bbox.AddBoundingVertex (radius, radius, radius);
-    break;
-  case CS_BBOX_ACCURATE:
-    // TODO: Vertices, not just points (and radius)
-    //for (int i = 0; i < max_particles; i++)
-    //  bbox.AddBoundingVertex (positions[i]);
-    bbox.AddBoundingVertex (-radius, -radius, -radius);
-    bbox.AddBoundingVertex (radius, radius, radius);
-    break;
-  case CS_BBOX_MAX:
-    // TODO: Come up with a better estimated maximum
-    bbox.AddBoundingVertex (-100000.0f, -100000.0f, -100000.0f);
-    bbox.AddBoundingVertex (100000.0f, 100000.0f, 100000.0f);
-    break;
-  }
+  bbox.Set (csVector3 (-radius, -radius, -radius),
+  	csVector3 (radius, radius, radius));
+}
+
+void csParticlesObject::SetObjectBoundingBox (const csBox3&)
+{
+  // @@@ TODO
 }
 
 void csParticlesObject::GetRadius(csVector3 &rad, csVector3 &c)
