@@ -19,26 +19,26 @@
 #ifndef __CS_PARTICLES_H__
 #define __CS_PARTICLES_H__
 
-#include "csutil/array.h"
-#include "csutil/cscolor.h"
-#include "csutil/garray.h"
-#include "csutil/weakref.h"
-
 #include "csgeom/objmodel.h"
 #include "csgeom/transfrm.h"
+#include "csgfx/shadervar.h"
+#include "csgfx/shadervarcontext.h"
+#include "csutil/array.h"
+#include "csutil/cscolor.h"
+#include "csutil/dirtyaccessarray.h"
+#include "csutil/flags.h"
+#include "csutil/leakguard.h"
+#include "csutil/ref.h"
+#include "csutil/weakref.h"
 
 #include "iengine/material.h"
-
 #include "imesh/object.h"
 #include "imesh/particles.h"
-
 #include "iutil/comp.h"
-
 #include "ivideo/graph3d.h"
 #include "ivideo/rendermesh.h"
 
-#include "csgfx/shadervar.h"
-#include "csgfx/shadervarcontext.h"
+
 
 /**
  * Particles type, instantiates factories which create meshes
@@ -75,11 +75,12 @@ class csParticlesFactory : public iMeshObjectFactory
 {
   friend class csParticlesObject;
 private:
-  iBase* parent;
+  iMeshFactoryWrapper* logparent;
+  csParticlesType* particles_type;
   iObjectRegistry *object_reg;
 
-  csRef<iGraphics3D> g3d;
-  csRef<iShaderManager> shmgr;
+  csWeakRef<iGraphics3D> g3d;
+  csWeakRef<iShaderManager> shmgr;
   csRef<iMaterialWrapper> material;
 
   csParticleEmitType emit_type;
@@ -90,6 +91,7 @@ private:
   csParticleForceType force_type;
 
   csVector3 force_direction;
+  csVector3 force_direction_variation;
   float force_range;
   csParticleFalloffType force_falloff;
   float force_cone_radius;
@@ -117,17 +119,23 @@ private:
   float particle_radius;
 
   csString physics_plugin;
+  bool physics_zsort;
 
-  csArray<csColor> gradient_colors;
+  csArray<csColor4> gradient_colors;
 
   float loop_time;
   float base_heat;
-  csColor constant_color;
+  csColor4 constant_color;
   csParticleColorMethod color_method;
-  csWeakRef<iParticlesColorCallback> color_callback;
+  csRef<iParticlesColorCallback> color_callback;
   csFlags flags;
 
+  uint mixmode;
+  bool zsort_enabled;
+
 public:
+  CS_LEAKGUARD_DECLARE (csParticlesFactory);
+
   SCF_DECLARE_IBASE;
 
   csParticlesFactory (csParticlesType* p, iObjectRegistry* objreg);
@@ -138,8 +146,15 @@ public:
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) {}
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { parent = lp; }
-  virtual iBase* GetLogicalParent () const { return parent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  {
+    logparent = lp;
+  }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  {
+    return logparent;
+  }
+  virtual iMeshObjectType* GetMeshObjectType () const { return particles_type; }
   virtual iObjectModel* GetObjectModel () { return 0; }
 
   void SetMaterial (iMaterialWrapper *mat)
@@ -185,20 +200,24 @@ public:
     force_range = range;
     force_falloff = falloff;
   }
-  void SetLinearForceType (const csVector3 &direction, float range,
-    csParticleFalloffType falloff)
+  void SetLinearForceType (const csVector3 &direction,
+  	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff)
   {
     force_type = CS_PART_FORCE_LINEAR;
     force_direction = direction;
+    force_direction_variation = direction_variation;
     force_range = range;
     force_falloff = falloff;
   }
-  void SetConeForceType (const csVector3 &direction, float range,
-    csParticleFalloffType falloff, float radius,
-    csParticleFalloffType radius_falloff)
+  void SetConeForceType (const csVector3 &direction,
+  	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff, float radius,
+	csParticleFalloffType radius_falloff)
   {
     force_type = CS_PART_FORCE_CONE;
     force_direction = direction;
+    force_direction_variation = direction_variation;
     force_range = range;
     force_falloff = falloff;
     force_cone_radius = radius;
@@ -217,13 +236,13 @@ public:
   void SetTimeVariation (float variation)
   { time_variation = variation; }
 
-  void AddColor (csColor color)
+  void AddColor (csColor4 color)
   {
     gradient_colors.Push(color);
   }
   void ClearColors ()
   { gradient_colors.DeleteAll (); }
-  void SetConstantColorMethod (csColor color)
+  void SetConstantColorMethod (csColor4 color)
   {
     color_method = CS_PART_COLOR_CONSTANT;
     constant_color = color;
@@ -276,6 +295,8 @@ public:
   { return force_range; }
   void GetForceDirection (csVector3 &dir)
   { dir = force_direction; }
+  void GetForceDirectionVariation (csVector3 &dirvar)
+  { dirvar = force_direction_variation; }
   float GetForceConeRadius ()
   { return force_cone_radius; }
   float GetForce ()
@@ -294,13 +315,13 @@ public:
   { return particle_radius; }
   csParticleColorMethod GetParticleColorMethod ()
   { return color_method; }
-  csColor GetConstantColor ()
-  { return constant_color; }
+  void GetConstantColor (csColor4& color)
+  { color = constant_color; }
   float GetColorLoopTime ()
   { return loop_time; }
   float GetBaseHeat ()
   { return base_heat; }
-  const csArray<csColor> &GetGradient ()
+  const csArray<csColor4> &GetGradient ()
   { return gradient_colors; }
   void SetDampener (float damp)
   { dampener = damp; }
@@ -322,6 +343,12 @@ public:
   { return transform_mode; }
   void SetPhysicsPlugin (const char *plugin)
   { physics_plugin = plugin; }
+  void SetMixMode (uint mode)
+  { mixmode = mode; }
+  uint GetMixMode () const
+  { return mixmode; }
+  void EnableZSort (bool en) { zsort_enabled = en; }
+  bool IsZSortEnabled () const { return zsort_enabled; }
 
   struct eiParticlesFactoryState : public iParticlesFactoryState
   {
@@ -344,14 +371,16 @@ public:
     {scfParent->SetCylinderEmitType (radius, height); }
     virtual void SetRadialForceType(float range, csParticleFalloffType falloff)
     { scfParent->SetRadialForceType(range, falloff); }
-    virtual void SetLinearForceType(const csVector3 &direction, float range,
-      csParticleFalloffType falloff)
-    { scfParent->SetLinearForceType(direction, range, falloff); }
-    virtual void SetConeForceType (const csVector3 &direction, float range,
-      csParticleFalloffType falloff, float radius,
-      csParticleFalloffType radius_falloff)
-    { scfParent->SetConeForceType (direction, range, falloff, radius,
-        radius_falloff); }
+    virtual void SetLinearForceType(const csVector3 &direction,
+    	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff)
+    { scfParent->SetLinearForceType(direction, direction_variation, range, falloff); }
+    virtual void SetConeForceType (const csVector3 &direction,
+    	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff, float radius,
+	csParticleFalloffType radius_falloff)
+    { scfParent->SetConeForceType (direction, direction_variation, range,
+    	falloff, radius, radius_falloff); }
     virtual void SetForce (float force)
     { scfParent->SetForce (force); }
     virtual void SetDiffusion (float size)
@@ -364,11 +393,11 @@ public:
     { scfParent->SetTimeToLive (time); }
     virtual void SetTimeVariation (float variation)
     { scfParent->SetTimeVariation (variation); }
-    virtual void AddColor (csColor color)
+    virtual void AddColor (csColor4 color)
     { scfParent->AddColor (color); }
     virtual void ClearColors ()
     { scfParent->ClearColors (); }
-    virtual void SetConstantColorMethod (csColor color)
+    virtual void SetConstantColorMethod (csColor4 color)
     { scfParent->SetConstantColorMethod (color); }
     virtual void SetLinearColorMethod ()
     { scfParent->SetLinearColorMethod (); }
@@ -384,11 +413,11 @@ public:
     { scfParent->SetParticleRadius (radius); }
     virtual csParticleColorMethod GetParticleColorMethod ()
     { return scfParent->GetParticleColorMethod (); }
-    virtual csColor GetConstantColor ()
-    { return scfParent->GetConstantColor (); }
+    virtual void GetConstantColor (csColor4& color )
+    { scfParent->GetConstantColor (color); }
     virtual float GetColorLoopTime ()
     { return scfParent->GetColorLoopTime (); }
-    virtual const csArray<csColor> &GetGradient ()
+    virtual const csArray<csColor4> &GetGradient ()
     { return scfParent->GetGradient (); }
     virtual float GetBaseHeat ()
     { return scfParent->GetBaseHeat (); }
@@ -417,6 +446,8 @@ public:
     { return scfParent->GetForceRange (); }
     virtual void GetForceDirection (csVector3 &dir)
     { scfParent->GetForceDirection (dir); }
+    virtual void GetForceDirectionVariation (csVector3 &dir)
+    { scfParent->GetForceDirectionVariation (dir); }
     virtual float GetForceConeRadius ()
     { return scfParent->GetForceConeRadius (); }
     virtual float GetForce ()
@@ -453,6 +484,14 @@ public:
     { return scfParent->GetMassVariation (); }
     virtual void SetPhysicsPlugin (const char *plugin)
     { scfParent->SetPhysicsPlugin (plugin); }
+    virtual void SetMixMode (uint mode)
+    { scfParent->SetMixMode (mode); }
+    virtual uint GetMixMode () const
+    { return scfParent->GetMixMode(); }
+    virtual void EnableZSort (bool en)
+    { scfParent->EnableZSort (en); }
+    virtual bool IsZSortEnabled () const
+    { return scfParent->IsZSortEnabled (); }
   } scfiParticlesFactoryState;
   friend struct eiParticlesFactoryState;
 };
@@ -464,10 +503,11 @@ public:
 class csParticlesObject : public iMeshObject
 {
 private:
-  iBase* logparent;
+  iMeshWrapper* logparent;
   csParticlesFactory* pFactory;
   iMeshObjectDrawCallback* vis_cb;
   csRef<csShaderVariableContext> svcontext;
+  csRef<csRenderBufferHolder> bufferHolder;
   csRef<iParticlesPhysics> physics;
 
   csRef<iMaterialWrapper> matwrap;
@@ -499,6 +539,7 @@ private:
   csParticleForceType force_type;
 
   csVector3 force_direction;
+  csVector3 force_direction_variation;
   float force_range;
   csParticleFalloffType force_falloff;
   float force_cone_radius;
@@ -528,12 +569,12 @@ private:
   float particle_radius;
   bool radius_changed;
 
-  csArray<csColor> gradient_colors;
+  csArray<csColor4> gradient_colors;
   float loop_time;
   float base_heat;
-  csColor constant_color;
+  csColor4 constant_color;
   csParticleColorMethod color_method;
-  csWeakRef<iParticlesColorCallback> color_callback;
+  csRef<iParticlesColorCallback> color_callback;
 
   const csArray<csParticlesData> *point_data;
   struct i_vertex
@@ -543,10 +584,11 @@ private:
   };
   csDirtyAccessArray<i_vertex> vertex_data;
 
-  int buffer_length;
+  size_t buffer_length;
 
   bool point_sprites;
 
+  csRef<iRenderBuffer> masterBuffer;
   csRef<iRenderBuffer> vertex_buffer;
   csRef<iRenderBuffer> color_buffer;
   csRef<iRenderBuffer> texcoord_buffer;
@@ -560,7 +602,12 @@ private:
 
   csFlags flags;
 
+  uint mixmode;
+  bool zsort_enabled;
+
 public:
+  CS_LEAKGUARD_DECLARE (csParticlesObject);
+
   SCF_DECLARE_IBASE;
 
   csParticlesObject (csParticlesFactory* f);
@@ -573,18 +620,9 @@ public:
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone ();
 
-  /**
-   * Does all pre-render calculation.  Determines which LOD children in the
-   * tree should be drawn
-   */
-  bool DrawTest (iRenderView* rview, iMovable* movable, uint32 frustum_mask);
-
   /// Updates the lighting
   void UpdateLighting (iLight** lights, int num_lights, iMovable* movable);
 
-  virtual bool Draw (iRenderView*, iMovable*, csZBufMode)
-  { /* deprecated */ return false; }
-
   /// Returns the mesh, ready for rendering
   csRenderMesh** GetRenderMeshes (int& n, iRenderView* rview,
     iMovable* movable, uint32 frustum_mask);
@@ -606,11 +644,12 @@ public:
 	csVector3& isect, float* pr);
   /// Find exact position of a beam hit
   bool HitBeamObject (const csVector3& start, const csVector3& end,
-	csVector3& isect, float* pr, int* polygon_idx = 0);
+	csVector3& isect, float* pr, int* polygon_idx = 0,
+	iMaterialWrapper** material = 0);
 
   /// Set/Get logical parent
-  void SetLogicalParent (iBase* lp) { logparent = lp; }
-  iBase* GetLogicalParent () const { return logparent; }
+  void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
+  iMeshWrapper* GetMeshWrapper () const { return logparent; }
 
   /// Gets the object model
   iObjectModel *GetObjectModel () { return &scfiObjectModel; }
@@ -620,7 +659,7 @@ public:
   /// Get the constant base color
   bool GetColor (csColor &c) const { c = basecolor; return true; }
 
-  iRenderBuffer *GetRenderBuffer (csStringID name);
+  iRenderBuffer *GetRenderBuffer (csRenderBufferName name);
 
   /// Set the material wrapper
   bool SetMaterialWrapper (iMaterialWrapper* m)
@@ -629,7 +668,8 @@ public:
   iMaterialWrapper* GetMaterialWrapper () const { return matwrap; }
   void InvalidateMaterialHandles () {}
 
-  void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL);
+  void GetObjectBoundingBox (csBox3& bbox);
+  void SetObjectBoundingBox (const csBox3& bbox);
   void GetRadius (csVector3& rad, csVector3& c);
 
   bool LoadPhysicsPlugin (const char *plugin_id);
@@ -675,20 +715,24 @@ public:
     force_range = range;
     force_falloff = falloff;
   }
-  void SetLinearForceType (const csVector3 &direction, float range,
-    csParticleFalloffType falloff)
+  void SetLinearForceType (const csVector3 &direction,
+  	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff)
   {
     force_type = CS_PART_FORCE_LINEAR;
     force_direction = direction;
+    force_direction_variation = direction_variation;
     force_range = range;
     force_falloff = falloff;
   }
-  void SetConeForceType (const csVector3 &direction, float range,
-    csParticleFalloffType falloff, float radius,
-    csParticleFalloffType radius_falloff)
+  void SetConeForceType (const csVector3 &direction,
+  	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff, float radius,
+	csParticleFalloffType radius_falloff)
   {
     force_type = CS_PART_FORCE_CONE;
     force_direction = direction;
+    force_direction_variation = direction_variation;
     force_range = range;
     force_falloff = falloff;
     force_cone_radius = radius;
@@ -706,7 +750,7 @@ public:
   { time_to_live = time; }
   void SetTimeVariation (float variation)
   { time_variation = variation; }
-  void AddColor (csColor color)
+  void AddColor (csColor4 color)
   {
     gradient_colors.Push(color);
   }
@@ -714,7 +758,7 @@ public:
   {
     gradient_colors.DeleteAll ();
   }
-  void SetConstantColorMethod (csColor color)
+  void SetConstantColorMethod (csColor4 color)
   {
     color_method = CS_PART_COLOR_CONSTANT;
     constant_color = color;
@@ -770,6 +814,8 @@ public:
   { return force_range; }
   void GetForceDirection (csVector3 &dir)
   { dir = force_direction; }
+  void GetForceDirectionVariation (csVector3 &dirvar)
+  { dirvar = force_direction_variation; }
   float GetForceConeRadius ()
   { return force_cone_radius; }
   float GetForce ()
@@ -800,13 +846,13 @@ public:
   { return particle_mass; }
   csParticleColorMethod GetParticleColorMethod ()
   { return color_method; }
-  csColor GetConstantColor ()
-  { return constant_color; }
+  void GetConstantColor (csColor4& color)
+  { color = constant_color; }
   float GetColorLoopTime ()
   { return loop_time; }
   float GetBaseHeat ()
   { return base_heat; }
-  const csArray<csColor> &GetGradient ()
+  const csArray<csColor4> &GetGradient ()
   { return gradient_colors; }
   void SetTransformMode (bool transform)
   { transform_mode = transform; }
@@ -821,8 +867,14 @@ public:
   void Stop ();
   bool IsRunning ()
   { return running; }
+  void SetMixMode (uint mode)
+  { mixmode = mode; }
+  uint GetMixMode () const
+  { return mixmode; }
+  void EnableZSort (bool en);
+  bool IsZSortEnabled () const { return zsort_enabled; }
 
-  virtual void PositionChild (iMeshObject* child,csTicks current_time) {}
+  virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) {}
 
   struct eiParticlesObjectState : public iParticlesObjectState
   {
@@ -843,14 +895,16 @@ public:
     {scfParent->SetCylinderEmitType (radius, height); }
     virtual void SetRadialForceType(float range, csParticleFalloffType falloff)
     { scfParent->SetRadialForceType(range, falloff); }
-    virtual void SetLinearForceType(const csVector3 &direction, float range,
-      csParticleFalloffType falloff)
-    { scfParent->SetLinearForceType(direction, range, falloff); }
-    virtual void SetConeForceType (const csVector3 &direction, float range,
-      csParticleFalloffType falloff, float radius,
-      csParticleFalloffType radius_falloff)
-    { scfParent->SetConeForceType (direction, range, falloff, radius,
-        radius_falloff); }
+    virtual void SetLinearForceType(const csVector3 &direction,
+    	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff)
+    { scfParent->SetLinearForceType(direction, direction_variation, range, falloff); }
+    virtual void SetConeForceType (const csVector3 &direction,
+    	const csVector3& direction_variation, float range,
+	csParticleFalloffType falloff, float radius,
+	csParticleFalloffType radius_falloff)
+    { scfParent->SetConeForceType (direction, direction_variation, range,
+    	falloff, radius, radius_falloff); }
     virtual void SetForce (float force)
     { scfParent->SetForce (force); }
     virtual void SetDiffusion (float size)
@@ -863,11 +917,11 @@ public:
     { scfParent->SetTimeToLive (time); }
     virtual void SetTimeVariation (float variation)
     { scfParent->SetTimeVariation (variation); }
-    virtual void AddColor (csColor color)
+    virtual void AddColor (csColor4 color)
     { scfParent->AddColor (color); }
     virtual void ClearColors ()
     { scfParent->ClearColors (); }
-    virtual void SetConstantColorMethod (csColor color)
+    virtual void SetConstantColorMethod (csColor4 color)
     { scfParent->SetConstantColorMethod (color); }
     virtual void SetLinearColorMethod ()
     { scfParent->SetLinearColorMethod (); }
@@ -881,11 +935,11 @@ public:
     { return scfParent->GetColorCallback (); }
     virtual csParticleColorMethod GetParticleColorMethod ()
     { return scfParent->GetParticleColorMethod (); }
-    virtual csColor GetConstantColor ()
-    { return scfParent->GetConstantColor (); }
+    virtual void GetConstantColor (csColor4& color)
+    { scfParent->GetConstantColor (color); }
     virtual float GetColorLoopTime ()
     { return scfParent->GetColorLoopTime (); }
-    virtual const csArray<csColor> &GetGradient ()
+    virtual const csArray<csColor4> &GetGradient ()
     { return scfParent->GetGradient (); }
     virtual float GetBaseHeat ()
     { return scfParent->GetBaseHeat (); }
@@ -918,6 +972,8 @@ public:
     { return scfParent->GetForceRange (); }
     virtual void GetForceDirection (csVector3 &dir)
     { scfParent->GetForceDirection (dir); }
+    virtual void GetForceDirectionVariation (csVector3 &dir)
+    { scfParent->GetForceDirectionVariation (dir); }
     virtual float GetForceConeRadius ()
     { return scfParent->GetForceConeRadius (); }
     virtual float GetForce ()
@@ -962,32 +1018,55 @@ public:
     { scfParent->Stop (); }
     virtual bool IsRunning ()
     { return scfParent->IsRunning (); }
-   
+    virtual void SetMixMode (uint mode)
+    { scfParent->SetMixMode (mode); }
+    virtual uint GetMixMode () const
+    { return scfParent->GetMixMode(); }
+    virtual void EnableZSort (bool en)
+    { scfParent->EnableZSort (en); }
+    virtual bool IsZSortEnabled () const
+    { return scfParent->IsZSortEnabled (); }
   } scfiParticlesObjectState;
   friend struct eiParticlesObjectState;
 
   struct eiObjectModel : public csObjectModel
   {
     SCF_DECLARE_EMBEDDED_IBASE (csParticlesObject);
-    void GetObjectBoundingBox (csBox3& b, int t = CS_BBOX_NORMAL)
-    { scfParent->GetObjectBoundingBox (b, t); }
-    void GetRadius (csVector3& r, csVector3& c)
+    virtual void GetObjectBoundingBox (csBox3& b)
+    { scfParent->GetObjectBoundingBox (b); }
+    virtual void SetObjectBoundingBox (const csBox3& b)
+    { scfParent->SetObjectBoundingBox (b); }
+    virtual void GetRadius (csVector3& r, csVector3& c)
     { scfParent->GetRadius (r, c); }
   } scfiObjectModel;
   friend struct eiObjectModel;
 
-  class eiShaderVariableAccessor : public iShaderVariableAccessor
+  class eiRenderBufferAccessor : public iRenderBufferAccessor
   {
+  private:
+    csParticlesObject* parent;
   public:
-    SCF_DECLARE_EMBEDDED_IBASE (csParticlesObject);
-    virtual void PreGetValue (csShaderVariable* variable)
+    CS_LEAKGUARD_DECLARE (eiRenderBufferAccessor);
+    SCF_DECLARE_IBASE;
+    eiRenderBufferAccessor (csParticlesObject* parent)
     {
-      scfParent->PreGetShaderVariableValue (variable);
+      SCF_CONSTRUCT_IBASE (0);
+      eiRenderBufferAccessor::parent = parent;
     }
-  } scfiShaderVariableAccessor;
-  friend class eiShaderVariableAccessor;
+    virtual ~eiRenderBufferAccessor ()
+    {
+      SCF_DESTRUCT_IBASE ();
+    }
+    virtual void PreGetBuffer (csRenderBufferHolder* holder,
+    	csRenderBufferName buffer)
+    {
+      parent->PreGetBuffer (holder, buffer);
+    }
+  };
+  csRef<eiRenderBufferAccessor> scfiRenderBufferAccessor;
+  friend class eiRenderBufferAccessor;
 
-  void PreGetShaderVariableValue (csShaderVariable* variable);
+  void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer);
 };
 
 #endif // __CS_PARTICLES_H__
