@@ -19,7 +19,7 @@
 #ifndef __CS_RAIN_H__
 #define __CS_RAIN_H__
 
-#include "plugins/mesh/partgen/particle.h"
+#include "csplugincommon/particlesys/particle.h"
 #include "imesh/rain.h"
 
 /**
@@ -57,7 +57,7 @@ public:
   void SetCollisionDetection (bool cd);
   bool GetCollisionDetection () const;
 
-  SCF_DECLARE_IBASE_EXT (csMeshObject);
+  SCF_DECLARE_IBASE_EXT (csNewParticleSystem);
 
   //------------------------- iRainState implementation ----------------
   class RainState : public iRainState
