@@ -23,7 +23,7 @@
 #include "csgeom/box.h"
 #include "csutil/cscolor.h"
 #include "csutil/refarr.h"
-#include "plugins/mesh/partgen/partgen.h"
+#include "csplugincommon/particlesys/partgen.h"
 #include "imesh/snow.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
@@ -72,7 +72,7 @@ public:
     initialized = false;
     drop_width = dropwidth;
     drop_height = dropheight;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get the size of the drops.
   void GetDropSize (float& dropwidth, float& dropheight) const
@@ -82,7 +82,7 @@ public:
   {
     initialized = false;
     rainbox.Set (minbox, maxbox);
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get box.
   void GetBox (csVector3& minbox, csVector3& maxbox) const
@@ -101,7 +101,7 @@ public:
   {
     initialized = false;
     rain_dir = fspeed;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get fall speed.
   const csVector3& GetFallSpeed () const
@@ -111,7 +111,7 @@ public:
   {
     initialized = false;
     swirl_amount = sw;
-    scfiObjectModel.ShapeChanged ();
+    ShapeChanged ();
   }
   /// Get swirl.
   float GetSwirl () const
@@ -155,7 +155,7 @@ public:
       scfParent->SetSwirl (sw);
     }
     virtual int GetParticleCount () const
-    { return scfParent->GetParticleCount (); }
+    { return (int)scfParent->GetParticleCount (); }
     virtual void GetDropSize (float& dropwidth, float& dropheight) const
     { scfParent->GetDropSize (dropwidth, dropheight); }
     virtual void GetBox (csVector3& minbox, csVector3& maxbox) const
@@ -177,12 +177,14 @@ class csSnowMeshObjectFactory : public iMeshObjectFactory
 {
 private:
   iObjectRegistry* object_reg;
-  iBase* logparent;
+  iMeshFactoryWrapper* logparent;
+  iMeshObjectType* snow_type;
   csFlags flags;
 
 public:
   /// Constructor.
-  csSnowMeshObjectFactory (iBase *pParent, iObjectRegistry* object_reg);
+  csSnowMeshObjectFactory (iMeshObjectType* pParent,
+  	iObjectRegistry* object_reg);
 
   /// Destructor.
   virtual ~csSnowMeshObjectFactory ();
@@ -195,8 +197,11 @@ public:
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  { logparent = lp; }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  { return logparent; }
+  virtual iMeshObjectType* GetMeshObjectType () const { return snow_type; }
   virtual iObjectModel* GetObjectModel () { return 0; }
 };
 
