@@ -20,10 +20,10 @@
 #include "cssysdef.h"
 #include "csgeom/matrix3.h"
 #include "csgeom/transfrm.h"
-#include "spiral.h"
 #include "ivideo/material.h"
 #include "iengine/material.h"
-#include "qsqrt.h"
+#include "spiral.h"
+#include "csqsqrt.h"
 #include <math.h>
 #include <stdlib.h>
 
@@ -62,10 +62,10 @@ void csSpiralMeshObject::SetupObject ()
     if (size.y > max_size) max_size = size.y;
     if (size.z > max_size) max_size = size.z;
     float a = max_size/2.;
-    radius = qsqrt (a*a + a*a);
+    radius = csQsqrt (a*a + a*a);
 
     // create particles
-    int i;
+    size_t i;
     for (i=0 ; i<number ; i++)
     {
       RestartParticle(FindOldest(), (part_time / float(number)) * float(number-i));
@@ -122,17 +122,17 @@ void csSpiralMeshObject::RestartParticle (int index, float pre_move)
 
 int csSpiralMeshObject::FindOldest ()
 {
-  int num = GetNumParticles ();
+  int num = (int)GetNumParticles ();
   int part_idx;
-  if (num >= number)
+  if ((size_t)num >= number)
   {
     part_idx = last_reuse;
-    last_reuse = (last_reuse+1)%number;
+    last_reuse = (last_reuse+1)%(int)number;
   }
   else
   {
     AppendRectSprite (part_width, part_height, mat, false);	// @@@ PARAMETER
-    part_idx = GetNumParticles ()-1;
+    part_idx = (int)GetNumParticles ()-1;
     GetParticle(part_idx)->SetMixMode(MixMode);
   }
   return part_idx;
@@ -143,10 +143,10 @@ void csSpiralMeshObject::SetSource (const csVector3& source)
 {
   initialized = false;
   csSpiralMeshObject::source = source;
-  part_source.Set( qsqrt(source.x*source.x+source.z*source.z),
+  part_source.Set( csQsqrt(source.x*source.x+source.z*source.z),
                   source.y,
                   atan2(source.x,-source.z));
-  scfiObjectModel.ShapeChanged ();
+  ShapeChanged ();
 }
 
 void csSpiralMeshObject::Update (csTicks elapsed_time)
@@ -157,7 +157,7 @@ void csSpiralMeshObject::Update (csTicks elapsed_time)
 
   // Update position
   int i;
-  for (i=0 ; i < particles.Length () ; i++)
+  for (i=0 ; i < (int)particles.Length () ; i++)
   {
     part_pos[i] += part_speed* delta_t;
     SetPosition (i);
@@ -179,7 +179,7 @@ void csSpiralMeshObject::HardTransform (const csReversibleTransform& t)
 {
   source = t.This2Other (source);
   initialized = false;
-  scfiObjectModel.ShapeChanged ();
+  ShapeChanged ();
 }
 
 //----------------------------------------------------------------------
@@ -188,12 +188,13 @@ SCF_IMPLEMENT_IBASE (csSpiralMeshObjectFactory)
   SCF_IMPLEMENTS_INTERFACE (iMeshObjectFactory)
 SCF_IMPLEMENT_IBASE_END
 
-csSpiralMeshObjectFactory::csSpiralMeshObjectFactory (iBase *p,
+csSpiralMeshObjectFactory::csSpiralMeshObjectFactory (iMeshObjectType* p,
 	iObjectRegistry* s)
 {
   SCF_CONSTRUCT_IBASE (p);
   object_reg = s;
   logparent = 0;
+  spiral_type = p;
 }
 
 csSpiralMeshObjectFactory::~csSpiralMeshObjectFactory ()
