@@ -17,24 +17,34 @@
 */
 
 #include "cssysdef.h"
+#include "csqsqrt.h"
+
+#include "csgeom/box.h"
+#include "csgeom/math.h"
 #include "csgeom/math3d.h"
 #include "csgeom/math2d.h"
-#include "csgeom/box.h"
 #include "csgeom/transfrm.h"
+#include "csgfx/renderbuffer.h"
 #include "cstool/rbuflock.h"
-#include "spr2d.h"
+
 #include "iengine/movable.h"
 #include "iengine/rview.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/material.h"
+#include "ivideo/rendermesh.h"
 #include "iengine/material.h"
 #include "iengine/camera.h"
 #include "igeom/clip2d.h"
 #include "iengine/engine.h"
 #include "iengine/light.h"
 #include "iutil/objreg.h"
-#include "qsqrt.h"
+
+#include "spr2d.h"
+
+
+CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObject);
+CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObjectFactory);
 
 CS_IMPLEMENT_PLUGIN
 
@@ -57,17 +67,10 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::Particle)
   SCF_IMPLEMENTS_INTERFACE (iParticle)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-#ifdef CS_USE_NEW_RENDERER
-SCF_IMPLEMENT_IBASE (csSprite2DMeshObject::eiShaderVariableAccessor)
-  SCF_IMPLEMENTS_INTERFACE (iShaderVariableAccessor)
+SCF_IMPLEMENT_IBASE (csSprite2DMeshObject::eiRenderBufferAccessor)
+  SCF_IMPLEMENTS_INTERFACE (iRenderBufferAccessor)
 SCF_IMPLEMENT_IBASE_END
 
-csStringID csSprite2DMeshObject::vertex_name = csInvalidStringID;
-csStringID csSprite2DMeshObject::texel_name = csInvalidStringID;
-csStringID csSprite2DMeshObject::color_name = csInvalidStringID;
-csStringID csSprite2DMeshObject::index_name = csInvalidStringID;
-#endif
-
 csSprite2DMeshObject::csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory)
 {
   SCF_CONSTRUCT_IBASE (0);
@@ -85,12 +88,11 @@ csSprite2DMeshObject::csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory
   current_lod = 1;
   current_features = 0;
   uvani = 0;
-#ifdef CS_USE_NEW_RENDERER
+
   vertices_dirty = true;
   texels_dirty = true;
   colors_dirty = true;
   indicesSize = (size_t)-1;
-#endif
 }
 
 csSprite2DMeshObject::~csSprite2DMeshObject ()
@@ -109,7 +111,7 @@ void csSprite2DMeshObject::SetupObject ()
   {
     initialized = true;
     float max_sq_dist = 0;
-    int i;
+    size_t i;
     bbox_2d.StartBoundingBox(vertices[0].pos);
     for (i = 0 ; i < vertices.Length () ; i++)
     {
@@ -125,63 +127,21 @@ void csSprite2DMeshObject::SetupObject ()
       float sqdist = v.pos.x*v.pos.x + v.pos.y*v.pos.y;
       if (sqdist > max_sq_dist) max_sq_dist = sqdist;
     }
-    float max_dist = qsqrt (max_sq_dist);
+    float max_dist = csQsqrt (max_sq_dist);
     radius.Set (max_dist, max_dist, max_dist);
 
-#ifdef CS_USE_NEW_RENDERER
-    if ((vertex_name == csInvalidStringID) ||
-      (color_name == csInvalidStringID) ||
-      (texel_name == csInvalidStringID) ||
-      (index_name  == csInvalidStringID))
-    {
-      csRef<iStringSet> strings = 
-	CS_QUERY_REGISTRY_TAG_INTERFACE (factory->object_reg,
-	"crystalspace.shared.stringset", iStringSet);
-      vertex_name = strings->Request ("vertices");
-      texel_name = strings->Request ("texture coordinates");
-      color_name = strings->Request ("colors");
-      index_name = strings->Request ("indices");
-    }
-    
-    svcontext.AttachNew (new csShaderVariableContext ());
-    csRef<iShaderVariableAccessor> accessor;
-    accessor.AttachNew (new eiShaderVariableAccessor (this));
-    csShaderVariable* sv;
-    sv = svcontext->GetVariableAdd (index_name);
-    sv->SetAccessor (accessor);
-    sv = svcontext->GetVariableAdd (vertex_name);
-    sv->SetAccessor (accessor);
-    sv = svcontext->GetVariableAdd (texel_name);
-    sv->SetAccessor (accessor);
-    sv = svcontext->GetVariableAdd (color_name);
-    sv->SetAccessor (accessor);
-#endif
+    bufferHolder.AttachNew (new csRenderBufferHolder);
+    csRef<iRenderBufferAccessor> newAccessor;
+    newAccessor.AttachNew (new eiRenderBufferAccessor (this));
+    bufferHolder->SetAccessor (newAccessor, (uint32)CS_BUFFER_ALL_MASK);
+    svcontext.AttachNew (new csShaderVariableContext);
   }
 }
 
 static csVector3 cam;
 
-bool csSprite2DMeshObject::DrawTest (iRenderView* rview, iMovable* movable,
-	uint32)
-{
-  SetupObject ();
-
-  // Camera transformation for the single 'position' vector.
-  cam = rview->GetCamera ()->GetTransform ().Other2This (
-  	movable->GetFullPosition ());
-  if (cam.z < SMALL_Z) return false;
-
-  if (factory->light_mgr)
-  {
-    const csArray<iLight*>& relevant_lights = factory->light_mgr
-    	->GetRelevantLights (logparent, -1, false);
-    UpdateLighting (relevant_lights, movable, csVector3 (0.0f));
-  }
-
-  return true;
-}
-
-void csSprite2DMeshObject::UpdateLighting (const csArray<iLight*>& lights,
+void csSprite2DMeshObject::UpdateLighting (
+    const csArray<iLightSectorInfluence*>& lights,
     const csVector3& pos)
 {
   if (!lighting) return;
@@ -197,33 +157,35 @@ void csSprite2DMeshObject::UpdateLighting (const csArray<iLight*>& lights,
   //}
 
   int i;
-  int num_lights = lights.Length ();
+  int num_lights = (int)lights.Length ();
   for (i = 0; i < num_lights; i++)
   {
-    csColor light_color = lights[i]->GetColor () * (256. / CS_NORMAL_LIGHT_LEVEL);
-    float sq_light_radius = lights [i]->GetInfluenceRadiusSq ();
+    iLight* li = lights[i]->GetLight ();
+    csColor light_color = li->GetColor ()
+    	* (256. / CS_NORMAL_LIGHT_LEVEL);
+    float sq_light_radius = csSquare (li->GetCutoffDistance ());
     // Compute light position.
-    csVector3 wor_light_pos = lights [i]->GetCenter ();
+    csVector3 wor_light_pos = li->GetMovable ()->GetFullPosition ();
     float wor_sq_dist =
       csSquaredDist::PointPoint (wor_light_pos, pos);
     if (wor_sq_dist >= sq_light_radius) continue;
-    float wor_dist = qsqrt (wor_sq_dist);
+    float wor_dist = csQsqrt (wor_sq_dist);
     float cosinus = 1.;
     cosinus /= wor_dist;
-    light_color *= cosinus * lights [i]->GetBrightnessAtDistance (wor_dist);
+    light_color *= cosinus * li->GetBrightnessAtDistance (wor_dist);
     color += light_color;
   }
-  for (i = 0 ; i < vertices.Length () ; i++)
+  for (size_t j = 0 ; j < vertices.Length () ; j++)
   {
-    vertices[i].color = vertices[i].color_init + color;
-    vertices[i].color.Clamp (2, 2, 2);
+    vertices[j].color = vertices[j].color_init + color;
+    vertices[j].color.Clamp (2, 2, 2);
   }
-#ifdef CS_USE_NEW_RENDERER
   colors_dirty = true;
-#endif
+
 }
 
-void csSprite2DMeshObject::UpdateLighting (const csArray<iLight*>& lights,
+void csSprite2DMeshObject::UpdateLighting (
+    const csArray<iLightSectorInfluence*>& lights,
     iMovable* movable, csVector3 offset)
 {
   if (!lighting) return;
@@ -231,223 +193,12 @@ void csSprite2DMeshObject::UpdateLighting (const csArray<iLight*>& lights,
   UpdateLighting (lights, pos + offset);
 }
 
-#define INTERPOLATE1_S(var) \
-  g3dpoly->var [i]= inpoly_##var [vt]+ \
-    t * (inpoly_##var [vt2]- inpoly_##var [vt]);
-
-#define INTERPOLATE1(var,component) \
-  g3dpoly->var [i].component = inpoly_##var [vt].component + \
-    t * (inpoly_##var [vt2].component - inpoly_##var [vt].component);
-
-#define INTERPOLATE_S(var) \
-{ \
-  float v1 = inpoly_##var [edge_from [0]] + \
-    t1 * (inpoly_##var [edge_to [0]] - inpoly_##var [edge_from [0]]); \
-  float v2 = inpoly_##var [edge_from [1]] + \
-    t2 * (inpoly_##var [edge_to [1]] - inpoly_##var [edge_from [1]]); \
-  g3dpoly->var [i] = v1 + t * (v2 - v1); \
-}
-
-#define INTERPOLATE(var,component) \
-{ \
-  float v1 = inpoly_##var [edge_from [0]].component + \
-    t1 * (inpoly_##var [edge_to [0]].component - inpoly_##var [edge_from [0]].component); \
-  float v2 = inpoly_##var [edge_from [1]].component + \
-    t2 * (inpoly_##var [edge_to [1]].component - inpoly_##var [edge_from [1]].component); \
-  g3dpoly->var [i].component = v1 + t * (v2 - v1); \
-}
-
-static void PreparePolygonFX2 (G3DPolygonDPFX* g3dpoly,
-  csVector2* clipped_verts, int num_vertices, csVertexStatus* clipped_vtstats,
-  int orig_num_vertices, bool gouraud)
-{
-  // first we copy the first texture coordinates to a local buffer
-  // to avoid that they are overwritten when interpolating.
-  CS_ALLOC_STACK_ARRAY (csVector2, inpoly_vertices, orig_num_vertices);
-  CS_ALLOC_STACK_ARRAY (csVector2, inpoly_texels, orig_num_vertices);
-  CS_ALLOC_STACK_ARRAY (csColor, inpoly_colors, orig_num_vertices);
-  CS_ALLOC_STACK_ARRAY (float, inpoly_z, orig_num_vertices);
-  int i;
-  memcpy (inpoly_vertices, g3dpoly->vertices,
-  	orig_num_vertices * sizeof (csVector2));
-  memcpy (inpoly_texels, g3dpoly->texels,
-  	orig_num_vertices * sizeof (csVector2));
-  memcpy (inpoly_colors, g3dpoly->colors, orig_num_vertices * sizeof (csColor));
-  memcpy (inpoly_z, g3dpoly->z, orig_num_vertices * sizeof (float));
-
-  int vt, vt2;
-  float t;
-  for (i = 0; i < num_vertices; i++)
-  {
-    g3dpoly->vertices [i] = clipped_verts [i];
-    switch (clipped_vtstats[i].Type)
-    {
-      case CS_VERTEX_ORIGINAL:
-        vt = clipped_vtstats[i].Vertex;
-        g3dpoly->z [i] = inpoly_z [vt];
-        g3dpoly->texels [i] = inpoly_texels [vt];
-	if (gouraud)
-          g3dpoly->colors [i] = inpoly_colors [vt];
-	break;
-      case CS_VERTEX_ONEDGE:
-        vt = clipped_vtstats[i].Vertex;
-	vt2 = vt + 1; if (vt2 >= orig_num_vertices) vt2 = 0;
-	t = clipped_vtstats [i].Pos;
-	INTERPOLATE1_S (z);
-	INTERPOLATE1 (texels,x);
-	INTERPOLATE1 (texels,y);
-	if (gouraud)
-	{
-	  INTERPOLATE1 (colors,red);
-	  INTERPOLATE1 (colors,green);
-	  INTERPOLATE1 (colors,blue);
-	}
-	break;
-      case CS_VERTEX_INSIDE:
-        float x = clipped_verts [i].x;
-        float y = clipped_verts [i].y;
-        int edge_from [2], edge_to [2];
-	int edge = 0;
-	int j, j1;
-	j1 = orig_num_vertices - 1;
-	for (j = 0; j < orig_num_vertices; j++)
-	{
-          if ((y >= inpoly_vertices [j].y && y <= inpoly_vertices [j1].y) ||
-	      (y <= inpoly_vertices [j].y && y >= inpoly_vertices [j1].y))
-	  {
-	    edge_from [edge] = j;
-	    edge_to [edge] = j1;
-	    edge++;
-	    if (edge >= 2) break;
-	  }
-	  j1 = j;
-	}
-	if (edge == 1)
-	{
-	  // Safety if we only found one edge.
-	  edge_from [1] = edge_from [0];
-	  edge_to [1] = edge_to [0];
-	}
-	csVector2& A = inpoly_vertices [edge_from [0]];
-	csVector2& B = inpoly_vertices [edge_to [0]];
-	csVector2& C = inpoly_vertices [edge_from [1]];
-	csVector2& D = inpoly_vertices [edge_to [1]];
-	float t1 = (y - A.y) / (B.y - A.y);
-	float t2 = (y - C.y) / (D.y - C.y);
-	float x1 = A.x + t1 * (B.x - A.x);
-	float x2 = C.x + t2 * (D.x - C.x);
-	t = (x - x1) / (x2 - x1);
-	INTERPOLATE_S (z);
-	INTERPOLATE (texels,x);
-	INTERPOLATE (texels,y);
-	if (gouraud)
-	{
-	  INTERPOLATE (colors,red);
-	  INTERPOLATE (colors,green);
-	  INTERPOLATE (colors,blue);
-	}
-	break;
-    }
-  }
-}
-
-#undef INTERPOLATE
-#undef INTERPOLATE1
-
-bool csSprite2DMeshObject::Draw (iRenderView* rview, iMovable* /*movable*/,
-	csZBufMode mode)
-{
-// @@@ TODO:
-//     - Z fill vs Z use
-  if (!material)
-  {
-    printf ("INTERNAL ERROR: sprite2D used without material!\n");
-    return false;
-  }
-  iMaterialHandle* mat = material->GetMaterialHandle ();
-  if (!mat)
-  {
-    printf ("INTERNAL ERROR: sprite2D used without valid material handle!\n");
-    return false;
-  }
-
-  if (vis_cb) if (!vis_cb->BeforeDrawing (this, rview)) return false;
-
-  iGraphics3D* g3d = rview->GetGraphics3D ();
-  iCamera* camera = rview->GetCamera ();
-
-  // Prepare for rendering.
-  g3d->SetRenderState (G3DRENDERSTATE_ZBUFFERMODE, mode);
-
-  material->Visit ();
-
-  G3DPolygonDPFX& g3dpolyfx = factory->g3dpolyfx;
-  g3dpolyfx.num = vertices.Length ();
-  g3dpolyfx.mat_handle = mat;
-  g3dpolyfx.mat_handle->GetTexture ()->GetMeanColor (g3dpolyfx.flat_color_r,
-    g3dpolyfx.flat_color_g, g3dpolyfx.flat_color_b);
-
-  CS_ALLOC_STACK_ARRAY (csVector2, poly2d, vertices.Length ());
-  csVector2 clipped_poly2d[MAX_OUTPUT_VERTICES];
-  csVertexStatus clipped_vtstats[MAX_OUTPUT_VERTICES];
-
-  float iz = 1. / cam.z;
-  float iza = iz * camera->GetFOV ();
-
-  int i;
-
-  for (i = 0; i < vertices.Length (); i++)
-  {
-    g3dpolyfx.z [i] = iz;
-    g3dpolyfx.vertices [i].x =
-    poly2d [i].x = (cam.x + vertices [i].pos.x) * iza + camera->GetShiftX ();
-    g3dpolyfx.vertices [i].y =
-    poly2d [i].y = (cam.y + vertices [i].pos.y) * iza + camera->GetShiftY ();
-    g3dpolyfx.colors [i] = vertices [i].color;
-  }
-
-  if (!uvani)
-  {
-    for (i = 0; i < vertices.Length (); i++)
-    {
-      g3dpolyfx.texels [i].x = vertices [i].u;
-      g3dpolyfx.texels [i].y = vertices [i].v;
-    }
-  }
-  else
-  {
-    int n;
-    const csVector2 *uv = uvani->GetVertices (n);
-    for (i = 0; i < n; i++)
-    {
-      g3dpolyfx.texels [i].x = uv [i].x;
-      g3dpolyfx.texels [i].y = uv [i].y;
-    }
-  }
-
-  int num_clipped_verts;
-  uint8 clip_result = rview->GetClipper ()->Clip (poly2d, vertices.Length (),
-    clipped_poly2d, num_clipped_verts, clipped_vtstats);
-  if (clip_result == CS_CLIP_OUTSIDE) return false;
-  g3dpolyfx.num = num_clipped_verts;
-
-  if (clip_result != CS_CLIP_INSIDE)
-    PreparePolygonFX2 (&g3dpolyfx, clipped_poly2d, num_clipped_verts,
-    	clipped_vtstats, vertices.Length (), true);
-
-  rview->CalculateFogPolygon (g3dpolyfx);
-  g3dpolyfx.mixmode = MixMode;
-  g3d->DrawPolygonFX (g3dpolyfx);
-  return true;
-}
-
 csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n, 
 						      iRenderView* rview, 
 						      iMovable* movable, 
 						      uint32 frustum_mask,
 						      csVector3 offset)
 {
-#ifdef CS_USE_NEW_RENDERER
   SetupObject ();
 
   iCamera* camera = rview->GetCamera ();
@@ -463,7 +214,7 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
 
   if (factory->light_mgr)
   {
-    const csArray<iLight*>& relevant_lights = factory->light_mgr
+    const csArray<iLightSectorInfluence*>& relevant_lights = factory->light_mgr
     	->GetRelevantLights (logparent, -1, false);
     UpdateLighting (relevant_lights, movable, offset);
   }
@@ -475,7 +226,6 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
   int clip_portal, clip_plane, clip_z_plane;
   rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane, 
     clip_z_plane);
-  csVector3 camera_origin = temp.GetT2OTranslation ();
 
   csReversibleTransform tr_o2c;
   tr_o2c.SetO2TTranslation (-temp.Other2This (offset));
@@ -486,11 +236,15 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
   if (meshCreated)
   {
     rm->meshtype = CS_MESHTYPE_TRIANGLEFAN;
-    rm->material = material;
+    rm->buffers = bufferHolder;
     rm->variablecontext = svcontext;
     rm->geometryInstance = this;
   }
   
+  rm->material = material;//Moved this statement out of the above 'if'
+    //to make the change of the material possible at any time
+    //(thru a call to either iSprite2DState::SetMaterialWrapper () or
+    //csSprite2DMeshObject::SetMaterialWrapper (). Luca
   rm->mixmode = MixMode;
   rm->clip_portal = clip_portal;
   rm->clip_plane = clip_plane;
@@ -501,122 +255,121 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
       disappear. 
      */
   rm->indexstart = 0;
-  rm->indexend = vertices.Length();
-  rm->object2camera = tr_o2c;
-  rm->camera_origin = camera_origin;
+  rm->worldspace_origin = movable->GetFullPosition ();
+
+  rm->object2world = tr_o2c.GetInverse () * camera->GetTransform ();
+  rm->indexend = (uint)vertices.Length();
+
 
   n = 1; 
   return &rm; 
-#endif
-  n = 0; 
-  return 0;
 }
 
-#ifdef CS_USE_NEW_RENDERER
-void csSprite2DMeshObject::PreGetShaderVariableValue (csShaderVariable* variable)
+void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer)
 {
-  const csStringID name = variable->GetName ();
-  if (name == index_name)
+  if (!holder) return;
+
+  if (buffer == CS_BUFFER_INDEX)
   {
-    size_t indexSize = sizeof (uint) * vertices.Length();
+    size_t indexSize = vertices.Length();
     if (!index_buffer.IsValid() || 
       (indicesSize != indexSize))
     {
-      index_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+      index_buffer = csRenderBuffer::CreateIndexRenderBuffer (
 	indexSize, CS_BUF_DYNAMIC, 
-	CS_BUFCOMP_UNSIGNED_INT, 1, true));
-      variable->SetValue (index_buffer);
+	CS_BUFCOMP_UNSIGNED_INT, 0, vertices.Length() - 1);
+      
+      holder->SetRenderBuffer (CS_BUFFER_INDEX, index_buffer);
 
       csRenderBufferLock<uint> indexLock (index_buffer);
       uint* ptr = indexLock;
 
-      for (int i = 0; i < vertices.Length(); i++)
+      for (size_t i = 0; i < vertices.Length(); i++)
       {
-	*ptr++ = i;
+	*ptr++ = (uint)i;
       }
       indicesSize = indexSize;
     }
   }
-  else if (name == texel_name)
+  else if (buffer == CS_BUFFER_TEXCOORD0)
   {
     if (texels_dirty)
     {
-		
-	  int texels_count;
-	  const csVector2 *uvani_uv;
-	  if (!uvani)
-	    texels_count = vertices.Length ();
-	  else
-	 	uvani_uv = uvani->GetVertices (texels_count);
+      int texels_count;
+      const csVector2 *uvani_uv = 0;
+      if (!uvani)
+	texels_count = (int)vertices.Length ();
+      else
+	uvani_uv = uvani->GetVertices (texels_count);
 	  
-	  size_t texelSize = sizeof (float) * texels_count * 2;
-      if (!texel_buffer.IsValid() || (size_t)texel_buffer->GetSize() != texelSize)
+      size_t texelSize = texels_count;
+      if (!texel_buffer.IsValid() || (texel_buffer->GetSize()
+      	!= texelSize * sizeof(float) * 2))
       {
-		texel_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
-		texelSize, CS_BUF_STATIC, 
-		CS_BUFCOMP_FLOAT, 2, false));
-		variable->SetValue (texel_buffer);
+	texel_buffer = csRenderBuffer::CreateRenderBuffer (
+	  texelSize, CS_BUF_STATIC, CS_BUFCOMP_FLOAT, 2);
+	holder->SetRenderBuffer (CS_BUFFER_TEXCOORD0, texel_buffer);
       }
 
       csRenderBufferLock<csVector2> texelLock (texel_buffer);
 
-	  for (int i = 0; i < texels_count; i++)
-	  {
-	  	csVector2& v = texelLock[i];
-	  	if (!uvani)
-		{
-			v.x = vertices[i].u;
-			v.y = vertices[i].v;
-		}
-		else
-		{
-			v.x = uvani_uv[i].x;
-			v.y = uvani_uv[i].y;
-		}
-	  }
+      for (size_t i = 0; i < (size_t)texels_count; i++)
+      {
+	csVector2& v = texelLock[i];
+	if (!uvani)
+	{
+	  v.x = vertices[i].u;
+	  v.y = vertices[i].v;
+	}
+	else
+	{
+	  v.x = uvani_uv[i].x;
+	  v.y = uvani_uv[i].y;
+	}
+      }
       texels_dirty = false;
     }
   }
-  else if (name == color_name)
+  else if (buffer == CS_BUFFER_COLOR)
   {
     if (colors_dirty)
     {
-      size_t color_size = sizeof (float) * vertices.Length() * 3;
-      if (!color_buffer.IsValid() ||
-	(size_t)color_buffer->GetSize() != color_size)
+      size_t color_size = vertices.Length();
+      if (!color_buffer.IsValid() || (color_buffer->GetSize() 
+	!= color_size * sizeof(float) * 2))
       {
-	color_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+	color_buffer = csRenderBuffer::CreateRenderBuffer (
 	  color_size, CS_BUF_STATIC, 
-	  CS_BUFCOMP_FLOAT, 3, false));
-	variable->SetValue (color_buffer);
+	  CS_BUFCOMP_FLOAT, 3);
+	holder->SetRenderBuffer (CS_BUFFER_COLOR, color_buffer);
       }
 
       csRenderBufferLock<csColor> colorLock (color_buffer);
 
-      for (int i = 0; i < vertices.Length(); i++)
+      for (size_t i = 0; i < vertices.Length(); i++)
       {
 	colorLock[i] = vertices[i].color;
       }
       colors_dirty = false;
     }
   }
-  else if (name == vertex_name)
+  else if (buffer == CS_BUFFER_POSITION)
   {
     if (vertices_dirty)
     {
-      size_t vertices_size = sizeof (float) * vertices.Length() * 3;
-      if (!vertex_buffer.IsValid() ||
-	(size_t)vertex_buffer->GetSize() != vertices_size)
+      size_t vertices_size = vertices.Length();
+      if (!vertex_buffer.IsValid() || (vertex_buffer->GetSize() 
+	!= vertices_size * sizeof(float) * 3))
       {
-	vertex_buffer.AttachNew (factory->g3d->CreateRenderBuffer (
+	vertex_buffer = csRenderBuffer::CreateRenderBuffer (
 	  vertices_size, CS_BUF_STATIC, 
-	  CS_BUFCOMP_FLOAT, 3, false));
-	variable->SetValue (vertex_buffer);
+	  CS_BUFCOMP_FLOAT, 3);
+	holder->SetRenderBuffer (CS_BUFFER_POSITION, vertex_buffer);
       }
 
       csRenderBufferLock<csVector3> vertexLock (vertex_buffer);
 
-      for (int i = 0; i < vertices.Length(); i++)
+      for (size_t i = 0; i < vertices.Length(); i++)
       {
 	vertexLock[i].Set (vertices[i].pos.x, vertices[i].pos.y, 0.0f);
       }
@@ -624,14 +377,18 @@ void csSprite2DMeshObject::PreGetShaderVariableValue (csShaderVariable* variable
     }
   }
 }
-#endif
 
-void csSprite2DMeshObject::GetObjectBoundingBox (csBox3& bbox, int /*type*/)
+void csSprite2DMeshObject::GetObjectBoundingBox (csBox3& bbox)
 {
   SetupObject ();
   bbox.Set (-radius, radius);
 }
 
+void csSprite2DMeshObject::SetObjectBoundingBox (const csBox3&)
+{
+  // @@@ TODO
+}
+
 void csSprite2DMeshObject::HardTransform (const csReversibleTransform& t)
 {
   (void)t;
@@ -643,7 +400,7 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
   double angle_inc = TWO_PI / n;
   double angle = 0.0;
   vertices.SetLength (n);
-  int i;
+  size_t i;
   for (i = 0; i < vertices.Length (); i++, angle += angle_inc)
   {
     vertices [i].pos.y = cos (angle);
@@ -657,47 +414,34 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
     vertices [i].color.Set (1, 1, 1);
     vertices [i].color_init.Set (1, 1, 1);
   }
-#ifdef CS_USE_NEW_RENDERER
+
   vertices_dirty = true;
   texels_dirty = true;
   colors_dirty = true;
-#endif
+
   scfiObjectModel.ShapeChanged ();
 }
 
-void csSprite2DMeshObject::NextFrame (csTicks current_time, const csVector3& /*pos*/)
+void csSprite2DMeshObject::NextFrame (csTicks current_time,
+	const csVector3& /*pos*/)
 {
   if (uvani && !uvani->halted)
-#ifdef CS_USE_NEW_RENDERER
   {
-	int old_frame_index = uvani->frameindex;
-#endif
+    int old_frame_index = uvani->frameindex;
     uvani->Advance (current_time);
-#ifdef CS_USE_NEW_RENDERER
-	texels_dirty = old_frame_index != uvani->frameindex;
+    texels_dirty |= (old_frame_index != uvani->frameindex);
   }
-#endif
 }
 
-void csSprite2DMeshObject::Particle::UpdateLighting (const csArray<iLight*>& lights,
-    const csReversibleTransform& transform)
+void csSprite2DMeshObject::Particle::UpdateLighting (
+	const csArray<iLightSectorInfluence*>& lights,
+	const csReversibleTransform& transform)
 {
   csVector3 new_pos = transform.This2Other (part_pos);
   scfParent->UpdateLighting (lights, new_pos);
 }
 
-void csSprite2DMeshObject::Particle::Draw (iRenderView* rview,
-    const csReversibleTransform& transform, csZBufMode mode)
-{
-  scfParent->SetupObject ();
-
-  // Camera transformation for the single 'position' vector.
-  csVector3 new_pos = transform.This2Other (part_pos);
-  cam = rview->GetCamera ()->GetTransform ().Other2This (new_pos);
-  if (cam.z < SMALL_Z) return;
-  scfParent->Draw (rview, 0, mode);
-}
-    
+   
 csRenderMesh** csSprite2DMeshObject::Particle::GetRenderMeshes (int& n, 
 	iRenderView* rview, iMovable* movable, uint32 frustum_mask)
 {
@@ -707,52 +451,52 @@ csRenderMesh** csSprite2DMeshObject::Particle::GetRenderMeshes (int& n,
 void csSprite2DMeshObject::Particle::SetColor (const csColor& col)
 {
   csColoredVertices& vertices = scfParent->GetVertices ();
-  int i;
+  size_t i;
   for (i = 0 ; i < vertices.Length () ; i++)
     vertices[i].color_init = col;
   if (!scfParent->lighting)
     for (i = 0 ; i < vertices.Length () ; i++)
       vertices[i].color = col;
-#ifdef CS_USE_NEW_RENDERER  
+
   scfParent->colors_dirty = true;
-#endif
+
 }
 
 void csSprite2DMeshObject::Particle::AddColor (const csColor& col)
 {
   csColoredVertices& vertices = scfParent->GetVertices ();
-  int i;
+  size_t i;
   for (i = 0 ; i < vertices.Length () ; i++)
     vertices[i].color_init += col;
   if (!scfParent->lighting)
     for (i = 0 ; i < vertices.Length () ; i++)
       vertices[i].color = vertices[i].color_init;
-#ifdef CS_USE_NEW_RENDERER  
+
   scfParent->colors_dirty = true;
-#endif
+
 }
 
 void csSprite2DMeshObject::Particle::ScaleBy (float factor)
 {
   csColoredVertices& vertices = scfParent->GetVertices ();
-  int i;
+  size_t i;
   for (i = 0; i < vertices.Length (); i++)
     vertices[i].pos *= factor;
-#ifdef CS_USE_NEW_RENDERER
+
   scfParent->vertices_dirty = true;
-#endif
+
   scfParent->scfiObjectModel.ShapeChanged ();
 }
 
 void csSprite2DMeshObject::Particle::Rotate (float angle)
 {
   csColoredVertices& vertices = scfParent->GetVertices ();
-  int i;
+  size_t i;
   for (i = 0; i < vertices.Length (); i++)
     vertices[i].pos.Rotate (angle);
-#ifdef CS_USE_NEW_RENDERER
+
   scfParent->vertices_dirty = true;
-#endif
+
   scfParent->scfiObjectModel.ShapeChanged ();
 }
 
@@ -833,17 +577,25 @@ void csSprite2DMeshObject::Sprite2DState::RemoveUVAnimation (
 }
 
 iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
-	const char *name)
+	const char *name) const
 {
   return scfParent->factory->GetUVAnimation (name);
 }
 
 iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
-	int idx)
+	int idx) const
 {
   return scfParent->factory->GetUVAnimation (idx);
 }
 
+iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
+	int idx, int &style, bool &loop) const
+{
+  style = scfParent->uvani->style;
+  loop = scfParent->uvani->loop;
+  return scfParent->factory->GetUVAnimation (idx);
+}
+
 
 void csSprite2DMeshObject::uvAnimationControl::Advance (csTicks current_time)
 {
@@ -915,7 +667,8 @@ void csSprite2DMeshObject::uvAnimationControl::Advance (csTicks current_time)
 
 }
 
-const csVector2 *csSprite2DMeshObject::uvAnimationControl::GetVertices (int &num)
+const csVector2 *csSprite2DMeshObject::uvAnimationControl::GetVertices (
+	int &num)
 {
   num = frame->GetUVCount ();
   return frame->GetUVCoo ();
@@ -923,10 +676,10 @@ const csVector2 *csSprite2DMeshObject::uvAnimationControl::GetVertices (int &num
 
 // The hit beam methods in sprite2d make a couple of small presumptions.
 // 1) The sprite is always facing the start of the beam.
-// 2) Since it is always facing the the beam, only one side
+// 2) Since it is always facing the beam, only one side
 // of its bounding box can be hit (if at all).
 
-void csSprite2DMeshObject::CheckBeam (const csVector3& start,
+void csSprite2DMeshObject::CheckBeam (const csVector3& /*start*/,
       const csVector3& pl, float sqr, csMatrix3& o2t)
 {
   // This method is an optimized version of LookAt() based on
@@ -937,10 +690,10 @@ void csSprite2DMeshObject::CheckBeam (const csVector3& start,
   // The transformation matrix is stored and used again if the
   // start vector for the beam is in the same position. MHV.
 
-  csVector3 pl2 = pl * qisqrt (sqr);
+  csVector3 pl2 = pl * csQisqrt (sqr);
   csVector3 v1( pl2.z, 0, -pl2.x);
   sqr = v1*v1;
-  v1 *= qisqrt(sqr);
+  v1 *= csQisqrt(sqr);
   csVector3 v2(pl2.y * v1.z, pl2.z * v1.x - pl2.x * v1.z, -pl2.y * v1.x);
   o2t.Set (v1.x, v2.x, pl2.x,
            v1.y, v2.y, pl2.y,
@@ -956,12 +709,12 @@ bool csSprite2DMeshObject::HitBeamOutline(const csVector3& start,
   float sqr = pl * pl;
   if (sqr < SMALL_EPSILON) return false; // Too close, Cannot intersect
   float dist;
-  csIntersect3::Plane(start, end, pl, 0, isect, dist);
+  csIntersect3::SegmentPlane(start, end, pl, 0, isect, dist);
   if (pr) *pr = dist;
   csMatrix3 o2t;
   CheckBeam (start, pl, sqr, o2t);
   csVector3 r = o2t * isect;
-  int trail, len = vertices.Length();
+  int trail, len = (int)vertices.Length();
   trail = len - 1;
   csVector2 isec(r.x, r.y);
   int i;
@@ -982,7 +735,7 @@ SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObjectFactory::Sprite2DFactoryState)
   SCF_IMPLEMENTS_INTERFACE (iSprite2DFactoryState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
-csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iBase *pParent,
+csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iMeshObjectType* pParent,
 	iObjectRegistry* object_reg)
 {
   SCF_CONSTRUCT_IBASE (pParent);
@@ -991,9 +744,12 @@ csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iBase *pParent,
   MixMode = 0;
   lighting = true;
   logparent = 0;
+  spr2d_type = pParent;
   csSprite2DMeshObjectFactory::object_reg = object_reg;
   light_mgr = CS_QUERY_REGISTRY (object_reg, iLightManager);
   g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
+    "crystalspace.shared.stringset", iStringSet);
 }
 
 csSprite2DMeshObjectFactory::~csSprite2DMeshObjectFactory ()
