@@ -24,18 +24,22 @@
 #include "csgeom/objmodel.h"
 #include "csgfx/shadervar.h"
 #include "csgfx/shadervarcontext.h"
-#include "csutil/cscolor.h"
+
 #include "csutil/array.h"
+#include "csutil/cscolor.h"
+#include "csutil/flags.h"
 #include "csutil/refarr.h"
-#include "cstool/rendermeshholder.h"
+#include "csutil/leakguard.h"
 #include "csutil/weakref.h"
+#include "cstool/rendermeshholder.h"
+
 #include "imesh/object.h"
 #include "imesh/sprite2d.h"
 #include "imesh/particle.h"
 #include "ivideo/graph3d.h"
-#include "iutil/config.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
+#include "iutil/pluginconfig.h"
 #include "iengine/lightmgr.h"
 #include "spr2duv.h"
 
@@ -64,35 +68,35 @@ protected:
 
   uvAnimationControl* uvani;
 
-#ifdef CS_USE_NEW_RENDERER
-  class eiShaderVariableAccessor : public iShaderVariableAccessor
+  class eiRenderBufferAccessor : public iRenderBufferAccessor
   {
+  private:
     csWeakRef<csSprite2DMeshObject> parent;
   public:
+    CS_LEAKGUARD_DECLARE (eiRenderBufferAccessor);
     SCF_DECLARE_IBASE;
 
-    eiShaderVariableAccessor (csSprite2DMeshObject* parent)
+    eiRenderBufferAccessor (csSprite2DMeshObject* parent)
     {
       SCF_CONSTRUCT_IBASE(0);
-      eiShaderVariableAccessor::parent = parent;
+      eiRenderBufferAccessor::parent = parent;
     }
-    virtual ~eiShaderVariableAccessor() 
+    virtual ~eiRenderBufferAccessor() 
     {
       SCF_DESTRUCT_IBASE();
     }
 
-    virtual void PreGetValue (csShaderVariable* variable)
+    void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer)
     {
-      if (parent) parent->PreGetShaderVariableValue (variable);
+      if (parent) parent->PreGetBuffer (holder, buffer);
     }
   };
-  friend class eiShaderVariableAccessor;
+  friend class eiRenderBufferAccessor;
 
-  void PreGetShaderVariableValue (csShaderVariable* variable);
+  void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer);
 
-  csRef<csShaderVariableContext> svcontext;
-  csRenderMeshHolderSingle rmHolder;
-  static csStringID vertex_name, texel_name, color_name, index_name;
+  csRef<csRenderBufferHolder> bufferHolder;
+  csRenderMeshHolder rmHolder;
   csRef<iRenderBuffer> vertex_buffer;
   bool vertices_dirty;
   csRef<iRenderBuffer> texel_buffer;
@@ -101,14 +105,14 @@ protected:
   bool colors_dirty;
   csRef<iRenderBuffer> index_buffer;
   size_t indicesSize;
-#endif
+  csRef<csShaderVariableContext> svcontext;
 
 private:
   csRef<iMeshObjectFactory> ifactory;
-  iBase* logparent;
+  iMeshWrapper* logparent;
   csSprite2DMeshObjectFactory* factory;
 
-  iMaterialWrapper* material;
+  csRef<iMaterialWrapper> material;
   uint MixMode;
   bool initialized;
   iMeshObjectDrawCallback* vis_cb;
@@ -135,8 +139,9 @@ private:
   void SetupObject ();
 
   /// Update lighting given a position.
-  void UpdateLighting (const csArray<iLight*>& lights, const csVector3& pos);
-  void UpdateLighting (const csArray<iLight*>& lights,
+  void UpdateLighting (const csArray<iLightSectorInfluence*>& lights,
+      const csVector3& pos);
+  void UpdateLighting (const csArray<iLightSectorInfluence*>& lights,
       	iMovable* movable, csVector3 offset);
 
   /// Check the start vector and recalculate the LookAt matrix if changed.
@@ -147,6 +152,8 @@ public:
   /// Constructor.
   csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory);
 
+  CS_LEAKGUARD_DECLARE (csSprite2DMeshObject);
+
   /// Destructor.
   virtual ~csSprite2DMeshObject ();
 
@@ -159,7 +166,8 @@ public:
    */
   void CreateRegularVertices (int n, bool setuv);
 
-  void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL);
+  void GetObjectBoundingBox (csBox3& bbox);
+  void SetObjectBoundingBox (const csBox3& bbox);
   void GetRadius (csVector3& rad, csVector3& cent)
   { rad = radius; cent.Set (0,0,0); }
 
@@ -169,8 +177,6 @@ public:
   virtual iMeshObjectFactory* GetFactory () const { return ifactory; }
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone () { return 0; }
-  virtual bool DrawTest (iRenderView* rview, iMovable* movable,
-  	uint32 frustum_mask);
   csRenderMesh **GetRenderMeshes (int &n, iRenderView* rview, 
     iMovable* movable, uint32 frustum_mask, csVector3 offset);
   virtual csRenderMesh **GetRenderMeshes (int &n, iRenderView* rview, 
@@ -178,7 +184,6 @@ public:
   {
     return GetRenderMeshes (n, rview, movable, frustum_mask, csVector3 (0.0f));
   }
-  virtual bool Draw (iRenderView* rview, iMovable* movable, csZBufMode mode);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
     if (cb) cb->IncRef ();
@@ -196,21 +201,27 @@ public:
         csVector3&, float*);
   /// 2D sprites have no depth, so this is equivalent to HitBeamOutline.
   virtual bool HitBeamObject (const csVector3& start, const csVector3& end,
-  	csVector3& isect, float* pr, int* polygon_idx = 0)
+  	csVector3& isect, float* pr, int* polygon_idx = 0,
+	iMaterialWrapper** material = 0)
   {
+    if (material) *material = csSprite2DMeshObject::material;
     if (polygon_idx) *polygon_idx = -1;
     return HitBeamOutline(start, end, isect, pr);
   }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
+  virtual iMeshWrapper* GetMeshWrapper () const { return logparent; }
 
   //------------------------- iObjectModel implementation ----------------
   class ObjectModel : public csObjectModel
   {
     SCF_DECLARE_EMBEDDED_IBASE (csSprite2DMeshObject);
-    virtual void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL)
+    virtual void GetObjectBoundingBox (csBox3& bbox)
     {
-      scfParent->GetObjectBoundingBox (bbox, type);
+      scfParent->GetObjectBoundingBox (bbox);
+    }
+    virtual void SetObjectBoundingBox (const csBox3& bbox)
+    {
+      scfParent->SetObjectBoundingBox (bbox);
     }
     virtual void GetRadius (csVector3& rad, csVector3& cent)
     {
@@ -228,12 +239,14 @@ public:
     return true;
   }
   virtual iMaterialWrapper* GetMaterialWrapper () const { return material; }
+  virtual void SetMixMode (uint mode) { MixMode = mode; }
+  virtual uint GetMixMode () const { return MixMode; }
   virtual void InvalidateMaterialHandles () { }
   /**
    * see imesh/object.h for specification. The default implementation
    * does nothing.
    */
-  virtual void PositionChild (iMeshObject* child,csTicks current_time) { }
+  virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
 
   //------------------------- iSprite2DState implementation ----------------
   class Sprite2DState : public iSprite2DState
@@ -269,8 +282,10 @@ public:
     virtual int GetUVAnimationCount () const;
     virtual iSprite2DUVAnimation *CreateUVAnimation ();
     virtual void RemoveUVAnimation (iSprite2DUVAnimation *anim);
-    virtual iSprite2DUVAnimation *GetUVAnimation (const char *name);
-    virtual iSprite2DUVAnimation *GetUVAnimation (int idx);
+    virtual iSprite2DUVAnimation *GetUVAnimation (const char *name) const;
+    virtual iSprite2DUVAnimation *GetUVAnimation (int idx) const;
+    virtual iSprite2DUVAnimation *GetUVAnimation (int idx, int &style,
+    	bool &loop) const;
 
   } scfiSprite2DState;
   friend class Sprite2DState;
@@ -294,11 +309,9 @@ public:
       scfParent->MixMode = mode;
     }
     virtual void Rotate (float angle);
-    virtual void Draw (iRenderView* rview,
-    	const csReversibleTransform& transform, csZBufMode mode);
     virtual csRenderMesh** GetRenderMeshes (int& n, iRenderView* rview, 
       iMovable* movable, uint32 frustum_mask);
-    virtual void UpdateLighting (const csArray<iLight*>& lights,
+    virtual void UpdateLighting (const csArray<iLightSectorInfluence*>& lights,
 	const csReversibleTransform& transform);
   } scfiParticle;
   friend class Particle;
@@ -310,7 +323,6 @@ public:
 class csSprite2DMeshObjectFactory : public iMeshObjectFactory
 {
 protected:
-
   class animVector : public csArray<csSprite2DUVAnimation*>
   {
   public:
@@ -329,8 +341,9 @@ protected:
   animVector vAnims;
 
 private:
-  iMaterialWrapper* material;
-  iBase* logparent;
+  csRef<iMaterialWrapper> material;
+  iMeshFactoryWrapper* logparent;
+  iMeshObjectType* spr2d_type;
   uint MixMode;
   /**
    * If false then we don't do lighting but instead use
@@ -340,15 +353,15 @@ private:
   csFlags flags;
 
 public:
-  /// Polygon.
-  G3DPolygonDPFX g3dpolyfx;
+  CS_LEAKGUARD_DECLARE (csSprite2DMeshObjectFactory);
 
   csRef<iLightManager> light_mgr;
-  csRef<iObjectRegistry> object_reg;
-  csRef<iGraphics3D> g3d;
+  iObjectRegistry* object_reg;
+  csWeakRef<iGraphics3D> g3d;
 public:
   /// Constructor.
-  csSprite2DMeshObjectFactory (iBase *pParent, iObjectRegistry* object_reg);
+  csSprite2DMeshObjectFactory (iMeshObjectType* pParent,
+  	iObjectRegistry* object_reg);
 
   /// Destructor.
   virtual ~csSprite2DMeshObjectFactory ();
@@ -360,7 +373,7 @@ public:
   /// Get mixmode.
   uint GetMixMode () const { return MixMode; }
 
-  int GetUVAnimationCount () const {return vAnims.Length ();}
+  int GetUVAnimationCount () const {return (int)vAnims.Length ();}
   iSprite2DUVAnimation *CreateUVAnimation ()
   {
     csSprite2DUVAnimation *p = new csSprite2DUVAnimation (0);
@@ -369,7 +382,7 @@ public:
   }
   void RemoveUVAnimation (iSprite2DUVAnimation *anim)
   {
-    int idx = vAnims.Find ((csSprite2DUVAnimation*)anim);
+    int idx = (int)vAnims.Find ((csSprite2DUVAnimation*)anim);
     if (idx != -1)
     {
       anim->DecRef ();
@@ -378,7 +391,7 @@ public:
   }
   iSprite2DUVAnimation *GetUVAnimation (const char *name)
   {
-    int idx = vAnims.FindKey (vAnims.KeyCmp(name));
+    int idx = (int)vAnims.FindKey (vAnims.KeyCmp(name));
     return (iSprite2DUVAnimation *)(idx != -1 ? vAnims.Get (idx) : 0);
   }
   iSprite2DUVAnimation *GetUVAnimation (int idx)
@@ -394,8 +407,11 @@ public:
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  { logparent = lp; }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  { return logparent; }
+  virtual iMeshObjectType* GetMeshObjectType () const { return spr2d_type; }
   virtual iObjectModel* GetObjectModel () { return 0; }
 
   //---------------------- iSprite2DFactoryState implementation ----------------
@@ -419,11 +435,10 @@ public:
     { return scfParent->CreateUVAnimation (); }
     virtual void RemoveUVAnimation (iSprite2DUVAnimation *anim)
     { scfParent->RemoveUVAnimation(anim); }
-    virtual iSprite2DUVAnimation *GetUVAnimation (const char *name)
+    virtual iSprite2DUVAnimation *GetUVAnimation (const char *name) const
     { return scfParent->GetUVAnimation (name); }
-    virtual iSprite2DUVAnimation *GetUVAnimation (int idx)
+    virtual iSprite2DUVAnimation *GetUVAnimation (int idx) const
     { return scfParent->GetUVAnimation (idx); }
-
   } scfiSprite2DFactoryState;
   friend class Sprite2DFactoryState;
 };
