@@ -18,34 +18,43 @@
 */
 
 #include "cssysdef.h"
-#include "csutil/sysfunc.h"
+
+
 #include "csgeom/math3d.h"
 #include "csgeom/matrix3.h"
-#include "csgeom/quaterni.h"
+#include "csgeom/quaternion.h"
 #include "csgeom/transfrm.h"
-#include "csutil/scanstr.h"
-#include "csutil/util.h"
-#include "imesh/object.h"
-#include "iengine/mesh.h"
-#include "iengine/engine.h"
-#include "iutil/plugin.h"
-#include "imesh/sprite3d.h"
-#include "ivideo/graph3d.h"
-#include "qint.h"
-#include "iutil/vfs.h"
+#include "csgeom/tri.h"
+#include "csutil/cscolor.h"
 #include "csutil/csstring.h"
-#include "iutil/object.h"
+#include "csutil/scanstr.h"
+#include "csutil/sysfunc.h"
+#include "csutil/util.h"
+
+#include "iengine/engine.h"
 #include "iengine/material.h"
-#include "ivaria/reporter.h"
-#include "iutil/objreg.h"
-#include "iutil/eventh.h"
+#include "iengine/mesh.h"
+#include "imap/ldrctxt.h"
+#include "imesh/object.h"
+#include "imesh/sprite3d.h"
 #include "iutil/comp.h"
 #include "iutil/document.h"
-#include "imap/ldrctxt.h"
+#include "iutil/eventh.h"
+#include "iutil/object.h"
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "iutil/vfs.h"
+#include "ivaria/reporter.h"
+#include "ivideo/graph3d.h"
+
+
 #include "spr3dldr.h"
 
 CS_IMPLEMENT_PLUGIN
 
+namespace cspluginSpr3dLoader
+{
+
 enum
 {
   XMLTOKEN_ACTION,
@@ -68,65 +77,24 @@ enum
   XMLTOKEN_V
 };
 
-SCF_IMPLEMENT_IBASE (csSprite3DFactoryLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite3DFactoryLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite3DFactorySaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite3DFactorySaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite3DLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite3DLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite3DSaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite3DSaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csSprite3DFactoryLoader)
 SCF_IMPLEMENT_FACTORY (csSprite3DFactorySaver)
 SCF_IMPLEMENT_FACTORY (csSprite3DLoader)
 SCF_IMPLEMENT_FACTORY (csSprite3DSaver)
 
-
-csSprite3DFactoryLoader::csSprite3DFactoryLoader (iBase* pParent)
+csSprite3DFactoryLoader::csSprite3DFactoryLoader (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite3DFactoryLoader::~csSprite3DFactoryLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite3DFactoryLoader::Initialize (iObjectRegistry* object_reg)
 {
   csSprite3DFactoryLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("action", XMLTOKEN_ACTION);
   xmltokens.Register ("f", XMLTOKEN_F);
@@ -147,17 +115,18 @@ bool csSprite3DFactoryLoader::Initialize (iObjectRegistry* object_reg)
 }
 
 csPtr<iBase> csSprite3DFactoryLoader::Parse (iDocumentNode* node,
+				       iStreamSource*,
 				       iLoaderContext* ldr_context, 
 				       iBase* context)
 {
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-  	iPluginManager));
-  csRef<iMeshObjectType> type (CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	"crystalspace.mesh.object.sprite.3d", iMeshObjectType));
+  csRef<iPluginManager> plugin_mgr (csQueryRegistry<iPluginManager>
+    (object_reg));
+  csRef<iMeshObjectType> type (csQueryPluginClass<iMeshObjectType> (
+    plugin_mgr, "crystalspace.mesh.object.sprite.3d"));
   if (!type)
   {
-    type = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.mesh.object.sprite.3d",
-    	iMeshObjectType);
+    type = csLoadPlugin<iMeshObjectType> (plugin_mgr, 
+      "crystalspace.mesh.object.sprite.3d");
   }
   if (!type)
   {
@@ -172,14 +141,14 @@ csPtr<iBase> csSprite3DFactoryLoader::Parse (iDocumentNode* node,
   // below should be removed.
   csRef<iMeshObjectFactory> fact;
   if (context)
-    fact = SCF_QUERY_INTERFACE (context, iMeshObjectFactory);
+    fact = scfQueryInterface<iMeshObjectFactory> (context);
   // DecRef of fact will be handled later.
   // If there was no factory we create a new one.
   if (!fact)
     fact = type->NewFactory ();
 
   csRef<iSprite3DFactoryState> spr3dLook (
-  	SCF_QUERY_INTERFACE (fact, iSprite3DFactoryState));
+    scfQueryInterface<iSprite3DFactoryState> (fact));
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -378,49 +347,148 @@ csPtr<iBase> csSprite3DFactoryLoader::Parse (iDocumentNode* node,
 
 //---------------------------------------------------------------------------
 
-csSprite3DFactorySaver::csSprite3DFactorySaver (iBase* pParent)
+csSprite3DFactorySaver::csSprite3DFactorySaver (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite3DFactorySaver::~csSprite3DFactorySaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite3DFactorySaver::Initialize (iObjectRegistry* object_reg)
 {
   csSprite3DFactorySaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
 
-#define MAXLINE 100 /* max number of chars per line... */
-
-void csSprite3DFactorySaver::WriteDown (iBase*, iFile*)
+bool csSprite3DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
+	iStreamSource*)
 {
+  if (!parent) return false; //you never know...
+  
+  csRef<iDocumentNode> paramsNode = 
+    parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+  paramsNode->SetValue("params");
+
+  if (obj)
+  {
+    csRef<iSprite3DFactoryState> spritefact = 
+      scfQueryInterface<iSprite3DFactoryState> (obj);
+    csRef<iMeshObjectFactory> meshfact = 
+      scfQueryInterface<iMeshObjectFactory> (obj);
+    if (!spritefact) return false;
+    if (!meshfact) return false;
+
+    //Writedown Material tag
+    iMaterialWrapper* mat = spritefact->GetMaterialWrapper();
+    if (mat)
+    {
+      const char* matname = mat->QueryObject()->GetName();
+      if (matname && *matname)
+      {
+        csRef<iDocumentNode> matNode = 
+          paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        matNode->SetValue("material");
+        csRef<iDocumentNode> matnameNode = 
+          matNode->CreateNodeBefore(CS_NODE_TEXT, 0);
+        matnameNode->SetValue(matname);
+      }    
+    }    
+    int i;
+    //Write Frame Tags
+    for (i=0; i<spritefact->GetFrameCount(); i++)
+    {
+      csRef<iDocumentNode> frameNode = 
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      frameNode->SetValue("frame");
+      iSpriteFrame* sprite_frame = spritefact->GetFrame(i);
+      frameNode->SetAttribute("name", sprite_frame->GetName());
+      //Write v Tags
+      csRef<iDocumentNode> vertexNode;
+      for (int j=0; j<spritefact->GetVertexCount(); j++)
+      {
+        vertexNode = 
+          frameNode->CreateNodeBefore(CS_NODE_ELEMENT, vertexNode);
+        vertexNode->SetValue("v");
+        csVector3 vertex = spritefact->GetVertex(i,j);
+        csVector2 texel = spritefact->GetTexel(i,j);
+        //vertexNode->SetAttribute("name", sprite_frame->GetName());
+        synldr->WriteVector(vertexNode, &vertex);
+        vertexNode->SetAttributeAsFloat("u", texel.x);
+        vertexNode->SetAttributeAsFloat("v", texel.y);
+      }
+    }
+
+    //Write Action Tags
+    for (i=0; i<spritefact->GetActionCount(); i++)
+    {
+      csRef<iDocumentNode> actionNode = 
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      actionNode->SetValue("action");
+      iSpriteAction* sprite_action = spritefact->GetAction(i);
+      actionNode->SetAttribute("name", sprite_action->GetName());
+      //Write f Tags
+      for (int j=0; j<sprite_action->GetFrameCount(); j++)
+      {
+        csRef<iDocumentNode> frameNode = 
+          actionNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        frameNode->SetValue("f");
+        iSpriteFrame* sprite_frame = sprite_action->GetFrame(j);
+        frameNode->SetAttribute("name", sprite_frame->GetName());
+        frameNode->SetAttributeAsInt("delay", 
+          sprite_action->GetFrameDelay(j));
+      }
+    }
+
+    //Write Triangle Tags
+    for (i=0; i<spritefact->GetTriangleCount(); i++)
+    {
+      csRef<iDocumentNode> triaNode = 
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      triaNode->SetValue("t");
+      csTriangle sprite_tria = spritefact->GetTriangle(i);
+      triaNode->SetAttributeAsInt("v1", sprite_tria.a);
+      triaNode->SetAttributeAsInt("v2", sprite_tria.b);
+      triaNode->SetAttributeAsInt("v3", sprite_tria.c);
+    }
+
+    //Write Socket Tags
+    for (i=0; i<spritefact->GetSocketCount(); i++)
+    {
+      csRef<iDocumentNode> socketNode = 
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      socketNode->SetValue("socket");
+      iSpriteSocket* sprite_socket = spritefact->GetSocket(i);
+      socketNode->SetAttribute("name", sprite_socket->GetName());
+      socketNode->SetAttributeAsInt("tri", 
+        sprite_socket->GetTriangleIndex());
+    }
+
+    //Writedown Tween tag
+    synldr->WriteBool(paramsNode, "tween", 
+      spritefact->IsTweeningEnabled(), true);
+
+    //TBD: Writedown Smooth tag
+
+  }
+  return true;
 }
-
 //---------------------------------------------------------------------------
-csSprite3DLoader::csSprite3DLoader (iBase* pParent)
+csSprite3DLoader::csSprite3DLoader (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite3DLoader::~csSprite3DLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite3DLoader::Initialize (iObjectRegistry* object_reg)
 {
   csSprite3DLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("action", XMLTOKEN_ACTION);
   xmltokens.Register ("basecolor", XMLTOKEN_BASECOLOR);
@@ -433,7 +501,7 @@ bool csSprite3DLoader::Initialize (iObjectRegistry* object_reg)
 }
 
 csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
-	iLoaderContext* ldr_context, iBase*)
+	iStreamSource*, iLoaderContext* ldr_context, iBase*)
 {
   csRef<iMeshObject> mesh;
   csRef<iSprite3DState> spr3dLook;
@@ -459,7 +527,7 @@ csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
 	    return 0;
 	  }
 	  mesh = fact->GetMeshObjectFactory ()->NewInstance ();
-          spr3dLook = SCF_QUERY_INTERFACE (mesh, iSprite3DState);
+	  spr3dLook = scfQueryInterface<iSprite3DState> (mesh);
 	  if (!spr3dLook)
 	  {
       	    synldr->ReportError (
@@ -479,7 +547,18 @@ csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
 		"No Factory! Please define 'factory' before 'action'!");
 	  return 0;
 	}
-	spr3dLook->SetAction (child->GetContentsValue ());
+	else
+	{
+	  const char* action = child->GetContentsValue ();
+	  if (!spr3dLook->SetAction (action))
+	  {
+      	    synldr->ReportError (
+		  "crystalspace.sprite3dloader.parse.action",
+		  child,
+		  "Action '%s' failed to start!", action);
+	    return 0;
+	  }
+	}
         break;
       case XMLTOKEN_BASECOLOR:
 	if (!spr3dLook)
@@ -495,7 +574,7 @@ csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
 	  csColor col;
 	  if (!synldr->ParseColor (child, col))
 	    return 0;
-	  spr3dLook->SetBaseColor (col);
+	  mesh->SetColor (col);
 	}
         break;
       case XMLTOKEN_LIGHTING:
@@ -535,7 +614,7 @@ csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
 		child, "Couldn't find material '%s'!", matname);
             return 0;
 	  }
-	  spr3dLook->SetMaterialWrapper (mat);
+	  mesh->SetMaterialWrapper (mat);
 	}
 	break;
       case XMLTOKEN_MIXMODE:
@@ -583,26 +662,100 @@ csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
 
 //---------------------------------------------------------------------------
 
-csSprite3DSaver::csSprite3DSaver (iBase* pParent)
+csSprite3DSaver::csSprite3DSaver (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite3DSaver::~csSprite3DSaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite3DSaver::Initialize (iObjectRegistry* object_reg)
 {
   csSprite3DSaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
 
-void csSprite3DSaver::WriteDown (iBase*, iFile*)
+bool csSprite3DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
+	iStreamSource*)
 {
+  if (!parent) return false; //you never know...
+  
+  csRef<iDocumentNode> paramsNode = 
+    parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+  paramsNode->SetValue("params");
+
+  if (obj)
+  {
+    csRef<iSprite3DState> sprite = scfQueryInterface<iSprite3DState> (obj);
+    csRef<iMeshObject> mesh = scfQueryInterface<iMeshObject> (obj);
+    if (!sprite) return false;
+    if (!mesh) return false;
+
+    //Writedown Factory tag
+    iMeshFactoryWrapper* fact = mesh->GetFactory()->GetMeshFactoryWrapper ();
+    if (fact)
+    {
+      const char* factname = fact->QueryObject()->GetName();
+      if (factname && *factname)
+      {
+        csRef<iDocumentNode> factNode = 
+          paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        factNode->SetValue("factory");
+        factNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue(factname);
+      }    
+    }
+
+    //Writedown Lighting tag
+    synldr->WriteBool(paramsNode, "lighting", sprite->IsLighting(), true);
+
+    //Writedown Tween tag
+    synldr->WriteBool(paramsNode, "tween", sprite->IsTweeningEnabled(),true);
+
+    //Writedown Basecolor tag
+    csColor col;
+    mesh->GetColor (col);
+    csRef<iDocumentNode> colorNode = 
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    colorNode->SetValue("basecolor");
+    synldr->WriteColor(colorNode, &col);
+
+    //Writedown Action tag
+    const char* actionname = sprite->GetCurAction()->GetName();
+    if (actionname && *actionname)
+    {
+      csRef<iDocumentNode> actionNode = 
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      actionNode->SetValue("action");
+      actionNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue(actionname);
+    }    
+    
+    //Writedown Material tag
+    iMaterialWrapper* mat = mesh->GetMaterialWrapper();
+    if (mat)
+    {
+      const char* matname = mat->QueryObject()->GetName();
+      if (matname && *matname)
+      {
+        csRef<iDocumentNode> matNode = 
+          paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        matNode->SetValue("material");
+        csRef<iDocumentNode> matnameNode = 
+          matNode->CreateNodeBefore(CS_NODE_TEXT, 0);
+        matnameNode->SetValue(matname);
+      }    
+    }    
+
+    //Writedown Mixmode tag
+    int mixmode = sprite->GetMixMode();
+    csRef<iDocumentNode> mixmodeNode = 
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    mixmodeNode->SetValue("mixmode");
+    synldr->WriteMixmode(mixmodeNode, mixmode, true);
+  }
+  return true;
 }
 
+} // namespace cspluginSpr3dLoader
