@@ -20,7 +20,7 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "csutil/sysfunc.h"
 #include "csgeom/math3d.h"
 #include "csgeom/matrix3.h"
-#include "csgeom/quaterni.h"
+#include "csgeom/quaternion.h"
 #include "csgeom/transfrm.h"
 #include "csutil/scanstr.h"
 #include "csutil/util.h"
@@ -30,7 +30,7 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "iutil/plugin.h"
 #include "imesh/spritecal3d.h"
 #include "ivideo/graph3d.h"
-#include "qint.h"
+#include "csqint.h"
 #include "iutil/vfs.h"
 #include "csutil/csstring.h"
 #include "iutil/object.h"
@@ -43,10 +43,19 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "imap/ldrctxt.h"
 #include "sprcal3dldr.h"
 
+// Hack: work around problems caused by #defining 'new'
+#if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
+# undef new
+#endif
+#include <new>
+
 #include <cal3d/loader.h>
 
 CS_IMPLEMENT_PLUGIN
 
+namespace cspluginSprCal3dLdr
+{
+
 enum
 {
   XMLTOKEN_PATH,
@@ -59,69 +68,32 @@ enum
   XMLTOKEN_MORPHANIMATION,
   XMLTOKEN_OPTIONS,
   XMLTOKEN_HARDTRANSFORM,
-  XMLTOKEN_SOCKET
+  XMLTOKEN_SOCKET,
+  XMLTOKEN_FACTORY,
+  XMLTOKEN_ANIMCYCLE,
+  XMLTOKEN_IDLEANIM,
+  XMLTOKEN_IDLE
 };
 
-SCF_IMPLEMENT_IBASE (csSpriteCal3DFactoryLoader)
-SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSpriteCal3DFactoryLoader::eiComponent)
-SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSpriteCal3DFactorySaver)
-SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSpriteCal3DFactorySaver::eiComponent)
-SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSpriteCal3DLoader)
-SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSpriteCal3DLoader::eiComponent)
-SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSpriteCal3DSaver)
-SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSpriteCal3DSaver::eiComponent)
-SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csSpriteCal3DFactoryLoader)
 SCF_IMPLEMENT_FACTORY (csSpriteCal3DFactorySaver)
 SCF_IMPLEMENT_FACTORY (csSpriteCal3DLoader)
 SCF_IMPLEMENT_FACTORY (csSpriteCal3DSaver)
 
-
-csSpriteCal3DFactoryLoader::csSpriteCal3DFactoryLoader (iBase* pParent)
+csSpriteCal3DFactoryLoader::csSpriteCal3DFactoryLoader (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSpriteCal3DFactoryLoader::~csSpriteCal3DFactoryLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSpriteCal3DFactoryLoader::Initialize (iObjectRegistry* object_reg)
 {
   csSpriteCal3DFactoryLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
-  vfs    = CS_QUERY_REGISTRY (object_reg, iVFS);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
+  vfs    = csQueryRegistry<iVFS> (object_reg);
 
   xmltokens.Register ("path",           XMLTOKEN_PATH);
   xmltokens.Register ("scale",          XMLTOKEN_SCALE);
@@ -139,17 +111,18 @@ bool csSpriteCal3DFactoryLoader::Initialize (iObjectRegistry* object_reg)
 
 
 csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
+						iStreamSource*,
 						iLoaderContext* ldr_context, 
 						iBase* context)
 {
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-    iPluginManager));
-  csRef<iMeshObjectType> type (CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-    "crystalspace.mesh.object.sprite.cal3d", iMeshObjectType));
+  csRef<iPluginManager> plugin_mgr (
+    csQueryRegistry<iPluginManager> (object_reg));
+  csRef<iMeshObjectType> type (csQueryPluginClass<iMeshObjectType> (plugin_mgr,
+    "crystalspace.mesh.object.sprite.cal3d"));
   if (!type)
   {
-    type = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.mesh.object.sprite.cal3d",
-      iMeshObjectType);
+    type = csLoadPlugin<iMeshObjectType> (plugin_mgr, 
+      "crystalspace.mesh.object.sprite.cal3d");
   }
   if (!type)
   {
@@ -159,19 +132,23 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
     return 0;
   }
 
+  // sprcal3d is absolutely predicated upon having skeleton and misbehaves (or
+  // crashes) if the skeleton is missing, thus we must take care to only
+  // consider the mesh valid if the skeleton is present.
+  bool skel_present = false;
+
   // @@@ Temporary fix to allow to set actions for objects loaded
   // with impexp. Once those loaders move to another plugin this code
   // below should be removed.
   csRef<iMeshObjectFactory> fact;
   if (context)
-    fact = SCF_QUERY_INTERFACE (context, iMeshObjectFactory);
-  // DecRef of fact will be handled later.
+    fact = scfQueryInterface<iMeshObjectFactory> (context);
   // If there was no factory we create a new one.
   if (!fact)
     fact = type->NewFactory ();
 
   csRef<iSpriteCal3DFactoryState> newspr (
-    SCF_QUERY_INTERFACE (fact, iSpriteCal3DFactoryState));
+    scfQueryInterface<iSpriteCal3DFactoryState> (fact));
 
   if (!newspr->Create("dummy"))
   {
@@ -194,7 +171,8 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
       {
           bool rotate = child->GetAttributeValueAsBool("rotate_x_axis");
           bool invert = child->GetAttributeValueAsBool("flip_textures");
-          newspr->SetLoadFlags( rotate?LOADER_ROTATE_X_AXIS:0 | invert?LOADER_INVERT_V_COORD:0 );
+          newspr->SetLoadFlags( rotate?LOADER_ROTATE_X_AXIS:0
+	      | invert?LOADER_INVERT_V_COORD:0 );
           break;
       }
     case XMLTOKEN_PATH:
@@ -205,8 +183,8 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	else
 	{
 	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badpath",
-	    child,"dir is a required attribute of <path> token in cal3d files.");
+	    "crystalspace.spritecal3dfactoryloader.parse.badpath", child,
+	    "dir is a required attribute of <path> token in cal3d files.");
 	  return 0;
 	}
 	break;
@@ -217,8 +195,8 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	if (!scale)
 	{
 	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badvalue",
-	    child,"value is a required attribute of <scale> token in cal3d files.");
+	    "crystalspace.spritecal3dfactoryloader.parse.badvalue", child,
+	    "value is a required attribute of <scale> token in cal3d files.");
 	  return 0;
 	}
 	break;
@@ -247,7 +225,7 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	{
 	  if (!newspr->LoadCoreSkeleton(vfs,file))
 	  {
-        synldr->ReportError (
+	    synldr->ReportError (
   	      "crystalspace.spritecal3dfactoryloader.parse.badfile",
 	      child,"Could not load cal3d skeleton file <%s>.",file);
 
@@ -258,10 +236,13 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	else
 	{
 	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badfile",
-	    child,"file is a required attribute of <skeleton> token in cal3d files.");
+	    "crystalspace.spritecal3dfactoryloader.parse.badfile", child,
+	    "file is a required attribute of <skeleton> token in cal3d "
+	    "files.");
 	  return 0;
 	}
+	// We've loaded the skeleton
+	skel_present = true;
 	break;
       }
     case XMLTOKEN_ANIMATION:
@@ -271,32 +252,33 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	if (!name)
 	{
 	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badfile",
-	    child,"name is a required attribute of <animation> token in cal3d files.");
+	    "crystalspace.spritecal3dfactoryloader.parse.badfile", child,
+	    "name is a required attribute of <animation> token in cal3d "
+	    "files.");
 	  return 0;
 	}
 	int type;
-    csString ctype = child->GetAttributeValue("type");
-    if (ctype=="idle")
-      type = iSpriteCal3DState::C3D_ANIM_TYPE_IDLE;
-    else if (ctype == "travel")
-      type = iSpriteCal3DState::C3D_ANIM_TYPE_TRAVEL;
-    else if (ctype == "cycle")
-      type = iSpriteCal3DState::C3D_ANIM_TYPE_CYCLE;
-    else if (ctype == "style_cycle")
-      type = iSpriteCal3DState::C3D_ANIM_TYPE_STYLE_CYCLE;
-    else if (ctype == "action")
-      type = iSpriteCal3DState::C3D_ANIM_TYPE_ACTION;
-    else
-      type = iSpriteCal3DState::C3D_ANIM_TYPE_NONE;
+        csString ctype = child->GetAttributeValue("type");
+        if (ctype=="idle")
+          type = iSpriteCal3DState::C3D_ANIM_TYPE_IDLE;
+        else if (ctype == "travel")
+          type = iSpriteCal3DState::C3D_ANIM_TYPE_TRAVEL;
+        else if (ctype == "cycle")
+          type = iSpriteCal3DState::C3D_ANIM_TYPE_CYCLE;
+        else if (ctype == "style_cycle")
+          type = iSpriteCal3DState::C3D_ANIM_TYPE_STYLE_CYCLE;
+        else if (ctype == "action")
+          type = iSpriteCal3DState::C3D_ANIM_TYPE_ACTION;
+        else
+          type = iSpriteCal3DState::C3D_ANIM_TYPE_NONE;
 
-	float base_vel = child->GetAttributeValueAsInt("base_vel");
+        float base_vel = child->GetAttributeValueAsInt("base_vel");
 	float min_vel = child->GetAttributeValueAsFloat("min_vel");
 	float max_vel = child->GetAttributeValueAsFloat("max_vel");
 	int  max_interval = child->GetAttributeValueAsInt("max_random");
 	int  min_interval = child->GetAttributeValueAsInt("min_random");
 	int  idle_pct     = child->GetAttributeValueAsInt("idle_pct");
-    bool lock         = child->GetAttributeValueAsBool("lock");
+        bool lock         = child->GetAttributeValueAsBool("lock");
 	if (file)
 	{
 	  int animID = newspr->LoadCoreAnimation(vfs,file,
@@ -305,25 +287,26 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	    base_vel,
 	    min_vel,
 	    max_vel,
-        min_interval,
-        max_interval,
-        idle_pct, lock);
+            min_interval,
+            max_interval,
+            idle_pct, lock);
 
 	  if (animID == -1)
 	  {
-        synldr->ReportError (
+            synldr->ReportError (
   	      "crystalspace.spritecal3dfactoryloader.parse.badfile",
 	      child,"Could not load cal3d anim file <%s>.",file);
 
-        newspr->ReportLastError();
+            newspr->ReportLastError();
 	    return 0;
 	  }
 	}
 	else
 	{
 	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badfile",
-	    child,"file is a required attribute of <animation> token in cal3d files.");
+	    "crystalspace.spritecal3dfactoryloader.parse.badfile", child,
+	    "file is a required attribute of <animation> token in cal3d "
+	    "files.");
 	  return 0;
 	}
 	break;
@@ -348,14 +331,15 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	    mat = LoadMaterialTag(newspr,child,ldr_context,def_matl,def_matl);
 	  }
 	  int mesh_index = newspr->LoadCoreMesh(vfs,file,name,attach,mat);
-      if (mesh_index == -1)
+          if (mesh_index == -1)
 	  {
-	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badfile",
-	    child,"Could not load mesh file <%s>.",file);
-	    newspr->ReportLastError();
-	    return 0;
+	      synldr->ReportError (
+	        "crystalspace.spritecal3dfactoryloader.parse.badfile",
+	        child,"Could not load mesh file <%s>.",file);
+	        newspr->ReportLastError();
+	        return 0;
 	  }
+
           csRef<iDocumentNodeIterator> child_it = child->GetNodes ();
           while (child_it->HasNext ())
           {
@@ -371,7 +355,8 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
                 const char *morph_name = childchild->GetAttributeValue("name");
                 if (morph_file)
                 {
-                  int morph_index = newspr->LoadCoreMorphTarget(vfs,mesh_index,morph_file,morph_name);
+                  int morph_index = newspr->LoadCoreMorphTarget(
+		      vfs,mesh_index,morph_file,morph_name);
                   if (morph_index == -1)
                   {
                     newspr->ReportLastError();
@@ -385,8 +370,8 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
 	else
 	{
 	  synldr->ReportError (
-	    "crystalspace.spritecal3dfactoryloader.parse.badfile",
-	    child,"file is a required attribute of <mesh> token in cal3d files.");
+	    "crystalspace.spritecal3dfactoryloader.parse.badfile", child,
+	    "file is a required attribute of <mesh> token in cal3d files.");
 	  return 0;
 	}
 	break;
@@ -407,7 +392,8 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
             case XMLTOKEN_MORPHTARGET:
             {
               const char *mesh_name = childchild->GetAttributeValue("mesh");
-              const char *morph_name = childchild->GetAttributeValue("morphtarget");
+              const char *morph_name =
+		childchild->GetAttributeValue("morphtarget");
               newspr->AddMorphTarget(morphanimationid,mesh_name,morph_name);
             }
           }
@@ -416,13 +402,24 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
       }
     case XMLTOKEN_MATERIAL:
       {
+#ifdef CS_DEBUG
+	static bool styleReminder = true;
+	if (styleReminder)
+	{
+	  synldr->Report ("crystalspace.mesh.loader.factory.sprite.cal3d", 
+	    CS_REPORTER_SEVERITY_NOTIFY, child, 
+	    "Material definitions in meshes is not considered good style, "
+	    "if you need to define a mesh and material at the same time use "
+	    "libraries.");
+	  styleReminder = false;
+	}
+#endif
         const char *file = child->GetAttributeValue("file");
         const char *matName = child->GetAttributeValue("name");
         if (!LoadMaterialTag(newspr,child,ldr_context,file, matName))
           return 0;
         break;
       }
-
     case XMLTOKEN_SOCKET:
       {
         int a = child->GetAttributeValueAsInt ("tri");
@@ -433,19 +430,27 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
         sprite_socket->SetTriangleIndex (a);
         sprite_socket->SetSubmeshIndex (submesh);
         sprite_socket->SetMeshIndex (mesh);
-    
         break;
       }
-
-      break;
-
     default:
       synldr->ReportBadToken (child);
       return 0;
     }
   }
+	
+  // If we haven't loaded the skeleton, report error and abort load.
+  if (!skel_present)
+  {
+    synldr->ReportError(
+      "crystalspace.spritecal3dfactoryloader.parse.badfile",
+      node, "No <skeleton> token found in cal3d file.");
+    return 0;
+  }
+    
   if (scale)
-    newspr->RescaleFactory(scale);
+    newspr->RescaleFactory(scale);  // this calls the function below itself
+  else
+    newspr->CalculateAllBoneBoundingBoxes();
 
   // Wrapup cal3d initialization
   newspr->BindMaterials();
@@ -453,11 +458,12 @@ csPtr<iBase> csSpriteCal3DFactoryLoader::Parse (iDocumentNode* node,
   return csPtr<iBase> (fact);
 }
 
-iMaterialWrapper *csSpriteCal3DFactoryLoader::LoadMaterialTag(iSpriteCal3DFactoryState *newspr,
-							      iDocumentNode* child,
-							      iLoaderContext* ldr_context,
-							      const char *file, 
-                                  const char* name)
+iMaterialWrapper *csSpriteCal3DFactoryLoader::LoadMaterialTag(
+	iSpriteCal3DFactoryState *newspr,
+	iDocumentNode* child,
+	iLoaderContext* ldr_context,
+	const char *file, 
+	const char* name)
 {
   iMaterialWrapper* mat=0;
   if (file)
@@ -480,8 +486,8 @@ iMaterialWrapper *csSpriteCal3DFactoryLoader::LoadMaterialTag(iSpriteCal3DFactor
   else
   {
     synldr->ReportError (
-      "crystalspace.spritecal3dfactoryloader.parse.badfile",
-      child,"file is a required attribute of <material> token in cal3d files.");
+      "crystalspace.spritecal3dfactoryloader.parse.badfile", child,
+      "file is a required attribute of <material> token in cal3d files.");
     return 0;
   }
   return mat;
@@ -490,76 +496,219 @@ iMaterialWrapper *csSpriteCal3DFactoryLoader::LoadMaterialTag(iSpriteCal3DFactor
 
 //---------------------------------------------------------------------------
 
-csSpriteCal3DFactorySaver::csSpriteCal3DFactorySaver (iBase* pParent)
+csSpriteCal3DFactorySaver::csSpriteCal3DFactorySaver (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSpriteCal3DFactorySaver::~csSpriteCal3DFactorySaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSpriteCal3DFactorySaver::Initialize (iObjectRegistry* object_reg)
 {
   csSpriteCal3DFactorySaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
   return true;
 }
 
 #define MAXLINE 100 /* max number of chars per line... */
 
-
-void csSpriteCal3DFactorySaver::WriteDown (iBase*, iFile*)
+//TBD
+bool csSpriteCal3DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
+	iStreamSource*)
 {
-}
-
-
-
-
+  if (!parent || !obj)
+    return false;
+ 
+  csRef<iDocumentNode> paramsNode =
+    parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+  paramsNode->SetValue("params");
 
+  paramsNode->CreateNodeBefore(CS_NODE_COMMENT, 0)->SetValue
+    ("iSaverPlugin not yet supported for cal3d mesh");
+ 
+  csRef<iSpriteCal3DFactoryState> cal3dfact = 
+    scfQueryInterface<iSpriteCal3DFactoryState> (obj);
+  csRef<iMeshObjectFactory> meshfact =
+    scfQueryInterface<iMeshObjectFactory> (obj);
+#if 0
+  if ( cal3dfact && meshfact )
+  {
+    //Write Option Tag
+    int flags = cal3dfact->GetLoadFlags();
+    bool rotate = flags & LOADER_ROTATE_X_AXIS;
+    bool invert = flags & LOADER_INVERT_V_COORD;
+    csRef<iDocumentNode> optionNode =
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    optionNode->SetValue("socket");
+    optionNode->SetAttribute("rotate_x_axis", (rotate)?"yes":"no");
+    optionNode->SetAttribute("flip_textures", (invert)?"yes":"no");
+ 
+    //Write Path Tag
+    const char* dir = cal3dfact->GetBasePath();
+    csRef<iDocumentNode> pathNode =
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    pathNode->SetValue("path");
+    pathNode->SetAttribute("dir", dir);
+ 
+    //Write Scale Tag
+    float scale = cal3dfact->GetScale();
+    csRef<iDocumentNode> scaleNode =
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    scaleNode->SetValue("scale");
+    scaleNode->SetAttributeAsFloat("value", scale);
+ 
+    //Write Skeleton Tag
+    const char* skeleton = cal3dfact->GetCoreSkeleton()->GetFileName();
+    csRef<iDocumentNode> pathNode =
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    pathNode->SetValue("skeleton");
+    pathNode->SetAttribute("file", skeleton);
+ 
+    //Write Material Tag
+    const char* file = cal3dfact->Get...
+    const char* matName = cal3dfact->Get...
+    csRef<iDocumentNode> pathNode =
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    pathNode->SetValue("material");
+    child->SetAttribute("file", file );
+    child->SetAttribute("name", matName);
+ 
+    //Write Mesh Tags
+    for (int i=0; i<cal3dfact->GetMeshCount(); i++)
+    {
+      csRef<iDocumentNode> meshNode =
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      meshNode->SetValue("mesh");
+ 
+      const char* meshname = cal3dfact->GetMeshName(i);
+      meshNode->SetAttribute("name", meshname);
+ 
+      //Write MorphTargets Tags
+      for (int j=0; j<cal3dfact->GetMorphTargetCount(i); j++)
+      {
+        const char *morph_file, *morph_name;
+        cal3dfact->GetCoreMorphTarget(i,morph_file,morph_name);
+ 
+        csRef<iDocumentNode> morphtargetNode =
+          meshNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        morphtargetNode->SetValue("morphtarget");
+        meshNode->SetAttribute("name", morph_name);
+        meshNode->SetAttribute("file", morph_file);
+      }
+ 
+    }
+ 
+    //Write MorphAnimation Tags
+    for (int i=0; i<cal3dfact->GetMorphAnimationCount(); i++)
+    {
+      csRef<iDocumentNode> morphNode =
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      morphNode->SetValue("morphanimation");
+ 
+      const char* morphname = cal3dfact->GetMorphAnimationName(i);
+      meshNode->SetAttribute("name", morphname);
+ 
+      //Write MorphTargets Tags
+      for (int j=0; j<cal3dfact->GetMorphTargetCount(i); j++)
+      {
+        const char *morph_file, *morph_name;
+        cal3dfact->GetCoreMorphTarget(i,morph_file,morph_name);
+ 
+        csRef<iDocumentNode> morphtargetNode =
+          meshNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        morphtargetNode->SetValue("morphtarget");
+        meshNode->SetAttribute("name", morph_name);
+        meshNode->SetAttribute("file", morph_file);
+      }
+    }
+ 
+    //Write Animation Tags
+    for (int i=0; i<cal3dfact->GetAnimationCount(); i++)
+    {
+      const char *name, *file;
+      bool lock;
+      int  type, max_interval, min_interval, idle_pct;
+      float base_vel, min_vel, max_vel;
+      cal3dfact->GetCoreAnimation(i, file, name, type, base_vel,
+	min_vel, max_vel, min_interval, max_interval, idle_pct, lock);
+ 
+      csRef<iDocumentNode> aniNode =
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      aniNode->SetValue("animation");
+ 
+      aniNode->SetAttribute("name", name);
+      aniNode->SetAttribute("file", file);
+ 
+      if (type == iSpriteCal3DState::C3D_ANIM_TYPE_IDLE)
+        aniNode->SetAttribute("type", "idle");
+      else if (type == iSpriteCal3DState::C3D_ANIM_TYPE_TRAVEL)
+        aniNode->SetAttribute("type", "travel");
+      else if (type == iSpriteCal3DState::C3D_ANIM_TYPE_CYCLE)
+        aniNode->SetAttribute("type", "cycle");
+      else if (type == iSpriteCal3DState::C3D_ANIM_TYPE_STYLE_CYCLE)
+        aniNode->SetAttribute("type", "style_cycle");
+      else if (type == iSpriteCal3DState::C3D_ANIM_TYPE_ACTION)
+        aniNode->SetAttribute("type", "action");
+ 
+      aniNode->SetAttributeAsInt("base_vel", base_vel);
+      aniNode->SetAttributeAsFloat("min_vel", min_vel);
+      aniNode->SetAttributeValueAsFloat("max_vel", max_vel);
+      aniNode->SetAttributeValueAsInt("max_random", max_interval);
+      aniNode->SetAttributeValueAsInt("min_random", min_interval);
+      aniNode->SetAttributeValueAsInt("idle_pct", idle_pct);
+      aniNode->SetAttributeValueAsBool("lock", lock);
+    }
+ 
+    //Write Socket Tags
+    for (int i=0; i<cal3dfact->GetSocketCount(); i++)
+    {
+      csRef<iDocumentNode> socketNode =
+	paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      socketNode->SetValue("socket");
+      iSpriteCal3DSocket* sprite_socket = cal3dfact->GetSocket(i);
+      socketNode->SetAttribute("name", sprite_socket->GetName());
+      socketNode->SetAttributeAsInt("tri", sprite_socket->GetTriangleIndex());
+      socketNode->SetAttributeAsInt("submesh",
+        sprite_socket->GetSubmeshIndex());
+      socketNode->SetAttributeAsInt("mesh", sprite_socket->GetMeshIndex());
+    }
+ 
+  }
+#endif
+  return true;
+}  
 
 //---------------------------------------------------------------------------
 
-csSpriteCal3DLoader::csSpriteCal3DLoader (iBase* pParent)
+csSpriteCal3DLoader::csSpriteCal3DLoader (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSpriteCal3DLoader::~csSpriteCal3DLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSpriteCal3DLoader::Initialize (iObjectRegistry* object_reg)
 {
-#if 0
-  csSprite3DLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  csSpriteCal3DLoader::object_reg = object_reg;
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
-  xmltokens.Register ("action", XMLTOKEN_ACTION);
-  xmltokens.Register ("basecolor", XMLTOKEN_BASECOLOR);
   xmltokens.Register ("factory", XMLTOKEN_FACTORY);
-  xmltokens.Register ("lighting", XMLTOKEN_LIGHTING);
-  xmltokens.Register ("material", XMLTOKEN_MATERIAL);
-  xmltokens.Register ("mixmode", XMLTOKEN_MIXMODE);
-  xmltokens.Register ("tween", XMLTOKEN_TWEEN);
-#endif
+  xmltokens.Register ("animcycle", XMLTOKEN_ANIMCYCLE);
+  xmltokens.Register ("idleanim", XMLTOKEN_IDLEANIM);
+  xmltokens.Register ("idle", XMLTOKEN_IDLE);
 
   return true;
 }
 
 csPtr<iBase> csSpriteCal3DLoader::Parse (iDocumentNode* node,
+					 iStreamSource*,
 					 iLoaderContext* ldr_context, iBase*)
 {
-#if 0
   csRef<iMeshObject> mesh;
-  csRef<iSprite3DState> spr3dLook;
+  csRef<iSpriteCal3DState> sprCal3dLook;
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -577,116 +726,71 @@ csPtr<iBase> csSpriteCal3DLoader::Parse (iDocumentNode* node,
 	if (!fact)
 	{
 	  synldr->ReportError (
-	    "crystalspace.sprite3dloader.parse.unknownfactory",
+	    "crystalspace.spritecal3dloader.parse.unknownfactory",
 	    child, "Couldn't find factory '%s'!", factname);
 	  return 0;
 	}
 	mesh = fact->GetMeshObjectFactory ()->NewInstance ();
-	spr3dLook = SCF_QUERY_INTERFACE (mesh, iSprite3DState);
+	sprCal3dLook = scfQueryInterface<iSpriteCal3DState> (mesh);
       }
       break;
-    case XMLTOKEN_ACTION:
-      if (!spr3dLook)
+    case XMLTOKEN_ANIMCYCLE:
+      if (!sprCal3dLook)
       {
 	synldr->ReportError (
-	  "crystalspace.sprite3dloader.parse.motion.missingfactory",
+	  "crystalspace.spritecal3dloader.parse.motion.missingfactory",
 	  child,
-	  "No Factory! Please define 'factory' before 'applymotion'!");
+	  "No Factory! Please define 'factory' before 'animcycle'!");
 	return 0;
       }
-      spr3dLook->SetAction (child->GetContentsValue ());
-      break;
-    case XMLTOKEN_BASECOLOR:
-      if (!spr3dLook)
+      if (sprCal3dLook->FindAnim(child->GetContentsValue ()) != -1)
       {
-	synldr->ReportError (
-	  "crystalspace.sprite3dloader.parse.motion.missingfactory",
-	  child,
-	  "No Factory! Please define 'factory' before 'applymotion'!");
-	return 0;
+         sprCal3dLook->SetAnimCycle (child->GetContentsValue (),1.0);
       }
       else
       {
-	csColor col;
-	if (!synldr->ParseColor (child, col))
-	  return 0;
-	spr3dLook->SetBaseColor (col);
+	synldr->ReportError (
+	  "crystalspace.spritecal3dloader.parse.motion.missingfactory",
+	  child,
+	  "Anim cycle not found!!!");
+	  // Non fatal error?
       }
+	 
       break;
-    case XMLTOKEN_LIGHTING:
-      if (!spr3dLook)
+    case XMLTOKEN_IDLEANIM:
+      if (!sprCal3dLook)
       {
 	synldr->ReportError (
-	  "crystalspace.sprite3dloader.parse.motion.missingfactory",
+	  "crystalspace.spritecal3dloader.parse.motion.missingfactory",
 	  child,
-	  "No Factory! Please define 'factory' before 'applymotion'!");
+	  "No Factory! Please define 'factory' before 'idleanim'!");
 	return 0;
       }
+      if (sprCal3dLook->FindAnim(child->GetContentsValue ()) != -1)
+      {
+      	sprCal3dLook->SetDefaultIdleAnim (child->GetContentsValue ());
+      }
       else
       {
-	bool do_lighting;
-	if (!synldr->ParseBool (child, do_lighting, true))
-	  return 0;
-	spr3dLook->SetLighting (do_lighting);
+	synldr->ReportError (
+	  "crystalspace.spritecal3dloader.parse.motion.missingfactory",
+	  child,
+	  "Anim cycle not found!!!");
+	  // Non fatal error?
       }
       break;
-    case XMLTOKEN_MATERIAL:
-      if (!spr3dLook)
+    case XMLTOKEN_IDLE:
+      if (!sprCal3dLook)
       {
 	synldr->ReportError (
-	  "crystalspace.sprite3dloader.parse.motion.missingfactory",
+	  "crystalspace.spritecal3dloader.parse.motion.missingfactory",
 	  child,
-	  "No Factory! Please define 'factory' before 'applymotion'!");
+	  "No Factory! Please define 'factory' before 'idle'!");
 	return 0;
       }
-      else
-      {
-	const char* matname = child->GetContentsValue ();
-	iMaterialWrapper* mat = ldr_context->FindMaterial (matname);
-	if (!mat)
-	{
-	  synldr->ReportError (
-	    "crystalspace.sprite3dloader.parse.unknownmaterial",
-	    child, "Couldn't find material '%s'!", matname);
-	  return 0;
-	}
-	spr3dLook->SetMaterialWrapper (mat);
-      }
-      break;
-    case XMLTOKEN_MIXMODE:
-      if (!spr3dLook)
-      {
-	synldr->ReportError (
-	  "crystalspace.sprite3dloader.parse.motion.missingfactory",
-	  child,
-	  "No Factory! Please define 'factory' before 'applymotion'!");
-	return 0;
-      }
-      else
-      {
-	uint mm;
-	if (!synldr->ParseMixmode (child, mm))
-	  return 0;
-	spr3dLook->SetMixMode (mm);
-      }
-      break;
-    case XMLTOKEN_TWEEN:
-      if (!spr3dLook)
-      {
-	synldr->ReportError (
-	  "crystalspace.sprite3dloader.parse.motion.missingfactory",
-	  child,
-	  "No Factory! Please define 'factory' before 'applymotion'!");
-	return 0;
-      }
-      else
-      {
-	bool do_tween;
-	if (!synldr->ParseBool (child, do_tween, true))
-	  return 0;
-	spr3dLook->EnableTweening (do_tween);
-      }
+      sprCal3dLook->SetVelocity(0);
       break;
+
     default:
       synldr->ReportBadToken (child);
       return 0;
@@ -694,31 +798,39 @@ csPtr<iBase> csSpriteCal3DLoader::Parse (iDocumentNode* node,
   }
 
   return csPtr<iBase> (mesh);
-#endif
-  return 0;
 }
 
 //---------------------------------------------------------------------------
 
-csSpriteCal3DSaver::csSpriteCal3DSaver (iBase* pParent)
+csSpriteCal3DSaver::csSpriteCal3DSaver (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSpriteCal3DSaver::~csSpriteCal3DSaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSpriteCal3DSaver::Initialize (iObjectRegistry* object_reg)
 {
   csSpriteCal3DSaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
   return true;
 }
 
-void csSpriteCal3DSaver::WriteDown (iBase*, iFile*)
+bool csSpriteCal3DSaver::WriteDown (iBase* /*obj*/, iDocumentNode* parent,
+	iStreamSource*)
 {
+  if (!parent)
+    return false;
+  
+  csRef<iDocumentNode> paramsNode =
+    parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+  paramsNode->SetValue("params");
+  paramsNode->CreateNodeBefore(CS_NODE_COMMENT, 0)->SetValue
+    ("iSaverPlugin not yet supported for cal3d mesh");
+  paramsNode=0;
+  
+  return true;
 }
+
+} // namespace cspluginSprCal3dLdr
