@@ -23,7 +23,9 @@
 #include "csgeom/transfrm.h"
 #include "csgeom/objmodel.h"
 #include "csutil/cscolor.h"
+#include "csutil/flags.h"
 #include "csutil/refarr.h"
+
 #include "imesh/object.h"
 #include "imesh/stars.h"
 #include "ivideo/graph3d.h"
@@ -40,7 +42,7 @@ class csStarsMeshObject : public iMeshObject
 {
 private:
   iMeshObjectFactory* factory;
-  iBase* logparent;
+  iMeshWrapper* logparent;
   csBox3 box;
   iMeshObjectDrawCallback* vis_cb;
   csColor color;
@@ -119,7 +121,8 @@ public:
   /// Return true if max color is used.
   bool IsMaxColorUsed () const { return use_max_color; }
 
-  void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL);
+  void GetObjectBoundingBox (csBox3& bbox);
+  void SetObjectBoundingBox (const csBox3& bbox);
   void GetRadius (csVector3& rad, csVector3& cent)
   { rad = max_radius; cent = box.GetCenter(); }
 
@@ -129,11 +132,8 @@ public:
   virtual iMeshObjectFactory* GetFactory () const { return factory; }
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone () { return 0; }
-  virtual bool DrawTest (iRenderView* rview, iMovable* movable,
-  	uint32 frustum_mask);
   virtual csRenderMesh **GetRenderMeshes (int &n, iRenderView*,
     iMovable*, uint32) { n = 0; return 0; }
-  virtual bool Draw (iRenderView* rview, iMovable* movable, csZBufMode mode);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
     if (cb) cb->IncRef ();
@@ -144,24 +144,29 @@ public:
   {
     return vis_cb;
   }
-  virtual void NextFrame (csTicks /*current_time*/, const csVector3& /*pos*/) { }
+  virtual void NextFrame (csTicks /*current_time*/, const csVector3& /*pos*/)
+  { }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
   virtual bool HitBeamOutline (const csVector3&, const csVector3&,
         csVector3&, float*)
   { return false; }
   virtual bool HitBeamObject (const csVector3&, const csVector3&,
-  	csVector3&, float*, int* = 0) { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  	csVector3&, float*, int* = 0, iMaterialWrapper** = 0) { return false; }
+  virtual void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
+  virtual iMeshWrapper* GetMeshWrapper () const { return logparent; }
 
   //------------------------- iObjectModel implementation ----------------
   class ObjectModel : public csObjectModel
   {
     SCF_DECLARE_EMBEDDED_IBASE (csStarsMeshObject);
-    virtual void GetObjectBoundingBox (csBox3& bbox, int type = CS_BBOX_NORMAL)
+    virtual void GetObjectBoundingBox (csBox3& bbox)
     {
-      scfParent->GetObjectBoundingBox (bbox, type);
+      scfParent->GetObjectBoundingBox (bbox);
+    }
+    virtual void SetObjectBoundingBox (const csBox3& bbox)
+    {
+      scfParent->SetObjectBoundingBox (bbox);
     }
     virtual void GetRadius (csVector3& rad, csVector3& cent)
     {
@@ -175,12 +180,14 @@ public:
   virtual bool GetColor (csColor& col) const { col = color; return true; }
   virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
   virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
+  virtual void SetMixMode (uint) { }
+  virtual uint GetMixMode () const { return CS_FX_COPY; }
   virtual void InvalidateMaterialHandles () { }
   /**
    * see imesh/object.h for specification. The default implementation
    * does nothing.
    */
-  virtual void PositionChild (iMeshObject* child,csTicks current_time) { }
+  virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
 
   //------------------------- iStarsState implementation ----------------
   class StarsState : public iStarsState
@@ -242,12 +249,13 @@ public:
 class csStarsMeshObjectFactory : public iMeshObjectFactory
 {
 private:
-  iBase* logparent;
+  iMeshFactoryWrapper* logparent;
+  iMeshObjectType* stars_type;
   csFlags flags;
 
 public:
   /// Constructor.
-  csStarsMeshObjectFactory (iBase *pParent);
+  csStarsMeshObjectFactory (iMeshObjectType* pParent);
 
   /// Destructor.
   virtual ~csStarsMeshObjectFactory ();
@@ -260,8 +268,11 @@ public:
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
-  virtual void SetLogicalParent (iBase* lp) { logparent = lp; }
-  virtual iBase* GetLogicalParent () const { return logparent; }
+  virtual void SetMeshFactoryWrapper (iMeshFactoryWrapper* lp)
+  { logparent = lp; }
+  virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
+  { return logparent; }
+  virtual iMeshObjectType* GetMeshObjectType () const { return stars_type; }
   virtual iObjectModel* GetObjectModel () { return 0; }
 };
 
