@@ -15,28 +15,37 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#include <math.h>
 #include "cssysdef.h"
-#include "qint.h"
-#include "qsqrt.h"
-#include "csgeom/polyclip.h"
-#include "csgeom/polyaa.h"
-#include "csgeom/poly3d.h"
+#include <math.h>
+
+#include "csqint.h"
+#include "csqsqrt.h"
+
+#include "csutil/debug.h"
 #include "csgeom/frustum.h"
+#include "csgeom/math.h"
+#include "csgeom/poly3d.h"
+#include "csgeom/polyaa.h"
+#include "csgeom/polyclip.h"
+
+#include "iengine/engine.h"
+#include "iengine/light.h"
+#include "iengine/material.h"
+#include "iengine/movable.h"
+#include "iengine/shadows.h"
+#include "iengine/texture.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/material.h"
+#include "ivideo/texture.h"
+
 #include "thing.h"
 #include "polytext.h"
 #include "polygon.h"
 #include "lppool.h"
 #include "lghtmap.h"
-#include "iengine/texture.h"
-#include "iengine/material.h"
-#include "iengine/shadows.h"
-#include "iengine/light.h"
-#include "iengine/movable.h"
-#include "csutil/debug.h"
-#include "ivideo/texture.h"
-#include "ivideo/material.h"
-#include "ivideo/graph3d.h"
+
+namespace cspluginThing
+{
 
 // Option variable: cosinus factor.
 float csPolyTexture::cfg_cosinus_factor = 0;
@@ -88,18 +97,17 @@ bool csPolyTexture::RecalculateDynamicLights (
 	const csMatrix3& m_world2tex,
 	const csVector3& v_world2tex,
 	csPolygon3D* polygon,
-	const csPlane3& polygon_world_plane)
+	const csPlane3& polygon_world_plane,
+	const csColor& amb,
+	csLightingScratchBuffer& finalLM)
 {
   if (!lm) return false;
 
   // first combine the static and pseudo-dynamic lights
   csThing* thing = polygon->GetParent ();
-  csColor amb = thing->GetDynamicAmbientLight();
 
-  if (!lm->UpdateRealLightMap (amb.red,
-      amb.green,
-      amb.blue,
-      thing->GetLightVersion () != light_version ))
+  if (!lm->UpdateRealLightMap (amb.red, amb.green, amb.blue,
+      thing->GetLightVersion () != light_version, finalLM))
     return false;
 
   light_version = thing->GetLightVersion ();
@@ -111,7 +119,7 @@ bool csPolyTexture::RecalculateDynamicLights (
   while (lp)
   {
     ShineDynLightMap (lp, m_world2tex, v_world2tex, polygon,
-    	polygon_world_plane);
+    	polygon_world_plane, finalLM);
     lp = lp->GetNext ();
   }
 
@@ -236,7 +244,7 @@ static bool CanCastShadow (
 }
 
 
-void csPolyTexture::FillLightMap (
+bool csPolyTexture::FillLightMap (
   iFrustumView *lview,
   csLightingPolyTexQueue* lptq,
   bool vis,
@@ -246,7 +254,7 @@ void csPolyTexture::FillLightMap (
   const csPlane3& subpoly_plane,
   csPolygon3DStatic* spoly)
 {
-  if (!lm) return ;
+  if (!lm) return false;
 
   int j;
 
@@ -254,9 +262,9 @@ void csPolyTexture::FillLightMap (
   csVector3 &lightpos = light_frustum->GetOrigin ();
   float inv_lightcell_size = 1.0f / csLightMap::lightcell_size;
   int ww, hh;
-  iMaterialHandle* mat_handle = spoly->GetMaterialHandle ();
-  if (mat_handle && mat_handle->GetTexture ())
-    mat_handle->GetTexture ()->GetMipMapDimensions (0, ww, hh);
+  iMaterial* mat = spoly->GetMaterial ();
+  if (mat && mat->GetTexture ())
+    mat->GetTexture ()->GetRendererDimensions (ww, hh);
   else
     ww = hh = 64;
 
@@ -273,7 +281,7 @@ void csPolyTexture::FillLightMap (
     lptq->AddPolyTexture (this, subpoly);
   }
 
-  if (shadow_bitmap->IsFullyShadowed ()) return ;
+  if (shadow_bitmap->IsFullyShadowed ()) return false;
 
   // Room for our shadow in lightmap space.
   csVector2 s2d[MAX_OUTPUT_VERTICES];
@@ -283,9 +291,12 @@ void csPolyTexture::FillLightMap (
     // If completely shadowed due to external causes (i.e. c-buffer)
     // then we just fill it as entirely shadowed.
     shadow_bitmap->RenderTotal (1);
+    return false;
   }
   else
   {
+    bool affect = true;
+  
     // We will now start rendering an optional light frustum and several
     // shadow frustums on our shadow bitmap. But first we will create a
     // clipping polygon on our polygon which is guaranteed to be larger
@@ -446,11 +457,12 @@ void csPolyTexture::FillLightMap (
         }
 
         shadow_bitmap->RenderPolygon (s2d, nv, 1);
-        if (shadow_bitmap->IsFullyShadowed ()) break;
+        if (shadow_bitmap->IsFullyShadowed ()) { affect = false; break; }
       }
     }
 
     shadow_it->DecRef ();
+    return affect;
   }
 }
 
@@ -459,24 +471,21 @@ void csPolyTexture::ShineDynLightMap (csLightPatch *lp,
   const csMatrix3& m_world2tex,
   const csVector3& v_world2tex,
   csPolygon3D* polygon,
-  const csPlane3& polygon_world_plane)
+  const csPlane3& polygon_world_plane,
+  csLightingScratchBuffer& finalLM)
 {
-  csPolyTextureMapping* tmapping = polygon->GetStaticPoly ()->GetTextureMapping ();
+  csPolyTextureMapping* tmapping = polygon->GetStaticPoly ()
+  	->GetTextureMapping ();
   int lw = 1 +
-    ((tmapping->GetLitOriginalWidth () + csLightMap::lightcell_size - 1) >>
+    ((tmapping->GetLitWidth () + csLightMap::lightcell_size - 1) >>
       csLightMap::lightcell_shift);
   int lh = 1 +
     ((tmapping->GetLitHeight () + csLightMap::lightcell_size - 1) >>
       csLightMap::lightcell_shift);
 
-  int u, uv;
-
-  int l1, l2 = 0, l3 = 0;
-  float d;
-
   int ww, hh;
-  iMaterialHandle* mat_handle = polygon->GetStaticPoly()->GetMaterialHandle();
-  mat_handle->GetTexture ()->GetMipMapDimensions (0, ww, hh);
+  iMaterial* mat = polygon->GetStaticPoly()->GetMaterial();
+  mat->GetTexture ()->GetRendererDimensions (ww, hh);
 
   float cosfact = polygon->GetParent ()->GetStaticData ()->GetCosinusFactor ();
   if (cosfact == -1) cosfact = cfg_cosinus_factor;
@@ -490,16 +499,12 @@ void csPolyTexture::ShineDynLightMap (csLightPatch *lp,
 
   csVector3 v1, v2;
 
-  int ru, rv;
   float invww, invhh;
   invww = 1.0f / (float)ww;
   invhh = 1.0f / (float)hh;
 
-  csRGBpixel* map = lm->GetRealMap ();
+  csRGBpixel* map = finalLM.GetArray();
   iLight *light = lp->GetLight ();
-#if defined(CS_DEBUG)
-  long lm_size = lm->GetWidth () * lm->GetHeight (); // Used only for asserts.
-#endif
 
   int i;
   float miny = 1000000, maxy = -1000000;
@@ -510,15 +515,16 @@ void csPolyTexture::ShineDynLightMap (csLightPatch *lp,
   if (lp->GetLightFrustum ())
     lightpos = lp->GetLightFrustum ()->GetOrigin ();
   else
-    lightpos = light->GetCenter ();
+    lightpos = light->GetMovable ()->GetFullPosition ();
+
+  float infradius_sq = csSquare (light->GetCutoffDistance ());
 
   // Calculate the uv's for all points of the frustum (the
   // frustum is actually a clipped version of the polygon).
-  csVector2 *f_uv = 0;
+  csVector2 f_uv[100];
   if (lp->GetVertices ())
   {
     int mi;
-    f_uv = new csVector2[lp->GetVertexCount ()];
     for (i = 0; i < lp->GetVertexCount (); i++)
     {
       //if (lview.IsMirrored ()) mi = lview.num_frustum-i-1;
@@ -534,6 +540,7 @@ void csPolyTexture::ShineDynLightMap (csLightPatch *lp,
       if (f_uv[i].y > maxy) maxy = f_uv[MaxIndex = i].y;
     }
   }
+  else return;
 
   csColor color = light->GetColor () * CS_NORMAL_LIGHT_LEVEL;
 
@@ -548,9 +555,9 @@ void csPolyTexture::ShineDynLightMap (csLightPatch *lp,
   scanL2 = scanR2 = MaxIndex;
 
   // sy = fyL = fyR =
-  //   (QRound (f_uv[scanL2].y)>lh-1)?lh-1:QRound (f_uv[scanL2].y);
-  sy = fyL = fyR = (QRound (ceil (f_uv[scanL2].y)) > lh - 1) ?
-    lh - 1 : QRound (ceil (f_uv[scanL2].y));
+  //   (csQround (f_uv[scanL2].y)>lh-1)?lh-1:csQround (f_uv[scanL2].y);
+  sy = fyL = fyR = (csQround (ceil (f_uv[scanL2].y)) > lh - 1) ?
+    lh - 1 : csQround (ceil (f_uv[scanL2].y));
 
   for (;;)
   {
@@ -565,7 +572,7 @@ void csPolyTexture::ShineDynLightMap (csLightPatch *lp,
       {
         // Check first if polygon has been finished
 a:
-        if (scanR2 == MinIndex) goto finish;
+        if (scanR2 == MinIndex) return;
         scanR1 = scanR2;
         scanR2 = (scanR2 + 1) % lp->GetVertexCount ();
 
@@ -575,11 +582,11 @@ a:
           goto a;
         }
 
-        /*      if (scanR2 == MinIndex) goto finish;
+        /*      if (scanR2 == MinIndex) return;
         scanR1 = scanR2;
         scanR2 = (scanR2 + 1) % lp->num_vertices;
 */
-        fyR = QRound (floor (f_uv[scanR2].y));
+        fyR = csQround (floor (f_uv[scanR2].y));
 
         float dyR = (f_uv[scanR1].y - f_uv[scanR2].y);
         sxR = f_uv[scanR1].x;
@@ -598,7 +605,7 @@ a:
       if (sy <= fyL)
       {
 b:
-        if (scanL2 == MinIndex) goto finish;
+        if (scanL2 == MinIndex) return;
         scanL1 = scanL2;
         scanL2 = (scanL2 - 1 + lp->GetVertexCount ()) % lp->GetVertexCount ();
 
@@ -611,7 +618,7 @@ b:
         //scanL1 = scanL2;
 
         //scanL2 = (scanL2 - 1 + lp->num_vertices) % lp->num_vertices;
-        fyL = QRound (floor (f_uv[scanL2].y));
+        fyL = csQround (floor (f_uv[scanL2].y));
 
         float dyL = (f_uv[scanL1].y - f_uv[scanL2].y);
         sxL = f_uv[scanL1].x;
@@ -641,7 +648,7 @@ b:
       // @@@ The check below should not be needed but it is.
       if (sy < 0)
       {
-        goto finish;
+        return;
       }
 
       // Compute the rounded screen coordinates of horizontal strip
@@ -654,112 +661,374 @@ b:
         _l = _;
       }
 
-      xL = 1 + QRound (ceil (_l));
-      xR = QRound (floor (_r));
+      xL = 1 + csQround (ceil (_l));
+      xR = csQround (floor (_r));
 
       //if (xR > xL) { int xswap = xR; xR = xL; xL = xswap; }
       if (xR < 0) xR = 0;
       if (xL > lw) xL = lw;
 
-      for (u = xR; u < xL; u++)
-      {
-        uv = sy * new_lw + u;
+      v1.x = float ((xR << csLightMap::lightcell_shift)
+      	+ tmapping->GetIMinU ()) * invww;
+      v1.y = float ((sy << csLightMap::lightcell_shift)
+      	+ tmapping->GetIMinV ()) * invhh;
+      v1.z = 0;
+      float dv1x = float (csLightMap::lightcell_size) * invww;
 
-        ru = u << csLightMap::lightcell_shift;
-        rv = sy << csLightMap::lightcell_shift;
+      int uv = sy * new_lw + xR;
+      csRGBpixel* map_uv = map + uv;
 
-        v1.x = (float)(ru + tmapping->GetIMinU ()) * invww;
-        v1.y = (float)(rv + tmapping->GetIMinV ()) * invhh;
-        v1.z = 0;
-        v2 = vv + m_t2w * v1;
+      // We have to calculate v2 = vv + m_t2w * v1 every time in the
+      // loop below. However, v1.z is 0 and v1.y never changes so we
+      // can optimize this a little bit.
+      v2.x = vv.x + m_t2w.m11 * v1.x + m_t2w.m12 * v1.y;
+      v2.y = vv.y + m_t2w.m21 * v1.x + m_t2w.m22 * v1.y;
+      v2.z = vv.z + m_t2w.m31 * v1.x + m_t2w.m32 * v1.y;
+      float dv2x_x = m_t2w.m11 * dv1x;
+      float dv2x_y = m_t2w.m21 * dv1x;
+      float dv2x_z = m_t2w.m31 * dv1x;
 
-        // Check if the point on the polygon is shadowed. To do this
-        // we traverse all shadow frustums and see if it is contained in
-  // any of them.
-        iShadowIterator *shadow_it = lp->GetShadowBlock ()
-    		->GetShadowIterator ();
-        bool shadow = false;
-        while (shadow_it->HasNext ())
-        {
-          csFrustum *shadow_frust = shadow_it->Next ();
-          if (
-            shadow_it->IsRelevant () &&
-            ((csPolygon3D *) (shadow_it->GetUserData ())) != polygon)
-            if (shadow_frust->Contains (v2 - shadow_frust->GetOrigin ()))
-            {
-              shadow = true;
-              break;
-            }
-        }
+      // We work relative to lightpos.
+      v2 -= lightpos;
 
-        shadow_it->DecRef ();
+      int du = xL - xR;
+      if (cosfact > 0.0001)
+        ShineDynLightMapHorizCosfact (du, map_uv, v2,
+	  dv2x_x, dv2x_y, dv2x_z,
+	  light, color, infradius_sq, cosfact,
+	  polygon_world_plane);
+      else
+        ShineDynLightMapHoriz (du, map_uv, v2,
+	  dv2x_x, dv2x_y, dv2x_z,
+	  light, color, infradius_sq,
+	  polygon_world_plane);
 
-        if (!shadow)
-        {
-          d = csSquaredDist::PointPoint (lightpos, v2);
-
-          if (d >= light->GetInfluenceRadiusSq ()) continue;
-          d = qsqrt (d);
-
-          float cosinus = (v2 - lightpos) * polygon_world_plane.Normal ();
-          cosinus /= d;
-          cosinus += cosfact;
-          if (cosinus < 0)
-            cosinus = 0;
-          else if (cosinus > 1)
-            cosinus = 1;
-
-          float brightness = cosinus * light->GetBrightnessAtDistance (d);
-
-          if (color.red > 0)
-          {
-            l1 = QRound (color.red * brightness);
-            if (l1)
-            {
-              CS_ASSERT (uv >= 0 && uv < lm_size);
-              l1 += map[uv].red;
-              if (l1 > 255) l1 = 255;
-              map[uv].red = l1;
-            }
-          }
-
-          if (color.green > 0)
-          {
-            l2 = QRound (color.green * brightness);
-            if (l2)
-            {
-              CS_ASSERT (uv >= 0 && uv < lm_size);
-              l2 += map[uv].green;
-              if (l2 > 255) l2 = 255;
-              map[uv].green = l2;
-            }
-          }
-
-          if (color.blue > 0)
-          {
-            l3 = QRound (color.blue * brightness);
-            if (l3)
-            {
-              CS_ASSERT (uv >= 0 && uv < lm_size);
-              l3 += map[uv].blue;
-              if (l3 > 255) l3 = 255;
-              map[uv].blue = l3;
-            }
-          }
-        }
-      }
-
-      if (!sy) goto finish;
+      if (!sy) return;
       sxL += dxL;
       sxR += dxR;
       sy--;
     }
   }
-
-finish:
-  delete[] f_uv;
 }
 
+#undef UPDATE_MAP
+#define UPDATE_MAP(cname) \
+      if (color.cname > 0) \
+      { \
+        float fl = color.cname * brightness; \
+        if (fl > 0.5) \
+        { \
+          int l = csQint (fl) + map_uv->cname; \
+          if (l > 255) l = 255; \
+          map_uv->cname = l; \
+        } \
+      }
+
+void csPolyTexture::ShineDynLightMapHorizCosfact (
+	int du, csRGBpixel* map_uv,
+	csVector3& v2,
+	float dv2x_x, float dv2x_y, float dv2x_z,
+	iLight* light, const csColor& color, float infradius_sq,
+	float cosfact,
+	const csPlane3& polygon_world_plane)
+{
+  float inv_dist;
+
+  // We select a different loop based on attenuation type for most
+  // effiency. This makes a huge difference.
+  switch (light->GetAttenuationMode ())
+  {
+    case CS_ATTN_INVERSE:
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          float inv_d = csQisqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          cosinus *= inv_d;
+          cosinus += cosfact;
+          if (cosinus < 0.02) continue;
+          else if (cosinus > 1) cosinus = 1;
+
+          float brightness = cosinus * inv_d;
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    case CS_ATTN_REALISTIC:
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float sqd = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (sqd < infradius_sq)
+        {
+          float inv_d = csQisqrt (sqd);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          cosinus *= inv_d;
+          cosinus += cosfact;
+          if (cosinus < 0.02) continue;
+          else if (cosinus > 1) cosinus = 1;
+
+          float brightness = cosinus / sqd;
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    case CS_ATTN_NONE:
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          float inv_d = csQisqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          cosinus *= inv_d;
+          cosinus += cosfact;
+	  float brightness;
+          if (cosinus < 0.001) continue;
+          else if (cosinus > 1) brightness = 1;
+	  else brightness = cosinus;
+
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    case CS_ATTN_LINEAR:
+      inv_dist = 1.0 / light->GetAttenuationConstants ().x;
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          d = csQsqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          cosinus /= d;
+          cosinus += cosfact;
+	  float brightness;
+          if (cosinus < 0.02) continue;
+          else if (cosinus > 1) brightness = 1.0 - d * inv_dist;
+	  else brightness = cosinus - cosinus * d * inv_dist;
+
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    default:
+      // Most general loop.
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          d = csQsqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          cosinus /= d;
+          cosinus += cosfact;
+          if (cosinus < 0.02) continue;
+          else if (cosinus > 1) cosinus = 1;
+
+          float brightness = cosinus * light->GetBrightnessAtDistance (d);
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+  }
+}
+
+void csPolyTexture::ShineDynLightMapHoriz (
+	int du, csRGBpixel* map_uv,
+	csVector3& v2,
+	float dv2x_x, float dv2x_y, float dv2x_z,
+	iLight* light, const csColor& color, float infradius_sq,
+	const csPlane3& polygon_world_plane)
+{
+  float inv_dist;
+
+  // We select a different loop based on attenuation type for most
+  // effiency. This makes a huge difference.
+  switch (light->GetAttenuationMode ())
+  {
+    case CS_ATTN_INVERSE:
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float brightness = v2 * polygon_world_plane.Normal ();
+          brightness /= d;
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    case CS_ATTN_REALISTIC:
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float sqd = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (sqd < infradius_sq)
+        {
+          float d = csQsqrt (sqd);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          float brightness = cosinus / (d * sqd);
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    case CS_ATTN_NONE:
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          float inv_d = csQisqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float brightness = v2 * polygon_world_plane.Normal ();
+          brightness *= inv_d;
+          if (brightness < 0.005) continue;
+
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    case CS_ATTN_LINEAR:
+      inv_dist = 1.0 / light->GetAttenuationConstants ().x;
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          float inv_d = csQisqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+	  float brightness = cosinus * (inv_d - inv_dist);
+
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+    default:
+      // Most general loop.
+      while (du > 0)
+      {
+        du--;
+        // Calculate squared distance between lightpos and transformed point.
+        float d = v2.x * v2.x + v2.y * v2.y + v2.z * v2.z;
+        if (d < infradius_sq)
+        {
+          d = csQsqrt (d);
+
+          // Remember that v2 is point on lightmap in 3D minutes lightpos.
+          float cosinus = v2 * polygon_world_plane.Normal ();
+          cosinus /= d;
+          float brightness = cosinus * light->GetBrightnessAtDistance (d);
+          if (brightness < 0.005) continue;
+	  UPDATE_MAP (red);
+	  UPDATE_MAP (green);
+	  UPDATE_MAP (blue);
+        }
+        v2.x += dv2x_x;
+        v2.y += dv2x_y;
+        v2.z += dv2x_z;
+        map_uv++;
+      }
+      break;
+  }
+}
+#undef UPDATE_MAP
+
 void csPolyTexture::UpdateFromShadowBitmap (
   iLight *light,
   const csVector3 &lightpos,
@@ -772,9 +1041,9 @@ void csPolyTexture::UpdateFromShadowBitmap (
   CS_ASSERT (shadow_bitmap != 0);
 
   int ww, hh;
-  iMaterialHandle* mat_handle = polygon->GetStaticPoly()->GetMaterialHandle();
-  if (mat_handle && mat_handle->GetTexture ())
-    mat_handle->GetTexture ()->GetMipMapDimensions (0, ww, hh);
+  iMaterial* mat = polygon->GetStaticPoly()->GetMaterial();
+  if (mat && mat->GetTexture ())
+    mat->GetTexture ()->GetRendererDimensions (ww, hh);
   else
     ww = hh = 64;
 
@@ -840,6 +1109,18 @@ void csPolyTexture::UpdateFromShadowBitmap (
   else
   {
     csRGBpixel *lightmap = lm->GetStaticMap ();
+    if (!lightmap)
+    {
+      lm->ReAlloc ();
+      lightmap = lm->GetStaticMap ();
+      csColor ambient;
+      polygon->GetParent ()->GetStaticData ()
+      	->thing_type->engine->GetAmbientLight (ambient);
+      lm->InitColor (
+          int(ambient.red * 255.0f),
+          int(ambient.green * 255.0f),
+          int(ambient.blue * 255.0f));
+    }
     shadow_bitmap->UpdateLightMap (
         lightmap,
         csLightMap::lightcell_shift,
@@ -1215,6 +1496,25 @@ void csShadowBitmap::UpdateLightMap (
   v_rv *= rv_step;
   v_ru *= ru_step;
 
+  csThing* poly_parent = poly->GetParent ();
+  csThingStatic* poly_parent_static = poly_parent->GetStaticData ();
+  bool do_smooth = poly_parent->GetStaticData ()->GetSmoothingFlag();
+  float* distances = 0;
+  csVector3* nearestNormals = 0;
+  csVector3* worldNormals = 0;
+  if (do_smooth)
+  {
+    distances = new float[100];
+    nearestNormals = new csVector3[100];
+    poly_parent->WorUpdate ();
+    csReversibleTransform mesh_transf = poly_parent->GetCachedMovable ()
+    	->GetFullTransform ();
+    worldNormals = new csVector3[poly_parent_static->num_vertices];
+    csVector3* normals = poly_parent_static->GetNormals ();
+    for (i = 0 ; i < poly_parent_static->num_vertices ; i++)
+      worldNormals[i] = mesh_transf.This2OtherRelative (normals[i]);
+  }
+
   for (i = 0; i < lm_h; i++)
   {
     int uv = base_uv;
@@ -1238,22 +1538,17 @@ void csShadowBitmap::UpdateLightMap (
       // @@@ Optimization: It should be possible to combine these
       // calculations into a more efficient formula.
       float d = csSquaredDist::PointPoint (lightpos, v);
-      if (d >= light->GetInfluenceRadiusSq ()) continue;
+      if (d >= csSquare (light->GetCutoffDistance ())) continue;
 
-      d = qsqrt (d);
+      d = csQsqrt (d);
 
       // Initialize normal with the flat one
       csVector3 normal = poly_world_plane.Normal ();
-      if (poly->GetParent ()->GetStaticData ()->GetSmoothingFlag())
+      if (do_smooth)
       {
-        csThing* poly_parent = poly->GetParent ();
 	int* vertexs = poly->GetStaticPoly ()->GetVertexIndices ();
-	csVector3* normals = poly_parent->GetStaticData ()
-	  ->GetNormals ();
 	int vCount = poly->GetStaticPoly ()->GetVertexCount ();
 
-	csVector3* nearestNormals = new csVector3[vCount];
-	float *distances = new float[vCount];
 	float shortestDistance = 10000000.0f;  // Big enought?
 	int nearestNormal = 0;
 
@@ -1261,8 +1556,8 @@ void csShadowBitmap::UpdateLightMap (
 	{
 	  // Create the 3D Segments
 	  csSegment3 seg;
-	  seg.SetStart (poly_parent->Vwor(vertexs[act]));
-	  seg.SetEnd (poly_parent->Vwor(vertexs[(act+1)%vCount]));
+	  seg.SetStart (poly_parent->Vwor (vertexs[act]));
+	  seg.SetEnd (poly_parent->Vwor (vertexs[(act+1)%vCount]));
 
 	  // Find the nearest point from v to the segment
 	  // a: Start
@@ -1289,17 +1584,17 @@ void csShadowBitmap::UpdateLightMap (
 
 	  // Find the normal in the nearest point of the segment
 	  // ==> Linear interpolation between vertexs
-	  float AllDistance = qsqrt (
+	  float AllDistance = csQsqrt (
 	    csSquaredDist::PointPoint(seg.Start (), seg.End()));
-	  float factorA = 1.0f - (qsqrt (csSquaredDist::PointPoint (
+	  float factorA = 1.0f - (csQsqrt (csSquaredDist::PointPoint (
 	    seg.Start (),nearest) ) / AllDistance);
-	  float factorB = 1.0f - (qsqrt (csSquaredDist::PointPoint
+	  float factorB = 1.0f - (csQsqrt (csSquaredDist::PointPoint
 	    (seg.End (),nearest) ) / AllDistance);
-	  nearestNormals[act] = factorA * normals[vertexs[act]]
-	    + factorB * normals[vertexs[((act+1)%vCount)]];
+	  nearestNormals[act] = factorA * worldNormals[vertexs[act]]
+	    + factorB * worldNormals[vertexs[((act+1)%vCount)]];
 
 	  // Get the distance
-	  distances[act] = qsqrt (csSquaredDist::PointPoint(v,nearest));
+	  distances[act] = csQsqrt (csSquaredDist::PointPoint(v,nearest));
 	  if (distances[act] < shortestDistance)
 	  {
 	    nearestNormal = act;
@@ -1324,9 +1619,6 @@ void csShadowBitmap::UpdateLightMap (
 	}
 
 	normal.Normalize();
-
-	delete [] nearestNormals;
-	delete [] distances;
       }
 
       float cosinus = (v - lightpos) * normal;
@@ -1339,17 +1631,20 @@ void csShadowBitmap::UpdateLightMap (
 
       int l;
       csRGBpixel &lumel = lightmap[uv];
-      l = lumel.red + QRound (light_r * scale);
+      l = lumel.red + csQround (light_r * scale);
 //    if (l<20) l = 20;
       lumel.red = l < 255 ? l : 255;
-      l = lumel.green + QRound (light_g * scale);
+      l = lumel.green + csQround (light_g * scale);
 //    if (l<20) l = 20;
       lumel.green = l < 255 ? l : 255;
-      l = lumel.blue + QRound (light_b * scale);
+      l = lumel.blue + csQround (light_b * scale);
 //    if (l<20) l = 20;
       lumel.blue = l < 255 ? l : 255;
     }
   }
+  delete [] distances;
+  delete [] nearestNormals;
+  delete [] worldNormals;
 }
 
 bool csShadowBitmap::UpdateShadowMap (
@@ -1385,7 +1680,30 @@ bool csShadowBitmap::UpdateShadowMap (
   v_rv *= rv_step;
   v_ru *= ru_step;
 
-  csThingStatic* static_data = poly->GetParent ()->GetStaticData ();
+  csThing* poly_parent = poly->GetParent ();
+  csThingStatic* static_data = poly_parent->GetStaticData ();
+  bool do_smooth = static_data->GetSmoothingFlag();
+  csSegment3* segments = 0;
+  csVector3* nearest = 0;
+  csVector3* nearestNormals = 0;
+  float *distances = 0;
+  float *weights = 0;
+  csVector3* worldNormals = 0;
+  if (do_smooth)
+  {
+    segments = new csSegment3[100];
+    nearest = new csVector3[100];
+    nearestNormals = new csVector3[100];
+    distances = new float[100];
+    weights = new float[100];
+    csReversibleTransform mesh_transf = poly_parent->GetCachedMovable ()
+    	->GetFullTransform ();
+    worldNormals = new csVector3[static_data->num_vertices];
+    csVector3* normals = static_data->GetNormals ();
+    for (i = 0 ; i < static_data->num_vertices ; i++)
+      worldNormals[i] = mesh_transf.This2OtherRelative (normals[i]);
+  }
+
   bool relevant = false;
   for (i = 0; i < lm_h; i++)
   {
@@ -1412,112 +1730,99 @@ bool csShadowBitmap::UpdateShadowMap (
 
       // calculations into a more efficient formula.
       float d = csSquaredDist::PointPoint (lightpos, v);
-      if (d >= light->GetInfluenceRadiusSq ()) continue;
+      if (d >= csSquare (light->GetCutoffDistance ())) continue;
 
-      d = qsqrt (d);
+      d = csQsqrt (d);
 
       csVector3 normal = poly_world_plane.Normal ();
       if ( static_data->GetSmoothingFlag() )
       {
-  csThing* poly_parent = poly->GetParent ();
-  int* vertexs = poly->GetStaticPoly ()->GetVertexIndices ();
-  csVector3* normals = static_data->GetNormals ();
-  int vCount = poly->GetStaticPoly ()->GetVertexCount ();
-  csSegment3* segments = new csSegment3[vCount];
+        int* vertexs = poly->GetStaticPoly ()->GetVertexIndices ();
+        int vCount = poly->GetStaticPoly ()->GetVertexCount ();
 
-  // int nearest;
-  csVector3* nearest = new csVector3[vCount];
-  csVector3* nearestNormals = new csVector3[vCount];
-  float *distances = new float[vCount];
-  float shortestDistance = 10000000.0f;  // Big enought?
-  int nearestNormal = 0;
+        // int nearest;
+        float shortestDistance = 10000000.0f;  // Big enought?
+        int nearestNormal = 0;
 
-  for (act = 0; act < vCount ; act++)
-  {
-    // Create the 3D Segments
-    segments[act].SetStart (poly_parent->Vwor(vertexs[act]));
-    segments[act].SetEnd (poly_parent->Vwor(vertexs[(act+1)%vCount]));
+        for (act = 0; act < vCount ; act++)
+        {
+          // Create the 3D Segments
+          segments[act].SetStart (poly_parent->Vwor(vertexs[act]));
+          segments[act].SetEnd (poly_parent->Vwor(vertexs[(act+1)%vCount]));
 
-    // Find the nearest point from v to the segment
-    // a: Start
-    // b: End
-    // c: point
-    csVector3 v_a = segments[act].Start ();
-    csVector3 v_b = segments[act].End ();
-    csVector3 v_c = v;
-    float dt_a = (v_c.x - v_a.x) * (v_b.x - v_a.x) +
-    (v_c.y - v_a.y) * (v_b.y - v_a.y) +
-    (v_c.z - v_a.z) * (v_b.z - v_a.z);
-    float dt_b = (v_c.x - v_b.x) * (v_a.x - v_b.x) +
-    (v_c.y - v_b.y) * (v_a.y - v_b.y) +
-    (v_c.z - v_b.z) * (v_a.z - v_b.z);
+          // Find the nearest point from v to the segment
+          // a: Start
+          // b: End
+          // c: point
+          csVector3 v_a = segments[act].Start ();
+          csVector3 v_b = segments[act].End ();
+          csVector3 v_c = v;
+          float dt_a = (v_c.x - v_a.x) * (v_b.x - v_a.x) +
+                (v_c.y - v_a.y) * (v_b.y - v_a.y) +
+                (v_c.z - v_a.z) * (v_b.z - v_a.z);
+          float dt_b = (v_c.x - v_b.x) * (v_a.x - v_b.x) +
+                (v_c.y - v_b.y) * (v_a.y - v_b.y) +
+                (v_c.z - v_b.z) * (v_a.z - v_b.z);
 
-    if (dt_a <= 0)
-    {
-      nearest[act] = v_a;
-    }
-    else
-    {
-      if (dt_b <=0)
-      {
-        nearest[act] = v_b;
-      }
-      else
-      {
-        nearest[act] = v_a + ((v_b - v_a) * dt_a)/(dt_a + dt_b);
-      }
-    }
+          if (dt_a <= 0)
+          {
+            nearest[act] = v_a;
+          }
+          else
+          {
+            if (dt_b <=0)
+            {
+              nearest[act] = v_b;
+            }
+            else
+            {
+              nearest[act] = v_a + ((v_b - v_a) * dt_a)/(dt_a + dt_b);
+            }
+          }
 
-    // Find the normal in the nearest point of the segment
-    // ==> Linear interpolation between vertexs
-    float AllDistance = qsqrt (csSquaredDist::PointPoint
-      (segments[act].Start (),segments[act].End()));
-    float factorA = 1.0f - (qsqrt (csSquaredDist::PointPoint
-      (segments[act].Start (),nearest[act]) ) / AllDistance);
-    float factorB = 1.0f - (qsqrt (csSquaredDist::PointPoint
-      (segments[act].End (),nearest[act]) ) / AllDistance);
-    nearestNormals[act] = factorA * normals[vertexs[act]]
-      + factorB * normals[vertexs[((act+1)%vCount)]];
+          // Find the normal in the nearest point of the segment
+          // ==> Linear interpolation between vertexs
+          float AllDistance = csQsqrt (csSquaredDist::PointPoint
+            (segments[act].Start (),segments[act].End()));
+          float factorA = 1.0f - (csQsqrt (csSquaredDist::PointPoint
+            (segments[act].Start (),nearest[act]) ) / AllDistance);
+          float factorB = 1.0f - (csQsqrt (csSquaredDist::PointPoint
+            (segments[act].End (),nearest[act]) ) / AllDistance);
+          nearestNormals[act] = factorA * worldNormals[vertexs[act]]
+            + factorB * worldNormals[vertexs[((act+1)%vCount)]];
 
-    // Get the distance
-    distances[act] = qsqrt (csSquaredDist::PointPoint(v,nearest[act]));
-    if (distances[act] < shortestDistance)
-    {
-      nearestNormal = act;
-      shortestDistance = distances[act];
-    }
-  }
+          // Get the distance
+          distances[act] = csQsqrt (csSquaredDist::PointPoint(v,nearest[act]));
+          if (distances[act] < shortestDistance)
+          {
+            nearestNormal = act;
+            shortestDistance = distances[act];
+          }
+        }
 
-  float *weights = new float[vCount];
-  // Get the weights of vertexes
-  for (act = 0; act < vCount ; act++)
-  {
-    if (distances[act] < 0.001f)
-      weights[act] = 1.0f;
-    else
-      weights[act] = shortestDistance / distances[act];
-  }
+        // Get the weights of vertexes
+        for (act = 0; act < vCount ; act++)
+        {
+          if (distances[act] < 0.001f)
+            weights[act] = 1.0f;
+          else
+            weights[act] = shortestDistance / distances[act];
+        }
 
-  if (!poly->GetStaticPoly ()->PointOnPolygon (v))
-  {
-    normal = nearestNormals[nearestNormal];
-  }
-  else
-  {
-    normal.x = normal.y = normal.z = 0.0f;
-    for (act = 0; act < vCount ; act++)
-    {
-      normal += weights[act]*nearestNormals[act];
-    }
-  }
+        if (!poly->GetStaticPoly ()->PointOnPolygon (v))
+        {
+          normal = nearestNormals[nearestNormal];
+        }
+        else
+        {
+          normal.x = normal.y = normal.z = 0.0f;
+          for (act = 0; act < vCount ; act++)
+          {
+            normal += weights[act]*nearestNormals[act];
+          }
+        }
 
-  normal.Normalize();
-
-  delete [] nearest;
-  delete [] nearestNormals;
-  delete [] distances;
-  delete [] weights;
-  delete [] segments;
+        normal.Normalize();
       }
 
       float cosinus = (v - lightpos) * normal;
@@ -1530,33 +1835,37 @@ bool csShadowBitmap::UpdateShadowMap (
 
       float brightness = cosinus * light->GetBrightnessAtDistance (d);
 
-      int l = shadowmap[uv] + QRound (
+      int l = shadowmap[uv] + csQround (
           CS_NORMAL_LIGHT_LEVEL * lightness * brightness);
       shadowmap[uv] = l < 255 ? l : 255;
       if ((!relevant) && shadowmap[uv] > 0) relevant = true;
     }
   }
+
+  delete [] segments;
+  delete [] nearest;
+  delete [] nearestNormals;
+  delete [] distances;
+  delete [] weights;
+  delete [] worldNormals;
+
   return relevant;
 }
 
 //-----------------------------------------------------------------------------
-SCF_IMPLEMENT_IBASE(csLightingPolyTexQueue)
-  SCF_IMPLEMENTS_INTERFACE(iLightingProcessData)
-SCF_IMPLEMENT_IBASE_END
 
 csLightingPolyTexQueue::csLightingPolyTexQueue (
-  iLight *light)
+  iLight *light) : scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   csLightingPolyTexQueue::light = light;
 }
 
 csLightingPolyTexQueue::~csLightingPolyTexQueue ()
 {
-  SCF_DESTRUCT_IBASE ();
 }
 
-void csLightingPolyTexQueue::AddPolyTexture (csPolyTexture *pt, csPolygon3D* polygon)
+void csLightingPolyTexQueue::AddPolyTexture (csPolyTexture *pt, 
+					     csPolygon3D* polygon)
 {
   polytxts.Push (pt);
   polygons.Push (polygon);
@@ -1570,7 +1879,7 @@ void csLightingPolyTexQueue::UpdateMaps (
   if (light->GetDynamicType () == CS_LIGHT_DYNAMICTYPE_DYNAMIC)
     return; // No update maps for dynamic lights.
 
-  int i;
+  size_t i;
   for (i = 0; i < polytxts.Length (); i++)
   {
     csPolyTexture *pt = polytxts[i];
@@ -1602,3 +1911,5 @@ void csLightingPolyTexQueue::UpdateMaps (
   polytxts.DeleteAll ();
   polygons.DeleteAll ();
 }
+
+} // namespace cspluginThing
