@@ -17,42 +17,51 @@
 */
 
 #include "cssysdef.h"
+
 #include "csgeom/math3d.h"
 #include "csgeom/matrix3.h"
 #include "csgeom/transfrm.h"
-#include "csutil/util.h"
-#include "csutil/scanstr.h"
 #include "csutil/cscolor.h"
 #include "csutil/csstring.h"
+#include "csutil/dirtyaccessarray.h"
+#include "csutil/flags.h"
+#include "csutil/scanstr.h"
 #include "csutil/scfstr.h"
-#include "csutil/garray.h"
-#include "iutil/vfs.h"
-#include "iutil/document.h"
-#include "iutil/object.h"
-#include "iutil/objreg.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/plugin.h"
-#include "iengine/mesh.h"
+#include "csutil/util.h"
+
+
 #include "iengine/engine.h"
-#include "iengine/texture.h"
+#include "iengine/mesh.h"
 #include "iengine/material.h"
 #include "iengine/portal.h"
 #include "iengine/portalcontainer.h"
+#include "iengine/texture.h"
+#include "iengine/region.h"
+#include "imap/ldrctxt.h"
+#include "imap/loader.h"
+#include "imap/services.h"
 #include "imesh/object.h"
 #include "imesh/thing.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/texture.h"
-#include "ivideo/material.h"
-#include "imap/services.h"
-#include "imap/ldrctxt.h"
-#include "imap/parser.h"
+#include "iutil/comp.h"
+#include "iutil/document.h"
+#include "iutil/eventh.h"
+#include "iutil/object.h"
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "iutil/vfs.h"
 #include "ivaria/reporter.h"
+#include "ivideo/graph3d.h"
+#include "ivideo/material.h"
+#include "ivideo/texture.h"
+
+
 #include "thingldr.h"
-#include "qint.h"
 
 CS_IMPLEMENT_PLUGIN
 
+namespace cspluginThingLdr
+{
+
 enum
 {
   XMLTOKEN_CLONE = 1,
@@ -90,53 +99,31 @@ enum
   XMLTOKEN_TEXLEN,
   XMLTOKEN_VISTREE,
   XMLTOKEN_V,
-  XMLTOKEN_SMOOTH
+  XMLTOKEN_SMOOTH,
+  XMLTOKEN_RENDERBUFFER
 };
 
-SCF_IMPLEMENT_IBASE (csThingLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csThingLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csThingSaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csThingSaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csThingLoader)
 SCF_IMPLEMENT_FACTORY (csThingFactoryLoader)
 SCF_IMPLEMENT_FACTORY (csThingSaver)
-
-
-#define MAXLINE 200 /* max number of chars per line... */
+SCF_IMPLEMENT_FACTORY (csThingFactorySaver)
 
 //---------------------------------------------------------------------------
 
-csThingLoader::csThingLoader (iBase* pParent)
+csThingLoader::csThingLoader (iBase* pParent) : 
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csThingLoader::~csThingLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csThingLoader::Initialize (iObjectRegistry* object_reg)
 {
   csThingLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("clone", XMLTOKEN_CLONE);
   xmltokens.Register ("cosfact", XMLTOKEN_COSFACT);
@@ -174,6 +161,7 @@ bool csThingLoader::Initialize (iObjectRegistry* object_reg)
   xmltokens.Register ("matrix", XMLTOKEN_MATRIX);
   xmltokens.Register ("orig", XMLTOKEN_ORIG);
   xmltokens.Register ("origref", XMLTOKEN_ORIGREF);
+  xmltokens.Register ("renderbuffer", XMLTOKEN_RENDERBUFFER);
   return true;
 }
 
@@ -416,13 +404,23 @@ bool csThingLoader::ParsePortal (
   return true;
 }
 
+struct ParsedRB
+{ 
+  csString name;
+  csRef<iRenderBuffer> buf; 
+  csRef<iDocumentNode> node;
+  ParsedRB (const char* name, iRenderBuffer* b, iDocumentNode* n) : 
+    name (name), buf (b), node (n) {}
+};
+
 bool csThingLoader::ParsePoly3d (
         iDocumentNode* node,
 	iLoaderContext* ldr_context,
 	iEngine* engine,
 	float default_texlen,
 	iThingFactoryState* thing_fact_state, int vt_offset,
-	bool& poly_delete, iMeshWrapper* mesh)
+	bool& poly_delete, iMeshWrapper* mesh,
+	bool& baduv)
 {
   poly_delete = false;
   iMaterialWrapper* mat = 0;
@@ -430,18 +428,18 @@ bool csThingLoader::ParsePoly3d (
   if (!thing_type)
   {
     csRef<iPluginManager> plugin_mgr =
-    	CS_QUERY_REGISTRY (object_reg, iPluginManager);
+    	csQueryRegistry<iPluginManager> (object_reg);
     CS_ASSERT (plugin_mgr != 0);
-    thing_type = CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	  "crystalspace.mesh.object.thing", iMeshObjectType);
+    thing_type = csQueryPluginClass<iMeshObjectType> (plugin_mgr,
+  	  "crystalspace.mesh.object.thing" );
     if (!thing_type)
-      thing_type = CS_LOAD_PLUGIN (plugin_mgr,
-    	  "crystalspace.mesh.object.thing", iMeshObjectType);
+      thing_type = csLoadPlugin<iMeshObjectType> (plugin_mgr,
+    	  "crystalspace.mesh.object.thing");
   }
 
   CS_ASSERT (thing_type != 0);
-  csRef<iThingEnvironment> te = SCF_QUERY_INTERFACE (
-  	thing_type, iThingEnvironment);
+  csRef<iThingEnvironment> te = scfQueryInterface<iThingEnvironment> (
+  	thing_type);
 
   uint texspec = 0;
   int tx_uv_i1 = 0;
@@ -461,6 +459,7 @@ bool csThingLoader::ParsePoly3d (
   int set_colldet = 0; // If 1 then set, if -1 then reset, else default.
   int set_viscull = 0; // If 1 then set, if -1 then reset, else default.
   csDirtyAccessArray<int> vertices_to_add;
+  csArray<ParsedRB> renderbuffers;
 
   // For portals.
   csRef<iDocumentNode> portal_node;
@@ -540,7 +539,7 @@ bool csThingLoader::ParsePoly3d (
         {
 	  int vt_idx = child->GetContentsValueAsInt ();
 	  bool ignore = false;
-	  int cnt = vertices_to_add.Length ();
+	  int cnt = (int)vertices_to_add.Length ();
 	  for (int i = 0 ; i < cnt ; i++)
 	  {
 	    if (vertices_to_add[i] == vt_idx+vt_offset)
@@ -575,6 +574,21 @@ bool csThingLoader::ParsePoly3d (
 	  "crystalspace.thingldr.polygon",
 	  child, "<mixmode> for polygons is no longer supported! Use <mixmode> for the entire mesh instead!");
 	return false;
+      case XMLTOKEN_RENDERBUFFER:
+	{
+	  const char *name = child->GetAttributeValue("name");
+	  if ((name == 0) || (*name == 0))
+	  {
+	    synldr->ReportError ("crystalspace.thingldr.polygon",
+	      child, "<renderbuffer>s must have names");
+	    return false;
+	  }
+
+	  csRef<iRenderBuffer> rb = synldr->ParseRenderBuffer (child);
+	  if (!rb.IsValid()) return false;
+	  renderbuffers.Push (ParsedRB (name, rb, child));
+	}
+	break;
       default:
         synldr->ReportBadToken (child);
         return false;
@@ -591,11 +605,11 @@ bool csThingLoader::ParsePoly3d (
   }
 
   thing_fact_state->SetPolygonVertexIndices (CS_POLYRANGE_LAST,
-  	vertices_to_add.Length (), vertices_to_add.GetArray ());
+  	(int)vertices_to_add.Length (), vertices_to_add.GetArray ());
 
   mat = thing_fact_state->GetPolygonMaterial (CS_POLYINDEX_LAST);
-  csRef<iMaterialEngine> mateng = SCF_QUERY_INTERFACE (mat->GetMaterial (),
-  	iMaterialEngine);
+  csRef<iMaterialEngine> mateng = scfQueryInterface<iMaterialEngine> (
+    mat->GetMaterial ());
   bool is_texture_transparent = false;
   if (mateng)
   {
@@ -611,7 +625,7 @@ bool csThingLoader::ParsePoly3d (
         iImage* im = tw->GetImageFile ();
         if (im)
         {
-          is_texture_transparent = im->HasKeycolor () ||
+          is_texture_transparent = im->HasKeyColor () ||
 	    (im->GetFormat () & CS_IMGFMT_ALPHA);
         }
       }
@@ -639,11 +653,17 @@ bool csThingLoader::ParsePoly3d (
 	  "Bad texture specification: vertex index 3 doesn't exist!");
       return false;
     }
-    thing_fact_state->SetPolygonTextureMapping (
+    if (!thing_fact_state->SetPolygonTextureMapping (
     	CS_POLYRANGE_LAST,
-        thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, tx_uv_i1), tx_uv1,
-        thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, tx_uv_i2), tx_uv2,
-        thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, tx_uv_i3), tx_uv3);
+        thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, tx_uv_i1),
+		tx_uv1,
+        thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, tx_uv_i2),
+		tx_uv2,
+        thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, tx_uv_i3),
+		tx_uv3))
+    {
+      baduv = true;
+    }
   }
   else if (texspec & CSTEX_V1)
   {
@@ -661,8 +681,11 @@ bool csThingLoader::ParsePoly3d (
           "Bad texture specification!");
 	return false;
       }
-      else thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST,
-        tx_orig, tx1, tx_len.y, tx2, tx_len.z);
+      else if (!thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST,
+            tx_orig, tx1, tx_len.y, tx2, tx_len.z))
+      {
+	baduv = true;
+      }
     }
     else
     {
@@ -672,8 +695,11 @@ bool csThingLoader::ParsePoly3d (
           "Bad texture specification!");
 	return false;
       }
-      else thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST,
-      	tx_orig, tx1, tx_len.x);
+      else if (!thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST,
+      	tx_orig, tx1, tx_len.x))
+      {
+        baduv = true;
+      }
     }
   }
   else if (tx_len.x)
@@ -683,55 +709,83 @@ bool csThingLoader::ParsePoly3d (
     // a standard offset. Otherwise we will just create a plane specific
     // for this case given the first two vertices.
     bool same_x = true, same_y = true, same_z = true;
-    const csVector3& v = thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, 0);
-    for (int i = 1 ; i < thing_fact_state->GetPolygonVertexCount (CS_POLYINDEX_LAST) ; i++)
+    const csVector3& v = thing_fact_state->GetPolygonVertex (
+    	CS_POLYINDEX_LAST, 0);
+    for (int i = 1 ; i < thing_fact_state->GetPolygonVertexCount (
+    	CS_POLYINDEX_LAST) ; i++)
     {
-      const csVector3& v2 = thing_fact_state->GetPolygonVertex (CS_POLYINDEX_LAST, i);
+      const csVector3& v2 = thing_fact_state->GetPolygonVertex (
+      	CS_POLYINDEX_LAST, i);
       if (same_x && ABS (v.x-v2.x) >= SMALL_EPSILON) same_x = false;
       if (same_y && ABS (v.y-v2.y) >= SMALL_EPSILON) same_y = false;
       if (same_z && ABS (v.z-v2.z) >= SMALL_EPSILON) same_z = false;
     }
     if (same_x)
     {
-      thing_fact_state->SetPolygonTextureMapping (
+      if (!thing_fact_state->SetPolygonTextureMapping (
         CS_POLYRANGE_LAST,
       	csVector3 (v.x, 0, 0), csVector3 (v.x, 0, 1),
-	tx_len.x, csVector3 (v.x, 1, 0), tx_len.x);
+	tx_len.x, csVector3 (v.x, 1, 0), tx_len.x))
+      {
+        baduv = true;
+      }
     }
     else if (same_y)
     {
-      thing_fact_state->SetPolygonTextureMapping (
+      if (!thing_fact_state->SetPolygonTextureMapping (
         CS_POLYRANGE_LAST,
       	csVector3 (0, v.y, 0), csVector3 (1, v.y, 0),
-	tx_len.x, csVector3 (0, v.y, 1), tx_len.x);
+	tx_len.x, csVector3 (0, v.y, 1), tx_len.x))
+      {
+        baduv = true;
+      }
     }
     else if (same_z)
     {
-      thing_fact_state->SetPolygonTextureMapping (
+      if (!thing_fact_state->SetPolygonTextureMapping (
       	CS_POLYRANGE_LAST,
 	csVector3 (0, 0, v.z), csVector3 (1, 0, v.z),
-	tx_len.x, csVector3 (0, 1, v.z), tx_len.x);
+	tx_len.x, csVector3 (0, 1, v.z), tx_len.x))
+      {
+        baduv = true;
+      }
     }
     else
-      thing_fact_state->SetPolygonTextureMapping (
+    {
+      if (!thing_fact_state->SetPolygonTextureMapping (
       	CS_POLYRANGE_LAST,
-	tx_len.x);
+	tx_len.x))
+      {
+        baduv = true;
+      }
+    }
   }
   else
-    thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST, tx_matrix, tx_vector);
+  {
+    if (!thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST,
+    	tx_matrix, tx_vector))
+    {
+      baduv = true;
+    }
+  }
 
   if (texspec & CSTEX_UV_SHIFT)
   {
     if (thing_fact_state->IsPolygonTextureMappingEnabled (CS_POLYINDEX_LAST))
     {
-      thing_fact_state->GetPolygonTextureMapping (CS_POLYINDEX_LAST, tx_matrix, tx_vector);
+      thing_fact_state->GetPolygonTextureMapping (CS_POLYINDEX_LAST,
+      	tx_matrix, tx_vector);
       // T = Mot * (O - Vot)
       // T = Mot * (O - Vot) + Vuv      ; Add shift Vuv to final texture map
       // T = Mot * (O - Vot) + Mot * Mot-1 * Vuv
       // T = Mot * (O - Vot + Mot-1 * Vuv)
       csVector3 shift (uv_shift.x, uv_shift.y, 0);
       tx_vector -= tx_matrix.GetInverse () * shift;
-      thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST, tx_matrix, tx_vector);
+      if (!thing_fact_state->SetPolygonTextureMapping (CS_POLYRANGE_LAST,
+      	tx_matrix, tx_vector))
+      {
+        baduv = true;
+      }
     }
   }
 
@@ -768,14 +822,17 @@ bool csThingLoader::ParsePoly3d (
       int portal_pri = engine->GetPortalRenderPriority ();
       if (portal_pri == 0)
         portal_pri = mesh->GetRenderPriority ();
-      char pc_name[100];
-      sprintf (pc_name, "__portals_%d_%s__", portal_pri,
+      csString pc_name;
+      pc_name.Format ("__portals_%d_%s__", portal_pri,
       	destSectorName.GetData ());
 
       iPortal* portal;
       csRef<iMeshWrapper> portal_mesh = engine->CreatePortal (
       	pc_name, mesh, destSector,
       	portal_verts, cnt, portal);
+      if (ldr_context->GetRegion ())
+	ldr_context->GetRegion ()->QueryObject ()->ObjAdd (
+		portal_mesh->QueryObject ());
       delete[] portal_verts;
 
       portal_mesh->SetRenderPriority (portal_pri);
@@ -838,6 +895,28 @@ bool csThingLoader::ParsePoly3d (
   else if (set_colldet == -1)
     thing_fact_state->ResetPolygonFlags (CS_POLYRANGE_LAST, CS_POLY_COLLDET);
 
+  for (size_t i  = 0; i < renderbuffers.Length(); i++)
+  {
+    const ParsedRB& rb = renderbuffers[i];
+    if (rb.buf->GetElementCount() != vertices_to_add.Length())
+    {
+      synldr->ReportError ("crystalspace.thingldr.polygon", rb.node,
+	"Render buffer element count does not match polygon vertex count: "
+	"%zu != %zu", rb.buf->GetElementCount(), 
+	vertices_to_add.Length());
+      return false;
+    }
+    if (!thing_fact_state->AddPolygonRenderBuffer (CS_POLYINDEX_LAST, 
+      rb.name, rb.buf))
+    {
+      synldr->ReportError ("crystalspace.thingldr.polygon", rb.node,
+	"Either a renderbuffer '%s' was already attached to the polygon "
+	"or the format does not match other buffers of the same name attached "
+	"to other polygons.", rb.name.GetData());
+      return false;
+    }
+  }
+
   return true;
 }
 
@@ -846,7 +925,7 @@ bool csThingLoader::LoadThingPart (iThingEnvironment* te, iDocumentNode* node,
 	iObjectRegistry* object_reg, iReporter* reporter,
 	iSyntaxService *synldr, ThingLoadInfo& info,
 	iEngine* engine, int vt_offset, bool isParent,
-	iMeshWrapper* mesh)
+	iMeshWrapper* mesh, bool& baduv)
 {
 #define CHECK_TOPLEVEL(m) \
 if (!isParent) { \
@@ -879,11 +958,9 @@ if (!info.thing_fact_state) \
 { \
   csRef<iMeshObjectFactory> fact; \
   fact = info.type->NewFactory (); \
-  info.thing_fact_state = SCF_QUERY_INTERFACE (fact, \
-	iThingFactoryState); \
+  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (fact); \
   info.obj = fact->NewInstance (); \
-  info.thing_state = SCF_QUERY_INTERFACE (info.obj, \
-	iThingState); \
+  info.thing_state = scfQueryInterface<iThingState> (info.obj); \
 }
 
   uint mixmode = CS_FX_COPY;
@@ -902,7 +979,7 @@ if (!info.thing_fact_state) \
 	synldr->ReportError (
 	    "crystalspace.thingloader.parse.vistree",
 	    child, "'vistree' no longer supported! Convert your level to Dynavis using 'levtool'!\n");
-	printf ("'vistree' no longer supported! Convert your level to Dynavis using 'levtool'!\n");
+	csPrintf ("'vistree' no longer supported! Convert your level to Dynavis using 'levtool'!\n");
 	return false;
       case XMLTOKEN_COSFACT:
         CHECK_TOPLEVEL("cosfact");
@@ -947,8 +1024,8 @@ if (!info.thing_fact_state) \
               child, "Couldn't find thing factory '%s'!", factname);
             return false;
           }
-	  info.thing_fact_state = SCF_QUERY_INTERFACE (
-	  	fact->GetMeshObjectFactory (), iThingFactoryState);
+	  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (
+	  	fact->GetMeshObjectFactory ());
 	  if (!info.thing_fact_state)
 	  {
 	    synldr->ReportError (
@@ -957,7 +1034,7 @@ if (!info.thing_fact_state) \
             return false;
 	  }
 	  info.obj = fact->GetMeshObjectFactory ()->NewInstance ();
-	  info.thing_state = SCF_QUERY_INTERFACE (info.obj, iThingState);
+	  info.thing_state = scfQueryInterface<iThingState> (info.obj);
         }
         break;
       case XMLTOKEN_CLONE:
@@ -989,8 +1066,8 @@ if (!info.thing_fact_state) \
             return false;
           }
 
-	  csRef<iThingState> other_thing_state (SCF_QUERY_INTERFACE (
-	  	wrap->GetMeshObject (), iThingState));
+	  csRef<iThingState> other_thing_state (scfQueryInterface<iThingState> (
+	  	wrap->GetMeshObject ()));
 	  if (!other_thing_state)
 	  {
 	    synldr->ReportError (
@@ -998,17 +1075,18 @@ if (!info.thing_fact_state) \
               child, "Object '%s' is not a thing!", meshname);
             return false;
 	  }
-	  info.thing_fact_state = other_thing_state->GetFactory ();
+	  info.thing_fact_state = scfQueryInterface<iThingFactoryState>
+	    (wrap->GetMeshObject ()->GetFactory());
 	  info.obj = wrap->GetFactory ()->GetMeshObjectFactory ()
 	  	->NewInstance ();
-	  info.thing_state = SCF_QUERY_INTERFACE (info.obj, iThingState);
+	  info.thing_state = scfQueryInterface<iThingState> (info.obj);
         }
         break;
       case XMLTOKEN_PART:
 	if (!LoadThingPart (te, child, ldr_context, object_reg, reporter,
 		synldr, info, engine, info.thing_fact_state
 			? info.thing_fact_state->GetVertexCount () : 0,
-		false, mesh))
+		false, mesh, baduv))
 	  return false;
         break;
       case XMLTOKEN_V:
@@ -1040,8 +1118,9 @@ Nag to Jorrit about this feature if you want it.");
 	    	info.default_material);
 	  bool poly_delete = false;
 	  bool success = ParsePoly3d (child, ldr_context,
-	  			    engine, info.default_texlen, info.thing_fact_state,
-				    vt_offset, poly_delete, mesh);
+	  			    engine, info.default_texlen,
+				    info.thing_fact_state,
+				    vt_offset, poly_delete, mesh, baduv);
 	  if (!success)
 	  {
 	    info.thing_fact_state->RemovePolygon (idx);
@@ -1058,7 +1137,7 @@ Nag to Jorrit about this feature if you want it.");
         CHECK_TOPLEVEL("replacematerial");
         CHECK_OBJECTONLY("replacematerial");
 	{
-	  int idx = info.replace_materials.Push (RepMaterial ());
+	  int idx = (int)info.replace_materials.Push (RepMaterial ());
 	  info.replace_materials[idx].oldmat = csStrNew (
 		child->GetAttributeValue ("old"));
 	  info.replace_materials[idx].newmat = csStrNew (
@@ -1092,11 +1171,13 @@ Nag to Jorrit about this feature if you want it.");
 
       case XMLTOKEN_MIXMODE:
         CHECK_TOPLEVEL("mixmode");
-        CHECK_OBJECTONLY("mixmode");
         {
 	  if (synldr->ParseMixmode (child, mixmode))
 	  {
-            info.thing_state->SetMixMode (mixmode);
+	    if (info.load_factory)
+              info.thing_fact_state->SetMixMode (mixmode);
+	    else
+              info.thing_state->SetMixMode (mixmode);
 	  }
 	}
         break;
@@ -1117,8 +1198,8 @@ Nag to Jorrit about this feature if you want it.");
   if (mixmode == CS_FX_COPY)
   {
     // Mixmode is copy, delete all polygons that became portals.
-    int i;
-    for (i = polygons_to_delete.Length ()-1 ; i >= 0 ; i--)
+    size_t i = polygons_to_delete.Length ();
+    while (i-- > 0)
       info.thing_fact_state->RemovePolygon (polygons_to_delete[i]);
   }
 
@@ -1126,20 +1207,21 @@ Nag to Jorrit about this feature if you want it.");
 }
 
 csPtr<iBase> csThingLoader::Parse (iDocumentNode* node,
+			     iStreamSource*,
 			     iLoaderContext* ldr_context, iBase* context)
 {
   ThingLoadInfo info;
   info.load_factory = false;
   info.global_factory = false;
 
-  csRef<iPluginManager> plugin_mgr = CS_QUERY_REGISTRY (object_reg,
-  	iPluginManager);
-  info.type = CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	"crystalspace.mesh.object.thing", iMeshObjectType);
+  csRef<iPluginManager> plugin_mgr = 
+    csQueryRegistry<iPluginManager> (object_reg);
+  info.type = csQueryPluginClass<iMeshObjectType> (plugin_mgr,
+  	"crystalspace.mesh.object.thing");
   if (!info.type)
   {
-    info.type = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.mesh.object.thing",
-    	iMeshObjectType);
+    info.type = csLoadPlugin<iMeshObjectType> (plugin_mgr, 
+      "crystalspace.mesh.object.thing");
   }
   if (!info.type)
   {
@@ -1148,15 +1230,16 @@ csPtr<iBase> csThingLoader::Parse (iDocumentNode* node,
 		node, "Could not load the thing mesh object plugin!");
     return 0;
   }
-  csRef<iThingEnvironment> te = SCF_QUERY_INTERFACE (info.type,
-  	iThingEnvironment);
-  csRef<iEngine> engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  csRef<iThingEnvironment> te = 
+    scfQueryInterface<iThingEnvironment> (info.type);
+  csRef<iEngine> engine = csQueryRegistry<iEngine> (object_reg);
 
   // It is possible that the mesh wrapper is null.
-  csRef<iMeshWrapper> mesh = SCF_QUERY_INTERFACE_SAFE (context, iMeshWrapper);
+  csRef<iMeshWrapper> mesh = scfQueryInterfaceSafe<iMeshWrapper> (context);
 
+  bool baduv = false;
   if (!LoadThingPart (te, node, ldr_context, object_reg, reporter, synldr, info,
-  	engine, 0, true, mesh))
+  	engine, 0, true, mesh, baduv))
   {
     info.obj = 0;
   }
@@ -1169,7 +1252,7 @@ csPtr<iBase> csThingLoader::Parse (iDocumentNode* node,
 	"This is not supported!");
       return 0;
     }
-    int i;
+    size_t i;
     for (i = 0 ; i < info.replace_materials.Length () ; i++)
     {
       RepMaterial& rm = info.replace_materials[i];
@@ -1193,24 +1276,32 @@ csPtr<iBase> csThingLoader::Parse (iDocumentNode* node,
     }
   }
 
+  if (baduv)
+  {
+    synldr->Report ("crystalspace.thingloader.parse",
+		CS_REPORTER_SEVERITY_WARNING,
+		node, "Bad UV coordinates for polygons in this thing!");
+  }
+
   return csPtr<iBase> (info.obj);
 }
 
 csPtr<iBase> csThingFactoryLoader::Parse (iDocumentNode* node,
+			     iStreamSource*,
 			     iLoaderContext* ldr_context, iBase*)
 {
   ThingLoadInfo info;
   info.load_factory = true;
   info.global_factory = false;
 
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-  	iPluginManager));
-  info.type = CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	"crystalspace.mesh.object.thing", iMeshObjectType);
+  csRef<iPluginManager> plugin_mgr (
+    csQueryRegistry<iPluginManager> (object_reg));
+  info.type = csQueryPluginClass<iMeshObjectType> (plugin_mgr,
+  	"crystalspace.mesh.object.thing");
   if (!info.type)
   {
-    info.type = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.mesh.object.thing",
-    	iMeshObjectType);
+    info.type = csLoadPlugin<iMeshObjectType> (plugin_mgr, 
+      "crystalspace.mesh.object.thing");
   }
   if (!info.type)
   {
@@ -1219,52 +1310,175 @@ csPtr<iBase> csThingFactoryLoader::Parse (iDocumentNode* node,
 		node, "Could not load the thing mesh object plugin!");
     return 0;
   }
-  csRef<iThingEnvironment> te = SCF_QUERY_INTERFACE (info.type,
-  	iThingEnvironment);
-  csRef<iEngine> engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  csRef<iThingEnvironment> te = 
+    scfQueryInterface<iThingEnvironment> (info.type);
+  csRef<iEngine> engine = csQueryRegistry<iEngine> (object_reg);
 
   csRef<iMeshObjectFactory> fact;
   fact = info.type->NewFactory ();
-  info.thing_fact_state = SCF_QUERY_INTERFACE (fact, iThingFactoryState);
+  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (fact);
 
+  bool baduv = false;
   if (!LoadThingPart (te, node, ldr_context, object_reg, reporter, synldr, info,
-  	engine, 0, true, 0))
+  	engine, 0, true, 0, baduv))
   {
     fact = 0;
   }
+
+  if (baduv)
+  {
+    synldr->Report ("crystalspace.thingloader.parse",
+    		CS_REPORTER_SEVERITY_WARNING,
+		node, "Bad UV coordinates for polygons in this thing!");
+  }
+
   return csPtr<iBase> (fact);
 }
 
 //---------------------------------------------------------------------------
 
-csThingSaver::csThingSaver (iBase* pParent)
+csThingSaver::csThingSaver (iBase* pParent) : 
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csThingSaver::~csThingSaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csThingSaver::Initialize (iObjectRegistry* object_reg)
 {
   csThingSaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
+  return true;
+}
+//TBD
+bool csThingSaver::WriteDown (iBase* obj, iDocumentNode* parent,
+	iStreamSource*)
+{
+  if (!parent) return false; //you never know...
+  
+  csRef<iDocumentNode> paramsNode = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+  paramsNode->SetValue("params");
+  if (obj)
+  {
+    csRef<iThingState> thing = scfQueryInterface<iThingState> (obj);
+    csRef<iMeshObject> mesh = scfQueryInterface<iMeshObject> (obj);
+    if (!thing) return false;
+    if (!mesh) return false;
+
+    iMeshWrapper* meshwrap = mesh->GetMeshWrapper ();
+    iMeshFactoryWrapper* factwrap = meshwrap->GetFactory();
+    if (factwrap)
+    {
+      const char* factname = factwrap->QueryObject()->GetName();
+      if (factname && *factname)
+      {
+        csRef<iDocumentNode> factNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        factNode->SetValue("factory");
+        factNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue(factname);
+      }    
+    }
+    else
+    {
+      csRef<iThingFactoryState> fact = scfQueryInterface<iThingFactoryState>
+        (mesh->GetFactory());
+      WriteFactory (fact, paramsNode);
+    }
+    //Writedown Mixmode tag
+    int mixmode = thing->GetMixMode();
+    csRef<iDocumentNode> mixmodeNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    mixmodeNode->SetValue("mixmode");
+    synldr->WriteMixmode(mixmodeNode, mixmode, true);
+  }
   return true;
 }
 
-void csThingSaver::WriteDown (iBase* /*obj*/, iFile *file)
+//TBD
+bool csThingSaver::WriteFactory (iBase* obj, iDocumentNode* parent)
 {
-  csString str;
-  csRef<iFactory> fact (SCF_QUERY_INTERFACE (this, iFactory));
-  char buf[MAXLINE];
-  char name[MAXLINE];
-  csFindReplace (name, fact->QueryDescription (), "Saver", "Loader", MAXLINE);
-  sprintf (buf, "FACTORY ('%s')\n", name);
-  str.Append (buf);
-  file->Write ((const char*)str, str.Length ());
+  if (!parent) return false; //you never know...
+  
+  iDocumentNode* paramsNode = parent;
+
+  if (obj)
+  {
+    csRef<iThingFactoryState> thingfact = 
+      scfQueryInterface<iThingFactoryState> (obj);
+    if (!thingfact) return false;
+
+    for (int vertidx = 0; vertidx < thingfact->GetVertexCount(); vertidx++)
+    {
+      csRef<iDocumentNode> vertNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      vertNode->SetValue("v");
+      csVector3 vertex = thingfact->GetVertex(vertidx);
+      synldr->WriteVector(vertNode, &vertex);
+    }  
+    iMaterialWrapper* material = 0;
+    for (int polyidx = 0; polyidx < thingfact->GetPolygonCount(); polyidx++)
+    {
+      if (material != thingfact->GetPolygonMaterial(polyidx))
+      {
+        material = thingfact->GetPolygonMaterial(polyidx);
+        csRef<iDocumentNode> materialNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        materialNode->SetValue("material");
+        const char* materialname = material->QueryObject()->GetName();
+        if (materialname && *materialname)
+        {
+          materialNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue(materialname);
+        }
+      }
+      csRef<iDocumentNode> polyNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      polyNode->SetValue("p");
+      const char* polyname = thingfact->GetPolygonName(polyidx);
+      if (polyname && *polyname)
+      {
+        polyNode->SetAttribute("name", polyname);
+      }
+      for (int pvertidx = 0; pvertidx < thingfact->GetPolygonVertexCount(polyidx); pvertidx++)
+      {
+        csRef<iDocumentNode> vertNode = polyNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        vertNode->SetValue("v");
+        int vertex = thingfact->GetPolygonVertexIndices(polyidx)[pvertidx];
+        vertNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValueAsInt(vertex);
+      }
+      if (thingfact->IsPolygonTextureMappingEnabled(polyidx))
+      {
+        csRef<iDocumentNode> texmapNode = polyNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        texmapNode->SetValue("texmap");
+        csMatrix3 m; csVector3 v;
+        thingfact->GetPolygonTextureMapping(polyidx, m, v);
+        csRef<iDocumentNode> matrixNode = texmapNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        matrixNode->SetValue("matrix");
+        synldr->WriteMatrix(matrixNode, &m);
+        csRef<iDocumentNode> vectorNode = texmapNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        vectorNode->SetValue("v");
+        synldr->WriteVector(vectorNode, &v);
+      }
+      //Writedown Colldet tag
+      bool colldet = thingfact->GetPolygonFlags(polyidx).Check(CS_POLY_COLLDET);
+      synldr->WriteBool(polyNode, "colldet", colldet, true);
+ 
+      //Writedown Lighting tag
+      bool lighting = thingfact->GetPolygonFlags(polyidx).Check(CS_POLY_LIGHTING);
+      synldr->WriteBool(polyNode, "lighting", lighting, true);
+
+      //Writedown Viscull tag
+      bool viscull = thingfact->GetPolygonFlags(polyidx).Check(CS_POLY_VISCULL);
+      synldr->WriteBool(polyNode, "viscull", viscull, true);
+    }
+    //Writedown Smooth tag
+    synldr->WriteBool(paramsNode, "smooth", thingfact->GetSmoothingFlag(), false);
+
+    //Writedown Cosfact tag
+    float cosfact = thingfact->GetCosinusFactor();
+    csRef<iDocumentNode> cosfactNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    cosfactNode->SetValue("cosfact");
+    cosfactNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValueAsFloat(cosfact);
+  }
+  return true;
 }
 
+} // namespace cspluginThingLdr
+
