@@ -17,17 +17,21 @@
 */
 
 #include "cssysdef.h"
-#include "iutil/objreg.h"
-#include "iutil/document.h"
-#include "iutil/object.h"
+
+#include "csgeom/plane3.h"
+#include "csgeom/transfrm.h"
 #include "iengine/engine.h"
 #include "iengine/mesh.h"
 #include "iengine/movable.h"
-#include "imap/services.h"
 #include "imap/ldrctxt.h"
-#include "imap/parser.h"
-#include "ivaria/reporter.h"
+#include "imap/loader.h"
+#include "imap/services.h"
+#include "iutil/document.h"
+#include "iutil/object.h"
+#include "iutil/objreg.h"
 #include "ivaria/dynamics.h"
+#include "ivaria/reporter.h"
+#include "ivaria/ode.h"
 #include "physldr.h"
 
 CS_IMPLEMENT_PLUGIN
@@ -55,6 +59,7 @@ enum
   XMLTOKEN_COLLIDERCYLINDER,
   XMLTOKEN_LENGTH,
   XMLTOKEN_COLLIDERBOX,
+  XMLTOKEN_COLLIDERPLANE,
   XMLTOKEN_MOVE,
   XMLTOKEN_ROTATE,
   XMLTOKEN_JOINT,
@@ -64,7 +69,12 @@ enum
   XMLTOKEN_CONSTRAINED,
   XMLTOKEN_FREE,
   XMLTOKEN_MIN,
-  XMLTOKEN_MAX
+  XMLTOKEN_MAX,
+  XMLTOKEN_SIMULATIONMODE,
+  XMLTOKEN_AUTODISABLE,
+  XMLTOKEN_WORLDSTEP,
+  XMLTOKEN_STEPFAST,
+  XMLTOKEN_QUICKSTEP
 };
 
 SCF_IMPLEMENT_IBASE (csPhysicsLoader)
@@ -116,6 +126,7 @@ bool csPhysicsLoader::Initialize (iObjectRegistry* object_reg)
   xmltokens.Register ("collidersphere", XMLTOKEN_COLLIDERSPHERE);
   xmltokens.Register ("collidercylinder", XMLTOKEN_COLLIDERCYLINDER);
   xmltokens.Register ("colliderbox", XMLTOKEN_COLLIDERBOX);
+  xmltokens.Register ("colliderplane", XMLTOKEN_COLLIDERPLANE);
   xmltokens.Register ("radius", XMLTOKEN_RADIUS);
   xmltokens.Register ("length", XMLTOKEN_LENGTH);
   xmltokens.Register ("move", XMLTOKEN_MOVE);
@@ -126,17 +137,26 @@ bool csPhysicsLoader::Initialize (iObjectRegistry* object_reg)
   xmltokens.Register ("angle", XMLTOKEN_ANGLE);
   xmltokens.Register ("min", XMLTOKEN_MIN);
   xmltokens.Register ("max", XMLTOKEN_MAX);
+  // csODE* related tokens
+  xmltokens.Register ("simulationmode", XMLTOKEN_SIMULATIONMODE);
+  xmltokens.Register ("autodisable", XMLTOKEN_AUTODISABLE);
+  xmltokens.Register ("worldstep", XMLTOKEN_WORLDSTEP);
+  xmltokens.Register ("stepfast", XMLTOKEN_STEPFAST);
+  xmltokens.Register ("quickstep", XMLTOKEN_QUICKSTEP);
   return true;
 }
 
 csPtr<iBase> csPhysicsLoader::Parse (iDocumentNode* node,
-		iLoaderContext* /*ldr_context*/,
+		iStreamSource*, iLoaderContext* /*ldr_context*/,
 		iBase* /*context*/)
 {
   engine = CS_QUERY_REGISTRY (object_reg, iEngine);
   CS_ASSERT (engine != 0);
   csRef<iDynamics> dynamics = CS_QUERY_REGISTRY (object_reg, iDynamics);
-  if (dynamics == 0) {
+  if (dynamics == 0)
+  {
+    synldr->ReportError ("crystalspace.dynamics.loader",
+                node, "No dynamics in object registry!");
     return 0;
   }
 
@@ -147,12 +167,24 @@ csPtr<iBase> csPhysicsLoader::Parse (iDocumentNode* node,
     if (child->GetType () != CS_NODE_ELEMENT) continue;
     const char *value = child->GetValue ();
     csStringID id = xmltokens.Request (value);
-    if (id == XMLTOKEN_SYSTEM) {
-      csRef<iDynamicSystem> system = dynamics->CreateSystem ();
-      if (!ParseSystem (child, system)) {
-	    return 0;
-	  }
-    } else {
+    if (id == XMLTOKEN_SYSTEM)
+    {
+      csRef<iDocumentAttribute> attr = child->GetAttribute ("name");
+      csRef<iDynamicSystem> system;
+      if (attr)
+	system = dynamics->FindSystem (attr->GetValue ());
+      if (!system)
+      {
+        system = dynamics->CreateSystem ();
+	if (attr)
+	  system->QueryObject ()->SetName (attr->GetValue ());
+      }
+      if (!ParseSystem (child, system))
+      {
+	return 0;
+      }
+    } else
+    {
       synldr->ReportBadToken (child);
       return 0;
     }
@@ -162,8 +194,26 @@ csPtr<iBase> csPhysicsLoader::Parse (iDocumentNode* node,
 
 bool csPhysicsLoader::ParseSystem (iDocumentNode* node, iDynamicSystem* system)
 {
+  // Get name for system
   const char *name = node->GetAttributeValue ("name");
   system->QueryObject()->SetName (name);
+  // Look for ODE specific properties
+  csRef<iODEDynamicSystemState> osys= SCF_QUERY_INTERFACE (
+			system, iODEDynamicSystemState);
+  if (osys)
+  {
+    if (node->GetAttribute("cfm"))
+    {
+      float cfm = node->GetAttributeValueAsFloat ("cfm");
+      osys->SetCFM(cfm);
+    }
+    if (node->GetAttribute("erp"))
+    {
+      float erp = node->GetAttributeValueAsFloat ("erp");
+      osys->SetERP(erp);
+    }
+  }
+  // Get all tokens
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
   {
@@ -174,16 +224,17 @@ bool csPhysicsLoader::ParseSystem (iDocumentNode* node, iDynamicSystem* system)
     switch (id)
     {
       case XMLTOKEN_GRAVITY:
-      { 
+      {
         csVector3 v;
-        if (!synldr->ParseVector (child, v)) {
+        if (!synldr->ParseVector (child, v))
+	{
           synldr->ReportError ("crystalspace.dynamics.loader",
                 child, "Error processing gravity token");
           return false;
         }
         system->SetGravity (v);
         break;
-      } 
+      }
       case XMLTOKEN_DAMPENER:
       {
         float angular = child->GetAttributeValueAsFloat ("angular");
@@ -193,7 +244,7 @@ bool csPhysicsLoader::ParseSystem (iDocumentNode* node, iDynamicSystem* system)
         break;
       }
       case XMLTOKEN_GROUP:
-	  {
+      {
         csRef<iBodyGroup> group = system->CreateGroup ();
         csRef<iDocumentNodeIterator> git = child->GetNodes ();
         while (git->HasNext ())
@@ -202,44 +253,105 @@ bool csPhysicsLoader::ParseSystem (iDocumentNode* node, iDynamicSystem* system)
           if (gchild->GetType () != CS_NODE_ELEMENT) continue;
           const char *value = gchild->GetValue ();
           csStringID id = xmltokens.Request (value);
-          if (id == XMLTOKEN_BODY) {
+          if (id == XMLTOKEN_BODY)
+	  {
             csRef<iRigidBody> body = system->CreateBody ();
             group->AddBody (body);
-            if (!ParseBody (gchild, body)) {
-			  return false;
-			}
-          } else {
+            if (!ParseBody (gchild, body))
+	    {
+	      return false;
+	    }
+          }
+	  else
+	  {
             synldr->ReportBadToken (gchild);
           }
         }
         break;
       }
       case XMLTOKEN_COLLIDERMESH:
-	    if (!ParseSystemColliderMesh (child, system)) return false;
-		break;
+      {
+	if (!ParseSystemColliderMesh (child, system)) return false;
+	break;
+      }
       case XMLTOKEN_COLLIDERSPHERE:
-	    if (!ParseSystemColliderSphere (child, system)) return false;
-		break;
+      {
+	if (!ParseSystemColliderSphere (child, system)) return false;
+	break;
+      }
+      case XMLTOKEN_COLLIDERPLANE:
+      {
+        if (!ParseSystemColliderPlane (child, system)) return false;
+    	break;
+      }
       case XMLTOKEN_COLLIDERCYLINDER:
-	    if (!ParseSystemColliderCylinder (child, system)) return false;
-		break;
+      {
+	if (!ParseSystemColliderCylinder (child, system)) return false;
+	break;
+      }
       case XMLTOKEN_COLLIDERBOX:
-	    if (!ParseSystemColliderBox (child, system)) return false;
-		break;
+      {
+	if (!ParseSystemColliderBox (child, system)) return false;
+	break;
+      }
       case XMLTOKEN_BODY:
       {
         csRef<iRigidBody> body = system->CreateBody ();
-        if (!ParseBody (child, body)) {
-		  return false;
-		}
+        if (!ParseBody (child, body)) return false;
         break;
       }
       case XMLTOKEN_JOINT:
       {
         csRef<iJoint> joint = system->CreateJoint ();
-        if (!ParseJoint (child, joint, system)) {
-		  return false;
-		}
+        if (!ParseJoint (child, joint, system)) return false;
+        break;
+      }
+      case XMLTOKEN_AUTODISABLE:
+      {
+	csRef<iODEDynamicSystemState> osys= SCF_QUERY_INTERFACE (
+			system, iODEDynamicSystemState);
+	if (osys)
+	{
+	  bool autodisable;
+	  if (!synldr->ParseBool (child, autodisable, false))
+	  {
+	    synldr->ReportError ("crystalspace.dynamics.loader",
+		child, "Error processing autodisable token");
+	    return false;
+	  }
+	  osys->EnableAutoDisable(autodisable);
+	}
+	break;
+
+      }
+      case XMLTOKEN_SIMULATIONMODE:
+      {
+    	const char* sm = child->GetContentsValue ();
+	csStringID sm_id = xmltokens.Request (sm);
+
+	csRef<iODEDynamicSystemState> osys= SCF_QUERY_INTERFACE (
+			system, iODEDynamicSystemState);
+	if (osys)
+	{
+	  switch (sm_id)
+	  {
+	    case XMLTOKEN_WORLDSTEP:
+		osys->EnableQuickStep(false);
+		osys->EnableStepFast(false);
+		break;
+	    case XMLTOKEN_STEPFAST:
+		osys->EnableStepFast(true);
+		break;
+	    case XMLTOKEN_QUICKSTEP:
+		osys->EnableQuickStep(true);
+		break;
+	    default:
+          	synldr->ReportBadToken (child);
+        	return false;
+	  }
+	}
+	// if there is no iODEDynamicSystemState simply
+	// ignore the property so maps can still be loaded.
         break;
       }
       default:
@@ -255,6 +367,11 @@ bool csPhysicsLoader::ParseBody (iDocumentNode* node, iRigidBody* body)
   const char *name = node->GetAttributeValue ("name");
   body->QueryObject()->SetName (name);
   float mass = node->GetAttributeValueAsFloat ("mass");
+  bool enabled=true;
+  if (node->GetAttribute ("enabled"))
+    enabled=node->GetAttributeValueAsBool("enabled");
+  if (!enabled)
+    body->Disable();
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -263,7 +380,8 @@ bool csPhysicsLoader::ParseBody (iDocumentNode* node, iRigidBody* body)
     if (child->GetType () != CS_NODE_ELEMENT) continue;
     const char *value = child->GetValue ();
     csStringID id = xmltokens.Request (value);
-    switch (id) {
+    switch (id)
+    {
       case XMLTOKEN_STATIC:
         body->MakeStatic ();
         break;
@@ -312,7 +430,7 @@ bool csPhysicsLoader::ParseCollider (iDocumentNode* node, iRigidBody* body)
   float d = node->GetAttributeValueAsFloat ("density");
   float e = node->GetAttributeValueAsFloat ("elasticity");
   float s = node->GetAttributeValueAsFloat ("softness");
-   
+
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
   {
@@ -330,12 +448,13 @@ bool csPhysicsLoader::ParseCollider (iDocumentNode* node, iRigidBody* body)
 	    child, "No mesh specified for collidermesh");
 	  return false;
         }
-        iMeshWrapper *m = engine->FindMeshObject (child->GetAttributeValue ("mesh"));
+        iMeshWrapper *m = engine->FindMeshObject (child->GetAttributeValue (
+				"mesh"));
 	csOrthoTransform t;
 	ParseTransform (child, t);
         if (m)
 	{
-          if( s > 0)
+          if (s > 0)
             body->AttachColliderMesh (m, t, f, d, e, s);
           else  //no softness parameter, so use default
             body->AttachColliderMesh (m, t, f, d, e);
@@ -365,10 +484,17 @@ bool csPhysicsLoader::ParseCollider (iDocumentNode* node, iRigidBody* body)
         body->AttachColliderCylinder (l, r, t, f, d, e, s);
         break;
       }
+      case XMLTOKEN_COLLIDERPLANE:
+      {
+        csPlane3 plane;
+        synldr->ParsePlane (node, plane);
+        body->AttachColliderPlane (plane, f, d, e, s);
+      }
       case XMLTOKEN_COLLIDERBOX:
       {
         csVector3 v;
-        if (!synldr->ParseVector (child, v)) {
+        if (!synldr->ParseVector (child, v))
+	{
           synldr->ReportError ("crystalspace.dynamics.loader",
                 child, "Error processing box parameters");
           return false;
@@ -410,24 +536,45 @@ bool csPhysicsLoader::ParseSystemColliderMesh (
   return true;
 }
 
-bool csPhysicsLoader::ParseSystemColliderSphere (iDocumentNode* node, iDynamicSystem* system)
+static float GetFloat (iDocumentNode* node, const char* name, float def)
 {
-  float f = node->GetAttributeValueAsFloat ("friction");
-  float e = node->GetAttributeValueAsFloat ("elasticity");
-  float s = node->GetAttributeValueAsFloat ("softness");
+  csRef<iDocumentAttribute> attr = node->GetAttribute (name);
+  if (!attr) return def;
+  return attr->GetValueAsFloat ();
+}
+
+bool csPhysicsLoader::ParseSystemColliderPlane (iDocumentNode *node,
+		iDynamicSystem* system)
+{
+  float f = GetFloat (node, "friction", 1.0f);
+  float e = GetFloat (node, "elasticity", 0.0f);
+  float s = GetFloat (node, "softness", 0.0f);
+  csPlane3 plane;
+  synldr->ParsePlane (node, plane);
+  system->AttachColliderPlane (plane, f, e, s);
+  return true;
+}
+
+bool csPhysicsLoader::ParseSystemColliderSphere (iDocumentNode* node,
+		iDynamicSystem* system)
+{
+  float f = GetFloat (node, "friction", 1.0f);
+  float e = GetFloat (node, "elasticity", 0.0f);
+  float s = GetFloat (node, "softness", 0.0f);
   float r = node->GetAttributeValueAsFloat ("radius");
   csOrthoTransform t;
   ParseTransform (node, t);
   system->AttachColliderSphere (r, t.GetOrigin(), f, e, s);
   return true;
 }
-  
 
-bool csPhysicsLoader::ParseSystemColliderCylinder (iDocumentNode* node, iDynamicSystem* system)
+
+bool csPhysicsLoader::ParseSystemColliderCylinder (iDocumentNode* node,
+		iDynamicSystem* system)
 {
-  float f = node->GetAttributeValueAsFloat ("friction");
-  float e = node->GetAttributeValueAsFloat ("elasticity");
-  float s = node->GetAttributeValueAsFloat ("softness");
+  float f = GetFloat (node, "friction", 1.0f);
+  float e = GetFloat (node, "elasticity", 0.0f);
+  float s = GetFloat (node, "softness", 0.0f);
   float l = node->GetAttributeValueAsFloat ("length");
   float r = node->GetAttributeValueAsFloat ("radius");
   csOrthoTransform t;
@@ -436,13 +583,15 @@ bool csPhysicsLoader::ParseSystemColliderCylinder (iDocumentNode* node, iDynamic
   return true;
 }
 
-bool csPhysicsLoader::ParseSystemColliderBox (iDocumentNode* node, iDynamicSystem* system)
+bool csPhysicsLoader::ParseSystemColliderBox (iDocumentNode* node,
+		iDynamicSystem* system)
 {
-  float f = node->GetAttributeValueAsFloat ("friction");
-  float e = node->GetAttributeValueAsFloat ("elasticity");
-  float s = node->GetAttributeValueAsFloat ("softness");
+  float f = GetFloat (node, "friction", 1.0f);
+  float e = GetFloat (node, "elasticity", 0.0f);
+  float s = GetFloat (node, "softness", 0.0f);
   csVector3 v;
-  if (!synldr->ParseVector (node, v)) {
+  if (!synldr->ParseVector (node, v))
+  {
     synldr->ReportError ("crystalspace.dynamics.loader",
       node, "Error processing box parameters");
     return false;
@@ -483,9 +632,10 @@ bool csPhysicsLoader::ParseTransform (iDocumentNode* node, csOrthoTransform &t)
   return true;
 }
 
-bool csPhysicsLoader::ParseJoint (iDocumentNode* node, iJoint* joint, iDynamicSystem* system)
+bool csPhysicsLoader::ParseJoint (iDocumentNode* node, iJoint* joint,
+		iDynamicSystem* system)
 {
-  joint->SetTransConstraints (false, false, false); 
+  joint->SetTransConstraints (false, false, false);
   joint->SetRotConstraints (false, false, false);
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
@@ -503,9 +653,16 @@ bool csPhysicsLoader::ParseJoint (iDocumentNode* node, iJoint* joint, iDynamicSy
       case XMLTOKEN_BODY:
         if (body1 == 0)
 	{
-          if (child->GetContentsValue ())
+	  const char* name = child->GetContentsValue ();
+          if (name)
 	  {
-            body1 = system->FindBody (child->GetContentsValue());
+            body1 = system->FindBody (name);
+	    if (!body1)
+	    {
+              synldr->ReportError ("crystalspace.dynamics.loader",
+                child, "Can't find body with name '%s'!", name);
+              return false;
+	    }
           }
 	  else
 	  {
@@ -516,9 +673,16 @@ bool csPhysicsLoader::ParseJoint (iDocumentNode* node, iJoint* joint, iDynamicSy
         }
 	else if (body2 == 0)
 	{
-          if (child->GetContentsValue ())
+	  const char* name = child->GetContentsValue ();
+          if (name)
 	  {
-            body2 = system->FindBody (child->GetContentsValue());
+            body2 = system->FindBody (name);
+	    if (!body2)
+	    {
+              synldr->ReportError ("crystalspace.dynamics.loader",
+                child, "Can't find body with name '%s'!", name);
+              return false;
+	    }
           }
 	  else
 	  {
@@ -596,7 +760,8 @@ bool csPhysicsLoader::ParseJoint (iDocumentNode* node, iJoint* joint, iDynamicSy
   return true;
 }
 
-bool csPhysicsLoader::ParseConstraint (iDocumentNode *node, bool &x, bool &y, bool &z, csVector3 &min, csVector3 &max)
+bool csPhysicsLoader::ParseConstraint (iDocumentNode *node, bool &x,
+		bool &y, bool &z, csVector3 &min, csVector3 &max)
 {
   x = strcmp (node->GetAttributeValue ("x"), "true") == 0;
   y = strcmp (node->GetAttributeValue ("y"), "true") == 0;
