@@ -16,8 +16,8 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <string.h>
 #include "cssysdef.h"
+#include <string.h>
 #include "csver.h"
 #include "csutil/scf.h"
 #include "csutil/util.h"
@@ -35,7 +35,7 @@ class csReporterIterator : public iReporterIterator
 {
 public:
   csPDelArray<csReporterMessage> messages;
-  int idx;
+  size_t idx;
 
 public:
   SCF_DECLARE_IBASE;
@@ -122,19 +122,17 @@ void csReporter::Report (int severity, const char* msgId,
   va_end (arg);
 }
 
-CS_IMPLEMENT_STATIC_VAR(GetReportBuf, csString, ());
-
 void csReporter::ReportV (int severity, const char* msgId,
   	const char* description, va_list arg)
 {
-  csString& buf = *GetReportBuf();
+  csStringFast<768> buf;
   buf.FormatV (description, arg);
 
   // To ensure thread-safety we first copy the listeners.
   csRefArray<iReporterListener> copy;
   {
     csScopedMutexLock lock (mutex);
-    int i;
+    size_t i;
     for (i = 0 ; i < listeners.Length () ; i++)
     {
       iReporterListener* listener = listeners[i];
@@ -143,7 +141,7 @@ void csReporter::ReportV (int severity, const char* msgId,
   }
 
   bool add_msg = true;
-  int i;
+  size_t i;
   for (i = 0 ; i < copy.Length () ; i++)
   {
     iReporterListener* listener = copy[i];
@@ -175,8 +173,8 @@ void csReporter::Clear (int severity)
 {
   csScopedMutexLock lock (mutex);
 
-  int i = 0;
-  int len = messages.Length ();
+  size_t i = 0;
+  size_t len = messages.Length ();
   while (i < len)
   {
     csReporterMessage* msg = messages[i];
@@ -195,8 +193,8 @@ void csReporter::Clear (int severity)
 void csReporter::Clear (const char* mask)
 {
   csScopedMutexLock lock (mutex);
-  int i = 0;
-  int len = messages.Length ();
+  size_t i = 0;
+  size_t len = messages.Length ();
   while (i < len)
   {
     csReporterMessage* msg = messages[i];
@@ -216,7 +214,7 @@ csPtr<iReporterIterator> csReporter::GetMessageIterator ()
 {
   csScopedMutexLock lock (mutex);
   csReporterIterator* it = new csReporterIterator ();
-  int i;
+  size_t i;
   for (i = 0 ; i < messages.Length () ; i++)
   {
     csReporterMessage* msg = new csReporterMessage ();
@@ -237,8 +235,8 @@ void csReporter::AddReporterListener (iReporterListener* listener)
 void csReporter::RemoveReporterListener (iReporterListener* listener)
 {
   csScopedMutexLock lock (mutex);
-  int idx = listeners.Find (listener);
-  if (idx != -1)
+  size_t idx = listeners.Find (listener);
+  if (idx != csArrayItemNotFound)
   {
     listeners.DeleteIndex (idx);
   }
@@ -247,8 +245,8 @@ void csReporter::RemoveReporterListener (iReporterListener* listener)
 bool csReporter::FindReporterListener (iReporterListener* listener)
 {
   csScopedMutexLock lock (mutex);
-  int idx = listeners.Find (listener);
-  return idx != -1;
+  size_t idx = listeners.Find (listener);
+  return idx != csArrayItemNotFound;
 }
 
 csReporterMessage::~csReporterMessage ()
