@@ -15,10 +15,11 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#include <string.h>
 
 #include "cssysdef.h"
+#include <string.h>
 #include "csutil/sysfunc.h"
+#include "csutil/event.h"
 #include "sequence.h"
 #include "csutil/scf.h"
 #include "iutil/objreg.h"
@@ -81,12 +82,13 @@ void csSequence::DeleteFirstSequence ()
 }
 
 void csSequence::AddOperation (csTicks time, iSequenceOperation* operation,
-	iBase* params)
+	iBase* params, uint sequence_id)
 {
   csSequenceOp* op = new csSequenceOp ();
   op->time = time;
   op->operation = operation;
   op->params = params;
+  op->sequence_id = sequence_id;
   // Insert this operation at the right time.
   if (first)
   {
@@ -122,10 +124,10 @@ void csSequence::AddOperation (csTicks time, iSequenceOperation* operation,
 }
 
 void csSequence::AddRunSequence (csTicks time, iSequence* sequence,
-	iBase* params)
+	iBase* params, uint sequence_id)
 {
   RunSequenceOp* op = new RunSequenceOp (seqmgr, sequence);
-  AddOperation (time, op, params);
+  AddOperation (time, op, params, sequence_id);
   op->DecRef ();
 }
 
@@ -137,11 +139,11 @@ void csSequence::RunSequenceOp::Do (csTicks dt, iBase* params)
 
 void csSequence::AddCondition (csTicks time, iSequenceCondition* condition,
   	iSequence* trueSequence, iSequence* falseSequence,
-	iBase* params)
+	iBase* params, uint sequence_id)
 {
   RunCondition* op = new RunCondition (seqmgr, condition, trueSequence,
   	falseSequence);
-  AddOperation (time, op, params);
+  AddOperation (time, op, params, sequence_id);
   op->DecRef ();
 }
 
@@ -160,16 +162,16 @@ void csSequence::RunCondition::Do (csTicks dt, iBase* params)
 }
 
 void csSequence::AddLoop (csTicks time, iSequenceCondition* condition,
-  	iSequence* sequence, iBase* params)
+  	iSequence* sequence, iBase* params, uint sequence_id)
 {
   RunLoop* op = new RunLoop (seqmgr, condition, sequence);
-  AddOperation (time, op, params);
+  AddOperation (time, op, params, sequence_id);
   op->DecRef ();
 }
 
 void csSequence::RunLoop::Do (csTicks dt, iBase* params)
 {
-  while (condition->Condition (dt, params))
+  while (sequence && condition->Condition (dt, params))
     seqmgr->RunSequence (-(signed)dt, sequence, params);
 }
 
@@ -202,6 +204,7 @@ csSequenceManager::csSequenceManager (iBase *iParent) :
   previous_time_valid = false;
   main_time = 0;
   suspended = true;
+  sequence_id = 0;
 }
 
 csSequenceManager::~csSequenceManager ()
@@ -223,11 +226,12 @@ bool csSequenceManager::Initialize (iObjectRegistry *r)
 {
   object_reg = r;
   vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
+  CS_INITIALIZE_EVENT_SHORTCUTS (object_reg);
   if (!scfiEventHandler)
     scfiEventHandler = new EventHandler (this);
   csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
   if (q != 0)
-    q->RegisterListener (scfiEventHandler, CSMASK_Nothing);
+    q->RegisterListener (scfiEventHandler, FinalProcess);
   return true;
 }
 
@@ -235,8 +239,8 @@ bool csSequenceManager::HandleEvent (iEvent &event)
 {
   // Sequence manager must be final because engine sequence manager
   // must come first. @@@ HACKY
-  if (event.Type != csevBroadcast
-   || event.Command.Code != cscmdFinalProcess)
+  // \todo : convert this to a subscription ordering constraint
+  if (event.Name != FinalProcess)
     return false;
 
   if (!suspended)
@@ -267,7 +271,7 @@ void csSequenceManager::Clear ()
   main_sequence->Clear ();
   main_time = 0;
   previous_time_valid = false;
-  int i;
+  size_t i;
   for (i = 0 ; i < sequences.Length () ; i++)
   {
     csWeakRef<csSequence>* seq = sequences[i];
@@ -338,7 +342,7 @@ csPtr<iSequence> csSequenceManager::NewSequence ()
     cnt = 0;
     if (sequences.Length () > 100)
     {
-      int i;
+      size_t i;
       csArray<csWeakRef<csSequence>* > copy;
       for (i = 0 ; i < sequences.Length () ; i++)
       {
@@ -353,18 +357,35 @@ csPtr<iSequence> csSequenceManager::NewSequence ()
 }
 
 void csSequenceManager::RunSequence (csTicks time, iSequence* sequence,
-	iBase* params)
+	iBase* params, uint sequence_id)
 {
   csSequence* seq = (csSequence*)sequence;
   csSequenceOp* op = seq->GetFirstSequence ();
   while (op)
   {
     main_sequence->AddOperation (main_time + time + op->time, op->operation,
-    	params ? params : (iBase*)op->params);
+    	params ? params : (iBase*)op->params, sequence_id);
     op = op->next;
   }
 }
 
+void csSequenceManager::DestroySequenceOperations (uint sequence_id)
+{
+  csSequenceOp* op = main_sequence->GetFirstSequence ();
+  while (op)
+  {
+    csSequenceOp* op_next = op->next;
+    if (op->sequence_id == sequence_id)
+    {
+      if (op->next) op->next->prev = op->prev;
+      if (op->prev) op->prev->next = op->next;
+      else main_sequence->SetFirstSequence (op->next);
+      delete op;
+    }
+    op = op_next;
+  }
+}
+
 SCF_IMPLEMENT_IBASE (csSequence::StandardOperation)
   SCF_IMPLEMENTS_INTERFACE (iSequenceOperation)
 SCF_IMPLEMENT_IBASE_END
