@@ -2,7 +2,7 @@
 	Copyright (C) 1998, 1999 by Nathaniel 'NooTe' Saint Martin
 	Copyright (C) 1998, 1999 by Jorrit Tyberghein
 	Written by Nathaniel 'NooTe' Saint Martin
-      Linux sound driver by Gert Steenssens <gsteenss@eps.agfa.be>
+	GNU/Linux sound driver by Gert Steenssens <gsteenss@eps.agfa.be>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -29,11 +29,11 @@
 #include <signal.h>
 
 #include "cssysdef.h"
-#if defined(CS_HAS_MACHINE_SOUNDCARD_H)
+#if defined(CS_HAVE_MACHINE_SOUNDCARD_H)
 #  include <machine/soundcard.h>
-#elif defined(CS_HAS_SYS_SOUNDCARD_H)
+#elif defined(CS_HAVE_SYS_SOUNDCARD_H)
 #  include <sys/soundcard.h>
-#else // CS_HAS_SOUNDCARD_H
+#else // CS_HAVE_SOUNDCARD_H
 #  include <soundcard.h>
 #endif
 #include "csutil/scf.h"
@@ -54,7 +54,7 @@ SCF_IMPLEMENT_FACTORY(csSoundDriverOSS)
 SCF_IMPLEMENT_IBASE(csSoundDriverOSS)
   SCF_IMPLEMENTS_INTERFACE(iSoundDriver)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iComponent)
-SCF_IMPLEMENT_IBASE_END;
+SCF_IMPLEMENT_IBASE_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csSoundDriverOSS::eiComponent)
   SCF_IMPLEMENTS_INTERFACE (iComponent)
