@@ -16,17 +16,17 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "cssysdef.h"
 #include "isound/loader.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
-#include "../common/soundraw.h"
-#include "../common/sndload.h"
+#include "csplugincommon/soundloader/soundraw.h"
+#include "csplugincommon/soundloader/sndload.h"
 
 CS_IMPLEMENT_PLUGIN
 
@@ -57,13 +57,13 @@ public:
     SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
     SCF_DESTRUCT_IBASE();
   }
-  virtual csPtr<iSoundData> LoadSound(void *Buffer, uint32 Size);
+  virtual csPtr<iSoundData> LoadSound(void *Buffer, size_t Size);
 };
 
 SCF_IMPLEMENT_IBASE(csSoundLoader_AIFF)
   SCF_IMPLEMENTS_INTERFACE(iSoundLoader)
   SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iComponent)
-SCF_IMPLEMENT_IBASE_END;
+SCF_IMPLEMENT_IBASE_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csSoundLoader_AIFF::eiComponent)
   SCF_IMPLEMENTS_INTERFACE (iComponent)
@@ -84,7 +84,7 @@ SCF_IMPLEMENT_FACTORY(csSoundLoader_AIFF)
 #define Stream buf[index]
 
 csPtr<iSoundData>
-csSoundLoader_AIFF::LoadSound(void* databuf, uint32 size)
+csSoundLoader_AIFF::LoadSound(void* databuf, size_t size)
 {
   uint8 *buf = (uint8*) databuf;
   unsigned long index=0;
