@@ -27,10 +27,17 @@ SCF_IMPLEMENT_IBASE (csOggSoundData)
   SCF_IMPLEMENTS_INTERFACE (iSoundData)
 SCF_IMPLEMENT_IBASE_END
 
+// As of 2004/10/05 the specs to which these functions (should) adhere is located at
+// http://www.xiph.org/ogg/vorbis/doc/vorbisfile/callbacks.html
+
 static size_t cs_ogg_read (void *ptr, size_t size, size_t nmemb,
   void *datasource)
 {
   csOggSoundData::datastore *ds = (csOggSoundData::datastore*)datasource;
+
+  // size_t is unsigned, be careful with subtraction.  A 0 return indicates end of stream.
+  if (ds->length <= ds->pos)
+    return 0;
   size_t br = MIN (size*nmemb, ds->length - ds->pos);
 
   memcpy (ptr, ds->data+ds->pos, br);
@@ -47,13 +54,15 @@ static int cs_ogg_seek (void *datasource, ogg_int64_t offset, int whence)
   if (whence == SEEK_SET)
     np = offset;
   else if (whence == SEEK_CUR)
-    np = ds->pos + offset;
+    np = (size_t)(ds->pos + offset);
   else if (whence == SEEK_END)
-    np = ds->length + offset - 1;
+    np = (size_t)(ds->length + offset);
   else
     return -1;
 
-  if (np >= ds->length)
+  // Since np is unsigned, a negative value will wrap around to a huge positive value, and 
+  //  inevitably be well beyond the length.
+  if (np > ds->length)
     return -1;
 
   ds->pos = np;
@@ -68,7 +77,7 @@ static int cs_ogg_close (void *)
 static long cs_ogg_tell (void *datasource)
 {
   csOggSoundData::datastore *ds = (csOggSoundData::datastore*)datasource;
-  return ds->pos;
+  return (long)ds->pos;
 }
 
 csOggSoundData::cs_ov_callbacks::cs_ov_callbacks ()
@@ -191,10 +200,10 @@ void *csOggSoundData::ReadStreamed(long &NumSamples)
     while (bytes_read && buffersize)
     {
       
-      bytes_read = ov_read (&vf, (char *)write_ptr, buffersize, endian,
+      bytes_read = ov_read (&vf, (char *)write_ptr, (int)buffersize, endian,
 			       fmt.Bits>>3, 1, &current_section);
 
-      NumSamples += bytes_read / ((fmt.Bits >> 3) * fmt.Channels);
+      NumSamples += (long)bytes_read / ((fmt.Bits >> 3) * fmt.Channels);
       buffersize-=bytes_read;
       write_ptr += bytes_read;
     }
@@ -231,11 +240,11 @@ public:
     SCF_DESTRUCT_IBASE();
   }
 
-  virtual csPtr<iSoundData> LoadSound (void *Buffer, uint32 Size)
+  virtual csPtr<iSoundData> LoadSound (void *Buffer, size_t Size)
   {
     csOggSoundData *sd=0;
     if (csOggSoundData::IsOgg (Buffer, Size))
-      sd = new csOggSoundData ((iBase*)this, (uint8*)Buffer, (size_t) Size);
+      sd = new csOggSoundData ((iBase*)this, (uint8*)Buffer, Size);
 
     return csPtr<iSoundData> (sd);
   }
