@@ -19,18 +19,20 @@
 */
 
 #include "cssysdef.h"
+#include "csplugincommon/directx/guids.h"
 #include "csutil/sysfunc.h"
+#include "csutil/event.h"
+#include "csutil/scf.h"
 #include <stdio.h>
 
-#include "csutil/scf.h"
 #include "iutil/cfgfile.h"
 #include "iutil/event.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
 #include "iutil/virtclk.h"
 #include "ivaria/reporter.h"
+#include "ivideo/graph2d.h"
 
-#define INITGUID
 #include "sndrdr.h"
 #include "sndlstn.h"
 #include "sndsrc.h"
@@ -74,10 +76,12 @@ csSoundRenderDS3D::csSoundRenderDS3D(iBase *piBase)
 bool csSoundRenderDS3D::Initialize(iObjectRegistry *r)
 {
   object_reg = r;
+  CS_INITIALIZE_EVENT_SHORTCUTS (object_reg);
   csRef<iEventQueue> q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
-  if (q != 0)
-    q->RegisterListener (&scfiEventHandler,
-    CSMASK_Command | CSMASK_Broadcast | CSMASK_Nothing);
+  if (q != 0) {
+    csEventID events[] = { PreProcess, SystemOpen, SystemClose, CS_EVENTLIST_END };
+    q->RegisterListener (&scfiEventHandler, events);
+  }
   
   win32Assistant = CS_QUERY_REGISTRY (object_reg, iWin32Assistant);
   if (!win32Assistant)
@@ -108,10 +112,7 @@ csSoundRenderDS3D::~csSoundRenderDS3D()
 //  Pre-Multi Threaded section
 bool csSoundRenderDS3D::Open()
 {
-  csRef<iReporter> reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  if (reporter)
-    reporter->Report (CS_REPORTER_SEVERITY_NOTIFY,
-    "crystalspace.sound.ds3d",
+  Report (CS_REPORTER_SEVERITY_NOTIFY,
     "SoundRender DirectSound3D selected");
 
   HRESULT r;
@@ -120,29 +121,30 @@ bool csSoundRenderDS3D::Open()
     r = DirectSoundCreate(0, &AudioRenderer, 0);
     if (r != DS_OK)
     {
-      if (reporter)
-        reporter->Report (CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.sound.ds3d",
+      Report (CS_REPORTER_SEVERITY_ERROR,
         "Error : Cannot Initialize DirectSound3D (%s).", GetError(r));
-      else
-        csPrintf (
-        "Error : Cannot Initialize DirectSound3D (%s).\n", GetError(r));
       Close();
       return false;
     }
 
+    csRef<iGraphics2D> g2d = csQueryRegistry<iGraphics2D> (object_reg);
+    if (!g2d.IsValid())
+    {
+      Report (CS_REPORTER_SEVERITY_ERROR, 
+	"A canvas is required");
+      return false;
+    }
+    csRef<iWin32Canvas> canvas = scfQueryInterface<iWin32Canvas> (g2d);
+    CS_ASSERT (canvas.IsValid());
+
     DWORD dwLevel = DSSCL_PRIORITY;
     r = AudioRenderer->SetCooperativeLevel(
-      win32Assistant->GetApplicationWindow(), dwLevel);
+      canvas->GetWindowHandle(), dwLevel);
     if (r != DS_OK)
     {
-      if (reporter)
-        reporter->Report (CS_REPORTER_SEVERITY_ERROR,
+      Report (CS_REPORTER_SEVERITY_ERROR,
         "crystalspace.sound.ds3d",
         "Error : Cannot Set Cooperative Level (%s).", GetError(r));
-      else
-        csPrintf (
-        "Error : Cannot Set Cooperative Level (%s).\n", GetError(r));
       Close();
       return false;
     }
@@ -159,8 +161,7 @@ bool csSoundRenderDS3D::Open()
 
   float vol = Config->GetFloat("Sound.Volume",-1);
   if (vol>=0) SetVolume(vol);
-  if (reporter)
-    reporter->Report (CS_REPORTER_SEVERITY_NOTIFY,
+  Report (CS_REPORTER_SEVERITY_NOTIFY,
     "crystalspace.sound.ds3d",
     "  Volume: %g\n", GetVolume());
 
@@ -249,8 +250,8 @@ void csSoundRenderDS3D::UnregisterSound(iSoundHandle *snd)
 {
   mutex_SoundHandles->LockWait();
   csRef<csSoundHandleDS3D> hdl = (csSoundHandleDS3D *)snd;
-  int n = SoundHandles.Find(hdl);
-  if (n != -1)
+  size_t n = SoundHandles.Find(hdl);
+  if (n != csArrayItemNotFound)
   {
     SoundHandles.DeleteIndex (n);
     hdl->Unregister();
@@ -265,7 +266,7 @@ iSoundListener *csSoundRenderDS3D::GetListener()
 
 void csSoundRenderDS3D::Update()
 {
-  int i;
+  size_t i;
   csTicks ct, et;
   ct = csGetTicks ();
 
@@ -315,7 +316,7 @@ void csSoundRenderDS3D::SetDirty()
 void csSoundRenderDS3D::AddSource(csSoundSourceDS3D *src)
 {
   mutex_ActiveSources->LockWait();
-  if (ActiveSources.Find(src)==-1)
+  if (ActiveSources.Find (src) == csArrayItemNotFound)
   {
     ActiveSources.Push(src);
   }
@@ -325,8 +326,8 @@ void csSoundRenderDS3D::AddSource(csSoundSourceDS3D *src)
 void csSoundRenderDS3D::RemoveSource(csSoundSourceDS3D *src)
 {
   mutex_ActiveSources->LockWait();
-  int n=ActiveSources.Find(src);
-  if (n!=-1)
+  size_t n = ActiveSources.Find(src);
+  if (n != csArrayItemNotFound)
   {
     ActiveSources.DeleteIndex (n);
   }
@@ -375,23 +376,28 @@ void csSoundRenderDS3D::ThreadProc ()
   }
 }
 
+void csSoundRenderDS3D::Report (int severity, const char* msg, ...)
+{
+  va_list args;
+  va_start (args, msg);
+  csReportV (object_reg, severity, "crystalspace.sound.ds3d", msg, args);
+  va_end (args);
+}
+
 bool csSoundRenderDS3D::HandleEvent (iEvent &e)
 {
-  if (e.Type == csevCommand || e.Type == csevBroadcast)
+  if (e.Name == PreProcess)
   {
-    switch (e.Command.Code)
-    {
-    case cscmdPreProcess:
-      if (!BackgroundProcessing)
-        Update(); // Only perform updates here if there is no background thread
-      break;
-    case cscmdSystemOpen:
-      Open();
-      break;
-    case cscmdSystemClose:
-      Close();
-      break;
-    }
+    if (!BackgroundProcessing)
+      Update(); // Only perform updates here if there is no background thread
+  }
+  else if (e.Name == SystemOpen)
+  {
+    Open();
+  }
+  else if (e.Name == SystemClose)
+  {
+    Close();
   }
   return false;
 }
