@@ -16,10 +16,10 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#ifndef __SNDHDL_H__
-#define __SNDHDL_H__
+#ifndef __CS_SNDHDLSOFTWARE_H__
+#define __CS_SNDHDLSOFTWARE_H__
 
-#include "../common/shdl.h"
+#include "csplugincommon/soundrenderer/shdl.h"
 
 class csSoundRenderSoftware;
 
@@ -33,7 +33,18 @@ public:
 
   void Unregister();
   virtual void vUpdate(void *buf, long NumSamples);
-  virtual iSoundSource *CreateSource(int Mode3d);
+  virtual csPtr<iSoundSource> CreateSource(int Mode3d);
+
+  virtual void IncSourceCount();
+  virtual void DecSourceCount();
+
+  virtual void ResetStream();
+
+  void ProcessReset();
+
+private:
+  bool need_reset;
+  int source_count;
 };
 
-#endif // __SNDHDL_H__
+#endif // __CS_SNDHDLSOFTWARE_H__
