@@ -62,6 +62,11 @@ class csSoundRenderSoftware : public iSoundRender
   csRef<csCondition> data;
   csRef<csThread> mixer;
 
+  // event names (optimization)
+  csEventID PreProcess;
+  csEventID SystemOpen;
+  csEventID SystemClose;
+
 public:
   SCF_DECLARE_IBASE;
   // The system driver.
@@ -131,6 +136,9 @@ public:
   // previous time the sound handles were updated
   csTicks LastTime;
 
+  // sound renderer has been opened
+  bool isOpen;
+
   struct eiComponent : public iComponent
   {
     SCF_DECLARE_EMBEDDED_IBASE(csSoundRenderSoftware);
@@ -153,6 +161,8 @@ public:
       SCF_DESTRUCT_IBASE();
     }
     virtual bool HandleEvent (iEvent& e) { return parent->HandleEvent(e); }
+    CS_EVENTHANDLER_NAMES("crystalspace.sound")
+    CS_EVENTHANDLER_NIL_CONSTRAINTS
   } * scfiEventHandler;
 };
 
