@@ -16,13 +16,15 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <string.h>
 #include "cssysdef.h"
+#include <string.h>
 #include "stdrep.h"
 #include "csver.h"
 #include "csutil/scf.h"
 #include "csutil/util.h"
 #include "csutil/sysfunc.h"
+#include "csutil/event.h"
+#include "iutil/cfgmgr.h"
 #include "iutil/event.h"
 #include "iutil/eventh.h"
 #include "iutil/eventq.h"
@@ -30,12 +32,13 @@
 #include "iutil/cmdline.h"
 #include "iutil/comp.h"
 #include "iutil/objreg.h"
+#include "iutil/verbositymanager.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/fontserv.h"
 #include "ivideo/natwin.h"
 #include "ivaria/conout.h"
-
+  
 CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_FACTORY (csReporterListener)
@@ -80,6 +83,7 @@ csReporterListener::csReporterListener (iBase *iParent)
   reporter = 0;
   scfiEventHandler = 0;
   silent = false;
+  append = false;
 
   debug_filename = DefaultDebugFilename ();
 #ifdef CS_DEBUG
@@ -163,13 +167,26 @@ bool csReporterListener::Initialize (iObjectRegistry* r)
   }
   csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
   if (q != 0)
-    q->RegisterListener (scfiEventHandler, CSMASK_Nothing);
-
+    q->RegisterListener (scfiEventHandler, csevPostProcess(object_reg));
+  
+  csRef<iConfigManager> cfg(CS_QUERY_REGISTRY (r, iConfigManager));
+  if ( cfg )
+  {
+    append = cfg->GetBool("Reporter.FileAppend", false );
+  }
+  
   csRef<iCommandLineParser> cmdline = CS_QUERY_REGISTRY (object_reg,
     iCommandLineParser);
   if (cmdline)
   {
-    bool verbose = cmdline->GetOption ("verbose") != 0;
+    silent = cmdline->GetOption ("silent") != 0;
+    append = cmdline->GetOption ("append") != 0;
+  }
+  csRef<iVerbosityManager> verbosemgr (
+    CS_QUERY_REGISTRY (object_reg, iVerbosityManager));
+  if (verbosemgr) 
+  {
+    bool verbose = verbosemgr->Enabled ("stdrep");
     if (verbose)
     {
       dest_stdout[CS_REPORTER_SEVERITY_WARNING] = true;
@@ -179,12 +196,29 @@ bool csReporterListener::Initialize (iObjectRegistry* r)
       dest_stdout[CS_REPORTER_SEVERITY_DEBUG] = true;
       dest_stderr[CS_REPORTER_SEVERITY_DEBUG] = false;
     }
-    silent = cmdline->GetOption ("silent") != 0;
   }
 
   return true;
 }
 
+static const char* consolePrefix[5] = 
+{
+  CS_ANSI_BK CS_ANSI_FM CS_ANSI_FI,
+  CS_ANSI_BK CS_ANSI_FR CS_ANSI_FI,
+  CS_ANSI_BK CS_ANSI_FY CS_ANSI_FI,
+  "",
+  ""
+};
+
+static const char* consoleSuffix[5] = 
+{
+  CS_ANSI_RST,
+  CS_ANSI_RST,
+  CS_ANSI_RST,
+  "",
+  ""
+};
+
 bool csReporterListener::Report (iReporter*, int severity,
 	const char* msgID, const char* description)
 {
@@ -204,8 +238,9 @@ bool csReporterListener::Report (iReporter*, int severity,
   {
     if (!repeatedID)
     {
-      csPrintf ("\n%s:\n", msgID);
+      stdoutTmp << '\n' << msgID << ":\n";
     }
+    stdoutTmp << consolePrefix[severity];
     int offset = 0;
     while (strlen(description+offset)>77)
     {
@@ -215,20 +250,25 @@ bool csReporterListener::Report (iReporter*, int severity,
       if (linebreak>0)
       {
         str.Truncate (linebreak);
-        csPrintf ("  %s\n", str.GetData ());
+	stdoutTmp << "  " << str << '\n';
         offset += linebreak+1;
       }
       else
       {
         //csPrintf ("  %0.77s\n", str.GetData ());
-        csPrintf ("  %.77s\n", str.GetData ());
+	stdoutTmp << "  " << str << '\n';
         offset += 77;
       }
     }
-    csPrintf ("  %s\n", description+offset);
+    //csPrintf ("  %s\n", description+offset);
+    stdoutTmp << "  " << description+offset << '\n';
+    stdoutTmp << consoleSuffix[severity];
+    csPrintf ("%s", stdoutTmp.GetData());
+    stdoutTmp.Truncate (0);
   }
   if (dest_stderr[severity])
-    csFPutErr (msg.GetData());
+    csPrintfErr ("%s%s%s", consolePrefix[severity], msg.GetData(), 
+      consoleSuffix[severity]);
   if (dest_console[severity] && console)
     console->PutText ("%s", msg.GetData());
   if (dest_alert[severity] && nativewm)
@@ -240,7 +280,25 @@ bool csReporterListener::Report (iReporter*, int severity,
     {
       csRef<iVFS> vfs (CS_QUERY_REGISTRY (object_reg, iVFS));
       if (vfs.IsValid())
-        debug_file = vfs->Open (debug_filename, VFS_FILE_WRITE);
+      {  
+        // If log does not exists then create a new one    
+        if ( !(vfs->Exists(debug_filename)) )
+        {
+            debug_file = vfs->Open (debug_filename, VFS_FILE_WRITE);
+        } 
+        else               
+        {
+          // if the log file exists open up using desired behaviour.
+          if ( append )
+          {
+            debug_file = vfs->Open (debug_filename, VFS_FILE_APPEND);            
+          }
+          else
+          { 
+            debug_file = vfs->Open (debug_filename, VFS_FILE_WRITE);
+          }    
+        }        
+      }        
     }
     if (debug_file.IsValid())
     {
@@ -272,15 +330,13 @@ bool csReporterListener::Report (iReporter*, int severity,
 
 bool csReporterListener::HandleEvent (iEvent& event)
 {
-  if (event.Type == csevBroadcast)
+  if (event.Name == csevPostProcess(object_reg))
   {
-    if (event.Command.Code == cscmdPostProcess)
-    {
       csScopedMutexLock lock (mutex);
-      int l = messages.Length ();
+      size_t l = messages.Length ();
       if (l > 0)
       {
-	int i;
+	size_t i;
         csRef<iGraphics3D> g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
 
         csRef<iGraphics2D> g2d = g3d->GetDriver2D ();
@@ -306,7 +362,7 @@ bool csReporterListener::HandleEvent (iEvent& event)
           int sep = g2d->FindRGB (int (255*0.7), int (255*0.7),
 	      int (180*0.7));
 
-	  int max_l = (sh-4-6-4-6) / (fh+6);
+	  size_t max_l = (sh-4-6-4-6) / (fh+6);
 	  if (l > max_l) l = max_l;
           int h = 0;
           int c = 0;
@@ -348,7 +404,7 @@ bool csReporterListener::HandleEvent (iEvent& event)
                 }
                 str.Format ("  %s", msg.GetData ());
                 h++;
-                l--;
+                if (l > 0) l--;
               }
               str.Format ("  %s", msg.GetData ());
               g2d->DrawBox (4, 4+h*(fh+6), sw-8, fh+6, bg[c]);
@@ -378,7 +434,6 @@ bool csReporterListener::HandleEvent (iEvent& event)
 	  }
         }
       }
-    }
   }
 
   return false;
@@ -403,10 +458,11 @@ void csReporterListener::SetReporter (iReporter* reporter)
   if (reporter) reporter->AddReporterListener (&scfiReporterListener);
 }
 
-void csReporterListener::SetDebugFile (const char* s)
+void csReporterListener::SetDebugFile (const char* s, bool appendFile )
 {
   debug_file = 0;
   debug_filename = s;
+  append = appendFile;
 }
 
 void csReporterListener::SetDefaults ()
