@@ -21,10 +21,8 @@
 #include "csgfx/rgbpixel.h"
 
 #include "igraphic/image.h"
-
 #include "iutil/objreg.h"
 #include "iutil/plugin.h"
-
 #include "ivaria/reporter.h"
 
 #include "simpleformer.h"
@@ -63,7 +61,7 @@ float BiLinearData (float* data, int width, int height, float x, float z)
 }
 
 // Bicubic weight function
-float WeightFunction (float x)
+static float WeightFunction (float x)
 {
   float a = (x+2)<0?0:(x+2);
   float b = (x+1)<0?0:(x+1);
@@ -82,6 +80,8 @@ float BiCubicData (float* data, int width, int height, float x, float z)
 
   // If deltaX/Z is close to 0, we're pretty much 
   // right on a heightmap sample point, which means it's useless to blend
+// Jorrit: Disabled this because it gives small irregularities.
+#if 0
   if (fabs (deltaX) <= SMALL_EPSILON && 
       fabs (deltaZ) <= SMALL_EPSILON)
   {
@@ -91,26 +91,39 @@ float BiCubicData (float* data, int width, int height, float x, float z)
     intZ = MAX (MIN (intZ, height-1), 0);
     return data[(int)intX+(int)intZ*width];
   }
+#endif
+
+  int ix = (int) floor (x) - 1;
+  int iy = (int) floor (z) - 1;
 
   // Grab 16 surrounding heights and blend them
-  for (int i=0; i<4; ++i)
+  for (int j=0; j<4; ++j)
   {
-    for (int j=0; j<4; ++j)
+    // Calculate integer position
+    int intZ = iy + j;
+    // Clamp coordinates to heightmap size
+    if (intZ < 0) intZ = 0;
+    else if (intZ > height-1) intZ = height-1;
+
+    // Multiply with width.
+    intZ *= width;
+
+    // Precalc weight function for 'j'.
+    float jweight = WeightFunction (j-1.0f-deltaZ);
+
+    for (int i=0; i<4; ++i)
     {
       // Calculate integer position
-      int intX = (int) floor (x) - 1 + i;
-      int intZ = (int) floor (z) - 1 + j;
-
+      int intX = ix + i;
       // Clamp coordinates to heightmap size
-      intX = MAX (MIN (intX, width-1), 0);
-      intZ = MAX (MIN (intZ, height-1), 0);
+      if (intX < 0) intX = 0;
+      else if (intX > width-1) intX = width-1;
 
       // Grab the height
-      float height = data[intX+intZ*width];
+      float height = data[intX+intZ];
 
       // Weight the height using a cubic weight function
-      height *= WeightFunction (i-1.0f-deltaX) * 
-                WeightFunction (j-1.0f-deltaZ);
+      height *= WeightFunction (i-1.0f-deltaX) * jweight;
 
       // Add the weighted height to the result
       result += height;
@@ -126,17 +139,17 @@ float BiCubicData (float* data, int width, int height, float x, float z)
 //////////////////////////////////////////////////////////////////////////
 
 SCF_IMPLEMENT_IBASE (csSimpleFormer)
-SCF_IMPLEMENTS_INTERFACE (iTerraFormer)
-SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iSimpleFormerState)
-SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
+  SCF_IMPLEMENTS_INTERFACE (iTerraFormer)
+  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iSimpleFormerState)
+  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
 SCF_IMPLEMENT_IBASE_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csSimpleFormer::SimpleFormerState)
-SCF_IMPLEMENTS_INTERFACE (iSimpleFormerState)
+  SCF_IMPLEMENTS_INTERFACE (iSimpleFormerState)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_EMBEDDED_IBASE (csSimpleFormer::Component)
-SCF_IMPLEMENTS_INTERFACE (iComponent)
+  SCF_IMPLEMENTS_INTERFACE (iComponent)
 SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csSimpleFormer)
@@ -167,40 +180,45 @@ csSimpleFormer::~csSimpleFormer ()
   SCF_DESTRUCT_IBASE();
 }
 
-void csSimpleFormer::SetHeightmap (iImage *heightmap)
+bool csSimpleFormer::SetIntegerMap (csStringID type, iImage* map,
+	int scale, int offset)
 {
-  // Grab dimensions
-  width = heightmap->GetWidth ();
-  height = heightmap->GetHeight ();
+  size_t w = map->GetWidth ();
+  size_t h = map->GetHeight ();
+  if (width != w || height != h) return false;
 
-  // Allocate height data
-  heightData = new float[width*height];
+  // First check if we already have an intmap of this type.
+  size_t intmap_idx = (size_t)~0;
+  size_t i = 0;
+  while (i < intmaps.Length ())
+    if (intmaps[i].type == type)
+    {
+      intmap_idx = i;
+      break;
+    }
+    else i++;
+  if (intmap_idx == (size_t)~0)
+    intmap_idx = intmaps.Push (csIntMap ());
+
+  csIntMap& intmap = intmaps[intmap_idx];
+  intmap.type = type;
+
+  // Allocate data
+  delete[] intmap.data;
+  intmap.data = new int[width*height];
 
   // Check what type of image we got
-  if (heightmap->GetFormat () & CS_IMGFMT_TRUECOLOR)
+  if (map->GetFormat () & CS_IMGFMT_TRUECOLOR)
   {
-    // It's a RGBA image
-    csRGBpixel *data = (csRGBpixel *)heightmap->GetImageData ();
-
-    unsigned int x, y;
-    // Loop through the data
-    for (y = 0; y < height; ++y)
-    {
-      for (x = 0; x < width; ++x)
-      {
-        // Grab the intensity as height
-        // We're reversing Y to later get negative Y in heightmap image 
-        // to positive Z in terrain
-        heightData[x+(height-y-1)*width] = 
-          data[x+y*width].Intensity ()/255.0;
-      }
-    }
+    // We don't support this!
+    intmaps.DeleteIndex (intmap_idx);
+    return false;
   }
-  else if (heightmap->GetFormat () & CS_IMGFMT_PALETTED8)
+  else if (map->GetFormat () & CS_IMGFMT_PALETTED8)
   {
     // It's a paletted image, so we grab data & palette
-    unsigned char *data = (unsigned char*)heightmap->GetImageData ();
-    csRGBpixel *palette = heightmap->GetPalette ();
+    unsigned char *data = (unsigned char*)map->GetImageData ();
+    const csRGBpixel *palette = map->GetPalette ();
 
     // Keep an index to avoid uneccesary x+y*w calculations
     // Start at last line, since we're gonna want it reversed in Y later
@@ -216,11 +234,165 @@ void csSimpleFormer::SetHeightmap (iImage *heightmap)
         // Grab the intensity as height
         // We're reversing Y to later get negative Y in heightmap image 
         // to positive Z in terrain
-        heightData[x+(height-y-1)*width] = 
-          palette[data[x+y*width]].Intensity () / 255.0;
+        intmap.data[x+(height-y-1)*width] = 
+          int (palette[data[x+y*width]].Intensity ()) * scale + offset;
       }
     }
   }
+
+  return true;
+}
+
+bool csSimpleFormer::SetFloatMap (csStringID type, iImage* map,
+	float scale, float offset)
+{
+  size_t w = map->GetWidth ();
+  size_t h = map->GetHeight ();
+  if (width != w || height != h) return false;
+
+  // First check if we already have an floatmap of this type.
+  size_t floatmap_idx = (size_t)~0;
+  size_t i = 0;
+  while (i < floatmaps.Length ())
+    if (floatmaps[i].type == type)
+    {
+      floatmap_idx = i;
+      break;
+    }
+    else i++;
+  if (floatmap_idx == (size_t)~0)
+    floatmap_idx = floatmaps.Push (csFloatMap ());
+
+  csFloatMap& floatmap = floatmaps[floatmap_idx];
+  floatmap.type = type;
+
+  // Allocate data
+  delete[] floatmap.data;
+  floatmap.data = new float[width*height];
+
+  // Check what type of image we got
+  if (map->GetFormat () & CS_IMGFMT_TRUECOLOR)
+  {
+    // It's a RGBA image
+    csRGBpixel *data = (csRGBpixel *)map->GetImageData ();
+
+    unsigned int x, y;
+    // Loop through the data
+    for (y = 0; y < height; ++y)
+    {
+      for (x = 0; x < width; ++x)
+      {
+        // Grab the intensity as height
+        // We're reversing Y to later get negative Y in heightmap image 
+        // to positive Z in terrain
+        floatmap.data[x+(height-y-1)*width] = 
+          data[x+y*width].Intensity () * scale /255.0 + offset;
+      }
+    }
+  }
+  else if (map->GetFormat () & CS_IMGFMT_PALETTED8)
+  {
+    // It's a paletted image, so we grab data & palette
+    unsigned char *data = (unsigned char*)map->GetImageData ();
+    const csRGBpixel *palette = map->GetPalette ();
+
+    // Keep an index to avoid uneccesary x+y*w calculations
+    // Start at last line, since we're gonna want it reversed in Y later
+    // (negative Y in heightmap image goes to positive Z in terrain)
+    //int idx = (height-1)*width;
+
+    unsigned int x, y;
+    // Loop through the data
+    for (y = 0; y < height; ++y)
+    {
+      for (x = 0; x < width; ++x)
+      {
+        // Grab the intensity as height
+        // We're reversing Y to later get negative Y in heightmap image 
+        // to positive Z in terrain
+        floatmap.data[x+(height-y-1)*width] = 
+          palette[data[x+y*width]].Intensity () * scale / 255.0 + offset;
+      }
+    }
+  }
+
+  return true;
+}
+
+void csSimpleFormer::SetHeightmap (float* data, unsigned int width,
+	unsigned int height)
+{
+  csSimpleFormer::width = width;
+  csSimpleFormer::height = height;
+
+  delete [] heightData;
+  heightData = new float[width*height];
+  memcpy (heightData, data, width*height*sizeof(float));
+}
+
+void csSimpleFormer::SetHeightmap (iImage *heightmap)
+{
+  // Grab dimensions
+  width = heightmap->GetWidth ();
+  height = heightmap->GetHeight ();
+
+  // Allocate height data
+  heightData = new float[width*height];
+
+  // Check what type of image we got
+  if ((heightmap->GetFormat () & CS_IMGFMT_MASK) == CS_IMGFMT_TRUECOLOR)
+  {
+    // It's a RGBA image
+    csRGBpixel *data = (csRGBpixel *)heightmap->GetImageData ();
+
+    unsigned int x, y;
+    // Loop through the data
+    for (y = 0; y < height; ++y)
+    {
+      // Keep an index to avoid uneccesary x+y*w calculations
+      uint index1 = (height-y-1)*width;
+      uint index2 = y*width;
+      for (x = 0; x < width; ++x)
+      {
+        // Grab height from R,G,B (triplet is effectively a 24-bit number)
+        // We're reversing Y to later get negative Y in heightmap image 
+        // to positive Z in terrain - sampler space has an inverted Y
+	// axis in comparison to image space.
+	const csRGBpixel& heixel = data[index2++];
+	int h = heixel.red * 65536 + heixel.green * 256 + heixel.blue;
+        heightData[index1++] = (float)h/16777215.0;
+      }
+    }
+  }
+  else if ((heightmap->GetFormat () & CS_IMGFMT_MASK) == CS_IMGFMT_PALETTED8)
+  {
+    // It's a paletted image, so we grab data & palette
+    unsigned char *data = (unsigned char*)heightmap->GetImageData ();
+
+    unsigned int x, y;
+    // Loop through the data
+    for (y = 0; y < height; ++y)
+    {
+      // Keep an index to avoid uneccesary x+y*w calculations
+      uint index1 = (height-y-1)*width;
+      uint index2 = y*width;
+      for (x = 0; x < width; ++x)
+      {
+        // Grab the index value as height
+        // We're reversing Y to later get negative Y in heightmap image 
+        // to positive Z in terrain - sampler space has an inverted Y
+	// axis in comparison to image space.
+        heightData[index1++] = (float)data[index2++] / 255.0;
+      }
+    }
+  }
+  else
+  {
+    // Well...
+    memset (heightData, 0, sizeof(float) * width * height);
+    csReport (objectRegistry, CS_REPORTER_SEVERITY_WARNING,
+      "crystalspace.terraformer.simple", "Odd image format");
+  }
 }
 
 void csSimpleFormer::SetScale (csVector3 scale)
@@ -274,11 +446,28 @@ bool csSimpleFormer::SampleFloat (csStringID type, float x, float z,
       scale.y + offset.y;
     return true;
   }
+  else
+  {
+    // Check if it is one of the float maps.
+    size_t i;
+    for (i = 0 ; i < floatmaps.Length () ; i++)
+      if (floatmaps[i].type == type)
+      {
+        // Transform input coordinates to heightmap space.
+        // See CachePositions for details
+        x = ((x-offset.x)/scale.x+1)*(width/2);
+        z = ((z-offset.z)/scale.z+1)*(height/2);
+
+        // Calculate height and return it
+        value = BiLinearData (floatmaps[i].data, width, height, x, z);
+        return true;
+      }
+  }
   return false;
 }
 
-bool csSimpleFormer::SampleVector2 (csStringID type, float x, float z, 
-                                    csVector2 &value)
+bool csSimpleFormer::SampleVector2 (csStringID /*type*/, float /*x*/,
+                                    float /*z*/, csVector2& /*value*/)
 {
   return false;
 }
@@ -298,6 +487,22 @@ bool csSimpleFormer::SampleVector3 (csStringID type, float x, float z,
 bool csSimpleFormer::SampleInteger (csStringID type, float x, float z, 
                                     int &value)
 {
+  {
+    // Check if it is one of the int maps.
+    size_t i;
+    for (i = 0 ; i < intmaps.Length () ; i++)
+      if (intmaps[i].type == type)
+      {
+        // Transform input coordinates to heightmap space.
+        // See CachePositions for details
+        x = ((x-offset.x)/scale.x+1)*(width/2);
+        z = ((z-offset.z)/scale.z+1)*(height/2);
+
+        // Calculate height and return it
+        value = intmaps[i].data[int (z) * width + int (x)];
+        return true;
+      }
+  }
   return false;
 }
 
@@ -306,7 +511,7 @@ bool csSimpleFormer::SampleInteger (csStringID type, float x, float z,
 //////////////////////////////////////////////////////////////////////////
 
 SCF_IMPLEMENT_IBASE (csSimpleSampler)
-SCF_IMPLEMENTS_INTERFACE (iTerraSampler)
+  SCF_IMPLEMENTS_INTERFACE (iTerraSampler)
 SCF_IMPLEMENT_IBASE_END
 
 csSimpleSampler::csSimpleSampler (csSimpleFormer *terraFormer,
@@ -530,19 +735,19 @@ void csSimpleSampler::CacheTexCoords ()
 
   // Iterate through the samplepoints in the region
   // Sample texture coordinates as x/z positions in heightmap space
-  const float startx = minCorner.x / (float)terraFormer->width;
+  const float startx = minCorner.x / (float)(terraFormer->width);
   csVector2 texCoord;
-  texCoord.y = minCorner.z / (float)terraFormer->height;
+  texCoord.y = minCorner.z / (float)(terraFormer->height);
   const csVector2 tcStep (
-    sampleDistanceHeight.x / (float)terraFormer->width, 
-    sampleDistanceHeight.z / (float)terraFormer->height);  
+    sampleDistanceHeight.x / (float)(terraFormer->width), 
+    sampleDistanceHeight.z / (float)(terraFormer->height));
   for (unsigned int i = 0; i<resolution; ++i)
   {
     texCoord.x = startx; 
     for (unsigned int j = 0; j<resolution; ++j)
     {
       // Just assign the texture coordinate
-      texCoords[idx++] = texCoord;
+      texCoords[idx++].Set (texCoord.x, 1.0f-texCoord.y); //= texCoord;
       texCoord.x += tcStep.x;
     }
     texCoord.y += tcStep.y;
