@@ -17,6 +17,8 @@
 */
 
 #include "cssysdef.h"
+#include "csplugincommon/directx/guids.h"
+
 #include "csutil/sysfunc.h"
 #include "csutil/scf.h"
 #include "csutil/csstring.h"
@@ -28,13 +30,13 @@
 #include "csutil/win32/win32.h"
 #include "iutil/cmdline.h"
 
-#define INITGUID
-#include "plugins/video/canvas/directxcommon/directdetection.h"
+#include "csplugincommon/directx/directdetection.h"
 
 #include "csutil/win32/wintools.h"
-#include "plugins/video/canvas/common/softfontcache.h"
+#include "csplugincommon/canvas/softfontcache.h"
 
 #include "g2d.h"
+#include "csutil/win32/psdk-compat.h"
 
 #ifndef DD_FALSE
   // This is normally being done in the ddraw.h file
@@ -44,23 +46,6 @@
 #define WINDOW_STYLE (WS_CAPTION | WS_MINIMIZEBOX | WS_POPUP | WS_SYSMENU)
 #define FULLSCREEN_STYLE (WS_POPUP | WS_SYSMENU)
 
-// These don't exist on some older SDKs
-#ifndef _WIN64
-  #ifndef SetWindowLongPtrA
-    #define SetWindowLongPtrA SetWindowLongA
-  #endif
-  #ifndef SetWindowLongPtrW
-    #define SetWindowLongPtrW SetWindowLongW
-  #endif
-  
-  #ifndef GetWindowLongPtrA
-    #define GetWindowLongPtrA GetWindowLongA
-  #endif
-  #ifndef GetWindowLongPtrW
-    #define GetWindowLongPtrW GetWindowLongW
-  #endif
-#endif
-
 //--//--//--//--//--//--//--//--//--//--//--//--//-- csGraphics2DDDraw3 --//--//
 
 CS_IMPLEMENT_PLUGIN
@@ -69,7 +54,7 @@ SCF_IMPLEMENT_FACTORY (csGraphics2DDDraw3)
 
 
 csGraphics2DDDraw3::csGraphics2DDDraw3(iBase *iParent) :
-  csGraphics2D (iParent),
+  scfImplementationType (this, iParent),
   m_lpDD (0),
   m_lpddsPrimary (0),
   m_lpddsBack (0),
@@ -96,14 +81,7 @@ void csGraphics2DDDraw3::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep = CS_QUERY_REGISTRY (object_reg, iReporter);
-  if (rep)
-    rep->ReportV (severity, "crystalspace.canvas.ddraw", msg, arg);
-  else
-  {
-    csPrintfV (msg, arg);
-    csPrintf ("\n");
-  }
+  csReportV (object_reg, severity, "crystalspace.canvas.ddraw", msg, arg);
   va_end (arg);
 }
 
@@ -152,39 +130,30 @@ bool csGraphics2DDDraw3::Open ()
   m_rcWindow.bottom = m_rcWindow.top + wheight;
 
   // create the window.
-  if (cswinIsWinNT ())
-  {
-    m_hWnd = CreateWindowW (CS_WIN32_WINDOW_CLASS_NAMEW, 0, 0,
-      m_rcWindow.left, m_rcWindow.top, m_rcWindow.right - m_rcWindow.left,
-      m_rcWindow.bottom - m_rcWindow.top, 0, 0, m_hInstance, 0);
-  }
-  else
-  {
-    m_hWnd = CreateWindowA (CS_WIN32_WINDOW_CLASS_NAME, 0, 0,
-      m_rcWindow.left, m_rcWindow.top, m_rcWindow.right - m_rcWindow.left,
-      m_rcWindow.bottom - m_rcWindow.top, 0, 0, m_hInstance, 0);
-  }
-  ASSERT (m_hWnd);
+  m_hWnd = m_piWin32Assistant->CreateCSWindow (this, 0, 0,
+    m_rcWindow.left, m_rcWindow.top, 
+    m_rcWindow.right - m_rcWindow.left, m_rcWindow.bottom - m_rcWindow.top);
+  CS_ASSERT (m_hWnd);
 
   SetTitle (win_title);
 
   // Subclass the window
   if (cswinIsWinNT ())
   {
-    m_OldWndProc = (WNDPROC)GetWindowLongPtrW (m_hWnd, GWL_WNDPROC);
-    SetWindowLongPtrW (m_hWnd, GWL_WNDPROC, (LONG_PTR)WindowProc);
-    SetWindowLongPtrW (m_hWnd, GWL_USERDATA, (LONG_PTR)this);
+    m_OldWndProc = (WNDPROC)GetWindowLongPtrW (m_hWnd, GWLP_WNDPROC);
+    SetWindowLongPtrW (m_hWnd, GWLP_WNDPROC, (LONG_PTR)WindowProc);
+    SetWindowLongPtrW (m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
   }
   else
   {
-    m_OldWndProc = (WNDPROC)GetWindowLongA (m_hWnd, GWL_WNDPROC);
-    SetWindowLongPtrA (m_hWnd, GWL_WNDPROC, (LONG_PTR)WindowProc);
-    SetWindowLongPtrA (m_hWnd, GWL_USERDATA, (LONG_PTR)this);
+    m_OldWndProc = (WNDPROC)GetWindowLongA (m_hWnd, GWLP_WNDPROC);
+    SetWindowLongPtrA (m_hWnd, GWLP_WNDPROC, (LONG_PTR)WindowProc);
+    SetWindowLongPtrA (m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
   }
 
   // Get ahold of the main DirectDraw object...
-  DDetection.checkDevices2D ();
-  DirectDevice = DDetection.findBestDevice2D (DisplayNumber);
+  DDetection.CheckDevices2D ();
+  DirectDevice = DDetection.FindBestDevice (DisplayNumber);
 
   if (DirectDevice == 0)
   {
@@ -192,7 +161,7 @@ bool csGraphics2DDDraw3::Open ()
     return false;
   }
 
-  LPGUID pGuid = 0;
+  const GUID* pGuid = 0;
   if (!DirectDevice->IsPrimary2D)
     pGuid = &DirectDevice->Guid2D;
 
@@ -201,7 +170,7 @@ bool csGraphics2DDDraw3::Open ()
 
   // Create a DD object for either the primary device or the secondary.
   HRESULT ddrval;
-  if ((ddrval = DirectDrawCreate (pGuid, &m_lpDD, 0)) != DD_OK)
+  if ((ddrval = DirectDrawCreate ((LPGUID)pGuid, &m_lpDD, 0)) != DD_OK)
   {
     InitFail (ddrval, "DirectDrawCreate FAILED (Code: %08lx)\n");
     return false;
@@ -511,16 +480,38 @@ void csGraphics2DDDraw3::SetRGB (int i, int r, int g, int b)
 
 bool csGraphics2DDDraw3::SetMouseCursor (csMouseCursorID iShape)
 {
-bool rc;
-  if (!m_bHardwareCursor) {
+  bool rc;
+  if (!m_bHardwareCursor)
+  {
     m_piWin32Assistant->SetCursor (csmcNone);
     rc = false;
-  } else {
+  }
+  else
+  {
     rc = m_piWin32Assistant->SetCursor (iShape);
   }
   return rc;
 }
 
+bool csGraphics2DDDraw3::SetMouseCursor (iImage *image, const csRGBcolor* keycolor, 
+					 int hotspot_x, int hotspot_y,
+					 csRGBcolor fg, csRGBcolor bg)
+{
+  if (!m_bHardwareCursor)
+  {
+    m_piWin32Assistant->SetCursor (csmcNone);
+    return false;
+  }
+  HCURSOR cur = cursors.GetMouseCursor (image, keycolor, hotspot_x, 
+    hotspot_y, fg, bg);
+  if (cur == 0)
+  {
+    m_piWin32Assistant->SetCursor (csmcNone);
+    return false;
+  }
+  return m_piWin32Assistant->SetHCursor (cur);
+}
+
 bool csGraphics2DDDraw3::SetMousePosition (int x, int y)
 {
   POINT p;
@@ -536,33 +527,30 @@ bool csGraphics2DDDraw3::SetMousePosition (int x, int y)
 
 bool csGraphics2DDDraw3::PerformExtensionV (char const* command, va_list args)
 {
-  bool rc = true;
   if (!strcmp (command, "fullscreen"))
   {
-    bool fs = bool(va_arg (args, int));
+    bool fs = (((int)va_arg (args, int)) != 0);
     if (fs != FullScreen
-     && (m_bAllowWindowed || fs))
+      && (m_bAllowWindowed || fs))
     {
       // Save window position
       if (!FullScreen)
         GetWindowRect (m_hWnd, &m_rcWindow);
       FullScreen = fs;
       if (FAILED(ChangeCoopLevel ()))
-	Report (CS_REPORTER_SEVERITY_WARNING,
+        Report (CS_REPORTER_SEVERITY_WARNING,
 	  "ChangeCoopLevel() failed!");
     } 
     else
     {
       if (!(m_bAllowWindowed || fs))
-      {
-	Report (CS_REPORTER_SEVERITY_NOTIFY,
+        Report (CS_REPORTER_SEVERITY_NOTIFY,
 	  "Windowed mode not available!");
-      }
     }
+    return true;
   }
   else
-    rc = false;
-  return rc;
+    return csGraphics2D::PerformExtensionV (command, args);
 }
 
 //-----------------------------------------------------------------------------
@@ -704,15 +692,13 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
   pfmt.RedMask = ddpf.dwRBitMask;
   pfmt.GreenMask = ddpf.dwGBitMask;
   pfmt.BlueMask = ddpf.dwBBitMask;
+  pfmt.AlphaMask = ddpf.dwRGBAlphaBitMask;
   Depth = ddpf.dwRGBBitCount;
 
-  if (fontCache) delete fontCache;
   if (Depth == 8)
   {
     pfmt.PalEntries = 256;
     pfmt.PixelBytes = 1;
-
-    fontCache = new csSoftFontCache8 (this);
   }
   else if (Depth == 16)
   {
@@ -722,15 +708,6 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     // Set pixel format
     pfmt.PixelBytes = 2;
     pfmt.PalEntries = 0;
-
-    if (ddpf.dwGBitMask == 0x03e0)
-    {
-      fontCache = new csSoftFontCache16_555 (this);
-    }
-    else
-    {
-      fontCache = new csSoftFontCache16_565 (this);
-    }
   }
   else if (Depth == 32)
   {
@@ -740,10 +717,11 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     // calculate CS's pixel format structure.
     pfmt.PixelBytes = 4;
     pfmt.PalEntries = 0;
-    
-    fontCache = new csSoftFontCache32 (this);
   }
   pfmt.complete ();
+
+  if (fontCache) delete fontCache; fontCache = 0;
+  CreateDefaultFontCache ();
   fontCache->SetClipRect (ClipX1, ClipY1, ClipX2, ClipY2);
 
   m_lpddsBack->GetSurfaceDesc (&ddsd);
@@ -910,7 +888,7 @@ bool csGraphics2DDDraw3::CreateIdentityPalette (csRGBpixel *p)
 LRESULT CALLBACK csGraphics2DDDraw3::WindowProc (HWND hWnd, UINT message,
   WPARAM wParam, LPARAM lParam)
 {
-  csGraphics2DDDraw3 *This = (csGraphics2DDDraw3 *)GetWindowLongPtrA (hWnd, GWL_USERDATA);
+  csGraphics2DDDraw3 *This = (csGraphics2DDDraw3 *)GetWindowLongPtrA (hWnd, GWLP_USERDATA);
   switch (message)
   {
     case WM_PAINT:
