@@ -6,7 +6,7 @@
 //  Copyright (c) 2001 Matt Reda. All rights reserved.
 //
 
-#import "plugins/video/canvas/macosx/common/OSXDelegate2D.h"
+#import "csplugincommon/macosx/OSXDelegate2D.h"
 
 
 // Create a small category for NSOpenGLContext to give access to the CGL context
@@ -58,12 +58,8 @@ NSOpenGLContext *context;
     NSOpenGLPixelFormatAttribute attribs[] = {
         NSOpenGLPFAWindow, NSOpenGLPFADoubleBuffer, NSOpenGLPFAAccelerated,
         NSOpenGLPFAColorSize, depth, NSOpenGLPFADepthSize, 32,
-#ifndef CS_USE_NEW_RENDERER
-        NSOpenGLPFAStencilSize, 1,
-#else
         NSOpenGLPFAStencilSize, 8,
         NSOpenGLPFAAlphaSize, 8,
-#endif
         NSOpenGLPFAScreenMask, CGDisplayIDToOpenGLDisplayMask(display), nil
     };
 
@@ -125,7 +121,7 @@ NSOpenGLContext *context;
 
 
 // C API to OSXDelegate2D class (OpenGL category)
-#define DEL2D_FUNC(ret, func) __private_extern__ inline ret OSXDelegate2D_##func
+#define DEL2D_FUNC(ret, func) __private_extern__ ret OSXDelegate2D_##func
 
 typedef void *OSXDelegate2DHandle;
 
