@@ -35,25 +35,15 @@
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE_EXT (csGraphics2DGLX)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iOpenGLInterface)
-SCF_IMPLEMENT_IBASE_EXT_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csGraphics2DGLX::eiOpenGLInterface)
-  SCF_IMPLEMENTS_INTERFACE (iOpenGLInterface)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csGraphics2DGLX)
 
-
 #define DEF_OGLDISP "crystalspace.graphics2d.glx.disp.empty"
 #define XWIN_SCF_ID "crystalspace.window.x"
 
 // csGraphics2DGLX function
 csGraphics2DGLX::csGraphics2DGLX (iBase *iParent) :
-  csGraphics2DGLCommon (iParent), cmap (0), hardwareaccelerated(false)
+  scfImplementationType (this, iParent), cmap (0), hardwareaccelerated(false)
 {
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiOpenGLInterface);
 }
 
 void csGraphics2DGLX::Report (int severity, const char* msg, ...)
@@ -81,8 +71,6 @@ bool csGraphics2DGLX::Initialize (iObjectRegistry *object_reg)
   if (!csGraphics2DGLCommon::Initialize (object_reg))
     return false;
 
-  csConfigAccess config(object_reg, "/config/opengl.cfg");
-
   csRef<iPluginManager> plugin_mgr (
   	CS_QUERY_REGISTRY (object_reg, iPluginManager));
   if ((strDriver = config->GetStr ("Video.OpenGL.Display.Driver", 0)))
@@ -106,15 +94,19 @@ bool csGraphics2DGLX::Initialize (iObjectRegistry *object_reg)
   if (!xwin)
   {
     Report (CS_REPORTER_SEVERITY_WARNING,
-      "Could not create an instance of %s !", XWIN_SCF_ID);
+      "Could not create an instance of " XWIN_SCF_ID "!");
     return false;
   }
 
   dpy = xwin->GetDisplay ();
   screen_num = xwin->GetScreen ();
 
-  // The texture manager only needs to know this:
-  pfmt.PalEntries = 0;
+  pfmt.RedMask = 0;
+  pfmt.GreenMask = 0;
+  pfmt.BlueMask = 0;
+  pfmt.AlphaMask = 0;
+  pfmt.PalEntries = 0; // Texture mananger needs to know this.
+  pfmt.PixelBytes = 0;
 
   // Create the event outlet
   csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
@@ -129,25 +121,26 @@ csGraphics2DGLX::~csGraphics2DGLX ()
   // Destroy your graphic interface
   XFree ((void*)xvis);
   Close ();
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiOpenGLInterface);
 }
 
 bool csGraphics2DGLX::Open()
 {
   if (is_open) return true;
 
-  Report (CS_REPORTER_SEVERITY_NOTIFY, "Opening GLX2D\n");
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "Opening GLX2D");
   // We now select the visual here as with a mesa bug it is not possible
   // to destroy double buffered contexts and then create a single buffered
   // one.
 
-  if (!CreateVisuals ())
+  if (!ChooseVisual ())
     return false;
 
+  cmap = XCreateColormap (dpy, RootWindow (dpy, xvis->screen),
+    xvis->visual, AllocNone);
+
   xwin->SetColormap (cmap);
   xwin->SetVisualInfo (xvis);
   xwin->SetCanvas ((iGraphics2D *)this);
-
   if (!xwin->Open ())
   {
     Report (CS_REPORTER_SEVERITY_ERROR,
@@ -155,8 +148,9 @@ bool csGraphics2DGLX::Open()
     return false;
   }
   window = xwin->GetWindow ();
+  active_GLContext = glXCreateContext(dpy, xvis, 0, True);
 
-  // this makes the context we created in Initialize() the current
+  // this makes the context we just created the current
   // context, so that all subsequent OpenGL calls will set state and
   // draw stuff on this context.  You could of couse make
   // some more contexts and switch around between several of them...
@@ -165,10 +159,14 @@ bool csGraphics2DGLX::Open()
   
   XSync (dpy, False);
   
+  GetCurrentAttributes();
+  
   // Open your graphic interface
   if (!csGraphics2DGLCommon::Open ())
     return false;
 
+  xwin->SetTitle (win_title);
+
   return true;
 }
 
@@ -191,6 +189,21 @@ void csGraphics2DGLX::Close(void)
     xwin->Close ();
 }
 
+const char* csGraphics2DGLX::GetVersionString (const char* ver)
+{
+  if (strcmp (ver, "mesa") == 0)
+  {
+    static const char needle[] = "Mesa ";
+    const char* glVersion = (const char*)glGetString (GL_VERSION);
+    const char* p = strstr (glVersion, needle);
+    if (p != 0)
+      return p + sizeof (needle) - 1;
+    return 0;
+  }
+  else
+    return csGraphics2DGLCommon::GetVersionString (ver);
+}
+
 static const char *visual_class_name (int cls)
 {
   switch (cls)
@@ -212,28 +225,55 @@ static const char *visual_class_name (int cls)
   }
 }
 
-bool csGraphics2DGLX::CreateVisuals ()
+bool csGraphics2DGLX::ChooseVisual ()
 {
-  int desired_attributes[] =
+  bool do_verbose = false;
+  csRef<iVerbosityManager> verbosemgr (
+    CS_QUERY_REGISTRY (object_reg, iVerbosityManager));
+  if (verbosemgr) 
+    do_verbose = verbosemgr->Enabled ("renderer.x.visual");
+    
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "Creating Context");
+  
+  GLPixelFormat format;
+  csGLPixelFormatPicker picker (this);
+  
+  while (picker.GetNextFormat (format))
   {
-    GLX_RGBA,
-    GLX_DEPTH_SIZE, 8,
-    GLX_RED_SIZE, 4,
-    GLX_BLUE_SIZE, 4,
-    GLX_GREEN_SIZE, 4,
-    GLX_DOUBLEBUFFER,
-#ifdef CS_USE_NEW_RENDERER
-    GLX_ALPHA_SIZE, 8,
-    GLX_STENCIL_SIZE, 8,
-#else
-    GLX_STENCIL_SIZE, 1,
-#endif
-    None
-  };
-  Report (CS_REPORTER_SEVERITY_NOTIFY, "Creating Context\n");
-  // find a visual that supports all the features we need
-  xvis = glXChooseVisual (dpy, screen_num, desired_attributes);
-  hardwareaccelerated = true;
+    if (do_verbose)
+    {
+      csString pfStr;
+      GetPixelFormatString (format, pfStr);
+  
+      Report (CS_REPORTER_SEVERITY_NOTIFY,
+	"Probing pixel format: %s", pfStr.GetData());
+    }
+    const int colorBits = format[glpfvColorBits];
+    const int colorComponentSize = 
+	((colorBits % 32) == 0) ? colorBits / 4 : colorBits / 3;
+    const int accumBits = format[glpfvAccumColorBits];
+    const int accumComponentSize = 
+	((accumBits % 32) == 0) ? accumBits / 4 : accumBits / 3;
+    int desired_attributes[] =
+    {
+      GLX_RGBA,
+      GLX_DEPTH_SIZE, format[glpfvDepthBits],
+      GLX_RED_SIZE, colorComponentSize,
+      GLX_BLUE_SIZE, colorComponentSize,
+      GLX_GREEN_SIZE, colorComponentSize,
+      GLX_DOUBLEBUFFER,
+      GLX_ALPHA_SIZE, format[glpfvAlphaBits],
+      GLX_STENCIL_SIZE, format[glpfvStencilBits],
+      GLX_ACCUM_RED_SIZE, accumComponentSize,
+      GLX_ACCUM_BLUE_SIZE, accumComponentSize,
+      GLX_ACCUM_GREEN_SIZE, accumComponentSize,
+      GLX_ACCUM_ALPHA_SIZE, format[glpfvAccumAlphaBits],
+      None
+    };
+    // find a visual that supports all the features we need
+    xvis = glXChooseVisual (dpy, screen_num, desired_attributes);
+    if (xvis) break;
+  }
 
   // if a visual was found that we can use, make a graphics context which
   // will be bound to the application window.  If a visual was not
@@ -281,13 +321,21 @@ bool csGraphics2DGLX::CreateVisuals ()
       }
     }
   }
+  return true;
+}
 
-  active_GLContext = glXCreateContext(dpy, xvis, 0, GL_TRUE);
-  cmap = XCreateColormap (dpy, RootWindow (dpy, xvis->screen),
-    xvis->visual, AllocNone);
-
+void csGraphics2DGLX::GetCurrentAttributes ()
+{
+  hardwareaccelerated = glXIsDirect (dpy, active_GLContext);
   Report (CS_REPORTER_SEVERITY_NOTIFY, "Video driver GL/X version %s",
-    glXIsDirect (dpy, active_GLContext) ? "(direct renderer)" : "");
+    hardwareaccelerated ? "(direct renderer)" : 
+    "(indirect renderer)");
+  if (!hardwareaccelerated)
+  {
+    Report (CS_REPORTER_SEVERITY_WARNING,
+      "Indirect rendering may indicate a flawed OpenGL setup if you run on "
+      "a local X server.");
+  }
 
   Depth = xvis->depth;
 
@@ -296,7 +344,7 @@ bool csGraphics2DGLX::CreateVisuals ()
   else
     pfmt.PixelBytes = 2;
 
-  Report (CS_REPORTER_SEVERITY_NOTIFY, "Visual ID: %x, %dbit %sn",
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "Visual ID: %p, %dbit %s",
     xvis->visualid, Depth, visual_class_name (xvis->c_class));
 
   int ctype, frame_buffer_depth, size_depth_buffer, level;
@@ -306,6 +354,7 @@ bool csGraphics2DGLX::CreateVisuals ()
   glXGetConfig(dpy, xvis, GLX_DEPTH_SIZE, &size_depth_buffer);
   glXGetConfig(dpy, xvis, GLX_LEVEL, &level);
 
+  int color_bits = 0;
   int alpha_bits = 0;
   if (ctype)
   {
@@ -313,20 +362,50 @@ bool csGraphics2DGLX::CreateVisuals ()
     pfmt.GreenMask = xvis->green_mask;
     pfmt.BlueMask = xvis->blue_mask;
     glXGetConfig(dpy, xvis, GLX_RED_SIZE, &pfmt.RedBits);
+    color_bits += pfmt.RedBits;
     glXGetConfig(dpy, xvis, GLX_GREEN_SIZE, &pfmt.GreenBits);
+    color_bits += pfmt.GreenBits;
     glXGetConfig(dpy, xvis, GLX_BLUE_SIZE, &pfmt.BlueBits);
+    color_bits += pfmt.BlueBits;
     glXGetConfig(dpy, xvis, GLX_ALPHA_SIZE, &alpha_bits);
+    pfmt.AlphaBits = alpha_bits;
       
     int bit;
+    // Fun hack, xvis doesn't provide alpha mask
+    bit=0; while (bit < alpha_bits) pfmt.AlphaMask |= (1<<(bit++));
+    pfmt.AlphaMask = pfmt.AlphaMask << color_bits;
+
     bit=0; while (!(pfmt.RedMask & (1<<bit))) bit++; pfmt.RedShift = bit;
     bit=0; while (!(pfmt.GreenMask & (1<<bit))) bit++; pfmt.GreenShift = bit;
     bit=0; while (!(pfmt.BlueMask & (1<<bit))) bit++; pfmt.BlueShift = bit;
+    if (pfmt.AlphaMask)
+    {
+      bit=0; while (!(pfmt.AlphaMask & (1<<bit))) bit++; pfmt.AlphaShift = bit;
+    }
   }
 
   // Report Info
-  Report (CS_REPORTER_SEVERITY_NOTIFY,
-    "Frame buffer: %dbit ", frame_buffer_depth);
-    
+  currentFormat[glpfvColorBits] = color_bits;
+  currentFormat[glpfvAlphaBits] = alpha_bits;
+  currentFormat[glpfvDepthBits] = size_depth_buffer;
+  int stencilSize = 0;
+  glXGetConfig(dpy, xvis, GLX_STENCIL_SIZE, &stencilSize);
+  currentFormat[glpfvStencilBits] = stencilSize;
+  int accumBits = 0;
+  int accumAlpha = 0;
+  {
+    int dummy;
+    glXGetConfig(dpy, xvis, GLX_ACCUM_RED_SIZE, &dummy);
+    accumBits += dummy;
+    glXGetConfig(dpy, xvis, GLX_ACCUM_GREEN_SIZE, &dummy);
+    accumBits += dummy;
+    glXGetConfig(dpy, xvis, GLX_ACCUM_BLUE_SIZE, &dummy);
+    accumBits += dummy;
+    glXGetConfig(dpy, xvis, GLX_ACCUM_ALPHA_SIZE, &accumAlpha);
+  }
+  currentFormat[glpfvAccumColorBits] = accumBits;
+  currentFormat[glpfvAccumAlphaBits] = accumAlpha;
+
   if (ctype)
   {
     if (pfmt.RedMask > pfmt.BlueMask)
@@ -342,10 +421,8 @@ bool csGraphics2DGLX::CreateVisuals ()
   }
     
   Report (CS_REPORTER_SEVERITY_NOTIFY, "level %d, double buffered", level);
-  Report (CS_REPORTER_SEVERITY_NOTIFY, "Depth buffer: %dbit", size_depth_buffer);
 
   pfmt.complete ();
-  return true;
 }
 
 bool csGraphics2DGLX::PerformExtensionV (char const* command, va_list args)
