@@ -19,10 +19,11 @@
 
 #include <stdarg.h>
 #include "cssysdef.h"
-#include "cssys/sysfunc.h"
+#include "csutil/sysfunc.h"
 #include "x2d.h"
 #include "csgeom/csrect.h"
 #include "csutil/cfgacc.h"
+#include "csutil/event.h"
 #include "iutil/plugin.h"
 #include "iutil/cfgmgr.h"
 #include "iutil/cmdline.h"
@@ -34,61 +35,35 @@ CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_FACTORY (csGraphics2DXLib)
 
-SCF_EXPORT_CLASS_TABLE (x2d)
-  SCF_EXPORT_CLASS_DEP (csGraphics2DXLib, "crystalspace.graphics2d.x2d",
-    "X-Windows 2D graphics driver for Crystal Space", "crystalspace.font.server.")
-SCF_EXPORT_CLASS_TABLE_END
-
-SCF_IMPLEMENT_IBASE_EXT (csGraphics2DXLib)
-  SCF_IMPLEMENTS_INTERFACE (iEventPlug)
-SCF_IMPLEMENT_IBASE_EXT_END
-
-
-
 #define CS_XWIN_SCF_ID "crystalspace.window.x"
 #define CS_XEXT_SHM_SCF_ID "crystalspace.window.x.extshm"
 #define CS_XEXT_SHM "MIT-SHM"
 
-
 csGraphics2DXLib::csGraphics2DXLib (iBase *iParent) :
-  csGraphics2D (iParent), xwin (NULL), xshm (NULL),  xim (NULL),
-  dpy (NULL), cmap (0), real_Memory (NULL),
-  sim_lt8 (NULL), sim_lt16 (NULL)
+  scfImplementationType (this, iParent), xim (0),
+  dpy (0), cmap (0), real_Memory (0),
+  sim_lt8 (0), sim_lt16 (0)
 {
-  xwin = NULL;
-  xshm = NULL;
-  EventOutlet = NULL;
+  EventOutlet = 0;
 }
 
 csGraphics2DXLib::~csGraphics2DXLib(void)
 {
-  if (xshm)
-  {
-    xshm->DecRef (); // decref before we close the win since we use the dpy in xshm
-    xshm = NULL;
-  }
+  xshm = 0;
 
   Close();
 
   delete [] sim_lt8;
   delete [] sim_lt16;
-
-  if (EventOutlet)
-      EventOutlet->DecRef();
-  if (xwin)
-    xwin->DecRef ();
 }
 
 void csGraphics2DXLib::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  iReporter* rep = CS_QUERY_REGISTRY (object_reg, iReporter);
+  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
   if (rep)
-  {
     rep->ReportV (severity, "crystalspace.canvas.softx", msg, arg);
-    rep->DecRef ();
-  }
   else
   {
     csPrintfV (msg, arg);
@@ -99,15 +74,19 @@ void csGraphics2DXLib::Report (int severity, const char* msg, ...)
 
 bool csGraphics2DXLib::Initialize (iObjectRegistry *object_reg)
 {
+  CanvasResize = csevCanvasResize (object_reg, this);
+
   if (!csGraphics2D::Initialize (object_reg))
     return false;
 
-  iPluginManager* plugin_mgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
+  csRef<iPluginManager> plugin_mgr (
+  	CS_QUERY_REGISTRY (object_reg, iPluginManager));
 
   xwin = CS_LOAD_PLUGIN (plugin_mgr, CS_XWIN_SCF_ID, iXWindow);
   if (!xwin)
   {
-    plugin_mgr->DecRef ();
+    Report (CS_REPORTER_SEVERITY_ERROR,
+	      "Couldn't find 'xwin' plugin!");
     return false;
   }
   dpy = xwin->GetDisplay ();
@@ -116,14 +95,13 @@ bool csGraphics2DXLib::Initialize (iObjectRegistry *object_reg)
   bool do_shm;
   // Query system settings
   csConfigAccess Config(object_reg, "/config/video.cfg");
-  iCommandLineParser* cmdline = CS_QUERY_REGISTRY (object_reg,
-						   iCommandLineParser);
+  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
+						   iCommandLineParser));
   sim_depth = Config->GetInt ("Video.SimulateDepth", 0);
 
   do_shm = Config->GetBool ("Video.XSHM", true);
   if (cmdline->GetOption ("XSHM")) do_shm = true;
   if (cmdline->GetOption ("noXSHM")) do_shm = false;
-  cmdline->DecRef ();
 
   if (do_shm)
   {
@@ -138,20 +116,18 @@ bool csGraphics2DXLib::Initialize (iObjectRegistry *object_reg)
     else
     {
       Report (CS_REPORTER_SEVERITY_WARNING,
-	      "No shared memory X-extension detected....disabling\n");
+	      "No shared memory X-extension detected....disabling");
     }
   }
 
-  iEventQueue* q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
+  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
   if (q != 0)
   {
     // Tell event queue to call us on broadcast messages
-    q->RegisterListener (scfiEventHandler, CSMASK_Broadcast);
+    q->RegisterListener (scfiEventHandler, csevCommandLineHelp(object_reg));
     // Create the event outlet
     EventOutlet = q->CreateEventOutlet (this);
-    q->DecRef ();
   }
-  plugin_mgr->DecRef ();
   return true;
 }
 
@@ -179,7 +155,7 @@ bool csGraphics2DXLib::Open()
   if (xshm)
     Report (CS_REPORTER_SEVERITY_NOTIFY, "(Using SHM extension plugin)");
 
-  Report (CS_REPORTER_SEVERITY_NOTIFY, "Using %d bit %sColor visual",
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "Using %u bit %sColor visual",
               xvis.depth, (xvis.visual->c_class == PseudoColor) ? "Pseudo" : "True");
 
   // Open your graphic interface
@@ -189,6 +165,8 @@ bool csGraphics2DXLib::Open()
   if (!AllocateMemory ())
     return false;
 
+  xwin->SetTitle (win_title);
+
   Clear (0);
   return true;
 }
@@ -206,7 +184,7 @@ void csGraphics2DXLib::Close ()
   if (Memory && sim_depth && !xshm)
   {
     delete [] Memory;
-    Memory = NULL;
+    Memory = 0;
   }
 
   csGraphics2D::Close ();
@@ -236,6 +214,7 @@ bool csGraphics2DXLib::CreateVisuals ()
   pfmt.RedMask = xvis.red_mask;
   pfmt.GreenMask = xvis.green_mask;
   pfmt.BlueMask = xvis.blue_mask;
+  pfmt.AlphaMask = 0;
 
   pfmt.complete ();
   pfmt.PalEntries = xvis.colormap_size;
@@ -263,7 +242,7 @@ bool csGraphics2DXLib::CreateVisuals ()
     switch (sim_depth)
     {
       case 8:
-        pfmt.RedMask = pfmt.GreenMask = pfmt.BlueMask = 0;
+        pfmt.RedMask = pfmt.GreenMask = pfmt.BlueMask = pfmt.AlphaMask = 0;
       	pfmt.PalEntries = 256;
       	pfmt.PixelBytes = 1;
       	break;
@@ -271,6 +250,7 @@ bool csGraphics2DXLib::CreateVisuals ()
       	pfmt.RedMask   = 0x1f << 10;
       	pfmt.GreenMask = 0x1f << 5;
       	pfmt.BlueMask  = 0x1f;
+        pfmt.AlphaMask = 0;
       	pfmt.PalEntries = 0;
       	pfmt.PixelBytes = 2;
       	break;
@@ -278,6 +258,7 @@ bool csGraphics2DXLib::CreateVisuals ()
       	pfmt.RedMask   = 0x1f << 11;
       	pfmt.GreenMask = 0x3f << 5;
       	pfmt.BlueMask  = 0x1f;
+        pfmt.AlphaMask = 0;
       	pfmt.PalEntries = 0;
       	pfmt.PixelBytes = 2;
       	break;
@@ -285,6 +266,7 @@ bool csGraphics2DXLib::CreateVisuals ()
       	pfmt.RedMask = 0xff << 16;
       	pfmt.GreenMask = 0xff << 8;
       	pfmt.BlueMask = 0xff;
+        pfmt.AlphaMask = 0xff << 24;
       	pfmt.PalEntries = 0;
       	pfmt.PixelBytes = 4;
       	break;
@@ -354,13 +336,11 @@ bool csGraphics2DXLib::CreateVisuals ()
   if (pfmt.PixelBytes == 2)
   {
     _DrawPixel = DrawPixel16;
-    _WriteString = WriteString16;
     _GetPixelAt = GetPixelAt16;
   }
   else if (pfmt.PixelBytes == 4)
   {
     _DrawPixel = DrawPixel32;
-    _WriteString = WriteString32;
     _GetPixelAt = GetPixelAt32;
   } /* endif */
 
@@ -683,13 +663,13 @@ void csGraphics2DXLib::SetRGB(int i, int r, int g, int b)
     if (sim_lt16)
     {
       delete [] sim_lt16;
-      sim_lt16 = NULL;
+      sim_lt16 = 0;
     }
 
   csGraphics2D::SetRGB (i, r, g, b);
 }
 
-void csGraphics2DXLib::Print (csRect *area)
+void csGraphics2DXLib::Print (csRect const* area)
 {
   // If we won't tell X-Windows to flush, the applications starts to response
   // slowly in some circumstances (on some machines, OSes, configurations etc)
@@ -864,8 +844,7 @@ bool csGraphics2DXLib::AllocateMemory ()
   {
     Report (CS_REPORTER_SEVERITY_NOTIFY,
 	    "SHM available but could not allocate. Trying without SHM.");
-    xshm->DecRef ();
-    xshm = NULL;
+    xshm = 0;
     mem_valid = TryAllocateMemory();
   }
 
@@ -887,7 +866,7 @@ bool csGraphics2DXLib::TryAllocateMemory ()
     int bitmap_pad = (disp_depth + 7) / 8;
     bitmap_pad = (bitmap_pad == 3) ? 32 : bitmap_pad*8;
     xim = XCreateImage(dpy, DefaultVisual(dpy,screen_num),
-		       disp_depth, ZPixmap, 0, NULL,
+		       disp_depth, ZPixmap, 0, 0,
 		       Width, Height, bitmap_pad, 0);
     xim->data = new char[xim->bytes_per_line*xim->height];
     real_Memory = (unsigned char*)(xim->data);
@@ -921,13 +900,13 @@ bool csGraphics2DXLib::Resize (int width, int height)
   {
     delete [] real_Memory;
     XDestroyImage (xim);
-    xim = NULL;
+    xim = 0;
   }
   if (!AllocateMemory())
     return false;
 //    Clear (0);
 //    XSync (dpy, false);
-  EventOutlet->Broadcast (cscmdContextResize, (iGraphics2D *)this);
+  EventOutlet->Broadcast (CanvasResize, (intptr_t)this);
   return true;
 }
 
@@ -946,20 +925,19 @@ void csGraphics2DXLib::AllowResize (bool iAllow)
 
 bool csGraphics2DXLib::HandleEvent (iEvent &Event)
 {
-  if ((Event.Type == csevBroadcast)
-   && (Event.Command.Code == cscmdCommandLineHelp)
-   && object_reg)
+  if ((Event.Name == csevCommandLineHelp(object_reg))
+      && object_reg)
   {
-    printf ("Options for X-Windows 2D graphics driver:\n");
-    printf ("  -sdepth=<depth>    set simulated depth (8, 15, 16, or 32) (default=none)\n");
-    printf ("  -shm/noshm         SHM extension (default '%sshm')\n",
+    csPrintf ("Options for X-Windows 2D graphics driver:\n");
+    csPrintf ("  -sdepth=<depth>    set simulated depth (8, 15, 16, or 32) (default=none)\n");
+    csPrintf ("  -XSHM/noXSHM       SHM extension (default '%sXSHM')\n",
       xshm ? "" : "no");
     return true;
   }
   return false;
 }
 
-bool csGraphics2DXLib::PerformExtensionV (char const* command, va_list)
+bool csGraphics2DXLib::PerformExtensionV (char const* command, va_list args)
 {
   if (!strcasecmp (command, "sim_pal"))
   {
@@ -986,7 +964,7 @@ bool csGraphics2DXLib::PerformExtensionV (char const* command, va_list)
     XSync (dpy, False);
     return true;
   }
-  return false;
+  return csGraphics2D::PerformExtensionV (command, args);
 }
 
 #undef CS_XWIN_SCF_ID
