@@ -19,7 +19,7 @@
 #ifndef __CS_X2D_H__
 #define __CS_X2D_H__
 
-#include "plugins/video/canvas/common/graph2d.h"
+#include "csplugincommon/canvas/graph2d.h"
 #include "iutil/event.h"
 #include "plugins/video/canvas/xwindowcommon/xwindow.h"
 #include "plugins/video/canvas/xwindowcommon/xextshm.h"
@@ -30,7 +30,9 @@
 #include <X11/Xatom.h>
 
 /// XLIB version.
-class csGraphics2DXLib : public csGraphics2D, public iEventPlug
+class csGraphics2DXLib : public scfImplementationExt1<csGraphics2DXLib, 
+							csGraphics2D, 
+							iEventPlug>
 {
   csRef<iXWindow> xwin;
   /// Shared memory extension (manages the shared memory backbuffer)
@@ -52,18 +54,21 @@ class csGraphics2DXLib : public csGraphics2D, public iEventPlug
   int sim_depth;
   csPixelFormat real_pfmt;	// Contains the real pfmt is simulating stuff
   unsigned char* real_Memory;	// Real memory to the display
-  unsigned char* sim_lt8;	// 8-bit lookup table (with 16-bit index) for simulated depth
-  uint16* sim_lt16;		// 16-bit lookup table (with 8-bit index) for simulated depth
+  unsigned char* sim_lt8;	// 8-bit lookup table (with 16-bit index)
+  				// for simulated depth
+  uint16* sim_lt16;		// 16-bit lookup table (with 8-bit index)
+  				// for simulated depth
 
   bool CreateVisuals ();
   bool AllocateMemory ();
   bool TryAllocateMemory ();
 
 public:
-  SCF_DECLARE_IBASE_EXT(csGraphics2D);
   csGraphics2DXLib (iBase*);
   virtual ~csGraphics2DXLib ();
 
+  csEventID CanvasResize;
+
   virtual bool Initialize (iObjectRegistry*);
   virtual bool Open ();
   virtual void Close ();
@@ -99,6 +104,13 @@ public:
 
   virtual bool Resize (int width, int height);
 
+  virtual void AlertV (int type, const char* title, const char* okMsg,
+  	const char* msg, va_list arg) 
+  {
+    if (!xwin->AlertV (type, title, okMsg, msg, arg))
+      csGraphics2D::AlertV (type, title, okMsg, msg, arg);
+  }
+
   virtual void SetTitle (const char* title)
   { xwin->SetTitle (title); }
 
@@ -119,8 +131,8 @@ public:
 
   /// Set mouse cursor shape
   // should be the window manager
-  virtual bool SetMouseCursor (iImage *image, csRGBcolor keycolor, int x, int y,
-                                               csRGBcolor fg, csRGBcolor bg)
+  virtual bool SetMouseCursor (iImage *image, 
+    const csRGBcolor* keycolor, int x, int y, csRGBcolor fg, csRGBcolor bg)
   { return xwin->SetMouseCursor (image, keycolor, x, y, fg, bg); }
 
   //------------------------ iEventPlug interface ---------------------------//
