@@ -20,9 +20,11 @@
 #define XK_XKB_KEYS
 #include <stdarg.h>
 #include "cssysdef.h"
-#include "cssys/sysfunc.h"
+#include "csutil/sysfunc.h"
+#include "csutil/event.h"
 #include "xwindow.h"
 #include "csgeom/csrect.h"
+#include "csutil/callstack.h"
 #include "csutil/cfgacc.h"
 #include "csutil/scf.h"
 #include "iutil/plugin.h"
@@ -32,73 +34,48 @@
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
 #include "ivaria/reporter.h"
-#include "video/canvas/common/scancode.h"
+#include "csplugincommon/canvas/scancode.h"
+#include "igraphic/image.h"
+#include "csplugincommon/canvas/cursorconvert.h"
+
+// Define this if you want keyboard-grabbing behavior enabled.  For now it is
+// disabled by default.  In the future, we should probably provide an API for
+// setting this at run-time (though this API, when properly generalized to be
+// platform-neutral, does not belong in iGraphics2D; but rather in some input-
+// or keyboard-related interface).
+#undef CS_XWIN_GRAB_KEYBOARD
 
 CS_IMPLEMENT_PLUGIN
 
 SCF_IMPLEMENT_FACTORY (csXWindow)
 
-SCF_EXPORT_CLASS_TABLE (xwin)
-  SCF_EXPORT_CLASS (csXWindow, "crystalspace.window.x",
-    "X-Window plugin for Crystal Space")
-SCF_EXPORT_CLASS_TABLE_END
-
-SCF_IMPLEMENT_IBASE(csXWindow)
-  SCF_IMPLEMENTS_INTERFACE(iXWindow)
-  SCF_IMPLEMENTS_INTERFACE (iEventPlug)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csXWindow::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csXWindow::EventHandler)
-  SCF_IMPLEMENTS_INTERFACE (iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
 #define CS_XEXT_XF86VM_SCF_ID "crystalspace.window.x.extf86vm"
 #define CS_XEXT_XF86VM "XFree86-VidModeExtension"
 
-csXWindow::csXWindow (iBase* parent)
+csXWindow::csXWindow (iBase* parent) : scfImplementationType (this, parent)
 {
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
-
-  scfiEventHandler = NULL;
-
   EmptyMouseCursor = 0;
   memset (&MouseCursor, 0, sizeof (MouseCursor));
   wm_win = ctx_win = 0;
-  win_title = NULL;
-  EventOutlet = NULL;
+  EventOutlet = 0;
 
   wm_width = wm_height = 0;
 
-  xvis = NULL;
+  dpy = 0;
+  xvis = 0;
   gc = 0;
   screen_num = 0;
-  Canvas = NULL;
+  Canvas = 0;
 
   keyboard_grabbed = 0;
-  xf86vm = NULL;
+  oldErrorHandler = 0;
 }
 
 void csXWindow::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  iReporter* rep = CS_QUERY_REGISTRY (object_reg, iReporter);
-  if (rep)
-  {
-    rep->ReportV (severity, "crystalspace.window.x", msg, arg);
-    rep->DecRef ();
-  }
-  else
-  {
-    csPrintfV (msg, arg);
-    csPrintf ("\n");
-  }
+  csReportV (object_reg, severity, "crystalspace.window.x", msg, arg);
   va_end (arg);
 }
 
@@ -106,39 +83,72 @@ csXWindow::~csXWindow ()
 {
   if (scfiEventHandler)
   {
-    iEventQueue* q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
+    csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
     if (q != 0)
-    {
       q->RemoveListener (scfiEventHandler);
-      q->DecRef ();
-    }
-    scfiEventHandler->DecRef ();
   }
-  if (xf86vm)
-    xf86vm->DecRef ();
-  if (EventOutlet)
-    EventOutlet->DecRef ();
-  delete [] win_title;
+
+  cachedCursors.DeleteAll ();
+  
+  if (oldErrorHandler != 0)
+  {
+    XSetErrorHandler (oldErrorHandler);
+  }
+}
+
+static int MyErrorHandler (Display* dpy, XErrorEvent* ev)
+{
+  char errorText[256];
+  XGetErrorText (dpy, ev->error_code, errorText, sizeof (errorText));
+  csFPrintf (stderr, "X error!\n");
+  csFPrintf (stderr, " type:         %d\n", ev->type);
+  csFPrintf (stderr, " serial:       %ld\n", ev->serial);
+  csFPrintf (stderr, " error code:   %d (%s)\n", ev->error_code, errorText);
+  csFPrintf (stderr, " request code: %d\n", ev->request_code);
+  csFPrintf (stderr, " minor code:   %d\n", ev->minor_code);
+  csFPrintf (stderr, " resource:     %jx\n", (uintmax_t)ev->resourceid);
+  fflush (stderr);
+  
+  csCallStack* stack = csCallStackHelper::CreateCallStack (1);
+  if (stack != 0)
+  {
+    stack->Print (stderr);
+    fflush (stderr);
+    stack->Free();
+  }
+#ifdef CS_DEBUG
+  CS_DEBUG_BREAK;
+#else
+  abort();
+#endif
+  return 0;
 }
 
 bool csXWindow::Initialize (iObjectRegistry *object_reg)
 {
   this->object_reg = object_reg;
+  this->name_reg = csEventNameRegistry::GetRegistry(object_reg);
   csConfigAccess Config(object_reg, "/config/video.cfg");
-  iCommandLineParser* cmdline = CS_QUERY_REGISTRY (object_reg,
-						   iCommandLineParser);
+  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
+						   iCommandLineParser));
   do_hwmouse = Config->GetBool ("Video.SystemMouseCursor", true);
   if (cmdline->GetOption ("sysmouse")) do_hwmouse = true;
   if (cmdline->GetOption ("nosysmouse")) do_hwmouse = false;
-  cmdline->DecRef ();
   // Open display
-  dpy = XOpenDisplay (NULL);
+  dpy = XOpenDisplay (0);
 
   if (!dpy)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "FATAL: Cannot open X display");
     return false;
   }
+  
+  bool Xsynchronous = Config->GetBool ("Video.X.Sync", false);
+  if (Xsynchronous)
+  {
+    XSynchronize (dpy, True);
+    oldErrorHandler = XSetErrorHandler (MyErrorHandler);
+  }
 
   // Set user locale for national character support
   if (XSupportsLocale ())
@@ -149,20 +159,17 @@ bool csXWindow::Initialize (iObjectRegistry *object_reg)
   memset (MouseCursor, 0, sizeof (MouseCursor));
 
   // Create the event outlet
-  iEventQueue* q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
+  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
   if (q != 0)
-  {
     EventOutlet = q->CreateEventOutlet (this);
-    q->DecRef ();
-  }
 
   int opcode, first_event, first_error;
   if (XQueryExtension (dpy, CS_XEXT_XF86VM,
 		       &opcode, &first_event, &first_error))
   {
-    iPluginManager* plugin_mgr = CS_QUERY_REGISTRY(object_reg, iPluginManager);
+    csRef<iPluginManager> plugin_mgr (
+    	CS_QUERY_REGISTRY(object_reg, iPluginManager));
     xf86vm = CS_LOAD_PLUGIN (plugin_mgr, CS_XEXT_XF86VM_SCF_ID, iXExtF86VM);
-    plugin_mgr->DecRef ();
   }
   return true;
 }
@@ -226,7 +233,7 @@ bool csXWindow::Open ()
     if (!xvis)
       Report (CS_REPORTER_SEVERITY_ERROR, "No XVisualInfo!");
     if (!Canvas)
-      Report (CS_REPORTER_SEVERITY_ERROR, "No Canvas!\n");
+      Report (CS_REPORTER_SEVERITY_ERROR, "No Canvas!");
     return false;
   }
 
@@ -256,11 +263,11 @@ bool csXWindow::Open ()
   XSetWMProtocols (dpy, wm_win, &wm_delete_window, 1);
 
   XClassHint *class_hint = XAllocClassHint();
-  class_hint->res_name = win_title;
-  class_hint->res_class = win_title;
+  class_hint->res_name = CS_CONST_CAST (char*, win_title.GetData());
+  class_hint->res_class = CS_CONST_CAST (char*, win_title.GetData());
   XmbSetWMProperties (dpy, wm_win,
-		      NULL, NULL, NULL, 0,
-		      NULL, NULL, class_hint);
+		      0, 0, 0, 0,
+		      0, 0, class_hint);
 
   XFree (class_hint);
 
@@ -299,9 +306,9 @@ bool csXWindow::Open ()
   XChangeProperty (dpy, ctx_win, wm_client_leader, XA_WINDOW, 32,
 		   PropModeReplace, (const unsigned char*)&wm_win, 1);
   XmbSetWMProperties (dpy, ctx_win, win_title, win_title,
-                      NULL, 0, NULL, NULL, NULL);
+                      0, 0, 0, 0, 0);
   XmbSetWMProperties (dpy, wm_win, win_title, win_title,
-                      NULL, 0, NULL, NULL, NULL);
+                      0, 0, 0, 0, 0);
   XMapWindow (dpy, ctx_win);
   XMapRaised (dpy, wm_win);
 
@@ -348,12 +355,13 @@ bool csXWindow::Open ()
 
   // Tell event queue to call us on every frame
   if (!scfiEventHandler)
-    scfiEventHandler = new EventHandler (this);
-  iEventQueue* q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
-  if (q != 0)
-  {
-    q->RegisterListener (scfiEventHandler, CSMASK_Nothing);
-    q->DecRef ();
+    scfiEventHandler.AttachNew (new EventHandler (this));
+  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  if (q != 0) {
+    csEventID events[3] = { csevPreProcess(name_reg), 
+			    csevCommandLineHelp(name_reg), 
+			    CS_EVENTLIST_END };
+    q->RegisterListener (scfiEventHandler, events);
   }
 
   if (xf86vm)
@@ -421,13 +429,14 @@ void csXWindow::Close ()
     XDestroyWindow (dpy, wm_win);
     wm_win = 0;
   }
+  XSync (dpy, true);
 }
 
 void csXWindow::SetTitle (const char* title)
 {
-  delete [] win_title;
-  win_title = new char [strlen (title) + 1];
-  strcpy (win_title, title);
+  win_title = title;
+  if(dpy && wm_win)   
+      XStoreName(dpy, wm_win, win_title);
 }
 
 void csXWindow::AllowResize (bool iAllow)
@@ -482,26 +491,38 @@ static Bool AlwaysTruePredicate (Display*, XEvent*, char*)
   return True;
 }
 
+static int TranslateXMouseButton (int xbutton)
+{
+  static int buttonMapping[] = {csmbNone, csmbLeft, csmbMiddle, csmbRight};
+  const int buttonMapCount =
+    sizeof (buttonMapping) / sizeof (buttonMapping[0]);
+  
+  if (xbutton < buttonMapCount)
+    return buttonMapping[xbutton];
+  else
+    return xbutton-1; // Since mouse buttons are 0-based
+}
+
 bool csXWindow::HandleEvent (iEvent &Event)
 {
-  static int button_mapping[6] = {0, 1, 3, 2, 4, 5};
   XEvent event;
-  KeySym key;
+  KeySym xKey;
+  utf32_char csKeyRaw = 0, csKeyCooked = 0;
   int charcount;
   char charcode [8];
   bool down;
   bool resize = false;
 
-  if ((Event.Type == csevBroadcast)
-   && (Event.Command.Code == cscmdCommandLineHelp))
+  if (Event.Name == csevCommandLineHelp(name_reg))
   {
-    printf ("Options for X-Window Plugin:\n");
-    printf ("  -[no]sysmouse      use/don't use system mouse cursor "
+    csPrintf ("Options for X-Window Plugin:\n");
+    csPrintf ("  -[no]sysmouse      use/don't use system mouse cursor "
       "(default=%s)\n", do_hwmouse ? "use" : "don't");
     return true;
   }
 
   while (XCheckIfEvent (dpy, &event, AlwaysTruePredicate, 0))
+  {
     switch (event.type)
     {
       case ConfigureNotify:
@@ -520,22 +541,22 @@ bool csXWindow::HandleEvent (iEvent &Event)
         XRefreshKeyboardMapping (&event.xmapping);
 	break;
       case ClientMessage:
-	if (STATIC_CAST(Atom, event.xclient.data.l[0]) == wm_delete_window)
+	if (CS_STATIC_CAST(Atom, event.xclient.data.l[0]) == wm_delete_window)
 	{
-	  EventOutlet->Broadcast (cscmdContextClose, (iGraphics2D *)this);
-	  EventOutlet->Broadcast (cscmdQuit);
+	  EventOutlet->Broadcast (csevCanvasClose (name_reg, Canvas), (intptr_t)this);
+	  EventOutlet->Broadcast (csevQuit (name_reg));
 	}
 	break;
       case ButtonPress:
-        EventOutlet->Mouse (button_mapping [event.xbutton.button],
+        EventOutlet->Mouse (TranslateXMouseButton (event.xbutton.button),
           true, event.xbutton.x, event.xbutton.y);
         break;
       case ButtonRelease:
-        EventOutlet->Mouse (button_mapping [event.xbutton.button],
+        EventOutlet->Mouse (TranslateXMouseButton (event.xbutton.button),
           false, event.xbutton.x, event.xbutton.y);
         break;
       case MotionNotify:
-        EventOutlet->Mouse (0, false, event.xbutton.x, event.xbutton.y);
+        EventOutlet->Mouse (csmbNone, false, event.xbutton.x, event.xbutton.y);
         break;
       case KeyPress:
       case KeyRelease:
@@ -546,73 +567,100 @@ bool csXWindow::HandleEvent (iEvent &Event)
 	  (XPointer)&event);
         down = (event.type == KeyPress);
         charcount = XLookupString ((XKeyEvent *)&event, charcode,
-				   sizeof (charcode), &key, NULL);
-        switch (key)
+				   sizeof (charcode), &xKey, 0);
+#define MAP_KEY(xKey, rawCode, cookedCode)	\
+  case xKey:					\
+    csKeyRaw = rawCode;				\
+    csKeyCooked = cookedCode;			\
+    break
+    
+        switch (xKey)
         {
-          case XK_Meta_L:
-	  case XK_Meta_R:
-	  case XK_Alt_L:
-          case XK_Alt_R:      key = CSKEY_ALT; break;
-          case XK_Control_L:
-          case XK_Control_R:  key = CSKEY_CTRL; break;
-          case XK_Shift_L:
-          case XK_Shift_R:    key = CSKEY_SHIFT; break;
-	  case XK_KP_Up:
-	  case XK_KP_8:
-          case XK_Up:         key = CSKEY_UP; break;
-	  case XK_KP_Down:
-	  case XK_KP_2:
-          case XK_Down:       key = CSKEY_DOWN; break;
-	  case XK_KP_Left:
-	  case XK_KP_4:
-          case XK_Left:       key = CSKEY_LEFT; break;
-	  case XK_KP_Right:
-	  case XK_KP_6:
-          case XK_Right:      key = CSKEY_RIGHT; break;
-          case XK_BackSpace:  key = CSKEY_BACKSPACE; break;
-	  case XK_KP_Insert:
-	  case XK_KP_0:
-          case XK_Insert:     key = CSKEY_INS; break;
-	  case XK_KP_Delete:
-	  case XK_KP_Decimal:
-          case XK_Delete:     key = CSKEY_DEL; break;
-	  case XK_KP_Page_Up:
-	  case XK_KP_9:
-          case XK_Page_Up:    key = CSKEY_PGUP; break;
-	  case XK_KP_Page_Down:
-	  case XK_KP_3:
-          case XK_Page_Down:  key = CSKEY_PGDN; break;
-	  case XK_KP_Home:
-	  case XK_KP_7:
-          case XK_Home:       key = CSKEY_HOME; break;
-	  case XK_KP_End:
-	  case XK_KP_1:
-          case XK_End:        key = CSKEY_END; break;
-          case XK_Escape:     key = CSKEY_ESC; break;
+	  MAP_KEY (XK_Meta_L, CSKEY_ALT_LEFT, CSKEY_ALT);
+	  MAP_KEY (XK_Meta_R, CSKEY_ALT_RIGHT, CSKEY_ALT);
+	  
+	  MAP_KEY (XK_Alt_L, CSKEY_ALT_LEFT, CSKEY_ALT);
+	  MAP_KEY (XK_Alt_R, CSKEY_ALT_RIGHT, CSKEY_ALT);
+	  
+	  MAP_KEY (XK_Control_L, CSKEY_CTRL_LEFT, CSKEY_CTRL);
+	  MAP_KEY (XK_Control_R, CSKEY_CTRL_RIGHT, CSKEY_CTRL);
+	  
+	  MAP_KEY (XK_Shift_L, CSKEY_SHIFT_LEFT, CSKEY_SHIFT);
+	  MAP_KEY (XK_Shift_R, CSKEY_SHIFT_RIGHT, CSKEY_SHIFT);
+	  
+	  MAP_KEY (XK_KP_Up, CSKEY_PAD8, CSKEY_UP);
+	  MAP_KEY (XK_KP_8, CSKEY_PAD8, '8');
+	  MAP_KEY (XK_Up, CSKEY_UP, CSKEY_UP);
+	  
+	  MAP_KEY (XK_KP_Down, CSKEY_PAD2, CSKEY_DOWN);
+	  MAP_KEY (XK_KP_2, CSKEY_PAD2, '2');
+	  MAP_KEY (XK_Down, CSKEY_DOWN, CSKEY_DOWN);
+	  
+	  MAP_KEY (XK_KP_Left, CSKEY_PAD4, CSKEY_LEFT);
+	  MAP_KEY (XK_KP_4, CSKEY_PAD4, '4');
+	  MAP_KEY (XK_Left, CSKEY_LEFT, CSKEY_LEFT);
+	  
+	  MAP_KEY (XK_KP_Right, CSKEY_PAD6, CSKEY_RIGHT);
+	  MAP_KEY (XK_KP_6, CSKEY_PAD6, '6');
+	  MAP_KEY (XK_Right, CSKEY_RIGHT, CSKEY_RIGHT);
+	  
+	  MAP_KEY (XK_BackSpace, CSKEY_BACKSPACE, CSKEY_BACKSPACE);
+	  
+	  MAP_KEY (XK_KP_Insert, CSKEY_PAD0, CSKEY_INS);
+	  MAP_KEY (XK_KP_0, CSKEY_PAD0, '0');
+	  MAP_KEY (XK_Insert, CSKEY_INS, CSKEY_INS);
+	  
+	  MAP_KEY (XK_KP_Delete, CSKEY_PADDECIMAL, CSKEY_DEL);
+	  MAP_KEY (XK_KP_Decimal, CSKEY_PADDECIMAL, '.'); 
+	    // @@@ Not necessary '.' on non-English keyboards
+	  MAP_KEY (XK_Delete, CSKEY_DEL, CSKEY_DEL);
+	  
+	  MAP_KEY (XK_KP_Page_Up, CSKEY_PAD9, CSKEY_PGUP);
+	  MAP_KEY (XK_KP_9, CSKEY_PAD9, '9');
+	  MAP_KEY (XK_Page_Up, CSKEY_PGUP, CSKEY_PGUP);
+	  
+	  MAP_KEY (XK_KP_Page_Down, CSKEY_PAD3, CSKEY_PGDN);
+	  MAP_KEY (XK_KP_3, CSKEY_PAD3, '3');
+	  MAP_KEY (XK_Page_Down, CSKEY_PGDN, CSKEY_PGDN);
+	  
+	  MAP_KEY (XK_KP_Home, CSKEY_PAD7, CSKEY_HOME);
+	  MAP_KEY (XK_KP_7, CSKEY_PAD7, '7');
+	  MAP_KEY (XK_Home, CSKEY_HOME, CSKEY_HOME);
+	  
+	  MAP_KEY (XK_KP_End, CSKEY_PAD1, CSKEY_END);
+	  MAP_KEY (XK_KP_1, CSKEY_PAD1, '1');
+	  MAP_KEY (XK_End, CSKEY_END, CSKEY_END);
+
+	  MAP_KEY (XK_Escape, CSKEY_ESC, CSKEY_ESC);
 #ifdef XK_ISO_Left_Tab
-          case XK_ISO_Left_Tab:
+	  MAP_KEY (XK_ISO_Left_Tab, CSKEY_TAB, CSKEY_TAB);
 #endif
-          case XK_KP_Tab:
-          case XK_Tab:        key = CSKEY_TAB; break;
-          case XK_F1:         key = CSKEY_F1; break;
-          case XK_F2:         key = CSKEY_F2; break;
-          case XK_F3:         key = CSKEY_F3; break;
-          case XK_F4:         key = CSKEY_F4; break;
-          case XK_F5:         key = CSKEY_F5; break;
-          case XK_F6:         key = CSKEY_F6; break;
-          case XK_F7:         key = CSKEY_F7; break;
-          case XK_F8:         key = CSKEY_F8; break;
-          case XK_F9:         key = CSKEY_F9; break;
-          case XK_F10:        key = CSKEY_F10; break;
-          case XK_F11:        key = CSKEY_F11; break;
-          case XK_F12:        key = CSKEY_F12; break;
+	  MAP_KEY (XK_KP_Tab, CSKEY_TAB, CSKEY_TAB);
+	  MAP_KEY (XK_Tab, CSKEY_TAB, CSKEY_TAB);
+	  
+	  MAP_KEY (XK_F1, CSKEY_F1, CSKEY_F1);
+	  MAP_KEY (XK_F2, CSKEY_F2, CSKEY_F2);
+	  MAP_KEY (XK_F3, CSKEY_F3, CSKEY_F3);
+	  MAP_KEY (XK_F4, CSKEY_F4, CSKEY_F4);
+	  MAP_KEY (XK_F5, CSKEY_F5, CSKEY_F5);
+	  MAP_KEY (XK_F6, CSKEY_F6, CSKEY_F6);
+	  MAP_KEY (XK_F7, CSKEY_F7, CSKEY_F7);
+	  MAP_KEY (XK_F8, CSKEY_F8, CSKEY_F8);
+	  MAP_KEY (XK_F9, CSKEY_F9, CSKEY_F9);
+	  MAP_KEY (XK_F10, CSKEY_F10, CSKEY_F10);
+	  MAP_KEY (XK_F11, CSKEY_F11, CSKEY_F11);
+	  MAP_KEY (XK_F12, CSKEY_F12, CSKEY_F12);
+	  
           case XK_KP_Add:
 	    {
 	      if (xf86vm && xf86vm->IsFullScreen () &&
 		  down && (event.xkey.state & Mod1Mask))
 		SetVideoMode (true, true, false);
 	      else
-		key = CSKEY_PADPLUS;
+	      {
+		csKeyRaw = CSKEY_PADPLUS;
+		csKeyCooked = '+';
+	      }
 	      break;
 	    }
           case XK_KP_Subtract:
@@ -621,7 +669,10 @@ bool csXWindow::HandleEvent (iEvent &Event)
 		  down && (event.xkey.state & Mod1Mask))
 		SetVideoMode (true, false, true);
 	      else
-		key = CSKEY_PADMINUS;
+	      {
+		csKeyRaw = CSKEY_PADMINUS;
+		csKeyCooked = '-';
+	      }
 	      break;
 	    }
           case XK_KP_Multiply:
@@ -631,11 +682,15 @@ bool csXWindow::HandleEvent (iEvent &Event)
 		SetVideoMode (!xf86vm->IsFullScreen (), false, false);
 	      else
 #endif
-		key = CSKEY_PADMULT;
+	      {
+		csKeyRaw = CSKEY_PADMULT;
+		csKeyCooked = '*';
+	      }
 	      break;
 	    }
-          case XK_KP_Divide:  key = CSKEY_PADDIV; break;
-          case XK_KP_Begin:   key = CSKEY_CENTER; break;
+	  MAP_KEY (XK_KP_Divide, CSKEY_PADDIV, '/');
+	  MAP_KEY (XK_KP_Begin, CSKEY_PAD5, '5');
+	  MAP_KEY (XK_KP_5, CSKEY_PAD5, '5');
 	  case XK_KP_Enter:
           case XK_Return:
 	    {
@@ -644,19 +699,31 @@ bool csXWindow::HandleEvent (iEvent &Event)
 		SetVideoMode (!xf86vm->IsFullScreen (), false, false);
 	      else
 #endif
-		key = CSKEY_ENTER;
+	      {
+		csKeyRaw = (xKey == XK_Return) ? CSKEY_ENTER : CSKEY_PADENTER;
+		csKeyCooked = CSKEY_ENTER;
+	      }
 	      break;
 	    }
-          default:            key = (key <= 127) ? ScanCodeToChar [key] : 0;
+          default:            
+	    {
+	      if (xKey < sizeof(ScanCodeToChar)/sizeof(ScanCodeToChar[0]))
+	      {
+		csKeyRaw = ScanCodeToChar [xKey]; 
+		  // @@@ Remove scancodes, if possible
+		csKeyCooked = charcount == 1 ? charcode[0] : 0;
+	      }
+	    }
+#undef MAP_KEY
         }
-        if (key)
-          EventOutlet->Key(key, charcount == 1 ? uint8(charcode[0]) : 0, down);
+        if (csKeyRaw || csKeyCooked)
+          EventOutlet->Key(csKeyRaw, csKeyCooked, down);
         break;
       case FocusIn:
 	{
-	  EventOutlet->Broadcast (cscmdFocusChanged,
-				  (void *)(event.type == FocusIn));
+	  EventOutlet->Broadcast (csevFocusGained (name_reg), true);
 #ifndef CS_DEBUG
+#ifdef CS_XWIN_GRAB_KEYBOARD
 	  if (xf86vm && !keyboard_grabbed &&
 	      event.xfocus.window == wm_win)
 	  {
@@ -668,16 +735,18 @@ bool csXWindow::HandleEvent (iEvent &Event)
 			   CurrentTime);
 	    keyboard_grabbed = 2;
 	  }
+#endif
 #endif
 	  break;
 	}
       case FocusOut:
 	{
-	  EventOutlet->Broadcast (cscmdFocusChanged,
-				  (void *)(event.type == FocusIn));
+	  EventOutlet->Broadcast (csevFocusLost (name_reg), false);
 #ifndef CS_DEBUG
+#ifdef CS_XWIN_GRAB_KEYBOARD
 	  if (xf86vm && keyboard_grabbed && !--keyboard_grabbed)
 	    XUngrabKeyboard (dpy, CurrentTime);
+#endif
 #endif
 	  break;
 	}
@@ -694,25 +763,26 @@ bool csXWindow::HandleEvent (iEvent &Event)
 	switch(event.xvisibility.state)
 	{
 	  case VisibilityUnobscured:
-	    EventOutlet->Broadcast (cscmdCanvasExposed, NULL);
+	    EventOutlet->Broadcast (csevCanvasExposed (name_reg, Canvas), 0);
 	    break;
 	  case VisibilityPartiallyObscured:
-	    EventOutlet->Broadcast (cscmdCanvasExposed, NULL);
+	    EventOutlet->Broadcast (csevCanvasExposed (name_reg, Canvas), 0);
 	    break;
 	  case VisibilityFullyObscured:
-	    EventOutlet->Broadcast (cscmdCanvasHidden, NULL);
+	    EventOutlet->Broadcast (csevCanvasHidden (name_reg, Canvas), 0);
 	    break;
 	}
 	break;
       case UnmapNotify:
-	EventOutlet->Broadcast (cscmdCanvasHidden, NULL);
+	EventOutlet->Broadcast (csevCanvasHidden (name_reg, Canvas), 0);
 	break;
       case MapNotify:
-	EventOutlet->Broadcast (cscmdCanvasExposed, NULL);
+	EventOutlet->Broadcast (csevCanvasExposed (name_reg, Canvas), 0);
 	break;
       default:
         break;
     }
+  }
 
   if (resize)
     if (Canvas->Resize (wm_width, wm_height))
@@ -742,3 +812,60 @@ bool csXWindow::SetMouseCursor (csMouseCursorID iShape)
     return (iShape == csmcNone);
   } /* endif */
 }
+
+bool csXWindow::SetMouseCursor (iImage *image, const csRGBcolor* keycolor,
+                                int hotspot_x, int hotspot_y,
+                                csRGBcolor fg, csRGBcolor bg)
+{
+  if (!image) return false;
+
+  // Check for cached cursor - we can only cache images with a name
+  if (image->GetName())
+  {
+    Cursor cur = cachedCursors.Get (image->GetName(), 0);
+    if (cur) 
+    {
+      XDefineCursor (dpy, ctx_win, cur);
+      return true;
+    }
+  }
+
+  // In X we must have a monochrome pointer. csCursorConverter takes care of
+  // the conversion.
+  uint8* source;
+  uint8* mask;
+  if (!csCursorConverter::ConvertTo1bpp (image, source, mask, fg, bg, 
+    keycolor, true)) 
+    return false;
+
+  // Create Xwindow compatible Pixmaps
+  Pixmap srcPixmap = XCreatePixmapFromBitmapData (dpy, ctx_win, (char *) source,
+                                image->GetWidth(), image->GetHeight(), 1, 0, 1);
+  Pixmap maskPixmap = XCreatePixmapFromBitmapData (dpy, ctx_win, (char *) mask,
+                                image->GetWidth(), image->GetHeight(), 1, 0, 1);
+  delete[] source;
+  delete[] mask;
+
+  // Create foreground color.  X colors are 16 bit so we must scale them to
+  // keep relative values
+  XColor xfg; 
+  xfg.red = fg.red * 257; xfg.green = fg.green * 257; xfg.blue = fg.blue * 257;
+
+  // Create foreground color.  X colors are 16 bit so we must scale them to
+  // keep relative values
+  XColor xbg; 
+  xbg.red = bg.red * 257; xbg.green = bg.green * 257; xbg.blue = bg.blue * 257;
+    
+  // Create XWindow mouse cursor
+  Cursor mouseCursor = XCreatePixmapCursor (dpy, srcPixmap, maskPixmap, 
+                                            &xfg, &xbg, hotspot_x, hotspot_y);
+  
+  // Select the cursor
+  XDefineCursor (dpy, ctx_win, mouseCursor);
+
+  // Cache a pointer to the cursor (will it will be deleted in the destructor?)
+  if (image->GetName())
+    cachedCursors.Put (image->GetName(), mouseCursor);
+
+  return true;
+}
