@@ -17,9 +17,8 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <math.h>
-
 #include "cssysdef.h"
+#include <math.h>
 #include "cstypes.h"
 #include "csgeom/math3d.h"
 #include "bmpimage.h"
@@ -29,16 +28,10 @@
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csBMPImageIO)
-  SCF_IMPLEMENTS_INTERFACE (iImageIO)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
+namespace cspluginBMPimg
+{
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBMPImageIO::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_FACTORY (csBMPImageIO);
+SCF_IMPLEMENT_FACTORY (csBMPImageIO)
 
 
 #define BMP_MIME "image/bmp"
@@ -64,23 +57,17 @@ static iImageIO::FileFormatDescription formatlist[] =
 
 static inline uint16 us_endian (const uint8* ptr)
 {
-  uint16 n;
-  memcpy(&n, ptr, sizeof(n));
-  return convert_endian(n);
+  return csLittleEndian::Convert (csGetFromAddress::UInt16 (ptr));
 }
 
 static inline uint32 ul_endian (const uint8* ptr)
 {
-  uint32 n;
-  memcpy(&n, ptr, sizeof(n));
-  return convert_endian(n);
+  return csLittleEndian::Convert (csGetFromAddress::UInt32 (ptr));
 }
 
 static inline long l_endian (const uint8* ptr)
 {
-  int32 n;
-  memcpy(&n, ptr, sizeof(n));
-  return convert_endian(n);
+  return csLittleEndian::Convert (csGetFromAddress::Int32 (ptr));
 }
 
 #define BFTYPE(x)    us_endian((x) +  0)
@@ -149,11 +136,9 @@ struct bmpHeader
 
 //---------------------------------------------------------------------------
 
-
-csBMPImageIO::csBMPImageIO (iBase *pParent)
+csBMPImageIO::csBMPImageIO (iBase *pParent) : 
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   formats.Push (&formatlist[0]);
   formats.Push (&formatlist[1]);
   formats.Push (&formatlist[2]);
@@ -161,8 +146,6 @@ csBMPImageIO::csBMPImageIO (iBase *pParent)
 
 csBMPImageIO::~csBMPImageIO()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 const csImageIOFileFormatDescriptions& csBMPImageIO::GetDescription ()
@@ -170,10 +153,10 @@ const csImageIOFileFormatDescriptions& csBMPImageIO::GetDescription ()
   return formats;
 }
 
-csPtr<iImage> csBMPImageIO::Load (uint8* iBuffer, uint32 iSize, int iFormat)
+csPtr<iImage> csBMPImageIO::Load (iDataBuffer* buf, int iFormat)
 {
   ImageBMPFile* i = new ImageBMPFile (iFormat);
-  if (i && !i->Load (iBuffer, iSize))
+  if (i && !i->Load (buf->GetUint8(), buf->GetSize()))
   {
     delete i;
     return 0;
@@ -209,7 +192,7 @@ csPtr<iDataBuffer> csBMPImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
   } /* endswitch */
 
   // alpha channel?
-  bool hasAlpha = format & CS_IMGFMT_ALPHA;
+  bool hasAlpha = (format & CS_IMGFMT_ALPHA) != 0;
   // number of bytes per pixel in truecolor mode
   int bytesPerPixel = palette?1:(hasAlpha?4:3);
 
@@ -225,24 +208,25 @@ csPtr<iDataBuffer> csBMPImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
   bmpHeader hdr;
   hdr.bfTypeLo = 'B';
   hdr.bfTypeHi = 'M';
-  hdr.bfSize = little_endian_long (len);
+  hdr.bfSize = (uint32)csLittleEndian::Convert ((uint32)len);
   hdr.bfRes1 = 0;
   hdr.bfRes2 = 0;
-  hdr.bfOffBits = little_endian_long (sizeof (bmpHeader)-2 + 256*(palette?4:0));
-  hdr.biSize = little_endian_long (40);
-  hdr.biWidth = little_endian_long (w);
-  hdr.biHeight = little_endian_long (h);
-  hdr.biPlanes = little_endian_short (1);
-  hdr.biBitCount = little_endian_short (bytesPerPixel*8);
-  hdr.biCompression = little_endian_long (0);
-  hdr.biSizeImage = little_endian_long (0);
-  hdr.biXPelsPerMeter = little_endian_long (0);
-  hdr.biYPelsPerMeter = little_endian_long (0);
-  hdr.biClrUsed = little_endian_long (0);
-  hdr.biClrImportant = little_endian_long (0);
+  hdr.bfOffBits = csLittleEndian::UInt32 (sizeof (bmpHeader)-2 + 256*(palette?4:0));
+  hdr.biSize = csLittleEndian::UInt32 (40);
+  hdr.biWidth = csLittleEndian::UInt32 (w);
+  hdr.biHeight = csLittleEndian::UInt32 (h);
+  hdr.biPlanes = csLittleEndian::UInt16 (1);
+  hdr.biBitCount = csLittleEndian::UInt16 (bytesPerPixel*8);
+  hdr.biCompression = csLittleEndian::UInt32 (0);
+  hdr.biSizeImage = csLittleEndian::UInt32 (0);
+  hdr.biXPelsPerMeter = csLittleEndian::UInt32 (0);
+  hdr.biYPelsPerMeter = csLittleEndian::UInt32 (0);
+  hdr.biClrUsed = csLittleEndian::UInt32 (0);
+  hdr.biClrImportant = csLittleEndian::UInt32 (0);
 
   csDataBuffer *db = new csDataBuffer (len);
   unsigned char *p = (unsigned char *)db->GetData ();
+  memset (p, 0, len);
   memcpy (p, &hdr.bfTypeLo, sizeof (bmpHeader)-2);
 
   p += sizeof (bmpHeader)-2;
@@ -250,7 +234,7 @@ csPtr<iDataBuffer> csBMPImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
   int x, y, i;
   if (palette)
   {
-    csRGBpixel *pal = Image->GetPalette ();
+    const csRGBpixel *pal = Image->GetPalette ();
     for (i= 0; i < 256; i++)
     {
       *p++ = pal[i].blue;
@@ -297,16 +281,16 @@ csPtr<iDataBuffer> csBMPImageIO::Save (iImage *Image, const char *mime,
   return 0;
 }
 
-bool ImageBMPFile::Load (uint8* iBuffer, uint32 iSize)
+bool ImageBMPFile::Load (uint8* iBuffer, size_t iSize)
 {
   if ((memcmp (iBuffer, BM, 2) == 0) && BISIZE(iBuffer) == WinHSize)
     return LoadWindowsBitmap (iBuffer, iSize);
   return false;
 }
 
-bool ImageBMPFile::LoadWindowsBitmap (uint8* iBuffer, uint32 iSize)
+bool ImageBMPFile::LoadWindowsBitmap (uint8* iBuffer, size_t iSize)
 {
-  set_dimensions (BIWIDTH(iBuffer), BIHEIGHT(iBuffer));
+  SetDimensions (BIWIDTH(iBuffer), BIHEIGHT(iBuffer));
   const int bmp_size = Width * Height;
 
   uint8 *iPtr = iBuffer + BFOFFBITS(iBuffer);
@@ -359,7 +343,7 @@ bool ImageBMPFile::LoadWindowsBitmap (uint8* iBuffer, uint32 iSize)
     else
       return false;
 
-    convert_pal8 (buffer, palette, 16);
+    ConvertFromPal8 (buffer, 0, palette, 16);
     return true;
   }
   else if (BITCOUNT(iBuffer) == _256Color)
@@ -443,7 +427,7 @@ bool ImageBMPFile::LoadWindowsBitmap (uint8* iBuffer, uint32 iSize)
       }
     }
     // Now transform the image data to target format
-    convert_pal8 (buffer, palette);
+    ConvertFromPal8 (buffer, 0, palette);
     return true;
   }
   else if (BITCOUNT(iBuffer) == TRUECOLOR24)
@@ -470,7 +454,7 @@ bool ImageBMPFile::LoadWindowsBitmap (uint8* iBuffer, uint32 iSize)
       buffer_y -= Width;
     }
     // Now transform the image data to target format
-    convert_rgba (buffer);
+    ConvertFromRGBA (buffer);
     return true;
   }
   else if (BITCOUNT(iBuffer) == TRUECOLOR32)
@@ -495,10 +479,11 @@ bool ImageBMPFile::LoadWindowsBitmap (uint8* iBuffer, uint32 iSize)
       buffer_y -= Width;
     }
     // Now transform the image data to target format
-    convert_rgba (buffer);
+    ConvertFromRGBA (buffer);
     return true;
   }
 
   return false;
 }
 
+} // namespace cspluginBMPimg
