@@ -20,39 +20,39 @@
 #ifndef __CS_BMPIMAGE_H__
 #define __CS_BMPIMAGE_H__
 
-#include "csgfx/csimage.h"
+#include "csgfx/memimage.h"
 #include "igraphic/imageio.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"
 #include "iutil/databuff.h"
 
+namespace cspluginBMPimg
+{
+
 /**
  * The BMP image file format loader.
  */
-class csBMPImageIO : public iImageIO
+class csBMPImageIO : public scfImplementation2<csBMPImageIO,
+                                               iImageIO,
+                                               iComponent>
 {
  protected:
   csImageIOFileFormatDescriptions formats;
 
  public:
-  SCF_DECLARE_IBASE;
-
   csBMPImageIO (iBase *pParent);
   virtual ~csBMPImageIO ();
 
   virtual const csImageIOFileFormatDescriptions& GetDescription ();
-  virtual csPtr<iImage> Load (uint8* iBuffer, uint32 iSize, int iFormat);
+  virtual csPtr<iImage> Load (iDataBuffer* buf, int iFormat);
   virtual void SetDithering (bool iEnable);
   virtual csPtr<iDataBuffer> Save (iImage *image, const char *mime = 0,
     const char* extraoptions = 0);
-  virtual csPtr<iDataBuffer> Save (iImage *image, iImageIO::FileFormatDescription *format = 0,
-    const char* extraoptions = 0);
+  virtual csPtr<iDataBuffer> Save (iImage *image,
+  	iImageIO::FileFormatDescription *format = 0,
+	const char* extraoptions = 0);
 
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csBMPImageIO);
-    virtual bool Initialize (iObjectRegistry*) { return true; }
-  } scfiComponent;
+  virtual bool Initialize (iObjectRegistry*) { return true; }
 };
 
 /**
@@ -62,17 +62,19 @@ class csBMPImageIO : public iImageIO
  * Only able to read 24 bits RGB encoded with no palette files and
  * 8 bits RGB encoded files.
  */
-class ImageBMPFile : public csImageFile
+class ImageBMPFile : public csImageMemory
 {
   friend class csBMPImageIO;
   // Load a Windows-format BMP file
-  bool LoadWindowsBitmap (uint8* iBuffer, uint32 iSize);
+  bool LoadWindowsBitmap (uint8* iBuffer, size_t iSize);
 
 private:
   /// Initialize the image object
-  ImageBMPFile (int iFormat) : csImageFile (iFormat) { };
+  ImageBMPFile (int iFormat) : csImageMemory (iFormat) { };
   /// Try to read the BMP file from the buffer and return success status
-  bool Load (uint8* iBuffer, uint32 iSize);
+  bool Load (uint8* iBuffer, size_t iSize);
 };
 
+} // namespace cspluginBMPimg
+
 #endif // __CS_BMPIMAGE_H__
