@@ -16,26 +16,19 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "cssysdef.h"
 #include "gifimage.h"
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csGIFImageIO)
-  SCF_IMPLEMENTS_INTERFACE (iImageIO)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csGIFImageIO::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_FACTORY (csGIFImageIO);
+namespace cspluginGIFimg
+{
 
+SCF_IMPLEMENT_FACTORY (csGIFImageIO)
 
 static iImageIO::FileFormatDescription formatlist[2] =
 {
@@ -43,18 +36,15 @@ static iImageIO::FileFormatDescription formatlist[2] =
   {"image/gif", "GIF89a", CS_IMAGEIO_LOAD}
 };
 
-csGIFImageIO::csGIFImageIO (iBase *pParent)
+csGIFImageIO::csGIFImageIO (iBase *pParent) : 
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   formats.Push (&formatlist[0]);
   formats.Push (&formatlist[1]);
 }
 
 csGIFImageIO::~csGIFImageIO()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 const csImageIOFileFormatDescriptions& csGIFImageIO::GetDescription ()
@@ -62,10 +52,10 @@ const csImageIOFileFormatDescriptions& csGIFImageIO::GetDescription ()
   return formats;
 }
 
-csPtr<iImage> csGIFImageIO::Load (uint8* iBuffer, uint32 iSize, int iFormat)
+csPtr<iImage> csGIFImageIO::Load (iDataBuffer* buf, int iFormat)
 {
   ImageGifFile* i = new ImageGifFile (iFormat);
-  if (i && !i->Load (iBuffer, iSize))
+  if (i && !i->Load (buf->GetUint8(), buf->GetSize()))
   {
     delete i;
     return 0;
@@ -119,18 +109,18 @@ class GIFStream
 {
 private:
   uint8 *buf, *ptr, *bmark;
-  long size, remaining;
+  size_t size, remaining;
   uint8 bitoffs;
   bool EOFcode;
 
 public:
-  GIFStream(uint8* b, long fsize, int offs=0) :
+  GIFStream(uint8* b, size_t fsize, size_t offs=0) :
    buf(b), ptr(b+offs), bmark(0), size(fsize), remaining(fsize-offs),
    bitoffs(0), EOFcode( fsize <= offs ) {}
 
   uint8 operator* () const
   { if (EOFcode) return 0;  else return *ptr; }
-  uint8 operator[] (int i) const
+  uint8 operator[] (size_t i) const
   { if (i >= remaining) return 0;  else return ptr[i]; }
   GIFStream& operator++ ()
   { ptr++;  remaining--;  EOFcode = remaining<=0;  return *this; }
@@ -256,7 +246,7 @@ public:
 
 //---------------------------------------------------------------------------
 
-int ImageGifFile::decode_gif (uint8* iBuffer, long iSize, int* Prefix,
+int ImageGifFile::decode_gif (uint8* iBuffer, size_t iSize, int* Prefix,
   int* Suffix, int* OutCode)
 {
   GIFStream gptr (iBuffer,iSize);
@@ -276,7 +266,7 @@ int ImageGifFile::decode_gif (uint8* iBuffer, long iSize, int* Prefix,
 
   ch = gptr.nextbyte();
 
-  bool has_cmap = ch & COLORMAPMASK;
+  bool has_cmap = (ch & COLORMAPMASK) != 0;
   int cmap_size = 1 << ( (ch & 7) + 1 );
 
   gptr += 2;  // skip 2 bytes : background color and '0'
@@ -301,10 +291,8 @@ int ImageGifFile::decode_gif (uint8* iBuffer, long iSize, int* Prefix,
 	  // get transparent color index - (ch==4), so the 3'th byte
 	  transp_index = gptr[3];
 	  csRGBcolor tcol = palette(transp_index);
-	  has_keycolour = 1;
-	  keycolour_r = tcol.red;
-	  keycolour_g = tcol.green;
-	  keycolour_b = tcol.blue;
+	  has_keycolour = true;
+	  keycolour = tcol;
 	}
 	gptr += ch;
 	break;
@@ -327,9 +315,9 @@ int ImageGifFile::decode_gif (uint8* iBuffer, long iSize, int* Prefix,
 
   int width  = gptr.nextword();
   int height = gptr.nextword();
-  set_dimensions (width, height);
+  SetDimensions (width, height);
 
-  GIFOutput optr(width, height, (gptr.nextbyte() & INTERLACEMASK));
+  GIFOutput optr(width, height, (gptr.nextbyte() & INTERLACEMASK) != 0);
 
   // Note that I ignore the possible existence of a local color map.
   // I'm told there aren't many files around that use them, and the spec
@@ -415,7 +403,7 @@ int ImageGifFile::decode_gif (uint8* iBuffer, long iSize, int* Prefix,
       // Now we put the data out to the image buffer.
       // It's been stacked LIFO, so deal with it that way...
 
-	  int j;
+      int j;
       for (j = OutCount - 1; j >= 0; j--)
       {
         *optr = OutCode [j];
@@ -443,12 +431,12 @@ int ImageGifFile::decode_gif (uint8* iBuffer, long iSize, int* Prefix,
   }
 
   Format &= ~CS_IMGFMT_ALPHA;
-  convert_pal8 (optr.get_image (), palette.get_palette ());
+  ConvertFromPal8 (optr.get_image (), 0, palette.get_palette ());
 
   return 0;
 }
 
-bool ImageGifFile::Load (uint8* iBuffer, uint32 iSize)
+bool ImageGifFile::Load (uint8* iBuffer, size_t iSize)
 {
   int* Prefix  = new int [4096]; // Hash table used by decompressor.
   int* Suffix  = new int [4096]; // Hash table used by decompressor.
@@ -462,3 +450,5 @@ bool ImageGifFile::Load (uint8* iBuffer, uint32 iSize)
 
   return (rc == 0);
 }
+
+} // namespace cspluginGIFimg
