@@ -18,16 +18,15 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-// note: the order is not quite CS standard... However, if changed you'll
-// get a couple of compile errors, mostly due to conflicts in some #defines 
-// and __declspec()s
 #include "cssysdef.h"
 #include "csutil/scf.h"
 #include "csutil/sysfunc.h"
 #include "csgeom/csrect.h"
+#include "csgeom/math.h"
 #include "csgfx/rgbpixel.h"
 #include "csgfx/packrgb.h"
 #include "csutil/databuf.h"
+#include "csplugincommon/imageloader/optionsparser.h"
 #include "ivaria/reporter.h"
 
 extern "C"
@@ -54,17 +53,10 @@ extern "C"
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csJNGImageIO)
-  SCF_IMPLEMENTS_INTERFACE (iImageIO)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csJNGImageIO::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_FACTORY (csJNGImageIO);
+namespace cspluginJNGimg
+{
 
+SCF_IMPLEMENT_FACTORY (csJNGImageIO)
 
 #define JNG_MIME "image/x-jng" 
 #define MNG_MIME "image/x-mng"
@@ -107,22 +99,19 @@ void ReportLibmngError (iObjectRegistry *object_reg, mng_handle hMNG, char* msg)
     &extra2, &errortext);
 
   Report (object_reg, CS_REPORTER_SEVERITY_WARNING,
-    "%s: %s (severity %d, chunkname %.8x, chunkseq %d, %.8x, %.8x)",
+    "%s: %s (severity %" PRId8 ", chunkname %.8" PRIx32 ", chunkseq %" PRIu32 ", %.8" PRIx32 ", %.8" PRIx32 ")",
     msg, errortext, severity, chunkname, chunkseq, extra1, extra2);
 }
 
-csJNGImageIO::csJNGImageIO (iBase *pParent)
+csJNGImageIO::csJNGImageIO (iBase *pParent) : 
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   formats.Push (&formatlist[0]);
   formats.Push (&formatlist[1]);
 }
 
 csJNGImageIO::~csJNGImageIO()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 const csImageIOFileFormatDescriptions& csJNGImageIO::GetDescription ()
@@ -130,10 +119,10 @@ const csImageIOFileFormatDescriptions& csJNGImageIO::GetDescription ()
   return formats;
 }
 
-csPtr<iImage> csJNGImageIO::Load (uint8* iBuffer, uint32 iSize, int iFormat)
+csPtr<iImage> csJNGImageIO::Load (iDataBuffer* buf, int iFormat)
 {
   ImageJngFile* i = new ImageJngFile (iFormat, object_reg);
-  if (i && !i->Load (iBuffer, iSize))
+  if (i && !i->Load (buf->GetUint8(), buf->GetSize()))
   {
     delete i;
     return 0;
@@ -184,7 +173,7 @@ mng_bool csJNGImageIO::cb_writedata (mng_handle hHandle, mng_ptr pBuf,
 
   this_ = (csJNGImageIO *)mng_get_userdata (hHandle);
 
-  *pWritten = this_->outfile->Write ((char*)pBuf, iBuflen);
+  *pWritten = (mng_uint32)this_->outfile->Write ((char*)pBuf, iBuflen);
 
   return MNG_TRUE;
 }
@@ -355,57 +344,28 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image,
        compress=50
        progressive,compress=30
    */
-  const char *current_opt = extraoptions;
-  while (current_opt && *current_opt)
+  csImageLoaderOptionsParser optparser (extraoptions);
+  optparser.GetBool ("progressive", progressive);
+  if (optparser.GetInt ("compress", quality))
   {
-    if (*current_opt == ',') current_opt++;
-
-    const char *opt_end = strchr (current_opt, ',');
-    if (!opt_end) opt_end = strchr (current_opt, 0);
-
-    char *opt_key = new char[opt_end - current_opt + 1];
-    strncpy (opt_key, current_opt, opt_end - current_opt);
-    opt_key[opt_end - current_opt] = 0;
-    current_opt = opt_end;
-
-    char *opt_value = strchr (opt_key, '=');
-    if (opt_value) *opt_value++ = 0;
-
-    if (!strcmp (opt_key, "compress"))
-    {
-      if (opt_value)
-      {
-	quality = 100 - atoi (opt_value);
-        if (alpha_jpeg_quality == -1) alpha_jpeg_quality = quality;
-      }
-    }
-    else if (!strcmp (opt_key, "progressive"))
-    {
-      progressive = true;
-    }
-    else if (!strcmp (opt_key, "jng_lossy_alpha"))
-    {
-      alpha_jpeg = true;
-      if (alpha_jpeg_quality == -1) alpha_jpeg_quality = quality;
-    }
-    else if (!strcmp (opt_key, "jng_alpha_compress"))
-    {
-      if (opt_value)
-      {
-	alpha_jpeg_quality = 100 - atoi (opt_value);
-	alpha_png_compress = atoi (opt_value) / 10;
-      }
-    }
-
-    delete opt_key;
+    quality = 100 - quality;
+    if (quality < 0) quality = 0;
+    if (quality > 100) quality = 100;
+    if (alpha_jpeg_quality == -1) alpha_jpeg_quality = quality;
+  }
+  if (optparser.GetBool ("jng_lossy_alpha", alpha_jpeg))
+  {
+    if (alpha_jpeg_quality == -1) alpha_jpeg_quality = quality;
+  }
+  if (optparser.GetInt ("jng_alpha_compress", alpha_png_compress))
+  {
+    alpha_jpeg_quality = 100 - alpha_png_compress;
+    if (alpha_jpeg_quality < 0) alpha_jpeg_quality = 0;
+    if (alpha_jpeg_quality > 100) alpha_jpeg_quality = 100;
+    alpha_png_compress /= 10;
+    if (alpha_png_compress < 0) alpha_png_compress = 0;
+    if (alpha_png_compress > 9) alpha_png_compress = 9;
   }
-
-  if (quality < 0) quality = 0;
-  if (quality > 100) quality = 100;
-  if (alpha_jpeg_quality < 0) alpha_jpeg_quality = 0;
-  if (alpha_jpeg_quality > 100) alpha_jpeg_quality = 100;
-  if (alpha_png_compress < 0) alpha_png_compress = 0;
-  if (alpha_png_compress > 9) alpha_png_compress = 9;
 
   mng_handle handle = mng_initialize ( mng_ptr(this), cb_alloc, 
                                       cb_free, MNG_NULL);
@@ -436,7 +396,7 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image,
     return 0;
   }
 
-  bool has_alpha = imgRGBA->GetFormat() & CS_IMGFMT_ALPHA;
+  bool has_alpha = (imgRGBA->GetFormat() & CS_IMGFMT_ALPHA) != 0;
 
   if (mng_putchunk_jhdr (handle, imgRGBA->GetWidth(), imgRGBA->GetHeight(),
 			 has_alpha ? MNG_COLORTYPE_JPEGCOLORA 
@@ -527,7 +487,8 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image,
 
       // funny, mng_putchunk_jdaa is missing from libmng
       //if (mng_putchunk_jdaa (handle, ds.len, ds.data) != MNG_NOERROR)
-      if (mng_putchunk_unknown (handle, MNG_UINT_JDAA, ds.len, ds.data) != MNG_NOERROR)
+      if (mng_putchunk_unknown (handle, MNG_UINT_JDAA, 
+	(mng_uint32)ds.len, ds.data) != MNG_NOERROR)
       {
 	ReportLibmngError (object_reg, handle, "failed to put JDAA chunk");
 	mng_cleanup (&handle);
@@ -593,7 +554,7 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image,
         size_t size = sizeof (buff) - zs.avail_out;
 
 	// create a chuk w/compressed data.
-	if (mng_putchunk_idat (handle, size, &buff) != MNG_NOERROR)
+	if (mng_putchunk_idat (handle, (mng_uint32)size, &buff) != MNG_NOERROR)
 	{
 	  ReportLibmngError (object_reg, handle, "failed to put IDAT chunk");
           deflateEnd (&zs);
@@ -650,7 +611,7 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image,
   jpeg_start_compress (&cinfo, true);
 
   JSAMPROW row_pointer[1];
-  JSAMPLE *image = (JSAMPLE*)csPackRGBpixelToRGB
+  JSAMPLE *image = (JSAMPLE*)csPackRGB::PackRGBpixelToRGB
     ((csRGBpixel*)Image->GetImageData (),
     Image->GetWidth () * Image->GetHeight ());
   row_pointer[0] = (JSAMPLE*)&row[0];
@@ -667,7 +628,7 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image,
 
   delete [] row;
 
-  if (mng_putchunk_jdat (handle, ds.len, ds.data) != MNG_NOERROR)
+  if (mng_putchunk_jdat (handle, (mng_uint32)ds.len, ds.data) != MNG_NOERROR)
   {
     ReportLibmngError (object_reg, handle, "failed to put JDAT chunk");
     mng_cleanup (&handle);
@@ -729,10 +690,6 @@ csPtr<iDataBuffer> csJNGImageIO::Save (iImage *Image, const char *mime,
  * (http://www.libmng.com/MNGsuite) and see for yourself.
  */
 
-SCF_IMPLEMENT_IBASE_EXT (ImageJngFile)
-  SCF_IMPLEMENTS_INTERFACE (iAnimatedImage)
-SCF_IMPLEMENT_IBASE_EXT_END
-
 mng_bool ImageJngFile::cb_readdata (mng_handle hHandle, mng_ptr pBuf,
 					      mng_uint32 iBuflen, mng_uint32 *pRead)
 {
@@ -741,7 +698,8 @@ mng_bool ImageJngFile::cb_readdata (mng_handle hHandle, mng_ptr pBuf,
   this_ = (ImageJngFile *)mng_get_userdata (hHandle);
 
   // determine amount of data to read; 0 if EOF
-  *pRead = MIN (iBuflen, this_->bufferSize - (this_->bufptr - this_->buffer));
+  *pRead = (mng_uint32)csMin ((size_t)iBuflen, 
+    this_->bufferSize - (this_->bufptr - this_->buffer));
   if (*pRead > 0)
   {
     memcpy (pBuf, this_->bufptr, *pRead);
@@ -812,7 +770,7 @@ mng_bool ImageJngFile::cb_settimer (mng_handle hHandle, mng_uint32 iMsecs)
 }
 
 ImageJngFile::ImageJngFile (int iFormat, iObjectRegistry* p) : 
-  csImageFile (iFormat)
+  scfImplementationType (this, iFormat)
 { 
   object_reg = p; 
   vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
@@ -830,7 +788,7 @@ ImageJngFile::~ImageJngFile ()
   delete[] NewImage; 
 }
 
-bool ImageJngFile::Load (uint8 *iBuffer, uint32 iSize)
+bool ImageJngFile::Load (uint8 *iBuffer, size_t iSize)
 {
   mng_retcode retcode;
 
@@ -911,9 +869,11 @@ bool ImageJngFile::Load (uint8 *iBuffer, uint32 iSize)
 
   if (NewImage)
   {
-    csRGBpixel *rgbImage = csCopyUnpackRGBAtoRGBpixel (NewImage, Width*Height);
-    convert_rgba (rgbImage);
-    CheckAlpha();
+    csRGBpixel *rgbImage = 
+      csPackRGBA::CopyUnpackRGBAtoRGBpixel (NewImage, Width*Height);
+    ConvertFromRGBA (rgbImage);
+    // Subsequent images may contain alpha, so don't check
+    if (!doWait) CheckAlpha();
   }
 
   if (mng_get_sigtype (handle) != mng_it_mng)
@@ -935,6 +895,9 @@ bool ImageJngFile::Animate (csTicks time, csRect* dirtyrect)
 
   ImageJngFile::dirtyrect = dirtyrect;
 
+  // Prevent very slow updates if the elapsed time is too big.
+  if (time > 500) time = 500;
+
   bool updated = false;
   total_time_elapsed += time;
   time_elapsed += time;
@@ -947,9 +910,16 @@ bool ImageJngFile::Animate (csTicks time, csRect* dirtyrect)
 
   if (updated)
   {
-    csRGBpixel *rgbImage = csCopyUnpackRGBAtoRGBpixel (NewImage, Width*Height);
-    convert_rgba (rgbImage);
-    CheckAlpha();
+    if (csPackRGBA::IsRGBpixelSane())
+    {
+      memcpy (Image, NewImage, Width * Height * sizeof (csRGBpixel));
+    }
+    else
+    {
+      csRGBpixel *rgbImage = csPackRGBA::CopyUnpackRGBAtoRGBpixel (
+	NewImage, Width*Height);
+      ConvertFromRGBA (rgbImage);
+    }
   }
 
   return updated;
@@ -966,3 +936,4 @@ bool ImageJngFile::IsAnimated ()
    */
 }
 
+} // namespace cspluginJNGimg
