@@ -16,42 +16,28 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "cssysdef.h"
 #include <math.h>
 #include <stdio.h>
 #include <setjmp.h>
 #include <string.h>
 
-#include "cssysdef.h"
 #include "csutil/scf.h"
+#include "csplugincommon/imageloader/optionsparser.h"
 #include "jpgimage.h"
 #include "csgfx/rgbpixel.h"
 #include "csgfx/packrgb.h"
 #include "csutil/databuf.h"
 #include "ivaria/reporter.h"
 
-extern "C"
-{
-#define jpeg_boolean boolean
-#define JDCT_DEFAULT JDCT_FLOAT	// use floating-point for decompression
-#define INT32 JPEG_INT32
-#include <jpeglib.h>
-#include <jerror.h>
-#undef INT32
-}
-
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csJPGImageIO)
-  SCF_IMPLEMENTS_INTERFACE (iImageIO)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
+namespace cspluginJPGimg
+{
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csJPGImageIO::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_FACTORY (csJPGImageIO);
+CS_LEAKGUARD_IMPLEMENT (ImageJpgFile);
 
+SCF_IMPLEMENT_FACTORY (csJPGImageIO)
 
 #define JPG_MIME "image/jpg"
 
@@ -79,18 +65,15 @@ void Report (iObjectRegistry *object_reg, int severity, const char* msg, ...)
   va_end (arg);
 }
 
-csJPGImageIO::csJPGImageIO (iBase *pParent)
+csJPGImageIO::csJPGImageIO (iBase *pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   formats.Push (&formatlist[0]);
   formats.Push (&formatlist[1]);
 }
 
 csJPGImageIO::~csJPGImageIO()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 const csImageIOFileFormatDescriptions& csJPGImageIO::GetDescription ()
@@ -100,14 +83,6 @@ const csImageIOFileFormatDescriptions& csJPGImageIO::GetDescription ()
 
 /* ==== Error mgmnt ==== */
 
-struct my_error_mgr
-{
-  struct jpeg_error_mgr pub;	/* "public" fields */
-  jmp_buf setjmp_buffer;	/* for return to caller */
-};
-
-typedef struct my_error_mgr *my_error_ptr;
-
 static void my_error_exit (j_common_ptr cinfo)
 {
   /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
@@ -196,10 +171,10 @@ static void jpeg_buffer_dest (j_compress_ptr cinfo, jpg_datastore *ds)
   dest->ds = ds;
 }
 
-csPtr<iImage> csJPGImageIO::Load (uint8* iBuffer, uint32 iSize, int iFormat)
+csPtr<iImage> csJPGImageIO::Load (iDataBuffer* buf, int iFormat)
 {
-  ImageJpgFile* i = new ImageJpgFile (iFormat, object_reg);
-  if (i && !i->Load (iBuffer, iSize))
+  ImageJpgFile* i = new ImageJpgFile (object_reg, iFormat);
+  if (i && !i->Load (buf))
   {
     delete i;
     return 0;
@@ -226,8 +201,8 @@ csPtr<iDataBuffer> csJPGImageIO::Save(iImage *Image, iImageIO::FileFormatDescrip
   } /* endswitch */
 
   // compression options
-  volatile int quality = 80;
-  volatile bool progressive = false;
+  int quality = 80;
+  bool progressive = false;
 
   /*
      parse output options.
@@ -243,38 +218,15 @@ csPtr<iDataBuffer> csJPGImageIO::Save(iImage *Image, iImageIO::FileFormatDescrip
        compress=50
        progressive,compress=30
    */
-  const char *current_opt = extraoptions;
-  while (current_opt && *current_opt)
+  csImageLoaderOptionsParser optparser (extraoptions);
+  optparser.GetBool ("progressive", progressive);
+  if (optparser.GetInt ("compress", quality))
   {
-    if (*current_opt == ',') current_opt++;
-
-    const char *opt_end = strchr (current_opt, ',');
-    if (!opt_end) opt_end = strchr (current_opt, 0);
-
-    char *opt_key = new char[opt_end - current_opt + 1];
-    strncpy (opt_key, current_opt, opt_end - current_opt);
-    opt_key[opt_end - current_opt] = 0;
-    current_opt = opt_end;
-
-    char *opt_value = strchr (opt_key, '=');
-    if (opt_value) *opt_value++ = 0;
-
-    if (!strcmp (opt_key, "compress"))
-    {
-      if (opt_value)
-	quality = 100 - atoi (opt_value);
-    }
-    else if (!strcmp (opt_key, "progressive"))
-    {
-      progressive = true;
-    }
-
-    delete opt_key;
+    quality = 100 - quality;
+    if (quality < 0) quality = 0;
+    if (quality > 100) quality = 100;
   }
 
-  if (quality < 0) quality = 0;
-  if (quality > 100) quality = 100;
-
   JSAMPLE* volatile row = 0;
   struct jpg_datastore ds;
   struct jpeg_compress_struct cinfo;
@@ -307,7 +259,7 @@ csPtr<iDataBuffer> csJPGImageIO::Save(iImage *Image, iImageIO::FileFormatDescrip
   jpeg_start_compress (&cinfo, true);
 
   JSAMPROW row_pointer[1];
-  JSAMPLE *image = (JSAMPLE*)csPackRGBpixelToRGB
+  JSAMPLE *image = (JSAMPLE*)csPackRGB::PackRGBpixelToRGB
     ((csRGBpixel*)Image->GetImageData (),
      Image->GetWidth () * Image->GetHeight ());
   row_pointer[0] = (JSAMPLE*)&row[0];
@@ -447,12 +399,18 @@ static void jpeg_memory_src (j_decompress_ptr cinfo, char *inbfr, int len)
 
 bool ImageJpgFile::dither = true;
 
-bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
+ImageJpgFile::JpegLoader::~JpegLoader()
 {
-  struct jpeg_decompress_struct cinfo;
-  struct my_error_mgr jerr;
-  int row_stride;		/* physical row width in output buffer */
+  if (setjmp (jerr.setjmp_buffer))
+  {
+    return;
+  }
+  if (decompCreated) jpeg_destroy_decompress (&cinfo);
+  decompCreated = false;
+}
 
+bool ImageJpgFile::JpegLoader::InitOk()
+{
   // For now we don't support alpha-map images
   // The libjpeg docs are unclear on this subject, it seems that alpha
   // mapped images could be supported by libjpeg (it supports a random
@@ -460,8 +418,6 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
   // alpha-mapped JPEG images and can't test it.
   Format &= ~CS_IMGFMT_ALPHA;
 
-  int i;
-
   /* ==== Step 1: allocate and initialize JPEG decompression object */
   /* We set up the normal JPEG error routines, then override error_exit. */
   cinfo.err = jpeg_std_error(&jerr.pub);
@@ -475,15 +431,17 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
       Report (object_reg, CS_REPORTER_SEVERITY_WARNING,
 	"%s\n", errmsg);
     }
-    jpeg_destroy_decompress (&cinfo);
+    if (decompCreated) jpeg_destroy_decompress (&cinfo);
+    decompCreated = false;
     return false;
   }
 
   /* Now we can initialize the JPEG decompression object. */
   jpeg_create_decompress (&cinfo);
+  decompCreated = true;
 
   /* ==== Step 2: specify data source (memory buffer, in this case) */
-  jpeg_memory_src (&cinfo, (char *)iBuffer, iSize);
+  jpeg_memory_src (&cinfo, dataSource->GetData(), (int)dataSource->GetSize());
 
   /* ==== Step 3: read file parameters with jpeg_read_header() */
   (void) jpeg_read_header(&cinfo, TRUE);
@@ -496,7 +454,10 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
     cinfo.dither_mode = dither ? JDITHER_FS : JDITHER_NONE;
     cinfo.quantize_colors = TRUE;
     cinfo.desired_number_of_colors = 256;
+    dataType = rdtIndexed;
   }
+  else
+    dataType = rdtR8G8B8;
   // We almost always want RGB output (no grayscale, yuv etc)
   if (cinfo.jpeg_color_space != JCS_GRAYSCALE)
     cinfo.out_color_space = JCS_RGB;
@@ -506,21 +467,50 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
 
   /* ==== Step 5: Start decompressor */
 
-  (void) jpeg_start_decompress (&cinfo);
+  jpeg_start_decompress (&cinfo);
   /* We may need to do some setup of our own at this point before reading
    * the data.  After jpeg_start_decompress() we have the correct scaled
    * output image dimensions available, as well as the output colormap
    * if we asked for color quantization.
    * In this example, we need to make an output work buffer of the right size.
    */
+  Width = cinfo.output_width;
+  Height = cinfo.output_height;
 
-  set_dimensions (cinfo.output_width, cinfo.output_height);
+  if ((Format & CS_IMGFMT_MASK) == CS_IMGFMT_ANY)
+    Format = (Format & ~CS_IMGFMT_MASK) |
+      (cinfo.quantize_colors ? CS_IMGFMT_PALETTED8 : CS_IMGFMT_TRUECOLOR);
+
+  return true;
+}
+
+bool ImageJpgFile::JpegLoader::LoadData ()
+{
+  CS_ASSERT (decompCreated);
+
+  int row_stride;		/* physical row width in output buffer */
+
+  int i;
+
+  if (setjmp (jerr.setjmp_buffer))
+  {
+    char errmsg [256];
+    if (cinfo.err->msg_code != JERR_NO_SOI)
+    {
+      cinfo.err->format_message ((jpeg_common_struct *)&cinfo, errmsg);
+      Report (object_reg, CS_REPORTER_SEVERITY_WARNING,
+	"%s\n", errmsg);
+    }
+    if (decompCreated) jpeg_destroy_decompress (&cinfo);
+    decompCreated = false;
+    return false;
+  }
 
   int pixelcount = Width * Height;
   if ((Format & CS_IMGFMT_MASK) == CS_IMGFMT_PALETTED8)
-    Image = new uint8 [pixelcount];
+    indexData = new uint8 [pixelcount];
   else
-    Image = new csRGBpixel [pixelcount];
+    rawData.AttachNew (new csDataBuffer (pixelcount * 3));
 
   /* JSAMPLEs per row in output buffer */
   row_stride = cinfo.output_width * cinfo.output_components;
@@ -549,18 +539,20 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
 	// Safety.
 	if (bufp + row_stride > pixelcount) break;
         /* paletted image */
-        memcpy (((uint8 *)Image) + bufp, buffer [0], row_stride);
+	memcpy (indexData + bufp, buffer [0], row_stride);
       }
       else
       {
 	// Safety.
 	if (bufp + (int)cinfo.output_width > pixelcount) break;
 	/* Grayscale image */
-        csRGBpixel *out = ((csRGBpixel *)Image) + bufp;
+	uint8* out = rawData->GetUint8() + bufp*3;
         for (i = 0; i < (int)cinfo.output_width; i++)
         {
-          out->red = out->green = out->blue = buffer [0][i];
-          out++;
+	  const uint8 v = buffer [0][i];
+	  *out++ = v;
+	  *out++ = v;
+	  *out++ = v;
         }
       }
     else
@@ -568,9 +560,8 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
       // Safety.
       if (bufp + (int)cinfo.output_width > pixelcount) break;
       /* rgb triplets */
-      csRGBpixel *out = ((csRGBpixel *)Image) + bufp;
-      for (i = 0; i < (int)cinfo.output_width; i++)
-        memcpy (out++, buffer [0] + i * 3, 3);
+      uint8* out = rawData->GetUint8() + bufp*3;
+      memcpy (out, buffer[0], cinfo.output_width * 3);
     }
     bufp += cinfo.output_width;
   }
@@ -578,28 +569,24 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
   /* Get palette */
   if (cinfo.quantize_colors)
   {
-    Palette = new csRGBpixel [256];
+    palette = new csRGBpixel [256];
     int cshift = 8 - cinfo.data_precision;
     for (i = 0; i < cinfo.actual_number_of_colors; i++)
     {
-      Palette [i].red   = cinfo.colormap [0] [i] << cshift;
+      palette [i].red   = cinfo.colormap [0] [i] << cshift;
       if (cinfo.jpeg_color_space != JCS_GRAYSCALE)
       {
-        Palette [i].green = cinfo.colormap [1] [i] << cshift;
-        Palette [i].blue  = cinfo.colormap [2] [i] << cshift;
+        palette [i].green = cinfo.colormap [1] [i] << cshift;
+        palette [i].blue  = cinfo.colormap [2] [i] << cshift;
       }
       else
-        Palette [i].green = Palette [i].blue = Palette [i].red;
+        palette [i].green = palette [i].blue = palette [i].red;
     }
   }
 
-  if ((Format & CS_IMGFMT_MASK) == CS_IMGFMT_ANY)
-    Format = (Format & ~CS_IMGFMT_MASK) |
-      (cinfo.quantize_colors ? CS_IMGFMT_PALETTED8 : CS_IMGFMT_TRUECOLOR);
-
   /* ==== Step 7: Finish decompression */
 
-  (void) jpeg_finish_decompress(&cinfo);
+  jpeg_finish_decompress(&cinfo);
   /* We can ignore the return value since suspension is not possible
    * with the buffer data source.
    */
@@ -607,12 +594,23 @@ bool ImageJpgFile::Load (uint8* iBuffer, uint32 iSize)
   /* ==== Step 8: Release JPEG decompression object */
   /* This is an important step since it will release a good deal of memory. */
   jpeg_destroy_decompress(&cinfo);
-
+  decompCreated = false;
 
   /* At this point you may want to check to see whether any corrupt-data
    * warnings occurred (test whether jerr.pub.num_warnings is nonzero).
    */
 
   /* And we're done! */
+  dataSource = 0;
   return true;
 }
+
+csRef<iImageFileLoader> ImageJpgFile::InitLoader (csRef<iDataBuffer> source)
+{
+  csRef<JpegLoader> loader;
+  loader.AttachNew (new JpegLoader (Format, object_reg, source));
+  if (!loader->InitOk()) return 0;
+  return loader;
+}
+
+} // namespace cspluginJPGimg
