@@ -18,10 +18,13 @@
 */
 
 #include "cssysdef.h"
+#include "csgfx/imagetools.h"
 #include "csgfx/rgbpixel.h"
 #include "csgfx/packrgb.h"
-#include "csutil/databuf.h"
 #include "csutil/csendian.h"
+#include "csutil/csstring.h"
+#include "csutil/databuf.h"
+#include "csplugincommon/imageloader/optionsparser.h"
 #include <math.h>
 
 extern "C"
@@ -36,24 +39,19 @@ extern "C"
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csPNGImageIO)
-  SCF_IMPLEMENTS_INTERFACE (iImageIO)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
+namespace cspluginPNGimg
+{
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csPNGImageIO::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_FACTORY (csPNGImageIO);
+CS_LEAKGUARD_IMPLEMENT (ImagePngFile);
 
+SCF_IMPLEMENT_FACTORY (csPNGImageIO)
 
 #define PNG_MIME "image/png"
 
 struct datastore{
   unsigned char *data;
-  long pos;
-  long length;
+  size_t pos;
+  size_t length;
 
   datastore () { data = 0; pos = 0; length = 0; }
   ~datastore () { free (data); }
@@ -87,10 +85,9 @@ static iImageIO::FileFormatDescription formatlist[5] =
   {PNG_MIME, "RGBA", CS_IMAGEIO_LOAD|CS_IMAGEIO_SAVE}
 };
 
-csPNGImageIO::csPNGImageIO (iBase *pParent)
+csPNGImageIO::csPNGImageIO (iBase *pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   formats.Push (&formatlist[0]);
   formats.Push (&formatlist[1]);
   formats.Push (&formatlist[2]);
@@ -100,8 +97,6 @@ csPNGImageIO::csPNGImageIO (iBase *pParent)
 
 csPNGImageIO::~csPNGImageIO()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 const csImageIOFileFormatDescriptions& csPNGImageIO::GetDescription ()
@@ -109,10 +104,10 @@ const csImageIOFileFormatDescriptions& csPNGImageIO::GetDescription ()
   return formats;
 }
 
-csPtr<iImage> csPNGImageIO::Load (uint8* iBuffer, uint32 iSize, int iFormat)
+csPtr<iImage> csPNGImageIO::Load (iDataBuffer* buf, int iFormat)
 {
-  ImagePngFile* i = new ImagePngFile (iFormat);
-  if (i && !i->Load (iBuffer, iSize))
+  ImagePngFile* i = new ImagePngFile (object_reg, iFormat);
+  if (i && !i->Load (buf))
   {
     delete i;
     return 0;
@@ -124,38 +119,6 @@ void csPNGImageIO::SetDithering (bool)
 {
 }
 
-static inline unsigned sqr (int x)
-{
-  return (x * x);
-}
-
-int closest_index (iImage *Image, csRGBpixel *iColor)
-{
-  csRGBpixel *Palette = Image->GetPalette();
-
-  if (!Palette)
-    return -1;
-
-  int closest_idx = -1;
-  unsigned closest_dst = (unsigned)-1;
-
-  int idx;
-  for (idx = 0; idx < 256; idx++)
-  {
-    unsigned dst = sqr (iColor->red   - Palette [idx].red)   * R_COEF_SQ +
-                   sqr (iColor->green - Palette [idx].green) * G_COEF_SQ +
-                   sqr (iColor->blue  - Palette [idx].blue)  * B_COEF_SQ;
-    if (dst == 0)
-      return idx;
-    if (dst < closest_dst)
-    {
-      closest_dst = dst;
-      closest_idx = idx;
-    } /* endif */
-  }
-  return closest_idx;
-}
-
 csPtr<iDataBuffer> csPNGImageIO::Save (iImage *Image, iImageIO::FileFormatDescription *,
   const char* extraoptions)
 {
@@ -163,7 +126,7 @@ csPtr<iDataBuffer> csPNGImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
     return 0;
 
   int compress = 6;
-  volatile bool interlace = false;
+  bool interlace = false;
   /*
      parse output options.
      options are a comma-separated list and can be either
@@ -178,38 +141,15 @@ csPtr<iDataBuffer> csPNGImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
        compress=50
        progressive,compress=30
    */
-  const char *current_opt = extraoptions;
-  while (current_opt && *current_opt)
+  csImageLoaderOptionsParser optparser (extraoptions);
+  optparser.GetBool ("progressive", interlace);
+  if (optparser.GetInt ("compress", compress))
   {
-    if (*current_opt == ',') current_opt++;
-
-    const char *opt_end = strchr (current_opt, ',');
-    if (!opt_end) opt_end = strchr (current_opt, 0);
-
-    char *opt_key = new char[opt_end - current_opt + 1];
-    strncpy (opt_key, current_opt, opt_end - current_opt);
-    opt_key[opt_end - current_opt] = 0;
-    current_opt = opt_end;
-
-    char *opt_value = strchr (opt_key, '=');
-    if (opt_value) *opt_value++ = 0;
-
-    if (!strcmp (opt_key, "compress"))
-    {
-      if (opt_value)
-	compress = atoi (opt_value) / 10;
-    }
-    else if (!strcmp (opt_key, "progressive"))
-    {
-      interlace = true;
-    }
-
-    delete opt_key;
+    compress /= 10;
+    if (compress < 0) compress = 0;
+    if (compress > 9) compress = 9;
   }
 
-  if (compress < 0) compress = 0;
-  if (compress > 9) compress = 9;
-
   datastore ds;
 
   png_structp png = png_create_write_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0);
@@ -281,7 +221,7 @@ error2:
   /* set the palette if there is one. */
   if (colortype & PNG_COLOR_MASK_PALETTE)
   {
-    csRGBpixel *pal = Image->GetPalette ();
+    const csRGBpixel *pal = Image->GetPalette ();
     
     palette = (png_colorp)malloc (256 * sizeof (png_color));
     int i;
@@ -303,14 +243,15 @@ error2:
     }
     png_set_PLTE (png, info, palette, max_color+1);
 
-    if (Image->HasKeycolor())
+    if (Image->HasKeyColor())
     {
       // Get the keycolor palette index and write the appropriate
       // tRNS chunk.
       int key_r, key_g, key_b;
-      Image->GetKeycolor (key_r, key_g, key_b);
+      Image->GetKeyColor (key_r, key_g, key_b);
       csRGBpixel key (key_r, key_g, key_b);
-      int key_index = closest_index (Image, &key);
+      int key_index = csImageTools::ClosestPaletteIndex (
+	Image->GetPalette(), key);
       png_bytep trans = new png_byte[key_index + 1];
       memset (trans, 0xff, key_index);
       trans[key_index] = 0;
@@ -321,15 +262,15 @@ error2:
   else
   {
     //Write tRNS chunk with keycolor.
-    if (Image->HasKeycolor())
+    if (Image->HasKeyColor())
     {
       int key_r, key_g, key_b;
-      Image->GetKeycolor (key_r, key_g, key_b);
+      Image->GetKeyColor (key_r, key_g, key_b);
       png_color_16 trans;
       memset (&trans, 0, sizeof(trans));
-      trans.red = big_endian_short (key_r << 8);
-      trans.green = big_endian_short (key_g << 8);
-      trans.blue = big_endian_short (key_b << 8);
+      trans.red = csBigEndian::UInt16 (key_r << 8);
+      trans.green = csBigEndian::UInt16 (key_g << 8);
+      trans.blue = csBigEndian::UInt16 (key_b << 8);
       png_set_tRNS (png, info, 0, 0, &trans);
     }
   }
@@ -360,7 +301,7 @@ error2:
   png_bytep packedData;
   if ((format & CS_IMGFMT_MASK) == CS_IMGFMT_TRUECOLOR)
   {
-    packedData = (png_bytep)csPackRGBpixelToRGBA 
+    packedData = (png_bytep)csPackRGBA::PackRGBpixelToRGBA 
       ((csRGBpixel *)Image->GetImageData (), width * height);
   }
   else
@@ -373,7 +314,7 @@ error2:
   png_write_image (png, row_pointers);
   if ((format & CS_IMGFMT_MASK) == CS_IMGFMT_TRUECOLOR)
   {
-    csDiscardPackedRGBA (packedData);
+    csPackRGBA::DiscardPackedRGBA (packedData);
   }
 
   /* It is REQUIRED to call this to finish writing the rest of the file */
@@ -408,15 +349,8 @@ csPtr<iDataBuffer> csPNGImageIO::Save (iImage *Image, const char *mime,
 
 //---------------------------------------------------------------------------
 
-struct ImagePngRawData
-{
-  // The buffer to "read" from
-  uint8 *r_data;
-  // The buffer size
-  size_t r_size;
-};
-
-void ImagePngRead (png_structp png, png_bytep data, png_size_t size)
+void ImagePngFile::PngLoader::ImagePngRead (png_structp png, png_bytep data, 
+					    png_size_t size)
 {
   ImagePngRawData *self = (ImagePngRawData *) png->io_ptr;
 
@@ -430,64 +364,64 @@ void ImagePngRead (png_structp png, png_bytep data, png_size_t size)
   } /* endif */
 }
 
-bool ImagePngFile::Load (uint8 *iBuffer, uint32 iSize)
+ImagePngFile::PngLoader::~PngLoader()
 {
-  size_t rowbytes, exp_rowbytes;
-  png_infop info;
+  if (png != 0)
+    png_destroy_read_struct (&png, &info, (png_infopp) 0);
+}
 
-  if (!png_check_sig (iBuffer, iSize))
+bool ImagePngFile::PngLoader::InitOk ()
+{
+  const png_bytep iBuffer = dataSource->GetUint8();
+  const size_t iSize = dataSource->GetSize();
+
+  if (!png_check_sig (iBuffer, (int)iSize))
     return false;
-
-  png_structp png =
-    png_create_read_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0);
-
+  png = png_create_read_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0);
   if (!png)
   {
-nomem:
-    FreeImage ();
     return false;
   }
+
   info = png_create_info_struct (png);
   if (!info)
   {
-nomem2:
     png_destroy_read_struct (&png, (png_infopp) 0, (png_infopp) 0);
-    goto nomem;
+    png = 0;
+    return false;
   }
 
   if (setjmp (png->jmpbuf))
+  {
+nomem2:
     // If we get here, we had a problem reading the file
-    goto nomem2;
+    png_destroy_read_struct (&png, &info, (png_infopp) 0);
+    png = 0; info = 0;
+    return false;
+  }
 
-  ImagePngRawData raw = { iBuffer, iSize };
+  raw.r_data = iBuffer;
+  raw.r_size = iSize;
   png_set_read_fn (png, &raw, ImagePngRead);
 
   png_read_info (png, info);
 
   // Get picture info
-  png_uint_32 Width, Height;
-  int bit_depth, color_type;
+  png_uint_32 W, H;
 
-  png_get_IHDR (png, info, &Width, &Height, &bit_depth, &color_type,
+  png_get_IHDR (png, info, &W, &H, &bit_depth, &color_type,
     0, 0, 0);
+  Width = W;
+  Height = H;
 
-  if (bit_depth > 8)
-  {
-    // tell libpng to strip 16 bit/color files down to 8 bits/color
-    png_set_strip_16 (png);
-    bit_depth = 8;
-  }
-  else if (bit_depth < 8)
-    // Expand pictures with less than 8bpp to 8bpp
-    png_set_packing (png);
-
-  volatile enum { imgRGB, imgPAL, imgGrayAlpha } ImageType;
-  volatile int keycolor_index = -1;
+  int NewFormat;
 
   switch (color_type)
   {
     case PNG_COLOR_TYPE_GRAY:
     case PNG_COLOR_TYPE_GRAY_ALPHA:
+      dataType = rdtIndexed;
+      NewFormat = (Format & ~CS_IMGFMT_MASK) | CS_IMGFMT_PALETTED8;
       if ((Format & CS_IMGFMT_ALPHA) && (color_type & PNG_COLOR_MASK_ALPHA))
       {
 	ImageType = imgGrayAlpha;
@@ -501,15 +435,17 @@ nomem2:
 	{
 	  png_color_16p trans_values;
 	  png_get_tRNS (png, info, 0, 0, &trans_values);
-	  has_keycolour = true;
-	  keycolour_r = convert_endian (trans_values->gray) & 0xff;
-	  keycolour_g = convert_endian (trans_values->gray) & 0xff;
-	  keycolour_b = convert_endian (trans_values->gray) & 0xff;
+	  hasKeycolor = true;
+	  keycolor.red = csLittleEndian::Convert (trans_values->gray) & 0xff;
+	  keycolor.green = csLittleEndian::Convert (trans_values->gray) & 0xff;
+	  keycolor.blue = csLittleEndian::Convert (trans_values->gray) & 0xff;
 	}
       }
       break;
     case PNG_COLOR_TYPE_PALETTE:
+      NewFormat = (Format & ~CS_IMGFMT_MASK) | CS_IMGFMT_PALETTED8;
       ImageType = imgPAL;
+      dataType = rdtIndexed;
       // If we need alpha, take it. If we don't, strip it.
       if (Format & CS_IMGFMT_ALPHA)
       {
@@ -540,26 +476,30 @@ nomem2:
 	    ImageType = imgRGB;
 	  }
 	  else
-	    Format &= ~CS_IMGFMT_ALPHA;
+	    NewFormat &= ~CS_IMGFMT_ALPHA; // We're basically keycolored
 	}
+	else
+	  NewFormat &= ~CS_IMGFMT_ALPHA; // Nope, not transparent
       }
       break;
     case PNG_COLOR_TYPE_RGB:
     case PNG_COLOR_TYPE_RGB_ALPHA:
       ImageType = imgRGB;
+      dataType = rdtRGBpixel;
+      NewFormat = (Format & ~CS_IMGFMT_MASK) | CS_IMGFMT_TRUECOLOR;
       // If there is no alpha information, fill with 0xff
       if (!(color_type & PNG_COLOR_MASK_ALPHA))
       {
-        Format &= ~CS_IMGFMT_ALPHA;
+        NewFormat &= ~CS_IMGFMT_ALPHA;
         // Check if we have keycolor transparency.
         if (png_get_valid (png, info, PNG_INFO_tRNS))
 	{
 	  png_color_16p trans_values;
 	  png_get_tRNS (png, info, 0, 0, &trans_values);
-	  has_keycolour = true;
-	  keycolour_r = convert_endian (trans_values->red) & 0xff;
-	  keycolour_g = convert_endian (trans_values->green) & 0xff;
-	  keycolour_b = convert_endian (trans_values->blue) & 0xff;
+	  hasKeycolor = true;
+	  keycolor.red = csLittleEndian::Convert (trans_values->red) & 0xff;
+	  keycolor.green = csLittleEndian::Convert (trans_values->green) & 0xff;
+	  keycolor.blue = csLittleEndian::Convert (trans_values->blue) & 0xff;
 	}
         png_set_filler (png, 0xff, PNG_FILLER_AFTER);
       }
@@ -568,11 +508,42 @@ nomem2:
       goto nomem2;
   }
 
+  if ((Format & CS_IMGFMT_MASK) == CS_IMGFMT_ANY)
+    Format = NewFormat;
+  else
+    // Copy alpha flag
+    Format = (Format & CS_IMGFMT_MASK) | (NewFormat & ~CS_IMGFMT_MASK);
+
+  return true;
+}
+
+bool ImagePngFile::PngLoader::LoadData ()
+{
+  size_t rowbytes, exp_rowbytes;
+
+  if (setjmp (png->jmpbuf))
+  {
+nomem2:
+    // If we get here, we had a problem reading the file
+    png_destroy_read_struct (&png, &info, (png_infopp) 0);
+    png = 0; info = 0;
+    return false;
+  }
+
+  if (bit_depth > 8)
+  {
+    // tell libpng to strip 16 bit/color files down to 8 bits/color
+    png_set_strip_16 (png);
+    bit_depth = 8;
+  }
+  else if (bit_depth < 8)
+    // Expand pictures with less than 8bpp to 8bpp
+    png_set_packing (png);
+
   // Update structure with the above settings
   png_read_update_info (png, info);
 
   // Allocate the memory to hold the image
-  set_dimensions (Width, Height);
   if (ImageType == imgRGB)
     exp_rowbytes = Width * 4;	  // RGBA
   else if (ImageType == imgGrayAlpha)
@@ -602,7 +573,7 @@ nomem2:
   if (!NewImage)
     goto nomem2;
 
-  for (png_uint_32 row = 0; row < Height; row++)
+  for (int row = 0; row < Height; row++)
     row_pointers [row] = ((png_bytep)NewImage) + row * rowbytes;
 
   // Read image data
@@ -613,16 +584,19 @@ nomem2:
 
   if (ImageType == imgRGB)
   {
-    csRGBpixel *rgbImage = csCopyUnpackRGBAtoRGBpixel
-      (NewImage, Width * Height);
-    convert_rgba (rgbImage);
-    delete[] NewImage;
+    if (csPackRGBA::IsRGBpixelSane())
+      rgbaData = (csRGBpixel*)NewImage;
+    else
+    {
+      rgbaData = csPackRGBA::CopyUnpackRGBAtoRGBpixel
+	(NewImage, Width * Height);
+      delete[] NewImage;
+    }
   }
   else if (ImageType == imgPAL)
   {
     png_color graypal [256];
     png_color *png_palette = 0;
-    csRGBcolor *palette;
     int colors, i;
     if (!png_get_PLTE (png, info, &png_palette, &colors))
     {
@@ -635,7 +609,8 @@ nomem2:
           (i * 255) / entries;
     }
     // copy PNG palette.
-    palette = new csRGBcolor[colors];
+    palette = new csRGBpixel[colors];
+    paletteCount = colors;
     for (i = 0; i < colors; i++)
     {
       palette[i].red = png_palette[i].red;
@@ -644,44 +619,52 @@ nomem2:
     }
     if (keycolor_index != -1)
     {
-      has_keycolour = true;
-      keycolour_r = palette[keycolor_index].red;
-      keycolour_g = palette[keycolor_index].green;
-      keycolour_b = palette[keycolor_index].blue;
+      hasKeycolor = true;
+      keycolor.red = palette[keycolor_index].red;
+      keycolor.green = palette[keycolor_index].green;
+      keycolor.blue = palette[keycolor_index].blue;
     }
-    convert_pal8 ((uint8 *)NewImage, palette, colors);
-    delete[] palette;
+    indexData = NewImage;
   }
   else // grayscale + alpha
   {
     // This is a grayscale image, build a grayscale palette
-    csRGBpixel *palette = new csRGBpixel [256];
+    paletteCount = 256;
+    palette = new csRGBpixel [256];
     int i, entries = (1 << bit_depth) - 1;
     for (i = 0; i <= entries; i++)
       palette [i].red = palette [i].green = palette [i].blue =
         (i * 255) / entries;
 
     int pixels = Width * Height;
-    uint8 *image = new uint8 [pixels];
-    Alpha = new uint8 [pixels];
+    indexData = new uint8 [pixels];
+    alpha = new uint8 [pixels];
     uint8 *src = (uint8 *)NewImage;
     for (i = 0; i < pixels; i++)
     {
-      image [i] = *src++;
-      Alpha [i] = *src++;
+      indexData [i] = *src++;
+      alpha [i] = *src++;
     }
     delete [] (uint8 *)NewImage;
-    convert_pal8 (image, palette);
   }
 
   // clean up after the read, and free any memory allocated
   png_destroy_read_struct (&png, &info, (png_infopp) 0);
+  png = 0; info = 0;
 
   // Free the row pointers array that is not needed anymore
   delete [] row_pointers;
 
-  // Check if the alpha channel is valid
-  CheckAlpha ();
-
+  dataSource = 0;
   return true;
 }
+
+csRef<iImageFileLoader> ImagePngFile::InitLoader (csRef<iDataBuffer> source)
+{
+  csRef<PngLoader> loader;
+  loader.AttachNew (new PngLoader (Format, source));
+  if (!loader->InitOk()) return 0;
+  return loader;
+}
+
+} // namespace cspluginPNGimg
