@@ -28,10 +28,10 @@
 ** implied warranty.
 */
 
+#include "cssysdef.h"
 #include <math.h>
 #include <stdio.h>
 
-#include "cssysdef.h"
 #include "tgaimage.h"
 #include "csgfx/rgbpixel.h"
 #include "csgfx/packrgb.h"
@@ -39,53 +39,23 @@
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csTGAImageIO)
-  SCF_IMPLEMENTS_INTERFACE (iImageIO)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csTGAImageIO::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_FACTORY (csTGAImageIO);
-
-
-/* Header definition. */
-struct TGAheader
+namespace cspluginTGAimg
 {
-  unsigned char IDLength;		/* length of Identifier String */
-  unsigned char CoMapType;		/* 0 = no map */
-  unsigned char ImgType;		/* image type (see below for values) */
-  unsigned char Index_lo, Index_hi;	/* index of first color map entry */
-  unsigned char Length_lo, Length_hi;	/* number of entries in color map */
-  unsigned char CoSize;			/* size of color map entry (15,16,24,32) */
-  unsigned char X_org_lo, X_org_hi;	/* x origin of image */
-  unsigned char Y_org_lo, Y_org_hi;	/* y origin of image */
-  unsigned char Width_lo, Width_hi;	/* width of image */
-  unsigned char Height_lo, Height_hi;	/* height of image */
-  unsigned char PixelSize;		/* pixel size (8,16,24,32) */
-  /* 
-    bits 7-6, interleaving flag
-    bit  5, origin: 0=lower left, 1=upper left
-    bit  4, reserved
-    bits 3-0, number of attribute bits per pixel 
-   */
-  unsigned char flags;
-};
+
+SCF_IMPLEMENT_FACTORY (csTGAImageIO)
 
 typedef char ImageIDField[256];
 
 /* Definitions for image types. */
-#define TGA_Null 0
-#define TGA_Map 1
-#define TGA_RGB 2
-#define TGA_Mono 3
-#define TGA_RLEMap 9
-#define TGA_RLERGB 10
-#define TGA_RLEMono 11
-#define TGA_CompMap 32
-#define TGA_CompMap4 33
+#define TGA_Null	0
+#define TGA_Map		1
+#define TGA_RGB		2
+#define TGA_Mono	3
+#define TGA_RLEMap	9
+#define TGA_RLERGB     10
+#define TGA_RLEMono    11
+#define TGA_CompMap    32
+#define TGA_CompMap4   33
 
 /* Definitions for interleave flag. */
 #define TGA_IL_None   0x00
@@ -97,22 +67,9 @@ typedef char ImageIDField[256];
 #define TGA_Org_BL    0x00
 #define TGA_Org_TL    0x20
 
-#define MAXCOLORS 16384
-
-#define CSTGA_ID "Made with CrystalSpace, see http://crystal.sf.net/"
+#define CSTGA_ID "Made with Crystal Space, see http://www.crystalspace3d.org/"
 #define TGA_MIME "image/tga"
 
-static int mapped, rlencoded;
-
-CS_IMPLEMENT_STATIC_VAR (GetColorMap, csRGBpixel, [MAXCOLORS])
-
-static int RLE_count = 0, RLE_flag = 0;
-
-static void readtga (uint8*& ptr, struct TGAheader* tgaP);
-static void get_map_entry (uint8*& ptr, csRGBpixel* Value, int Size, bool alpha);
-static void get_pixel (uint8*& ptr, csRGBpixel* dest, int Size, bool alpha);
-
-
 static iImageIO::FileFormatDescription formatlist[6] =
 {
   {TGA_MIME, "Map", CS_IMAGEIO_LOAD},
@@ -123,10 +80,9 @@ static iImageIO::FileFormatDescription formatlist[6] =
   {TGA_MIME, "RLEMono", CS_IMAGEIO_LOAD}
 };
 
-csTGAImageIO::csTGAImageIO (iBase *pParent)
+csTGAImageIO::csTGAImageIO (iBase *pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
   formats.Push (&formatlist[0]);
   formats.Push (&formatlist[1]);
   formats.Push (&formatlist[2]);
@@ -137,8 +93,6 @@ csTGAImageIO::csTGAImageIO (iBase *pParent)
 
 csTGAImageIO::~csTGAImageIO()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 const csImageIOFileFormatDescriptions& csTGAImageIO::GetDescription ()
@@ -146,10 +100,10 @@ const csImageIOFileFormatDescriptions& csTGAImageIO::GetDescription ()
   return formats;
 }
 
-csPtr<iImage> csTGAImageIO::Load (uint8* iBuffer, uint32 iSize, int iFormat)
+csPtr<iImage> csTGAImageIO::Load (iDataBuffer* buf, int iFormat)
 {
-  ImageTgaFile* i = new ImageTgaFile (iFormat);
-  if (i && !i->Load (iBuffer, iSize))
+  ImageTgaFile* i = new ImageTgaFile (object_reg, iFormat);
+  if (i && !i->Load (buf))
   {
      delete i ;
     return 0;
@@ -189,26 +143,23 @@ csPtr<iDataBuffer> csTGAImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
   TGAheader hdr;
   int w = Image->GetWidth ();
   int h = Image->GetHeight ();
-  bool has_alpha = Image->GetFormat() & CS_IMGFMT_ALPHA;
+  bool has_alpha = (Image->GetFormat() & CS_IMGFMT_ALPHA) != 0;
 
-  hdr.IDLength  = 0;//strlen (CSTGA_ID);
+  hdr.IDLength  = (uint8)(strlen (CSTGA_ID) + 1);
   hdr.CoMapType = (palette ? 1 : 0);
   hdr.ImgType = (palette ? TGA_Map : TGA_RGB);
-  hdr.Index_lo = hdr.Index_hi = 0;
-  hdr.Length_lo = 0;
-  hdr.Length_hi = (palette ? 1 : 0);
+  hdr.Index= 0;
+  hdr.Length = (palette ? 256 : 0);
   hdr.CoSize = (palette ? 24 : 0);
-  hdr.X_org_lo = hdr.X_org_hi = 0;
-  hdr.Y_org_lo = hdr.Y_org_hi = 0;
-  hdr.Width_lo = (w%256);
-  hdr.Width_hi = (w/256);
-  hdr.Height_lo = h%256;
-  hdr.Height_hi = h/256;
+  hdr.X_org = 0;
+  hdr.Y_org = 0;
+  hdr.Width = w;
+  hdr.Height = h;
   hdr.PixelSize = (palette ? 8 : (has_alpha?32:24));
   hdr.flags = TGA_IL_None | TGA_Org_BL;
 
-
-  size_t size = w*h*hdr.PixelSize/8 + MAXCOLORS * hdr.CoSize / 8 + sizeof (TGAheader) + hdr.IDLength;
+  size_t size = w * h * hdr.PixelSize/8 
+    + 256 * hdr.CoSize / 8 + sizeof (TGAheader) + hdr.IDLength;
 
   csDataBuffer *db = new csDataBuffer (size);
 
@@ -223,7 +174,7 @@ csPtr<iDataBuffer> csTGAImageIO::Save (iImage *Image, iImageIO::FileFormatDescri
   int i, x, y;
   if (palette)
   {
-    csRGBpixel *pal = Image->GetPalette ();
+    const csRGBpixel *pal = Image->GetPalette ();
     for (i= 0; i < 256; i++)
     {
       *(p++) = pal[i].blue;
@@ -267,216 +218,76 @@ csPtr<iDataBuffer> csTGAImageIO::Save (iImage *Image, const char *mime,
 
 //---------------------------------------------------------------------------
 
-bool ImageTgaFile::Load (uint8* iBuffer, uint32 iSize)
+csRef<iImageFileLoader> ImageTgaFile::InitLoader (csRef<iDataBuffer> source)
 {
-  (void)iSize;
-  struct TGAheader tga_head;
-  int i;
-  unsigned int temp1, temp2;
-  int rows, cols, row, col, realrow, truerow, baserow;
-  int maxval;
-
-  /* @@todo: Add TGA format detection */
-
-  /* Read the Targa file header. */
-  readtga (iBuffer, &tga_head);
-
-  rows = (int (tga_head.Height_lo)) + (int (tga_head.Height_hi)) * 256;
-  cols = (int (tga_head.Width_lo))  + (int (tga_head.Width_hi))  * 256;
-
-  switch (tga_head.ImgType)
-  {
-    case TGA_Map:
-    case TGA_RGB:
-    case TGA_Mono:
-    case TGA_RLEMap:
-    case TGA_RLERGB:
-    case TGA_RLEMono:
-      break;
-
-    default:
-      return false;
-  }
-
-  if (tga_head.ImgType == TGA_Map ||
-      tga_head.ImgType == TGA_RLEMap ||
-      tga_head.ImgType == TGA_CompMap ||
-      tga_head.ImgType == TGA_CompMap4)
-  { /* Color-mapped image */
-    if (tga_head.CoMapType != 1)
-      return false;
-    mapped = 1;
-    /* Figure maxval from CoSize. */
-    switch (tga_head.CoSize)
-    {
-      case 8:
-      case 24:
-      case 32:
-        maxval = 255;
-        break;
-
-      case 15:
-      case 16:
-        maxval = 31;
-        break;
-
-      default:
-	return false;
-    }
-  }
-  else
-  { /* Not colormap, so figure maxval from PixelSize. */
-    mapped = 0;
-    switch (tga_head.PixelSize)
-    {
-      case 8:
-      case 24:
-      case 32:
-        maxval = 255;
-        break;
-
-      case 15:
-      case 16:
-        maxval = 31;
-        break;
-
-      default:
-	return false;
-    }
-  }
-
-  /* If required, read the color map information. */
-  if (tga_head.CoMapType != 0)
-  {
-    temp1 = int (tga_head.Index_lo) + int (tga_head.Index_hi) * 256;
-    temp2 = int (tga_head.Length_lo) + int (tga_head.Length_hi) * 256;
-    if ((temp1 + temp2 + 1) >= MAXCOLORS)
-      return false;
-    for (i = temp1; i < int (temp1 + temp2); ++i)
-      get_map_entry (iBuffer, &(GetColorMap() [i]), (int) tga_head.CoSize,
-        Format & CS_IMGFMT_ALPHA);
-  }
-
-  /* Check run-length encoding. */
-  rlencoded = (tga_head.ImgType == TGA_RLEMap ||
-               tga_head.ImgType == TGA_RLERGB ||
-               tga_head.ImgType == TGA_RLEMono);
-
-  /* Read the Targa file body and convert to portable format. */
-  set_dimensions (cols, rows);
-
-  // @@todo: avoid converting colormapped images into RGB,
-  // instead pass a pointer to convert_pal8
-  csRGBpixel *pixels = new csRGBpixel [Width * Height];
-
-  truerow = 0;
-  baserow = 0;
-  for (row = 0; row < rows; ++row)
-  {
-    realrow = truerow;
-    if ((tga_head.flags & TGA_Org_MASK) == TGA_Org_BL)
-      realrow = rows - realrow - 1;
-
-    for (col = 0; col < cols; ++col)
-      get_pixel (iBuffer, &(pixels [realrow * cols + col]),
-        (int) tga_head.PixelSize, Format & CS_IMGFMT_ALPHA);
-    if ((tga_head.flags & TGA_IL_MASK) == TGA_IL_Four)
-      truerow += 4;
-    else if ((tga_head.flags & TGA_IL_MASK) == TGA_IL_Two)
-      truerow += 2;
-    else
-      ++truerow;
-    if (truerow >= rows)
-      truerow = ++baserow;
-  }
-
-  // Convert image from RGB to requested format
-  convert_rgba (pixels);
-
-  // Check if the alpha channel is valid
-  CheckAlpha ();
-
-  return true;
+  csRef<TgaLoader> loader;
+  loader.AttachNew (new TgaLoader (Format, source));
+  if (!loader->InitOk()) return 0;
+  return loader;
 }
 
-
-static void readtga (uint8*& iBuffer, TGAheader* tgaP)
+bool ImageTgaFile::TgaLoader::readtga (uint8*& iBuffer, TGAheader* tgaP)
 {
-//  unsigned char flags;
-
-  tgaP->IDLength = *iBuffer++;
-  tgaP->CoMapType = *iBuffer++;
-  tgaP->ImgType = *iBuffer++;
-  tgaP->Index_lo = *iBuffer++;
-  tgaP->Index_hi = *iBuffer++;
-  tgaP->Length_lo = *iBuffer++;
-  tgaP->Length_hi = *iBuffer++;
-  tgaP->CoSize = *iBuffer++;
-  tgaP->X_org_lo = *iBuffer++;
-  tgaP->X_org_hi = *iBuffer++;
-  tgaP->Y_org_lo = *iBuffer++;
-  tgaP->Y_org_hi = *iBuffer++;
-  tgaP->Width_lo = *iBuffer++;
-  tgaP->Width_hi = *iBuffer++;
-  tgaP->Height_lo = *iBuffer++;
-  tgaP->Height_hi = *iBuffer++;
-  tgaP->PixelSize = *iBuffer++;
-  tgaP->flags = *iBuffer++;
+  if (!GetBytes (iBuffer, (uint8*)tgaP, sizeof (TGAheader))) return false;
 
   if (tgaP->IDLength != 0)
     iBuffer += tgaP->IDLength;
+  return (iBuffer <= bufferEnd);
 }
 
-static void get_map_entry (uint8*& iBuffer, csRGBpixel* Value, int Size, bool alpha)
+bool ImageTgaFile::TgaLoader::get_map_entry (uint8*& iBuffer, 
+					     csRGBpixel* Value, int Size, 
+					     bool alpha)
 {
-  unsigned char j, k;
+  uint8 bytes[4];
+  uint l;
 
   /* Read appropriate number of bytes, break into rgb & put in map. */
   switch (Size)
   {
     case 8:				/* Grey scale, read and triplicate. */
-      Value->red = Value->green = Value->blue = *iBuffer++;
+      if (!GetBytes (iBuffer, bytes, 1)) return false;
+      Value->red = Value->green = Value->blue = bytes[0];
       break;
 
     case 16:				/* 5 bits each of red green and blue. */
     case 15:				/* Watch for byte order. */
-      j = *iBuffer++;
-      k = *iBuffer++;
-      Value->red = (k & 0x7C) >> 2;
-      Value->green = ((k & 0x03) << 3) + ((j & 0xE0) >> 5);
-      Value->blue = j & 0x1F;
+      if (!GetBytes (iBuffer, bytes, 2)) return false;
+      l = ((unsigned int) bytes[1] << 8) + bytes[0];
+      Value->red = (bytes[1] & 0x7C) << 1;
+      Value->red |= Value->red >> 5;
+      Value->green = ((bytes[1] & 0x03) << 6) | ((bytes[0] & 0xE0) >> 2);
+      Value->green |= Value->green >> 5;
+      Value->blue = (bytes[0] & 0x1F) << 3;
+      Value->blue |= Value->blue >> 5;
       break;
 
     case 32:
     case 24:				/* 8 bits each of blue green and red. */
-      Value->blue = *iBuffer++;
-      Value->green = *iBuffer++;
-      Value->red = *iBuffer++;
-      if (Size == 32)
-        if (alpha)
-          Value->alpha = *iBuffer++;	/* Read alpha byte */
-        else
-          iBuffer++;
+      if (!GetBytes (iBuffer, bytes, (Size == 32) ? 4 : 3)) return false;
+      Value->blue = bytes[0];
+      Value->green = bytes[1];
+      Value->red = bytes[2];
+      if ((Size == 32) && alpha)
+          Value->alpha = bytes[3];	/* Read alpha byte */
       break;
 
     default:
-      return;
+      return false;
   }
+  return true;
 }
 
-static void get_pixel (uint8*& iBuffer, csRGBpixel* dest, int Size, bool alpha)
+bool ImageTgaFile::TgaLoader::get_current_pixel (uint8*& iBuffer, int Size, 
+						 bool alpha)
 {
-  static int Red, Grn, Blu, Alpha;
-  static unsigned int l;
-  unsigned char j, k;
-
   /* Check if run length encoded. */
   if (rlencoded)
   {
     if (RLE_count == 0)
     { /* Have to restart run. */
-      unsigned char i;
-      i = *iBuffer++;
+      uint8 i;
+      if (!GetBytes (iBuffer, &i, 1)) return false;
       RLE_flag = (i & 0x80);
       if (RLE_flag == 0)
         /* Stream of unencoded pixels. */
@@ -492,52 +303,245 @@ static void get_pixel (uint8*& iBuffer, csRGBpixel* dest, int Size, bool alpha)
       --RLE_count;
       if (RLE_flag != 0)
       /* Replicated pixels. */
-        goto PixEncode;
+        return true;
     }
   }
 
+  uint8 bytes[4];
   /* Read appropriate number of bytes, break into RGB. */
   switch (Size)
   {
     case 8:				/* Grey scale, read and triplicate. */
-      Red = Grn = Blu = l = *iBuffer++;
-      Alpha = 0;
+      if (!GetBytes (iBuffer, bytes, 1)) return false;
+      currentPixel.Red = currentPixel.Grn = currentPixel.Blu = currentPixel.l = bytes[0];
+      currentPixel.Alpha = 0xff;
       break;
 
     case 16:				/* 5 bits each of red green and blue. */
     case 15:				/* Watch byte order. */
-      j = *iBuffer++;
-      k = *iBuffer++;
-      l = ((unsigned int) k << 8)  + j;
-      Red = (k & 0x7C)  >> 2;
-      Grn = ( (k & 0x03)  << 3)  + ( (j & 0xE0)  >> 5);
-      Blu = j & 0x1F;
-      Alpha = 0;
+      if (!GetBytes (iBuffer, bytes, 2)) return false;
+      currentPixel.l = ((unsigned int) bytes[1] << 8) + bytes[0];
+      currentPixel.Red = (bytes[1] & 0x7C) << 1;
+      currentPixel.Red |= currentPixel.Red >> 5;
+      currentPixel.Grn = ((bytes[1] & 0x03) << 6) | ((bytes[0] & 0xE0) >> 2);
+      currentPixel.Grn |= currentPixel.Grn >> 5;
+      currentPixel.Blu = (bytes[0] & 0x1F) << 3;
+      currentPixel.Blu |= currentPixel.Blu >> 5;
+      currentPixel.Alpha = 0xff;
       break;
 
     case 32:
     case 24:			/* 8 bits each of blue green and red. */
-      Blu = *iBuffer++;
-      Grn = *iBuffer++;
-      Red = *iBuffer++;
-      Alpha = 0xff;
-      if (Size == 32)
-        if (alpha)
-          Alpha = *iBuffer++;	/* Read alpha byte */
-        else
-          iBuffer++;
-      l = 0;
+      if (!GetBytes (iBuffer, bytes, (Size == 32) ? 4 : 3)) return false;
+      currentPixel.Blu = bytes[0];
+      currentPixel.Grn = bytes[1];
+      currentPixel.Red = bytes[2];
+      currentPixel.Alpha = 0xff;
+      if ((Size == 32) && alpha)
+        currentPixel.Alpha = bytes[3];	/* Read alpha byte */
+      currentPixel.l = 0;
       break;
 
     default:
-      return;
+      return false;
   }
+  return true;
+}
+
+bool ImageTgaFile::TgaLoader::get_pixel (uint8*& iBuffer, csRGBpixel* dest, 
+					 int Size, bool alpha)
+{
+  if (!get_current_pixel (iBuffer, Size, alpha)) return false;
 
-PixEncode:
   if (mapped)
-    *dest = GetColorMap() [l];
+    *dest = colorMap[currentPixel.l];
   else
   {
-    dest->red = Red; dest->green = Grn; dest->blue = Blu; dest->alpha = Alpha;
+    dest->red = currentPixel.Red;
+    dest->green = currentPixel.Grn; 
+    dest->blue = currentPixel.Blu; 
+    dest->alpha = currentPixel.Alpha;
   }
+  return true;
 }
+
+bool ImageTgaFile::TgaLoader::get_pixel (uint8*& iBuffer, uint8* dest, 
+					 int Size, bool alpha)
+{
+  if (!get_current_pixel (iBuffer, Size, alpha)) return false;
+
+  if (tga_head.PixelSize <= 8)
+    *dest = currentPixel.l << indexShift;
+  else
+    *dest = currentPixel.l >> indexShift;
+  return true;
+}
+
+ImageTgaFile::TgaLoader::~TgaLoader()
+{
+  delete[] colorMap;
+}
+
+bool ImageTgaFile::TgaLoader::InitOk()
+{
+  int NewFormat;
+  iBuffer = dataSource->GetUint8();
+  bufferEnd = iBuffer + dataSource->GetSize();
+  if (!readtga (iBuffer, &tga_head)) return false;
+
+  switch (tga_head.ImgType)
+  {
+    case TGA_Map:
+    case TGA_RGB:
+    case TGA_Mono:
+    case TGA_RLEMap:
+    case TGA_RLERGB:
+    case TGA_RLEMono:
+      break;
+
+    default:
+      return false;
+  }
+
+  Height = tga_head.Height;
+  Width = tga_head.Width;
+  colorMapSize = tga_head.Length;
+
+  if (tga_head.ImgType == TGA_Map ||
+      tga_head.ImgType == TGA_RLEMap ||
+      tga_head.ImgType == TGA_CompMap ||
+      tga_head.ImgType == TGA_CompMap4)
+  { /* Color-mapped image */
+    if (tga_head.CoMapType != 1)
+      return false;
+    mapped = true;
+  }
+  else if ((tga_head.ImgType == TGA_Mono)
+    || (tga_head.ImgType == TGA_RLEMono))
+  {
+    // Grayscale image
+    colorMapSize = 1 << tga_head.PixelSize;
+    mapped = true;
+  }
+  else
+    mapped = false;
+  if (mapped && (colorMapSize <= 256))
+  {
+    dataType = rdtIndexed;
+    NewFormat = (Format & ~CS_IMGFMT_MASK) | CS_IMGFMT_PALETTED8;
+  }
+  else
+  {
+    dataType = rdtRGBpixel;
+    NewFormat = (Format & ~CS_IMGFMT_MASK) | CS_IMGFMT_TRUECOLOR;
+  }
+
+  if ((Format & CS_IMGFMT_MASK) == CS_IMGFMT_ANY)
+    Format = NewFormat;
+  else
+    // Copy alpha flag
+    Format = (Format & CS_IMGFMT_MASK) | (NewFormat & ~CS_IMGFMT_MASK);
+
+  return true;
+}
+
+bool ImageTgaFile::TgaLoader::LoadData ()
+{
+  if (colorMapSize != 0)
+  {
+    if (tga_head.PixelSize <= 8)
+      indexShift = 8 - tga_head.PixelSize;
+    else
+      indexShift = tga_head.PixelSize - 8;
+
+    uint colorMapOffs = tga_head.Index;
+    colorMap = new csRGBpixel [colorMapOffs + colorMapSize];
+    if (tga_head.CoMapType != 0)
+    {
+      for (uint i = colorMapOffs; i < colorMapOffs + colorMapSize; ++i)
+      {
+	get_map_entry (iBuffer, colorMap + i, tga_head.CoSize,
+	  (Format & CS_IMGFMT_ALPHA) != 0);
+      }
+    }
+    else
+    {
+      if (tga_head.PixelSize <= 8)
+      {
+	for (uint i = 0; i < colorMapSize; i++)
+	  colorMap[i].Set (i << indexShift, i << indexShift, i << indexShift);
+      }
+      else
+      {
+	for (uint i = 0; i < colorMapSize; i++)
+	  colorMap[i].Set (i >> indexShift, i >> indexShift, i >> indexShift);
+      }
+    }
+  }
+
+  /* Check run-length encoding. */
+  rlencoded = (tga_head.ImgType == TGA_RLEMap ||
+               tga_head.ImgType == TGA_RLERGB ||
+               tga_head.ImgType == TGA_RLEMono);
+
+  if (dataType == rdtIndexed)
+  {
+    indexData = new uint8[Width * Height];
+    palette = colorMap;
+    paletteCount = colorMapSize;
+    colorMap = 0;
+  }
+  else
+    rgbaData = new csRGBpixel [Width * Height];
+
+  int truerow = 0;
+  int baserow = 0;
+  for (int row = 0; row < Height; ++row)
+  {
+    int realrow = truerow;
+    if ((tga_head.flags & TGA_Org_MASK) == TGA_Org_BL)
+      realrow = Height - realrow - 1;
+
+    if (dataType == rdtIndexed)
+    {
+      for (int col = 0; col < Width; ++col)
+      {
+	if (!get_pixel (iBuffer, indexData + (realrow * Width + col),
+	  tga_head.PixelSize, (Format & CS_IMGFMT_ALPHA) != 0))
+        {
+          // Can't read pixel.... file is probably damaged.
+          // Use obnoxious pattern for replacement.
+          int v = ((realrow >> 1) & 1) ^ ((col >> 1) & 1);
+          *(indexData + (realrow * Width + col)) = v * 255;
+        }
+      }
+    }
+    else
+    {
+      for (int col = 0; col < Width; ++col)
+      {
+	if (!get_pixel (iBuffer, rgbaData + (realrow * Width + col),
+	  tga_head.PixelSize, (Format & CS_IMGFMT_ALPHA) != 0))
+        {
+          // Can't read pixel.... file is probably damaged.
+          // Use obnoxious pattern for replacement.
+          int v = ((realrow >> 1) & 1) ^ ((col >> 1) & 1);
+          (rgbaData + (realrow * Width + col))->Set (255*v, 255*(1-v), 255);
+        }
+      }
+    }
+    if ((tga_head.flags & TGA_IL_MASK) == TGA_IL_Four)
+      truerow += 4;
+    else if ((tga_head.flags & TGA_IL_MASK) == TGA_IL_Two)
+      truerow += 2;
+    else
+      ++truerow;
+    if (truerow >= Height)
+      truerow = ++baserow;
+  }
+
+  dataSource = 0;
+  return true;
+}
+
+} // namespace cspluginTGAimg
